(function () {
    'use strict';

    class Utils {

        constructor() {
        }

        static getSign(d) {
            if(d >= 0) {
                return 1
            }
            return -1;
        }

        static randomSign() {
            var n = Math.random();
            if(n > 0.5) {
                return -1;
            }
            return 1;
        }

        static randomExt(num) {
            let n = Math.random();
            return Math.floor(n*num);
        }

        static randomExt2(min, max) {
            let d = max - min;
            return min + Utils.randomExt(d);
        }

        static randomRangePoint(point, r) {
            let rx = point.x + Utils.randomSign() * Utils.randomExt(r);
            let ry = point.y + Utils.randomSign() * Utils.randomExt(r);
            return new Laya.Point(rx, ry);
        }

        //依据权重随机，要求arr的每一项的w属性为权重
        //totalWeight为空时自动从arr计算总权重
        static calcRandomWeight(arr, totalWeight) {
            let weightSum = totalWeight;
            let weightArr = arr;
            if(!weightSum) {
                weightSum = 0;
                for(const data of arr) {
                    weightSum = weightSum+data.w;
                }
            }
            let d = Utils.randomExt(weightSum);

            let pre = {w: 0};
            let ret = null;
            for(const cur of arr) {
                if(d >= pre.w && d < (cur.w + pre.w)) {
                    ret = cur;
                    break;
                } else {
                    pre.w += cur.w;
                }
            }
            return ret;
        }

        static map2Obj(map) {
            let obj = Object.create(null);
            for(let[k,v] of map) {
                obj[k] = v;
            }
            return obj;
        }

        static obj2Map(obj) {
            let map = new Map();
            for(let k of Object.keys(obj)) {
                map.set(k, obj[k]);
            }
            return map;
        }

        static fadeout(owner, time, handler) {
            owner.alpha = 1;
            Laya.Tween.to(owner, { alpha: 0 }, time,
                null,
                handler, 0);
        }

        static fadein(owner, time, handler) {
            owner.alpha = 0;
            Laya.Tween.to(owner, { alpha: 1 }, time,
                null,
                handler, 0);
        }
    }

    class TempletMgr {

        constructor() {
            this._templetMap = new Map();
            this._waitingMap = new Map();
        }

        clear() {
            for(const [key, templet] of this._templetMap) {
                templet._removeReference();
                templet.destroy();
            }
            this._templetMap.clear();
            this._waitingMap.clear();
        }

        purge() {
            for(const [key, templet] of this._templetMap) {
                if(!this._waitingMap.has(key)){
                    if(templet.referenceCount==1) {
                        templet._removeReference();
                        templet.destroy();
                        this._templetMap.delete(key);
                    }
                }
            }
        }

        check(key) {
            let templet = this._templetMap.get(key);
            if(templet && templet.referenceCount==1){
                templet._removeReference();
                templet.destroy();
                this._templetMap.delete(key);
            }
        }

        static getInstance() {
            return TempletMgr.instance?TempletMgr.instance:TempletMgr.instance = new TempletMgr();
        }

        load(path, skin, aniMode, completeHandler) {
            // console.debug("TempletMgr load", path, skin, aniMode);
            let key = skin ? path+"_"+skin : path+"_"+(aniMode?1:0);
            let templet = this._templetMap.get(key);
            if(!templet){
                templet = new Laya.Templet();
                templet.on(Laya.Event.COMPLETE, this, this.onTempletLoaded, [key, aniMode]);
                templet.on(Laya.Event.ERROR, this, this.onTempletLoadError, [key, aniMode]);
                templet.loadAni(path);
                templet._addReference();
                this._templetMap.set(key, templet);
                let handlerArray = new Array();
                this._waitingMap.set(key, handlerArray);
                if(completeHandler != null) {
                    handlerArray.push(completeHandler);
                }
            }else {
                let handlerArray = this._waitingMap.get(key);
                if(handlerArray){
                    if(completeHandler != null) {
                        handlerArray.push(completeHandler);
                    }
                }else {
                    if(completeHandler != null) {
                        Laya.timer.frameOnce(1, completeHandler, completeHandler.runWith, [templet, aniMode]);
                        // completeHandler.runWith([templet, aniMode]);
                    }
                }
            }
            return key;
        }

        onTempletLoaded(key, aniMode) {
            let templet = this._templetMap.get(key);
            let handlerArray = this._waitingMap.get(key);
            if(templet && handlerArray) {
                for(const handler of handlerArray) {
                    handler.runWith(templet, aniMode);
                }
            }
            this._waitingMap.delete(key);
        }

        onTempletLoadError(key, aniMode, templet) {
            //报个错？
            console.error("onTempletLoadError", key, aniMode);
            if(templet) {
                console.debug("onTempletLoadError templet:", templet._isDestroyed, templet);
                // templet.destroy(); templet是个错误字符串
            }
            this._waitingMap.delete(key);
        }
    }

    class Spine extends Laya.Skeleton {
        constructor(skin=null) {
            super();
            this.__skin = skin;
            this.loop = true;
            this.__loadTimes = 3;
            this.curAnim = "";
            this.loadedHandlers = new Array();
            this.eventHandlers = new Array();
            this.completeHandlers = new Array();

        }

        //添加加载完成回调
        addLoadedHandler(handler) {
            if(handler)
                this.loadedHandlers.push(handler);
        }

        //添加动画播放完成回调
        addCompleteHandler(handler) {
            if(handler)
                this.completeHandlers.push(handler);
        }

        clearCompleteHandler() {
            this.completeHandlers.length = 0;
        }

        //添加帧事件回调
        addEventHandler(handler) {
            if(handler)
                this.eventHandlers.push(handler);
        }

        removeEventHandler(eventHandler) {
            for(let i = this.eventHandlers.length - 1; i>=0; --i) {
                let handler = this.eventHandlers[i];
                if(handler==eventHandler) {
                    this.eventHandlers.splice(i,1);
                    return true;
                }
            }
            return false;
        }

        //加载动画
        loadAnim(path, skin, aniMode) {
            if(--this.__loadTimes<=0) {
                return;
            }
            this.__path = path;
            this.__skin = skin;
            this.__aniMode = aniMode;

            this.__templetKey = TempletMgr.getInstance().load(path, skin, aniMode, Laya.Handler.create(this, this.onLoadedExt));
            this.on(Laya.Event.STOPPED, this, this.onCompleteExt);
            this.on(Laya.Event.LABEL, this, this.onEventExt);
        }

        playAnim(aniName, loop = true, force = false) {
            if(this._inited){
                this.play(aniName, loop, force);
            }
            this.force = force;
            this.loop = loop;
            this.curAnim = aniName;
        }

        stopAnim() {
            if(this._inited) {
                this.stop();
            }
            this.loop = null;
            this.curAnim = null;
        }

        getCurAnim() {
            return this.curAnim;
        }

        //========================================================================
        onCompleteExt() {
            for(const handler of this.completeHandlers) {
                handler.runWith(this);
            }
            this.completeHandlers.length = 0;
            // if(this.curAnim&&this.loop) {
            //     this.play(this.curAnim, this.loop, this.force);
            // }
        }

        onEventExt(eventData) {
            // console.debug("onEventExt", eventData.name);
            for(let i = this.eventHandlers.length - 1; i>=0; --i) {
                let handler = this.eventHandlers[i];
                if(handler.once) {
                    handler.runWith(eventData);
                    this.eventHandlers.splice(i,1);
                }else
                    handler.runWith(eventData);
            }
            // this.eventHandlers.length = 0;
        }

        onLoadedExt(templet, aniMode) {
            if(this.destroyed)
                return;
            if(!templet || templet._isDestroyed) {
                this.loadAnim(this.__path, this.__skin, this.__aniMode);
                return;
            }
            this.init(templet, aniMode?1:0);
            if(this.__skin){
                this.showSkinByName(this.__skin);
            }else {
                this.showSkinByIndex(1);
            }
            this._inited = true;

            if(this.curAnim!=null&&!this._played) {
                this.play(this.curAnim, this.loop, this.force);
                this._played = true;
            }
            for(const handler of this.loadedHandlers) {
                handler.runWith(this);
            }
            this.loadedHandlers.length = 0;
        }

        destroy(destroyChild = true) {
            this._inited = false;
            super.destroy(destroyChild);
            TempletMgr.getInstance().check(this.___templetKey);
        }



    }

    class BaseCat extends Spine {
        constructor(data) {
            super();
            this.loopEndHandlers = new Array();
        }

        playAnim(animName, loop = true) {
            this.loopEndHandlers.length = 0;
            this._animState = null;
            this._loopTimes = null;
            super.playAnim(animName, loop);
        }

        onEventExt(event) {
            super.onEventExt(event);

            if(event && event.name && event.name.substring(0,8)=="animLoop") {
                let data = JSON.parse(event.stringValue);
                if(this._animState==null) {
                    this._animState = "loop";
                    if(data.loopTimes)
                        this._loopTimes = Number(data.loopTimes);
                    else
                        this._loopTimes = null;
                    this.play(data.mid, true);
                }else if(this._animState=="loop") {
                    if(this._loopTimes) {
                        if(--this._loopTimes<=0) {
                            this._animState = "ending";
                            if(data.end)
                                this.play(data.end, false);
                            else {
                                // this.play(data.mid, false);
                                this._animState = null;
                                this._loopTimes = null;
                                this.onLoopEnd();
                            }
                            this._loopTimes = null;
                        }else {
                            // this.play(data.mid, false);
                        }
                    }else {
                        // this.play(data.mid, false);
                    }
                }else if(this._animState=="ending") {
                    this._animState = null;
                    this._loopTimes = null;
                    this.onLoopEnd();
                }else if(this._animState=="stop") {
                    this._loopTimes = null;
                    this._animState = "ending";
                    if(data.end)
                        this.play(data.end, false);
                    else {
                        this._animState = null;
                        this._loopTimes = null;
                        this.onLoopEnd();
                        // this.play(data.mid, false);
                    }

                }
            }
        }

        onLoopEnd() {
            for(const handler of this.loopEndHandlers) {
                handler.runWith(this);
            }
            this.loopEndHandlers.length = 0;
        }

        addLoopEndHandler(loopEndHandler) {
            if(loopEndHandler)
                this.loopEndHandlers.push(loopEndHandler);
        }

        clearLoopEndHandler() {
            this.loopEndHandlers.length = 0;
        }

        stopLoop(loopEndHandler) {
            if(this.player && this.player.state == Laya.AnimationState.playing){
                if(this._animState){
                    this.addLoopEndHandler(loopEndHandler);
                    if(this._animState=="loop")
                        this._animState = "stop";
                }else {
                    this.addCompleteHandler(loopEndHandler);
                    this.player.stop(false);
                }
            }else {
                if(loopEndHandler)
                    loopEndHandler.runWith(this);
                this.stopAnim();
            }
        }

        stopAnim() {
            this.loopEndHandlers.length = 0;
            super.stopAnim();
        }

        onDestroy() {
            this.loopEndHandlers.length = 0;
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }

    }

    class EventMgr {

        constructor() {
            this.eventDispatcher = new Laya.EventDispatcher();
        }

        static getInstance() {
            return EventMgr.instance?EventMgr.instance:EventMgr.instance = new EventMgr();
        }

        //广播消息
        postEvent(eventName, argv) {
            this.eventDispatcher.event(eventName, argv);
        }

        //注册消息
        registEvent(eventName, caller, listener, args) {
            this.removeEvent(eventName, caller, listener);
            this.eventDispatcher.on(eventName, caller, listener, args);
        }

        //移除消息
        removeEvent(eventName, caller, listener) {
            this.eventDispatcher.off(eventName, caller, listener, false);
        }

        //慎用，不传能清掉所有的事件
        removeEventAll(eventName) {
            this.eventDispatcher.offAll(eventName);
        }

        offAllCaller(caller) {
            this.eventDispatcher.offAllCaller(caller);
        }

        //移除所有消息
        removeAllEvent() {
            this.eventDispatcher.offAll(eventName);
        }
    }

    class Cat extends BaseCat {
        constructor(data) {
            super();
            this.catData = data;
            this.maskAnim = null;
            this.zOrder = 50;
            this._zOrderOffset = 0;
        }

        get buildingId() {
            return this._buildingId;
        }

        set buildingId(buildingId) {
            this._buildingId = buildingId;
        }

        get spaceId() {
            return this._spaceId;
        }

        set spaceId(spaceId) {
            this._spaceId = spaceId;
        }

        getId() {
            return this.catData.getCatId();
        }

        //获得信息
        getData() {
            return this.catData;
        }

        get zOrderOffset() {
            return this._zOrderOffset;
        }

        set zOrderOffset(offset) {
            this._zOrderOffset = offset;
            this.yy = this.y;
        }

        posInMap(x, y) {
            this.x = x;
            this.yy = y;
        }

        get yy() {
            return this.y;
        }

        set yy(y) {
            this.y = y;
            // this._setY(y);
            this.zOrder = Math.floor(y)+this._zOrderOffset;
        }

        set scaleX(scale){
            super.scaleX = scale;
            if (this.maskAnim && !this.maskAnim.destroyed) {
                if(this.maskAnim.scaleX < 0){
                    this.maskAnim.scaleX = -1 * this.maskAnim.scaleX;
                }
            }
        }

        get scaleX(){
            return super.scaleX;
        }

        registerMouseDown(listener, func) {
            let panel =  new Laya.Panel();
            panel.mouseThrough = true;
            // panel.bgColor = "#c3504e";
            panel.width = 200;
            panel.height = 200;
            panel.centerX = 0.5;
            panel.y = -200;
            panel.on(Laya.Event.MOUSE_DOWN, listener, func);
            this.panel = panel;
            this.addChild(panel);
        }

        //初始化叹号
        initMaskAnim() {
            let anim = new Spine();
            this.maskAnim = anim;
            anim.y = -250;
            anim.visible = false;
            let outSelf = this;
            anim.addLoadedHandler(Laya.Handler.create(this, this._setMaskAnimLoaded));

            this.addChild(anim);
            // setTimeout(() => {
                anim.loadAnim("anim1/UIdonghua.sk", null, 0);
            // },100);
        }

        _setMaskAnimLoaded() {
            this.maskAnimLoaded = true;
            if(this._maskAnim_visible!=null) {
                this.maskAnim.visible = this._maskAnim_visible;
                this._maskAnim_visible = null;
            }
        }

        set maskAnimVisible(visible) {
            if(this.maskAnim)
                this.maskAnim.visible = visible;
            else
                this._maskAnim_visible = visible;
        }

        showMaskAnim(animName, showDuration, handler, x, y, once) {
            if(!this.maskAnim) {
                this.initMaskAnim();
            }
            this.maskAnimVisible = true;
            if (animName != "tanhao") {
                this.maskAnim.y = -200;
            } else {
                this.maskAnim.y = -250;
            }
            if (x != null) {
                this.maskAnim.x = x;
            } else {
                this.maskAnim.x = 0;
            }
            if (y != null) {
                this.maskAnim.y = y;
            }
            if (once) {
                this.setEmojiScaleX(this.scaleX);
                this.maskAnim.playAnim(animName, false);
                this.maskAnim.addCompleteHandler(Laya.Handler.create(this, function () {
                    this.hideMaskAnim();
                }));
                return;
            }
            this.maskAnim.playAnim(animName, true);
            if(!showDuration) {
                Laya.timer.clear(this, this.hideMaskAnim);
            }else {
                Laya.timer.once(showDuration, this, this.hideMaskAnim, [handler]);
            }
        }

        hideMaskAnim(handler) {
            if(this.maskAnimLoaded){
                this.maskAnim.stop();
                this.maskAnim.visible = false;
            }
            if(handler != null) {
                handler.run();
            }
        }

        onDestroy() {
            super.onDestroy();
            if(this.catMenuTips) {
                this.catMenuTips.destroy();
                this.catMenuTips= null;
            }
        }

        onEnable() {
            super.onEnable();
            if(this.catMenuTips){
                this.catMenuTips.onEnable();
            }
        }

        onDisable() {
            super.onDisable();
            if(this.catMenuTips){
                this.catMenuTips.onDisable();
            }
        }

        clone() {
            let cat = new Cat(this.catData);
            cat.posInMap(this.x, this.yy);
            cat.zOrderOffset = this.zOrderOffset;
            cat.loadAnim(this.__path, this.__skin, this.__aniMode);
            cat.playAnim("lick1");
            return cat;
        }

        enableTouch() {
            let panel = this._panel;
            if (!panel) {
                panel = new Laya.Panel();
                // panel.bgColor = "#c3504e";
                panel.width = 200;
                panel.height = 190;
                panel.centerX = 0.5;
                panel.y = -panel.height;
                panel.mouseThrough = false;
                this._panel = panel;
            }
            this.addChild(panel);
            panel.offAll(Laya.Event.CLICK);
            panel.on(Laya.Event.CLICK, this, this.onClick);
        }

        disableTouch() {
            let panel = this._panel;
            if (panel) {
                this.removeChild(panel);
                panel.offAll(Laya.Event.CLICK);
            }
        }

        onClick() {
            EventMgr.getInstance().postEvent("COFFEE_CAT_CLICK", [this.getId()]);
        }

        setEmojiScaleX(scale) {
            if (this.maskAnim != null && !this.maskAnim.destroyed) {
                if (scale <= 0) {
                    this.maskAnim.scaleX = 0 - Math.abs(this.maskAnim.scaleX);
                    this.maskAnim.x = Math.abs(this.maskAnim.x);
                } else {
                    this.maskAnim.scaleX = Math.abs(this.maskAnim.scaleX);
                    this.maskAnim.x = 0 - Math.abs(this.maskAnim.x);
                }
            }
        }

        playAnim(animName, loop = true) {
            super.playAnim(animName, loop);
            this.palySkillTagAnim(animName, loop);
        }

        palySkillTagAnim(anim) {
            if (!this.catData.isSkillCat()) {
                return;
            }
            if (anim != "lick1" || anim != "move1" || anim != "moive2" || anim != "rest1" || anim != "rest2") {
                if (this.skillAnim) {
                    this.skillAnim.visible = false;
                }
            }
            let source = this.catData.getSource();
            source = source.replace("mao", "Rmao") + ".sk";
            if (!this.skillAnim) {
                this.skillAnim = new Spine();
                this.addChild(this.skillAnim);
                this.skillAnim.loadAnim(source);
            }
            this.skillAnim.visible = true;
            this.skillAnim.playAnim(anim);
        }

    }

    let ConfigExt = ".txt";
    class ConfigMgr$1 {

        constructor() {
            this._Protobuf = protobuf;
            this._configs = new Map();
            this.configClassMap = new Map();
            this.subConfigClassMap = new Map();
            this._registClass();
        }

        static getInstance() {
            return ConfigMgr$1.instance?ConfigMgr$1.instance:ConfigMgr$1.instance = new ConfigMgr$1();
        }

        _registClass() {
            this.configClassMap.set("Const", pattern.Const);
            this.configClassMap.set("ActionEvent", pattern.ActionEvent);
            this.configClassMap.set("Role", pattern.Role);
            // this.configClassMap.set("BuildingBase", pattern.BuildingBase);
            this.configClassMap.set("Building", pattern.Building);
            // this.configClassMap.set("Scene", pattern.Scene);
            this.configClassMap.set("Translate", pattern.Translate);
            this.configClassMap.set("Foods", pattern.Foods);
            // this.configClassMap.set("FoodsList", pattern.FoodsList);
            this.configClassMap.set("Task", pattern.Task);
            this.configClassMap.set("TaskList", pattern.TaskList);
            // this.configClassMap.set("FishingBaits", pattern.FishingBaits);
            // this.configClassMap.set("FishingFishes", pattern.FishingFishes);
            // this.configClassMap.set("FishingResult", pattern.FishingResult);
            // this.configClassMap.set("FishingHitZone", pattern.FishingHitZone);
            // this.configClassMap.set("NauticalFoods", pattern.NauticalFoods);
            // this.configClassMap.set("NauticalTools", pattern.NauticalTools);
            // this.configClassMap.set("Sound", pattern.Sound);
            // this.configClassMap.set("ActionPoint", pattern.ActionPoint);
            this.configClassMap.set("CatPhotos", pattern.CatPhotos);
            // this.configClassMap.set("CatToys", pattern.CatToys);
            // this.configClassMap.set("GuideGroup", pattern.GuideGroup);
            // this.configClassMap.set("Guide", pattern.Guide);
            // this.configClassMap.set("Barrage", pattern.Barrage);
            // this.configClassMap.set("BarrageGroup", pattern.BarrageGroup);
            // this.configClassMap.set("Entrustment", pattern.Entrustment);
            // this.configClassMap.set("EntrustmentList", pattern.EntrustmentList);
            // this.configClassMap.set("SpecialGame", pattern.SpecialGame);
            // this.configClassMap.set("SpecialGameGroup", pattern.SpecialGameGroup);
            this.configClassMap.set("TaskMain", pattern.TaskMain);
            // this.configClassMap.set("Kanban", pattern.Kanban);
            // this.configClassMap.set("SpecialGameStrengthRole", pattern.SpecialGameStrengthRole);
            // this.configClassMap.set("SpecialGameStrength", pattern.SpecialGameStrength);
            // this.configClassMap.set("Mail", pattern.Mail);
            // this.configClassMap.set("Staff", pattern.Staff);
            // this.configClassMap.set("StaffSkill", pattern.StaffSkill);
            // this.configClassMap.set("StaffSkillGroup", pattern.StaffSkillGroup);
            // this.configClassMap.set("Sign", pattern.Sign);
            // this.configClassMap.set("NauticalProjectList", pattern.NauticalProjectList);
            // this.configClassMap.set("NauticalProject", pattern.NauticalProject);
            this.configClassMap.set("Milestone", pattern.Milestone);
            this.configClassMap.set("Mission", pattern.Mission);

            // subconfig
            // this.subConfigClassMap.set("CatPhotos", pattern.CatPhotos);
            this.subConfigClassMap.set("Translate", pattern.Translate);
            this.subConfigClassMap.set("Purchase", pattern.Purchase);
        }

        getConfigList() {
            let arr = new Array();
            this.configClassMap.forEach((value , key) => {
                arr.push(key);
            });
            return arr;
        }

        getConfigCount() {
            return this.configClassMap.size;
        }

        getSubConfigMap() {
            return this.subConfigClassMap;
        }

        loadConfig(ConfigName, handler) {
            if(this._configs.has(ConfigName)){
                if(handler){
                    handler.runWith(this._configs.get(ConfigName));
                }
                return;
            }
            let classType = this.configClassMap.get(ConfigName);
            if(classType){
                Laya.loader.load("config/" + ConfigName + ConfigExt, Laya.Handler.create(this, this._onBufferLoaded,[classType, ConfigName, handler]), null, Laya.Loader.BUFFER);
                return;
            } else {
                console.debug("不存在的proto: " + ConfigName + "请在_registClass中注册!");
            }
        }

        loadSubConfig(ConfigName, handler) {
            let classType = this.subConfigClassMap.get(ConfigName);
            if(classType){
                Laya.loader.load("subconfig/" + ConfigName + ConfigExt, Laya.Handler.create(this, this._onBufferLoaded,[classType, ConfigName, handler]), null, Laya.Loader.BUFFER);
                return;
            } else {
                console.debug("不存在的proto: " + ConfigName + "请在_registClass中注册!");
            }
        }

        getConfig(ConfigName) {
            if(this._configs.has(ConfigName)) {
                return this._configs.get(ConfigName);
            }
            this.loadConfig(ConfigName);
            return null;
        }

        getItemById(ConfigName, id) {
            let config = this.getConfig(ConfigName);
            if(config) {
                return config.get(id);
            }
            return null;
        }

        _onBufferLoaded(proto, name, handler, buffer) {
            let config = this._configs.get(name);
            if(config == null)
                config = new Map();
    		let data = new Laya.Byte(buffer);
    		data.endian = Laya.Byte.BIG_ENDIAN;
    		while(data.pos<data.length) {
    			let len = data.readInt16();
    			let array = data.readUint8Array(data.pos, len);
                let item = proto.decode(array); 
                config.set(item.Id, item);
            }
            this._configs.set(name, config);
            if(handler != null) {
                handler.runWith(config);
            }
        }

        insertConfig(name, content) {
            let config = this._configs.get(name);
            if(config == null)
                config = new Map();
            let props = content[0];
            for(var i = 1;i<content.length;i++) {
                let item = Object.create(null);
                let line = content[i];
                for(var j = 0; j<props.length;j++) {
                    item[props[j]] = line[j];
                }
                config.set(item.Id, item);
            }
            this._configs.set(name, config);
        }
    }

    class CatData {

        constructor(configId) {
            this.config = ConfigMgr$1.getInstance().getItemById("Role", String(configId));
            this.catId = ++CatData.catIdMax;
            let foodWeightedMap = new Map();
            for(let i = 0 ;i<this.config.FavorFoodsList.length ; ++i) {
                let c = this.config.FavorFoodsList[i];
                let d = c.split(":");
                foodWeightedMap.set(String(d[1]), Number(d[0]));
            }
            this.foodWeighted = 0;
            this.foodWeightedArray = new Array();
            let outSelf = this;
            foodWeightedMap.forEach(function (value, key) {
                outSelf.foodWeighted += value;
                outSelf.foodWeightedArray.push({c: key, w: Number(value)});
            });
            outSelf.foodWeightedArray.sort(function(a, b) {
                return a.w < b.w;
            });
            this.sceneId = 0;
            this.eventCount = 0;
        }

        setSceneId(sceneId) {
            this.sceneId = sceneId;
        }

        getSceneId() {
            return this.sceneId;
        }

        setParams(params) {
            this.params = params;
        }

        getParams() {
            return this.params;
        }

        randomFood() {
            let ret = Utils.calcRandomWeight(this.foodWeightedArray, this.foodWeighted);
            return ret?ret.c:null;
        }

        getCatId() {
            return this.catId;
        }

        getConfigId() {
            return this.config.Id;
        }

        getSource() {
            return this.config.Source;
        }

        getSkin() {
            return this.config.Skin;
        }

        getHeadIcon() {
            return "cat_collect/head/"+this.config.HeadIcon+".png";
        }

        getEventVisit() {
            return this.config.EventVisit;
        }

        getEventTravel() {
            return this.config.EventTravel;
        }

        getStarBase() {
            return this.config.StarBase;
        }

        isSkillCat() {
            return this.params.skillType ? true : false;
        }

        getDropRate() {
            return JSON.parse(this.config.DropRate);
        }
    }

    CatData.catIdMax = 0;

    class CoffeeCatData extends CatData {
        setPos(pos) {
            this.x = pos.x;
            this.y = pos.y;
        }

        getX(){
            return this.x;
        }

        getY(){
            return this.y;
        }

        pos(x, y) {
            this.x = x;
            this.y = y;
        }
    }

    class BeachCatData extends CatData {

        setEvent(eventString) {
            if(this.params) {
                this.params.event = eventString;
            }else {
                this.params = {"event": eventString};
            }
        }

    }

    class Constants {
    }

    Constants.dev = false;       //上线时候这个要记得改为 false

    Constants.LoginSceneId = 0;

    Constants.CoffeeSceneId = 1;

    Constants.BeachSceneId = 2;

    Constants.NauticalSceneId = 3;

    Constants.HomelandId = 4;

    Constants.FishingSceneId = 20;

    Constants.CoffeeBuildingScale = 0.85;

    Constants.BeachBuildingScale = 0.85;


    Constants.SwitchSceneDist = 200;

    Constants.ppversion = 3;//线上包的版本号  发布前需要修改

    Constants.TaskType = {
        "kDaily": 2,
        "kTrophy": 1
    };

    Constants.TaskStatus = {
        "kRewarded": -1,
        "kCompleted": 1,
        "kRunning": 0
    };

    Constants.RedDotType = {
        "kTask": 100,
        "kCollect": 200,
        "kMail": 300
    };


    Constants.RewardType = {
        "FISH": "FISH",
        "STAR": "STAR",
        "SHELL": "SHELL",
        "DIAMOND": "DIAMOND",
        "MEDAL": "MEDAL",
        "BOOK": "BOOK"
    };

    Constants.FlyType = {
        "NORM": 0,
        "ZCM": 1,
        "LW": 2,
        "QQ": 3,
        "ST": 4,
        "STNORM": 5,
        "ENHANCE": 6,
        "STENHANCE": 7,
        "TYPE1": "1",
        "TYPE2": "2",
        "TYPE3": "3",
        "TYPE4": "4",
        "TYPE5": "5",
    };

    Constants.FetchType = {
        "DIRECT": 0,
        "AD": 1,
        "LOTTERY": 2
    };

    Constants.ItemType = {
        "Currency": "Currency",
        "NauticalFoods": "NauticalFoods",
        "NauticalTools": "NauticalTools",
        "FishingBaits": "FishingBaits",
        "CatPhotos": "CatPhotos",
        "CatToys": "CatToys",
        "Coupon": "Coupon"
    };

    Constants.InteractState = {
        "drag": "drag",
        "petting": "petting",
        "toy": "toy"
    };

    Constants.SyncDelay = 15000;

    Constants.LetterBoxInterval = 60000;

    //一些客户端错误码定义
    Constants.NetError = 7770001;

    Constants.OperatingUpdateCD = 5000;

    Constants.NetReconnectId = -10001;

    Constants.GuideZOrder = 99999;

    //新手引导类型1 对话
    Constants.GuideTypeTalk = "1";

    //新手引导类型2 按钮
    Constants.GuideTypeButton = "2";

    //新手引导类型3 对话按钮
    Constants.GuideTypeTalkButton = "3";

    //新手引导类型4 程序控制
    Constants.GuideTypeSpace = "4";

    //新手引导类型5 高亮区域对话
    Constants.GuideTypeHoleTalk = "5";

    //弹幕颜色
    Constants.BarrageColors = ["#73CBCB", "#F9E5C3", "#FFE675", "#D7F56F", "#9B64B4"];

    //小游戏类型
    Constants.MiniGameType = {
        "MORA": "MORA",
        "COIN": "COIN",
        "STRENGTH": "STRENGTH"
    };

    //shareId
    Constants.ShareId = {
        "FortuneCat": 10,
        "Zhaolan": 11,
        "VisitorGift": 12,
        "Fishing": 13,
        "MoraGame": 14,
        "GuessGame": 15,
        "Task": 16,
        "FoodLevelUp": 17,
        "Building": 18,
        "ComeOn": 19,
        "CatFood": 20,
        "Traveling": 21,
        "FishReward": 22,
        "GuessGameDouble": 23,
        "MoraGameDouble": 24,
        "AdoptPre": 25,
        "PowerGame": 26,
        "PowerGameDouble": 27,
        "LuckyBagDouble": 28,
        "NuticalPhoto": 29,
        "ToyUnlock": 30,
        "AdoptSuccess": 31,
        "CatNewCome": 32,
        "RankShare": 33,//这个先不删了。。。
        "LocalRank": 34,
        "GlobalRank": 35,
        "InviteFriend": 36,
        "StaffSkillId": 37,
        "Balloon": 38,
        "NauticalReward": 39,
        "FriendShare": 40,
        "Bargaining": 41,
        "FishNewCome": 42,
        "Zhaolan_AD_View": 43,
        "FriendZhuli": 44,
        "MileStone": 45,
        'zhuliBaoxiangAward': 46,
        'Housekeeper': 47,
        "Luckydraw": 48,
        "guideBowl": 49,
        "SailUnlock": 50,
        "GuessGameMore": 51,
        "BuildingCoupon": 52,
        "MissionCatCD": 53,
        "MissionADRefresh": 54,
    };

    Constants.PeacockShareType = {
        "ToyUnlock": 1,
        "CatNewCome": 2,
        "AdoptSuccess": 3,
        "FishNewCome": 4
    };

    Constants.DialogStatus = {
        "BottomUI" : 1,
        "Sign" : 2,
        "HomeLand" : 3,
        "Fishing" : 4,
        "TaskMain" : 5,
        "ForturneCat" : 6,
        "MileStone" : 7,
        "CatNewcome" : 8,
        "VisitorGift" : 9,
        "Photo" : 10,
        "BuildingCouponPop" : 11,
        "PayRewards" : 12
    };

    Constants.BannerId = {
        "Task": 100,
        "CatFood": 101
    };

    Constants.MiniGameResult = {
        "Fail": 0,
        "Draw": 1,
        "Win": 2,
        "KO": 3
    };

    Constants.HomeRubbishType = {
        "Rubbush_1": 0,
        "Rubbush_2": 1,
        "Rubbush_3": 2
    };

    Constants.HealthyType = {
        "HUNGER": 1,
        "THIRST": 2,
        "DIRTY": 3,
    };

    Constants.WxMessageTmplIdKeys = {
        "ENTRUST": "ENTRUST",
        "FORTUNE_CAT": "FORTUNE_CAT",
        "CAT_FOOD": "CAT_FOOD",
        "NAUTICAL_START": "NAUTICAL_START",
        "FISHING": "FISHING",
    };

    Constants.WxMessageType = {
        "INTERACTIVE": "INTERACTIVE",
        "RANK": "RANK"
    };

    Constants.TipAnimType = {
        "GAILIANG": "success",
        "JIASU": "success",
        "SHENGJI": "success",
        "SHENGJIE": "success",
        "YANJIU": "success",
        "GOUMAI": "success",
        "SHENGXING": "success",
        "ADOPT": "success",
        "LEVELUP": "success",
        "INTIMACYUP": "intimacyup"
    };

    Constants.PotentialPic = {
        "A": "common/potential_a.png",
        "S": "common/potential_s.png",
        "SS": "common/potential_ss.png",
        "SSS": "common/potential_sss.png"
    };

    Constants.CouponType = {
        "BUILD": "BUILD",
        "FISHING": "FISHING",
        "SPEED": "SPEED"
    };

    class CatDataMgr {

        constructor() {
            this.catMap = new Map();
        }

        static getInstance() {
            return CatDataMgr.instance?CatDataMgr.instance:CatDataMgr.instance = new CatDataMgr();
        }

        //添加一个猫咪数据
        addCatData(configId, sceneId) {
            let catData = null;
            if(sceneId==Constants.CoffeeSceneId) {
                catData = new CoffeeCatData(configId);
            }else if(sceneId==Constants.BeachSceneId){
                catData = new BeachCatData(configId);
            }

            this.catMap.set(catData.getCatId(), catData);
            catData.setSceneId(sceneId);
            return catData;
        }

        //获取当前猫咪的个数
        getCatCountBySceneId(sceneId) {
            let ret = 0;
            this.catMap.forEach((value, key) => {
                if(value.getSceneId() == sceneId) {
                    ret++;
                }
            });
            return ret;
        }

        getCatsBySceneId(sceneId) {
            let ret = new Array;
            this.catMap.forEach((value, key) => {
                if(value.getSceneId() == sceneId) {
                    ret.push(value);
                }
            });
            return ret;
        }

        removeCatData(catId) {
            this.catMap.delete(catId);
        }

        getCatData(id) {
            return this.catMap.get(id);
        }


        dump() {
            let scene1 = new Array();
            let scene2 = new Array();
            this.catMap.forEach((value, key) => {
                if(value.getSceneId() == 1) {
                    if(value.eventCount==0){
                        scene1.push({"id": value.getConfigId(), "params": value.getParams()});
                    }
                }else if(value.getSceneId() == 2) {
                    if(value.eventCount==0){
                        scene2.push({"id": value.getConfigId(), "params": value.getParams()});
                    }
                }
            });
            let customData = {"scene1": scene1, "scene2": scene2};
            return customData;
        }

        clear() {
            this.catMap.clear();
        }

    }

    class GraphMatrix {
        constructor(matrix) {
            this._matrix = matrix;
        }

        getRelatedNodes(node) {
            let line = this._matrix[node];
            if(line){
                let arr = new Array();
                for(let i = 0; i<line.length; ++i) {
                    if(line[i]==1&&i!=node)
                        arr.push(i);
                }
                return arr;
            }
        }
    }

    class SearchPath {

        static isNodeInStack(node, stack) {
            for(const n of stack) {
                if(n==node)
                    return true;
            }
            return false;
        }

        //存储路径
        static savePath(stack, paths) {
            paths.push(stack.slice(0));
        }

        //寻找路径
        //cNode 当前的起始节点
        //pNode cNode的上一个节点
        //sNode 最初的起始节点
        //eNode 终点
        static getPaths(cNode, pNode, sNode, eNode, graph, stack, paths) {
            //如果符合条件说明出现环路，不能继续，返回false
            if(cNode!=null && pNode!=null && cNode==pNode)
                return false;
            if(cNode!=null) {
                let i = 0;
                stack.push(cNode);
                if(cNode==eNode) {//找到路径
                    SearchPath.savePath(stack, paths);
                    return true;
                }else {//继续寻路
                    let relatedNodes = graph.getRelatedNodes(cNode);
                    /*
                    * 如果nNode是最初的起始节点或者nNode就是cNode的上一节点或者nNode已经在栈中 ，
                    * 说明产生环路 ，应重新在与当前起始节点有连接关系的节点集中寻找nNode
                    */
                    for(let nNode of relatedNodes) {
                        if(pNode != null && (nNode==sNode||nNode==pNode||SearchPath.isNodeInStack(nNode, stack))) {
                            continue;
                        }
                        if(SearchPath.getPaths(nNode, cNode, sNode, eNode, graph, stack, paths))
                            stack.pop();//如果找到一条路径，就弹出栈顶节点
                    }
                    /*
                    * 当遍历完所有与cNode有连接关系的节点后，
                    * 说明在以cNode为起始节点到终点的路径已经全部找到
                    */
                    stack.pop();
                    return false;
                }
            } else {
                return false;
            }
        }
    }

    //0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14
    let matrix1 = [[0, 1, 1],//0
                   [1, 0, 0, 0, 0, 0, 0, 0, 1],//1
                   [1, 0, 0, 1, 0, 0, 0, 0, 0],//2
                   [0, 0, 1, 0, 1, 0, 1, 1, 0],//3
                   [0, 0, 0, 1, 0, 1, 0, 0, 0],//4
                   [0, 0, 0, 0, 1, 0, 1, 0, 0],//5
                   [0, 0, 0, 1, 0, 1, 0, 1, 0],//6
                   [0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1],//7
                   [0, 1, 0, 0, 0, 0, 0, 0, 0, 1],//8
                   [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0],//9
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0],//10
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1],//11
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1],//12
                   [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1],//13
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]];//14

                 //16,17,18,19,20,21,22
    let matrix2 = [[0, 1, 1],//16
                   [1, 0, 1],//17
                   [1, 1, 0, 1, 0, 1, 0],//18
                   [0, 0, 1, 0, 1, 0, 0],//19
                   [0, 0, 0, 1, 0, 1, 0],//20
                   [0, 0, 0, 0, 1, 0, 1],//21
                   [0, 0, 0, 0, 0, 1, 0]];//22

                  //0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14
    let fMatrix1 =[[0, 1, 1],//0
                   [1, 0, 1, 0, 0, 0, 0, 0, 1],//1
                   [1, 0, 0, 1, 0, 0, 0, 0, 0],//2
                   [0, 1, 1, 0, 1, 0, 1, 1, 0, 1],//3
                   [0, 0, 0, 1, 0, 1, 0, 0, 0],//4
                   [0, 0, 0, 0, 1, 0, 1, 0, 0],//5
                   [0, 0, 0, 1, 0, 1, 0, 1, 0, 1],//6
                   [0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1],//7
                   [0, 1, 0, 1, 0, 0, 0, 0, 0, 1],//8
                   [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0],//9
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1],//10
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 13],//11
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1],//12
                   [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1],//13
                   [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]];//14

                 //16,17,18,19,20,21,22
    let fMatrix2 =[[0, 1, 1],//16
                   [1, 0, 1, 0, 0, 0, 1],//17
                   [1, 1, 0, 1, 0, 1, 0],//18
                   [0, 0, 1, 0, 1, 1, 0],//19
                   [0, 0, 0, 1, 0, 1, 0],//20
                   [0, 0, 1, 1, 1, 0, 0],//21
                   [0, 0, 0, 0, 0, 1, 0]];//22


    class ScenePathMgr {

        constructor() {
            this._graph1 = new GraphMatrix(matrix1);
            this._graph2 = new GraphMatrix(matrix2);
            this._fastGraph1 = new GraphMatrix(fMatrix1);
            this._fastGraph2 = new GraphMatrix(fMatrix2);
            this.scene1BasePath = new Array();
            this.scene1ExtPath0 = new Array();
            this.scene1ExtPath1 = new Array();
            this.scene1ExtPath2 = new Array();
            this.scene1ExtPath3 = new Array();
            this.scene1StaffPath = new Array();
        }

        static getInstance() {
            return ScenePathMgr.instance?ScenePathMgr.instance:ScenePathMgr.instance = new ScenePathMgr();
        }


        isZOrderPathIndex(index) {
            if(index == 14 || index == 16 || index == 22)
                return true;
            return false;
        }

        getStartPathPoint() {
            return this.scene1BasePath[0];
        }

        _excludePathOverNode(paths, node1, node2) {
            let res = new Array();
            for(const path of paths) {
                let exclude = false;
                for(let i = 1; i<path.length-1; ++i) {
                    if ((node1 && path[i] == node1) || (node2 && path[i] == node2) || path[i] == 0) {
                        exclude = true;
                        break;
                    }
                }
                if(!exclude)
                    res.push(path);
            }
            return res;
        }

        _dealWithPath1(path) {
            let res = new Array();
            for(let i = 0; i<path.length - 1; ++i) {
                res.push(this.getBaseNode(path[i]));
                if(path[i]==0&&path[i+1]==1) {
                    this._insertExt([0,1,3,6,4,-1], res);
                }else if(path[i]==0&&path[i+1]==2) {
                    this._insertExt([2,4,5,-1], res);
                }else if(path[i]==3&&path[i+1]==4) {
                    this._insertExt([11,-1], res);
                }else if(path[i]==3&&path[i+1]==6) {
                    this._insertExt([10,11,12,-1], res);
                }else if(path[i]==3&&path[i+1]==7) {
                    this._insertExt([10,15,12,-1], res);
                }else if(path[i]==8&&path[i+1]==9) {
                    this._insertExt([10,13,14,15,-1], res);
                }else if((path[i]==12||path[i]==13)&&path[i+1]==14) {
                    this._insertExt([20,21,-1], res);
                }
            }
            res.push(this.getBaseNode(path[path.length-1]));
            return res;
        }

        findPathInMatrix1(start, end) {
            let stack = new Array();
            let paths = new Array();
            SearchPath.getPaths(start, null, start, end, this._graph1, stack, paths);
            paths = this._excludePathOverNode(paths, 0, 14);
            let path = paths[Utils.randomExt(paths.length)];
            path = this._dealWithPath1(path);
            return path;
        }

        _insertExt(exts, res) {
            let ext = exts[Utils.randomExt(exts.length)];
            if(ext>=0)
                res.push(this.getExtNode(ext));
        }

        _dealWithPath2(path) {
            for(let i = 0; i<path.length; ++i) {
                path[i] = path[i] + 16;
            }
            let res = new Array();
            for(let i = 0; i<path.length - 1; ++i) {
                res.push(this.getBaseNode(path[i]));
                if(path[i]==18&&path[i+1]==19) {
                    this._insertExt([30,31,32], res);
                }else if(path[i]==20&&path[i+1]==21) {
                    this._insertExt([33,34], res);
                    this._insertExt([35,36], res);
                }else if(path[i]==18&&path[i+1]==21) {
                    this._insertExt([30,37], res);
                }else if(path[i]==21&&path[i+1]==22) {
                    this._insertExt([37,38, -1], res);
                }
            }
            res.push(this.getBaseNode(path[path.length-1]));
            return res;
        }

        findPathInMatrix2(start, end) {
            let stack = new Array();
            let paths = new Array();
            SearchPath.getPaths(start-16, null, start-16, end-16, this._graph2, stack, paths);
            paths = this._excludePathOverNode(paths, 0, 14);
            let path = paths[Utils.randomExt(paths.length)];
            path = this._dealWithPath2(path);
            return path;
        }

        getBaseNode(i) {
            return this.scene1BasePath[i];
        }

        getStaffNode(i) {
            return this.scene1StaffPath[i];
        }

        getExtNode(i) {
            let decimal = Math.floor(i/10);
            let unit = i%10;
            switch(decimal) {
                case 0:
                    return this.scene1ExtPath0[unit];
                case 1:
                    return this.scene1ExtPath1[unit];
                case 2:
                    return this.scene1ExtPath2[unit];
                case 3:
                    return this.scene1ExtPath3[unit];
            }
        }

        randomPath(from, to) {
            if(from<15&&to<15) {
                // if (this.belongDifferent(from, to)) {
                //     return this.findPathInMatrix1(from, 14).concat(this.findPathInMatrix2(16, 22).concat(this.findPathInMatrix1(14, to)));
                // }
                return this.findPathInMatrix1(from, to);
            }else if(from>15&&to>15) {
                return this.findPathInMatrix2(from, to);
            }else if(from<15&&to>15) {
                return this.findPathInMatrix1(from, 14).concat(this.findPathInMatrix2(16, to));
            }
            else if(from>15&&to<15) {
                return this.findPathInMatrix2(from, 22).concat(this.findPathInMatrix1(14, to));
            }
        }

        // belongDifferent(path1, path2) {
        //     return (path1 >= 2 && path1 <= 7 || path1 == 13) ^ (path2 >= 2 && path2 <= 7 || path2 == 13) && (path1 != 0 && path2 != 0);
        // }

        load() {
            Laya.loader.create("prefab/path/Scene1Path.prefab", Laya.Handler.create(this, function(prefabDef) {
                let prefab = prefabDef.create();
                //base path
                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("b"+String(i));
                    if(c != null) {
                        this.scene1BasePath.push({i: i, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }
                //ext path
                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("p0"+String(i));
                    if(c != null) {
                        this.scene1ExtPath0.push({i: -1, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }

                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("p1"+String(i));
                    if(c != null) {
                        this.scene1ExtPath1.push({i: -1, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }
                
                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("p2"+String(i));
                    if(c != null) {
                        this.scene1ExtPath2.push({i: -1, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }
                
                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("p3"+String(i));
                    if(c != null) {
                        this.scene1ExtPath3.push({i: -1, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }
                //staff path
                for(let i = 0 ; i < 100; ++i) {
                    let c = prefab.getChildByName("s"+String(i));
                    if(c != null) {
                        this.scene1StaffPath.push({i: i, x: c.x, y: c.y});
                    } else {
                        break;
                    }
                }
            }));
        }

        randomFastPath(from, to) {
            if(from<15&&to<15) {
                return this.findFastPathInMatrix1(from, to);
            }else if(from>15&&to>15) {
                return this.findFastPathInMatrix2(from, to);
            }else if(from<15&&to>15) {
                return this.findFastPathInMatrix1(from, 14).concat(this.findFastPathInMatrix2(16, to));
            }
            else if(from>15&&to<15) {
                return this.findFastPathInMatrix2(from, 22).concat(this.findFastPathInMatrix1(14, to));
            }
        }

        findFastPathInMatrix1(start, end) {
            let stack = new Array();
            let paths = new Array();
            SearchPath.getPaths(start, null, start, end, this._fastGraph1, stack, paths);
            paths = this._excludePathOverNode(paths, 0, 14);
            let path = this.getShortestPath(paths);
            path = this._dealWithFastPath1(path);
            return path;
        }

        findFastPathInMatrix2(start, end) {
            let stack = new Array();
            let paths = new Array();
            SearchPath.getPaths(start - 16, null, start - 16, end - 16, this._fastGraph2, stack, paths);
            paths = this._excludePathOverNode(paths, 0, 14);
            let path = this.getShortestPath(paths);
            path = this._dealWithFastPath2(path);
            return path;
        }

        getShortestPath(paths) {
            let id = 0;
            for (let index = 0; index < paths.length; index++) {
                const path = paths[index];
                if (path.length < paths[id].length) {
                    id = index;
                }
            }
            return paths[id];
        }

        _dealWithFastPath1(path) {
            let res = new Array();
            for (let i = 0; i < path.length; ++i) {
                res.push(this.getStaffNode(path[i]));
            }
            return res;
        }

        _dealWithFastPath2(path) {
            for (let i = 0; i < path.length; ++i) {
                path[i] = path[i] + 16;
            }
            let res = new Array();
            for (let i = 0; i < path.length; ++i) {
                res.push(this.getStaffNode(path[i]));
            }
            return res;
        }

    }

    class TimeUtils {

        //服务器上的毫秒时间
        static getTimeNowOnServer() {
            return (+ new Date()) + this.serverTsModify+this.timeOffSet;
        }

        static getTimeLocal() {
            return (+ new Date());
        }

        //TODO 临时写的读取服务器时间戳, 这里以后要优化一下，因为时间戳会有传输延迟
        static updateServerTime(serverTsHigh, serverTsLow) {
            let serverTs = (serverTsHigh * 4294967296) + serverTsLow;
            //先计算出本次传输的 delay，如果 delay 计算的过大，则直接认为是系统调时间了，清零之前的累计delay，重新开始调整
            //暂定超过2s，认为是调时间了
            let nowServerTs = this.getTimeNowOnServer();
            let delayOnce = nowServerTs - serverTs;
            if(Math.abs(delayOnce) < 2000) {
                this.averageDelay = Math.trunc((this.averageDelay * this.calcCount + delayOnce) / (this.calcCount + 1));
                this.calcCount++;
                //如果 calcCount 太大了，适当取小了再计算
                if(this.calcCount > 1000) this.calcCount = 100;
            } else {
                this.averageDelay = 50;
                this.calcCount = 0;
            }

            this.serverTsModify = serverTs + this.averageDelay - (+ new Date());
        }
        static setLoginTime(timeOffSet){
            if(timeOffSet){
                this.timeOffSet=timeOffSet;
            }
        }

        static getServerTimeModify() {
            return this.serverTsModify + this.timeOffSet;
        }
    }

    TimeUtils.serverTsModify = 0;
    TimeUtils.calcCount = 0;
    //计算服务器的平均延迟，初始时假定为50ms
    TimeUtils.averageDelay = 50;
    TimeUtils.timeOffSet=0;

    class BaseCatMgr {
        constructor() {
            this._catMap = new Map();
            this._removingCat = new Map();
        }

        addCat(cat) {
            this._catMap.set(cat.getId(), cat);
        }


        removeCat(catId) {
            let cat = this._catMap.get(catId);
            this._catMap.delete(catId);
            if(cat) {
                this._removingCat.set(catId, cat);
            }
        }

        finallyRemove(catId) {
            let cat = this._catMap.get(catId);
            cat = cat ? cat : this._removingCat.get(catId);
            this._catMap.delete(catId);
            this._removingCat.delete(catId);
            if(cat)
                cat.destroy();
        }

        clearRemoved() {
            for(const [catId, cat] of this._removingCat) {
                if(cat)
                    cat.destroy();
            }
            this._removingCat.clear();
        }

        _getMap() {
            // for(const [key, cat] of this._catMap) {
            //     if(cat.destroyed) {
            //         this._catMap.set(key, cat.clone());
            //     }
            // }
            return this._catMap;
        }

        allCats() {
            let result = new Array();
            for(const [key, cat] of this._getMap()) {
                if(cat && !cat.destroyed)
                    result.push(cat);
            }
            return result;
        }

        getCat(catId) {
            let cat = this._catMap.get(catId);
            return cat;
        }

        clear() {
            for(const [catId, cat] of this._catMap) {
                if(cat)
                    cat.destroy();
            }
            this._catMap.clear();
            for(const [catId, cat] of this._removingCat) {
                if(cat)
                    cat.destroy();
            }
            this._removingCat.clear();
        }
    }

    class CoffeeCatMgr extends BaseCatMgr {

        static getInstance() {
            return CoffeeCatMgr.instance?CoffeeCatMgr.instance:CoffeeCatMgr.instance = new CoffeeCatMgr();
        }

        catsInBuilding(buildingId) {
            let result = new Array();
            for(const [key, cat] of this._getMap()) {
                if(cat.buildingId==buildingId && !cat.destroyed){
                    result.push(cat);
                }
            }
            return result;
        }

        clear() {
            super.clear();
        }
    }

    class BeachCatMgr extends BaseCatMgr {

        static getInstance() {
            return BeachCatMgr.instance?BeachCatMgr.instance:BeachCatMgr.instance = new BeachCatMgr();
        }

        idleCatsInBuilding(buildingId) {
            let result = new Array();
            for(const [key, cat] of this._getMap()) {
                if(cat.buildingId==buildingId && cat.parent == null && !cat.destroyed){
                    result.push(cat);
                }
            }
            return result;
        }

        clear() {
            super.clear();
        }
    }

    class Const {

        constructor() {
        }

        //获得原始数据
        static get(string_id) {
           let config = ConfigMgr$1.getInstance().getItemById("Const", String(string_id));
           return config?config.Value:null;
        }

        //获得数组
        static getArray(string_id) {
            let d = Const.get(string_id);
            if(d) {
                let ret = JSON.parse(d);
                return ret;
            }else {
                return null;
            }
        }

        //获得数组
        static getMap(string_id) {
            let d = Const.get(string_id);
            if(d) {
                let ret = JSON.parse(d);
                return ret;
            }else {
                return null;
            }
        }

    }

    //CatDataMgr 和 ScenePathMgr 都放到CatSystem里处理
    class CatSystem {
        static init() {
            EventMgr.getInstance().registEvent("CUSTOMDATA_CAT_RESTORE", null, CatSystem.restore);
            EventMgr.getInstance().registEvent("REMOVE_CAT", null, CatSystem.removeCatData);
        }

        static clear() {
            CoffeeCatMgr.getInstance().clear();
            BeachCatMgr.getInstance().clear();
            CatDataMgr.getInstance().clear();
        }

        //添加猫咪数据
        static addCatData(configId, sceneId, params) {
            let catData = CatDataMgr.getInstance().addCatData(configId, sceneId);
            catData.setParams(params);
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [sceneId, "ADD_CAT", catData.getCatId()]);
            return catData;
        }

        static getCatsBySceneId(sceneId) {
            return CatDataMgr.getInstance().getCatsBySceneId(sceneId);
        }

        static getCatCountBySceneId(sceneId) {
            return CatDataMgr.getInstance().getCatCountBySceneId(sceneId);
        }

        static removeCatData(catId, sceneId) {
            let catData = CatDataMgr.getInstance().removeCatData(catId, sceneId);
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [sceneId, "REMOVE_CAT", catId]);
        }

        static getCatData(id) {
            return CatDataMgr.getInstance().getCatData(id);
        }

        static restore(catsData, lastTime, scene2ExhaustTime) {
            EventMgr.getInstance().postEvent("SIMULATOR_CREATE_SESSION", Constants.CoffeeSceneId);
            EventMgr.getInstance().postEvent("SIMULATOR_CREATE_SESSION", Constants.BeachSceneId);
            console.debug("restore cats", catsData);
            if(catsData&&catsData.scene1){
                catsData.scene1.forEach(value=>{
                    // console.debug("restore", value)
                    CatSystem.addCatData(value.id, Constants.CoffeeSceneId, value.params);
                });
            }
            let now = TimeUtils.getTimeNowOnServer();
            if(lastTime && lastTime >0) {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.BeachSceneId, "SET_TIME", lastTime]);
            }else {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.BeachSceneId, "SET_TIME", now]);
            }
            if(catsData&&catsData.scene2){
                catsData.scene2.forEach(value=>{
                    CatSystem.addCatData(value.id, Constants.BeachSceneId, value.params);
                });
            }

            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.BeachSceneId, "REFRESH_FOOD", scene2ExhaustTime]);
            if(lastTime && lastTime >0) {

                let simulateTime = now - lastTime;
                if(scene2ExhaustTime!=null && now > Math.max(scene2ExhaustTime, lastTime)+Number(Const.get("CatFoodFinishCheckTime"))*1000) {
                    simulateTime = Math.max(scene2ExhaustTime, lastTime)+Number(Const.get("CatFoodFinishCheckTime"))*1000 - lastTime;
                }

                EventMgr.getInstance().postEvent("SIMULATOR_SIMULATE_SESSION", [Constants.BeachSceneId, simulateTime]);
            }
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.BeachSceneId, "SET_TIME", now]);
            EventMgr.getInstance().postEvent("SIMULATOR_RUN_SESSION", Constants.CoffeeSceneId);
            EventMgr.getInstance().postEvent("SIMULATOR_RUN_SESSION", Constants.BeachSceneId);
        }

    }

    class CommonUIMgr {

        //管理各个场景中通用的UI部分，目前包括 TopUI、BottomUINew、LeftUI、ArrowUI、LeftBtnDraw

        constructor() {
            this.topUI = null;
            this.bottomUI = null;
            this.leftUI = null;
            this.arrowUI = null;
            this.rightUI = null;
            this.leftBtnDraw = null;
        }

        static getInstance() {
            return CommonUIMgr.instance?CommonUIMgr.instance:CommonUIMgr.instance = new CommonUIMgr();
        }

        hideAll() {
            if(this.topUI)
                this.topUI.visible = false;
            if(this.bottomUI)
                this.bottomUI.visible = false;
            if(this.leftUI)
                this.leftUI.visible = false;
            if(this.arrowUI)
                this.arrowUI.visible = false;
            if(this.rightUI)
                this.rightUI.visible = false;
            if(this.leftBtnDraw)
                this.leftBtnDraw.visible = false;    
        }

        showCommonUI(sceneId, handler, args) {
            let handlerId = handler ? handler._id : null;
            let completeHandler = Laya.Handler.create(this, function () {
                if (this.topUI != null && this.bottomUI != null && this.leftUI != null && this.arrowUI != null && this.leftBtnDraw != null) {
                    if (handler != null) {
                        if (handlerId == handler._id) {
                            handler.runWith(args);
                        } else {
                            console.warn("handler_Id_Change", handlerId, handler._id, handlerId.caller);
                            // handler.recover();
                        }
                    }
                }
            }, null, false);

            if(sceneId == Constants.CoffeeSceneId) {
                this.showTopUI(sceneId,completeHandler);
                this.showBottomUI(sceneId,completeHandler);
                this.showLeftUI(sceneId,completeHandler);
                this.showArrowUI(sceneId, completeHandler);
                this.showRightUI(sceneId, completeHandler);
                this.showLeftBtnDraw(sceneId, completeHandler);
            } else if(sceneId == Constants.BeachSceneId) {
                this.showTopUI(sceneId,completeHandler);
                this.showBottomUI(sceneId);
                this.showLeftUI(sceneId);
                this.showArrowUI(sceneId, completeHandler);
                this.showRightUI(sceneId, completeHandler);
                this.showLeftBtnDraw(sceneId, completeHandler);
            } else if(sceneId == Constants.NauticalSceneId) {
                this.showTopUI(sceneId,completeHandler);
                this.showBottomUI(sceneId,completeHandler);
                this.showLeftUI(sceneId,completeHandler);
                this.showArrowUI(sceneId, completeHandler);
                this.showRightUI(sceneId, completeHandler);
                this.showLeftBtnDraw(sceneId, completeHandler);
            }else if(sceneId == Constants.HomelandId) {
                this.showTopUI(sceneId,completeHandler);
                this.hideBottomUI();
                this.showLeftUI(sceneId);
                this.hideArrowUI();
                this.hideRightUI();
                this.showLeftBtnDraw(sceneId, completeHandler);
            }else if (sceneId == -1){
                this.showTopUI(sceneId,completeHandler);
                this.showBottomUI(Constants.CoffeeSceneId,completeHandler);
                this.showLeftUI(sceneId,completeHandler);
                this.showArrowUI(Constants.CoffeeSceneId, completeHandler);
                this.hideRightUI();
                this.showLeftBtnDraw(sceneId, completeHandler);
            } else if (sceneId == Constants.FishingSceneId) {
                this.showTopUI(sceneId);
                this.hideBottomUI();
                this.hideArrowUI();
                this.hideLeftUI();
                this.hideRightUI();
                this.hideLeftBtnDraw();
                if (handler) {
                    handler.runWith(args);
                }
            }else {
                this.hideBottomUI();
                this.hideArrowUI();
                this.hideLeftUI();
                this.hideRightUI();
                this.hideLeftBtnDraw();
                if(handler) {
                    handler.runWith(args);
                }
            }
        }

        showTopUI(sceneId, completeHandler) {
            if (this.topUI != null) {
                this.topUI.visible = true;
                if (completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                this.loadResource("topUI", "prefab/ui/TopUI.prefab", completeHandler);
            }
            EventMgr.getInstance().postEvent("SHOW_BEIKE");
        }

        hideTopUI() {
            if(this.topUI != null) this.topUI.visible = false;
        }

        showBottomUI(sceneId,completeHandler) {
            if(this.bottomUI != null) {
                this.bottomUI.visible = true;
                this.bottomUI.showBtn(sceneId);
                if(completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                this.loadResource("bottomUI", "prefab/ui/BottomUI.prefab", completeHandler);
            }
        }

        hideBottomUI() {
            if(this.bottomUI != null) this.bottomUI.visible = false;
        }

        showLeftUI(sceneId,completeHandler) {
            if(this.leftUI != null) {
                this.leftUI.visible = true;
                let script = this.leftUI.getComponent(Laya.Script);
                // script.showBtn(sceneId);
                if(completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                this.loadResource("leftUI", "prefab/ui/LeftUI.prefab", completeHandler);
            }
        }

        hideLeftUI() {
            if(this.leftUI != null) this.leftUI.visible = false;
        }

        showArrowUI(sceneId, completeHandler) {
            if(this.arrowUI != null) {
                this.arrowUI.visible = true;
                EventMgr.getInstance().postEvent("SWITCH_ARROW_BTN", [sceneId]);
                if(completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                setTimeout(() => {
                    this.loadResource("arrowUI", "prefab/ui/ArrowUI.prefab", completeHandler, sceneId);
                },100);
            }
        }

        hideArrowUI() {
            if(this.arrowUI != null) this.arrowUI.visible = false;
        }

        showRightUI(sceneId, completeHandler) {
            if(this.rightUI != null) {
                this.rightUI.visible = true;
                if(completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                setTimeout(() => {
                    this.loadResource("rightUI", "prefab/ui/RightUI.prefab", completeHandler, sceneId);
                },100);
            }
        }

        hideRightUI() {
            if(this.rightUI != null) this.rightUI.visible = false;
        }

        showLeftBtnDraw(sceneId, completeHandler) {
            if (this.leftBtnDraw != null) {
                this.leftBtnDraw.visible = true;
                if (completeHandler != null) {
                    completeHandler.run();
                }
            } else {
                this.loadResource("leftBtnDraw", "prefab/ui/LeftBtnDraw.prefab", completeHandler);
            }
        }

        hideLeftBtnDraw() {
            if(this.leftBtnDraw != null){
                this.leftBtnDraw.visible = false;
            } 
        }

        loadResource(name, path, completeHandler, args) {
            if(path.indexOf(".prefab") != -1) {
                Laya.loader.create(path, Laya.Handler.create(this, function(prefabDef) {
                    if(name == "topUI") {
                        this.topUI = prefabDef.create();
                        this.topUI.x = (Laya.stage.width - this.topUI.width) / 2;
                        this.topUI.y = 0;
                        this.topUI.zOrder = 2;
                        Laya.stage.addChild(this.topUI);
                    } else if(name == "bottomUI") {
                        this.bottomUI = prefabDef.create();
                        this.bottomUI.x = (Laya.stage.width - this.bottomUI.width) / 2;
                        this.bottomUI.y = 2128;
                        this.bottomUI.zOrder = 1;
                        Laya.stage.addChild(this.bottomUI);
                    } else if(name == "leftUI") {
                        this.leftUI = prefabDef.create();
                        this.leftUI.x = (Laya.stage.width - 1132) / 2;
                        this.leftUI.y = 0;
                        this.leftUI.zOrder = 1;
                        Laya.stage.addChild(this.leftUI);
                    } else if(name == "arrowUI") {
                        this.arrowUI = prefabDef.create();
                        this.arrowUI.x = 0;
                        this.arrowUI.y = 0;
                        this.arrowUI.zOrder = 1;
                        Laya.stage.addChild(this.arrowUI);
                        EventMgr.getInstance().postEvent("SWITCH_ARROW_BTN", [args]);
                    } else if(name == "rightUI") {
                        this.rightUI = prefabDef.create();
                        this.rightUI.x = Laya.stage.width-this.rightUI.width;
                        this.rightUI.y = 0;
                        this.rightUI.zOrder = 1;
                        Laya.stage.addChild(this.rightUI);
                    }else if(name == "leftBtnDraw") {
                        this.leftBtnDraw = prefabDef.create();
                        this.leftBtnDraw.x = 0;
                        this.leftBtnDraw.y = 0;
                        this.leftBtnDraw.zOrder = 500;
                        Laya.stage.addChild(this.leftBtnDraw);
                    }
                     else {
                        return;
                    }
                    if(completeHandler != null) {
                        completeHandler.run();
                    }
                }));
            }
        }

        clear() {
            if(this.topUI) {
                this.topUI.destroy();
                this.topUI = null;
            }
            if(this.bottomUI) {
                this.bottomUI.destroy();
                this.bottomUI = null;
            }
            if(this.leftUI) {
                this.leftUI.destroy();
                this.leftUI = null;
            }
            if(this.arrowUI) {
                this.arrowUI.destroy();
                this.arrowUI = null;
            }
            if(this.rightUI) {
                this.rightUI.destroy();
                this.rightUI = null;
            }
        }
    }

    class PlayerData {

        constructor() {
            this._fish = -1;
            this._star = -1;
            this._shell=-1;
            this._diamond = -1;
            this._registerDays = 0;
            this._medal = -1;
            this._book = -1;
        }

        update(data) {
            this.updateFish(data.fish);
            this.updateStar(data.star);
            this.updateBeike(data.shell);
        }

        updateFish(fish) {
            this._fish = Number(fish);
        }

        updateStar(star) {
            this._star = Number(star);
            // if (Laya.Browser.onMiniGame) {
            //     wx.setUserCloudStorage({
            //       KVDataList: [{ key: 'STAR', value: String(this._star) }],
            //       success: res => {
            //           console.debug("上传开放域成功:");
            //           console.debug(res);
            //       },
            //       fail: res => {
            //         console.debug("上传开放域失败:");
            //         console.debug(res);
            //       }
            //     });
            //   }
        }
        updateShell(shell) {
            this._shell=  Number(shell);
        }

        updateDiamond(diamond) {
            this._diamond = Number(diamond);
        }

        updateMedal(value) {
            this._medal = Number(value);
        }

        updateBook(value) {
            this._book = Number(value);
        }

        getStar() { return this._star;}

        getFish() {  return this._fish;}

        getShell() { return this._shell; }
        
        getDiamond() { return this._diamond; }
        
        getMedal() {  return  this._medal;}
        
        getBook() { return this._book; }

        setRegisterDays(days) {
            this._registerDays = days;
        }

        getRegisterDays() {
            return this._registerDays;
        }

        clear() {
            this._fish = -1;
            this._star = -1;
            this._shell = -1;
            this._diamond = -1;
            this._registerDays = 0;
            this._medal = -1;
            this._book = -1;
        }
    }

    let star_count_events = [10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 120, 150, 200, 300, 400, 500, 700, 900, 1200, 1500, 2000, 2500,
                            3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500, 8000, 8500, 9000,
                            9500, 10000, 11000, 12000, 13000, 14000, 15000, 16000, 17000, 18000, 19000, 20000, 22000, 24000, 26000, 28000, 30000];
    //DataMgr都应该只包含ConfigMgr和对应的Data
    class PlayerDataMgr {

        constructor() {
            this._player = new PlayerData();
        }

        static getInstance() {
            return PlayerDataMgr.instance?PlayerDataMgr.instance:PlayerDataMgr.instance = new PlayerDataMgr();
        }

        getFish() {
            return this._player.getFish() > 0 ? this._player.getFish() : 0;
        }

        updateFish(fish) {
            let beforFish = this.getFish();
            this._player.updateFish(fish);
            if (beforFish != fish) {
                EventMgr.getInstance().postEvent("FISH_REALLY_CHANGE", [fish]);
            }
        }

        getStar() {
            return this._player.getStar() > 0 ? this._player.getStar() : 0;
        }

        updateStar(star) {
            let prevStar = this._player.getStar();
            if(prevStar==-1 && star==0) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["bell_0"]);
            }else if(star > 0 && prevStar!=-1){
                for( const count of star_count_events) {
                    if(prevStar<count && star >= count) {
                        console.log('增加新星++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++',"bell_" + String(count));
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["bell_" + String(count),null,1]);
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["bell_" + String(count)]);
                    }else if(star < count) {
                        break;
                    }
                }
            }
            let beforStar = this.getStar();
            this._player.updateStar(star);
            if (star != beforStar) {
                EventMgr.getInstance().postEvent("STAR_REALLY_CHANGE", [star]);
                EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
                let qiaodantj=Number(Const.get('SignSystemOpenCondition'));
                if(star>=qiaodantj){
                if(qiaodantj>beforStar){
                    EventMgr.getInstance().postEvent("SHOW_SIGNBTN");
                }
               }
               if(star>=400){
                   if(!Laya.LocalStorage.getJSON('threeEvent')){
                    Laya.LocalStorage.setJSON('threeEvent','1');
                    let str = "ftue_harbourUnlock";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                   }
               }
            }
        }
        getShell() {
            return this._player.getShell() > 0 ? this._player.getShell() : 0;
        }

        updateShell(beike) {
            let beforShell = this.getShell();
            this._player.updateShell(beike);
            if (beforShell != beike) {
                // EventMgr.getInstance().postEvent("SHELL_REALLY_CHANGE", [beike]);
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["tool"]);
            }
        }

        getDiamond() {
            return this._player.getDiamond() > 0 ? this._player.getDiamond() : 0;
        }

        updateDiamond(diamond) {
            let beforDiamond = this.getDiamond();
            this._player.updateDiamond(diamond);
            if (diamond != beforDiamond) {
                EventMgr.getInstance().postEvent("DIAMOND_REALLY_CHANGE", [diamond]);
            }
        }

        getMedal() {
            return this._player.getMedal() > 0 ? this._player.getMedal() : 0;
        }

        updateMedal(value) {
            let befor = this.getMedal();
            this._player.updateMedal(value);
            if (befor != value) {
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
            }
        }

        getBook() {
            return this._player.getBook() > 0 ? this._player.getBook() : 0;
        }

        updateBook(value) {
            let befor = this.getBook();
            this._player.updateBook(value);
            if (befor != value) {
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
            }
        }


        clear() {
            this._player.clear();
            this._adRewardsInfo = null;
            this._giveTimes = null;
            this._gameCreateTime = null;
        }

        setRegisterDays(days) {
            this._player.setRegisterDays(days);
        }

        getRegisterDays() {
            return this._player.getRegisterDays();
        }

        setAdRewardsInfo(adRewardsInfo) {
            this._adRewardsInfo = adRewardsInfo;
        }

        getMakeUpTimes() {
            if (this._adRewardsInfo && this._adRewardsInfo.makeUpTimes) {
                return this._adRewardsInfo.makeUpTimes;
            }
            return 0;
        }

        getTaskRewardTimes() {
            if (this._adRewardsInfo && this._adRewardsInfo.taskRewardTimes) {
                return this._adRewardsInfo.taskRewardTimes;
            }
            return 0;
        }

        addMakeUpTimes(makeUpTimes) {
            this._adRewardsInfo.makeUpTimes += makeUpTimes;
        }

        addTaskRewardTimes(taskRewardTimes) {
            this._adRewardsInfo.taskRewardTimes += taskRewardTimes;
        }

        setLoginDays(days){
            this._loginDays = days;
        }

        getLoginDays(){
            return this._loginDays;
        }

        //gata注册时间
        setCreateTime(time) {
            this._createTime = time;
        }
        //只静态页展示使用，其他用getGameCreateDays按零点计算
        getCreateDays() {
            if (this._createTime == null) {
                return null;
            }
            let day = 0;
            let now = TimeUtils.getTimeNowOnServer();
            let date = new Date(now);
            let year = date.getUTCFullYear(now);
            let mouth = date.getUTCMonth(now);
            let today = date.getUTCDate(now);
            let todayTime = Date.UTC(year, mouth, today) / 1000;
            let createDate = new Date(this._createTime * 1000);
            let creatYear = createDate.getUTCFullYear();
            let creatMouth = createDate.getUTCMonth();
            let creatDay = createDate.getUTCDate();
            let creatTime = Date.UTC(creatYear, creatMouth, creatDay) / 1000;
            todayTime = Math.max(todayTime, creatTime);
            day = Math.floor((todayTime - creatTime) / 3600 / 24) + 1;
            return day;
        }

        //game注册时间
        setGameCreateTime(time) {
            this._gameCreateTime = time;
        }
        //按5点计算
        getGameCreateDays() {
            if (this._gameCreateTime == null) {
                return null;
            }
            let day = 0;
            let now = TimeUtils.getTimeNowOnServer();
            let date = new Date(now);
            let year = date.getUTCFullYear(now);
            let mouth = date.getUTCMonth(now);
            let today = date.getUTCDate(now);
            let nowHour = date.getUTCHours(now);
            today = nowHour >= 21 ? today : today - 1;
            let todayTime = Date.UTC(year, mouth, today) / 1000;
            let createDate = new Date(this._gameCreateTime * 1000);
            let creatYear = createDate.getUTCFullYear();
            let creatMouth = createDate.getUTCMonth();
            let creatDay = createDate.getUTCDate();
            let creatHour = createDate.getUTCHours();
            creatDay = creatHour >= 21 ? creatDay : creatDay - 1;
            let creatTime = Date.UTC(creatYear, creatMouth, creatDay) / 1000;
            todayTime = Math.max(todayTime, creatTime);
            day = Math.floor((todayTime - creatTime) / 3600 / 24) + 1;
            return day;
        }
        
        setRid(rid) {
            this._rid = rid;
        }

        checkIsGroupB() {
            let ABTestTriggerInvite = Const.get("ABTestTriggerInvite");
            if (ABTestTriggerInvite != "1") {
                return false;
            }
            let ABTestRunningTime = JSON.parse(Const.get("ABTestRunningTime"));
            let ABTestBGroupRegisterTime = JSON.parse(Const.get("ABTestBGroupRegisterTime"));
            let runningStart = ABTestRunningTime.start;
            let runningEnd = ABTestRunningTime.end;
            let registerStart = ABTestBGroupRegisterTime.start;
            let registerEnd = ABTestBGroupRegisterTime.end;
            let now = Math.floor(TimeUtils.getTimeNowOnServer() / 1000);
            let ridNum = Number(this._rid.split("_")[0]);
            if (this._createTime && registerStart <= this._createTime && this._createTime <= registerEnd && now >= runningStart && now <= runningEnd && ridNum % 2 == 0) {
                return true;
            }
            return false;
        }

        setGiveTimes(times) {
            this._giveTimes = times ? times : 0;
        }

        getGiveTimes(times) {
            return this._giveTimes;
        }
    }

    class SceneContainer extends Laya.Panel {
        constructor(sceneId, scene) {
            super();
            this.sceneId = sceneId;
            this._scene = scene;
            scene.removeSelf();
            this.addChild(scene);
            scene.zOrder = 1;
            scene.mouseThrough = true;
            this.initView();
        }

        initView() {
            // console.debug("initView", Laya.stage.width, Laya.stage.height);
            let scene = this._scene;
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            this.box = this;
            // this.box = new Laya.Box();
            // this.addChild(this.box);
            // this.box.width = this.width;
            // this.box.height = this.height;
            // this.box.mouseThrough = false;
            // this.box.zOrder = -1;
        }

        open(params) {
            if(this.parent==null)
                Laya.Scene.root.addChild(this);
            this._scene.onOpened(params);
            EventMgr.getInstance().postEvent("SCENE_OPENED", [this]);
        }

        close() {
            if(this.parent!=null){
                this.removeSelf();
                this._scene.onClosed();
                EventMgr.getInstance().postEvent("SCENE_CLOSED", [this]);
            }
        }

        getScene() {
            return this._scene;
        }

    }

    class GameContext {
    }

    class SceneMgr {

        constructor() {
            this.sceneMap = new Map();
            this.sceneContainerMap = new Map();
            this.sceneLockMap = new Map();
            EventMgr.getInstance().registEvent("SWITCH_SCENE", this, this.switchScene);
            EventMgr.getInstance().registEvent("UPDATE_STAR", this, this.changeLock);
            EventMgr.getInstance().registEvent("SCENE_OPENED", this, this.addSwitchSceneEvent);
            EventMgr.getInstance().registEvent("SCENE_CLOSED", this, this.removeSwitchSceneEvent);
            EventMgr.getInstance().registEvent("SWITCH_LAST_SCENE", this, this.switchLastScene);
            EventMgr.getInstance().registEvent("SWITCH_SCENE_SMOOTH", this, this.switchToScene);
            EventMgr.getInstance().registEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            this.initScengSwitch();
        }

        initScengSwitch() {
            this.switchMap = new Map();
            ConfigMgr$1.getInstance().loadConfig("Scene", Laya.Handler.create(this, function(config) {
                    config.forEach((item, sceneId) => {
                        let obj = {};
                        let changed = false;
                        item.goto.forEach(element => {
                            if (element != "") {
                                let data = element.split(":");
                                obj[data[0]] = Number(data[1]);
                                changed = true;
                            }
                        });
                        if(changed)
                            this.switchMap.set(Number(sceneId), obj);
                    });
                }));

            this._switchBar = new Laya.Image();
            this._switchBar.skin = "sc_coffee/scenesplit.png";
            this._switchBar.sizeGrid = "6,10,16,8";
            this._switchBar.height = 2436;
            this._switchBar.width = 40;
            this._switchBar.anchorX = 0.5;
            this._switchBar.anchorY = 0;
        }

        static getInstance() {
            return SceneMgr.instance?SceneMgr.instance:SceneMgr.instance = new SceneMgr();
        }

        getSceneSwitchConfig(sceneId) {
            return this.switchMap.get(Number(sceneId));
        }

        getSceneUrl(sceneId) {
            if(sceneId == Constants.CoffeeSceneId) {
                return "scene/CoffeeScene.scene";
            } else if(sceneId == Constants.BeachSceneId) {
                return "scene/BeachScene.scene";
            } else if(sceneId == Constants.NauticalSceneId) {
                return "scene/NauticalScene.scene";
            } else if(sceneId == Constants.HomelandId) {
                return "scene/Homeland.scene";
            } else if(sceneId == Constants.FishingSceneId) {
                return "scene/FishingScene.scene";
            } else if(sceneId == Constants.LoginSceneId) {
                return "scene/Login.scene";
            }
        }

        getScene(sceneId) {
            return this.sceneMap.get(sceneId);
        }

        getSceneContainer(sceneId) {
            return this.sceneContainerMap.get(sceneId);
        }

        getCurrentSceneId() {
            return this.currentSceneId;
        }

        loadScene(sceneId, handler) {
            let container = this.sceneContainerMap.get(sceneId);
            if(container == null) {
                let sceneUrl = this.getSceneUrl(sceneId);
                Laya.Scene.load(sceneUrl, Laya.Handler.create(this, function(scene) {
                    this.sceneMap.set(sceneId, scene);
                    let container = new SceneContainer(sceneId, scene);
                    this.sceneContainerMap.set(sceneId, container);
                    container.visible = false;
                    handler.runWith(container);
                }));
            }else {
                handler.runWith(container);
            }
        }

        _openScene(sceneId, handler) {
            let container = this.sceneContainerMap.get(sceneId);
            if(container != null) {
                container.visible = true;
                this.showLock(sceneId);
                container.x = 0;
                container.y = 0;
                container.open();
                CommonUIMgr.getInstance().showCommonUI(sceneId, handler, container.getScene());
                EventMgr.getInstance().postEvent("SWITCH_SCENE_COMPLETE", sceneId);
                return;
            } else {
                this.loadScene(sceneId, Laya.Handler.create(this, function(container) {
                    container.visible = true;
                    container.open();
                    this.showLock(sceneId);
                    CommonUIMgr.getInstance().showCommonUI(sceneId, handler, container.getScene());
                    EventMgr.getInstance().postEvent("SWITCH_SCENE_COMPLETE", sceneId);
                }));
            }
        }

        switchScene(sceneId, handler) {
            if (this._cameraType) {
                return;
            }
            if(this.currentSceneId == sceneId) {
                return;
            }
            let container = this.getSceneContainer(this.currentSceneId);
            if(container) {
                console.debug("close Scene", this.currentSceneId);
                container.close();
                this.hideSceneSwitching();
            }
            this.lastSceneId = this.currentSceneId;
            this.currentSceneId = sceneId;
            this.closeOthers();
            this._switchBar.removeSelf();
            this._openScene(sceneId, handler);
            GameContext.Sliding = false;
        }
        closeAllScene() {
            CommonUIMgr.getInstance().hideAll();
            CommonUIMgr.getInstance().clear();
            this.currentSceneId = null;
            for(const [sceneId, container] of this.sceneContainerMap) {
                container.destroy();
            }
            this.sceneContainerMap.clear();
            for(const [sceneId, scene] of this.sceneMap) {
                scene.destroy();
            }
            this.sceneMap.clear();
            for(let i = Laya.Scene.unDestroyedScenes.length - 1 ; i >= 0  ; --i) {
                let s = Laya.Scene.unDestroyedScenes[i];
                s.destroy();
            }
            Laya.Scene.gc();
            this._switchAllowed = false;
        }

        closeOthers() {
            if(this.currentSceneId!=null) {
                for(const [sceneId, container] of this.sceneContainerMap) {
                    if(sceneId!=this.currentSceneId)
                        container.close();
                }
            }
            Laya.timer.frameOnce(10, null, Laya.Scene.gc);
        }

        setSwitchAllowed() {
            this._switchAllowed = true;
        }

        addSwitchSceneEvent(container) {
            this._switchDisabled = false;
            container.box.off(Laya.Event.MOUSE_DOWN, this, this.mouseDown);
            container.box.off(Laya.Event.MOUSE_UP, this, this.mouseUp);
            container.box.off(Laya.Event.MOUSE_OUT, this, this.mouseUp);
            container.box.off(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);

            let obj = this.getSceneSwitchConfig(container.sceneId);
            if (obj) {
                container.box.on(Laya.Event.MOUSE_DOWN, this, this.mouseDown);
                container.box.on(Laya.Event.MOUSE_UP, this, this.mouseUp);
                container.box.on(Laya.Event.MOUSE_OUT, this, this.mouseUp);
            }
        }

        removeSwitchSceneEvent(container) {
            container.box.off(Laya.Event.MOUSE_DOWN, this, this.mouseDown);
            container.box.off(Laya.Event.MOUSE_UP, this, this.mouseUp);
            container.box.off(Laya.Event.MOUSE_OUT, this, this.mouseUp);
            container.box.off(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);
        }

        mouseDown() {
            if(!this._switchAllowed)
                return;
            this.hideSceneSwitching();
            let container = this.sceneContainerMap.get(this.currentSceneId);
            this.isSwitching = false;
            this.switchingType = null;
            this._sliding = false;
            GameContext.Sliding = false;
            if(container) {
                if (this.currentSceneId != container.sceneId) {
                    return;
                }
                if (this.mousePoint) {
                    this.mousePoint.x = Laya.stage.mouseX;
                    this.mousePoint.y = Laya.stage.mouseY;
                } else {
                    this.mousePoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
                }
                container.box.off(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);
                container.box.on(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);
            }
        }

        mouseUp() {
            if(!this._switchAllowed)
                return;
            let container = this.sceneContainerMap.get(this.currentSceneId);
            if(container) {
                // console.debug("mouseUp", this._sliding, this.switchingType, this.isSwitching);
                container.box.off(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);
                if(this._sliding) {
                    if(this.isSwitching) {
                        this.touchSwitchScene(this.switchingType);
                    }else {
                        if(this.switchingType) {
                            this.backSceneSwitch(this.switchingType);
                        }
                    }
                }
                this._sliding = false;
                this.isSwitching = false;
                this.switchingType = null;
            }
        }

        mouseMoveCall() {
            if(!this._switchAllowed)
                return;
            let container = this.sceneContainerMap.get(this.currentSceneId);
            let obj = this.getSceneSwitchConfig(this.currentSceneId);
            let offsetX = Laya.stage.mouseX - this.mousePoint.x;
            if(obj) {
                if(offsetX>0) {
                    if((offsetX>15 || this._sliding) && obj.left!=null) {
                        this._sliding = true;
                        GameContext.Sliding = true;
                        this.switchingType = "left";
                        let offset = Math.abs(offsetX);
                        if(offset>=Constants.SwitchSceneDist) {
                            this.isSwitching = true;
                            this.showSceneSwitching("left", obj.left, offsetX);
                        }else {
                            this.isSwitching = false;
                            this.showSceneSwitching("left", obj.left, offsetX);
                        }
                    }else {
                        this.isSwitching = false;
                        this.hideSceneSwitching();
                    }
                }else if(offsetX<=0) {
                    if((offsetX<-15 || this._sliding) && obj.right!=null) {
                        this._sliding = true;
                        GameContext.Sliding = true;
                        this.switchingType = "right";
                        let offset = Math.abs(offsetX);
                        if(offset>=Constants.SwitchSceneDist) {
                            this.isSwitching = true;
                            this.showSceneSwitching("right", obj.right, offsetX);
                        }else {
                            this.isSwitching = false;
                            // this.switchingType = null;
                            this.showSceneSwitching("right", obj.right, offsetX);
                        }
                    }else {
                        this.isSwitching = false;
                        // this.switchingType = null;
                        this.hideSceneSwitching();
                    }
                }
            }
        }

        touchSwitchScene(type) {
            let obj = this.getSceneSwitchConfig(this.currentSceneId);
            let container = this.sceneContainerMap.get(this.currentSceneId);
            container.box.off(Laya.Event.MOUSE_MOVE, this, this.mouseMoveCall);
            if(obj[type]) {
                this.switchToScene(type, obj[type]);
                this.isSwitching = false;
            }
        }

        switchToScene(type, sceneId) {
            if(!this._switchAllowed)
                return;
            if(this._switchDisabled)
                return;
            this._switchDisabled = true;
            let container = this.sceneContainerMap.get(this.currentSceneId);
            let next = this.sceneContainerMap.get(sceneId);
            if(!container||!next)
                return;
            let offsetX = container.x;
            this.showSceneSwitching(type, sceneId, offsetX);
            this._nextScene = null;
            let targetX_container = - Laya.stage.width - this._switchBar.width;
            let targetX_bar = - this._switchBar.width/2;
            let targetX_next = 0;
            if(type=="left") {
                targetX_container = Laya.stage.width + this._switchBar.width;
                targetX_bar = Laya.stage.width + this._switchBar.width/2;
            }
            this.lastSceneId = this.currentSceneId;
            this.currentSceneId = sceneId;
            let time = 300;
            for(const [sceneId, container] of this.sceneContainerMap) {
                this.removeSwitchSceneEvent(container);
            }

            Laya.Tween.clearAll(this._switchBar);
            Laya.Tween.to(container, { x: targetX_container}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        container.close();
                    }), 0);
            Laya.Tween.to(next, { x: targetX_next}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        GameContext.Sliding = false;
                        this.closeOthers();
                        this._switchBar.removeSelf();
                        next.x = 0;
                        next.open();
                        this.showLock(sceneId);
                        CommonUIMgr.getInstance().showCommonUI(sceneId, null, next.getScene());
                    }), 0);
            Laya.Tween.to(this._switchBar, { x: targetX_bar}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        this._switchBar.removeSelf();
                    }), 0);
            EventMgr.getInstance().postEvent("SWITCH_SCENE_COMPLETE", sceneId);
        }

        backSceneSwitch(type) {
            let container = this.sceneContainerMap.get(this.currentSceneId);
            let next = this._nextScene;
            let offsetX = container.x;
            if(next)
                this.showSceneSwitching(type, next.sceneId, offsetX);
            this._nextScene = null;
            let targetX_container = 0;
            let targetX_bar = Laya.stage.width + this._switchBar.width/2;
            let targetX_next = Laya.stage.width + this._switchBar.width;
            if(type=="left") {
                targetX_next = - this._switchBar.width - Laya.stage.width;
                targetX_bar = - this._switchBar.width/2;
            }
            let time = 100;
            Laya.Tween.clearAll(container);
            Laya.Tween.clearAll(this._switchBar);
            for(const [sceneId, container] of this.sceneContainerMap) {
                this.removeSwitchSceneEvent(container);
            }
            if(next) {
                Laya.Tween.clearAll(next);
                Laya.Tween.to(next, { x: targetX_next}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        next.removeSelf();
                    }), 0);
            }
            Laya.Tween.to(container, { x: targetX_container}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        GameContext.Sliding = false;
                        container.x = 0;
                        this.addSwitchSceneEvent(container);
                        this.closeOthers();
                        this._switchBar.removeSelf();
                        // this.showLock(sceneId);
                        // CommonUIMgr.getInstance().showCommonUI(sceneId, null, next.getScene());
                    }), 0);
            Laya.Tween.to(this._switchBar, { x: targetX_bar}, time,
                    null,
                    Laya.Handler.create(this, function() {
                        this._switchBar.removeSelf();
                    }), 0);
        }

        showSceneSwitching(type, sceneId, offsetX) {
            let container = this.sceneContainerMap.get(this.currentSceneId);
            let next = this.sceneContainerMap.get(sceneId);
            container.x = offsetX;
            if(this._switchBar.parent==null) {
                Laya.Scene.root.addChild(this._switchBar);
            }
            if(type=="left") {
                this._switchBar.x = offsetX - this._switchBar.width/2;
                if(this._nextScene!=next) {
                    if(this._nextScene)
                        this._nextScene.removeSelf();
                    this._nextScene = next;
                    next.visible = true;
                    Laya.Scene.root.addChild(next);
                }
                next.x = offsetX - this._switchBar.width - next.width;

            }else {
                this._switchBar.x = Laya.stage.width + offsetX + this._switchBar.width/2;
                if(this._nextScene!=next) {
                    if(this._nextScene)
                        this._nextScene.removeSelf();
                    this._nextScene = next;
                    next.visible = true;
                    Laya.Scene.root.addChild(next);
                }
                next.x = Laya.stage.width + offsetX + this._switchBar.width;
            }
        }
        hideSceneSwitching() {
            if(this._nextScene) {
                Laya.Tween.clearAll(this._nextScene);
                this._nextScene.removeSelf();
                this._nextScene = null;
            }
            this._switchBar.removeSelf();
            let container = this.sceneContainerMap.get(this.currentSceneId);
            container.x = 0;
            Laya.Tween.clearAll(container);
            Laya.Tween.clearAll(this._switchBar);
        }

        getSceneLock(id) {
            return this.sceneLockMap.get(id);
        }

        changeLock(){
            let allUnlock = true;
            for (let id = Constants.LoginSceneId; id <= Constants.HomelandId; id++) {
                let item = ConfigMgr$1.getInstance().getItemById("Scene", id.toString());
                if (item) {
                    let lock = true;
                    let unlockStar = JSON.parse(item.Unlock);
                    if (unlockStar && unlockStar[0] && unlockStar[0].Star <= PlayerDataMgr.getInstance().getStar()) {
                        lock = false;
                    }else {
                        allUnlock=false;
                    }
                    this.sceneLockMap.set(id, lock);
                    this.showLock(id);
                }
            }
            if(allUnlock){
                EventMgr.getInstance().removeEvent("UPDATE_STAR", this, this.changeLock);
                this.addUnlock = true;
            }
        }

        showLock(id) {
            if (this.addUnlock) {
                return;
            }
            let lock = this.getSceneLock(id);
            let scene = this.getScene(id);
            if (scene && scene.lockTypeChange) {
                scene.lockTypeChange(lock);
            }
        }

        switchLastScene() {
            this.switchScene(this.lastSceneId);
        }

        cameraSwitch(type) {
            if (type) {
                CommonUIMgr.getInstance().hideBottomUI();
                CommonUIMgr.getInstance().hideLeftUI();
                CommonUIMgr.getInstance().hideArrowUI();
                CommonUIMgr.getInstance().hideTopUI();
                this._cameraType = true;
            } else {
                CommonUIMgr.getInstance().showCommonUI(this.currentSceneId);
                this._cameraType = false;
            }
        }

    }

    class BuildingExtInfo {
        constructor() {
            this.buildingRoot = null;
            this.isBusy = false;
            this.buildingExtInfoMgr = null;
            this.actionArr = null;
        }

        setBuildingExtInfoMgr(mgr) {
            this.buildingExtInfoMgr = mgr;
        }

        getBuildingExtInfoMgr() {
            return this.buildingExtInfoMgr;
        }

        addToBuildingRoot(render) {
            if (this.buildingRoot != null) {
                this.buildingRoot.removeChildByName("cat");
                render.name = "cat";
                this.buildingRoot.addChild(render);
            }
        }

        setBuildingRoot(buildingRoot) {
            this.buildingRoot = buildingRoot;
            if (this.buildingRoot != null) {
                this.buildingRoot.mouseThrough = true;
            }
        }

        getBuildingRoot() {
            return this.buildingRoot;
        }

        getBuildingRootWidth() {
            return this.buildingRoot.width;
        }

        getBuildingRootHalfWidth() {
            return this.getBuildingRootWidth() / 2;
        }

        getBuildingRootHeight() {
            return this.buildingRoot.height;
        }

        getBuildingRootHalfHeight() {
            return this.getBuildingRootHeight() / 2;
        }

        randomCatMove() {
            if (this.actionArr != null) {
                let i = Utils.randomExt(this.actionArr.length);
                return this.actionArr[i];
            }
            return null;
        }

        cleanBuildingExtInfo() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }

        setActionSetting(actionArr) {
            this.actionArr = actionArr;
        }

        getActionSetting() {
            return this.actionArr;
        }

        hideOthers() {
            let building = this.buildingRoot.parent;
            let numChildren = building.numChildren;
            for (let i = 0; i < numChildren; i++) {
                if (building.getChildAt(i) != this.buildingRoot) {
                    building.getChildAt(i).visible = false;
                } else {
                    building.getChildAt(i).visible = true;
                }
            }
        }

        showAll() {
            let building = this.buildingRoot.parent;
            let numChildren = building.numChildren;
            for (let i = 0; i < numChildren; i++) {
                building.getChildAt(i).visible = true;
            }
        }
    }

    class VisitorReward {

        constructor() {

        }

        setData(id, catConfigId, buildingId, reward, rewardType) {
            this._catConfigId = catConfigId;
            this._buildingId = buildingId;
            this._reward = reward;
            this._rewardType = rewardType;
            this._id = id;
        }

        restore(info) {
            // console.debug("VisitorReward restore", info);
            this._id = info.id;
            this._catConfigId = info.roleId;
            this._buildingId = info.buildingId;
            if(info.rewards && info.rewards.rewards) {
                for(const item of info.rewards.rewards) {
                    this._reward = item.amount;
                    this._rewardType = item.id;
                }
            }
        }

        get fetched() {
            return this._fetched;
        }

        fetch() {
            this._fetched = true;
        }

        get id() {
            return this._id;
        }

        get catConfigId() {
            return this._catConfigId;
        }

        get buildingId() {
            return this._buildingId;
        }

        get reward() {
            return [this._reward, this._rewardType];
        }

        dump() {
            return {"id": this._id, "name":"", "buildingId":this._buildingId, "rewards": {"rewards": [{"type": "Currency", "id": this._rewardType, "amount": this._reward}]}, "roleId": String(this._catConfigId)};
        }

        clear() {
        }

    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class SyncDataMgr {

        constructor() {
            this._cacheFishMap = new Map();
            this._cacheStarMap = new Map();
            this._fishReward = 0;
            this._starReward = 0;
            this._medalReward = 0;
            this._bookReward = 0;
            this._scene2ExhaustTime = null;
            this._visitorRewardCache = new Array();
            this._visitorRewardStash = new Array();

            this._visitorGift = null;

            this._eatTimes = 0;
        }

        static getInstance() {
            return SyncDataMgr.instance?SyncDataMgr.instance:SyncDataMgr.instance = new SyncDataMgr();
        }

        getCachedFish(buildingBaseId) {
            let fish = this._cacheFishMap.get(buildingBaseId);
            return fish?fish:0;
        }

        addCachedFish(buildingBaseId, count) {
            let fish = this.getCachedFish(buildingBaseId);
            fish = fish+count;
            this._cacheFishMap.set(buildingBaseId, fish);
        }

        getCachedStar(buildingBaseId) {
            let star = this._cacheStarMap.get(buildingBaseId);
            return star?star:0;
        }

        addCachedStar(buildingBaseId, count) {
            let star = this.getCachedStar(buildingBaseId);
            star = star+count;
            this._cacheStarMap.set(buildingBaseId, star);
        }

        collectReward(buildingBaseId) {
            let fish = this.getCachedFish(buildingBaseId);
            this._cacheFishMap.delete(buildingBaseId);
            this._fishReward += fish;
            let star = this.getCachedStar(buildingBaseId);
            this._cacheStarMap.delete(buildingBaseId);
            this._starReward += star;
            return [fish, star];
        }

        dumpRewards() {
            let obj = { "rewards": [] };
            if (this._fishReward > 0) {
                obj.rewards.push(this.getReward(Constants.RewardType.FISH, this._fishReward));
            }
            if (this._starReward > 0) {
                obj.rewards.push(this.getReward(Constants.RewardType.STAR, this._starReward));
            }
            if (this._bookReward > 0) {
                obj.rewards.push(this.getReward(Constants.RewardType.BOOK, this._bookReward));
            }
            if (this._medalReward > 0) {
                obj.rewards.push(this.getReward(Constants.RewardType.MEDAL, this._medalReward));
            }
            return obj;
        }

        getReward(id, amount) {
            return { "type": "Currency", "id": id, "amount": amount };
        }

        dropRewards() {
            this._fishReward = 0;
            this._starReward = 0;
            this._medalReward = 0;
            this._bookReward = 0;
        }

        dumpCachedFish() {
            let fishCache = new Array();
            for(const[key, value] of this._cacheFishMap) {
                if(value>0) {
                    fishCache.push([key, value]);
                }
            }
            return fishCache;
        }

        restoreFish(fishCache) {
            this.clear();
            if(fishCache){
                for(const value of fishCache) {
                    this._cacheFishMap.set(value[0], value[1]);
                }
            }
        }

        //场景2 投食
        getScene2ExhaustTime() {
            return this._scene2ExhaustTime?this._scene2ExhaustTime:0;
        }

        addScene2Food(time, quality) {
            if(!this._scene2ExhaustTime || this._scene2ExhaustTime < TimeUtils.getTimeNowOnServer()+time*1000) {
                this._scene2ExhaustTime = TimeUtils.getTimeNowOnServer()+time*1000;
                return this._scene2ExhaustTime;
            }
            return null;
        }

        restoreScene2ExhaustTime(time, quality) {
            this._scene2ExhaustTime = time;
        }

        //场景2 常客礼物
        cacheVisitorReward(catId, catConfigId, buildingId, reward, rewardType) {
            let visitorReward = new VisitorReward();
            let id = "00000"+String(catId);
            id = String(TimeUtils.getTimeNowOnServer()) + id.substr(id.length-4);
            visitorReward.setData(id, catConfigId, buildingId, reward, rewardType);
            let giftsCountLimit = Number(Const.get("GiftsCountLimit"));
            if (this._visitorRewardCache.length >= giftsCountLimit) {
                this._visitorRewardCache.shift();
            }
            this._visitorRewardCache.push(visitorReward);
            this.setVisitorRewardsExist(true);
        }

        dumpVisitorRewardsCache() {
            for(const visitorReward of this._visitorRewardCache) {
                this._visitorRewardStash.push(visitorReward.dump());
            }
            this._visitorRewardCache.length = 0;
            return this._visitorRewardStash;
        }

        dropVisitorRewardsCache() {
            this._visitorRewardStash.length = 0;
        }

        updateVisitorGift(visitorRewards) {
            if(!this._visitorGift) {
                this._visitorGift = new Array();
            }
            this._visitorGift.length = 0;
            for(const rewardInfo of visitorRewards) {
                let reward = new VisitorReward();
                reward.restore(rewardInfo);
                this._visitorGift.push(reward);
            }
            if (this._visitorGift.length == 0 && this._visitorRewardCache.length == 0 && this._visitorRewardStash.length == 0) {
                this.setVisitorRewardsExist(false);
            }
        }

        clearVisitorGift() {
            if(this._visitorGift)
                this._visitorGift.length = 0;
            if (this._visitorRewardCache && this._visitorRewardCache.length > 0) {
                this.setVisitorRewardsExist(true);
            } else {
                this.setVisitorRewardsExist(false);
            }
        }

        getVisitorGifts() {
            return this._visitorGift;
        }

        setVisitorRewardsExist(value) {
            this._visitorRewardsExist = value;
            EventMgr.getInstance().postEvent("UPDATA_GIFTBTN_SHOW");
        }

        getVisitorRewardsExist() {
            return this._visitorRewardsExist;
        }

        // 打点
        addEatTimes() {
            this._eatTimes++;
        }

        dumpEatTimes() {
            let times = this._eatTimes;
            this._eatTimes = 0;
            return times;
        }

        clear() {
            this._cacheFishMap.clear();
            this._cacheStarMap.clear();
            this._fishReward = 0;
            this._starReward = 0;
            this._medalReward = 0;
            this._bookReward = 0;
            this._visitorRewardCache.length = 0;
            this._visitorRewardStash.length = 0;
            this._visitorGift = null;
        }

        addHomeRubbishRewards(star, fish, medal, book) {
            this._fishReward += fish;
            this._starReward += star;
            this._medalReward += medal;
            this._bookReward += book;
        }

        getCacheFishMap() {
            return this._cacheFishMap;
        }
    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class LetterBoxMgr {

        constructor() {
            this.cacheFish = 0;
        }

        static getInstance() {
            return LetterBoxMgr.instance?LetterBoxMgr.instance:LetterBoxMgr.instance = new LetterBoxMgr();
        }

        getCapacity() {
            return this.capacity;
        }

        setCapacity(capacity) {
            this.capacity = capacity;
        }

        getCacheFish() {
            return this.cacheFish;
        }

        setCacheFish(fish) {
            this.cacheFish = fish;
        }

        setLastUpdateTime(millis) {
            this.lastUpdateMillis = millis;
        }

        getLastUpdateTime() {
            return this.lastUpdateMillis;
        }


        clear() {
            this.cacheFish = 0;
            this.capacity = 0;
        }
    }

    class PlayerSystem {

        static init() {
            EventMgr.getInstance().registEvent("REFRESH_FISH", null, PlayerSystem.cumulateFish);
            EventMgr.getInstance().registEvent("REFRESH_STAR", null, PlayerSystem.cumulateStar);
            EventMgr.getInstance().registEvent("SECRETLY_REFRESH_FISH", null, PlayerSystem.secretlyCumulateFish);
            EventMgr.getInstance().registEvent("SECRETLY_REFRESH_STAR", null, PlayerSystem.secretelyCumulateStar);
            PlayerDataMgr.getInstance().clear();
        }

        static getFish() {
            return PlayerDataMgr.getInstance().getFish();
        }

        static cumulateFish(fish, handler) {
            PlayerSystem.updateFish(PlayerSystem.getFish() + fish, true);
        }

        static secretlyCumulateFish(fish, handler) {
            PlayerDataMgr.getInstance().updateFish(PlayerSystem.getFish() + fish);
        }

        static secretlyUpdateFish(fish) {
            PlayerDataMgr.getInstance().updateFish(fish);
        }

        static updateFish(fish, ifTween = false) {
            PlayerDataMgr.getInstance().updateFish(fish);
            EventMgr.getInstance().postEvent("UPDATE_FISH", [fish, ifTween]);
        }

        static getStar() {
            return PlayerDataMgr.getInstance().getStar();
        }

        static getDirectlyReceiveTime(){
            let ADTaskTextHideTimes = JSON.parse(Const.get("ADTaskTextHideTimes"));
            let star = PlayerDataMgr.getInstance().getStar();
            let time = 0;
            for(let i = 0;i < ADTaskTextHideTimes.length;i ++){
                if(star >= ADTaskTextHideTimes[i].Stars){
                    if(i == ADTaskTextHideTimes.length - 1 || star < ADTaskTextHideTimes[i+1].Stars){
                        time = ADTaskTextHideTimes[i].Times * 1000 | 0;
                        return time;
                    }
                }
            }
        }

        static updateStar(star) {
            PlayerDataMgr.getInstance().updateStar(star);
            EventMgr.getInstance().postEvent("UPDATE_STAR", star);
        }

        static secretlyUpdateStar(star) {
            PlayerDataMgr.getInstance().updateStar(star);
        }

        static cumulateStar(star) {
            PlayerSystem.updateStar(PlayerSystem.getStar()+star);
        }

        static secretelyCumulateStar(star) {
            PlayerDataMgr.getInstance().updateStar(PlayerSystem.getStar()+star);
        }

        static clear() {
            PlayerDataMgr.getInstance().clear();
        }

        static setPlayerInfo(info) {
            console.log(info,dyz的info);
            PlayerSystem.playerInfo = info;
        }

        static getPlayerInfo() {
            return PlayerSystem.playerInfo;
        }

        static setRegisterDays(days) {
            PlayerDataMgr.getInstance().setRegisterDays(days);
        }

        static getRegisterDays() {
            return PlayerDataMgr.getInstance().getRegisterDays();
        }

        static setAdRewardsInfo(adRewardsInfo) {
            PlayerDataMgr.getInstance().setAdRewardsInfo(adRewardsInfo);
        }

        static addMakeUpTimes(makeUpTimes = 1) {
            PlayerDataMgr.getInstance().addMakeUpTimes(makeUpTimes);
        }

        static getMakeUpTimes() {
            return PlayerDataMgr.getInstance().getMakeUpTimes();
        }

        static addTaskRewardTimes(taskRewardTimes = 1) {
            PlayerDataMgr.getInstance().addTaskRewardTimes(taskRewardTimes);
        }

        static getTaskRewardTimes() {
            return PlayerDataMgr.getInstance().getTaskRewardTimes();
        }
        
        static setLoginDays(days){
            PlayerDataMgr.getInstance().setLoginDays(days);
        }

        static getLoginDays(){
            return PlayerDataMgr.getInstance().getLoginDays();
        }

        static setCreateTime(time){
            PlayerDataMgr.getInstance().setCreateTime(time);
        }

        static getCreateDays(){
            return PlayerDataMgr.getInstance().getCreateDays();
        }

        static setGameCreateTime(time){
            PlayerDataMgr.getInstance().setGameCreateTime(time);
        }

        static getGameCreateDays(){
            return PlayerDataMgr.getInstance().getGameCreateDays();
        }

        static setRid(rid){
            PlayerDataMgr.getInstance().setRid(rid);
        }

        static checkIsGroupB(){
            return PlayerDataMgr.getInstance().checkIsGroupB();
        }

        static setGiveTimes(times){
            PlayerDataMgr.getInstance().setGiveTimes(times);
        }

        static getGiveTimes(times){
            return PlayerDataMgr.getInstance().getGiveTimes();
        }
        static getShell(){
            return PlayerDataMgr.getInstance().getShell();
        }
        static updateShell(shell) {
            PlayerDataMgr.getInstance().updateShell(shell);
            EventMgr.getInstance().postEvent("UPDATE_BEIKE", shell);
        }

        static cumulateShell(shell) {
            PlayerSystem.updateShell(PlayerSystem.getShell() + shell);
        }

        static secretlyUpdateShell(shell) {
            PlayerDataMgr.getInstance().updateShell(shell);
        }

        static updateDiamond(d) {
            PlayerDataMgr.getInstance().updateDiamond(d);
            EventMgr.getInstance().postEvent("UPDATE_ZUANSHI", d);
        }

        static getDiamond() {
            return PlayerDataMgr.getInstance().getDiamond();
        }

        static cumulateDiamond(d) {
            PlayerSystem.updateDiamond(PlayerSystem.getDiamond() + d);
        }
            
        static updateMedal(d) {
            PlayerDataMgr.getInstance().updateMedal(d);
        }

        static getMedal() {
            return PlayerDataMgr.getInstance().getMedal();
        }
        
        static cumulateMedal(d) {
            PlayerDataMgr.getInstance().updateMedal(PlayerSystem.getMedal() + d);
        }

        static updateBook(d) {
            PlayerDataMgr.getInstance().updateBook(d);
        }

        static getBook() {
            return PlayerDataMgr.getInstance().getBook();
        }
        
        static cumulateBook(d) {
            PlayerDataMgr.getInstance().updateBook(PlayerSystem.getBook() + d);
        }

    }

    PlayerSystem.playerInfo = {};
    PlayerSystem.playerInfo.nickName = "sunwen";
    PlayerSystem.playerInfo.avatarUrl = "sunwen";

    class PetData {

        constructor(configId, config) {
            this.configId = configId;
            this.config = config?config:ConfigMgr$1.getInstance().getItemById("Role", String(configId));
            this.intimacy = 0;
            this.visitTimes = 0;
            this.adopted = false;
            this.newCome = true;
            this.redPointStatus = 0;       //在可领养后是否看过，看过则不需要再亮红点，0代表不满足领养条件，1代表满足条件未看过，2代表满足条件并且已看过
            this._inHome = false;
            this._inNautical = false;
            this._dailyIntimacy = 0;
            this._fakeInHome = null;
            this._level = -1;
            this._quality = -1;
            this._attributeChanged = true;
            this._attribute = null;
            this.favor=0;
           // this._baseValue = JSON.parse(this.config.BaseValue);
        }

        setFavor(favor) {
            this.favor = favor;
        }

        getFavor() {
            return this.favor?this.favor:0;
        }

        setIntimacy(intimacy) {
            EventMgr.getInstance().postEvent("CHECK_INTIMACYUP",[this.getId(),this.intimacy,intimacy]);
            this.intimacy = intimacy;
        }

        getIntimacy() {
            return this.intimacy?this.intimacy:0;
        }

        addIntimacy(intimacy) {
            EventMgr.getInstance().postEvent("CHECK_INTIMACYUP",[this.getId(),this.intimacy,this.intimacy + intimacy]);
            this.intimacy += intimacy;
        }

        setVisitTimes(visitTimes) {
            this.visitTimes = visitTimes;

            //判断是否可以领养了，然后需要更改亮红点的状态
            if((this.canAdopt() ||this.canPreAdopt()) && this.redPointStatus == 0) {
                this.setRedPointStatus(1);
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
                EventMgr.getInstance().postEvent("UPDATE_PETS");
            }
        }

        getVisitTimes() {
            return this.visitTimes;
        }

        addVisitTimes(visitTimes) {
            this.visitTimes += visitTimes;

            //判断是否可以领养了，然后需要更改亮红点的状态
            if((this.canAdopt() ||this.canPreAdopt()) && this.redPointStatus == 0) {
                this.setRedPointStatus(1);
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
                EventMgr.getInstance().postEvent("UPDATE_PETS");
            }
        }

        setAdopted(adopted) {
            this.adopted = adopted?true:false;
        }

        getAdopted() {
            return this.adopted;
        }

        setNewCome(newCome) {
            this.newCome = newCome ? true : false;
        }

        getNewCome() {
            return this.newCome;
        }

        getRedPointStatus() {
            return this.redPointStatus;
        }

        setRedPointStatus(status) {
            this.redPointStatus = status;
        }

        //讲红点状态置为已读，必须是原来为1才置为2
        changeRedPointRead() {
            if(this.redPointStatus == 1) {
                this.redPointStatus = 2;
            }
        }

        needRedPoint() {
            return ((this.canAdopt() || this.canPreAdopt() || this.showCatArchivesRedPoint()) && this.redPointStatus == 1) || this.canLevelUp() || this.canStarUp();
        }

        showCatArchivesRedPoint(){
            if(!this.getAdopted()){
                return 0;
            }
            if(this.visitTimes<1){
                return 0;
            }
            if(!this.storyUnlock){
                return 0;
            }
            let red = 0;
            for(let i=0;i<0;i++){
                if(!this.storyUnlock.hasOwnProperty(i+1) && this.getIntimacyGrade()>i){
                    red = 1;
                }
            }
            if(red){
                this.setRedPointStatus(1);
            }
            return red ;
        }

        getId() {
            return this.configId;
        }

        getSource() {
            return this.config.Source;
        }

        getSkin() {
            return this.config.Skin;
        }

        getHeadIcon() {
            return "cat_collect/head/"+this.config.HeadIcon+".png";
        }

        getName() {
            return "tid#catName_"+this.getId();
        }

        getDefaultAction() {
            return this.config.DefaultAction;
        }

        getNormalPic() {
            return "cat_collect/normal/" + this.config.Icon + ".png";
        }

        getGrayPic() {
            return "cat_collect/gray/" + this.config.IconShadow + ".png";
        }

        getValueBase() {
            return this.config.ValueBase[this.getIntimacyGrade()];
        }

        get FavorToys() {
            return JSON.parse(this.config.FavorToys);
        }

        getAdoptLimit() {
            if(!this.adoptLimit)
                this.adoptLimit = JSON.parse(this.config.AdoptLimit);

            return this.adoptLimit;
        }

        getAdoptSound() {
            if(!this._adoptSound) {
                let info = JSON.parse(this.config.CatSound);
                this._adoptSound = info[0].Adopt;
            }
            return this._adoptSound;
        }

        getActionSound() {
            if(!this._actionSound) {
                let info = JSON.parse(this.config.CatSound);
                this._actionSound = info[0].Action;
            }
            return this._actionSound;
        }

        getNormalActions() {
            if(!this._normalActions) {
                this._normalActions = JSON.parse(this.config.NormalActions);
            }
            return this._normalActions;
        }

        getGift() {
            if(!this._gift) {
                this._gift = JSON.parse(this.config.Gift);
            }
            return this._gift;
        }

        getTag() {
            return JSON.parse(this.config.Label);
        }

        //未领取并且已经达到领取状态，则返回true
        canAdopt() {
            if(this.getAdopted()) return false;
            let minStar = Number(Const.get("CatAdoptStars"));
            if (minStar > PlayerSystem.getStar()) {
                return false;
            }
            let visitLimit = JSON.parse(this.config.AdoptLimit).VisitTimes?JSON.parse(this.config.AdoptLimit).VisitTimes:0;
            let favorslimit = JSON.parse(this.config.AdoptLimit).CatFavors?JSON.parse(this.config.AdoptLimit).CatFavors:0;
            return  ((this.visitTimes >= visitLimit)&&(this.favor >= favorslimit));
        }

        canPreAdopt(){
            if(this.getAdopted()) return false;
            let minStar = Number(Const.get("CatAdoptStars"));
            if (minStar > PlayerSystem.getStar()) {
                return false;
            }
            let CatAdoptADCondition = Const.get("CatAdoptADCondition");
            let CatAdoptADFavorsCondition = Const.get("CatAdoptADFavorsCondition");
            let visitLimit = JSON.parse(this.config.AdoptLimit).VisitTimes?JSON.parse(this.config.AdoptLimit).VisitTimes:0;
            let favorslimit = JSON.parse(this.config.AdoptLimit).CatFavors?JSON.parse(this.config.AdoptLimit).CatFavors:0;
            return ((this.visitTimes >= visitLimit * CatAdoptADCondition)&&(this.favor >= favorslimit * Number(CatAdoptADFavorsCondition)));
        }

        canShow() {
            if (this.getAdopted()) return false;
            let intimacyLimit = this.getAdoptLimit()["Intimacy"];
            return this.intimacy >= intimacyLimit && this.visitTimes > 0;
        }

        getIntimacyGrade() {
            let intimacy = this.getIntimacy();
            for(const k in this.config.Intimacy) {
                let v = this.config.Intimacy[k];
                if(intimacy < v){
                    return k - 1;
                }
            }
            return this.config.Intimacy.length - 1;
        }

        getIntimacyMax() {
            return this.config.IntimacyLimit;
        }

        setHomeStatus(homeStatus) {
            this.homeStatus=homeStatus.inHome;
        }

        set homeStatus(inHome) {
            if(this._inHome!=inHome) {
                this._inHome = inHome;
                if(!inHome) {
                    this.clearParams();
                }
                if(inHome)
                    EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PUT_PET", this.getId()]);
                else
                    EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "REMOVE_PET", this.getId()]);
            }
        }

        get homeStatus() {
            return this._inHome;
        }

        get fakeInHome() {
            if(this._fakeInHome==null)
                return this._inHome;
            else
                return this._fakeInHome;
        }

        set fakeInHome(inHome) {
            this._fakeInHome = inHome;
        }

        set inNautical(inNautical) {
            if(this._inNautical!=inNautical) {
                this._inNautical = inNautical;
                // if(inNautical)
                //     EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PET_NAUTICAL", this.getId()]);
                // else
                //     EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PET_NAUTICAL_BACK", this.getId()]);
            }
        }

        get inNautical() {
            return this._inNautical;
        }

        set dailyIntimacy(intimacy) {
            this._dailyIntimacy = intimacy;
        }

        get dailyIntimacy() {
            return this._dailyIntimacy;
        }

        // params

        set funTimer(timer) {
            this._funTimer = timer;
        }

        get funTimer() {
            return this._funTimer;
        }

        set cuteTimer(timer) {
            this._cuteTimer = timer;
        }

        get cuteTimer() {
            return this._cuteTimer;
        }

        set lastUpdateTime(time) {
            this._lastUpdateTime = time;
        }

        get lastUpdateTime() {
            return this._lastUpdateTime;
        }

        set cuted(cuted) {
            this._cuted = cuted;
        }

        get cuted() {
            return this._cuted;
        }

        set sad(rubbishNum) {
            let emotionValue = JSON.parse(Const.get("HomeBadEmotionValue"));
            this._sad = rubbishNum * Number(emotionValue.Rubbish) + (this.hunger ? emotionValue.Hunger : 0) + (this.thirst ? emotionValue.Thirsty : 0) + (this.dirty ? emotionValue.Dirty : 0);
        }

        get sad() {
            return this._sad;
        }

        get happy() {
            let emotionEffects = Const.getArray("HomeBadEmotionEffect");
            let emotion = null;
            for (const emotionEffect of emotionEffects) {
                if (this.sad >= Number(emotionEffect.key)) {
                    emotion = emotionEffect.effect;
                }
            }
            return emotion == "Good" ? true : false;
        }

        set hungerTimer(timer) {
            this._hungerTimer = timer;
        }

        get hungerTimer() {
            return this._hungerTimer;
        }

        set hunger(value) {
            this._hunger = value;
        }

        get hunger() {
            return this._hunger;
        }

        set thirstTimer(timer) {
            this._thirstTimer = timer;
        }

        get thirstTimer() {
            return this._thirstTimer;
        }

        set thirst(value) {
            this._thirst = value;
        }

        get thirst() {
            return this._thirst;
        }

        set dirtyTimer(timer) {
            this._dirtyTimer = timer;
        }

        get dirtyTimer() {
            return this._dirtyTimer;
        }

        set dirty(value) {
            this._dirty = value;
        }

        get dirty() {
            return this._dirty;
        }

        pos(x, y) {
            this._x = x;
            this._y = y;
        }

        get x() {
            return this._x;
        }

        get y() {
            return this._y;
        }

        get speed() {
            return 300;
        }

        dumpParams() {
            return {
                "lastUpdateTime": this.lastUpdateTime, "cuteTimer": this.cuteTimer,
                "funTimer": this.funTimer, "cuted": this.cuted, "x": this.x, "y": this.y,
                "hungerTimer": this.hungerTimer, "hunger": this.hunger, "thirstTimer": this.thirstTimer, "thirst": this.thirst, "dirtyTimer": this.dirtyTimer, "dirty": this.dirty
            };
        }

        restoreParams(params) {
            this.lastUpdateTime = params.lastUpdateTime;
            this.cuteTimer = params.cuteTimer;
            this.funTimer = params.funTimer;
            this.cuted = params.cuted;
            this.hungerTimer = params.hungerTimer;
            this.hunger = params.hunger;
            this.thirstTimer = params.thirstTimer;
            this.thirst = params.thirst;
            this.dirtyTimer = params.dirtyTimer;
            this.dirty = params.dirty;
            this.pos(params.x, params.y);
        }

        clearParams() {
            this.lastUpdateTime = null;
            this.cuteTimer = null;
            this.funTimer = null;
            this.cuted = null;
            this.hungerTimer = null;
            this.thirstTimer = null;
            this.dirtyTimer = null;
            this.pos(null, null);
        }

        set increase(v) {
            this._increase = v;
        }

        get increase() {
            return this._increase;
        }

        clear() {
            this.intimacy = 0;
            this.visitTimes = 0;
            this.adopted = false;
            this.newCome = true;
            this.redPointStatus = 0;
            this._inHome = false;
            this._inNautical = false;
            this._dailyIntimacy = 0;
            this._fakeInHome = null;

            this.clearParams();

            this._increase = null;
        }

        //指定的亲密度是否可领养
        canAdoptByIntimacy(intimacy) {
            if (this.getAdopted()) return false;
            let intimacyLimit = this.getAdoptLimit()["Intimacy"];
            return intimacy >= intimacyLimit && this.visitTimes > 0;
        }

        set level(value) {
            if(value!=this._level) {
                this._level = value;
                this._attributeChanged = true;
            }
        }

        get level() {
            return this._level;
        }

        set star(value) {
            if(value!=this._star) {
                this._star = value;
                this._attributeChanged = true;
            }
        }

        get star() {
            return this._star;
        }

        get Potential() {
            return this.config.Potential;
        }

        get Flag() {
            return this.config.Flag;
        }

        _calcAttribute() {
            let potentialRate = Number(Const.getMap("CatCultivatePotentialRate")[this.Potential]);
            let flagRate = Const.getMap("CatCultivateFlagRate")[this.Flag];
            let qualityRate = this._star>0?Number(Const.getMap("CatCultivateQualityRate")[this._star-1]):0;
            this._attribute = {};
            for(const attr of ["STA", "INT", "SPE"]) {
                this._attribute[attr] = (this._baseValue[attr] + flagRate[attr]*(qualityRate + this._level)) * potentialRate;
            }
        }

        get attribute() {
            if(this._attributeChanged) {
                this._calcAttribute();
            }
            return this._attribute;
        }

        canLevelUp() {
            if (!this.adopted) {
                return false;
            }
            let level = this.level;
            let costList = Const.getMap("CatCultivateLvUpCost");
            if (costList[level - 1] != null) {
                let cost = Math.floor(costList[level - 1] * Const.getArray("CatCultivateLvUpCostPRate")[this.Potential]);
                let medal = PlayerSystem.getMedal();
                return cost <= medal;
            }
            return false;
        }

        canStarUp() {
            if (!this.adopted) {
                return false;
            }
            let level = this.level;
            let starUpCondition = Const.getMap("CatCultivateQualityUpCondition");
            let condition = starUpCondition[String(this.star + 1)];
            if (condition != null) {
                if (condition.Condition > level) {
                    return false;
                }
                let book = PlayerSystem.getBook();
                let cost = Math.floor(starUpCondition[String(this.star + 1)].Cost * Const.getArray("CatCultivateQualityUpCostRate")[this.Potential]);
                return cost <= book;
            }
            return false;
        }
    }

    class PetReward {

        constructor(configId) {
            this.id = configId;
            this.intimacyIncr = 0;
            this.visitTimesIncr = 0;
            this.taskVisitTimesIncr = 0;
        }

        addIntimacy(intimacy) {
            this.intimacyIncr += intimacy;
        }

        addVisitTimes(visitTimes) {
            this.visitTimesIncr += visitTimes;
            this.taskVisitTimesIncr += visitTimes;
        }

        beachAddVisitTimes(visitTimes) {
            this.visitTimesIncr += visitTimes;
        }

        dumpAndStash() {
            if(this.stash){
                this.intimacyIncr += this.stash.intimacyIncr?this.stash.intimacyIncr:0;
                this.visitTimesIncr += this.stash.visitTimesIncr?this.stash.visitTimesIncr:0;
            }
            this.stash = null;
            if(this.intimacyIncr>0 || this.visitTimesIncr>0) {
                this.stash = {"intimacyIncr":this.intimacyIncr, "visitTimesIncr":this.visitTimesIncr};
                this.intimacyIncr = 0;
                this.visitTimesIncr = 0;
                return this.stash;
            }
            return null;
        }

        getTaskVisitTimes() {
            let taskVisitTimesIncr = this.taskVisitTimesIncr;
            this.taskVisitTimesIncr = 0;
            return taskVisitTimesIncr;
        }

        dropStash() {
            this.stash = null;
            if(this.intimacyIncr>0 || this.visitTimesIncr>0) {
                return true;
            }
            return false;
        }

        clear() {
            this.intimacyIncr = 0;
            this.visitTimesIncr = 0;
            this.stash = null;
        }

        getCurrentAddIntimacy() {
            return this.intimacyIncr;
        }

    }

    class PetDataMgr {

        constructor() {
            this._petsMap = new Map();
            this._petRewards = new Map();
            this._toysMap = new Map();
            this._itemMap = new Map();
            this.initPets();
        }

        static getInstance() {
            return PetDataMgr.instance?PetDataMgr.instance:PetDataMgr.instance = new PetDataMgr();
        }

        initPets() {
            let configs = ConfigMgr$1.getInstance().getConfig("Role");
            let petsMap = this._petsMap;
            for(const [petId, info] of configs) {
                let pet = new PetData(petId, info);
                petsMap.set(petId, pet);
            }
        }

        clear() {
            this._petsMap.forEach((pet)=>{
                pet.clear();
            });

            this._petRewards.clear();
            this._toysMap.clear();
            this._petsMap.clear();
            this._itemMap.clear();
            this.initPets();
        }

        getAllPets() {
            return this._petsMap;
        }

        getPet(petId) {
            return this._petsMap.get(petId);
        }

        getPetValueBase(petId){
            return this._petsMap.get(petId).getValueBase();
        }

        hasToy(toyId) {
            return this._toysMap.get(toyId);
        }

        addToy(toyId) {
            this._toysMap.set(toyId, true);
            let config = ConfigMgr$1.getInstance().getItemById("CatToys", toyId);
            if (config && config.Type == "ITEM") {
                this._itemMap.set(toyId, config);
            }
        }

        updatePet(petId, petInfo) {
            let pet = this.getPet(petId);
            if(!pet){
                return false;
            }
            let intimacy = petInfo.intimacy;
            let visitTimes = petInfo.visitTimes;
            let adopted = petInfo.adopted;
            let newCome = petInfo.newCome;
            let homeStatus = petInfo.homeStatus;
            let inNautical = petInfo.inNautical;
            let dailyIntimacy = petInfo.dailyIntimacy; // 今天获取到的亲密度
            let storyUnlock = petInfo.storyUnlock;
            let catLevel = petInfo.catLevel;
            let catStarGrade = petInfo.catStarGrade;
            let favor=petInfo.favor;

            //因为计算红点逻辑的遗留问题，喜爱值 在访问前赋值
            if(typeof(favor) != "undefined" && favor != null) {
                pet.favor = favor;
            }
            if(typeof(visitTimes) != "undefined" && visitTimes != null) {
                if(visitTimes > pet.getVisitTimes()) {
                    //防止服务器的数据更新略慢，因为客户端是定时同步这些数据给服务器
                    pet.setVisitTimes(visitTimes);
                }
            }
            if(typeof(adopted) != "undefined" && adopted != null) {
                pet.setAdopted(adopted);
            }
            if(typeof(intimacy) != "undefined" && intimacy != null) {
                if(intimacy > pet.getIntimacy()) {
                    pet.setIntimacy(intimacy);
                }
            }
            if(typeof(newCome) != "undefined" && newCome != null) {
                if(pet.getNewCome()) {
                    //如果原来的newCome已经是false了，则不用再改，防止后端数据较慢的情况导致newCome的更新错误
                    pet.setNewCome(newCome);
                }
            }

            if(typeof(homeStatus) != "undefined" && homeStatus != null) {
                pet.setHomeStatus(homeStatus);
            }

            if(typeof(inNautical) != "undefined" && inNautical != null) {
                pet.inNautical = inNautical;
            }

            if(typeof(dailyIntimacy) != "undefined" && dailyIntimacy != null) {
                pet.dailyIntimacy = dailyIntimacy;
            }

            if(typeof(storyUnlock) != "undefined" && storyUnlock != null) {
                pet.storyUnlock = storyUnlock;
            }

            if(typeof(catLevel) != "undefined" && catLevel != null) {
                pet.level = catLevel;
            }

            if(typeof(catStarGrade) != "undefined" && catStarGrade != null) {
                pet.star = catStarGrade;
            }
       
            return pet;
        }

        getPetReward(petId) {
            let petReward = this._petRewards.get(petId);
            if(!petReward){
                petReward = new PetReward(petId);
                this._petRewards.set(petId, petReward);
            }
            return petReward;
        }

        addIntimacy(petId, intimacy) {
            let pet = this.getPet(petId);
            if(!pet || intimacy==null || intimacy==0) {
                return false;
            }
            let current = pet.getIntimacy();
            let max = pet.getIntimacyMax();
            if (current >= max) {
                return false;
            }
            intimacy = Math.min(intimacy, max - current);
            pet.addIntimacy(intimacy);
            let petReward = this.getPetReward(petId);
            petReward.addIntimacy(intimacy);
            return true;
        }

        addVisitTimes(petId, visitTimes) {
            let pet = this.getPet(petId);
            if(!pet || visitTimes==null || visitTimes==0) {
                return false;
            }
            pet.addVisitTimes(visitTimes);
            let petReward = this.getPetReward(petId);
            petReward.addVisitTimes(visitTimes);
            return true;
        }

        beachAddVisitTimes(petId, visitTimes){
            let pet = this.getPet(petId);
            if(!pet || visitTimes==null || visitTimes==0) {
                return false;
            }
            pet.addVisitTimes(visitTimes);
            let petReward = this.getPetReward(petId);
            petReward.beachAddVisitTimes(visitTimes);
            return true;
        }

        adoptPet(petId) {
            let pet = this.getPet(petId);
            if(!pet){
                return false;
            }
            pet.setAdopted(true);
            return true;
        }

        dumpAndStashPetRewards() {
            let result = null;
            for(const [petId, petReward] of this._petRewards) {
                let info = petReward.dumpAndStash();
                if(info){
                    if(!result){
                        result = {};
                    }
                    result[petId] = info;
                }
            }
            return result;
        }

        getTaskVisitTimes() {
            let times = 0;
            let map = new Map();
            for (const [petId, petReward] of this._petRewards) {
                let t = petReward.getTaskVisitTimes();
                map.set(petId, t);
                times += t;
            }
            let obj = { times: times, result: map };
            return obj;
        }


        dropPetRewardsStash() {
            for(const [petId, petReward] of this._petRewards) {
                petReward.dropStash();
            }
        }

        clearPetRewards() {
            this._petRewards.clear();
        }

        toyUnlock(config){
            let needToyId = JSON.parse(config.AdoptLimit).CatToys;
            let hasToy = true;
            if (needToyId) {
                needToyId = String(needToyId);
                hasToy = this.hasToy(needToyId);
            }
            return hasToy ? true : false;
        }

        needRedPoint() {
            let needRedPoint = false;
            for(const [petId, petData] of this._petsMap) {
                needRedPoint =  needRedPoint || (this.toyUnlock(petData.config) ? petData.needRedPoint() : false);
            }
            return needRedPoint;
        }

        //是否有可领养的猫
        needGuide() {
            let needGuide = false;
            for(const [petId, petData] of this._petsMap) {
               if(petData.canAdopt()){
                needGuide =  petData.canAdopt();
               }
            }
            return needGuide;
        }

        //将所有状态为1的petData，置为2，表示已读取红点，之后不需要再显示
        clearAllRedPoint() {
            for(const [petId, petData] of this._petsMap) {
                if(petData.getRedPointStatus() == 1) {
                    petData.setRedPointStatus(2);
                }
            }
        }

        //将所有的来访次数大于0，并且newCome为true的置为false
        clearAllNewCome() {
            for(const [petId, petData] of this._petsMap) {
                if(petData.getNewCome() && petData.getVisitTimes() > 0) {
                    petData.setNewCome(false);
                }
            }
        }

        //家园

        get PetsAdopted() {
            let res = new Array();
            for(const [petId, pet] of this._petsMap) {
                if(pet.getAdopted())
                    res.push(pet);
            }
            return res;
        }

        get PetIdsAdopted() {
            let res = new Array();
            for(const [petId, pet] of this._petsMap) {
                if(pet.getAdopted())
                    res.push(petId);
            }
            return res;
        }

        get PetsFakeInHome() {
            let res = new Array();
            for(const [petId, pet] of this._petsMap) {
                if(pet.fakeInHome)
                    res.push(pet);
            }
            return res;
        }

        updatePetsInhome(petsInHome) {
            for(const [petId, pet] of this._petsMap) {
                pet.tempInHome = false;
            }
            for(const petId of petsInHome) {
                this._petsMap.get(petId).tempInHome = true;
            }

            for(const [petId, pet] of this._petsMap) {
                if(pet.homeStatus!=pet.tempInHome) {
                    pet.homeStatus = pet.tempInHome;
                }
                pet.tempInHome = null;
            }
        }

        setPetsInNautical(team) {
            for(const petId of team) {
                this._petsMap.get(petId).inNautical = true;
            }
        }

        updateUnlockPetToys(unlockPetToys) {
            this._toysMap.clear();
            for(const toyId of unlockPetToys) {
                this._toysMap.set(toyId, true);
                let config = ConfigMgr$1.getInstance().getItemById("CatToys", toyId);
                if (config && config.Type == "ITEM") {
                    this._itemMap.set(toyId, config);
                }
            }
        }

        dumpHomeParams() {
            let home = new Array();
            for(const [petId, pet] of this._petsMap) {
                if(pet.homeStatus) {
                    let params = pet.dumpParams();
                    if(params)
                        home.push({"id":petId, "params": params});
                }
            }
            return home;
        }

        getIfNeedUpdataToAdopt(petId) {
            let pet = this.getPet(petId);
            let petReward = this.getPetReward(petId);
            let truePetIntimacy = pet.getIntimacy() - petReward.getCurrentAddIntimacy();
            return !pet.canAdoptByIntimacy(truePetIntimacy);
        }

        setRubbishNum(num) {
            let pets = this.PetsFakeInHome;
            for (const pet of pets) {
                pet.sad = num;
            }
        }

        getItemBySpecialType(id) {
            let arr = [];
            this._itemMap.forEach(element => {
                if (element.SpecialFunctionType == id) {
                    arr.push(element);
                }
            });
            return arr;
        }
    }

    let trim = function(num, divisor, unit) {
        if(num%divisor==0)
            return String(num/divisor)+unit;
        else {
            return String((num / divisor).toFixed(1)) + unit;
        }
    };

    class Strings$1 {

        static get(textId, arg = []) {
            let item = ConfigMgr$1.getInstance().getItemById("Translate", textId);
            if (item != null) {
                let str = String(item.zh_CH);
                if (arg && arg.length > 0) {
                    for (let i = 1; i <= arg.length; i++) {
                        let s = "%" + i + "%";
                        if (str != null) {
                            str = str.replace(s, String(arg[i - 1]));
                        }
                    }
                }
                return str;
            }
            return null;
        }

        static short(num) {
            let K = 1000;
            let M = K * K;
            let G = K * M;
            if(num<100 * K) {
                return String(num);
            }else if(num < 100 * M) {
                return trim(num, K, "K");
            }else if(num < 10 * G) {
                return trim(num, M, "M");
            }else  {
                return trim(num, G, "G");
            }
        }

        static dealFishNumberText(num,type){
            let K = 1000;
            let M = K * K;
            let G = K * M;
            let N = "";
            if(type == 1){
                if(num < 0){
                    return;
                }
                if(num < 100 * K){
                    N = num | 0 + "";
                }
                else if(num < 100 * M){
                    N = (num / K).toFixed(0) + "K";
                }
                else if(num < 100 * G){
                    N = (num / M).toFixed(0) + "M";
                }
                else {
                    N = (num / G).toFixed(0) + "G";
                }
            }
            else {
                if(num < 0){
                    return;
                }
                if(num < 100 * K){
                    N = num | 0 + "";
                }
                else if(num < 100 * M){
                    N = (num / K).toFixed(1) + "K";
                }
                else if(num < 100 * G){
                    N = (num / M).toFixed(1) + "M";
                }
                else {
                    N = (num / G).toFixed(1) + "G";
                }
            }
            return N;
        }

        static floatFishNumberText1(num){
            if(num < 0){
                return ;
            }
            if(num < 1000){
                return num.toString();
            }
            if(num < 1000000){
                return ((num/100|0)/10).toFixed(1).toString() + "K";
            }
            if(num < 1000000000){
                return ((num/100000|0)/10).toFixed(1).toString() + "M";
            }
            return ((num/100000000|0)/10).toFixed(1).toString() + "G";
        }

        static floatFishNumberText(num){
            if(num < 0){
                return ;
            }
            if(num < 100000){
                return num.toString();
            }
            if(num < 100000000){
                return ((num/100|0)/10).toFixed(1).toString() + "K";
            }
            if(num < 100000000000){
                return ((num/100000|0)/10).toFixed(1).toString() + "M";
            }
            return ((num/100000000|0)/10).toFixed(1).toString() + "G";
        }

        static getTimeBySeconds(seconds) {
            let str = "";
            if (seconds >= 0 && seconds < 300) {
                str = Strings$1.get("tid#UI_149");
            } else if (seconds >= 300 && seconds < 3600) {
                str = Math.floor(seconds / 60) + " " + Strings$1.get("tid#UI_150")+ " ";
            } else if (seconds >= 3600 && seconds < 86400) {
                str = Math.floor(seconds / 3600) + " " + Strings$1.get("tid#UI_151")+ " ";
            } else {
                str = Math.floor(seconds / 86400) + " " + Strings$1.get("tid#UI_152")+ " ";
            }
            return str;
        }

        static getDaysBySeconds(seconds) {
            let str = Strings$1.get("tid#UI_153");
            return Math.ceil(seconds / 86400) + " " + str+ " ";
        }

        static getTimeByUTCMilliseconds(time) {
            let date = new Date(time);
            let year = date.getFullYear(time);
            let mouth = date.getMonth(time) + 1;
            let day = date.getDate(time);
            let str = year + "-" + mouth + "-" + day;
            return str;
        }

        static getLeftTimeBySeconds(seconds) {
            let str = "";
            if (seconds >= 0 && seconds < 60) {
                // str = seconds + "秒";
                // str = Strings.get("tid#UI_119");
                str = seconds + " " + Strings$1.get("tid#UI_119");
            } else if (seconds >= 60 && seconds < 3600) {
                // str = Math.floor(seconds / 60) + "分钟";
                // str = Strings.get("tid#UI_154");
                str = Math.floor(seconds / 60) + " " + Strings$1.get("tid#UI_154");
            } else if (seconds >= 3600 && seconds < 86400) {
                // str = Math.floor(seconds / 3600) + "小时";
                // str = Strings.get("tid#UI_155");
                str = Math.floor(seconds / 3600) + " " + Strings$1.get("tid#UI_155");
            } else {
                // str = Math.floor(seconds / 86400) + "天";
                // str = Strings.get("tid#UI_153");
                str = Math.floor(seconds / 86400) + " " + Strings$1.get("tid#UI_153");
            }
            return str;
        }
    }

    class TipsPopMgr {

        constructor() {
            this.tipsArr = [];
            this.isShowing = false;
        }

        static getInstance() {
            return TipsPopMgr.instance ? TipsPopMgr.instance : TipsPopMgr.instance = new TipsPopMgr();
        }

        init() {
            EventMgr.getInstance().registEvent("SHOW_TIPS", this, this.showTips);
            EventMgr.getInstance().registEvent("SHOW_TIPS_BY_ID", this, this.showTipsById);
            EventMgr.getInstance().registEvent("SHOW_ANIM_TIP", this, this.showAnimTip);
        }

        clearTips() {
            this.tipsArr = [];
        }

        addTips(tip, hideHandler, time, hasBg, up, offsetY) {
            let obj = { tip: tip, hideHandler: hideHandler, time: time, hasBg: hasBg, up: up, offsetY: offsetY };
            this.tipsArr.push(obj);
        }

        popTips() {
            let tip = this.tipsArr[0];
            this.tipsArr.splice(0, 1);
            return tip;
        }

        getTipsNum() {
            return this.tipsArr.length;
        }

        showTips(tip, hideHandler = null, time = 1500, hasBg = true, up = false, offsetY = 0) {
            if (tip == null) {
                return;
            }
            if (GameContext.isGuiding) {
                return;
            }
            this.addTips(tip, hideHandler, time, hasBg, up, offsetY);
            if (this.isShowing) {
                return;
            }
            this.isShowing = true;
            if (this.tipUI) {
                this._show();
            } else {
                this.loadTipsUI(Laya.Handler.create(this, this._show));
            }
        }

        showTipsById(tipId, arg = [], hideHandler = null, time = 1500, hasBg = true, up = false, offsetY = 0) {
            let tip = Strings$1.get(tipId, arg);
            this.showTips(tip, hideHandler, time, hasBg, up, offsetY);
        }

        _show() {
            this.isShowing = true;
            this.tipUI.zOrder = 999999;
            Laya.stage.addChild(this.tipUI);
            let nowTip = this.popTips();
            EventMgr.getInstance().postEvent("tipShow", nowTip);
        }

        showComplete() {
            this.isShowing = false;
            if (this.getTipsNum() > 0) {
                this._show();
            }
        }

        loadTipsUI(completeHandler) {
            this.loadResource("prefab/ui/TipsPop.prefab", completeHandler);
        }

        loadResource(path, completeHandler) {
            if (path.indexOf(".prefab") != -1) {
                Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                    this.tipUI = prefabDef.create();
                    this.tipUI.x = Laya.stage.width / 2;
                    this.tipUI.y = Laya.stage.height / 2;
                    if (completeHandler != null) {
                        completeHandler.run();
                    }
                }));
            }
        }

        showAnimTip(type) {
            if (this._anim && this._anim.destroyed) {
                this._anim.destroy();
            }
            this._anim = new Spine();
            this._anim.addCompleteHandler(Laya.Handler.create(this, function () {
                if (this._anim && this._anim.destroyed) {
                    this._anim.destroy();
                }
                this._anim = null;
            }));
            this._anim.loadAnim("anim3/shengjidonghua.sk", type);
            this._anim.x = Laya.stage.width / 2;
            this._anim.y = 800;
            this._anim.zOrder = 1000000;
            Laya.stage.addChild(this._anim);
            this._anim.playAnim("chenggong", false);
        }

    }

    class BarrageItem extends Laya.Box {

        constructor() {
            super();
        }

        onAwake() {
            this.mouseEnabled = false;
            this.barrageText = new Laya.Text();
            this.barrageText1 = new Laya.Text();
            this.barrageText.fontSize = 45;
            this.barrageText1.fontSize = 45;
            this.barrageText.bold=true;
            this.barrageText1.bold=true;
            this.addChild(this.barrageText);
            this.addChild(this.barrageText1);
            this.barrageImage = new Laya.Image();
            this.addChild(this.barrageImage);
            this.barrageText.mouseEnabled = false;
            this.barrageImage.mouseEnabled = false;
        }

        onEnable() {
            let str = Strings$1.get(this.config.Desc);
            let str1 = str.substring(Math.floor(str.length / 2));
            str = str.substring(0, Math.floor(str.length / 2));
            this.barrageText.text = str;
            this.barrageText.color = this.color;
            this.barrageText1.text = str1;
            this.barrageText1.color = this.color;
            this.barrageText1.x = this.barrageText.width;
            if (this.config.Picture != null && this.config.Picture != "") {
                Laya.loader.load(this.config.Picture, Laya.Handler.create(this, function () {
                    this.barrageImage.skin = this.config.Picture;
                    this.barrageImage.width *= 64 / this.barrageImage.height;
                    this.barrageImage.height = 64;
                    this.barrageText.x = this.barrageImage.width;
                }));
            }
            this.width = this.barrageText.x + this.barrageText.width + this.barrageText1.width;
            this.time = (Laya.stage.width + this.width) * Number(Const.get("BarrageSpeed"));
            this.x = Laya.stage.width;
            this.tween();
          
        }

        setValue(config, color, obj, handler) {
            this.config = config;
            this.color = color;
            this.aisleIndex = obj.index;
            let arr = obj.arr;
            this.y = Utils.randomExt2(arr[0] + 32, arr[1] - 32);
            this.handler = handler;
        }

        tween() {
            Laya.Tween.to(this, { x: 0 - this.width }, this.time, null, Laya.Handler.create(this, function () {
                this.clear();
            }));
        }

        clear() {
            Laya.Tween.clearAll(this);
            if (this.handler != null && this.aisleIndex != null) {
                this.handler.runWith(this.aisleIndex);
            }
            this.handler = null;
            this.aisleIndex = null;
            this.config = null;
            this.visible = false;
            this.barrageText.x = 0;
            this.barrageImage.skin = null;
            this.removeSelf();
            this.destroy();
            // Laya.Pool.recover("barrageItem", this);
        }
    }

    class BarrageMgr {

        constructor() {
            this._init();
        }

        static getInstance() {
            return BarrageMgr.instance ? BarrageMgr.instance : BarrageMgr.instance = new BarrageMgr();
        }

        _init() {
            this.openDown=false;
            this._barrageMap = new Map();
            let start = 300;
            this.aisleNum = 15;
            let aisleHeight = 100;
            this.aises = [];
            this._randomY_map = new Map();
            for (let i = 0; i < this.aisleNum; i++) {
                this.aises.push([start + aisleHeight * i, start + aisleHeight * i + aisleHeight]);
                this._randomY_map.set(i, this.aises[i]);
            }
            this._currentArray = [];
            this._time = 0;
            this._nextTime = 0;
            this._initConfig();
            this._initUI();
            EventMgr.getInstance().registEvent("SWITCH_SCENE_COMPLETE", this, this.autoRandomPlay);
            EventMgr.getInstance().registEvent("CLEAR_ALL_BARRAGES", this, this.onBlur);
            EventMgr.getInstance().registEvent("RESTART_BARRAGES", this, this.onFocus);
            if (!Laya.Browser.onMiniGame) {
                Laya.stage.on(Laya.Event.FOCUS, this, this.onFocus);
                Laya.stage.on(Laya.Event.BLUR, this, this.onBlur);
            } else {
                let obj = this;
                wx.onShow(function () {
                    Laya.timer.frameOnce(1, obj, obj.onShow);
                });
            }
        }

        _initUI() {
            this._barrageBox = new Laya.Box();
            this._barrageBox.width = Laya.stage.width * 3;
            this._barrageBox.height = Laya.stage.height;
            this._barrageBox.mouseEnabled = false;
            this._barrageBox.zOrder = 1;
            this._barrageBox.visible=!this.getOpenDown();
            Laya.stage.addChild(this._barrageBox);
        }

        _initConfig() {
            this._config = ConfigMgr$1.getInstance().getConfig("Barrage");
        }

        startWithGroup(group, delay) {
            this.groupConfig = group;
            this.timerClear();
            this.show();
            this.getCurrentBarrage();
            Laya.timer.once(delay * 1000, this, this.startLoop);
        }

        startLoop() {
            if (GameContext.isGuiding) {
                this.stop();
                return;
            }
            Laya.timer.frameLoop(10, this, this.updata);
        }

        randomStart(randomGroupId, randomDelayId) {
            if (GameContext.isGuiding) {
                return;
            }
            let groupArr = Const.getArray(randomGroupId);
            let delay = 0;
            if (randomDelayId) {
                let timeArr = Const.getArray(randomDelayId);
                delay = Utils.randomExt2(timeArr[0], timeArr[1]);
            }
            let unlockGroupArr = [];
            groupArr.forEach(groupId => {
                let groupConfig = ConfigMgr$1.getInstance().getItemById("BarrageGroup", groupId);
                if (groupConfig.Star <= PlayerSystem.getStar()) {
                    unlockGroupArr.push(groupConfig);
                }
            });
            if (unlockGroupArr.length > 0) {
                let group = unlockGroupArr[Math.floor(Utils.randomExt(unlockGroupArr.length))];
                this.startWithGroup(group, delay);
            }
        }

        updata() {
            if (this._nextTime == this._time) {
                this.playOnce();
            }
            this._time++;
        }

        playOnce() {
            this.randomNextTime();
            let obj = this.getRandomY();
            if (obj == null) {
                return;
            }
            // let barrageItem = Laya.Pool.getItemByCreateFun("barrageItem", this.createBarrageItem, this);
            let barrageItem = new BarrageItem();
            this._barrageMap.set(obj.index, barrageItem);
            barrageItem.setValue(this.nextConfig(), this.getRandomColor(), obj, Laya.Handler.create(this, this.barragePlayComplete));
            this._barrageBox.addChild(barrageItem);
            this._barrageBox.visible=!this.getOpenDown();
        }

        getRandomColor() {
            return Constants.BarrageColors[Math.floor(Utils.randomExt2(0, Constants.BarrageColors.length))];
        }

        getRandomY() {
            let keys = Array.from(this._randomY_map.keys());
            let randomKey = keys[Math.floor(Utils.randomExt(keys.length))];
            if (!this._randomY_map.has(randomKey) || !randomKey) {
                return null;
            }
            let ret = { index: randomKey, arr: this._randomY_map.get(randomKey) };
            this._randomY_map.delete(randomKey);
            return ret;
        }

        randomNextTime() {
            this._nextTime = this._time + Math.floor(Utils.randomExt2(1, 5));
        }

        nextConfig() {
            let config = this._currentArray[0];
            this._currentArray.splice(0, 1);
            if (this._currentArray.length == 0) {
                this.stop();
            }
            return config;
        }

        stop() {
            this._currentArray = [];
            this.groupConfig = null;
            this.timerClear();
        }

        timerClear() {
            Laya.timer.clear(this, this.updata);
            Laya.timer.clear(this, this.startLoop);
        }

        hide() {
            //this._barrageBox.visible = false;
        }

        show() {
            //this._barrageBox.visible = true;
        }

        getCurrentBarrage() {
            this.groupConfig.Group.forEach(element => {
                this._currentArray.push(this._config.get(element));
            });
            let fun = function (a, b) {
                return a.Sort - b.Sort;
            };
            this._currentArray.sort(fun);
        }

        autoRandomPlay(sceneId) {
            if (sceneId == Constants.FishingSceneId) {
                Laya.timer.clear(this, this.autoPlay);
                this.timerClear();
                this.hide();
            } else {
                let timeArr = Const.getArray("BarrageTriggerTime3");
                this._autoTime = Utils.randomExt2(timeArr[0], timeArr[1]);
                Laya.timer.clear(this, this.autoPlay);
                Laya.timer.once(this._autoTime * 1000, this, this.autoPlay);
                this.show();
            }
        }

        autoPlay() {
            this.randomStart("BarrageCycle");
            this.autoRandomPlay();
        }

        onFocus() {
            this.autoRandomPlay();
        }

        onBlur() {
            Laya.timer.clear(this, this.autoPlay);
            this.timerClear();
            this.hide();
            this._barrageMap.forEach((value, key) => {
                Laya.Tween.clearAll(value);
                value.clear();
            });
        }

        onShow() {
            this.onBlur();
            this.onFocus();
        }

        barragePlayComplete(index) {
            this._randomY_map.set(index, this.aises[index]);
            this._barrageMap.delete(index);
        }

        clear() {
            Laya.timer.clear(this, this.autoPlay);
            this.timerClear();
            this._barrageMap.forEach((value, key) => {
                Laya.Tween.clearAll(value);
                value.clear();
            });
            this._currentArray = [];
            this._time = 0;
            this._nextTime = 0;
            this._barrageMap.clear();
            this._randomY_map.clear();
            this.aises = [];
            let start = 300;
            this.aisleNum = 15;
            let aisleHeight = 100;
            for (let i = 0; i < this.aisleNum; i++) {
                this.aises.push([start + aisleHeight * i, start + aisleHeight * i + aisleHeight]);
                this._randomY_map.set(i, this.aises[i]);
            }
        }
        getOpenDown(){
          let a=Laya.LocalStorage.getJSON("barrage");
           if(Laya.LocalStorage.getJSON("barrage")==null){
              Laya.LocalStorage.setJSON("barrage",true); 
           }
           if(Laya.LocalStorage.getJSON("barrage")==false){
            return false;
           }
           else if(Laya.LocalStorage.getJSON("barrage")==true){
            return true;
           }
            
           
            
        }
        setOpenDown(openDown){
            // let barrageData={
            //     barrage: openDown
            // }
            Laya.LocalStorage.setJSON("barrage",openDown);
            this._barrageBox.visible=!this.getOpenDown();
        }
    }

    class TaskMainData {

        constructor(taskId) {
            this.config = ConfigMgr$1.getInstance().getItemById("TaskMain", taskId);
        }

        getId() {
            return this.config.Id;
        }

        getDesc() {
            return this.config.Desc;
        }

        getConditionStart() {
            return this.config.ConditionStart;
        }

        setProgress(current, target) {
            if (current == null) {
                current = 0;
            }
            this.current = current;
            this.target = target;
            return true;
        }

        getCurrent() {
            return this.current ? (this.current > this.getTarget() ? this.getTarget() : this.current) : 0;
        }

        getTarget() {
            return this.target ? this.target : this.config.ConditionValue;
        }

        getGuide() {
            return this.config.Guide;
        }

        getReward() {
            return this.config.Reward;
        }

        getStartDialog() {
            return this.config.StartDialog;
        }

        getEndDialog() {
            return this.config.EndDialog;
        }

        getConditionType() {
            return this.config.ConditionType;
        }

        addProgress(num) {
            if (this.current >= this.getTarget()) {
                return false;
            }
            this.current += num;
            return true;
        }

        updataProgress(num) {
            if (this.current >= this.getTarget()) {
                return false;
            }
            this.current = num;
            return true;
        }

    }

    class TaskMainMgr {

        constructor() {
            this.currentTask = null;
            this._lastStar = PlayerSystem.getStar();
            this.configs = ConfigMgr$1.getInstance().getConfig("TaskMain");
            EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.checkTask, [false]);
            EventMgr.getInstance().registEvent("GUIDE_SET_GUIDING", this, this.checkTask);
            this._isTasking = false;
            this._lastComplete = true;
        }

        static getInstance() {
            return TaskMainMgr.instance ? TaskMainMgr.instance : TaskMainMgr.instance = new TaskMainMgr();
        }

        clear() {
            this.currentTask = null;
            this._lastStar = PlayerSystem.getStar();
            this.configs = ConfigMgr$1.getInstance().getConfig("TaskMain");
            this._isTasking = false;
            this._lastComplete = true;
        };

        updata(data) {
            if (data) {
                let keys = Object.getOwnPropertyNames(data);
                if (keys.length <= 0) {
                    this._isTasking = false;
                    this.currentTask = null;
                    return this._isTasking;
                }
                keys.forEach(key => {
                    let task = data[key];
                    let taskId = task.id;
                    let current = task.current;
                    let target = task.target;
                    let taskMain = new TaskMainData(taskId);
                    let conditionStart = JSON.parse(taskMain.getConditionStart()).Star;
                    if (this.currentTask == null || taskId != this.currentTask.getId() && conditionStart <= PlayerSystem.getStar()) {
                        this.currentTask = taskMain;
                    }
                    if (this.currentTask && conditionStart <= PlayerSystem.getStar()) {
                        this.currentTask.setProgress(current, target);
                        this._isTasking = true;
                    } else {
                        this.currentTask = null;
                        this._isTasking = false;
                    }
                });
            } else {
                this._isTasking = false;
                this.currentTask = null;
            }
            return this._isTasking;
        }

        checkTask(type = false) {
            if (type) {
                return;
            }
            let currentStar = PlayerSystem.getStar();
            let send = false;
            this.configs.forEach((element, key) => {
                if (send) {
                    return;
                }
                let conditionStart = JSON.parse(element.ConditionStart).Star;
                if (this._lastStar <= conditionStart && currentStar >= conditionStart) {
                    send = true;
                    this._lastStar = conditionStart;
                    EventMgr.getInstance().postEvent("SERVICE_GET_ALLTASKS");
                }
            });
            if (!send) {
                this._lastStar = currentStar;
            }
        }

        getCurrentData() {
            return this.currentTask;
        }

        getIsTasking() {
            return this._isTasking;
        }

        getLastComplete() {
            return this._lastComplete;
        }

        setLastComplete(type) {
            this._lastComplete = type;
        }
    }

    class LocationData {

        constructor() {
            this.locationData = null;
            this._curBuildingId = null;
        }

        setData(data) {
            this.locationData = data;
        }

        getId() {
            return this.locationData.Id;
        }

        getX() {
            return Number(this.locationData.Pos[0]);
        }

        getY() {
            return Number(this.locationData.Pos[1]);
        }

        getSelectionX() {
            return Number(this.locationData.PosSelection[0]);
        }

        getSelectionY() {
            return Number(this.locationData.PosSelection[1]);
        }

        getOrder() {
            return Number(this.locationData.Order);
        }

        getBaseType() {
            return this.locationData.BaseType;
        }

        getZOrder() {
            let zorder = Number(this.locationData.ZOrder);
            return zorder;
        }

        getListName() {
            return this.locationData.ListName;
        }

        getBaseSize() {
            return this.locationData.BaseSize;
        }

        getPathPointIndex() {
            return Number(this.locationData.PathPointIndex);
        }

        getFixedBuildings() {
            return this.locationData.FixedBuildings;
        }

        getBelongScene() {
            return this.locationData.BelongScene;
        }

        checkBuildingAvailable(buildingId) {
            let buildings = this.getFixedBuildings();
            let result = false;
            for(let element of buildings){
                if(element==buildingId){
                    result = true;
                    return true;
                }
            }
            return result;
        }

        setBuildingId(buildingId) {
            this._curBuildingId = buildingId;
        }

        getBuildingId() {
            return this._curBuildingId;
        }

        checkIsHigher(buildingId) {
            if(this.getBaseType()==1){
                let buildings = this.getFixedBuildings();
                if(!this._curBuildingId){
                    return true;
                }
                for(let element of buildings){
                    if(element==this._curBuildingId){
                        return true;
                    }
                    if(element==buildingId){
                        return false;
                    }
                }
            }else {
                return true;
            }
        }

        clear() {
            this._curBuildingId = null;
        }
    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class LocationDataMgr {

        constructor() {
            this.locationMap = new Map();
            this.initLocations();
        }

        static getInstance() {
            return LocationDataMgr.instance?LocationDataMgr.instance:LocationDataMgr.instance = new LocationDataMgr();
        }

        //初始化点位
        initLocations() {
            let config = ConfigMgr$1.getInstance().getConfig("BuildingBase");
            let outSelf = this;
            config.forEach(function (item) {
                let data = new LocationData();
                data.setData(item);
                outSelf.locationMap.set(Number(item["Id"]), data);
            });
        }

        clear() {
            this.locationMap.forEach((location) => {
                location.clear();
            });
            this.locationMap.clear();
            this.initLocations();
            this._scene2Buildings = null;
        }

        getLocation(id) {
            return this.locationMap.get(Number(id));
        }

        getLocationsByBaseType(baseType) {
            let list = new Array();
            this.locationMap.forEach((location)=>{
                if(location.getBaseType()==baseType){
                    list.push(location);
                }
            });
            function sortByOrder(a,b)
            {
                return a.getOrder()-b.getOrder();
            }
            list.sort(sortByOrder);
            return list;
        }

        getLocationByBelongScene(sceneId) {
            let list = new Array();
            this.locationMap.forEach((location)=>{
                if(location.getBelongScene()==sceneId){
                    list.push(location);
                }
            });
            function sortByOrder(a,b)
            {
                return a.getOrder()-b.getOrder();
            }
            list.sort(sortByOrder);
            return list;
        }

        getLocationByBelongScene2() {
            if (this._scene2Buildings != null) {
                return this._scene2Buildings;
            }
            this._scene2Buildings = [];
            let data = this.getLocationByBelongScene(2);
            let list = new Array();
            data.forEach(element => {
                list = list.concat(element.getFixedBuildings());
            });
            let ret = Array.from(new Set(list));
            let res = new Array();
            let series = null;
            let seriesType = null;
            for (const bid of ret) {
                let config = ConfigMgr$1.getInstance().getItemById("Building", String(bid));
                if (config) {
                    if (config.SeriesType != seriesType) {
                        seriesType = config.SeriesType;
                        series = [bid];
                        res.push(series);
                    } else {
                        series.push(bid);
                    }
                }
            }
            res.forEach(element => {
                this._scene2Buildings.push({ buildings: element });
            });
            return this._scene2Buildings;
        }

    }

    class BuildingData{

        constructor(bid) {
            this.catWeightedMap = new Map();
            this.catWeightedSum = 0;
            this.config = ConfigMgr$1.getInstance().getItemById("Building", String(bid));
            this.config.CustomerInvite.forEach(element => {
                if(element != "") {
                    let data = element.split(":");
                    this.catWeightedMap.set(String(data[0]), Number(data[1]));
                    this.catWeightedSum += Number(data[1]);
                }
            });
            this.locationData = null;


            this.spaceMap = new Map();

            this.attractCD = null;
            this.pointSetting = null;
        }

        getCatWeightedSum() {
            return this.catWeightedSum;
        }

        getId() {
            return this.config.Id;
        }

        getSource() {
            return "prefab/component/"+this.config.Source+".prefab";
        }

        getLocationData() {
            return this.locationData;
        }

        setLocationData(data) {
            this.locationData = data;
            if(data==null) {
                for(const [key, catId] of this.spaceMap) {
                    EventMgr.getInstance().postEvent("REMOVE_CAT", [catId,this.getBelongScene()]);
                }
                this.spaceMap.clear();
                this.attractCD = null;
            }
        }

        getCatWeightedMap() {
            return this.catWeightedMap;
        }

        getCatWeightedArray() {
            let ar = new Array();
            this.catWeightedMap.forEach((value, key) => {
                ar.push({w: value, n: key});
            });
            ar.sort(function(a, b) {
                return a.w < b.w;
            });
            return ar;
        }

        getBelongScene() {
            return this.config.BelongScene;
        }

        getCost() {
            return this.config.Cost;
        }

        getStar() {
            return this.config.StarReward;
        }

        getLimit() {
            return this.config.Limit;
        }

        getSpecialType() {
            return this.config.SpecialTpye;
        }

        getSpecialValue() {
            return this.config.SpecialValue;
        }

        getAutoIncome() {
            let ret = 0;
            if(this.config.AutoIncome != null) {
                ret = Number(this.config.AutoIncome);
            }
            return ret;
        }

        canStay() {
            return this.getLocationData()!= null && this.getCapacity()>0;
        }

        getCapacity() {
            return this.config.PointCount;
        }

        hasSpace() {
            for(let i = 0; i<this.getCapacity(); ++i) {
                if(!this.spaceMap.get(i)) {
                    return true;
                }
            }
            return false;
        }

        getSpaces() {
            let arr = new Array();
            for(let i = 0; i<this.getCapacity(); ++i) {
                if(!this.spaceMap.get(i)) {
                    arr.push(i);
                }
            }
            if(arr.length==0){
                return null;
            }

            return arr;
        }

        occupy(spaceId, catId) {
            spaceId = Number(spaceId);
            if(spaceId>=this.getCapacity()||spaceId<0) {
                return false;
            }
            let prevCat = this.spaceMap.get(spaceId);
            if(prevCat) {
                if(prevCat==catId) {
                    return true;
                }
                return false;
            }
            this.spaceMap.set(spaceId, catId);
            return true;
        }

        leave(spaceId, catId) {
            spaceId = Number(spaceId);
            if(spaceId>=this.getCapacity()||spaceId<0) {
                return false;
            }
            let prevCat = this.spaceMap.get(spaceId);
            if(!prevCat || prevCat!=catId) {
                return false;
            }
            this.spaceMap.delete(spaceId);
            return true;
        }

        isSupportAction(eventString) {
            for(let i = 0 ; i < this.config.SupportActions.length; ++i) {
                let d = this.config.SupportActions[i];
                if(d != null && d != ""
                 && eventString == d) {
                    return true;
                }
            }
            return false;
        }

        getPointSetting() {
            if (this.pointSetting == null) {
                if (this.config.PointSetting) {
                    this.pointSetting = JSON.parse(this.config.PointSetting);
                }
            }
            return this.pointSetting;
        }

        getMagnification() {
            return this.config.Magnification;
        }

        getBuildingBaseId() {
            let location = this.getLocationData();
            return location? location.getId(): null;
        }

    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class BuildingDataMgr {

        constructor() {
            this.buildingMap = new Map();

            this.buildingMap.set(1, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});
            this.buildingMap.set(2, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});
            this.buildingMap.set(3, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});

        }

        static getInstance() {
            return BuildingDataMgr.instance?BuildingDataMgr.instance:BuildingDataMgr.instance = new BuildingDataMgr();
        }

        //获得属于的scene
        getSceneIdByBid(bid) {
            let config = ConfigMgr$1.getInstance().getItemById("Building", String(bid));
            if(config){
                return config.BelongScene;
            }
            return ;
        }

        //获得建筑物列表
        getBuildingDataListInfo(sceneId) {
           return this.buildingMap.get(Number(sceneId));
        }

        //获取建筑物通过bid
        getBuildingDataByBid(bid) {
            let info = this.getBuildingDataListInfo(this.getSceneIdByBid(bid));
            if (info == null) {
                return null;
            }
            for(let i = 0 ; i< info.list.length ; ++i) {
                let b = info.list[i];
                if(String(b.getId()) == String(bid)) {
                    return b;
                }
            }
            return null;
        }

        hasBuildingDataByBid(buildingId) {
            let info = this.getBuildingDataListInfo(this.getSceneIdByBid(buildingId));
            if(info == null) {
                return false;
            }
            for(let i = 0 ; i < info.list.length ; ++i) {
                let b = info.list[i];
                if(String(buildingId) == b.getId()) {
                    return true;
                }
            }
            return false;
        }

        getNowBuildingNumber(){
            let config = ConfigMgr$1.getInstance().getConfig("Building");
            let num = 0;
            config.forEach(Building => {
                if(this.hasBuildingDataByBid(Building.Id)) {
                    num ++;
                }
            });
            return num;
        }

        getBuildingsBySpecialType(type) {
            let buildings = new Array();
            for(const [sceneId, buildingDataListInfo] of this.buildingMap) {
                for(const buildingData of buildingDataListInfo.list) {
                    if(buildingData.getSpecialType()==type)
                        buildings.push(buildingData);
                }
            }
            return buildings;
        }

        //移除建筑物
        removeBuildingDataByBid(buildingId) {
            let info = this.getBuildingDataListInfo(this.getSceneIdByBid(buildingId));
            if(info == null) {
                return;
            }
            for(let i = 0 ; i < info.list.length ; ++i) {
                let b = info.list[i];
                if(String(buildingId) == b.getId()) {
                    let d = b.getCatWeightedMap();
                    let a = b.getAutoIncome();
                    info.autoIncome = info.autoIncome - a;
                    d.forEach(function (value, key) {
                        info.weighted = info.weighted - value;
                        let v = info.catWeightedMap.get(key) - value;
                        info.catWeightedMap.set(key, v);
                    });
                    info.list.splice(i, 1);
                    return b;
                }
            }
        }

        //添加建筑物
        addBuildingDataByBid(bid) {
            let sceneId = this.getSceneIdByBid(bid);
            let ret = this.getBuildingDataByBid(bid);
            if(ret != null) {
                return ret;
            }
            ret  = new BuildingData(String(bid));
            let d = ret.getCatWeightedMap();
            let a = ret.getAutoIncome();
            let info = this.buildingMap.get(sceneId);
            info.autoIncome = info.autoIncome + a;
            d.forEach(function (value, key) {
                info.weighted = info.weighted + value;
                if(info.catWeightedMap.get(key)) {
                   let v = info.catWeightedMap.get(key) + value;
                   info.catWeightedMap.set(key, v);
                } else {
                   info.catWeightedMap.set(key, value);
                }
           });
           info.list.push(ret);
           return ret;
        }

        getCatWeightedMap(sceneId) {
            let info = this.buildingMap.get(sceneId);
            return info.catWeightedMap;
        }

        getCatWeightedArray(sceneId) {
            let catWeightedMap = this.getCatWeightedMap(sceneId);
            let arr = new Array();
            catWeightedMap.forEach((value, key) => {
                arr.push({w: value, n: key});
            });
            arr.sort(function(a, b) {
                return a.w < b.w;
            });
            return arr;
        }

        getAutoIncomeSum() {
            let a1 = this.buildingMap.get(1);
            let a2 = this.buildingMap.get(2);
            return Number(a1.autoIncome) + Number(a2.autoIncome);
        }


        clear() {
            this.buildingMap.clear();
            this.buildingMap.set(1, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});
            this.buildingMap.set(2, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});
            this.buildingMap.set(3, {weighted: 0, list: new Array(), catWeightedMap: new Map(), autoIncome: 0});
        }

        getHasBuildingByPid(pid) {
            let config = ConfigMgr$1.getInstance().getItemById("BuildingBase", String(pid));
            this.buildingMap.get(Number(config.BelongScene));
            for(let i in config.FixedBuildings){
                if (this.getBuildingDataByBid(config.FixedBuildings[i]) != null) {
                    return true;
                }
            }
            return false;
        }

        getAllSceneUnlockBuilding() {
            let configMap = ConfigMgr$1.getInstance().getConfig("Building");
            let arr = [];
            let coffeeBuildings = [];
            let beachBuildings = [];
            let nauticalBuildings = [];
            arr.push(coffeeBuildings, beachBuildings, nauticalBuildings);
            configMap.forEach((value, key) => {
                if (this.getBuildingDataByBid(key) == null) {
                    if (value.BelongScene == Constants.CoffeeSceneId) {
                        coffeeBuildings.push(value);
                    } else if (value.BelongScene == Constants.BeachSceneId) {
                        beachBuildings.push(value);
                    } else {
                        nauticalBuildings.push(value);
                    }
                }
            });
            return arr;
        }

        updateTicketBuildingInfos(ticketBuildingInfos) {
            this.ticketBuildingInfos = ticketBuildingInfos;
        }

        getTicketBuildingInfos() {
            return this.ticketBuildingInfos;
        }
    }

    class LevelToyDataMgr{
        constructor() {

        }

        static getInstance() {
            return LevelToyDataMgr.instance?LevelToyDataMgr.instance:LevelToyDataMgr.instance = new LevelToyDataMgr();
        }

        updateData(levelToyMap){
            this.levelToyMap = levelToyMap;
        }

        getDataById(Id){
            let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", Id);
            let Rate = JSON.parse(toyConfig.SpecialValue).Rate;
            if(this.levelToyMap.hasOwnProperty(Id) && this.levelToyMap[Id].level > 0){
                let data = this.levelToyMap[Id];
                let level = data.level;
                let exp = data.exp;
                let cost = toyConfig.LvUpCost.length > level ? toyConfig.LvUpCost[level] : -1;
                let nowRate = Rate[level - 1];
                let arr = new Array(level,exp,cost,nowRate);
                return arr;
            }
            else {
                let level = 0;
                let exp = 0;
                let cost = -1;
                let nowRate = Rate[0];
                let arr = new Array(level,exp,cost,nowRate);
                return arr;
            }
        }
    }

    //先拆分出来，涉及EventMgr的以及其他DataMgr的都放到System中做，其他的之后再细化应该放在System还是DataMgr
    class BuildingSystem {

        static init() {
            EventMgr.getInstance().registEvent("CHECK_SHOW_BUILDING_TICKET", null, BuildingSystem.checkShowBuildingCouponPop);
        }

        static clear() {
            BuildingDataMgr.getInstance().clear();
            LocationDataMgr.getInstance().clear();
        }

        static addBuildingDataByBid(bid) {
            BuildingDataMgr.getInstance().addBuildingDataByBid(bid);
        }

        //替换建筑物数据
        static replaceBuildingData(bid, pid, isNew) {
            let locationData = LocationDataMgr.getInstance().getLocation(Number(pid));
            if (locationData == null) {
                return;
            }
            let preBuildingId = locationData.getBuildingId();
            if (preBuildingId != null) {
                let toRemoveBuildingData = BuildingDataMgr.getInstance().getBuildingDataByBid(preBuildingId);
                EventMgr.getInstance().postEvent("REMOVE_BUILDING", toRemoveBuildingData);
                BuildingSystem.removeBuildingDataByPid(pid);
            }
            let buildingData = BuildingDataMgr.getInstance().addBuildingDataByBid(bid);
            if (buildingData == null) {
                return;
            }
            let prevLocation = buildingData.getLocationData();
            if (prevLocation) {
                EventMgr.getInstance().postEvent("REMOVE_BUILDING", buildingData);
                prevLocation.setBuildingId(null);
            }
            locationData.setBuildingId(bid);
            buildingData.setLocationData(locationData);
            EventMgr.getInstance().postEvent("ADD_BUILDING", [buildingData, isNew]);
        }

        //获取建筑物通过location
        static getBuildingDataByPid(pid) {
            let location = LocationDataMgr.getInstance().getLocation(Number(pid));
            if (location == null) {
                return null;
            }
            let buildingId = location.getBuildingId();
            if (buildingId == null) {
                return null;
            }
            let building = BuildingDataMgr.getInstance().getBuildingDataByBid(buildingId);
            return building;
        }

        //移除建筑物通过pid
        static removeBuildingDataByPid(pid) {
            let locationData = LocationDataMgr.getInstance().getLocation(Number(pid));
            let buildingId = locationData.getBuildingId();
            if (buildingId) {
                let buildingData = BuildingDataMgr.getInstance().getBuildingDataByBid(buildingId);
                buildingData.setLocationData(null);
                locationData.setBuildingId(null);
                // return BuildingDataMgr.getInstance().removeBuildingDataByBid(buildingId);
            }
            return null;
        }

        //一场景使用，后续可以做预约？
        static randomVisitBuilding(sceneId, actionEvent) {
            let arr1 = new Array();
            let arr2 = new Array();
            let buildingDataList = BuildingDataMgr.getInstance().getBuildingDataListInfo(sceneId).list;
            buildingDataList.forEach(buildingData => {
                if (buildingData != null && buildingData.canStay() &&
                    buildingData.isSupportAction(actionEvent)) {
                    if (buildingData.hasSpace()) {
                        arr1.push(buildingData);
                    } else {
                        arr2.push(buildingData);
                    }
                    // let buildId = buildingData.getId()
                    // if ( buildId >= "1010" && buildId <= "1017") {

                    //     if (buildingData.hasSpace()) {

                    //         arr1.push(buildingData);

                    //     } else {

                    //         arr2.push(buildingData);

                    //     }

                    // }
                }
            });
            if (arr1.length == 0 && arr2.length == 0) {
                return null;
            }
            if (arr1.length > 0) {
                let i = Utils.randomExt(arr1.length);
                let ret = arr1[i];
                return ret;
            }
            if (arr2.length > 0) {
                let i = Utils.randomExt(arr2.length);
                let ret = arr2[i];
                return ret;
            }
        }

        //2场景使用
        static getAvailableBuilding(sceneId) {
            let arr = new Array();
            let buildingDataList = BuildingDataMgr.getInstance().getBuildingDataListInfo(sceneId).list;
            buildingDataList.forEach(buildingData => {
                if (buildingData != null && buildingData.canStay() && buildingData.hasSpace()) {
                    arr.push(buildingData);
                }
            });
            if (arr.length == 0) {
                return null;
            }
            return arr;
        }

        static getTotalSpaceCountInScene(sceneId) {
            let count = 0;
            let buildingDataList = BuildingDataMgr.getInstance().getBuildingDataListInfo(sceneId).list;
            buildingDataList.forEach(buildingData => {
                if (buildingData != null && buildingData.canStay()) {
                    count += buildingData.getCapacity();
                }
            });
            return count;
        }

        static getEmptySpaceCountInScene(sceneId) {
            let count = 0;
            let buildingDataList = BuildingDataMgr.getInstance().getBuildingDataListInfo(sceneId).list;
            buildingDataList.forEach(buildingData => {
                if (buildingData != null && buildingData.canStay() && buildingData.hasSpace()) {
                    count += buildingData.getSpaces().length;
                }
            });
            return count;
        }

        static getBuildingDataListInfo(sceneId) {
            return BuildingDataMgr.getInstance().getBuildingDataListInfo(sceneId);
        }

        static getLocationsByBaseType(baseType) {
            return LocationDataMgr.getInstance().getLocationsByBaseType(baseType);
        }

        static getLocationByBelongScene(sceneId) {
            return LocationDataMgr.getInstance().getLocationByBelongScene(Number(sceneId));
        }

        //是否存在建筑物
        static hasBuildingDataByBid(buildingId) {
            let info = BuildingDataMgr.getInstance().getBuildingDataListInfo(BuildingDataMgr.getInstance().getSceneIdByBid(buildingId));
            if (info == null) {
                return false;
            }
            for (let i = 0; i < info.list.length; ++i) {
                let b = info.list[i];
                if (String(buildingId) == b.getId()) {
                    return true;
                }
            }
            return false;
        }

        static getBuildingDataByBid(bid) {
            return BuildingDataMgr.getInstance().getBuildingDataByBid(bid);
        }

        static getCatWeightedArray(sceneId) {
            return BuildingDataMgr.getInstance().getCatWeightedArray(sceneId);
        }

        static getAutoIncomeSum() {
            return BuildingDataMgr.getInstance().getAutoIncomeSum();
        }

        /**----------------------------------------------------------------------
        原System调用
        */

        static checkBuildingUnlocked(buildingId) {
            if (BuildingSystem.hasBuildingDataByBid(buildingId)) {
                return true;
            }

            let building = new BuildingData(buildingId);
            if (PlayerSystem.getStar() < building.getLimit()) {
                return false;
            }
            return true;
        }

        static checkBuyBuildingAvailable(buildingId) {
            if (BuildingSystem.hasBuildingDataByBid(buildingId)) {
                return false;
            }
            let building = new BuildingData(buildingId);
            if (PlayerSystem.getFish() < building.getCost()) {
                return false;
            }
            return true;
        }

        static checkAdShowFish(buildingId) {
            let building = new BuildingData(buildingId);
            let num = building.getCost() - PlayerSystem.getFish();
            return [num, building.getCost()];
        }

        static checkPlaceBuildingAvailable(buildingId, baseId) {
            let location = LocationDataMgr.getInstance().getLocation(baseId);
            if (!location.checkBuildingAvailable(buildingId)) {
                return false;
            }
        }

        static buyBuilding(buildingId, fetchType, callback) {
            if (BuildingSystem.checkBuyBuildingAvailable(buildingId) || fetchType == Constants.FetchType.AD) {
                let func = function (res) {
                    if (fetchType == Constants.FetchType.AD) {
                        PlayerSystem.addMakeUpTimes(1);
                    }
                    BuildingDataMgr.getInstance().addBuildingDataByBid(buildingId);
                    if (callback) {
                        callback(res);
                        //打点
                        let id = Number(buildingId);
                        let scence;
                        if (id < 2000) {
                            scence = 'Coffee';
                        } else if (id >= 2000 && id < 3000) {
                            scence = 'Beach';
                        } else {
                            scence = 'Nautical';
                        }
                        let str = 'itemBuild';
                        let obj = { 'item': buildingId, 'zone': scence };
                        console.log('买买买啊买建筑+++++++++++++++++++++++++++++++++++++++++++++++++');
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                    }
                };
                // System.putBuilding(buildingId, "", Laya.Handler.create(null, func))
                EventMgr.getInstance().postEvent("SERVICE_PUT_BUILDING", [buildingId, "", fetchType, Laya.Handler.create(null, func)]);
            }
        }

        static buyBuildingAndPlace(buildingId, baseId, fetchType) {
            if (BuildingSystem.checkBuyBuildingAvailable(buildingId) || fetchType == Constants.FetchType.AD) {
                let func = function () {
                    if (fetchType == Constants.FetchType.AD) {
                        PlayerSystem.addMakeUpTimes(1);
                    }
                    BuildingSystem.replaceBuildingData(buildingId, baseId, true);
                    let location = LocationDataMgr.getInstance().getLocation(baseId);
                    SceneMgr.getInstance().switchScene(location.getBelongScene());
                };
                // System.putBuilding(buildingId, baseId, Laya.Handler.create(null, func))
                EventMgr.getInstance().postEvent("SERVICE_PUT_BUILDING", [buildingId, baseId, fetchType, Laya.Handler.create(null, func)]);
            }
        }

        static placeBuilding(buildingId, baseId, isNew) {
            let func = function () {
                BuildingSystem.replaceBuildingData(buildingId, baseId, isNew);
                let location = LocationDataMgr.getInstance().getLocation(baseId);
                SceneMgr.getInstance().switchScene(location.getBelongScene());
            };
            // System.putBuilding(buildingId, baseId, Laya.Handler.create(null, func))
            EventMgr.getInstance().postEvent("SERVICE_PUT_BUILDING", [buildingId, baseId, Constants.FetchType.DIRECT, Laya.Handler.create(null, func)]);
        }

        static getHasBuildingByPid(pid) {
            return BuildingDataMgr.getInstance().getHasBuildingByPid(pid);
        }

        static checkSceneCanBuyBuilding(id, maxLimit = true, ifAD = true) {
            if (maxLimit) {
                let max = Number(Const.get("ShopRedDot"));
                if (PlayerSystem.getStar() >= max) {
                    return false;
                }
            }
            let buildings = BuildingDataMgr.getInstance().getAllSceneUnlockBuilding();
            let ret = false;

            for (const element of buildings[id]) {
                if (element.Limit <= PlayerSystem.getStar()) {
                    if (element.Cost <= PlayerSystem.getFish()) {
                        ret = true;
                        return ret;
                    } else {
                        if (ifAD && BuildingSystem.checkADBuy(element.Id)) {
                            ret = true;
                            return ret;
                        }
                    }
                }
            }
            return ret;
        }

        static checkCoffeeCanBuyBuilding(maxLimit = true, ifAD = true) {
            return BuildingSystem.checkSceneCanBuyBuilding(0, maxLimit, ifAD);
        }

        static checkBeachCanBuyBuilding(maxLimit = true, ifAD = true) {
            return BuildingSystem.checkSceneCanBuyBuilding(1, maxLimit, ifAD);
        }

        static checkNauticalCanBuyBuilding(maxLimit = true, ifAD = true) {
            return BuildingSystem.checkSceneCanBuyBuilding(2, maxLimit, ifAD);
        }

        static checkCanBuyCheapestBuilding() {
            let buildings = BuildingDataMgr.getInstance().getAllSceneUnlockBuilding();
            let building = null;
            buildings.forEach(sceneBuildings => {
                sceneBuildings.forEach(element => {
                    if (element.Limit <= PlayerSystem.getStar() && element.Cost <= PlayerSystem.getFish()) {
                        if (building != null && building.Cost < element.Cost) {
                            return;
                        }
                        building = element;
                    }
                });
            });
            return building;
        }

        static getPidByBid(bid) {
            let sceneId = BuildingDataMgr.getInstance().getSceneIdByBid(bid);
            let buildingBases = BuildingSystem.getLocationByBelongScene(sceneId);
            for (const iterator of buildingBases) {
                let fixedBuildings = iterator.locationData.FixedBuildings;
                for (const buildingId of fixedBuildings) {
                    if (buildingId == bid) {
                        return iterator.locationData.Id;
                    }
                }
            }
        }

        static checkADBuy(buildingId) {
            if (BuildingSystem.hasBuildingDataByBid(buildingId)) {
                return false;
            }
            let needFish = BuildingSystem.checkAdShowFish(buildingId)[0];
            let costFish = BuildingSystem.checkAdShowFish(buildingId)[1];
            if (!needFish) {
                return false;
            }
            let maxAd = Number(Const.get("ADShoppingTimes"));
            let items = PetDataMgr.getInstance().getItemBySpecialType(4);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToyDataMgr.getInstance().getDataById(item.Id)[3]);
            });
            if (PlayerSystem.getMakeUpTimes() >= maxAd + itemAddTimes) {
                return false;
            }
            let adShoppingRange = JSON.parse(Const.get("ADShoppingRange"));
            let showAd = false;
            let buildingNum = BuildingDataMgr.getInstance().getNowBuildingNumber();
            let yugan = costFish - needFish;
            let adShoppingCondition = Const.get("ADShoppingCondition");
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].BUILDING_AMOUNT[0];
                let max = adShoppingRange[i].BUILDING_AMOUNT[1];
                if (max < 0) {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                } else {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                }
            }
            if (showAd) {
                return true;
            }
            return false;
        }

        static getLocationByBelongScene2() {
            return LocationDataMgr.getInstance().getLocationByBelongScene2();
        }

        static updateTicketBuildingInfos(ticketBuildingInfos) {
            BuildingDataMgr.getInstance().updateTicketBuildingInfos(ticketBuildingInfos);
        }

        static getTicketBuildingInfos() {
            return BuildingDataMgr.getInstance().ticketBuildingInfos;
        }

        static buyBuildingByTicket(buildingId, ticketIndex, handler) {
            let posId = "";
            let dealHandler = handler;
            if (buildingId && buildingId != null) {
                let pid = BuildingSystem.getPidByBid(buildingId);
                let locationData = LocationDataMgr.getInstance().getLocation(Number(pid));
                posId = locationData.checkIsHigher(buildingId) ? pid : "";
                let func = function (res) {
                    if (locationData.getBelongScene() != Constants.BeachSceneId) {
                        if (posId != "") {
                            BuildingSystem.replaceBuildingData(buildingId, posId, true);
                            let location = LocationDataMgr.getInstance().getLocation(posId);
                            SceneMgr.getInstance().switchScene(location.getBelongScene());
                        } else {
                            BuildingDataMgr.getInstance().addBuildingDataByBid(buildingId);
                        }
                    } else {
                        BuildingDataMgr.getInstance().addBuildingDataByBid(buildingId);
                        Laya.Dialog.closeAll();
                        GameContext.PlacingBuilding = buildingId;
                        GameContext.PlacingNewBuilding = true;
                        SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                        EventMgr.getInstance().postEvent("PLACE_BUILDING", buildingId);
                    }

                    if (handler) {
                        handler.runWith(res);
                    }
                };
                dealHandler = Laya.Handler.create(null, func);
            }
            EventMgr.getInstance().postEvent("SERVICE_FETCH_BUILDING_TICKET_REWARD", [ticketIndex, posId, dealHandler]);
        }

        static checkShowBuildingCouponPop() {
            Laya.timer.once(200, null, function () {
                let ticketBuildingInfos = BuildingSystem.getTicketBuildingInfos();
                if (ticketBuildingInfos != null && ticketBuildingInfos.length > 0) {
                    EventMgr.getInstance().postEvent("DIALOG_CHANGE_STATUS",[Constants.DialogStatus.BuildingCouponPop,1]);
                    Laya.Dialog.open("ui/BuildingCouponPop.scene");
                }
            });
        }

    }

    class FoodData {

        constructor(config, level) {
            this._level = level;
            this._unlocked = false;
            this._config = config;
        }

        unlock() {
            this._unlocked = true;
        }

        isLocked() {
            return !this._unlocked;
        }

        getId() {
            return this._config.Id;
        }

        getName() {
            return "tid#foodsName"+this.getId();
        }

        getDesc1() {
            return this._config.Desc1;
        }

        get Source() {
            return this.getShopItems();
        }

        getShopItems() {
            return "cookbook/"+this._config.ShopItems+".png";
        }

        getLimit() {
            return this._config.Limit;
        }

        getCost() {
            return this._config.Cost;
        }

        getIncome() {
            return this._config.Income;
        }

        getCookingTime() {
            return this._config.CookingTime;
        }

        getLevel() {
            return this._level;
        }

        // setLevel(level) {
        //     this._level = level;
        // }

        clear() {
            this._unlocked = false;
        }

        getType() {
            return this._config.Type;
        }
    }

    class FoodDataMgr {

        constructor() {
            this._foodsMap = new Map();
            this.foodListMap = new Map();
            this.cookBook = new Map();
            this.initFoods();
        }

        static getInstance() {
            return FoodDataMgr.instance?FoodDataMgr.instance:FoodDataMgr.instance = new FoodDataMgr();
        }

        //初始化点位
        initFoods() {
            let config = ConfigMgr$1.getInstance().getConfig("FoodsList");
            let foodConfig = ConfigMgr$1.getInstance().getConfig("Foods");
            let outSelf = this;
            config.forEach(function (item) {
                let foodsList = new Array();
                let list = JSON.parse(item.foodsList);
                list.forEach((foodInfo)=>{
                    let foodId = String(foodInfo["foodId"]);
                    let foodLv = Number(foodInfo["lv"]);
                    let food = new FoodData(foodConfig.get(foodId), foodLv);
                    outSelf._foodsMap.set(foodId, food);
                    foodsList.push(food);
                });
                foodsList.sort(function(a,b){
                    return a.getLevel()-b.getLevel();
                });
                outSelf.foodListMap.set(item["Id"], foodsList);
                outSelf.cookBook.set(item["Id"], foodsList[0]);
            });
        }

        clear() {
            this._foodsMap.forEach((food)=>{
                food.clear();
            });
            this._foodsMap.clear();
            this.foodListMap.clear();
            this.cookBook.clear();
            this.initFoods();
        }

        getFood(id) {
            return this._foodsMap.get(id);
        }

        getFoodList(foodListId) {
            return this.foodListMap.get(foodListId);
        }

        getCurFoodInList(listId) {
            return this.cookBook.get(String(listId));
        }

        updateFood(foodListId, level) {
            let list = this.foodListMap.get(foodListId);
            let food = this.cookBook.get(foodListId);
            if(food.isLocked()&&food.getLevel()==level){
                food.unlock();
                return true;
            }else if(food.getLevel()!=level) {
                let food = list[level - 1];
                if(food){
                    this.cookBook.set(foodListId, food);
                    for(let i=0;i<=level-1;i++){
                        list[i].unlock();
                    }
                    return true;
                }
            }
            return false;
        }

        getNextLevelUpFoodInList(foodListId) {
            let food = this.getCurFoodInList(foodListId);
            if(food.isLocked()){
                return food;
            }else {
                let list = this.getFoodList(foodListId);
                return list[food.getLevel()];
            }
        }

        getAvailableFood(type) {
            let food = this.cookBook.get(type);
            if(food.isLocked()) {
                return null;
            }
            return food;
        }

        getAllFood(){
            return this.cookBook;
        }

        get CurFoodInListUnLocked() {
            let res = new Array();
            for(const [Id, food] of this.cookBook) {
                if(!food.isLocked())
                    res.push(Id);
            }
            return res;
        }
    }

    class FoodSystem$1 {
        static init() {
        }

        static clear() {
            FoodDataMgr.getInstance().clear();
        }

        static updateFoods(FoodMenu) {
            if (FoodMenu.menu) {
                let needUpdate = false;
                Object.getOwnPropertyNames(FoodMenu.menu).forEach(foodListId => {
                    let level = FoodMenu.menu[foodListId];
                    needUpdate = FoodDataMgr.getInstance().updateFood(foodListId, level) || needUpdate;
                });
                if (needUpdate) {
                    EventMgr.getInstance().postEvent("UPDATE_COOKMENU");
                }
            }
        }

        static getFood(foodId) {
            return FoodDataMgr.getInstance().getFood(foodId);
        }

        static getFoodList(listId) {
            return FoodDataMgr.getInstance().getFoodList(listId);
        }

        static getCurFoodInList(listId) {
            return FoodDataMgr.getInstance().getCurFoodInList(listId);
        }

        static getAvailableFood(listId) {
            return FoodDataMgr.getInstance().getAvailableFood(listId);
        }

        static getNextLevelUpFoodInList(listId) {
            return FoodDataMgr.getInstance().getNextLevelUpFoodInList(listId);
        }

        static checkLevelUpUnAvailable(listId) {
            let food = FoodSystem$1.getNextLevelUpFoodInList(listId);
            if (food && food.getLimit() > PlayerSystem.getStar()) {
                return  Strings$1.get('tid#UI_060');//铃铛不足
            } else if (food && food.getCost() > PlayerSystem.getFish()) {
                //return "还差" + String(food.getCost() - PlayerSystem.getFish()) + "鱼干";
                return Strings$1.get('tid#UI_094')+' '+String(food.getCost() - PlayerSystem.getFish())+' '+Strings$1.get('tid#UI_095');
            }
            return false;
        }

        static checkFoodRedPoint(listId) {
            let food = FoodSystem$1.getNextLevelUpFoodInList(listId);
            if (food && food.getLimit() <= PlayerSystem.getStar() && food.getCost() <= PlayerSystem.getFish() && PlayerSystem.getStar() < Number(Const.get("ShopRedDot"))) {
                return true;
            }
            if (FoodSystem$1.checkAdLevelUp(listId)) {
                return true;
            }
            return false;
        }

        static checkLevelUpStarEnough(listId) {
            let food = FoodSystem$1.getNextLevelUpFoodInList(listId);
            if (food && food.getLimit() <= PlayerSystem.getStar()) {
                return true;
            }
            return false;
        }

        static checkAdShowFish(listId) {
            let food = FoodSystem$1.getNextLevelUpFoodInList(listId);
            if (food == null) {
                return null;
            }
            let num = food.getCost() - PlayerSystem.getFish();
            if (num > 0 && food && food.getLimit() <= PlayerSystem.getStar() && food.getCost() > PlayerSystem.getFish()) {
                return [num,food.getCost()];
            }
            return null;
        }

        static getPlayerFoodMenu(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_PLAYER_FOOD_MENU", [handler]);
        }

        static foodLevelUp(foodListId, fetchType, handler) {
            let dealFunc = function (res) {
                let str='recipeResearch';
                let obj={'recipe':foodListId};
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str,obj]);
                if (fetchType == Constants.FetchType.AD) {
                    PlayerSystem.addMakeUpTimes(1);
                }
                if (handler)
                    handler.runWith(res);
            };
            EventMgr.getInstance().postEvent("SERVICE_FOOD_LEVELUP", [foodListId, fetchType, Laya.Handler.create(null, dealFunc)]);
        }

        static checkFoodShopRedShow() {
            let max = Number(Const.get("ShopRedDot"));
            if (PlayerSystem.getStar() >= max) {
                return false;
            }
            let ret = FoodSystem$1.checkFoodCanLevelUp(true) || FoodSystem$1.checkFoodCanBuy(true);
            return ret;
        }

        static checkFoodCanLevelUp(ifAD = false) {
            let foodMap = FoodDataMgr.getInstance().getAllFood();
            let ret = false;
            foodMap.forEach((value, key) => {
                if (!value.isLocked()) {
                    let food = FoodSystem$1.getNextLevelUpFoodInList(key);
                    if (food && food.getLimit() <= PlayerSystem.getStar()) {
                        if (food.getCost() <= PlayerSystem.getFish()) {
                            ret = true;
                            return;
                        } else {
                            if (ifAD && FoodSystem$1.checkAdLevelUp(key)) {
                                ret = true;
                                return;
                            }
                        }
                    }
                }
            });
            return ret;
        }

        static checkFoodCanBuy(ifAD = false) {
            let foodMap = FoodDataMgr.getInstance().getAllFood();
            let ret = false;
            foodMap.forEach((value, key) => {
                if (value.isLocked()) {
                    if (value.getLimit() <= PlayerSystem.getStar()) {
                        if (value.getCost() <= PlayerSystem.getFish()) {
                            ret = true;
                            return;
                        } else {
                            if (ifAD && FoodSystem$1.checkAdLevelUp(key)) {
                                ret = true;
                                return;
                            }
                        }
                    }
                }
            });
            return ret;
        }

        static checkCanBuyCheapestFood() {
            let foodMap = FoodDataMgr.getInstance().getAllFood();
            let food = null;
            foodMap.forEach((value, key) => {
                if (value.isLocked()) {
                    if (value.getLimit() <= PlayerSystem.getStar() && value.getCost() <= PlayerSystem.getFish()) {
                        if (food != null && food.getCost() < value.getCost()) {
                            return;
                        }
                        food = value;
                    }
                }
            });
            return food;
        }

        static checkLevelUpCheapestFood() {
            let foodMap = FoodDataMgr.getInstance().getAllFood();
            let food = null;
            foodMap.forEach((value, key) => {
                if (!value.isLocked()) {
                    let nextFood = FoodSystem$1.getNextLevelUpFoodInList(key);
                    if (nextFood && nextFood.getLimit() <= PlayerSystem.getStar() && nextFood.getCost() <= PlayerSystem.getFish()) {
                        if (food != null && food.getCost < nextFood.getCost()) {
                            return;
                        }
                        food = value;
                    }
                }
            });
            return food;
        }

        static checkAdLevelUp(foodListId) {
            let needFish = FoodSystem$1.checkAdShowFish(foodListId) ? FoodSystem$1.checkAdShowFish(foodListId)[0] : 0;
            let costFish = FoodSystem$1.checkAdShowFish(foodListId) ? FoodSystem$1.checkAdShowFish(foodListId)[1] : 0;
            let yugan = costFish - needFish;
            let adShoppingCondition = Const.get("ADShoppingCondition");
            let maxAd = Number(Const.get("ADShoppingTimes"));
            let items = PetDataMgr.getInstance().getItemBySpecialType(4);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToyDataMgr.getInstance().getDataById(item.Id)[3]);
            });
            if (PlayerSystem.getMakeUpTimes() >= maxAd + itemAddTimes) {
                return false;
            }
            let adShoppingRange = JSON.parse(Const.get("ADShoppingRange"));
            let showAd = false;
            let buildingNum=BuildingDataMgr.getInstance().getNowBuildingNumber();
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].BUILDING_AMOUNT[0];
                let max = adShoppingRange[i].BUILDING_AMOUNT[1];
                if (max < 0) {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                } else {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                }
            }
            if (showAd) {
                return true;
            }
            return false;
        }
    }

    class TaskMainUIMgr {

        constructor() {
            this.init();
        }

        static getInstance() {
            return TaskMainUIMgr.instance ? TaskMainUIMgr.instance : TaskMainUIMgr.instance = new TaskMainUIMgr();
        }

        init() {
            EventMgr.getInstance().registEvent("CLEAR_MAIN_TASK_UI", this, this.clear);
            EventMgr.getInstance().registEvent("GUIDE_SET_GUIDING", this, this.changeVisible);
            EventMgr.getInstance().registEvent("FISH_REALLY_CHANGE", this, this.checkAnimShow);
            EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.checkAnimShow);
            EventMgr.getInstance().registEvent("TASK_POP_SHOW", this, this.checkAnimShow);
            EventMgr.getInstance().registEvent("TASK_MAIN_BOX_SHOW", this, this.taskMainShow);
            EventMgr.getInstance().registEvent("TASK_MAIN_BOX_HIDE", this, this.taskMainHide);
            this.buttonMap = new Map();
        }

        show(currentData) {
            if (!this.mainTaskBox) {
                Laya.loader.create("prefab/ui/TaskMainBox.prefab", Laya.Handler.create(this, function (prefabDef) {
                    this.mainTaskBox = prefabDef.create();
                    this.mainTaskBox.x = 0;
                    this.mainTaskBox.y = 0;
                    this.mainTaskBox.visible = true;
                    this.mainTaskBox.zOrder = 100;
                    Laya.stage.addChild(this.mainTaskBox);
                }));
                this.currentData = currentData;
                if (currentData == null) {
                    this.clear();
                    return;
                }
                this.addBtnAnim(currentData.getGuide());
                this.showTalk(true);
                return true;
            } else {
                this.mainTaskBox.updatView();
                return false;
            }
        }

        updatView() {
            if (this.mainTaskBox) {
                this.mainTaskBox.updatView();
            }
            if (this.currentData && this.currentData.getTarget() <= this.currentData.getCurrent()) {
                this.destroyBtnAnim();
            }
        }

        clear() {
            this.currentData = null;
            this.clearShow();
        }

        clearShow() {
            if (this.mainTaskBox != null) {
                this.mainTaskBox.clear();
                this.mainTaskBox = null;
            }
            this.destroyBtnAnim();
        }

        registButton(key, button) {
            this.buttonMap.set(key, button);
            if (this.currentData) {
                this.addBtnAnim(this.currentData.getGuide());
            }
        }

        addBtnAnim(Guide) {
            if (this.currentData.getTarget() <= this.currentData.getCurrent()) {
                return;
            }
            this.currentBtn = this.buttonMap.get(Guide);
            if (this.currentBtn && !this.currentBtn.destroyed && this.btnAnim == null) {
                this.btnAnim = new Spine();
                this.btnAnim.loadAnim("anim1/xinshouyindao.sk");
                this.btnAnim.playAnim("zhuazi2", true);
                this.btnAnim.x = this.currentBtn.width / 2;
                this.btnAnim.y = this.currentBtn.height / 2;
                this.btnAnim.scaleX = 1 / this.currentBtn.scaleX;
                this.btnAnim.scaleY = 1 / this.currentBtn.scaleY;
                this.btnAnim.zOrder = 100;
                this.currentBtn.addChildren(this.btnAnim);
                this.checkAnimShow();
            }
        }

        showTalk(start) {
            Laya.Dialog.close("ui/TaskMainTalkPop.scene");
            Laya.Dialog.open("ui/TaskMainTalkPop.scene", true, [this.currentData, start]);
        }

        destroyBtnAnim() {
            if (this.btnAnim) {
                this.btnAnim.destroy();
            }
            this.btnAnim = null;
        }

        changeVisible(type) {
            if (type) {
                Laya.Dialog.close("ui/TaskMainTalkPop.scene");
                this.clearShow();
            } else {
                if (this.currentData) {
                    Laya.timer.once(200, this, function () {
                        if (!GameContext.isGuiding && SceneMgr.getInstance().getCurrentSceneId() != Constants.FishingSceneId) {
                            this.show(this.currentData);
                        }
                    });
                }
            }
        }

        setBtnAnimVisible(visible) {
            if (this.btnAnim) {
                this.btnAnim.visible = visible && (!GameContext.isGuiding);
            }
        }

        checkAnimShow() {
            if (this.currentData) {
                let guideName = this.currentData.getGuide();
                let ifBuy = false;
                if (guideName == "Building") {
                    ifBuy = BuildingSystem.checkBeachCanBuyBuilding(false, false) || BuildingSystem.checkCoffeeCanBuyBuilding(false, false) || BuildingSystem.checkNauticalCanBuyBuilding(false, false);
                    this.setBtnAnimVisible(ifBuy);
                } else if (guideName == "Food_Unlock") {
                    ifBuy = FoodSystem$1.checkFoodCanBuy();
                    this.setBtnAnimVisible(ifBuy);
                } else if (guideName == "Food_LvUp") {
                    ifBuy = FoodSystem$1.checkFoodCanLevelUp();
                    this.setBtnAnimVisible(ifBuy);
                }else if(guideName=="Click_Adopt"){
                    ifBuy = PetDataMgr.getInstance().needGuide();
                    this.setBtnAnimVisible(ifBuy);
                }
            }
        }

        taskMainShow() {
            if (this.mainTaskBox && !this.mainTaskBox.destroyed) {
                this.mainTaskBox.show();
            }
        }

        taskMainHide() {
            if (this.mainTaskBox && !this.mainTaskBox.destroyed) {
                this.mainTaskBox.hide();
                EventMgr.getInstance().postEvent("CHECK_AUTO_POP_ALL");
            }
        }

        setTaskMainVisible(visible) {
            if (this.mainTaskBox && !this.mainTaskBox.destroyed) {
                this.mainTaskBox.visible = visible;
            } else {
                if (visible) {
                    this.changeVisible(false);
                }
            }
        }
    }

    class DialogStatusMgr {
        constructor() {
            this.clear();
        }

        static getInstance() {
            return DialogStatusMgr.instance ? DialogStatusMgr.instance : DialogStatusMgr.instance = new DialogStatusMgr();
        }

        init() {
            EventMgr.getInstance().registEvent("CHECK_AUTO_POP_ALL", this, this.dialogDisplay);
            EventMgr.getInstance().registEvent("GUIDE_SET_GUIDING", this, this.afterGuide);
            EventMgr.getInstance().registEvent("DIALOG_CHANGE_STATUS", this, this.changeStatus);
            this.FortuneCat = 0;
        }

        clear() {
            this.bottomUI = 0;
            this.sign = 0;
            this.homeLand = 0;
            this.fishing = 0;
            this.taskMain = 1;
            this.forturneCat = 0;
            this.mileStone = 0;
            this.catNewcome = 0;
            this.visitorGift = 0;
            this.photo = 0;
            this.isGuiding = 0;
            this.buildingCouponPop = 0;
            this.dialogDisplay();
            this.FortuneCat = 0;
            this.PayRewards = 0;
        }

        changeStatus(type, value) {
            switch (type) {
                case Constants.DialogStatus.BottomUI: this.bottomUI = value; break;
                case Constants.DialogStatus.Sign: this.sign = value; break;
                case Constants.DialogStatus.HomeLand: this.homeLand = value; break;
                case Constants.DialogStatus.Fishing: this.fishing = value; break;
                case Constants.DialogStatus.TaskMain: this.taskMain = value; break;
                case Constants.DialogStatus.ForturneCat: this.forturneCat = value; break;
                case Constants.DialogStatus.MileStone: this.mileStone = value; break;
                case Constants.DialogStatus.CatNewcome: this.catNewcome = value; break;
                case Constants.DialogStatus.VisitorGift: this.visitorGift = value; break;
                case Constants.DialogStatus.Photo: this.photo = value; break;
                case Constants.DialogStatus.BuildingCouponPop: this.buildingCouponPop = value; break;
                case Constants.DialogStatus.PayRewards: this.PayRewards = value; break;
            };
            this.dialogDisplay();
        }

        getStatus() {
            this.isGuiding = GameContext.isGuiding ? 1 : 0;
            let isHsowDialog = Laya.Dialog.manager.numChildren | 0;
            return this.isGuiding + this.bottomUI + this.sign + this.homeLand + this.fishing + this.taskMain + this.forturneCat
                + this.mileStone + this.catNewcome + this.visitorGift + this.photo + isHsowDialog + this.buildingCouponPop;
        }

        dialogDisplay() {
            Laya.timer.clearAll(this);
            Laya.timer.once(100, this, function () {
                if (this.getStatus() == 0) {
                    this.autoFortuneCat();
                }
            });
            Laya.timer.once(200, this, function () {
                if (this.getStatus() == 0) {
                    EventMgr.getInstance().postEvent("CHECK_SIGN_AUTO_POP");
                }
            });
            Laya.timer.once(300, this, function () {
                if (this.getStatus() == 0) {
                    EventMgr.getInstance().postEvent("PAY_REWARDS_TAN");
                }
            });
            Laya.timer.once(400, this, function () {
                if (this.getStatus() == 0) {
                    EventMgr.getInstance().postEvent("MAY_PLAY_MILESTONE");
                }
            });
            Laya.timer.once(500, this, function () {
                if (this.getStatus() == 0) {
                    EventMgr.getInstance().postEvent("CATNEEWCOME_DONE");
                }
            });
            Laya.timer.once(600, this, function () {
                if (this.getStatus() == 0) {
                    EventMgr.getInstance().postEvent("CHECK_SHOW_BUILDING_TICKET");
                }
            });
        }

        autoFortuneCat() {
            if (this.FortuneCat == 0) {
                this.FortuneCat = 1;
                EventMgr.getInstance().postEvent("ZCM_AUTO");
            }
        }

        afterGuide(value) {
            if (value) {
                return;
            }
            this.dialogDisplay();
        }

    }

    class TaskMainSystem {

        static init() {
            EventMgr.getInstance().registEvent("FISH_REALLY_CHANGE", null, TaskMainSystem.updataProgress, ["FISHES_GET"]);
            EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", null, TaskMainSystem.updataProgress, ["STARS_GOAL"]);
        }

        static clear() {
            TaskMainMgr.getInstance().clear();
            TaskMainUIMgr.getInstance().clear();
            TaskMainSystem.ready = null;
        }

        static updata(data) {
            if (GameContext.isGuiding) {
                return;
            }
            let isTasking = TaskMainMgr.getInstance().updata(data);
            let lastComplete = TaskMainMgr.getInstance().getLastComplete();
            if (!TaskMainSystem.ready) {
                return;
            }
            if (lastComplete) {
                Laya.timer.once(1000, TaskMainSystem, function () {
                    if (GameContext.isGuiding) {
                        return;
                    }
                    if (isTasking) {
                        TaskMainMgr.getInstance().setLastComplete(false);
                        let currentData = TaskMainSystem.getCurrentData();
                        if (currentData == null) {
                            return;
                        }
                        let str = "main_task" + currentData.getId().substring(9) + "_1";
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str]);
                        TaskMainUIMgr.getInstance().show(currentData);
                    } else {
                        TaskMainUIMgr.getInstance().clear();
                    }
                });
            } else {
                TaskMainUIMgr.getInstance().updatView();
            }
        }

        static getIsTasking() {
            return TaskMainMgr.getInstance().getIsTasking();
        }

        static getCurrentData() {
            return TaskMainMgr.getInstance().getCurrentData();
        }

        static checkTask() {
            TaskMainMgr.getInstance().checkTask();
        }

        static getReward(handler) {
            let currentData = TaskMainSystem.getCurrentData();
            let arr = currentData.getEndDialog();
            if (arr != null && arr.length > 0) {
                TaskMainUIMgr.getInstance().showTalk(false);
            } else {
                TaskMainSystem.completeTask();
            }
            EventMgr.getInstance().postEvent("SERVICE_GET_MAIN_TASK_REWARD", [TaskMainSystem.getCurrentData().getId(), handler]);
        }

        static completeTask() {
            TaskMainMgr.getInstance().setLastComplete(true);
            let isTasking = TaskMainSystem.getIsTasking();
            if (isTasking) {
                let currentData = TaskMainSystem.getCurrentData();
                let str = "main_task" + currentData.getId().substring(9) + "_1";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str]);
                if(str=='main_task1_1'){
                    let str1 = "ftue_55_taskClaim";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str1,null,1]);
                }
                TaskMainUIMgr.getInstance().show(currentData);
            } else {
                TaskMainUIMgr.getInstance().clear();
            }
        }

        static addProgress(conditionType, num) {
            let isTasking = TaskMainSystem.getIsTasking();
            let currentData = TaskMainSystem.getCurrentData();
            if (isTasking && currentData) {
                if (currentData.getConditionType() == conditionType) {
                    let result = currentData.addProgress(num);
                    TaskMainUIMgr.getInstance().updatView(currentData);
                    if (result && currentData.getCurrent() == currentData.getTarget()) {
                          let str = "main_task" + currentData.getId().substring(9) + "_3";
                        if(str=='main_task1_3'){
                            let str1 = "ftue_50_chair2Build";
                            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str1,null,1]);
                        }
                        TaskMainUIMgr.getInstance().taskMainShow();
                    }
                }
            }
        }

        static updataProgress(conditionType, num) {
            let isTasking = TaskMainSystem.getIsTasking();
            let currentData = TaskMainSystem.getCurrentData();
            if (isTasking && currentData) {
                if (currentData.getConditionType() == conditionType) {
                    let result = currentData.updataProgress(num);
                    TaskMainUIMgr.getInstance().updatView(currentData);
                    if (result && currentData.getCurrent() == currentData.getTarget()) {
                        let str = "main_task" + currentData.getId().substring(9) + "_3";
                        TaskMainUIMgr.getInstance().taskMainShow();
                    }
                }
            }
        }

        static startCheck(delay = 1000) {
            TaskMainSystem.ready = true;
            if (TaskMainSystem.getCurrentData()) {
                Laya.timer.once(delay, TaskMainSystem, function () {
                    if (GameContext.isGuiding) {
                        return;
                    }
                    TaskMainMgr.getInstance().setLastComplete(false);
                    let currentData = TaskMainSystem.getCurrentData();
                    let type = TaskMainUIMgr.getInstance().show(currentData);
                    if (type) {
                        let str = "main_task" + currentData.getId().substring(9) + "_1";
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str]);
                    }
                });
            } else {
                DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.TaskMain, 0);
            }
        }

        static checkEnought() {
            let task = TaskMainSystem.getCurrentData();
            if (task && task.getCurrent() >= task.getTarget()) {
                return true;
            }
            return false;
        }

        static setTaskMainVisible(visible) {
            TaskMainSystem.ready = visible;
            TaskMainUIMgr.getInstance().setTaskMainVisible(visible);
        }

    }

    class MiniGameData {

        constructor(data) {
            this.config = ConfigMgr$1.getInstance().getItemById("SpecialGame", data.gameId);
            this.listConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameGroup", data.listId);
            this.catId = ++MiniGameData.catIdMax;
        }

        getId() {
            return this.config.Id;
        }

        getCatId() {
            return "MiniGameCat" + this.catId;
        }

        getListId() {
            return this.listConfig.Id;
        }

        getDesc() {
            return this.config.Desc;
        }

        getReward() {
            return JSON.parse(this.config.RewardConfig);
        }

        getType() {
            return this.config.Type;
        }

        getStartTalk() {
            return this.config.Desc1;
        }

        getFinishTalk() {
            return this.config.Desc2;
        }

        getResultStatus() {
            return this._resultStatus;
        }

        // 0 失败 1胜利 2放弃
        setResultStatus(resultStatus) {
            this._resultStatus = resultStatus;
        }

        getScore() {
            return this._score;
        }

        setScore(score) {
            this._score = score;
        }

        getDataName() {
            return "MiniGame" + this.getType();
        }

        setActive(type) {
            this._active = type;
        }

        getCatName() {
            return this.config.Name;
        }

        setCatTalkShow() {
            this._catTalkShow = true;
        }

        getCatTalkShow() {
            return this._catTalkShow;
        }

    }
    MiniGameData.catIdMax = 0;

    class MiniGameDataMgr {

        constructor() {
            // this._map = new Map();
            this._data = null;
            this._gameActive = false;
            this.restEventTimes = null;
            this.talkID=null;//力气猫在对话时候，因为被KO会切换到下一只猫，在开始时记下这个，结束的对话通过这个值去取猫 
        }

        static getInstance() {
            return MiniGameDataMgr.instance ? MiniGameDataMgr.instance : MiniGameDataMgr.instance = new MiniGameDataMgr();
        }

        updata(data) {
            this._gameActive = data.gameActive; // 当前是否有小游戏处于激活状态
            let game = data.game; // 小游戏； 如果当前没有小游戏处于激活状态，则返回为空
            this.restEventTimes = game.restEventTimes;
            if (!this._gameActive) {
                this._data = null;
                return;
            }
            this._data = new MiniGameData(game);
        }

        getCurrentMiniGame() {
            return this._data;
        }

        getActive() {
            return this._gameActive;
        }

        setActive(type) {
            this._gameActive = type;
        }

        start(data) {
            this._gameActive = true; // 当前是否有小游戏处于激活状态
            let game = data; // 小游戏； 如果当前没有小游戏处于激活状态，则返回为空
            this.restEventTimes = game.restEventTimes;
            this._data = new MiniGameData(game);
        }

        clear() {
            this._data = null;
            this._gameActive = false;
            this.restEventTimes = null;
        }

        addProgress(type, num, handler) {
            if (this._gameActive) {
                handler.run();
                return;
            }
            if (this.restEventTimes != null && this.restEventTimes[type] != null) {
                this.restEventTimes[type] -= num;
                if (this.restEventTimes[type] <= 0) {
                    EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW", [handler]);
                }
            }
        }


        //储存小游戏其他数据
        updateOtherData(customData) {
            if (customData.minigame != null) {
                this.otherData = JSON.parse(customData.minigame);
            } else {
                this.otherData = {};
            }
            if (this.otherData.powerGameData == null) {
                let powerGameData = {
                    KOTimes: 0,
                    isFirst: true,
                    difficulty: 1,
                    failTimes: 0,
                    playerLevel: 1,
                    playerId: null
                };
                this.otherData.powerGameData = powerGameData;
            }
            let playerLevel = this.otherData.powerGameData.playerLevel;
            let configs = ConfigMgr$1.getInstance().getConfig("SpecialGameStrengthRole");
            for (const [id, powerConfig] of configs) {
                if (Number(playerLevel) == Number(powerConfig.Order)) {
                    this.otherData.powerGameData.playerId = powerConfig.Id;
                    break;
                }
            }
        }

        getOtherData() {
            return this.otherData;
        }

        //控制比力气小游戏难度
        addPowerPlayTimes(result) {
            if (result == Constants.MiniGameResult.KO) {
                this.otherData.powerGameData.KOTimes++;
                let playerLevel = this.otherData.powerGameData.playerLevel;
                playerLevel++;
                let playerId = null;
                let configs = ConfigMgr$1.getInstance().getConfig("SpecialGameStrengthRole");
                let config = null;
                for (const [id, powerConfig] of configs) {
                    if (Number(playerLevel) == Number(powerConfig.Order)) {
                        playerId = powerConfig.Id;
                        config = powerConfig;
                        break;
                    }
                }
                if (config != null) {
                    this.otherData.powerGameData.playerId = playerId;
                    this.otherData.powerGameData.playerLevel = playerLevel;
                    this.otherData.powerGameData.isFirst = true;
                }
                else {
                    this.otherData.powerGameData.isFirst = false;
                }
                let difficulty = this.otherData.powerGameData.difficulty;
                difficulty++;
                let difficultyConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameStrength", String(difficulty));
                if (difficultyConfig) {
                    this.otherData.powerGameData.difficulty = difficulty;
                }
                this.otherData.powerGameData.failTimes = 0;
            } else if (result == Constants.MiniGameResult.Win) {
                this.otherData.powerGameData.failTimes = 0;
                this.otherData.powerGameData.isFirst = false;
            } else {
                this.otherData.powerGameData.failTimes++;
                // let arr = Const.getArray("GameStrengthDowngrade");
                let arr = [3, 5, 8, 10];
                if (this.otherData.powerGameData.failTimes >= arr[0]) {
                    this.otherData.powerGameData.failTimes = 0;
                    if (this.otherData.powerGameData.difficulty > 1) {
                        this.otherData.powerGameData.difficulty--;
                    }
                }
                this.otherData.powerGameData.isFirst = false;
            }

        }
        getTalkID(){
            return this.talkID;
        }
        setTalkID(talkID){
            this.talkID=talkID;
        }

    }

    class SpecialCatMgr {

        constructor() {
            this._currentData = null;
            this._dataArr = [];
        }

        static getInstance() {
            return SpecialCatMgr.instance ? SpecialCatMgr.instance : SpecialCatMgr.instance = new SpecialCatMgr();
        }

        pushCat(data) {
            let canPush = true;
            if (data.getDataName() == "Entrust") {
                this._dataArr.forEach(entrustData => {
                    if (entrustData.getListId() == data.getListId() && entrustData.getId() == data.getId()) {
                        canPush = false;
                    }
                });
            }
            if (canPush) {
                this._dataArr.push(data);
            }
        }

        popCat() {
            if (this._currentData != null) {
                return;
            }
            this._currentData = this._dataArr[0];
            if (this._currentData != null) {
                this._dataArr.splice(0, 1);
                EventMgr.getInstance().postEvent("FORCE_ATTRACT_ENTRUST_CAT", this._currentData);
                if (this._currentData) {
                    EventMgr.getInstance().postEvent("SPECIAL_CAT_ZHAOLAN");
                }
            }
        }

        catLeave() {
            EventMgr.getInstance().postEvent("ENTRUST_CAT_GO", this._currentData);
            this._currentData = null;
        }

        clear() {
            this._currentData = null;
            this._dataArr = [];
        }
    }

    class MiniGameSystem {
        static init() {
        }

        static clear() {
            Laya.timer.clear(null, MiniGameSystem.triggerMiniGame);
            MiniGameDataMgr.getInstance().clear();
        }

        static updataMiniGame(data) {
            // return ; //屏蔽
            MiniGameDataMgr.getInstance().updata(data);
            let currentMiniGame = MiniGameSystem.getCurrentMiniGame();
            if (currentMiniGame != null) {
                SpecialCatMgr.getInstance().pushCat(currentMiniGame);
            } else {
                MiniGameSystem.miniGameTriggerTimer();
            }
        }

        static getCurrentMiniGame() {
            return MiniGameDataMgr.getInstance().getCurrentMiniGame();
        }

        // 0 失败 1胜利 2放弃
        static finishGame(gameId, listId, resultStatus, fetchType, handler) {
            MiniGameSystem.miniGameTriggerTimer();
            MiniGameSystem.setActive(false);
            if(resultStatus == 3){//放弃游戏
                EventMgr.getInstance().postEvent("SERVICE_GIVEUP_MINIGAME");
                SpecialCatMgr.getInstance().catLeave();
                return;
            }
            let score = 0;
            let result = { "resultStatus": resultStatus, "score": score };
            EventMgr.getInstance().postEvent("SERVICE_FINISH_MINIGAME", [listId, result, fetchType, handler]);
            SpecialCatMgr.getInstance().catLeave();
            switch (gameId) {
                case "SpecialGame1": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_end_finger_guess"]);
                    break;
                }
                case "SpecialGame2": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_end_coin_guess"]);
                    break;
                }
                case "SpecialGame3": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_end_hand_wrestling"]);
                    break;
                }
            }
        }

        static getActive() {
            return MiniGameDataMgr.getInstance().getActive();
        }

        static setActive(type) {
            return MiniGameDataMgr.getInstance().setActive(type);
        }

        static startMiniGame(data) {
            // return ; //屏蔽
            MiniGameDataMgr.getInstance().start(data);
            let currentMiniGame = MiniGameSystem.getCurrentMiniGame();
            if (currentMiniGame != null) {
                SpecialCatMgr.getInstance().pushCat(currentMiniGame);
            }
            switch (data.gameId) {
                case "SpecialGame1": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_appear_finger_guess"]);
                    break;
                }
                case "SpecialGame2": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_appear_coin_guess"]);
                    break;
                }
                case "SpecialGame3": {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_appear_hand_wrestling"]);
                    break;
                }
            }
        }

        static addProgress(type, num) {
            MiniGameDataMgr.getInstance().addProgress(type, num, Laya.Handler.create(null, function () {
                SpecialCatMgr.getInstance().popCat();
            }));
        }

        static updateOtherData(customData) {
            if (customData != null) {
                MiniGameDataMgr.getInstance().updateOtherData(customData);
            }
        }

        static sendOtherData() {
            let otherData = MiniGameDataMgr.getInstance().getOtherData();
            let customData = { "minigame": JSON.stringify(otherData) };
            EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", [customData]);
        }

        static getOtherData() {
            return MiniGameDataMgr.getInstance().getOtherData();
        }

        static addPowerPlayTimes(restlt) {
            MiniGameDataMgr.getInstance().addPowerPlayTimes(restlt);
            MiniGameSystem.sendOtherData();
        }
        static getTalkID() {
            return MiniGameDataMgr.getInstance().getTalkID();
        }
        static setTalkID(talkID) {
            MiniGameDataMgr.getInstance().setTalkID(talkID);
        }

        static triggerMiniGame() {
            GameContext.MinigameTimerRunning = null;
            if (MiniGameSystem.getActive()) {
                return;
            }
            EventMgr.getInstance().postEvent("SERVICE_CLIENT_TRIGGER_MINIGAME");
        }

        static miniGameTriggerTimer() {
            if (GameContext.MinigameTimerRunning) {
                return;
            }
            let miniGameAutoLimit = Number(Const.get("MiniGameAutoLimit"));
            let star = PlayerDataMgr.getInstance().getStar();
            if (star < miniGameAutoLimit) {
                EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", null, MiniGameSystem.miniGameTriggerTimer);
                EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", null, MiniGameSystem.miniGameTriggerTimer);
                return;
            }
            EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", null, MiniGameSystem.miniGameTriggerTimer);
            Laya.timer.clear(null, MiniGameSystem.triggerMiniGame);
            let miniGameAutoTime = Number(Const.get("MiniGameAutoTime")) * 1000;
            Laya.timer.once(miniGameAutoTime, null, MiniGameSystem.triggerMiniGame);
            GameContext.MinigameTimerRunning = true;
        }
    }

    class StaffCatMgr extends BaseCatMgr {

        static getInstance() {
            return StaffCatMgr.instance ? StaffCatMgr.instance : StaffCatMgr.instance = new StaffCatMgr();
        }

        addStaff(staff) {
            this.addCat(staff);
        }

        removeStaff(staffId) {
            this.removeCat(staffId);
        }

        getStaff(staffId) {
            return this.getCat(staffId);
        }

        allStaffs() {
            let result = new Array();
            for (const [key, staff] of this._getMap()) {
                result.push(staff);
            }
            return result;
        }

        idleStaffs() {
            let result = new Array();
            for (const [key, staff] of this._getMap()) {
                if (staff.parent == null) {
                    result.push(staff);
                }
            }
            return result;
        }


        clear() {
            super.clear();
        }
    }

    class SpecialCatSystem {
        static init() {
        }
        static pushCat(data) {
            SpecialCatMgr.getInstance().pushCat(data);
        }

        static popCat() {
            SpecialCatMgr.getInstance().popCat();
        }

        static catLeave() {
            SpecialCatMgr.getInstance().catLeave();
        }

        static clear() {
            SpecialCatMgr.getInstance().clear();
            StaffCatMgr.getInstance().clear();
        }

    }

    class AutoZhaolanMgr {

        constructor() {
            this.maxCount = Number(Const.get("CatInviteClickTimes"));
            this.count = 0;
            this.isFull = false;
            this.fast = false;
            this._manual = 0;
            this._adZhaolanNum = 0;
            this._catsInvite = 0;
            this._show = true;
            EventMgr.getInstance().registEvent("COFFEE_CAT_NUMBER_CHANGE", this, this.ifFull);
            EventMgr.getInstance().registEvent("GUIDE_SET_GUIDING", this, this.openAuto);
            EventMgr.getInstance().registEvent("SWITCH_SCENE_COMPLETE", this, this.adAnmiPlay);
            EventMgr.getInstance().registEvent("USING_SKILL3", this, this.staffCatSkill3);
            if (!Laya.Browser.onMiniGame) {
                Laya.stage.on(Laya.Event.FOCUS, this, function () {
                    this.onShow();
                });
                Laya.stage.on(Laya.Event.BLUR, this, function () {
                    this._show = false;
                    Laya.timer.clear(this, this.autoZhaolan);
                });
            } else {
                let obj = this;
                wx.onShow(function () {
                    Laya.timer.frameOnce(1, obj, obj.onShow);
                });
                wx.onHide(function () {
                    obj._show = false;
                    Laya.timer.clear(obj, obj.autoZhaolan);
                });
            }
        }

        static getInstance() {
            return AutoZhaolanMgr.instance ? AutoZhaolanMgr.instance : AutoZhaolanMgr.instance = new AutoZhaolanMgr();
        }

        ifFull() {
            let catCount = CatSystem.getCatCountBySceneId(Constants.CoffeeSceneId);
            let catLimit = Number(Const.get("CatVisitLimit"));
            if (catCount >= catLimit) {
                if (this.isFull) {
                    return;
                }
                this.isFull = true;
            } else {
                if (this.isFull == false) {
                    return;
                }
                this.isFull = false;
                this.count = 0;
                EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", [0]);
            }
            EventMgr.getInstance().postEvent("ZHAOLAN_FULL", [this.isFull]);
        }

        getProgress() {
            return this.count / this.maxCount;
        }

        progressAdd() {
            if (this.isFull) {
                TipsPopMgr.getInstance().showTipsById("tid#catSenceNumber");
                return;
            }
            if (GameContext.isGuiding) {
                this.count += Math.ceil(this.maxCount / 3);
                this.count = Math.min(this.maxCount, this.count);
            } else {
                ++this.count;
            }
            ++this._manual;
            EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", [this.getProgress()]);
            TaskMainSystem.addProgress("CLICK#Manual", 1);
            if (this.count >= this.maxCount) {
                this.count = 0;
                if (GameContext.isGuiding) {
                    EventMgr.getInstance().postEvent("GUIDE_GUIDE6_INVITE");
                }
                ++this._catsInvite;
                MiniGameSystem.addProgress("CATS_INVITE", 1);
                EventMgr.getInstance().postEvent("NORMAL_CAT_ZHAOLAN");
                EventMgr.getInstance().postEvent("FORCE_ATTRACT_COFFEE_CAT", ["special"]);
                SpecialCatSystem.popCat();
                EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", [0]);
            }
        }

        setZhaolanSpeed(fast = false) {
            this.fast = fast;
            if (fast) {
                BarrageMgr.getInstance().randomStart("BarrageAdvanceInvite", "BarrageTriggerTime2");
                this.catInviteAutoClick = Const.get("CatInviteAutoClickADS") * 1000;
                this.adAnmiPlay();
            } else {
                this.catInviteAutoClick = Const.get("CatInviteAutoClick") * 1000;
                this.adAnmiStop();
            }
            if (!this.isFull) {
                EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", []);
            }
            Laya.timer.clear(this, this.autoZhaolan);
            Laya.timer.loop(this.catInviteAutoClick, this, this.autoZhaolan);
        }

        startFastZhaolan() {
            this.setZhaolanSpeed(true);
            this._adZhaolanNum++;
            TaskMainSystem.addProgress("AD_INVITE", 1);
        }

        autoZhaolan() {
            if (!this._show || this.isFull) {
                return;
            }
            ++this.count;
            if (this.count >= this.maxCount) {
                this.count = 0;
                ++this._catsInvite;
                MiniGameSystem.addProgress("CATS_INVITE", 1);
                EventMgr.getInstance().postEvent("NORMAL_CAT_ZHAOLAN");
                EventMgr.getInstance().postEvent("FORCE_ATTRACT_COFFEE_CAT", ["special"]);
                SpecialCatSystem.popCat();
                EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", [0]);
            }
        }

        getCatInviteAutoClick() {
            return this.catInviteAutoClick;
        }

        getLeftTimes() {
            return this.maxCount - this.count;
        }

        getFast() {
            return this.fast;
        }

        openAuto(value) {
            if (value) {
                this.count = 0;
                Laya.timer.clear(this, this.autoZhaolan);
                EventMgr.getInstance().postEvent("STOP_ZHAOLAN_BAR_TWEEN", [0]);
                this.adAnmiStop();
            } else {
                this.count = 0;
                Laya.timer.clear(this, this.autoZhaolan);
                this.setZhaolanSpeed(this.fast);
            }
        }

        adAnmiPlay(sceneid) {
            if (!this.fast) {
                return;
            }
            if (GameContext.isGuiding) {
                return;
            }
            if (sceneid == null) {
                sceneid = SceneMgr.getInstance().getCurrentSceneId();
            }
            if (sceneid == Constants.HomelandId || sceneid == Constants.FishingSceneId) {
                this.adAnmiStop();
                return;
            }
            if (this.anAnmi == null || this.anAnmi.destroyed) {
                this.anAnmi = new Spine();
                this.anAnmi.load("anim2/yinghua.sk");
                this.anAnmi.playAnim(0, true);
                this.anAnmi.zOrder = 0;
            } else {
                this.anAnmi.removeSelf();
            }

            let scene = SceneMgr.getInstance().getScene(sceneid);
            scene.addChild(this.anAnmi);
            this.anAnmi.x = scene.width / 2;
            this.anAnmi.y = -scene.height / 2;
        }

        adAnmiStop() {
            if (this.anAnmi == null) {
                return;
            }
            this.anAnmi.stop();
            this.anAnmi.destroy();
            this.anAnmi = null;
        }

        getManualNum() {
            return this._manual;
        }

        dumpAndStashZhaolanNum() {
            let arr = [];
            if (this._manual > 0) {
                arr.push({ "type": "CLICK#Manual", "value": this._manual, "extraInfo": null });
            }
            if (this._adZhaolanNum > 0) {
                arr.push({ "type": "AD_INVITE", "value": this._adZhaolanNum, "extraInfo": null });
            }
            if (this._catsInvite > 0) {
                arr.push({ "type": "CATS_INVITE", "value": this._catsInvite, "extraInfo": null });
            }
            this._manual = 0;
            this._adZhaolanNum = 0;
            this._catsInvite = 0;
            return arr;
        }

        clear() {
            if (this.anAnmi) {
                this.anAnmi.stop();
                this.anAnmi.destroy();
                this.anAnmi = null;
            }
            this.count = 0;
            this.isFull = false;
            this.fast = false;
            this._manual = 0;
            this._adZhaolanNum = 0;
            this._catsInvite = 0;
            Laya.timer.clearAll(this);
        }

        onShow() {
            this._show = true;
            this.count = 0;
            EventMgr.getInstance().postEvent("RESET_ZHAOLAN_BAR", [0]);
            this.setZhaolanSpeed(this.fast);
        }

        staffCatSkill3(num) {
            for (let index = 0; index < num; index++) {
                MiniGameSystem.addProgress("CATS_INVITE", 1);
                EventMgr.getInstance().postEvent("FORCE_ATTRACT_COFFEE_CAT", ["special","skill"]);
            } 
        }
    }

    class EntrustData {

        constructor(data) {
            this.config = ConfigMgr$1.getInstance().getItemById("Entrustment", data.id);
            this.listConfig = ConfigMgr$1.getInstance().getItemById("EntrustmentList", data.listId);
            this.catId = ++EntrustData.catIdMax;
            if (data.status != null) {
                this.status = data.status;
            } else {
                this.status = 0;
            }
            this.conditions = [];
            data.conditions.forEach(element => {
                let arr = [];
                Object.getOwnPropertyNames(element.values).forEach(key => {
                    let condition = element.values[key];
                    arr.push(condition);
                });
                this.conditions.push(arr);
            });
        }

        getId() {
            return this.config.Id;
        }

        getCatId() {
            return "EntrustCat" + this.catId;
        }

        getListId() {
            return this.listConfig.Id;
        }

        getName() {
            return this.config.Name;
        }

        getDesc() {
            return this.config.Desc;
        }

        getConditionDesc() {
            return this.config.ConditionDesc;
        }

        getReward() {
            return JSON.parse(this.config.Reward);
        }

        getAcceptTalk() {
            return this.config.AcceptTalk;
        }

        getFinishTalk() {
            return this.config.FinishTalk;
        }

        getWXMessageTalk() {
            return this.config.WXMessage;
        }

        inDate() {
            if (this.listConfig.RegistrationDate.length > 0) {
                let registerDays = PlayerSystem.getRegisterDays();
                if (registerDays >= this.listConfig.RegistrationDate[0] && this.listConfig.RegistrationDate <= this.listConfig.RegistrationDate[1]) {
                    return true;
                }
                return false;
            } else {
                return true;
            }
        }

        getStatus() {
            return this.status;
        }

        getFastConditionProgress() {
            let fastCondition = null;
            let progress = 0;
            let conditions = this.getConditions();
            for (const condition of conditions) {
                let cProgress = 0;
                let current = 0;
                let target = 1;
                for (const entrustValue of condition) {
                    if (entrustValue.current != null) {
                        current += entrustValue.current;
                    }
                    target += entrustValue.target;
                }
                cProgress = current / target;
                if (cProgress >= progress) {
                    fastCondition = condition;
                    progress = cProgress;
                }
            }
            let ret = { condition: fastCondition, progress: progress };
            return ret;
        }

        getConditions() {
            return this.conditions;
        }

        getDataName() {
            return "Entrust";
        }

        setCatTalkShow() {
            this._catTalkShow = true;
        }

        getCatTalkShow() {
            return this._catTalkShow;
        }

    }
    EntrustData.catIdMax = 0;

    class EntrustDataMgr {

        constructor() {
            this._map = new Map();
        }

        static getInstance() {
            return EntrustDataMgr.instance ? EntrustDataMgr.instance : EntrustDataMgr.instance = new EntrustDataMgr();
        }

        updateEntrusts(entrusts) {
            this.clear();
            Object.getOwnPropertyNames(entrusts.entrustMap).forEach(key => {
                let entrust = entrusts.entrustMap[key];
                let entrustData = new EntrustData(entrust);
                this._map.set(key, entrustData);
            });
        }

        updataOne(entrust) {
            let entrustData = new EntrustData(entrust);
            this._map.set(entrustData.getListId(), entrustData);
            return entrustData;
        }

        getEntrusts() {
            let array = [];
            this._map.forEach(element => {
                if (element != null) {
                    array.push(element);
                }
            });
            return array;
        }

        clear() {
            this._map.clear();
        }

        getCurrentEntrust() {
            let current = null;
            this._map.forEach(element => {
                if (element.getStatus() == 0 || element.getStatus() == 2) {
                    current = element;
                    return;
                }
            });
            return current;
        }

        getCurrentDoingEntrusts() {
            let currents = [];
            this._map.forEach(element => {
                if (element.getStatus() == 1) {
                    currents.push(element);
                }
            });
            return currents;
        }

        getCurrentConditionEvents() {
            let arr = [];
            let currentEntrusts = this.getCurrentDoingEntrusts();
            for (const entrust of currentEntrusts) {
                let conditions = entrust.getConditions();
                conditions.forEach(condition => {
                    condition.forEach(element => {
                        if (element.eventType != "TIME") {
                            arr.push(element.eventType);
                        }
                    });
                });
            }
            return arr;
        }

    }

    class LuckyBagDataMgr {

        constructor() {
            this._onlineTime = null;
            this._receiveTimes = null;
            this._startTime = TimeUtils.getTimeNowOnServer();
            Laya.timer.once(500,this,function(){
                this._registerDays = PlayerSystem.getGameCreateDays();
                if(this._registerDays <= Number(Const.get("LuckBagMaxDays"))){
                    this._luckBagReward = JSON.parse(Const.get("LuckBagReward"))[this._registerDays <= 7 ? this._registerDays : 7];
                    if(this.checkReceiveTimes()){
                        this.setWaitingTime();
                    }              
                }
            });
            this.flag = 1;
        }

        static getInstance() {
            return LuckyBagDataMgr.instance ? LuckyBagDataMgr.instance : LuckyBagDataMgr.instance = new LuckyBagDataMgr();
        }

        setLock(num){
            this.lock = num;
            this.setStartTime();
        }

        setStartTime(){
            if(this.lock == 1){
                this._startTime = TimeUtils.getTimeNowOnServer();
                this._onlineTime = 0;
                this._baseTime = 0;
            }
        }

        update(luckyBagData) {
            this._receiveTimes = 0;
            if (luckyBagData != null) {
                this._receiveTimes = luckyBagData.receiveTimes ? luckyBagData.receiveTimes : 0;
                this._fishAmount = luckyBagData.fishAmount ? luckyBagData.fishAmount : 0;
            } 
            let days = this._registerDays;
            this._registerDays = PlayerSystem.getGameCreateDays();
            if(days){
                this.upDateOnlineDays(days);
            }
            if(this._registerDays <= Number(Const.get("LuckBagMaxDays")) && this._luckBagReward){
                this._luckBagReward = JSON.parse(Const.get("LuckBagReward"))[this._registerDays <= 7 ? this._registerDays : 7];
                this._fishAmount = luckyBagData.fishAmount;
                this.setWaitingTime();
                if(this.flag){
                    this._baseTime = 0;
                }
                else {
                    this.flag = 1;
                }
                this._startTime = TimeUtils.getTimeNowOnServer();
                this.getOnlineTime();
            }
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
            EventMgr.getInstance().postEvent("CHECK_CLOSE_LUCKYBAG");
            EventMgr.getInstance().postEvent("UPDATE_LUCKYBAG_DIALOG");
        }

        checkCloseluckyBag(){
            if(this.waitingTime>=0){
                EventMgr.getInstance().postEvent("CHECK_CLOSE_LUCKYBAG");
            }
        }

        //秒
        getOnlineTime() {
            this._onlineTime = Math.floor((TimeUtils.getTimeNowOnServer() - this._startTime) / 1000) + this._baseTime;
            return this._onlineTime ;
        }

        getRegisterDays(){
            this._registerDays = PlayerSystem.getGameCreateDays();
            return this._registerDays;
        }

        getReceiveTimes() {
            return this._receiveTimes;
        }

        upDateOnlineTime(onlineTime){
            this._baseTime = onlineTime;
        }

        upDateOnlineDays(days){
            Laya.timer.once(100,this,function(){
                if(days < this._registerDays){
                    this._onlineTime = 0;
                    this._baseTime = 0;
                }
            });
        }


        clear() {
            this.flag = 0;
            this._onlineTime = null;
            this._receiveTimes = null;
            this._startTime = null;
            this._baseTime = null;
            this._registerDays = null;
        }

        getReward(){
            return this._fishAmount;
        }

        setWaitingTime(){
            if(this._receiveTimes < this._luckBagReward.length){
                this.waitingTime = this._luckBagReward[this._receiveTimes].Time*60;
            }
        }

        getRemainingTime(){
            this.getOnlineTime();
            if(this.waitingTime > this._onlineTime){
                this.remainingTime = this.waitingTime - this._onlineTime;
            }else {
                this.remainingTime = 0;
            }
            return this.remainingTime;
        }

        checkReceiveTimes(){
            if(this._receiveTimes>=0 && this._luckBagReward){
                return this._receiveTimes < this._luckBagReward.length ;
            }
            return false;
        }
    }

    class StaffSkillData {

        constructor(groupId, id, staffGroupId) {
            this.staffGroupId = staffGroupId;
            this.groupId = groupId;
            this.skillId = id;
            if (id != null) {
                this.config = ConfigMgr$1.getInstance().getItemById("StaffSkill", id);
            }
            this.groupConfig = ConfigMgr$1.getInstance().getItemById("StaffSkillGroup", groupId);
        }

        getUnlock() {
            return this.skillId ? true : false;
        }

        getId() {
            return this.skillId;
        }

        getConfigId() {
            return this.config.Id;
        }

        getEffect() {
            return this.config.Effect;
        }

        getSkillTimeData() {
            return this.config.SkillData;
        }

        getStaffLvUpCost() {
            return this.config.LvUpCost;
        }

        getStaffLvUpTime() {
            return this.config.LvUpTime;
        }

        getName() {
            return this.groupConfig.Name;
        }

        getDesc() {
            return this.groupConfig.Desc;
        }

        getLevel() {
            return this.config ? this.config.Level : 0;
        }

        getGroupId() {
            return this.groupId;
        }

        getStaffId() {
            return this.staffGroupId;
        }

        set skillCdTimer(timer) {
            this._skill1CdTimer = timer;
        }

        get skillCdTimer() {
            return this._skill1CdTimer;
        }

        set skillTimer(timer) {
            this._skill1Timer = timer;
        }

        get skillTimer() {
            return this._skill1Timer;
        }

        set skillLevelUpEndTimer(timer) {
            this._skillLevelUpEndTimer = timer;
        }

        get skillLevelUpEndTimer() {
            return this._skillLevelUpEndTimer;
        }

        updata(id) {
            this.skillId = id;
            if (id != null) {
                this.config = ConfigMgr$1.getInstance().getItemById("StaffSkill", id);
            }
        }

        updataTimerData(data) {
            if (data == null) {
                return;
            }
            if (data.groupId == this.groupId) {
                this.skillCdTimer = data.skillCdTimer;
                this.skillTimer = data.skillTimer;
                // this.skillLevelUpEndTimer = data.skillLevelUpEndTimer;
            }
        }

        dumpParams() {
            return {
                "groupId": this.groupId,
                "skillCdTimer": this.skillCdTimer,
                "skillTimer": this.skillTimer,
                // "skillLevelUpEndTimer": this.skillLevelUpEndTimer
            };
        }

    }

    class StaffCatData {

        constructor(id, staffData) {//{ "staffGroupId": staffGroupId, "staffId": staffId, "skillGroup": skillGroup }
            this.config = ConfigMgr$1.getInstance().getItemById("Staff", staffData.staffId);
            this.staffGroupId = id;
            this.currentPathIndex = 0;
            if (staffData.skillAmount) {
                this._skillAmount = staffData.skillAmount;
            }
            this._skillMap = new Map();
            this.updataSkillData(staffData.skillGroup, staffData.endTimeStamp);
        }

        setParams(params) {
            this.params = params;
        }

        getParams() {
            return this.params;
        }

        getId() {
            return this.staffGroupId;
        }

        getConfigId() {
            return this.config.Id;
        }

        getSkin() {
            return this.config.Skin;
        }

        getStaffName() {
            return this.config.Name;
        }

        getStaffDesc() {
            return this.config.Desc;
        }

        getStaffLevel() {
            return this.config.Level;
        }

        getNextStaff() {
            return this.config.NextStaff;
        }

        getPrice() {
            return this.config.Price;
        }

        getSkillLvLimite() {
            return this.config.SkillLvLimite;
        }

        getHeadImage() {
            return this.config.Image;
        }

        getSkillGroup() {
            return this.config.SkillGroup;
        }

        getLvUpCondition() {
            return this.config.LvUpCondition;
        }

        getSkillAmount() {
            return this._skillAmount ? this._skillAmount : 0;
        }

        setSkillAmount(value) {
            this._skillAmount = value[this.getId()];
        }

        updata(staffId, skillAmount) {
            this.config = ConfigMgr$1.getInstance().getItemById("Staff", staffId);
            if (skillAmount != null) {
                this._skillAmount = skillAmount;
            }
        }

        pos(x, y) {
            this._x = x;
            this._y = y;
        }

        get x() {
            return this._x;
        }

        get y() {
            return this._y;
        }

        get speed() {
            return 180;//250
        }

        set skill1CdTimer(timer) {
            let skill = this._skillMap.get("SkillGroup_1");
            skill.skillCdTimer = timer;
        }

        get skill1CdTimer() {
            let skill = this._skillMap.get("SkillGroup_1");
            return skill ? skill.skillCdTimer : null;
        }

        set skill2CdTimer(timer) {
            let skill = this._skillMap.get("SkillGroup_2");
            skill.skillCdTimer = timer;
        }

        get skill2CdTimer() {
            let skill = this._skillMap.get("SkillGroup_2");
            return skill ? skill.skillCdTimer : null;
        }

        set skill3CdTimer(timer) {
            let skill = this._skillMap.get("SkillGroup_3");
            skill.skillCdTimer = timer;
        }

        get skill3CdTimer() {
            let skill = this._skillMap.get("SkillGroup_3");
            return skill ? skill.skillCdTimer : null;
        }

        set skill1Timer(timer) {
            let skill = this._skillMap.get("SkillGroup_1");
            skill.skillTimer = timer;
        }

        get skill1Timer() {
            let skill = this._skillMap.get("SkillGroup_1");
            return skill ? skill.skillTimer : null;
        }

        set skill2Timer(timer) {
            let skill = this._skillMap.get("SkillGroup_2");
            skill.skillTimer = timer;
        }

        get skill2Timer() {
            let skill = this._skillMap.get("SkillGroup_2");
            return skill ? skill.skillTimer : null;
        }

        set skill3Timer(timer) {
            let skill = this._skillMap.get("SkillGroup_3");
            skill.skillTimer = timer;
        }

        get skill3Timer() {
            let skill = this._skillMap.get("SkillGroup_3");
            return skill ? skill.skillTimer : null;
        }

        //{"CDTime":300,"EffectTime":60}
        getSkill1Data() {
            let skill = this._skillMap.get("SkillGroup_1");
            if (skill) {
                return JSON.parse(skill.getSkillTimeData());
            }
            return null;
        }
        //{"CDTime":300,"EffectTime":60}
        getSkill2Data() {
            let skill = this._skillMap.get("SkillGroup_2");
            if (skill) {
                return JSON.parse(skill.getSkillTimeData());
            }
            return null
        }
        //{"CDTime":360,"InviteAmount":5}
        getSkill3Data() {
            let skill = this._skillMap.get("SkillGroup_3");
            if (skill) {
                return JSON.parse(skill.getSkillTimeData());
            }
            return null
        }

        getSkillData() {
            return this._skillMap;
        }

        getSkillDataByGroupId(id) {
            return this._skillMap.get(id);
        }

        updataSkillData(data, endCDTimeMap) {
            let groups = ["SkillGroup_1", "SkillGroup_2", "SkillGroup_3"];
            groups.forEach(skillGroupId => {
                let skillId = data[skillGroupId];
                if (this._skillMap.has(skillGroupId)) {
                    let skill = this._skillMap.get(skillGroupId);
                    skill.updata(skillId);
                    if (endCDTimeMap && endCDTimeMap[skillGroupId]) {
                        skill.skillLevelUpEndTimer = Number(endCDTimeMap[skillGroupId]) + 1;
                    }
                } else {
                    let skill = new StaffSkillData(skillGroupId, skillId, this.getId());
                    this._skillMap.set(skillGroupId, skill);
                    if (endCDTimeMap && endCDTimeMap[skillGroupId]) {
                        skill.skillLevelUpEndTimer = Number(endCDTimeMap[skillGroupId]) + 1;
                    }
                }
            });
        }

        updataSkillDataById(id, data) {
            let skill = this._skillMap.get(id);
            skill.upData(data);
        }

        dumpParams() {
            let skill1 = this._skillMap.get("SkillGroup_1").dumpParams();
            let skill2 = this._skillMap.get("SkillGroup_2").dumpParams();
            let skill3 = this._skillMap.get("SkillGroup_3").dumpParams();
            return {
                "SkillGroup_1": skill1,
                "SkillGroup_2": skill2,
                "SkillGroup_3": skill3,
            };
        }

        updataSkillTimerData(data) {
            if (data == null) {
                return;
            }
            let skill1 = this._skillMap.get("SkillGroup_1");
            let skill2 = this._skillMap.get("SkillGroup_2");
            let skill3 = this._skillMap.get("SkillGroup_3");
            skill1.updataTimerData(data.SkillGroup_1);
            skill2.updataTimerData(data.SkillGroup_2);
            skill3.updataTimerData(data.SkillGroup_3);
        }

        getSkillLevelNum() {
            let level = 0;
            this._skillMap.forEach(element => {
                level += element.getLevel();
            });
            return level;
        }

    }

    class StaffCatDataMgr {

        constructor() {
            this._staffsMap = new Map();
            this._initData = false;
            this._params = null;
        }

        static getInstance() {
            return StaffCatDataMgr.instance ? StaffCatDataMgr.instance : StaffCatDataMgr.instance = new StaffCatDataMgr();
        }

        updataAllStaff(data) {
            data.forEach(staffData => {
                if (this._staffsMap.has(staffData.staffGroupId)) {
                    let staffCatData = this._staffsMap.get(staffData.staffGroupId);
                    staffCatData.updata(staffData.staffId, staffData.skillAmount);
                    staffCatData.updataSkillData(staffData.skillGroup, staffData.endTimeStamp);
                } else {
                    let staffCatData = new StaffCatData(staffData.staffGroupId, staffData);
                    this._staffsMap.set(staffData.staffGroupId, staffCatData);
                    this._currentId = staffData.staffGroupId;
                }
                this._initData = true;
                if (this._params != null) {
                    this.updataParams(this._params);
                    this._params = null;
                }
            });
        }

        clear() {
            this._staffsMap.clear();
            this._initData = false;
            this._params = null;
        }

        getAllStaffs() {
            return this._staffsMap;
        }

        getStaffData(staffId) {
            return this._staffsMap.get(staffId);
        }

        updateStaff(staffId, staffInfo) {
            let staff = this.getStaff(staffId);
            if (!staff) {
                return false;
            }
            staff.updata(staffInfo);
            return staff;
        }

        getCurrentId() {
            return this._currentId;
        }

        updataAllSkill(data) {
            StaffCatDataMgr.getInstance().updataAllSkill(data);
        }

        dumpParams() {
            let arr = [];
            this._staffsMap.forEach((staff) => {
                arr.push({ k: staff.getId(), v: staff.dumpParams() });
            });
            return JSON.stringify(arr);
        }

        updataParams(data) {
            if (data == null) {
                return;
            }
            if (!this._initData) {
                this._params = data;
                return;
            }
            let arr = JSON.parse(data);
            arr.forEach(element => {
                let staff = this._staffsMap.get(element.k);
                staff.updataSkillTimerData(element.v);
            });
        }

        getSkillData(staffGroupId, skillGroupId) {
            let staff = this.getStaffData(staffGroupId);
            let skill = staff.getSkillDataByGroupId(skillGroupId);
            return skill;
        }
    }

    class SyncSystem {
        static init() {
            EventMgr.getInstance().registEvent("UPDATE_SCENE_INFO_NOW", null, SyncSystem.updateSceneInfo);
            EventMgr.getInstance().registEvent("ADD_HOME_RUBBISH_REWARDS", null, SyncSystem.addHomeRubbishRewards);
        }

        static scheduleUpdate() {
            // console.debug("scheduleUpdate");
            SyncSystem.operatingTime = null;
            SyncSystem.sysytemUpdteTime = TimeUtils.getTimeNowOnServer() + Constants.SyncDelay;
            Laya.timer.loop(1000, SyncSystem, SyncSystem.updateLoop, [], true, false);
        }

        static forceUpdate(handler) {
            SyncSystem.updateSceneInfo(handler);
            SyncSystem.scheduleUpdate();
        }

        static updateLoop() {
            if ((TimeUtils.getTimeNowOnServer() >= SyncSystem.sysytemUpdteTime) || (SyncSystem.operatingTime && TimeUtils.getTimeNowOnServer() >= SyncSystem.operatingTime)) {
                SyncSystem.updateSceneInfo();
            }
        }

        static updateSceneInfo(handler) {
            SyncSystem.operatingTime = null;
            SyncSystem.sysytemUpdteTime = TimeUtils.getTimeNowOnServer() + Constants.SyncDelay;
            // let customData = SyncDataMgr.getInstance().dump();
            let customData = JSON.stringify({lastTime: TimeUtils.getTimeNowOnServer(),
                scene2ExhaustTime: SyncDataMgr.getInstance().getScene2ExhaustTime(),
                cats: CatDataMgr.getInstance().dump(), fish: SyncDataMgr.getInstance().dumpCachedFish(),
                homeParams: PetDataMgr.getInstance().dumpHomeParams(),
                luckyBagTime: LuckyBagDataMgr.getInstance().getOnlineTime(),
                registerDays: LuckyBagDataMgr.getInstance().getRegisterDays(),
                staffParams: StaffCatDataMgr.getInstance().dumpParams()
            });
            // console.debug("updateSceneInfo", customData);
            let rewards = SyncDataMgr.getInstance().dumpRewards();

            let petRewards = PetDataMgr.getInstance().dumpAndStashPetRewards();
            let visitorRewards = SyncDataMgr.getInstance().dumpVisitorRewardsCache();
            let events = [];
            let taskObj=PetDataMgr.getInstance().getTaskVisitTimes();
            let visitTimesIncr = taskObj.times;
            let map = taskObj.result;
            if (visitTimesIncr > 0) {
                events = [{ "type": "CATS_VISIT", "value": visitTimesIncr, "extraInfo": null }];
            }
            let entrustEvents = EntrustDataMgr.getInstance().getCurrentConditionEvents();
            if(entrustEvents!=null){
                entrustEvents.forEach(element => {
                    let num = map.get("Cat" + element.substring(16));
                    if (num != null) {
                        events.push({ "type": element, "value": num, "extraInfo": null });
                    }
                });
            }
            let zhaolanArr = AutoZhaolanMgr.getInstance().dumpAndStashZhaolanNum();
            events = events.concat(zhaolanArr);

            let eatTimes = SyncDataMgr.getInstance().dumpEatTimes();
              
            if (eatTimes && eatTimes > 0) {
                events.push({ "type": "ORDER", "value": eatTimes, "extraInfo": null });
            }
            EventMgr.getInstance().postEvent("SERVICE_UPDATE_SCENE_INFO", [customData, rewards, petRewards, visitorRewards, events, handler]);
        }

        static dropCache() {
            SyncDataMgr.getInstance().dropRewards();
            PetDataMgr.getInstance().dropPetRewardsStash();
            SyncDataMgr.getInstance().dropVisitorRewardsCache();
        }


        //场景2 投食
        static getScene2ExhaustTime() {
            return SyncDataMgr.getInstance().getScene2ExhaustTime();
        }

        static addScene2Food(time, quality) {
            let exhaustTime = SyncDataMgr.getInstance().addScene2Food(time, quality);
            if(exhaustTime!=null) {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.BeachSceneId, "REFRESH_FOOD", exhaustTime]);
                EventMgr.getInstance().postEvent("REFRESH_FOOD", [exhaustTime]);
            }
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catfood_put_success"]);
            return exhaustTime;
        }

        //场景2 常客礼物
        static cacheVisitorReward(catId, catConfigId, buildingId, reward, rewardType) {
            SyncDataMgr.getInstance().cacheVisitorReward(catId, catConfigId, buildingId, reward, rewardType);
        }

        //常客礼物 的相关操作
        static findVisitorGift(handler) {
            EventMgr.getInstance().postEvent("SERVICE_FIND_VISITOR_GIFT", [handler]);
        }

        static getVisitorGiftAll(fetchType, handler) {
            TaskMainSystem.addProgress("BEACH_GIFT", 1);
            EventMgr.getInstance().postEvent("SERVICE_GET_VISITOR_GIFT_ALL", [fetchType, handler]);
            SyncSystem.startOperatingUpdate();
        }

        static getVisitorGiftSingle(giftId, handler) {
            TaskMainSystem.addProgress("BEACH_GIFT", 1);
            EventMgr.getInstance().postEvent("SERVICE_GET_VISITOR_GIFT_SINGLE", [giftId, handler]);
            SyncSystem.startOperatingUpdate();
        }

        static clearVisitorGift() {
            SyncDataMgr.getInstance().clearVisitorGift();
        }

        static updateVisitorGift(visitorRewards) {
            SyncDataMgr.getInstance().updateVisitorGift(visitorRewards);
        }

        static getVisitorGifts() {
            return SyncDataMgr.getInstance().getVisitorGifts();
        }

        static setVisitorRewardsExist(value) {
            SyncDataMgr.getInstance().setVisitorRewardsExist(value);
        }

        static getVisitorRewardsExist() {
            return SyncDataMgr.getInstance().getVisitorRewardsExist();
        }


        static restoreCustomData(customData) {
            // console.debug("restoreCustomData", customData);
            let data = customData?JSON.parse(customData):{};
            let lastTime = data.lastTime;
            let scene2ExhaustTime = data.scene2ExhaustTime;
            let luckyBagTime = data.luckyBagTime ? data.luckyBagTime: 0;
            let registerDays = data.registerDays;
            let staffParams = data.staffParams;
            SyncDataMgr.getInstance().restoreScene2ExhaustTime(scene2ExhaustTime);
            SyncDataMgr.getInstance().restoreFish(data.fish);
            LuckyBagDataMgr.getInstance().upDateOnlineTime(luckyBagTime);
            LuckyBagDataMgr.getInstance().upDateOnlineDays(registerDays);
            EventMgr.getInstance().postEvent("UPDATE_DROPFISH_ALL", []);
            EventMgr.getInstance().postEvent("CUSTOMDATA_CAT_RESTORE", [data.cats, lastTime, scene2ExhaustTime]);
            EventMgr.getInstance().postEvent("CUSTOMDATA_PET_RESTORE", [data.homeParams, lastTime]);
            EventMgr.getInstance().postEvent("CUSTOMDATA_STAFF_RESTORE", [staffParams, lastTime]);
        }

        //rewards SyncData

        static getCachedFish(buildingBaseId) {
            return SyncDataMgr.getInstance().getCachedFish(buildingBaseId);
        }

        static getCachedStar(buildingBaseId) {
            return SyncDataMgr.getInstance().getCachedStar(buildingBaseId);
        }

        static addCachedFish(buildingBaseId, count) {
            if(buildingBaseId==null)
                return;
            SyncDataMgr.getInstance().addCachedFish(buildingBaseId, count);
        }

        static addCachedStar(buildingBaseId, count) {
            SyncDataMgr.getInstance().addCachedStar(buildingBaseId, count);
        }

        static collectReward(buildingBaseId) {
            let reward = SyncDataMgr.getInstance().collectReward(buildingBaseId);
            let fish = reward[0];
            let star = reward[1];
            SyncSystem.startOperatingUpdate();
            if(fish>0) {
                EventMgr.getInstance().postEvent("REFRESH_FISH", fish);
            }
            if(star>0) {
                EventMgr.getInstance().postEvent("REFRESH_STAR", star);
            }
        }


        //LetterBox

        static getLetterBoxInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_LETTER_BOX_INFO", [handler]);
        }

        static fetchLetterBox(fetchType, handler) {
            EventMgr.getInstance().postEvent("SERVICE_FETCH_LETTER_BOX", [fetchType, handler]);
            SyncSystem.startOperatingUpdate();
        }

        static updateLetterBox(letterBox) {
            if(letterBox.capacity!=null) {
                // console.debug("updateLetterBox capacity", letterBox.capacity);
                LetterBoxMgr.getInstance().setCapacity(letterBox.capacity);
            }
            // console.debug("updateLetterBox fishToReward", letterBox.fishToReward);
            LetterBoxMgr.getInstance().setCacheFish(letterBox.fishToReward||0);

            if(letterBox.lastUpdateMillis) {
                // console.debug("updateLetterBox lastUpdateMillis", letterBox.lastUpdateMillis);
                let lastUpdateTime = letterBox.lastUpdateMillis;
                if(lastUpdateTime==0) {
                    lastUpdateTime = LetterBoxMgr.getInstance().getLastUpdateTime();
                    if(lastUpdateTime==null)
                        lastUpdateTime = TimeUtils.getTimeNowOnServer();
                }
                LetterBoxMgr.getInstance().setLastUpdateTime(lastUpdateTime);
            }
            EventMgr.getInstance().postEvent("UPDATE_LETTER_BOX");
        }
        static updateZhuliBox(){
            EventMgr.getInstance().postEvent("UPDATE_ZHULI_BOX");
        }

        static setCacheFish(fish) {
            LetterBoxMgr.getInstance().setCacheFish(fish);
        }

        static getCacheFish() {
            return LetterBoxMgr.getInstance().getCacheFish();
        }

        static getCacheFishCapacity() {
            return LetterBoxMgr.getInstance().getCapacity();
        }

        static getLetterBoxNextUpdateInterval() {
            let lastUpdateTime = LetterBoxMgr.getInstance().getLastUpdateTime()||TimeUtils.getTimeNowOnServer();
            let result = lastUpdateTime + Constants.LetterBoxInterval - TimeUtils.getTimeNowOnServer();
            return result;
        }

        static clear() {
            Laya.timer.clear(null, SyncSystem.updateLoop);
            SyncDataMgr.getInstance().clear();
            LetterBoxMgr.getInstance().clear();
        }

        static startOperatingUpdate() {
            if (SyncSystem.operatingTime) {
                if (TimeUtils.getTimeNowOnServer() < SyncSystem.operatingTime && TimeUtils.getTimeNowOnServer() > SyncSystem.operatingTime - 2000) {
                    SyncSystem.operatingTime = TimeUtils.getTimeNowOnServer() + 2000;
                }
            } else {
                SyncSystem.operatingTime = TimeUtils.getTimeNowOnServer() + Constants.OperatingUpdateCD;
            }
        }


        // 打点

        static addEatTimes() {
            SyncDataMgr.getInstance().addEatTimes();
        }
        static setLoginTime(timeOffSet){
            TimeUtils.setLoginTime(timeOffSet);
        }

        //家园垃圾奖励
        static addHomeRubbishRewards(star, fish) {
            SyncSystem.startOperatingUpdate();
            if(fish>0) {
                EventMgr.getInstance().postEvent("SECRETLY_REFRESH_FISH", fish);
            }
            if(star>0) {
                EventMgr.getInstance().postEvent("SECRETLY_REFRESH_STAR", star);
            }
            SyncDataMgr.getInstance().addHomeRubbishRewards(star, fish);
        }

        static getNextFishBuildingBaseId() {
            let fishMap = SyncDataMgr.getInstance().getCacheFishMap();
            let maxId = null;
            let maxFish = 0;
            fishMap.forEach((value, id) => {
                if (value > maxFish) {
                    maxFish = value;
                    maxId = id;
                }
            });
            return maxId;
        }

    }

    class SoundManager {

        constructor() {
            this.soundConfigMap = this._getSoundMap();
            this.soundList = [];
            this.soundListPlaying = false;
            this.bgmID = null;
            this.getLocalData();
            Laya.SoundManager.autoReleaseSound=false;
            EventMgr.getInstance().registEvent("SOUND_REPLAY_BGM", this, this.replayBgm);
            EventMgr.getInstance().registEvent("SOUND_SET_ALL_MUTED", this, this.setAllMuted);
            EventMgr.getInstance().registEvent("SOUND_RESET_LOCAL_SOUND_SETTING", this, this.getLocalData);
        }

        static getInstance() {
            return SoundManager.instance ? SoundManager.instance : SoundManager.instance = new SoundManager();
        }


        _getSoundMap() {
            return ConfigMgr$1.getInstance().getConfig("Sound");
        }

        _getSoundByID(id) {
            return this.soundConfigMap.get(id);
        }

        play(id, times = null, handler = null) {
            let config = this._getSoundByID(id);
            if (times == null) {
                if (config.Type == 1) {
                    times = 0;
                } else {
                    times = 1;
                }
            }
            if (config.Type == 1) {
                if (this.bgmID && this.bgmID == id) {
                    return;
                }
                this.bgmID = id;
                if (this.getMusicMuted() || this.getAllMuted()) {
                    return;
                }
                if(SoundManager._isLoading == true) {
                    console.log("SoundManager is _isLoading");
                    return;
                }
                if(Constants.lastMusic){
                  Laya.SoundManager.destroySound(Constants.lastMusic);
                }
                 Laya.SoundManager.playMusic(config.Sound, times, handler);
                 Constants.lastMusic=config.Sound;
            } else {
                if(SoundManager._isLoading == true) {
                    console.log("SoundManager is _isLoading");
                    return;
                }
                 if(Constants.lastSound){
                     Laya.SoundManager.destroySound(Constants.lastSound);
                 }
                Laya.SoundManager.playSound(config.Sound, times, handler);
               Constants.lastSound=config.Sound;
            }
            // console.log(config.Sound, "     ", times);
        }

        /**
         * 背景音乐音量。
         * @default 0~1
         */
        setBgmVolume(volume) {
            Laya.SoundManager.musicVolume = volume;
        }

        /**
         * 音效音量。
         * @default 0~1
         */
        setSoundVolume(volume) {
            Laya.SoundManager.soundVolume = volume;
        }

        /**
         * 修改指定音效或背景音乐音量。
         * @param
         * @default 0~1
         */
        setVolumeByID(id, volume) {
            let config = this._getSoundByID(id);
            if (config) {
                Laya.SoundManager.setSoundVolume(volume, config.Sound);
            }
        }

        _popSoundList() {
            let soundId = this.soundList[0];
            if (!soundId) {
                return null;
            }
            this.soundList.splice(0, 1);
            return soundId;
        }

        /**
         * 按顺序播放音效。
         */
        playSoundList(id) {
            this.soundList.push(id);
            if (this.soundListPlaying) {
                return;
            }
            this._playByList();
        }

        _playByList() {
            let sound = this._popSoundList();
            if (sound != null) {
                this.soundListPlaying = true;
                this.play(sound, 1, Laya.Handler.create(this, this._playByList));
            } else {
                this.soundListPlaying = false;
            }
        }

        /**
         * 停止播放所有音效（不包括背景音乐）
         */
        stopAllSound() {
            Laya.SoundManager.stopAllSound();
        }

        /**
         * 停止播放背景音乐（不包括音效）
         */
        stopBgm() {
            Laya.SoundManager.stopMusic();
        }

        /**
         * 停止声音播放。此方法能够停止任意声音的播放（包括背景音乐和音效），只需传入对应id。
         */
        stopById(id) {
            let config = this._getSoundByID(id);
            if (config) {
                Laya.SoundManager.stopSound(config.Sound);
            }
        }

        /**
         * 所有音效（不包括背景音乐）是否静音。
         */
        setSoundMuted(value) {
            Laya.SoundManager.soundMuted = value;
            this.localSave();
        }

        /**
         * 所有音效（不包括背景音乐）是否静音。
         */
        getSoundMuted() {
            return Laya.SoundManager.soundMuted;
        }

        /**
         * 背景音乐（不包括音效）是否静音。
         */
        setMusicMuted(value) {
            Laya.SoundManager.musicMuted = value;
            this.localSave();
            if (!value) {
                this.replayBgm("bgm");
            } else {
                this.stopBgm();
            }
        }

        /**
         * 背景音乐（不包括音效）是否静音。
         */
        getMusicMuted() {
            return Laya.SoundManager.musicMuted;
        }

        /**
         * 背景音乐和所有音效是否静音。
         */
        setAllMuted(value) {
            Laya.SoundManager.muted = value;
        }

        getAllMuted() {
            return Laya.SoundManager.muted;
        }

        /**
         * 重新播放当前背景音乐
         */
        replayBgm(args) {
            if(args!="bgm"){
                return;
            }
            if (this.bgmID == null) {
                return;
            }
            let id = this.bgmID;
            this.bgmID = null;
            this.play(id, 0);
        }

        localSave() {
            let soundData = {
                musicMuted: this.getMusicMuted(),
                soundMuted: this.getSoundMuted()
            };
            Laya.LocalStorage.setJSON("soundData", soundData);
        }

        getLocalData() {
            let soundData = Laya.LocalStorage.getJSON("soundData");
            if (!soundData) {
                this.localSave();
            } else {
                this.setSoundMuted(soundData.soundMuted);
                this.setMusicMuted(soundData.musicMuted);
            }
        }

    }

    class BuildingExtInfoMgr {

        constructor(buildingData) {
            this.buildingData = buildingData;
            this.dropPoint = null;
            this.dropFish = [];
            this.buildingExtInfoArray = new Array();
        }

        pushBuildingExtInfo(buildingExtInfo) {
            buildingExtInfo.setBuildingExtInfoMgr(this);
            this.buildingExtInfoArray.push(buildingExtInfo);
        }

        getBuildingExtInfo(spaceId) {
            return this.buildingExtInfoArray[spaceId];
        }

        getBuildingExtInfoCount() {
            return this.buildingExtInfoArray.length;
        }

        setDropFishPoint(dropPoint) {
            this.dropPoint = dropPoint;
            if(this.dropPoint != null) {
                this.dropPoint.on(Laya.Event.CLICK, this, function () {
                    if (this._dropPointClicked) {
                        return;
                    }
                    this._dropPointClicked = true;
                    if (SyncSystem.getCachedFish(this.buildingData.getBuildingBaseId()) <= 0) {
                        return;
                    }
                    this.dropPoint.mouseEnabled = false;
                    let fish = this.dropFish;
                    if (SceneMgr.getInstance().getCurrentSceneId() != Constants.CoffeeSceneId) {
                        SyncSystem.collectReward(this.buildingData.getBuildingBaseId());
                        this.dropFish = [];
                        this._dropPointClicked = null;
                        fish.forEach(element => {
                            if (element) {
                                element.destroy();
                            }
                        });
                        return;
                    }
                    SoundManager.getInstance().play("Sound_clickReward");
                    let first = true;
                    fish.forEach(element => {
                        if(element){
                            element.onClick();
                            element.zOrder = 999;
                            let fishTime = 200 + Utils.randomExt(200);
                            let elementPoint = new Laya.Point(element.x, element.y);
                            elementPoint = element.parent.localToGlobal(elementPoint);
                            element.removeSelf();
                            element.x = elementPoint.x;
                            element.y = elementPoint.y;
                            Laya.stage.addChild(element);
                            // TODO  这个坐标需要做一下适配，或者想办法获得到真实的鱼干位置
                            Laya.Tween.clearAll(element);
                            Laya.Tween.to(element, { x: Constants.fishImagePoint.x, y: Constants.fishImagePoint.y }, fishTime,
                                null,
                                Laya.Handler.create(this, function () {
                                    element.destroy();
                                    if (first) {
                                        first=false;
                                        SyncSystem.collectReward(this.buildingData.getBuildingBaseId());
                                        this.dropFish.forEach(element => {
                                            Laya.Tween.clearAll(element);
                                            element.destroy();
                                        });
                                        this.dropFish = [];
                                        this._dropPointClicked = null;
                                    }
                                }), 0);
                        }
                    });
                    this.dropFish = [];
                    //文本播放增加的数量“+xxx”
                    let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
                    let dropPointPoistion = this.getDropFishPointPosition();
                    this.dropPoint.localToGlobal(dropPointPoistion);
                    scene.playFishNumber(dropPointPoistion.x, dropPointPoistion.y, SyncSystem.getCachedFish(this.buildingData.getBuildingBaseId()));
                });
            }
        }

        setDropFish(dropFish) {
            this.dropFish.push(dropFish);
            this._dropPointClicked = null;
            if(this.dropPoint){
                this.dropPoint.mouseEnabled = true;
            }
        }

        getDropFish() {
            return this.dropFish;
        }

        getDropFishPoint() {
            return this.dropPoint;
        }

        getDropFishPointPosition() {
            return new Laya.Point(this.dropPoint.width/2, this.dropPoint.height/2);
        }

        cleanBuildingExtInfoMgr() {
            for(let i = 0; i < this.buildingExtInfoArray.length ; ++i) {
                this.buildingExtInfoArray[i].cleanBuildingExtInfo();
            }
            let dropFish = this.getDropFish();
            if(dropFish){
                dropFish.forEach(element => {
                    element.destroy();
                });
            }
        }
    }

    //存储了钓鱼相关的基本数据，剩余次数、恢复时间、鱼饵使用 等等

    class FishingBaseData {
        constructor() {
            this.lastBaitId = "";
            this.remainTimes = 0;
            this.maxTimes = 0;
            this.nextUpdateTs = 0;
            this.dailyFreeTimes = 0;
            this.fishingTicketNum = 0;
            this.fullMills=0;
            // this.nextUpdateTs = TimeUtils.getTimeNowOnServer() + 1 * 60 * 1000;
        }

        getLastBaitId() {
            return this.lastBaitId;
        }

        setLastBaitId(lastId) {
            if(lastId != null && lastId != "") {
                //lastId 不为空才设置，否则不设置
                this.lastBaitId = lastId;
            }
        }

        setUnlockedBaits(baits) {
            this._unlockedBaits = baits;
        }

        unlockBait(baitId) {
            for(const bait of this._unlockedBaits) {
                if(bait == baitId)
                    return true;
            }
            this._unlockedBaits.push(baitId);
        }

        isBaitsUnlocked(baitId) {
            for(const bait of this._unlockedBaits) {
                if(bait == baitId)
                    return true;
            }
            return false;
        }

        getRemainTimes() {
            return this.remainTimes;
        }

        setRemainTimes(remainTimes) {
            this.remainTimes = remainTimes;
        }

        addRemainTimes(incr) {
            this.remainTimes += incr;
            if(this.remainTimes > this.maxTimes) {
                this.remainTimes == this.maxTimes;
            }
        }

        getMaxTimes() {
            return this.maxTimes;
        }

        setMaxTimes(maxTimes) {
            this.maxTimes = maxTimes;
        }

        getNextUpdateTime() {
            return this.nextUpdateTs;
        }

        setNextUpdateTime(nextTs) {
            this.nextUpdateTs = nextTs;
        }

        getDailyFreeTimes() {
            return this.dailyFreeTimes;
        }

        setDailyFreeTimes(times) {
            this.dailyFreeTimes = times;
        }

        setFishingTicketNum(n) {
            this.fishingTicketNum = n | 0;
        }

        getFishingTicketNum() {
            return this.fishingTicketNum;
        }
        getFullMills(){
            return this.fullMills;
        }
        setFullMills(fullMills){
             this.fullMills=fullMills;
        }
    }

    //钓鱼的图鉴信息

    class FishingCollectData {

        constructor(configId, config) {
            this.configId = configId;
            this.config = config?config:ConfigMgr.getInstance().getItemById("FishingFishes", String(configId));
            this.acquireTimes = 0;
            this.newCome = true;
            this.maxSize = 0;
        }

        getId() {
            return this.configId;
        }

        getAcquireTimes() {
            return this.acquireTimes;
        }

        setAcquireTimes(times) {
            this.acquireTimes = times;
        }

        getNewCome() {
            return this.newCome;
        }

        setNewCome(newCome) {
            this.newCome = newCome;
        }

        getMaxSize() {
            return this.maxSize;
        }

        setMaxSize(maxSize) {
            this.maxSize = maxSize;
        }

        getName() {
            return this.config.name;
        }

        //图片
        getSource() {
            return this.config.img;
        }

        //剪影图片
        getGrayPic() {
            return this.config.imgShadow;
        }

    }

    class FishingDataMgr {

        constructor() {
            this.fishingBaseData = new FishingBaseData();
            this.fishingCollectDatas = new Map();
            this.initCollects();
            this.fishingRetryFreeTimes=Const.get("FishingRetryFreeTimes");
        }

        static getInstance() {
            return FishingDataMgr.instance ? FishingDataMgr.instance : FishingDataMgr.instance = new FishingDataMgr();
        }

        initCollects() {
            let configs = ConfigMgr$1.getInstance().getConfig("FishingFishes");
            let m = this.fishingCollectDatas;
            for(const [fishId, info] of configs) {
                let c = new FishingCollectData(fishId, info);
                m.set(fishId, c);
            }
        }

        getFishingRetryFreeTimes(){
            return this.fishingRetryFreeTimes;
        }

        getBaseData() {
            return this.fishingBaseData;
        }

        getCollectById(collectId) {
            return this.fishingCollectDatas.get(collectId);
        }

        updateCollectData(collectId, acquireTimes, maxSize, newCome) {
            let collect = this.getCollectById(collectId);
            if(!collect){
                return false;
            }
            if(acquireTimes) {
                collect.setAcquireTimes(acquireTimes);
            }
            if(maxSize) {
                collect.setMaxSize(maxSize);
            }
            if(typeof(newCome) != "undefined" && newCome != null) {
                collect.setNewCome(newCome);
            }

            return collect;
        }

        clear() {
            this.fishingBaseData = new FishingBaseData();
            this.fishingCollectDatas.clear();
            this.initCollects();
        }
    }

    class FishingNewBaitMgr {
        constructor() {
            this.loadData();
        }

        static getInstance() {
            return FishingNewBaitMgr.instance?FishingNewBaitMgr.instance:FishingNewBaitMgr.instance = new FishingNewBaitMgr();
        }

        loadData(){
            this.seeArr = new Array();
            let FishingNewBait = Laya.LocalStorage.getJSON("FishingNewBait");
            let config = ConfigMgr$1.getInstance().getConfig("FishingBaits");
            if(FishingNewBait){
                this.FishingNewBait =FishingNewBait;
            }
            else {
                this.FishingNewBait = new Array();
                config.forEach((listConfig) => {
                    let obj = {
                        "Id":listConfig.Id,
                        "Value" : 0,
                        "Level" : listConfig.Level,
                        "Order" : listConfig.Order
                    };
                    this.FishingNewBait.push(obj);
                });
                this.FishingNewBait[0].Value = 2;
            }
        }

        updataData(unlockBatis){
            this.unlockBatis = unlockBatis;
            this.checkBaits();
        }

        checkBaits(){
            for(let i = 0;i < this.unlockBatis.length; i ++){
                for(let j=0 ; j<this.FishingNewBait.length ;j++){
                    if(this.unlockBatis[i] == this.FishingNewBait[j].Id && this.FishingNewBait[j].Value == 0){
                        let config = ConfigMgr$1.getInstance().getItemById("FishingBaits", this.FishingNewBait[j].Id);
                        let playerStar = PlayerSystem.getStar();
                        if(config.StarLimit<=playerStar){
                            this.FishingNewBait[j].Value = 1;
                        }
                    }
                }
            }
            this.saveData();
        }

        saveData(){
            Laya.LocalStorage.setJSON("FishingNewBait",this.FishingNewBait);
        }

        setValueByNO(num,value){
            this.FishingNewBait[num].Value = value;
            this.saveData();
        }

        seeBait(){
            for(let i = 0;i < this.seeArr.length;i++){
                if(this.seeArr[i]>=0){
                    let data = this.getBaitStatusByNO(this.seeArr[i]);
                    if(data.Value == 1){
                        this.setValueByNO(this.seeArr[i],2);
                    }
                }
            }
            this.seeArr.length = 0;
        }

        haveNewBait(){
            for(let i = 0; i< this.FishingNewBait.length; i++){
                if(this.FishingNewBait[i].Value == 1){
                    return true;
                }
            }
            return false;
        }

        haveHigherBait(){
            var highBait = 0;
            for(let i = 0; i< this.FishingNewBait.length; i++){
                if(this.FishingNewBait[i].Value > 0 && this.FishingNewBait[i].Level > this.FishingNewBait[highBait].Level){
                    highBait = i;
                }
            }
            if(this.FishingNewBait[highBait].Value == 1){
                return this.FishingNewBait[highBait].Order;
            }
            return -1;
        }

        getBaitNO(Id){
            for(let i = 0;i<this.FishingNewBait.length;i++){
                if(this.FishingNewBait[i].Id == Id){
                    return i;
                }
            }
            return -1;
        }

        getBaitStatusByNO(num){
            return this.FishingNewBait[num];
        }

        getBaitStatus(Id){
            let num = this.getBaitNO(Id);
            return this.getBaitStatusByNO(num);
        }

        setSeeArr(Id){
            let num = this.getBaitNO(Id);
            this.seeArr.push(num);
        }
    }

    class FishingSystem {
        static init() {
        }

        static clear() {
            FishingDataMgr.getInstance().clear();
        }

        static updateBaseData(gameInfo) {
            let baseData = FishingDataMgr.getInstance().getBaseData();
            baseData.setLastBaitId(gameInfo.lastBaitsId);
            baseData.setRemainTimes(gameInfo.remainTimes);
            baseData.setMaxTimes(gameInfo.maxTimes);
            baseData.setNextUpdateTime(gameInfo.nextUpdateTs);
            baseData.setUnlockedBaits(gameInfo.unlockBatis);
            baseData.setDailyFreeTimes(gameInfo.dailyFreeTimes);
            baseData.setFishingTicketNum(gameInfo.fishingTicketNum);
            baseData.setFullMills(gameInfo.fullMills);
            FishingNewBaitMgr.getInstance().updataData(gameInfo.unlockBatis);
            EventMgr.getInstance().postEvent("UPDATE_FISHING_TIP");
        }

        static updateCollectData(collectInfos) {
            Object.getOwnPropertyNames(collectInfos).forEach(key=>{

                let info = collectInfos[key];
                let collectId = info.fishId;
                let acquireTimes = info.acquireTimes;
                let maxSize = info.maxSize;
                let newCome = info.newCome;
                FishingDataMgr.getInstance().updateCollectData(collectId, acquireTimes, maxSize, newCome);
            });
        }

        static getFishingRetryFreeTimes(){
            return FishingDataMgr.getInstance().getFishingRetryFreeTimes();
        }

        static getMaxTimes() {
            return FishingDataMgr.getInstance().getBaseData().getMaxTimes();
        }

        static setMaxTimes(maxTimes) {
            FishingDataMgr.getInstance().getBaseData().setMaxTimes(maxTimes);
        }

        static getRemainTimes() {
            return FishingDataMgr.getInstance().getBaseData().getRemainTimes();
        }

        static addRemainTimes(incr) {
            FishingDataMgr.getInstance().getBaseData().addRemainTimes(incr);
            EventMgr.getInstance().postEvent("UPDATE_FISHING_TIP");
        }

        static getNextUpdateTs() {
            return FishingDataMgr.getInstance().getBaseData().getNextUpdateTime();
        }

        static setNextUpdateTs(nextTs) {
            FishingDataMgr.getInstance().getBaseData().setNextUpdateTime(nextTs);
        }

        static getLastBaitId() {
            return FishingDataMgr.getInstance().getBaseData().getLastBaitId();
        }

        static setLastBaitId(baitId) {
            FishingDataMgr.getInstance().getBaseData().setLastBaitId(baitId);
        }

        static isBaitUnlocked(baitId) {
            let baseData = FishingDataMgr.getInstance().getBaseData();
            return baseData.isBaitsUnlocked(baitId);
        }

        static unlockBait(baitId) {
            let baseData = FishingDataMgr.getInstance().getBaseData();
            baseData.unlockBait(baitId);
        }

        static getDailyFreeTimes() {
            return FishingDataMgr.getInstance().getBaseData().getDailyFreeTimes();
        }

        static getFishingTicketNum() {
            return FishingDataMgr.getInstance().getBaseData().getFishingTicketNum();
        }

        static setFishingTicketNum(fishingTicketNum){
            let baseData = FishingDataMgr.getInstance().getBaseData();
            baseData.setFishingTicketNum(fishingTicketNum);
        }

        //获取下次刷新时间，并且刷新最新次数
        static getAndUpdateTimes() {
            //-1表示当前已满，不需要更新
            let nextTs = this.getNextUpdateTs();
            if(nextTs == -1){
                return -1;
            }

            let currentTs = TimeUtils.getTimeNowOnServer();
            if(currentTs >= nextTs) {
                //这里如果已经到了刷新时间，则+1次，并且由客户端计算一下下次的回复时间
                this.addRemainTimes(1);
                let updateTs = ConfigMgr$1.getInstance().getItemById("Const", "FishingTimeIncrease").Value * 1000;
                if(this.getRemainTimes() >= this.getMaxTimes()) {
                    this.setNextUpdateTs(-1);
                } else {
                    this.setNextUpdateTs(this.getNextUpdateTs() + updateTs);
                }
            }

            return this.getNextUpdateTs();
        }

        //获取下次刷新距离现在的时间差，单位秒
        static getNextUpdateSecs() {
            let nextTs = this.getAndUpdateTimes();
            if(nextTs <= 0) return -1;

            return Math.ceil((nextTs - TimeUtils.getTimeNowOnServer()) / 1000);
        }

        static getCollectById(collectId) {
            return FishingDataMgr.getInstance().getCollectById(collectId);
        }

        static updateServerCollectsInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_FISHING_COLLECTS", [handler]);
        }

        static updateServerBaseInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_FISHING_BASE_INFO", [handler]);
        }

        static finishGameToServer(startType, fishingFishId, fishingBaitId, rewards, fishingFishSize, fetchType, handler, errorHandler) {
            EventMgr.getInstance().postEvent("SERVICE_FINISH_FISHING_GAME", [startType, fishingFishId, fishingBaitId, rewards, fishingFishSize, fetchType, handler, errorHandler]);
            SyncSystem.startOperatingUpdate();
        }
        static getFullMills(){
            return Number(FishingDataMgr.getInstance().getBaseData().getFullMills());
        }
    }

    class Building$1 extends Laya.Tree {

        constructor() {
            super();
            this.buildingData = null;
            this.anim = null;
            this.prefab = null;
            this.buildingExtInfoMgr = null;
            this.mouseThrough = true;
            this.loaded = false;
            this._buildingScale = 1;
            this._completeHandlers = new Array();
        }

        isLoaded() {
            return this.loaded;
        }

        getId() {
            return this.buildingData.getId();
        }

        getBuildingBaseId() {
            return this.buildingData.getBuildingBaseId();
        }

        //设置信息
        setData(data) {
            this.buildingData = data;
            this.buildingExtInfoMgr = new BuildingExtInfoMgr(this.buildingData);
        }

        //获得信息
        getData() {
            return this.buildingData;
        }

        set buildingScale(scale) {
            this._buildingScale = scale;
        }

        get buildingScale() {
            return this._buildingScale
        }

        onDisable() {

        }


        cleanBuilding() {
            this.getBuildingExtInfoMgr().cleanBuildingExtInfoMgr();
        }

        getBuildingExtInfoMgr() {
            return this.buildingExtInfoMgr;
        }

        judgeTouch() {
            let btn = this.prefab.getChildByName("touch");
            if(btn != null) {
                btn.on(Laya.Event.CLICK, this,this.onClick);
            }
        }

        judgeAnim() {
            for(let i = 0; i < 5; ++i) {
                let spineAnimText = this.prefab.getChildByName("spineAnim_"+String(i));
                if(spineAnimText != null &&
                    spineAnimText.text != null &&
                     spineAnimText.text != "") {
                        let animInfo = spineAnimText.text.split(":");
                        let animPath = animInfo[0];
                        let animMove = animInfo[1];
                        let animSkin = animInfo[2];
                        let anim = new Spine();
                        anim.x = spineAnimText.x;
                        anim.y = spineAnimText.y;
                        anim.zOrder = spineAnimText.zOrder;
                        anim.playAnim(animMove, true);
                        this.prefab.addChild(anim);
                        anim.name = "_spineAnim_"+String(i);
                        anim.loadAnim(animPath, animSkin, 0);
                        this.youxidatingAnim(anim,animMove);
                        if(this.prefab.name=='hall_3'||this.prefab.name=='hall_2'||this.prefab.name=='hall_1'){
                            this.animhall=anim;
                            EventMgr.getInstance().registEvent("PLAY_DONGHUA3_YOUKE",this,this.youxiqipao);
                        }

                } else {
                    break;
                }
            }
        }

        judgeDropPoint() {
            let dropPoint = this.prefab.getChildByName("dropPoint");
            if(dropPoint) {
                this.buildingExtInfoMgr.setDropFishPoint(dropPoint, this.buildingScale);
            }
        }

        judgeBuildingRoot() {
            for(let i = 0; i < 5; ++i) {
                let info = new BuildingExtInfo();
                let buildingRoot = this.prefab.getChildByName("buildingRoot_"+String(i));

                let move = this.prefab.getChildByName("move_"+String(i));
                let flip = 1;
                let f1 = this.prefab.getChildByName("flip_"+String(i));
                if(f1 != null) {
                    flip = -1;
                }
                let str = "p" + (i + 1);
                let pointSeting = this.buildingData.getPointSetting()[str];
                // console.log(pointSeting);
                let actionArr = null;
                if (pointSeting) {
                    actionArr = JSON.parse(ConfigMgr$1.getInstance().getItemById("ActionPoint", pointSeting).ActionsSetting);
                }
                if(buildingRoot != null) {
                    buildingRoot.x = buildingRoot.x + buildingRoot.width/2* (1-1/this.buildingScale);
                    buildingRoot.y = buildingRoot.y + buildingRoot.height/2* (1-1/this.buildingScale);
                    buildingRoot.scaleX = 1/this.buildingScale;
                    buildingRoot.scaleY = 1/this.buildingScale;
                    info.setBuildingRoot(buildingRoot);
                    info.setActionSetting(actionArr);
                    this.buildingExtInfoMgr.pushBuildingExtInfo(info);
                } else {
                    break;
                }
            }
        }


        //加载资源
        loadResource(path, completeHandler) {
            if(path.indexOf(".prefab") != -1) {
                if(completeHandler)
                    this._completeHandlers.push(completeHandler);
                this._path = path;
                Laya.loader.create(path, Laya.Handler.create(this, this._onLoaded));
            }
        }

        _onLoaded(prefabDef) {
            console.debug("building loaded", this._path, prefabDef);
            this.prefab = prefabDef.create();
            this.prefab.mouseThrough = true;
            this.width = this.prefab.width;
            this.height = this.prefab.height;
            this.pivotX = this.prefab.width/2*this.buildingScale;
            this.pivotY = this.prefab.height/2*this.buildingScale;
            this.prefab.scaleX = this.buildingScale;
            this.prefab.scaleY = this.buildingScale;
            this.buildingImageScale();
            this.addChild(this.prefab);
            //掉落点
            this.judgeDropPoint();
            //判断是否有触摸区域
            this.judgeTouch();
            //判断动画
            this.judgeAnim();
            //判断buildingRoot
            this.judgeBuildingRoot();
            this.loaded = true;

            for(const handler of this._completeHandlers) {
                handler();
            }
            this._completeHandlers.length = 0;
        }

        //点击建筑物
        onClick() {
            if(GameContext.Sliding)
                return;
            let afterHandler = Laya.Handler.create(this, function (res) {
                SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
            });
            FishingSystem.updateServerBaseInfo(afterHandler);
        }

        buildingImageScale(){
            let scale = 1 / this.buildingData.getMagnification();
            let buildingImageScale= this.prefab.getComponent(Laya.Script);
            if(buildingImageScale){
                buildingImageScale.setScale(scale);
                buildingImageScale.imageScale();
            }
        }
        //游戏大厅动画
        youxidatingAnim(anim,animMove){
            if(this.prefab.name=='hall_3'||this.prefab.name=='hall_2'||this.prefab.name=='hall_1'){
                let anim2Time=Number(Const.get('QuayHallActionTime'))*1000;
                Laya.timer.loop(anim2Time,this,function(){
                    if(this._donghua3==true){
                      //  anim.play('3',false);  
                       // this._donghua3=false;
                    }else {
                        anim.play('2',false);
                    }
                   anim.once(Laya.Event.STOPPED, this, function(){
                        anim.playAnim('1',true);
                    });
                });     
                Laya.timer.loop(15000,this,function(){
                     // this._donghua3=true;
            });
                this.youxidatingBaoxiang();
                EventMgr.getInstance().postEvent("SHOW_ZHULI_SHOW_FIRST");
                EventMgr.getInstance().postEvent("SHOW_BEIKE");
        }
        }
        youxiqipao(){
            if(this.prefab.name=='hall_3'||this.prefab.name=='hall_2'||this.prefab.name=='hall_1'){
                this.animhall.play('3',false);
                this.animhall.once(Laya.Event.STOPPED, this, function(){
                    this.animhall.playAnim('1',true);
                });
            }
           
        }
        //游戏大厅增加宝箱
        youxidatingBaoxiang(){
            Laya.loader.load("prefab/ui/zhuliBaoxiang.json", Laya.Handler.create(this, this.onPrefabBaoxiangLoaded), null, Laya.Loader.JSON);    
        }
        onPrefabBaoxiangLoaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
            let zhuliBaoxiang = Laya.Pool.getItemByCreateFun("zhuliBaoxiang", prefab.create, prefab);
            if(this.prefab.name=='hall_3'){
                zhuliBaoxiang.x=480;
                zhuliBaoxiang.y=120;
            }
            else if(this.prefab.name=='hall_1'){
                zhuliBaoxiang.x=270;
                zhuliBaoxiang.y=-280;
            }
            else if(this.prefab.name=='hall_2'){
                zhuliBaoxiang.x=385;
                zhuliBaoxiang.y=-120;
            }
            this.prefab.addChild(zhuliBaoxiang);
        }
    }

    class BaseGuide {

        constructor(guideKey) {
            this._guideKey = guideKey;
            this._objMap = new Map();
            this._clickMap = new Map();
            this.steps = [];
            this.showBtns = [];
            this.showBtnParents = [];
            this.showBtnPoints = [];
        }

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
        }

        start() {//开始引导
            if (this._start) {
                return;
            };
            this._start = true;
            EventMgr.getInstance().postEvent("GUIDE_SET_GUIDING", [true]);
            let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", this._guideKey);
            guideGroup.Group.forEach(element => {
                this.steps.push(element);
            });
            this.waitingRegisterKey = null;
            this.btnNum = 0;
            this._begin();
            EventMgr.getInstance().registEvent("GOUDE_COLSE_TALK", this, this.closetalk);    
        }

        _begin() {
            //继承该方法，引导的内容都直接添加在Laya.stage上，可以单独建立一个guide节点来添加，方便清理,在Constants中设置一个通用的GuideZOrder,所有的引导都在这个ZOrder中
            this.guideBox = new Laya.Box();
            this.guideBox.width = Laya.stage.width;
            this.guideBox.height = Laya.stage.height;
            this.guideBox.zOrder = Constants.GuideZOrder;
            this.guideBox.mouseEnabled = true;
            Laya.stage.addChild(this.guideBox);
            // this.baseSprite = new Laya.Sprite();
            // this.baseSprite.width = Laya.stage.width;
            // this.baseSprite.height = Laya.stage.height;
            // this.baseSprite.mouseEnabled = true;
            // this.baseSprite.zOrder = 0;
            // this.guideBox.addChild(this.baseSprite);
            this.btnSprite = new Laya.Sprite();
            this.btnSprite.width = Laya.stage.width;
            this.btnSprite.height = Laya.stage.height;
            this.btnSprite.zOrder = 1;
            this.btnSprite.loadImage("guide/mask0.png");
            this.guideBox.addChild(this.btnSprite);
            this.talkSprite = new Laya.Sprite();
            this.talkSprite.width = Laya.stage.width;
            this.talkSprite.height = Laya.stage.height;
            this.guideBox.addChild(this.talkSprite);
            this.talkSprite.zOrder = 2;
            this.talkSprite.on(Laya.Event.CLICK, this, this.nextStep);
            this.nextStep();
        }

        exit() {//中途退出引导
            EventMgr.getInstance().postEvent("GUIDE_SET_GUIDING", [false]);
            this.destroy();
        }

        complete() {
            EventMgr.getInstance().postEvent("GUIDE_COMPLETE_GUIDE", [this._guideKey]);
            EventMgr.getInstance().postEvent("GUIDE_SET_GUIDING", [false]);
            EventMgr.getInstance().removeEvent("GUIDE_NEXT_STEP", this, this.eventNextStep);
            Laya.timer.clear(this, this.showTalk);
            this.destroy();
        }

        onDestroy() {
            Laya.timer.clearAll(this);
        }

        destroy() {
            this._objMap.clear();
            this._clickMap.clear();
            //继承该方法，清理所有该引导的残留,以及注册的事件等
            if (this.talkSprite) {
                this.talkSprite.offAll();
            }
            if (this.clickImage) {
                this.clickImage.offAll();
            }
            this.currentGuide = null;
            this.steps = null;
            this.showBtns = null;
            this.showBtnParents = null;
            this.showBtnPoints = null;
            this.talkStr = null;
            this.currentCharNum = 0;
            if (this.btnAni) {
                this.btnAni.removeSelf();
                this.btnAni.destroy();
            }
            if (this.talkAni) {
                this.talkAni.removeSelf();
                this.talkAni.destroy();
            }
            this.guideBox.removeSelf();
            this.guideBox.destroy();
        }

        registerObj(key, obj) {
            this._objMap.set(key, obj);
        }

        getObj(key) {
            return this._objMap.get(key);
        }

        registerClick(key, btn, rect, showBtns, func) {
            this._clickMap.set(key, [btn, rect, func, showBtns]);
            if (this.waitingRegisterKey != null) {
                this.createClickArea(this.waitingRegisterKey);
            }
        }

        createClickArea(key) {
            let info = this._clickMap.get(key);
            if (!info) {
                this.waitingRegisterKey = key;
                return;
            }
            this.waitingRegisterKey = null;
            let btn = info[0];
            let rect = info[1];
            let func = info[2];
            let showBtns = info[3];
            //TODO 在引导层上根据rect或者btn来创建一个点击区域
            if (!btn || !btn.parent) {
                this.waitingRegisterKey = key;
                return;
            }
            this.creatMaskImages(btn, rect);
            this.upBtn_zOrder(btn, showBtns);
            this.clickImage.offAll();
            this.clickImage.on(Laya.Event.MOUSE_DOWN, this, function (event) {
                event.currentTarget = btn;
                btn.event(Laya.Event.MOUSE_DOWN, event);
            });
            this.clickImage.on(Laya.Event.MOUSE_UP, this, function (event) {
                event.currentTarget = btn;
                btn.event(Laya.Event.MOUSE_UP, event);
            });
            this.clickImage.on(Laya.Event.MOUSE_OUT, this, function (event) {
                event.currentTarget = btn;
                btn.event(Laya.Event.MOUSE_OUT, event);
            });
            this.clickImage.on(Laya.Event.CLICK, this, function (event) {
                this.nextStep();
                event.currentTarget = btn;
                btn.event(Laya.Event.CLICK, event);
                if (func) {
                    func().bind(this.getObj(key));
                }

            });
        }

        upBtn_zOrder(btn, showBtns) {
            if (showBtns && showBtns.length > 0) {
                this.showBtns = showBtns;
            } else {
                this.showBtns = [btn];
            }
            this.showBtnParents = [];
            this.showBtnPoints = [];
            this.showBtns.forEach(element => {
                this.showBtnParents.push(element.parent);
                let btnPoint = new Laya.Point(element.x, element.y);
                this.showBtnPoints.push(btnPoint);
                let btnNewPoint = element.parent.localToGlobal(btnPoint, true);
                Laya.Tween.clearAll(element);
                element.alpha = 1;
                element.x = btnNewPoint.x;
                element.y = btnNewPoint.y;
                this.btnShowSprite.addChild(element);
            });
        }

        backBtn_zOrder() {
            if(this.showBtns){
                for (let i = 0; i < this.showBtns.length; i++) {
                    let element = this.showBtns[i];
                    let point = this.showBtnPoints[i];
                    let parent = this.showBtnParents[i];
                    this.btnShowSprite.removeChild(element);
                    element.x = point.x;
                    element.y = point.y;
                    if (element != null && !element.destroyed) {
                        parent.addChild(element);
                    }
                }
            }
            this.showBtns = [];
            this.showBtnParents = [];
            this.showBtnPoints = [];
        }

        doClick(key) {
            let info = this._clickMap.get(key);
            let btn = info[0];
            btn.event(Laya.Event.CLICK);
        }

        //展示文本，点击提醒等通用功能，都可以在BaseGuide里写，其他的引导就继承BaseGuide，实现流程即可

        creatMaskImages(btn, rect) {
            this.btnSprite.visible = true;
            if (!this.btnShowSprite) {
                this.btnShowSprite = new Laya.Sprite();
                this.btnShowSprite.width = this.btnSprite.width;
                this.btnShowSprite.height = this.btnSprite.height;
                this.btnSprite.addChild(this.btnShowSprite);
                this.btnShowSprite.zOrder = 1;
            }
            let btnPoint = new Laya.Point(btn.x - (btn.pivotX ? btn.pivotX * btn.scaleX : (btn.anchorX ? btn.anchorX * btn.width * btn.scaleX : 0)),
                btn.y - (btn.pivotY ? btn.pivotY * btn.scaleY : (btn.anchorY ? btn.anchorY * btn.height * btn.scaleY : 0)));
            let btnRightDownPoint = new Laya.Point(btnPoint.x + btn.width * btn.scaleX, btnPoint.y + btn.height * btn.scaleY);
            if (rect != null) {
                btnPoint.x = rect.x;
                btnPoint.y = rect.y;
                btnRightDownPoint.x = rect.x + rect.width;
                btnRightDownPoint.y = rect.y + rect.height;
            }
            btnPoint = btn.parent.localToGlobal(btnPoint);
            btnRightDownPoint = btn.parent.localToGlobal(btnRightDownPoint);
            btnPoint.x -= 4;
            btnPoint.y -= 4;
            btnRightDownPoint.x += 4;
            btnRightDownPoint.y += 4;
            let btnWidth = btnRightDownPoint.x - btnPoint.x;
            let btnHeight = btnRightDownPoint.y - btnPoint.y;
            if (!this.clickImage) {
                this.clickImage = new Laya.Image("guide/mask0.png");
                this.clickImage.alpha = 0.01;
                this.clickImage.mouseEnabled = true;
            }
            this.clickImage.width = btnWidth;
            this.clickImage.height = btnHeight;
            this.clickImage.x = btnPoint.x;
            this.clickImage.y = btnPoint.y;
            this.clickImage.zOrder = 99;
            this.btnSprite.addChild(this.clickImage);
            if (!this.btnAniBox) {
                this.btnAniBox = new Laya.Box();
                this.btnAni = new Spine();
                this.btnAni.loadAnim("anim1/xinshouyindao.sk");
                this.btnAni.playAnim("zhuazi", true);
                this.btnSprite.addChild(this.btnAniBox);
                this.btnAniBox.addChild(this.btnAni);
                this.btnAniBox.zOrder = 10;
            }
            if (this.currentGuide.Type == Constants.GuideTypeHoleTalk) {
                this.btnAniBox.visible = false;
            } else {
                this.btnAniBox.visible = true;
            }
            this.btnAniBox.x = (btnPoint.x + btnRightDownPoint.x) / 2;
            this.btnAniBox.y = (btnPoint.y + btnRightDownPoint.y) / 2;
        }

        creatTalk() {
            this.talkSprite.visible = true;
            this.talkSprite.mouseEnabled = true;
            if (!this.talkBg) {
                this.talkBg = new Laya.Sprite();
                this.talkBg.loadImage("guide/mask0.png");
                this.talkBg.width = Laya.stage.width;
                this.talkBg.height = Laya.stage.height;
                this.talkSprite.addChild(this.talkBg);
                this.talkBg.zOrder = 0;
                this.talkBg.visible = true;
            }
            if (!this.talkAniBox) {
                this.talkAniBox = new Laya.Box();
                this.talkAni = new Laya.Skeleton();
                this.talkAniBox.addChild(this.talkAni);
                this.talkAni.load("anim1/xinshouyindao.sk", Laya.Handler.create(this, function () {
                    this.talkAni.play("maomao1", true);
                }));
            }
            if (!this.talkBox) {
                Laya.loader.create("prefab/ui/TalkBox.prefab", Laya.Handler.create(this, function (prefabDef) {
                    this.talkBox = prefabDef.create();
                    this.talkBox.x = (Laya.stage.width - this.talkBox.width) / 2;
                    this.talkBox.y = (Laya.stage.height - 310);
                    this.talkSprite.addChild(this.talkBox);
                    this.talkAniBox.x = 150;
                    this.talkAniBox.y = 120;
                    this.talkBox.addChild(this.talkAniBox);
                    this.talkLab = this.talkBox.getChildByName("talkLab");
                    this.completeImage = this.talkBox.getChildByName("completeImage");
                    this.currentCharNum = 0;
                    Laya.timer.loop(1, this, this.showTalk);
                    if (this.currentGuide.Hight != null) {
                        this.talkBox.y = this.currentGuide.Hight;
                    } else {
                        this.talkBox.y = (Laya.stage.height - 310);
                    }
                }));
            } else {
                this.talkBox.visible = true;
                this.currentCharNum = 0;
                Laya.timer.loop(1, this, this.showTalk);
                if (this.currentGuide.Hight != null) {
                    this.talkBox.y = this.currentGuide.Hight;
                } else {
                    this.talkBox.y = (Laya.stage.height - 310);
                }
            }
        }

        //一个字一个字蹦
        showTalk(showAll = false) {
            if (!this.talkBox) {
                return;
            }
            if (showAll) {
                this.clicktalk=0;
                this.talkLab.text = this.talkStr;
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                Laya.timer.frameOnce(1,this,function(){
                    this.clicktalk=1;
                });
                this.completeImageTween();
                if (this.currentGuide.Type == Constants.GuideTypeTalkButton || this.currentGuide.Type == Constants.GuideTypeSpace) {
                    this.talkSprite.mouseEnabled = false;
                }
                return;
            }
            if (this.talkStr == null) {
                return;
            }
            if(this.currentCharNum%2==0){
                ++this.currentCharNum;
            }
            this.talkLab.text = this.talkStr.substr(0, ++this.currentCharNum);
            if (this.currentCharNum >= this.talkStr.length) {
                this.completeImageTween();
                Laya.timer.clear(this, this.showTalk);
                this.clicktalk=0;
                this.talkStr = null;
                Laya.timer.frameOnce(1,this,function(){
                    this.clicktalk=1;
                });
                if (this.currentGuide.Type == Constants.GuideTypeTalkButton || this.currentGuide.Type == Constants.GuideTypeSpace) {
                    this.talkSprite.mouseEnabled = false;
                }
            }
        }

        completeImageTween() {
            if (!this.completeImage) {
                return;
            }
            this.stopCompleteImageTween();
            if (this.completeImage.alpha == 1) {
                Laya.Tween.to(this.completeImage, { alpha: 0 }, 500, null, Laya.Handler.create(this, this.completeImageTween));
            } else {
                Laya.Tween.to(this.completeImage, { alpha: 1 }, 500, null, Laya.Handler.create(this, this.completeImageTween));
            }
        }

        stopCompleteImageTween() {
            if (!this.completeImage) {
                return;
            }
            Laya.Tween.clearAll(this.completeImage);
        }

        creatAnmi(anmi) {
            EventMgr.getInstance().registEvent("GUIDE_NEXT_STEP", this, this.eventNextStep); 
        }
        closetalk(){
            this.guideBox.on(Laya.Event.CLICK,this,function(){
         
            if(this.clicktalk==1){
                this.talkSprite.visible=false;
                this.guideBox.offAll();
            }
            
            });
        }

        nextStep() {
            if (this.talkStr && (this.currentCharNum < this.talkStr.length)) {
                this.showTalk(true);
                return;
            }
            this.backBtn_zOrder();
            this.sdkSendEvent();
            //是否有下一步?
            if (this.steps == null || this.steps.length == 0) {
                this.complete();
                return;
            }
            this.currentGuide = ConfigMgr$1.getInstance().getItemById("Guide", this.steps[0]);
            this.steps.splice(0, 1);
            this.reomoveCurrentStepEvent();
            if (this.btnSprite) {
                this.btnSprite.visible = false;
            }
            if (this.talkSprite) {
                this.talkSprite.visible = false;
            }
            switch (this.currentGuide.Type) {
                //对话
                case Constants.GuideTypeTalk: {
                    this.talkStr = Strings$1.get(this.currentGuide.Content);
                    this.creatTalk();
                    this.talkBg.visible = true;
                    break;
                }
                //按钮
                case Constants.GuideTypeButton: {
                    this.createClickArea("btn" + this.btnNum);
                    this.btnNum++;
                    break;
                }
                //对话同时点按钮
                case Constants.GuideTypeTalkButton: {
                    this.createClickArea("btn" + this.btnNum);
                    this.talkStr = Strings$1.get(this.currentGuide.Content);
                    this.creatTalk();
                    this.btnNum++;
                    this.talkBg.visible = false;
                    break;
                }
                //播放动画或空白
                case Constants.GuideTypeSpace: {
                    this.creatAnmi();
                    if (this.currentGuide.Content != null) {
                        this.talkStr = Strings$1.get(this.currentGuide.Content);
                        if (this.talkStr) {
                            this.creatTalk();
                        }
                        if (this.talkBg) {
                            this.talkBg.visible = false;
                        }
                    }
                    break;
                }
                //高亮对话
                case Constants.GuideTypeHoleTalk: {
                    this.createClickArea("btn" + this.btnNum);
                    this.talkStr = Strings$1.get(this.currentGuide.Content);
                    this.creatTalk();
                    this.talkSprite.zOrder = 2;
                    this.talkBg.visible = false;
                    this.btnNum++;
                    break;
                }

            }
        }

        reomoveCurrentStepEvent() {
            EventMgr.getInstance().removeEvent("GUIDE_NEXT_STEP", this, this.eventNextStep);
            Laya.timer.clear(this, this.showTalk);
            this.stopCompleteImageTween();
            if (this.clickImage) {
                this.clickImage.offAll();
            }
        }

        eventNextStep(guideGroupId) {
            if (guideGroupId == this._guideKey) {
                this.nextStep();
                EventMgr.getInstance().removeEvent("GUIDE_NEXT_STEP", this, this.eventNextStep);
            }
        }

        sdkSendEvent() {

        }

        clear() {
            if(this.showBtns){
                this.backBtn_zOrder();
            }
            Laya.timer.clearAll(this);
            this.destroy();
            this._start = false;
            this.steps = [];
            this.showBtns = [];
            this.showBtnParents = [];
            this.showBtnPoints = [];
        }

        // restart() {
        //     this.backBtn_zOrder();
        //     Laya.timer.clearAll(this);
        //     this._start = false;

        //     this.steps = [];
        //     this.showBtns = [];
        //     this.showBtnParents = [];
        //     this.showBtnPoints = [];
        //     if (this.btnAniBox) {
        //         this.btnAniBox.offAll();
        //         this.btnAniBox.destroy();
        //         this.btnAniBox = null;
        //     }
        //     if (this.talkSprite) {
        //         this.talkSprite.offAll();
        //         this.talkSprite.destroy();
        //         this.talkSprite = null;
        //     }
        //     if (this.btnShowSprite) {
        //         this.btnShowSprite.offAll();
        //         this.btnShowSprite.destroy();
        //         this.btnShowSprite = null;
        //     }
        //     if (this.btnSprite) {
        //         this.btnSprite.offAll();
        //         this.btnSprite.destroy();
        //         this.btnSprite = null;
        //     }
        //     if (this.clickImage) {
        //         this.clickImage.offAll();
        //         this.clickImage.destroy();
        //         this.clickImage = null;
        //     }
        //     this.currentGuide = null;
        //     this.talkStr = null;
        //     this.currentCharNum = 0;
        //     if (this.btnAni) {
        //         this.btnAni.removeSelf();
        //         this.btnAni.destroy();
        //         this.btnAni = null;
        //     }
        //     if (this.talkAni) {
        //         this.talkAni.removeSelf();
        //         this.talkAni.destroy();
        //         this.talkAni = null;
        //     }
        //     if (this.talkAniBox) {
        //         this.talkAniBox.removeSelf();
        //         this.talkAniBox.destroy();
        //         this.talkAniBox = null;
        //     }
        //     if (this.talkBox) {
        //         this.talkBox.removeSelf();
        //         this.talkBox.destroy();
        //         this.talkBox = null;
        //     }
        //     this.guideBox.removeSelf();
        //     this.guideBox.destroy();
        //     this.start();
        // }

    }

    class Constants$1 {
    }

    Constants$1.RankUnlockStr = "解锁钓鱼功能可赠送口粮（<star>>200)";

    Constants$1.GiftMaxStr = "赠礼和奖励用完啦~再邀请%1%个好友就可以增加数量了！快去邀请吧。";
    Constants$1.GiftMaxLimitStr = "今日赠品和奖励用完啦~明天再来吧！";

    Constants$1.GiftSendStr = "收到礼物裙带菜Ⅰ×1";
    Constants$1.GiftSendSelfStr = "";
    Constants$1.GiftUnlock = null;
    Constants$1.TimeModify = 0;
    Constants$1.RankGiftsSendLimit = {};
    Constants$1.TouchEnabled = true;

    class Guide1 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_1") {
                //埋点 第一次 加载游戏、
                if(!Laya.LocalStorage.getJSON('diyici')){
                    let str='ftue_05_gameload';
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                    Laya.LocalStorage.setJSON('diyici','1');
                }
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.Star != null && PlayerSystem.getStar() >= Number(guideCondition.Star)) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }

        start(){
            if (!this._start) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["pre_guide"]);
            }
            super.start();
        }

        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_2");
        }
        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                 let s1 = "welcome_" + this.currentGuide.Id.substring(13);
                 EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
                let guidid=Number(this.currentGuide.Id.substring(13));
                if(guidid==1){
                    let str = "ftue_10_firstdialog";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }    
            }
        }

    }

    class Guide2 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_2") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.GuideGroup != null) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }

        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_3");
        }
        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(13)=='1'){
                let s1 = "begin_buy_chair_" + this.currentGuide.Id.substring(13);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
            }
                let guidid=this.currentGuide.Id;
                if(guidid=='Guide1_Build_3'){
                    let str = "ftue_15_shopEnter";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }else if(guidid=='Guide1_Build_6'){
                    let str = "ftue_20_chair1Build";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }
            }
        }

    }

    class Guide3 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_3") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        start() {//开始引导
            if (!this._start && CatSystem.getCatsBySceneId(Constants.CoffeeSceneId).length <= 1) {
                EventMgr.getInstance().postEvent("GUIDE_ATTRACT_COFFEE_CAT", ["special"]);
            }
            super.start();
        }

        nextStep() {
            if (this.currentGuide && this.currentGuide.Id == "Guide1_Food_6") {
                EventMgr.getInstance().postEvent("GUIDE_COMPLETE_GUIDE", [this._guideKey]);
            }
            super.nextStep();
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_4");
        }

        upBtn_zOrder(btn, showBtns) {
            if (this.currentGuide && this.currentGuide.Id == "Guide1_Food_4") {
                this.replaceImage = new Laya.Image("shopui/wupin1.png");
                this.replaceImage.scaleX = this.replaceImage.scaleY = 2;
                let btnPoint = new Laya.Point(btn.x, btn.y);
                let btnNewPoint = btn.parent.localToGlobal(btnPoint);
                this.replaceImage.x = btnNewPoint.x - 170;
                this.replaceImage.y = btnNewPoint.y - 194;
                this.btnShowSprite.addChild(this.replaceImage);
                btn.visible = false;
                this.showBtns.push(btn);
                this.btnAniBox.y+=150;
            } else {
                super.upBtn_zOrder(btn, showBtns);
            }
        }

        backBtn_zOrder() {
            if (this.currentGuide && this.currentGuide.Id == "Guide1_Food_4") {
                this.btnShowSprite.removeChild(this.replaceImage);
                if(this.showBtns && this.showBtns[0])
                    this.showBtns[0].visible = true;
                this.showBtns = [];
                this.showBtnParents = [];
                this.showBtnPoints = [];
                this.replaceImage = null;
            } else {
                super.backBtn_zOrder();
            }
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                switch (this.currentGuide.Id) {
                    case "Guide1_Food_1": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_buy_menu_1"]);
                        break;
                    }
                    case "Guide1_Food_6": {
                      
                        let str = "ftue_25_doughnutResearch";
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                        break;
                    }
                }
            }
        }
    }

    class Guide4 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_4") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        start() {
            if (CatSystem.getCatsBySceneId(Constants.CoffeeSceneId).length <= 0) {
                EventMgr.getInstance().postEvent("GUIDE_ATTRACT_COFFEE_CAT", ["special"]);
            }
            super.start();
        }

        creatMaskImages(btn, rect) {
            super.creatMaskImages(btn, rect);
            if (this.currentGuide && this.currentGuide.Id == "Guide1_Order_1") {
                this.btnAniBox.y += 100;
            }
        }

        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_5");
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(13)=='1'){
                    let s1 = "begin_eat_" + this.currentGuide.Id.substring(13);
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
                }
                let guidid=this.currentGuide.Id;
                if(guidid=='Guide1_Order_3'){
                    let str = "ftue_30_fishCollect";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }
            }
        }
    }

    class Guide5 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_5") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    // guideCondition.BuildingBsae = 107;
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_6");
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(15)=='1'){
                    let s1 = "begin_buy_toy_" + this.currentGuide.Id.substring(15);
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
                }
                let guidid=this.currentGuide.Id;
                if(guidid=='Guide1_Build_2_4'){
                    let str = "ftue_35_catScratchBuild";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }
            }
        }
    }

    class Guide6 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_6") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        start() {
            EventMgr.getInstance().registEvent("GUIDE_GUIDE6_INVITE", this, this.setInviteComplete);
            super.start();
        }
        nextStep() {
            if (this.currentGuide && !this.inviteComplete) {
                return;
            }
            super.nextStep();
            if (!this.btnAniBox) {
                this.btnAniBox = new Laya.Box();
                this.btnAni = new Spine();
                this.btnAni.loadAnim("anim1/xinshouyindao.sk");
                this.btnAni.playAnim("zhuazi", true);
                this.btnSprite.addChild(this.btnAniBox);
                this.btnAniBox.addChild(this.btnAni);
                this.btnAniBox.zOrder = 10;
            }
            if (!this.zhaolanAni) {
                this.zhaolanAni = new Laya.Skeleton();
                this.zhaolanAni.load("anim1/xinshouyindao.sk", Laya.Handler.create(this, function () {
                    this.zhaolanAni.play("anniu", true);
                }));
                this.zhaolanAni.y = -300;
                this.btnAniBox.addChild(this.zhaolanAni);
            } else {
                this.btnAniBox.removeChild(this.zhaolanAni);
                this.zhaolanAni.destroy();
            }
        }

        setInviteComplete() {
            this.inviteComplete = true;
            if (this.clickImage) {
                this.clickImage.offAll();
            }
            this.nextStep();
        }
        destroy() {
            this.btn = null;
            EventMgr.getInstance().removeEvent("GUIDE_GUIDE6_INVITE", this, this.setInviteComplete);
            super.destroy();
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(14)=='1'){
                let s1 = "begin_solicit_" + this.currentGuide.Id.substring(14);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
            }
                let guidid=this.currentGuide.Id;
                if(guidid=='Guide1_Invite_3'){
                    let str = "ftue_40_attractTap";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }
            }
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_10");
        }
    }

    class Guide7 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_7") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        creatAnmi(anmi) {
            // super.creatAnmi(anmi);
            if (!this.showAni) {
                this.showAni = new Laya.Skeleton();
                this.showAni.load("anim1/xinshouyindao.sk", Laya.Handler.create(this, function () {
                    this.showAni.play("maomao2", true);
                    this.showAni.x = this.guideBox.width / 2;
                    this.showAni.y = this.guideBox.height / 2;
                    this.guideBox.addChild(this.showAni);
                }));
                Laya.timer.once(3000, this, function () {
                    this.showAni.removeSelf();
                    this.showAni.destroy();
                    this.showAni = null;
                    this.nextStep();
                });
            }
        }
    }

    class Guide8 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_8") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.Star != null && PlayerSystem.getStar() >= Number(guideCondition.Star)) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }
        start() {
            Laya.Dialog.closeAll();
            SceneMgr.getInstance().hideSceneSwitching();
            SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId);
            super.start();
        }
        nextStep() {
            super.nextStep();
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_9");
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                switch (this.currentGuide.Id) {
                    case "Guide2_Build_1": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_beach_1"]);
                        let str = "ftue_beachUnlock";
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);            
                        break;
                    }
                    case "Guide2_Build_3": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_buy_sand_1"]);
                        break;
                    }
                }
            }
        }
    }

    class Guide9 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_9") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }

        start() {
            if (this._start) {
                return;
            };
            EventMgr.getInstance().registEvent("GUIDE_GUIDE8_AD", this, this.setADComplete);
            super.start();
        }

        creatAnmi(anmi) {
            if (this.currentGuide && this.currentGuide.Id == "Guide2_CatFood_2") {
                EventMgr.getInstance().postEvent("FORCE_ATTRACT_BEACH_CAT");
                Laya.timer.once(600, this, this.nextStep);
            }
            if (this.currentGuide && this.currentGuide.Id == "Guide2_CatReward_1") {
                if (!this.showAni) {
                    this.showAni = new Laya.Skeleton();
                    this.showAni.load("anim1/xinshouyindao.sk", Laya.Handler.create(this, function () {
                        this.showAni.play("shizhong", false);
                        this.showAni.x = this.guideBox.width / 2;
                        this.showAni.y = this.guideBox.height / 2;
                        this.guideBox.addChild(this.showAni);
                        this.showAni.zOrder = 4;
                        this.btnSprite.visible = true;
                        this.btnAniBox.visible = false;
                        this.showAni.on(Laya.Event.STOPPED, this, function () {
                            this.showAni.removeSelf();
                            this.showAni.destroy();
                            this.showAni = null;
                            this.nextStep();
                            Laya.timer.clear(this, this.claerShiZhong);
                        });
                    }));
                    Laya.timer.frameOnce(120, this, this.claerShiZhong);
                }
            }
        }

        claerShiZhong() {
            this.showAni.offAll(Laya.Event.STOPPED);
            this.showAni.removeSelf();
            this.showAni.destroy();
            this.showAni = null;
            this.nextStep();
        }

        nextStep() {
            if (this.currentGuide && this.currentGuide.Id == "Guide2_CatFood_1_1" && !this.adComplete) {
                return;
            }
            super.nextStep();
            if (this.currentGuide && this.currentGuide.Id == "Guide2_CatFood_1") {
                this.btnSprite.mouseEnabled = false;
                if (SceneMgr.getInstance().getCurrentSceneId() != Constants.BeachSceneId || GameContext.Sliding) {
                    SceneMgr.getInstance().switchScene(Constants.BeachSceneId, Laya.Handler.create(this, function () {
                        Laya.timer.once(500, this, function () {
                            let scene = SceneMgr.getInstance().getScene(Constants.BeachSceneId);
                            let rect = new Laya.Rectangle(1202, 763, 248, 212);
                            this.registerObj("btn0", scene);
                            this.registerClick("btn0", scene.getChildByName("root").getChildByName("bowl"), rect);
                            this.btnSprite.mouseEnabled = true;
                        });
                    }));
                } else {
                    let scene = SceneMgr.getInstance().getScene(Constants.BeachSceneId);
                    let rect = new Laya.Rectangle(1202, 763, 248, 212);
                    this.registerObj("btn0", scene);
                    this.registerClick("btn0", scene.getChildByName("root").getChildByName("bowl"), rect);
                    this.btnSprite.mouseEnabled = true;
                }
            }
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("UPDATA_GIFTBTN_SHOW");
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_11");
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                switch (this.currentGuide.Id) {
                    case "Guide2_CatFood_1": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_add_cat_food_1"]);
                        break;
                    }
                    case "Guide2_CatReward_1": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_obtain_cat_gift_1"]);
                        break;
                    }
                }
            }
        }

        setADComplete() {
            if (this.freeLabel) {
                this.freeLabel.destroy();
                this.freeLabel = null;
            }
            this.adComplete = true;
            this.nextStep();
        }

        destroy() {
            EventMgr.getInstance().removeEvent("GUIDE_GUIDE8_AD", this, this.setADComplete);
            super.destroy();
        }

    }

    class CommonButton extends Laya.Button {

        onAwake() {
            super.onAwake();
            if (this.anchorX != 0.5 || this.anchorY != 0.5) {
                let anchorX = this.anchorX | 0;
                let anchorY = this.anchorY | 0;
                this.anchorX = this.anchorY = 0.5;
                this.x += (this.anchorX - anchorX) * this.width * this.scaleX;
                this.y += (this.anchorY - anchorY) * this.height * this.scaleY;
            }
            this._sX = this.scaleX;
            this._sY = this.scaleY;
        }

        changeState() {
            super.changeState();
            if (this._stateNum == 1 && !this._banClickEffect) {
                this.scaleX = this._state > 0 ? this._effectScale ? this._sX * this._effectScale : this._sX * 1.2 : this._sX;
                this.scaleY = this._state > 0 ? this._effectScale ? this._sY * this._effectScale : this._sY * 1.2 : this._sY;
            }
        }

        //禁用点击放大效果
        setBanClickEffect(type) {
            this._banClickEffect = type;
        }

        //设置点击放大倍率，默认1.2
        setEffectScale(num) {
            this._effectScale = num;
        }
    }

    class Guide10 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_10") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }

        start() {
            if (this._start) {
                return;
            };
            Laya.Dialog.closeAll();
            EventMgr.getInstance().registEvent("GUIDE_GUIDE10_AD", this, this.setADComplete);
            SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId);
            super.start();
        }

        nextStep() {
            if (this.talkStr && (this.currentCharNum != this.talkStr.length)) {
                this.showTalk(true);
                return;
            }
            if (this.currentGuide && this.currentGuide.Id == "Guide1_ADInvite_3" && !this.adComplete) {
                return;
            }
            super.nextStep();
            if (this.currentGuide && this.currentGuide.Id == "Guide1_ADInvite_3") {
                // let free = new CommonButton("pinjie_btn/btn_blue.png", "");
                // free.scaleX = free.scaleY = 2;
                // free.sizeGrid = '21,24,23,22';
                // free.width = 170;
                // free.height = 70
                // free.stateNum = 1;
                // free.x = this.btnShowSprite.width / 2 - 410 + 30;
                // free.y = 1200;
                // this.btnShowSprite.addChild(free);
                // free.zOrder = 2;

                // let freeText = new Laya.Image('pinjie_btn/free.png');
                // free.addChild(freeText);
                // freeText.x = (170 - 114) / 2;
                // freeText.y = (70 - 32) / 2;

                let close = new CommonButton("pinjie_btn/guanbi.png", "");
                close.width = close.height = 50;
                close.scaleX = close.scaleY = 2.5;
                close.stateNum = 1;
                close.x = this.btnShowSprite.width / 2 + 338;
                close.y = 958;
                this.btnShowSprite.addChild(close);
                close.zOrder = 2;


                // free.on(Laya.Event.CLICK, this, function () {
                //     free.destroy();
                //     close.destroy();
                //     Laya.Dialog.close("ui/VideoShareUI.scene");
                //     EventMgr.getInstance().postEvent("GUIDE10_FREE_ZHAOLAN");
                // });
                close.on(Laya.Event.CLICK, this, function () {
                    //free.destroy();
                    close.destroy();
                    Laya.Dialog.close("ui/VideoShareUI.scene");
                    EventMgr.getInstance().postEvent("GUIDE10_FREE_ZHAOLAN");
                });
                this.btnAni.playAnim("zhuazi2", true);
            }
        }

        creatAnmi(anmi) {
            Laya.Dialog.close("ui/VideoShareUI.scene");
            let shareStr = Strings$1.get("tid#UI_033");
            let videoStr = Strings$1.get("tid#UI_032");
            Laya.Dialog.open("ui/VideoShareUI.scene", true, [Constants.ShareId.Zhaolan_AD_View, "DO_AD_ZHAOLAN", videoStr, shareStr]);
            this.nextStep();
        }

        setADComplete() {
            this.adComplete = true;
            if (this.clickImage != null && !this.clickImage.destroyed) {
                this.clickImage.offAll();
            }
            this.nextStep();
        }

        destroy() {
            this.btn = null;
            EventMgr.getInstance().removeEvent("GUIDE_GUIDE10_AD", this, this.setADComplete);
            super.destroy();
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(16)=='1'){
                let s1 = "begin_ADsolicit_" + this.currentGuide.Id.substring(16);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + s1]);
                }
                let guidid=this.currentGuide.Id;
                if(guidid=='Guide1_ADInvite_3'){
                    let str = "ftue_45_bonus5xMode";
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str,null,1]);
                }
            }
        }
    }

    class Guide11 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_11") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.Star != null && PlayerSystem.getStar() >= Number(guideCondition.Star)) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }
        start() {
            Laya.Dialog.closeAll();
            EventMgr.getInstance().postEvent("CHOOSE_PLACE_BUILDING");
            super.start();
        }
        nextStep() {
            super.nextStep();
            if (this.currentGuide && this.currentGuide.Id == "Guide2_Build2_2") {
                if (SceneMgr.getInstance().getCurrentSceneId() == Constants.BeachSceneId) {
                    this.nextStep();
                } else {
                    if (SceneMgr.getInstance().getCurrentSceneId() != Constants.CoffeeSceneId) {
                        SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId);
                    }
                }
            }
        }
        complete() {
            super.complete();
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(14)=='1'){
                let str = "begin_buy_fishing_stage_" + this.currentGuide.Id.substring(14);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + str]);
            }
            }
        }

    }

    class Guide12 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_12") {
                ifGuide = true;
            }
            return ifGuide;
        }
        start() {
            Laya.Dialog.closeAll();
            super.start();
        }
        nextStep() {
            super.nextStep();
            if (this.currentGuide && this.currentGuide.Id == "Guide2_FishEnter_2") {
                let scene = SceneMgr.getInstance().getScene(Constants.BeachSceneId);
                Laya.timer.once(200, this, function () {
                    let building = scene.getBuilding(2020);
                    if (building) {
                        this.registerObj("btn1", scene);
                        this.registerClick("btn1", building.prefab.getChildByName("touch"), null, [building.prefab]);
                        return;
                    }
                });
            } else if (this.currentGuide && this.currentGuide.Id == "Guide2_FishEnter_1") {
                if (SceneMgr.getInstance().getCurrentSceneId() == Constants.BeachSceneId) {
                    this.nextStep();
                }
            }
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(17)=='1'){
                let str = "begin_fishing_" + this.currentGuide.Id.substring(17);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + str]);
            }
            }
        }

    }

    class Guide13 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_13") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (SceneMgr.getInstance().getCurrentSceneId() == Constants.FishingSceneId) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        creatAnmi(anmi) {
            if (this.currentGuide && this.currentGuide.Id == "Guide2_FishGame_3") {
                EventMgr.getInstance().postEvent("GUIDE2_FISHGAME", ["Guide2_FishGame_3"]);
                this.nextStep();
            } else if (this.currentGuide && this.currentGuide.Id == "Guide2_FishGame_5") {
                EventMgr.getInstance().postEvent("GUIDE2_FISHGAME", ["Guide2_FishGame_5"]);
                this.nextStep();
            } else if (this.currentGuide && this.currentGuide.Id == "Guide2_FishGame_7") {
                EventMgr.getInstance().postEvent("GUIDE2_FISHGAME", ["Guide2_FishGame_7"]);
                this.nextStep();
            } else if (this.currentGuide && this.currentGuide.Id == "Guide2_FishGame_10") {
                EventMgr.getInstance().postEvent("GUIDE2_FISHGAME", ["Guide2_FishGame_10"]);
                this.nextStep();
            }
        }
    }

    class HomePetMgr extends BaseCatMgr {

        static getInstance() {
            return HomePetMgr.instance?HomePetMgr.instance:HomePetMgr.instance = new HomePetMgr();
        }

        addPet(pet) {
            this.addCat(pet);
        }

        removePet(petId) {
            this.removeCat(petId);
        }

        getPet(petId) {
            return this.getCat(petId);
        }

        allPets() {
            let result = new Array();
            for(const [key, pet] of this._getMap()) {
                result.push(pet);
            }
            return result;
        }

        idlePets() {
            let result = new Array();
            for(const [key, pet] of this._getMap()) {
                if(pet.parent == null){
                    result.push(pet);
                }
            }
            return result;
        }


        clear() {
            super.clear();
        }
    }

    class GetNewCatMgr$1{
        static getInstance() {
            return GetNewCatMgr$1.instance ? GetNewCatMgr$1.instance : GetNewCatMgr$1.instance = new GetNewCatMgr$1();
        }
        constructor(){
        }
        _onLoaded(prefabDef){
            this.getNewCatPrefab = prefabDef.create();
            this.getNewCatPrefab.getNewCatSkin(this.catSkinName);
            this.getNewCatPrefab.setCatStr(this.catStr);
        }
        catSkin(catSkinName, catStr) {
            if (GameContext.isGuiding) {
                return;
            }
            this.catSkinName=catSkinName;
            this.catStr = catStr;
            Laya.loader.create("prefab/ui/GetNewCat.prefab",Laya.Handler.create(this,this._onLoaded));
            
        }
    }

    class HomeRubbishData {

        constructor(id) {
            this.id = id;
            this.type = Utils.randomExt2(0, 3);
            let x = Utils.randomExt2(500, 1950);
            let y = Utils.randomExt2(850, 2080);
            this.pos(x, y);
        }

        pos(x, y) {
            this._x = x;
            this._y = y;
        }

        get x() {
            return this._x;
        }

        get y() {
            return this._y;
        }

        set type(value) {
            this._type = value;
        }

        get type() {
            return this._type;
        }

        get id() {
            return this._id;
        }

        set id(value) {
            this._id = value;
        }

        getObject() {
            let obj = {
                id: this.id,
                x: this.x,
                y: this.y,
                type: this._type
            };
            return obj;
        }

        updata(customData) {
            this.pos(customData.x, customData.y);
            this.id = customData.id;
            this.type = customData.type;
        }

    }

    class HomeRubbishDataMgr {

        constructor() {
            this._map = new Map();
            this._lastTime = null;
            this._maxId = 0;
        }

        static getInstance() {
            return HomeRubbishDataMgr.instance ? HomeRubbishDataMgr.instance : HomeRubbishDataMgr.instance = new HomeRubbishDataMgr();
        }

        updata(customData) {
            let rubbishDataArr = [];
            let lastTime = null;
            if (customData.homeRubbishData != null) {
                let rubbishData = JSON.parse(customData.homeRubbishData);
                rubbishDataArr = rubbishData.rubbishDataArr;
                lastTime = rubbishData.lastTime;
            }
            this._map.clear();
            for (const rubbishData of rubbishDataArr) {
                let homeRubbishData = new HomeRubbishData(rubbishData.id);
                homeRubbishData.updata(rubbishData);
                this._maxId = homeRubbishData.id;
                this._map.set(homeRubbishData.id, homeRubbishData);
            }
            this._lastTime = lastTime;
        }

        createCustomData() {
            let homeRubbishDataArr = [];
            this._map.forEach((rubbishData, k) => {
                let obj = rubbishData.getObject();
                homeRubbishDataArr.push(obj);
            });
            let rubbishData = {
                "rubbishDataArr": homeRubbishDataArr,
                "lastTime": this._lastTime
            };
            return JSON.stringify(rubbishData);
        }

        clear() {
            this._map.clear();
            this._lastTime = null;
            this._maxId = 0;
        }

        getLastTime() {
            return this._lastTime;
        }

        setLastTime(time) {
            this._lastTime = time;
        }

        getRubbishNum() {
            return this._map.size;
        }

        addRubbish(num) {
            if (num <= 0) {
                return;
            }
            let maxRubbishIdNum = Number(Const.get("HomeRubbishLimit")) * 2;
            for (let i = 0; i < num; i++) {
                let id = this._maxId % maxRubbishIdNum;
                id++;
                while (this._map.has(id)) {
                    ++id;
                    id = id % maxRubbishIdNum;
                }
                let rubbish = new HomeRubbishData(id);
                this._maxId = id;
                this._map.set(id, rubbish);
            }
        }

        getRubbishs() {
            return this._map;
        }

        removeRubbish(id) {
            this._map.delete(id);
        }

    }

    class NauticalToolData {

        constructor(config) {
            this._config = config;
            this._count = 0;
        }

        get Id() {
            return this._config.Id;
        }

        get Name() {
            return this._config.Name;
        }

        get Desc() {
            return this._config.Desc;
        }

        get Icon() {
            return this._config.Icon;
        }

        get PointRate() {
            return this._config.PointRate + 1;
        }

        get IntimacyRate() {
            return this._config.IntimacyRate + 1;
        }

        get Rank() {
            return this._config.Rank;
        }

        get count() {
            return this._count>0?this._count:0;
        }

        set count(count) {
            this._count = count;
        }

        clear() {
            this._count = 0;
        }
    }

    class NauticalFoodData {

        constructor(config) {
            this._config = config;
            this._count = 0;
        }

        get Id() {
            return this._config.Id;
        }

        get Name() {
            return this._config.Name;
        }

        get Desc() {
            return this._config.Desc;
        }

        get Icon() {
            return this._config.Icon;
        }

        get Point() {
            return this._config.Point;
        }

        get Intimacy() {
            return this._config.Intimacy;
        }

        get UpgradeLine() {
            return this._config.UpgradeLine;
        }

        get Rank() {
            return this._config.Rank;
        }

        get count() {
            return this._count>0?this._count:0;
        }

        set count(count) {
            this._count = count;
        }

        clear() {
            this._count = 0;
        }
    }

    class NauticalData {

        constructor() {
            this._sailIndex = null;
            this._adLock = null;
            this._inTrip = false;
            this._startSec = null;
            this._endSec = null;
            this._adReduceTimes = null;
            this._originEndSec = null;
            this._team = new Array();
            this._foods = new Map();
            this._tools = new Array();
        }

        update(nauticalInfo,sailIndex) {
            this.clearTripData();
            if(nauticalInfo) {
                this._inTrip = nauticalInfo.inTrip;
                this._adLock = nauticalInfo.adLock;
                if(sailIndex >= 0){
                    this._sailIndex = sailIndex;
                }
                if(this._inTrip) {
                    this._startSec = nauticalInfo.tripStartSec;
                    this._endSec = nauticalInfo.tripEndSec;
                    this._originEndSec = nauticalInfo.originEndSec;
                    this._team.length = 0;
                    this._foods.clear();
                    this._tools.length = 0;

                    let team = nauticalInfo.team;
                    if(team) {
                        for(const petId of team) {
                            this._team.push(petId);
                        }
                    }

                    let foods = Utils.obj2Map(nauticalInfo.foodInTrip);
                    if(foods) {
                        for(const [id, amount] of foods) {
                            this._foods.set(id, amount);
                        }
                    }

                    let tools = nauticalInfo.toolInTrip;
                    if(tools) {
                        for(const toolId of tools) {
                            this._tools.push(toolId);
                        }
                    }
                    this._adReduceTimes = nauticalInfo.adReduceTimes;
                }else {
                    this.food1 = this.food1;
                    this.food2 = this.food2;
                    this.tool = this.tool;
                }
            }else {
                //TODO 无信息按理是有问题，应该干点啥呢？
            }

        }

        get sailIndex(){
            return this._sailIndex;
        }

        get adLock(){
            return this._adLock;
        }

        get inTrip() {
            return this._inTrip;
        }

        get startSec() {
            return this._startSec;
        }

        get endSec() {
            return this._endSec;
        }

        get originEndSec() {
            return this._originEndSec;
        }

        get team() {
            return this._team;
        }

        get foods() {
            return this._foods;
        }

        get tools() {
            return this._tools;
        }

        get food1() {
            return this._food1Id;
        }

        set food1(foodId) {
            if(this._food1Id) {
                this.removeFood(this._food1Id);
            }
            this._food1Id = foodId;
            if(foodId) {
                this.addFood(foodId);
            }
        }

        get food2() {
            return this._food2Id;
        }

        set food2(foodId) {
            if(this._food2Id) {
                this.removeFood(this._food2Id);
            }
            this._food2Id = foodId;
            if(foodId) {
                this.addFood(foodId);
            }
        }

        get tool() {
            return this._toolId;
        }

        set tool(toolId) {
            if(this._toolId) {
                this.removeTool(this._toolId);
            }
            this._toolId = toolId;
            if(toolId) {
                this.addTool(toolId);
            }
        }

        hasPet(petId) {
            for(let i = 0; i<this._team.length; ++i) {
                if(this._team[i]==petId) {
                    return true;
                }
            }
            return false;
        }

        getFoodCount(foodId) {
            let count = this._foods.get(foodId);
            return count?count:0;
        }

        hasTool(toolId) {
            for(let i = 0; i<this._tools.length; ++i) {
                if(this._tools[i]==toolId) {
                    return true;
                }
            }
            return false;
        }

        addPet(petId) {
            if(!this._inTrip) {
                for(const id of this._team) {
                    if(id==petId) {
                        return false;
                    }
                }
                this._team.push(petId);
                return true;
            }
        }

        removePet(petId) {
            if(!this._inTrip) {
                for(let i = 0; i<this._team.length; ++i) {
                    if(this._team[i]==petId) {
                        this._team.splice(i, 1);
                        return true;
                    }
                }
                return false;
            }
        }

        removeAllPet() {
            if(!this._inTrip) {
                this._team.length = 0;
                return true;
            }
            else {
                return false;
            }
        }

        addFood(foodId) {
            if(!this._inTrip) {
                for(const [k,v] of this._foods) {
                    if(k==foodId) {
                        this._foods.set(k, v + 1);
                        return true;
                    }
                }
                this._foods.set(foodId, 1);
                return true;
            }
        }

        removeFood(foodId) {
            if(!this._inTrip) {
                for(const [k,v] of this._foods) {
                    if(k==foodId) {
                        if(v-1>0)
                            this._foods.set(k, v - 1);
                        else
                            this._foods.delete(k);
                        return true;
                    }
                }
                return false;
            }
        }

        addTool(toolId) {
            if(!this._inTrip) {
                for(const id of this._tools) {
                    if(id==toolId) {
                        return false;
                    }
                }
                this._tools.push(toolId);
                return true;
            }
        }

        removeTool(toolId) {
            if(!this._inTrip) {
                for(let i = 0; i<this._tools.length; ++i) {
                    if(this._tools[i]==toolId) {
                        this._tools.splice(i, 1);
                        return true;
                    }
                }
                return false;
            }
        }

        dumpStartReqInfo(tripIndex) {
            let catIds = this._team;
            let foodInTrip = Utils.map2Obj(this._foods);
            let toolInTrip = this._tools;
            if(tripIndex != 0 && tripIndex != 1 && tripIndex != 2 ){
                tripIndex = this._sailIndex;
            }
            return {"catIds": catIds, "foodInTrip": foodInTrip, "toolInTrip": toolInTrip,"tripIndex" : tripIndex};
        }

        clearTripData() {
            this._inTrip = false;
            this._startSec = null;
            this._endSec = null;
            this._originEndSec = null;
            this._team.length = 0;
            this._foods.clear();
            this._tools.length = 0;
        }

        clear() {
            this._inTrip = false;
            this._startSec = null;
            this._endSec = null;
            this._originEndSec = null;
            this._team.length = 0;
            this._foods.clear();
            this._tools.length = 0;
            this._food1Id = null;
            this._food2Id = null;
            this._toolId = null;
            this._adReduceTimes = null;
        }

        getAdReduceTimes() {
            return this._adReduceTimes ? this._adReduceTimes : 0;
        }
    }

    class NauticalDataMgr {

        constructor() {
            this._foodsMap = new Map();
            this._toolsMap = new Map();
            this._foodsList = new Array();
            this._toolsList = new Array();
            this._petIds = new Array();
            this._nauticalInfo = new Array();
            this.btnList = new Array();
            this._nauticalTicketNum = 0;
            this.init();
        }

        static getInstance() {
            return NauticalDataMgr.instance?NauticalDataMgr.instance:NauticalDataMgr.instance = new NauticalDataMgr();
        }

        init() {
            this._foodsMap.clear();
            this._toolsMap.clear();
            let foodsConfig = ConfigMgr$1.getInstance().getConfig("NauticalFoods");
            for(const [k, food] of foodsConfig) {
                this._foodsMap.set(food.Id, new NauticalFoodData(food));
            }
            let toolsConfig = ConfigMgr$1.getInstance().getConfig("NauticalTools");
            for(const [k, tool] of toolsConfig) {
                this._toolsMap.set(tool.Id, new NauticalToolData(tool));
            }
            this._nauticalNum = Const.get("NauticalSailLineADAmount");
            for(let i = 0;i < this._nauticalNum; i++){
                this._nauticalInfo[i] = new NauticalData();
            }
        }

        getNauticalInfo() {
            return this._nauticalInfo;
        }

        getAvailablePets() {
            return this._petIds.slice(0);
        }

        getFoods() {
            return this._foodsList.slice(0);
        }

        getTools() {
            return this._toolsList.slice(0);
        }

        getBtnList() {
            return this.btnList;
        }

        getFood(foodId) {
            return this._foodsMap.get(foodId);
        }

        getTool(toolId) {
            return this._toolsMap.get(toolId);
        }

       addToolById(toolId){
            let tool=this.getTool(toolId);
            tool.count=Number(tool.count)+1;
            this._toolsMap.set(toolId,tool);    
        }


        updateUsage(nauticalInfo) {
            let petIds = nauticalInfo.team;
            let foods = nauticalInfo.foods;
            let tools = nauticalInfo.tools;

            //宠物和道具暂不处理
            // for(let i = this._petIds.length - 1; i>=0; --i) {
            //     for(let j = petIds.length - 1; j>=0; --j) {
            //         if(this._petIds[i] == petIds[j]){
            //             this._petIds.splice(i, 1);
            //             break;
            //         }
            //     }
            // }

            for(let i = this._foodsList.length - 1; i>=0; --i) {
                let food = this._foodsList[i];
                let count = foods.get(food.Id);
                if(count) {
                    food.count = food.count - count;
                    if(food.count==0) {
                        this._foodsList.splice(i,1);
                    }
                }
            }
        }



        _clearFoods() {
            for(const [k, food] of this._foodsMap) {
                food.clear();
            }
            this._foodsList.length = 0;
        }

        _clearTools() {
            for(const [k, tool] of this._toolsMap) {
                tool.clear();
            }
            this._toolsList.length = 0;
        }

        updateNauticalFoods(nauticalFoods) {
            this._clearFoods();
            if(nauticalFoods) {
                for(const [k, item] of Utils.obj2Map(nauticalFoods)) {
                    let nauticalFoodData = this._foodsMap.get(item.id);
                    if(nauticalFoodData) {
                        nauticalFoodData.count = item.amount;
                        this._foodsList.push(nauticalFoodData);
                    }
                }
            }
        }

        updateNauticalTools(nauticalTools) {
            this._clearTools();
            if(nauticalTools) {
                for(const [k, item] of Utils.obj2Map(nauticalTools)) {
                    let nauticalToolData = this._toolsMap.get(item.id);
                    if(nauticalToolData) {
                        nauticalToolData.count = item.amount;
                        this._toolsList.push(nauticalToolData);
                    }
                }
            }
        }

        updateAvailablePets(petIds) {
            this._petIds.length = 0;
            if(petIds) {
                for(const petId of petIds) {
                    this._petIds.push(petId);
                }
            }
        }

        updateNauticalInfo(nauticalInfo) {
            for(let i = 0; i < this._nauticalNum ; i ++){
                this._nauticalInfo[i].update(nauticalInfo[i],i);
            }
        }

        updateOnlyNauticalInfo(sailIndex,nauticalInfo) {
            this._nauticalInfo[sailIndex].update(nauticalInfo);
            this.updateOnlyBtnList(sailIndex);
        }

        updateBtnList(btnList){
            this.btnList = btnList;
            EventMgr.getInstance().postEvent("UPDATE_SAILBTN");
        }

        updateOnlyBtnList(sailIndex){
            for(let i = 0; i< this.btnList.length; i++){
                if(this.btnList[i].nauticalIndex == sailIndex){
                    this.btnList[i].nauticalInfo = this._nauticalInfo[sailIndex];
                }
            }
            EventMgr.getInstance().postEvent("UPDATE_SAILBTN");
        }

        clearNauticalInfo(tripIndex) {
            this._nauticalInfo[tripIndex].clear();
            this.updateOnlyBtnList(tripIndex,this._nauticalInfo[tripIndex]);
        }

        clear() {
            this._clearFoods();
            this._clearTools();
            this._nauticalInfo.forEach(element => {
                element.clear();
            });
            this._nauticalInfo = new Array();
            this._petIds.length = 0;
            this._nauticalTicketNum = 0;
            this.init();
        }

        getFoodCanMergeInfo(foodId) {
            let nextFood = null;
            this._foodsMap.forEach(food => {
                let upgradeLine = JSON.parse(food.UpgradeLine)[0];
                if (upgradeLine == null) {
                    return;
                }
                if (upgradeLine.id == foodId) {
                    nextFood = food;
                    return;
                }
            });
            return nextFood;
        }

        getAdReduceTimes(nauticalIndex){
            return this._nauticalInfo[nauticalIndex].getAdReduceTimes();
        }

        updateNauticalTicketNum(num) {
            this._nauticalTicketNum = num;
        }

        getNauticalTicketNum() {
            return this._nauticalTicketNum;
        }

    }

    class CatNewComeMgr {

        constructor() {
            this.PopArr = new Array();
            EventMgr.getInstance().registEvent("CATNEEWCOME_DONE", this, this.doPop);
        }

        static getInstance(){
            return CatNewComeMgr.instance ? CatNewComeMgr.instance : CatNewComeMgr.instance = new CatNewComeMgr();
        }


        pushPop(petId){
            this.PopArr.push(petId);
        }

        doPop(){
            if(this.PopArr.length <= 0){
                return ;
            }
            for (let i = 0; i < this.PopArr.length; i++){
                EventMgr.getInstance().postEvent("DIALOG_CHANGE_STATUS",[Constants.DialogStatus.CatNewcome,1]);
                Laya.Dialog.open("ui/CatNewCome.scene",false,this.PopArr[i]);
            }
            this.PopArr.length = 0;
        }

        clear(){
            this.PopArr.length = 0;
        }

    }

    //管理PetDataMgr
    class PetSystem {
        static init() {
            EventMgr.getInstance().registEvent("CUSTOMDATA_PET_RESTORE", null, PetSystem.restoreHomeStatus);
        }

        static clear() {
            PetDataMgr.getInstance().clear();
            HomePetMgr.getInstance().clear();
            CatNewComeMgr.getInstance().clear();
        }

        static getPetInfo(petId) {
            return PetDataMgr.getInstance().getPet(petId);
        }

        static addIntimacy(petId, intimacy) {
            PetDataMgr.getInstance().addIntimacy(petId, intimacy);
            SyncSystem.startOperatingUpdate();
        }

        static addVisitTimes(petId, visitTimes) {
            PetSystem.newCatTip(petId);
            PetDataMgr.getInstance().addVisitTimes(petId, visitTimes);
            SyncSystem.startOperatingUpdate();
        }

        static newCatTip(petId){
            if(PetSystem.getPetInfo(petId).getVisitTimes()==0&&petId!="Cat1"){
                let pet = PetSystem.getPetInfo(petId);
                if(JSON.parse(pet.config.ShareReward).FirstVisitReward.amount){
                    if(DialogStatusMgr.getInstance().getStatus() == 0 ){
                        Laya.Dialog.open("ui/CatNewCome.scene",false,petId);
                    }
                    else {
                        CatNewComeMgr.getInstance().pushPop(petId);
                    }
                }
                else {
                    let catHeadSkin=pet.getHeadIcon();
                    GetNewCatMgr$1.getInstance().catSkin(catHeadSkin);
                }
            }
        }

        static beachAddVisitTimes(petId, visitTimes) {
            PetDataMgr.getInstance().beachAddVisitTimes(petId, visitTimes);
        }

        static doAdoptPet(petId){
            PetDataMgr.getInstance().adoptPet(petId);
        }

        static dumpAndStashPetRewards() {
            return PetDataMgr.getInstance().dumpAndStashPetRewards();
        }

        static clearPetRewards() {
            PetDataMgr.getInstance().clearPetRewards();
        }

        // 下面是与System交互

        static updatePets(pets) {
            Object.getOwnPropertyNames(pets).forEach(key=>{

                let petInfo = pets[key];
                PetSystem.updatePet(petInfo);
            });
        }

        static updatePet(petInfo) {
            let petId = petInfo.id;
            PetDataMgr.getInstance().updatePet(petId, petInfo);
        }

        static adoptPet(petId,type, handler) {
            EventMgr.getInstance().postEvent("SERVICE_ADOPT_PET", [petId,type, handler]);
        }

        static getPetsInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_PETSINFO", [handler]);
        }

        static needRedPoint() {
            return PetDataMgr.getInstance().needRedPoint();
        }

        static clearAllRedPoint() {
            PetDataMgr.getInstance().clearAllRedPoint();
        }

        static clearAllNewCome() {
            PetDataMgr.getInstance().clearAllNewCome();
        }

        static toyUnlock(config) {
            return PetDataMgr.getInstance().toyUnlock(config);
        }


        // 家园

        static get PetsAdopted() {
            return PetDataMgr.getInstance().PetsAdopted;
        }

         static get PetIdsAdopted() {
            return PetDataMgr.getInstance().PetIdsAdopted;
        }

        static get PetsFakeInHome() {
            return PetDataMgr.getInstance().PetsFakeInHome;
        }

        static get HomePetsLimit() {
            let limit = 0;
            let star = PlayerDataMgr.getInstance().getStar();
            let arr = Const.getArray("HomeCatSetLimit");
            for(const info of arr) {
                if(Number(info.Star)<=star)
                    limit = Number(info.Amount);
                else
                    break;
            }
            return limit;
        }

        static get HomePetsNextLimitInfo() {
            let star = PlayerDataMgr.getInstance().getStar();
            let arr = Const.getArray("HomeCatSetLimit");
            for(const info of arr) {
                if(Number(info.Star) > star)
                    return info;
            }
            return null;
        }

        static hasToy(toyId) {
            return PetDataMgr.getInstance().hasToy(toyId);
        }

        static addToy(toyId) {
            let itemConfig = PetDataMgr.getInstance().addToy(toyId);
            if (itemConfig) {
                PetSystem.updataItemSpecialFun(itemConfig);
            }
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
        }

        static updateHomeInfo(homeInfo) {
            if(homeInfo) {
                let photoWall = homeInfo.photoWall;
                // TODO 处理照片墙 或者新建个System用于处理照片墙；
                let petsInHome = homeInfo.petsInHome;
                let unlockPetToys = homeInfo.unlockPetToys;
                PetDataMgr.getInstance().updatePetsInhome(petsInHome);
                PetDataMgr.getInstance().updateUnlockPetToys(unlockPetToys);
            }
        }

        static restoreHomeStatus(homeStatus, lastTime) {
            EventMgr.getInstance().postEvent("SIMULATOR_CREATE_SESSION", Constants.HomelandId);
            let now = TimeUtils.getTimeNowOnServer();
            // if(lastTime && lastTime >0) {
            //     EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SET_TIME", lastTime]);
            // }else{
            //     EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SET_TIME", now]);
            // }
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SET_TIME", now]);

            let petsMap = PetDataMgr.getInstance().getAllPets();
            if(homeStatus) {
                for(const info of homeStatus) {
                    let pet = petsMap.get(info.id);
                    if(pet.homeStatus) {
                        pet.restoreParams(info.params);
                    }
                }
            }
            for(const [petId, pet] of petsMap) {
                if(pet.homeStatus)
                    EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PUT_PET", petId]);
            }
            EventMgr.getInstance().postEvent("SIMULATOR_RUN_SESSION", Constants.HomelandId);
        }

        static changePetInHome(params, handler) {
            // console.debug("changePetInHome", params);
            let dealHandler = Laya.Handler.create(null, function (res) {
                if (handler) {
                    handler.runWith(res);
                }
                let petNum = PetSystem.PetsFakeInHome.length;
                let str = 'catHomed';
                let obj = { 'homedCount': petNum};
                if(Laya.LocalStorage.getJSON('petNum')){
                    if(petNum>Laya.LocalStorage.getJSON('petNum')){
                        Laya.LocalStorage.setItem('petNum',petNum);
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                    }
                }else {
                    Laya.LocalStorage.setItem('petNum',petNum);
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                }
                if (petNum <= 0) {
                    HomeRubbishDataMgr.getInstance().setLastTime(null);
                    PetSystem.saveHomeRubbushData();
                } else {
                    if (HomeRubbishDataMgr.getInstance().getLastTime() == null) {
                        HomeRubbishDataMgr.getInstance().setLastTime(TimeUtils.getTimeNowOnServer() / 1000);
                        PetSystem.saveHomeRubbushData();
                    }
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_HOME_SET_PET", [params, dealHandler]);
        }
        static dealReward (petId, handler, res) {
            let poped = false;
            let rewards = res.rewards;
            if(rewards) {
                let items = rewards.rewards;
                let amount = res.amount;
                if(items) {
                    for(const item of items) {
                        if(item.type == Constants.ItemType.CatToys) {
                            PetSystem.addToy(item.id);
                            if(!poped) {
                                Laya.Dialog.close("ui/ToyUnlockPop.scene");
                                Laya.Dialog.open("ui/ToyUnlockPop.scene", false, {item:item, petId:petId,amount:amount});
                                poped = true;//只展示一个，如果有多个，后续增加弹窗序列吧
                            }
                        }else if(item.type == Constants.ItemType.NauticalTools || item.type == Constants.ItemType.FishingBaits) {
                            if(item.type == Constants.ItemType.NauticalTools){
                                NauticalDataMgr.getInstance().addToolById(item.id);
                            }
                           if(item.type == Constants.ItemType.FishingBaits){
                            FishingDataMgr.getInstance().getBaseData().unlockBait(item.id);
                           }
                            if(!poped) {
                                Laya.Dialog.close("ui/ToyUnlockPop.scene");
                                Laya.Dialog.open("ui/ToyUnlockPop.scene", false, {item:item, petId:petId,amount:amount});
                                poped = true;//只展示一个，如果有多个，后续增加弹窗序列吧
                            }
                        }
                    }
                }
            }
            // if(res.increase && res.increase>0) {
            //     let pet = PetDataMgr.getInstance().getPet(petId);
            //     pet.increase = res.increase;
            // }
            if(handler)
                handler.runWith(res);
        }
        static cuteInteract(petId, toyId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_HOME_CUTE_INTERACT", [petId, toyId, Laya.Handler.create(null, PetSystem.dealReward, [petId, handler])]);
        }

        static commonInteract(petId, toyId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_HOME_COMMON_INTERACT", [petId, toyId, Laya.Handler.create(null, PetSystem.dealReward, [petId, handler])]);
        }

        static fetchHomeInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_HOME_FETCH_INFO", [handler]);
        }
        /**获取领养前是否需要同步数据 */
        static getIfNeedUpdataToAdopt(pid) {
            return PetDataMgr.getInstance().getIfNeedUpdataToAdopt(pid);
        }

        //家园垃圾
        static updataHomeRubbish(customData) {
            if (customData != null) {
                HomeRubbishDataMgr.getInstance().updata(customData);
                PetDataMgr.getInstance().setRubbishNum(PetSystem.getRubbishNum());
            }
        }

        static saveHomeRubbushData() {
            let data = HomeRubbishDataMgr.getInstance().createCustomData();
            let customData = { "homeRubbishData": data };
            EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", [customData]);
        }

        static creatRubbish() {
            let petNum = PetSystem.PetsFakeInHome.length;
            if (petNum <= 0) {
                return HomeRubbishDataMgr.getInstance().getRubbishs();
            }
            let lastTime = HomeRubbishDataMgr.getInstance().getLastTime();
            if (lastTime == null) {
                lastTime = TimeUtils.getTimeNowOnServer() / 1000;
                HomeRubbishDataMgr.getInstance().setLastTime(TimeUtils.getTimeNowOnServer() / 1000);
                return HomeRubbishDataMgr.getInstance().getRubbishs();
            }
            let createTime = TimeUtils.getTimeNowOnServer() / 1000 - lastTime;
            let gap = 0;
            while (createTime >= gap) {
                let rubbishNum = HomeRubbishDataMgr.getInstance().getRubbishNum();
                let maxRubbishNum = Number(Const.get("HomeRubbishLimit"));
                if (rubbishNum >= maxRubbishNum) {
                    return HomeRubbishDataMgr.getInstance().getRubbishs();
                }
                let homeRubbishCreateGaps = Const.getArray("HomeRubbishCreateGaps");
                let cleanliness = rubbishNum / maxRubbishNum;
                let geps = null;
                for (const obj of homeRubbishCreateGaps) {
                    if (cleanliness >= Number(obj.Cleanness)) {
                        geps = obj.Gaps;
                    } else {
                        break;
                    }
                }
                if (geps == null) {
                    return HomeRubbishDataMgr.getInstance().getRubbishs();
                }
                gap = Utils.randomExt2(geps[0], geps[1]);

                let homeRubbishSpeeds = Const.getArray("HomeRubbishSpeedByCats");
                let speed = null;
                for (const obj of homeRubbishSpeeds) {
                    if (petNum >= Number(obj.CatAmount)) {
                        speed = obj.Speed;
                    } else {
                        break;
                    }
                }
                if (speed == null) {
                    return HomeRubbishDataMgr.getInstance().getRubbishs();
                }
                gap /= speed;
                createTime -= gap;
                if (createTime > 0) {
                    HomeRubbishDataMgr.getInstance().setLastTime(TimeUtils.getTimeNowOnServer() / 1000);
                    HomeRubbishDataMgr.getInstance().addRubbish(1);
                }
            }
            PetSystem.saveHomeRubbushData();
            PetDataMgr.getInstance().setRubbishNum(PetSystem.getRubbishNum());
            return HomeRubbishDataMgr.getInstance().getRubbishs();
        }

        static getMaxValueBase() {
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            let maxValueBase = 0;
            config.forEach((listConfig, petId) => {
                let pet = PetSystem.getPetInfo(petId);
                if (pet && pet.getVisitTimes() > 0 && PetDataMgr.getInstance().getPetValueBase(petId) > maxValueBase) {
                    maxValueBase = PetDataMgr.getInstance().getPetValueBase(petId);
                }
            });
            return maxValueBase;
        }

        static getRubbishRewards(id, adType) {
            let maxValueBase = PetSystem.getMaxValueBase();
            let homeRubbishRewards = Const.getArray("HomeRubbishReward");
            let star = 0;
            let fish = 0;
            let weight = 0;
            for (const reward of homeRubbishRewards) {
                weight += Number(reward.weight);
            }
            let i = 0;
            let randomNum = Utils.randomExt2(0, weight);
            while (homeRubbishRewards[i] != null && randomNum >= Number(homeRubbishRewards[i].weight) && homeRubbishRewards[i + 1] != null) {
                ++i;
            }
            let rubbishReward = homeRubbishRewards[i];
            if (rubbishReward.id == Constants.RewardType.FISH) {
                fish = Math.floor(maxValueBase * Number(rubbishReward.amountRate));
            } else if (rubbishReward.id == Constants.RewardType.STAR) {
                star = Number(rubbishReward.amount);
            }

            let cultivate = PetSystem.randomHomeCultivate("HomeRubbishCultivateRate");
            let medal = cultivate.MEDAL;
            let book = cultivate.BOOK;
            fish += cultivate.FISH;
            star += cultivate.STAR;

            if (adType == Constants.FetchType.AD) {
                fish *= 2;
                star *= 2;
                medal *= 2;
                book *= 2;
            }
            EventMgr.getInstance().postEvent("ADD_HOME_RUBBISH_REWARDS", [star, fish, medal, book]);
            HomeRubbishDataMgr.getInstance().removeRubbish(id);
            PetSystem.saveHomeRubbushData();
            PetDataMgr.getInstance().setRubbishNum(PetSystem.getRubbishNum());
            return {
                FISH: fish,
                STAR: star,
                MEDAL: medal,
                BOOK: book
            }
        }

        static getRubbishNum() {
            return HomeRubbishDataMgr.getInstance().getRubbishNum();
        }

        static getHealthyRewards(type, adType) {
            let maxValueBase = PetSystem.getMaxValueBase();
            let homeRewards = null;
            if (type == Constants.HealthyType.HUNGER) {
                homeRewards = Const.getArray("HomeHealthyHungerReward");
            } else if (type == Constants.HealthyType.THIRST) {
                homeRewards = Const.getArray("HomeHealthyThirstyReward");
            } else if (type == Constants.HealthyType.DIRTY) {
                homeRewards = Const.getArray("HomeHealthyDirtyReward");
            }
            if (homeRewards == null) {
                return;
            }

            let star = 0;
            let fish = 0;
            let weight = 0;
            for (const reward of homeRewards) {
                weight += Number(reward.weight);
            }
            let i = 0;
            let randomNum = Utils.randomExt2(0, weight);
            while (homeRewards[i] != null && randomNum >= Number(homeRewards[i].weight) && homeRewards[i + 1] != null) {
                ++i;
            }
            let homeReward = homeRewards[i];
            if (homeReward.id == Constants.RewardType.FISH) {
                fish = Math.floor(maxValueBase * Number(homeReward.amountRate));
            } else if (homeReward.id == Constants.RewardType.STAR) {
                star = Number(homeReward.amount);
            }
            let cultivate = PetSystem.randomHomeCultivate("HomeHealthyCultivateRate");
            let medal = cultivate.MEDAL;
            let book = cultivate.BOOK;
            fish += cultivate.FISH;
            star += cultivate.STAR;

            if (adType == Constants.FetchType.AD) {
                fish *= 2;
                star *= 2;
                medal *= 2;
                book *= 2;
            }
            EventMgr.getInstance().postEvent("ADD_HOME_RUBBISH_REWARDS", [star, fish, medal, book]);
            return {
                FISH: fish,
                STAR: star,
                MEDAL: medal,
                BOOK: book
            }
        }

        static healthInteract(petId, type,handler) {
            EventMgr.getInstance().postEvent("SERVICE_HOME_HEALTHINTERACT", [petId, type, Laya.Handler.create(null, PetSystem.dealReward, [petId, handler])]);
        }

        static getItemBySpecialType(id) {
            return PetDataMgr.getInstance().getItemBySpecialType(id);
        }

        static updataItemSpecialFun(config) {
            if (config.SpecialFunctionType == 1 || config.SpecialFunctionType == 11) {
                EventMgr.getInstance().postEvent("FETCH_LETTER_BOX_UPDATA_LIMIT");
            }
        }

        static levelUp(petId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_PET_LEVELUP", [petId, handler]);
        }

        static starUp(petId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_PET_STARUP", [petId, handler]);
        }

        static randomHomeCultivate(configName) {
            let medal = 0;
            let book = 0;
            let star = 0;
            let fish = 0;
            let rubbishCultivate = Laya.LocalStorage.getJSON(configName);
            if (rubbishCultivate == null) {
                rubbishCultivate = { allTimes: 0, weightTimes: 0, fixedTimes: 0 };
            } else {
                let day = rubbishCultivate.time;
                let current = PlayerDataMgr.getInstance().getGameCreateDays();
                if (day != current) {
                    rubbishCultivate = { allTimes: 0, weightTimes: 0, fixedTimes: 0 };
                }
            }
            rubbishCultivate.time = PlayerDataMgr.getInstance().getGameCreateDays();
            let config = Const.getArray(configName);
            rubbishCultivate.allTimes++;
            let names = Object.getOwnPropertyNames(config);
            names.sort(function (a, b) {
                return Number(b) - Number(a);
            });
            let rate = null;
            let begin = 0;
            names.forEach(element => {
                let n = Number(element);
                if (rubbishCultivate.allTimes >= n && rate == null) {
                    begin = n;
                    rate = config[element];
                }
            });
            rate.HomeCultivateWeightR = rate.HomeCultivateWeightR | 0;
            rate.HomeCultivateFixedR = rate.HomeCultivateFixedR | 0;
            let unit = (rate.Null | 0) + (rate.HomeCultivateWeightR | 0) + (rate.HomeCultivateFixedR | 0);
            let ran = (rubbishCultivate.allTimes - begin ) % unit;
            if (ran == 0) {
                rubbishCultivate.weightTimes = rubbishCultivate.fixedTimes = 0;
            }
            let weightGet = null;
            if (rubbishCultivate.weightTimes < (rate.HomeCultivateWeightR | 0)) {
                if (unit - ran <= rate.HomeCultivateFixedR - rubbishCultivate.fixedTimes + rate.HomeCultivateWeightR - rubbishCultivate.weightTimes || Utils.randomSign() > 0) {
                    rubbishCultivate.weightTimes++;
                    weightGet = true;
                    dealCultivate("HomeCultivateWeightR");
                }
            }
            if (!weightGet && rubbishCultivate.fixedTimes < (rate.HomeCultivateFixedR | 0)) {
                if (unit - ran <= rate.HomeCultivateFixedR - rubbishCultivate.fixedTimes + rate.HomeCultivateFixedR - rubbishCultivate.fixedTimes || Utils.randomSign() > 0) {
                    rubbishCultivate.fixedTimes++;
                    dealCultivate("HomeCultivateFixedR");
                }
            }
            Laya.LocalStorage.setJSON(configName, rubbishCultivate);
            function dealCultivate(config) {
                let c = Const.getArray(config);
                let fixeds = c.Fixed;
                if (fixeds) {
                    fixeds.forEach(fixed => {
                        if (fixed.id == Constants.RewardType.BOOK) {
                            book += fixed.amount;
                        } else if (fixed.id === Constants.RewardType.MEDAL) {
                            medal += fixed.amount;
                        } else if (fixed.id === Constants.RewardType.FISH) {
                            fish += fixed.amount;
                        } else if (fixed.id === Constants.RewardType.STAR) {
                            star += fixed.amount;
                        }
                    });
                }
                let weights = c.Weight;
                let totalWeight = 0;
                weights.forEach(element => {
                    element.w = element.weight;
                    totalWeight += element.weight;
                });
                if (weights) {
                    let r = Utils.calcRandomWeight(weights, totalWeight);
                    if (r.id == Constants.RewardType.BOOK) {
                        book += r.amount;
                    } else if (r.id === Constants.RewardType.MEDAL) {
                        medal += r.amount;
                    }else if (fieed.id === Constants.RewardType.FISH) {
                        fish += r.amount;
                    } else if (fieed.id === Constants.RewardType.STAR) {
                        star += r.amount;
                    }
                }
            }
            return { "MEDAL": medal, "BOOK": book, "FISH": fish, "STAR": star };
        }

    }

    class Guide14 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_14") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                let catAmount = guideCondition.CatAmount;
                let pets = PetSystem.PetIdsAdopted;
                if (pets && catAmount <= pets.length) {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        ignore(id) {
            this.ignoreId = id;
        }
        creatAnmi(){
            Laya.timer.once(600, this, this.nextStep);
        }
        nextStep() {
            super.nextStep();
            if (this.ignoreId != null) {
                if (this.currentGuide && this.currentGuide.Id == this.ignoreId) {
                    this.nextStep();
                    this.ignoreId = null;
                }
            }
            if (this.currentGuide && this.currentGuide.Id == "Guide4_HouseCat_9") {
                Constants.guid14=1;
            }
            else if (this.currentGuide && this.currentGuide.Id == "Guide4_HouseCat_10") {
                Laya.timer.frameOnce(10,this,function(){
                    EventMgr.getInstance().postEvent("GUIDE_CHECK_14");
                });
            } 
            else if (this.currentGuide && this.currentGuide.Id == "Guide4_HouseCat_14") {
                Constants.guid14=0;
            }
            else if (this.currentGuide && this.currentGuide.Id == "Guide4_HouseCat_4") {
                Laya.Dialog.closeAll();
            }

        }
        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                switch (this.currentGuide.Id) {
                    case "Guide4_HouseCat_1": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_home_enter_1"]);
                        break;
                    }
                    case "Guide4_HouseCat_5": {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_begin_cat_put_1"]);
                        break;
                    }
                }
            }
        }
    }

    class Guide15 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_15") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.Star != null && PlayerSystem.getStar() >= Number(guideCondition.Star)) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }

        start() {
            if (this._start) {
                return;
            };
            Laya.Dialog.closeAll();
            super.start();
        }

        createClickArea(key) {
            super.createClickArea(key);
            let info = this._clickMap.get(key);
            if (!info) {
                return;
            }
            this.waitingRegisterKey = null;
            let btn = info[0];
            if (this.currentGuide && this.currentGuide.Id == "Guide4_AdvancedBuilding_3" && btn && btn.selected) {
                this.nextStep();
            }
        }

        sdkSendEvent() {
            super.sdkSendEvent();
            if (this.currentGuide) {
                if(this.currentGuide.Id.substring(16)=='1'){
                let str = "begin_ADsolicit_" + this.currentGuide.Id.substring(16);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_" + str]);            
            }
            }
        }
    }

    class Guide16 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_16") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition && guideCondition.Interface == "Staff") {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }

        creatAnmi(anmi) {
            if (this.currentGuide && this.currentGuide.Id == "Guide4_Staff_3") {
                EventMgr.getInstance().postEvent("GUIDE16_OPEN_SKILL");
                this.nextStep();
            } else if (this.currentGuide && this.currentGuide.Id == "Guide4_Staff_6") {
                EventMgr.getInstance().postEvent("GUIDE16_CLOSE_SKILL");
                this.nextStep();
            }
        }

    }

    class Guide17 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_17") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition && guideCondition.Interface == "CatMission") {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }

    }

    class Guide18 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_18") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition) {
                    if (guideCondition.Star != null && PlayerSystem.getStar() >= Number(guideCondition.Star)) {
                        ifGuide = true;
                    }
                }
            }
            return ifGuide;
        }
        start() {
            Laya.Dialog.closeAll();
            SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId);
            super.start();
            EventMgr.getInstance().postEvent("GUIDEGROUP_18_START");
        }
        complete() {
            super.complete();
            EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
        }

    }

    class Guide19 extends BaseGuide {

        static check(eventString) {
            //继承，然后写各自的判断，返回是不是需要开始引导；
            let ifGuide = false;
            if (eventString == "GuideGroup_19") {
                let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", eventString);
                let guideCondition = JSON.parse(guideGroup.Condition);
                if (guideCondition && guideCondition.Interface == "OrderSystem") {
                    ifGuide = true;
                }
            }
            return ifGuide;
        }
        nextStep() {
            if (this.currentGuide && this.currentGuide.Id == "Guide4_Order_5") {
                EventMgr.getInstance().postEvent("GUIDE_COMPLETE_GUIDE", [this._guideKey]);
            }
            super.nextStep();
        }

    }

    let GuideMap = {
        'GuideGroup_1': Guide1,
        'GuideGroup_2': Guide2,
        'GuideGroup_3': Guide3,
        'GuideGroup_4': Guide4,
        'GuideGroup_5': Guide5,
        'GuideGroup_6': Guide6,
        // 'GuideGroup_7': Guide7,
        'GuideGroup_8': Guide8,
        'GuideGroup_9': Guide9,
        'GuideGroup_10': Guide10,
        'GuideGroup_11': Guide11,
        'GuideGroup_12': Guide12,
        // 'GuideGroup_13': Guide13,
        'GuideGroup_14': Guide14,
        'GuideGroup_15': Guide15,
        'GuideGroup_16': Guide16,
        'GuideGroup_17': Guide17,
        'GuideGroup_18': Guide18,
        'GuideGroup_19': Guide19
    };

    //按优先级排列
    let GuideList = [
        "GuideGroup_1",
        "GuideGroup_2",
        "GuideGroup_3",
        "GuideGroup_4",
        "GuideGroup_5",
        "GuideGroup_6",
        // "GuideGroup_7",
        "GuideGroup_8",
        "GuideGroup_9",
        "GuideGroup_10",
        "GuideGroup_11",
        "GuideGroup_12",
        // "GuideGroup_13",
        "GuideGroup_14",
        "GuideGroup_15",
        "GuideGroup_16",
        "GuideGroup_17",
        "GuideGroup_18",
        "GuideGroup_19"
    ];
    class GuideMgr {

        constructor() {
            this._guideProgress = new Map();
            this._currentGuide = null;
            this._currentGuideId = null;
            EventMgr.getInstance().registEvent("GUIDE_SET_GUIDING", this, this.setGuiding);
            EventMgr.getInstance().registEvent("GUIDE_COMPLETE_GUIDE", this, this.completeGuide);
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.commonStartGuide);
            EventMgr.getInstance().registEvent("UPDATE_STAR", this, this.starChange);
            // if (Laya.Browser.onMiniGame) {
            //     let obj = this;
            //     wx.onHide(function () {
            //         let currentGuide = obj._currentGuide;
            //         if (currentGuide) {
            //             currentGuide.backBtn_zOrder();
            //         }
            //     });
            //     wx.onShow(function () {
            //         let currentGuide = obj._currentGuide;
            //         if (currentGuide) {
            //             Laya.timer.frameOnce(1, currentGuide, currentGuide.restart);
            //         }
            //     });
            // }
        }

        static getInstance() {
            return GuideMgr.instance ? GuideMgr.instance : GuideMgr.instance = new GuideMgr();
        }

        clear() {
            this._guideProgress.clear();
            this._isGuiding = false;
            GameContext.isGuiding = false;
            if (this._currentGuide) {
                this._currentGuide.clear();
            }
            this._currentGuideId = null;
            this._currentGuide = null;
        }

        getGuide(eventString) {
            if (!this.getConditionGroupComplete(eventString)) {
                return;
            }
            for (const guideKey of GuideList) {
                if (!this.isGuideComplete(guideKey)) {
                    let guide = GuideMap[guideKey];
                    if (guide.check(eventString)) {
                        if (this._currentGuideId == guideKey) {
                            return this._currentGuide;
                        }
                        this._currentGuideId = guideKey;
                        // console.log("startGuide==================================", guideKey);
                        this._currentGuide = new guide(guideKey);
                        return this._currentGuide;
                    }
                }
            }
        }

        isGuideComplete(guideKey) {
            if (this._guideProgress.get(guideKey) == true) {
                return true;
            }
            return false;
        }

        isGuiding() {
            // return GameContext.isGuiding;
            return this._isGuiding;
        }

        setGuiding(v) {
            GameContext.isGuiding = v;
            this._isGuiding = v;
        }

        completeGuide(guideKey) {
            this._currentGuide = null;
            this._currentGuideId = null;
            // console.log("completeGuide==========================", guideKey);
            this._guideProgress.set(guideKey, true);
            EventMgr.getInstance().postEvent("SEND_GUIDE_PROGRESS");

        }

        updateProgress(progress) {
            if (progress) {
                for (const [k, v] of progress) {
                    if (k.substr(0, 10) != "GuideGroup") {
                        continue;
                    }
                    if (v == "1")
                        this._guideProgress.set(k, true);
                    else
                        this._guideProgress.delete(k, true);
                }
            }
            if (this._guideProgress.size == 0) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_enter_coffee"]);
            }
            // console.log(this._guideProgress);
            //TODO
        }

        getProgress() {
            //TODO
            let map = new Map();
            for (const [k, v] of this._guideProgress) {
                if (v) {
                    map.set(k, "1");
                } else {
                    map.set(k, "0");
                }
            }

            return map;
        }

        getCurrentGuideId() {
            return this._currentGuideId;
        }

        getConditionGroupComplete(groupId) {
            let guideGroup = ConfigMgr$1.getInstance().getItemById("GuideGroup", groupId);
            if (!guideGroup) {
                return false;
            }
            let guideCondition = JSON.parse(guideGroup.Condition);
            if (guideCondition && (guideCondition.GuideGroup == null || this.isGuideComplete(guideCondition.GuideGroup))) {
                return true;
            }
            return false;
        }

        startCheckGuide() {
            for (const guideKey of GuideList) {
                if (!this.isGuideComplete(guideKey)) {
                    Laya.timer.once(1000, this, function () {
                        if (!this._isGuiding) {
                            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", guideKey);
                        }
                    });
                }
            }
        }

        commonStartGuide(eventString, unSkip) {
            if (eventString == "GuideGroup_1") {
                let guide = this.getGuide("GuideGroup_1");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_4") {
                let guide = this.getGuide("GuideGroup_4");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_9") {
                let guide = this.getGuide("GuideGroup_9");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_10") {
                let guide = this.getGuide("GuideGroup_10");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_11") {
                let guide = this.getGuide("GuideGroup_11");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_12") {
                let guide = this.getGuide("GuideGroup_12");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_14") {
                let guide = this.getGuide("GuideGroup_14");
                if (guide) {
                    guide.start();
                    if (unSkip == null) {
                        guide.ignore("Guide4_HouseCat_3");
                    }
                }
            } else if (eventString == "GuideGroup_15") {
                let guide = this.getGuide("GuideGroup_15");
                if (guide) {
                    guide.start();
                }
            } else if (eventString == "GuideGroup_18") {
                let guide = this.getGuide("GuideGroup_18");
                if (guide) {
                    guide.start();
                }
            }
        }
        starChange() {
            if (!this.firstStar) {
                this.firstStar = true;
                return;
            }
            if (this.isGuiding()) {
                return;
            }
            if (this._lastSar == null || this._lastSar != PlayerSystem.getStar()) {
                this._lastSar = PlayerSystem.getStar();
                this.startCheckGuide();
            }
        }
    }

    class GuideSystem {
        static init() {
            EventMgr.getInstance().registEvent("SEND_GUIDE_PROGRESS", GuideSystem, GuideSystem.sendGuideProgress);
        }

        static clear() {
            GuideMgr.getInstance().clear();
        }

        static getGuide(eventString) {
            //return ; //屏蔽
            return GuideMgr.getInstance().getGuide(eventString);
        }

        static isGuiding() {
            return GuideMgr.getInstance().isGuiding();
        }

        static setGuiding(v) {
            GuideMgr.getInstance().setGuiding(v);
        }

        static completeGuide(guideKey) {
            GuideMgr.getInstance().completeGuide(guideKey);
            GuideSystem.sendGuideProgress();
        }

        static updateGuideProgress(customData) {
            // TODO 获取新手引导引导进度
            // let rid = Laya.LocalStorage.getItem("user_rid");
            // let progress = Laya.LocalStorage.getJSON("guide_" + rid);
            GuideMgr.getInstance().updateProgress(customData);
        }

        static sendGuideProgress() {
            let progress = GuideMgr.getInstance().getProgress();
            //TODO 把新手引导进度存储到本地或者发送到后端
            // let rid = Laya.LocalStorage.getItem("user_rid");
            // Laya.LocalStorage.setJSON("guide_" + rid, progress);
            let customData = Utils.map2Obj(progress);
            EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", [customData]);
        }

        static isGuideComplete(guideKey) {
            return GuideMgr.getInstance().isGuideComplete(guideKey);
        }

        static getCurrentGuideId() {
            return GuideMgr.getInstance().getCurrentGuideId();
        }

        static getConditionGroupComplete(groupId) {
            return GuideMgr.getInstance().getConditionGroupComplete(groupId);
        }

        static startCheckGuide() {
            //return ; //屏蔽
            GuideSystem.updateGuideProgress();
            GuideMgr.getInstance().startCheckGuide();
        }
    }

    class LevelToySystem{
        constructor() {

        }

        static updateData(levelToyMap){
            LevelToyDataMgr.getInstance(). updateData(levelToyMap);
        }

        static getDataById(Id){
            return LevelToyDataMgr.getInstance(). getDataById(Id);
        }
    }

    class BuildingTipLogic extends Laya.Script {

        constructor() {
            super();
            this.costText = null;
            this.letterTip = null;
        }

        onEnable() {
            //道具加成
            let items = PetSystem.getItemBySpecialType(1);
            let itemAddTimes = 0;
            let buildingData = this.owner.getData();
            items.forEach(item => {
                if (String(item.NeedBId) == buildingData.getId()) {
                    itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
                }
            });
            this.autoIncomeLimit = SyncSystem.getCacheFishCapacity() || buildingData.getSpecialValue() * (1 + itemAddTimes);
            EventMgr.getInstance().registEvent("UPDATE_LETTER_BOX", this, this.updateLetterBox);
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().registEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            EventMgr.getInstance().registEvent("FETCH_LETTER_BOX_UPDATA_LIMIT", this, this.updataAutoIncomeLimit);
        }

        onAwake() {
            this.loadLetterTip();
        }

        loadLetterTip() {
            Laya.loader.create("prefab/component/letter_box_tip.prefab", Laya.Handler.create(this, function (prefabDef) {
                this.letterTip = prefabDef.create();
                this.letterTip.y=15;
                this.letterTip.on(Laya.Event.CLICK, this, this.onClickTip);
                this.owner.prefab.addChild(this.letterTip);
                this.aniBox = this.letterTip.getChildByName("aniBox");
                this.costText = this.aniBox.getChildByName("cost_text");
                this.letterTip.ani1.play(0, true);
                let xiaoFeiText= this.aniBox.getChildByName("xiaofei");
                xiaoFeiText.text = Strings$1.get("tid#UI_001");
                this.costText.x = xiaoFeiText.x + xiaoFeiText.textWidth;
                this.updateLetterBox();
            }));
        }

        onClickTip() {
            if(GameContext.Sliding)
                return;
            let cacheFish =  SyncSystem.getCacheFish();
            if(cacheFish > 0) {
                Laya.Dialog.close("ui/FortuneCatUI.scene");
                Laya.Dialog.open("ui/FortuneCatUI.scene", true);
            }
        }


        onAddCatTip() {
            let sum = BuildingSystem.getAutoIncomeSum();
            let cacheFish =  SyncSystem.getCacheFish();
            // console.debug("onAddCatTip", cacheFish, sum);
            let cost = Number(cacheFish) + Number(sum);
            if(cost >= this.autoIncomeLimit) {
                cost = this.autoIncomeLimit;
            }
            SyncSystem.setCacheFish(cost);
            if (this.costText) {
                this.costText.text = Strings$1.dealFishNumberText(cost);
                if (GuideSystem.isGuiding() && GuideSystem.getCurrentGuideId() != "GuideGroup_7") {
                    this.letterTip.visible = false;
                } else {
                    this.letterTip.visible = true;
                    this.letterTip.ani1.play(0, true);
                }
            }
            SyncSystem.getLetterBoxInfo();
            let interval = SyncSystem.getLetterBoxNextUpdateInterval()+500;
            while(interval<=5000) {
                interval = interval + Constants.LetterBoxInterval;
            }
            Laya.timer.once(interval, this, this.onAddCatTip, [], true);
        }

        updateLetterBox() {
            let cacheFish = SyncSystem.getCacheFish();
            if (this.costText) {
                this.costText.text = Strings$1.dealFishNumberText(cacheFish);
                if (cacheFish > 0) {
                    if (GuideSystem.isGuiding() && GuideSystem.getCurrentGuideId() != "GuideGroup_7") {
                        this.letterTip.visible = false;
                    } else {
                        this.letterTip.visible = true;
                        this.letterTip.ani1.play(0, true);
                    }
                } else {
                    this.letterTip.visible = false;
                    this.letterTip.ani1.stop();
                }
            }
            let interval = SyncSystem.getLetterBoxNextUpdateInterval()+500;
            while(interval<=0) {
                interval = interval + Constants.LetterBoxInterval;
            }
            Laya.timer.once(interval, this, this.onAddCatTip, [], true);
        }

        onDisable() {
            if (this.letterTip && this.letterTip.ani1) {
                this.letterTip.ani1.stop();
            }
            EventMgr.getInstance().removeEvent("UPDATE_LETTER_BOX", this, this.updateLetterBox);
            EventMgr.getInstance().removeEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("FETCH_LETTER_BOX_UPDATA_LIMIT", this, this.updataAutoIncomeLimit);
            Laya.timer.clear(this, this.onAddCatTip);
        }

        onDestroy(){
            if (this.letterTip && this.letterTip.ani1) {
                this.letterTip.ani1.stop();
            }
            EventMgr.getInstance().removeEvent("UPDATE_LETTER_BOX", this, this.updateLetterBox);
            EventMgr.getInstance().removeEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("FETCH_LETTER_BOX_UPDATA_LIMIT", this, this.updataAutoIncomeLimit);
            Laya.timer.clear(this, this.onAddCatTip);
        }

        checkGuide(eventString){
            if (eventString == "GuideGroup_7") {
                let guide = GuideSystem.getGuide("GuideGroup_7");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.letterTip);
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", this.letterTip);
                    guide.start();
                    this.updateLetterBox();
                }
            }
        }

        cameraSwitch(type) {
            if (type) {
                this.letterTip.visible = false;
                this.letterTip.ani1.stop();
            } else {
                this.updateLetterBox();
            }
        }

        updataAutoIncomeLimit() {
            //道具加成
            let items = PetSystem.getItemBySpecialType(1);
            let itemAddTimes = 0;
            let buildingData = this.owner.getData();
            items.forEach(item => {
                if (String(item.NeedBId) == buildingData.getId()) {
                    itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
                }
            });
            this.autoIncomeLimit = SyncSystem.getCacheFishCapacity() || buildingData.getSpecialValue() * (1 + itemAddTimes);
            Laya.timer.clear(this, this.onAddCatTip);
            this.onAddCatTip();
        }
    }

    class NauticalSystem$1 {
        static init() {
        }

        static clear() {
            NauticalDataMgr.getInstance().clear();
        }

        static updateAll(nauticalInfo, petIds, nauticalFoods, nauticalTools,nauticalTicketNum) {
            let mgr = NauticalDataMgr.getInstance();
            if(nauticalInfo){
                mgr.updateNauticalInfo(nauticalInfo);
                this.updateBtnList(nauticalInfo);
            }
            if(petIds){
                mgr.updateAvailablePets(petIds);
            }
            if(nauticalFoods){
                mgr.updateNauticalFoods(nauticalFoods);
            }
            if(nauticalTools){
                mgr.updateNauticalTools(nauticalTools);
            }
            if (nauticalTicketNum) {
                mgr.updateNauticalTicketNum(nauticalTicketNum);
            }
        }

        static updateNauticalFoods(nauticalFoods) {
            let mgr = NauticalDataMgr.getInstance();
            mgr.updateNauticalFoods(nauticalFoods);
            EventMgr.getInstance().postEvent("UPDATA_NAUTICAL_FOODS");
        }

        static updateNauticalTools(nauticalTools) {
            let mgr = NauticalDataMgr.getInstance();
            mgr.updateNauticalTools(nauticalTools);
            EventMgr.getInstance().postEvent("UPDATA_NAUTICAL_TOOLS");
        }


        static updateNauticalInfo(nauticalInfo) {
            NauticalDataMgr.getInstance().updateNauticalInfo(nauticalInfo);
            this.updateBtnList(nauticalInfo);
            if(nauticalInfo.inTrip) {
                PetDataMgr.getInstance().setPetsInNautical(nauticalInfo.team);
            }
        }

        static updateOnlyNauticalInfo(sailIndex,nauticalInfo) {
            NauticalDataMgr.getInstance().updateOnlyNauticalInfo(sailIndex,nauticalInfo);
            if(nauticalInfo.inTrip) {
                PetDataMgr.getInstance().setPetsInNautical(nauticalInfo.team);
            }
        }

        static updateBtnList(nauticalInfo){
            let btnList = new Array();
            let nauticalNum = nauticalInfo.length;
            btnList.length = 0;
            for(let i = 0; i< nauticalNum; i++){
                if(BuildingDataMgr.getInstance().hasBuildingDataByBid(3010 + i)){
                    let obj = {
                        "BuildingId" :3010 + i,
                        "nauticalIndex" : i,
                        "nauticalInfo" : this.NauticalInfo[i]
                    };
                    btnList.push(obj);
                }
            }
            NauticalDataMgr.getInstance().updateBtnList(btnList);
        }

        static updateBtnListAfterBuy(){
            let nauticalInfo = this.NauticalInfo;
            this.updateBtnList(nauticalInfo);
        }

        static get NauticalInfo() {
            return NauticalDataMgr.getInstance().getNauticalInfo();
        }

        static get Pets() {
            return NauticalDataMgr.getInstance().getAvailablePets();
        }

        static get Foods() {
            return NauticalDataMgr.getInstance().getFoods();
        }

        static get Tools() {
            return NauticalDataMgr.getInstance().getTools();
        }

        static get btnList() {
            return NauticalDataMgr.getInstance().getBtnList();
        }

        static get PetsLimit() {
            let limit = new Array();
            let buildings = BuildingDataMgr.getInstance().getBuildingsBySpecialType(4);
            for(const buildingData of buildings) {
                limit.push(Number(buildingData.getSpecialValue()));
            }
            return limit;
        }

        static getFoodById(foodId) {
            return NauticalDataMgr.getInstance().getFood(foodId);
        }
      
        static getToolById(toolId) {
            return NauticalDataMgr.getInstance().getTool(toolId);
        }

        static getBuildingDataByPid(pid) {
            let location = LocationDataMgr.getInstance().getLocation(Number(pid));
            if(location == null) {
                return null;
            }
            let buildingId = location.getBuildingId();
            if(buildingId == null) {
                return null;
            }
            let building = BuildingDataMgr.getInstance().getBuildingDataByBid(buildingId);
            return building;
        }

        static getBoatCanSail(style){// 0 最高匹配  1 当前船 ////// return -1 没船 -2 出航了 -3 都没解锁
            let nowId = this.getBuildingDataByPid(302) ? this.getBuildingDataByPid(302).config.Id : 0;
            if(style){
                if(nowId){
                    return nowId - 3010;
                }
                else {
                    return -1;
                }
            }
            if(this.btnList.length <= 0){
                return -1;
            }
            let inTrip = 0;
            for(let i = this.btnList.length - 1 ; i >= 0 ; i--){
                if(this.btnList[i].nauticalInfo.adLock == 0 ){
                    if(!this.btnList[i].nauticalInfo.inTrip){
                        return this.btnList[i].nauticalIndex;
                    }
                    else {
                        inTrip = 1;
                    }
                }
            }
            if(inTrip){
                return -2;
            }
            else {
                return -3;
            }
        }

        static clearNauticalInfo(tripIndex) {
            NauticalDataMgr.getInstance().clearNauticalInfo(tripIndex);
        }

        static checkFood(nauticalInfo) {
            let foodId1 = nauticalInfo.food1;
            let foodId2 = nauticalInfo.food2;
            let food1 = NauticalSystem$1.getFoodById(foodId1);
            let food2 = NauticalSystem$1.getFoodById(foodId2);
            if(food1&&food1.count==0)
                nauticalInfo.food1 = null;
            if(food2)
                if(food2.count==0)
                    nauticalInfo.food2 = null;
                else if(foodId1==foodId2 && food2.count==1)
                    nauticalInfo.food2 = null;
        }

        static checkTool(nauticalInfo) {
            let tool = NauticalSystem$1.getToolById(nauticalInfo.tool);
            if(tool&&tool.count==0)
                nauticalInfo.tool = null;
        }

        static checkPets(nauticalInfo) {
            let pets = nauticalInfo.team;
            let availablePets = NauticalSystem$1.Pets;
            for(let i = pets.length - 1; i>=0;--i) {
                let petId = pets[i];
                let needRemove = true;
                for(const value of availablePets) {
                    if(petId==value) {
                        needRemove = false;
                        break;
                    }
                }
                if(needRemove)
                    pets.splice(i,1);
            }
            for(let i = 0;i<this.btnList.length;i++){
                if(this.btnList[i].nauticalIndex == this.nauticalIndex){
                    var limit = this.PetsLimit[i];
                    break;
                }
            }
            if(pets.length>limit)
                pets.splice(pets.length, pets.length - limit);

        }

        static updateNauticalTicketNum(v) {
            NauticalDataMgr.getInstance().updateNauticalTicketNum(v);
        }

        static getNauticalTicketNum() {
            return NauticalDataMgr.getInstance().getNauticalTicketNum();
        }

        static addTicket(v) {
            NauticalSystem$1.updateNauticalTicketNum(NauticalSystem$1.getNauticalTicketNum() + v);
        }

        //===============Service===============
        static getNauticalInfoReq(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_NAUTICAL_INFO", [handler]);
        }

        static mergeNauticalFood(nauticalFoodId, nauticalFoodCount, handler) {
            EventMgr.getInstance().postEvent("SERVICE_MERGE_NAUTICAL_FOOD", [nauticalFoodId, nauticalFoodCount, handler]);
        }

        static startNauticalTrip(sailIndex,handler) {
            let nauticalInfo = NauticalSystem$1.NauticalInfo[sailIndex];
            NauticalSystem$1.checkFood(nauticalInfo);
            NauticalSystem$1.checkPets(nauticalInfo);
            NauticalSystem$1.checkTool(nauticalInfo);
            let info = nauticalInfo.dumpStartReqInfo(sailIndex);
            let dealFunc = function(res) {
                NauticalDataMgr.getInstance().updateUsage(nauticalInfo);
                if(sailIndex == NauticalSystem$1.getBoatCanSail(1)){
                    EventMgr.getInstance().postEvent("BOAT_START_TRIP");
                }
                if(handler)
                    handler.runWith([res]);
            };
            EventMgr.getInstance().postEvent("SERVICE_START_NAUTICAL_TRIP", [sailIndex,info, Laya.Handler.create(null, dealFunc)]);
        }

        static finishNauticalTrip(type,tripIndex,handler) {
            EventMgr.getInstance().postEvent("SERVICE_FINISH_NAUTICAL_TRIP", [type,tripIndex,handler]);
            SyncSystem.startOperatingUpdate();
        }

        static previewNauticalTripReq(tripIndex,handler) {
            EventMgr.getInstance().postEvent("SERVICE_PREVIEW_NAUTICAL_TRIPREQ", [tripIndex,handler]);
            SyncSystem.startOperatingUpdate();
        }

        static unlockNautical(tripIndex ,handler) {
            EventMgr.getInstance().postEvent("SERVICE_UNLOCK_NAUTICAL", [tripIndex,handler]);
        }

        static getFoodCanMergeInfo(foodId) {
            return NauticalDataMgr.getInstance().getFoodCanMergeInfo(foodId);
        }

        static getAdReduceTimes(nauticalIndex) {
            return NauticalDataMgr.getInstance().getAdReduceTimes(nauticalIndex);
        }

    }

    class NauticalBoatLogic extends Laya.Script {

        constructor() {
            super();
        }

        onEnable() {
            Laya.timer.clear(this, this._checkTime);
            this.boat = this.owner.prefab.getChildByName("_spineAnim_0");
            this.animUI = this.owner.prefab.getChildByName("_spineAnim_1");
            this.animUI.scale(1.3,1.3);
            this.animUI.pos(160,200);
            this.animX = this.animUI.x;
            this.animY = this.animUI.y;
            Laya.loader.create("prefab/ui/time.prefab",Laya.Handler.create(this, this._onTimeLoaded));
            if(!this.sailBtnPrefab){
                Laya.loader.create("prefab/ui/sailBtn.prefab",Laya.Handler.create(this, this._onSailBtnLoaded));
            }
            this.owner.prefab.getChildByName("touchFrame").on(Laya.Event.CLICK, this, this.onClickUI);
            EventMgr.getInstance().registEvent("BOAT_START_TRIP", this, this._startTrip);
            this.nauticalIndex = NauticalSystem$1.getBoatCanSail(1);
            this._nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            EventMgr.getInstance().registEvent("BOAT_UPDATE_INFO", this, this._updateNauticalInfo);
            EventMgr.getInstance().registEvent("BOAT_GET_REWARD", this, this.getReward);
            EventMgr.getInstance().registEvent("BOAT_AD_CHECK_TIME", this, this.adCheckTime);
            this._updateNauticalInfo();
            TaskMainUIMgr.getInstance().registButton("Catfood", this.owner.prefab.getChildByName("Sail"));
            this.owner.mouseEnabled = true;
        }

        _updateNauticalInfo() {
            if(!this._nauticalInfo.inTrip) {
                this._idle();
            }else {
                if(TimeUtils.getTimeNowOnServer()>this._nauticalInfo.endSec*1000+500) {
                    this._endTrip();
                }else {
                    this._startTrip();
                }
            }
        }

        hideAnim() {
            this.animUI.visible = false;
        }

        _resumeAnim() {
            this.animUI.visible = true;
        }

        _startTrip() {
            Laya.timer.clear(this, this._checkTime);
            this.boat.visible = false;
            this.animUI.visible = true;
            this.animUI.playAnim("paizi");
            this.donghuazhanshi(false);
            // Utils.fadein(this.animUI, 500);
            Laya.timer.once(this._nauticalInfo.endSec*1000-TimeUtils.getTimeNowOnServer()+500, this, this._checkTime, [], true);
        }

        _onTimeLoaded(prefabDef){
            if(this.timePrefab){
                this.timePrefab.destroy();
            }
            this.timePrefab = prefabDef.create();
            this.animUI.addChild(this.timePrefab);
            this.timePrefab.x = -25;
            this.timePrefab.y = -125;
            this.timePrefab.visible = false;
            Laya.timer.once(10,this,function(){
                this.showRemainingTime();
            });
        }

        _onSailBtnLoaded(prefabDef){
            this.sailBtnPrefab = prefabDef.create();
            this.owner.addChild(this.sailBtnPrefab);
            this.sailBtnPrefab.x = -100;
            this.sailBtnPrefab.y = -300;
        }

        showRemainingTime(){
            this.showTime();
            Laya.timer.loop(1000,this,function(){
                this.showTime();
            });    
        }

        showTime(){
            let str = "";
            this.endTime = this._nauticalInfo.endSec * 1000 + 500;
            this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
            this.remainingTime = this.leftTime / 1000 | 0;
            if(this.remainingTime > 0){
                this.timePrefab.visible = true;
                let minute = this.remainingTime / 60 | 0;
                let second = this.remainingTime % 60;
                if(minute<10){
                    str += "0";
                }
                str += minute + ":";
                if(second<10){
                    str += "0";
                }
                str += second;
            }else {
                this.timePrefab.visible = false;
            }
            if(str!=""){
                this.timePrefab.value = str;
            }
        }

        _checkTime() {
            if(this._nauticalInfo.inTrip) {
                if(TimeUtils.getTimeNowOnServer()>this._nauticalInfo.endSec*1000) {
                    this._endTrip();
                    EventMgr.getInstance().postEvent("left_RED_POINT_SHOW",[true]);
                }else {
                    Laya.timer.once(1000, this, this._checkTime, null, true);
                }
            }else {
                this._idle();
            }
        }

        _endTrip() {
            Laya.timer.clear(this, this._checkTime);
            this.boat.visible = true;
            this.animUI.visible = false;
            // this.donghuazhanshi(false);
            //Utils.fadein(this.animUI, 500);
        }

        _idle() {
            this._rewarded = false;
            this.boat.visible = true;
            // Utils.fadein(this.animUI, 500);
            this.animUI.stopAnim();
            // this.donghuazhanshi(true);
         
        }

        onClickUI() {
            if(GameContext.Sliding)
                return;
            // this.hideAnim();
            if(this._nauticalInfo.adLock > 0){
                Laya.Dialog.close("ui/SailDialog.scene");
                Laya.Dialog.open("ui/SailDialog.scene",true,this.nauticalIndex);
            }
            else if(!this._nauticalInfo.inTrip) {
                Laya.Dialog.close("ui/NauticalStartPop");
                Laya.Dialog.open("ui/NauticalStartPop.scene", false,this.nauticalIndex);
            }else {
                if(TimeUtils.getTimeNowOnServer()>this._nauticalInfo.endSec*1000+500) {
                    if(!this._rewarded) {
                        this._rewarded = true;
                        NauticalSystem$1.previewNauticalTripReq(this.nauticalIndex,Laya.Handler.create(this, this._showReward));
                    }
                }else {
                    Laya.Dialog.close("ui/NauticalTravelingPop.scene");
                    Laya.Dialog.open("ui/NauticalTravelingPop.scene", false,["",this.nauticalIndex]);
                }
            }
        }

        _showReward(res,nauticalIndex) {
            Laya.Dialog.close("ui/NauticalRewardPop.scene");
            Laya.Dialog.open("ui/NauticalRewardPop.scene", false,[res,nauticalIndex]);
        }

        onDisable() {
            this._rewarded = null;
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("BOAT_START_TRIP", this, this._startTrip);
            EventMgr.getInstance().removeEvent("BOAT_UPDATE_INFO", this, this._updateNauticalInfo);
            EventMgr.getInstance().removeEvent("BOAT_GET_REWARD", this, this.getReward);
            EventMgr.getInstance().removeEvent("BOAT_AD_CHECK_TIME", this, this.adCheckTime);
        }

        getReward(type,tripIndex,handler){
            this.owner.mouseEnabled = false;
            this.serverGetReward(type,tripIndex,handler);
            if(tripIndex == this.nauticalIndex){
                this._idle();
            }
        }

        serverGetReward(type,tripIndex,handler) {
            NauticalSystem$1.finishNauticalTrip(type,tripIndex,handler);
            this.owner.mouseEnabled = true;
        }

        adCheckTime(){
            this._nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            Laya.timer.clear(this, this._checkTime);
            if(this._nauticalInfo.inTrip) {
                this.boat.visible = false;
                if (TimeUtils.getTimeNowOnServer() > this._nauticalInfo.endSec * 1000) {
                    this._endTrip();
                } else {
                    Laya.timer.once(this._nauticalInfo.endSec*1000-TimeUtils.getTimeNowOnServer()+500, this, this._checkTime, [], true);
                }
            } else {
                this._endTrip();
            }
        }
        //因出航前展示动画修改 但 出航中 和出航后 仍用到原动画 所以 分开展示
        donghuazhanshi(chuhangqian){
            if(chuhangqian==true){
                this.animUI.visible = false;
            }else {     
                this.animUI.visible = true;
            }
        }  
    }

    class BuildingFishingTip extends Laya.Sprite {

        constructor() {
            super();
            this.tipAnim = new Spine();
            this.tipAnim.loadAnim("anim3/tanhaotixing.sk");
            this.addChild(this.tipAnim);
            this.tipAnim.scaleX = this.tipAnim.scaleY = 1.3;
            this.tipAnim.x = 10;
            this.tipAnim.y = -30;
        }

        onEnable() {
            EventMgr.getInstance().registEvent("UPDATE_FISHING_TIP", this, this.updateTip);
            this.tipAnim.playAnim("tanhaotixing", true);
            this.updateTip();
            Laya.timer.clear(this, this.updateServerBaseInfo);
            let nextUpdateSecs = FishingSystem.getNextUpdateSecs();
            if (nextUpdateSecs > 0) {
                Laya.timer.once(nextUpdateSecs * 1000, this, this.updateServerBaseInfo);
            } 
        }

        onAwake() {
            this._loadComplete = false;
            this.x = 770;
            this.y = 890;
            this.visible = false;
        }


        onDisable() {
            this.tipAnim.stopAnim();
            EventMgr.getInstance().removeEvent("UPDATE_FISHING_TIP", this, this.updateTip);
        }

        onDestroy() {
            this.tipAnim.destroy();
            Laya.timer.clear(this,this.updateServerBaseInfo);
            EventMgr.getInstance().removeEvent("UPDATE_FISHING_TIP", this, this.updateTip);
        }

        updateTip() {
            if (FishingSystem.getRemainTimes() > 0) {
                this.visible = true;
                this.tipAnim.playAnim("tanhaotixing", true);
            } else {
                this.visible = false;
                this.tipAnim.stopAnim();
            }
        }

        updateServerBaseInfo(){
            FishingSystem.updateServerBaseInfo();
        }
    }

    class BuildingFactory {

        constructor() {
        }

        static getInstance() {
            return BuildingFactory.instance?BuildingFactory.instance:BuildingFactory.instance = new BuildingFactory();
        }

        createBuilding(buildingData, parent, scale, completeHandler) {
            var building = new Building$1();
            building.setData(buildingData);
            if (Number(buildingData.getId()) >= 2010 && Number(buildingData.getId() < 2030)) {
                scale = null;
            }
            building.buildingScale = scale ? scale : 1;
            building.loadResource(buildingData.getSource(), function() {
                parent.addChild(building);
                let locationData = buildingData.getLocationData();
                building.pos(Number(locationData.getX()), Number(locationData.getY()));
                building.zOrder = Math.floor(building.y) + Number(locationData.getZOrder());
                let sceneId = buildingData.getBelongScene();
                if(sceneId == Constants.CoffeeSceneId) {
                } else if (sceneId == Constants.BeachSceneId) {
                    // building.addComponent(BeachAttractCatLogic);
                }
                let specialType = buildingData.getSpecialType();
                if(specialType == 1) {
                    building.addComponent(BuildingTipLogic);
                }else if(specialType == 4) {
                    building.addComponent(NauticalBoatLogic);
                }
                if(completeHandler != null) {
                    completeHandler.runWith(building);
                }
                if (Number(buildingData.getId()) >= 2020 && Number(buildingData.getId() < 2030)) {
                    let buildingFishingTip = new BuildingFishingTip();
                    building.addChild(buildingFishingTip);
                }else if(Number(buildingData.getId()) >= 3010 && Number(buildingData.getId() < 3020)){
                    EventMgr.getInstance().postEvent("left_RED_POINT_SHOW",[true]);
                }
                Utils.fadein(building, 1000);
            });
            return building;
        }

        static createBuildingBaseSelector(location, parent) {
            let sprite = new Laya.Sprite();
            sprite.loadImage("facility/jianzhu"+(location.getBaseSize()+1)+".png");
            sprite.scaleX = sprite.scaleY = Number(location.getBaseSize());
            sprite.mouseEnabled = true;
            parent.addChild(sprite);
            sprite.pos(Number(location.getSelectionX()), Number(location.getSelectionY()));
            sprite.zOrder = Math.floor(sprite.y) + Number(location.getZOrder());
            return sprite;
        }
    }

    /**
     * 本示例采用非脚本的方式实现，而使用继承页面基类，实现页面逻辑。在IDE里面设置场景的Runtime属性即可和场景进行关联
     * 相比脚本方式，继承式页面类，可以直接使用页面定义的属性（通过IDE内var属性定义），比如this.tipLbll，this.scoreLbl，具有代码提示效果
     * 建议：如果是页面级的逻辑，需要频繁访问页面内多个元素，使用继承式写法，如果是独立小模块，功能单一，建议用脚本方式实现，比如子弹脚本。
     */

    class BeachScene extends Laya.Scene {
        constructor() {
            super();
            this.sceneId = Constants.BeachSceneId;//2
            this.buildingMap = new Map();
        }

        onAwake() {
            this.root = this.getChildByName("root");
            EventMgr.getInstance().registEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().registEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().registEvent("PLACE_BUILDING", this, this.placeBuilding);
            EventMgr.getInstance().registEvent("CHOOSE_PLACE_BUILDING", this, this.onChooseLayerClick);
            this.onBg();
            this.onInitBuilding();
            this.chooseLayer.on(Laya.Event.CLICK,this,this.onChooseLayerClick);
            if(GameContext.PlacingBuilding) {
                this.placeBuilding(GameContext.PlacingBuilding);
            }
        }


        onDestroy() {
            EventMgr.getInstance().removeEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().removeEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().removeEvent("PLACE_BUILDING", this, this.placeBuilding);
            EventMgr.getInstance().removeEvent("CHOOSE_PLACE_BUILDING", this, this.onChooseLayerClick);
            if(this.jianzao) {
                this.jianzao.destroy();
                this.jianzao = null;
            }
        }

        onBg() {
            if(!this.bgAnim) {
                this.bgAnim = new Laya.Skeleton();
                setTimeout(() => {
                    this.bgAnim.load("anim1/hailangdonghua.sk", Laya.Handler.create(this, function() {
                        this.root.addChild(this.bgAnim);
                        this.bgAnim.scaleX = 1.2;
                        this.bgAnim.scaleY = 1.2;
                        this.bgAnim.x = 890;
                        this.bgAnim.y = 1075;
                        this._bgAnimReady = true;
                        SoundManager.getInstance().stopById("Sound_beachWave");
                        SoundManager.getInstance().play("Sound_beachWave", 1);
                        this.bgAnim.play("sea", false, true);
                        this.bgAnim.on(Laya.Event.STOPPED, this, function () {
                            SoundManager.getInstance().play("Sound_beachWave", 1);
                            this.bgAnim.play("sea", false, true);
                        });
                    }));
                }, 100);
            }
        }

        onInitBuilding() {
            let buildingList = BuildingSystem.getBuildingDataListInfo(this.sceneId).list;
            for(var i = 0 ; i < buildingList.length ; ++i) {
                let data = buildingList[i];
                let buildingId = data.getId();
                if(this.buildingMap.get(buildingId) == null && data.getLocationData() != null) {
                    // console.debug("onInitBuilding", buildingId)
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root, Constants.BeachBuildingScale, Laya.Handler.create(this, this._buildingLoaded));
                    this.buildingMap.set(buildingId, building);
                }
            }
        }

        _buildingLoaded(building) {
            let buildingId = building.getId();
            let cats = BeachCatMgr.getInstance().idleCatsInBuilding(buildingId);
            for(const cat of cats) {
                let spaceId = cat.spaceId;
                this.addCatToBuilding(cat, buildingId, spaceId);
            }
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", "GuideGroup_12");
        }

        addCatToBuilding(cat, buildingId, spaceId) {
            let buildingExtInfo = this.getSpaceInfo(buildingId, spaceId);
            if(buildingExtInfo != null) {
                if(cat.parent==buildingExtInfo.getBuildingRoot())
                    return;
                buildingExtInfo.addToBuildingRoot(cat);
                cat.x = buildingExtInfo.getBuildingRootHalfWidth();
                cat.y = buildingExtInfo.getBuildingRootHalfHeight();
                let actionSetting = buildingExtInfo.randomCatMove();
                if (actionSetting.flip) {
                    cat.scaleX = -1 * Math.abs(cat.scaleX);
                } else {
                    cat.scaleX = Math.abs(cat.scaleX);
                }
                Utils.fadein(cat, 500, Laya.Handler.create(this, function() {
                    cat.alpha = 1;
                    cat.playAnim(actionSetting.name);
                    if (actionSetting.hide) {
                        buildingExtInfo.hideOthers();
                    }
                }));
            }
        }

        initCats() {
            let cats = BeachCatMgr.getInstance().allCats();
            for(const cat of cats) {
                if(cat.buildingId == null || cat.buildingId == undefined) {
                }else {
                    this.addCatToBuilding(cat, cat.buildingId, cat.spaceId);
                }
            }
        }

        onOpened() {
            this.initCats();
            BeachCatMgr.getInstance().clearRemoved();
            if (this._bgAnimReady) {
                this.bgAnim.play("sea", false);
                SoundManager.getInstance().stopById("Sound_beachWave");
                SoundManager.getInstance().play("Sound_beachWave", 1);
            }
        }

        onClosed() {
            if (this._bgAnimReady) {
                this.bgAnim.paused();
            }
            SoundManager.getInstance().stopById("Sound_beachWave");
        }

        addBuilding(data, isNew) {
            if (data && data.getLocationData && data.getBelongScene) {
                if (data.getBelongScene() == this.sceneId) {
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root, Constants.CoffeeBuildingScale, Laya.Handler.create(this, this._onBuildingAdded));
                    this.buildingMap.set(data.getId(), building);

                    if (this.jianzao && !this.jianzao.destroyed) {
                        this.jianzao.destroy();
                    }
                    this.jianzao = new Spine();
                    this.root.addChild(this.jianzao);
                    let locationData = data.getLocationData();
                    let scale = Number(locationData.getBaseSize());
                    if (isNew) {
                        this.jianzao.loadAnim("anim1/jianzao1.sk");
                        this.jianzao.playAnim("jianzao1", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale * 2;
                    } else {
                        this.jianzao.loadAnim("anim3/jianzao.sk");
                        this.jianzao.playAnim("yanwu", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale / 2;
                    }
                    this.jianzao.visible = true;
                    this.jianzao.addCompleteHandler(Laya.Handler.create(this, function () {
                        this.jianzao.destroy();
                        this.jianzao = null;
                    }));
                    this.jianzao.pos(Number(locationData.getSelectionX()) + 152 * scale, Number(locationData.getSelectionY()) + 80 * scale);
                    this.jianzao.zOrder = Math.floor(this.jianzao.y) + Number(locationData.getZOrder());
                }
            }
        }

        removeBuilding(data) {
            if(data && data.getBelongScene && data.getBelongScene()==this.sceneId){
                if(this.buildingMap.has(data.getId())){
                    let building = this.buildingMap.get(data.getId());
                    building.destroy();
                    this.buildingMap.delete(data.getId());
                }
            }
        }

        placeBuilding(buildingId) {
            let locationList = BuildingSystem.getLocationsByBaseType(2);
            let list = new Array();
            locationList.forEach(element=>{
                if(element.getBelongScene()==this.sceneId && element.checkBuildingAvailable(buildingId)){
                    list.push(element);
                }
            });
            if(list.length>0){
                GameContext.PlacingBuilding = null;
                this.showLocationSelect(list, buildingId);
            }
        }

        getBuilding(id) {
            for (var [key, value] of this.buildingMap) {
                if(Number(key) == Number(id)) {
                    if(value.isLoaded()) {
                        return value;
                    }
                    return null;
                }
            }
            return null;
        }

        getSpaceInfo(buildingId, spaceId) {
            let building = this.getBuilding(buildingId);
            if(building){
                return building.getBuildingExtInfoMgr().getBuildingExtInfo(spaceId);
            }

        }

        showLocationSelect(locationList, buildingId) {
            this.chooseLayer.removeChildren();
            this.chooseLayer.visible = true;
            let firstSelector=null;
            locationList.forEach(element=>{
                let selector = BuildingFactory.createBuildingBaseSelector(element, this.chooseLayer, Constants.BeachBuildingScale);
                selector.baseId = element.getId();
                selector.buildingId = buildingId;
                selector.on(Laya.Event.CLICK,this,this.onClick);
                if (!firstSelector) {
                    firstSelector = selector;
                }
            });
            if (Number(buildingId) == 2080) {
                let guide = GuideSystem.getGuide("GuideGroup_8");
                if (guide) {
                    guide.registerObj("btn4", this);
                    let rect = new Laya.Rectangle(firstSelector.x, firstSelector.y, 301, 161);
                    guide.registerClick("btn4", firstSelector, rect);
                }
            }else if(Number(buildingId) == 2020){
                let guide = GuideSystem.getGuide("GuideGroup_11");
                if (guide) {
                    let rect = new Laya.Rectangle(firstSelector.x, firstSelector.y, 602, 322);
                    guide.registerObj("btn4", this);
                    guide.registerClick("btn4", firstSelector, rect);
                }
            }
        }

        onClick(event) {
            let buildingId = event.currentTarget.buildingId;
            let baseId = event.currentTarget.baseId;
            BuildingSystem.placeBuilding(buildingId, baseId, GameContext.PlacingNewBuilding);
            GameContext.PlacingNewBuilding = null;
            this.chooseLayer.visible = false;
            SoundManager.getInstance().play("Sound_placeBuilding");
        }

        onChooseLayerClick(){
            BuildingSystem.checkShowBuildingCouponPop();
            this.chooseLayer.visible = false;
        }

        lockTypeChange(lock) {
            if (lock != null && lock) {
                EventMgr.getInstance().postEvent("BOWL_SET_VISIBLE", [0]);
                if (this.lockBox) {
                    this.lockBox.visible = true;
                    this.mouseEnabled = false;
                } else {
                    this.lockBox = this.getChildByName("SceneLock");
                    let mid = new Laya.Point(Laya.stage.width / 2, 0);
                    mid = this.globalToLocal(mid);
                    this.lockBox.x = mid.x;
                    mid = null;
                    this.lockNum = this.lockBox.getChildByName("lockNum");
                    this.get=this.lockBox.getChildByName("get");
                    this.get.text=Strings$1.get('tid#UI_170');
                    let item = ConfigMgr$1.getInstance().getItemById("Scene", Constants.BeachSceneId.toString());
                    if (item) {
                        let unlockStar = JSON.parse(item.Unlock);
                        this.lockNum.text = "" + unlockStar[0].Star;
                    }
                    this.lockBox.visible = true;
                    this.mouseEnabled = false;
                }
            } else {
                EventMgr.getInstance().postEvent("BOWL_SET_VISIBLE", [1]);
                this.mouseEnabled = true;
                if (this.lockBox) {
                    this.lockBox.visible = false;
                    this.lockBox.removeSelf();
                    this.lockBox.destroy();
                }
            }
        }

    }

    class WxConfigMgr {

        constructor() {
            if (Laya.Browser.onMiniGame) {
                this._platform = "prod";
            }else if((Laya.Browser.onAndroid||Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone)&&Constants.shouji){
                this._platform = "pp";
            }
            else {
                this._platform = "dev";
            }
            this._compressed = false;
        }

        static getInstance() {
            return WxConfigMgr.instance ? WxConfigMgr.instance : WxConfigMgr.instance = new WxConfigMgr();
        }

        set compressed(compressed) {
            this._compressed = compressed;
        }

        get compressed() {
            return this._compressed;
        }

        getAppid() {
            return WxConfigMgr.GlobalUrl[this._platform]['appid'];
        }

        getAppVersion() {
            return WxConfigMgr.GlobalUrl[this._platform]['version'];
        }

        getAdUnitId() {
            if(this._platform=='pp'){
      
            if(PLAY68_SDK.getPlatform() == 'ios') {
                return WxConfigMgr.GlobalUrl[this._platform]['adUnitIdIos'];
            }else {
                return WxConfigMgr.GlobalUrl[this._platform]['adUnitIdAndroid'];
            }   
                      
        }
        else {
            return WxConfigMgr.GlobalUrl[this._platform]['adUnitId'];
        }
        }

        getBannerId() {
            return WxConfigMgr.GlobalUrl[this._platform]['bannerId'];
        }

        getPlatform() {
            return this._platform;
        }

        setPlatform(platform) {
            if(WxConfigMgr.GlobalUrl[platform] == null) {
                console.error(".... platform error: " + platform);
                return;
            }
            this._platform = platform;
        }

        getGlobalUrl() {
            return WxConfigMgr.GlobalUrl[this._platform]['url'];
        }

        get subPackageName() {
            if(this._compressed) {
                return 'sub_inloading_compressed';
            } else {
                return 'sub_inloading';
            }
        }

        get subPackageCfgName() {
            if(this._compressed) {
                return "subpackage_compressed.json";
            } else {
                return "subpackage.json";
            }
        }

        get CdnBaseUrl() {
            if(this._platform == "jy") {
                if(this._compressed) {
                    return "https://download-fy.hulai.com/cat_jy_compressed/";
                } else {
                    return "https://download-fy.hulai.com/cat_jy/";
                }
            }else {
                if(this._compressed) {
                    return "https://download-fy.hulai.com/cat_compressed/";
                } else {
                    return "https://download-fy.hulai.com/cat/";
                }
            }
        }

        get CdnShareUrl() {
            return "https://download-fy.hulai.com/share/cat/";
        }

        getWxMessageTmplIds(keys) {
            let tmplIds = [];
            for (const key of keys) {
                let id = WxConfigMgr.WxMessageTmplIds[this._platform][key];
                if (id != null) {
                    tmplIds.push(id);
                }
            }
            return tmplIds;
        }
        getWxMessageType(key) {
            return WxConfigMgr.WxMessageType[key];
        }
    }

    //GlobalUrl 是 appid 对应不同 global 地址的map
    WxConfigMgr.GlobalUrl = {
        "prod": {
            "url": "fy.hulai.com:17305",
            "appid": "wxee291f857e1d91ef",   //default appid，以后应该写正式的appid，防止偶尔没传的情况下线上出现问题
            "version": "1.0.0",
            "adUnitId": "",
            "bannerId": ""
        },
        "jy": {
            "url": "fy.hulai.com:17401",
            "appid": "wxae673ae58e5d7c96",   //default appid，以后应该写正式的appid，防止偶尔没传的情况下线上出现问题
            "version": "1.0.11",
            "adUnitId": "adunit-153db474a089472c",
            "bannerId": "adunit-b45a44735112fcdd"
        },
        "pp":{
            //"url":"fy.hulai.com:17301",//测试服
            "url":"game-cats-life.cp.freegames123.com:17405",//正式服
            "appid": "catslife",   //default appid，以后应该写正式的appid，防止偶尔没传的情况下线上出现问题
            "version": "1.0.0",
            "adUnitIdAndroid": "ca-app-pub-9882247603724774/3684342023",
            "adUnitIdIos": "ca-app-pub-9882247603724774/2343550599",
        },

        // 长短都有 adunit-153db474a089472c   长广告 adunit-c237d58e3f5c8cf9  短广告 adunit-c26f2cd898739fdb

        // "dev": {
        //     "url" : "",
        //     "appid": "",
        //     "version": "1.0.0",
        //     "adUnitId": ""
        // },
        "test": {
            "url" : "fy.hulai.com:17305",
            "appid": "wxee291f857e1d91ef",
            "version": "1.0.0",
            "adUnitId": "",
            "bannerId": ""
        }
    };

    WxConfigMgr.WxMessageTmplIds = {
        "prod": {
            "ENTRUST": "_1HY3-yQV36ofClgBFvC04J7EQLiNxqqWOjQUo7KA1A",
            "FORTUNE_CAT": "yzkRrzCMg13qNwR2LJTd94njeqaz0I31eM-v13Yz5Sc",
            "CAT_FOOD": "4lJhHg00H3q2Ol5P-LJ9cWC1xpuXjZ10M5psIyCf208",
            "NAUTICAL_START": "4lJhHg00H3q2Ol5P-LJ9cfuS8ogINMjHN5Ny5VhRjwE",
            "FISHING": "4lJhHg00H3q2Ol5P-LJ9cTY27NY_m1vaoWIyNxPxuDQ"
        },
        "jy": {
            "ENTRUST": "r9v4Kx5DRTedTEWK3GPAh9oHZj1Gy0Y5YqKP2zbMB6s",
            "FORTUNE_CAT": "E20w65SZFsZOEGoHbW9zCO9yJD1P7IeDZ1iOD44Sbik",
            "CAT_FOOD": "Y-oFcUl3cWJgYOQ31neC_OYdB3Ki_AsneeCmnnHtgtY",
            "NAUTICAL_START": "Y-oFcUl3cWJgYOQ31neC_EEKOZ_tF9irAyQbfAUhsYY",
            "FISHING": "Y-oFcUl3cWJgYOQ31neC_MDtRGylRKm2UxCehY-Avmw"
        }
    };

    WxConfigMgr.WxMessageType = {
        "INTERACTIVE": "SYS_MSG_TYPE_INTERACTIVE",
        "RANK": "SYS_MSG_TYPE_RANK"
    };

    // WxConfigMgr.CdnBaseUrl = "https://download-fy.hulai.com/cat/"

    //网络层是否要加密，true是加密，false不加密
    WxConfigMgr.NetEncrypt = true;
    //与后端的协议是否使用 proto，true是使用, false不使用
    WxConfigMgr.ProtoEnable = true;

    class ADShareMgr {

        constructor() {
            this._shareTimes = 0;
            this._shareFailTimes = 0;
            this._hasVideo = false;
        }

        static getInstance() {
            return ADShareMgr.instance ? ADShareMgr.instance : ADShareMgr.instance = new ADShareMgr();
        }

        setShareTimes(shareTimes) {
            this._shareTimes = shareTimes;
        }

        getShareTimes() {
            return this._shareTimes;
        }

        setShareFailTimes(shareFailTimes) {
            this._shareFailTimes = shareFailTimes;
        }

        getShareFailTimes() {
            return this._shareFailTimes;
        }

        get hasVideo() {
            return this._hasVideo;
        }

        set hasVideo(v) {
            this._hasVideo = v;
        }
    }

    var rewardedVideoPP;
    class SdkHelper {

        static init(version, id) {

            EventMgr.getInstance().registEvent("SDK_SEND_EVENT", null, SdkHelper.sendEvent);
            EventMgr.getInstance().registEvent("SDK_SEND_EVENT_PP", null, SdkHelper.sendEventPPObj);

            let param = SdkHelper.params();
            param['method'] = "createInit";
            param['aid'] = WxConfigMgr.getInstance().getAdUnitId();
            param['traceid'] = "null";
            SdkHelper.trackEvent(param);
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                JYSDK.init({ version: version, debug: false });
            } else if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                PLAY68_SDK.debug = true;
                PLAY68_SDK.appId = id;
                PLAY68_SDK.init(function (status, player) {
                    if (status) {
                        console.log(player.id); // 1738266482889185
                        console.log(player.name); // null 此时还拿不到用户名和头像。
                        // 此处可以开始加载资源，设置加载进度，见2.2
                        console.log("压 土豆   ftue_account_login");
                        let str = "ftue_account_login";
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
                        EventMgr.getInstance().postEvent('PP_INIT_FISHING');
                    } else {
                        console.log(player); // {code: 1, message: "错误信息"}
                    }
                });
            }
        }

        static showPay(goodsId,productID) {
            //PurchaseCDTime  20  付费间隔时间//单位s
            if(!(Laya.Browser.onAndroid||Laya.Browser.onIOS&&Constants.shouji)){
                // EventMgr.getInstance().postEvent("PURCHASE_SHOWFAILED");
                return ;
            }
            // 第一步 检测支付环境是否准备就绪（必要检查，可能有的环境不支持支付，在所有支付操作之前检查，只需检查一次就行。）
            console.log("判断当前环境是否可以发起支付");
            var time = TimeUtils.getTimeNowOnServer();
            PLAY68_SDK.onReady(function(res, error){
                console.log("res,error",res,error);
                if (!res) {
                    // 不支持支付 报错
                    console.error(error.message);
                    EventMgr.getInstance().postEvent("PURCHASE_SHOWFAILED");
                    return;
                }
                var time1 = TimeUtils.getTimeNowOnServer();
                if(time1 - time > Number(Const.get("PurchasePayTime")) * 1000){
                    return ;
                }
                EventMgr.getInstance().postEvent("PURCHASE_CHANGEPAYTIMEFLAG",[1]);
                // 第二步 拉起支付界面
                console.log("拉起支付，开始购买商品");
                PLAY68_SDK.purchaseAsync(productID, null, function(res, purchase){
                    console.log("res,purchase",res,purchase);
                    // 此处拿到purchase信息 可以看作是订单信息
                    // 此时 purchase.isConsumed 应该是false
                    // purchase.signedRequest 签名数据 服务器通过这个验证请求确实来自sdk
        
                    // 这里判断下 支付成功了才消费。
                    if (res === true) {
                        console.log("purchase.paymentID",purchase.paymentID);
                        EventMgr.getInstance().postEvent("PURCHASEASYNC_COMPLETE",[purchase.paymentID,goodsId]);
                        // 第三步 消费购买的purchase（消费了之后才可以再次购买此商品）
                        SdkHelper.consumePurchaseAsync(goodsId,purchase);
                    }
                    else {
                        EventMgr.getInstance().postEvent("PURCHASE_SHOWFAILED");
                    }
                });
            });
        }

        static consumePurchaseAsync(goodsId,purchase){
            console.log("消费购买的商品");
            PLAY68_SDK.consumePurchaseAsync(purchase.purchaseToken , function(res,error){
                console.log("res,error",res,error);
                if (res === true) {
                    // 第四步 消费成功，此处可以开始发货逻辑
                    // 发货时可以把purchase.signedRequest发送到服务器检查签名 验证解密方式见11.签名验证
                    EventMgr.getInstance().postEvent("CONSUMEPURCHASEASYNC_COMPLETE",[purchase.paymentID,goodsId]);
                    // EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["gemBuy",{"gemUnit":goodsId}]);
                } else {
                    EventMgr.getInstance().postEvent("PURCHASE_SHOWFAILED");
                    console.error(error.message);
                }
            });
        }

        static getPurchasesAsync(){  
               //登陆时重新检测
         if(WxConfigMgr.getInstance().getPlatform() == "pp") {
            console.log("获取用户所有未消费的购买商品");
            PLAY68_SDK.getPurchasesAsync(Laya.Handler.create(this,function(res,purchaseList){
                console.log("res,purchaseList",res,purchaseList);
                if (res === true) {
                    for(let i = 0; i < purchaseList.length; i ++){
                        let config = ConfigMgr$1.getInstance().getConfig("Purchase");
                        config.forEach((listConfig, Id) => {
                            if(purchaseList[i] == listConfig.AndroidPoint || purchaseList[i] == listConfig.IOSPoint){
                                SdkHelper.consumePurchaseAsync(Id,purchaseList[i]);
                            }
                        });
                        
                    }
                }
            }));
        }
        }

        static setShareTimes(shareTimes) {
            ADShareMgr.getInstance().setShareTimes(shareTimes);
        }

        static getShareTimes() {
            return ADShareMgr.getInstance().getShareTimes();
        }

        static addShareTimes() {
            let shareTimes = ADShareMgr.getInstance().getShareTimes() + 1;
            ADShareMgr.getInstance().setShareTimes(shareTimes);
            ADShareMgr.getInstance().setShareFailTimes(0);
            EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", { 'shareTimes': String(shareTimes) });
            // EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", {'shareTimes': String(shareTimes), 'shareFailTimes': String(0)});
        }

        static setShareFailTimes(shareFailTimes) {
            ADShareMgr.getInstance().setShareFailTimes(shareFailTimes);
        }

        static getShareFailTimes() {
            return ADShareMgr.getInstance().getShareFailTimes();
        }

        static addShareFailTimes() {
            let shareFailTimes = ADShareMgr.getInstance().getShareFailTimes() + 1;
            ADShareMgr.getInstance().setShareFailTimes(shareFailTimes);
            // EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", {'shareFailTimes': String(shareFailTimes)});
        }

        static hasVideo() {
            // return ADShareMgr.getInstance().hasVideo;
            return true;
        }

        static sendLoginLog(obj) {
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                JYSDK.sendLoginLog(
                    obj
                    // {
                    //     app_id: _appId,
                    //     open_id: _openId,
                    //     uid: System.getRid(),
                    //     create_time: Math.floor(res.extraInfo.createTs),
                    //     login_times: res.extraInfo.loginTimes
                    // }
                );
            }
        }
        static sendLioginPP(id) {
            let param = SdkHelper.params();
            SdkHelper.uid = id;
            param['method'] = "createLogin";
            param['uid'] = id;
            param['aid'] = WxConfigMgr.getInstance().getAdUnitId();
            param['traceid'] = "null";
            SdkHelper.trackEvent(param);
        }

        static sendLoadingLogStart() {
            if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                console.log("压 土豆   ftue_loadingProgress_start");
                let str = "ftue_loadingProgress_start";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
            }
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                JYSDK.sendLoadingLog('start');
            }
        }

        static sendLoadingLogEnd() {
            if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                console.log("压 土豆   ftue_loadingProgress_end");
                let str = "ftue_loadingProgress_end";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
            }
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                JYSDK.sendLoadingLog('end');
            }
        }
        static loadRewardVideoAd() {
            if (Laya.Browser.onMiniGame) {
                let adUnitId = WxConfigMgr.getInstance().getAdUnitId();
                if (adUnitId && adUnitId != "") {
                    ADShareMgr.getInstance().hasVideo = true;
                    EventMgr.getInstance().postEvent("SDK_AD_HAS_VIDEO_CHANGE");
                    let i = wx.createRewardedVideoAd({ 'adUnitId': adUnitId });
                    i.load();
                    i.onError(e => {
                        ADShareMgr.getInstance().hasVideo = false;
                        EventMgr.getInstance().postEvent("SDK_AD_HAS_VIDEO_CHANGE");
                    });
                }
            }
            else if((Laya.Browser.onAndroid || Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone) && Constants.shouji){
                console.log('尝试加载pp广告');
                let placementID = WxConfigMgr.getInstance().getAdUnitId();
                if (placementID && placementID != "") {
                    // EventMgr.getInstance().postEvent("SDK_AD_HAS_VIDEO_CHANGE");
                    PLAY68_SDK.getRewardedVideoAsyncV3(placementID, function (status, obj) {
                        if (status) {
                            ADShareMgr.getInstance().hasVideo = true;
                            console.log('加载pp广告成功');
                            rewardedVideoPP = obj;   // 这里拿到对象，可以使用6.4调用显示广告
                        } else {
                            ADShareMgr.getInstance().hasVideo = false;
                            console.error(obj);    // status不为true，返回的是error对象
                            console.log('加载pp广告失败', placementID);
                            // todo 加载失败，可以延迟几秒后再尝试重新加载
                            SdkHelper.loadRewardVideoAdFail(placementID);
                        }
                    });
                }
            }
        }
        static loadRewardVideoAdFail(placementID) {
            console.log('加载pp广告失败33333333333333333333333', placementID);
            Laya.timer.once(6000, this, function () {
                PLAY68_SDK.getRewardedVideoAsyncV3(placementID, function (status, obj) {
                    if (status) {
                        ADShareMgr.getInstance().hasVideo = true;
                        console.log('加载pp广告成功');
                        rewardedVideoPP = obj;   // 这里拿到对象，可以使用6.4调用显示广告
                    } else {
                        ADShareMgr.getInstance().hasVideo = false;
                        console.error(obj);    // status不为true，返回的是error对象
                        console.log('加载pp广告失败', placementID);
                        SdkHelper.loadRewardVideoAdFail(placementID);
                    }
                });
            });
        }

        static loadInterstitialAd() {

        }

        static showRewardVideoAd(videoId, title = null, imageUrl = null, query = null) {
           // EventMgr.getInstance().postEvent("SDK_VIDEO_DONE", videoId);return;
            if (SdkHelper._currentShardId && SdkHelper._currentShardId == videoId) {
                return;
            }
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                if (SdkHelper.hasVideo()) {
                    SdkHelper._currentShardId = videoId;
                    EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [1, videoId]);
                    EventMgr.getInstance().postEvent("SOUND_SET_ALL_MUTED", [true]);
                    JYSDK.showVideo(videoId, (code) => {
                        if (code == 0) {
                            EventMgr.getInstance().postEvent("SDK_VIDEO_DONE", videoId);
                            console.log("视频播放完成");
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [2, videoId]);
                        } else if (code == 1) {
                            EventMgr.getInstance().postEvent("SDK_VIDEO_UNDONE", videoId);
                            console.log("视频正在拉起");
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [3, videoId]);
                        } else if (code == 2) {
                            EventMgr.getInstance().postEvent("SDK_VIDEO_UNDONE", videoId);
                            console.log("视频播放提前取消");
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [3, videoId]);
                        } else if (code == 3) {
                            EventMgr.getInstance().postEvent("SDK_VIDEO_ERROR", videoId);
                            console.log("视频播放出错");
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [5, videoId]);
                            ADShareMgr.getInstance().hasVideo = false;
                            SdkHelper.shareAppMessage(videoId, title, imageUrl, query);
                            EventMgr.getInstance().postEvent("SDK_AD_HAS_VIDEO_CHANGE");
                        } else {
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [4, videoId]);
                        }
                        EventMgr.getInstance().postEvent("SOUND_SET_ALL_MUTED", [false]);
                        EventMgr.getInstance().postEvent("SOUND_RESET_LOCAL_SOUND_SETTING");
                        EventMgr.getInstance().postEvent("SOUND_REPLAY_BGM", "bgm");
                        SdkHelper._currentShardId = null;
                    }, { 'adUnitId': WxConfigMgr.getInstance().getAdUnitId() });
                } else {
                    SdkHelper.shareAppMessage(videoId, title, imageUrl, query);
                }
            }
            else if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                // if (SdkHelper.hasVideo()) {
                SdkHelper._currentShardId = videoId;
                EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [1, videoId]);
                EventMgr.getInstance().postEvent("SOUND_SET_ALL_MUTED", [true]);
                PLAY68_SDK.showAsyncV3(rewardedVideoPP, function (status, obj) {  // rewardedVideo 即为上面返回的广告对象
                    let param = SdkHelper.params();
                    param['method'] = "reportAd";
                    param['uid'] = SdkHelper.uid;
                    param['aid'] = WxConfigMgr.getInstance().getAdUnitId();
                    param['traceid'] = "null";
                    param['adType'] = "1";
                    if (status) {
                        console.log('播放pp广告成功');
                        // 这里 obj === rewardedVideo
                        EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [2, videoId]);
                        EventMgr.getInstance().postEvent("SDK_VIDEO_DONE", videoId);
                        SdkHelper.ppAdEvent(videoId, true);
                        param['loadStatus'] = 1;
                        param['showStatus'] = 1;
                        param['playStatus'] = 1;

                    } else {
                        console.log('播放pp广告失败', obj.code);
                        if (obj.code == -1) {
                            //用户强制关闭广告
                            param['loadStatus'] = 1;
                            param['showStatus'] = 1;
                            param['playStatus'] = 5;
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [3, videoId]);
                            SdkHelper.ppAdEvent(videoId, false);
                        } else if (obj.code == -6) {
                            //没有广告
                            param['loadStatus'] = 2;
                            param['showStatus'] = 2;
                            param['playStatus'] = 2;
                            EventMgr.getInstance().postEvent("SERVICE_VIEW_AD", [5, videoId]);
                        }
                        TipsPopMgr.getInstance().showTipsById("tid#TipsOverseas1");
                        console.error(obj);    // status不为true，返回的是error对象
                    }
                    SdkHelper.trackEvent(param);
                    EventMgr.getInstance().postEvent("SOUND_SET_ALL_MUTED", [false]);
                    EventMgr.getInstance().postEvent("SOUND_RESET_LOCAL_SOUND_SETTING");
                    EventMgr.getInstance().postEvent("SOUND_REPLAY_BGM", "bgm");
                    SdkHelper._currentShardId = null;
                    // todo 不管播放成功或者失败，这里都可以尝试加载下一个广告
                    SdkHelper.loadRewardVideoAd();
                });
                // }
            }
        }

        static showBanner(bannerId, left, top, width, height) {
            if (WxConfigMgr.getInstance().getPlatform() == 'jy') {
                JYSDK.showBanner(bannerId, { 'adUnitId': WxConfigMgr.getInstance().getBannerId(), 'style': { 'left': left, 'top': top, 'width': width, 'height': height } });
            }
        }

        static hideBanner(bannerId) {
            if (WxConfigMgr.getInstance().getPlatform() == 'jy') {
                JYSDK.hideBanner(bannerId);
            }
        }

        static shareAppMessage(shareId, title = null, imageUrl = null, query = null, failTip = true) {
            if (SdkHelper._currentShardId && SdkHelper._currentShardId == shareId) {
                return;
            }
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                if (imageUrl == null) {
                    SdkHelper._currentShardId = shareId;
                    let shareContents = Const.getArray("ShareContents");
                    if (shareContents) {
                        let info = shareContents[Utils.randomExt(shareContents.length)];
                        imageUrl = WxConfigMgr.getInstance().CdnShareUrl + info.img + ".jpg";
                    } else {
                        imageUrl = "https://download-fy.hulai.com/share/cat/share_img1.jpg";
                    }
                }
                if (title == null) {
                    let shareContents = Const.getArray("ShareContents");
                    if (shareContents) {
                        let info = shareContents[Utils.randomExt(shareContents.length)];
                        title = Strings$1.get(info.desc);
                    } else {
                        title = "快来一起养宠物吧!";
                    }
                }
                let shareTimes = SdkHelper.getShareTimes();
                let shareFailTimes = SdkHelper.getShareFailTimes();
                let delay = 3;
                if (shareTimes >= 3)
                    delay = 5;
                JYSDK.shareAppMessage({
                    shareId: shareId,
                    title: title,
                    imageUrl: imageUrl,
                    query: query,
                    delay: delay,
                    success: function () {
                        //分享成功
                        SdkHelper.addShareTimes();
                        EventMgr.getInstance().postEvent("SDK_SHARE_DONE", shareId);
                        EventMgr.getInstance().postEvent("SERVICE_VIEW_SHARE", [1, shareId]);
                        SdkHelper._currentShardId = null;
                    },
                    fail: function () {
                        //分享失败
                        SdkHelper.addShareFailTimes();
                        if (failTip) {
                            if (shareTimes == 0) {
                                if (shareFailTimes == 0) {
                                    EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#ShareFailedA1");
                                } else if (shareFailTimes == 1) {
                                    EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#ShareFailedA1");
                                } else {
                                    let tips = ["tid#ShareFailedA1", "tid#ShareFailedA2"];
                                    let tip = tips[Utils.randomExt(tips.length)];
                                    if (tip) {
                                        EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", tip);
                                    }
                                }
                            } else {
                                if (shareFailTimes == 0) {
                                    EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#ShareFailedB1");
                                } else if (shareFailTimes == 1) {
                                    EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#ShareFailedB2");
                                } else {
                                    let tips = ["tid#ShareFailedB1", "tid#ShareFailedB2"];
                                    let tip = tips[Utils.randomExt(tips.length)];
                                    if (tip) {
                                        EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", tip);
                                    }
                                }
                            }
                        }
                        EventMgr.getInstance().postEvent("SDK_SHARE_FAIL", shareId);
                        EventMgr.getInstance().postEvent("SERVICE_VIEW_SHARE", [2, shareId]);
                        SdkHelper._currentShardId = null;
                    },
                });
            } else if (Laya.Browser.onMiniGame) {
                wx.shareAppMessage({
                    title: title,
                    imageUrl: imageUrl,
                    query: query,
                    success(res) {
                        console.log("success", res);
                        SdkHelper._currentShardId = null;
                    },
                    fail(res) {
                        console.log("fail", res);
                        SdkHelper._currentShardId = null;
                    }
                });
            }
            //  else{
            //     EventMgr.getInstance().postEvent("SDK_SHARE_DONE", shareId);
            //  }
        }

        static onShareAppMessage(shareId, title = null, imageUrl = null, query = null) {
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                wx.showShareMenu();
                // if(title==null) {
                //     let shareContents = Const.getArray("ShareContents");
                //     if(shareContents) {
                //         let info = shareContents[Utils.randomExt(shareContents.length)];
                //         title = Strings.get(info.desc);
                //         imageUrl = WxConfigMgr.getInstance().CdnShareUrl + info.img + ".jpg";
                //     }else{
                //         title = "快来一起养宠物吧!";
                //         imageUrl = "https://download-fy.hulai.com/share/cat/share_img1.jpg";
                //     }
                // }
                // JYSDK.onShareAppMessage({
                //     shareId: shareId,
                //     title: title,
                //     imageUrl: imageUrl,
                //     query: query,
                //     delay: 3,
                //     success: function() {
                //         //分享成功
                //     },
                //     fail: function() {
                //         //分享失败
                //     },
                // });
                console.log("onShareAppMessage");
                wx.onShareAppMessage(() => {
                    if (title == null) {
                        let shareContents = Const.getArray("ShareContents");
                        if (shareContents) {
                            let info = shareContents[Utils.randomExt(shareContents.length)];
                            title = Strings$1.get(info.desc);
                            imageUrl = WxConfigMgr.getInstance().CdnShareUrl + info.img + ".jpg";
                        } else {
                            title = "快来一起养宠物吧!";
                            imageUrl = "https://download-fy.hulai.com/share/cat/share_img1.jpg";
                        }
                    }
                    return {
                        title: title,
                        imageUrl: imageUrl // 图片 URL
                    }
                });
            }
        }

        static sendEvent(eventId, data = null, type) {
            if (WxConfigMgr.getInstance().getPlatform() == "jy") {
                JYSDK.sendEvent(eventId, data);
            } else if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                if (type && type == 1) {
                    console.log('ppevent', eventId);
                    PLAY68_SDK.logEvent(eventId);
                } else {
                    let param = SdkHelper.params();
                    param['method'] = "reportEvent";
                    if (SdkHelper.uid) {
                        param['uid'] = SdkHelper.uid;
                    }
                    if (WxConfigMgr.getInstance().getAdUnitId()) {
                        param['aid'] = WxConfigMgr.getInstance().getAdUnitId();
                    }
                    param['traceid'] = "0";
                    param['evnetName'] = eventId;
                    param['eventId'] = eventId;
                    param['eventRes'] = 1;
                    SdkHelper.trackEvent(param);
                }
            }
        }
        static sendEventPPObj(eventId, obj) {
            if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                if (obj) {
                    PLAY68_SDK.logEvent(eventId, '', obj);
                } else {
                    PLAY68_SDK.logEvent(eventId);
                }
            }
        }

        static requestSubscribeMessage(keys) {
            if (!Laya.Browser.onMiniGame) {
                return;
            }
            let tmplIdArr = WxConfigMgr.getInstance().getWxMessageTmplIds(keys);
            wx.requestSubscribeMessage({
                tmplIds: tmplIdArr,
            });
        }

        static checkSubscriptions(key, handler) {
            if (!Laya.Browser.onMiniGame) {
                handler.runWith(false);
                return;
            }
            let id = WxConfigMgr.getInstance().getWxMessageType(key);
            if (id == null) {
                id = WxConfigMgr.getInstance().getWxMessageTmplIds([key])[0];
            }
            if (!id) {
                if (handler) {
                    handler.runWith(false);
                }
                return;
            }
            wx.getSetting({
                withSubscriptions: true,
                success(res) {
                    let setting = res.subscriptionsSetting;
                    if (setting && setting.mainSwitch) {
                        if (setting[id] != null /* && setting[id] == "accept" */) {
                            //已勾选不在提醒
                            if (handler) {
                                handler.runWith(true);
                            }
                        } else {
                            if (handler) {
                                handler.runWith(false);
                            }
                        }
                    } else {
                        //已关闭总订阅开关
                        if (handler) {
                            handler.runWith(true);
                        }
                    }
                }
            });
        }

        static requestSubscribeSystemMessage(key) {
            let msgType = WxConfigMgr.getInstance().getWxMessageType(key);
            if (Laya.Browser.onMiniGame) {
                wx.requestSubscribeSystemMessage({
                    msgTypeList: [msgType],
                    complete(res) {
                        console.log(res);
                    }
                });
            }
        }

        //获取小程序启动时的参数
        static getLaunchOptionsSync() {
            if (Laya.Browser.onMiniGame) {
                return wx.getLaunchOptionsSync();
            }
        }
        //广告打点
        static ppAdEvent(id, ifover) {
            let adComplet;
            if (ifover) {
                adComplet = 'Complete';
            } else {
                adComplet = 'incomplete';
            }
            let str = 'adWatch';
            let obj = { 'adUnit': id, 'adComplete': adComplet };
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
        }

        //米壳
        //排序
        static objKeySort(obj) {
            var newkey = Object.keys(obj).sort();
            var newObj = {};
            for (var i = 0; i < newkey.length; i++) {
                newObj[newkey[i]] = obj[newkey[i]];
            }
            return newObj;
        }

        //加密
        static genSign(obj, apikey) {
            if (obj.sign) {
                delete obj.sign;
            }
            var str = "";
            var newkey = Object.keys(obj);
            for (var i = 0; i < newkey.length; i++) {
                str += newkey[i] + '=' + obj[newkey[i]];
            }
            str += apikey;
            return md5(str);
        }

        static trackEvent(param) {
            let ret = SdkHelper.objKeySort(param); //函数执行
            let api_key = 'db96abc4109f6d516b9e5e0a67aa79e9';
            let sign = SdkHelper.genSign(ret, api_key);
            param['sign'] = sign;
            console.log(param, '数据上报');
            $.ajax({
                type: "post",
                url: "https://sgptrack.mikeyouxi.com/track",
                data: param,
                async: false,
                success: function (ret) {
                    console.log(ret, '数据上报');
                },
                fail: function (ret) {
                    console.log(ret, '数据上报');
                }
            });
        }
        static params() {
            let deviceType="";
            if(Laya.Browser.onAndroid){
                deviceType="Android";
            }else if(Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone){
                deviceType="IOS";
            }  
            let param = {
                "gameId": "47",
                "channelId": "223",
                "subGameId": "1417",
                "platformId": '2',
                "buildVersion": Constants.ppversion,
                "devId": "211065FE-8E15-49E5-A03C-273B877BE48C",
                "deviceType":deviceType,
                "os": "11.3",
                "source": "2",
            };
            return param;
        }
        static onGiveMarking() {
            if (this.onGiveMarkingSign) {
                return;
            }
            this.onGiveMarkingSign = 1;
            if ((Laya.Browser.onAndroid || Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone) && Constants.shouji){
                console.log("打开去商店评分弹窗");
                PLAY68_SDK.giveMarking();
            }
            Laya.timer.once(1000, this, function () {
                this.onGiveMarkingSign = 0;
            });
        }
        static goTuisong(id, title, desc, times) {
            if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                PLAY68_SDK.addNotification(id, title, desc, times, function (res, data) {
                    if (res) {
                        console.log('添加成功', id, times);
                    } else {
                        console.log('添加失败', id, title, desc, times);
                    }
                });
            }
        }
        static shanchuTuisong(id) {
            if ((Laya.Browser.onAndroid || Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone) && Constants.shouji){
                PLAY68_SDK.removeNotification(id, function (res, data) {
                    if (res) {
                        console.log('删除/取消成功');
                    } else {
                        console.log('删除/取消失败');
                    }
                });
            }
        }
        //8点推送
        static tuisongDays() {
            if (WxConfigMgr.getInstance().getPlatform() == "pp") {
                console.log("压 土豆   ftue_loadingPicture_end");
                let str = "ftue_loadingPicture_end";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
            }
            if ((Laya.Browser.onAndroid || Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone )&& Constants.shouji){
                // 2/5/9/14day
                let date = new Date();
                let oneday = 24 * 3600;
                //当天剩余时间  ----到第二天 的 20点
                let time = oneday - (date.getHours() * 3600 + date.getMinutes() * 60 + date.getSeconds()) + 20 * 3600;
                //第二天
                SdkHelper.shanchuTuisong('DI_JITIAN_2');
                SdkHelper.goTuisong('DI_JITIAN_2', Strings$1.get('tid#WXMessage_LoginName'), Strings$1.get('tid#WXMessage_LoginDesc'), '' + (time + oneday));
                SdkHelper.shanchuTuisong('DI_JITIAN_5');
                SdkHelper.goTuisong('DI_JITIAN_5', Strings$1.get('tid#WXMessage_LoginName'), Strings$1.get('tid#WXMessage_LoginDesc'), '' + (time + 4 * oneday));
                SdkHelper.shanchuTuisong('DI_JITIAN_9');
                SdkHelper.goTuisong('DI_JITIAN_9', Strings$1.get('tid#WXMessage_LoginName'), Strings$1.get('tid#WXMessage_LoginDesc'), '' + (time + 8 * oneday));
                SdkHelper.shanchuTuisong('DI_JITIAN_14');
                SdkHelper.goTuisong('DI_JITIAN_14', Strings$1.get('tid#WXMessage_LoginName'), Strings$1.get('tid#WXMessage_LoginDesc'), '' + (time + 13 * oneday));
            }
        }
    }

    class Bowl extends Laya.Box {
        constructor() {
            super();
            this.visible = false;
        }

        update() {
            var bowlType = Laya.LocalStorage.getJSON("bowlType");
            if(!bowlType){
                bowlType = 0;
                Laya.LocalStorage.setJSON("bowlType",bowlType);
            }
            let exhaustTime = SyncSystem.getScene2ExhaustTime();
            let now = TimeUtils.getTimeNowOnServer();
            let total = Number(Const.get("CatFoodKeepTime"));
            let full = Number(Const.get("CatFoodNotFull"));
            if(exhaustTime==null || exhaustTime<=now) {
                if(this._image.texture!="common/bowl_0.png" && this._image.texture!="common/Gbowl_0.png"){
                    if(bowlType){
                        this._image.texture = "common/Gbowl_0.png";
                    }
                    else {
                        this._image.texture = "common/bowl_0.png";
                    }   
                }
                this.barValue = 0;
            }else {
                let ratio = (exhaustTime - now)/1000/total;
                this.barValue = ratio;
                if (ratio <= 1) {
                    this._label.text = "";
                    this._label.visible = false;
                } else if (ratio > 1 && ratio <= 2) {
                    this._label.text = "X2";
                    this._label.visible = true;
                } else {
                    this._label.text = "X" + Math.ceil(ratio);
                    this._label.visible = true;
                }
                if(ratio>full) {
                    if(this._image.texture!="common/bowl_2.png" && this._image.texture!="common/Gbowl_2.png"){
                        if(bowlType){
                            this._image.texture = "common/Gbowl_2.png";
                        }
                        else {
                            this._image.texture = "common/bowl_2.png";
                        }   
                    }
                }else {
                    if(this._image.texture!="common/bowl_1.png" && this._image.texture!="common/Gbowl_1.png"){
                        if(bowlType){
                            this._image.texture = "common/Gbowl_1.png";
                        }
                        else {
                            this._image.texture = "common/bowl_1.png";
                        }   
                    }
                }
            }
            this._image.visible = true;
            this._progress.visible = true;
            this.checkWeakGuideShow();
        }

        onEnable() {
            this._image = this.getChildByName("image");
            this._progress = this.getChildByName("bar");
            this._label = this.getChildByName("label");
            this._label.visible = false;
            EventMgr.getInstance().registEvent("BOWL_SET_VISIBLE", this, this.setVisible);
            this.bar1 = this._progress.getChildByName("bar1");
            this.bar2 = this._progress.getChildByName("bar2");
            this.bar3 = this._progress.getChildByName("bar3");
            this.bar4 = this._progress.getChildByName("bar4");
            if (SceneMgr.getInstance().getSceneLock(Constants.BeachSceneId)) {
                return;
            }
            if(this.visible==false) {
                this.visible = true;
                this._image.visible = false;
                this._progress.visible = false;
                this._label.visible = false;
            }

            Laya.timer.loop(1000, this, this.update, null, true, true);
            EventMgr.getInstance().registEvent("REFRESH_FOOD", this, this.update);
            this.on(Laya.Event.CLICK, this, this.onClick);
            TaskMainUIMgr.getInstance().registButton("Catfood", this);
            EventMgr.getInstance().registEvent("ADD_CAT_FOOD_AD", this, this.adAddFood);
            EventMgr.getInstance().registEvent("ADVERTISING_AD_CLICK", this, this.onADclick);
            EventMgr.getInstance().registEvent("GUIDE8_ADD_FOOD", this, this.guideAddFood);
            this.weakGuideFoodLimit = Number(Const.get("CatFoodWeekGuideRate"));
            this.weakGuideFoodLimitStar = Const.getArray("CatFoodWeekGuideRange");
            this.weakGuideFoodLimitStar[0] = Number(this.weakGuideFoodLimitStar[0]);
            this.weakGuideFoodLimitStar[1] = Number(this.weakGuideFoodLimitStar[1]);
        }

        onClick() {
            let total = Number(Const.get("CatFoodKeepTime"));
            let adTime = Number(Const.get("CatFoodKeepTimeAD"));
            let items = PetSystem.getItemBySpecialType(7);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
            });
            let videoStr = Strings$1.get("tid#UI_027", [String(Math.floor((total + adTime + itemAddTimes) / 3600))]);
            let shareStr = Strings$1.get("tid#UI_028", [String(Math.floor((total + adTime + itemAddTimes) / 3600))]);
            Laya.Dialog.close("ui/BowlAdPop.scene");
            Laya.Dialog.open("ui/BowlAdPop.scene", false, [Constants.ShareId.CatFood, "ADD_CAT_FOOD_AD", videoStr, shareStr]);
        }

        onDisable() {
            Laya.timer.clearAll(this);
            this.offAll();
            EventMgr.getInstance().removeEvent("REFRESH_FOOD", this, this.update);
            EventMgr.getInstance().removeEvent("ADD_CAT_FOOD_AD", this, this.adAddFood);
            EventMgr.getInstance().removeEvent("ADVERTISING_AD_CLICK", this, this.onADclick);
            EventMgr.getInstance().removeEvent("BOWL_SET_VISIBLE", this, this.setVisible);
            EventMgr.getInstance().removeEvent("GUIDE8_ADD_FOOD", this, this.guideAddFood);
        }

        adAddFood(type){
            if(type=="error"){
                return;
            }
            let total = Number(Const.get("CatFoodKeepTime"));
            let items = PetSystem.getItemBySpecialType(7);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
            });
            if(type==true){
                Laya.LocalStorage.setJSON("bowlType",1);
                EventMgr.getInstance().postEvent("GUIDE_GUIDE8_AD");
                let adTime = Number(Const.get("CatFoodKeepTimeAD"));
                let result= SyncSystem.addScene2Food(adTime+total+itemAddTimes);
                this.update();
                SyncSystem.updateSceneInfo();
                EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["CATFOOD_PUT", 1, null]);
                TaskMainSystem.addProgress("CATFOOD_PUT", 1);
                EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["ADS_CATFOOD_PUT", 1, null]);
                TaskMainSystem.addProgress("ADS_CATFOOD_PUT", 1);
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon21", [String(Math.floor((total + adTime + itemAddTimes) / 3600))]);
                    SdkHelper.shanchuTuisong('maomi_liangshi_1');
                    SdkHelper.goTuisong('maomi_liangshi_1',Strings$1.get('tid#WXMessage_CatFoodName'),Strings$1.get('tid#WXMessage_CatFoodDesc'),''+Math.floor((total + adTime + itemAddTimes)));
                if (result != null) {
                    EventMgr.getInstance().postEvent("SERVICE_CAT_FOOD_SUBSCRIBE", result);
                }
            } else {
                let exhaustTime = SyncSystem.getScene2ExhaustTime();
                let now = TimeUtils.getTimeNowOnServer();
                let full = Number(Const.get("CatFoodNotFull"));
                let result = SyncSystem.addScene2Food(total + itemAddTimes);
                if (exhaustTime && (exhaustTime - now) / 1000 / total > full) {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon22");
                    return;
                } else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon20", [String(Math.floor((total + itemAddTimes) / 3600))]);
                        SdkHelper.shanchuTuisong('maomi_liangshi_1');
                        SdkHelper.goTuisong('maomi_liangshi_1',Strings$1.get('tid#WXMessage_CatFoodName'),Strings$1.get('tid#WXMessage_CatFoodDesc'),''+Math.floor((total + itemAddTimes)));
                    Laya.LocalStorage.setJSON("bowlType", 0);
                }
                this.update();
                SyncSystem.updateSceneInfo();
                EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["CATFOOD_PUT", 1, null]);
                TaskMainSystem.addProgress("CATFOOD_PUT", 1);
                if (result != null) {
                    EventMgr.getInstance().postEvent("SERVICE_CAT_FOOD_SUBSCRIBE", result);
                }
            }
        }

        onADclick(shareId) {
            if (shareId == Constants.ShareId.CatFood) {
             
            }
        }

        setVisible(num){
            if(num==0){
                this.visible = false;
                this._image.visible = false;
                this._progress.visible = false;
                this._label.visible = false;
            }
            else if(num==1){
                this.visible = true;
                this._image.visible = true;
                this._progress.visible = true;
                this._label.visible = true;
            }
        }

        set barValue(value) {
            this._barValue = Math.max(0, value);
            this.bar1.width = Math.floor(Math.min(1, this._barValue) * 95);
            this.bar2.width = Math.floor(Math.min(1, Math.max(0, this._barValue - 1)) * 95);
            this.bar3.width = Math.floor(Math.min(1, Math.max(0, this._barValue - 2)) * 95);
            this.bar4.width = Math.floor(Math.min(1, Math.max(0, this._barValue - 3)) * 95);
        }

        get barValue() {
            return this._barValue ? this._barValue : 0;
        }
        
        checkWeakGuideShow() {
            if (!GuideSystem.isGuiding() && this.visible && PlayerSystem.getStar() < this.weakGuideFoodLimitStar[1] && PlayerSystem.getStar() > this.weakGuideFoodLimitStar[0] && this.barValue < this.weakGuideFoodLimit) {
                if (!this.maoliangTipAnim || this.maoliangTipAnim.destroyed) {
                    this.maoliangTipAnim = new Spine();
                    this.maoliangTipAnim.loadAnim("anim3/tanhaotixing.sk");
                    this.addChild(this.maoliangTipAnim);
                    this.maoliangTipAnim.x = 120;
                    this.maoliangTipAnim.y = -20;
                }
                this.maoliangTipAnim.playAnim("tanhaotixing", true);
                this.maoliangTipAnim.visible = true;
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [1, true]);
            } else {
                if (this.maoliangTipAnim && this.maoliangTipAnim.visible == true) {
                    this.maoliangTipAnim.visible = false;
                    this.maoliangTipAnim.stopAnim();
                }
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [1, false]);
            }
        }

        guideAddFood() {
            let total = Number(Const.get("CatFoodKeepTime"));
            SyncSystem.addScene2Food(total);
            this.update();
            SyncSystem.updateSceneInfo();
            EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["CATFOOD_PUT", 1, null]);
            TaskMainSystem.addProgress("CATFOOD_PUT", 1);
        }

    }

    class ViewAdjust extends Laya.Script {

        constructor() {
            super();
            //0 View剧中 1Dialog剧中
            /** @prop {name:align, tips:"0:View居中 1:Dialog居中", type:int, default:0}*/
            this.align = 0;
        }

        onStart() {
            if(this.owner) {
                if(this.align == 0) {
                    let heightScale = Laya.Browser.height/2436;
                    let realW = 1827*heightScale;
                    // let realH = 2436*heightScale;
                    this.owner.x = (Laya.Browser.width/2 - realW/2)/heightScale;
                } else if(this.align == 1) {
                    // this.owner.popup();
                    this.owner.isModal = true;
                }

            }
        }

        onEnable() {
            if(this.align == 1) {
                this.owner.isModal = true;
            }
        }
    }

    class Guide extends Laya.Script {

        constructor() {
            super();
            this.guideAnim = null;
            EventMgr.getInstance().registEvent("NEXT_GUIDE", this, this.onNextGuide);
        }

        onAwake() {
            // this.guideAnim = new Laya.Skeleton();
            // this.guideAnim.load("anim1/maozhua.sk", Laya.Handler.create(this, function() {
            //     this.guideAnim.x = 406;
            //     this.guideAnim.y = 2051;
            //     this.guideAnim.scaleX = 0.6;
            //     this.guideAnim.scaleY = 0.6;
            //     this.guideAnim.play("2", true);
            // }));
            // let root = this.owner.getChildByName("root");
            // root.addChild(this.guideAnim);
        }
        
        onEnable() {
            
        }

        onNextGuide() {
            // this.guideAnim.stop();
            // this.guideAnim.visible = false;
        }

        onDisable() {
        }

        onDestroy(){
            EventMgr.getInstance().removeEvent("NEXT_GUIDE", this, this.onNextGuide);
        }
    }

    class DropFish extends Laya.Sprite {

        constructor(buildingId) {
            super();
            this._buildingId = buildingId;
            this.prefab = null;
            this.countText = null;
            this.dropFishCount = 0;
        }

        onEnable() {
            Laya.loader.create("prefab/component/drop_fish.prefab", Laya.Handler.create(this, this._onLoaded));
            EventMgr.getInstance().registEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
        }

        _onLoaded(prefabDef) {
            this.prefab = prefabDef.create();
            this.width = this.prefab.width;
            this.height = this.prefab.height;
            this.pivotX = this.prefab.width/2;
            this.pivotY = this.prefab.height/2;
            this.addChild(this.prefab);
            this.countText = this.prefab.getChildByName("coutText");
            // this.countText.visible = this.dropFishCount>0?true:false;
            this.countText.visible = false;
            this.countText.text = "+"+String(this.dropFishCount);
        }

        onClick() {
            // this.setDropFish(0);
            if(this.countText) {
                this.countText.visible = false;
            }
        }

        addDropFish(count) {
            this.setDropFish(this.dropFishCount + count);
        }

        setDropFish(count) {
            this.dropFishCount = count;
            if(this.countText) {
                // this.countText.visible = count>0?true:false;
                this.countText.text = "+"+String(this.dropFishCount);
            }
            this.visible = count>0?true:false;
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
        }

        get factor() {
            return 0;
        }

        set factor(value) {
            if (value <= 0.8) {
                value = value / 0.8;
                this.x = (1 - value) * (1 - value) * this.startX + 2 * value * (1 - value) * this.midX + value * value * this.groundX;
                this.y = (1 - value) * (1 - value) * this.startY + 2 * value * (1 - value) * this.midY + value * value * this.groundY;
            } else {
                value = (value - 0.8) / 0.2;
                this.x = (1 - value) * (1 - value) * this.groundX + 2 * value * (1 - value) * this.midX2 + value * value * this.endX;
                this.y = (1 - value) * (1 - value) * this.groundY + 2 * value * (1 - value) * this.midY2 + value * value * this.endY;
            }
        }

        saveStartPoint(x,y) {
            let jump = 200;
            this.startX = x;
            this.startY = y;
            this.endX = this.x;
            this.endY = this.y;
            this.groundX = this.startX + (this.endX - this.startX) * 0.8;
            this.groundY = this.endY;
            let height1 = Math.abs(this.groundY - this.startY);
            let width1 = this.groundX - this.startX;
            this.midX = this.startX + width1 / (Math.pow((height1 / jump), 0.5) + 2);
            this.midY = Math.min(this.startY, this.groundY) - jump;
            this.midX2 = (this.endX + this.groundX) / 2;
            this.midY2 = this.endY - jump / 5;
        }

        cameraSwitch(type) {
            this.visible = !type;
        }
    }

    class ShowCatMenuTip {

        constructor(cat, foodId, buildingExtInfo, time) {
            this.cat = cat;
            this.foodId = foodId;
            this.buildingExtInfo = buildingExtInfo;
            this.time = time;
            this.__enabled = true;
            Laya.loader.create("prefab/tip/cat_menu_tip.prefab", Laya.Handler.create(this, this._onLoaded));
        }

        _onLoaded(prefabDef) {
            if(this._destroyed){
                return;
            }
            this.menuTipPrefab = prefabDef.create();
            if (this.__enabled) {
                this.menuTipPrefab.ani2.play(0, false);
                this.menuTipPrefab.ani2.once(Laya.Event.COMPLETE, this, function () {
                    this.menuTipPrefab.ani1.play(0,true);
                });
            }
            this.initCatMenuTip();
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.guide4);
            this.guide4("GuideGroup_4");
        }

        onEnable() {
            this.__enabled = true;
            if(this.menuTipPrefab){
                if(this.menuTipPrefab.visible) {
                    if (!this._eating) {
                        this.menuTipPrefab.ani1.play(0,true);
                    }
                }
            }
            this.guide4("GuideGroup_4");
        }

        onDisable() {
            this.__enabled = false;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
                this.menuTipPrefab.ani1.stop();
            }
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.guide4);
        }

        destroy() {
            if(this._destroyed)
                return;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.destroy();
                this.menuTipPrefab = null;
            }
            if(this.panel) {
                this.panel.destroy();
                this.panel = null;
            }
            this._destroyed = true;
        }

        set TipClicked(clicked) {
            this._tipClicked = clicked;
        }

        get TipClicked() {
            return this._tipClicked;
        }

        set TipVisible(visible) {
            if(this.menuTipPrefab){
                this.menuTipPrefab.visible = visible;
                if (visible && this.__enabled) {
                    if (!this._eating) {
                        this.menuTipPrefab.ani1.play(0,true);
                    }
                }else {
                    this.menuTipPrefab.ani2.stop();
                    this.menuTipPrefab.ani3.stop();
                    this.menuTipPrefab.ani1.stop();
                }
            }
            else
                this._tipVisible = visible;
        }

        get TipVisible() {
            return this._tipVisible;
        }

        initCatMenuTip() {
            let cat = this.cat;
            let menuTipPrefab = this.menuTipPrefab;

            this.cat.addChild(this.menuTipPrefab);
            let food = FoodSystem$1.getFood(this.foodId);
            let aniBox=this.menuTipPrefab.getChildByName("aniBox");
            let catTipImage = aniBox.getChildByName("meals_img");
            catTipImage.skin = food.Source;
            let barBox=this.menuTipPrefab.getChildByName("barBox");
            let catEatImage = barBox.getChildByName("food");
            catEatImage.skin = food.Source;
            this.tweenTime = food.getCookingTime() * 1000;
            this.menuTipPrefab.y = -340.5;
            this.menuTipPrefab.x = -70;

            if(cat.scaleX<0) {
                this.menuTipPrefab.scaleX = -1;
                this.menuTipPrefab.x = 83;
            }

            let panel =  new Laya.Panel();
            // panel.bgColor = "#c3504e";
            panel.width = 200;
            panel.height = 340;
            panel.centerX = 0.5;
            panel.y = -panel.height;
            this.cat.addChild(panel);
            this.panel = panel;

            if(this._tipVisible==false){
                menuTipPrefab.visible = false;
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
                this.menuTipPrefab.ani1.stop();
            }
            this._tipVisible = null;

            panel.on(Laya.Event.CLICK, this, this.onTipClicked);
        }

        onTipClicked() {
            EventMgr.getInstance().postEvent("COFFEE_CAT_CLICK", [this.cat.getId()]);
            if(this._tipClicked == true) {
                return;
            }
            let catData = this.cat.getData();
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.CoffeeSceneId, "CAT_EAT", catData.getCatId()]);
        }

        guide4(eventString) {
            if (eventString != "GuideGroup_4") {
                return;
            }
            let guide = GuideSystem.getGuide("GuideGroup_4");
            if (guide) {
                this.guide4 = function (eventString) {

                };
                guide.registerObj("btn0", this);
                let rect = new Laya.Rectangle(100, -340, -200, 340);
                guide.registerClick("btn0", this.panel, rect, [this.cat]);
                EventMgr.getInstance().postEvent('GOUDE_COLSE_TALK');
            }
        }

        skillEat() {
            if (this._tipClicked == true) {
                return;
            }
            this._tipClicked = true;
            let anim = new Spine();
            anim.loadAnim("anim2/renzhemao.sk");
            anim .x=-20;
            anim.y = 100;
            anim.addCompleteHandler(Laya.Handler.create(this, function () {
                anim.destroy();
            }));
            anim.playAnim("skill3-1", false);
            this.cat.addChild(anim);
            let catData = this.cat.getData();
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.CoffeeSceneId, "CAT_EAT", catData.getCatId()]);
        }

        showEatingBar() {
            this._eating = true;
            if (this.menuTipPrefab) {
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.ani3.play(0, true);
                let barBox = this.menuTipPrefab.getChildByName("barBox");
                let bar = barBox.getChildByName("bar");
                bar.scaleX = 0;
                Laya.Tween.to(bar, { scaleX: 2 }, this.tweenTime);
            }
        }
    }

    class EntrustSystem {
        static init() {
        }

        static clear() {
            EntrustDataMgr.getInstance().clear();
        }

        static updateEntrusts(entrusts) {
            EntrustDataMgr.getInstance().updateEntrusts(entrusts);
        }

        static updataOne(one) {
            let currentEntrust = EntrustDataMgr.getInstance().updataOne(one);
            SpecialCatMgr.getInstance().pushCat(currentEntrust);
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_entrust_accept"]);
        }

        static getEntrusts() {
            return EntrustDataMgr.getInstance().getEntrusts();
        }

        static getAllEntrusts(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_ENTRUST", [handler]);
        }

        static acceptEntrust(listId) {
            EventMgr.getInstance().postEvent("SERVICE_ACCEPT_ENTRUST", [listId]);
        }

        static receiveRewards(listId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_RECEIVE_REWARDS", [listId, handler]);
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_entrust_finish"]);
        }

        static getCurrentEntrust() {
            return EntrustDataMgr.getInstance().getCurrentEntrust();
        }

        static acceptComplete(entrust) {
            EntrustDataMgr.getInstance().updataOne(entrust);
        }

        static startCheck() {
            EntrustSystem.getAllEntrusts(Laya.Handler.create(EntrustSystem, function () {
                let arr = EntrustSystem.getEntrusts();
                arr.forEach(element => {
                    if (element.getStatus() == 0 || element.getStatus() == 2) {
                        SpecialCatMgr.getInstance().pushCat(element);
                    }
                });
            }));
        }

        static setCurrentEntrustId(id) {
            EntrustDataMgr.getInstance().setCurrentEntrustId(id);
        }


    }

    class TaskData {

        constructor(taskId) {
            this.config = ConfigMgr$1.getInstance().getItemById("Task", taskId);
        }

        getId() {
            return this.config.Id;
        }

        getName() {
            return this.config.Name;
        }

        getDesc() {
            return this.config.Desc;
        }

        setProgress(current, target) {
            if(current==this.current&&target==this.target&&this.rewarded!=true)
                return false;
            this.current = current;
            this.target = target;
            this.rewarded = false;
            return true;
        }

        getCurrent() {
            return this.current?(this.current>this.getTarget()?this.getTarget():this.current):0;
        }

        getTarget() {
            return this.target?this.target:this.config.ConditionValue;
        }

        getRewardType() {
            return this.config.RewardType;
        }

        getReward() {
            return this.config.Reward;
        }

        getStatus() {
            if(this.rewarded)
                return Constants.TaskStatus.kRewarded;
            else if(this.getCurrent()>=this.getTarget())
                return Constants.TaskStatus.kCompleted;
            else
                return Constants.TaskStatus.kRunning;
        }

        setRewarded() {
            if(this.rewarded)
                return false;
            this.rewarded = true;
            this.current = this.getTarget();
            return true;
        }

    }

    class TaskDataMgr {

        constructor() {
            this.dailyTaskMap = new Map();
            this.trophyTaskMap = new Map();
            this.initTasks();
        }

        initTasks() {
            let config = ConfigMgr$1.getInstance().getConfig("TaskList");
            let outSelf = this;
            config.forEach(function(taskList, id) {
                let taskMap = outSelf.dailyTaskMap;
                if(taskList.TaskType==Constants.TaskType.kTrophy)
                    taskMap = outSelf.trophyTaskMap;
                taskMap.set(taskList.Id, new TaskData(taskList.List[0]));
            });
        }

        static getInstance() {
            return TaskDataMgr.instance?TaskDataMgr.instance:TaskDataMgr.instance = new TaskDataMgr();
        }

        setDirty(dirtyFlag) {
            this.dirtyFlag = dirtyFlag;
        }

        hasCompletedTask() {
            if(this.dirtyFlag==1)
                return true;
            if(this.dirtyFlag==-1)
                return false;
            let ret = this.hasCompletedDailyTask()||this.hasCompletedTrophyTask();
            return ret;
        }

        hasCompletedDailyTask() {
            let ret = false;
            this.dailyTaskMap.forEach(function(task, listId){
                if(task.getStatus()==Constants.TaskStatus.kCompleted)
                    ret = true;
            });
            return ret;
        }

        hasCompletedTrophyTask() {
            let ret = false;
            this.trophyTaskMap.forEach(function(task, listId){
                if(task.getStatus()==Constants.TaskStatus.kCompleted)
                    ret = true;
            });
            return ret;
        }

        getCurrentTask(taskListId) {
            let config = ConfigMgr$1.getInstance().getItemById("TaskList", taskListId);
            let taskMap = this.dailyTaskMap;
            if(config.TaskType==Constants.TaskType.kTrophy)
                taskMap = this.trophyTaskMap;
            return taskMap.get(taskListId);
        }

        updateTask(taskListId, taskId, current, target, checklist) {
            if(checklist)
                checklist.delete(taskListId);
            let config = ConfigMgr$1.getInstance().getItemById("TaskList", taskListId);
            if(config==null) {
                console.error("error taskListId: ", taskListId, " not exists");
                return false;
            }
            let taskMap = this.dailyTaskMap;
            if(config){
               if (config.TaskType==Constants.TaskType.kTrophy)
                taskMap = this.trophyTaskMap;
            }
            else
            {
                console.log("TaskList error");
            }
            let task = taskMap.get(taskListId);
            if(!task) {
                console.log(" error,Task is null");
                return false;
            }
            if(task.getId()==taskId){
                return task.setProgress(current, target);
            }else {
                task = new TaskData(taskId);
                task.setProgress(current, target);
                taskMap.set(taskListId, task);
                return true;
            }
        }

        getChecklist() {
            let checklist = new Map();
            let config = ConfigMgr$1.getInstance().getConfig("TaskList");
            config.forEach(function(taskList, id) {
                checklist.set(id, true);
            });
            return checklist;
        }

        setUnUpdatedTaskCompleted(checklist) {
            let ret = false;
            if(checklist){
                let outSelf = this;
                checklist.forEach(function(v, taskListId){
                    if(v){
                        let config = ConfigMgr$1.getInstance().getItemById("TaskList", taskListId);
                        let taskMap = outSelf.dailyTaskMap;
                        if(config.TaskType==Constants.TaskType.kTrophy)
                            taskMap = outSelf.trophyTaskMap;
                        let task = taskMap.get(taskListId);
                        if(task.getId()==config.List[config.List.length-1]){
                            if(task.setRewarded())
                                ret = true;
                        }else {
                            task = new TaskData(config.List[config.List.length-1]);
                            task.setRewarded();
                            taskMap.set(taskListId, task);
                            ret = true;
                        }
                    }
                });
            }
            return ret;
        }

        clear() {
            this.dailyTaskMap.clear();
            this.trophyTaskMap.clear();
            this.initTasks();
        }

        getCurrentUnlock(taskListId) {
            let config = ConfigMgr$1.getInstance().getItemById("TaskList", taskListId);
            let unLock = true;
            let registerDays=PlayerSystem.getRegisterDays();
            if(config.RegistrationDate){
                if(config.RegistrationDate.length==2){
                    if(registerDays<config.RegistrationDate[0]||registerDays>config.RegistrationDate[1]){
                        unLock=false;
                    }
                }else if(config.RegistrationDate.length==1){
                    if(registerDays<config.RegistrationDate[0]){
                        unLock=false;
                    }
                }
            }
            return unLock;
        }

    }

    class TaskSystem {
        static init() {
            if (Laya.Browser.onMiniGame) {
                wx.onShow(function (res) {
                    TaskSystem.collectionTask(res);
                });
            }
        }

        static clear() {
            TaskDataMgr.getInstance().clear();
        }

        static setDirty(dirty) {
            TaskDataMgr.getInstance().setDirty(dirty);
        }

        static updateTasks(tasks) {
            TaskDataMgr.getInstance().setDirty(null);
            let dailyTasks = tasks.dailyTasks;
            let trophyTasks = tasks.trophyTasks;
            let checklist = TaskDataMgr.getInstance().getChecklist();
            let needUpdate = false;
            [dailyTasks, trophyTasks].forEach(taskLists=>{
                if(taskLists){
                    Object.getOwnPropertyNames(taskLists).forEach(key=>{

                        let task = taskLists[key];
                        let taskId = task.id;
                        let taskListId = task.taskListId;
                        let current = task.current;
                        let target = task.target;
                        needUpdate = TaskDataMgr.getInstance().updateTask(
                            taskListId, taskId, current, target , checklist)
                            || needUpdate;
                    });
                }
            });
            needUpdate = TaskDataMgr.getInstance().setUnUpdatedTaskCompleted(checklist) || needUpdate;
            if(needUpdate) {
                EventMgr.getInstance().postEvent("UPDATE_TASKS");
            }

            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["task"]);
        }

        static hasCompletedTask() {
            return TaskDataMgr.getInstance().hasCompletedTask();
        }

        static hasCompletedDailyTask() {
            return TaskDataMgr.getInstance().hasCompletedDailyTask();
        }

        static hasCompletedTrophyTask() {
            return TaskDataMgr.getInstance().hasCompletedTrophyTask();
        }

        static getCurrentTask(taskListId) {
            return TaskDataMgr.getInstance().getCurrentTask(taskListId);
        }

        static getAllTasks(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_ALLTASKS", [handler]);
        }

        static getDailyTaskReward(taskId, fetchType, handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_DAILY_TASK_REWARD", [taskId, fetchType, handler]);
            SyncSystem.startOperatingUpdate();
        }

        static getTrophyTaskReward(taskId, fetchType, handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_TROPHY_TASK_REWARD", [taskId, fetchType, handler]);
            SyncSystem.startOperatingUpdate();
        }

        static getCurrentUnlock(taskListId) {
            let unlock = TaskDataMgr.getInstance().getCurrentUnlock(taskListId);
            return unlock;
        }

        static collectionTask(obj) {
            if (obj && obj.scene) {
                if (obj.scene == 1089) {
                    EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["COLLECT_APP", 1, null]);
                } else if (obj.scene == 1131) {
                    EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["COLLECT_WINDOW", 1, null]);
                } else if (obj.scene == 1023) {
                    EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["COLLECT_DESKTOP", 1, null]);
                }
            }
        }

    }

    class RedPointMgr {

        constructor() {
            this.redPointMap = new Map();
            EventMgr.getInstance().registEvent("FISH_REALLY_CHANGE", this, this.refreshAll);
            EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.refreshAll);
        }

        static getInstance() {
            return RedPointMgr.instance?RedPointMgr.instance:RedPointMgr.instance = new RedPointMgr();
        }

        registerObject(redPointType, object) {
            let map = this.redPointMap.get(redPointType);
            if(map==null){
                map = new Map();
                this.redPointMap.set(redPointType, map);
            }
            map.set(object, true);
        }

        unregisterObject(redPointType, object) {
            let map = this.redPointMap.get(redPointType);
            if(map!=null){
                map.delete(object);
            }
        }

        refreshRedPoint(redPointType, data) {
            let map = this.redPointMap.get(redPointType);
            // console.debug("refresh red point map", map);
            if(map!=null){
                map.forEach((v, object)=>{
                    // console.debug("map object", v, object);
                    object.refreshRedPoint(data);
                });
            }
        }

        clear() {
            this.redPointMap.clear();
        }

        refreshAll() {
            this.redPointMap.forEach((value, key) => {
                this.refreshRedPoint(key);
            });
        }

    }

    class RedPointSystem {
        static init() {
            EventMgr.getInstance().registEvent("REDPOINT_REFRESH", null, RedPointSystem.refreshRedPoint);
        }

        static registerObject(redPointType, object) {
            // console.debug("registerObject", redPointType, object);
            RedPointMgr.getInstance().registerObject(redPointType, object);
        }

        static unregisterObject(redPointType, object) {
            // console.debug("unregisterObject", redPointType, object);
            RedPointMgr.getInstance().unregisterObject(redPointType, object);
        }

        static refreshRedPoint(redPointType, data) {
            // console.debug("refreshRedPoint", redPointType);
            RedPointMgr.getInstance().refreshRedPoint(redPointType, data);
        }

        static clear() {
            RedPointMgr.getInstance().clear();
        }

    }

    let DefaultFrameInterval = 50;  // milliseconds
    class SceneDirector {
        start() {
            this._frameCounter = 0;
            this._actime = 0;
            this._frameInterval = DefaultFrameInterval;
            this._paused = false;
            this._interpreter.start();
            this._simulator.start();
        }

        pause() {
            this._paused = true;
        }

        update(dt) {
            let frameInterval = this._frameInterval;
            let actime = this._actime + dt;
            let simulator = this._simulator;
            let interpreter = this._interpreter;
            this._actime = actime;
            while(actime>=frameInterval) {
                actime = actime - frameInterval;
                this._actime = actime;
                if(this._paused) {
                    return "paused";
                }
                this._frameCounter ++;
                if(simulator) {
                    simulator.tick(frameInterval, this._frameCounter);
                }
                if(interpreter) {
                    interpreter.update(this._frameCounter);
                }
            }
        }

        flush() {
            this._interpreter.flush();
        }

        setSimulator(simulator) {
            this._simulator = simulator;
        }

        //解释器 调用后端接口与 改变表现
        setInterpreter(interpreter) {
            this._interpreter = interpreter;
        }

        //1 正常 2快进
        setInterpreterMode(mode) {
            this._interpreter.setMode(mode);
        }
    }

    class Context {

        setRecorder(recorder) {
            this._recorder = recorder;
        }

        getRecorder() {
            return this._recorder;
        }

        setRandomizer(randomizer) {
            this._randomizer = randomizer;
        }

        getRandomizer() {
            return this._randomizer;
        }

        random() {

        }

        randomExt(num) {

        }

        newTimeline(objId, typeName) {
            this._recorder.newTimeline(objId, typeName);
        }

        recordEvent(objId, evt, data) {
            this._recorder.recordEvent(objId, evt, data);
        }

        recordEventNextFrame(objId, evt, data) {
            this._recorder.recordEventNextFrame(objId, evt, data);
        }

        recordMetaEvent(objId, evt, data, typeName) {
            this._recorder.recordMetaEvent(objId, evt, data, typeName);
        }

        removeTimeline(objId) {
            this._recorder.setTimelineRemovable(objId);
        }

        newMainLine() {
            if(!this._mainlineCreated){
                this._recorder.newTimeline("Main", "MainLine");
                this._mainlineCreated = true;
            }
        }

        recordMainEvent(evt, data) {
            this.newMainLine();
            this._recorder.recordEvent("Main", evt, data);
        }

    }

    class SceneSimulator {

        constructor() {
        }

        start() {
            this._context = new Context();
            this._context.setRecorder(this._recorder);
            this._recorder.start();
            this._logic.start(this._context);
        }

        setLogic(logic) {
            this._logic = logic;
        }

        setRecorder(recorder) {
            this._recorder = recorder;
        }

        setController(controller) {
            this._controller = controller;
        }

        tick(frameInterval, frameCounter) {
            this._recorder.gotoFrame(frameCounter);

            this._processInputs(this._controller, this._logic);

            this._controller.cleanUp();

            this._logic.step(frameInterval);

        }

        _processInputs(controller, logic) {
            if(!logic.isReadyForInput()) {
                return
            }

            while(true) {
                let input = controller.nextInput();
                if(input==null) {
                    return;
                }
                logic.handlerInput(input);
            }
        }
    }

    class Controller {
        constructor() {
            // this._inputHistory = new Array();
            this._inputs = new Array();
            this._processingIndex = 0;
        }

        sendOpCommand(op, args, handler) {
            if(this._inputs==null) {
                return null;
            }
            let input = {
                "frame"  : this._currentFrame,
                "op"     : op,
                "args"   : args,
                "handler": handler
            };

            this._inputs.push(input);
            return true
        }

        nextInput() {
            return this._inputs[this._processingIndex++];
        }

        cleanUp() {
            this._inputs.length = 0;
            this._processingIndex = 0;
        }

    }

    class Recorder {
        //recorder
        start() {
            this._timelineDict = new Map();
            this._timelineArray = new Array();
            this._eventsOnFrames = new Map();
            this._currentFrame = 0;
        }

        gotoFrame(frame) {
            this._currentFrame = frame;
        }

        getCurrentFrame() {
            return this._currentFrame;
        }

        newTimeline(objId, typeName) {
            return this.newTimelineAtFrame(objId, this._currentFrame, typeName);
        }

        recordEvent(objId, evt, data) {
            this.recordEventAtFrame(objId, this._currentFrame, evt, data);
        }

        recordEventNextFrame(objId, evt, data) {
            this.recordEventAtFrame(objId, this._currentFrame + 1, evt, data);
        }

        recordMetaEvent(objId, evt, data, typeName) {
            this.newTimeline(objId, typeName);
            this.recordEvent(objId, evt, data);
        }

        newTimelineAtFrame(objId, frame, typeName) {
            if(this._timelineDict.get(objId)){
                console.debug("ERROR!!!!!!objId not unique", objId);
                return;
            }
            let timeline = {"id":objId, "type":typeName, "start":frame, "end":frame};
            this._timelineDict.set(objId, timeline);
            this._timelineArray.push(timeline);
            return timeline;
        }

        recordEventAtFrame(objId, frame, evt, data) {
            let timeline = this._timelineDict.get(objId);
            if(timeline==null){
                console.debug("ERROR!!!!!!objId not exists", objId);
                return;
            }
            if(timeline.ended==null||timeline.ended<frame) {
                timeline.ended = frame;
            }

            let frameEvents = this._eventsOnFrames.get(frame);
            if(frameEvents==null){
                frameEvents = new Array();
                this._eventsOnFrames.set(frame, frameEvents);
            }
            frameEvents.push({"id":objId, "e":evt, "d":data});
        }

        //provider
        allNewTimelinesAtFrame(frame) {
            let newTimelines = null;
            let timelineArray = this._timelineArray;
            for(let i = 0; i < timelineArray.length; ++i) {
                let tl = timelineArray[i];
                if(tl.start==frame){
                    if(newTimelines==null) {
                        newTimelines = new Array();
                    }
                    newTimelines.push([tl.id, tl.type]);
                }
            }
            return newTimelines;
        }

        allRecordsAtFrame(frame) {
            let frameEvents = this._eventsOnFrames.get(frame);
            if(frameEvents==null) {
                return null;
            }

            let records = new Array();
            for(let i = 0; i< frameEvents.length; ++i) {
                let event = frameEvents[i];
                records.push([event.id, event.e, event.d]);
            }
            return records;
        }

        //清理记录

        setTimelineRemovable(objId) {
            let timeline = this._timelineDict.get(objId);
            timeline.removable = true;
        }

        cleanUpBeforeFrame(frame) {
            for(const [f, events] of this._eventsOnFrames) {
                if(f < frame){
                    this._eventsOnFrames.delete(f);
                }
            }
            let timelineArray = this._timelineArray;
            let timelineDict = this._timelineDict;
            for(let i = timelineArray.length-1; i >=0 ; --i) {
                let tl = timelineArray[i];
                if(tl.removable && tl.start<frame){
                    timelineArray.splice(i,1);
                    timelineDict.delete(tl.id);
                }
            }
        }

    }

    class TLInterpreter {

        setId(id) {
            this._id = id;
        }

        getId() {
            return this._id;
        }

        setInterpreter(interpreter) {
            this._interpreter = interpreter;
        }

        getInterpreter() {
            return this._interpreter;
        }

        getTLInterpreterById(id) {
            if(this._interpreter==null){
                return null;
            }
            return this._interpreter.findTLInterpreter(id);
        }

        _removeSelf() {
            if(this._interpreter==null){
                return null;
            }
            return this._interpreter.removeTLInterpreter(this._id);
        }

        doAction(action, args, mode) {
            let func = this[action];
            if(func==null){
                console.debug("warning", "'"+this._id+"' skipped action '"+ action + "'");
                return;
            }
            return func.bind(this)(action, args, mode)
        }

    }

    class MainTLInterpreter extends TLInterpreter {
        DropFish(action, args, mode) {
            let buildingId = args.buildingId;
            let reward     = args.reward;
            let sceneId    = args.sceneId;
            let spaceId = args.spaceId;
            let multiple = args.multiple;
            EventMgr.getInstance().postEvent("UPDATE_DROPFISH", [buildingId, sceneId, spaceId, multiple]);
        }

        DropStar(action, args, mode) {
            let buildingId = args.buildingId;
            let reward     = args.reward;
            let sceneId    = args.sceneId;
            // EventMgr.getInstance().postEvent("UPDATE_DROPFISH", [buildingId, sceneId]);
        }

        flush(){

        }

    }

    class EntrustCat extends BaseCat {
        constructor(data) {
            super();
            this.data = data;
            this._registerClick();
            this.panel.mouseEnabled = false;
            this.catAnimgo = new Laya.Skeleton();
            this.catAnimgo.visible = false;
            this.catAnimgo.load("anim3/UIdonghua.sk", Laya.Handler.create(this, function () {
                this.catAnimgo.play("tanhao1", true);
            }));
            this.addChild(this.catAnimgo);
        }

        get zOrderOffset() {
            return this._zOrderOffset;
        }

        set zOrderOffset(offset) {
            this._zOrderOffset = offset;
            this.yy = this.y;
        }

        posInMap(x, y) {
            this.x = x;
            this.yy = y;
        }

        get yy() {
            return this.y;
        }

        set yy(y) {
            this.y = y;
            this.zOrder = Math.floor(y) + this._zOrderOffset;
        }

        _registerClick() {
            let panel = new Laya.Panel();
            panel.mouseEnabled = true;
            // panel.bgColor = "#c3504e";
            // panel.alpha=0.5;
            panel.width = 150;
            panel.height = 200;
            panel.centerX = 0.5;
            panel.y = -200;
            panel.on(Laya.Event.CLICK, this, this.onClick);
            panel.zOrder = 1;
            this.panel = panel;
            this.addChild(panel);
        }

        _onLoaded() {
            this._setMaskAnimLoaded();
        }

        _setMaskAnimLoaded() {
            if (this._tips_visible != null) {
                this.catAnimgo.visible = this._tips_visible;
                //this.menuTipPrefab.visible = this._tips_visible;
                this._tips_visible = null;
            }
            if (this._type != null) {
                this.typeChange(this._type);
            }
        }

        set tipsVisible(visible) {
            if (this.catAnimgo)
                this.catAnimgo.visible = visible;

            else
                this._tips_visible = visible;
        }
        showHideTips() {
            this.tipsVisible = true;
            if ( this.aniType == 1) {
                this.catAnimgo.play("tanhao1", false, true);
                this.catAnimgo.on(Laya.Event.STOPPED, this, function () {
                    this.catAnimgo.play("tanhao2", true, true);
                });
                Laya.timer.once(3000, this, function () {
                    this.tipsVisible = false;
                });
            }
            else {
                this.catAnimgo.play("wenhao1", false, true);
                this.catAnimgo.on(Laya.Event.STOPPED, this, function () {
                    this.catAnimgo.play("wenhao2", true, true);
                });
            }
        }
        showTips() {
            this.panel.mouseEnabled = true;
            this.showTime = 2500;
          if (this.data.getDataName() == "Entrust") {
                if (this.data.getStatus() == 0) {
                    this.typeChange(1);
                } else {
                    this.typeChange(2);
                }
                this.catAnimgo.y = -170;
                this.catAnimgo.x+=-10;
            } else {
                this.catAnimgo.y = -170;
                this.catAnimgo.x+=-10;
                this.typeChange(1);
            }
            Laya.timer.loop((Number(Const.get("SpecialGameManBubble"))*1000)+ this.showTime, this, this.showHideTips, false);
        }

        typeChange(type) {
            this.aniType=type;
            if (this.catAnimgo == null) {
                this._type = type;
                return;
            }
            if (type == 1) {
                this.catAnimgo.stop();
                this.catAnimgo.visible = true;
                this.catAnimgo.play("tanhao1", false, true);
                this.catAnimgo.on(Laya.Event.STOPPED, this, function () {
                    this.catAnimgo.play("tanhao2", true, true);
                    Laya.timer.once(3000, this, function () {
                        this.tipsVisible = false;
                    });
                });
            } else {
                this.catAnimgo.stop();
                this.catAnimgo.visible = true;
                this.catAnimgo.play("wenhao1", false, true);
                this.catAnimgo.on(Laya.Event.STOPPED, this, function () {
                    this.catAnimgo.play("wenhao2", true, true);
                    Laya.timer.once(3000, this, function () {
                        this.tipsVisible = false;
                    });
                });
            }
            if (this.scaleX < 0) {
                this.catAnimgo.scaleX = -2;
                this.catAnimgo.scaleY = 2;
            } else {
                this.catAnimgo.scaleX = 2;
                this.catAnimgo.scaleY = 2;
            }
        }

        hideMaskAnim() {
            this.catAnimgo.visible = false;
            this.catAnimgo.stop();
            this.panel.mouseEnabled = false;
            Laya.timer.clear(this, this.showHideTips);
        }

        onDestroy() {
            Laya.timer.clear(this, this.showHideTips);
            super.onDestroy();
            if (this.catAnimgo) {
                this.catAnimgo.stop();
            }
        }

        onClick() {
            if (GameContext.Sliding)
                return;
            if (this.data.getDataName() == "Entrust") {
                this.showEntrust();
            } else {
                this.showMiniGame();
                this.data.setCatTalkShow();
            }
        }

        showEntrust() {
            Laya.Dialog.close("ui/EntrustPop.scene");
            Laya.Dialog.open("ui/EntrustPop.scene", true, this.data);
        }

        showMiniGame() {
            if (this.data.getCatTalkShow()) {
                if (this.data.getType() == Constants.MiniGameType.MORA) {
                    Laya.Dialog.close("ui/MoraGamePop.scene");
                    Laya.Dialog.open("ui/MoraGamePop.scene", true, this.data);
                } else if (this.data.getType() == Constants.MiniGameType.COIN) {
                    Laya.Dialog.close("ui/GuessGamePop.scene");
                    Laya.Dialog.open("ui/GuessGamePop.scene", true, this.data);
                } else if (this.data.getType() == Constants.MiniGameType.STRENGTH) {
                    Laya.Dialog.close("ui/PowerGamePop.scene");
                    Laya.Dialog.open("ui/PowerGamePop.scene", true, this.data);
                }
            } else {
                Laya.Dialog.close("ui/MiniGameTalkPop.scene");
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.data]);
            }
        }

    }

    class CatFactory {

        constructor() {
        }

        static getInstance() {
            return CatFactory.instance?CatFactory.instance:CatFactory.instance = new CatFactory();
        }

        createCoffeeCat(catData, parent, initX, initY, completeHandler) {
            let cat = new Cat(catData);
            cat.pos(initX, initY);
            cat.addLoadedHandler(completeHandler);
            Utils.fadein(cat, 500);
            if(parent)
                parent.addChild(cat);
            cat.scaleX = cat.scaleY = Constants.CoffeeBuildingScale;
            cat.playAnim("lick1", true);
            cat.loadAnim(catData.getSource()+".sk", catData.getSkin(), 0);
            return cat;
        }

        createBeachCat(catData, buildingId, spaceId, completeHandler) {
            let cat = new Cat(catData);
            cat.buildingId = buildingId;
            cat.scaleX = cat.scaleY = Constants.BeachBuildingScale;
            cat.spaceId = spaceId;
            cat.addLoadedHandler(completeHandler);
            cat.loadAnim(catData.getSource()+".sk", catData.getSkin(), 0);
            return cat;
        }

        createEntrustCat(args, parent, initX, initY, completeHandler) {
            let cat = new EntrustCat(args);
            cat.pos(initX, initY);
            cat.addLoadedHandler(completeHandler);
            Utils.fadein(cat, 500);
            if (parent)
                parent.addChild(cat);
            // cat.scaleX = cat.scaleY = Constants.CoffeeBuildingScale;
            cat.playAnim("lick1", true);
            if (args.getDataName() == "Entrust") {
                cat.loadAnim("anim2/NPCmao.sk",'skin2');

            } else if (args.getDataName() == "MiniGameMORA") {
                cat.loadAnim("anim2/NPCmao.sk",'skin1');
            } else if (args.getDataName() == "MiniGameCOIN") {
                cat.loadAnim("anim2/NPCmao.sk",'skin3');
            } else if (args.getDataName() == "MiniGameSTRENGTH") {
                let powerGameData = MiniGameSystem.getOtherData().powerGameData;;
                let playerId = powerGameData.playerId;
                let playerLevel = Number(playerId.substr(11));
                cat.loadAnim("anim2/NPCmao.sk", "skin4-" + playerLevel);
            }
            return cat;
        }
    }

    let CatFace = {
        Left: 1,
        Right: -1,
    };
    class CoffeeCatTLInterpreter extends TLInterpreter{

        AddCat(action, args, mode) {
            let owner = null;
            let catId = args;
            let catData = CatSystem.getCatData(catId);
            let initX = catData.getX();
            let initY = catData.getY();

            let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
            let cat = CatFactory.getInstance().createCoffeeCat(
                        catData,
                        scene?scene.root:null, initX, initY, null);
            this._cat = cat;
            this._catId = catId;

            cat._toX = initX;
            cat._toY = initY;
            // cat.enableTouch();
            CoffeeCatMgr.getInstance().addCat(this._cat);
            EventMgr.getInstance().postEvent("COFFEE_CAT_NUMBER_CHANGE");

        }

        RemoveCat(action, args, mode) {
            if(mode==1){
                let owner = this._cat;
                if(owner != null && !owner.destroyed) {
                    CoffeeCatMgr.getInstance().finallyRemove(this._catId);
                    EventMgr.getInstance().postEvent("COFFEE_CAT_NUMBER_CHANGE");
                }
                this._removeSelf();
            }
            this._cat = null;
        }

        MoveTo(action, args, mode) {
            let toX = args.x;
            let toY = args.y;
            let time = args.time;
            let zOrderOffset = args.zOrderOffset;
            let owner = this._cat;
            if(owner != null && !owner.destroyed) {
                owner._toX = toX;
                owner._toY = toY;
                owner.alpha = 1;
                if(zOrderOffset!=null){
                    owner.zOrderOffset = zOrderOffset;
                    owner.updateZOrder();
                }
                let targetPoint = Utils.randomRangePoint({x: toX, y: toY}, 20);
                var curAnim = owner.getCurAnim();
                if (targetPoint.y - owner.y > 0) {
                    if (curAnim != "move1") {
                        owner.playAnim("move1", true);
                    }
                } else if (targetPoint.y - owner.y < 0) {
                    if (curAnim != "move2") {
                        owner.playAnim("move2", true);
                    }
                }
                let face = CatFace.Left;
                if (targetPoint.x - owner.x > 0) {
                    face = CatFace.Right;
                }
                owner.scaleX = Math.abs(owner.scaleX) * face;
                owner.setEmojiScaleX(owner.scaleX);
                if(mode==1){

                    let act = Laya.Tween.to(owner, { x: targetPoint.x, yy: targetPoint.y }, time,
                        null,
                        Laya.Handler.create(this, function() {

                        }), 0);
                    return act;
                }
            }

        }

        RandomStand(action, args, mode) {
            if(mode==1){
                let owner = this._cat;
                let animArray = new Array("lick1", "rest1", "rest2");
                let i = Utils.randomExt(animArray.length);
                let anim = animArray[i];
                if(owner != null && !owner.destroyed) {
                    owner.alpha = 1;
                    // owner.playAnim(anim, true);
                    if(owner.getCurAnim() != anim) {
                        owner.playAnim(anim, true);
                    }
                }
            }
        }

        Wait(action, args, mode) {
            if(mode==1){
                let owner = this._cat;
                if(owner != null && !owner.destroyed) {
                    if(owner.getCurAnim() != "lick1") {
                        owner.playAnim("lick1", true);
                    }
                }
            }
        }

        IntoBuilding(action, args, mode) {
            let owner = this._cat;
            let type = args.type;
            let buildingId = args.buildingId;
            let spaceId = args.spaceId;
            let foodId = args.food;
            let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
            let time = args.time;

            if(owner != null && !owner.destroyed) {
                owner.buildingId = buildingId;
                owner.spaceId = spaceId;
                owner._eventType = type;
                owner._foodId = foodId;
                owner._eventTime = time;

                if(mode==1 && scene){
                    owner.playAnim("lick1");
                    owner.showMaskAnim("tanhao", 400, Laya.Handler.create(this, function() {
                        Utils.fadeout(owner, 500, Laya.Handler.create(this, function() {
                            scene.addCatToBuilding(owner);
                        }));
                    }));
                }
            }
        }

        OutBuilding(action, args, mode) {
            let owner = this._cat;
            if(owner != null && !owner.destroyed) {
                let pos = args.pos;
                owner.buildingId = null;
                owner.spaceId = null;
                owner._eventType = null;
                owner._foodId = null;
                owner._eventTime = null;
                owner._toX = pos.x;
                owner._toY = pos.y;
                if(mode==1){
                    if(owner.catMenuTips){
                        owner.catMenuTips.destroy();
                        owner.catMenuTips = null;
                    }
                    owner.hideMaskAnim();
                    owner.disableTouch();
                    owner.stopLoop(Laya.Handler.create(this, function(){
                        owner.removeSelf();
                        owner.playAnim("lick1");
                        let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
                        if(scene) {
                            let buildingExtInfo = scene.getSpaceInfo(args.buildingId, args.spaceId);
                            if(buildingExtInfo)
                                buildingExtInfo.showAll();
                            scene.root.addChild(owner);
                        }
                        owner.posInMap(pos.x, pos.y);
                        let zOrderOffset = pos.zOrderOffset;
                        if(zOrderOffset!=null) {
                            owner.zOrderOffset = zOrderOffset;
                            owner.updateZOrder();
                        }
                        Utils.fadein(owner, 500);
                    }));
                }
            }
        }

        BackToScene(action, args, mode) {
            let owner = this._cat;
            let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
            if(owner != null && !owner.destroyed) {
                let pos = args.pos;
                owner.buildingId = null;
                owner.spaceId = null;
                owner._eventType = null;
                owner._foodId = null;
                owner._eventTime = null;
                owner._toX = pos.x;
                owner._toY = pos.y;
                if(mode==1) {
                    owner.clearLoopEndHandler();
                    owner.clearCompleteHandler();

                    if(scene && owner.parent!=scene.root) {
                        owner.removeSelf();
                        owner.playAnim("lick1");
                        if(args.buildingId && args.spaceId) {
                            let buildingExtInfo = scene.getSpaceInfo(args.buildingId, args.spaceId);
                            if(buildingExtInfo)
                                buildingExtInfo.showAll();
                        }
                        scene.root.addChild(owner);
                        owner.posInMap(pos.x, pos.y);
                        let zOrderOffset = pos.zOrderOffset;
                        if(zOrderOffset!=null) {
                            owner.zOrderOffset = zOrderOffset;
                            owner.updateZOrder();
                        }
                        Utils.fadein(owner, 500);
                    }
                }
            }
        }

        CatEat(action, args, mode) {
            let owner = this._cat;
            if(owner != null && !owner.destroyed) {
                owner.playAnim("lick1");
                if(owner.catMenuTips){
                    owner.catMenuTips.TipClicked = true;
                    // owner.catMenuTips.TipVisible = false;
                    owner.catMenuTips.showEatingBar();
                    // owner.showMaskAnim("chifan2");
                    owner.enableTouch();
                }
            }
        }

        WaitEatTimeOut(action, args, mode) {
            let owner = this._cat;
            if(owner.catMenuTips){
                owner.catMenuTips.TipClicked = true;
                owner.catMenuTips.TipVisible = false;
                owner.showMaskAnim("shengqi");
                owner.enableTouch();
            }
        }

        RemoveFade(action, args, mode) {
            let owner = this._cat;
            this._cat = null;
            if(this._catId != null)
                CoffeeCatMgr.getInstance().removeCat(this._catId);
            if(mode==1){
                EventMgr.getInstance().postEvent("COFFEE_CAT_NUMBER_CHANGE");
                if(owner != null && !owner.destroyed && owner.parent!=null){
                    Utils.fadeout(owner, 500, Laya.Handler.create(CoffeeCatMgr.getInstance(), CoffeeCatMgr.finallyRemove, [this._catId]));
                }else {
                    CoffeeCatMgr.getInstance().finallyRemove(this._catId);
                }
            }
        }

        flush() {

        }

        ShowEmoji(action, args, mode) {
            let owner = this._cat;
            if (owner != null && !owner.destroyed) {
                owner.showMaskAnim(args.anim, 2000, null, args.x, args.y, true);
            }
        }

    }

    let CatFace$1 = {
        Left: 1,
        Right: -1,
    };
    class BeachCatTLInterpreter extends TLInterpreter{

        AddCat(action, args, mode) {
            this._catId = args.catId;
            this._buildingId = args.buildingId;
            this._spaceId = args.spaceId;

            if(mode==1){
                this._createCat();
            }
        }

        _createCat() {
            if(this._catId!=null) {
                console.debug("BeachCatTLInterpreter._createCat: " + String(this._catId));
                console.debug(CatDataMgr.getInstance().catMap);
                let catData = CatSystem.getCatData(this._catId);
                if(catData) {

                    let cat = CatFactory.getInstance().createBeachCat(
                        catData, this._buildingId, this._spaceId, Laya.Handler.create(this, this._addCatToBuildingExt));
                    this._cat = cat;
                    BeachCatMgr.getInstance().addCat(this._cat);
                } else {
                    console.debug("Warning BeachCatTLInterpreter._createCat-->未找到对应的猫的数据:" + String(this._catId));
                }

            }
        }

        _addCatToBuildingExt(cat){
            let scene = SceneMgr.getInstance().getScene(Constants.BeachSceneId);

            if(scene && scene.parent && cat != null && !cat.destroyed && cat.parent==null) {
                // console.debug("_addCatToBuildingExt", 1);
                scene.addCatToBuilding(cat, cat.buildingId, cat.spaceId);
            }
        }

        RemoveCat(action, args, mode) {
            let owner = this._cat;
            if(owner != null && !owner.destroyed) {
                BeachCatMgr.getInstance().finallyRemove(owner.getId());
            }
            if(mode==1 && this._cat) {

            }
            this._cat = null;
            this._catId = null;
            this._removeSelf();
        }

        RemoveFade(action, args, mode) {
            let owner = this._cat;
            if(this._catId!=null)
                BeachCatMgr.getInstance().removeCat(this._catId);
            if(mode==1 && owner != null && !owner.destroyed) {
                let scene = SceneMgr.getInstance().getScene(Constants.BeachSceneId);
                if(scene) {
                    let buildingExtInfo = scene.getSpaceInfo(args.buildingId, args.spaceId);
                    if (buildingExtInfo) {
                        buildingExtInfo.showAll();
                    }
                }
                owner.stopLoop();
                if(owner.parent!=null){
                    Utils.fadeout(owner, 500, Laya.Handler.create(BeachCatMgr.getInstance(), BeachCatMgr.finallyRemove, [this._catId]));
                }else {
                    BeachCatMgr.getInstance().finallyRemove(this._catId);
                }
            }
            this._cat = null;
            this._catId = null;
            this._removeSelf();
        }

        flush() {
            this._createCat();
        }

    }

    class ShowCuteMenuTip {

        constructor(pet, toyId) {
            this.pet = pet;
            this.toyId = toyId;
            Laya.loader.create("prefab/tip/cute_menu_tip.prefab", Laya.Handler.create(this, this._onLoaded));
        }

        _onLoaded(prefabDef) {
            if(this._destroyed){
                return;
            }
            this.menuTipPrefab = prefabDef.create();
            this.menuTipPrefab.ani1.play(0, true);
            this.initCuteMenuTip();
        }

        onEnable() {
            this.__enabled = true;
            if(this.menuTipPrefab){
                if(this.menuTipPrefab.visible) {
                    this.menuTipPrefab.ani1.play(0,true);
                }
            }
        }

        onDisable() {
            this.__enabled = false;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani1.stop();
            }
        }

        destroy() {
            if(this._destroyed)
                return;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.destroy();
                this.menuTipPrefab = null;
            }
            if(this.panel) {
                this.panel.destroy();
                this.panel = null;
            }
            this._destroyed = true;
        }

        set TipClicked(clicked) {
            this._tipClicked = clicked;
        }

        set TipVisible(visible) {
            if(this.menuTipPrefab){
                this.menuTipPrefab.visible = visible;
                if (visible&& this.__enabled) {
                    this.menuTipPrefab.ani1.play(0,true);
                }else {
                    this.menuTipPrefab.ani1.stop();
                }
            }
            else
                this._tipVisible = visible;
        }

        initCuteMenuTip() {
            let pet = this.pet;
            let menuTipPrefab = this.menuTipPrefab;

            this.pet.addChild(this.menuTipPrefab);

            let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", this.toyId);
            let aniBox=this.menuTipPrefab.getChildByName("aniBox");
            let toyTipImage = aniBox.getChildByName("toy_img");
            toyTipImage.skin=toyConfig.Icon;
            this.menuTipPrefab.y = -340.5;
            this.menuTipPrefab.x = -70;

            if(pet.scaleX<0) {
                this.menuTipPrefab.scaleX = -1;
                this.menuTipPrefab.x = 83;
            }

            let panel =  new Laya.Panel();
            // panel.bgColor = "#c3504e";
            panel.width = 200;
            panel.height = 340;
            panel.centerX = 0.5;
            panel.y = -panel.height;
            this.pet.addChild(panel);
            this.panel = panel;

            if(this._tipVisible==false){
                menuTipPrefab.visible = false;
                this.menuTipPrefab.ani1.stop();
            }
            this._tipVisible = null;

            panel.on(Laya.Event.CLICK, this, this.onTipClicked);
        }

        onTipClicked() {
            if(this._tipClicked == true) {
                return;
            }
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY_CUTE_TOY", {"petId":this.pet.getId()}]);
            // EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_home_cute_play_ball"]);
        }

    }

    class ShowToysMenuTip {

        constructor(pet, toys) {
            this.pet = pet;
            this.toys = toys;
            Laya.loader.create("prefab/tip/toys_menu_tip.prefab", Laya.Handler.create(this, this._onLoaded));
        }

        _onLoaded(prefabDef) {
            if(this._destroyed){
                return;
            }
            this.menuTipPrefab = prefabDef.create();
            this.menuTipPrefab.ani1.play(0, true);
            this.menuTipPrefab.ani2.play(0, true);
            this.menuTipPrefab.ani3.play(0, true);
            this.initToysMenuTip();
        }

        onEnable() {
            this.__enabled = true;
            if(this.menuTipPrefab){
                if(this.menuTipPrefab.visible) {
                    this.menuTipPrefab.ani1.play(0, true);
                    this.menuTipPrefab.ani2.play(0, true);
                    this.menuTipPrefab.ani3.play(0, true);
                }
            }
        }

        onDisable() {
            this.__enabled = false;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
            }
        }

        destroy() {
            if(this._destroyed)
                return;
            if(this.menuTipPrefab){
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
                this.menuTipPrefab.destroy();
                this.menuTipPrefab = null;
            }
            this._destroyed = true;
        }

        set TipClicked(clicked) {
            this._tipClicked = clicked;
        }

        set TipVisible(visible) {
            if(this.menuTipPrefab){
                this.menuTipPrefab.visible = visible;
                if (visible&& this.__enabled) {
                    this.menuTipPrefab.ani1.play(0,true);
                    this.menuTipPrefab.ani2.play(0,true);
                    this.menuTipPrefab.ani3.play(0,true);
                }else {
                    this.menuTipPrefab.ani1.stop();
                    this.menuTipPrefab.ani2.stop();
                    this.menuTipPrefab.ani3.stop();
                }
            }
            else
                this._tipVisible = visible;
        }

        initToysMenuTip() {
            let pet = this.pet;
            let menuTipPrefab = this.menuTipPrefab;

            this.pet.addChild(this.menuTipPrefab);

            this.menuTipPrefab.y = -340.5;
            this.menuTipPrefab.x = -70;

            if(pet.scaleX<0) {
                this.menuTipPrefab.scaleX = -1;
                this.menuTipPrefab.x = 83;
            }

            if(this._tipVisible==false){
                menuTipPrefab.visible = false;
                this.menuTipPrefab.ani1.stop();
                this.menuTipPrefab.ani2.stop();
                this.menuTipPrefab.ani3.stop();
            }
            this._tipVisible = null;

            this.setToyMenu(this.menuTipPrefab.getChildByName("toy1"), this.toys[0]);

            this.setToyMenu(this.menuTipPrefab.getChildByName("toy2"), this.toys[1]);

            this.setToyMenu(this.menuTipPrefab.getChildByName("toy3"), this.toys[2]);

        }

        setToyMenu(menu, toy) {
            menu.offAll();
            if(toy) {
                let toyId = toy.NeedToyId;
                menu.visible = true;
                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", toyId);
                let toyTipImage = menu.getChildByName("toy_img");
                toyTipImage.skin=toyConfig.Icon;
                if(PetSystem.hasToy(toyId)) {
                    toyTipImage.gray = false;
                }else {
                    toyTipImage.gray = true;
                }
                menu.on(Laya.Event.CLICK, this, this.onTipClicked, [toyId]);
            }else {
                menu.visible = false;
            }
        }

        onTipClicked(toyId) {
            if(this._tipClicked == true) {
                return;
            }
            if (!PetSystem.hasToy(toyId)) {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon7");
            }
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY_TOY", {"petId":this.pet.getId(), "toyId": toyId}]);
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_home_play_ball"]);
        }

    }

    let dragStartDist = 15;
    class HomePet extends BaseCat {
        constructor(data) {
            super();
            this.petData = data;
            this.speed = data.speed;
            EventMgr.getInstance().registEvent("GUIDE_CHECK_14",this,function(){
                let guide = GuideSystem.getGuide("GuideGroup_14");
                if (guide) {
                    guide.registerObj("btn6", this);
                    guide.registerClick("btn6",this._panel,null,[this]);
            }  
            EventMgr.getInstance().postEvent("GUIDE_COMPLETE_GUIDE", ["GuideGroup_14"]);
        });
        }

        setMapInfo(info) {
            this._unitX = info[0];
            this._unitY = info[1];
            this._maxX = info[2];
            this._maxY = info[3];

            this.yy = this.yy;
        }

        getId() {
            return this.petData.getId();
        }

        posInMap(x, y) {
            this.x = x;
            this.yy = y;
        }

        get yy() {
            return this.y;
        }

        set yy(y) {
            this.y = y;
            // this._setY(y);
            if(this._unitY) {
                this.zOrder = this.mapY+3;
            }
        }

        get mapX() {
            return Math.floor(this.x/this._unitX);
        }

        get mapY() {
            return Math.floor(this.y/this._unitY);
        }

        getRealX(mapX) {
            return (mapX+0.5)*this._unitX;
        }

        getRealY(mapY) {
            return (mapY+0.5)*this._unitY;
        }

        calcMapX(realX) {
            return Math.floor(realX/this._unitX);
        }

        calcMapY(realY) {
            return Math.floor(realY/this._unitY);
        }

        /**

        registerStep(step) {
            if(this.__distroyed) {
                return;
            }
            this._step = step;
            this._startTime = step.startTime?step.startTime : TimeUtils.getTimeNowOnServer();
        }

        doStep() {
            if(this.__distroyed) {
                return;
            }
            if(this._step) {
                Laya.Tween.clearAll(this);
                Laya.timer.clearAll(this);
                let stepInfo = this._step;
                switch(stepInfo.type) {
                    case "Move":
                        this._handleMove(stepInfo);
                        break;
                    case "Remove":
                        this.destroy();
                        break;
                    case "Idle":
                        this._handleIdle(stepInfo);
                        break;
                    case "CommonMenu":
                        this._handleCommonMenu(stepInfo);
                        break;
                    case "CuteMenu":
                        this._handleCuteMenu(stepInfo);
                        break;
                    case "Play":
                        this._handlePlay(stepInfo);
                        break;
                }
            }
        }

        _handleMove(stepInfo) {
            let startX = stepInfo.startX, startY = stepInfo.startY;
            let endX = stepInfo.endX, endY = stepInfo.endY;
            let duration = stepInfo.duration;
            let endTime = this._startTime+duration;
            let now = TimeUtils.getTimeNowOnServer();
            let remainDuration = endTime - now;
            if(remainDuration<=0) {
                this.playMoveAnim(startX, startY, endX, endY);
                this.posInMap(endX, endY);
            }else{
                let x = startX + (endX - startX) * (duration - remainDuration)/duration;
                let y = startY + (endY - startY) * (duration - remainDuration)/duration;
                this.posInMap(x, y);
                this.moveTo(endX, endY, remainDuration);
            }
        }

        _handleIdle(stepInfo) {
            this.playAnim(stepInfo.anim);
        }

        _handleCommonMenu(stepInfo) {

        }

        _handleCuteMenu(stepInfo) {

        }

        _handlePlay(stepInfo) {
            this.playAnim(stepInfo.anim);
        }
        **/// 由于逻辑变化，暂时用不上了，可以考虑用到其他猫里

        moveTo(targetX, targetY, duration) {
            // console.debug("moveTo", this.x, this.y, targetX, targetY, duration);
            this.clearTween();
            let time = duration;
            if(duration==null) {
                let dx = this.x - targetX;
                let dy = this.y - targetY;
                let distance = Math.sqrt(dx * dx + dy * dy);
                time = distance/this.speed * 1000;
            }

            this.playMoveAnim(this.x, this.y, targetX, targetY);

            // console.debug("move Info", dx, dy, distance, this.speed, time);

            Laya.Tween.to(this, { x: targetX, yy: targetY}, time,
                    null,Laya.Handler.create(this, this.next));
        }

        playMoveAnim(x, y, targetX, targetY) {
            let curAnim = this.getCurAnim();
            if(Math.abs(targetY - y) < Math.abs(targetX -x)/2) {
                if (curAnim != "move3") {
                    this.playAnim("move3", true);
                }
            }else if (targetY - y > 0) {
                if (curAnim != "move1") {
                    this.playAnim("move1", true);
                }
            } else if (targetY - y < 0) {
                if (curAnim != "move2") {
                    this.playAnim("move2", true);
                }
            }
            if(targetX - x>0) {
                this.scaleX = -1*Math.abs(this.scaleX);
            }else {
                this.scaleX = Math.abs(this.scaleX);
            }
        }

        startMove(path) {
            this._path = path;
            if(path) {
                let pos = this._path.pop();
                if(pos)
                    this.moveTo(pos[0], pos[1], pos[2]);
            }
        }

        next() {
            if(this._path) {
                let pos = this._path.pop();
                if(pos) {
                    this.moveTo(pos[0], pos[1], pos[2]);
                    return;
                }
                this._path = null;
            }
            this.playAnim("lick1");
        }

        stopMove() {
            this._path = null;
            this.clearTween();
        }

        clearTween() {
            if (this.readyRemove) {
                return;
            }
            Laya.Tween.clearAll(this);
            if(this.alpha!=1) {
                Laya.Tween.to(this, { alpha: 1 }, (1-this.alpha)*500,
                    null,
                    null, 0);
            }
        }

        showMenu(toys) {
            if(this.toysMenuTips)
                this.toysMenuTips.destroy();
            this.toysMenuTips = new ShowToysMenuTip(this, toys);
        }

        showCuteMenu(toy) {
            if(this.cuteMenuTips)
                this.cuteMenuTips.destroy();
            let toyId = toy.NeedToyId;
            this.cuteMenuTips = new ShowCuteMenuTip(this, toyId);
        }

        showIntimacyIncrease(increase) {
            if(increase<1 && increase > 0){
                increase = 1;
            }
            increase = (increase + 0.5)|0;
            if(!this._intimacyAnim){
                let anim = new Spine();
                this._intimacyAnim = anim;
                anim.playAnim("xin", false, true);
                this._intimacyAnim.y -= 30;
                if (!this.showSprite || this.showSprite.destroyed) {
                    this.showSprite = new Laya.Sprite();
                    this.showSprite.scaleX = 1 / this.scaleX;
                    this.addChild(this.showSprite);
                }
                this.showSprite.addChild(anim);
                anim.loadAnim("anim2/haogandu.sk", null, 0);
                Laya.loader.create("prefab/ui/addNum.prefab", Laya.Handler.create(this, function (prefabDef) {
                    this.intimacyAdd = prefabDef.create();
                    this.intimacyAdd.alpha = 0;
                    this.intimacyAdd.value = increase;
                    this.showSprite.addChild(this.intimacyAdd);
                    this.intimacyAdd.scaleX = 2;
                    Laya.timer.once(700,this,function(){
                        this.intimacyAdd.alpha = 1;
                        this.intimacyAdd.x=this._intimacyAnim.parent.x + 120;
                        this.intimacyAdd.y=this._intimacyAnim.parent.y - 230;
                        Laya.Tween.to(this.intimacyAdd,{alpha:0},500,null,Laya.Handler.create(this,function(){
                            EventMgr.getInstance().postEvent("MAY_SHOW_INTIMACYUP");
                        }));
                    });
                    
                }));
                this._intimacyAnim.visible = true;
            }else {
                this.intimacyAdd.alpha = 0;
                this._intimacyAnim.playAnim("xin", false, true);
                this.intimacyAdd.value = increase;
                Laya.timer.once(700,this,function(){
                    this.intimacyAdd.alpha = 1;
                    this.intimacyAdd.x=this._intimacyAnim.parent.x + 120;
                    this.intimacyAdd.y=this._intimacyAnim.parent.y - 230;
                    Laya.Tween.to(this.intimacyAdd,{alpha:0},500,null,Laya.Handler.create(this,function(){
                        EventMgr.getInstance().postEvent("MAY_SHOW_INTIMACYUP");
                    }));
                });
            }
        }

        clearMenus() {
            this.clearTween();
            // if(this._panel)
            //     this._panel.releaseMouseEvent();
            if(this.toysMenuTips) {
                this.toysMenuTips.destroy();
                this.toysMenuTips = null;
            }
            if(this.cuteMenuTips) {
                this.cuteMenuTips.destroy();
                this.cuteMenuTips = null;
            }
        }

        enableTouch(){
            let panel = this._panel;
            if(!panel) {
                panel =  new Laya.Panel();
                // panel.bgColor = "#c3504e";
                panel.width = 200;
                panel.height = 190;
                panel.centerX = 0.5;
                panel.y = -panel.height;
                panel.mouseThrough = false;
                this.addChild(panel);
                this._panel = panel;
            }
            // panel.releaseMouseEvent();
            panel.offAll(Laya.Event.MOUSE_DOWN);
            panel.offAll(Laya.Event.MOUSE_MOVE);
            panel.offAll(Laya.Event.MOUSE_UP);
            panel.offAll(Laya.Event.MOUSE_OUT);
            panel.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            // panel.on(Laya.Event.MOUSE_UP, this, this.onMouseUp, [1]);
            // panel.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp, [2]);
            this._moving = null;
            this._mousing = null;
            this._lastDragTime = null;
        }

        disableTouch(){
            let panel = this._panel;
            if(panel) {
                // panel.releaseMouseEvent();
                panel.offAll(Laya.Event.MOUSE_DOWN);
                panel.offAll(Laya.Event.MOUSE_MOVE);
                panel.offAll(Laya.Event.MOUSE_UP);
                panel.offAll(Laya.Event.MOUSE_OUT);
            }

            this._moving = null;
            this._mousing = null;
            this._lastDragTime = null;
            this.banDrag();
        }

        allowDrag() {
            this._dragable = true;
        }

        banDrag() {
            this._dragable = false;
            if (this._panel) {
                this._panel.releaseMouseEvent();
            }
        }

        onMouseDown(event) {
            if(!event || event.stopPropagation==null)
                return;
            event.stopPropagation();
            console.debug("pet Mouse down");
            if (!this.petData.happy || this.petData.hunger || this.petData.thirst || this.petData.dirty) {
                this._panel.on(Laya.Event.MOUSE_UP, this, this.onMouseUp, [1]);
                this._panel.on(Laya.Event.MOUSE_OUT, this, this.onMouseOut, [2]);
                return;
            }
            if(GameContext.HomelandInteractState == Constants.InteractState.petting) {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY", this.getId()]);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_home_fondle"]);
                let lumao = new Laya.Skeleton();
                this.addChild(lumao);
                lumao.load("anim2/lumao.sk", Laya.Handler.create(this, function () {
                    lumao.once(Laya.Event.STOPPED, this, function () {
                        lumao.removeSelf();
                        lumao.destroy();
                    });
                    lumao.play(0, false);
                }));
                return;
            }else if(GameContext.HomelandInteractState == Constants.InteractState.toy){
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SHOW_PET_MENU", this.getId()]);
                return;
            }
            // EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "STOP_PET", this.getId()]);
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "DRAG_PET", this.getId()]);
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_home_grab"]);
            this._mousing = true;
            this._panel.captureMouseEvent(true);
            this._moving = true;
            if (this.mousePoint) {
                this.mousePoint.x = Laya.stage.mouseX;
                this.mousePoint.y = Laya.stage.mouseY;
            } else {
                this.mousePoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            }

            this._startX = this.x;
            this._startY = this.y;
            this._mouseStartTime = TimeUtils.getTimeNowOnServer();

            this._panel.offAll(Laya.Event.MOUSE_MOVE);
            this._panel.on(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
            this._panel.on(Laya.Event.MOUSE_UP, this, this.onMouseUp, [1]);
            this._panel.on(Laya.Event.MOUSE_OUT, this, this.onMouseOut, [2]);
        }

        onMouseMove(event) {
            if(!event || event.stopPropagation==null)
                return;
            event.stopPropagation();
            if(GameContext.HomelandInteractState != Constants.InteractState.drag) {
                return;
            }
            // console.debug("onMouseMove");
            if(this._mousing) {
                // event.stopPropagation();
                if(this._moving!=null) {
                    let dx = Laya.stage.mouseX-this.mousePoint.x;
                    let dy = Laya.stage.mouseY-this.mousePoint.y;
                    if(this._moving==false) {
                        if(Math.sqrt(dx*dx+dy+dy)>dragStartDist) {
                            this._moving = true;
                        }
                    }
                    if(this._moving) {
                        if(this._dragable) {
                            this.x = this._startX + dx;
                            this.y = this._startY + dy;
                            this.zOrder = 100000;
                        }
                        let time = TimeUtils.getTimeNowOnServer();
                        if(this._lastDragTime == null || time-this._lastDragTime>=1000) {
                            this._lastDragTime = time;
                            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "DRAG_PET", this.getId()]);
                        }
                    }
                }
            }
        }

        onMouseUp(d, event) {
            if(!event || event.stopPropagation==null)
                return;
            event.stopPropagation();
            this._panel.releaseMouseEvent();
            console.log(this.petData.happy);
            if (this.petData.hunger || this.petData.thirst || this.petData.dirty) {
                this._panel.offAll(Laya.Event.MOUSE_UP);
                this._panel.offAll(Laya.Event.MOUSE_OUT);
                this._isPlayHealthy = true;
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY_BE_HEALTHY", { "petId": this.getId() }]);
                return;
            }
            if (!this.petData.happy) {
                this._panel.offAll(Laya.Event.MOUSE_UP);
                this._panel.offAll(Laya.Event.MOUSE_OUT);
                return;
            }
            if(GameContext.HomelandInteractState != Constants.InteractState.drag) {
                return;
            }
            if(this._mousing) {
                console.debug("onMouseUp", d, this._moving), TimeUtils.getTimeNowOnServer(), this._mouseStartTime;
                if(this._moving) {
                    this.yy = this.y;
                    EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "MOVE_PET", {"petId":this.getId(), "x": this.x, "y": this.y}]);
                }else {
                    let time = TimeUtils.getTimeNowOnServer();
                    let duration = time - this._mouseStartTime;
                    if(duration<=1500) {
                        EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY", this.getId()]);
                    }else {
                        EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SHOW_PET_MENU", this.getId()]);
                    }
                }
            }
            this._moving = null;
            this._mousing = null;
            this._lastDragTime = null;
            this._panel.offAll(Laya.Event.MOUSE_MOVE);
            this._panel.offAll(Laya.Event.MOUSE_UP);
            this._panel.offAll(Laya.Event.MOUSE_OUT);
        }

        onMouseOut() {
            this._panel.releaseMouseEvent();
        }

        onDestroy() {
            if(this._panel)
                this._panel.releaseMouseEvent();
            this.disableTouch();
            this.clearMenus();
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
            this.readyRemove = null;
            this.__distroyed = true;
            this._step = null;
            super.onDestroy();
            if(this.cuteMenuTips) {
                this.cuteMenuTips.destroy();
                this.cuteMenuTips = null;
            }
            if(this.toysMenuTips) {
                this.toysMenuTips.destroy();
                this.toysMenuTips = null;
            }
        }

        onEnable(){
            super.onEnable();
            if(this.cuteMenuTips){
                this.cuteMenuTips.onEnable();
            }
            if(this.toysMenuTips) {
                this.toysMenuTips.onEnable();
            }
            this.showHealth();
        }

        onDisable(){
            if(this._panel)
                this._panel.releaseMouseEvent();
            super.onDisable();
            if(this.cuteMenuTips){
                this.cuteMenuTips.onDisable();
            }
            if(this.toysMenuTips) {
                this.toysMenuTips.onDisable();
            }
        }


        clone() {
            let pet = new HomePet(this.petData);
            pet._unitX = this._unitX;
            pet._unitY = this._unitY;
            pet._maxX = this._maxX;
            pet._maxY = this._maxY;
            pet.posInMap(this.x, this.yy);
            pet.loadAnim(this.__path, this.__skin, this.__aniMode);
            pet.playAnim("lick1");
            return pet;
        }

        set scaleX(scale) {
            super.scaleX = scale;
            if (this.showSprite && !this.showSprite.destroyed) {
                this.showSprite.scaleX = 1 / scale;
            }
        }

        get scaleX() {
            return super.scaleX;
        }

        showHealth() {
            if (!this.showSprite) {
                this.showSprite = new Laya.Sprite();
                this.showSprite.mouseEnabled = false;
                this.addChild(this.showSprite);
            }
            let arr = [];
            if (this.petData.hunger) {
                if (!this.hungerImage || this.hungerImage.destroyed) {
                    this.hungerImage = new Laya.Image();
                    this.hungerImage.skin = "homeland/ui/hunger.png";
                    this.hungerImage.anchorX = 0.5;
                    this.hungerImage.anchorY = 0.5;
                    this.hungerImage.scaleX = this.hungerImage.scaleY = 2.5;
                    this.showSprite.addChild(this.hungerImage);
                    this.hungerImage.y = -200;
                }
                this.hungerImage.visible = true;
                arr.push(this.hungerImage);
            } else {
                if (this.hungerImage) {
                    this.hungerImage.destroy();
                    this.hungerImage = null;
                }
            }
            if (this.petData.thirst) {
                if (!this.thirstImage || this.thirstImage.destroyed) {
                    this.thirstImage = new Laya.Image();
                    this.thirstImage.skin = "homeland/ui/thirst.png";
                    this.thirstImage.anchorX = 0.5;
                    this.thirstImage.anchorY = 0.5;
                    this.thirstImage.scaleX = this.thirstImage.scaleY = 2.5;
                    this.showSprite.addChild(this.thirstImage);
                    this.thirstImage.y = -200;
                }
                this.thirstImage.visible = true;
                arr.push(this.thirstImage);
            } else {
                if (this.thirstImage) {
                    this.thirstImage.destroy();
                    this.thirstImage = null;
                }
            }
            if (this.petData.dirty) {
                if (!this.dirtyImage || this.dirtyImage.destroyed) {
                    this.dirtyImage = new Laya.Image();
                    this.dirtyImage.skin = "homeland/ui/dirty.png";
                    this.dirtyImage.anchorX = 0.5;
                    this.dirtyImage.anchorY = 0.5;
                    this.dirtyImage.scaleX = this.dirtyImage.scaleY = 2.5;
                    this.showSprite.addChild(this.dirtyImage);
                    this.dirtyImage.y = -200;
                }
                this.dirtyImage.visible = true;
                arr.push(this.dirtyImage);
            } else {
                if (this.dirtyImage) {
                    this.dirtyImage.destroy();
                    this.dirtyImage = null;
                }
            }
            let startX = (arr.length - 1) * (-30);
            for (let i = 0; i < arr.length; i++) {
                arr[i].x = startX + i * 60;
            }
        }

        playBeHealthy(type, handler) {
            this._isPlayHealthy = true;
            this.disableTouch();
            if (this.readyRemove) {
                return;
            }
            let playAnim = new Spine();
            playAnim.zOrder = 10;
            playAnim.y = -85;
            playAnim.loadAnim("anim2/jiayuanhudong.sk");
            let progress = new Laya.ProgressBar("menu/progress2.png");
            let img = new Laya.Image("menu/kuangdi.png");
            img.width = 55;
            img.height = 10;
            img.sizeGrid = "7,10,6,9";
            progress.sizeGrid = "7,10,6,9";
            progress.addChild(img);
            progress.scale(2.5,2.5);
            progress.width = 55;
            progress.height = 10;
            progress.y = -1;
            progress.value = 0;
            progress.anchorX = 0.5;
            progress.anchorY = 0.5;
            progress.y = 20;
            Laya.Tween.to(progress, { value: 1 }, 2000);
            let dealHandler = Laya.Handler.create(this, function () {
                if (this.readyRemove) {
                    return;
                }
                let point = new Laya.Point(this.x, this.y - 100);
                point = this.parent.localToGlobal(point);
                if (handler) {
                    handler.runWith({
                        "type": type,
                        "point": point
                    });
                }
                playAnim.destroy();
                Laya.Tween.clearAll(progress);
                progress.destroy();
                this.enableTouch();
                this.showHealth();
                this._isPlayHealthy = null;
            });
            playAnim.addCompleteHandler(dealHandler);
            if (type == Constants.HealthyType.HUNGER) {
                playAnim.playAnim("weishi", false);
            } else if (type == Constants.HealthyType.THIRST) {
                playAnim.playAnim("weishui", false);
            } else if (type == Constants.HealthyType.DIRTY) {
                playAnim.playAnim("xizao", false);
            }
            this.showSprite.addChild(playAnim);
            this.showSprite.addChild(progress);
        }

        set readyRemove(value) {
            this._readyRemove = value;
        }

        get readyRemove() {
            return this._readyRemove
        }

        getIsHealthyIng() {
            return this._isPlayHealthy;
        }

    }

    class HomePetTLInterpreter extends TLInterpreter {

        PutPet(action, args, mode) {
            console.debug("PutPet");
            let petId = args.petId;
            let x = args.x;
            let y = args.y;
            let mapInfo = args.mapInfo;
            let inNautical = args.inNautical;
            let petData = PetSystem.getPetInfo(petId);

            let pet = new HomePet(petData);
            pet.posInMap(x, y);
            pet.playAnim("rest1", true);
            pet.loadAnim(petData.getSource()+".sk", petData.getSkin(), 0);

            pet.setMapInfo(mapInfo);
            this._pet = pet;
            this._petId = petId;

            if(mode==1) {
                let scene = SceneMgr.getInstance().getScene(Constants.HomelandId);
                if(scene && scene.root != null) {
                    scene.addPet(pet);

                }
            }
            if(inNautical)
                pet.visible = false;

            HomePetMgr.getInstance().addPet(this._pet);

        }

        RemovePet(action, args, mode) {
            if(mode==1){
                let owner = this._pet;
                if(owner != null) {
                    HomePetMgr.getInstance().removePet(this._petId);
                    owner.alpha = 1;
                    owner.readyRemove = true;
                    Utils.fadeout(owner, 500, Laya.Handler.create(this, function() {
                        HomePetMgr.getInstance().finallyRemove(this._petId);
                    }));
                }
                this._removeSelf();
            }
        }

        Hide(action, args, mode) {
            this._pet.visible = false;
        }

        Show(action, args, mode) {
            this._pet.visible = true;
        }

        //清理宠物身上的弹窗，事件, 计时等等
        clearOwner() {
            if(this._pet) {
                this._pet.clearMenus();
                this._pet.stopMove();
            }
        }

        //禁用点击
        disableTouch() {
            if(this._pet) {
                this._pet.disableTouch();
            }
        }

        //启用点击
        enableTouch() {
            if(this._pet) {
                this._pet.enableTouch();
            }
        }

        Reset(action, args, mode) {
            this.clearOwner();
            this.disableTouch();
            this.enableTouch();
            let x = args.x;
            let y = args.y;
            if(this._pet) {
                this._pet.posInMap(x, y);
            }
        }

        Idle(action, args, mode) {
            this.clearOwner();
            this.enableTouch();
            if(mode==1){
                let owner = this._pet;
                let animArray = new Array("lick1");
                let i = Utils.randomExt(animArray.length);
                let anim = animArray[i];
                if(owner != null) {
                    // owner.playAnim(anim, true);
                    if(owner.getCurAnim() != anim) {
                        owner.playAnim(anim, true);
                    }
                }
            }
        }

        Move(action, args, mode) {
            this.clearOwner();
            this.enableTouch();
            let path = args;
            let owner = this._pet;
            if(mode==1){
                owner.startMove(path);
            }
        }

        MoveIng(action, args, mode) {
            // console.debug("MoveIng",this._pet.x, this._pet.y, args);
        }

        MoveEnd(action, args, mode) {
            // let x = args.x;
            // let y = args.y;
            let owner = this._pet;
            owner.stopMove();
            // if(x!=null && y!=null)
            //     owner.posInMap(x, y);
            // EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "UPDATE_POS", {"petId": this._petId, "x": owner.x, "y": owner.y}]);

        }

        Stop(action, args, mode) {
            let owner = this._pet;
            owner.stopMove();
            owner.playAnim("rest1", true);
        }

        StopEnd(action, args, mode) {
            let owner = this._pet;
            owner.stopMove();
            this.disableTouch();
        }

        StartDrag(action, args, mode) {
            console.debug("StartDrag");
            let anim = args.anim;
            let owner = this._pet;
            owner.stopMove();
            owner.allowDrag();
            if(anim)
                owner.playAnim(anim, true);
        }

        EndDrag(action, args, mode) {
            console.debug("EndDrag");
            let anim = args.anim;
            let x = args.x;
            let y = args.y;
            let owner = this._pet;
            this.disableTouch();
            owner.banDrag();
            if(anim)
                owner.playAnim(anim, true);
            if(x!=null && y!=null)
                owner.posInMap(x, y);
        }

        ShowMenu(action, args, mode) {
            let toys = args.toys;
            let owner = this._pet;
            this.clearOwner();
            this.disableTouch();
            owner.playAnim("lick1");
            owner.showMenu(toys);
        }

        Play(action, args, mode) {
            this.clearOwner();
            let owner = this._pet;
            this.clearOwner();
            this.disableTouch();
            let pet = PetSystem.getPetInfo(this._petId);
            let normalActions = pet.getNormalActions();
            let anim = normalActions[Utils.randomExt(normalActions.length)];
            if(anim)
                owner.playAnim(anim, true);
            else
                owner.playAnim("rest2", true);

            this._actionSound();
        }

        _actionSound() {
            let pet = PetSystem.getPetInfo(this._petId);
            let actionSounds = pet.getActionSound();
            let actionSound = actionSounds[Utils.randomExt(actionSounds.length)];
            if(actionSound)
                SoundManager.getInstance().play(actionSound, 1);
        }

        PlayToy(action, args, mode) {
            this.clearOwner();
            let anim = args.anim;
            let owner = this._pet;
            this.clearOwner();
            this.disableTouch();
            if(anim)
                owner.playAnim(anim, true);
            else
                owner.playAnim("rest2", true);
            this._actionSound();
        }

        CuteMenu(action, args, mode) {
            let toy = args.toy;
            let owner = this._pet;
            this.clearOwner();
            this.disableTouch();
            owner.playAnim("play_beg");
            if(toy)
                owner.showCuteMenu(toy);
        }

        ActionEnd(action, args, mode) {
            if(args.increase!=null && args.increase>0) {
                let owner = this._pet;
                if(owner)
                    owner.showIntimacyIncrease(args.increase);
            }
        }

        CuteActionEnd(action, args, mode) {
            if(args.increase!=null && args.increase>0) {
                let owner = this._pet;
                if(owner)
                    owner.showIntimacyIncrease(args.increase);
            }
        }


        flush() {

        }

        ShowHealth(action, args, mode) {
            let owner = this._pet;
            if (owner != null) {
                owner.showHealth();
            }
        }

        PlayBeHealthy(action, args, mode) {
            let owner = this._pet;
            let type = args.type;
            let handler = args.handler;
            if (owner != null) {
                owner.playBeHealthy(type, handler);
            }
        }

        PlayHealthyEnd(action, args, mode) {
            if(args.increase!=null && args.increase>0) {
                let owner = this._pet;
                if(owner)
                    owner.showIntimacyIncrease(args.increase);
            }
        }

    }

    let CatFace$2 = {
        Left:1,
        Right: -1,
    };
    class CoffeeEntrustCatTLInterpreter extends TLInterpreter {

        AddCat(action, args, mode) {
            let owner = null;
            let initX = 1094;
            let initY = 2574;
            let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
            let cat = CatFactory.getInstance().createEntrustCat(
                args,
                scene ? scene.root : null, initX, initY, null);
            this._cat = cat;
            cat._toX = initX;
            cat._toY = initY;

        }

        RemoveCat(action, args, mode) {
            if (mode == 1) {
                let owner = this._cat;
                if (owner != null) {
                    owner.removeSelf();
                    owner.destroy();
                }
                this._removeSelf();
            }
        }

        MoveTo(action, args, mode) {
            let toX = args.x;
            let toY = args.y;
            let time = args.time;
            let zOrderOffset = args.zOrderOffset;
            let owner = this._cat;
            if (owner) {
                owner._toX = toX;
                owner._toY = toY;
                owner.alpha = 1;
                if (zOrderOffset != null) {
                    owner.zOrderOffset = zOrderOffset;
                    owner.updateZOrder();
                }
                let targetPoint = Utils.randomRangePoint({ x: toX, y: toY }, 20);
                var curAnim = owner.getCurAnim();
                if (targetPoint.y - owner.y > 0) {
                    if (curAnim != "move1") {
                        owner.playAnim("move1", true);
                    }
                } else if (targetPoint.y - owner.y < 0) {
                    if (curAnim != "move2") {
                        owner.playAnim("move2", true);
                    }
                }
                let face = CatFace$2.Left;
                if (targetPoint.x - owner.x > 0) {
                    face = CatFace$2.Right;
                }
                owner.scaleX = Math.abs(owner.scaleX) * face;
                if (mode == 1) {

                    let act = Laya.Tween.to(owner, { x: targetPoint.x, yy: targetPoint.y }, time,
                        null,
                        Laya.Handler.create(this, function () {

                        }), 0);
                    return act;
                }
            }

        }

        RandomStand(action, args, mode) {
            if (mode == 1) {
                let owner = this._cat;
                let anim = "lick1";
                if (owner != null) {
                    owner.alpha = 1;
                    if (owner.getCurAnim() != anim) {
                        owner.playAnim(anim, true);
                    }
                }
            }
        }

        Wait(action, args, mode) {
            if (mode == 1) {
                let owner = this._cat;
                if (owner != null) {
                    owner.scaleX = 1;
                    if (owner.getCurAnim() != "lick1") {
                        owner.playAnim("lick1", true);
                    }
                    this.showTip();
                }
            }
        }


        BackToScene(action, args, mode) {
            let owner = this._cat;
            let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
            if (owner) {
                let pos = args.pos;
                owner.buildingId = null;
                owner.spaceId = null;
                owner._eventType = null;
                owner._foodId = null;
                owner._eventTime = null;
                owner._toX = pos.x;
                owner._toY = pos.y;
                if (mode == 1) {
                    owner.clearLoopEndHandler();
                    owner.clearCompleteHandler();

                    if (scene && owner.parent != scene.root) {
                        owner.removeSelf();
                        owner.playAnim("lick1");
                        if (args.buildingId && args.spaceId) {
                            let buildingExtInfo = scene.getSpaceInfo(args.buildingId, args.spaceId);
                            if (buildingExtInfo)
                                buildingExtInfo.showAll();
                        }
                        scene.root.addChild(owner);
                        owner.posInMap(pos.x, pos.y);
                        let zOrderOffset = pos.zOrderOffset;
                        if (zOrderOffset != null) {
                            owner.zOrderOffset = zOrderOffset;
                            owner.updateZOrder();
                        }
                        Utils.fadein(owner, 500);
                    }
                }
            }
        }

        RemoveFade(action, args, mode) {
            let owner = this._cat;
            if (owner)
                CoffeeCatMgr.getInstance().removeCat(this._catId);
            if (mode == 1) {
                if (owner.parent != null) {
                    Utils.fadeout(owner, 500, Laya.Handler.create(this, this.removeFadeDestroy, [this._catId]));
                } else {
                    this.removeFadeDestroy();
                }
            }
        }

        removeFadeDestroy() {
            let owner = this._cat;
            owner.removeSelf();
            owner.destroy();
        }

        flush() {

        }

        showTip() {
            let owner = this._cat;
            owner.showTips();
        }

        Leave(action, args, mode) {
            let owner = this._cat;
            owner.hideMaskAnim();
        }

        SpecialWait(action, args, mode) {
            if (mode == 1) {
                let owner = this._cat;
                if (owner != null) {
                    owner.scaleX=1;
                    if (owner.getCurAnim() != "lick1") {
                        owner.playAnim("lick1", true);
                    }
                    this.showTip();
                }
            }
        }

    }

    //管理StaffDataMgr
    class StaffSystem {
        static init() {
            EventMgr.getInstance().registEvent("CUSTOMDATA_STAFF_RESTORE", null, StaffSystem.updataParams);
            EventMgr.getInstance().registEvent("FISRST_GET_STAFF", null, StaffSystem.getStaffGroup);
        }

        static updataAllStaff(data) {
            StaffCatDataMgr.getInstance().updataAllStaff(data);
            let staffs = StaffSystem.getAllStaffs();
            staffs.forEach((v, k) => {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.CoffeeSceneId, "PUT_STAFF", v.getId()]);
            });
            EventMgr.getInstance().postEvent("UPDATA_STAFF_DATA");
            EventMgr.getInstance().postEvent("UPDATA_GIFTBTN_SHOW");
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
        }

        static clear() {
            StaffCatDataMgr.getInstance().clear();
        }

        static getAllStaffs() {
            return StaffCatDataMgr.getInstance().getAllStaffs();
        }

        static getStaffData(id) {
            return StaffCatDataMgr.getInstance().getStaffData(id);
        }

        static skill1() {
            EventMgr.getInstance().postEvent("USING_SKILL1");
        }

        static skill2() {
            EventMgr.getInstance().postEvent("USING_SKILL2");
        }

        static skill3() {
            EventMgr.getInstance().postEvent("USING_SKILL3", 1);
        }

        static getCurrendStaffData() {
            let id = StaffCatDataMgr.getInstance().getCurrentId();
            return StaffSystem.getStaffData(id);
        }

        static getNextLevelSkillId(groupId, id) {
            let groupConfig = ConfigMgr$1.getInstance().getItemById("StaffSkillGroup", groupId);
            let skills = JSON.parse(groupConfig.Group1);
            let nextIndex = null;
            if (id == null) {
                return skills[0];
            }
            for (let index = 0; index < skills.length; index++) {
                if (skills[index] == id) {
                    nextIndex = index + 1;
                    break;
                }
            }
            if (nextIndex) {
                return skills[nextIndex];
            }
            return null;
        }

        static skillLevelUp(skillGroupId, staffGroupId, fetchType, handler) {
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
            let dealHandler = Laya.Handler.create(this, function (res) {
                let level;
                switch (skillGroupId) {
                    case 'SkillGroup_1':
                        level = res.group.group.SkillGroup_1.substring(8,10);
                        break;
                    case 'SkillGroup_2':
                        level = res.group.group.SkillGroup_2.substring(8,10);
                        break;
                    case 'SkillGroup_3':
                        level = res.group.group.SkillGroup_3.substring(8,10);
                        break;
                    default:
                        break;
                }
                let str = 'ninjaUpgrade';
                level=Number(level);
                let obj = { 'skill': skillGroupId, 'level': level };
                console.log('ninjaUpgrade',level);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                let staffData = StaffSystem.getStaffData(staffGroupId);
                staffData.updataSkillData(res.group.group);
                let skillData = staffData.getSkillDataByGroupId(skillGroupId);
                skillData.skillLevelUpEndTimer = res.endTimeStamp + 1;
                EventMgr.getInstance().postEvent("UPDATA_STAFF_DATA");
                EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
                if (handler) {
                    handler.runWith(res);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_SKILL_LEVEL_UP", [skillGroupId, staffGroupId, fetchType, dealHandler]);
        }

        static staffLevelUp(staffGroupId, handler) {
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
            let dealHandler = Laya.Handler.create(this, function (res) {
                // StaffSystem.updataStaffData(staffGroupId, res.group);
                StaffSystem.getAllStaffGroupSkillGroup(res);
                if (handler) {
                    handler.runWith(res);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_STAFF_LEVEL_UP", [staffGroupId, dealHandler]);
        }

        static getStaffUnlock() {
            let staff = StaffSystem.getAllStaffs();
            if (staff && staff.size > 0) {
                return true;
            }
            return false;
        }

        static getStaffGroup(handler) {
            let dealHandler = Laya.Handler.create(null, function (res) {
                StaffSystem.getAllStaffGroupSkillGroup(res);
                if (handler) {
                    handler.runWith(res);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_GET_STAFF_GROUP", dealHandler);
        }

        static getStaffSkillGroup(staffGroupId, handler) {
            let dealHandler = Laya.Handler.create(null, function (res) {
                if (handler) {
                    handler.runWith(res);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_GET_STAFF_SKILL_GROUP", [staffGroupId, dealHandler]);
        }

        static getAllStaffGroupSkillGroup(obj) {
            let group = obj.group.group;
            let skillAmount = obj.skillAmount ? obj.skillAmount.skillAmount : null;
            if (group == null) {
                return;
            }
            let allStaff = [];
            let length = Object.getOwnPropertyNames(group).length;
            let i = 0;
            Object.getOwnPropertyNames(group).forEach(staffGroupId => {
                let staffId = group[staffGroupId];
                let amount = 0;
                amount = skillAmount ? skillAmount[staffGroupId] : null;
                let skillHandler = Laya.Handler.create(null, function (res) {
                    if (res.group == null) {
                        console.warn("handler_Id_Change(may)", res);
                        return;
                    }
                    let skillGroup = res.group.group;
                    let endTimeStamp = res.endTimeStamp ? res.endTimeStamp.endTimeStamp : null;
                    allStaff.push({ "staffGroupId": staffGroupId, "staffId": staffId, "skillGroup": skillGroup, "skillAmount": amount, "endTimeStamp": endTimeStamp });
                    i++;
                    if (i == length) {
                        StaffSystem.updataAllStaff(allStaff);
                    }
                });
                StaffSystem.getStaffSkillGroup(staffGroupId, skillHandler);
            });
        }

        static unlockSkill(skillGroupId, staffGroupId, handler) {
            let dealHandler = Laya.Handler.create(null, function (res) {
                let skill = res.skill.group;
                let skillAmount = res.skillAmount.skillAmount;
                StaffSystem.updataSkillDataById(staffGroupId, skill, skillAmount);
                let staffData = StaffSystem.getStaffData(staffGroupId);
                let skillData = staffData.getSkillDataByGroupId(skillGroupId);
                skillData.skillCdTimer = 50;
                if (res.endTimeStamp != null) {
                    skillData.skillLevelUpEndTimer = res.endTimeStamp + 1;
                }
                EventMgr.getInstance().postEvent("UPDATA_STAFF_DATA");
                EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
                if (handler) {
                    handler.runWith(res);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_SELECT_SKILL", [skillGroupId, staffGroupId, dealHandler]);
        }

        static updataStaffDataById(staffGroupId, staffId) {
            let staffData = StaffSystem.getStaffData(staffGroupId);
            staffData.updata(staffId);
        }

        static updataSkillDataById(staffGroupId, skillData, skillAmount) {
            let staffData = StaffSystem.getStaffData(staffGroupId);
            staffData.setSkillAmount(skillAmount);
            staffData.updataSkillData(skillData);
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
        }

        static getSkillUnlock(staffGroupId, skillGroupId) {
            let skillData = StaffCatDataMgr.getInstance().getSkillData(staffGroupId, skillGroupId);
            return skillData.getUnlock();
        }

        static updataParams(data) {
            StaffCatDataMgr.getInstance().updataParams(data);
        }

        static reduceCDTime(skillGroupId, staffGroupId, handler) {
            let dealHandler = Laya.Handler.create(null, function (res) {
                let staffData = StaffSystem.getStaffData(staffGroupId);
                let skillData = staffData.getSkillDataByGroupId(skillGroupId);
                skillData.skillLevelUpEndTimer = res.endTimeStamp + 1;
                EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
                EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
                if (handler)
                    handler.runWith([res]);
            });
            EventMgr.getInstance().postEvent("SERVICE_REDUCE_CD_TIME", [skillGroupId, staffGroupId, dealHandler]);
        }


        static getAllskillAmount() {
            let staffData = StaffSystem.getCurrendStaffData();
            let skillAmount = 0;
            if (staffData) {
                skillAmount = staffData.getSkillAmount();
            }
            return skillAmount;
        }

        static checkRedPoint() {
            let red = false;
            let staffs = StaffCatDataMgr.getInstance().getAllStaffs();
            staffs.forEach(staff => {
                if (staff.getSkillAmount() > 0) {
                    red = true;
                    return;
                }
                let skillLevelNum = staff.getSkillLevelNum();
                if (staff.getNextStaff() && staff.getLvUpCondition() != null && skillLevelNum >= staff.getLvUpCondition() && staff.getPrice() <= PlayerDataMgr.getInstance().getFish()) {
                    red = true;
                    return;
                }
                let skillDatas = staff.getSkillData();
                skillDatas.forEach(skillData => {
                    let skillLevelUpEndTimer = skillData.skillLevelUpEndTimer;
                    if ((skillData.getUnlock() && staff.getSkillLvLimite() > skillData.getLevel() && PlayerDataMgr.getInstance().getFish() >= skillData.getStaffLvUpCost())
                        || (skillLevelUpEndTimer != null && skillLevelUpEndTimer > TimeUtils.getTimeNowOnServer() / 1000)) {
                        red = true;
                    }
                });
            });
            return red;
        }
    }

    class StaffCat extends BaseCat {
        constructor(data) {
            super();
            this.staffData = data;
            this.speed = data.speed;
            this.zOrder = 50;
            this._zOrderOffset = 0;
            this.upOrder = 11000;
            this.downOrder = 0;
            this.showSprite = new Laya.Sprite();
            this.addChild(this.showSprite);
        }

        getId() {
            return this.staffData.getId();
        }

        posInMap(x, y) {
            this.x = x;
            this.yy = y;
        }

        get yy() {
            return this.y;
        }

        set yy(y) {
            this.y = y;
            this.staffData.pos(this.x, y);
            this.zOrder = Math.floor(y) + this._zOrderOffset;
        }

        get zOrderOffset() {
            return this._zOrderOffset;
        }

        set zOrderOffset(offset) {
            this._zOrderOffset = offset;
            this.yy = this.y;
        }

        moveTo(targetX, targetY, duration, index) {
            // console.log("moveTo", this.x, this.y, targetX, targetY, duration);
            if (targetX == this.x && targetY == this.y) {
                this.next();
            }
            // console.debug(this.staffData.currentPathIndex, "============>", index);
            this.clearTween();
            let time = 0;
            // if (duration == null) {
            let dx = this.x - targetX;
            let dy = this.y - targetY;
            let distance = Math.sqrt(dx * dx + dy * dy);
            time = distance / this.speed * 1000;
            // }
            if (index != null) {
                let zOrderOffset = this.zOrderOffset;
                if (index == -1) {
                    for (let j = 0; j < this._path.length; ++j) {
                        if (this._path[j].i != null && this._path[j].i != -1) {
                            index = this._path[j].i;
                            break;
                        }
                    }
                }
                // if (this.staffData.currentPathIndex == 14 && index > this.staffData.currentPathIndex) {
                //     this.showSprite.visible = false;
                // } else {
                //     this.showSprite.visible = true;
                // }
                if (ScenePathMgr.getInstance().isZOrderPathIndex(this.staffData.currentPathIndex)) {
                    if (ScenePathMgr.getInstance().isZOrderPathIndex(index)) {
                        zOrderOffset = this.downOrder;
                        // this.showSprite.visible = false;
                    } else {
                        if (index > this.staffData.currentPathIndex) {
                            zOrderOffset = this.upOrder;
                        } else if (index < this.staffData.currentPathIndex) {
                            zOrderOffset = this.downOrder;
                        }
                    }
                }
                if (index != -1) {
                    this.staffData.currentPathIndex = index;
                }
                this.zOrderOffset = zOrderOffset;
                this.updateZOrder();
            }
            this.playMoveAnim(this.x, this.y, targetX, targetY);
            Laya.Tween.to(this, { x: targetX, yy: targetY }, time,
                null, Laya.Handler.create(this, this.next));
        }

        playMoveAnim(x, y, targetX, targetY) {
            let curAnim = this.getCurAnim();
            if (targetY - y > 0) {
                if (curAnim != "move1") {
                    this.playAnim("move1", true);
                }
            } else if (targetY - y < 0) {
                if (curAnim != "move2") {
                    this.playAnim("move2", true);
                }
            }
            if (targetX - x > 0) {
                this.scaleX = -1 * Math.abs(this.scaleX);
            } else {
                this.scaleX = Math.abs(this.scaleX);
            }
        }

        startMove(path) {
            this._path = path;
            if (path) {
                let pos = this._path.pop();
                if (pos)
                    this.moveTo(pos[0], pos[1], pos[2], pos[3]);
            }
        }

        next() {
            if (this._path) {
                let pos = this._path.pop();
                if (pos) {
                    this.moveTo(pos[0], pos[1], pos[2], pos[3]);
                    return;
                }
                this._path = null;
            }
            this.playAnim("lick1");
        }

        stopMove() {
            this._path = null;
            this.clearTween();
        }

        clearTween() {
            if (this.readyRemove) {
                return;
            }
            Laya.Tween.clearAll(this);
            if (this.alpha != 1) {
                Laya.Tween.to(this, { alpha: 1 }, (1 - this.alpha) * 500,
                    null,
                    null, 0);
            }
        }

        onDestroy() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
            this.readyRemove = null;
            this.__distroyed = true;
            this._step = null;
            super.onDestroy();
        }

        onEnable() {
            super.onEnable();
        }

        onDisable() {
            super.onDisable();
        }


        clone() {
            let staff = new StaffCat(this.staffData);
            staff.posInMap(this.x, this.yy);
            staff.loadAnim(this.__path, this.__skin, this.__aniMode);
            staff.playAnim("lick1");
            return staff;
        }

        set readyRemove(value) {
            this._readyRemove = value;
        }

        get readyRemove() {
            return this._readyRemove
        }

        _playSkill(anim, handler) {
            this.addCompleteHandler(Laya.Handler.create(this, function () {
                this.clearCompleteHandler();
                this.playAnim(anim, false);
                Laya.timer.once(100, this, function () {
                    this.addCompleteHandler(handler);
                });
            }));
            this.playAnim("skill0", false);
        }

        skill1(handler) {
            let dealHandler = Laya.Handler.create(this, function () {
                this.playAnim("lick1");
                if (handler) {
                    handler.run();
                }
            });
            this._playSkill("skill1", dealHandler);
        }

        skill2(handler) {
            let dealHandler = Laya.Handler.create(this, function () {
                this.playAnim("lick1");
                if (handler) {
                    handler.run();
                }
            });
            this._playSkill("skill3", dealHandler);
        }

        skill3(handler) {
            let dealHandler = Laya.Handler.create(this, function () {
                this.playAnim("lick1");
                if (handler) {
                    handler.run();
                }
            });
            this._playSkill("skill2", dealHandler);
        }

        addSkillTip(staffData) {
            if (!this.tipSprite || this.tipSprite.destroyed) {
                this.tipSprite = new Laya.Sprite();
                this.tipSprite.zOrder = 5;
                this.showSprite.addChild(this.tipSprite);
            }
            Laya.loader.create("prefab/tip/StaffSkillTip.prefab", Laya.Handler.create(this, function (prefabDef) {
                if (this._destroyed) {
                    return;
                }
                this.tipSprite.destroyChildren();
                let skillDatas = staffData.getSkillData();
                let size = skillDatas.size;
                if (size == 0) {
                    return;
                }
                let arr = [];
                skillDatas.forEach(skillData => {
                    if (skillData.skillTimer != null) {
                        arr.push(skillData);
                    }
                });
                let startX = 0 - (arr.length - 1) * 32;
                for (let index = 0; index < arr.length; index++) {
                    let skill = arr[index];
                    let tip = prefabDef.create();
                    this.tipSprite.addChild(tip);
                    tip.x = startX;
                    tip.y = -220;
                    tip.addValue(skill);
                    startX += 64;
                }
            }));
        }

        resetSkillTip(staffData) {
            if (this.tipSprite && !this.tipSprite.destroyed) {
                this.tipSprite.destroyChildren();
            }
            this.addSkillTip(staffData);
        }

        set scaleX(value) {
            super.scaleX = value;
            if (this.showSprite && !this.showSprite.destroyed) {
                this.showSprite.scaleX = Math.abs(value) / value;
            }
        }

        get scaleX() {
            return super.scaleX;
        }

        skillPickFish() {
            if (this.staffData.skill1Timer != null) {
                EventMgr.getInstance().postEvent("USING_SKILL1", [this.x, this.y, this.zOrder]);
            }
        }

        showSkillTag() {
            let tag = new Laya.Image("staff/skill3Tag.png");
            tag.scaleX = tag.scaleY = 2.5 / Math.abs(this.scaleX);
            tag.y = -300;
            tag.anchorX = 0.5;
            Laya.Tween.to(tag, { y: -350 }, 500, null, Laya.Handler.create(this, function () {
                tag.destroy();
            }));
            this.showSprite.addChild(tag);
        }

    }

    class StaffCatTLInterpreter extends TLInterpreter {

        PutStaff(action, args, mode) {
            let staffId = args.staffId;
            let x = args.x;
            let y = args.y;
            let staffData = StaffSystem.getStaffData(staffId);

            let staff = new StaffCat(staffData);
            staff.scaleX = staff.scaleY = Constants.CoffeeBuildingScale;
            staff.posInMap(x, y);
            staff.playAnim("lick1", true);
            staff.loadAnim("anim2/renzhemao.sk");

            this._staff = staff;
            this._staffId = staffId;

            if (mode == 1) {
                let scene = SceneMgr.getInstance().getScene(Constants.CoffeeSceneId);
                if (scene && scene.root != null) {
                    scene.root.addChild(staff);
                }
            }
            StaffCatMgr.getInstance().addStaff(this._staff);
        }

        RemoveStaff(action, args, mode) {
            if (mode == 1) {
                let owner = this._staff;
                if (owner != null) {
                    StaffCatMgr.getInstance().removeStaff(this._staffId);
                    owner.alpha = 1;
                    owner.readyRemove = true;
                    Utils.fadeout(owner, 500, Laya.Handler.create(this, function () {
                        StaffCatMgr.getInstance().finallyRemove(this._staffId);
                    }));
                }
                this._removeSelf();
            }
        }

        Hide(action, args, mode) {
            this._staff.visible = false;
        }

        Show(action, args, mode) {
            this._staff.visible = true;
        }

        //清理宠物身上的弹窗，事件, 计时等等
        clearOwner() {
            if (this._staff) {
                this._staff.stopMove();
            }
        }

        Reset(action, args, mode) {
            this.clearOwner();
            let x = args.x;
            let y = args.y;
            if (this._staff) {
                this._staff.posInMap(x, y);
            }
        }

        Idle(action, args, mode) {
            this.clearOwner();
            if (mode == 1) {
                let owner = this._staff;
                let animArray = new Array("lick1");
                let i = Utils.randomExt(animArray.length);
                let anim = animArray[i];
                if (owner != null) {
                    owner.playAnim(anim, true);
                }
            }
        }

        Move(action, args, mode) {
            this.clearOwner();
            let path = args;
            let owner = this._staff;
            if (mode == 1) {
                owner.startMove(path);
            }
        }

        MoveIng(action, args, mode) {
            // console.debug("MoveIng",this._staff.x, this._staff.y, args);
        }

        MoveEnd(action, args, mode) {
            let owner = this._staff;
            owner.stopMove();
        }

        Stop(action, args, mode) {
            let owner = this._staff;
            owner.stopMove();
            owner.playAnim("rest1", true);
        }

        StopEnd(action, args, mode) {
            let owner = this._staff;
            owner.stopMove();
        }

        _actionSound() {
            return;
            let staff = StaffSystem.getStaffData(this._staffId);
            let actionSounds = staff.getActionSound();
            let actionSound = actionSounds[Utils.randomExt(actionSounds.length)];
            if (actionSound)
                SoundManager.getInstance().play(actionSound, 1);
        }

        Skill_1(action, args, mode) {
            this.clearOwner();
            let owner = this._staff;
            let handler = args.handler;
            this.clearOwner();
            owner.skill1(handler);
            this._actionSound();
        }

        Skill_2(action, args, mode) {
            this.clearOwner();
            let owner = this._staff;
            let handler = args.handler;
            this.clearOwner();
            owner.skill2(handler);
            this._actionSound();
        }


        Skill_3(action, args, mode) {
            this.clearOwner();
            let owner = this._staff;
            let handler = args.handler;
            this.clearOwner();
            owner.skill3(handler);
            this._actionSound();
        }

        ResetSkillTip(action, args, mode) {
            let owner = this._staff;
            owner.resetSkillTip(args);
        }

        SkillPickFish(action, args, mode) {
            let owner = this._staff;
            owner.skillPickFish(args);
        }

        ShowSkillTag(action, args, mode) {
            let owner = this._staff;
            owner.showSkillTag(args);
        }

        flush() {

        }

    }

    class TLInterpreterFactory {

        static createTLInterpreter(typeName) {
            let interpreter = null;
            switch(typeName) {
                case "MainLine":
                    interpreter = new MainTLInterpreter();
                    break;
                case "CoffeeCatLine":
                    interpreter = new CoffeeCatTLInterpreter();
                    break;
                case "BeachCatLine":
                    interpreter = new BeachCatTLInterpreter();
                    break;
                case "HomePetLine":
                    interpreter = new HomePetTLInterpreter();
                    break;
                case "EntrustCatLine":
                    interpreter = new CoffeeEntrustCatTLInterpreter();
                    break;
                case "StaffCatLine":
                    interpreter = new StaffCatTLInterpreter();
                    break;
            }
            return interpreter;
        }

    }

    class Interpreter {
        constructor(){
            this._mode = 1; //1 Normal 2 Fastforward
        }

        setMode(mode) {
            this._mode = mode;
        }

        setRecordsProvider(provider) {
            this._recordsProvider = provider;
        }

        getRecordsProvider() {
            return this._recordsProvider;
        }

        addTLInterpreter(id, tlTypeName) {
            let tlInterpreter = TLInterpreterFactory.createTLInterpreter(tlTypeName);

            if(tlInterpreter==null){
                return null;
            }

            tlInterpreter.setId(id);
            tlInterpreter.setInterpreter(this);
            this._timelineInterpreters.set(id, tlInterpreter);
            return tlInterpreter;
        }

        removeTLInterpreter(id) {
            this._timelineInterpreters.delete(id);
        }

        findTLInterpreter(id) {
            if(this._timelineInterpreters) {
                return this._timelineInterpreters.get(id);
            }
        }

        start() {
            this._timelineInterpreters = new Map();
            this._lastIndex = -1;
        }

        update(frame) {
            let lastIndex  = this._lastIndex?this._lastIndex:0;
            if(frame<=lastIndex){
                return;
            }
            for(let i = lastIndex+1; i<=frame; ++i) {
                this._lastIndex = i;
                this._processOneFrame(i);
            }

            //清理记录
            this._recordsProvider.cleanUpBeforeFrame(frame);
        }

        _processOneFrame(frame) {
            let recordsProvider = this._recordsProvider;
            if(recordsProvider==null){
                return;
            }

            // add new timelines
            let newTimelines = recordsProvider.allNewTimelinesAtFrame(frame);
            if(newTimelines!=null){
                newTimelines.forEach(tlInfo => {
                    this.addTLInterpreter(tlInfo[0], tlInfo[1]);
                });
            }

            // interpret records of this frame
            let timelineInterpreters  = this._timelineInterpreters;
            let frameRecords = recordsProvider.allRecordsAtFrame(frame);
            if(timelineInterpreters!=null && frameRecords!=null){
                frameRecords.forEach(record => {
                    let tlInterpreter = timelineInterpreters.get(record[0]);
                    if(tlInterpreter!=null) {
                        tlInterpreter.doAction(record[1], record[2], this._mode);
                    }
                });
            }
        }

        //将mode2 堆积的结果进行最终体现
        flush() {
            if(this._timelineInterpreters){
                this._timelineInterpreters.forEach(tlInterpreter=> {
                    tlInterpreter.flush();
                });
            }
        }

    }

    class Logic {
        start(context) {
            this._context = context;
        }

        step(dt) {}

        handlerInput(input) {}

        isReadyForInput() {
            return true;
        }

    }

    var CatFace$3 = {
        Left: 1,
        Right: -1,
    };

    class CatLogicHelper {
            //获得奖励
        static getFishReward(eventData, valueBase) {
            let completeReward = eventData.CompleteReward;

            let weightArr = new Array();
            let totalWeight = 0;
            for(const key in completeReward) {
                let element = completeReward[key];
                if( element != null && element != "") {
                    let arr =  element.split(":");
                    let weight = Number(arr[0]);
                    let reward = Number(arr[1]);
                    weightArr.push({"w":weight, "reward":reward});
                    totalWeight +=weight;
                }
            }

            let ret = Utils.calcRandomWeight(weightArr, totalWeight);

            return ret?ret.reward*valueBase:null;
        }

        static getStarReward(eventData, valueBase) {
            let ret = null;
            let completeReward = JSON.parse(eventData.SCompleteReward);
            let probability = Math.random();

            if(probability<Number(completeReward.rate)) {
                ret = Number(completeReward.times)*valueBase;
            }
            return ret;
        }

        //随机下一个事件
        static randomNextEvent(sceneId, eventData) {
            let n = eventData.NextEvent.length;
            if(n == 0) {
                return null;
            }
            let arr = new Array();
            for(let i = 0 ; i < eventData.NextEvent.length ; ++i) {
               let event = eventData.NextEvent[i];
               let b = BuildingSystem.randomVisitBuilding(sceneId, event);
               if(b != null) {
                   arr.push(event);
               }
            }
            if(arr.length == 0) {
                return null;
            }
            let m = Utils.randomExt(arr.length);
            let ret = arr[m];
            return ret;
        }

        static randomNextEventInBuilding(buildingData, eventData) {
            let n = eventData.NextEvent.length;
            if(n == 0) {
                return null;
            }
            let arr = new Array();
            for(let i = 0 ; i < eventData.NextEvent.length ; ++i) {
               let event = eventData.NextEvent[i];
               if(buildingData.isSupportAction(event)) {
                   arr.push(event);
               }
            }
            if(arr.length == 0) {
                return null;
            }
            let m = Utils.randomExt(arr.length);
            let ret = arr[m];
            return ret;
        }


        //寻找可以执行动作的空位
        //1 有空位 2 没有空位 3 building消失了或者无载猫功能
        static findCatMoveBlock(buildingId, sceneId) {
            let buildingData = BuildingSystem.getBuildingDataByBid(buildingId);
            if(buildingData == null || buildingData.getCapacity()<=0) {
                return {"t": 3, "s": null};
            }
            let spaces = buildingData.getSpaces();
            if(spaces==null || spaces.length == 0) {
                return {"t": 2, "s": null};
            }
            let space = spaces[Utils.randomExt(spaces.length)];
            if (space != null) {
                return {"t": 1, "s": space};
            }
            return {"t": 2, "s": null};
        }

    }

    //后续改成状态机
    class CoffeeCatRole {
        constructor(data) {
            this.catData = data;
            this.activeAI = false;

            this.eventActionConfig = ConfigMgr$1.getInstance().getConfig("ActionEvent");
            this.targetBuildingData = null;
            this.speed = 180;

            this.pathIndex = -1;
            this.upOrder = 11000;
            this.downOrder = 0;

            this._eventStack = new Array();

        }

        getId() {
            return this.catData.getCatId();
        }

        getConfigId() {
            return this.catData.getConfigId();
        }

        restore(restoreData) {

        }

        getIsActive() {
            return this.activeAI;
        }

        setIsActive(active) {
            this.activeAI = active;
        }

        //获得信息
        getData() {
            return this.catData;
        }

        _popEvent() {
            let info = this._eventStack.pop();
            if (info) {
                this._setNextStep(info.event, info.delay, info.args);
                this.update(0);
            }
        }

        _stackEvent(event, delay, args) {
            this._eventStack.push({ "event": event, "delay": delay, "args": args });
        }

        _setNextStep(event, delay, args) {
            this._scheduleTime = delay;
            this._nextEvent = event;
            this._nextEventArgs = args;
        }


        start(context) {
            let isGuiding = GuideSystem.isGuiding() && (GuideSystem.getCurrentGuideId() == "GuideGroup_3" || GuideSystem.getCurrentGuideId() == "GuideGroup_4");
            if (isGuiding) {
                this.guideStart(context);
                return;
            }
            let p = null;
            p = ScenePathMgr.getInstance().getStartPathPoint();
            this._curPos = p;
            this._curPathIndex = p.i;
            this.zOrderOffset = this.downOrder;
            this.catData.setPos(Utils.randomRangePoint(new Laya.Point(p.x, p.y), 100));
            this._context = context;
            this._state = "start";
            this._setNextStep("onActive", 2100);
            this._context.recordMetaEvent(this.getId(), "AddCat", this.getId(), "CoffeeCatLine");
        }

        update(dt) {
            if (this._scheduleTime && this._scheduleTime > 0) {
                this._scheduleTime -= dt;
                return;
            }
            if (this._nextEvent) {
                let event = this._nextEvent;
                this._nextEvent = null;
                switch (event) {
                    case "onActive":
                        this._onActive(this._nextEventArgs);
                        break;
                    case "gotoTargetIndex":
                        this._gotoTargetIndex(this._nextEventArgs);
                        break;
                    case "gotoTargetIndexAndStand":
                        this._gotoTargetIndexAndStand(this._nextEventArgs);
                        break;
                    // case "gotoBuilding":
                    //     this._gotoBuilding(this._nextEventArgs);
                    //     break;
                    case "waitEmpty":
                        this._waitEmpty(this._nextEventArgs);
                        break;
                    case "intoBuilding":
                        this._intoBuilding(this._nextEventArgs);
                        break;
                    case "outBuilding":
                        this._outBuilding(this._nextEventArgs);
                        break;
                    case "newDecision":
                        this._newDecision(this._nextEventArgs);
                        break;
                    case "remove":
                        this._remove(this._nextEventArgs);
                        break;
                    case "waitEatTimeOut":
                        this._waitEatTimeOut(this._nextEventArgs);
                        break;
                    case "guideWait":
                        this._guideWait(this._nextEventArgs);
                        break;
                }
            }
        }

        eat() {
            if (this._state == "waitEat") {
                let foodId = this._eatInfo[3];
                let food = FoodSystem$1.getFood(foodId);
                if (food.isLocked()) {
                    this._waitEatTimeOut(this._eatInfo);
                    return;
                }
                let income = food.getIncome();
                this._eatInfo[4] = income;
                this._state = "eating";
                this._setNextStep("outBuilding", food.getCookingTime() * 1000, this._eatInfo);
                this._context.recordEvent(this.getId(), "CatEat", null);
            }
        }

        remove() {
            if (!this._context) {
                return true;
            }
            if (this._state != "leaving") {
                this._context.recordEvent(this.getId(), "RemoveCat", null);
                return true;
            }
            return false;
        }

        _onActive() {
            let catAppearLimitB = Number(Const.get("CatAppearLimitB"));
            let catAppearLimitK = Number(Const.get("CatAppearLimitK"));
            let spaceCount = BuildingSystem.getTotalSpaceCountInScene(Constants.CoffeeSceneId);
            let activeCatCount = this._context._roleMgr.getActiveRoleCount();
            if (activeCatCount <= Math.floor(spaceCount * catAppearLimitK) + catAppearLimitB) {
                this._resetToVisitEvent();
                this.setIsActive(true);
            } else {
                this._setNextStep("onActive", 2000);
            }
        }

        _resetToVisitEvent() {
            let eventVisit = this.catData.getEventVisit();
            if (GuideSystem.isGuiding() && GuideSystem.getCurrentGuideId() == "GuideGroup_6") {
                this.guideRole(eventVisit);
            } else {
                this._doActionEvent(eventVisit);
            }
        }

        _doActionEvent(eventString) {
            this._genTarget(eventString);
            let eventData = this.eventActionConfig.get(eventString);
            if (eventData != null) {
                this._doWork(eventData);
                return;
            }
            if (this.targetBuildingData == null) {
                this._doLeave();
                return;
            }
        }

        _doLeave() {
            this.setIsActive(false);
            this._stackEvent("remove");
            this._genPath(0);
            this._gotoTargetIndex();
            this._state = "leaving";
        }

        _remove() {
            CatSystem.removeCatData(this.getId(), Constants.CoffeeSceneId);
            this._context.recordEvent(this.getId(), "RemoveFade", null);
            this._context._roleMgr.removeRole(this.getId());
        }

        _genTarget(eventString) {
            this.targetBuildingData = null;
            let b = BuildingSystem.randomVisitBuilding(Constants.CoffeeSceneId, eventString);
            if (b != null) {
                this.targetBuildingData = b;
            }
        }

        _genPath(targetIndex) {
            let curIndex = this._curPos.i;
            let startIndex = this._curPathIndex;
            let prePath = [curIndex];
            if (curIndex != startIndex) {
                if (this.path && this.pathIndex >= 0) {
                    for (let j = this.pathIndex + 1; j < this.path.length; ++j) {
                        if (this.path[j].i != -1) {
                            startIndex = this.path[j].i;
                            break;
                        } else {
                            prePath.push(this.path[j]);
                        }
                    }
                }
            }
            if (startIndex == this._curPathIndex)
                prePath.length = 0;
            this.path = prePath.concat(ScenePathMgr.getInstance().randomPath(startIndex, targetIndex));
            this.pathIndex = 0;
        }

        _getNextIndexPointInPath() {
            if (this.path && this.pathIndex >= 0) {
                for (let j = this.pathIndex + 1; j < this.path.length; ++j) {
                    if (this.path[j].i != -1) {
                        return this.path[j];
                    }
                }
            }
        }

        _gotoBuilding(targetBuildingData) {
            if (!targetBuildingData) {
                this._doLeave();
                return;
            }
            let targetIndex = targetBuildingData.getLocationData().getPathPointIndex();
            this._genPath(targetIndex);
            this._gotoTargetIndexAndStand();
        }

        _gotoTargetIndexAndStand() {
            let g = Utils.randomExt(100);
            if (g > 97) {
                let time = 3000 + Utils.randomExt(6000);
                this._state = "randomStand";
                this._context.recordEvent(this.getId(), "RandomStand", null);
                this._setNextStep("gotoTargetIndex", time);
            } else {
                this._gotoTargetIndex();
            }
        }

        _gotoTargetIndex() {
            if (this.pathIndex < this.path.length - 1) {
                if (this.pathIndex == -1) {
                    this.pathIndex = 0;
                }
                let prevP = this.path[this.pathIndex];
                this.pathIndex++;
                let p = this.path[this.pathIndex];

                let dx = this.catData.getX() - p.x;
                let dy = this.catData.getY() - p.y;
                let distance = Math.sqrt(dx * dx + dy * dy);
                let time = distance / this.speed * 1000;
                let zOrderOffset = this.zOrderOffset;
                let nextP = p;
                if (nextP.i == -1)
                    nextP = this._getNextIndexPointInPath();
                if (prevP && nextP) {
                    if (ScenePathMgr.getInstance().isZOrderPathIndex(prevP.i)) {
                        if (ScenePathMgr.getInstance().isZOrderPathIndex(nextP.i)) {
                            zOrderOffset = this.downOrder;
                        } else {
                            if (nextP.i > prevP.i) {
                                zOrderOffset = this.upOrder;
                            } else if (nextP.i < prevP.i) {
                                zOrderOffset = this.downOrder;
                            }
                        }
                    }
                }
                // console.debug(prevP?prevP.i:null, nextP?nextP.i:null, zOrderOffset);
                this._curPos = p;
                if (p.i != -1)
                    this._curPathIndex = p.i;
                this.zOrderOffset = zOrderOffset;
                this.catData.setPos(p);
                this._state = "moving";
                this._context.recordEvent(this.getId(), "MoveTo", { "x": p.x, "y": p.y, "time": time, "zOrderOffset": zOrderOffset });
                this._setNextStep("gotoTargetIndexAndStand", time);
            } else {
                this._popEvent();
            }
        }

        _getCurrentPos() {
            let p = this._curPos;
            return { "x": p.x, "y": p.y, "zOrderOffset": this.zOrderOffset };
        }

        //执行动作
        _doWork(eventData) {
            if (eventData.Type == "VISIT") {
                this._doActionEvent(CatLogicHelper.randomNextEvent(Constants.CoffeeSceneId, eventData));
            } else {
                this._stackEvent("waitEmpty", 0, eventData);
                this._gotoBuilding(this.targetBuildingData);
            }
        }

        _waitEmpty(eventData) {
            this._state = "waitEmpty";
            this._context.recordEvent(this.getId(), "Wait", { "anim": "lick1" });
            if (this.targetBuildingData) {
                let buildingId = this.targetBuildingData.getId();
                let info = CatLogicHelper.findCatMoveBlock(buildingId, Constants.CoffeeSceneId);
                if (info.t == 1) {
                    if (info.s != null) {
                        this._setNextStep("intoBuilding", 0, [eventData, buildingId, info.s]);
                    }
                } else if (info.t == 2) {
                    this._setNextStep("waitEmpty", 1000);
                } else {
                    this._setNextStep("intoBuilding", 0);
                }
            } else {
                this._doLeave();
            }
        }

        _intoBuilding(info) {
            if (info && info[0] && info[1]) {
                let eventData = info[0];
                let buildingId = info[1];
                let spaceId = info[2];
                let buildingData = BuildingSystem.getBuildingDataByBid(buildingId);
                if (!buildingData.occupy(spaceId, this.getId())) {
                    this._setNextStep("waitEmpty", 1000);
                    return;
                }
                let type = 1;
                if (eventData.Type == "REST") {
                    type = 2;
                }
                if (type == 1) {
                    //吃饭
                    this._state = "waitEat";
                    let foodType = this.catData.randomFood();
                    let food = FoodSystem$1.getCurFoodInList(foodType);
                    // console.debug('randomfood', foodType, food);
                    info[3] = food.getId();
                    this._eatInfo = info;
                    let time = Number(Const.get("CookingOvertime")) * 1000;
                    if (!(GuideSystem.isGuiding() && (GuideSystem.getCurrentGuideId() == "GuideGroup_3") || GuideSystem.getCurrentGuideId() == "GuideGroup_4")) {
                        this._setNextStep("waitEatTimeOut", time + 900, info);
                    }
                    this._context.recordEvent(this.getId(), "IntoBuilding", { "type": type, "time": info[3] ? info[3] : 10900, "buildingId": buildingId, "spaceId": spaceId, "food": food.getId() });
                } else if (type == 2) {
                    //休息
                    this._state = "rest";
                    this._setNextStep("outBuilding", 6900, info);
                    this._context.recordEvent(this.getId(), "IntoBuilding", { "type": type, "time": 6900, "buildingId": buildingId, "spaceId": spaceId });
                }

            } else {
                // TODO 没building怎么处理 因为换场景肯定容易出现 by sunwen
                this._resetToVisitEvent();
            }
        }

        _waitEatTimeOut(info) {
            this._state = "waitEatTimeOut";
            this._setNextStep("outBuilding", 1000, info);
            this._context.recordEvent(this.getId(), "WaitEatTimeOut", null);
        }

        _outBuilding(info) {//假如猫没了？
            let eventData = info[0];
            let buildingId = info[1];
            let spaceId = info[2];
            let income = info[4];
            let buildingData = BuildingSystem.getBuildingDataByBid(buildingId);
            buildingData.leave(spaceId, this.getId());
            if (this.catData.eventCount == 0) {
                PetSystem.addVisitTimes(this.getConfigId(), 1);
                TaskMainSystem.addProgress("CATS_VISIT", 1);
            }
            if (this._state != "waitEatTimeOut") {
                //增加亲密度
                PetSystem.addIntimacy(this.getConfigId(), eventData.IntimacyUp);

                let reward = CatLogicHelper.getStarReward(eventData, this.catData.getStarBase());

                let obj = this.catData.getDropRate();
                let names = Object.getOwnPropertyNames(obj);
                //掉落鱼干倍数
                let multiple = 1;
                if (names.length > 0) {
                    names.sort(function (a, b) {
                        return Number(b) - Number(a);
                    });
                    let r = Math.random();
                    names.forEach(element => {
                        if (r <= Number(element)) {
                            multiple = obj[element];
                        }
                    });
                }
                if (reward && reward > 0) {
                    SyncSystem.addCachedStar(buildingData.getBuildingBaseId(), reward * multiple);
                    this._context.recordMainEvent("DropStar", { "buildingId": buildingId, "reward": reward, "sceneId": Constants.CoffeeSceneId, "multiple": multiple });
                } else {
                    let id = this.catData.getConfigId();
                    let valueBase = PetSystem.getPetInfo(id).getValueBase();
                    reward = CatLogicHelper.getFishReward(eventData, valueBase);
                    if (reward && reward > 0) {
                        SyncSystem.addCachedFish(buildingData.getBuildingBaseId(), reward * multiple);
                        this._context.recordMainEvent("DropFish", { "buildingId": buildingId, "reward": reward, "sceneId": Constants.CoffeeSceneId, "spaceId": spaceId, "multiple": multiple });
                    }
                }

                // console.debug("outBuilding", reward, income, this._state);
                if (this._state == "eating") {
                    SyncSystem.addEatTimes();
                    TaskMainSystem.addProgress("ORDER", 1);
                    if (income) {
                        SyncSystem.addCachedFish(buildingData.getBuildingBaseId(), income * multiple);
                        this._context.recordMainEvent("DropFish", { "buildingId": buildingId, "reward": income, "sceneId": Constants.CoffeeSceneId, "spaceId": spaceId, "multiple": multiple });
                    }
                }
            } else {
                if (this._state == "waitEatTimeOut") {
                    this._hungry = true;
                }
                this.catData.eventCount = 2;
            }

            this._state = "idle";

            this.catData.eventCount++;

            this._context.recordEvent(this.getId(), "OutBuilding", { "time": 500, "buildingId": buildingId, "spaceId": spaceId, "pos": this._getCurrentPos() });
            this._setNextStep("newDecision", 1000, info);
        }

        _newDecision(info) {
            let eventData = info[0];
            let buildingId = info[1];
            let spaceId = info[2];
            this._context.recordEvent(this.getId(), "BackToScene", { "time": 500, "buildingId": buildingId, "spaceId": spaceId, "pos": this._getCurrentPos() });
            if (this.catData.eventCount >= 2) {
                this._doLeave();
                return;
            } else {
                this._doActionEvent(CatLogicHelper.randomNextEvent(Constants.CoffeeSceneId, eventData));
            }
        }





        destroy() {
            this._context.removeTimeline(this.getId());
        }


        guideRole(eventString) {
            this.targetBuildingData = null;
            let eventData = this.eventActionConfig.get(eventString);
            if (eventData.Type == "VISIT") {
                this._doActionEvent("RestNLeave");
            }
        }

        guideStart(context) {
            let p = null;
            p = ScenePathMgr.getInstance().getBaseNode(7);
            this._curPos = p;
            this._curPathIndex = p.i;
            this.zOrderOffset = this.upOrder;
            this.catData.setPos(new Laya.Point(p.x - 100, p.y + 80));
            this._context = context;
            let eventData = this.eventActionConfig.get("EatNLeave");
            let info = CatLogicHelper.findCatMoveBlock("1060", Constants.CoffeeSceneId);
            if (info.s != null) {
                this._state = "intoBuilding";
                this._setNextStep("intoBuilding", 0, [eventData, "1060", info.s, 10000000]);
                this._context.recordMetaEvent(this.getId(), "AddCat", this.getId(), "CoffeeCatLine");
            } else {
                this._state = "guideWait";
                this._setNextStep("guideWait", 100);
            }
        }

        _guideWait() {
            let eventData = this.eventActionConfig.get("EatNLeave");
            let info = CatLogicHelper.findCatMoveBlock("1060", Constants.CoffeeSceneId);
            if (info.s != null) {
                this._state = "intoBuilding";
                this._setNextStep("intoBuilding", 0, [eventData, "1060", info.s, 10000000]);
                this._context.recordMetaEvent(this.getId(), "AddCat", this.getId(), "CoffeeCatLine");
            } else {
                this._state = "guideWait";
                this._setNextStep("guideWait", 100);
            }
        }

        showEmoji() {
            if (this._state == "eating") {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon50", [], null, 1500, false, true, -200]);
            } if (this._state == "waitEatTimeOut") {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon51", [], null, 1500, false, true, -200]);
            }
            return;
            if (GuideSystem.isGuiding()) {
                return;
            }
            if (this._state == "waitEat") {
                return;
            }
            let anim = null;
            let x = null;
            let y = this.catData.config.BubbleHight;
            if (this.targetBuildingData && this.catData.eventCount < 2) {
                if (Number(this.targetBuildingData.locationData.locationData.Id) >= 107 && Number(this.targetBuildingData.locationData.locationData.Id) <= 109) {
                    anim = "fan";
                } else {
                    anim = "wan";
                }
            } else {
                if (this._hungry) {
                    anim = "xin";
                } else {
                    anim = "xin";
                }
            }
            if (this._state == "rest" && this._nextEvent == "outBuilding") {
                anim = "kaixin";
                if (this.targetBuildingData.locationData.locationData.Id == "101") {
                    if (this._nextEventArgs[2] == 0) {
                        anim=null;
                    }
                } else if (this.targetBuildingData.locationData.locationData.Id == "103") {
                    x = 120;
                    y = -160;
                } else if (this.targetBuildingData.locationData.locationData.Id == "105") {
                    x = 100;
                    y = -20;
                }
            } else if (this._state == "eating") {
                anim = null;
                let catHeadSkin = PetSystem.getPetInfo(this.getConfigId()).getHeadIcon();
                GetNewCatMgr.getInstance().catSkin(catHeadSkin, "正在用餐中，就不打扰了吧");
            }
            if (anim != null) {
                this._context.recordEventNextFrame(this.getId(), "ShowEmoji", { "anim": anim, x: x, y: y });
            }
        }


    }

    class CoffeeRoleMgr {

        constructor() {
            this._roleMap = new Map();
            this._roleList = new Array();
            this._roleStack = new Array();
            EventMgr.getInstance().registEvent("COFFEE_CAT_CLICK", this, this.onCatClick);
        }

        addRole(catData) {
            let role = new CoffeeCatRole(catData);
            this._roleMap.set(String(catData.getCatId()), role);
            this._roleStack.push(role);
            return role;
        }

        removeRole(roleId) {
            this._roleMap.delete(String(roleId));
            for(const key in this._roleList) {
                let role = this._roleList[key];
                if(role.getId() == roleId){
                    role.destroy();
                    this._roleList.splice(key, 1);
                    return role;
                }
            }
            for(const key in this._roleStack) {
                let role = this._roleStack[key];
                if(role.getId() == roleId){
                    role.destroy();
                    this._roleStack.splice(key, 1);
                    return;
                }
            }
        }

        showRole() {
            let role = this._roleStack.shift();
            if(role) {
                this._roleList.push(role);
            }
            return role;
        }

        getRoleById(id) {
            return this._roleMap.get(String(id));
        }

        getRoles() {
            return this._roleList.slice(0);
        }

        getRoleCount() {
            return this._roleMap.size;
        }

        getRoleListCount() {
            return this._roleList.length;
        }

        getStackRoleCount() {
            return this._roleStack.length;
        }

        getActiveRoleCount() {
            let activeCatCount = 0;
            for (const role of this._roleList) {
                if(role.getIsActive()) {
                    activeCatCount++;
                }
            }
            return activeCatCount;
        }

        onCatClick(catId){
            let role=this.getRoleById(catId);
            if(role){
                role.showEmoji();
            }
        }
    }

    //后续改成状态机
    class EntrustCatRole {
        constructor(entrustData) {
            this.entrustData = entrustData;
            this.activeAI = false;

            this.eventActionConfig = ConfigMgr$1.getInstance().getConfig("ActionEvent");
            this.targetBuildingData = null;
            this.speed = 180;

            this.pathIndex = -1;
            this.upOrder = 11000;
            this.downOrder = 0;

            this._eventStack = new Array();
            this.pos = new Laya.Point();
            EventMgr.getInstance().registEvent("ENTRUST_CAT_GO", this, this._doLeave);
        }

        restore(restoreData) {

        }

        getId() {
            return this.entrustData.getCatId();
        }

        getIsActive() {
            return this.activeAI;
        }

        setIsActive(active) {
            this.activeAI = active;
        }

        _popEvent() {
            let info = this._eventStack.pop();
            if (info) {
                this._setNextStep(info.event, info.delay, info.args);
                this.update(0);
            }
        }

        _stackEvent(event, delay, args) {
            this._eventStack.push({ "event": event, "delay": delay, "args": args });
        }

        _setNextStep(event, delay, args) {
            this._scheduleTime = delay;
            this._nextEvent = event;
            this._nextEventArgs = args;
        }


        start(context) {
            let p = null;
            p = ScenePathMgr.getInstance().getStartPathPoint();
            this._curPos = p;
            this._curPathIndex = p.i;
            this.zOrderOffset = this.downOrder;
            this.pos = Utils.randomRangePoint(new Laya.Point(p.x, p.y), 100);
            this._context = context;
            this._state = "start";
            this._setNextStep("onActive", 0);
            this._context.recordMetaEvent(this.getId(), "AddCat", this.entrustData, "EntrustCatLine");
        }

        update(dt) {
            if (this._scheduleTime && this._scheduleTime > 0) {
                this._scheduleTime -= dt;
                return;
            }
            if (this._nextEvent) {
                let event = this._nextEvent;
                this._nextEvent = null;
                switch (event) {
                    case "onActive":
                        this._onActive(this._nextEventArgs);
                        break;
                    case "gotoTargetIndex":
                        this._gotoTargetIndex(this._nextEventArgs);
                        break;
                    case "waitEmpty":
                        this._waitEmpty(this._nextEventArgs);
                        break;
                    case "remove":
                        this._remove(this._nextEventArgs);
                        break;
                }
            }
        }

        remove() {
            if (!this._context) {
                return true;
            }
            if (this._state != "leaving") {
                this._context.recordEvent(this.getId(), "RemoveCat", null);
                return true;
            }
            return false;
        }

        _onActive() {
            this._resetToVisitEvent();
            this.setIsActive(true);
        }

        _resetToVisitEvent() {
            this._stackEvent("waitEmpty");
            this._genPath(4);
            this._gotoTargetIndex();
        }

        _doLeave(currentData) {
            if (this.entrustData != currentData) {
                return;
            }
            this._context.recordEventNextFrame(this.getId(), "Leave", null);
            this.setIsActive(false);
            this._stackEvent("remove");
            this._genPath(0);
            this._gotoTargetIndex();
            this._state = "leaving";
        }

        _remove() {
            CatSystem.removeCatData(this.getId(), Constants.CoffeeSceneId);
            this._context.recordEvent(this.getId(), "RemoveFade", null);
            this._context._roleMgr.removeRole(this.getId());
        }

        _genTarget(eventString) {
            this.targetBuildingData = null;
            let b = BuildingSystem.randomVisitBuilding(Constants.CoffeeSceneId, eventString);
            if (b != null) {
                this.targetBuildingData = b;
            }
        }

        _genPath(targetIndex) {
            let curIndex = this._curPos.i;
            let startIndex = this._curPathIndex;
            let prePath = [curIndex];
            if (curIndex != startIndex) {
                if (this.path && this.pathIndex >= 0) {
                    for (let j = this.pathIndex + 1; j < this.path.length; ++j) {
                        if (this.path[j].i != -1) {
                            startIndex = this.path[j].i;
                            break;
                        } else {
                            prePath.push(this.path[j]);
                        }
                    }
                }
            }
            if (startIndex == this._curPathIndex)
                prePath.length = 0;
            this.path = prePath.concat(ScenePathMgr.getInstance().randomPath(startIndex, targetIndex));
            if (targetIndex == 4) {
                this.path = [
                    { i: 1, x: 1042, y: 2524 },
                    { i: 2, x: 864, y: 1961 },
                    { i: 3, x: 1044, y: 1821 },
                    { i: 4, x: 1174, y: 1755 }
                ];
            } else {
                this.path = [
                    { i: 4, x: 1174, y: 1755 },
                    { i: 3, x: 1044, y: 1821 },
                    { i: 2, x: 864, y: 1961 },
                    { i: 0, x: 1042, y: 2524 }
                ];
            }
            this.pathIndex = 0;
        }

        _getNextIndexPointInPath() {
            if (this.path && this.pathIndex >= 0) {
                for (let j = this.pathIndex + 1; j < this.path.length; ++j) {
                    if (this.path[j].i != -1) {
                        return this.path[j];
                    }
                }
            }
        }


        _gotoTargetIndex() {
            if (this.pathIndex < this.path.length - 1) {
                if (this.pathIndex == -1) {
                    this.pathIndex = 0;
                }
                let prevP = this.path[this.pathIndex];
                this.pathIndex++;
                let p = this.path[this.pathIndex];

                let dx = this.pos.x - p.x;
                let dy = this.pos.y - p.y;
                let distance = Math.sqrt(dx * dx + dy * dy);
                let time = distance / this.speed * 1000;
                let zOrderOffset = this.zOrderOffset;
                let nextP = p;
                if (nextP.i == -1)
                    nextP = this._getNextIndexPointInPath();
                if (prevP && nextP) {
                    if (ScenePathMgr.getInstance().isZOrderPathIndex(prevP.i)) {
                        if (ScenePathMgr.getInstance().isZOrderPathIndex(nextP.i)) {
                            zOrderOffset = this.downOrder;
                        } else {
                            if (nextP.i > prevP.i) {
                                zOrderOffset = this.upOrder;
                            } else if (nextP.i < prevP.i) {
                                zOrderOffset = this.downOrder;
                            }
                        }
                    }
                }
                this._curPos = p;
                if (p.i != -1)
                    this._curPathIndex = p.i;
                this.zOrderOffset = zOrderOffset;
                this.pos = p;
                this._state = "moving";
                this._context.recordEvent(this.getId(), "MoveTo", { "x": p.x, "y": p.y, "time": time, "zOrderOffset": zOrderOffset });
                this._setNextStep("gotoTargetIndex", time);
            } else {
                this._popEvent();
            }
        }

        //执行动作
        _doWork(eventData) {
            if (eventData.Type == "VISIT") {
                this._doActionEvent(CatLogicHelper.randomNextEvent(Constants.CoffeeSceneId, eventData));
            } else {
                this._stackEvent("waitEmpty", 0, eventData);
                this._gotoBuilding(this.targetBuildingData);
            }
        }

        _waitEmpty(eventData) {
            this._state = "waitEmpty";
            if (this.entrustData.getDataName() == "MiniGame" + Constants.MiniGameType.MORA) {
                this._context.recordEvent(this.getId(), "SpecialWait", { "anim": "lick1" });
            } else {
                this._context.recordEvent(this.getId(), "Wait", { "anim": "lick1" });
            }
        }

        destroy() {
            this._context.removeTimeline(this.getId());
            EventMgr.getInstance().removeEvent("ENTRUST_CAT_GO", this);
        }

    }

    class EntrustRoleMgr {

        constructor() {
            this._roleMap = new Map();
        }

        addRole(entrustData) {
            let role = new EntrustCatRole(entrustData);
            this._roleMap.set(String(entrustData.getCatId()), role);
            return role;
        }

        removeRole(roleId) {
            let role = this._roleMap.get(roleId);
            this._roleMap.delete(String(roleId));
            role.destroy();
        }


        getRole(id) {
            return this._roleMap.get(String(id));
        }

        getRoles() {
            return this._roleMap;
        }

    }

    //后续改成状态机
    class BeachCatRole {
        constructor(data) {
            this.catData = data;

            this.eventActionConfig = ConfigMgr$1.getInstance().getConfig("ActionEvent");
            this.targetEvent = null;


            this._eventStack = new Array();

        }

        getId() {
            return this.catData.getCatId()
        }

        getConfigId() {
            return this.catData.getConfigId();
        }

        //获得信息
        getData() {
            return this.catData;
        }

        _popEvent() {
            let info = this._eventStack.pop();
            if(info) {
                this._setNextStep(info.event, info.delay, info.args);
                this.update(0);
            }
        }

        _stackEvent(event, delay, args) {
            this._eventStack.push({"event":event, "delay":delay, "args":args});
        }

        _setNextStep(event, delay, args) {
            this._scheduleTime = delay;
            this._nextEvent = event;
            this._nextEventArgs = args;
        }


        start(context, currentTime) {
            let params = this.catData.getParams();
            this.buildingId = params.BuildingId;
            this.spaceId = params.SpaceId;
            this.travelTime = params.TravelTime;
            this.startTime = params.StartTime;
            if(this.buildingId==null) {
                CatSystem.removeCatData(this.getId(), Constants.BeachSceneId);
                return;
            }
            let travelTime = this.travelTime;
            if(currentTime && this.startTime) {
                travelTime = this.travelTime - currentTime + this.startTime;
                if(travelTime<0) {
                    travelTime = 0;
                }
            }
            let buildingData = BuildingSystem.getBuildingDataByBid(this.buildingId);
            buildingData.occupy(this.spaceId, this.getId());


            let eventString = params.event;
            if(!eventString) {
                eventString = this.catData.getEventTravel();
            }
            let eventData = this.eventActionConfig.get(eventString);
            if(eventData==null){
                if(buildingData!=null){
                    buildingData.leave(this.spaceId, this.getId());
                }
                CatSystem.removeCatData(this.getId(), Constants.BeachSceneId);
                return;
            }
            if(eventData.Type == "VISIT") {
                eventString = CatLogicHelper.randomNextEventInBuilding(buildingData, eventData);
                if(eventString==null){
                    buildingData.leave(this.spaceId, this.getId());
                    CatSystem.removeCatData(this.getId(), Constants.BeachSceneId);
                    return;
                }
            }

            this._context = context;
            this._state = "inBuilding";

            this._eventString = eventString;

            this.catData.setEvent(eventString);

            this._context.recordMetaEvent(this.getId(), "AddCat", {"catId":this.getId(), "buildingId":this.buildingId, "spaceId":this.spaceId, "event":eventString}, "BeachCatLine");

            this._setNextStep("travelEnd", travelTime);
            this.update(0);
        }

        update(dt) {
            if(this._scheduleTime && this._scheduleTime>0) {
                this._scheduleTime -= dt;
                return;
            }
            if(this._nextEvent){
                let event = this._nextEvent;
                this._nextEvent = null;
                switch(event) {
                    case "travelEnd":
                        this._travelEnd(this._nextEventArgs);
                        break;
                    case "remove":
                        this._remove(this._nextEventArgs);
                        break;
                 }
            }
        }

        remove() {
            if(!this._context) {
                return true;
            }
            if(this._state!="leaving"){
                let buildingData = BuildingSystem.getBuildingDataByBid(this.buildingId);
                buildingData.leave(this.spaceId, this.getId());
                this._context.recordEvent(this.getId(), "RemoveCat", null);
                return true;
            }
            return false;
        }

        _travelEnd() {
            if(this._eventString) {
                let eventData = this.eventActionConfig.get(this._eventString);
                if (this.catData.eventCount == 0) {
                    PetSystem.beachAddVisitTimes(this.getConfigId(), 1);
                }
                PetSystem.addIntimacy(this.getConfigId(), eventData.IntimacyUp);
                let reward = CatLogicHelper.getStarReward(eventData, this.catData.getStarBase());
                let rewardType = Constants.RewardType.STAR;
                if(reward && reward>0){
                    SyncSystem.cacheVisitorReward(this.getId(), this.getConfigId(), this.buildingId, reward, rewardType);
                }else {
                    let id = this.catData.getConfigId();
                    let valueBase = PetSystem.getPetInfo(id).getValueBase();
                    reward = CatLogicHelper.getFishReward(eventData, valueBase);
                    rewardType = Constants.RewardType.FISH;
                    if(reward && reward>0){
                        SyncSystem.cacheVisitorReward(this.getId(), this.getConfigId(), this.buildingId, reward, rewardType);
                    }
                }
            }


            this._state = "leaving";
            this.catData.eventCount++;
            this._context.recordEvent(this.getId(), "RemoveFade", { "buildingId": this.buildingId, "spaceId": this.spaceId });
            this._setNextStep("remove", 500);
        }

        _remove() {
            let buildingData = BuildingSystem.getBuildingDataByBid(this.buildingId);
            buildingData.leave(this.spaceId, this.getId());
            CatSystem.removeCatData(this.getId(), Constants.BeachSceneId);
            this._context._roleMgr.removeRole(this.getId());
        }

        destroy() {
            if(this._context) {
                this._context.removeTimeline(this.getId());
            }
        }

    }

    class BeachRoleMgr {

        constructor() {
            this._roleMap = new Map();
            this._roleList = new Array();
        }

        addRole(catData) {
            let role = new BeachCatRole(catData);
            this._roleMap.set(String(catData.getCatId()), role);
            this._roleList.push(role);
            return role;
        }

        removeRole(roleId) {
            this._roleMap.delete(String(roleId));
            for(const key in this._roleList) {
                let role = this._roleList[key];
                if(role.getId() == roleId){
                    role.destroy();
                    this._roleList.splice(key, 1);
                    return role;
                }
            }
        }

        getRoleById(id) {
            return this._roleMap.get(String(id));
        }

        getRoles() {
            return this._roleList.slice(0);
        }

        getRoleCount() {
            return this._roleList.length;
        }

    }

    /**
    *    Filename:    StateMachine.lua
    *    Author:      pengtao
    *    Datetime:    2020-03-28
    *    Description: The normal state machine.
    */

    class StateMachine {
        constructor(agent, initialState, globalState){
            this._agent = agent;
            this.initialState = initialState;
            this.globalState = globalState;
        }

        set agent(agent) {
            this._agent = agent;
        }

        get agent() {
            return this._agent;
        }

        set initialState(initialState) {
            this._previousState = null;
            this._currentState = initialState;
        }

        get currentState() {
            return this._currentState;
        }

        get previousState() {
            return this._previousState;
        }

        set globalState(globalState) {
            this._globalState = globalState;
        }

        get globalState() {
            return this._globalState;
        }

        update() {
            let args = Array.from(arguments);
            args.unshift(this._agent);
            if(this._globalState)
                this._globalState.update.apply(this._globalState, args);
            if(this._currentState)
                this._currentState.update.apply(this._currentState, args);
        }

        changeState(newState) {
            let args = Array.from(arguments);
            args[0] = this._agent;
            this._previousState = this._currentState;
            if(this._currentState)
                this._currentState.exit.apply(this._currentState, args);
            this._currentState = newState;
            if(this._currentState)
                this._currentState.enter.apply(this._currentState, args);
        }

        revertToPreviousState() {
            if(this._previousState==null)
                return false;
            let args = Array.from(arguments);
            args.unshift(this._previousState);
            this.changeState.apply(this, args);
        }

        isInState(state) {
            let current = this._currentState;
            return current==state?true:(current!=null&&current.name==state);
        }

        handleMessage(message) {
            if(this._currentState&&
                this._currentState.onMessage(this._agent, message))
                return true;
            if(this._globalState&&
                this._globalState.onMessage(this._agent, message))
                return true;
            return false;
        }
    }

    class IState {
        constructor(name) {
            this._name = name;
        }

        get name() {
            return this._name;
        }

        enter(agent) {

        }

        update(agent) {

        }

        exit(agent) {

        }

        onMessage(agent, message) {
            return false;
        }
    }

    let __NULL__ = "NULL";
    class BaseState extends IState{
        constructor(name) {
            super(name);
            this._transitions = new Map();
            this._enabledOps = new Map();
        }

        setupContext(context) {
            return this;
        }

        addTransition(msgType, state) {
            this._transitions.set(msgType, state);
            return this;
        }

        getTransition(msgType) {
            return this._transitions.get(msgType);
        }

        enableOp(op) {
            this._enabledOps.set(op, true);
            return this;
        }

        isOpEnabled(op) {
            return this._enabledOps.get(op)?true:false;
        }

        onMessage(agent, message) {
            let msgType = message.type;
            let nextState = this._transitions.get(msgType);
            if (nextState) {
                agent.switchToState(nextState, message);
                return true;
            }
        }

        exit() {
            this._timers = null;
        }

        update(agent, context, dt) {
            this.updateTimers(dt);
        }

        handlePlayerInput(agent, context, op, args, callback) {
            return false;
        }

        startTimer(timeout, handler) {
            let timer = new LogicTimer(timeout, handler);
            let timers = this._timers;
            if(timers==null)
                this._timers = [timer];
            else
                timers.push(timer);
            timer.restart();
            return timer;
        }

        updateTimers(dt) {
            let timers = this._timers;
            if(timers) {
                let cntTimers = timers.length;
                let done = 0;
                for(let i = 0; i < cntTimers; ++i) {
                    let timer = timers[i];
                    if(timer && timer.tick(dt)) {
                        timers[i] = __NULL__;
                        ++done;
                    }
                }
                if(done > 0) {
                    let eidx = 0;
                    for(let i = 0; i < cntTimers; ++i) {
                        let timer = timers[i];
                        if(timer==null) {
                            timers.splice(i);
                            return;
                        }
                        if(timer==__NULL__) {
                            timers[i] = null;
                        }else {
                            if(i!=eidx) {
                                timers[eidx] = timer;
                                timers[i] = null;
                            }
                            ++eidx;
                        }
                    }
                }
            }
        }

    }

    class LogicTimer {
        constructor(timeout, handler) {
            this._timeout = timeout;
            this._handler = handler;
            // this._index = LogicTimer.__index++;
        }

        set handler(handler) {
            this._handler = handler;
        }

        get timeout() {
            return this._timeout;
        }

        set timeout(timeout) {
            this._timeout = timeout;
        }

        get elapsed() {
            return this._elapsed;
        }

        restart() {
            this._elapsed = 0;
        }

        cancel() {
            this._elapsed = null;
        }

        isDone() {
            return this._elapsed == null;
        }

        tick(dt) {
            let elapsed = this._elapsed;
            if(elapsed!=null) {
                elapsed += dt;
                if(elapsed<this._timeout) {
                    this._elapsed = elapsed;
                }else {
                    this._elapsed = null;
                    if(this._handler)
                        this._handler.runWith(this);
                }
            }
            return this._elapsed==null;
        }
    }
    // LogicTimer.__index = 0;

    class StaffRole {
        constructor(data) {
            this.staffData = data;
            this.staffData.currentPathIndex = 0;

            this.createStateMachine();
            this._orderTime = Number(Const.get("StaffAutoOrderTime")) * 1000;
            this._autoInviteTime = Number(Const.get("StaffAutoInviteTime")) * 1000;
        }

        getId() {
            return this.staffData.getId();
        }

        //获得信息
        getData() {
            return this.staffData;
        }

        createStateMachine() {
            let startIdleState = new StartIdleState("StartIdle");
            let idleRandomState = new RandomState("idleRandom");
            let idleState = new IdleState("Idle");
            let idleMoveState = new MoveState("IdleMove");
            let skill1ActionState = new Skill1ActionState("Skill1Action");
            let skill2ActionState = new Skill2ActionState("Skill2Action");
            let skill3ActionState = new Skill3ActionState("Skill3Action");
            let skillMoveState = new SkillMoveState("SkillMove");

            startIdleState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("Reset", startIdleState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState)
                .addTransition("SkillMove", skillMoveState);

            idleRandomState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState);

            idleState.addTransition("IdleMove", idleMoveState)
                .addTransition("Idle", idleState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState);

            idleMoveState.addTransition("Idle", idleState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState);

            skill1ActionState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("SkillMove", skillMoveState);

            skill2ActionState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("SkillMove", skillMoveState);

            skill3ActionState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState)
                .addTransition("SkillMove", skillMoveState);

            skillMoveState.addTransition("Idle", idleState)
                .addTransition("SkillMove", skillMoveState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("Skill1", skill1ActionState)
                .addTransition("Skill2", skill2ActionState)
                .addTransition("Skill3", skill3ActionState);

            this._stateMachine = new StateMachine(this, startIdleState);
        }

        switchToState(newState, message) {
            return this._stateMachine.changeState(newState, this._context, message);
        }

        sendMsg(msgType, params) {
            // console.log("==================", msgType);
            this._stateMachine.handleMessage({ "type": msgType, "params": params });
        }

        start(context, currentTime) {
            this._context = context;
            let staffData = this.staffData;
            let x = staffData.x;
            let y = staffData.y;
            this.isIdle = true;
            // if (x == null || y == null) {
            x = 1042;
            y = 2324;
            // }
            this.setPos(x, y);

            this._context.recordMetaEvent(this.getId(), "PutStaff", { "staffId": this.getId(), "x": x, "y": y }, "StaffCatLine");
            if (this.staffData.skill1Timer != null || this.staffData.skill2Timer != null) {
                this.sendMsg("SkillMove");
                this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            } else {
                this.sendMsg("Reset");
            }
            this.setSkill1CdTimer();
            this.setSkill2CdTimer();
            this.setSkill3CdTimer();
            staffData.lastUpdateTime = currentTime;
        }

        updateTimer(dt) {
            if (dt <= 0)
                return;
            if (this.staffData.skill1CdTimer != null) {
                this.staffData.skill1CdTimer -= dt;
                if (this.staffData.skill1CdTimer <= 0 && this.canStartSkill()) {
                    this.sendMsg("Skill1");
                }
            }
            if (this.staffData.skill2CdTimer != null) {
                this.staffData.skill2CdTimer -= dt;
                if (this.staffData.skill2CdTimer <= 0 && this.canStartSkill()) {
                    this.sendMsg("Skill2");
                }
            }
            if (this.staffData.skill3CdTimer != null) {
                this.staffData.skill3CdTimer -= dt;
                if (this.staffData.skill3CdTimer <= 0 && this.canStartSkill()) {
                    this.sendMsg("Skill3");
                }
            }

            if (this.staffData.skill1Timer != null) {
                this.staffData.skill1Timer -= dt;
                this._context.recordEvent(this.getId(), "SkillPickFish");
                if (this.staffData.skill1Timer <= 0) {
                    this.clearSkill1Timer();
                    this.setSkill1CdTimer();
                    if (!this.isSkilling()) {
                        this.sendMsg("Idle");
                    }
                }
            }

            if (this.staffData.skill2Timer != null) {
                this.staffData.skill2Timer -= dt;
                if (this.staffData.skill2Timer % this._orderTime == 0) {
                    StaffSystem.skill2();
                }
                if (this.staffData.skill2Timer <= 0) {
                    this.clearSkill2Timer();
                    this.setSkill2CdTimer();
                    if (!this.isSkilling()) {
                        this.sendMsg("Idle");
                    }
                }
            }

            if (this.staffData.skill3Timer != null) {
                this.staffData.skill3Timer -= dt;
                if (this.staffData.skill3Timer % this._autoInviteTime == 0) {
                    StaffSystem.skill3();
                    this._context.recordEvent(this.getId(), "ShowSkillTag", this.getData());
                }
                if (this.staffData.skill3Timer <= 0) {
                    this.clearSkill3Timer();
                    this.setSkill3CdTimer();
                    if (!this.isSkilling()) {
                        this.sendMsg("Idle");
                    }
                }
            }
            this.staffData.getSkillData().forEach(skill => {
                if (skill.getUnlock() && skill.skillLevelUpEndTimer != null && Math.abs(TimeUtils.getTimeNowOnServer() - skill.skillLevelUpEndTimer * 1000) <= 100) {
                    EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
                    EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["staff"]);
                }
            });
        }

        update(dt, currentTime) {
            // this.updateTimer(currentTime - this.staffData.lastUpdateTime);
            this._context.recordEvent(this.getId(), "Show");
            this.staffData.lastUpdateTime = currentTime;
            this._stateMachine.update(this._context, dt);
        }

        setSkill1CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_1")) {
                return;
            }
            if ((this.staffData.skill1Timer != null && this.staffData.skill1Timer > 0) || (this.staffData.skill1CdTimer != null && this.staffData.skill1CdTimer > 0)) {
                return;
            }
            this.staffData.skill1CdTimer = this.staffData.getSkill1Data().CDTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        clearSkill1CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_1")) {
                return;
            }
            this.staffData.skill1CdTimer = null;
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        setSkill2CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_2")) {
                return;
            }
            if ((this.staffData.skill2Timer != null && this.staffData.skill2Timer > 0) || (this.staffData.skill2CdTimer != null && this.staffData.skill2CdTimer > 0)) {
                return;
            }
            this.staffData.skill2CdTimer = this.staffData.getSkill2Data().CDTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        clearSkill2CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_2")) {
                return;
            }
            this.staffData.skill2CdTimer = null;
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        setSkill3CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_3")) {
                return;
            }
            if ((this.staffData.skill3Timer != null && this.staffData.skill3Timer > 0) || (this.staffData.skill3CdTimer != null && this.staffData.skill3CdTimer > 0)) {
                return;
            }
            this.staffData.skill3CdTimer = this.staffData.getSkill3Data().CDTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        clearSkill3CdTimer() {
            if (!StaffSystem.getSkillUnlock(this.staffData.getId(), "SkillGroup_3")) {
                return;
            }
            this.staffData.skill3CdTimer = null;
            EventMgr.getInstance().postEvent("UPDATE_SCENE_INFO_NOW");
        }

        calcIdle(message, stateName) {
            return { "duration": Utils.randomExt2(2000, 5000) };
        }

        startSkill1() {
            this.isIdle = false;
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
            this.staffData.skill1Timer = this.staffData.getSkill1Data().EffectTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
        }

        clearSkill1Timer() {
            this.staffData.skill1Timer = null;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
        }

        startSkill2() {
            this.isIdle = false;
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
            this.staffData.skill2Timer = this.staffData.getSkill2Data().EffectTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
        }

        clearSkill2Timer() {
            this.staffData.skill2Timer = null;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
        }

        startSkill3() {
            this.isIdle = false;
            this.staffData.skill3Timer = this.staffData.getSkill3Data().EffectTime * 1000;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
        }

        clearSkill3Timer() {
            this.staffData.skill3Timer = null;
            this._context.recordEvent(this.getId(), "ResetSkillTip", this.getData());
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
        }

        canStartSkill() {
            return this.isIdle && (this.staffData.currentPathIndex < 12 || this.staffData.currentPathIndex > 18) && this.staffData.currentPathIndex != 22 && this.staffData.currentPathIndex > 2;
        }

        isSkilling() {
            return !this.isIdle || this.staffData.skill1Timer != null || this.staffData.skill2Timer != null || this.staffData.skill3Timer != null;
        }

        randomTarget() {
            let targetIndex = 1;
            if (Utils.randomSign() == 1) {
                targetIndex = Utils.randomExt2(1, 13);
            } else {
                let endArr = [17, 20, 21];
                targetIndex = endArr[Utils.randomExt2(0, 3)];
            }
            return targetIndex;
        }

        calcRandomMove(message, stateName) {
            this._pathStartX = null;
            this._pathStartY = null;
            this._realPath = null;
            this._moveDuration = null;
            // let blockMap = this._context._blockMap;
            let x = this.staffData.x;
            let y = this.staffData.y;
            let targetIndex = this.randomTarget();
            while (targetIndex == this.staffData.currentPathIndex) {
                targetIndex = this.randomTarget();
            }
            let path = this._genPath(targetIndex);
            if (path) {
                if (path && path.length > 0) {
                    let realPath = [];
                    let lastX = this.staffData.x;
                    let lastY = this.staffData.y;
                    let totalDuration = 0;
                    for (let i = 0; i < path.length; ++i) {
                        let nextX = path[i].x;
                        let nextY = path[i].y;
                        let index = path[i].i;
                        let dx = lastX - nextX;
                        let dy = lastY - nextY;
                        let distance = Math.sqrt(dx * dx + dy * dy);
                        let duration = distance / this.staffData.speed * 1000;
                        realPath.push([nextX, nextY, duration, index]);
                        totalDuration += duration;

                        lastX = nextX;
                        lastY = nextY;
                    }
                    this._realPath = realPath.reverse();
                    this._moveDuration = 0;
                    this._pathStartX = this.staffData.x;
                    this._pathStartY = this.staffData.y;
                    return { "duration": totalDuration, "params": this._realPath.slice(0) };
                }
            }
            return null;
        }

        skill1Move(message, stateName) {
            this._pathStartX = null;
            this._pathStartY = null;
            this._realPath = null;
            this._moveDuration = null;
            // let blockMap = this._context._blockMap;
            let x = this.staffData.x;
            let y = this.staffData.y;
            let targetIndex = this.randomTarget();
            if (this.staffData.skill1Timer != null) {
                targetIndex = this.getNextFishBuildingPath();
            }
            if (targetIndex == 0) {
                targetIndex = this.randomTarget();
            }
            let path = this._genPath(targetIndex, true);
            if (path) {
                if (path && path.length > 0) {
                    let realPath = [];
                    let lastX = this.staffData.x;
                    let lastY = this.staffData.y;
                    let totalDuration = 0;
                    for (let i = 0; i < path.length; ++i) {
                        let nextX = path[i].x;
                        let nextY = path[i].y;
                        let index = path[i].i;
                        let dx = lastX - nextX;
                        let dy = lastY - nextY;
                        let distance = Math.sqrt(dx * dx + dy * dy);
                        let duration = distance / this.staffData.speed * 1000;
                        realPath.push([nextX, nextY, duration, index]);
                        totalDuration += duration;

                        lastX = nextX;
                        lastY = nextY;
                    }
                    this._realPath = realPath.reverse();
                    this._moveDuration = 0;
                    this._pathStartX = this.staffData.x;
                    this._pathStartY = this.staffData.y;
                    return { "duration": totalDuration, "params": this._realPath.slice(0) };
                }
            }
            return null;
        }

        _genPath(targetIndex, fast = false) {
            let path = null;
            if (fast) {
                path = ScenePathMgr.getInstance().randomFastPath(this.staffData.currentPathIndex, targetIndex);
            } else {
                path = ScenePathMgr.getInstance().randomPath(this.staffData.currentPathIndex, targetIndex);
            }
            return path;
        }

        _updatePos(dt) {
            if (this._moveDuration != null) {
                this._moveDuration += dt;
                let duration = this._moveDuration;
                let lastX = this._pathStartX;
                let lastY = this._pathStartY;
                for (let i = this._realPath.length - 1; i >= 0; --i) {
                    let pathInfo = this._realPath[i];
                    if (duration > pathInfo[2]) {
                        lastX = pathInfo[0];
                        lastY = pathInfo[1];
                        duration -= pathInfo[2];
                    } else {
                        this.setPos(lastX + (pathInfo[0] - lastX) * duration / pathInfo[2],
                            lastY + (pathInfo[1] - lastY) * duration / pathInfo[2]);
                        return;
                    }
                }
            }
        }

        setPos(x, y) {
            this.staffData.pos(x, y);
        }

        getPos() {
            return { "x": this.staffData.x, "y": this.staffData.y };
        }

        calcStop(message, stateName) {
            return { "duration": 4000, "event": "Stop" };
        }

        afterSkillAction(context, stateName) {
            this.isIdle = true;
            EventMgr.getInstance().postEvent("UPDATA_SATFFPOP_LIST");
        }

        calcSkill1Action(message, stateName) {
            // let duration = this.staffData.getSkill1Data().EffectTime * 1000;
            return { "duration": 8000, "event": "Skill_1", "params": { staffData: this.staffData } };
        }

        calcSkill2Action(message, stateName) {
            // let duration = this.staffData.getSkill2Data().EffectTime * 1000;
            return { "duration": 8000, "event": "Skill_2", "params": { staffData: this.staffData } };
        }

        calcSkill3Action(message, stateName) {
            return { "duration": 8000, "event": "Skill_3", "params": { staffData: this.staffData } };
        }

        destroy() {
            if (this._context) {
                this._context.removeTimeline(this.getId());
                this._context = null;
            }
            this._stateMachine = null;
        }

        getNextFishBuildingPath() {
            let maxPath = 0;
            let maxId = SyncSystem.getNextFishBuildingBaseId();
            if (maxId != null) {
                let config = ConfigMgr$1.getInstance().getItemById("BuildingBase", maxId);
                maxPath = Number(config.PathPointIndex);
            }
            return maxPath;
        }
    }

    class StartIdleState extends BaseState {

        enter(agent, context, message) {
            context.recordEvent(agent.getId(), "Reset", agent.getPos());
            if (Utils.randomSign() == 1) {
                this.startTimer(5, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(5, Laya.Handler.create(this, this.next2, [agent, context]));
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }

        next2(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class RandomState extends BaseState {

        enter(agent, context, message) {
            context.recordEvent(agent.getId(), "Reset", agent.getPos());
            if (Utils.randomSign() == 1) {
                this.startTimer(1, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(1, Laya.Handler.create(this, this.next2, [agent, context]));
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }

        next2(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class IdleState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcIdle(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), "Idle", args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        update(agent, context, dt) {
            super.update(agent, context, dt);
            agent.updateTimer(dt);
        }

        exit(agent, context, message) {
            super.exit(agent, context);
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class MoveState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcRandomMove(message, this._name);
            if (args) {
                let duration = args.duration;
                context.recordEvent(agent.getId(), "Move", args.params);
                this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(5, Laya.Handler.create(this, this.next, [agent, context]));
            }
        }

        update(agent, context, dt) {
            super.update(agent, context, dt);
            agent.updateTimer(dt);
            // agent._updatePos(dt);
            // context.recordEvent(agent.getId(), "MoveIng", agent.getPos());
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            context.recordEvent(agent.getId(), "MoveEnd", agent.getPos());
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class Skill1ActionState extends BaseState {

        enter(agent, context, message) {
            agent.clearSkill1CdTimer();
            agent.startSkill1();
            let args = agent.calcSkill1Action(message, this._name);
            let duration = args.duration;
            args.params.handler = Laya.Handler.create(this, function () {
                this.next(agent, context);
            });
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            agent.afterSkillAction(context, this._name);
            this.onMessage(agent, { "type": "SkillMove" });
        }
    }

    class Skill2ActionState extends BaseState {

        enter(agent, context, message) {
            agent.clearSkill2CdTimer();
            agent.startSkill2();
            let args = agent.calcSkill2Action(message, this._name);
            let duration = args.duration;
            args.params.handler = Laya.Handler.create(this, function () {
                this.next(agent, context);
            });
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            agent.afterSkillAction(context, this._name);
            this.onMessage(agent, { "type": "SkillMove" });
        }
    }


    class Skill3ActionState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcSkill3Action(message, this._name);
            let duration = args.duration;
            agent.clearSkill3CdTimer();
            agent.startSkill3();
            args.params.handler = Laya.Handler.create(this, function () {
                this.next(agent, context);
            });
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }


        next(agent, context) {
            agent.afterSkillAction(context, this._name);
            this.onMessage(agent, { "type": "SkillMove" });
        }
    }

    class SkillMoveState extends BaseState {

        enter(agent, context, message) {
            let args = agent.skill1Move(message, this._name);
            let duration = args.duration;
            context.recordEventNextFrame(agent.getId(), "Move", args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        update(agent, context, dt) {
            super.update(agent, context, dt);
            agent.updateTimer(dt);
            // agent._updatePos(dt);
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            context.recordEvent(agent.getId(), "MoveEnd", agent.getPos());
        }

        next(agent, context) {
            agent.afterSkillAction(context, this._name);
            this.onMessage(agent, { "type": "SkillMove" });
        }
    }

    class StaffRoleMgr extends BeachRoleMgr {
        addRole(catData) {
            let role = new StaffRole(catData);
            this._roleMap.set(String(catData.getId()), role);
            this._roleList.push(role);
            return role;
        }
    }

    /**
     * A node in grid.
     * This class holds some basic information about a node and custom
     * attributes may be added, depending on the algorithms' needs.
     * @constructor
     * @param {number} x - The x coordinate of the node on the grid.
     * @param {number} y - The y coordinate of the node on the grid.
     * @param {boolean} [walkable] - Whether this node is walkable.
     */
    class Node {
        constructor(x, y, walkable) {
            /**
             * The x coordinate of the node on the grid.
             * @type number
             */
            this.x = x;
            /**
             * The y coordinate of the node on the grid.
             * @type number
             */
            this.y = y;
            /**
             * Whether this node can be walked through.
             * @type boolean
             */
            this.walkable = (walkable === undefined ? true : walkable);
        }

        clearExtraInfo() {
            this.f = null;
            this.g = null;
            this.h = null;
            this.opened = null;
            this.closed = null;
            this.parent = null;
        }
    }

    let DiagonalMovement = {
        Always: 1,
        Never: 2,
        IfAtMostOneObstacle: 3,
        OnlyWhenNoObstacles: 4
    };

    /**
     * The Grid class, which serves as the encapsulation of the layout of the nodes.
     * @constructor
     * @param {number|Array<Array<(number|boolean)>>} width_or_matrix Number of columns of the grid, or matrix
     * @param {number} height Number of rows of the grid.
     * @param {Array<Array<(number|boolean)>>} [matrix] - A 0-1 matrix
     *     representing the walkable status of the nodes(0 or false for walkable).
     *     If the matrix is not supplied, all the nodes will be walkable.  */
    class Grid {

        constructor(width_or_matrix, height, matrix) {
            var width;

            if (typeof width_or_matrix !== 'object') {
                width = width_or_matrix;
            } else {
                height = width_or_matrix.length;
                width = width_or_matrix[0].length;
                matrix = width_or_matrix;
            }

            /**
             * The number of columns of the grid.
             * @type number
             */
            this.width = width;
            /**
             * The number of rows of the grid.
             * @type number
             */
            this.height = height;

            /**
             * A 2D array of nodes.
             */
            this.nodes = this._buildNodes(width, height, matrix);
        }

        /**
         * Build and return the nodes.
         * @private
         * @param {number} width
         * @param {number} height
         * @param {Array<Array<number|boolean>>} [matrix] - A 0-1 matrix representing
         *     the walkable status of the nodes.
         * @see Grid
         */
        _buildNodes(width, height, matrix) {
            var i, j,
                nodes = new Array(height);

            for (i = 0; i < height; ++i) {
                nodes[i] = new Array(width);
                for (j = 0; j < width; ++j) {
                    nodes[i][j] = new Node(j, i);
                }
            }


            if (matrix === undefined) {
                return nodes;
            }

            if (matrix.length !== height || matrix[0].length !== width) {
                throw new Error('Matrix size does not fit');
            }

            for (i = 0; i < height; ++i) {
                for (j = 0; j < width; ++j) {
                    if (matrix[i][j]) {
                        // 0, false, null will be walkable
                        // while others will be un-walkable
                        nodes[i][j].walkable = false;
                    }
                }
            }

            return nodes;
        }


        getNodeAt(x, y) {
            return this.nodes[y][x];
        }


        /**
         * Determine whether the node at the given position is walkable.
         * (Also returns false if the position is outside the grid.)
         * @param {number} x - The x coordinate of the node.
         * @param {number} y - The y coordinate of the node.
         * @return {boolean} - The walkability of the node.
         */
        isWalkableAt(x, y) {
            return this.isInside(x, y) && this.nodes[y][x].walkable;
        }


        /**
         * Determine whether the position is inside the grid.
         * XXX: `grid.isInside(x, y)` is wierd to read.
         * It should be `(x, y) is inside grid`, but I failed to find a better
         * name for this method.
         * @param {number} x
         * @param {number} y
         * @return {boolean}
         */
        isInside(x, y) {
            return (x >= 0 && x < this.width) && (y >= 0 && y < this.height);
        }


        /**
         * Set whether the node on the given position is walkable.
         * NOTE: throws exception if the coordinate is not inside the grid.
         * @param {number} x - The x coordinate of the node.
         * @param {number} y - The y coordinate of the node.
         * @param {boolean} walkable - Whether the position is walkable.
         */
        setWalkableAt(x, y, walkable) {
            this.nodes[y][x].walkable = walkable;
        }


        /**
         * Get the neighbors of the given node.
         *
         *     offsets      diagonalOffsets:
         *  +---+---+---+    +---+---+---+
         *  |   | 0 |   |    | 0 |   | 1 |
         *  +---+---+---+    +---+---+---+
         *  | 3 |   | 1 |    |   |   |   |
         *  +---+---+---+    +---+---+---+
         *  |   | 2 |   |    | 3 |   | 2 |
         *  +---+---+---+    +---+---+---+
         *
         *  When allowDiagonal is true, if offsets[i] is valid, then
         *  diagonalOffsets[i] and
         *  diagonalOffsets[(i + 1) % 4] is valid.
         * @param {Node} node
         * @param {DiagonalMovement} diagonalMovement
         */
        getNeighbors(node, diagonalMovement) {
            var x = node.x,
                y = node.y,
                neighbors = [],
                s0 = false, d0 = false,
                s1 = false, d1 = false,
                s2 = false, d2 = false,
                s3 = false, d3 = false,
                nodes = this.nodes;

            // ↑
            if (this.isWalkableAt(x, y - 1)) {
                neighbors.push(nodes[y - 1][x]);
                s0 = true;
            }
            // →
            if (this.isWalkableAt(x + 1, y)) {
                neighbors.push(nodes[y][x + 1]);
                s1 = true;
            }
            // ↓
            if (this.isWalkableAt(x, y + 1)) {
                neighbors.push(nodes[y + 1][x]);
                s2 = true;
            }
            // ←
            if (this.isWalkableAt(x - 1, y)) {
                neighbors.push(nodes[y][x - 1]);
                s3 = true;
            }

            if (diagonalMovement === DiagonalMovement.Never) {
                return neighbors;
            }

            if (diagonalMovement === DiagonalMovement.OnlyWhenNoObstacles) {
                d0 = s3 && s0;
                d1 = s0 && s1;
                d2 = s1 && s2;
                d3 = s2 && s3;
            } else if (diagonalMovement === DiagonalMovement.IfAtMostOneObstacle) {
                d0 = s3 || s0;
                d1 = s0 || s1;
                d2 = s1 || s2;
                d3 = s2 || s3;
            } else if (diagonalMovement === DiagonalMovement.Always) {
                d0 = true;
                d1 = true;
                d2 = true;
                d3 = true;
            } else {
                throw new Error('Incorrect value of diagonalMovement');
            }

            // ↖
            if (d0 && this.isWalkableAt(x - 1, y - 1)) {
                neighbors.push(nodes[y - 1][x - 1]);
            }
            // ↗
            if (d1 && this.isWalkableAt(x + 1, y - 1)) {
                neighbors.push(nodes[y - 1][x + 1]);
            }
            // ↘
            if (d2 && this.isWalkableAt(x + 1, y + 1)) {
                neighbors.push(nodes[y + 1][x + 1]);
            }
            // ↙
            if (d3 && this.isWalkableAt(x - 1, y + 1)) {
                neighbors.push(nodes[y + 1][x - 1]);
            }

            return neighbors;
        }


        /**
         * Get a clone of this grid.
         * @return {Grid} Cloned grid.
         */
        clone() {
            var i, j,

                width = this.width,
                height = this.height,
                thisNodes = this.nodes,

                newGrid = new Grid(width, height),
                newNodes = new Array(height);

            for (i = 0; i < height; ++i) {
                newNodes[i] = new Array(width);
                for (j = 0; j < width; ++j) {
                    newNodes[i][j] = new Node(j, i, thisNodes[i][j].walkable);
                }
            }

            newGrid.nodes = newNodes;

            return newGrid;
        }
    }

    class Heap {
        constructor(cmp){
            this.cmp=null;
            this.nodes=null;
            this.heapFunction = new HeapFunction();
            this.cmp=cmp !=null ? cmp :this.heapFunction.defaultCmp;
            this.nodes=[];
        }


        push(x){
            return this.heapFunction.heappush(this.nodes,x,this.cmp);
        }

        pop(){
            return this.heapFunction.heappop(this.nodes,this.cmp);
        }

        peek(){
            return this.nodes[0];
        }

        contains(x){
            return this.nodes.indexOf(x)!==-1;
        }

        replace(x){
            return this.heapFunction.heapreplace(this.nodes,x,this.cmp);
        }

        pushpop(x){
            return this.heapFunction.heappushpop(this.nodes,x,this.cmp);
        }

        heapify(){
            return this.heapFunction.heapify(this.nodes,this.cmp);
        }

        updateItem(x){
            return this.heapFunction.updateItem(this.nodes,x,this.cmp);
        }

        clear(){
            return this.nodes=[];
        }

        empty(){
            return this.nodes.length===0;
        }

        size(){
            return this.nodes.length;
        }

        clone(){
            var heap=new Heap();
            heap.nodes=this.nodes.slice(0);
            return heap;
        }

        toArray(){
            return this.nodes.slice(0);
        }
    }


    class HeapFunction {
        constructor(){

            this.defaultCmp=function(x,y){
                if (x < y){
                    return -1;
                }
                if (x > y){
                    return 1;
                }
                return 0;
            };
        }

        insort(a,x,lo,hi,cmp){
            var mid=NaN;
            if (lo==null){
                lo=0;
            }
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            if (lo < 0){
                throw new Error('lo must be non-negative');
            }
            if (hi==null){
                hi=a.length;
            }
            while (lo < hi){
                mid=Math.floor((lo+hi)/ 2);
                if (cmp(x,a[mid])< 0){
                    hi=mid;
                }
                else {
                    lo=mid+1;
                }
            }
            return ([].splice.apply(a,[lo,lo-lo].concat(x)),x);
        }


        heappush(array,item,cmp){
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            array.push(item);
            return this._siftdown(array,0,array.length-1,cmp);
        }


        heappop(array,cmp){
            var lastelt,returnitem;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            lastelt=array.pop();
            if (array.length){
                returnitem=array[0];
                array[0]=lastelt;
                this._siftup(array,0,cmp);
            }
            else {
                returnitem=lastelt;
            }
            return returnitem;
        }


        heapreplace(array,item,cmp){
            var returnitem;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            returnitem=array[0];
            array[0]=item;
            this._siftup(array,0,cmp);
            return returnitem;
        }


        heappushpop(array,item,cmp){
            var _ref;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            if (array.length && cmp(array[0],item)< 0){
                _ref=[array[0],item],item=_ref[0],array[0]=_ref[1];
                this._siftup(array,0,cmp);
            }
            return item;
        }


        heapify(array,cmp){
            var i=0,_i=0,_j=0,_len=0,_ref,_ref1,_results,_results1;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            _ref1=(function(){
                _results1=[];
                for (_j=0,_ref=Math.floor(array.length / 2);0 <=_ref ? _j < _ref :_j > _ref;0 <=_ref ? _j++:_j--){
                    _results1.push(_j);
                }
                return _results1;
            }).apply(this).reverse();
            _results=[];
            for (_i=0,_len=_ref1.length;_i < _len;_i++){
                i=_ref1[_i];
                _results.push(this._siftup(array,i,cmp));
            }
            return _results;
        }


        updateItem(array,item,cmp){
            var pos=0;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            pos=array.indexOf(item);
            if (pos===-1){
                return null;
            }
            this._siftdown(array,0,pos,cmp);
            return this._siftup(array,pos,cmp);
        }


        nlargest(array,n,cmp){
            var elem,result,_i=0,_len=0,_ref;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            result=array.slice(0,n);
            if (!result.length){
                return result;
            }
            this.heapify(result,cmp);
            _ref=array.slice(n);
            for (_i=0,_len=_ref.length;_i < _len;_i++){
                elem=_ref[_i];
                this.heappushpop(result,elem,cmp);
            }
            return result.sort(cmp).reverse();
        }


        nsmallest(array,n,cmp){
            var elem,i,los,result,_i=0,_j=0,_len,_ref,_ref1,_results;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            if (n *10 <=array.length){
                result=array.slice(0,n).sort(cmp);
                if (!result.length){
                    return result;
                }
                los=result[result.length-1];
                _ref=array.slice(n);
                for (_i=0,_len=_ref.length;_i < _len;_i++){
                    elem=_ref[_i];
                    if (cmp(elem,los)< 0){
                        this.insort(result,elem,0,null,cmp);
                        result.pop();
                        los=result[result.length-1];
                    }
                }
                return result;
            }
            this.heapify(array,cmp);
            _results=[];
            for (i=_j=0,_ref1=Math.min(n,array.length);0 <=_ref1 ? _j < _ref1 :_j > _ref1;i=0 <=_ref1 ?++_j :--_j){
                _results.push(this.heappop(array,cmp));
            }
            return _results;
        }


        _siftdown(array,startpos,pos,cmp){
            var newitem,parent,parentpos=0;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            newitem=array[pos];
            while (pos > startpos){
                parentpos=(pos-1)>> 1;
                parent=array[parentpos];
                if (cmp(newitem,parent)< 0){
                    array[pos]=parent;
                    pos=parentpos;
                    continue ;
                }
                break ;
            }
            return array[pos]=newitem;
        }


        _siftup(array,pos,cmp){
            var childpos=0,endpos=0,newitem,rightpos=0,startpos=0;
            if (cmp==null){
                cmp=this.defaultCmp;
            }
            endpos=array.length;
            startpos=pos;
            newitem=array[pos];
            childpos=2 *pos+1;
            while (childpos < endpos){
                rightpos=childpos+1;
                if (rightpos < endpos && !(cmp(array[childpos],array[rightpos])< 0)){
                    childpos=rightpos;
                }
                array[pos]=array[childpos];
                pos=childpos;
                childpos=2 *pos+1;
            }
            array[pos]=newitem;
            return this._siftdown(array,startpos,pos,cmp);
        }

    }

    /**
     * Backtrace according to the parent records and return the path.
     * (including both start and end nodes)
     * @param {Node} node End node
     * @return {Array<Array<number>>} the path
     */

    class Util {
        static backtrace(node) {
            var path = [{"x": node.x, "y": node.y}];
            while (node.parent) {
                node = node.parent;
                path.push({"x": node.x, "y": node.y});
            }
            return path.reverse();
        }

        /**
         * Backtrace from start and end node, and return the path.
         * (including both start and end nodes)
         * @param {Node}
         * @param {Node}
         */
        static biBacktrace(nodeA, nodeB) {
            var pathA = Util.backtrace(nodeA),
                pathB = Util.backtrace(nodeB);
            return pathA.concat(pathB.reverse());
        }

        /**
         * Compute the length of the path.
         * @param {Array<Array<number>>} path The path
         * @return {number} The length of the path
         */
        static pathLength(path) {
            var i, sum = 0, a, b, dx, dy;
            for (i = 1; i < path.length; ++i) {
                a = path[i - 1];
                b = path[i];
                dx = a[0] - b[0];
                dy = a[1] - b[1];
                sum += Math.sqrt(dx * dx + dy * dy);
            }
            return sum;
        }


        /**
         * Given the start and end coordinates, return all the coordinates lying
         * on the line formed by these coordinates, based on Bresenham's algorithm.
         * http://en.wikipedia.org/wiki/Bresenham's_line_algorithm#Simplification
         * @param {number} x0 Start x coordinate
         * @param {number} y0 Start y coordinate
         * @param {number} x1 End x coordinate
         * @param {number} y1 End y coordinate
         * @return {Array<Array<number>>} The coordinates on the line
         */
        static interpolate(x0, y0, x1, y1) {
            var abs = Math.abs,
                line = [],
                sx, sy, dx, dy, err, e2;

            dx = abs(x1 - x0);
            dy = abs(y1 - y0);

            sx = (x0 < x1) ? 1 : -1;
            sy = (y0 < y1) ? 1 : -1;

            err = dx - dy;

            while (true) {
                line.push([x0, y0]);

                if (x0 === x1 && y0 === y1) {
                    break;
                }

                e2 = 2 * err;
                if (e2 > -dy) {
                    err = err - dy;
                    x0 = x0 + sx;
                }
                if (e2 < dx) {
                    err = err + dx;
                    y0 = y0 + sy;
                }
            }

            return line;
        }


        /**
         * Given a compressed path, return a new path that has all the segments
         * in it interpolated.
         * @param {Array<Array<number>>} path The path
         * @return {Array<Array<number>>} expanded path
         */
        static expandPath(path) {
            var expanded = [],
                len = path.length,
                coord0, coord1,
                interpolated,
                interpolatedLen,
                i, j;

            if (len < 2) {
                return expanded;
            }

            for (i = 0; i < len - 1; ++i) {
                coord0 = path[i];
                coord1 = path[i + 1];

                interpolated = Util.interpolate(coord0[0], coord0[1], coord1[0], coord1[1]);
                interpolatedLen = interpolated.length;
                for (j = 0; j < interpolatedLen - 1; ++j) {
                    expanded.push(interpolated[j]);
                }
            }
            expanded.push(path[len - 1]);

            return expanded;
        }


        /**
         * Smoothen the give path.
         * The original path will not be modified; a new path will be returned.
         * @param {PF.Grid} grid
         * @param {Array<Array<number>>} path The path
         */
        static smoothenPath(grid, path) {
            var len = path.length,
                x0 = path[0][0],        // path start x
                y0 = path[0][1],        // path start y
                x1 = path[len - 1][0],  // path end x
                y1 = path[len - 1][1],  // path end y
                sx, sy,                 // current start coordinate
                ex, ey,                 // current end coordinate
                newPath,
                i, j, coord, line, testCoord, blocked;

            sx = x0;
            sy = y0;
            newPath = [[sx, sy]];

            for (i = 2; i < len; ++i) {
                coord = path[i];
                ex = coord[0];
                ey = coord[1];
                line = Util.interpolate(sx, sy, ex, ey);

                blocked = false;
                for (j = 1; j < line.length; ++j) {
                    testCoord = line[j];

                    if (!grid.isWalkableAt(testCoord[0], testCoord[1])) {
                        blocked = true;
                        break;
                    }
                }
                if (blocked) {
                    lastValidCoord = path[i - 1];
                    newPath.push(lastValidCoord);
                    sx = lastValidCoord[0];
                    sy = lastValidCoord[1];
                }
            }
            newPath.push([x1, y1]);

            return newPath;
        }


        /**
         * Compress a path, remove redundant nodes without altering the shape
         * The original path is not modified
         * @param {Array<Array<number>>} path The path
         * @return {Array<Array<number>>} The compressed path
         */
        static compressPath(path) {

            // nothing to compress
            if(path.length < 3) {
                return path;
            }

            var compressed = [],
                sx = path[0][0], // start x
                sy = path[0][1], // start y
                px = path[1][0], // second point x
                py = path[1][1], // second point y
                dx = px - sx, // direction between the two points
                dy = py - sy, // direction between the two points
                lx, ly,
                ldx, ldy,
                sq, i;

            // normalize the direction
            sq = Math.sqrt(dx*dx + dy*dy);
            dx /= sq;
            dy /= sq;

            // start the new path
            compressed.push([sx,sy]);

            for(i = 2; i < path.length; i++) {

                // store the last point
                lx = px;
                ly = py;

                // store the last direction
                ldx = dx;
                ldy = dy;

                // next point
                px = path[i][0];
                py = path[i][1];

                // next direction
                dx = px - lx;
                dy = py - ly;

                // normalize
                sq = Math.sqrt(dx*dx + dy*dy);
                dx /= sq;
                dy /= sq;

                // if the direction has changed, store the point
                if ( dx !== ldx || dy !== ldy ) {
                    compressed.push([lx,ly]);
                }
            }

            // store the last point
            compressed.push([px,py]);

            return compressed;
        }
    }

    /**
     * @namespace PF.Heuristic
     * @description A collection of heuristic functions.
     */

    let Heuristic = {

      /**
       * Manhattan distance.
       * @param {number} dx - Difference in x.
       * @param {number} dy - Difference in y.
       * @return {number} dx + dy
       */
      manhattan: function(dx, dy) {
          return dx + dy;
      },

      /**
       * Euclidean distance.
       * @param {number} dx - Difference in x.
       * @param {number} dy - Difference in y.
       * @return {number} sqrt(dx * dx + dy * dy)
       */
      euclidean: function(dx, dy) {
          return Math.sqrt(dx * dx + dy * dy);
      },

      /**
       * Octile distance.
       * @param {number} dx - Difference in x.
       * @param {number} dy - Difference in y.
       * @return {number} sqrt(dx * dx + dy * dy) for grids
       */
      octile: function(dx, dy) {
          var F = Math.SQRT2 - 1;
          return (dx < dy) ? F * dx + dy : F * dy + dx;
      },

      /**
       * Chebyshev distance.
       * @param {number} dx - Difference in x.
       * @param {number} dy - Difference in y.
       * @return {number} max(dx, dy)
       */
      chebyshev: function(dx, dy) {
          return Math.max(dx, dy);
      },


      diagonal: function(dx, dy) {
        let minD = Math.min(dx, dy);
        return minD * Math.SQRT2 + (dx + dy - 2 * minD);
      },

    };

    /**
     * A* path-finder. Based upon https://github.com/bgrins/javascript-astar
     * @constructor
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     * @param {number} opt.weight Weight to apply to the heuristic to allow for
     *     suboptimal paths, in order to speed up the search.
     */
    class AStarFinder {
        constructor(opt) {
            opt = opt || {};
            this.allowDiagonal = opt.allowDiagonal;
            this.dontCrossCorners = opt.dontCrossCorners;
            this.heuristic = opt.heuristic || Heuristic.manhattan;
            this.weight = opt.weight || 1;
            this.diagonalMovement = opt.diagonalMovement;

            if (!this.diagonalMovement) {
                if (!this.allowDiagonal) {
                    this.diagonalMovement = DiagonalMovement.Never;
                } else {
                    if (this.dontCrossCorners) {
                        this.diagonalMovement = DiagonalMovement.OnlyWhenNoObstacles;
                    } else {
                        this.diagonalMovement = DiagonalMovement.IfAtMostOneObstacle;
                    }
                }
            }

            // When diagonal movement is allowed the manhattan heuristic is not
            //admissible. It should be octile instead
            if (this.diagonalMovement === DiagonalMovement.Never) {
                this.heuristic = opt.heuristic || Heuristic.manhattan;
            } else {
                this.heuristic = opt.heuristic || Heuristic.octile;
            }
        }


        /**
         * Find and return the the path.
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            var openList = new Heap(function(nodeA, nodeB) {
                    return nodeA.f - nodeB.f;
                }),
                startNode = grid.getNodeAt(startX, startY),
                endNode = grid.getNodeAt(endX, endY),
                heuristic = this.heuristic,
                diagonalMovement = this.diagonalMovement,
                weight = this.weight,
                abs = Math.abs, SQRT2 = Math.SQRT2,
                node, neighbors, neighbor, i, l, x, y, ng;

            // set the `g` and `f` value of the start node to be 0
            startNode.g = 0;
            startNode.f = 0;

            // push the start node into the open list
            openList.push(startNode);
            startNode.opened = true;

            // while the open list is not empty
            while (!openList.empty()) {
                // pop the position of node which has the minimum `f` value.
                node = openList.pop();
                node.closed = true;

                // if reached the end position, construct the path and return it
                if (node === endNode) {
                    return Util.backtrace(endNode);
                }

                // get neigbours of the current node
                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    if (neighbor.closed) {
                        continue;
                    }

                    x = neighbor.x;
                    y = neighbor.y;

                    // get the distance between current node and the neighbor
                    // and calculate the next g score
                    ng = node.g + ((x - node.x === 0 || y - node.y === 0) ? 1 : SQRT2);

                    // check if the neighbor has not been inspected yet, or
                    // can be reached with smaller cost from the current node
                    if (!neighbor.opened || ng < neighbor.g) {
                        neighbor.g = ng;
                        neighbor.h = neighbor.h || weight * heuristic(abs(x - endX), abs(y - endY));
                        neighbor.f = neighbor.g + neighbor.h;
                        neighbor.parent = node;

                        if (!neighbor.opened) {
                            openList.push(neighbor);
                            neighbor.opened = true;
                        } else {
                            // the neighbor can be reached with smaller cost.
                            // Since its f value has been updated, we have to
                            // update its position in the open list
                            openList.updateItem(neighbor);
                        }
                    }
                } // end for each neighbor
            } // end while not open list empty

            // fail to find the path
            return [];
        }
    }

    /**
     * Best-First-Search path-finder.
     * @constructor
     * @extends AStarFinder
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     */

    class BestFirstFinder extends AStarFinder {
        constructor(opt) {
            super(opt);
            var orig = this.heuristic;
            this.heuristic = function(dx, dy) {
                return orig(dx, dy) * 1000000;
            };
        }
    }

    /**
     * Breadth-First-Search path finder.
     * @constructor
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     */
    class BreadthFirstFinder {
        constructor(opt) {
            opt = opt || {};
            this.allowDiagonal = opt.allowDiagonal;
            this.dontCrossCorners = opt.dontCrossCorners;
            this.diagonalMovement = opt.diagonalMovement;

            if (!this.diagonalMovement) {
                if (!this.allowDiagonal) {
                    this.diagonalMovement = DiagonalMovement.Never;
                } else {
                    if (this.dontCrossCorners) {
                        this.diagonalMovement = DiagonalMovement.OnlyWhenNoObstacles;
                    } else {
                        this.diagonalMovement = DiagonalMovement.IfAtMostOneObstacle;
                    }
                }
            }
        }

        /**
         * Find and return the the path.
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            var openList = [],
                diagonalMovement = this.diagonalMovement,
                startNode = grid.getNodeAt(startX, startY),
                endNode = grid.getNodeAt(endX, endY),
                neighbors, neighbor, node, i, l;

            // push the start pos into the queue
            openList.push(startNode);
            startNode.opened = true;

            // while the queue is not empty
            while (openList.length) {
                // take the front node from the queue
                node = openList.shift();
                node.closed = true;

                // reached the end position
                if (node === endNode) {
                    return Util.backtrace(endNode);
                }

                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    // skip this neighbor if it has been inspected before
                    if (neighbor.closed || neighbor.opened) {
                        continue;
                    }

                    openList.push(neighbor);
                    neighbor.opened = true;
                    neighbor.parent = node;
                }
            }

            // fail to find the path
            return [];
        }
    }

    /**
     * Dijkstra path-finder.
     * @constructor
     * @extends AStarFinder
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     */
    class DijkstraFinder extends AStarFinder {
        constructor(opt) {
            super(opt);
            this.heuristic = function(dx, dy) {
                return 0;
            };
        }
    }

    /**
     * A* path-finder.
     * based upon https://github.com/bgrins/javascript-astar
     * @constructor
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     * @param {number} opt.weight Weight to apply to the heuristic to allow for
     *     suboptimal paths, in order to speed up the search.
     */
    class BiAStarFinder {
        constructor(opt) {
            opt = opt || {};
            this.allowDiagonal = opt.allowDiagonal;
            this.dontCrossCorners = opt.dontCrossCorners;
            this.diagonalMovement = opt.diagonalMovement;
            this.heuristic = opt.heuristic || Heuristic.manhattan;
            this.weight = opt.weight || 1;

            if (!this.diagonalMovement) {
                if (!this.allowDiagonal) {
                    this.diagonalMovement = DiagonalMovement.Never;
                } else {
                    if (this.dontCrossCorners) {
                        this.diagonalMovement = DiagonalMovement.OnlyWhenNoObstacles;
                    } else {
                        this.diagonalMovement = DiagonalMovement.IfAtMostOneObstacle;
                    }
                }
            }

            //When diagonal movement is allowed the manhattan heuristic is not admissible
            //It should be octile instead
            if (this.diagonalMovement === DiagonalMovement.Never) {
                this.heuristic = opt.heuristic || Heuristic.manhattan;
            } else {
                this.heuristic = opt.heuristic || Heuristic.octile;
            }
        }

        /**
         * Find and return the the path.
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            var cmp = function(nodeA, nodeB) {
                    return nodeA.f - nodeB.f;
                },
                startOpenList = new Heap(cmp),
                endOpenList = new Heap(cmp),
                startNode = grid.getNodeAt(startX, startY),
                endNode = grid.getNodeAt(endX, endY),
                heuristic = this.heuristic,
                diagonalMovement = this.diagonalMovement,
                weight = this.weight,
                abs = Math.abs, SQRT2 = Math.SQRT2,
                node, neighbors, neighbor, i, l, x, y, ng,
                BY_START = 1, BY_END = 2;

            // set the `g` and `f` value of the start node to be 0
            // and push it into the start open list
            startNode.g = 0;
            startNode.f = 0;
            startOpenList.push(startNode);
            startNode.opened = BY_START;

            // set the `g` and `f` value of the end node to be 0
            // and push it into the open open list
            endNode.g = 0;
            endNode.f = 0;
            endOpenList.push(endNode);
            endNode.opened = BY_END;

            // while both the open lists are not empty
            while (!startOpenList.empty() && !endOpenList.empty()) {

                // pop the position of start node which has the minimum `f` value.
                node = startOpenList.pop();
                node.closed = true;

                // get neigbours of the current node
                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    if (neighbor.closed) {
                        continue;
                    }
                    if (neighbor.opened === BY_END) {
                        return Util.biBacktrace(node, neighbor);
                    }

                    x = neighbor.x;
                    y = neighbor.y;

                    // get the distance between current node and the neighbor
                    // and calculate the next g score
                    ng = node.g + ((x - node.x === 0 || y - node.y === 0) ? 1 : SQRT2);

                    // check if the neighbor has not been inspected yet, or
                    // can be reached with smaller cost from the current node
                    if (!neighbor.opened || ng < neighbor.g) {
                        neighbor.g = ng;
                        neighbor.h = neighbor.h ||
                            weight * heuristic(abs(x - endX), abs(y - endY));
                        neighbor.f = neighbor.g + neighbor.h;
                        neighbor.parent = node;

                        if (!neighbor.opened) {
                            startOpenList.push(neighbor);
                            neighbor.opened = BY_START;
                        } else {
                            // the neighbor can be reached with smaller cost.
                            // Since its f value has been updated, we have to
                            // update its position in the open list
                            startOpenList.updateItem(neighbor);
                        }
                    }
                } // end for each neighbor


                // pop the position of end node which has the minimum `f` value.
                node = endOpenList.pop();
                node.closed = true;

                // get neigbours of the current node
                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    if (neighbor.closed) {
                        continue;
                    }
                    if (neighbor.opened === BY_START) {
                        return Util.biBacktrace(neighbor, node);
                    }

                    x = neighbor.x;
                    y = neighbor.y;

                    // get the distance between current node and the neighbor
                    // and calculate the next g score
                    ng = node.g + ((x - node.x === 0 || y - node.y === 0) ? 1 : SQRT2);

                    // check if the neighbor has not been inspected yet, or
                    // can be reached with smaller cost from the current node
                    if (!neighbor.opened || ng < neighbor.g) {
                        neighbor.g = ng;
                        neighbor.h = neighbor.h ||
                            weight * heuristic(abs(x - startX), abs(y - startY));
                        neighbor.f = neighbor.g + neighbor.h;
                        neighbor.parent = node;

                        if (!neighbor.opened) {
                            endOpenList.push(neighbor);
                            neighbor.opened = BY_END;
                        } else {
                            // the neighbor can be reached with smaller cost.
                            // Since its f value has been updated, we have to
                            // update its position in the open list
                            endOpenList.updateItem(neighbor);
                        }
                    }
                } // end for each neighbor
            } // end while not open list empty

            // fail to find the path
            return [];
        }
    }

    /**
     * Bi-direcitional Best-First-Search path-finder.
     * @constructor
     * @extends BiAStarFinder
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     */
    class BiBestFirstFinder extends BiAStarFinder {
        constructor(opt) {
            super(opt);
            var orig = this.heuristic;
            this.heuristic = function(dx, dy) {
                return orig(dx, dy) * 1000000;
            };
        }
    }

    /**
     * Bi-directional Breadth-First-Search path finder.
     * @constructor
     * @param {object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     */
    class BiBreadthFirstFinder {
        constructor(opt) {
            opt = opt || {};
            this.allowDiagonal = opt.allowDiagonal;
            this.dontCrossCorners = opt.dontCrossCorners;
            this.diagonalMovement = opt.diagonalMovement;

            if (!this.diagonalMovement) {
                if (!this.allowDiagonal) {
                    this.diagonalMovement = DiagonalMovement.Never;
                } else {
                    if (this.dontCrossCorners) {
                        this.diagonalMovement = DiagonalMovement.OnlyWhenNoObstacles;
                    } else {
                        this.diagonalMovement = DiagonalMovement.IfAtMostOneObstacle;
                    }
                }
            }
        }


        /**
         * Find and return the the path.
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            var startNode = grid.getNodeAt(startX, startY),
                endNode = grid.getNodeAt(endX, endY),
                startOpenList = [], endOpenList = [],
                neighbors, neighbor, node,
                diagonalMovement = this.diagonalMovement,
                BY_START = 0, BY_END = 1,
                i, l;

            // push the start and end nodes into the queues
            startOpenList.push(startNode);
            startNode.opened = true;
            startNode.by = BY_START;

            endOpenList.push(endNode);
            endNode.opened = true;
            endNode.by = BY_END;

            // while both the queues are not empty
            while (startOpenList.length && endOpenList.length) {

                // expand start open list

                node = startOpenList.shift();
                node.closed = true;

                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    if (neighbor.closed) {
                        continue;
                    }
                    if (neighbor.opened) {
                        // if this node has been inspected by the reversed search,
                        // then a path is found.
                        if (neighbor.by === BY_END) {
                            return Util.biBacktrace(node, neighbor);
                        }
                        continue;
                    }
                    startOpenList.push(neighbor);
                    neighbor.parent = node;
                    neighbor.opened = true;
                    neighbor.by = BY_START;
                }

                // expand end open list

                node = endOpenList.shift();
                node.closed = true;

                neighbors = grid.getNeighbors(node, diagonalMovement);
                for (i = 0, l = neighbors.length; i < l; ++i) {
                    neighbor = neighbors[i];

                    if (neighbor.closed) {
                        continue;
                    }
                    if (neighbor.opened) {
                        if (neighbor.by === BY_START) {
                            return Util.biBacktrace(neighbor, node);
                        }
                        continue;
                    }
                    endOpenList.push(neighbor);
                    neighbor.parent = node;
                    neighbor.opened = true;
                    neighbor.by = BY_END;
                }
            }

            // fail to find the path
            return [];
        }
    }

    /**
     * Bi-directional Dijkstra path-finder.
     * @constructor
     * @extends BiAStarFinder
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     */
    class BiBestFirstFinder$1 extends BiAStarFinder {
        constructor(opt) {
            super(opt);
            this.heuristic = function(dx, dy) {
                return 0;
            };
        }
    }

    /**
     * Iterative Deeping A Star (IDA*) path-finder.
     *
     * Recursion based on:
     *   http://www.apl.jhu.edu/~hall/AI-Programming/IDA-Star.html
     *
     * Path retracing based on:
     *  V. Nageshwara Rao, Vipin Kumar and K. Ramesh
     *  "A Parallel Implementation of Iterative-Deeping-A*", January 1987.
     *  ftp://ftp.cs.utexas.edu/.snapshot/hourly.1/pub/AI-Lab/tech-reports/UT-AI-TR-87-46.pdf
     *
     * @author Gerard Meier (www.gerardmeier.com)
     *
     * @constructor
     * @param {Object} opt
     * @param {boolean} opt.allowDiagonal Whether diagonal movement is allowed.
     *     Deprecated, use diagonalMovement instead.
     * @param {boolean} opt.dontCrossCorners Disallow diagonal movement touching
     *     block corners. Deprecated, use diagonalMovement instead.
     * @param {DiagonalMovement} opt.diagonalMovement Allowed diagonal movement.
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     * @param {number} opt.weight Weight to apply to the heuristic to allow for
     *     suboptimal paths, in order to speed up the search.
     * @param {boolean} opt.trackRecursion Whether to track recursion for
     *     statistical purposes.
     * @param {number} opt.timeLimit Maximum execution time. Use <= 0 for infinite.
     */
    class IDAStarFinder {
        constructor(opt) {
            opt = opt || {};
            this.allowDiagonal = opt.allowDiagonal;
            this.dontCrossCorners = opt.dontCrossCorners;
            this.diagonalMovement = opt.diagonalMovement;
            this.heuristic = opt.heuristic || Heuristic.manhattan;
            this.weight = opt.weight || 1;
            this.trackRecursion = opt.trackRecursion || false;
            this.timeLimit = opt.timeLimit || Infinity; // Default: no time limit.

            if (!this.diagonalMovement) {
                if (!this.allowDiagonal) {
                    this.diagonalMovement = DiagonalMovement.Never;
                } else {
                    if (this.dontCrossCorners) {
                        this.diagonalMovement = DiagonalMovement.OnlyWhenNoObstacles;
                    } else {
                        this.diagonalMovement = DiagonalMovement.IfAtMostOneObstacle;
                    }
                }
            }

            // When diagonal movement is allowed the manhattan heuristic is not
            // admissible, it should be octile instead
            if (this.diagonalMovement === DiagonalMovement.Never) {
                this.heuristic = opt.heuristic || Heuristic.manhattan;
            } else {
                this.heuristic = opt.heuristic || Heuristic.octile;
            }
        }

        /**
         * Find and return the the path. When an empty array is returned, either
         * no path is possible, or the maximum execution time is reached.
         *
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            // Used for statistics:
            var nodesVisited = 0;

            // Execution time limitation:
            var startTime = new Date().getTime();

            // Heuristic helper:
            var h = function(a, b) {
                return this.heuristic(Math.abs(b.x - a.x), Math.abs(b.y - a.y));
            }.bind(this);

            // Step cost from a to b:
            var cost = function(a, b) {
                return (a.x === b.x || a.y === b.y) ? 1 : Math.SQRT2;
            };

            /**
             * IDA* search implementation.
             *
             * @param {Node} The node currently expanding from.
             * @param {number} Cost to reach the given node.
             * @param {number} Maximum search depth (cut-off value).
             * @param {Array<Array<number>>} The found route.
             * @param {number} Recursion depth.
             *
             * @return {Object} either a number with the new optimal cut-off depth,
             * or a valid node instance, in which case a path was found.
             */
            var search = function(node, g, cutoff, route, depth) {
                nodesVisited++;

                // Enforce timelimit:
                if (this.timeLimit > 0 &&
                    new Date().getTime() - startTime > this.timeLimit * 1000) {
                    // Enforced as "path-not-found".
                    return Infinity;
                }

                var f = g + h(node, end) * this.weight;

                // We've searched too deep for this iteration.
                if (f > cutoff) {
                    return f;
                }

                if (node == end) {
                    route[depth] = [node.x, node.y];
                    return node;
                }

                var min, t, k, neighbour;

                var neighbours = grid.getNeighbors(node, this.diagonalMovement);

                // Sort the neighbours, gives nicer paths. But, this deviates
                // from the original algorithm - so I left it out.
                //neighbours.sort(function(a, b){
                //    return h(a, end) - h(b, end);
                //});


                /*jshint -W084 *///Disable warning: Expected a conditional expression and instead saw an assignment
                for (k = 0, min = Infinity; neighbour = neighbours[k]; ++k) {
                /*jshint +W084 *///Enable warning: Expected a conditional expression and instead saw an assignment
                    if (this.trackRecursion) {
                        // Retain a copy for visualisation. Due to recursion, this
                        // node may be part of other paths too.
                        neighbour.retainCount = neighbour.retainCount + 1 || 1;

                        if(neighbour.tested !== true) {
                            neighbour.tested = true;
                        }
                    }

                    t = search(neighbour, g + cost(node, neighbour), cutoff, route, depth + 1);

                    if (t instanceof Node) {
                        route[depth] = [node.x, node.y];

                        // For a typical A* linked list, this would work:
                        // neighbour.parent = node;
                        return t;
                    }

                    // Decrement count, then determine whether it's actually closed.
                    if (this.trackRecursion && (--neighbour.retainCount) === 0) {
                        neighbour.tested = false;
                    }

                    if (t < min) {
                        min = t;
                    }
                }

                return min;

            }.bind(this);

            // Node instance lookups:
            var start = grid.getNodeAt(startX, startY);
            var end   = grid.getNodeAt(endX, endY);

            // Initial search depth, given the typical heuristic contraints,
            // there should be no cheaper route possible.
            var cutOff = h(start, end);

            var j, route, t;

            // With an overflow protection.
            for (j = 0; true; ++j) {

                route = [];

                // Search till cut-off depth:
                t = search(start, 0, cutOff, route, 0);

                // Route not possible, or not found in time limit.
                if (t === Infinity) {
                    return [];
                }

                // If t is a node, it's also the end node. Route is now
                // populated with a valid path to the end node.
                if (t instanceof Node) {
                    return route;
                }

                // Try again, this time with a deeper cut-off. The t score
                // is the closest we got to the end node.
                cutOff = t;
            }

            // This _should_ never to be reached.
            return [];
        }

    }

    /**
     * @author imor / https://github.com/imor
     */

    /**
     * Base class for the Jump Point Search algorithm
     * @param {object} opt
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     */
    class JumpPointFinderBase {
        constructor(opt) {
            opt = opt || {};
            this.heuristic = opt.heuristic || Heuristic.manhattan;
            this.trackJumpRecursion = opt.trackJumpRecursion || false;
        }

        /**
         * Find and return the path.
         * @return {Array<Array<number>>} The path, including both start and
         *     end positions.
         */
        findPath(startX, startY, endX, endY, grid) {
            var openList = this.openList = new Heap(function(nodeA, nodeB) {
                    return nodeA.f - nodeB.f;
                }),
                startNode = this.startNode = grid.getNodeAt(startX, startY),
                endNode = this.endNode = grid.getNodeAt(endX, endY), node;

            this.grid = grid;


            // set the `g` and `f` value of the start node to be 0
            startNode.g = 0;
            startNode.f = 0;

            // push the start node into the open list
            openList.push(startNode);
            startNode.opened = true;

            // while the open list is not empty
            while (!openList.empty()) {
                // pop the position of node which has the minimum `f` value.
                node = openList.pop();
                node.closed = true;

                if (node === endNode) {
                    return Util.expandPath(Util.backtrace(endNode));
                }

                this._identifySuccessors(node);
            }

            // fail to find the path
            return [];
        }

        /**
         * Identify successors for the given node. Runs a jump point search in the
         * direction of each available neighbor, adding any points found to the open
         * list.
         * @protected
         */
        _identifySuccessors(node) {
            var grid = this.grid,
                heuristic = this.heuristic,
                openList = this.openList,
                endX = this.endNode.x,
                endY = this.endNode.y,
                neighbors, neighbor,
                jumpPoint, i, l,
                x = node.x, y = node.y,
                jx, jy, dx, dy, d, ng, jumpNode,
                abs = Math.abs, max = Math.max;

            neighbors = this._findNeighbors(node);
            for(i = 0, l = neighbors.length; i < l; ++i) {
                neighbor = neighbors[i];
                jumpPoint = this._jump(neighbor[0], neighbor[1], x, y);
                if (jumpPoint) {

                    jx = jumpPoint[0];
                    jy = jumpPoint[1];
                    jumpNode = grid.getNodeAt(jx, jy);

                    if (jumpNode.closed) {
                        continue;
                    }

                    // include distance, as parent may not be immediately adjacent:
                    d = Heuristic.octile(abs(jx - x), abs(jy - y));
                    ng = node.g + d; // next `g` value

                    if (!jumpNode.opened || ng < jumpNode.g) {
                        jumpNode.g = ng;
                        jumpNode.h = jumpNode.h || heuristic(abs(jx - endX), abs(jy - endY));
                        jumpNode.f = jumpNode.g + jumpNode.h;
                        jumpNode.parent = node;

                        if (!jumpNode.opened) {
                            openList.push(jumpNode);
                            jumpNode.opened = true;
                        } else {
                            openList.updateItem(jumpNode);
                        }
                    }
                }
            }
        }
    }

    /**
     * @author imor / https://github.com/imor
     */

    /**
     * Path finder using the Jump Point Search algorithm allowing only horizontal
     * or vertical movements.
     */
    class JPFNeverMoveDiagonally extends JumpPointFinderBase {

        /**
         * Search recursively in the direction (parent -> child), stopping only when a
         * jump point is found.
         * @protected
         * @return {Array<Array<number>>} The x, y coordinate of the jump point
         *     found, or null if not found
         */
        _jump(x, y, px, py) {
            var grid = this.grid,
                dx = x - px, dy = y - py;

            if (!grid.isWalkableAt(x, y)) {
                return null;
            }

            if(this.trackJumpRecursion === true) {
                grid.getNodeAt(x, y).tested = true;
            }

            if (grid.getNodeAt(x, y) === this.endNode) {
                return [x, y];
            }

            if (dx !== 0) {
                if ((grid.isWalkableAt(x, y - 1) && !grid.isWalkableAt(x - dx, y - 1)) ||
                    (grid.isWalkableAt(x, y + 1) && !grid.isWalkableAt(x - dx, y + 1))) {
                    return [x, y];
                }
            }
            else if (dy !== 0) {
                if ((grid.isWalkableAt(x - 1, y) && !grid.isWalkableAt(x - 1, y - dy)) ||
                    (grid.isWalkableAt(x + 1, y) && !grid.isWalkableAt(x + 1, y - dy))) {
                    return [x, y];
                }
                //When moving vertically, must check for horizontal jump points
                if (this._jump(x + 1, y, x, y) || this._jump(x - 1, y, x, y)) {
                    return [x, y];
                }
            }
            else {
                throw new Error("Only horizontal and vertical movements are allowed");
            }

            return this._jump(x + dx, y + dy, x, y);
        }

        /**
         * Find the neighbors for the given node. If the node has a parent,
         * prune the neighbors based on the jump point search algorithm, otherwise
         * return all available neighbors.
         * @return {Array<Array<number>>} The neighbors found.
         */
        _findNeighbors(node) {
            var parent = node.parent,
                x = node.x, y = node.y,
                grid = this.grid,
                px, py, nx, ny, dx, dy,
                neighbors = [], neighborNodes, neighborNode, i, l;

            // directed pruning: can ignore most neighbors, unless forced.
            if (parent) {
                px = parent.x;
                py = parent.y;
                // get the normalized direction of travel
                dx = (x - px) / Math.max(Math.abs(x - px), 1);
                dy = (y - py) / Math.max(Math.abs(y - py), 1);

                if (dx !== 0) {
                    if (grid.isWalkableAt(x, y - 1)) {
                        neighbors.push([x, y - 1]);
                    }
                    if (grid.isWalkableAt(x, y + 1)) {
                        neighbors.push([x, y + 1]);
                    }
                    if (grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y]);
                    }
                }
                else if (dy !== 0) {
                    if (grid.isWalkableAt(x - 1, y)) {
                        neighbors.push([x - 1, y]);
                    }
                    if (grid.isWalkableAt(x + 1, y)) {
                        neighbors.push([x + 1, y]);
                    }
                    if (grid.isWalkableAt(x, y + dy)) {
                        neighbors.push([x, y + dy]);
                    }
                }
            }
            // return all neighbors
            else {
                neighborNodes = grid.getNeighbors(node, DiagonalMovement.Never);
                for (i = 0, l = neighborNodes.length; i < l; ++i) {
                    neighborNode = neighborNodes[i];
                    neighbors.push([neighborNode.x, neighborNode.y]);
                }
            }

            return neighbors;
        }
    }

    /**
     * @author imor / https://github.com/imor
     */

    /**
     * Path finder using the Jump Point Search algorithm which always moves
     * diagonally irrespective of the number of obstacles.
     */
    class JPFAlwaysMoveDiagonally extends JumpPointFinderBase {


        /**
         * Search recursively in the direction (parent -> child), stopping only when a
         * jump point is found.
         * @protected
         * @return {Array<Array<number>>} The x, y coordinate of the jump point
         *     found, or null if not found
         */
        _jump(x, y, px, py) {
            var grid = this.grid,
                dx = x - px, dy = y - py;

            if (!grid.isWalkableAt(x, y)) {
                return null;
            }

            if(this.trackJumpRecursion === true) {
                grid.getNodeAt(x, y).tested = true;
            }

            if (grid.getNodeAt(x, y) === this.endNode) {
                return [x, y];
            }

            // check for forced neighbors
            // along the diagonal
            if (dx !== 0 && dy !== 0) {
                if ((grid.isWalkableAt(x - dx, y + dy) && !grid.isWalkableAt(x - dx, y)) ||
                    (grid.isWalkableAt(x + dx, y - dy) && !grid.isWalkableAt(x, y - dy))) {
                    return [x, y];
                }
                // when moving diagonally, must check for vertical/horizontal jump points
                if (this._jump(x + dx, y, x, y) || this._jump(x, y + dy, x, y)) {
                    return [x, y];
                }
            }
            // horizontally/vertically
            else {
                if( dx !== 0 ) { // moving along x
                    if((grid.isWalkableAt(x + dx, y + 1) && !grid.isWalkableAt(x, y + 1)) ||
                       (grid.isWalkableAt(x + dx, y - 1) && !grid.isWalkableAt(x, y - 1))) {
                        return [x, y];
                    }
                }
                else {
                    if((grid.isWalkableAt(x + 1, y + dy) && !grid.isWalkableAt(x + 1, y)) ||
                       (grid.isWalkableAt(x - 1, y + dy) && !grid.isWalkableAt(x - 1, y))) {
                        return [x, y];
                    }
                }
            }

            return this._jump(x + dx, y + dy, x, y);
        }

        /**
         * Find the neighbors for the given node. If the node has a parent,
         * prune the neighbors based on the jump point search algorithm, otherwise
         * return all available neighbors.
         * @return {Array<Array<number>>} The neighbors found.
         */
        _findNeighbors(node) {
            var parent = node.parent,
                x = node.x, y = node.y,
                grid = this.grid,
                px, py, nx, ny, dx, dy,
                neighbors = [], neighborNodes, neighborNode, i, l;

            // directed pruning: can ignore most neighbors, unless forced.
            if (parent) {
                px = parent.x;
                py = parent.y;
                // get the normalized direction of travel
                dx = (x - px) / Math.max(Math.abs(x - px), 1);
                dy = (y - py) / Math.max(Math.abs(y - py), 1);

                // search diagonally
                if (dx !== 0 && dy !== 0) {
                    if (grid.isWalkableAt(x, y + dy)) {
                        neighbors.push([x, y + dy]);
                    }
                    if (grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y]);
                    }
                    if (grid.isWalkableAt(x + dx, y + dy)) {
                        neighbors.push([x + dx, y + dy]);
                    }
                    if (!grid.isWalkableAt(x - dx, y)) {
                        neighbors.push([x - dx, y + dy]);
                    }
                    if (!grid.isWalkableAt(x, y - dy)) {
                        neighbors.push([x + dx, y - dy]);
                    }
                }
                // search horizontally/vertically
                else {
                    if(dx === 0) {
                        if (grid.isWalkableAt(x, y + dy)) {
                            neighbors.push([x, y + dy]);
                        }
                        if (!grid.isWalkableAt(x + 1, y)) {
                            neighbors.push([x + 1, y + dy]);
                        }
                        if (!grid.isWalkableAt(x - 1, y)) {
                            neighbors.push([x - 1, y + dy]);
                        }
                    }
                    else {
                        if (grid.isWalkableAt(x + dx, y)) {
                            neighbors.push([x + dx, y]);
                        }
                        if (!grid.isWalkableAt(x, y + 1)) {
                            neighbors.push([x + dx, y + 1]);
                        }
                        if (!grid.isWalkableAt(x, y - 1)) {
                            neighbors.push([x + dx, y - 1]);
                        }
                    }
                }
            }
            // return all neighbors
            else {
                neighborNodes = grid.getNeighbors(node, DiagonalMovement.Always);
                for (i = 0, l = neighborNodes.length; i < l; ++i) {
                    neighborNode = neighborNodes[i];
                    neighbors.push([neighborNode.x, neighborNode.y]);
                }
            }

            return neighbors;
        }
    }

    /**
     * @author imor / https://github.com/imor
     */

    /**
     * Path finder using the Jump Point Search algorithm which moves
     * diagonally only when there are no obstacles.
     */
    class JPFMoveDiagonallyIfNoObstacles extends JumpPointFinderBase {

        /**
         * Search recursively in the direction (parent -> child), stopping only when a
         * jump point is found.
         * @protected
         * @return {Array<Array<number>>} The x, y coordinate of the jump point
         *     found, or null if not found
         */
        _jump(x, y, px, py) {
            var grid = this.grid,
                dx = x - px, dy = y - py;

            if (!grid.isWalkableAt(x, y)) {
                return null;
            }

            if(this.trackJumpRecursion === true) {
                grid.getNodeAt(x, y).tested = true;
            }

            if (grid.getNodeAt(x, y) === this.endNode) {
                return [x, y];
            }

            // check for forced neighbors
            // along the diagonal
            if (dx !== 0 && dy !== 0) {
                // if ((grid.isWalkableAt(x - dx, y + dy) && !grid.isWalkableAt(x - dx, y)) ||
                    // (grid.isWalkableAt(x + dx, y - dy) && !grid.isWalkableAt(x, y - dy))) {
                    // return [x, y];
                // }
                // when moving diagonally, must check for vertical/horizontal jump points
                if (this._jump(x + dx, y, x, y) || this._jump(x, y + dy, x, y)) {
                    return [x, y];
                }
            }
            // horizontally/vertically
            else {
                if (dx !== 0) {
                    if ((grid.isWalkableAt(x, y - 1) && !grid.isWalkableAt(x - dx, y - 1)) ||
                        (grid.isWalkableAt(x, y + 1) && !grid.isWalkableAt(x - dx, y + 1))) {
                        return [x, y];
                    }
                }
                else if (dy !== 0) {
                    if ((grid.isWalkableAt(x - 1, y) && !grid.isWalkableAt(x - 1, y - dy)) ||
                        (grid.isWalkableAt(x + 1, y) && !grid.isWalkableAt(x + 1, y - dy))) {
                        return [x, y];
                    }
                    // When moving vertically, must check for horizontal jump points
                    // if (this._jump(x + 1, y, x, y) || this._jump(x - 1, y, x, y)) {
                        // return [x, y];
                    // }
                }
            }

            // moving diagonally, must make sure one of the vertical/horizontal
            // neighbors is open to allow the path
            if (grid.isWalkableAt(x + dx, y) && grid.isWalkableAt(x, y + dy)) {
                return this._jump(x + dx, y + dy, x, y);
            } else {
                return null;
            }
        }

        /**
         * Find the neighbors for the given node. If the node has a parent,
         * prune the neighbors based on the jump point search algorithm, otherwise
         * return all available neighbors.
         * @return {Array<Array<number>>} The neighbors found.
         */
        _findNeighbors(node) {
            var parent = node.parent,
                x = node.x, y = node.y,
                grid = this.grid,
                px, py, nx, ny, dx, dy,
                neighbors = [], neighborNodes, neighborNode, i, l;

            // directed pruning: can ignore most neighbors, unless forced.
            if (parent) {
                px = parent.x;
                py = parent.y;
                // get the normalized direction of travel
                dx = (x - px) / Math.max(Math.abs(x - px), 1);
                dy = (y - py) / Math.max(Math.abs(y - py), 1);

                // search diagonally
                if (dx !== 0 && dy !== 0) {
                    if (grid.isWalkableAt(x, y + dy)) {
                        neighbors.push([x, y + dy]);
                    }
                    if (grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y]);
                    }
                    if (grid.isWalkableAt(x, y + dy) && grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y + dy]);
                    }
                }
                // search horizontally/vertically
                else {
                    var isNextWalkable;
                    if (dx !== 0) {
                        isNextWalkable = grid.isWalkableAt(x + dx, y);
                        var isTopWalkable = grid.isWalkableAt(x, y + 1);
                        var isBottomWalkable = grid.isWalkableAt(x, y - 1);

                        if (isNextWalkable) {
                            neighbors.push([x + dx, y]);
                            if (isTopWalkable) {
                                neighbors.push([x + dx, y + 1]);
                            }
                            if (isBottomWalkable) {
                                neighbors.push([x + dx, y - 1]);
                            }
                        }
                        if (isTopWalkable) {
                            neighbors.push([x, y + 1]);
                        }
                        if (isBottomWalkable) {
                            neighbors.push([x, y - 1]);
                        }
                    }
                    else if (dy !== 0) {
                        isNextWalkable = grid.isWalkableAt(x, y + dy);
                        var isRightWalkable = grid.isWalkableAt(x + 1, y);
                        var isLeftWalkable = grid.isWalkableAt(x - 1, y);

                        if (isNextWalkable) {
                            neighbors.push([x, y + dy]);
                            if (isRightWalkable) {
                                neighbors.push([x + 1, y + dy]);
                            }
                            if (isLeftWalkable) {
                                neighbors.push([x - 1, y + dy]);
                            }
                        }
                        if (isRightWalkable) {
                            neighbors.push([x + 1, y]);
                        }
                        if (isLeftWalkable) {
                            neighbors.push([x - 1, y]);
                        }
                    }
                }
            }
            // return all neighbors
            else {
                neighborNodes = grid.getNeighbors(node, DiagonalMovement.OnlyWhenNoObstacles);
                for (i = 0, l = neighborNodes.length; i < l; ++i) {
                    neighborNode = neighborNodes[i];
                    neighbors.push([neighborNode.x, neighborNode.y]);
                }
            }

            return neighbors;
        }
    }

    /**
     * @author imor / https://github.com/imor
     */

    /**
     * Path finder using the Jump Point Search algorithm which moves
     * diagonally only when there is at most one obstacle.
     */

    class JPFMoveDiagonallyIfAtMostOneObstacle extends JumpPointFinderBase {

        /**
         * Search recursively in the direction (parent -> child), stopping only when a
         * jump point is found.
         * @protected
         * @return {Array<Array<number>>} The x, y coordinate of the jump point
         *     found, or null if not found
         */
        _jump(x, y, px, py) {
            var grid = this.grid,
                dx = x - px, dy = y - py;

            if (!grid.isWalkableAt(x, y)) {
                return null;
            }

            if(this.trackJumpRecursion === true) {
                grid.getNodeAt(x, y).tested = true;
            }

            if (grid.getNodeAt(x, y) === this.endNode) {
                return [x, y];
            }

            // check for forced neighbors
            // along the diagonal
            if (dx !== 0 && dy !== 0) {
                if ((grid.isWalkableAt(x - dx, y + dy) && !grid.isWalkableAt(x - dx, y)) ||
                    (grid.isWalkableAt(x + dx, y - dy) && !grid.isWalkableAt(x, y - dy))) {
                    return [x, y];
                }
                // when moving diagonally, must check for vertical/horizontal jump points
                if (this._jump(x + dx, y, x, y) || this._jump(x, y + dy, x, y)) {
                    return [x, y];
                }
            }
            // horizontally/vertically
            else {
                if( dx !== 0 ) { // moving along x
                    if((grid.isWalkableAt(x + dx, y + 1) && !grid.isWalkableAt(x, y + 1)) ||
                       (grid.isWalkableAt(x + dx, y - 1) && !grid.isWalkableAt(x, y - 1))) {
                        return [x, y];
                    }
                }
                else {
                    if((grid.isWalkableAt(x + 1, y + dy) && !grid.isWalkableAt(x + 1, y)) ||
                       (grid.isWalkableAt(x - 1, y + dy) && !grid.isWalkableAt(x - 1, y))) {
                        return [x, y];
                    }
                }
            }

            // moving diagonally, must make sure one of the vertical/horizontal
            // neighbors is open to allow the path
            if (grid.isWalkableAt(x + dx, y) || grid.isWalkableAt(x, y + dy)) {
                return this._jump(x + dx, y + dy, x, y);
            } else {
                return null;
            }
        }

        /**
         * Find the neighbors for the given node. If the node has a parent,
         * prune the neighbors based on the jump point search algorithm, otherwise
         * return all available neighbors.
         * @return {Array<Array<number>>} The neighbors found.
         */
        _findNeighbors(node) {
            var parent = node.parent,
                x = node.x, y = node.y,
                grid = this.grid,
                px, py, nx, ny, dx, dy,
                neighbors = [], neighborNodes, neighborNode, i, l;

            // directed pruning: can ignore most neighbors, unless forced.
            if (parent) {
                px = parent.x;
                py = parent.y;
                // get the normalized direction of travel
                dx = (x - px) / Math.max(Math.abs(x - px), 1);
                dy = (y - py) / Math.max(Math.abs(y - py), 1);

                // search diagonally
                if (dx !== 0 && dy !== 0) {
                    if (grid.isWalkableAt(x, y + dy)) {
                        neighbors.push([x, y + dy]);
                    }
                    if (grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y]);
                    }
                    if (grid.isWalkableAt(x, y + dy) || grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y + dy]);
                    }
                    if (!grid.isWalkableAt(x - dx, y) && grid.isWalkableAt(x, y + dy)) {
                        neighbors.push([x - dx, y + dy]);
                    }
                    if (!grid.isWalkableAt(x, y - dy) && grid.isWalkableAt(x + dx, y)) {
                        neighbors.push([x + dx, y - dy]);
                    }
                }
                // search horizontally/vertically
                else {
                    if(dx === 0) {
                        if (grid.isWalkableAt(x, y + dy)) {
                            neighbors.push([x, y + dy]);
                            if (!grid.isWalkableAt(x + 1, y)) {
                                neighbors.push([x + 1, y + dy]);
                            }
                            if (!grid.isWalkableAt(x - 1, y)) {
                                neighbors.push([x - 1, y + dy]);
                            }
                        }
                    }
                    else {
                        if (grid.isWalkableAt(x + dx, y)) {
                            neighbors.push([x + dx, y]);
                            if (!grid.isWalkableAt(x, y + 1)) {
                                neighbors.push([x + dx, y + 1]);
                            }
                            if (!grid.isWalkableAt(x, y - 1)) {
                                neighbors.push([x + dx, y - 1]);
                            }
                        }
                    }
                }
            }
            // return all neighbors
            else {
                neighborNodes = grid.getNeighbors(node, DiagonalMovement.IfAtMostOneObstacle);
                for (i = 0, l = neighborNodes.length; i < l; ++i) {
                    neighborNode = neighborNodes[i];
                    neighbors.push([neighborNode.x, neighborNode.y]);
                }
            }

            return neighbors;
        }
    }

    /**
     * @author aniero / https://github.com/aniero
     */
    /**
     * Path finder using the Jump Point Search algorithm
     * @param {Object} opt
     * @param {function} opt.heuristic Heuristic function to estimate the distance
     *     (defaults to manhattan).
     * @param {DiagonalMovement} opt.diagonalMovement Condition under which diagonal
     *      movement will be allowed.
     */
    let JumpPointFinder = function (opt) {
        opt = opt || {};
        if (opt.diagonalMovement === DiagonalMovement.Never) {
            return new JPFNeverMoveDiagonally(opt);
        } else if (opt.diagonalMovement === DiagonalMovement.Always) {
            return new JPFAlwaysMoveDiagonally(opt);
        } else if (opt.diagonalMovement === DiagonalMovement.OnlyWhenNoObstacles) {
            return new JPFMoveDiagonallyIfNoObstacles(opt);
        } else {
            return new JPFMoveDiagonallyIfAtMostOneObstacle(opt);
        }
    };

    let PathFinding = {
        'AStarFinder'               : AStarFinder,
        'BestFirstFinder'           : BestFirstFinder,
        'BreadthFirstFinder'        : BreadthFirstFinder,
        'DijkstraFinder'            : DijkstraFinder,
        'BiAStarFinder'             : BiAStarFinder,
        'BiBestFirstFinder'         : BiBestFirstFinder,
        'BiBreadthFirstFinder'      : BiBreadthFirstFinder,
        'BiDijkstraFinder'          : BiBestFirstFinder$1,
        'IDAStarFinder'             : IDAStarFinder,
        'JumpPointFinder'           : JumpPointFinder
    };

    let nodesInLine = function(node1, node2, node3) {
        if((node1.x-node2.x)*(node2.y-node3.y)==(node2.x-node3.x)*(node1.y-node2.y))
            return true;
        return false;
    };

    let epsilon = 0.001;
    let equal = function(x, y) {
        if(Math.abs(x - y) <= epsilon)
            return true
        return false
    };

    class BlockMap extends Grid {
        constructor(width, height, blocks) {
            super(width, height, blocks);

            this.blocks = blocks;

            // console.debug("blocks", blocks);
            // console.debug("maps");
            // for(let i = 0; i<height; ++i) {
            //     let str = this.nodes[i][0].walkable?"0":"1";
            //     for(let j=1;j<width;++j) {
            //         str += this.nodes[i][j].walkable?",0":",1";
            //     }
            //     console.debug(str+" "+i);
            // }

            this._finder = new PathFinding.AStarFinder({
                allowDiagonal: true,
                dontCrossCorners: true,
                heuristic: Heuristic.manhattan,
            });
        }

        resumeData() {
            this._buildNodes(this.width, this.height, this.blocks);
        }

        _buildNodes(width, height, blocks) {
            var i, j,
                nodes = new Array(height);

            for (i = 0; i < height; ++i) {
                nodes[i] = new Array(width);
                for (j = 0; j < width; ++j) {
                    nodes[i][j] = new Node(j, i);
                }
            }


            if (blocks === undefined) {
                return nodes;
            }
            for (i = 0; i < height; ++i) {
                let line_block = blocks[i];
                let length = line_block.length;
                let index = 0;
                let interval_block = line_block[index];
                for (j = 0; j < width; ++j) {
                    if(index<length)
                        if (j>=interval_block[0]&&j<=interval_block[1])
                            nodes[i][j].walkable = false;
                        else {
                            while(index<length-1 && j>interval_block[1])
                                interval_block = line_block[++index];
                            if(j>interval_block[1])
                                break;
                            if(index<length&&j>=interval_block[0])
                                nodes[i][j].walkable = false;
                        }
                    else
                        break;
                }
            }

            return nodes;
        }

        findPath(startX, startY, endX, endY) {
            if(this.isWalkableAt(startX, startY)) {
                if(this.isWalkableAt(endX, endY)) {
                    let path = [];
                    let times = 0;
                    while(path.length==0) {
                        if(++times>10)
                            break;
                        this.clearNodeExtraInfos();
                        path = this._finder.findPath(startX, startY, endX, endY, this);
                    }
                    return this.smooth(path);
                }
            }
        }

        smooth(path) {//佛洛依德路径平滑算法（FLOYD）
            if(path.length<=2)
                return path;
            //去掉共线点
            let max = path.length - 2;
            for(let i = 0; i < max;) {
                if(nodesInLine(path[i], path[i + 1], path[i + 2])) {
                    path.splice(i + 1, 1);
                    --max;
                }else {
                    ++i;
                }
            }

            //判断是否中途无障碍直达
            max = path.length - 2;
            for(let i = 0; i < max;) {
                if(this.checkLineWalkable(path[i], path[i + 2])) {
                    path.splice(i + 1, 1);
                    --max;
                }else {
                    ++i;
                }
            }
            return path;
        }

        checkLineWalkable(node1, node2) {
            //相信寻路没那么蠢，沿坐标轴方向也能走歪了，如果有问题，还是需要修改的
            if(node1.x == node2.x || node1.y == node2.y)
                return false;
            //斜相邻的，判断是否因为遇到角而产生
            if(Math.abs(node1.x - node2.x) == 1 && Math.abs(node1.y - node2.y) == 1) {
                if(this.isWalkableAt(node1.x, node2.y) && this.isWalkableAt(node2.x, node1.y))
                    return true;
                return false;
            }

            let k = (node1.y - node2.y)/(node1.x - node2.x);
            let b = node1.y + 0.5 - k * (node1.x + 0.5);
            if(Math.abs(k) <= 1) {//k 为0或无穷的情况 实际已经被排除
                let width = Math.abs(node1.x - node2.x);
                if(node1.x>node2.x) {
                    for(let i = 0; i < width; ++i) {
                        let y = (node1.x - i) * k + b;
                        let intY = Math.floor(y);
                        if(equal(y, intY)) {//过格线，禁止跨角的情况下，需要4个格子都无阻挡
                            if(!this.isWalkableAt(node1.x - i, intY) || !this.isWalkableAt(node1.x - i - 1, intY) ||
                                !this.isWalkableAt(node1.x - i, intY-1) || !this.isWalkableAt(node1.x - i - 1, intY-1))
                                return false;
                        }else {
                            if(!this.isWalkableAt(node1.x + i, intY) || !this.isWalkableAt(node1.x + i + 1, intY))
                                return false;
                        }
                    }
                }else {
                    for(let i = 0; i < width; ++i) {
                        let y = (node1.x + i + 1) * k + b;
                        let intY = Math.floor(y);
                        if(equal(y, intY)) {//过格线，禁止跨角的情况下，需要4个格子都无阻挡
                            if(!this.isWalkableAt(node1.x + i, intY) || !this.isWalkableAt(node1.x + i + 1, intY) ||
                                !this.isWalkableAt(node1.x + i, intY-1) || !this.isWalkableAt(node1.x + i + 1, intY-1))
                                return false;
                        }else {
                            if(!this.isWalkableAt(node1.x + i, intY) || !this.isWalkableAt(node1.x + i + 1, intY))
                                return false;
                        }
                    }
                }
            }else {// k>1 k<-1
                let height = Math.abs(node1.y - node2.y);
                if(node1.y>node2.y) {
                    for(let i = 0; i < height; ++i) {
                        let x = (node1.y - i - b)/k;
                        let intX = Math.floor(x);
                        if(equal(x, intX)) {//过格线，禁止跨角的情况下，需要4个格子都无阻挡
                            if(!this.isWalkableAt(intX, node1.y - i) || !this.isWalkableAt(intX, node1.y - i - 1) ||
                                    !this.isWalkableAt(intX - 1, node1.y - i) || !this.isWalkableAt(intX - 1, node1.y - i - 1))
                                return false;
                        }else {
                            if(!this.isWalkableAt(intX, node1.y + i) || !this.isWalkableAt(intX, node1.y + i + 1))
                                return false;
                        }
                    }
                }else {
                    for(let i = 0; i < height; ++i) {
                        let x = (node1.y + i + 1 - b)/k;
                        let intX = Math.floor(x);
                        if(equal(x, intX)) {//过格线，禁止跨角的情况下，需要4个格子都无阻挡
                            if(!this.isWalkableAt(intX, node1.y + i) || !this.isWalkableAt(intX, node1.y + i + 1) ||
                                    !this.isWalkableAt(intX - 1, node1.y + i) || !this.isWalkableAt(intX - 1, node1.y + i + 1))
                                return false;
                        }else {
                            if(!this.isWalkableAt(intX, node1.y + i) || !this.isWalkableAt(intX, node1.y + i + 1))
                                return false;
                        }
                    }
                }
            }
        }

        findNearestValidNode(x, y, fromX, fromY) {
            if(this.isWalkableAt(x, y))
                return this.getNodeAt(x, y);
            for(let i = 1; i<this.height/2; ++i) {
                let arr = [];
                if(this.isWalkableAt(x + i, y + i))
                    arr.push(this.getNodeAt(x + i, y + i));
                if(this.isWalkableAt(x + i, y - i))
                    arr.push(this.getNodeAt(x + i, y - i));
                if(this.isWalkableAt(x - i, y + i))
                    arr.push(this.getNodeAt(x - i, y + i));
                if(this.isWalkableAt(x - i, y - i))
                    arr.push(this.getNodeAt(x - i, y - i));

                for(let d = -i + 1; d < i; ++d) {
                    if(this.isWalkableAt(x + d, y + i))
                        arr.push(this.getNodeAt(x + d, y + i));
                    if(this.isWalkableAt(x + d, y - i))
                        arr.push(this.getNodeAt(x + d, y - i));
                    if(this.isWalkableAt(x + i, y + d))
                        arr.push(this.getNodeAt(x + i, y + d));
                    if(this.isWalkableAt(x - i, y + d))
                        arr.push(this.getNodeAt(x - i, y + d));
                }

                if(arr.length>0) {
                    if(fromX&&fromY) {
                        function sortByDistance(a,b)
                        {
                            return Heuristic.euclidean(a.x - fromX, a.y - fromY) - Heuristic.euclidean(b.x - fromX,b.y - fromY);
                        }
                        arr.sort(sortByDistance);
                    }
                    return arr[0];
                }

            }
        }

        getValidNodes(x, y, distance) {
            let arr = [];
            for(let i = 0; i < this.height; ++i) {
                for(let j = 0;j < this.width; ++j) {
                    let node = this.nodes[i][j];
                    if(node.walkable) {
                        if(distance && distance > 0){
                            if(Heuristic.manhattan(x - j, y - i)>=distance)
                                arr.push(node);
                        }else if(distance && distance < 0) {
                            if(Heuristic.manhattan(x - j, y - i)<=-distance)
                                arr.push(node);
                        }else
                            arr.push(node);
                    }
                }
            }
            return arr;
        }


        addBlocks(blocks) {
            for(const block of blocks) {
                let node = this.getNodeAt(block[0],block[1]);
                if(node)
                    node.walkable = false;
            }
        }

        clearNodeExtraInfos() {
            for(let i = 0; i < this.height; ++i) {
                for(let j = 0;j < this.width; ++j) {
                    this.nodes[i][j].clearExtraInfo();
                }
            }
        }

    }

    var MapInfo = {"width": 107, "height": 87, "block": [[[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 12], [25, 48], [71, 106]], [[0, 10], [25, 48], [90, 106]], [[0, 10], [90, 106]], [[0, 10], [90, 106]], [[0, 9], [90, 106]], [[0, 9], [90, 106]], [[0, 9], [91, 106]], [[0, 9], [91, 106]], [[0, 9], [91, 106]], [[0, 8], [93, 106]], [[0, 8], [93, 106]], [[0, 8], [93, 106]], [[0, 8], [96, 106]], [[0, 7], [96, 106]], [[0, 7], [96, 106]], [[0, 13], [100, 106]], [[0, 13], [100, 106]], [[0, 14], [100, 106]], [[0, 14], [101, 106]], [[0, 14], [101, 106]], [[0, 18], [102, 106]], [[0, 18], [102, 106]], [[0, 18], [102, 106]], [[0, 18], [102, 106]], [[0, 12], [102, 106]], [[0, 11], [102, 106]], [[0, 10], [81, 106]], [[0, 10], [81, 106]], [[0, 10], [81, 106]], [[0, 10], [81, 106]], [[0, 10], [81, 106]], [[0, 3], [81, 106]], [[0, 3], [105, 106]], [[0, 2], [105, 106]], [[0, 2], [105, 106]], [[0, 2], [105, 106]], [[0, 1], [106, 106]], [[0, 1], [106, 106]], [[0, 1], [106, 106]], [[0, 1], [106, 106]], [[0, 1], [106, 106]], [[0, 1], [106, 106]], [[0, 28], [91, 106]], [[0, 28], [91, 106]], [[0, 28], [91, 106]], [[0, 28], [91, 106]], [[0, 28], [91, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]], [[0, 106]]], "decorate": {"pingfeng": 57}};

    let kShowCD = 2000;

    class CoffeeSceneLogic extends Logic {

        constructor() {
            super();
            this._roleMgr = new CoffeeRoleMgr();
            this._currentStaffTime = 0;
            this._staffRoleMgr = new StaffRoleMgr();
        }

        start(context) {
            this._context = context;
            // this._attractCD = 10000;
            // this._calcAttractCD();
            this._showCD = kShowCD;
            this._immediateShow = false;
            context._roleMgr = this._roleMgr;
            this._context.newMainLine();
        }

        step(dt) {
            let roles = this._roleMgr.getRoles();
            for(const role of roles) {
                role.update(dt);
            }
            // this._attractCD -= dt;
            // if(this._attractCD<=0){
            //     // this._attractCD = 10000+Utils.randomExt(5000);
            //     this._calcAttractCD();
            //     this._attractCat();

            // }
            this._showCD -= dt;
            if(this._showCD<=0) {
                this._showCD = kShowCD;
                this._showCat();
            }
            if(this._immediateShow) {
                this._immediateShow = false;
                this._showCat();
            }
            this.entrustCatStep(dt);
            this.staffStep(dt);
        }

        // _calcAttractCD() {
            // let array = Const.getArray("CatVisitGaps");
            // this._attractCD = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
        // }

        _attractCat(args) {
            let catVisitWeightedArray = BuildingSystem.getCatWeightedArray(Constants.CoffeeSceneId);
            let catVisitWeightedSum = BuildingSystem.getBuildingDataListInfo(Constants.CoffeeSceneId).weighted;

            let ret = Utils.calcRandomWeight(catVisitWeightedArray, catVisitWeightedSum);
            let catLimit = Number(Const.get("CatVisitLimit"));
            if(ret != null) {
               let cout =  this._roleMgr.getRoleCount();
               if (cout < catLimit) {
                    //此处触发刷
                    let p = ScenePathMgr.getInstance().getStartPathPoint();
                    if(p != null) {
                        let configId = ret.n;
                        let catData = CatSystem.addCatData(configId, Constants.CoffeeSceneId, { skillType: args });
                    }
                }
            }
        }

        _showCat() {
            if (this._roleMgr.getStackRoleCount() > 0) {
                let catAppearLimitB = Number(Const.get("CatAppearLimitB"));
                let catAppearLimitK = Number(Const.get("CatAppearLimitK"));
                let spaceCount = BuildingSystem.getTotalSpaceCountInScene(Constants.CoffeeSceneId);;
                if(this._roleMgr.getRoleListCount() < Math.floor(spaceCount * catAppearLimitK) + catAppearLimitB) {
                    let role = this._roleMgr.showRole();
                    role.start(this._context);
                }
            }
        }

        handlerInput(input) {
            let op = input.op;
            let args = input.args;
            let handler = input.handler;
            switch(op) {
                case "ADD_CAT":
                    this._addCat(args);
                    break;
                case "REMOVE_CAT":
                    this._removeCat(args);
                    break;
                case "FORCE_ATTRACT_CAT":
                    this._forceAttractCat(args);
                    break;
                case "RESTORE_CAT":
                    this._restoreCat(args);
                    break;
                case "CAT_EAT":
                    this._catEat(args);
                    break;
                case "GUIDE_ATTRACT_CAT":
                    this._guideAttractCat();
                    break;
                case "ENTRUST_ATTRACT_CAT":
                    this._entreustAttractCat(args);
                    break;
                case "ADD_ENTRUST_CAT":
                    this._addEntrustCat(args);
                    break;

                case "SET_TIME":
                    this._setTime(args);
                    break;
                case "UPDATE_POS":
                    this._updatePos(args);
                    break;
                case "PUT_STAFF":
                    this._addStaff(args);
                    break;
                case "REMOVE_STAFF":
                    this._removeStaff(args);
                    break;
                case "STOP_STAFF":
                    this._stopStaff(args);
                    break;
            }
        }


        _addCat(args) {
            let catId = args;
            if(!this._roleMgr.getRoleById(catId)) {
                let catData = CatSystem.getCatData(catId);
                let role = this._roleMgr.addRole(catData);
            }
        }

        _restoreCat(args) {

        }

        _removeCat(args) {
            let catId = args;
            let role = this._roleMgr.getRoleById(catId);
            if(role) {
                if(role.remove()) {
                    this._roleMgr.removeRole(catId);
                }
            }
        }

        _forceAttractCat(args) {
            this._attractCat(args);
            this._immediateShow = true;
        }

        _catEat(args) {
            let catId = args;
            let role = this._roleMgr.getRoleById(catId);
            role.eat();
        }

        _guideAttractCat() {
            //新手引导加猫
            let p = ScenePathMgr.getInstance().getBaseNode(7);
            if (p != null) {
                let configId = "Cat1";
                let catData = CatSystem.addCatData(configId, Constants.CoffeeSceneId, {});
            }
            this._immediateShow = true;
        }
        //订单猫
        _entreustAttractCat(args) {
            let p = ScenePathMgr.getInstance().getStartPathPoint();
            if (p != null) {
                EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.CoffeeSceneId, "ADD_ENTRUST_CAT", args]);
            }
            this._immediateShow = true;
        }
        //添加订单猫
        _addEntrustCat(args) {
            if(this._entrustRoleMgr==null){
                this._entrustRoleMgr=new EntrustRoleMgr();
            }
            if(args && args.getCatId) {
                let role = this._entrustRoleMgr.addRole(args);
                role.start(this._context);
            }
        }

        entrustCatStep(dt) {
            if(this._entrustRoleMgr==null){
                this._entrustRoleMgr=new EntrustRoleMgr();
            }
            let roles = this._entrustRoleMgr.getRoles();
            roles.forEach(role => {
                role.update(dt);
            });
        }


        staffStep(dt) {
            this._currentStaffTime += dt;
            if(this._stopUpdate)
                return;
            let roles = this._staffRoleMgr.getRoles();
            for(const role of roles) {
                role.update(dt, this._currentStaffTime);
            }
        }

        _updatePos(args) {
            let staffId = args.staffId;
            let role = this._staffRoleMgr.getRoleById(staffId);
            if (role)
                role.setPos(args.x, args.y);
        }

        _addStaff(args) {
            let id = args;
            if (!this._staffRoleMgr.getRoleById(id)) {
                let staffData = StaffSystem.getStaffData(id);
                let role = this._staffRoleMgr.addRole(staffData);
                if (role) {
                    role.start(this._context, this._currentStaffTime);
                }
            }
        }

        _removeStaff(args) {
            let staffId = args;
            let role = this._staffRoleMgr.getRoleById(staffId);
            if (role) {
                if (role.remove()) {
                    this._staffRoleMgr.removeRole(staffId);
                }
            }
        }

        _stopStaff(args) {
            let staffId = args;
            let role = this._staffRoleMgr.getRoleById(staffId);
            if (role)
                role.stop();
        }

        _setTime(args) {
            this._currentStaffTime = args;
        }

    }

    let kLoopInterval = 50;
    class CoffeeSession {
        static getOrCreateSession() {
            if(GameContext.coffeeSession){
                return GameContext.coffeeSession;
            }
            GameContext.coffeeSession = new CoffeeSession();
            GameContext.coffeeSession.build();
            GameContext.coffeeSession.prepare();
            return GameContext.coffeeSession;
        }

        static getSession() {
            if(GameContext.coffeeSession){
                return GameContext.coffeeSession;
            }
        }

        static runSession() {
            let session = CoffeeSession.getOrCreateSession();
            session.startLoop();
        }

        static sendOpCommand(op, args, handler) {
            let session = CoffeeSession.getSession();
            if(session) {
                session._sendOpCommand(op, args, handler);
            }
        }

        static simulate(time) {
            let session = CoffeeSession.getSession();
            if(session)
                session._simulate(time);
        }

        static clear() {
            if(GameContext.coffeeSession) {
                GameContext.coffeeSession.destroy();
                GameContext.coffeeSession = null;
            }
        }

        constructor() {
            this._director = new SceneDirector();
            this._simulator = new SceneSimulator();
            this._logic = new CoffeeSceneLogic();
            this._controller = new Controller();
            this._recorder = new Recorder();
            this._interpreter = new Interpreter();
        }

        build() {
            this._director.setSimulator(this._simulator);
            this._director.setInterpreter(this._interpreter);
            this._interpreter.setRecordsProvider(this._recorder);
            this._simulator.setRecorder(this._recorder);
            this._simulator.setLogic(this._logic);
            this._simulator.setController(this._controller);
        }

        prepare() {
            this._director.start();
            EventMgr.getInstance().offAllCaller(this);
            EventMgr.getInstance().registEvent("FORCE_ATTRACT_COFFEE_CAT", this, this.onForceAttractCat);
            EventMgr.getInstance().registEvent("GUIDE_ATTRACT_COFFEE_CAT", this, this.onGuideAttractCat);
            EventMgr.getInstance().registEvent("FORCE_ATTRACT_ENTRUST_CAT", this, this.onEntrustAttractCat);
        }

        onForceAttractCat(value,skillType) {
            if (value != "special") {
                return;
            }
            this._sendOpCommand("FORCE_ATTRACT_CAT", skillType);
        }

        startLoop() {
            Laya.timer.loop(kLoopInterval, this, this._update, [], true, true);
        }

        _update() {
            this._director.update(kLoopInterval);
        }

        //模拟多少时间
        _simulate(time) {
            this._director.setInterpreterMode(2);
            this._director.update(time);
            this._director.flush();
            this._director.setInterpreterMode(1);
        }

        _sendOpCommand(op, args, handler) {
            this._controller.sendOpCommand(op, args, handler);
        }


        destroy() {
            this.destroyed = true;
            EventMgr.getInstance().offAllCaller(this);
            Laya.timer.clearAll(this);
            // EventMgr.getInstance().removeEvent("FORCE_ATTRACT_COFFEE_CAT", this, this.onForceAttractCat);
            // EventMgr.getInstance().removeEvent("GUIDE_ATTRACT_COFFEE_CAT", this, this.onGuideAttractCat);
            // EventMgr.getInstance().removeEvent("FORCE_ATTRACT_ENTRUST_CAT", this, this.onEntrustAttractCat);
        }

        onGuideAttractCat(value){
            if (value != "special") {
                return;
            }
            this._sendOpCommand("GUIDE_ATTRACT_CAT");
        }

        onEntrustAttractCat(entrustData) {
            this._sendOpCommand("ENTRUST_ATTRACT_CAT", entrustData);
        }
    }

    let kShowCD$1 = 2000;

    class BeachSceneLogic extends Logic {

        constructor() {
            super();
            this._roleMgr = new BeachRoleMgr();
            this._currentTime = 0;
            this._exhaustTime = null;
        }

        start(context) {
            this._context = context;
            this._attractCD = 10000;
            this._immediateShow = false;
            context._roleMgr = this._roleMgr;
        }

        step(dt) {
            this._currentTime += dt;
            let roles = this._roleMgr.getRoles();
            for(const role of roles) {
                role.update(dt);
            }
            this._attractCat(dt);
        }

        _attractCat(dt) {
            if(this._exhaustTime==null || this._exhaustTime< this._currentTime) {
                // console.debug("wont _attractCat");
                return;
            }
            let arr = BuildingSystem.getAvailableBuilding(Constants.BeachSceneId);
            if(arr) {
                for(const buildingData of arr) {
                    let id = buildingData.getId();
                    if(!buildingData.attractCD) {
                         let array = Const.getArray("TravelTriggerGaps");
                         buildingData.attractCD =  Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
                    }
                    buildingData.attractCD -= dt;
                    if(buildingData.attractCD<=0) {

                        let ret = Utils.calcRandomWeight(buildingData.getCatWeightedArray(), buildingData.getCatWeightedSum());

                        if(ret != null) {
                            let configId = ret.n;
                            let spaces = buildingData.getSpaces();
                            let space = spaces[Utils.randomExt(spaces.length)];
                            // console.debug("_attractCat 4", configId, id, space, spaces);
                            if(space!=null) {
                                let array = Const.getArray("TravelKeepTime");
                                let time =  Utils.randomExt2(Number(array[0]), Number(array[1]))*1000;
                                let catData = CatSystem.addCatData(configId, Constants.BeachSceneId,{"BuildingId":id, "SpaceId":space, "TravelTime":time, "StartTime":this._currentTime});
                                buildingData.occupy(space, catData.getCatId());
                            }
                        }
                        buildingData.attractCD = null;
                    }
                }
            }
        }

        handlerInput(input) {
            let op = input.op;
            let args = input.args;
            let handler = input.handler;
            switch(op) {
                case "ADD_CAT":
                    this._addCat(args);
                    break;
                case "REMOVE_CAT":
                    this._removeCat(args);
                    break;
                case "SET_TIME":
                    this._setTime(args);
                    break;
                case "REFRESH_FOOD":
                    this._refreshFood(args);
                    break;
                case "GUIDE_ATTRACT_CAT":
                    this._guideAttractCat();
                    break;
            }
        }


        _addCat(args) {
            // console.debug("_addCat", args);
            let catId = args;
            if(!this._roleMgr.getRoleById(catId)) {
                let catData = CatSystem.getCatData(catId);
                let role = this._roleMgr.addRole(catData);
                role.start(this._context, this._currentTime);
            }
        }

        _removeCat(args) {
            let catId = args;
            let role = this._roleMgr.getRoleById(catId);
            if(role) {
                if(role.remove()) {
                    this._roleMgr.removeRole(catId);
                }
            }
        }

        _setTime(args) {
            this._currentTime = args;
        }

        _refreshFood(args) {
            this._exhaustTime = args;
        }

        _guideAttractCat() {
            let arr = BuildingSystem.getAvailableBuilding(Constants.BeachSceneId);
            if (arr) {
                for (const buildingData of arr) {
                    let id = buildingData.getId();
                    let ret = Utils.calcRandomWeight(buildingData.getCatWeightedArray(), buildingData.getCatWeightedSum());
                    if (ret != null) {
                        let configId = ret.n;
                        let spaces = buildingData.getSpaces();
                        let space = spaces[Utils.randomExt(spaces.length)];
                        if (space != null) {
                            let time = 4000;
                            let catData = CatSystem.addCatData(configId, Constants.BeachSceneId, { "BuildingId": id, "SpaceId": space, "TravelTime": time, "StartTime": this._currentTime });
                            buildingData.occupy(space, catData.getCatId());
                        }
                    }
                    buildingData.attractCD = null;
                }
            }
        }

    }

    let kLoopInterval$1 = 50;
    class BeachSession {
        static getOrCreateSession() {
            if(GameContext.beachSession){
                return GameContext.beachSession;
            }
            GameContext.beachSession = new BeachSession();
            GameContext.beachSession.build();
            GameContext.beachSession.prepare();
            return GameContext.beachSession;
        }

        static getSession() {
            if(GameContext.beachSession){
                return GameContext.beachSession;
            }
        }

        static runSession() {
            let session = BeachSession.getOrCreateSession();
            session.startLoop();
        }

        static sendOpCommand(op, args, handler) {
            let session = BeachSession.getSession();
            if(session)
                session._sendOpCommand(op, args, handler);
        }

        static simulate(time) {
            let session = BeachSession.getSession();
            if(session)
                session._simulate(time);
        }

        static clear() {
            if(GameContext.beachSession) {
                GameContext.beachSession.destroy();
                GameContext.beachSession = null;
            }
        }

        constructor() {
            this._director = new SceneDirector();
            this._simulator = new SceneSimulator();
            this._logic = new BeachSceneLogic();
            this._controller = new Controller();
            this._recorder = new Recorder();
            this._interpreter = new Interpreter();
        }

        build() {
            this._director.setSimulator(this._simulator);
            this._director.setInterpreter(this._interpreter);
            this._interpreter.setRecordsProvider(this._recorder);
            this._simulator.setRecorder(this._recorder);
            this._simulator.setLogic(this._logic);
            this._simulator.setController(this._controller);
        }

        prepare() {
            this._director.start();
            EventMgr.getInstance().offAllCaller(this);
            EventMgr.getInstance().registEvent("FORCE_ATTRACT_BEACH_CAT", this, this.onForceAttractCat);
        }

        onForceAttractCat() {
            this._sendOpCommand("GUIDE_ATTRACT_CAT");
        }

        startLoop() {
            Laya.timer.loop(kLoopInterval$1, this, this._update, [], true, true);
        }

        _update() {
            this._director.update(kLoopInterval$1);
        }

        //模拟多少时间
        _simulate(time) {
            this._director.setInterpreterMode(2);
            this._director.update(time);
            this._director.flush();
            this._director.setInterpreterMode(1);
        }

        _sendOpCommand(op, args, handler) {
            this._controller.sendOpCommand(op, args, handler);
        }


        destroy() {
            this.destroyed = true;
            EventMgr.getInstance().offAllCaller(this);
            Laya.timer.clearAll(this);
            // EventMgr.getInstance().removeEvent("FORCE_ATTRACT_BEACH_CAT", this);
        }
    }

    class FlyItem extends Laya.Sprite {

        constructor() {
            super();
            this.name = "flyItem";
            this.width = this.height = 100;
            this.pivot(50, 50);
            this._image = new Laya.Image();
            this._image.scale(0.8, 0.8);
            this._image.anchorX = this._image.anchorY = 0.5;
            this._image.pos(50, 50);
            this.addChild(this._image);
        }

        init(type) {
            switch (type) {
                case Constants.RewardType.FISH:
                    this._image.skin = "topui/yugan1.png";
                    break;
                case Constants.RewardType.STAR:
                    this._image.skin = "topui/lingdang1.png";
                    break;
                case Constants.RewardType.SHELL:
                    this._image.skin = "topui/beike.png";
                    break;
                case Constants.RewardType.MEDAL:
                    this._image.skin = "topui/huizhang1.png";
                    break;
                case Constants.RewardType.BOOK:
                    this._image.skin = "topui/miji1.png";
                    break;
            }
            this.alpha = 1;
        }

        onEnable() {
            EventMgr.getInstance().registEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
        }

        get factor() {
            return 0;
        }

        set factor(value) {
            this.x = (1 - value) * (1 - value) * this.startX + 2 * value * (1 - value) * this.midX + value * value * this.endX;
            this.y = (1 - value) * (1 - value) * this.startY + 2 * value * (1 - value) * this.midY + value * value * this.endY;
        }

        get speed() {
            return 0;
        }

        set speed(value) {
            this.x = this.startX + this.speedX * value + 0.5 * this.accelerationX * value * value;
            this.y = this.startY + this.speedY * value + 0.5 * this.accelerationY * value * value;
        }

        startTween(startX, startY, endX, endY, handler = null, time = 1000, autoDestroy = true, delay = 0, midX = null, minY = null) {
            this.x = this.startX = startX ? startX : this.x;
            this.y = this.startY = startY ? startY : this.y;
            this.endX = endX;
            this.endY = endY;
            this.midX = midX ? midX : (startX + endX) / 2 + Utils.randomExt2(-100, 100);
            this.midY = minY ? minY : (startY + endY) / 2 + Utils.randomExt2(-100, 100);
            this.tweenHandler = handler;
            let h = Laya.Handler.create(this, function () {
                if (autoDestroy) {
                    // this.destroy();
                    this.removeSelf();
                    Laya.Pool.recover("flyItem", this);
                }
            });
            if (this.midX == (this.endX + this.x) / 2 && this.minY == (this.endY + this.y) / 2) {
                Laya.Tween.to(this, { x: this.endX, y: this.endY }, time - 90, null, null, delay);
            } else {
                Laya.Tween.to(this, { factor: 1 }, time - 90, null, null, delay);
            }
            Laya.Tween.to(this, { scaleX: 1.5, scaleY: 1.5 }, 45, null, null, delay + time - 90);
            Laya.Tween.to(this, { scaleX: 1, scaleY: 1 }, 45, null, h, delay + time - 45);
        }

        startSpeedTween(startX, startY, endX, endY, handler = null, time = 1000, autoDestroy = true, delay = 0, speedX = null, speedY = null) {
            this.x = this.startX = startX ? startX : this.x;
            this.y = this.startY = startY ? startY : this.y;
            this.endX = endX;
            this.endY = endY;
            this.speedX = speedX ? speedX : 100;
            this.speedY = speedY ? speedY : 100;
            this.accelerationX = ((this.endX - this.startX) - this.speedX) * 2;
            this.accelerationY = ((this.endY - this.startY) - this.speedY) * 2;
            this.tweenHandler = handler;
            let h = Laya.Handler.create(this, function () {
                if (autoDestroy) {
                    // this.destroy();
                    this.removeSelf();
                    Laya.Pool.recover("flyItem", this);
                }
            });
            Laya.Tween.to(this, { speed: 1 }, time, null, h, delay);
        }

        onDisable() {
            if (this.tweenHandler) {
                this.tweenHandler.run();
                this.tweenHandler = null;
            }
            Laya.Tween.clearAll(this);
        }

        cameraSwitch(type) {
            this.visible = !type;
        }
    }

    class ItemFlyMgr {

        constructor() {
            if (!Laya.Browser.onMiniGame) {
                Laya.stage.on(Laya.Event.FOCUS, this, this.clear);
            } else {
                let obj = this;
                wx.onShow(function () {
                    Laya.timer.frameOnce(1, obj, obj.clear);
                });
            }
            this._initUI();
            this._handlerMap = new Map();
        }

        static getInstance() {
            return ItemFlyMgr.instance ? ItemFlyMgr.instance : ItemFlyMgr.instance = new ItemFlyMgr();
        }

        _initUI() {
            this._itemBox = new Laya.Box();
            this._itemBox.width = Laya.stage.width;
            this._itemBox.height = Laya.stage.height;
            this._itemBox.mouseEnabled = false;
            this._itemBox.zOrder = 100000;
            Laya.stage.addChild(this._itemBox);
        }
        //修改完毕后删掉
        fly(type, addNum, addType, startx, starty, handler = null, time = 400) {
            if (type == Constants.RewardType.FISH) {
                this.flyReward(type, addNum, Constants.FlyType.TYPE5, startx, starty, handler, 1500);
                return;
            }
            let dealHandler = Laya.Handler.create(this, function () {
                if (addType == Constants.FlyType.ST || addType == Constants.FlyType.STNORM || addType == Constants.FlyType.STENHANCE) {
                    EventMgr.getInstance().postEvent("SHOW_ADD_ST", [addNum, type, handler]);
                }
                else {
                    EventMgr.getInstance().postEvent("SHOW_ADD", [addNum, type, handler]);
                }
            });
            this._handlerMap.set(dealHandler._id, dealHandler);
            if (type == Constants.RewardType.STAR && (addType == Constants.FlyType.LW || addType == Constants.FlyType.ST)) {
                var locX = new Array(106, 41, -178);
                var locY = new Array(-20, 48, -5);
                var waitTime = new Array(80, 160, 130);
                for (var i = 0; i < 3; i++) {
                    let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);
                    let endx = null;
                    let endy = null;
                    item.skin = "topui/xingxing1.png";
                    endx = Constants.starImagePoint.x;
                    endy = Constants.starImagePoint.y;
                    item.scaleX = item.scaleY = 0.8;
                    item.x = startx + locX[i];
                    item.y = starty + locY[i];
                    item.wait = waitTime[i];
                    Laya.timer.once(500, this, function () {
                        this._itemBox.addChild(item);
                        Laya.timer.once(item.wait, this, function () {
                            Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                                item.removeSelf();
                                Laya.Pool.recover("itemImage", item);
                            }));
                        });

                    });

                }
                Laya.timer.once(900, this, function () {
                    this._handlerMap.delete(dealHandler._id);
                    dealHandler.run();
                });
            }
            else if (type == Constants.RewardType.FISH && (addType == Constants.FlyType.ZCM || addType == Constants.FlyType.LW || addType == Constants.FlyType.QQ || addType == Constants.FlyType.ST)) {
                var locX = new Array(120, 155, 110, 63, 24, -57, -130, -140, -151, -153, -133);
                var locY = new Array(-50, -5, 55, 76, 128, 105, 15, 50, -42, -15, -80);
                var waitTime = new Array(80, 80, 130, 130, 160, 160, 130, 130, 80, 80, 80);
                for (var i = 0; i < 11; i++) {
                    let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);
                    let endx = null;
                    let endy = null;
                    item.skin = "topui/yugan1.png";
                    endx = Constants.fishImagePoint.x;
                    endy = Constants.fishImagePoint.y;
                    item.scaleX = item.scaleY = 0.8;
                    item.x = startx + locX[i];
                    item.y = starty + locY[i];
                    item.wait = waitTime[i];
                    Laya.timer.once(500, this, function () {
                        this._itemBox.addChild(item);
                        Laya.timer.once(item.wait, this, function () {
                            Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                                item.removeSelf();
                                Laya.Pool.recover("itemImage", item);
                            }));
                        });

                    });
                }
                Laya.timer.once(900, this, function () {
                    this._handlerMap.delete(dealHandler._id);
                    dealHandler.run();
                });
            }
            else if (type == Constants.RewardType.SHELL) {
                var locX = new Array(106, 41, -178);
                var locY = new Array(-20, 48, -5);
                var waitTime = new Array(80, 160, 130);
                for (var i = 0; i < 3; i++) {
                    let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);
                    let endx = null;
                    let endy = null;
                    item.skin = "topui/beike.png";
                    endx = Constants.shellImagePoint.x;
                    endy = Constants.shellImagePoint.y;
                    item.scaleX = item.scaleY = 2;
                    item.x = startx + locX[i];
                    item.y = starty + locY[i];
                    item.wait = waitTime[i];
                    Laya.timer.once(500, this, function () {
                        this._itemBox.addChild(item);
                        Laya.timer.once(item.wait, this, function () {
                            Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                                item.removeSelf();
                                Laya.Pool.recover("itemImage", item);
                            }));
                        });

                    });

                }
                Laya.timer.once(900, this, function () {
                    this._handlerMap.delete(dealHandler._id);
                    dealHandler.run();
                });
            } else if (type == Constants.RewardType.DIAMOND) {
                var locX = new Array(106, 41, -178);
                var locY = new Array(-20, 48, -5);
                var waitTime = new Array(80, 160, 130);
                for (var i = 0; i < 3; i++) {
                    let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);
                    let endx = null;
                    let endy = null;
                    item.skin = "topui/diamond.png";
                    endx = Constants.zuanshiImagePoint.x;
                    endy = Constants.zuanshiImagePoint.y;
                    item.scaleX = item.scaleY = 0.5;
                    item.x = startx + locX[i];
                    item.y = starty + locY[i];
                    item.wait = waitTime[i];
                    Laya.timer.once(500, this, function () {
                        this._itemBox.addChild(item);
                        Laya.timer.once(item.wait, this, function () {
                            Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                                item.removeSelf();
                                Laya.Pool.recover("itemImage", item);
                            }));
                        });
                    });
                }
                Laya.timer.once(900, this, function () {
                    this._handlerMap.delete(dealHandler._id);
                    dealHandler.run();
                });
            }
            else {
                let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);
                let endx = null;
                let endy = null;
                if (type == Constants.RewardType.STAR) {
                    item.skin = "topui/xingxing1.png";
                    endx = Constants.starImagePoint.x;
                    endy = Constants.starImagePoint.y;
                    if (addType == Constants.FlyType.ENHANCE || addType == Constants.FlyType.STENHANCE) {
                        var enhanceAnim = new Spine();
                        this._itemBox.addChild(enhanceAnim);
                        enhanceAnim.pos(startx, starty);
                        enhanceAnim.scale(0.8, 0.8);
                        enhanceAnim.loadAnim("anim1/duoyugan.sk");
                        enhanceAnim.playAnim("lingdang2", false, true);
                    }
                } else {
                    item.skin = "topui/yugan1.png";
                    endx = Constants.fishImagePoint.x;
                    endy = Constants.fishImagePoint.y;
                    if (addType == Constants.FlyType.ENHANCE || addType == Constants.FlyType.STENHANCE) {
                        var enhanceAnim = new Spine();
                        this._itemBox.addChild(enhanceAnim);
                        enhanceAnim.pos(startx, starty);
                        enhanceAnim.scale(0.8, 0.8);
                        enhanceAnim.loadAnim("anim1/duoyugan.sk");
                        enhanceAnim.playAnim("yugan2", false, true);
                    }
                }
                item.scaleX = item.scaleY = 0.8;
                if (addType == Constants.FlyType.ENHANCE || addType == Constants.FlyType.STENHANCE) {
                    item.x = startx - 50;
                    item.y = starty - 250;
                    Laya.timer.once(1000, this, function () {
                        enhanceAnim.destroy();
                        this._itemBox.addChild(item);
                        Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                            item.removeSelf();
                            Laya.Pool.recover("itemImage", item);
                            this._handlerMap.delete(dealHandler._id);
                            dealHandler.run();
                        }));
                    });
                } else {
                    item.x = startx;
                    item.y = starty;
                    this._itemBox.addChild(item);
                    Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                        item.removeSelf();
                        Laya.Pool.recover("itemImage", item);
                        this._handlerMap.delete(dealHandler._id);
                        dealHandler.run();
                    }));
                }
            }

        }
        flyjuanzhou(startx, starty, type, handler = null, time = 400) {
            //let item = Laya.Pool.getItemByClass("itemImage", Laya.Image);

            let item = new Laya.Button();
            if (handler) {
                this._handlerMap.set(handler._id, handler);
            }

            if (type == 'StaffUnlock') {
                item.skin = "signui/yuangong.png";
                item.stateNum = 2;
                item.scaleX = item.scaleY = 2;
            }
            if (type == 'SkillAmount') {
                item.skin = "signui/skilljuan.png";
                item.stateNum = 1;
                item.scaleX = item.scaleY = 1;
            }


            let endx = null;
            let endy = null;
            //item.skin = "signui/yuangong.png";
            endx = Constants.yuangongImagePoint.x;
            endy = Constants.yuangongImagePoint.y;


            item.x = startx;
            item.y = starty;
            this._itemBox.addChild(item);
            Laya.Tween.to(item, { x: endx, y: endy }, time, null, Laya.Handler.create(this, function () {
                this._handlerMap.delete(handler._id);
                if (handler) {
                    handler.run();
                }
                item.removeSelf();
            }));
        }

        clear(runHandler = true) {
            Laya.timer.clearAll(this);
            while (this._itemBox._children.length > 0) {
                let item = this._itemBox._children[0];
                Laya.Tween.clearAll(item);
                item.removeSelf();
                if (item.name == "flyItem") {
                    Laya.Pool.recover("flyItem", item);
                } else {
                    Laya.Pool.recover("itemImage", item);
                }
            }
            if (runHandler) {
                this._handlerMap.forEach((value, key) => {
                    if (value != null) {
                        value.run();
                    }
                });
            }
            this._handlerMap.clear();
        }

        flyReward(rewardType, rewardNum, flyType, globalStartX, globalStartY, handler, time = 1500) {
            if (rewardType != Constants.RewardType.FISH && rewardType != Constants.RewardType.BOOK && rewardType != Constants.RewardType.MEDAL) {
                //修改完成后删掉
                this.fly(rewardType, rewardNum, Constants.FlyType.LW, globalStartX, globalStartY, handler, 400);
                return;
            }
            if (rewardType == Constants.RewardType.BOOK && rewardType == Constants.RewardType.MEDAL) {
                console.log(rewardType, rewardNum, flyType);
            }
            let dealHandler = Laya.Handler.create(this, function () {
                EventMgr.getInstance().postEvent("SHOW_ADD_ST", [rewardNum, rewardType, handler]);
                EventMgr.getInstance().postEvent("SHOW_ADD", [rewardNum, rewardType, handler]);
            });
            let endX = 0;
            let endY = 0;
            switch (rewardType) {
                case Constants.RewardType.FISH: {
                    endX = Constants.fishImagePoint.x + 24;
                    endY = Constants.fishImagePoint.y + 23;
                    break;
                }
                case Constants.RewardType.STAR: {
                    endX = Constants.starImagePoint.x;
                    endY = Constants.starImagePoint.y;
                    break;
                }
                case Constants.RewardType.SHELL: {
                    endX = Constants.shellImagePoint.x;
                    endY = Constants.shellImagePoint.y;
                    break;
                }
                case Constants.RewardType.DIAMOND: {
                    endX = Constants.luckyButtonPoint.x;
                    endY = Constants.luckyButtonPoint.y;
                    break;
                }
                default: {
                    endX = globalStartX;
                    endY = globalStartY - 200;
                }
            }
            switch (flyType) {
                case Constants.FlyType.TYPE1: {
                    time = 700;
                    let time1 = 200;
                    let num = 10;
                    let delay = 80;
                    for (let index = 0; index < num; index++) {
                        let item = this.createRewardItem(rewardType, globalStartX, globalStartY);
                        item.visible = false;
                        Laya.timer.once(index * delay, this, function () {
                            item.visible = true;
                        });
                        item.startTween(globalStartX, globalStartY, globalStartX + (index % 2 == 0 ? 200 : -200) + Utils.randomExt2(-50, 50), globalStartY, null, time1, false, index * delay, null, globalStartY - 100);
                        Laya.timer.once(time1 + num * delay, this, function () {
                            if (index == num - 1) {
                                item.startTween(null, null, endX, endY, dealHandler, time - time1, true, index * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                            } else {
                                item.startTween(null, null, endX, endY, null, time - time1, true, index * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                            }
                        });
                    }
                    break;
                }
                case Constants.FlyType.TYPE2: {
                    time = 700;
                    let time1 = 100;
                    let num = 30;
                    let delay = 30;
                    for (let index = 0; index < num; index++) {
                        let item = this.createRewardItem(rewardType, globalStartX, globalStartY);
                        let right = Utils.randomSign();
                        item.visible = false;
                        Laya.timer.once(index * delay, this, function () {
                            item.visible = true;
                        });
                        item.startTween(globalStartX, globalStartY, globalStartX + right * Utils.randomExt2(0, 150) * Math.pow((num - index) / num, 1 / 3), globalStartY - 80 * Math.pow(index / num, 3), null, time1, false, index * delay, globalStartX + right * Utils.randomExt2(0, 50), globalStartY - 300);
                        Laya.timer.once(time1 + num * delay, this, function () {
                            if (index == 0) {
                                item.startTween(null, null, endX, endY, dealHandler, time - time1, true, (num - index) * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                            } else {
                                item.startTween(null, null, endX, endY, null, time - time1, true, (num - index) * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                            }
                        });
                    }
                    break;
                }
                case Constants.FlyType.TYPE3: {
                    globalStartX = Laya.stage.width / 2;
                    globalStartY = Laya.stage.height / 2;
                    let time1 = 200;
                    let num = 64;
                    let count = 8;
                    let left = (this._itemBox.width - 1100) / 2;
                    let top = 250;
                    let width = (this._itemBox.width - left * 2) / count;
                    let height = (this._itemBox.height - top * 2) / count;
                    for (let index = 0; index < num; index++) {
                        let item = this.createRewardItem(rewardType, globalStartX, globalStartY);
                        let scale = 1 + Math.random();
                        let ix = Math.floor(index / count) % count;
                        let iy = Math.floor(index % count);
                        let xx = left + Utils.randomExt2(ix * width, (ix + 1) * width);
                        let yy = top + Utils.randomExt2(iy * height, (iy + 1) * height);
                        let time2 = Math.pow((xx - endX) * (xx - endX) + (yy - endY) * (yy - endY), 0.5) / 2;
                        Laya.Tween.to(item, { x: xx, y: yy, scaleX: scale, scaleY: scale }, time1);
                        Laya.timer.once(time1 + 300, this, function () {
                            if (index == 0) {
                                item.startTween(null, null, endX, endY, dealHandler, time2, true, 0, (endX + item.x) / 2, (endY + item.y) / 2);
                            } else {
                                item.startTween(null, null, endX, endY, null, time2, true, 0, (endX + item.x) / 2, (endY + item.y) / 2);
                            }
                        });
                    }
                    break;
                }
                case Constants.FlyType.TYPE4: {
                    let num = 64;
                    let delay = 60;
                    let left = (this._itemBox.width - 1100) / 2;
                    let width = (this._itemBox.width - left * 2) / 4;
                    for (let index = 0; index < num; index++) {
                        let ix = index % 4;
                        let item = this.createRewardItem(rewardType, left + Utils.randomExt2(ix * width, (ix + 1) * width), -100);
                        let scale = 1 + Math.random();
                        item.scale(scale, scale);
                        if (index == num - 1) {
                            item.startSpeedTween(item.x, item.y, item.x, this._itemBox.height + 100, dealHandler, time + Utils.randomExt(200), true, Math.floor(index / 4) * delay, 0, 50);
                        } else {
                            item.startSpeedTween(item.x, item.y, item.x, this._itemBox.height + 100, null, time + Utils.randomExt(200), true, Math.floor(index / 4) * delay, 0, 50);
                        }
                    }
                    break;
                }
                case Constants.FlyType.TYPE5: {
                    time = 500;
                    let num = 10;
                    let delay = 80;
                    for (let index = 0; index < num; index++) {
                        let item = this.createRewardItem(rewardType, globalStartX, globalStartY);
                        item.visible = false;
                        Laya.timer.once(index * delay, this, function () {
                            item.visible = true;
                        });
                        if (index == num - 1) {
                            item.startTween(globalStartX, globalStartY, endX, endY, dealHandler, time, true, index * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                        } else {
                            item.startTween(globalStartX, globalStartY, endX, endY, null, time, true, index * delay, (endX + item.x) / 2, (endY + item.y) / 2);
                        }
                    }
                    break;
                }
                default: {
                    time = 1000;
                    console.log(rewardType, globalStartX, globalStartY);
                    let item = this.createRewardItem(rewardType, globalStartX, globalStartY);
                    Laya.Tween.to(item, { y: endY, alpha: 0 }, time, null, Laya.Handler.create(this, function () {
                        if (handler != null) {
                            handler.run();
                        }
                        item.removeSelf();
                        Laya.Pool.recover("flyItem", item);
                        dealHandler.recover();
                    }));
                }
            }
        }

        createRewardItem(rewardType, x, y) {
            let item = Laya.Pool.getItemByClass("flyItem", FlyItem);
            item.init(rewardType);
            item.pos(x, y);
            this._itemBox.addChild(item);
            return item;
        }

    }

    class HomeRole {
        constructor(data) {
            this.petData = data;

            this.eventActionConfig = ConfigMgr$1.getInstance().getConfig("ActionEvent");

            this.createStateMachine();

            this._intimacyLimit = Number(Const.get("HomeIntimacyAddLimit"));
        }
        getId() {
            return this.petData.getId();
        }

        //获得信息
        getData() {
            return this.petData;
        }

        createStateMachine() {
            let startIdleState = new StartIdleState$1("StartIdle");
            let idleRandomState = new RandomState$1("idleRandom");
            let idleState = new IdleState$1("Idle");
            let idleMoveState = new MoveState$1("IdleMove");
            let idleStopState = new StopState("IdleStop");
            let idleDragState = new DragState("IdleDrag");
            let idleDragEndState = new DragEndState("IdleDragEnd");
            let idleMenuState = new MenuState("IdleMenu");
            let idleActionState = new ActionState("IdleAction");

            let startFunState = new StartFunState("StartFun");
            let funRandomState = new RandomState$1("funRandom");
            let funIdleState = new IdleState$1("FunIdle");
            let funMoveState = new MoveState$1("FunMove");
            let funStopState = new StopState("FunStop");
            let funDragState = new DragState("FunDrag");
            let funDragEndState = new DragEndState("FunDragEnd");
            let funMenuState = new MenuState("FunMenu");
            let funActionState = new ActionState("FunAction");
            let cuteMenuState = new CuteMenuState("CuteMenu");
            let cuteActionState = new CuteActionState("CuteAction");

            startIdleState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState)
                .addTransition("Reset", startIdleState)
                .addTransition("Fun", startFunState);

            idleRandomState.addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState);

            idleState.addTransition("IdleMove", idleMoveState)
                .addTransition("Fun", startFunState)
                // .addTransition("Stop", idleStopState)
                .addTransition("Drag", idleDragState)
                .addTransition("Menu", idleMenuState)
                .addTransition("Play", idleActionState)
                .addTransition("Back", idleRandomState)
                .addTransition("Idle", idleState);

            idleMoveState.addTransition("Idle", idleState)
                .addTransition("Fun", startFunState)
                // .addTransition("Stop", idleStopState)
                .addTransition("Drag", idleDragState)
                .addTransition("Menu", idleMenuState)
                .addTransition("Play", idleActionState)
                .addTransition("Back", idleRandomState);

            // idleStopState.addTransition("Back", idleRandomState)
            //          .addTransition("Idle", idleState)
            //          // .addTransition("IdleMove", idleMoveState)
            //          .addTransition("Drag", idleDragState)
            //          .addTransition("Menu", idleMenuState)
            //          .addTransition("Play", idleActionState);

            idleDragState.addTransition("Back", idleRandomState)
                .addTransition("Idle", idleDragEndState)
                .addTransition("Drag", idleDragState)
                .addTransition("StopDrag", idleDragEndState);

            idleDragEndState.addTransition("Back", idleRandomState)
                // .addTransition("IdleMove", idleMoveState)
                .addTransition("Idle", idleState);

            idleMenuState.addTransition("Back", idleRandomState)
                .addTransition("HideMenu", idleState)
                // .addTransition("IdleMove", idleMoveState)
                .addTransition("Idle", idleState)
                .addTransition("Toy", idleActionState);

            idleActionState.addTransition("Back", idleRandomState)
                .addTransition("Idle", idleState)
                .addTransition("IdleMove", idleMoveState);




            startFunState.addTransition("Idle", funIdleState)
                .addTransition("IdleMove", funMoveState)
                .addTransition("Reset", startFunState);

            funRandomState.addTransition("Idle", funIdleState)
                .addTransition("IdleMove", funMoveState);

            funIdleState.addTransition("IdleMove", funMoveState)
                .addTransition("Cute", cuteMenuState)
                // .addTransition("Stop", funStopState)
                .addTransition("Drag", funDragState)
                .addTransition("Menu", funMenuState)
                .addTransition("Play", funActionState)
                .addTransition("Back", funRandomState)
                .addTransition("Idle", funIdleState);

            funMoveState.addTransition("Idle", funIdleState)
                .addTransition("Cute", cuteMenuState)
                // .addTransition("Stop", funStopState)
                .addTransition("Drag", funDragState)
                .addTransition("Menu", funMenuState)
                .addTransition("Play", funActionState)
                .addTransition("Back", funRandomState);

            // funStopState.addTransition("Back", funRandomState)
            //          // .addTransition("IdleMove", funMoveState)
            //          .addTransition("Idle", funIdleState)
            //          .addTransition("Drag", funDragState)
            //          .addTransition("Menu", funMenuState)
            //          .addTransition("Play", funActionState);

            funDragState.addTransition("Back", funRandomState)
                .addTransition("Idle", funDragEndState)
                .addTransition("Drag", funDragState)
                .addTransition("StopDrag", funDragEndState);

            funDragEndState.addTransition("Back", funRandomState)
                // .addTransition("IdleMove", funMoveState)
                .addTransition("Idle", funIdleState);

            funMenuState.addTransition("Back", funRandomState)
                .addTransition("HideMenu", funIdleState)
                // .addTransition("IdleMove", funMoveState)
                .addTransition("Idle", funIdleState)
                .addTransition("Toy", funActionState);

            funActionState.addTransition("Back", funRandomState)
                .addTransition("Idle", funIdleState)
                .addTransition("IdleMove", funMoveState);

            cuteMenuState.addTransition("Back", startFunState)
                .addTransition("HideMenu", startFunState)
                .addTransition("CuteToy", cuteActionState);

            cuteActionState.addTransition("Back", funRandomState)
                .addTransition("CuteEnd", startIdleState);


            this._stateMachine = new StateMachine(this, startIdleState);
        }

        switchToState(newState, message) {
            return this._stateMachine.changeState(newState, this._context, message);
        }

        sendMsg(msgType, params) {
            this._stateMachine.handleMessage({ "type": msgType, "params": params });
        }

        setMapInfo(info) {
            this._unitX = info[0];
            this._unitY = info[1];
            this._maxX = info[2];
            this._maxY = info[3];
        }

        get mapX() {
            return Math.floor(this.petData.x / this._unitX);
        }

        get mapY() {
            return Math.floor(this.petData.y / this._unitY);
        }

        getRealX(mapX) {
            return (mapX + 0.5) * this._unitX;
        }

        randomRealX(mapX) {
            return (mapX + Math.random()) * this._unitX;
        }

        getRealY(mapY) {
            return (mapY + 0.5) * this._unitY;
        }

        randomRealY(mapY) {
            return (mapY + Math.random()) * this._unitY;
        }

        calcMapX(realX) {
            return Math.floor(realX / this._unitX);
        }

        calcMapY(realY) {
            return Math.floor(realY / this._unitY);
        }

        // 不用考虑猫的互相碰撞
        // getBlocks() {
        //     let x = this.mapX;
        //     let y = this.mapY;
        //     let arr = [];
        //     for(let i = x - 2; i <= x+2; ++i)
        //         for(let j = y - 2; j<=y+2; ++j)
        //             arr.push([i,j]);
        // }



        start(context, currentTime) {
            this._context = context;
            let petData = this.petData;
            let x = petData.x;
            let y = petData.y;
            let funTimer = petData.funTimer;
            let cuteTimer = petData.cuteTimer;
            let cuted = petData.cuted;
            let period = petData.lastUpdateTime ? currentTime - petData.lastUpdateTime : 0;
            let hungerTimer = petData.hungerTimer;
            let thirstTimer = petData.thirstTimer;
            let dirtyTimer = petData.dirtyTimer;
            petData.sad = PetSystem.getRubbishNum();
            if (x == null || y == null) {
                x = 500;
                y = 1000;
                let blockMap = this._context._blockMap;
                let nodes = blockMap.getValidNodes();
                let node = nodes[Utils.randomExt(nodes.length)];
                if (node) {
                    x = this.randomRealX(node.x);
                    y = this.randomRealY(node.y);;
                }
            }
            if (Constants.guid14==1) {
                this.setPos(1000, 1500);
                this.sendMsg("Reset");
                Laya.timer.frameOnce(2, this, function () {
                    this.petData.dirtyTimer = 10;
                });
            } else {
                this.setPos(x, y);
            }
            this._context.recordMetaEvent(this.getId(), "PutPet", { "petId": this.getId(), "inNautical": petData.inNautical, "x": this.petData.x, "y": this.petData.y, "mapInfo": [this._unitX, this._unitY, this._maxX, this._maxY] }, "HomePetLine");


            if (Constants.guid14!=1) {
                if (cuted) {
                    this.sendMsg("Reset");
                } else if (cuteTimer) {
                    this.sendMsg("Fun");
                    this.updateTimer(period);
                }
                else if (funTimer) {
                    this.sendMsg("Idle");
                    this.updateTimer(period);
                } else {
                    this.sendMsg("Reset");
                }
            }
            if (!hungerTimer) {
                this.setHealthyTimer(Constants.HealthyType.HUNGER);
            }
            if (!thirstTimer) {
                this.setHealthyTimer(Constants.HealthyType.THIRST);
            }
            if (!dirtyTimer) {
                this.setHealthyTimer(Constants.HealthyType.DIRTY);
            }
            petData.lastUpdateTime = currentTime;

            this._inNautical = petData.inNautical;
        }

        updateTimer(dt) {
            if (dt <= 0)
                return;
            if (this.petData.funTimer) {
                this.petData.funTimer -= dt;
                if (this.petData.funTimer <= 0) {
                    if (this.petData.dailyIntimacy < this._intimacyLimit && this.petData.getIntimacy() < this.petData.getIntimacyMax())
                        this.sendMsg("Fun");
                    else
                        this.setFunTimer();
                }
            }
            if (this.petData.cuteTimer) {
                this.petData.cuteTimer -= dt;
                if (this.petData.cuteTimer <= 0) {
                    if (this.getFavorToy() && this.petData.happy)
                        this.sendMsg("Cute");
                    else
                        this.setCuteTimer();
                }
            }
            if (this.petData.hungerTimer) {
                this.petData.hungerTimer -= dt;
                if (this.petData.hungerTimer <= 0) {
                    this.setHealthy(Constants.HealthyType.HUNGER, true);
                    this.clearHealthyTimer(Constants.HealthyType.HUNGER);
                }
            }
            if (this.petData.thirstTimer) {
                this.petData.thirstTimer -= dt;
                if (this.petData.thirstTimer <= 0) {
                    this.setHealthy(Constants.HealthyType.THIRST, true);
                    this.clearHealthyTimer(Constants.HealthyType.THIRST);
                }
            }
            if (this.petData.dirtyTimer) {
                this.petData.dirtyTimer -= dt;
                if (this.petData.dirtyTimer <= 0) {
                    this.setHealthy(Constants.HealthyType.DIRTY, true);
                    this.clearHealthyTimer(Constants.HealthyType.DIRTY);
                }
            }

        }

        update(dt, currentTime) {
            let petData = this.petData;
            if (this._inNautical != petData.inNautical) {
                this._inNautical = petData.inNautical;
                if (petData.inNautical)
                    this._context.recordEvent(this.getId(), "Hide");
                else {
                    this.updateTimer(currentTime - this.petData.lastUpdateTime);
                    this._context.recordEvent(this.getId(), "Show");
                }
            }
            if (!this._inNautical) {
                this.petData.lastUpdateTime = currentTime;
                this._stateMachine.update(this._context, dt);
            }
        }

        sleep() {
            this.sendMsg("Back");
        }

        back(currentTime) {
            this.sendMsg("Back");
            if (!this._inNautical) {
                this.updateTimer(currentTime - this.petData.lastUpdateTime);
                this.petData.lastUpdateTime = currentTime;
            }
        }

        remove() {
            if (!this._context) {
                return true;
            }
            this._context.recordEvent(this.getId(), "RemovePet", null);
            return true;
        }

        stop() {
            if (!this._context) {
                console.debug("HomeRole stop befrore start");
                return;
            }
            this.sendMsg("Stop");
        }

        drag(args) {
            if (!this._context) {
                console.debug("HomeRole drag befrore start");
                return;
            }
            this.sendMsg("Drag", args);
        }

        move(args) {
            if (!this._context) {
                console.debug("HomeRole move befrore start");
                return;
            }
            this.sendMsg("StopDrag", args);
        }

        showMenu(args) {
            if (!this._context) {
                console.debug("HomeRole showMenu befrore start");
                return;
            }
            this.sendMsg("Menu", args);
        }

        hideMenu(args) {
            if (!this._context) {
                console.debug("HomeRole hideMenu befrore start");
                return;
            }
            this.sendMsg("HideMenu", args);
        }

        play(args) {
            if (!this._context) {
                console.debug("HomeRole play befrore start");
                return;
            }
            this.sendMsg("Play", args);
        }

        playToy(args) {
            if (!this._context) {
                console.debug("HomeRole playToy befrore start");
                return;
            }
            this.sendMsg("Toy", args);
        }

        playCuteToy(args) {
            if (!this._context) {
                console.debug("HomeRole playCuteToy befrore start");
                return;
            }
            this.sendMsg("CuteToy", args);
        }

        playBeHealthy(args) {
            if (!this._context || !this.petData) {
                return;
            }
            let type = null;
            if (this.petData.hunger) {
                type = Constants.HealthyType.HUNGER;
            } else if (this.petData.thirst) {
                type = Constants.HealthyType.THIRST;
            } else if (this.petData.dirty) {
                type = Constants.HealthyType.DIRTY;
            }
            if (type == null) {
                return;
            }
            let handler = Laya.Handler.create(this, function (res) {
                this.setHealthy(res.type, false);
                let reward = PetSystem.getHealthyRewards(res.type, args.adType);
                let point = res.point;
                if (reward.FISH > 0) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.FISH, Number(reward.FISH), Constants.FlyType.ENHANCE, point.x, point.y);
                }
                if (reward.STAR > 0) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, Number(reward.STAR), Constants.FlyType.ENHANCE, point.x, point.y);
                }
                if (reward.MEDAL > 0) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.MEDAL, Number(reward.MEDAL), null, point.x, point.y);
                }
                if (reward.BOOK > 0) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.BOOK, Number(reward.BOOK), null, point.x, point.y);
                }
                PetSystem.healthInteract(this.getId(), res.type, Laya.Handler.create(this, this.afterHealthAction));
            });
            this.sendMsg("Idle");
            this._context.recordEvent(this.getId(), "PlayBeHealthy", { "handler": handler, "type": type });
        }

        setFunTimer() {
            let array = Const.getArray("HomeStateChangeToExciting");
            this.petData.funTimer = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
        }

        clearFunTimer() {
            this.petData.funTimer = null;
        }

        setCuteTimer() {
            let array = Const.getArray("HomeActionChangeToBeg");
            this.petData.cuteTimer = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
        }

        clearCuteTimer() {
            this.petData.cuteTimer = null;
        }

        setCuted(cuted) {
            this.petData.cuted = cuted;
        }

        setHealthyTimer(type) {
            if (type == Constants.HealthyType.HUNGER) {
                let array = Const.getArray("HomeHealthyHunger");
                this.petData.hungerTimer = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
            } else if (type == Constants.HealthyType.THIRST) {
                let array = Const.getArray("HomeHealthyThirsty");
                this.petData.thirstTimer = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
            } else if (type == Constants.HealthyType.DIRTY) {
                let array = Const.getArray("HomeHealthyDirty");
                this.petData.dirtyTimer = Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000;
            }
        }

        clearHealthyTimer(type) {
            if (type == Constants.HealthyType.HUNGER) {
                this.petData.hungerTimer = null;
            } else if (type == Constants.HealthyType.THIRST) {
                this.petData.thirstTimer = null;
            } else if (type == Constants.HealthyType.DIRTY) {
                this.petData.dirtyTimer = null;
            }
        }

        setHealthy(type, healthy) {
            if (!this._context) {
                return true;
            }
            if (type == Constants.HealthyType.HUNGER) {
                this.petData.hunger = healthy;
            } else if (type == Constants.HealthyType.THIRST) {
                this.petData.thirst = healthy;
            } else if (type == Constants.HealthyType.DIRTY) {
                this.petData.dirty = healthy;
            }
            if (!healthy) {
                this.setHealthyTimer(type);
            }
            this.petData.sad = PetSystem.getRubbishNum();
            // this._context.recordEventNextFrame(this.getId(), "ShowHealth");
            this._context.recordEvent(this.getId(), "ShowHealth");
        }

        calcIdle(message, stateName) {
            return { "duration": Utils.randomExt2(3000, 5000) };
        }

        calcRandomMove(message, stateName) {
            this._pathStartX = null;
            this._pathStartY = null;
            this._realPath = null;
            this._moveDuration = null;
            let blockMap = this._context._blockMap;
            let x = this.petData.x;
            let y = this.petData.y;
            let nodes = blockMap.getValidNodes(x, y, 50);
            let node = nodes[Utils.randomExt(nodes.length)];
            if (node) {
                let endX = node.x;
                let endY = node.y;
                let targetX = this.randomRealX(node.x);
                let targetY = this.randomRealY(node.y);
                let startX = this.mapX;
                let startY = this.mapY;
                let path = blockMap.findPath(startX, startY, endX, endY);
                if (path && path.length > 0) {
                    let realPath = [];
                    let lastX = this.petData.x;
                    let lastY = this.petData.y;
                    let totalDuration = 0;
                    for (let i = 1; i <= path.length - 2; ++i) {
                        let nextX = this.getRealX(path[i].x);
                        let nextY = this.getRealY(path[i].y);
                        let dx = lastX - nextX;
                        let dy = lastY - nextY;
                        let distance = Math.sqrt(dx * dx + dy * dy);
                        let duration = distance / this.petData.speed * 1000;
                        realPath.push([nextX, nextY, duration]);
                        totalDuration += duration;
                        lastX = nextX;
                        lastY = nextY;
                    }
                    let dx = lastX - targetX;
                    let dy = lastY - targetY;
                    let distance = Math.sqrt(dx * dx + dy * dy);
                    let duration = distance / this.petData.speed * 1000;
                    realPath.push([targetX, targetY, duration]);
                    totalDuration += duration;
                    this._realPath = realPath.reverse();
                    this._moveDuration = 0;
                    this._pathStartX = this.petData.x;
                    this._pathStartY = this.petData.y;
                    return { "duration": totalDuration, "params": this._realPath.slice(0) };
                }
            }
            return null;
        }

        _updatePos(dt) {
            if (this._moveDuration != null) {
                this._moveDuration += dt;
                let duration = this._moveDuration;
                let lastX = this._pathStartX;
                let lastY = this._pathStartY;
                for (let i = this._realPath.length - 1; i >= 0; --i) {
                    let pathInfo = this._realPath[i];
                    if (duration > pathInfo[2]) {
                        lastX = pathInfo[0];
                        lastY = pathInfo[1];
                        duration -= pathInfo[2];
                    } else {
                        // console.debug("updatePos", this._moveDuration, i, this._realPath);
                        this.setPos(lastX + (pathInfo[0] - lastX) * duration / pathInfo[2],
                            lastY + (pathInfo[1] - lastY) * duration / pathInfo[2]);
                        return;
                    }
                }
            }
        }

        setPos(x, y) {
            let targetX = this.calcMapX(x), targetY = this.calcMapY(y);
            let blockMap = this._context._blockMap;

            if (!blockMap.isWalkableAt(targetX, targetY)) {
                let node = blockMap.findNearestValidNode(targetX, targetY, this.mapX, this.mapY);
                if (node) {
                    targetX = node.x;
                    targetY = node.y;
                    this.petData.pos(this.randomRealX(targetX),
                        this.randomRealY(targetY));
                    return;
                }
            }
            this.petData.pos(x, y);
        }

        getPos() {
            return { "x": this.petData.x, "y": this.petData.y };
        }

        calcStop(message, stateName) {
            return { "duration": 4000, "event": "Stop" };
        }

        calcStartDrag(message, stateName) {
            return { "anim": "drag_up" };
        }

        calcDrag(message, stateName) {
            return { "duration": 5000 };
        }

        calcEndDrag(message, stateName) {
            if (message.type == "StopDrag") {
                let x = message.params.x;
                let y = message.params.y;
                this.setPos(x, y);
            }
            let args = this.getPos();
            args.anim = "drag_down";
            return args;
        }

        calcMenu(message, stateName) {
            return { "duration": 10000, "params": { "toys": this.petData.FavorToys } };
        }

        calcPlay() {
            let array = Const.getArray("HomeNormalPlayTime");
            return { "duration": Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000, "event": "Play" };
        }

        calcAction(message, stateName) {
            // this.petData.increase = null;
            this._toyId = null;
            if (message.type == "Play") {
                // PetSystem.commonInteract(this.getId());
                return this.calcPlay();
            } else {
                let toyId = message.params.toyId;
                if (toyId && PetSystem.hasToy(toyId)) {
                    // let rate = Math.max((1-this.petData.dailyIntimacy/this._intimacyLimit)*Number(Const.get("HomeIntimacyAddRateK")), 0)
                    let toys = this.petData.FavorToys;
                    for (const toyInfo of toys) {
                        if (toyInfo.NeedToyId == toyId) {
                            // PetSystem.commonInteract(this.getId(), toyId);
                            this._toyId = toyId;
                            let array = toyInfo.KeepTime;
                            return { "duration": Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000, "event": "PlayToy", "params": { "anim": toyInfo.Action } };
                        }
                    }
                }
                // PetSystem.commonInteract(this.getId());
                return this.calcPlay();
            }
            return args;
        }

        afterAction(context, stateName) {
            PetSystem.commonInteract(this.getId(), this._toyId, Laya.Handler.create(this, function (res) {
                context.recordEventNextFrame(this.getId(), "ActionEnd", { "increase": res.increase });
            }));
        }

        getFavorToy() {
            let toys = this.petData.FavorToys;
            let arr = [];
            let totalWeight = 0;
            for (const toyInfo of toys) {
                if (PetSystem.hasToy(toyInfo.NeedToyId)) {
                    totalWeight += Number(toyInfo.Weight);
                    arr.push({ "w": Number(toyInfo.Weight), "NeedToyId": toyInfo.NeedToyId, "Action": toyInfo.Action, "BegIntimacyAdd": toyInfo.BegIntimacyAdd, "KeepTime": toyInfo.KeepTime });
                }
            }
            return Utils.calcRandomWeight(arr, totalWeight);
        }

        calcCuteMenu(message, stateName) {
            let duration = Number(Const.get("HomeActionChangeToWalk")) * 1000;
            this._favorToy = this.getFavorToy();
            return { "duration": duration, "params": { "toy": this._favorToy } };
        }

        calcCuteAction(message, stateName) {
            // this.petData.increase = null;
            if (this._favorToy) {
                // PetSystem.cuteInteract(this.getId(), this._favorToy.NeedToyId);
                let array = this._favorToy.KeepTime;
                return { "duration": Utils.randomExt2(Number(array[0]), Number(array[1])) * 1000, "event": "PlayToy", "params": { "anim": this._favorToy.Action } };
            } else {
                return this.calcPlay();
            }
        }

        afterCuteAction(context, stateName) {
            if (this._favorToy) {
                PetSystem.cuteInteract(this.getId(), this._favorToy.NeedToyId, Laya.Handler.create(this, function (res) {
                    context.recordEventNextFrame(this.getId(), "CuteActionEnd", { "increase": res.increase });
                }));
            }
        }

        afterHealthAction(res) {
            this._context.recordEventNextFrame(this.getId(), "PlayHealthyEnd", { "increase": res.increase });
        }

        destroy() {
            if (this._context) {
                this._context.removeTimeline(this.getId());
                this._context = null;
            }
            this._stateMachine = null;
        }
    }

    class StartIdleState$1 extends BaseState {

        enter(agent, context, message) {
            agent.setCuted(null);
            agent.clearCuteTimer();
            agent.setFunTimer();
            context.recordEvent(agent.getId(), "Reset", agent.getPos());
            if (Constants.guid14==1) {
                this.startTimer(1, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                if (Utils.randomSign() == 1) {
                    this.startTimer(5, Laya.Handler.create(this, this.next, [agent, context]));
                } else {
                    this.startTimer(5, Laya.Handler.create(this, this.next2, [agent, context]));
                }
            }
        }
        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }

        next2(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class RandomState$1 extends BaseState {

        enter(agent, context, message) {
            context.recordEvent(agent.getId(), "Reset", agent.getPos());
            if (Utils.randomSign() == 1) {
                this.startTimer(1, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(1, Laya.Handler.create(this, this.next2, [agent, context]));
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }

        next2(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class IdleState$1 extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcIdle(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), "Idle", args.params);
            if (Constants.guid14==1) {
                this.startTimer(10000000000000, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
            }
        }
        update(agent, context, dt) {
            super.update(agent, context, dt);
            agent.updateTimer(dt);
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            if (message.type == "Drag") {
                let args = agent.calcStartDrag(message, this._name);
                context.recordEvent(agent.getId(), "StartDrag", args);
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class MoveState$1 extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcRandomMove(message, this._name);
            if (args) {
                let duration = args.duration;
                context.recordEvent(agent.getId(), "Move", args.params);
                this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(5, Laya.Handler.create(this, this.next, [agent, context]));
            }
        }

        update(agent, context, dt) {
            super.update(agent, context, dt);
            agent.updateTimer(dt);
            agent._updatePos(dt);
            // context.recordEvent(agent.getId(), "MoveIng", agent.getPos());
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            context.recordEvent(agent.getId(), "MoveEnd", agent.getPos());
            if (message.type == "Drag") {
                let args = agent.calcStartDrag(message, this._name);
                context.recordEvent(agent.getId(), "StartDrag", args);
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class StopState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcStop(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            if (message.type == "Drag") {
                let args = agent.calcStartDrag(message, this._name);
                context.recordEvent(agent.getId(), "StartDrag", args);
            } else {
                context.recordEvent(agent.getId(), "StopEnd");
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class DragState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcDrag(message, this._name);
            let duration = args.duration;
            // context.recordEvent(agent.getId(), "Drag", args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        exit(agent, context, message) {
            super.exit(agent, context);
            if (message.type != "Drag") {
                let args = agent.calcEndDrag(message, this._name);
                context.recordEvent(agent.getId(), "EndDrag", args);
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class DragEndState extends BaseState {
        enter(agent, context, message) {
            this.startTimer(400, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class MenuState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcMenu(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), "ShowMenu", args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }
    }

    class ActionState extends BaseState {

        enter(agent, context, message) {
            let args = agent.calcAction(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }


        next(agent, context) {
            agent.afterAction(context, this._name);
            this.onMessage(agent, { "type": "Idle" });
        }
    }


    class StartFunState extends BaseState {

        enter(agent, context, message) {
            agent.setCuted(null);
            agent.clearFunTimer();
            agent.setCuteTimer();
            context.recordEvent(agent.getId(), "Reset", agent.getPos());
            if (Utils.randomSign() == 1) {
                this.startTimer(5, Laya.Handler.create(this, this.next, [agent, context]));
            } else {
                this.startTimer(5, Laya.Handler.create(this, this.next2, [agent, context]));
            }
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Idle" });
        }

        next2(agent, context) {
            this.onMessage(agent, { "type": "IdleMove" });
        }
    }

    class CuteMenuState extends BaseState {

        enter(agent, context, message) {
            agent.clearCuteTimer();
            let args = agent.calcCuteMenu(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), "CuteMenu", args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            this.onMessage(agent, { "type": "Back" });
        }
    }

    class CuteActionState extends BaseState {

        enter(agent, context, message) {
            agent.setCuted(true);
            let args = agent.calcCuteAction(message, this._name);
            let duration = args.duration;
            context.recordEvent(agent.getId(), args.event, args.params);
            this.startTimer(duration, Laya.Handler.create(this, this.next, [agent, context]));
        }

        next(agent, context) {
            agent.afterCuteAction(context, this._name);
            this.onMessage(agent, { "type": "CuteEnd" });
        }
    }

    class HomeRoleMgr extends BeachRoleMgr {
        addRole(petData) {
            let role = new HomeRole(petData);
            this._roleMap.set(String(petData.getId()), role);
            this._roleList.push(role);
            return role;
        }
    }

    let kMapInfo = [25, 28, 107, 87];

    class HomelandLogic extends Logic {

        constructor() {
            super();
            this._roleMgr = new HomeRoleMgr();
            this._currentTime = 0;
        }

        start(context) {
            this._context = context;
            context._roleMgr = this._roleMgr;
            // this._context.newMainLine();
            this._blockMap = new BlockMap(MapInfo.width, MapInfo.height, MapInfo.block);
            context._blockMap = this._blockMap;
        }

        step(dt) {
            this._currentTime += dt;
            if(this._stopUpdate)
                return;
            let roles = this._roleMgr.getRoles();
            for(const role of roles) {
                role.update(dt, this._currentTime);
            }
        }

        handlerInput(input) {
            let op = input.op;
            let args = input.args;
            let handler = input.handler;
            switch(op) {
                case "SET_TIME":
                    this._setTime(args);
                    break;
                case "SCENE_HIDE":
                    this._sceneHide(args);
                    break;
                case "SCENE_BACK":
                    this._sceneBack(args);
                    break;
                case "UPDATE_POS":
                    this._updatePos(args);
                    break;
                case "PUT_PET":
                    this._addPet(args);
                    break;
                case "REMOVE_PET":
                    this._removePet(args);
                    break;
                case "STOP_PET":
                    this._stopPet(args);
                    break;
                case "DRAG_PET":
                    this._dragPet(args);
                    break;
                case "MOVE_PET":
                    this._movePet(args);
                    break;
                case "SHOW_PET_MENU":
                    this._showPetMenu(args);
                    break;
                case "HIDE_PET_MENU":
                    this._hidePetMenu(args);
                    break;
                case "PLAY":
                    this._play(args);
                    break;
                case "PLAY_TOY":
                    this._playToy(args);
                    break;
                case "PLAY_CUTE_TOY":
                    this._playCuteToy(args);
                    break;
                case "PLAY_BE_HEALTHY":
                    this._playBeHealthy(args);
                    break;
            }
        }

        _sceneHide() {
            this._stopUpdate = true;
            let roles = this._roleMgr.getRoles();
            for(const role of roles) {
                role.sleep(this._currentTime);
            }
        }

        _sceneBack() {
            this._stopUpdate = null;
            let roles = this._roleMgr.getRoles();
            for(const role of roles) {
                role.back(this._currentTime);
            }
        }

        _updatePos(args) {
            let petId = args.petId;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.setPos(args.x, args.y);
        }

        _addPet(args) {
            let petId = args;
            if(!this._roleMgr.getRoleById(petId)) {
                let petData = PetSystem.getPetInfo(petId);
                let role = this._roleMgr.addRole(petData);
                if(role) {
                    role.setMapInfo(kMapInfo);
                    role.start(this._context, this._currentTime);
                }
            }
        }

        _removePet(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role) {
                if(role.remove()) {
                    this._roleMgr.removeRole(petId);
                }
            }
        }

        _stopPet(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.stop();
        }

        _dragPet(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.drag(args);
        }

        _movePet(args) {
            let petId = args.petId;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.move(args);
        }

        _showPetMenu(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.showMenu(args);
        }

        _hidePetMenu(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.hideMenu(args);
        }

        _play(args) {
            let petId = args;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.play(args);
        }

        _playToy(args) {
            let petId = args.petId;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.playToy(args);
        }

        _playCuteToy(args) {
            let petId = args.petId;
            let role = this._roleMgr.getRoleById(petId);
            if(role)
                role.playCuteToy(args);
        }

        _setTime(args) {
            this._currentTime = args;
        }

        _playBeHealthy(args) {
            let petId = args.petId;
            let role = this._roleMgr.getRoleById(petId);
            if (role)
                role.playBeHealthy(args);
        }

    }

    let kLoopInterval$2 = 50;
    class HomeSession {
        static getOrCreateSession() {
            if(GameContext.homeSession){
                return GameContext.homeSession;
            }
            GameContext.homeSession = new HomeSession();
            GameContext.homeSession.build();
            GameContext.homeSession.prepare();
            return GameContext.homeSession;
        }

        static getSession() {
            if(GameContext.homeSession){
                return GameContext.homeSession;
            }
        }

        static runSession() {
            let session = HomeSession.getOrCreateSession();
            session.startLoop();
        }

        static sendOpCommand(op, args, handler) {
            let session = HomeSession.getSession();
            if(session)
                session._sendOpCommand(op, args, handler);
        }

        static simulate(time) {
            let session = HomeSession.getSession();
            if(session)
                session._simulate(time);
        }

        static clear() {
            if(GameContext.homeSession) {
                GameContext.homeSession.destroy();
                GameContext.homeSession = null;
            }
        }

        constructor() {
            this._director = new SceneDirector();
            this._simulator = new SceneSimulator();
            this._logic = new HomelandLogic();
            this._controller = new Controller();
            this._recorder = new Recorder();
            this._interpreter = new Interpreter();
        }

        build() {
            this._director.setSimulator(this._simulator);
            this._director.setInterpreter(this._interpreter);
            this._interpreter.setRecordsProvider(this._recorder);
            this._simulator.setRecorder(this._recorder);
            this._simulator.setLogic(this._logic);
            this._simulator.setController(this._controller);
        }

        prepare() {
            this._director.start();

        }

        startLoop() {
            Laya.timer.loop(kLoopInterval$2, this, this._update, [], true, true);
        }

        _update() {
            this._director.update(kLoopInterval$2);
        }

        //模拟多少时间
        _simulate(time) {
            this._director.setInterpreterMode(2);
            this._director.update(time);
            this._director.flush();
            this._director.setInterpreterMode(1);
        }

        _sendOpCommand(op, args, handler) {
            this._controller.sendOpCommand(op, args, handler);
        }


        destroy() {
            this.destroyed = true;
            EventMgr.getInstance().offAllCaller(this);
            Laya.timer.clearAll(this);
        }
    }

    class SimulatorSystem {
        static init() {
            EventMgr.getInstance().registEvent("SIMULATOR_SEND_CMD", null, SimulatorSystem.sendOpCommand);

            EventMgr.getInstance().registEvent("SIMULATOR_CREATE_SESSION", null, SimulatorSystem.CreateSession);

            EventMgr.getInstance().registEvent("SIMULATOR_RUN_SESSION", null, SimulatorSystem.RunSession);

            EventMgr.getInstance().registEvent("SIMULATOR_SIMULATE_SESSION", null, SimulatorSystem.SimulateSession);

        }

        static clear() {
            CoffeeSession.clear();
            BeachSession.clear();
            HomeSession.clear();
        }

        static getSession(sceneId) {
            if(sceneId==Constants.CoffeeSceneId) {
                return CoffeeSession;
            }else if(sceneId==Constants.BeachSceneId) {
                return BeachSession;
            }else if(sceneId==Constants.HomelandId) {
                return HomeSession;
            }
        }

        static CreateSession(sceneId) {
            let session = SimulatorSystem.getSession(sceneId);
            if(session) {
                session.getOrCreateSession();
            }
        }

        static SimulateSession(sceneId, time) {
            let session = SimulatorSystem.getSession(sceneId);
            if(session) {
                session.simulate(time);
            }
        }

        static RunSession(sceneId) {
            let session = SimulatorSystem.getSession(sceneId);
            if(session) {
                session.runSession();
            }
        }

        static sendOpCommand(sceneId, op, args, handler) {
            let session = SimulatorSystem.getSession(sceneId);
            if(session) {
                session.sendOpCommand(op, args, handler);
            }
        }
    }

    /**
     *
     * 协议映射表
     */
    class ProtoIds {
        constructor() {
        }

        static getRequestMap() {
            var protoIDs = {};
            protoIDs["Player.LoginReq"]                 = [10001, WxConfigMgr.ProtoEnable ? Player.LoginReq : null];
            protoIDs["Player.LogoutReq"]                = [10002, WxConfigMgr.ProtoEnable ? Player.LogoutReq : null];
            protoIDs["Player.OnlineCrossDayReq"]        = [10003, WxConfigMgr.ProtoEnable ? Player.OnlineCrossDayReq : null];
            protoIDs["Player.FullBagReq"]               = [10004, WxConfigMgr.ProtoEnable ? Player.FullBagReq : null];
            protoIDs["Player.PutCustomDataReq"]         = [10005, WxConfigMgr.ProtoEnable ? Player.PutCustomDataReq : null];
            protoIDs["Player.DelCustomDataReq"]         = [10006, WxConfigMgr.ProtoEnable ? Player.DelCustomDataReq : null];
            protoIDs["Player.GetCustomDataReq"]         = [10007, WxConfigMgr.ProtoEnable ? Player.GetCustomDataReq : null];
            protoIDs["Player.ViewAdReq"]                = [10008, WxConfigMgr.ProtoEnable ? Player.ViewAdReq : null];
            protoIDs["Player.AuthorizeReq"]             = [10009, WxConfigMgr.ProtoEnable ? Player.AuthorizeReq : null];
            protoIDs["Player.ClientLoadingLogReq"]      = [10010, WxConfigMgr.ProtoEnable ? Player.ClientLoadingLogReq : null];
            protoIDs["Player.ShareReq"]                 = [10011, WxConfigMgr.ProtoEnable ? Player.ShareReq : null];

            protoIDs["Building.GetAllBuildingReq"]      = [10101, WxConfigMgr.ProtoEnable ? Building.GetAllBuildingReq : null];
            protoIDs["Building.PutBuildingWithAdReq"]   = [10104, WxConfigMgr.ProtoEnable ? Building.PutBuildingWithAdReq : null];
            protoIDs["Building.RemoveBuildingReq"]      = [10103, WxConfigMgr.ProtoEnable ? Building.RemoveBuildingReq : null];
            protoIDs["Building.BargainReq"]      = [10105, WxConfigMgr.ProtoEnable ? Building.BargainReq : null];

            protoIDs["Scene.UpdateSceneInfoReq"]        = [10201, WxConfigMgr.ProtoEnable ? Scene.UpdateSceneInfoReq : null];
            protoIDs["Scene.FetchLetterBoxReq"]         = [10202, WxConfigMgr.ProtoEnable ? Scene.FetchLetterBoxReq : null];
            protoIDs["Scene.GetLetterBoxInfoReq"]       = [10203, WxConfigMgr.ProtoEnable ? Scene.GetLetterBoxInfoReq : null];
            protoIDs["Scene.FindVisitorGiftReq"]        = [10204, WxConfigMgr.ProtoEnable ? Scene.FindVisitorGiftReq : null];
            protoIDs["Scene.GetVisitorGiftAllReq"]      = [10205, WxConfigMgr.ProtoEnable ? Scene.GetVisitorGiftAllReq : null];
            protoIDs["Scene.GetVisitorGiftSingleReq"]   =  [10208, WxConfigMgr.ProtoEnable ? Scene.GetVisitorGiftSingleReq : null];
            protoIDs["Scene.CatFoodSubscribeReq"]       =  [10209, WxConfigMgr.ProtoEnable ? Scene.CatFoodSubscribeReq : null];

            protoIDs["Task.GetAllTasksReq"]             = [10301, WxConfigMgr.ProtoEnable ? Task.GetAllTasksReq : null];
            protoIDs["Task.getDailyTaskRewardWithAdReq"]      = [10304, WxConfigMgr.ProtoEnable ? Task.getDailyTaskRewardWithAdReq : null];
            protoIDs["Task.getTrophyTaskRewardWithAdReq"]     = [10305, WxConfigMgr.ProtoEnable ? Task.getTrophyTaskRewardWithAdReq : null];
            protoIDs["Task.getMainTaskRewardReq"]             = [10306, WxConfigMgr.ProtoEnable ? Task.getMainTaskRewardReq : null];

            protoIDs["Food.GetPlayerFoodMenuReq"]       = [10401, WxConfigMgr.ProtoEnable ? Food.GetPlayerFoodMenuReq : null];
            protoIDs["Food.FoodLevelUpWithAdReq"]             = [10403, WxConfigMgr.ProtoEnable ? Food.FoodLevelUpWithAdReq : null];

            protoIDs["Pet.GetPlayerPetsInfoReq"]        = [10501, WxConfigMgr.ProtoEnable ? Pet.GetPlayerPetsInfoReq : null];
            protoIDs["Pet.AdoptPetReq"]                 = [10502, WxConfigMgr.ProtoEnable ? Pet.AdoptPetReq : null];
            protoIDs["Pet.StoryUnlockReq"]                 = [10503, WxConfigMgr.ProtoEnable ? Pet.StoryUnlockReq : null];
            protoIDs["Pet.ShareGetRewardReq"]                 = [10504, WxConfigMgr.ProtoEnable ? Pet.ShareGetRewardReq : null];
            protoIDs["Pet.CatLevelUpReq"]                 = [10505, WxConfigMgr.ProtoEnable ? Pet.CatLevelUpReq : null];
            protoIDs["Pet.CatStarUpGradeReq"]                 = [10506, WxConfigMgr.ProtoEnable ? Pet.CatStarUpGradeReq : null];

            // protoIDs["Nautical.GetNauticalInfoReq"]     = [10601, WxConfigMgr.ProtoEnable ? Nautical.GetNauticalInfoReq : null];
            protoIDs["Nautical.MergeNauticalFoodReq"]   = [10602, WxConfigMgr.ProtoEnable ? Nautical.MergeNauticalFoodReq : null];
            // protoIDs["Nautical.StartNauticalTripRes"]     = [10603, WxConfigMgr.ProtoEnable ? Nautical.StartNauticalTripReq : null];
            // protoIDs["Nautical.FinishNauticalTripReq"]   = [10604, WxConfigMgr.ProtoEnable ? Nautical.FinishNauticalTripReq : null];
            // protoIDs["Nautical.PreviewNauticalTripReq"]   = [10605, WxConfigMgr.ProtoEnable ? Nautical.PreviewNauticalTripReq : null];
            // protoIDs["Nautical.AdReduceNauticalTimeReq"]   = [10606, WxConfigMgr.ProtoEnable ? Nautical.AdReduceNauticalTimeReq : null];
            protoIDs["Nautical.GetNauticalInfo0912Req"]     = [10607, WxConfigMgr.ProtoEnable ? Nautical.GetNauticalInfo0912Req : null];
            protoIDs["Nautical.StartNauticalTrip0912Req"]     = [10608, WxConfigMgr.ProtoEnable ? Nautical.StartNauticalTrip0912Req : null];
            protoIDs["Nautical.FinishNauticalTrip0912Req"]   = [10609, WxConfigMgr.ProtoEnable ? Nautical.FinishNauticalTrip0912Req : null];
            protoIDs["Nautical.PreviewNauticalTrip0912Req"]   = [10610, WxConfigMgr.ProtoEnable ? Nautical.PreviewNauticalTrip0912Req : null];
            protoIDs["Nautical.AdReduceNauticalTime0912Req"]   = [10611, WxConfigMgr.ProtoEnable ? Nautical.AdReduceNauticalTime0912Req : null];
            protoIDs["Nautical.unlockNauticalReq"]   = [10612, WxConfigMgr.ProtoEnable ? Nautical.unlockNauticalReq : null];

            protoIDs["FishingGame.GetFishGameInfoReq"]        = [10701, WxConfigMgr.ProtoEnable ? FishingGame.GetFishGameInfoReq : null];
            protoIDs["FishingGame.PlayFishGameReq"]           = [10702, WxConfigMgr.ProtoEnable ? FishingGame.PlayFishGameReq : null];
            protoIDs["FishingGame.GetFishingFishInfoReq"]     = [10703, WxConfigMgr.ProtoEnable ? FishingGame.GetFishingFishInfoReq : null];
            protoIDs["FishingGame.AdFreeFishingReq"]     = [10705, WxConfigMgr.ProtoEnable ? FishingGame.AdFreeFishingReq : null];


            protoIDs["Home.SetPetStatusReq"]           = [10901, WxConfigMgr.ProtoEnable ? Home.SetPetStatusReq : null];
            protoIDs["Home.CuteInteractReq"]           = [10902, WxConfigMgr.ProtoEnable ? Home.CuteInteractReq : null];
            protoIDs["Home.CommonInteractReq"]         = [10903, WxConfigMgr.ProtoEnable ? Home.CommonInteractReq : null];
            protoIDs["Home.FetchPhotoWallReq"]         = [10904, WxConfigMgr.ProtoEnable ? Home.FetchPhotoWallReq : null];
            protoIDs["Home.FetchHomeInfoReq"]          = [10905, WxConfigMgr.ProtoEnable ? Home.FetchHomeInfoReq : null];
            protoIDs["Home.HealthyInteractReq"]          = [10906, WxConfigMgr.ProtoEnable ? Home.HealthyInteractReq : null];

            protoIDs["Mail.FetchAllMailsReq"]          = [11001, WxConfigMgr.ProtoEnable ? Mail.FetchAllMailsReq : null];
            protoIDs["Mail.ReadMailReq"]          = [11002, WxConfigMgr.ProtoEnable ? Mail.ReadMailReq : null];
            protoIDs["Mail.DeleteMailReq"]          = [11003, WxConfigMgr.ProtoEnable ? Mail.DeleteMailReq : null];
            protoIDs["Mail.DeleteReadMailsReq"]          = [11004, WxConfigMgr.ProtoEnable ? Mail.DeleteReadMailsReq : null];
            protoIDs["Mail.ReceiveMailRewardsReq"]          = [11005, WxConfigMgr.ProtoEnable ? Mail.ReceiveMailRewardsReq : null];

            protoIDs["Event.FireEventReq"]          = [20001, WxConfigMgr.ProtoEnable ? Event.FireEventReq : null];

            protoIDs["Entrust.GetEntrustReq"]          = [11101, WxConfigMgr.ProtoEnable ? Entrust.GetEntrustReq : null];
            protoIDs["Entrust.AcceptEntrustReq"]          = [11102, WxConfigMgr.ProtoEnable ? Entrust.AcceptEntrustReq : null];
            protoIDs["Entrust.ReceiveRewardsReq"]          = [11103, WxConfigMgr.ProtoEnable ? Entrust.ReceiveRewardsReq : null];

            protoIDs["Suggest.SendSuggestReq"]          = [11201, WxConfigMgr.ProtoEnable ? Suggest.SendSuggestReq : null];


            protoIDs["Balloon.TouchBalloonReq"]          = [11501, WxConfigMgr.ProtoEnable ? Balloon.TouchBalloonReq : null];
            protoIDs["Balloon.GetBalloonInfoReq"]          = [11502, WxConfigMgr.ProtoEnable ? Balloon.GetBalloonInfoReq : null];

            protoIDs["MiniGames.GetMiniGameReq"]          = [11301, WxConfigMgr.ProtoEnable ? MiniGames.GetMiniGameReq : null];
            protoIDs["MiniGames.FinishMiniGameReq"]          = [11302, WxConfigMgr.ProtoEnable ? MiniGames.FinishMiniGameReq : null];
            protoIDs["MiniGames.ClientTriggerMiniGameReq"]         = [11303, WxConfigMgr.ProtoEnable ? MiniGames.ClientTriggerMiniGameReq : null];
            protoIDs["MiniGames.GiveUpMiniGameReq"]         = [11304, WxConfigMgr.ProtoEnable ? MiniGames.GiveUpMiniGameReq : null];

            protoIDs["LuckyBag.QueryLuckyBagInfoReq"]          = [11601, WxConfigMgr.ProtoEnable ? LuckyBag.QueryLuckyBagInfoReq : null];
            protoIDs["LuckyBag.ReceiveLuckyBagReq"]          = [11602, WxConfigMgr.ProtoEnable ? LuckyBag.ReceiveLuckyBagReq : null];

            protoIDs["Staff.GetStaffSkillGroupReq"]          = [11701, WxConfigMgr.ProtoEnable ? Staff.GetStaffSkillGroupReq : null];
            protoIDs["Staff.SkillLevelUpReq"]          = [11702, WxConfigMgr.ProtoEnable ? Staff.SkillLevelUpReq : null];
            protoIDs["Staff.StaffLevelUpReq"]          = [11703, WxConfigMgr.ProtoEnable ? Staff.StaffLevelUpReq : null];
            protoIDs["Staff.SelectSkillReq"]          = [11704, WxConfigMgr.ProtoEnable ? Staff.SelectSkillReq : null];
            protoIDs["Staff.GetStaffGroupReq"]          = [11705, WxConfigMgr.ProtoEnable ? Staff.GetStaffGroupReq : null];
            protoIDs["Staff.ReduceCDTimeReq"]          = [11706, WxConfigMgr.ProtoEnable ? Staff.ReduceCDTimeReq : null];

            //签到
            protoIDs["Sign.GetSignInfoReq"] =                    [11801, WxConfigMgr.ProtoEnable ? Sign.GetSignInfoReq : null];
            protoIDs["Sign.GetSignRewardReq"] =                    [11802, WxConfigMgr.ProtoEnable ? Sign.GetSignRewardReq : null];

            //游戏大厅助力
            protoIDs["Wharf.GetWharfInfoReq"]                   = [11901, WxConfigMgr.ProtoEnable ? Wharf.GetWharfInfoReq : null];// 获取一个总的场景信息
            protoIDs["Wharf.GetNauticalProjectGroupReq"]         = [11902, WxConfigMgr.ProtoEnable ? Wharf.GetNauticalProjectGroupReq : null];// 获取建筑信息
            protoIDs["Wharf.ProjectLevelUpReq"]                   = [11903, WxConfigMgr.ProtoEnable ? Wharf.ProjectLevelUpReq : null];// 升级游乐项目
            protoIDs["Wharf.GetWharfLetterBoxReq"]                   = [11904, WxConfigMgr.ProtoEnable ? Wharf.GetWharfLetterBoxReq : null];// 查看挂机奖励及助力信息
            protoIDs["Wharf.FetchWharfLetterBoxReq"]                   = [11905, WxConfigMgr.ProtoEnable ? Wharf.FetchWharfLetterBoxReq : null];//  领取挂机奖励
            protoIDs["Wharf.AidPlayerReq"]                   = [11906, WxConfigMgr.ProtoEnable ? Wharf.AidPlayerReq : null];//  助力
            protoIDs["Wharf.ReceiveAidSalaryReq"]                   = [11907, WxConfigMgr.ProtoEnable ? Wharf.ReceiveAidSalaryReq : null];// 领取助力奖励
            protoIDs["Wharf.FetchDailyShellReq"]                   = [11908, WxConfigMgr.ProtoEnable ? Wharf.FetchDailyShellReq : null];// 领取每日贝壳

            protoIDs["Debug.DebugReq"] = [90001, WxConfigMgr.ProtoEnable ? Debug.DebugReq : null];
            protoIDs["Sign.GetCollectGiftInfoReq"] = [12001, WxConfigMgr.ProtoEnable ? Collect.GetCollectGiftInfoReq : null];
            protoIDs["Milestone.FetchMilestoneRewardReq"] = [12101, WxConfigMgr.ProtoEnable ? Milestone.FetchMilestoneRewardReq : null];
            protoIDs["LuckyDraw.FetchLuckyDrawRewardReq"] = [12201, WxConfigMgr.ProtoEnable ? LuckyDraw.FetchLuckyDrawRewardReq : null];
            protoIDs["LuckyDraw.GetLuckyDrawInfoReq"] = [12202, WxConfigMgr.ProtoEnable ? LuckyDraw.GetLuckyDrawInfoReq : null];
            protoIDs["LuckyDraw.FetchBuildingTicketRewardReq"] = [12203, WxConfigMgr.ProtoEnable ? LuckyDraw.FetchBuildingTicketRewardReq : null];
            protoIDs["LuckyDraw.FinishBuildingCouponReq"] = [12204, WxConfigMgr.ProtoEnable ? LuckyDraw.FinishBuildingCouponReq : null];
            protoIDs["LuckyDraw.UseAdCouponReq"] = [12205, WxConfigMgr.ProtoEnable ? LuckyDraw.UseAdCouponReq : null];
            protoIDs["LuckyDraw.GetLotteryBagReq"] = [12206, WxConfigMgr.ProtoEnable ? LuckyDraw.GetLotteryBagReq : null];
            
            //万事屋
            protoIDs["CatMission.GetCatMissionInfoReq"] = [12501, WxConfigMgr.ProtoEnable ? CatMission.GetCatMissionInfoReq : null];
            protoIDs["CatMission.PlayerRefreshReq"] = [12502, WxConfigMgr.ProtoEnable ? CatMission.PlayerRefreshReq : null];
            protoIDs["CatMission.ChangeRefreshLevelReq"] = [12503, WxConfigMgr.ProtoEnable ? CatMission.ChangeRefreshLevelReq : null];
            protoIDs["CatMission.ReceiveMissionRewardReq"] = [12504, WxConfigMgr.ProtoEnable ? CatMission.ReceiveMissionRewardReq : null];
            protoIDs["CatMission.ResetCatStatusReq"] = [12505, WxConfigMgr.ProtoEnable ? CatMission.ResetCatStatusReq  : null];
            protoIDs["CatMission.AdRefreshMissionReq"] = [12506, WxConfigMgr.ProtoEnable ? CatMission.AdRefreshMissionReq  : null];

            //内购
            protoIDs["Pay.CreateOrderReq"] = [12601, WxConfigMgr.ProtoEnable ? Pay.CreateOrderReq : null];
            protoIDs["Pay.FinishOrderReq"] = [12602, WxConfigMgr.ProtoEnable ? Pay.FinishOrderReq : null];
            protoIDs["Pay.ReceiveFreeDiamondReq"] = [12603, WxConfigMgr.ProtoEnable ? Pay.ReceiveFreeDiamondReq : null];
            protoIDs["Pay.PayCardBonusReq"] = [12604, WxConfigMgr.ProtoEnable ? Pay.PayCardBonusReq : null];

            //资源购买
            protoIDs["Shop.ResourceShopReq"] = [12701, WxConfigMgr.ProtoEnable ? Shop.ResourceShopReq : null];

            protoIDs["Takeout.FetchTakeoutInfoReq"] = [12801, WxConfigMgr.ProtoEnable ? Takeout.FetchTakeoutInfoReq : null];
            protoIDs["Takeout.DeliveryReq"] = [12802, WxConfigMgr.ProtoEnable ? Takeout.DeliveryReq : null];
            protoIDs["Takeout.ReceiveRewardsReq"] = [12803, WxConfigMgr.ProtoEnable ? Takeout.ReceiveRewardsReq : null];
            protoIDs["Takeout.SpeedUpDeliveryReq"] = [12804, WxConfigMgr.ProtoEnable ? Takeout.SpeedUpDeliveryReq : null];
            protoIDs["Takeout.LeveUpTakeoutReq"] = [12805, WxConfigMgr.ProtoEnable ? Takeout.LeveUpTakeoutReq : null];
            protoIDs["Takeout.PutAdvertiserCatsReq"] = [12806, WxConfigMgr.ProtoEnable ? Takeout.PutAdvertiserCatsReq : null];
            protoIDs["Takeout.BuyOrderReq"] = [12807, WxConfigMgr.ProtoEnable ? Takeout.BuyOrderReq : null];


            return protoIDs;
        }

        static getResponseMap() {
            var protoIDs = {};
            protoIDs[10001] = ["Player.LoginRes", WxConfigMgr.ProtoEnable ? Player.LoginRes : null];
            protoIDs[10002] = ["Player.LogoutRes", WxConfigMgr.ProtoEnable ? Player.LogoutRes : null];
            protoIDs[10003] = ["Player.OnlineCrossDayRes", WxConfigMgr.ProtoEnable ? Player.OnlineCrossDayRes : null];
            protoIDs[10004] = ["Player.FullBagRes", WxConfigMgr.ProtoEnable ? Player.FullBagRes : null];
            protoIDs[10005] = ["Player.PutCustomDataRes", WxConfigMgr.ProtoEnable ? Player.PutCustomDataRes : null];
            protoIDs[10006] = ["Player.DelCustomDataRes", WxConfigMgr.ProtoEnable ? Player.DelCustomDataRes : null];
            protoIDs[10007] = ["Player.GetCustomDataRes", WxConfigMgr.ProtoEnable ? Player.GetCustomDataRes : null];
            protoIDs[10008] = ["Player.ViewAdRes", WxConfigMgr.ProtoEnable ? Player.ViewAdRes : null];
            protoIDs[10009] = ["Player.AuthorizeRes", WxConfigMgr.ProtoEnable ? Player.AuthorizeRes : null];
            protoIDs[10010] = ["Player.ClientLoadingLogRes", WxConfigMgr.ProtoEnable ? Player.ClientLoadingLogRes : null];
            protoIDs[10011] = ["Player.ShareRes", WxConfigMgr.ProtoEnable ? Player.ShareRes : null];

            protoIDs[10101] = ["Building.GetAllBuildingRes", WxConfigMgr.ProtoEnable ? Building.GetAllBuildingRes : null];
            protoIDs[10104] = ["Building.PutBuildingWithAdRes", WxConfigMgr.ProtoEnable ? Building.PutBuildingWithAdRes : null];
            protoIDs[10103] = ["Building.RemoveBuildingRes", WxConfigMgr.ProtoEnable ? Building.RemoveBuildingRes : null];
            protoIDs[10105] = ["Building.BargainRes", WxConfigMgr.ProtoEnable ? Building.BargainRes : null];

            protoIDs[10201] = ["Scene.UpdateSceneInfoRes", WxConfigMgr.ProtoEnable ? Scene.UpdateSceneInfoRes : null];
            protoIDs[10202] = ["Scene.FetchLetterBoxRes", WxConfigMgr.ProtoEnable ? Scene.FetchLetterBoxRes : null];
            protoIDs[10203] = ["Scene.GetLetterBoxInfoRes", WxConfigMgr.ProtoEnable ? Scene.GetLetterBoxInfoRes : null];
            protoIDs[10204] = ["Scene.FindVisitorGiftRes", WxConfigMgr.ProtoEnable ? Scene.FindVisitorGiftRes : null];
            protoIDs[10205] = ["Scene.GetVisitorGiftAllRes", WxConfigMgr.ProtoEnable ? Scene.GetVisitorGiftAllRes : null];
            protoIDs[10208] = ["Scene.GetVisitorGiftSingleRes", WxConfigMgr.ProtoEnable ? Scene.GetVisitorGiftSingleRes : null];
            protoIDs[10209] = ["Scene.CatFoodSubscribeRes", WxConfigMgr.ProtoEnable ? Scene.CatFoodSubscribeRes : null];

            protoIDs[10301] = ["Task.GetAllTasksRes", WxConfigMgr.ProtoEnable ? Task.GetAllTasksRes : null];
            protoIDs[10304] = ["Task.getDailyTaskRewardWithAdRes", WxConfigMgr.ProtoEnable ? Task.getDailyTaskRewardWithAdRes : null];
            protoIDs[10305] = ["Task.getTrophyTaskRewardWithAdRes", WxConfigMgr.ProtoEnable ? Task.getTrophyTaskRewardWithAdRes : null];
            protoIDs[10306] = ["Task.getMainTaskRewardRes", WxConfigMgr.ProtoEnable ? Task.getMainTaskRewardRes : null];

            protoIDs[10401] = ["Food.GetPlayerFoodMenuRes", WxConfigMgr.ProtoEnable ? Food.GetPlayerFoodMenuRes : null];
            protoIDs[10403] = ["Food.FoodLevelUpWithAdRes", WxConfigMgr.ProtoEnable ? Food.FoodLevelUpWithAdRes : null];

            protoIDs[10501] = ["Pet.GetPlayerPetsInfoRes", WxConfigMgr.ProtoEnable ? Pet.GetPlayerPetsInfoRes : null];
            protoIDs[10502] = ["Pet.AdoptPetRes", WxConfigMgr.ProtoEnable ? Pet.AdoptPetRes : null];
            protoIDs[10503] = ["Pet.StoryUnlockRes", WxConfigMgr.ProtoEnable ? Pet.StoryUnlockRes : null];
            protoIDs[10504] = ["Pet.ShareGetRewardRes", WxConfigMgr.ProtoEnable ? Pet.ShareGetRewardRes : null];
            protoIDs[10505] = ["Pet.CatLevelUpRes", WxConfigMgr.ProtoEnable ? Pet.CatLevelUpRes : null];
            protoIDs[10506] = ["Pet.CatStarUpGradeRes", WxConfigMgr.ProtoEnable ? Pet.CatStarUpGradeRes : null];

            // protoIDs[10601] = ["Nautical.GetNauticalInfoRes", WxConfigMgr.ProtoEnable ? Nautical.GetNauticalInfoRes : null];
            protoIDs[10602] = ["Nautical.MergeNauticalFoodRes", WxConfigMgr.ProtoEnable ? Nautical.MergeNauticalFoodRes : null];
            // protoIDs[10603] = ["Nautical.StartNauticalTripRes", WxConfigMgr.ProtoEnable ? Nautical.StartNauticalTripRes : null];
            // protoIDs[10604] = ["Nautical.FinishNauticalTripRes", WxConfigMgr.ProtoEnable ? Nautical.FinishNauticalTripRes : null];
            // protoIDs[10605] = ["Nautical.PreviewNauticalTripRes", WxConfigMgr.ProtoEnable ? Nautical.PreviewNauticalTripRes : null];
            // protoIDs[10606] = ["Nautical.AdReduceNauticalTimeRes", WxConfigMgr.ProtoEnable ? Nautical.AdReduceNauticalTimeRes : null];

            protoIDs[10607] = ["Nautical.GetNauticalInfo0912Res", WxConfigMgr.ProtoEnable ? Nautical.GetNauticalInfo0912Res : null];
            protoIDs[10608] = ["Nautical.StartNauticalTrip0912Res", WxConfigMgr.ProtoEnable ? Nautical.StartNauticalTrip0912Res : null];
            protoIDs[10609] = ["Nautical.FinishNauticalTrip0912Res", WxConfigMgr.ProtoEnable ? Nautical.FinishNauticalTrip0912Res : null];
            protoIDs[10610] = ["Nautical.PreviewNauticalTrip0912Res", WxConfigMgr.ProtoEnable ? Nautical.PreviewNauticalTrip0912Res : null];
            protoIDs[10611] = ["Nautical.AdReduceNauticalTime0912Res", WxConfigMgr.ProtoEnable ? Nautical.AdReduceNauticalTime0912Res : null];
            protoIDs[10612] = ["Nautical.unlockNauticalRes", WxConfigMgr.ProtoEnable ? Nautical.unlockNauticalRes : null];

            protoIDs[10701] = ["FishingGame.GetFishGameInfoRes", WxConfigMgr.ProtoEnable ? FishingGame.GetFishGameInfoRes : null];
            protoIDs[10702] = ["FishingGame.PlayFishGameRes", WxConfigMgr.ProtoEnable ? FishingGame.PlayFishGameRes : null];
            protoIDs[10703] = ["FishingGame.GetFishingFishInfoRes", WxConfigMgr.ProtoEnable ? FishingGame.GetFishingFishInfoRes : null];
            protoIDs[10705] = ["FishingGame.AdFreeFishingRes", WxConfigMgr.ProtoEnable ? FishingGame.AdFreeFishingRes : null];


            protoIDs[10901] = ["Home.SetPetStatusRes", WxConfigMgr.ProtoEnable ? Home.SetPetStatusRes : null];
            protoIDs[10902] = ["Home.CuteInteractRes", WxConfigMgr.ProtoEnable ? Home.CuteInteractRes : null];
            protoIDs[10903] = ["Home.CommonInteractRes", WxConfigMgr.ProtoEnable ? Home.CommonInteractRes : null];
            protoIDs[10904] = ["Home.FetchPhotoWallRes", WxConfigMgr.ProtoEnable ? Home.FetchPhotoWallRes : null];
            protoIDs[10905] = ["Home.FetchHomeInfoRes", WxConfigMgr.ProtoEnable ? Home.FetchHomeInfoRes : null];
            protoIDs[10906] = ["Home.HealthyInteractRes", WxConfigMgr.ProtoEnable ? Home.HealthyInteractRes : null];

            protoIDs[11001] = ["Mail.FetchAllMailsRes", WxConfigMgr.ProtoEnable ? Mail.FetchAllMailsRes : null];
            protoIDs[11002] = ["Mail.ReadMailRes", WxConfigMgr.ProtoEnable ? Mail.ReadMailRes : null];
            protoIDs[11003] = ["Mail.DeleteMailRes", WxConfigMgr.ProtoEnable ? Mail.DeleteMailRes : null];
            protoIDs[11004] = ["Mail.DeleteReadMailsRes", WxConfigMgr.ProtoEnable ? Mail.DeleteReadMailsRes : null];
            protoIDs[11005] = ["Mail.ReceiveMailRewardsRes", WxConfigMgr.ProtoEnable ? Mail.ReceiveMailRewardsRes : null];

            protoIDs[20001] = ["Event.FireEventRes", WxConfigMgr.ProtoEnable ? Event.FireEventRes : null];

            protoIDs[11101] = ["Entrust.GetEntrustRes", WxConfigMgr.ProtoEnable ? Entrust.GetEntrustRes : null];
            protoIDs[11102] = ["Entrust.AcceptEntrustRes", WxConfigMgr.ProtoEnable ? Entrust.AcceptEntrustRes : null];
            protoIDs[11103] = ["Entrust.ReceiveRewardsRes", WxConfigMgr.ProtoEnable ? Entrust.ReceiveRewardsRes : null];

            protoIDs[11201] = ["Suggest.SendSuggestRes", WxConfigMgr.ProtoEnable ? Suggest.SendSuggestRes : null];


            protoIDs[11501] = ["Balloon.TouchBalloonRes", WxConfigMgr.ProtoEnable ? Balloon.TouchBalloonRes : null];
            protoIDs[11502] = ["Balloon.GetBalloonInfoRes", WxConfigMgr.ProtoEnable ? Balloon.GetBalloonInfoRes : null];

            protoIDs[11301] = ["MiniGames.GetMiniGameRes", WxConfigMgr.ProtoEnable ? MiniGames.GetMiniGameRes : null];
            protoIDs[11302] = ["MiniGames.FinishMiniGameRes", WxConfigMgr.ProtoEnable ? MiniGames.FinishMiniGameRes : null];
            protoIDs[11303] = ["MiniGames.ClientTriggerMiniGameRes", WxConfigMgr.ProtoEnable ? MiniGames.ClientTriggerMiniGameRes : null];
            protoIDs[11304] = ["MiniGames.GiveUpMiniGameRes", WxConfigMgr.ProtoEnable ? MiniGames.GiveUpMiniGameRes : null];

            protoIDs[11601] = ["LuckyBag.QueryLuckyBagInfoRes", WxConfigMgr.ProtoEnable ? LuckyBag.QueryLuckyBagInfoRes : null];
            protoIDs[11602] = ["LuckyBag.ReceiveLuckyBagRes", WxConfigMgr.ProtoEnable ? LuckyBag.ReceiveLuckyBagRes : null];

            protoIDs[11701] = ["Staff.GetStaffSkillGroupRes", WxConfigMgr.ProtoEnable ? Staff.GetStaffSkillGroupRes : null];
            protoIDs[11702] = ["Staff.SkillLevelUpRes", WxConfigMgr.ProtoEnable ? Staff.SkillLevelUpRes : null];
            protoIDs[11703] = ["Staff.StaffLevelUpRes", WxConfigMgr.ProtoEnable ? Staff.StaffLevelUpRes : null];
            protoIDs[11704] = ["Staff.SelectSkillRes", WxConfigMgr.ProtoEnable ? Staff.SelectSkillRes : null];
            protoIDs[11705] = ["Staff.GetStaffGroupRes", WxConfigMgr.ProtoEnable ? Staff.GetStaffGroupRes : null];
            protoIDs[11706] = ["Staff.ReduceCDTimeRes", WxConfigMgr.ProtoEnable ? Staff.ReduceCDTimeRes : null];
            //签到
            protoIDs[11801] = ["Sign.GetSignInfoRes", WxConfigMgr.ProtoEnable ? Sign.GetSignInfoRes : null];
            protoIDs[11802] = ["Sign.GetSignRewardRes", WxConfigMgr.ProtoEnable ? Sign.GetSignRewardRes : null];


            //游戏大厅助力
            protoIDs[11901] = ["Wharf.GetWharfInfoRes ", WxConfigMgr.ProtoEnable ?Wharf.GetWharfInfoRes: null];// 获取一个总的场景信息
            protoIDs[11902] = ["Wharf.GetNauticalProjectGroupRes",WxConfigMgr.ProtoEnable ?Wharf.GetNauticalProjectGroupRes: null];// 获取建筑信息
            protoIDs[11903] = ["Wharf.ProjectLevelUpRes",WxConfigMgr.ProtoEnable ?Wharf.ProjectLevelUpRes: null];// 升级游乐项目
            protoIDs[11904] = ["Wharf.GetWharfLetterBoxRes",WxConfigMgr.ProtoEnable ?Wharf.GetWharfLetterBoxRes: null];// 查看挂机奖励及助力信息
            protoIDs[11905] = ["Wharf.FetchWharfLetterBoxRes", WxConfigMgr.ProtoEnable ?Wharf.FetchWharfLetterBoxRes: null];//  领取挂机奖励
            protoIDs[11906] = ["Wharf.AidPlayerRes",WxConfigMgr.ProtoEnable ?Wharf.AidPlayerRes: null];//  助力
            protoIDs[11907] = ["Wharf.ReceiveAidSalaryRes ",WxConfigMgr.ProtoEnable ?Wharf.ReceiveAidSalaryRes : null];//  领取助力奖励
            protoIDs[11908] = ["Wharf.FetchDailyShellRes",WxConfigMgr.ProtoEnable ?Wharf.FetchDailyShellRes : null];//  领取每日贝壳奖励

     
    //added by tenkin 
            protoIDs[12001] = ["Collect.GetCollectGiftInfoRes", WxConfigMgr.ProtoEnable ?Collect.GetCollectGiftInfoRes: null];
            protoIDs[12101] = ["Milestone.FetchMilestoneRewardRes",WxConfigMgr.ProtoEnable ? Milestone.FetchMilestoneRewardRes: null ];
            protoIDs[12201] = ["LuckyDraw.FetchLuckyDrawRewardRes", WxConfigMgr.ProtoEnable ?LuckyDraw.FetchLuckyDrawRewardRes: null];
            protoIDs[12202] = ["LuckyDraw.yDrGetLuckawInfoRes", WxConfigMgr.ProtoEnable ?LuckyDraw.GetLuckyDrawInfoRes : null];
            protoIDs[12203] = ["LuckyDraw.FetchBuildingTicketRewardRes", WxConfigMgr.ProtoEnable ?LuckyDraw.FetchBuildingTicketRewardRes : null];
            protoIDs[12204] = ["LuckyDraw.FinishBuildingCouponRes", WxConfigMgr.ProtoEnable ?LuckyDraw.FinishBuildingCouponRes : null];
            protoIDs[12205] = ["LuckyDraw.UseAdCouponRes", WxConfigMgr.ProtoEnable ?LuckyDraw.UseAdCouponRes : null];
            protoIDs[12206] = ["LuckyDraw.GetLotteryBagRes", WxConfigMgr.ProtoEnable ?LuckyDraw.GetLotteryBagRes : null];
        
            //万事屋
            protoIDs[12501] = ["CatMission.GetCatMissionInfoRes", WxConfigMgr.ProtoEnable ? CatMission.GetCatMissionInfoRes : null];
            protoIDs[12502] = ["CatMission.PlayerRefreshRes", WxConfigMgr.ProtoEnable ? CatMission.PlayerRefreshRes : null];
            protoIDs[12503] = ["CatMission.ChangeRefreshLevelRes", WxConfigMgr.ProtoEnable ? CatMission.ChangeRefreshLevelRes : null];
            protoIDs[12504] = ["CatMission.ReceiveMissionRewardRes", WxConfigMgr.ProtoEnable ? CatMission.ReceiveMissionRewardRes : null];
            protoIDs[12505] = ["CatMission.ResetCatStatusRes", WxConfigMgr.ProtoEnable ? CatMission.ResetCatStatusRes  : null];
            protoIDs[12506] = ["CatMission.AdRefreshMissionRes", WxConfigMgr.ProtoEnable ? CatMission.AdRefreshMissionRes  : null];

            protoIDs[12601] = ["Pay.CreateOrderRes", WxConfigMgr.ProtoEnable ? Pay.CreateOrderRes : null];
            protoIDs[12602] = ["Pay.FinishOrderRes", WxConfigMgr.ProtoEnable ? Pay.FinishOrderRes : null];
            protoIDs[12603] = ["Pay.ReceiveFreeDiamondRes", WxConfigMgr.ProtoEnable ? Pay.ReceiveFreeDiamondRes : null];
            protoIDs[12604] = ["Pay.PayCardBonusRes", WxConfigMgr.ProtoEnable ? Pay.PayCardBonusRes : null];

            protoIDs[12701] = ["Shop.ResourceShopRes", WxConfigMgr.ProtoEnable ? Shop.ResourceShopRes : null];

            protoIDs[12801] = ["Takeout.FetchTakeoutInfoRes", WxConfigMgr.ProtoEnable ? Takeout.FetchTakeoutInfoRes : null];
            protoIDs[12802] = ["Takeout.DeliveryRes", WxConfigMgr.ProtoEnable ? Takeout.DeliveryRes  : null];        
            protoIDs[12803] = ["Takeout.ReceiveRewardsRes", WxConfigMgr.ProtoEnable ? Takeout.ReceiveRewardsRes  : null];
            protoIDs[12804] = ["Takeout.SpeedUpDeliveryRes", WxConfigMgr.ProtoEnable ? Takeout.SpeedUpDeliveryRes  : null];
            protoIDs[12805] = ["Takeout.LevelUpTakeoutRes", WxConfigMgr.ProtoEnable ? Takeout.LevelUpTakeoutRes  : null];
            protoIDs[12806] = ["Takeout.PutAdvertiserCatsRes", WxConfigMgr.ProtoEnable ? Takeout.PutAdvertiserCatsRes  : null];
            protoIDs[12807] = ["Takeout.BuyOrderRes", WxConfigMgr.ProtoEnable ? Takeout.BuyOrderRes  : null];

            
            protoIDs[90001] = ["Debug.DebugRes", WxConfigMgr.ProtoEnable ? Debug.DebugRes : null];

            return protoIDs;
        }

        static getNotifyProtoMap() {
            var protoIDs = {};
            protoIDs[1001] = ["PlayerPushMsg.TestMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.TestMsg : null];
            protoIDs[1002] = ["PlayerPushMsg.ReddotMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.ReddotMsg : null];
            protoIDs[1003] = ["PlayerPushMsg.OnlineCrossDayMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.OnlineCrossDayMsg : null];
            protoIDs[1005] = ["PlayerPushMsg.MailMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.MailMsg : null];
            protoIDs[1006] = ["PlayerPushMsg.EntrustMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.EntrustMsg : null];
            protoIDs[1007] = ["PlayerPushMsg.MiniGameMsg", WxConfigMgr.ProtoEnable ? PlayerPushMsg.MiniGameMsg : null];
            return protoIDs;
        }
    }

    class NetUtils {

        //将字符串转换为字节数组
        static stringToByte(str) {
            var bytes = new Array();
            var len, c;
            len = str.length;
            for(var i = 0; i < len; i++) {
                c = str.charCodeAt(i);
                if(c >= 0x010000 && c <= 0x10FFFF) {
                    bytes.push(((c >> 18) & 0x07) | 0xF0);
                    bytes.push(((c >> 12) & 0x3F) | 0x80);
                    bytes.push(((c >> 6) & 0x3F) | 0x80);
                    bytes.push((c & 0x3F) | 0x80);
                } else if(c >= 0x000800 && c <= 0x00FFFF) {
                    bytes.push(((c >> 12) & 0x0F) | 0xE0);
                    bytes.push(((c >> 6) & 0x3F) | 0x80);
                    bytes.push((c & 0x3F) | 0x80);
                } else if(c >= 0x000080 && c <= 0x0007FF) {
                    bytes.push(((c >> 6) & 0x1F) | 0xC0);
                    bytes.push((c & 0x3F) | 0x80);
                } else {
                    bytes.push(c & 0xFF);
                }
            }
            return bytes;
        }

        //将字节数组转换为字符串
        static byteToString(arr) {
            if(typeof arr === 'string') {
                return arr;
            }
            var str = '',
                _arr = arr;
            for(var i = 0; i < _arr.length; i++) {
                var one = _arr[i].toString(2),
                    v = one.match(/^1+?(?=0)/);
                if(v && one.length == 8) {
                    var bytesLength = v[0].length;
                    var store = _arr[i].toString(2).slice(7 - bytesLength);
                    for(var st = 1; st < bytesLength; st++) {
                        store += _arr[st + i].toString(2).slice(2);
                    }
                    str += String.fromCharCode(parseInt(store, 2));
                    i += bytesLength - 1;
                } else {
                    str += String.fromCharCode(_arr[i]);
                }
            }
            return str;
        }

        static bytesToHexString(byteArray) {
    		return Array.from(byteArray, function(byte) {
    		  return ('0' + (byte & 0xFF).toString(16)).slice(-2);
    		}).join('')
    	}

    	static hexString2Bytes(str) {
    		var pos = 0;
    		var len = str.length;
    		if (len % 2 != 0) {
    		  return null;
    		}
    		len /= 2;
    		var hexA = new Array();
    		for (var i = 0; i < len; i++) {
    		  var s = str.substr(pos, 2);
    		  var v = parseInt(s, 16);
    		  hexA.push(v);
    		  pos += 2;
    		}
    		return hexA;
    	  }

    	static base64ToBytes(base64) {
    		var base64map = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    		base64 = base64.replace(/[^A-Z0-9+\/]/ig, "");
    		for (var bytes = [], i = 0, imod4 = 0; i < base64.length; imod4 = ++i % 4) {
    			if (imod4 == 0) continue;
    			bytes.push(((base64map.indexOf(base64.charAt(i - 1)) & (Math.pow(2, -2 * imod4 + 8) - 1)) << (imod4 * 2)) |
    					   (base64map.indexOf(base64.charAt(i)) >>> (6 - imod4 * 2)));
    		}
    		return bytes;
        }

        static base64_encode(str) {
            var c1, c2, c3;
            var base64EncodeChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
            var i = 0, len = str.length, string = '';
          
            while (i < len) {
              c1 = str.charCodeAt(i++) & 0xff;
              if (i == len) {
                string += base64EncodeChars.charAt(c1 >> 2);
                string += base64EncodeChars.charAt((c1 & 0x3) << 4);
                string += "==";
                break;
              }
              c2 = str.charCodeAt(i++);
              if (i == len) {
                string += base64EncodeChars.charAt(c1 >> 2);
                string += base64EncodeChars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
                string += base64EncodeChars.charAt((c2 & 0xF) << 2);
                string += "=";
                break;
              }
              c3 = str.charCodeAt(i++);
              string += base64EncodeChars.charAt(c1 >> 2);
              string += base64EncodeChars.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
              string += base64EncodeChars.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
              string += base64EncodeChars.charAt(c3 & 0x3F);
            }
            return string
          }
        
        static byteToBase64String( buffer ) {
            var binary = '';
            var bytes = new Uint8Array( buffer );
            var len = bytes.byteLength;
            for (var i = 0; i < len; i++) {
                binary += String.fromCharCode( bytes[ i ] );
            }
            return NetUtils.base64_encode( binary );
        }

    	static Int8parse(u8arr) {
    		var len = u8arr.length;
      
    		var words = [];
    		for (var i = 0; i < len; i++) {
    			words[i >>> 2] |= (u8arr[i] & 0xff) << (24 - (i % 4) * 8);
    		}
      
    		return CryptoJS.lib.WordArray.create(words, len);
        }
        
        /**
         * important important important ！！！
         * JSEncrypt 库的 rsa 加密有点问题，源代码中，对于 byte[] 类型数据进行解密的时候，会默认按照 pkcs1unpad2 的格式进行转换 string
         * 但是这种字符串无法再转回来，所以修改了 JSEncrypt 的源代码，在 decrypt 的函数中，直接返回解密的bitInteger，业务中再将 bigInteger 转为 byte[]
         * @param {*} d 
         * @param {*} n 
         */
        static rsaIntToBytes(d, n) {
            var b = d.toByteArray();
            var i = 0;
            while (i < b.length && b[i] == 0) {
                ++i;
            }
            if (b.length - i != n - 1 || b[i] != 2) {
                return null;
            }
            ++i;
            while (b[i] != 0) {
                if (++i >= b.length) {
                    return null;
                }
            }
            var ret = [];
            while (++i < b.length) {
                var c = b[i] & 255;
                ret.push(c);
            }
            return ret;
        }

        //生成一个长度为 16 的随机字节数组
        static createRandom16Bytes() {
            var ret = new Int8Array(16);
            for(var i = 0; i < 16; i++) {
                var random = Math.floor(65536 * Math.random());
                ret[i] = random & 255;
            }
            return ret;
        }

        //将 d 的数据量填充到 16 的倍数
        static paddingBytesTo16Times(d) {
            //d可能是 Array 可能是 ArrayBuffer，两者的拷贝方法不同
            var dLength = (d instanceof Array) ? d.length : d.byteLength;
            if(dLength % 16 == 0) {
                return new Int8Array(d);
            } else {
                var ret = new Int8Array(dLength + 16 - (dLength % 16));
                if(d instanceof Array) {
                    for(var i = 0; i < dLength; i++) {
                        ret[i] = d[i];
                    }
                } else {
                    ret.set(new Int8Array(d), 0);
                }
                for(var i = 0; i < 16 - (dLength % 16); i++) {
                    ret[dLength + i] = 0;
                }
                return ret;
            }
        }

        static aesEncrypt(originData, aesKey) {
            //TODO 先不加密，但是格式上按照 crc + originDataLenght + iv + encryptData 的格式来

            var ba = new Laya.Byte();
            ba.endian = Laya.Socket.BIG_ENDIAN;
            var crc = CRC32.buf(new Uint8Array(originData));
            if (WxConfigMgr.NetEncrypt) {
                var dataArray = NetUtils.paddingBytesTo16Times(originData);
                var randomIv = NetUtils.createRandom16Bytes();
                var encryptedData = CryptoJS.AES.encrypt(NetUtils.Int8parse(dataArray), aesKey, { iv: CryptoJS.lib.WordArray.create(randomIv, 16), mode: CryptoJS.mode.CBC, padding: CryptoJS.pad.NoPadding });
                var finalEncryptedData = NetUtils.base64ToBytes(encryptedData.toString());
            }

            ba.writeInt32(crc);                 //crc
            ba.writeInt32(originData.byteLength);   //原始数据长度
            if (WxConfigMgr.NetEncrypt) {
                ba.writeArrayBuffer(randomIv, 0, randomIv.length);  //iv向量
                ba.writeArrayBuffer(finalEncryptedData, 0, finalEncryptedData.length);    //加密后的数据
            } else {
                ba.writeArrayBuffer(originData, 0, originData.length);
            }

            return ba.buffer;
        }

        //AES 解密，返回解密后的内容，并且校验 CRC 是否对
        static aesDecrypt(crc, originDataLength, iv, encryptData, aesKey) {
            let decryptedData = null;
            if(WxConfigMgr.NetEncrypt) {
                let bytes  = CryptoJS.AES.decrypt(NetUtils.byteToBase64String(new Int8Array(encryptData)), aesKey, { iv: CryptoJS.lib.WordArray.create(new Int8Array(iv), 16), mode: CryptoJS.mode.CBC, padding: CryptoJS.pad.NoPadding });
                decryptedData = NetUtils.hexString2Bytes(bytes.toString());
            } else {
                decryptedData = new Int8Array(encryptData);
            }
            
            var ba = new Laya.Byte(decryptedData);
            var realData = ba.readArrayBuffer(originDataLength);

            var clientCrc = CRC32.buf(new Uint8Array(realData));
            if(clientCrc != crc) {
                //TODO 这里应该抛出异常，先只记录日志
                return null;
            }
            return realData;
        }
    }

    class ProtoManager {
        constructor() {
            this._requestProtoIDs = ProtoIds.getRequestMap();
            this._responseProtoIDs = ProtoIds.getResponseMap();
            this._notifyProtoIDs = ProtoIds.getNotifyProtoMap();

            this._loadedHandler = null;

            this._eventDispatcher = new Laya.EventDispatcher();
        }

        static getInstance() {
            return ProtoManager.instance ? ProtoManager.instance : ProtoManager.instance = new ProtoManager();
        }

        getOpcode(requestName) {
            return this._requestProtoIDs[requestName][0];
        }

        rawEncode(paramData) {
            return NetUtils.stringToByte(JSON.stringify(paramData));
        }

        rawDecode(paramData) {
            return JSON.parse(NetUtils.byteToString(paramData));
        }

        encodeRequest(requestName, paramData) {
            if (WxConfigMgr.ProtoEnable) {
                var protoMsg = this._requestProtoIDs[requestName][1];
                if (!protoMsg) {
                    console.error("没有找到协议信息: " + requestName);
                    return null;
                }
                var errMsg = protoMsg.verify(paramData);
                if (errMsg) throw Error(errMsg);

                var buffer = protoMsg.encode(paramData).finish();
                return buffer;
            } else {
                return this.rawEncode(paramData);
            }
        }

        decodeRequest(requestName, paramData) {
            if (WxConfigMgr.ProtoEnable) {
                var protoMsg = this._requestProtoIDs[requestName][1];
                if (!protoMsg) {
                    console.error("没有找到协议信息: " + requestName);
                    return null;
                }
                var newMessage = protoMsg.decode(paramData);
                return newMessage;
            } else {
                return this.rawDecode(paramData);
            }
        }

        decodeResponse(opCode, paramData) {
            if (WxConfigMgr.ProtoEnable) {
                var protoMsgName = this._responseProtoIDs[opCode][0];
                if (!protoMsgName) {
                    console.error("没有ID为: " + opCode + " 的协议！");
                    return null;
                }
                var protoMsg = this._responseProtoIDs[opCode][1];
                if (!protoMsg) {
                    console.error("没有找到协议信息: " + protoMsgName);
                    return null;
                }
                var newMessage = protoMsg.decode(paramData);
                return newMessage;
            } else {
                return this.rawDecode(paramData);
            }
        }

        decodeNotify(opCode, paramData) {
            if (WxConfigMgr.ProtoEnable) {
                var protoMsgName = this._notifyProtoIDs[opCode][0];
                if (!protoMsgName) {
                    console.error("没有ID为: " + opCode + " 的协议！");
                    return null;
                }
                var protoMsg = this._notifyProtoIDs[opCode][1];
                if (!protoMsg) {
                    console.error("没有找到协议信息: " + protoMsgName);
                    return null;
                }
                var newMessage = protoMsg.decode(paramData);
                return newMessage;
            } else {
                return this.rawDecode(paramData);
            }
        }

        //TODO 测试时添加的对notify数据进行，正常使用中应该是用不到这个函数的
        encodeNotify(opCode, paramData) {
            if (WxConfigMgr.ProtoEnable) {
                var protoMsgName = this._notifyProtoIDs[opCode][0];
                if (!protoMsgName) {
                    console.error("没有ID为: " + opCode + " 的协议！");
                    return null;
                }
                var protoMsg = this._notifyProtoIDs[opCode][1];
                if (!protoMsg) {
                    console.error("没有找到协议信息: " + protoMsgName);
                    return null;
                }
                var errMsg = protoMsg.verify(paramData);
                if (errMsg) throw Error(errMsg);

                var buffer = protoMsg.encode(paramData).finish();
                return buffer;
            } else {
                return this.rawEncode(paramData);
            }
        }

        registerNotifyCallback(opCode, caller, listener, args) {
            //只允许一个回调存在
            this._eventDispatcher.offAll("NotifyCallback_" + opCode);
            this._eventDispatcher.on("NotifyCallback_" + opCode, caller, listener, args);
        }

        //实际处理收到的服务器推送
        handlerNotify(opCode, params) {
            this._eventDispatcher.event("NotifyCallback_" + opCode, params);
        }
    }

    class WaitingRequest {

        constructor(requestName, param, handler, isSync) {
            this._requestName = requestName;
            this._param = param;
            this._handler = handler;
            this._isSync = isSync;
            this._buffer = null;
            this._msgId = -1;
            this._sendingTs = -1;
        }

        getRequestName() {
            return this._requestName;
        }

        getParam() {
            return this._param;
        }

        getHandler() {
            return this._handler;
        }

        setBuffer(dataBuffer) {
            this._buffer = dataBuffer;
        }

        getBuffer() {
            return this._buffer;
        }

        getIsSync() {
            return this._isSync;
        }

        getMsgId() {
            return this._msgId;
        }

        setMsgId(msgId) {
            this._msgId = msgId;
        }

        getSendingTs() {
            return this._sendingTs;
        }

        setSendingTs(ts) {
            this._sendingTs = ts;
        }
    }

    class NetLoadingMgr {

        constructor() {
            this._loadingIds = new Map();        //存储正在等待loading的元素，loading界面消失的条件是这个map为空
            //用map的原因，是担心特殊情况下出现有两个场景等待loading的时候
        }

        clear() {
            this._loadingIds.clear();
            Laya.timer.clear(this, this.timeout);
            Laya.Dialog.close("ui/LoadingReq.scene");
        }

        static getInstance() {
            return NetLoadingMgr.instance ? NetLoadingMgr.instance : NetLoadingMgr.instance = new NetLoadingMgr();
        }

        ifLoadingIdsEmpty() {
            return this._loadingIds.size == 0;
        }

        getMap() {
            return this._loadingIds.keys();
        }

        //开始loading的id
        beginLoading(loadingId, timeoutSecs) {
            if (this._loadingIds.size == 0) {
                Laya.Dialog.close("ui/LoadingReq.scene");
                Laya.Dialog.open("ui/LoadingReq.scene", false, {});
            }

            if (!this._loadingIds.has(loadingId)) {
                this._loadingIds.set(loadingId, 1);

                //重新定时一个超时的调度, 如果没有传则默认20s
                Laya.timer.clear(this, this.timeout);
                Laya.timer.once(timeoutSecs ? timeoutSecs : 20000, this, this.timeout);
            }
        }

        //某个loadingId已经结束了等待
        endLoading(loadingId) {
            if (this._loadingIds.has(loadingId)) {
                this._loadingIds.delete(loadingId);
                console.log("..... end loading ..... " + loadingId);
                EventMgr.getInstance().postEvent("REQ_LOADING_END", loadingId);
                if (this._loadingIds.size == 0) {
                    Laya.timer.clear(this, this.timeout);
                }
            }
        }

        timeout() {
            console.log("........ loading timeout ........");
            EventMgr.getInstance().postEvent("REQ_LOADING_TIMEOUT");
        }
    }

    class SocketEnv {

        constructor(param) {
            this._rid = param.rid;
            this._token = param.token;
            this._sdkName = param.sdkName;
            this._userInfo = param.userInfo;
            this._loginParam = param;
            this._msgId = 0;
        }

        getUserInfo() {
            return this._userInfo;
        }

        getLoginParam() {
            return this._loginParam;
        }

        getRid() {
            return this._rid;
        }

        getToken() {
            return this._token;
        }

        getSdkName() {
            return this._sdkName;
        }

        setAesKey(aesKey) {
            this._aesKey = aesKey;
        }

        getAesKey() {
            return this._aesKey;
        }

        incrMsgId() {
            this._msgId++;
            return this._msgId;
        }

        getMsgId() {
            return this._msgId;
        }

        setRsaKey(rsaKey) {
            this._rsaKey = rsaKey;
        }

        getRsaKey() {
            return this._rsaKey;
        }

        setRsaDataLength(length) {
            this._rsaDataLength = length;
        }

        getRsaDataLength() {
            return this._rsaDataLength;
        }

    }

    var key = [{"private": "-----BEGIN RSA PRIVATE KEY-----\nMIICWwIBAAKBgQDCdzSHiqD27+n300RHJEzBgcd7/j5jiQKbeJp7mP4+jRKXEdM/\nJkulGWiSkcBdGsjHmLvyRpAE/4C+s6UC3ST79Q73zZQdZIrwnTBYp9crQq8UCNFR\nO1vsovI/COADBcR/hwE7+XrC5nMSADGm89+Bw5+KlJgOEm/iK93S0cuRWQIDAQAB\nAoGAXV71FttmHArnsPl8gBgazpKsoIlblVkUqmeG7S9xa8E6mtSRNHIDwhroDe/z\niACzRLq+Cz78KZowS+Bfq3dZKNXD0rKX3D/QD4NKz1FnxBYxtLqdy4MeBydJrB1p\nnjkwzMjDxkQEiYMtpXCFqP0HJ4cQzu6ITHSZmynQd+96xz0CQQD0X0RPV8YUQe/q\nXThFCeEaKZCmhCjTWBjdalSuCyFwQeqo2PFdKHaVSBI7XE4uzaz/n/0kQRosKcyC\nS6VLK1B/AkEAy7gFHxQJ/xUHzEcw2bfKqsXPCqUqrUSj/1GfuiQl/SoVa6VE0/Y5\naZbnR5YhNPKvEnAl3v0jB4G+hQFBAsayJwJAaUJjxR1Up8MjGTsgErTHriI7OU5x\n2172DuFPKkyHEbiC4EnZ5TuZoIGTwNfooUZcfxslyaj1IhK7EWCOCUvV9wJADx9k\n+e6h41arUoXCYZccK1styGuEZIJg7jYf9lBypX13Q5tsJZP0pRr3pj/FzUlqp6K9\nnPULoaYG/5adPdiEJwJAFuT8dhRClca0lpIEgZIKlfDGHEpq1c9IZlTiVJTDKhYn\n2DJsW9tGp+dyf162jxo5vnIiF23xcmJ5hzaFUaJ7ow==\n-----END RSA PRIVATE KEY-----", "public": "-----BEGIN PUBLIC KEY-----\nMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDCdzSHiqD27+n300RHJEzBgcd7\n/j5jiQKbeJp7mP4+jRKXEdM/JkulGWiSkcBdGsjHmLvyRpAE/4C+s6UC3ST79Q73\nzZQdZIrwnTBYp9crQq8UCNFRO1vsovI/COADBcR/hwE7+XrC5nMSADGm89+Bw5+K\nlJgOEm/iK93S0cuRWQIDAQAB\n-----END PUBLIC KEY-----", "length": 128}];

    class SocketHelper {

        //生成建立连接的二进制数据
        static genConnectData(socketEnv) {
            //发送 Connect 的消息，用于生成秘钥
            let ba = new Laya.Byte();
            ba.endian = Laya.Socket.BIG_ENDIAN;
            let ridBytes = NetUtils.stringToByte(socketEnv.getRid());
            ba.writeByte(ridBytes.length);
            ba.writeArrayBuffer(ridBytes, 0, ridBytes.length);

            //这里生成一个随机密钥, 需要网络加密的时候才进行
            if (WxConfigMgr.NetEncrypt) {
                let rsaKey = new JSEncrypt();
                console.log("------ gen js encryt key ------ " + TimeUtils.getTimeNowOnServer());
                rsaKey.setPublicKey(key[0]["public"]);
                rsaKey.setPrivateKey(key[0]["private"]);
                socketEnv.setRsaKey(rsaKey);

                //原来的写法在 rsaKey.getKey() 时候会去真正生成非对称秘钥，在微信里非常耗时。。改为直接生成好存在配置中读取使用
                // socketEnv.setRsaDataLength((rsaKey.getKey().n.bitLength() + 7) >> 3);
                socketEnv.setRsaDataLength(key[0]["length"]);
                console.log("------ finish gen js encrypt key ------ " + TimeUtils.getTimeNowOnServer());
                let publicKey = NetUtils.base64ToBytes(rsaKey.getPublicKeyB64());
                ba.writeInt16(publicKey.length);                            //秘钥长度
                ba.writeArrayBuffer(publicKey, 0, publicKey.length);        //秘钥内容
            }
            //写最终的发送包
            let finalBa = new Laya.Byte();
            finalBa.endian = Laya.Socket.BIG_ENDIAN;

            finalBa.writeByte(SocketHelper.ConnectMsg);
            finalBa.writeInt32(ba.buffer.byteLength);
            finalBa.writeArrayBuffer(ba.buffer, 0, ba.buffer.byteLength);

            return finalBa;
        }

        static genReconnectData(socketEnv) {
            //表示是在重连, 发送 reconnect 信号
            let ba = new Laya.Byte();
            ba.endian = Laya.Socket.BIG_ENDIAN;
            let ridBytes = NetUtils.stringToByte(socketEnv.getRid());
            ba.writeByte(ridBytes.length);
            ba.writeArrayBuffer(ridBytes, 0, ridBytes.length);

            //进行aes加密
            if (WxConfigMgr.NetEncrypt) {
                let dataArray = NetUtils.paddingBytesTo16Times(ridBytes);
                let randomIv = NetUtils.createRandom16Bytes();
                let encryptedData = CryptoJS.AES.encrypt(NetUtils.Int8parse(dataArray), socketEnv.getAesKey(), { iv: CryptoJS.lib.WordArray.create(randomIv, 16), mode: CryptoJS.mode.CBC, padding: CryptoJS.pad.NoPadding });
                let finalEncryptedData = NetUtils.base64ToBytes(encryptedData.toString());

                ba.writeArrayBuffer(randomIv, 0, randomIv.length);  //iv向量
                ba.writeInt32(ridBytes.length);   //原始rid的长度
                ba.writeArrayBuffer(finalEncryptedData, 0, finalEncryptedData.length);    //加密后的数据
            }

            //写最终的发送包
            let finalBa = new Laya.Byte();
            finalBa.endian = Laya.Socket.BIG_ENDIAN;

            finalBa.writeByte(SocketHelper.ReconnectMsg);
            finalBa.writeInt32(ba.buffer.byteLength);
            finalBa.writeArrayBuffer(ba.buffer, 0, ba.buffer.byteLength);

            return finalBa;
        }

        //将请求的数据进行proto格式化，并且加密，生成最终发送给服务器的二进制数据
        static genRequestData(waitingRequest, socketEnv) {
            let requestName = waitingRequest.getRequestName();
            let param = waitingRequest.getParam();

            let opCode = ProtoManager.getInstance().getOpcode(requestName);
            let data = ProtoManager.getInstance().encodeRequest(requestName, param);

            //先拼内层数据 dataMsgType + msgID + opCode + ridLength + rid + dataLength + data
            let ba = new Laya.Byte();
            ba.endian = Laya.Socket.BIG_ENDIAN;

            ba.writeByte(SocketHelper.RequestData);
            ba.writeInt32(socketEnv.incrMsgId());
            ba.writeInt32(opCode);
            let ridBytes = NetUtils.stringToByte(socketEnv.getRid());
            ba.writeByte(ridBytes.length);
            ba.writeArrayBuffer(ridBytes, 0, ridBytes.length);
            ba.writeInt32(data.length);
            ba.writeArrayBuffer(data, 0, data.length);

            //开始加密
            let encryptData = NetUtils.aesEncrypt(ba.buffer, socketEnv.getAesKey());

            //写最终的发送包
            let finalBa = new Laya.Byte();
            finalBa.endian = Laya.Socket.BIG_ENDIAN;

            finalBa.writeByte(SocketHelper.DataMsg);
            finalBa.writeInt32(encryptData.byteLength);
            finalBa.writeArrayBuffer(encryptData, 0, encryptData.byteLength);

            return finalBa;
        }

        static dealEsblishMsg(msgData, socketEnv) {
            if (WxConfigMgr.NetEncrypt) {
                //先将整个数据用 rsa 解密
                let rsaDecryptedData = socketEnv.getRsaKey().decrypt(NetUtils.byteToBase64String(new Int8Array(msgData)));

                //从解析的数据中拿aes秘钥
                let establishBa = new Laya.Byte(NetUtils.rsaIntToBytes(rsaDecryptedData, socketEnv.getRsaDataLength()));
                establishBa.endian = Laya.Socket.BIG_ENDIAN;

                let keyLength = establishBa.getInt16();
                let originAesKey = establishBa.readArrayBuffer(keyLength);
                socketEnv.setAesKey(NetUtils.Int8parse(new Int8Array(originAesKey)));
            }
        }

        //解密data类型的数据，返回最终的有效数据
        static dealDataMsg(msgData, msgLength, socketEnv) {
            let dataMsgBa = new Laya.Byte(msgData);
            dataMsgBa.endian = Laya.Socket.BIG_ENDIAN;

            //先解密
            let crc = dataMsgBa.getUint32();
            let originDataLength = dataMsgBa.readInt32();
            let encryptData = null;
            let iv = null;
            if (WxConfigMgr.NetEncrypt) {
                iv = dataMsgBa.readArrayBuffer(16);
                encryptData = dataMsgBa.readArrayBuffer(msgLength - 4 - 4 - 16);
            } else {
                encryptData = dataMsgBa.readArrayBuffer(msgLength - 4 - 4);
            }

            let dataBa = new Laya.Byte(NetUtils.aesDecrypt(crc, originDataLength, iv, encryptData, socketEnv.getAesKey()));
            dataBa.endian = Laya.Socket.BIG_ENDIAN;

            return dataBa;
        }

        static genHeartBeatData() {
            let ba = new Laya.Byte();
            ba.endian = Laya.Byte.BIG_ENDIAN;
            ba.writeByte(SocketHelper.HeartbeatMsg);
            ba.writeInt32(0);
            return ba;
        }
    }

    SocketHelper.ConnectMsg = 0x21;
    SocketHelper.DataMsg = 0x22;
    SocketHelper.HeartbeatMsg = 0x23;
    SocketHelper.EstablishMsg = 0x34;
    SocketHelper.ErrorMsg = 0x35;
    SocketHelper.ReconnectMsg = 0x41;

    SocketHelper.RequestData = 0x01;
    SocketHelper.ResponseData = 0x02;
    SocketHelper.NotifyData = 0x03;

    class WebSocketClient {
        constructor() {
            this._heartTimer = new Laya.Timer();
            this._timer = this._heartTimer;
            this._waitSendRequests = new Array();
            this.initSocket();
        }

        static getInstance() {
            return WebSocketClient.instance ? WebSocketClient.instance : WebSocketClient.instance = new WebSocketClient();
        }

        init() {
            this._heartTimer.loop(10000, this, this._onHeartBeat);
        }

        initSocket() {
            this._reconnecting = false;
            this._connectReady = false;
            NetLoadingMgr.getInstance().clear();
            if (this._socket) {
                this._socket.cleanSocket();
            }
            this._heartTimer.clear(this, this.reconnect);
            this._socket = new Laya.Socket();
            this._socket.endian = Laya.Socket.BIG_ENDIAN;
            this._socket.on(Laya.Event.OPEN, this, this._openHandler);
            this._socket.on(Laya.Event.MESSAGE, this, this._receiveHandler);
            this._socket.on(Laya.Event.CLOSE, this, this._closeHandler);
            this._socket.on(Laya.Event.ERROR, this, this._errorHandler);

            this._waitSendRequests.length = 0;
            this._sendingRequest = null;

            this._timeoutReqsArray = new Array();
        }

        _check() {
            if (WxConfigMgr.NetEncrypt) {
                if (this._socketEnv && this._socketEnv.getRid() && this._socketEnv.getAesKey() && this._loginUrl) {
                    return true;
                }
                return false;
            } else {
                if (this._socketEnv && this._socketEnv.getRid() && this._loginUrl) {
                    return true;
                }
                return false;
            }
        }

        //连接, 必须传入 rid，收到连接成功的回调之后，自动的发送 Connect 消息
        login(url, loginParam, handler) {
            this._socketEnv = new SocketEnv(loginParam);
            this._socket.connectByUrl(url);
            let outSelf = this;
            this._loginHandler = Laya.Handler.create(this, function () {
                const args = Array.from(arguments);
                handler.runWith(args);
                outSelf._logined = true;
            });
            this._loginUrl = url;
            console.log("连接socket:" + String(this._loginUrl));
        }

        reconnect() {
            //必须有rid、aesKey、loginUrl 才能够进行重连
            if (this._check()) {
                this._socket.connectByUrl(this._loginUrl);
                this._reconnecting = true;
                console.log("....... reconnect waiting .......");

                NetLoadingMgr.getInstance().beginLoading(Constants.NetReconnectId, 5000);
            } else {
                //直接抛错退出
                TipsPopMgr.getInstance().showTipsById("tid#Tips_serversError_2");
                this._timer.once(2500, null, function () {
                    EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
                });
            }
        }

        /**
         * 发送消息，始终保持只有一个请求发送给了后端，在这条请求未收到返回前，不会发送其他请求
         * @param {*} opCode 
         * @param {*} param json格式的，未经proto转化
         * @param {*} handler 回调函数
         * @param {*} isSync 是否同步调用发送
         */
        sendRequest(requestName, param, handler, isSync) {
            if (!this._check())
                return;
            if (!this._logined) {
                var opCode = ProtoManager.getInstance().getOpcode(requestName);
                if (opCode != 10001) {
                    return;
                }
            }
            if (!this._connectReady && !this._reconnecting) {
                //如果连接未准备好，并且当前没在reconnect，则开始重连
                this.reconnect();
            }
            if (!this._connectReady) {
                if (isSync) {
                    //TODO: 做点提示
                }
                return;
            }

            //如果当前正有消息在发送，则把本条请求先缓存起来，如果没有消息在发送，则直接发送
            var waitingReqeust = new WaitingRequest(requestName, param, handler, isSync);
            if (this._sendingRequest != null) {
                this._waitSendRequests.push(waitingReqeust);
            } else {
                this._onceSendReq(waitingReqeust);
            }
        }

        _onceSendReq(waitingRequest) {
            if (!this._check())
                return;
            if (!this._logined) {
                var opCode = ProtoManager.getInstance().getOpcode(waitingRequest.getRequestName());
                if (opCode != 10001) {
                    return;
                } else {
                    console.log("login sending", (+ new Date()));
                }
            }
            let finalBa = SocketHelper.genRequestData(waitingRequest, this._socketEnv);
            waitingRequest.setBuffer(finalBa.buffer);
            waitingRequest.setMsgId(this._socketEnv.getMsgId());
            waitingRequest.setSendingTs(TimeUtils.getTimeLocal());
            this._sendingRequest = waitingRequest;

            if (waitingRequest.getIsSync()) {
                //同步请求则立马开始loading
                NetLoadingMgr.getInstance().beginLoading(waitingRequest.getMsgId());
            }
            //注册一个5秒的超时回调函数，如果进入该函数则直接再次重试请求
            this._timer.once(5000, this, this._reqTimeout);
            this._socket.send(finalBa.buffer);
        }

        _reqTimeout() {
            console.log("........ req timeout .......");
            if (this._sendingRequest != null && this._sendingRequest.getBuffer() != null) {
                console.log("........ waiting req msg id: " + this._sendingRequest.getMsgId());
                //有请求5s没收到返回，开始转菊花，并且尝试重新发送
                NetLoadingMgr.getInstance().beginLoading(this._sendingRequest.getMsgId());

                //直接重新发送一次该请求，并且重新注册 timeOut
                this._socket.send(this._sendingRequest.getBuffer());
                this._timer.once(5000, this, this._reqTimeout);
            }
        }

        _openHandler(event) {
            //socket open之后，执行 connect 或者 reconnect
            if (this._socketEnv.getRid()) {
                if (this._reconnecting) {
                    let finalBa = SocketHelper.genReconnectData(this._socketEnv);
                    this._socket.send(finalBa.buffer);
                    this._socket.flush();
                } else {
                    let finalBa = SocketHelper.genConnectData(this._socketEnv);
                    this._socket.send(finalBa.buffer);
                    this._socket.flush();
                }
                this._connectReady = true;
                NetLoadingMgr.getInstance().endLoading(Constants.NetReconnectId);
                this._heartTimer.clear(this, this.reconnect);
            }
        }

        _receiveHandler(msg) {
            //先解析msg的type，是 EstaBlish/Data/Error，如果是 Data 的话，再解析是 Response/Notify
            var bytes = new Laya.Byte();
            bytes.writeArrayBuffer(msg);
            bytes.pos = 0;
            bytes.endian = Laya.Socket.BIG_ENDIAN;
            var msgType = bytes.getByte();
            var msgLength = bytes.getInt32();
            var msgData = bytes.readArrayBuffer(msgLength);
            console.log("_receiveHandler", msgType);
            switch (msgType) {
                case SocketHelper.EstablishMsg:
                    SocketHelper.dealEsblishMsg(msgData, this._socketEnv);  //解密连接数据
                    if (this._reconnecting) {
                        this._reconnecting = false;         //重连结束, TODO 可以发送重连时需要请求的接口
                    } else {
                        this.sendRequest("Player.LoginReq", this._socketEnv.getLoginParam(), this._loginHandler);
                    }
                    break;
                case SocketHelper.DataMsg:
                    let dataBa = SocketHelper.dealDataMsg(msgData, msgLength, this._socketEnv);
                    //解密后的数据读取是 Response 还是 Notify
                    let dataMsgType = dataBa.getByte();
                    if (dataMsgType == SocketHelper.ResponseData) {
                        let msgId = dataBa.readInt32();
                        TimeUtils.updateServerTime(dataBa.readUint32(), dataBa.readUint32());   //同步下服务器的时间戳

                        let dataError = dataBa.readInt32();
                        let dataLength = dataBa.readInt32();
                        let dataParam = new Uint8Array(dataBa.readArrayBuffer(dataLength));
                        this._handlerResponse(msgId, dataParam, dataError);
                    } else if (dataMsgType == SocketHelper.NotifyData) {
                        let opCode = dataBa.readInt32();
                        let dataLength = dataBa.readInt32();
                        let dataParam = new Uint8Array(dataBa.readArrayBuffer(dataLength));
                        this._handlerNotify(opCode, dataParam);
                    } else {
                        //都不是，则直接返回，记录个错误日志
                        console.debug("error msg type: " + dataMsgType);
                    }
                    break;
                case SocketHelper.ErrorMsg:
                    let errorMsgBa = new Laya.Byte(msgData);
                    errorMsgBa.endian = Laya.Socket.BIG_ENDIAN;
                    let errorCode = errorMsgBa.getInt32();
                    console.log("*********** " + errorCode + " **********");
                    if (this._reconnecting) {
                        //重连过程中如果返回 ERROR，则直接退出走重新连接
                        console.log(".... error in reconnecting .... re login now");
                        EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
                    } else {
                        if (errorCode == 1030) {
                            console.log("===== network disconnect =====");      //多点登录
                            this.clearNetStatus();
                            this.showMultiLoginTip();
                        }
                    }
                    break;
            }
        }

        showMultiLoginTip() {
            TipsPopMgr.getInstance().showTipsById("tid#Tips_loginConflict_1");
            this._timer.once(1500, null, function () {
                EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
            });
            // Laya.Dialog.open("ui/MultiLogin.scene", false);
        }

        _closeHandler(e) {
            console.log("------ socket close ------");
            if (this._connectReady) {
                this._connectReady = false;

                //尝试重连
                this.reconnect();
            }
        }

        _errorHandler(e) {
            console.log("------- socket error ------");
            this._connectReady = false;
            if (this._reconnecting) {
                //如果正在重连，但是重连失败，等待2s再试一次
                this._heartTimer.once(2000, this, this.reconnect);
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#Tips_serversError_2", [], Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
                }));
            }
        }

        //清空当前
        clearNetStatus() {
            console.log("===== clear net status ======");
            if (this._socket) {
                this._socket.cleanSocket();
            }
            this._socket = null;
            this._connectReady = false;
            this._loginUrl = null;
            this._reconnecting = false;
            this._sendingRequest = null;
            this._waitSendRequests.length = 0;
        }

        _onHeartBeat() {
            // console.log("------- heart beat -------");
            if (this._connectReady) {
                let ba = SocketHelper.genHeartBeatData();
                this._socket.send(ba.buffer);
                this._socket.flush();
            }
        }

        _handlerResponse(msgId, dataParam, dataError) {
            //具体处理接收到返回数据的处理函数,只处理msgId等于当前缓存的this._msgId的消息,因为是逐条发送处理，所以序号不对就直接忽略
            if (this._socketEnv.getMsgId() == msgId && this._sendingRequest != null) {
                var opCode = ProtoManager.getInstance().getOpcode(this._sendingRequest.getRequestName());
                var ret = new Array();
                ret[0] = dataError;
                //当返回没有错误码的时候，才去解析数据，否则直接将数据复制一个空的json
                if (ret[0] == 0) {
                    ret[1] = ProtoManager.getInstance().decodeResponse(opCode, dataParam);
                    if (opCode == 10001) {
                        console.log("login responsed", (+ new Date()));
                        this._waitSendRequests.length = 0;
                    }
                } else {
                    ret[1] = {};
                }

                this._timer.clear(this, this._reqTimeout);
                NetLoadingMgr.getInstance().endLoading(msgId);
                var handler = this._sendingRequest.getHandler();
                if (handler)
                    handler.runWith(ret);
                if (this._timeoutReqsArray.length < 5) {
                    //超时次数小于5次时才统计，到达5次，则发送一条服务器日志记录下
                    var delayTs = TimeUtils.getTimeLocal() - this._sendingRequest.getSendingTs();
                    if (delayTs >= 150) {
                        console.log("fuck:   " + delayTs);
                        this._timeoutReqsArray.push(this._sendingRequest.getRequestName());
                        if (this._timeoutReqsArray.length >= 5) {
                            this._sendClientTimeoutLog();
                            this._timeoutReqsArray.splice(0, this._timeoutReqsArray.length);
                        }
                    }
                }
                this._sendingRequest = null;

                if (this._waitSendRequests.length > 0 && this._check()) {
                    this._onceSendReq(this._waitSendRequests.shift());
                }
            } else {
                console.error("!!!!! msgId wrong or sending request is null: " + msgId);
                console.error(this._socketEnv.getMsgId());
                console.error(this._sendingRequest);
            }
        }

        _handlerNotify(opCode, dataParam) {
            //具体处理服务器推送的处理函数
            var ret = ProtoManager.getInstance().decodeNotify(opCode, dataParam);
            ProtoManager.getInstance().handlerNotify(opCode, ret);
        }

        _sendClientTimeoutLog() {
            //复用下之前记录loading时长的日志
            let data = {};

            data.rid = this._socketEnv.getRid() != null ? this._socketEnv.getRid() : "null";
            data.timeoutReqs = this._timeoutReqsArray;

            let customData = JSON.stringify(data);
            EventMgr.getInstance().postEvent("SERVICE_SEND_LOADINGLOG", customData);
        }
    }

    class Service {

        static init() {
            WebSocketClient.getInstance().init();
        }

        static sendRequest(reqName, params, handler, errorHandler, isSync) {
            let commonHandler = function() {
                // 处理通用错误
                const args = Array.from(arguments);
                if(args[0]!=0) {
                    if(errorHandler != undefined && errorHandler != null) {
                        //走调用方传入的错误处理函数
                        errorHandler.runWith(args[0]);
                    } else {
                        //展示错误信息
                        TipsPopMgr.getInstance().showTipsById("tid#Tips_serversError_1");
                        console.error("Error Number:" + args[0]);
                    }
                }else {
                    if(handler) {
                        handler.runWith(args.slice(1));
                    }
                }
            };
            WebSocketClient.getInstance().sendRequest(reqName, params, Laya.Handler.create(null, commonHandler), isSync);
        }

        static login(url, serverId, rid, token, userInfo,ppInfo, hasAuthorized, handler) {
            let commonHandler = function(res,res1) {
                // 处理通用错误
                const args = Array.from(arguments);
                if(args[0]!=0) {
                    // TODO 展示错误信息
                    console.error("Error Number:"+args[0]);
                }else {
                    handler.runWith(args.slice(1));
                }
            };
            let sdkName = "dev";
            if (Laya.Browser.onMiniGame) {
                sdkName = "wx";
            }else if((Laya.Browser.onAndroid||Laya.Browser.onIOS||Laya.Browser.onIPad||Laya.Browser.onIPhone)&&Constants.shouji){
                sdkName = "pp";
                let param = SdkHelper.params();
                param['method'] = "createRole";
                param['serverId'] = serverId;
                param['roleId'] =rid;
                param['roleName'] =userInfo.nickName;
                param['aid'] =WxConfigMgr.getInstance().getAdUnitId();
                param['uid'] =SdkHelper.uid;
                if(arguments.isNew){
                  param['type'] ='3';
                }else {
                    param['type'] ='1';   
                }
                console.log(param);
                SdkHelper.trackEvent(param);
            }
     

            WebSocketClient.getInstance().login(url, {"rid":rid, "sdkName": sdkName, "token": token, "userInfo": userInfo,"ppInfo":ppInfo, 'hasAuthorized': hasAuthorized?true:false}, Laya.Handler.create(null, commonHandler));
        }

        static registerNotifyCallback(opCode, caller, listener, args) {
            ProtoManager.getInstance().registerNotifyCallback(opCode, caller, listener, args);
        }

        static logout(handler) {
            Service.sendRequest("Player.LogoutReq", {}, handler);
        }

        static onlineCrossDay(handler) {
            Service.sendRequest("Player.OnlineCrossDayReq", {}, handler);
        }

        static debug(code, param, handler) {
            Service.sendRequest("Debug.DebugReq", {code: Number(code), param: String(param)}, handler);
        }

        static getFullBag(handler) {
            Service.sendRequest("Player.FullBagReq", {}, handler);
        }

        static putCustomData(customKV, handler) {
            Service.sendRequest("Player.PutCustomDataReq", {customKV:customKV}, handler);
        }

        static delCustomData(keys, handler) {
            Service.sendRequest("Player.DelCustomDataReq", {keys : keys}, handler);
        }

        static getCustomData(handler) {
            Service.sendRequest("Player.GetCustomDataReq", {}, handler);
        }

        static viewAd(status, type, handler = null) {
            Service.sendRequest("Player.ViewAdReq", {'status': status, 'type': type}, handler);
        }

        static sendClientLoadingLog(customData, handler = null) {
            Service.sendRequest("Player.ClientLoadingLogReq", {'customData': customData}, handler);
        }

        static shareReq(status, type, handler = null) {
            Service.sendRequest("Player.ShareReq", {'status': status, 'type': type}, handler);
        }

        //建筑
        static getAllBuilding(handler) {
            // WebSocketClient.getInstance().sendRequest("Building.GetAllBuildingReq", {}, handler);
            Service.sendRequest("Building.GetAllBuildingReq", {}, handler);
        }

        static putBuilding(buildingId, baseId, fetchType, handler) {
            // WebSocketClient.getInstance().sendRequest("Building.PutBuildingReq", {"buildingId":buildingId, "pos":baseId}, handler);
            Service.sendRequest("Building.PutBuildingWithAdReq", { "buildingId": buildingId, "pos": baseId, "fetchType": fetchType }, handler);
        }

        static removeBuilding(buildingId, handler) {
            // WebSocketClient.getInstance().sendRequest("Building.RemoveBuildingReq", {"buildingId":buildingId}, handler);
            Service.sendRequest("Building.RemoveBuildingReq", {"buildingId":buildingId}, handler);
        }

        static bargain(fetchType, fishAmount, buildingId, handler) {
            Service.sendRequest("Building.BargainReq", {"fetchType":fetchType,"fishAmount":fishAmount,"buildingId":buildingId}, handler);
        }

        //任务
        static getAllTasks(handler) {
            Service.sendRequest("Task.GetAllTasksReq", {}, handler);
        }

        static getDailyTaskReward(taskId, fetchType, handler) {
            Service.sendRequest("Task.getDailyTaskRewardWithAdReq", { "taskId": taskId, "fetchType": fetchType }, handler);
        }

        static getTrophyTaskReward(taskId, fetchType, handler) {
            Service.sendRequest("Task.getTrophyTaskRewardWithAdReq", { "taskId": taskId, "fetchType": fetchType }, handler);
        }

        static getMainTaskReward(taskId, handler) {
            Service.sendRequest("Task.getMainTaskRewardReq", { "taskId": taskId }, handler);
        }

        //菜谱
        static getPlayerFoodMenu(handler) {
            Service.sendRequest("Food.GetPlayerFoodMenuReq", {}, handler);
        }

        static foodLevelUp(foodListId, fetchType, handler) {
            Service.sendRequest("Food.FoodLevelUpWithAdReq", { "foodListId": foodListId, "fetchType": fetchType }, handler);
        }

        //场景同步
        static getLetterBoxInfo(handler) {
            Service.sendRequest("Scene.GetLetterBoxInfoReq", {}, handler);
        }

        static fetchLetterBox(ad , handler) {
            Service.sendRequest("Scene.FetchLetterBoxReq", {"ad" : ad }, handler);
        }

        static updateSceneInfo(customData, rewards, petRewards, visitorRewards, events, handler) {
            if (petRewards) {
                if (events) {
                    Service.sendRequest("Scene.UpdateSceneInfoReq",
                        { "customData": customData, "rewards": rewards, "petRewards": petRewards, "visitorRewards": visitorRewards, "events": events},
                        handler);
                } else {
                    Service.sendRequest("Scene.UpdateSceneInfoReq",
                        { "customData": customData, "rewards": rewards, "petRewards": petRewards, "visitorRewards": visitorRewards},
                        handler);
                }
            } else {
                if (events) {
                    Service.sendRequest("Scene.UpdateSceneInfoReq",
                        { "customData": customData, "rewards": rewards, "visitorRewards": visitorRewards, "events": events}, handler);
                } else {
                    Service.sendRequest("Scene.UpdateSceneInfoReq",
                        { "customData": customData, "rewards": rewards, "visitorRewards": visitorRewards}, handler);
                }
            }
        }

        //宠物 图鉴
        static getPetsInfo(handler) {
            Service.sendRequest("Pet.GetPlayerPetsInfoReq", {}, handler);
        }

        static adoptPet(petId,type, handler) {
            Service.sendRequest("Pet.AdoptPetReq", {"roleId":petId,"type":type}, handler, null, true);
        }

        static storyLock(petId,num,handler){
            Service.sendRequest("Pet.StoryUnlockReq", {"roleId":petId,"unlockIndex":num}, handler);
        }

        //炫耀一下获得奖励
        static petShareGetReward(Id,type,handler){
            if(type == Constants.PeacockShareType.FishNewCome){
                var str = JSON.stringify({"fishId":Id,"type":type});
            }
            else {
                var str = JSON.stringify({"roleId":Id,"type":type});
            }
            Service.sendRequest("Pet.ShareGetRewardReq", {"json":str}, handler);
        }

        static catLevelUp(petId, handler) {
            Service.sendRequest("Pet.CatLevelUpReq",{"roleId":petId}, handler);
        }

        static catStarUpgrade(petId, handler) {
            Service.sendRequest("Pet.CatStarUpGradeReq",{"roleId":petId}, handler);
        }



        //长客
        static findVisitorGift(handler) {
            Service.sendRequest("Scene.FindVisitorGiftReq", {}, handler);
        }

        static getVisitorGiftAll(fetchType, handler) {
            Service.sendRequest("Scene.GetVisitorGiftAllReq", {"type":fetchType}, handler);
        }

        static getVisitorGiftSingle(giftId, handler) {
            Service.sendRequest("Scene.GetVisitorGiftSingleReq", {"id": giftId}, handler);
        }

        //放置猫粮，用于订阅
        static catFoodSubscribe(endTime, handler) {
            Service.sendRequest("Scene.CatFoodSubscribeReq", {"endTime": endTime}, handler);
        }

        //探险
        static getNauticalInfo(handler) {
            Service.sendRequest("Nautical.GetNauticalInfo0912Req", {}, handler);
        }

        static mergeNauticalFood(nauticalFoodId, nauticalFoodCount, handler) {
            Service.sendRequest("Nautical.MergeNauticalFoodReq",
                {"id": String(nauticalFoodId), "amount": Number(nauticalFoodCount)}, handler);
        }

        static startNauticalTrip(info, handler) {
            Service.sendRequest("Nautical.StartNauticalTrip0912Req",info, handler);
        }

        static finishNauticalTrip(type,tripIndex ,handler) {
            Service.sendRequest("Nautical.FinishNauticalTrip0912Req", {'fetchType':type,'tripIndex':tripIndex}, handler);
        }

        static previewNauticalTripReq(tripIndex ,handler) {
            Service.sendRequest("Nautical.PreviewNauticalTrip0912Req", {"tripIndex":tripIndex}, handler);
        }

        static adReduceNauticalTime(tripIndex, type, handler) {
            Service.sendRequest("Nautical.AdReduceNauticalTime0912Req", { "tripIndex": tripIndex, "type": type }, handler);
        }

        static unlockNautical(tripIndex,handler) {
            Service.sendRequest("Nautical.unlockNauticalReq", {"tripIndex":tripIndex}, handler);
        }

        //钓鱼
        static getFishGameInfo(handler) {
            Service.sendRequest("FishingGame.GetFishGameInfoReq", {}, handler, null, true);
        }

        static playFishGame(startType, fishingFishId, fishingBaitId, rewards, fishingFishSize, fetchType, handler, errorHandler) {
            Service.sendRequest("FishingGame.PlayFishGameReq",
                { "fishingFishId": fishingFishId, "fishingBaitId": fishingBaitId, "rewards": rewards, "fishingFishSize": fishingFishSize, "type": startType, "fetchType": fetchType },
                handler, errorHandler, true);
        }

        static getFishingFishInfo(handler) {
            Service.sendRequest("FishingGame.GetFishingFishInfoReq", {}, handler, null, true);
        }

        static getFishingAdFreeInfo(handler) {
            Service.sendRequest("FishingGame.AdFreeFishingReq", {}, handler, null, true);
        }


        //家园
        static setPetStatus(params, handler) {
            Service.sendRequest("Home.SetPetStatusReq", {"params": params}, handler);
        }

        static cuteInteract(petId, toyId, handler) {
            Service.sendRequest("Home.CuteInteractReq", {"petId": petId, "toyId": toyId}, handler);
        }

        static commonInteract(petId, toyId, handler) {
            Service.sendRequest("Home.CommonInteractReq", {"petId": petId}, handler);
        }

        static fetchPhotoWall(handler) {
            Service.sendRequest("Home.FetchPhotoWallReq", {}, handler);
        }

        static fetchHomeInfo(handler) {
            Service.sendRequest("Home.FetchHomeInfoReq", {}, handler);
        }

        static healthyInteract(petId,type,handler){
            Service.sendRequest("Home.HealthyInteractReq", {"petId":petId,"type":type}, handler);
        }   
        //邮件
        static fetchAllMails(handler) {
            Service.sendRequest("Mail.FetchAllMailsReq", {}, handler);
        }

        static readMail(mailId, handler) {
            Service.sendRequest("Mail.ReadMailReq", { "mailId": mailId }, handler);
        }

        static deleteMail(mailId, handler) {
            Service.sendRequest("Mail.DeleteMailReq", { "mailId": mailId }, handler);
        }

        static deleteReadMails(handler) {
            Service.sendRequest("Mail.DeleteReadMailsReq", {}, handler);
        }

        static receiveMailRewards(mailId, handler) {
            Service.sendRequest("Mail.ReceiveMailRewardsReq", { "mailId": mailId }, handler);
        }

        static fireEvent(type, value, extraInfo, handler) {
            Service.sendRequest("Event.FireEventReq", { "event": {"type": type, "value": value, "extraInfo": extraInfo} }, handler);
        }
        //订单
        static getEntrust(handler) {
            Service.sendRequest("Entrust.GetEntrustReq", {}, handler);
        }

        static acceptEntrust(listId, handler) {
            Service.sendRequest("Entrust.AcceptEntrustReq", { "listId": listId }, handler);
        }

        static receiveRewards(listId, handler) {
            Service.sendRequest("Entrust.ReceiveRewardsReq", { "listId": listId }, handler);
        }
        // 发送建议
        static sendSuggest(content, handler) {
            Service.sendRequest("Suggest.SendSuggestReq", { "content": content }, handler);
        }

        //气球
        static getBalloonInfo(fetchType,handler) {
            Service.sendRequest("Balloon.TouchBalloonReq", {"fetchType":fetchType}, handler);
        }

        static getBalloonNum(handler){
            Service.sendRequest("Balloon.GetBalloonInfoReq", {}, handler);
        }
        
        // 小游戏
        static getMiniGame(handler) {
            Service.sendRequest("MiniGames.GetMiniGameReq", {}, handler);
        }

        static finishMiniGame(listId, result, fetchType, handler) {
            Service.sendRequest("MiniGames.FinishMiniGameReq", { listId: listId, result: result, fetchType: fetchType }, handler);
        }

        static ClientTriggerMiniGame(handler) {
            Service.sendRequest("MiniGames.ClientTriggerMiniGameReq", {}, handler);
        }

        static GiveUpMiniGame(handler) {
            Service.sendRequest("MiniGames.GiveUpMiniGameReq", {}, handler);
        }

        // 挂机
        static queryLuckyBagInfo(handler) {
            Service.sendRequest("LuckyBag.QueryLuckyBagInfoReq", {}, handler);
        }

        static receiveLuckyBag(fetchType, handler) {
            Service.sendRequest("LuckyBag.ReceiveLuckyBagReq", { fetchType: fetchType }, handler);
        }
     

        
        static getAuthorize(userInfo, handler) {
            Service.sendRequest("Player.AuthorizeReq", {'userInfo': userInfo}, handler);
        }

        //员工
        static getStaffSkillGroup(staffGroupId, handler) {
            Service.sendRequest("Staff.GetStaffSkillGroupReq", { 'staffGroupId': staffGroupId }, handler);
        }

        static skillLevelUp(skillGroupId, staffGroupId, fetchType, handler) {
            Service.sendRequest("Staff.SkillLevelUpReq", { 'skillGroupId': skillGroupId, 'staffGroupId': staffGroupId, 'fetchType': fetchType }, handler);
        }

        static staffLevelUp(staffGroupId, handler) {
            Service.sendRequest("Staff.StaffLevelUpReq", { 'staffGroupId': staffGroupId }, handler);
        }

        static selectSkill(skillGroupId, staffGroupId, handler) {
            Service.sendRequest("Staff.SelectSkillReq", { 'skillGroupId': skillGroupId, 'staffGroupId': staffGroupId }, handler);
        }

        static getStaffGroup(handler) {
            Service.sendRequest("Staff.GetStaffGroupReq", {}, handler);
        }

        static reduceCDTime(skillGroupId, staffGroupId, handler) {
            Service.sendRequest("Staff.ReduceCDTimeReq", { "skillGroupId": skillGroupId, "staffGroupId": staffGroupId }, handler);
        }
        
        static getSignIngo(handler){
            Service.sendRequest("Sign.GetSignInfoReq", {}, handler);
        }
        static GetSignRewardReq(handler){
            Service.sendRequest("Sign.GetSignRewardReq", {}, handler);   
        }
        //助力
        static GetWharfInfo(handler) {
            Service.sendRequest("Wharf.GetWharfInfoReq", {}, handler);
        }
        static GetNauticalProjectGroup(handler) {
            Service.sendRequest("Wharf.GetNauticalProjectGroupReq", {}, handler);
        }
        static ProjectLevelUp(projectGroupId, handler) {
            Service.sendRequest("Wharf.ProjectLevelUpReq", {"projectGroupId":projectGroupId}, handler);
        }
        static GetWharfLetterBox(handler) {
            Service.sendRequest("Wharf.GetWharfLetterBoxReq", {}, handler);
        }
        static FetchWharfLetterBox(handler,fetchType) {
            Service.sendRequest("Wharf.FetchWharfLetterBoxReq",{'fetchType':fetchType}, handler);
        }

        static AidPlayer(targetRId,handler) {
            Service.sendRequest("Wharf.AidPlayerReq",{"targetRId": targetRId}, handler); 
        }
        static ReceiveAidSalary(handler){
            Service.sendRequest("Wharf.ReceiveAidSalaryReq",{}, handler); 
        }
        static FetchDailyShell(handler){
            Service.sendRequest("Wharf.FetchDailyShellReq",{}, handler); 
        }

        static FetchLuckyDrawReward(type, rewardInfo ,handler){
            Service.sendRequest("LuckyDraw.FetchLuckyDrawRewardReq",{"type":type, "rewardInfo":rewardInfo}, handler); 
        }

        static GetLuckyDrawInfo(handler){
            Service.sendRequest("LuckyDraw.GetLuckyDrawInfoReq",{}, handler); 
        }

        static FetchBuildingTicketReward(ticketIndex, posId, handler) {
            Service.sendRequest("LuckyDraw.FetchBuildingTicketRewardReq", { "ticketIndex": ticketIndex, "posId": posId }, handler);
        }

        static FinishBuildingCoupon(handler) {
            Service.sendRequest("LuckyDraw.FinishBuildingCouponReq", {}, handler);
        }

        static UseAdCoupon(adCouponType,handler) {
            Service.sendRequest("LuckyDraw.UseAdCouponReq", {"adCouponType":adCouponType}, handler);
        }

        static GetLotteryBag(handler){
            Service.sendRequest("LuckyDraw.GetLotteryBagReq", {}, handler);
        }
        
        static FetchMilestoneReward(milestoneId ,fetchType ,handler){
            Service.sendRequest("Milestone.FetchMilestoneRewardReq",{"milestoneId" : milestoneId ,"fetchType" : fetchType }, handler); 
        }
        static GetCollectGiftInfoReq(handler)
        {
            Service.sendRequest("Sign.GetCollectGiftInfoReq",{},handler);
        }


        //万事屋
        static getCatMissionInfo(handler) {
            Service.sendRequest("CatMission.GetCatMissionInfoReq", {}, handler);
        }

        static refreshCatMission(handler) {
            Service.sendRequest("CatMission.PlayerRefreshReq", {}, handler);
        }

        static changeRefreshLevel(level, handler) {
            Service.sendRequest("CatMission.ChangeRefreshLevelReq", {"level": level}, handler);
        }

        static receiveMissionReward(missionIndex, roleIds, handler) {
            Service.sendRequest("CatMission.ReceiveMissionRewardReq", { "missionIndex": missionIndex, "roleIds": roleIds }, handler);
        }

        static resetCatStatus(roleId, handler) {
            Service.sendRequest("CatMission.ResetCatStatusReq", { "roleId": roleId }, handler);
        }

        static adRefreshMission(handler) {
            Service.sendRequest("CatMission.AdRefreshMissionReq", {}, handler);
        }

        //内购
        static createOrder(orderId,purchaseId,handler) {
            Service.sendRequest("Pay.CreateOrderReq", {"orderId":orderId,"purchaseId":purchaseId}, handler);
        }

        static finishOrder(orderId,purchaseId,handler) {
            Service.sendRequest("Pay.FinishOrderReq", {"orderId":orderId,"purchaseId":purchaseId}, handler);
        }

        static receiveFreeDiamond(handler) {
            Service.sendRequest("Pay.ReceiveFreeDiamondReq", {}, handler);
        }

        static payCardBonus(cardType,handler) {
            Service.sendRequest("Pay.PayCardBonusReq", {"cardType":cardType}, handler);
        }

        //资源购买
        static resourceShop(purchaseId,handler) {
            Service.sendRequest("Shop.ResourceShopReq", {"purchaseId":purchaseId}, handler);
        }

        //外卖
        // 获取外卖公司信息 请求
        static fetchTakeoutInfo(handler) {
            Service.sendRequest("Takeout.FetchTakeoutInfoReq", {}, handler);
        }
        // 配送 请求
        static delivery(pets, free, handler) {
            Service.sendRequest("Takeout.DeliveryReq", { "pets": pets, "free": free }, handler);
        }
        // 领取配送奖励 请求
        static receiveOrderRewards(handler) {
            Service.sendRequest("Takeout.ReceiveRewardsReq", {}, handler);
        }
        // 加速配送 请求
        static speedUpDelivery(free, handler) {
            Service.sendRequest("Takeout.SpeedUpDeliveryReq", { free: free }, handler);
        }
        // 升级外卖公司 请求
        static levelUpTakeout(handler) {
            Service.sendRequest("Takeout.LeveUpTakeoutReq", {}, handler);
        }
        // 放置宣传员 请求
        static putAdvertiserCats(pets, handler) {
            Service.sendRequest("Takeout.PutAdvertiserCatsReq", { "pets": pets }, handler);
        }
        // 购买订单数量
        static buyOrder(handler) {
            Service.sendRequest("Takeout.BuyOrderReq", { }, handler);
        }
        
    }

    class MailDataMgr {

        constructor() {
            this.mailMap = new Map();
            this._new = false;
        }

        static getInstance() {
            return MailDataMgr.instance ? MailDataMgr.instance : MailDataMgr.instance = new MailDataMgr();
        }

        updataMail(mails) {
            this.clear();
            mails.forEach(element => {
                if (element.systemId != null) {
                    let config = ConfigMgr$1.getInstance().getItemById("Mail", element.systemId);
                    if (config) {
                        element.title = Strings$1.get(config.Name);
                        element.content = Strings$1.get(config.Content);
                    }
                }
                this.mailMap.set(element.id, element);
            });
            this._new = false;
            EventMgr.getInstance().postEvent("UDATA_MAIL");
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mail"]);
        }

        getMails() {
            return this.mailMap;
        }

        readMail(mail) {
            this.mailMap.set(mail.id, mail);
            this._new = false;
            EventMgr.getInstance().postEvent("UDATA_MAIL");
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mail"]);
        }

        deleteMail(mailId) {
            if (mailId != null)
                this.mailMap.delete(mailId);
        }

        deleteReadMails(mails) {
            if (mails != null) {
                this.updataMail(mails);
                EventMgr.getInstance().postEvent("UDATA_MAIL");
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mail"]);
            }
        }

        clear() {
            this.mailMap.clear();
            this._new = false;
        }

        hasUnRead() {
            let a = false;
            this.mailMap.forEach(element => {
                if (!element.read) {
                    a = true;
                    return;
                }
            });
            return a || this._new;
        }

        setNewMail(value) {
            this._new = value;
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mail"]);
        }

    }

    class MailSystem {
        static init() {
        }

        static clear() {
            MailDataMgr.getInstance().clear();
        }

        static updataMail(mails) {
            MailDataMgr.getInstance().updataMail(mails);
        }

        static getMails() {
            return MailDataMgr.getInstance().getMails();
        }

        static readMail(mail) {
            MailDataMgr.getInstance().readMail(mail);
        }

        static deleteMail(mailId) {
            MailDataMgr.getInstance().deleteMail(mailId);
        }

        static deleteReadMails(mails) {
            MailDataMgr.getInstance().deleteReadMails(mails);
        }

        static clear() {
            MailDataMgr.getInstance().clear();
        }

        static hasUnRead() {
            return MailDataMgr.getInstance().hasUnRead();
        }

        static setNewMail(value) {
            MailDataMgr.getInstance().setNewMail(value);
        }

    }

    class PhotoData {

        constructor(id, image, type) {
            this._image = image;
            this._id = id;
            this._frameId = null;
            this._inFrame = false;
            //1:用户照片 2:系统照片
            this._type = type;
            this._time = null;
            this._sysytemId = null;
            this._sysytemConfig = null;
        }

        getImage() {
            return this._image;
        }

        setImage(image) {
            this._image = image;
        }

        getId() {
            return this._id;
        }

        setId(id) {
            this._id = id;
        }

        getFrameId() {
            if (this._inFrame) {
                return this._frameId;
            }
        }

        setFrameId(id) {
            this._frameId = id;
        }


        getInFrame() {
            return this._inFrame;
        }

        setInFrame(value) {
            this._inFrame = value;
            if (!value) {
                this.setFrameId(null);
            }
        }

        getType() {
            return this._type;
        }

        setType(type) {
            this._type = type;
        }

        getTime() {
            return this._time;
        }

        setTime(value) {
            this._time = value;
        }

        getSaveData() {
            return { id: this._id, image: this._image, frameId: this._frameId, inFrame: this._inFrame, type: this._type, time: this._time };
        }

        setSystemConfig(config) {
            this._sysytemConfig = config;
            this._sysytemId = config.Id;
        }

        getSystemId() {
            return this._sysytemId;
        }

        getSystemConfig() {
            return this._sysytemConfig;
        }


    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class PhotoDataMgr {

        constructor() {
            this._photoMap = new Map();
            this._systemPhotoMap = new Map();
            this._photoFrameData = [null, null, null, null];
            this._maxId = -1;
            this.updataLocalPhotosData();
            this.updataPhotoFrameData();
        }

        static getInstance() {
            return PhotoDataMgr.instance ? PhotoDataMgr.instance : PhotoDataMgr.instance = new PhotoDataMgr();
        }

        setPhotoFrameData(photoId, index) {
            this._photoFrameData[index] = photoId;
            EventMgr.getInstance().postEvent("UPDATA_PHOTO_WALL");
        }

        getPhotoFrameData() {
            return this._photoFrameData;
        }

        updataPhotoFrameData() {
            //更新全部数据
            this._photoFrameData = [null, null, null, null];
            this._photoMap.forEach((value, key) => {
                if (value.getInFrame()) {
                    this._photoFrameData[value.getFrameId()] = value.getId();
                }
            });
            this._systemPhotoMap.forEach((value, key) => {
                if (value.getInFrame()) {
                    this._photoFrameData[value.getFrameId()] = value.getId();
                }
            });
        }

        clearFrameData() {
            this._photoFrameData = [null, null, null, null];
            this.photoDataLocalSave();
        }

        deleataPhotoFrameData(index) {
            if (this._photoFrameData[index] != null) {
                let photo = this.getPhotoData(this._photoFrameData[index]);
                photo.setInFrame(false);
                photo.setFrameId(null);
            }
            this._photoFrameData[index] = null;
            this.photoDataLocalSave();
            EventMgr.getInstance().postEvent("UPDATA_PHOTO_WALL");
        }

        exchangeFrameData(fromIndex, toIndex) {
            let exchangeData = this._photoFrameData[toIndex];
            this._photoFrameData[toIndex] = this._photoFrameData[fromIndex];
            this._photoFrameData[fromIndex] = exchangeData;
            if (this._photoFrameData[toIndex] != null) {
                this.getPhotoData(this._photoFrameData[toIndex]).setFrameId(toIndex);
            }
            if (this._photoFrameData[fromIndex] != null) {
                this.getPhotoData(this._photoFrameData[fromIndex]).setFrameId(fromIndex);
            }
            EventMgr.getInstance().postEvent("UPDATA_PHOTO_WALL");
            this.photoDataLocalSave();
        }

        getEmptyFrameId() {
            for (let i in this._photoFrameData) {
                if (this._photoFrameData[i] == null) {
                    return Number(i);
                }
            }
            return null;
        }


        //相册

        getPhotoData(id) {
            if (this._systemPhotoMap.has(id)) {
                return this._systemPhotoMap.get(id);
            }
            return this._photoMap.get(id);
        }

        pushPhotoData(image, type) {
            this._maxId--;
            let photo = new PhotoData(this._maxId, image, type);
            photo.setTime(TimeUtils.getTimeNowOnServer());
            this._photoMap.set(this._maxId, photo);
            this.photoDataLocalSave();
        }

        getSystemPhotosData() {
            let arr = [];
            this._systemPhotoMap.forEach(element => {
                if (element.getType() == 2) {
                    arr.push(element);
                }
            });
            return arr;
        }

        getUserPhotosData() {
            let arr = [];
            this._photoMap.forEach(element => {
                if (element.getType() == 1) {
                    arr.push(element);
                }
            });
           return arr;
        }

        deletePhotoData(id) {
            let photo = this.getPhotoData(id);
            if (photo.getInFrame()) {
                this.deleataPhotoFrameData(photo.getFrameId());
            }
            if (this._systemPhotoMap.has(id)) {
                this._systemPhotoMap.delete(id);
            } else {
                this._photoMap.delete(id);
            }
            EventMgr.getInstance().postEvent("PHOTOS_UPDATA");
            this.photoDataLocalSave();
        }

        placePhoto(id) {
            let frameId = this.getEmptyFrameId();
            if (frameId != null) {
                let photo = this.getPhotoData(id);
                photo.setFrameId(frameId);
                photo.setInFrame(true);
                this.setPhotoFrameData(id, frameId);
                this.photoDataLocalSave();
                EventMgr.getInstance().postEvent("PHOTOS_UPDATA");
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_picture_set"]);
                return true;
            } else {
                return false;
            }
        }

        removePhoto(id) {
            let photo = this.getPhotoData(id);
            this.deleataPhotoFrameData(photo.getFrameId());
            EventMgr.getInstance().postEvent("PHOTOS_UPDATA");
        }

        photoDataLocalSave() {
            let arr = [];
            this._photoMap.forEach((value, key) => {
                let str = value.getSaveData();
                arr.push(str);
            });
            this._systemPhotoMap.forEach((value, key) => {
                let str = value.getSaveData();
                arr.push(str);
            });
            Laya.LocalStorage.setJSON("photoData", arr);
        }

        updataLocalPhotosData() {
            let localArr = Laya.LocalStorage.getJSON("photoData");
            if (localArr != null && Object.prototype.toString.call(localArr) == '[object Array]') {
                // let systemConfig = ConfigMgr.getInstance().getConfig("CatPhotos");
                localArr.forEach(element => {
                    let photo = new PhotoData(element.id, element.image, element.type);
                    photo.setInFrame(element.inFrame);
                    photo.setFrameId(element.frameId);
                    photo.setTime(element.time);
                    // photo.setTime(TimeUtils.getTimeNowOnServer());
                    if (element.type == 1) {
                        this._photoMap.set(element.id, photo);
                        this._maxId = Math.min(this._maxId, element.id);
                    } else {
                        // if (systemConfig.has(element.id)) {
                        this._systemPhotoMap.set(element.id, photo);
                        // }
                    }
                });
            }
            // this.updataSystemPhotoData();
        }

        setSystemPhotoMap(id){
            let photo = new PhotoData(id,null,2);
            let systemConfig = ConfigMgr$1.getInstance().getItemById("CatPhotos",id);
            photo.setSystemConfig(systemConfig);
            this._systemPhotoMap.set(id, photo);
        }

        updataSystemPhotoData(data) {
            let systemConfig = ConfigMgr$1.getInstance().getConfig("CatPhotos");
            let map = new Map();
            data.forEach(id => {
                let config = systemConfig.get(id);
                if (this._systemPhotoMap.has(id)) {
                    this._systemPhotoMap.get(id).setSystemConfig(config);
                } else {
                    let photo = new PhotoData(id, null, 2);
                    photo.setInFrame(false);
                    photo.setSystemConfig(config);
                    this._systemPhotoMap.set(id, photo);
                }
                map.set(id, id);
            });
            this._systemPhotoMap.forEach(photo => {
                if (!map.has(photo.getId())) {
                    this._systemPhotoMap.delete(photo.getId());
                }
            });
            map = null;
        }

        getUserPhotoNum() {
            return this.getUserPhotosData().length;
        }

        //从服务器获取数据
        updataPhotosData(data) {
            this.updataSystemPhotoData(data.systemPhotos);
            this.updataPhotoFrameData();
        }

        clear() {
            this._photoMap = new Map();
            this._systemPhotoMap = new Map();
            this._photoFrameData = [null, null, null, null];
            this._maxId = -1;
            this.updataLocalPhotosData();
            this.updataPhotoFrameData();
        }

    }

    class PhotoSystem {
        //照片墙

        static clear() {
            PhotoDataMgr.getInstance().clear();
        }

        static getPhotoFrameData() {
            return PhotoDataMgr.getInstance().getPhotoFrameData();
        }

        static updataPhotoFrameData() {
            PhotoDataMgr.getInstance().updataPhotoFrameData();
        }

        static clearFrameData() {
            PhotoDataMgr.getInstance().clearFrameData();
        }

        static deleataPhotoFrameData(index) {
            PhotoDataMgr.getInstance().deleataPhotoFrameData(index);
        }

        static exchangeFrameData(fromIndex, toIndex) {
            PhotoDataMgr.getInstance().exchangeFrameData(fromIndex, toIndex);
        }


        //相册
        static getPhotoData(id) {
            return PhotoDataMgr.getInstance().getPhotoData(id);
        }

        static pushPhotoData(image, type) {
            PhotoDataMgr.getInstance().pushPhotoData(image, type);
        }

        static getSystemPhotosData() {
            return PhotoDataMgr.getInstance().getSystemPhotosData();
        }

        static getUserPhotosData() {
            return PhotoDataMgr.getInstance().getUserPhotosData();
        }

        static deletePhotoData(id) {
            PhotoDataMgr.getInstance().deletePhotoData(id);
        }

        static placePhoto(id) {
            return PhotoDataMgr.getInstance().placePhoto(id);
        }

        static removePhoto(id) {
            PhotoDataMgr.getInstance().removePhoto(id);
        }

        static updataPhotosData(photoWallData) {
            PhotoDataMgr.getInstance().updataPhotosData(photoWallData);
        }

        static getUserPhotoNum() {
            PhotoDataMgr.getInstance().getUserPhotoNum();
        }

        static setSystemPhotoMap(id){
            PhotoDataMgr.getInstance().setSystemPhotoMap(id);
        }

    }

    class BalloonDataMgr{
        constructor() {
            this.setConstData();
            this.loadADdata();
        }

        static getInstance() {
            return BalloonDataMgr.instance ? BalloonDataMgr.instance : BalloonDataMgr.instance = new BalloonDataMgr();
        }

        loadADdata(){
            let triggerBalloonAD = Laya.LocalStorage.getJSON("triggerBalloonAD");
            if(!triggerBalloonAD){
                this.triggerBalloonAD = 0;
            }
            else {
                this.triggerBalloonAD = JSON.parse(triggerBalloonAD);
            }
            let watchBalloonAD =  Laya.LocalStorage.getJSON("watchBalloonAD");
            if(!watchBalloonAD){
                this.watchBalloonAD = new Array(0,0,0,0,0,0,0,0,0,0);
            }
            else {
                this.watchBalloonAD = watchBalloonAD;
            }
        }

        getADInter(){
            let a = 0,inter = 1;
            for(let i = 0;i<10;i++){
                a += this.watchBalloonAD[i];
            }
            if(this.triggerBalloonAD >= 0){
                inter = (a+this.BalloonDoubleRewardConst)/(this.triggerBalloonAD+2*this.BalloonDoubleRewardConst);
            }
            return inter;
        }

        getADRate(){
            let inter = this.getADInter();
            for(let i=0;i<this.BalloonDoubleRewardRate.length;i++){
                if(inter <= this.BalloonDoubleRewardRate[i].inter){
                    return this.BalloonDoubleRewardRate[i].rate;
                }
            }
        }

        updateADdata(i){
            if(this.triggerBalloonAD < 10){
                this.triggerBalloonAD++;
                Laya.LocalStorage.setJSON("triggerBalloonAD",this.triggerBalloonAD);
            }
            this.watchBalloonAD.splice(0,1);
            this.watchBalloonAD.push(i);
            let str = JSON.stringify(this.watchBalloonAD);
            Laya.LocalStorage.setJSON("watchBalloonAD",str);
        }

        setConstData(){
            this.balloonMaxNum=0;
            this.balloonStarLimit = Const.getArray("BalloonStarLimit");
            this.arr = Const.getArray("BalloonCreatingGapsDaily");
            this.amount1= new Array();
            this.amount2= new Array();
            this.Gaps1= new Array();
            this.Gaps2= new Array();
            for(let i=0;i<this.arr.length;i++){
                this.amount1[i]=this.arr[i].Amount[0];
                this.amount2[i]=this.arr[i].Amount[1];
                this.Gaps1[i]=this.arr[i].Gaps[0];
                this.Gaps2[i]=this.arr[i].Gaps[1];
                if(this.balloonMaxNum<this.amount2[i]){
                    this.balloonMaxNum=this.amount2[i];
                }
            }
            this.BalloonDoubleRewardConst = JSON.parse(Const.get("BalloonDoubleRewardConst"));
            this.BalloonDoubleRewardRate = Const.getArray("BalloonDoubleRewardRate");
        }

        getBalloonSpeed(){
            return 50;
        }

        getBalloonMaxNum(){
            return this.balloonMaxNum;
        }

        getBalloonStarLimit(){
            return this.balloonStarLimit;
        }

        getNextTime(){
            for(var i=0;i<this.arr.length;i++){
                if(this.amount1[i]<=this.balloonNum+1&&this.balloonNum+1<=this.amount2[i]){
                    var time=Math.random();
                    time=time*(this.Gaps2[i]-this.Gaps1[i])+this.Gaps1[i];
                    break;
                }
            }
            if(i<this.arr.length){
                return time*1000|0;
            }
            else {
                EventMgr.getInstance().postEvent("BALLOON_DESTROY");
                return 100000000000000;
            }
        }

        getLaterTime(){
            return 30000;
        }

        setBalloonNum(num){
            this.balloonNum=num;
        }

        addBalloonNum(){
            this.balloonNum+=1;
        }

        getBalloonNum(){
            return this.balloonNum;
        }
        
    }

    class BalloonSystem{
        static beginBalloon(num){
            BalloonDataMgr.getInstance().setBalloonNum(num);
            this.balloonStarLimit=BalloonDataMgr.getInstance().getBalloonStarLimit();
            this.balloonMaxNum=BalloonDataMgr.getInstance().getBalloonMaxNum();
        }

        static onBalloonClick(){
            let data = Laya.Handler.create(this, this.sendBalloonClick, [Constants.FetchType.DIRECT]);
            EventMgr.getInstance().postEvent("SERVICE_SEND_BALLOON",[Constants.FetchType.DIRECT,data]);
        }

        static onADBalloonClick(type){
            let data = Laya.Handler.create(this, this.sendBalloonClick, [Constants.FetchType.AD]);
            if(type){
                BalloonSystem.updateADdata(1);
                EventMgr.getInstance().postEvent("SERVICE_SEND_BALLOON",[Constants.FetchType.AD,data]);
            }
            else {
                BalloonSystem.updateADdata(0);
                EventMgr.getInstance().postEvent("SERVICE_SEND_BALLOON",[Constants.FetchType.DIRECT,data]);
            }
        }

        static sendBalloonClick(fetchType, balloonNum, fishReward) {
            if (balloonNum != this.getBalloonNum()) {
                BalloonDataMgr.getInstance().setBalloonNum(balloonNum);
            }
            EventMgr.getInstance().postEvent("BALLOON_FLY_YUGAN", [fishReward, fetchType]);
        }

        static updateADdata(i){
            BalloonDataMgr.getInstance().updateADdata(i);
        }

        static checkWatchAD(){
            let rate = BalloonDataMgr.getInstance().getADRate() / 100;
            let r = Math.random();
            return r <= rate ;
        }

        static setRewardNum(num){
            this.rewardNum = num;
        }

        static getRewardNum(){
            return this.rewardNum;
        }

        static getBalloonNum(){
            return BalloonDataMgr.getInstance().getBalloonNum();
        }

        static getNextTime(){
            return BalloonDataMgr.getInstance().getNextTime();
        }

        static getLaterTime(){
            return BalloonDataMgr.getInstance().getLaterTime();
        }

        static getBalloonSpeed(){
            return BalloonDataMgr.getInstance().getBalloonSpeed();
        }

        static setSceneLicense(num){
            this.sceneLicense=num;
        }

        static addBalloonNum(){
            BalloonDataMgr.getInstance().addBalloonNum();
        }

        static getSceneLicense(){
            return this.sceneLicense;
        }

        static getBalloonLicense(){
            this.balloonNum=BalloonDataMgr.getInstance().getBalloonNum();
            let starNum=PlayerDataMgr.getInstance().getStar();
            if(starNum>=this.balloonStarLimit&&this.balloonNum<this.balloonMaxNum){
                return 1;
            }
            else {
                return 0;
            }
        }

        static balloonSwitch(){
            if (GameContext.isGuiding) {
                return 0;
            }
            if(BalloonSystem.getSceneLicense()&&BalloonSystem.getBalloonLicense()){
                return 1;
            }
            else {
                return 0;
            }
        }
    }

    class GameLoaderTask {
        constructor() {
            this.taskInfo = null;
            this.completeHandler = null;
            this.state = 0; //0 等待 1工作中 2完成
        }

        start() {
            this.state = 1;
            this.processTask();
        }

        setTaskInfo(taskInfo) {
            this.taskInfo = taskInfo;
        }

        processTask() {
            // console.debug("开始任务------------>");
            // console.debug(this.taskInfo);
            if(this.taskInfo.type == "scenePath") {
                ScenePathMgr.getInstance().load();
                this.state = 2;
                return;
            } else if (this.taskInfo.type == "config") {
                ConfigMgr$1.getInstance().loadConfig(this.taskInfo.source1, Laya.Handler.create(this, function(){
                    console.debug("加载数据表:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
                return;
            } else if (this.taskInfo.type == "image") {
                Laya.loader.load([{ url: this.info.source1, type: Laya.Loader.IMAGE}], Laya.Handler.create(this, function(){
                    // console.debug("加载atlas:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
            } else if (this.taskInfo.type == "image1") {
                Laya.loader.load([{ url: this.info.source1, type: Laya.Loader.IMAGE}], Laya.Handler.create(this, function(){
                    // console.debug("加载atlas:" + String(this.taskInfo.source1));
                    this.state = 2;
                }), null, null, 1, false, null, true);
            } else if (this.taskInfo.type == "atlas") {
                // Laya.Loader.load()
                Laya.loader.load([{ url: this.taskInfo.source1, type: Laya.Loader.ATLAS }], Laya.Handler.create(this, function(){
                    // console.debug("加载atlas:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
            } else if (this.taskInfo.type == "spine") {
                Laya.loader.load([{ url: this.taskInfo.source1, type: Laya.Loader.BUFFER }], Laya.Handler.create(this, function(){
                    // console.debug("加载文件夹:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
            } else if (this.taskInfo.type == "sound") {
                // Laya.loader.load([{ url: this.taskInfo.source1, type: Laya.Loader.SOUND }], Laya.Handler.create(this, function(){
                    // console.debug("加载文件夹:" + String(this.taskInfo.source1));
                    this.state = 2;
                // }));
            } else if (this.taskInfo.type == "scene") {
                Laya.loader.load([{ url: this.taskInfo.source1, type: Laya.Loader.JSON }], Laya.Handler.create(this, function(){
                    // console.debug("加载文件夹:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
            } else if (this.taskInfo.type == "dir") {
                let assets = [];
                for(const info of this.taskInfo.files) {
                    switch(info.type) {
                        case "image":
                            assets.push({ url: info.source1, type: Laya.Loader.IMAGE});
                            break;
                        case "atlas":
                            assets.push({ url: info.source1, type: Laya.Loader.ATLAS});
                            break;
                        case "spine":
                            assets.push({ url: info.source1, type: Laya.Loader.BUFFER});
                            break;
                        case "sound":
                            // assets.push({ url: info.source1, type: Laya.Loader.SOUND});
                            break;
                    }
                }
                Laya.loader.load(assets, Laya.Handler.create(this, function(){
                    // console.debug("加载文件夹:" + String(this.taskInfo.source1));
                    this.state = 2;
                }));
            } else if (this.taskInfo.type == "login") {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_login_start"]);
                EventMgr.getInstance().postEvent("SERVICE_LOGIN", [""]);
                this.state = 2;
            }
        }

        getState() {
            return this.state;
        }
    }

    class GameLoaderTaskMgr {
        constructor() {
            this.taskInfos = new Array();
            this.processHandler = null;
            this.completeHandler = null;
            this.taskIndex = 0;
            this.taskTotal = 0;
            this.taskIndexWeight = 0;
            this.taskTotalWeight = 0;
        }

        addTaskInfo(task, weight = 1) {
            let taskInfo = new GameLoaderTask();
            taskInfo.setTaskInfo(task);
            taskInfo.weight = weight;
            this.taskInfos.push(taskInfo);
            this.taskTotal = this.taskInfos.length;
            this.taskTotalWeight += weight;
        }

        setHandler(processHandler, completeHandler) {
            this.processHandler = processHandler;
            this.completeHandler = completeHandler;
        }

        update() {
            if(this.taskInfos.length != 0) {
                let taskInfo = this.taskInfos[0];
                if(taskInfo.getState() == 1) {
                    return;
                }
                if(taskInfo.getState() == 2) {
                    this.taskIndex ++;
                    this.taskIndexWeight+=taskInfo.weight;
                    this.taskInfos.splice(0, 1);
                    // console.debug("结束任务------------>");
                    // console.debug(taskInfo);
                    if(this.processHandler) {
                        this.processHandler.runWith([this.taskIndexWeight, this.taskTotalWeight, this.taskIndex, this.taskTotal]);
                    }
                    return;
                }
                if(taskInfo.getState() == 0) {
                    taskInfo.start();
                }
            } else {
                if(this.completeHandler) {
                    this.completeHandler.run();
                }
                Laya.timer.clear(this, this.update);
            }
        }

        start() {
            Laya.timer.loop(1, this, this.update);
        }
    }

    var loads = [{"type": "image", "source1": "res/cat_collect/gray.png"}, {"type": "image", "source1": "res/cat_collect/head.png"}, {"type": "image", "source1": "res/cat_collect/normal.png"}, {"type": "image", "source1": "res/collect.png"}, {"type": "image", "source1": "res/fortune.png"}, {"type": "image", "source1": "anim1/duoyugan.png"}, {"type": "spine", "source1": "anim1/duoyugan.sk"}, {"type": "image", "source1": "anim1/fudaidonghua.png"}, {"type": "spine", "source1": "anim1/fudaidonghua.sk"}, {"type": "image", "source1": "anim1/hailangdonghua.png"}, {"type": "spine", "source1": "anim1/hailangdonghua.sk"}, {"type": "image", "source1": "anim1/jianzao1.png"}, {"type": "spine", "source1": "anim1/jianzao1.sk"}, {"type": "image", "source1": "anim1/kanbanniang.png"}, {"type": "spine", "source1": "anim1/kanbanniang.sk"}, {"type": "image", "source1": "anim1/loading2.png"}, {"type": "spine", "source1": "anim1/loading2.sk"}, {"type": "image", "source1": "anim1/mao1.png"}, {"type": "spine", "source1": "anim1/mao1.sk"}, {"type": "image", "source1": "anim1/mao2.png"}, {"type": "spine", "source1": "anim1/mao2.sk"}, {"type": "image", "source1": "anim1/maomizengli.png"}, {"type": "spine", "source1": "anim1/maomizengli.sk"}, {"type": "image", "source1": "anim1/Rmao1.png"}, {"type": "spine", "source1": "anim1/Rmao1.sk"}, {"type": "image", "source1": "anim1/Rmao2.png"}, {"type": "spine", "source1": "anim1/Rmao2.sk"}, {"type": "image", "source1": "anim1/UIdonghua.png"}, {"type": "spine", "source1": "anim1/UIdonghua.sk"}, {"type": "image", "source1": "anim1/xinshouyindao.png"}, {"type": "spine", "source1": "anim1/xinshouyindao.sk"}, {"type": "image", "source1": "anim1/zhaocaimao1.png"}, {"type": "spine", "source1": "anim1/zhaocaimao1.sk"}, {"type": "image", "source1": "anim1/zhaocaimao2.png"}, {"type": "spine", "source1": "anim1/zhaocaimao2.sk"}, {"type": "image", "source1": "anim2/caidai.png"}, {"type": "spine", "source1": "anim2/caidai.sk"}, {"type": "image", "source1": "anim2/caijinbimao.png"}, {"type": "spine", "source1": "anim2/caijinbimao.sk"}, {"type": "image", "source1": "anim2/caiquan.png"}, {"type": "spine", "source1": "anim2/caiquan.sk"}, {"type": "image", "source1": "anim2/caiquanji.png"}, {"type": "spine", "source1": "anim2/caiquanji.sk"}, {"type": "image", "source1": "anim2/caiquannpc.png"}, {"type": "spine", "source1": "anim2/caiquannpc.sk"}, {"type": "image", "source1": "anim2/diaoyu.png"}, {"type": "spine", "source1": "anim2/diaoyu.sk"}, {"type": "image", "source1": "anim2/diaoyuchangjing.png"}, {"type": "spine", "source1": "anim2/diaoyuchangjing.sk"}, {"type": "image", "source1": "anim2/duibo.png"}, {"type": "spine", "source1": "anim2/duibo.sk"}, {"type": "image", "source1": "anim2/haogandu.png"}, {"type": "spine", "source1": "anim2/haogandu.sk"}, {"type": "image", "source1": "anim2/jiayou.png"}, {"type": "spine", "source1": "anim2/jiayou.sk"}, {"type": "image", "source1": "anim2/jiayuanhudong.png"}, {"type": "spine", "source1": "anim2/jiayuanhudong.sk"}, {"type": "image", "source1": "anim2/jiesuodonghua.png"}, {"type": "spine", "source1": "anim2/jiesuodonghua.sk"}, {"type": "image", "source1": "anim2/lumao.png"}, {"type": "spine", "source1": "anim2/lumao.sk"}, {"type": "image", "source1": "anim2/mao3.png"}, {"type": "spine", "source1": "anim2/mao3.sk"}, {"type": "image", "source1": "anim2/NPCmao.png"}, {"type": "spine", "source1": "anim2/NPCmao.sk"}, {"type": "image", "source1": "anim2/qiqiumao.png"}, {"type": "spine", "source1": "anim2/qiqiumao.sk"}, {"type": "image", "source1": "anim2/renwumao.png"}, {"type": "spine", "source1": "anim2/renwumao.sk"}, {"type": "image", "source1": "anim2/renzhemao.png"}, {"type": "spine", "source1": "anim2/renzhemao.sk"}, {"type": "image", "source1": "anim2/Rmao3.png"}, {"type": "spine", "source1": "anim2/Rmao3.sk"}, {"type": "image", "source1": "anim2/xiaoxuemao.png"}, {"type": "spine", "source1": "anim2/xiaoxuemao.sk"}, {"type": "image", "source1": "anim2/yinghua.png"}, {"type": "spine", "source1": "anim2/yinghua.sk"}, {"type": "image", "source1": "anim2/yutangdonghua.png"}, {"type": "spine", "source1": "anim2/yutangdonghua.sk"}, {"type": "image", "source1": "res/homeland/ui.png"}, {"type": "image", "source1": "res/homeland.png"}, {"type": "image", "source1": "res/miniGame.png"}, {"type": "image", "source1": "component_coffee2/bihua_3.png"}, {"type": "image", "source1": "component_coffee2/chuanghu_3.png"}, {"type": "image", "source1": "component_coffee2/denglong_3.png"}, {"type": "image", "source1": "component_coffee2/ditan_3.png"}, {"type": "image", "source1": "component_coffee2/guitai_3.png"}, {"type": "image", "source1": "component_coffee2/maojia_3.png"}, {"type": "image", "source1": "component_coffee2/mazhuanao1_3.png"}, {"type": "image", "source1": "component_coffee2/mazhuanao2_3.png"}, {"type": "image", "source1": "component_coffee2/mendian_3.png"}, {"type": "image", "source1": "component_coffee2/shugui_3.png"}, {"type": "image", "source1": "component_coffee2/xiaojing_3.png"}, {"type": "image", "source1": "component_coffee2/zhalan_3.png"}, {"type": "image", "source1": "component_coffee2/zuodian1_3.png"}, {"type": "image", "source1": "component_coffee2/zuoyi_3.png"}, {"type": "image", "source1": "component_beach1/diaoyu_0.png"}, {"type": "image", "source1": "component_beach1/diaoyu_1.png"}, {"type": "image", "source1": "component_beach1/diaoyu_2.png"}, {"type": "image", "source1": "component_beach1/maojin_1.png"}, {"type": "image", "source1": "component_beach1/maojin_2.png"}, {"type": "image", "source1": "component_beach1/paiqiu_1.png"}, {"type": "image", "source1": "component_beach1/paiqiu_2.png"}, {"type": "image", "source1": "component_beach1/qidian_1.png"}, {"type": "image", "source1": "component_beach1/qidian_2.png"}, {"type": "image", "source1": "component_beach1/san_1.png"}, {"type": "image", "source1": "component_beach1/san_2.png"}, {"type": "image", "source1": "component_beach1/shadiao_1.png"}, {"type": "image", "source1": "component_beach1/shadiao_2.png"}, {"type": "image", "source1": "component_beach1/shadui_1.png"}, {"type": "image", "source1": "component_beach1/shahua_1.png"}, {"type": "image", "source1": "component_beach1/shahua_2.png"}, {"type": "image", "source1": "component_beach1/waiqiang_1.png"}, {"type": "image", "source1": "component_beach1/waiqiang_2.png"}, {"type": "image", "source1": "component_beach1/yizi_1.png"}, {"type": "image", "source1": "component_beach1/yizi_2.png"}, {"type": "image", "source1": "res/nautical_food.png"}, {"type": "image", "source1": "res/fishing_game/baitShadow.png"}, {"type": "image", "source1": "res/fishing_game/shadow.png"}, {"type": "image", "source1": "res/fishing_game/bait.png"}, {"type": "image", "source1": "res/fishing_game/ui.png"}, {"type": "image", "source1": "res/mail.png"}, {"type": "image", "source1": "res/camera.png"}];

    class InGameLoaderTaskMgr extends GameLoaderTaskMgr {

        constructor() {
            super();
            this._frameCounter = 0;
            this._frameInterval = 3;
        }

        update() {
            if(++this._frameCounter>=this._frameInterval) {
                this._frameCounter = 0;
                super.update();
            }
        }

        start() {
            Laya.timer.frameLoop(1, this, this.update);
        }
    }


    class InGameLoader {

        constructor() {
        }

        static getInstance() {
            return InGameLoader.instance?InGameLoader.instance:InGameLoader.instance = new InGameLoader();
        }


        start(handler) {
            let loader = new InGameLoaderTaskMgr();
            let i = 0;
            let div = 2;
            let dirTask;
            for(const info of loads) {
                if(i%div==0) {
                    dirTask = {type: "dir", files:[info]};
                    loader.addTaskInfo(dirTask);
                }else {
                    dirTask.files.push(info);
                }
                i++;
            }
            loader.setHandler(Laya.Handler.create(this, function(taskIndex) {
                console.debug("InGameLoader task : " + String(taskIndex) + "/" + String(i/div));
            }, null, false),Laya.Handler.create(this, function() {
                console.log("InGameLoader done");
                if(handler)
                    handler.run();
            }));
            loader.start();
        }

    }

    var loads$1 = [{"type": "image", "source1": "res/sc_coffee.png"}, {"type": "image", "source1": "sc_coffee1/leftqiang.png"}, {"type": "image", "source1": "sc_coffee1/rightqiang.png"}, {"type": "image", "source1": "res/skin_coffee.png"}, {"type": "image", "source1": "component_coffee1/bihua_1.png"}, {"type": "image", "source1": "component_coffee1/bihua_2.png"}, {"type": "image", "source1": "component_coffee1/chuanghu_1.png"}, {"type": "image", "source1": "component_coffee1/chuanghu_2.png"}, {"type": "image", "source1": "component_coffee1/denglong_1.png"}, {"type": "image", "source1": "component_coffee1/denglong_2.png"}, {"type": "image", "source1": "component_coffee1/ditan_1.png"}, {"type": "image", "source1": "component_coffee1/ditan_2.png"}, {"type": "image", "source1": "component_coffee1/eatBar.png"}, {"type": "image", "source1": "component_coffee1/eatBar1.png"}, {"type": "image", "source1": "component_coffee1/eatBarBg.png"}, {"type": "image", "source1": "component_coffee1/guitai_1.png"}, {"type": "image", "source1": "component_coffee1/guitai_2.png"}, {"type": "image", "source1": "component_coffee1/letter_box_dialog.png"}, {"type": "image", "source1": "component_coffee1/maojia_1.png"}, {"type": "image", "source1": "component_coffee1/maojia_2.png"}, {"type": "image", "source1": "component_coffee1/mazhuanao1_1.png"}, {"type": "image", "source1": "component_coffee1/mazhuanao1_2.png"}, {"type": "image", "source1": "component_coffee1/mazhuanao2_1.png"}, {"type": "image", "source1": "component_coffee1/mazhuanao2_2.png"}, {"type": "image", "source1": "component_coffee1/meals_bg.png"}, {"type": "image", "source1": "component_coffee1/mendian_1.png"}, {"type": "image", "source1": "component_coffee1/mendian_2.png"}, {"type": "image", "source1": "component_coffee1/shugui_1.png"}, {"type": "image", "source1": "component_coffee1/shugui_2.png"}, {"type": "image", "source1": "component_coffee1/xiaojing_1.png"}, {"type": "image", "source1": "component_coffee1/xiaojing_2.png"}, {"type": "image", "source1": "component_coffee1/zhalan_1.png"}, {"type": "image", "source1": "component_coffee1/zhalan_2.png"}, {"type": "image", "source1": "component_coffee1/zuodian1_1.png"}, {"type": "image", "source1": "component_coffee1/zuodian1_2.png"}, {"type": "image", "source1": "component_coffee1/zuoyi_1.png"}, {"type": "image", "source1": "component_coffee1/zuoyi_2.png"}, {"type": "image", "source1": "res/topui.png"}, {"type": "image", "source1": "bg/beijingtu.jpg"}, {"type": "image", "source1": "bg/coffee_bg.jpg"}, {"type": "image", "source1": "bg/collect_bg.jpg"}, {"type": "image", "source1": "bg/ditu_beach.jpg"}, {"type": "image", "source1": "bg/nautical_bg.jpg"}, {"type": "image", "source1": "bg/shopbeijing.jpg"}, {"type": "image", "source1": "res/shopitems.png"}, {"type": "image", "source1": "res/scene_nautical.png"}, {"type": "image", "source1": "res/shopui.png"}, {"type": "image", "source1": "res/common.png"}, {"type": "image", "source1": "res/cookbook.png"}, {"type": "image", "source1": "res/facility.png"}, {"type": "image", "source1": "res/guide.png"}, {"type": "image", "source1": "res/menu.png"}, {"type": "image", "source1": "res/smallpop.png"}, {"type": "image", "source1": "res/task.png"}, {"type": "image", "source1": "res/tip.png"}];

    class TransitionGameLoaderTaskMgr extends GameLoaderTaskMgr {

        constructor() {
            super();
            this._frameCounter = 0;
            this._frameInterval = 3;
        }

        update() {
            if(++this._frameCounter>=this._frameInterval) {
                this._frameCounter = 0;
                super.update();
            }
        }

        start() {
            Laya.timer.frameLoop(1, this, this.update);
        }
    }


    class TransitionGameLoader {

        constructor() {
        }

        static getInstance() {
            return TransitionGameLoader.instance?TransitionGameLoader.instance:TransitionGameLoader.instance = new TransitionGameLoader();
        }


        start(handler) {
            let loader = new TransitionGameLoaderTaskMgr();
            let i = 0;
            let div = 20;
            let dirTask;
            for(const info of loads$1) {
                if(i%div==0) {
                    dirTask = {type: "dir", files:[info]};
                    loader.addTaskInfo(dirTask);
                }else {
                    dirTask.files.push(info);
                }
                i++;
            }
            loader.setHandler(Laya.Handler.create(this, function(taskIndex) {
                console.debug("TransitionGameLoader task : " + String(taskIndex) + "/" + String(i/div));
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_resources_"+ String(taskIndex) + "/" + String(Math.ceil(i/div))]);
            }, null, false),Laya.Handler.create(this, function() {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_resources_done"]);
                console.log("TransitionGameLoader done");
                if(handler)
                    handler.run();
            }));
            console.log("TransitionGameLoader");
            loader.start();
        }

    }

    class LuckyBagSystem {

        static init() {
        }

        static update(luckyData) {
            LuckyBagDataMgr.getInstance().update(luckyData);
        }

        static receiveLuckyBag(fetchType, handler) {
            EventMgr.getInstance().postEvent("SERVICE_RECEIVE_LUCKY_BAG", [fetchType, handler]);
        }

        static queryLuckyBagInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_QUERY_LUCKY_BAG_INFO", [handler]);
        }

        static checkCloseluckyBag(){
            LuckyBagDataMgr.getInstance().checkCloseluckyBag();
        }

        static checkDaysReceiveType() {
            let day = LuckyBagDataMgr.getInstance().getRegisterDays();
            let luckBagMaxDays = Number(Const.get("LuckBagMaxDays"));
            if (luckBagMaxDays < day) {
                return false;
            }
            return true;
        }

        static checkStarReceiveType() {
            let limitStar = Number(Const.get("LuckBagStarsCondition"));
            if (limitStar > PlayerSystem.getStar()) {
                LuckyBagDataMgr.getInstance().setLock(1);
                return false;
            }
            LuckyBagDataMgr.getInstance().setLock(0);
            return true;
        }

        static getRegisterDays(){
            return LuckyBagDataMgr.getInstance().getRegisterDays();
        }

        static getOnlineTime(){
            return LuckyBagDataMgr.getInstance().getOnlineTime();
        }

        static getRemainingTime(){
            return LuckyBagDataMgr.getInstance().getRemainingTime();
        }

        static getReward(){
            return LuckyBagDataMgr.getInstance().getReward();
        }

        static createDataMgr(){
            LuckyBagDataMgr.getInstance();
        }

        static checkReceiveTimes(){
            return LuckyBagDataMgr.getInstance().checkReceiveTimes();
        }

        static setStartTime(){
            LuckyBagDataMgr.getInstance().setStartTime();
        }

        static clear(){
            LuckyBagDataMgr.getInstance().clear();
        }

    }

    class SignSystem {
        static init() {
            EventMgr.getInstance().registEvent("SERVICE_SIGN_INFN", this, SignSystem.getSignInfo);
            EventMgr.getInstance().registEvent("CHECK_SIGN_AUTO_POP", this, SignSystem.checkAutoPop);
            SignSystem.qiandaotanchu = 0;
        }

        static clear() {
            SignSystem.qiandaotanchu = 0;
        }

        static getSignData(id){
           return ConfigMgr$1.getInstance().getItemById("Sign",id);
        }
        static getReward (id){
            let reward=ConfigMgr$1.getInstance().getItemById("Sign",id).Reward;
            return JSON.parse(reward);
        }
        static lastSignTime(){
            if(this._lastSignTime){
                return this._lastSignTime/1000;
            }else {
                return 0;
            }    
        }
        static getSignDays(){
            if(this._getSignDays){
                return this._getSignDays;
            }else {
                return 0;
            }  
        }

        static getSignInfo(qiandao) {
            this._qiandao = qiandao;
            let dealFunc = function (res) {
                let info = res.signInfo;
                this._lastSignTime = info.lastSignTime;
                this._getSignDays = info.signTimes;
                EventMgr.getInstance().postEvent("SHOW_SIGNBTN");
                EventMgr.getInstance().postEvent("SHOW_SIGNPOINT");
                if (this._qiandao == 0) {
                    EventMgr.getInstance().postEvent("CHECK_AUTO_POP_ALL");
                }
                if (this._getSignDays == 7) {
                    GameContext.signBtnState = false;
                    EventMgr.getInstance().postEvent("UnShowSignBtn_Event", true);
                }
                EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["sign"]);
            };
            Service.getSignIngo(Laya.Handler.create(this, dealFunc));
        }
        //根据比较上一次的签到时间来进行判断，如果上次时间 在5点之前 可以签到，否则不行 时间戳起始 并不是24点 
        //而是8点  所以进行计算的时候 取余24小时并不准确，需要0-8点 需要做处理

        static ifQiandao(){
            let nowTimes=Math.floor(TimeUtils.getTimeNowOnServer()/1000)-3600*8;
            let wudian=0;
            //拿到5点的时间戳，但当心拿成上一天的
            if((nowTimes-nowTimes%86400)>57600){
                wudian=(nowTimes+28800)-(nowTimes+28800)%86400-28800+18000;
            }
            else {
                wudian=nowTimes-nowTimes%86400-28800+18000;
            }
            let c=nowTimes%86400;
            if((nowTimes%86400)>=(16*3600)&&(nowTimes%86400)<(21*3600)){
               wudian=wudian-24*3600;
            }
            if(SignSystem.lastSignTime()>wudian||SignSystem.getSignDays()>=7){
                return true;
            }else {
                return false;
            }
        }

        static showRedPoint() {
            return PlayerDataMgr.getInstance().getStar() >= Const.get('SignSystemOpenCondition') && !SignSystem.ifQiandao();
        }

        static qiandao(){
            let dealFunc = function(res) {
                if(res.bag.currency.FISH){
                    PlayerDataMgr.getInstance().updateFish(res.bag.currency.FISH.amount);
                }
                if(res.bag.currency.STAR){
                    PlayerDataMgr.getInstance().updateStar(res.bag.currency.STAR.amount);
                }
                if(res.bag.currency.SHELL){
                    PlayerDataMgr.getInstance().updateShell(res.bag.currency.SHELL.amount);
                }
                if(res.bag.currency.DIAMOND){
                    PlayerDataMgr.getInstance().updateDiamond(res.bag.currency.DIAMOND.amount);
                }
                    SignSystem.getSignInfo(1);
            };
               Service.GetSignRewardReq(Laya.Handler.create(this, dealFunc));
        }

        static checkAutoPop() {
            if(PlayerDataMgr.getInstance().getStar()>=Const.get('SignSystemOpenCondition')&&!SignSystem.ifQiandao()){
                if( SignSystem.qiandaotanchu==0){
                    SignSystem.qiandaotanchu = 1;
                    EventMgr.getInstance().postEvent("DIALOG_CHANGE_STATUS",[Constants.DialogStatus.Sign,1]);
                    Laya.Dialog.open("ui/Sign.scene", false,[1]);      
                }  
             }
        }
    }

    class ProjectData {

        constructor(config, level,listType,listOrder) {
            this._level = level;
            this._unlocked = false;
            this._config = config;
            this._listType=listType;
            this._listOrder=listOrder;
        }

        unlock() {
            this._unlocked = true;
        }

        isLocked() {
            return !this._unlocked;
        }

        getId() {
            return this._config.Id;
        }

        getName() {
            return "tid#Name" + this.getId();
        }

        getDesc1() {
            return "tid#Desc" + this.getId();
        }

        get Source() {
            return this._config.Source;
        }

        getShopItems() {
            return "nautical_project/" + this._config.ShopItems + ".png";
        }

        getType() {
            return this._config.Type;
        }
        getWeight() {
            return this._config.Weight;
        }

        getBuildingLimit() {
            return this._config.BuildingLimit;
        }

        getStarLimit() {
            return this._config.StarLimit;
        }

        getCost() {
            return this._config.Cost;
        }
        getAutoIncome(){
            return this._config.AutoIncome;
        }
        
        getSpecialTpye() {
            return this._config.SpecialTpye;
        }

        getSpecialValue() {
            return this._config.SpecialValue;
        }

        getLevel() {
            return this._level;
        }

        // setLevel(level) {
        //     this._level = level;
        // }

        clear() {
            this._unlocked = false;
        }
        getlistOrder(){
            return this._listOrder;
        }
        getlistType(){
            return this._listType;
        }
    }

    class ProjectDataMgr {

        constructor() {
            this._projectMap = new Map();
            this.projectListMap = new Map();
            this.youlechang = new Map();
            this.initProjects();
        }

        static getInstance() {
            return ProjectDataMgr.instance?ProjectDataMgr.instance:ProjectDataMgr.instance = new ProjectDataMgr();
        }

        //初始化点位
        initProjects() {
            let config = ConfigMgr$1.getInstance().getConfig("NauticalProjectList");
            let projectConfig = ConfigMgr$1.getInstance().getConfig("NauticalProject");
            let outSelf = this;
            config.forEach(function (item) {
                let projectList = new Array();
                let list = JSON.parse(item.ProjectList);
                let listType=JSON.parse(item.Type);
                let listOrder=JSON.parse(item.Order);
                list.forEach((projectInfo)=>{
                    let projectId = String(projectInfo["ProjectId"]);
                    let projectLv = Number(projectInfo["lv"]);
                    let project = new ProjectData(projectConfig.get(projectId), projectLv,listType,listOrder);
                    outSelf._projectMap.set(projectId, project);
                    projectList.push(project);
                });
                projectList.sort(function(a,b){
                    return a.getLevel()-b.getLevel();
                });
                outSelf.projectListMap.set(item["Id"], projectList);
                outSelf.youlechang.set(item["Id"], projectList[0]);
            });
        }

        clear() {
            this._projectMap.forEach((project)=>{
                project.clear();
            });
            this._projectMap.clear();
            this.projectListMap.clear();
            this.youlechang.clear();
            this.initProjects();
        }

        getProject(id) {
            return this._projectMap.get(id);
        }

        getProjectList(projectListId) {
            return this.projectListMap.get(projectListId);
        }

        getCurProjectInList(listId) {
            return this.youlechang.get(String(listId));
        }

        updateProject(projectListId, level) {
            let list = this.projectListMap.get(projectListId);
            let project = this.youlechang.get(projectListId);
            if(project.isLocked()&&project.getLevel()==level){
                project.unlock();
                return true;
            }else if(project.getLevel()!=level) {
                let project = list[level - 1];
                if(project){
                    this.youlechang.set(projectListId, project);
                    for(let i=0;i<=level-1;i++){
                        list[i].unlock();
                    }
                    return true;
                }
            }
            return false;
        }

        getNextLevelUpProjectInList(projectListId) {
            let project = this.getCurProjectInList(projectListId);
            if(project.isLocked()){
                return project;
            }else {
                let list = this.getProjectList(projectListId);
                return list[project.getLevel()];
            }
        }
        getyoulechangList(){
            return this.youlechang;
        }
        getType(projectId){
            let type=this.youlechang.get(String(projectId)).getType();
            return type;
        }
        getlistType(projectId){
            let type=this.youlechang.get(String(projectId)).getlistType();
            return type;
        }
        
    }

    class ProjectSystem {
        static init() {
        }

        static clear() {
            ProjectDataMgr.getInstance().clear();
        }

        static updateProjects(projects) {
            if (projects.group) {
                let needUpdate = false;
                Object.getOwnPropertyNames(projects.group).forEach(projectsid => {
                    let ProjectId = projects.group[projectsid];
                    let project=ProjectSystem.getProject(ProjectId);
                    let level=project._level;
                    needUpdate = ProjectDataMgr.getInstance().updateProject(projectsid, level) || needUpdate;
                });
                if (needUpdate) {
                    EventMgr.getInstance().postEvent("UPDATE_PROJECTGROUP");
                }
            }
        }

        static getProject(ProjectId) {
            return ProjectDataMgr.getInstance().getProject(ProjectId);
        }

        static getProjectList(listId) {
            return ProjectDataMgr.getInstance().getProjectList(listId);
        }

        static getCurProjectInList(listId) {
            return ProjectDataMgr.getInstance().getCurProjectInList(listId);
        }
        static getyoulechangList() {
            return ProjectDataMgr.getInstance().getyoulechangList();
        }

        static getNextLevelUpProjectInList(listId) {
            return ProjectDataMgr.getInstance().getNextLevelUpProjectInList(listId);
        }

        static checkLevelUpUnAvailable(listId) {
            let project = ProjectSystem.getNextLevelUpProjectInList(listId);
            if (!BuildingSystem.hasBuildingDataByBid(project.getBuildingLimit())) {   
                let pid=project.getBuildingLimit();
                return Strings$1.get('tid#UI_059').replace('%1%',Strings$1.get("tid#BuildingName_"+pid));
            }
            else if (project && project.getStarLimit() > PlayerSystem.getStar()) {
                return  Strings$1.get('tid#UI_060');
            } else if (project && project.getCost() > PlayerSystem.getShell()) {
                //return "还差" + String(project.getCost() - PlayerSystem.getShell()) + "贝壳可升级";
                return Strings$1.get('tid#UI_050',[String(project.getCost() - PlayerSystem.getShell())])
            }
            return false;
        }

        static checkProjectRedPoint(listId) {
            let project = ProjectSystem.getNextLevelUpProjectInList(listId);
            if (project && BuildingSystem.hasBuildingDataByBid(project.getBuildingLimit())&&project.getStarLimit() <= PlayerSystem.getStar() && project.getCost() <= PlayerSystem.getShell() && PlayerSystem.getStar() < Number(Const.get("ShopRedDot"))) {
                return true;
            }
            // if (ProjectSystem.checkAdLevelUp(listId)) {
            //     return true;
            // }
            return false;
        }

        static checkLevelUpStarEnough(listId) {
            let project = ProjectSystem.getNextLevelUpProjectInList(listId);
            if (project && project.getStarLimit() <= PlayerSystem.getStar()) {
                return true;
            }
            return false;
        }

        static checkAdShowFish(listId) {
            let food = FoodSystem.getNextLevelUpFoodInList(listId);
            if (food == null) {
                return null;
            }
            let num = food.getCost() - PlayerSystem.getFish();
            if (num > 0 && food && food.getLimit() <= PlayerSystem.getStar() && food.getCost() > PlayerSystem.getFish()) {
                return [num,food.getCost()];
            }
            return null;
        }

        static getProjectGroupMenu(handler) {
            EventMgr.getInstance().postEvent("SERVICE_GET_PROJECT_GROUP", [handler]);
        }

        static projectLevelUp(projectListId, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith(res);
            };
            EventMgr.getInstance().postEvent("SERVICE_PROJECT_LEVELUP", [projectListId, Laya.Handler.create(null, dealFunc)]);
        }

        static checkProjectShopRedShow() {
            let max = Number(Const.get("ShopRedDot"));
            if (PlayerSystem.getStar() >= max) {
                return false;
            }
            let ret = ProjectSystem.checkProjectCanLevelUp(false)|| ProjectSystem.checkProjectCanBuy(false);
            return ret;
        }

        static checkProjectCanLevelUp(ifAD = false) {
            let youleMap = ProjectDataMgr.getInstance().getyoulechangList();
            let ret = false;
            youleMap.forEach((value, key) => {
                if (!value.isLocked()) {
                    let project = ProjectSystem.getNextLevelUpProjectInList(key);
                    if (project && project.getStarLimit() <= PlayerSystem.getStar()) {
                        if (project.getCost() <= PlayerSystem.getShell()) {
                            ret = true;
                            return;
                        } else {
                            if (ifAD && ProjectSystem.checkAdLevelUp(key)) {
                                ret = true;
                                return;
                            }
                        }
                    }
                }
            });
            return ret;
        }

        static checkProjectCanBuy(ifAD = false) {
            let youleMap = ProjectDataMgr.getInstance().getyoulechangList();
            let ret = false;
            youleMap.forEach((value, key) => {
                if (value.isLocked()) {
                    if(BuildingSystem.hasBuildingDataByBid(value.getBuildingLimit())){
                    if (value.getStarLimit() <= PlayerSystem.getStar()) {
                        if (value.getCost() <= PlayerSystem.getShell()) {
                            ret = true;
                            return;
                        } else {
                            if (ifAD && ProjectSystem.checkAdLevelUp(key)) {
                                ret = true;
                                return;
                            }
                        }
                    }}
                }
            });
            return ret;
        }

        static checkCanBuyCheapestFood() {
            let foodMap = ProjectDataMgr.getInstance().getAllFood();
            let food = null;
            foodMap.forEach((value, key) => {
                if (value.isLocked()) {
                    if(BuildingSystem.hasBuildingDataByBid(value.getBuildingLimit())){
                    if (value.getLimit() <= PlayerSystem.getStar() && value.getCost() <= PlayerSystem.getFish()) {
                        if (food != null && food.getCost() < value.getCost()) {
                            return;
                        }
                        food = value;
                    }
                }
                }
            });
            return food;
        }

        static checkLevelUpCheapestFood() {
            let foodMap = ProjectDataMgr.getInstance().getAllFood();
            let food = null;
            foodMap.forEach((value, key) => {
                if (!value.isLocked()) {
                    let nextFood = FoodSystem.getNextLevelUpFoodInList(key);
                    if (nextFood && nextFood.getLimit() <= PlayerSystem.getStar() && nextFood.getCost() <= PlayerSystem.getFish()) {
                        if (food != null && food.getCost < nextFood.getCost()) {
                            return;
                        }
                        food = value;
                    }
                }
            });
            return food;
        }

        static checkAdLevelUp(foodListId) {
            let needFish = FoodSystem.checkAdShowFish(foodListId) ? FoodSystem.checkAdShowFish(foodListId)[0] : 0;
            let costFish = FoodSystem.checkAdShowFish(foodListId) ? FoodSystem.checkAdShowFish(foodListId)[1] : 0;
            let yugan = costFish - needFish;
            let adShoppingCondition = Const.get("ADShoppingCondition");
            let maxAd = Number(Const.get("ADShoppingTimes"));
            let items = PetDataMgr.getInstance().getItemBySpecialType(4);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToyDataMgr.getInstance().getDataById(item.Id)[3]);
            });
            if (PlayerSystem.getMakeUpTimes() >= maxAd + itemAddTimes) {
                return false;
            }
            let adShoppingRange = JSON.parse(Const.get("ADShoppingRange"));
            let showAd = false;
            let buildingNum=BuildingDataMgr.getInstance().getNowBuildingNumber();
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].BUILDING_AMOUNT[0];
                let max = adShoppingRange[i].BUILDING_AMOUNT[1];
                if (max < 0) {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                } else {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                }
            }
            if (showAd) {
                return true;
            }
            return false;
        }
        static getType(projectid){
            return  ProjectDataMgr.getInstance().getType(projectid);
        }
        static getlistType(projectid){
            return ProjectDataMgr.getInstance().getlistType(projectid);
        }
        
    }

    class ZhuliData {

        constructor(id,endTimeStamp,skin,openId) {
          this.id=id;
          this.endTimeStamp=endTimeStamp;
          this.imageSkin=skin;
          this.openId=openId;

        }
        getId() {
            return this._config.Id;
        }
        getOpenId() {
            return this.openId;
        }
        clear() {
            this._unlocked = false;
        }
    }

    class ZhuliDataMgr {

        constructor() {
           this.showlist=new Map();
           this.zhuliFriendlist=new Map();
           this.cacheFish = 0;
           this.cacheShell= 0;
           this.shareFish = 0;
           this.shareShell = 0;
           this.zhulishangxian=false;
           this.biaoyuopenId='';
        }

        static getInstance() {
            return ZhuliDataMgr.instance?ZhuliDataMgr.instance:ZhuliDataMgr.instance = new ZhuliDataMgr();
        }
        updateZhuliList(updateZhuliList){
            let zhulilist=new Map();
            let max=this.getZhuliListLength();
            for(let i=0;i<max;i++){
           let id = String(i);
           let skin ="project/invintFriend.png";
           let openId='zhuli007';
           let zhuliInfo = new ZhuliData(id,null,skin,openId); 
           zhulilist.set(i,zhuliInfo);
       }
            let i=0;
            let i1=0;
            for (const key in updateZhuliList) {  
                if (updateZhuliList.hasOwnProperty(key)) {
                    const zhuliFriend = updateZhuliList[key];
                    let id= zhuliFriend.rid;
                    let endTimeStamp= zhuliFriend.endTimeStamp;
                    let skin ="project/head1.png";
                    let openId=zhuliFriend.openId;
                    if(endTimeStamp*1000>TimeUtils.getTimeNowOnServer()){
                    let zhuliInfo = new ZhuliData(id,endTimeStamp,skin,openId); 
                    zhulilist.set(i,zhuliInfo);
                    i1++;
                    i++;
                }
                }
            }
            this.zhulirenshu=i;
            let k=i1<2?i1:2;
            let showlist=new Map();
            for(let j=0;j<=k;j++){
                let zhuliinfo=zhulilist.get(j);   
                showlist.set(k-j,zhuliinfo);
            }
            this.showlist=showlist;//目前弃用了
            this.zhulilist=zhulilist;
            if(i>=this.getIfMaxzhuliTimes()){
                this.zhulishangxian=true;
            };
            EventMgr.getInstance().postEvent("UPDATE_ZHULI_LIST");
        }
        getZhuliList(){
            return this.zhulilist;
        }
        updateZhuXinxi(aidedInfo){
            if(aidedInfo){
           this.zhuliOverTime=aidedInfo.endTimeStamp;
           this.hasReceive=aidedInfo.hasReceive;
           this.zhuliName=aidedInfo.rid;
           this.aidTimes=aidedInfo.aidTimes;

        }
        }
        updateBiaoyu(slogan){
          if(slogan){
              this.biaoyuid=slogan.rid;
              this.biaoyuopenId=slogan.openId;
              this.biaoyuAward=slogan.shell;
              this.biaoyuType=slogan.type;
          }
        }
        getZhuliListLength(){
            let times=Number(Const.get('QuayAssistSameTimeLimit'));
            return times;
        }
        //是否到达助力上限
        getZhuliSX(){
            let times=Number(Const.get('QuayAssistTimes'));
            if(times<=Number(this.aidTimes)){
               return true;
            }  else {
               return false;
            }
        }
        getZhuliFriendList(){
          
        }
        setZhuliFriendList(){
          
        }
        getCapacity() {
            return this.capacity;
        }

        setCapacity(capacity) {
            this.capacity = capacity;
        }
        getCacheFish() {
            return this.cacheFish;
        }

        setCacheFish(fish) {
            this.cacheFish = fish;
        }
        getCacheShell() {
            return this.cacheShell;
        }

        setCacheShell(shell) {
            this.cacheShell = shell;
        }
        getShareFish() {
            return this.shareFish;
        }

        setShareFish(fish) {
            this.shareFish = fish;
        }
        getShareShell() {
            return this.shareShell;
        }

        setShareShell(shell) {
            this.shareShell = shell;
        }
        getZhuliOverTime(){
          return   this.zhuliOverTime;
        }
        getZhuliName(){
            return   this.zhuliName;
        }
        getHasReceive(){
            return this.hasReceive;
        }
        /**
         * 0  助力对象
         * 1  助力类型
         * 2  助力奖励
         */
        getBiaoyu(){
            return [this.biaoyuid,this.biaoyuType,this.biaoyuAward,this.biaoyuopenId];
        }
        getIfMaxzhuliTimes(){
            let times=Number(Const.get('QuayAssistInviteTimes'));
            return times;
        }
        getZhulishangxian(){
            return this.zhulishangxian;
        }
        clear() {
        }
        getzhulirenshu(){
           return this.zhulirenshu;
        }
    }

    //助力系统
    class ZhuliSystem {
        static init() {
            if (Laya.Browser.onMiniGame) {
                wx.onShow((res)=> {
                    ZhuliSystem.zhuliPanding(res);
                });
            }
        }

        static clear() {
        
        }
        static getZhuliMessage(){

        }
        static getZhuliListLength(){
            return 3;
        }
        static updateZhuliList(aidedInfoGroup){
            ZhuliDataMgr.getInstance().updateZhuliList(aidedInfoGroup);
        }
        static updateZhuXinxi(aidedInfo){
            ZhuliDataMgr.getInstance().updateZhuXinxi(aidedInfo);
        }
        static updateBiaoyu(slogan){
            ZhuliDataMgr.getInstance().updateBiaoyu(slogan);
            EventMgr.getInstance().postEvent("UPDATE_ZHULI_BIAOYU");
        }
        static getZhuliList(){
            return ZhuliDataMgr.getInstance().getZhuliList();
        }
        static getZhuliOverTime(){
            return  ZhuliDataMgr.getInstance().getZhuliOverTime();
          }
        static getZhuliName(){
              return ZhuliDataMgr.getInstance().getZhuliName();
        }
        //去助力
        static AidPlayer(targetRId,handler){
            let dealFunc = function (res) {
                EventMgr.getInstance().postEvent("ZHULI_TIPS",[res.tips]);
                ZhuliSystem.updateZhuXinxi(res.aidedGroupInfo.aidInfo);
           };
            Service.AidPlayer(targetRId,Laya.Handler.create(null, dealFunc));
        }

        //获取助力奖励信息
        static  updatewharfLetterBox(wharfLetterBox){
            if(wharfLetterBox.capacity!=null) {
                ZhuliDataMgr.getInstance().setCapacity(wharfLetterBox.capacity);
            }
            ZhuliDataMgr.getInstance().setCacheFish(wharfLetterBox.fishToReward||0);
            ZhuliDataMgr.getInstance().setCacheShell(wharfLetterBox.shellToReward||0);
            ZhuliDataMgr.getInstance().setShareFish(wharfLetterBox.shareFishToReward||0);
            ZhuliDataMgr.getInstance().setShareShell(wharfLetterBox.shareShellToReward||0);

            EventMgr.getInstance().postEvent("UPDATE_WHARFLETTER_BOX");
        }
        static getCacheFish(){
            return ZhuliDataMgr.getInstance().getCacheFish();
        }
        static getCapacity(){
            return ZhuliDataMgr.getInstance().getCapacity();
        }
        static getCacheShell(){
            return ZhuliDataMgr.getInstance().getCacheShell();
        }
        static getShareFish(){
            return ZhuliDataMgr.getInstance().getShareFish();
        }
        static getShareShell(){
            return ZhuliDataMgr.getInstance().getShareShell();
        }
        static getHasReceive(){
            return ZhuliDataMgr.getInstance().getHasReceive();
        }
          /**
         * 0  助力对象
         * 1  助力类型
         * 2  助力奖励
         */
        static getBiaoyu(){
            return ZhuliDataMgr.getInstance().getBiaoyu();
        }
        static getZhulishangxian(){
            return ZhuliDataMgr.getInstance().getZhulishangxian();
        }
        static getZhuliSX(){
            return ZhuliDataMgr.getInstance().getZhuliSX();
        }
        static getZhulirenshu(){
            return ZhuliDataMgr.getInstance().getzhulirenshu();
        }
        static zhuliPanding(res){
            let weixin=res;
            if(weixin){
            let xinxi=weixin.query;
            let zrid=xinxi.rid;
            let zhuli=xinxi.zhuli;
            if(zhuli=='zhuli'){
           //检测是否是分享的链接
          // if(!GameContext.isGuiding){
            let zhuliOverTime=ZhuliSystem.getZhuliOverTime()*1000;
            let nowTime=TimeUtils.getTimeNowOnServer()+3000;
            let zhuliitem=Number(Const.get('QuayAssistWorkTime'))*1000;
            let ifGet=ZhuliSystem.getHasReceive();//判断是否已经领取 true 已经领取了
            if(!ifGet){
                if(zhuliOverTime<nowTime){
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon52"); 
                    return;  
                }
                if(zhuliOverTime>nowTime){
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon42"); 
                    return;  
                }     
            }
            if(!BuildingSystem.hasBuildingDataByBid(3000)&&!BuildingSystem.hasBuildingDataByBid(3001)&&!BuildingSystem.hasBuildingDataByBid(3002)){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon41"); 
                return;  
            }
            if(ZhuliSystem.getZhuliSX()){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon40"); 
                return;
            } 
            if(this.rid==zrid){
               return;
            }
           ZhuliSystem.AidPlayer(zrid);
        //}   
            }
        }
        }
        static setRid(rid){
           this.rid=rid;
        }
    }

    class LuckydrawDataMgr extends Laya.Dialog {
        constructor() {
            super();
            this.loadConfig();
        }

        loadConfig(){
            this.PrizeWheelUnlockLimits = Const.get("PrizeWheelUnlockLimits");
            this.PrizeWheelADdrawLimits = Const.get("PrizeWheelADdrawLimits");
            this.Packet1GameTime = Const.get("Packet1GameTime");
            this.Packet1GameAutoTime = Const.get("Packet1GameAutoTime");
            this.PrizeWheelRewardRate = JSON.parse(Const.get("PrizeWheelRewardRate"));
            this.Packet1 = JSON.parse(Const.get("Packet1"));
            this.FishLow = Const.get("FishLow");
            this.FishHigh = Const.get("FishHigh");
            this.DiamondDrawSpend = Const.get("DiamondDrawSpend");

            this.setPrizeWheelTimesArr();
            this.setPacket1();
        }

        setPrizeWheelTimesArr(){
            this.prizeWheelTimesArr = new Array();
            for(let i = 0;i < this.PrizeWheelRewardRate.length; i++){
                for(let j = 0; j < this.PrizeWheelRewardRate[i].times; j ++){
                    this.prizeWheelTimesArr.push(i);
                }
            }
            Laya.LocalStorage.setJSON("prizeWheelTimesArr",this.prizeWheelTimesArr);
        }

        setPacket1(){
            for(let i = 1; i < this.Packet1.length; i++){
                this.Packet1[i].weight += this.Packet1[i-1].weight;
            }
        }

        static getInstance() {
            return LuckydrawDataMgr.instance ? LuckydrawDataMgr.instance : LuckydrawDataMgr.instance = new LuckydrawDataMgr();
        }

        ///////////////////////////随机获取数值/////////////////////////////

        getPrizeWheelNum(){
            let nowPrizeWheelTimesArr = Laya.LocalStorage.getJSON("prizeWheelTimesArr");
            let numMax = nowPrizeWheelTimesArr.length;
            let ran = Math.random() * numMax | 0;
            let i = nowPrizeWheelTimesArr[ran];
            let type = this.PrizeWheelRewardRate[i].type;
            nowPrizeWheelTimesArr.splice(ran,1);
            if(nowPrizeWheelTimesArr.length == 0){
                nowPrizeWheelTimesArr = this.prizeWheelTimesArr;
            }
            Laya.LocalStorage.setJSON("prizeWheelTimesArr",nowPrizeWheelTimesArr);
            return [i,type];
        }

        //////////////////////////数据获取与刷新///////////////////////////

        //免费次数
        setFreeTimes(times){
            this.freeTimes = times;
        }

        getFreeTimes(){
            return this.freeTimes;
        }

        //看广告次数
        setVideoTimes(times){
            this.videoTimes = times;
        }

        getVideoTimes(){
            return [this.videoTimes,this.PrizeWheelADdrawLimits];
        }

        getDiamondDrawSpend(){
            return this.DiamondDrawSpend;
        }

        getFishRate(){
            return [this.FishLow,this.FishHigh]
        }

        getPacket1(){
            return this.Packet1;
        }

        getPacket1GameTimeData(){
            return [this.Packet1GameTime,this.Packet1GameAutoTime];
        }

        getPrizeWheelUnlockLimits(){
            return this.PrizeWheelUnlockLimits;
        }

        getPrizeWheelReward() {
            return this.PrizeWheelRewardRate;
        }
    }

    class LuckydrawSystem extends Laya.Dialog {
        constructor() {
           
        }

        static getFishReward(){
            let maxValueBase = PetSystem.getMaxValueBase();
            let rate = LuckydrawDataMgr.getInstance().getFishRate();
            return [rate[0] * maxValueBase,rate[1] * maxValueBase,maxValueBase];
        }

        static getPrizeWheelNum(){
            return LuckydrawDataMgr.getInstance().getPrizeWheelNum();
        }

        static fetchLuckyDrawReward(type, rewardInfo ,handler){
            EventMgr.getInstance().postEvent("SERVICE_FETCH_LUCKYDRAWREWARD", [type, rewardInfo ,handler]);
        }

        static updateLuckydraw(res){
            let freeTimes = res.freeDraw ? res.freeDraw : 0;
            let videoTimes = res.adDrawTimes ? res.adDrawTimes : 0;
            LuckydrawSystem.setFreeTimes(freeTimes);
            LuckydrawSystem.setVideoTimes(videoTimes);
            EventMgr.getInstance().postEvent("UPDATE_ LUCKYDRAWPOP_UI");
        }

        //免费次数
        static setFreeTimes(times){
            LuckydrawDataMgr.getInstance().setFreeTimes(times);
        }

        static getFreeTimes(){
            return LuckydrawDataMgr.getInstance().getFreeTimes();
        }


        //看广告次数
        static setVideoTimes(times){
            LuckydrawDataMgr.getInstance().setVideoTimes(times);
        }

        static getVideoTimes(){
            return LuckydrawDataMgr.getInstance().getVideoTimes();
        }

        static getDiamondDrawSpend(){
            return LuckydrawDataMgr.getInstance().getDiamondDrawSpend();
        }

        static getPacket1Currency(){
            let Packet1 = LuckydrawDataMgr.getInstance().getPacket1();
            let len = Packet1.length;
            let max = Packet1[len-1].weight;
            let ran = Math.random() * max;
            for(let i = 0; i < len ; i++){
                if(ran <= Packet1[i].weight){
                    let amount = 0;
                    if(Packet1[i].id == Constants.RewardType.FISH){
                        let r = Math.random();
                        amount = (r * (Packet1[i].amountRate[1] - Packet1[i].amountRate[0]) + Packet1[i].amountRate[0]).toFixed(0);
                    }
                    else if(Packet1[i].id == Constants.RewardType.STAR){
                        let r = Math.random();
                        amount = (r * (Packet1[i].amount[1] - Packet1[i].amount[0]) + Packet1[i].amount[0]).toFixed(0);
                    }
                    return  [Packet1[i].id , amount];
                }
            }
        }

        static getPacket1GameTimeData(){
            return LuckydrawDataMgr.getInstance().getPacket1GameTimeData();
        }

        static getPrizeWheelUnlockLimits(){
            return LuckydrawDataMgr.getInstance().getPrizeWheelUnlockLimits();
        }

        static getRedPointFlag() {
            if (LuckydrawDataMgr.getInstance().freeTimes || PlayerDataMgr.getInstance().getDiamond() >= LuckydrawDataMgr.getInstance().getDiamondDrawSpend()) {
                return true;
            }
            return false;
        }

        static getPrizeWheelReward() {
            return LuckydrawDataMgr.getInstance().getPrizeWheelReward();
        }
    }

    class MileStoneDataMgr {
        constructor() {
        }

        static getInstance() {
            return MileStoneDataMgr.instance?MileStoneDataMgr.instance:MileStoneDataMgr.instance = new MileStoneDataMgr();
        }

        updateList(MileStonList){
            this.MileStonList = MileStonList;
        }

        loadData(){
            let config = ConfigMgr$1.getInstance().getConfig("Milestone");
            this.config = new Array();
            config.forEach((listConfig) => {
                this.config.push(listConfig);
            });
            this.doList = new Array();
            this.clickSign = 1;
            this.beginSign = 0;
        }

        getClickSign(){
            return this.clickSign;
        }

        changeClickSign(){
            this.clickSign = 0;
            Laya.timer.once(500,this,function(){
                this.clickSign = 1;
            });
        }

        clearDoList(){
            this.doList.length = 0;
        }

        pushDoList(Id){
            this.doList.push(Id);
        }

        getDoList(){
            this.doList.sort(this.sortByOrder);
            return this.doList;
        }


        getConfig(Id){
            if(Id == -1){
                return this.config;
            }
            for(let i = 0;i < this.config.length ; i ++){
                if(this.config[i].Id == Id){
                    return this.config[i];
                }
            }
        }

        sortByOrder(a,b){
            let i = MileStoneDataMgr.getInstance().getConfig(a).Order;
            let j = MileStoneDataMgr.getInstance().getConfig(b).Order;
            return i - j;
        }

        checkFinish(Id){
            if(this.MileStonList){
                for(let i = 0; i < this.MileStonList.length; i ++){
                    if(Id == this.MileStonList[i]){
                        return true;
                    }
                }
                return false;
            }
        }

        
    }

    class MileStoneSystem {
        constructor() {
            
        }

        static begin(){
            EventMgr.getInstance().registEvent("SET_FORTUNECAT_FLAG",this,MileStoneSystem.setFortuneCatFlag);
            EventMgr.getInstance().registEvent("MAY_PLAY_MILESTONE",this,MileStoneSystem.checkCanReceive);
            EventMgr.getInstance().registEvent("CHANGE_MILESTONE_BEGINSIGN",this,MileStoneSystem.changeBeginSign);
        }

        static loadData(){
            MileStoneDataMgr.getInstance().loadData();
        }

        static updateList(MileStonList){
            MileStoneDataMgr.getInstance().updateList(MileStonList);
        }

        static getMileStoneReward(milestoneId ,fetchType,handler){
            EventMgr.getInstance().postEvent("SERVICE_MILESTONE_REWARD",[milestoneId ,fetchType,handler]);
        }

        static getConfig(Id){
            return MileStoneDataMgr.getInstance().getConfig(Id);
        }

        static checkFinish(Id){
            return MileStoneDataMgr.getInstance().checkFinish(Id);
        }

        static pushDoList(Id){
           MileStoneDataMgr.getInstance().pushDoList(Id);
        }

        static playDoList(){
            let doList = MileStoneDataMgr.getInstance().getDoList();
            if(doList.length > 0 && MileStoneDataMgr.getInstance().getClickSign()){
                MileStoneDataMgr.getInstance().changeClickSign();
                EventMgr.getInstance().postEvent("DIALOG_CHANGE_STATUS",[Constants.DialogStatus.MileStone,1]);
                Laya.Dialog.close("ui/MileStoneNewsPaper.scene");
                Laya.Dialog.open("ui/MileStoneNewsPaper.scene",false,doList[0]);
            }
        }

        static mayPlayDoList(){
            if(!DialogStatusMgr.getInstance().getStatus() && !MileStoneSystem.checkGuide()){
                MileStoneSystem.playDoList();
            }
        }

        static checkGuide(){
            let guideId = JSON.parse(Const.get("BarginIgnoreGuideGruop"));
            for(let i = 0;i<guideId.length;i++){
                let condition = JSON.parse(ConfigMgr$1.getInstance().getItemById("GuideGroup", guideId[i]).Condition);
                let star = PlayerDataMgr.getInstance().getStar();
                if(star >= condition.Star && !GuideMgr.getInstance().isGuideComplete(guideId[i])){
                    return true;
                }
            }
            return false;
        }

        static checkCanReceive(){
            MileStoneDataMgr.getInstance().clearDoList();
            let config = MileStoneSystem.getConfig(-1);
            for(let i = 0; i < config.length ; i ++){
                let Id = config[i].Id;
                if(!MileStoneSystem.checkFinish(Id)){
                    let condition = JSON.parse(config[i].Condition);
                    var count = 0,sum = 0;
                    if(condition.Star){
                        count ++;
                        let star = PlayerDataMgr.getInstance().getStar();
                        if(star >= condition.Star){
                            sum ++;
                        }
                    }
                    if(condition.CatAmount){
                        count ++;
                        let CatAdopt = MileStoneSystem.getNum(1);
                        if(CatAdopt >= condition.CatAmount){
                            sum ++;
                        }
                    }
                    if(condition.FOOD_UNLOCK){
                        count ++;
                        let unlock = MileStoneSystem.getNum(2);
                        if(unlock >= condition.FOOD_UNLOCK){
                            sum ++;
                        }
                    }
                    if(condition.BUILDING_AMOUNT){
                        count ++;
                        let num = MileStoneSystem.getNum(3);
                        if(num >= condition.BUILDING_AMOUNT){
                            sum ++;
                        }
                    }
                    if(condition.GuideGroup){
                        count ++;
                        if(GuideMgr.getInstance().isGuideComplete(condition.GuideGroup)){
                            sum ++;
                        }
                    }
                }
                if(count <= sum){
                    MileStoneSystem.pushDoList(Id);
                }
            }
            MileStoneSystem.mayPlayDoList();
        }

        static getNum(type){
            let num = 0;
            if(type == 1){
                num = PetDataMgr.getInstance().PetsAdopted.length;
            }
            else if(type == 2){
                num = FoodDataMgr.getInstance().CurFoodInListUnLocked.length;
            }
            else if(type == 3){
                num = BuildingDataMgr.getInstance().getNowBuildingNumber();
            }
            return num;
        }
    }

    class MissionData {

        constructor(configId, config) {
            this.configId = configId;
            this.config = config ? config : ConfigMgr$1.getInstance().getItemById("Mission", String(configId));
            this._eventGroup = JSON.parse(this.config.EventGroup);
            this._rewards = JSON.parse(this.config.Reward);
            this._resetTime = 0;
            this._missionIndex = -1;
        }

        get Id() {
            return this.configId;
        }

        get EventGroup() {
            return this._eventGroup;
        }

        get MissionLv() {
            return this.config.MissionLv;
        }

        get StarLv() {
            return Number(this.config.StarLv);
        }

        get Name() {
            return this.config.Name;
        }

        get Desc() {
            return this.config.Desc;
        }

        get Icon() {
            return this.config.Icon;
        }

        get Reward() {
            return this._rewards;
        }

        get ValueRange() {
            return JSON.parse(this.config.ValueRange);
        }
    }

    class MissionDataMgr {

        constructor() {
            this._catLimit = 3;
            this._level = 0;
            this._refreshLevel = 0;
            this._refreshLimit = 0;
            this._dailyRefreshTimes = 0;
            this._petsMap = new Map();
            this._missionMap = new Map();
            this._missionList = new Array();
            this._team = new Array();
            this._curMissionIndex = null;
            this.initMissions();
        }

        static getInstance() {
            return MissionDataMgr.instance?MissionDataMgr.instance:MissionDataMgr.instance = new MissionDataMgr();
        }

        initMissions() {
            let configs = ConfigMgr$1.getInstance().getConfig("Mission");
            let missionMap = this._missionMap;
            for(const [missionId, info] of configs) {
                let mission = new MissionData(missionId, info);
                missionMap.set(missionId, mission);
            }
        }

        clear() {
            this._missionList.length = 0;
            this._petsMap.clear();
            this._level = 0;
            this._refreshLevel = 0;
            this._nextResetTs = -1;
            this._refreshLimit = 0;
            this._dailyRefreshTimes = 0;
            this._team = new Array();
            this._curMissionIndex = null;
        }

        getNextResetTs() {
            return this._nextResetTs;
        }

        getAdRefreshTimes() {
            return this._adRefreshTimes;
        }

        getAdsToLook() {
            return this._adsToLook;
        }

        updateMissionInfo(catMissionInfo) {
            this._level = catMissionInfo.level;
            this._refreshLevel = catMissionInfo.refreshLevel;
            this._exp = catMissionInfo.exp;
            this._dailyRefreshTimes = catMissionInfo.dailyRefreshTimes;
            this._nextResetTs = catMissionInfo.nextResetTs;
            this._adRefreshTimes = catMissionInfo.adRefreshTimes;
            this._adsToLook = catMissionInfo.adsToLook;
            let missionList = catMissionInfo.missionList;
            this._missionList.length = 0;
            for(let i = 0; i < missionList.length; i ++){
                let missionInfo = missionList[i];
                this._missionList.push({
                    "id": missionInfo.missionId,
                    "index": i,
                    "resetTime": missionInfo.resetTime
                });
            }
        }

        updatePetInfo(catMissionPetInfo) {
            let petResetTsMap = catMissionPetInfo.petResetTsMap;
            let map = this._petsMap;
            map.clear();
            for(let k of Object.keys(petResetTsMap)) {
                map.set(k, petResetTsMap[k]);
            }
        }

        resetCatStatusData(catId,resetTs){
            this._petsMap.set(catId,resetTs);
        }

        canChangeRefreshLevel() {
            let now = TimeUtils.getTimeNowOnServer();
            if(now >= this._refreshLimit) {
                return true;
            }
        }

        getChangeRefreshLevelTime() {
            let now = TimeUtils.getTimeNowOnServer();
            return (this._refreshLimit - now)/1000;
        }

        setRefreshLevel(level){
            this._refreshLevel = level;
            let now = TimeUtils.getTimeNowOnServer();
            this._refreshLimit = now + Number(Const.get("CatMissionLvChooseTime")) * 1000;
        }

        getRefreshLevel() {
            return this._refreshLevel;
        }

        getRefreshTimes() {
            return this._dailyRefreshTimes;
        }

        getLevel() {
            return this._level;
        }

        getExp(){
            return this._exp;
        }

        getMissionList() {
            return this._missionList;
        }

        getMissionById(missionId) {
            return this._missionMap.get(missionId);
        }

        getMissionByIndex(missionIndex) {
            return this._missionList[missionIndex];
        }

        getPetRestTime(petId) {
            return this._petsMap.get(petId);
        }

        get Team() {
            let pets = this._team;
            // this._team = new Array();
            // for(let i = 1; i < this._catLimit; i++) {
            //     if(pets[i]!=null)
            //         this._team.push(pets[i])
            // }
            // return this._team;
            let eidx = 0;
            for(let i = 0; i < this._catLimit; ++i) {
                let pet = pets[i];
                if(pet!=null){
                    if(i!=eidx) {
                        pets[eidx] = pet;
                        pets[i] = null;
                    }
                    ++eidx;
                }
            }
            return pets;
        }

        hasPet(petId) {
            for(let i = 0; i < this._catLimit; i++) {
                if(this._team[i]==petId)
                    return true;
            }

            return false;
        }

        setPet(petId, index) {
            this._team[index] = petId;
        }

        removePet(index) {
            this._team[index] = null;
        }

        clearTeam() {
            this._team.length = 0;
        }

        setCurrentMissionIndex(mission) {
            this._curMissionIndex = mission;
        }

        getCurrentMissionIndex() {
            return this._curMissionIndex;
        }

        getCatIndex(catId) {
            return this._team.indexOf(catId);
        }
    }

    //万事屋
    class MissionSystem {
        static init() {
        }

        static clear() {
            MissionDataMgr.getInstance().clear();
        }


        static updateMissionInfo(catMissionInfo) {
            MissionDataMgr.getInstance().updateMissionInfo(catMissionInfo);
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mission"]);
        }

        static updatePetInfo(catMissionPetInfo) {
            MissionDataMgr.getInstance().updatePetInfo(catMissionPetInfo);
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mission"]);
        }

        static resetCatStatusData(catId,resetTs) {
            MissionDataMgr.getInstance().resetCatStatusData(catId,resetTs);
            EventMgr.getInstance().postEvent("MISSION_CAT_LIST_UPDATE");
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["mission"]);
        }

        static canChangeRefreshLevel() {
            return MissionDataMgr.getInstance().canChangeRefreshLevel();
        }

        static getChangeRefreshLevelTime() {
            return MissionDataMgr.getInstance().getChangeRefreshLevelTime();
        }

        static setRefreshLevel(level) {
            MissionDataMgr.getInstance().setRefreshLevel(level);
        }

        static getRefreshLevel() {
            return MissionDataMgr.getInstance().getRefreshLevel();
        }

        static getRefreshTimes() {
            return MissionDataMgr.getInstance().getRefreshTimes();
        }
        
        static getAdRefreshTimes() {
            return MissionDataMgr.getInstance().getAdRefreshTimes();
        }

        static getAdsToLook() {
            return MissionDataMgr.getInstance().getAdsToLook();
        }

        static getLevel() {
            return MissionDataMgr.getInstance().getLevel();
        }

        static getExp() {
            return MissionDataMgr.getInstance().getExp();
        }

        static getMissionList() {
            return MissionDataMgr.getInstance().getMissionList();
        }

        static getMissionByIndex(missionIndex) {
            return MissionDataMgr.getInstance().getMissionByIndex(missionIndex);
        }

        static getMissionById(missionId) {
            return MissionDataMgr.getInstance().getMissionById(missionId);
        }

        static getPetRestTime(petId) {
            return MissionDataMgr.getInstance().getPetRestTime(petId);
        }

        static get Team() {
            return MissionDataMgr.getInstance().Team;
        }

        static hasPet(petId) {
            return MissionDataMgr.getInstance().hasPet(petId);
        }

        static setPet(petId, index) {
            MissionDataMgr.getInstance().setPet(petId, index);
        }

        static removePet(index) {
            MissionDataMgr.getInstance().removePet(index);
        }

        static clearTeam() {
            MissionDataMgr.getInstance().clearTeam();
        }

        static fight(missionIndex, petIds) {
            let missionInfo = MissionSystem.getMissionByIndex(missionIndex);
            let mission = MissionSystem.getMissionById(missionInfo.id);
            let ValueRange = mission.ValueRange;
            let pcValue = {};
            for (const key in ValueRange) {
                if (ValueRange.hasOwnProperty(key)) {
                    const element = ValueRange[key];
                    pcValue[key] = Utils.randomExt2(element[0], element[1]);
                }
            }
            let EventGroup = mission.EventGroup;
            let infos = [];
            let success = true;
            let petIndex = 0;
            for (const eventId of EventGroup) {
                let event = ConfigMgr$1.getInstance().getItemById("MissionEvent", eventId);
                let eventInfo = { event: event, cats: [] };
                let conditions = JSON.parse(event.Condition);
                let eventSuccess = false;
                while (petIds[petIndex] != null) {
                    let petId = petIds[petIndex];
                    let pet = PetDataMgr.getInstance().getPet(petId);
                    let attribute = pet.attribute;
                    let catResult = { petId: petId, petData: pet };
                    let catSuccess = true;
                    let result = [];
                    conditions.forEach(condition => {
                        let s = attribute[condition] >= pcValue[condition];
                        result.push({ condition: condition, success: s });
                        catSuccess = catSuccess && s;
                    });
                    catResult.success = catSuccess;
                    catResult.result = result;
                    eventSuccess = eventSuccess || catSuccess;
                    eventInfo.cats.push(catResult);
                    if (catSuccess) {
                        break;
                    }
                    petIndex++;
                }
                eventInfo.success = eventSuccess;
                success = success && eventSuccess;
                infos.push(eventInfo);
            }

            return { pcValue: pcValue, infos: infos, success: success };
        }

        static setCurrentMissionIndex(mission) {
            MissionDataMgr.getInstance().setCurrentMissionIndex(mission);
        }

        static getCatIndex(catId) {
            return MissionDataMgr.getInstance().getCatIndex(catId);
        }

        //仅用来记录当前委托
        static getCurrentMissionIndex() {
            return MissionDataMgr.getInstance().getCurrentMissionIndex();
        }

        static checkRedPoint() {
            let now = TimeUtils.getTimeNowOnServer();
            let missions = MissionSystem.getMissionList();
            for (const missionInfo of missions) {
                let resetTime = missionInfo.resetTime;
                if (resetTime * 1000 <= now) {
                    let pets = PetDataMgr.getInstance().PetIdsAdopted;
                    for (const petId of pets) {
                        let reset_time = MissionSystem.getPetRestTime(petId) * 1000;
                        if (reset_time == null || reset_time < now) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        // service
        static getMissionInfo(handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_GET_INFO", [handler]);
        }

        static refreshMission(handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_REFRESH", [handler]);
        }

        static changeRefreshLevel(level, handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_CHANGE_LEVEL", [level, handler]);
        }

        static receiveMissionReward(missionIndex, roleIds, handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_GET_REWARD", [missionIndex, roleIds, handler]);
        }

        static resetCatStatus(roleId, handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_RESET_CATSTATUS", [roleId, handler]);
        }

        static adRefreshMission(handler) {
            EventMgr.getInstance().postEvent("SERVICE_MISSION_AD_REFRESH", [handler]);
        }
    }

    class CollectGiftData {
        constructor(id, image, type, lock, realId, quality) {
            this._image = image;
            this._id = id;
            this._type = type;
            this._lock = lock;
            this._realId = realId;
            this._quality = quality;
        }
        getRealId() {
            return this._realId;
        }
        getImage() {
            return this._image;
        }
        setImage(img) {
            this._image = img;
        }
        getId() {
            return this._id;
        }

        setId(id) {
            this._id = id;
        }

        getType() {
            return this._type;
        }

        setType(type) {
            this._type = type;
        }
        getLock() {
            return this._lock;
        }
        setLock(state) {
            this._lock = state;
        }
        getQuality() {
            return this._quality;
        }
    }

    class CollectGiftDataMgr
    {
        constructor()
        {
            this._cgData=new Map();
            this.collectedCnt=0;
        }

        static getInstance() {
            return CollectGiftDataMgr.instance ? CollectGiftDataMgr.instance : CollectGiftDataMgr.instance = new CollectGiftDataMgr();
        }
        setData(data)
        {
            let catToysData=data.unlockPetToys;     //type=1
            let batisData=data.unlockBaits;         //type=2
            let nauticalTools=data.unlockNauticalTools; //type=3
            //state=1,unlocked ,state=0 locked
           
         let petToysCfgList= ConfigMgr$1.getInstance().getConfig("CatToys");
         petToysCfgList.forEach(
             (ptCell, id) => {
                 if (ptCell.Display == 0) {
                     return;
                 }
                let state=0;
                 if(ptCell.Type == "TOY"){
                    if(catToysData.includes(id))
                    {state=1;
                        this.collectedCnt+=1;}
                     else 
                     { state=0;
                    }
                     
                     this.pushCgData("TOY"+id,ptCell.Icon,1,state,id,ptCell.Quality);
                 }
                 else if(ptCell.Type == "ITEM"){
                    if(catToysData.includes(id))
                    {state=1;
                        this.collectedCnt+=1;}
                     else 
                     { state=0;
                    }
                     
                     this.pushCgData("TOY"+id,ptCell.Icon,4,state,id,ptCell.Quality);
                 }
             
         });
        
        
        let fishbaitList= ConfigMgr$1.getInstance().getConfig("FishingBaits");
        fishbaitList.forEach((fbCell,id)=>
            {
                let state2=0;
                if(batisData.includes(id))
                {
                    state2=PlayerSystem.getStar()> fbCell.StarLimit?1:0;
                    if(state2==1)
                    {
                        this.collectedCnt+=1;
                    }
                }
                else
                {state2=0;}
                this.pushCgData("BATIS"+id,fbCell.img,2,state2,id,fbCell.Quality);
            }
        );

         let toolsCfgList=ConfigMgr$1.getInstance().getConfig("NauticalTools");
         toolsCfgList.forEach(
             (ntCell,id)=>{
                 let state3=0;
                 if(nauticalTools.includes(id))
                 {
                     state3=1;
                    this.collectedCnt+=1;}
                 
                else 
                {
                    state3=0;
                }
                this.pushCgData("tool"+id,ntCell.Icon,3,state3,id,ntCell.Quality);
             }
         );


         EventMgr.getInstance().postEvent("Refresh_CollectGiftUI",null);

        }

        pushCgData(id, img, type, lockState, realId, quality) {
            let cgData = new CollectGiftData(id, img, type, lockState, realId, quality);
            this._cgData.set(id, cgData);
        }

        getGiftData()
        {
            return this._cgData;
        }
        getCollectedCnt()
        {
            return this.collectedCnt;
        }
        clear()
        {
            this._cgData.clear();
            this.collectedCnt=0;
        }
    }

    class PayDataMgr{
        constructor(){
           this.loadData();
        }

        static getInstance() {
            return PayDataMgr.instance ? PayDataMgr.instance : PayDataMgr.instance = new PayDataMgr();
        }

        loadData(){
            this.PurchaseDailyDiamond = Number(Const.get("PurchaseDailyDiamond"));
            this.PurchaseWeekDays = Number(Const.get("PurchaseWeekDays"));
            this.PurchaseMonthDays = Number(Const.get("PurchaseMonthDays"));
            this.PurchaseCDTime = Number(Const.get("PurchaseCDTime"));
            this.payLock = 1;
        }

        updatePayInfo(payInfo){
            this.payInfo = payInfo;
        }

        getPayInfo(){
            return this.payInfo;
        }

        setSkipAdInviteNum(skipAdInviteNum){
            this.skipAdInviteNum = skipAdInviteNum;
        }

        getSkipAdInviteNum(){
            return this.skipAdInviteNum;
        }

        setCatFoodNum(catFoodNum){
            this.catFoodNum = catFoodNum;
        }

        getCatFoodNum(){
            return this.catFoodNum;
        }

        setTrackData(serverId,roleId,userInfo){
            this.trackData = {
                "serverId": serverId,
                "roleId": roleId,
                "roleName":userInfo.nickName
            };
        }

        getTrackData(){
            return this.trackData;
        }

        setPayLock(num){
            this.payLock = num;
        }

        getPayLock(){
            return this.payLock;
        }
    }

    class PaySystem{
        static init(){
            EventMgr.getInstance().registEvent("PURCHASEASYNC_COMPLETE", null, PaySystem.createOrder);
            EventMgr.getInstance().registEvent("CONSUMEPURCHASEASYNC_COMPLETE", null,PaySystem.finishOrder);
            EventMgr.getInstance().registEvent("PAY_REWARDS_TAN", null,PaySystem.getPurchaseCard);
            EventMgr.getInstance().registEvent("GET_PURCHASE_CARD",this,PaySystem.getPurchaseCard);
        }

        static updatePayInfo(payInfo){
            PayDataMgr.getInstance().updatePayInfo(payInfo);
        }

        static getPayInfo(){
            return PayDataMgr.getInstance().getPayInfo();
        }

        static setSkipAdInviteNum(skipAdInviteNum){
            PayDataMgr.getInstance().setSkipAdInviteNum(skipAdInviteNum);
        }

        static getSkipAdInviteNum(){
            return PayDataMgr.getInstance().getSkipAdInviteNum();
        }

        static setCatFoodNum(catFoodNum){
            PayDataMgr.getInstance().setCatFoodNum(catFoodNum);
        }

        static getCatFoodNum(){
            return PayDataMgr.getInstance().getCatFoodNum();
        }

        static setTrackData(serverId,roleId,userInfo){
            PayDataMgr.getInstance().setTrackData(serverId,roleId,userInfo);
        }

        static getTrackData(){
            return PayDataMgr.getInstance().getTrackData();
        }

        static setPayLock(num){
            PayDataMgr.getInstance().setPayLock(num);
        }

        static getPayLock(){
            return PayDataMgr.getInstance().getPayLock();
        }
    ///////////////////////////////////////////////////////////////////////////////////
        static getSingleFish(){
            this.PurchaseFISHAmountLow = Number(Const.get("PurchaseFISHAmountLow"));
            this.PurchaseFISHAmountHigh = Number(Const.get("PurchaseFISHAmountHigh"));
            this.PurchaseFISHAmount = JSON.parse(Const.get("PurchaseFISHAmount"));
            let starNum = PlayerDataMgr.getInstance().getStar();
            let M = this.PurchaseFISHAmount.A * starNum * starNum + this.PurchaseFISHAmount.B * starNum + this.PurchaseFISHAmount.C;
            if(M < this.PurchaseFISHAmountLow){
                M = this.PurchaseFISHAmountLow;
            }
            if(M > this.PurchaseFISHAmountHigh || starNum >= Number(Const.get("PurchaseFISHAmountStarLimit"))){
                M = this.PurchaseFISHAmountHigh;
            }
            return M.toFixed(0);
        }

        static checkDoubleSign(listConfig){
            if(!listConfig.FirstDouble){
                return false;
            }
            let Id = listConfig.Id;
            let firstPay = PaySystem.getPayInfo().firstPay;
            for(let i = 0;i < firstPay.length; i ++){
                if(Id == firstPay[i]){
                    return false;
                }
            }
            return true;
        }

        static checkDay(receiveMills){
            let onDayTime = 24 * 60 * 60 * 1000;
            let now = TimeUtils.getTimeNowOnServer();
            let date = new Date(now);
            let nowHour = date.getUTCHours(now);
            if(nowHour < 5){
                now -= onDayTime;
                date = new Date(now);
            }
            let year = date.getUTCFullYear(now);
            let month = date.getUTCMonth(now);
            let today = date.getUTCDate(now);
            
            let olddate = new Date(receiveMills);
            let oldHour = olddate.getUTCHours(receiveMills);
            if(oldHour < 5){
                receiveMills -= onDayTime;
                olddate = new Date(receiveMills);
            }
            let oldyear = olddate.getUTCFullYear(receiveMills);
            let oldmonth = olddate.getUTCMonth(receiveMills);
            let oldday = olddate.getUTCDate(receiveMills);
            
            if(year == oldyear && month == oldmonth && today == oldday){
                return false;
            }
            else {
                return true;
            }
        }

        static  getRemainDays(EndMills){
            let day = 0;
            let now = TimeUtils.getTimeNowOnServer();
            if(EndMills < now){
                return -1;
            }
            let date = new Date(now);
            let year = date.getUTCFullYear(now);
            let mouth = date.getUTCMonth(now);
            let today = date.getUTCDate(now);
            let nowHour = date.getUTCHours(now);
            today = nowHour >= 5 ? today : today - 1;
            let todayTime = Date.UTC(year, mouth, today,5) / 1000;

            let createDate = new Date(EndMills);
            let creatYear = createDate.getUTCFullYear();
            let creatMouth = createDate.getUTCMonth();
            let creatDay = createDate.getUTCDate();
            let creatHour = createDate.getUTCHours();
            creatDay = creatHour >= 5 ? creatDay : creatDay - 1;
            let creatTime = Date.UTC(creatYear, creatMouth, creatDay,5) / 1000;
            day = Math.floor((creatTime - todayTime) / 3600 / 24);
            return day;
        }

        static purchaseTrackEvent(orderId,productID){
            let trackData = PaySystem.getTrackData();
            let purchaseConfig =  ConfigMgr$1.getInstance().getItemById("Purchase", productID);
            let directReward = JSON.parse(purchaseConfig.DirectReward)[0];
            let param = SdkHelper.params();
            param['method'] = "createOrder";
            param['cpOrderId'] = orderId;
            param['serverId'] = trackData.serverId;
            param['roleId'] = trackData.roleId;
            param['roleName'] = trackData.roleName;
            param['productId'] = productID;
            param['productName'] = purchaseConfig.Type;
            if(directReward.amount){
                param['productDes'] = "amount:" + directReward.amount;
            }
            else if(directReward.amountRate){
                param['productDes'] = "amountRate:" + directReward.amountRate;
            }
            param['customInfo'] = "null";
            param['totalFee'] = Number(purchaseConfig.Price) * 100;
            param['uid'] = SdkHelper.uid;
            if(WxConfigMgr.getInstance().getPlatform() =='pp'){
                param['aid'] = WxConfigMgr.getInstance().getAdUnitId();
            }
            param['traceid'] = "null";
            SdkHelper.trackEvent(param);
        }
        /////////////////////////////////service////////////////////////////////
        static createOrder(orderId,productID){
            EventMgr.getInstance().postEvent("SERVICE_CREATEORDER",[orderId,productID]);
            // PaySystem.purchaseTrackEvent(orderId,productID);
            PaySystem.setPayLock(1);
        }

        static finishOrder(orderId,productID){
            EventMgr.getInstance().postEvent("SERVICE_FINISHORDER",[orderId,productID]);
        }

        static receiveFreeDiamond(){
            EventMgr.getInstance().postEvent("SERVICE_RECEIVEFREEDIAMOND");
        }

        static payCardBonus(cardType){
            EventMgr.getInstance().postEvent("SERVICE_PAYCARDBONUS",[cardType]);
        }

        static resourceShop(productID){
            EventMgr.getInstance().postEvent("SERVICE_RESOURCESHOP",[productID]);
        }

        static getPurchaseCard(){
            let payCardInfo = PaySystem.getPayInfo().payCardInfo;
            let purchaseConfig = ConfigMgr$1.getInstance().getConfig("Purchase");
            if(payCardInfo.weekCardEndMills && PaySystem.getRemainDays(payCardInfo.weekCardEndMills) > 0 
            && (!payCardInfo.weekCardReceiveMills || PaySystem.checkDay(payCardInfo.weekCardReceiveMills))){
                let config = purchaseConfig.get("Pay_WEEK_1");
                Laya.Dialog.close("ui/PayRewards.scene");
                Laya.Dialog.open("ui/PayRewards.scene",false,[config,0]);
            }
            else if(payCardInfo.monthCardEndMills && PaySystem.getRemainDays(payCardInfo.monthCardEndMills) > 0 
            && (!payCardInfo.monthCardReceiveMills || PaySystem.checkDay(payCardInfo.monthCardReceiveMills))){
                let config = purchaseConfig.get("Pay_MONTH_1");
                Laya.Dialog.close("ui/PayRewards.scene");
                Laya.Dialog.open("ui/PayRewards.scene",false,[config,0]);
            }
        }
    }

    //DataMgr都应该只包含ConfigMgr和对应的Data
    class TakeoutDataMgr {

        constructor() {
            this.delivery = null;
            this.takeoutOrder = null;
            this.nextUpdateTime = null;
            this.config = null;
            this.loaclDeliveryPets = [];
            this.loaclPublicityPets = [];
            for (let index = 0; index < 3; index++) {
                let obj = {
                    Id: index,
                    petId: null,
                    lock: true
                };
                this.loaclDeliveryPets.push(obj);
            }
            for (let index = 0; index < 5; index++) {
                let obj = {
                    Id: index,
                    petId: null,
                    lock: true
                };
                this.loaclPublicityPets.push(obj);
            }
        }

        static getInstance() {
            return TakeoutDataMgr.instance ? TakeoutDataMgr.instance : TakeoutDataMgr.instance = new TakeoutDataMgr();
        }

        clear() {
            this.delivery = null;
            this.takeoutOrder = null;
            this.nextUpdateTime = null;
            this.config = null;
            this.loaclDeliveryPets = [];
            this.loaclPublicityPets = [];
            for (let index = 0; index < 3; index++) {
                if (this.loaclDeliveryPets[index]) {
                    this.loaclDeliveryPets[index].petId = null;
                    this.loaclDeliveryPets[index].lock = true;
                }
            }
            for (let index = 0; index < 5; index++) {
                if (this.loaclPublicityPets[index]) {
                    this.loaclPublicityPets[index].petId = null;
                    this.loaclPublicityPets[index].lock = true;
                }
            }
        }

        updateTakeoutInfo(takeoutInfo) {
            let level = takeoutInfo.level;
            this.delivery = takeoutInfo.delivery;
            this.config = ConfigMgr$1.getInstance().getItemById("OrderSystem", String(level));

            this.takeoutOrder = takeoutInfo.takeoutOrder;
            let lastUpdateTime = this.getLastUpdateTs();
            let orderSystemOrderCisterns = Number(Const.get("OrderSystemOrderCisterns"));
            let leftProgress = orderSystemOrderCisterns - this.getProgress();
            let pets = this.getPublicityPets();
            this.nextUpdateTime = lastUpdateTime + Math.ceil(leftProgress / (this.getSpeed()));
            for (let index = 0; index < this.loaclPublicityPets.length; index++) {
                const element = this.loaclPublicityPets[index];
                element.Id = index;
                element.petId = pets[index];
                element.lock = index >= this.getMaxAdvertisingStaff();
            }
            for (let index = 0; index < this.loaclDeliveryPets.length; index++) {
                const element = this.loaclDeliveryPets[index];
                element.Id = index;
                element.lock = index >= this.getMaxCourierStaff();
            }
        }



        getConfig() {
            return this.config;
        }

        getLevel() {
            return Number(this.config.Id);
        }

        getName() {
            return this.config.Name;
        }

        getMaxCourierStaff() {
            return this.config.CourierStaff;
        }

        getMaxOrderNum() {
            return this.config.Order;
        }

        getMaxAdvertisingStaff() {
            return this.config.AdvertisingStaff;
        }

        getSpendTime() {
            return this.config.SpendTime;
        }

        getRewardBase() {
            return this.config.RewardBase;
        }

        getLvUpLimit() {
            return this.config.LvUpLimit;
        }

        getLvUpCost() {
            return this.config.LvUpCost;
        }

        getStaffCar() {
            return this.config.StaffCar;
        }



        //满级返回空
        getNextLevelConfig() {
            if (this.config.LvUpLimit < 0) {
                return null;
            }
            let nextLevel = Number(this.getLevel()) + 1;
            return ConfigMgr$1.getInstance().getItemById("OrderSystem", String(nextLevel));
        }

        getDeliveryRewardAmount() {
            return this.delivery.rewardAmount;
        }

        getDeliveryEndTime() {
            return this.delivery.endTs;
        }

        getDeliveryReceive() {
            return this.delivery.receive;
        }

        getDeliveryPets() {
            return this.delivery.pets;
        }

        getDeliveryTag() {
            return this.delivery.tag;
        }

        getOrderNum() {
            return this.takeoutOrder.order;
        }

        getProgress() {
            return this.takeoutOrder.progress;
        }

        getSpeed() {
            return this.takeoutOrder.speed;
        }

        getLastUpdateTs() {
            return this.takeoutOrder.lastUpdateTs;
        }

        getPublicityPets() {
            return this.takeoutOrder.pets;
        }
        getBoughtTimes() {
            return this.takeoutOrder.boughtTimes;
        }

        getNextUpdateTime() {
            return this.nextUpdateTime;
        }


        localAddOrderNum() {
            this.takeoutOrder.order++;
        }

        addLoclPets(petId, index) {
            if (index != null) {
                if (this.loaclDeliveryPets[index] && !this.loaclDeliveryPets[index].lock) {
                    this.loaclDeliveryPets[index].petId = petId;
                }
                return;
            }
            for (let i = 0; i < this.loaclDeliveryPets.length; i++) {
                const element = this.loaclDeliveryPets[i];
                if (element.petId == null) {
                    element.petId = petId;
                    break;
                }
            }
        }

        removeLocalPets(petId) {
            for (let index = 0; index < this.loaclDeliveryPets.length; index++) {
                const element = this.loaclDeliveryPets[index];
                if (petId == element.petId) {
                    element.petId = null;
                    break;
                }
            }
        }

        getLoaclPets() {
            return this.loaclDeliveryPets;
        }

        clearLoaclPets() {
            this.loaclDeliveryPets.forEach(element => {
                element.petId = null;
            });
        }

        getLoaclPublicityPets() {
            return this.loaclPublicityPets;
        }
    }

    class TakeoutSystem {
        static init() {
        }

        static clear() {
            TakeoutDataMgr.getInstance().clear();
        }


        //===================service==========================
        static fetchTakeoutInfo(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }

        static delivery(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }

        static receiveRewards(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo, false);
            EventMgr.getInstance().postEvent("UPDATA_TAKEOUT_INFO_UI", true);
        }

        static speedUpDelivery(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }

        static levelUpTakeout(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }

        static putAdvertiserCats(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }

        static buyOrder(res) {
            let takeoutInfo = res.takeoutInfo;
            TakeoutSystem.updateTakeoutInfo(takeoutInfo);
        }
        //===================service==========================


        static updateTakeoutInfo(takeoutInfo, updateUI = true) {
            TakeoutDataMgr.getInstance().updateTakeoutInfo(takeoutInfo);
            //刷新UI
            if (updateUI) {
                EventMgr.getInstance().postEvent("UPDATA_TAKEOUT_INFO_UI");
            }
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["order"]);
        }



        static getConfig() {
            return TakeoutDataMgr.getInstance().getConfig();
        }

        static getLevel() {
            return TakeoutDataMgr.getInstance().getLevel();
        }

        static getName() {
            return Strings$1.get(TakeoutDataMgr.getInstance().getName());
        }

        static getMaxCourierStaff() {
            return TakeoutDataMgr.getInstance().getMaxCourierStaff();
        }

        static getMaxOrderNum() {
            return TakeoutDataMgr.getInstance().getMaxOrderNum();
        }

        static getMaxAdvertisingStaff() {
            return TakeoutDataMgr.getInstance().getMaxAdvertisingStaff();
        }

        static getSpendTime() {
            return TakeoutDataMgr.getInstance().getSpendTime();
        }

        static getRewardBase() {
            return TakeoutDataMgr.getInstance().getRewardBase();
        }

        static getLvUpLimit() {
            return TakeoutDataMgr.getInstance().getLvUpLimit();
        }

        static getLvUpCost() {
            return TakeoutDataMgr.getInstance().getLvUpCost();
        }

        static getStaffCar() {
            return TakeoutDataMgr.getInstance().getStaffCar();
        }


        //满级返回null
        static getNextLevelConfig() {
            return TakeoutDataMgr.getInstance().getNextLevelConfig();
        }

        static getDeliveryRewardAmount() {
            return TakeoutDataMgr.getInstance().getDeliveryRewardAmount();
        }

        static getDeliveryEndTime() {
            return TakeoutDataMgr.getInstance().getDeliveryEndTime();
        }

        static getDeliveryReceive() {
            return TakeoutDataMgr.getInstance().getDeliveryReceive();
        }

        static getDeliveryPets() {
            return TakeoutDataMgr.getInstance().getDeliveryPets();
        }

        static getDeliveryTag() {
            return TakeoutDataMgr.getInstance().getDeliveryTag();
        }

        static getOrderNum() {
            return TakeoutDataMgr.getInstance().getOrderNum();
        }

        static getPublicityPets() {
            return TakeoutDataMgr.getInstance().getPublicityPets();
        }

        static getBoughtTimes() {
            return TakeoutDataMgr.getInstance().getBoughtTimes();
        }

        /**
         * 下次订单刷新剩余时间S
         */
        static getOrderUpdaeTimeLeft_S() {
            if (TakeoutSystem.getOrderNum() >= TakeoutSystem.getMaxOrderNum()) {
                return null;
            }
            let nextTime = TakeoutDataMgr.getInstance().getNextUpdateTime() + 1;
            if (TimeUtils.getTimeNowOnServer() > nextTime * 1000) {
                let orderSystemOrderCisterns = Number(Const.get("OrderSystemOrderCisterns"));
                nextTime += Math.ceil(orderSystemOrderCisterns / (TakeoutDataMgr.getInstance().getSpeed()));
                TakeoutDataMgr.getInstance().localAddOrderNum();
                EventMgr.getInstance().postEvent("SERVICE_FETCH_TAKEOUT_INFO");
            }
            return Math.ceil((nextTime * 1000 - TimeUtils.getTimeNowOnServer()) / 1000);
        }

        static getDeliveryTimeLeft_S() {
            if (TimeUtils.getTimeNowOnServer() > TakeoutSystem.getDeliveryEndTime() * 1000) {
                EventMgr.getInstance().postEvent("UPDATA_TAKEOUT_INFO_UI");
                return null;
            }
            return Math.ceil((TakeoutSystem.getDeliveryEndTime() * 1000 - TimeUtils.getTimeNowOnServer()) / 1000);
        }

        /**
         * 获取当前外卖状态
         * 0:未开始
         * 1：进行中
         * 2：待领取
         */
        static getCurrentType() {
            let endTime = TakeoutSystem.getDeliveryEndTime();
            if (endTime != null && endTime > 0) {
                if (endTime * 1000 > TimeUtils.getTimeNowOnServer()) {
                    return 1;
                }
                return TakeoutSystem.getDeliveryReceive() ? 0 : 2;
            }
            return 0;
        }

        static addDeliveryLoclPet(petId, index) {
            TakeoutDataMgr.getInstance().addLoclPets(petId, index);
        }

        static removeDeliveryLocalPet(id) {
            TakeoutDataMgr.getInstance().removeLocalPets(id);
        }

        static pushDeliveryRecommendCat() {
            TakeoutSystem.clearDeliveryLocalPets();
            let petMap = PetDataMgr.getInstance().PetsAdopted;
            let pets = [];
            let xcPets = TakeoutSystem.getPublicityPets();
            petMap.forEach(element => {
                let id = element.getId();
                for (const iterator of xcPets) {
                    if (iterator == id) {
                        return;
                    }
                }
                pets.push(element);
            });
            pets.sort((a, b) => {
                return TakeoutSystem.getOnePetEfficiencyNum(b) - TakeoutSystem.getOnePetEfficiencyNum(a);
            });
            for (let index = 0; index < TakeoutSystem.getMaxCourierStaff(); index++) {
                if (pets[index] != null) {
                    TakeoutSystem.addDeliveryLoclPet(pets[index].getId(), index);
                }
            }
        }
        /**
         * 按排名计算推荐
         */
        static checkIfDeliveryRecommendCat(curId) {
            let petMap = PetDataMgr.getInstance().PetsAdopted;
            let xcPets = TakeoutSystem.getPublicityPets();
            // let kdPets = TakeoutSystem.getDeliveryPets();
            // let localKdPets = TakeoutSystem.getDeliveryLocalPets();
            let map = new Map();
            petMap.forEach(element => {
                let id = element.getId();
                for (const iterator of xcPets) {
                    if (iterator == id) {
                        return;
                    }
                }
                // for (const iterator of kdPets) {
                //     if (iterator == id) {
                //         return;
                //     }
                // }
                // for (const iterator of localKdPets) {
                //     if (iterator.petId == id && id != curId) {
                //         return;
                //     }
                // }
                let b = TakeoutSystem.getOnePetEfficiencyNum(element);
                let arr = map.get(b);
                if (arr) {
                    arr.push(id);
                } else {
                    map.set(b, [id]);
                }
            });
            let rarnks = Array.from(map.keys());
            rarnks.sort((a, b) => {
                return b - a;
            });
            let num = TakeoutSystem.getMaxCourierStaff();
            let num1 = 0;
            for (const key of rarnks) {
                let arr = map.get(key);
                num1 += arr.length;
                if (arr.indexOf(curId) >= 0) {
                    return true;
                }
                if (num1 > num) {
                    break;
                }
            }
            return false;
        }

        static getDeliveryLocalPets() {
            return TakeoutDataMgr.getInstance().getLoaclPets();
        }

        static clearDeliveryLocalPets() {
            TakeoutDataMgr.getInstance().clearLoaclPets();
        }

        static getAllCanUsePets(isDelivery, curId = null) {
            let petMap = PetDataMgr.getInstance().PetsAdopted;
            let pets = [];
            let xcPets = TakeoutSystem.getPublicityPets();
            let kdPets = TakeoutSystem.getDeliveryPets();
            let localKdPets = TakeoutSystem.getDeliveryLocalPets();
            let recommendIds = [];
            petMap.forEach(element => {
                let id = element.getId();
                for (const iterator of xcPets) {
                    if (iterator == id && id != curId) {
                        return;
                    }
                }
                for (const iterator of kdPets) {
                    if (iterator == id) {
                        return;
                    }
                }
                for (const iterator of localKdPets) {
                    if (iterator.petId == id && id != curId) {
                        return;
                    }
                }
                pets.push(element);
            });
            pets.sort((a, b) => {
                if (isDelivery) {
                    return TakeoutSystem.getOnePetEfficiencyNum(b) - TakeoutSystem.getOnePetEfficiencyNum(a);
                } else {
                    return b.getIntimacyGrade() - a.getIntimacyGrade();
                }
            });
            let petIds = [];
            pets.forEach(element => {
                let id = element.getId();
                petIds.push(id);
            });
            if (pets.length > 0) {
                let max = 0;
                if (isDelivery) {
                    max = TakeoutSystem.getOnePetEfficiencyNum(pets[0]);
                } else {
                    max = pets[0].getIntimacyGrade();
                }
                recommendIds.push(petIds[0]);
                for (let index = 1; index < pets.length; index++) {
                    const element = pets[index];
                    if (isDelivery) {
                        if (TakeoutSystem.getOnePetEfficiencyNum(element) < max) {
                            break;
                        }
                    } else {
                        if (element.getIntimacyGrade() < max) {
                            break;
                        }
                    }
                    recommendIds.push(petIds[index]);
                }
            }
            return { pets: petIds, recommendIds: recommendIds };
        }
        /**
         * 本地实际加成（小数）
         */
        static getLocalEfficiencyNum() {

            let pets = TakeoutSystem.getDeliveryLocalPets();
            let orderSystemRewardUpConfig1 = Const.getArray("OrderSystemRewardUp1");
            let orderSystemRewardUpConfig2 = Const.getArray("OrderSystemRewardUp2");
            let tag = TakeoutSystem.getDeliveryTag();
            let tagNum = 0;
            let intimacy = 0;
            let orderSystemRewardUp1 = 0;
            let orderSystemRewardUp2 = 0;
            pets.forEach(obj => {
                if (obj.petId == null) {
                    return;
                }
                let pet = PetDataMgr.getInstance().getPet(obj.petId);
                let isApply = false;
                pet.getTag().forEach(element => {
                    if (element == tag) {
                        isApply = true;
                    }
                });
                if (isApply) {
                    ++tagNum;
                    let a = orderSystemRewardUpConfig1[tagNum];
                    if (a != null) {
                        orderSystemRewardUp1 = a;
                    }
                }
                intimacy += pet.getIntimacyGrade();
                let b = orderSystemRewardUpConfig2[intimacy];
                if (b != null) {
                    orderSystemRewardUp2 = b;
                }
            });
            return orderSystemRewardUp1 + orderSystemRewardUp2;
        }

        static getOnePetEfficiencyNum(pet) {
            if (pet == null) {
                return null;
            }
            let orderSystemRewardUpConfig1 = Const.getArray("OrderSystemRewardUp1");
            let orderSystemRewardUpConfig2 = Const.getArray("OrderSystemRewardUp2");
            let tag = TakeoutSystem.getDeliveryTag();
            let tagNum = 0;
            let intimacy = 0;
            let orderSystemRewardUp1 = 0;
            let orderSystemRewardUp2 = 0;
            let isApply = false;
            pet.getTag().forEach(element => {
                if (element == tag) {
                    isApply = true;
                }
            });
            if (isApply) {
                ++tagNum;
                let a = orderSystemRewardUpConfig1[tagNum];
                if (a != null) {
                    orderSystemRewardUp1 = a;
                }
            }
            intimacy += pet.getIntimacyGrade();
            let b = orderSystemRewardUpConfig2[intimacy];
            if (b != null) {
                orderSystemRewardUp2 = b;
            }
            return orderSystemRewardUp1 + orderSystemRewardUp2;
        }

        static getLoclTrueReward() {
            let num = 0;
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            let maxValueBase = 0;
            config.forEach((listConfig, petId) => {
                let pet = PetDataMgr.getInstance().getPet(petId);
                if (pet && pet.getVisitTimes() > 0 && Number(pet.getValueBase()) > maxValueBase) {
                    maxValueBase = Number(pet.getValueBase());
                }
            });
            let orderSystemRewardK = Number(Const.get("OrderSystemRewardK"));
            let rewardBase = TakeoutSystem.getRewardBase();
            let efficiencyNum = TakeoutSystem.getLocalEfficiencyNum();
            num = Math.floor((maxValueBase * orderSystemRewardK + rewardBase) * (1 + efficiencyNum));
            return num;
        }

        /**
         * 实际加成（小数）
         */
        static getEfficiencyNum() {
            let pets = TakeoutSystem.getDeliveryPets();
            let orderSystemRewardUpConfig1 = Const.getArray("OrderSystemRewardUp1");
            let orderSystemRewardUpConfig2 = Const.getArray("OrderSystemRewardUp2");
            let tag = TakeoutSystem.getDeliveryTag();
            let tagNum = 0;
            let intimacy = 0;
            let orderSystemRewardUp1 = 0;
            let orderSystemRewardUp2 = 0;
            pets.forEach(petId => {
                let pet = PetDataMgr.getInstance().getPet(petId);
                let isApply = false;
                pet.getTag().forEach(element => {
                    if (element == tag) {
                        isApply = true;
                    }
                });
                if (isApply) {
                    ++tagNum;
                    let a = orderSystemRewardUpConfig1[tagNum];
                    if (a != null) {
                        orderSystemRewardUp1 = a;
                    }
                }
                intimacy += pet.getIntimacyGrade();
                let b = orderSystemRewardUpConfig2[intimacy];
                if (b != null) {
                    orderSystemRewardUp2 = b;
                }
            });
            return orderSystemRewardUp1 + orderSystemRewardUp2;
        }

        static getOrderSpeedUp() {
            let pets = TakeoutSystem.getPublicityPets();
            let orderSystemSpeedUpConfig1 = Const.getArray("OrderSystemSpeedUp1");
            let orderSystemSpeedUpConfig2 = Const.getArray("OrderSystemSpeedUp2");
            let petNum = 0;
            let intimacy = 0;
            let orderSystemSpeedUp1 = 0;
            let orderSystemSpeedUp2 = 0;
            pets.forEach(petId => {
                let pet = PetDataMgr.getInstance().getPet(petId);
                petNum += 1;
                let a = orderSystemSpeedUpConfig1[petNum];
                if (a != null) {
                    orderSystemSpeedUp1 = a;
                }
                intimacy += pet.getIntimacyGrade();
                let b = orderSystemSpeedUpConfig2[intimacy];
                if (b != null) {
                    orderSystemSpeedUp2 = b;
                }
            });
            return Math.floor((orderSystemSpeedUp1 + orderSystemSpeedUp2) * 100);
        }

        static removeAdvertiserCats2Service(petId) {
            let pets = TakeoutSystem.getPublicityPets();
            let arr = [];
            pets.forEach(element => {
                if (element != petId) {
                    arr.push(element);
                }
            });
            EventMgr.getInstance().postEvent("SERVICE_PUT_ADVERTISER_CATS", [arr]);
        }

        static putAdvertiserCats2Service(petId, index) {
            let pets = TakeoutSystem.getPublicityPets();
            let arr = [];
            arr = arr.concat(pets);
            if (index != null && arr[index] != null) {
                arr[index] = petId;
            } else {
                arr.push(petId);
            }
            EventMgr.getInstance().postEvent("SERVICE_PUT_ADVERTISER_CATS", [arr]);
        }

        static getLoaclPublicityPets() {
            return TakeoutDataMgr.getInstance().getLoaclPublicityPets();
        }

        static startDelivery2Service(free, handler) {
            let pets = [];
            let localPets = TakeoutSystem.getDeliveryLocalPets();
            localPets.forEach(element => {
                if (element.petId != null && !element.lock) {
                    pets.push(element.petId);
                }
            });
            if (pets.length <= 0) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#OrderSystemDesc13");
                return false;
            }
            EventMgr.getInstance().postEvent("SERVICE_DELIVERY", [pets, free, handler]);
            return true;
        }

        static checkLevelUpRedPoint() {
            return TakeoutSystem.getNextLevelConfig() && PetDataMgr.getInstance().PetsAdopted.length >= TakeoutSystem.getLvUpLimit() && TakeoutSystem.getLvUpCost() <= PlayerDataMgr.getInstance().getFish();
        }

        static checkOrderRedPoint() {
            return TakeoutSystem.getOrderNum() > 0 && TakeoutSystem.getCurrentType() != 1;
        }

        static getCurrentBuyOrderCost() {
            let orderTimesCost = Const.getArray("OrderSystemOrderTimesCost");
            let time = TakeoutSystem.getBoughtTimes();
            return orderTimesCost[time];
        }

        static getMaxBuyNum() {
            let orderTimesCost = Const.getArray("OrderSystemOrderTimesCost");
            return orderTimesCost.length;
        }

    }

    //let url = "ws://52.88.182.252:17405/gs1" //开发服
    //let url ="wss://game-cats-life.cp.freegames123.com:17405/gs1";                
    let url = "ws://49.233.17.104:14005/gs5";//海外 dev测试服
    // let url  = "ws://49.233.17.104:14003/gs3"//海外 dev测试服
    //let url  = "ws://49.232.98.28:14002/gs2"    //dev开发服
    // let url = "ws://49.232.18.82:14002/gs2" //开发服
    //let url="ws://49.232.98.28:14001/gs1"//Debug
    //let url="ws://49.232.98.28:14004/gs4"//balloon
    //let url="ws://49.232.98.28:14005/gs5"//测试
    //let url="ws://49.232.98.28:14003/gs3"
    // let url = "ws://49.232.18.82:14001/gs1"    //稳定服
    // let url = "ws://49.232.98.28:14001"    //稳定服
    // let url = "ws://49.232.98.28:14002"    //调试服
    // let url = "wss://shuangfeng33.cn:17305/gs1"    //稳定服
    // let url = "ws://10.0.72.156:9823"
    // let url = "ws://127.0.0.1:9823"
    // let serverId = "1"  //稳定服
    let serverId = "5"; //开发服

    let _appId;
    let _openId;
    let _wxLoginState = 0;
    let _rid;
    let _token = null;
    let _serverTag;
    let _authorized = false;
    let _logined = false;
    class System {
        static init() {
            if (System.inited == null) {
                System.inited = true;
                EventMgr.getInstance().registEvent("NETWORD_DISCONNECT", null, System.restart);

                EventMgr.getInstance().registEvent("SERVICE_LOGIN", null, System.login);

                EventMgr.getInstance().registEvent("SERVICE_ONLINE_CROSSDAY", null, System.onlineCrossDay);

                EventMgr.getInstance().registEvent("SYSTEM_SET_AUTHORIZED", null, System.setAuthorized);
                EventMgr.getInstance().registEvent("SERVICE_AUTHORIZE_REQUEST", null, System.authorizeRequest);

                EventMgr.getInstance().registEvent("SERVICE_PUT_BUILDING", null, System.putBuilding);
                EventMgr.getInstance().registEvent("SERVICE_BARGAIN", null, System.bargain);

                EventMgr.getInstance().registEvent("SERVICE_GET_ALLTASKS", null, System.getAllTasks);
                EventMgr.getInstance().registEvent("SERVICE_GET_TROPHY_TASK_REWARD", null, System.getTrophyTaskReward);
                EventMgr.getInstance().registEvent("SERVICE_GET_DAILY_TASK_REWARD", null, System.getDailyTaskReward);
                EventMgr.getInstance().registEvent("SERVICE_GET_MAIN_TASK_REWARD", null, System.getMainTaskReward);

                EventMgr.getInstance().registEvent("SERVICE_GET_PLAYER_FOOD_MENU", null, System.getPlayerFoodMenu);
                EventMgr.getInstance().registEvent("SERVICE_FOOD_LEVELUP", null, System.foodLevelUp);

                EventMgr.getInstance().registEvent("SERVICE_UPDATE_SCENE_INFO", null, System.updateSceneInfo);

                EventMgr.getInstance().registEvent("SERVICE_GET_LETTER_BOX_INFO", null, System.getLetterBoxInfo);
                EventMgr.getInstance().registEvent("SERVICE_FETCH_LETTER_BOX", null, System.fetchLetterBox);

                EventMgr.getInstance().registEvent("SERVICE_GET_PETSINFO", null, System.getPetsInfo);
                EventMgr.getInstance().registEvent("SERVICE_ADOPT_PET", null, System.adoptPet);
                EventMgr.getInstance().registEvent("SERVICE_STORY_LOCK", null, System.storyLock);
                EventMgr.getInstance().registEvent("SERVICE_PET_SHAREGETREWARD", null, System.petShareGetReward);
                EventMgr.getInstance().registEvent("SERVICE_PET_LEVELUP", null, System.catLevelUp);
                EventMgr.getInstance().registEvent("SERVICE_PET_STARUP", null, System.catStarUp);

                EventMgr.getInstance().registEvent("SERVICE_FIND_VISITOR_GIFT", null, System.findVisitorGift);
                EventMgr.getInstance().registEvent("SERVICE_GET_VISITOR_GIFT_ALL", null, System.getVisitorGiftAll);
                EventMgr.getInstance().registEvent("SERVICE_GET_VISITOR_GIFT_SINGLE", null, System.getVisitorGiftSingle);

                EventMgr.getInstance().registEvent("SERVICE_CAT_FOOD_SUBSCRIBE", null, System.catFoodSubscribe);

                EventMgr.getInstance().registEvent("SERVICE_GET_FISHING_COLLECTS", null, System.getFishingFishInfo);
                EventMgr.getInstance().registEvent("SERVICE_GET_FISHING_BASE_INFO", null, System.getFishGameInfo);
                EventMgr.getInstance().registEvent("SERVICE_FINISH_FISHING_GAME", null, System.playFishGame);
                EventMgr.getInstance().registEvent("SERVICE_AD_FREE_FISHING_GAME", null, System.getFishingAdFreeInfo);

                EventMgr.getInstance().registEvent("SERVICE_GET_NAUTICAL_INFO", null, System.getNauticalInfo);
                EventMgr.getInstance().registEvent("SERVICE_MERGE_NAUTICAL_FOOD", null, System.mergeNauticalFood);
                EventMgr.getInstance().registEvent("SERVICE_START_NAUTICAL_TRIP", null, System.startNauticalTrip);
                EventMgr.getInstance().registEvent("SERVICE_FINISH_NAUTICAL_TRIP", null, System.finishNauticalTrip);
                EventMgr.getInstance().registEvent("SERVICE_PREVIEW_NAUTICAL_TRIPREQ", null, System.previewNauticalTripReq);
                EventMgr.getInstance().registEvent("SERVICE_AD_REDUCE_NAUTICAL_TIME", null, System.adReduceNauticalTime);
                EventMgr.getInstance().registEvent("SERVICE_UNLOCK_NAUTICAL", null, System.unlockNautical);

                EventMgr.getInstance().registEvent("SERVICE_HOME_SET_PET", null, System.setPetStatus);
                EventMgr.getInstance().registEvent("SERVICE_HOME_CUTE_INTERACT", null, System.cuteInteract);
                EventMgr.getInstance().registEvent("SERVICE_HOME_COMMON_INTERACT", null, System.commonInteract);
                EventMgr.getInstance().registEvent("SERVICE_HOME_FETCH_PHOTOWALL", null, System.fetchPhotoWall);
                EventMgr.getInstance().registEvent("SERVICE_HOME_FETCH_INFO", null, System.fetchHomeInfo);
                EventMgr.getInstance().registEvent("SERVICE_HOME_HEALTHINTERACT", null, System.healthyInteract);

                EventMgr.getInstance().registEvent("SERVICE_MAIL_FETCH_ALL_MAILS", null, System.fetchAllMails);
                EventMgr.getInstance().registEvent("SERVICE_MAIL_READ_MAIL", null, System.readMail);
                EventMgr.getInstance().registEvent("SERVICE_MAIL_DELETE_MAIL", null, System.deleteMail);
                EventMgr.getInstance().registEvent("SERVICE_MAIL_DELETE_READ_MAILS", null, System.deleteReadMails);
                EventMgr.getInstance().registEvent("SERVICE_RECEIVE_MAIL_REWARDS", null, System.receiveMailRewards);

                EventMgr.getInstance().registEvent("SERVICE_EVENT_FIRE_EVENT", null, System.fireEvent);

                EventMgr.getInstance().registEvent("SERVICE_PUT_CUSTOM_DATA", null, System.putCustomData);
                EventMgr.getInstance().registEvent("SERVICE_DEL_CUSTOM_DATA", null, System.delCustomData);
                EventMgr.getInstance().registEvent("SERVICE_GET_CUSTOM_DATA", null, System.getCustomData);

                EventMgr.getInstance().registEvent("SERVICE_GET_ENTRUST", null, System.getEntrust);
                EventMgr.getInstance().registEvent("SERVICE_ACCEPT_ENTRUST", null, System.acceptEntrust);
                EventMgr.getInstance().registEvent("SERVICE_RECEIVE_REWARDS", null, System.receiveRewards);
                EventMgr.getInstance().registEvent("SERVICE_SEND_SUGGEST", null, System.sendSuggest);


                EventMgr.getInstance().registEvent("SERVICE_SEND_BALLOON", null, System.getBalloonInfo);
                EventMgr.getInstance().registEvent("SERVICE_GET_BALLOON_NUM", null, System.getBalloonNum);

                EventMgr.getInstance().registEvent("SERVICE_GET_MINIGAME", null, System.getMiniGame);
                EventMgr.getInstance().registEvent("SERVICE_FINISH_MINIGAME", null, System.finishMiniGame);
                EventMgr.getInstance().registEvent("SERVICE_CLIENT_TRIGGER_MINIGAME", null, System.ClientTriggerMiniGame);
                EventMgr.getInstance().registEvent("SERVICE_GIVEUP_MINIGAME", null, System.GiveUpMiniGame);

                EventMgr.getInstance().registEvent("SERVICE_QUERY_LUCKY_BAG_INFO", null, System.queryLuckyBagInfo);
                EventMgr.getInstance().registEvent("SERVICE_RECEIVE_LUCKY_BAG", null, System.receiveLuckyBag);

                EventMgr.getInstance().registEvent("SERVICE_REQUEST_TOTAL_RANKING", null, System.requestTotalRanking);
                EventMgr.getInstance().registEvent("SERVICE_REQUEST_REGION_RANKING", null, System.requestRegionRanking);
                EventMgr.getInstance().registEvent("SERVICE_SEND_FRIEND_GIFT", null, System.sendFriendGift);

                EventMgr.getInstance().registEvent("SERVICE_GET_STAFF_SKILL_GROUP", null, System.getStaffSkillGroup);
                EventMgr.getInstance().registEvent("SERVICE_SKILL_LEVEL_UP", null, System.skillLevelUp);
                EventMgr.getInstance().registEvent("SERVICE_STAFF_LEVEL_UP", null, System.staffLevelUp);
                EventMgr.getInstance().registEvent("SERVICE_SELECT_SKILL", null, System.selectSkill);
                EventMgr.getInstance().registEvent("SERVICE_GET_STAFF_GROUP", null, System.getStaffGroup);
                EventMgr.getInstance().registEvent("SERVICE_REDUCE_CD_TIME", null, System.reduceCDTime);

                EventMgr.getInstance().registEvent("SERVICE_VIEW_AD", null, System.viewAd);
                EventMgr.getInstance().registEvent("SERVICE_VIEW_SHARE", null, System.shareReq);
                EventMgr.getInstance().registEvent("SERVICE_SEND_DEBUG", null, System.debug);
                EventMgr.getInstance().registEvent("SERVICE_SEND_LOADINGLOG", null, System.sendClientLoadingLog);
                //added by tenkin
                EventMgr.getInstance().registEvent("SERVICE_GetCollectGiftInfoRes", null, System.getCollectGiftInfoRes);
                //助力
                EventMgr.getInstance().registEvent("SERVICE_GET_PROJECT_GROUP", null, System.GetNauticalProjectGroup);
                EventMgr.getInstance().registEvent("SERVICE_PROJECT_LEVELUP", null, System.projectLevelUp);
                EventMgr.getInstance().registEvent("FETCH_WHARF_LETTERBOX", null, System._FetchWharfLetterBox);
                EventMgr.getInstance().registEvent("RECEIVE_AID_SALARY", null, System._ReceiveAidSalary);
                EventMgr.getInstance().registEvent("FETCH_DAILY_SHELL", null, System._FetchDailyShell);

                //万事屋
                EventMgr.getInstance().registEvent("SERVICE_MISSION_GET_INFO", null, System.getCatMissionInfo);
                EventMgr.getInstance().registEvent("SERVICE_MISSION_REFRESH", null, System.refreshCatMission);
                EventMgr.getInstance().registEvent("SERVICE_MISSION_CHANGE_LEVEL", null, System.changeRefreshLevel);
                EventMgr.getInstance().registEvent("SERVICE_MISSION_GET_REWARD", null, System.receiveMissionReward);
                EventMgr.getInstance().registEvent("SERVICE_MISSION_RESET_CATSTATUS", null, System.resetCatStatus);
                EventMgr.getInstance().registEvent("SERVICE_MISSION_AD_REFRESH", null, System.adRefreshMission);

                EventMgr.getInstance().registEvent("SERVICE_MILESTONE_REWARD", null, System.getMileStoneReward);

                EventMgr.getInstance().registEvent("SERVICE_FETCH_LUCKYDRAWREWARD", null, System.FetchLuckyDrawReward);
                EventMgr.getInstance().registEvent("SERVICE_GET_LUCKYDRAWINFO", null, System.GetLuckyDrawInfo);
                EventMgr.getInstance().registEvent("SERVICE_FETCH_BUILDING_TICKET_REWARD", null, System.FetchBuildingTicketReward);
                EventMgr.getInstance().registEvent("SERVICE_FINISH_BUILDING_COUPON", null, System.FinishBuildingCoupon);
                EventMgr.getInstance().registEvent("SERVICE_USE_AD_COUPON", null, System.UseAdCoupon);
                EventMgr.getInstance().registEvent("SERVICE_GETLOTTERYBAG", null, System.GetLotteryBag);

                EventMgr.getInstance().registEvent("SERVICE_CREATEORDER", null, System.createOrder);
                EventMgr.getInstance().registEvent("SERVICE_FINISHORDER", null, System.finishOrder);
                EventMgr.getInstance().registEvent("SERVICE_RECEIVEFREEDIAMOND", null, System.receiveFreeDiamond);
                EventMgr.getInstance().registEvent("SERVICE_PAYCARDBONUS", null, System.payCardBonus);
                EventMgr.getInstance().registEvent("SERVICE_RESOURCESHOP", null, System.resourceShop);

                EventMgr.getInstance().registEvent("SERVICE_FETCH_TAKEOUT_INFO", null, System.fetchTakeoutInfo);
                EventMgr.getInstance().registEvent("SERVICE_DELIVERY", null, System.delivery);
                EventMgr.getInstance().registEvent("SERVICE_ORDER_RECEIVE_REWARDS", null, System.receiveOrderRewards);
                EventMgr.getInstance().registEvent("SERVICE_SPEED_UP_DELIVERY", null, System.speedUpDelivery);
                EventMgr.getInstance().registEvent("SERVICE_LEVEL_UP_TAKEOUT", null, System.levelUpTakeout);
                EventMgr.getInstance().registEvent("SERVICE_PUT_ADVERTISER_CATS", null, System.putAdvertiserCats);
                EventMgr.getInstance().registEvent("SERVICE_BUY_ORDER", null, System.buyOrder);

                EventMgr.getInstance().registEvent("UpdateFullBagByInviteTask_Event", null, System._updateFullBag);
                EventMgr.getInstance().registEvent("UpdateRewardsByInviteTask_Event", this, System._dealWithRewards);
            }
        }

        static initSystems() {
            if (System.systemsInited == null) {
                System.systemsInited = true;
                PlayerSystem.init();
                BuildingSystem.init();
                CatSystem.init();
                FoodSystem$1.init();
                TaskSystem.init();
                RedPointSystem.init();
                SyncSystem.init();
                SimulatorSystem.init();
                PetSystem.init();
                FishingSystem.init();
                NauticalSystem$1.init();

                GuideSystem.init();
                TaskMainSystem.init();
                StaffSystem.init();
                SignSystem.init();
                ProjectSystem.init();
                ZhuliSystem.init();

                MissionSystem.init();
                PaySystem.init();
            }
        }


        static restart() {
            Laya.stage.removeChildByName("MainTain");
            EventMgr.getInstance().postEvent("LOADING_SHOW_TRANSITION");
            System.clear();
            System.clearView();
            EventMgr.getInstance().postEvent("BALLOON_DESTROY");
            //总是报错，只能先reload
            // SceneMgr.getInstance().closeAllScene();
            // SceneMgr.getInstance().switchScene(0);
            // Laya.Browser.window.location.reload();      //这句话会重新执行main函数
            if (Laya.Browser.onMiniGame || Laya.Browser.onAndroid || Laya.Browser.onIPad || Laya.Browser.onIPhone) {
                // wx.exitMiniProgram({
                //     fail() {
                //         console.log("退出游戏失败")
                //     }
                // })

                SceneMgr.getInstance().closeAllScene();
                WebSocketClient.getInstance().initSocket();
                System.enter(false, false);
            } else {
                SceneMgr.getInstance().closeAllScene();
                WebSocketClient.getInstance().initSocket();
                SceneMgr.getInstance().switchScene(Constants.LoginSceneId);
            }
            System._needRestart = false;
            //TODO: 显示些啥
        }

        static enter(load = true, skipLogin = true) {
            console.log("enter", load, skipLogin);
            Laya.timer.clear(System, System.enter);
            if (!skipLogin) {
                System.login(System.getRid(), Laya.Handler.create(null, function () {
                    System.loadScenes(load);
                }));
            } else {
                // System.loadScenes(load);
                // SceneMgr.getInstance().loadScene(Constants.CoffeeSceneId);
                if (Laya.Browser.onMiniGame) {
                    if (System._loadState == 1) {
                        Laya.loader.load([{ url: "res/cookbook.atlas", type: Laya.Loader.ATLAS }]);
                        System.loadScenes(load);
                    } else if (System._loadState == 0) {
                        Laya.timer.frameOnce(1, System, System.enter, [load, skipLogin]);
                    }
                } else {
                    Laya.loader.load([{ url: "res/cookbook.atlas", type: Laya.Loader.ATLAS }]);
                    TransitionGameLoader.getInstance().start(Laya.Handler.create(System, System.loadScenes, [load]));
                }
            }
        }

        static loadScenes(load) {
            Laya.timer.clear(System, System.loadScenes);
            if (System.isLogined()) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_load_scene_start", { totalcost: (+ new Date()) - System._loader_begin }]);
                SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId, Laya.Handler.create(null, function (scene3) {
                    console.debug("handler3", 3);
                    Laya.stage.removeChildByName("LoadTransition");
                    Laya.stage.removeChildByName("MainTain");
                    if (load) {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_enter_scene", { totalcost: (+ new Date()) - System._loader_begin }]);
                    }
                    Laya.timer.frameOnce(1, System, function () {
                        SceneMgr.getInstance().loadScene(Constants.BeachSceneId, Laya.Handler.create(null, function (scene1) {
                            console.debug("handler1", 1);
                            SceneMgr.getInstance().loadScene(Constants.NauticalSceneId, Laya.Handler.create(null, function (scene2) {
                                console.debug("handler2", 2);
                                SceneMgr.getInstance().setSwitchAllowed();
                                if (load) {
                                    InGameLoader.getInstance().start();
                                }
                            }));
                        }));
                    });
                }));
            } else {
                Laya.timer.frameOnce(1, System, System.loadScenes, [load]);
            }
        }

        static setLogined() {
            _logined = true;
            console.log("setlogined");
        }

        static isLogined() {
            return _logined;
        }

        static clear() {
            Laya.timer.clear(System, System.enter);
            Laya.timer.clear(System, System.login);
            Laya.timer.clear(System, System.loadScenes);
            _logined = false;
            PlayerSystem.clear();
            BuildingSystem.clear();
            CatSystem.clear();
            FoodSystem$1.clear();
            TaskSystem.clear();
            RedPointSystem.clear();
            SyncSystem.clear();
            SimulatorSystem.clear();
            PetSystem.clear();
            FishingSystem.clear();
            NauticalSystem$1.clear();
            GuideSystem.clear();
            EntrustSystem.clear();
            MailSystem.clear();
            MiniGameSystem.clear();
            PhotoSystem.clear();
            TaskMainSystem.clear();
            SpecialCatSystem.clear();
            WebSocketClient.getInstance().clearNetStatus();
            LuckyBagSystem.clear();
            TempletMgr.getInstance().clear();
            StaffSystem.clear();
            ProjectSystem.clear();
            ZhuliSystem.clear();

            MissionSystem.clear();
            TakeoutSystem.clear();
        }

        static clearView() {
            Laya.Dialog.closeAll();
            BarrageMgr.getInstance().clear();
            ItemFlyMgr.getInstance().clear(false);
            AutoZhaolanMgr.getInstance().clear();
        }

        static setAppId(appId) {
            _appId = appId;
        }

        static setOpenId(openId) {
            _openId = openId;
        }

        static getOpenId() {
            return _openId;
        }

        static setRid(rid) {
            _rid = rid;
            PlayerSystem.setRid(rid);
            //助力系统会用到rid
            if (Laya.Browser.onMiniGame) {
                ZhuliSystem.setRid(rid);
            } else {
                ZhuliSystem.setRid(_rid + "_" + serverId);
            }
        }

        static getRid() {
            if (Laya.Browser.onMiniGame) {
                return _rid;
            }
            else if ((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) && Constants.shouji) {
                return _rid;
            } else {
                return _rid + "_" + serverId;
            }
        }

        static setToken(token) {
            _token = token;
        }

        static getToken() {
            return _token;
        }

        static setAuthorized(v) {
            _authorized = v;
        }

        static getAuthorized() {
            return _authorized;
        }

        static setServerTag(tag) {
            _serverTag = tag;
        }

        static getServerTag() {
            return _serverTag;
        }

        static getServerUrl() {
            if (Laya.Browser.onMiniGame) {
                return "wss://" + WxConfigMgr.getInstance().getGlobalUrl() + _serverTag;
                // return "wss://fy.hulai.com:17305"+_serverTag
            } else if (!Laya.Browser.onMiniGame && ((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) && Constants.shouji)) {
                return "wss://" + WxConfigMgr.getInstance().getGlobalUrl() + _serverTag;
            }
            return url
        }

        static _dealLoginRes(res) {
            let bag = res.bag;
            let playerBuilding = res.playerBuilding;
            let playerPets = res.playerPets;
            let tasks = res.tasks;
            let foodMenu = res.menu;
            let letterBox = res.letterbox;
            let customData = res.customData;
            let fishingGameInfo = res.fishingGameInfo;
            let homeInfo = res.homeInfo;
            let visitorRewardsExist = res.visitorRewardsExist;
            let customKV = res.customKV;
            let newMailCount = res.newMailCount;
            let adRewardsInfo = res.adRewardsInfo;
            let luckyBagInfo = res.luckyBagInfo;
            let giftTimes = res.giftTimes;
            let createTs = res.extraInfo ? res.extraInfo.createTs : null;
            let loginDays = res.extraInfo ? res.extraInfo.loginDays : null;
            let timeOffSet = res.timeOffSet;
            let nauticalProjectGroup = res.nauticalProjectGroup;
            let wharfLetterBox = res.wharfLetterBox;//挂机奖励
            let luckydraw = res.luckyDraw;
            let aidedGroupInfo = res.aidedGroupInfo;
            let finishedMilestone = res.finishedMilestone;
            let benefitsIfHasFetched = res.benefitsIfHasFetched;
            let levelToyMap = homeInfo ? homeInfo.levelToyMap : null;
            let ticketBuildingInfos = res.ticketBuildingInfos;
            let payInfo = res.payInfo;
            let lotteryBag = res.lotteryBag;
            let takeoutInfo = res.takeoutInfo;
            SyncSystem.setLoginTime(timeOffSet);
            System._updateFullBag(bag);
            System._initBuilding(playerBuilding);
            System._updatePets(res.playerPets);
            System._updateTasks(tasks);
            System._updateFoods(foodMenu);
            System._updateLetterBox(letterBox);
            System._updatewharfLetterBox(wharfLetterBox);
            System._updateZhuliBox();
            System._updateFishGameInfo(fishingGameInfo);
            SyncSystem.setVisitorRewardsExist(visitorRewardsExist);
            System._restoreCustomData(customData);

            System._updateAidedGroupInfo(aidedGroupInfo);
            System._updateProjects(nauticalProjectGroup);



            SyncSystem.scheduleUpdate();
            if (customKV && customKV.shareTimes != null) {
                SdkHelper.setShareTimes(Number(customKV.shareTimes));
            }
            // if(customKV.shareFailTimes != null) {
            //     SdkHelper.setShareFailTimes(Number(customKV.shareFailTimes));
            // }
            if (createTs) {
                PlayerSystem.setGameCreateTime(Number(createTs));
            }
            if (loginDays) {
                PlayerSystem.setLoginDays(loginDays);
            }
            MiniGameSystem.updateOtherData(customKV);
            if (customKV != null) {
                GuideSystem.updateGuideProgress(Utils.obj2Map(customKV));
            }
            PetSystem.updataHomeRubbish(customKV);
            if (homeInfo) {
                PetSystem.updateHomeInfo(homeInfo);
                let photoWall = homeInfo.photoWall;
                PhotoSystem.updataPhotosData(photoWall);
            }
            if (res.balloon) {
                BalloonSystem.beginBalloon(res.balloon.dailyCount);
            }
            if (newMailCount) {
                MailSystem.setNewMail(true);
            }
            if (adRewardsInfo) {
                PlayerSystem.setAdRewardsInfo(adRewardsInfo);
            }
            if (luckyBagInfo) {
                LuckyBagSystem.update(luckyBagInfo);
            }
            else {
                LuckyBagSystem.checkCloseluckyBag();
            }
            if (giftTimes) {
                PlayerSystem.setGiveTimes(giftTimes);
            }
            if (luckydraw) {
                let diamound = luckydraw.diamondAmount ? luckydraw.diamondAmount : 0;
                PlayerSystem.updateDiamond(diamound);
                LuckydrawSystem.updateLuckydraw(luckydraw);
            }
            if (finishedMilestone) {
                MileStoneSystem.updateList(finishedMilestone);
            }
            let activityData = res.ifOpenInfo;
            if (activityData) {
                System.SetActivityState(activityData);
            }
            if (levelToyMap) {
                LevelToySystem.updateData(levelToyMap);
            }
            if (payInfo) {
                PaySystem.updatePayInfo(payInfo);
            }
            BuildingSystem.updateTicketBuildingInfos(ticketBuildingInfos);
            if (lotteryBag) {
                let fishingTicketNum = lotteryBag.fishingTicketNum ? lotteryBag.fishingTicketNum : 0;
                let nauticalSpeedUpNum = lotteryBag.nauticalSpeedUpNum ? lotteryBag.nauticalSpeedUpNum : 0;
                let skipAdInviteNum = lotteryBag.skipAdInviteNum ? lotteryBag.skipAdInviteNum : 0;
                let catFoodNum = lotteryBag.catFoodNum ? lotteryBag.catFoodNum : 0;
                FishingSystem.setFishingTicketNum(fishingTicketNum);
                NauticalSystem$1.updateNauticalTicketNum(nauticalSpeedUpNum);
                PaySystem.setSkipAdInviteNum(skipAdInviteNum);
                PaySystem.setCatFoodNum(catFoodNum);
            }
            if (takeoutInfo) {
                TakeoutSystem.fetchTakeoutInfo(res);
            }
        }

        static _updateFullBag(bag, updataUI = true) {
            if (bag) {
                // console.debug("bag", bag);
                if (bag.currency) {
                    let currency = bag.currency;
                    // console.debug("currency:", currency);
                    let fishUpdated = false;
                    Object.getOwnPropertyNames(currency).forEach(function (key) {

                        let item = currency[key];
                        let itemId = item.id;
                        let count = item.amount;
                        // console.debug("updateItem", itemId, count, item);
                        if (itemId == Constants.RewardType.FISH) {
                            if (updataUI) {
                                PlayerSystem.updateFish(count);
                            } else {
                                PlayerSystem.secretlyUpdateFish(count);
                            }
                            fishUpdated = true;
                        } else if (itemId == Constants.RewardType.STAR) {
                            if (updataUI) {
                                PlayerSystem.updateStar(count);
                            } else {
                                PlayerSystem.secretlyUpdateStar(count);
                            }
                        }
                        else if (itemId == Constants.RewardType.SHELL) {
                            if (updataUI) {
                                PlayerSystem.updateShell(count);
                            } else {
                                PlayerSystem.secretlyUpdateShell(count);
                            }
                        } else if (itemId == Constants.RewardType.DIAMOND) {
                            PlayerSystem.updateDiamond(count);
                        } else if (itemId == Constants.RewardType.MEDAL) {
                            PlayerSystem.updateMedal(count);
                        } else if (itemId == Constants.RewardType.BOOK) {
                            PlayerSystem.updateBook(count);
                        }
                    });
                    if (!fishUpdated) {
                        PlayerSystem.updateFish(0);
                    }
                }
                if (bag.nauticalFoods)
                    NauticalSystem$1.updateNauticalFoods(bag.nauticalFoods);
                if (bag.nauticalTools)
                    NauticalSystem$1.updateNauticalTools(bag.nauticalTools);
                if (bag.foodMenu)
                    System._updateFoods(bag.foodMenu);

                EventMgr.getInstance().postEvent("PURCHASE_UPDATE_CURRENCY");
            }
        }

        static _dealWithRewards(rewards) {
            if (rewards) {
                let items = rewards.rewards;
                if (items) {
                    for (const item of items) {
                        switch (item.type) {
                            case Constants.ItemType.NauticalFoods:
                                let food = NauticalSystem$1.getFoodById(item.id);
                                food.count = food.count + item.amount;
                                break;
                            case Constants.ItemType.NauticalTools:
                                let tool = NauticalSystem$1.getToolById(item.id);
                                tool.count = tool.count + item.amount;
                                break;
                            case Constants.ItemType.FishingBaits:
                                FishingSystem.unlockBait(item.id);
                                break;
                            case Constants.ItemType.CatPhotos:
                                PhotoSystem.setSystemPhotoMap(item.id);
                                break;
                            case Constants.ItemType.CatToys:
                                PetSystem.addToy(item.id);
                                break;
                            case Constants.ItemType.Currency:
                                if (item.id == Constants.RewardType.STAR) {
                                    PlayerSystem.cumulateStar(item.amount);
                                } else if (item.id == Constants.RewardType.DIAMOND) {
                                    PlayerSystem.cumulateDiamond(item.amount);
                                } else if (item.id == Constants.RewardType.SHELL) {
                                    PlayerSystem.cumulateShell(item.amount);
                                } else if (item.id == Constants.RewardType.MEDAL) {
                                    PlayerSystem.cumulateMedal(item.amount);
                                } else if (item.id == Constants.RewardType.BOOK) {
                                    PlayerSystem.cumulateBook(item.amount);
                                } else {
                                    PlayerSystem.cumulateFish(item.amount);
                                }
                                break;
                            case Constants.ItemType.Coupon:
                                if (item.id == Constants.CouponType.SPEED) {
                                    NauticalSystem$1.addTicket(item.amount);
                                }
                                break;
                        }
                    }
                }
            }
        }

        static _dealWithPhotoRewards(rewards) {
            if (rewards) {
                for (const item of rewards) {
                    PhotoSystem.setSystemPhotoMap(item.photoId);
                }
            }
        }

        static _initBuilding(playerBuilding) {
            if (playerBuilding) {
                // console.debug("_initBuilding arguments", arguments);
                let buildings = playerBuilding.buildings;
                // console.debug("buildings:", buildings);
                Object.getOwnPropertyNames(buildings).forEach(function (key) {

                    let building = buildings[key];
                    let buildingId = building.id;
                    let baseId = building.position;
                    // console.debug("initBuilding", buildingId, baseId);
                    if (baseId) {
                        BuildingSystem.replaceBuildingData(buildingId, baseId);
                    } else {
                        BuildingSystem.addBuildingDataByBid(buildingId);
                    }
                });
            }
        }

        static _updateBuilding(playerBuilding) {
            //TODO 先清除现有的建筑信息
            BuildingSystem.clear();
            System._initBuilding(playerBuilding);
        }

        static _updatePets(pets) {
            if (pets) {
                if (pets.cats) {
                    PetSystem.updatePets(pets.cats);
                }
            }
        }

        static _updateTasks(tasks, registerDays) {
            if (tasks) {
                // console.debug("_updateTasks:", tasks);
                if (registerDays != null) {
                    PlayerSystem.setRegisterDays(registerDays);
                }
                TaskSystem.updateTasks(tasks);
                if (tasks.mainTasks) {
                    TaskMainSystem.updata(tasks.mainTasks);
                }
            }
        }

        static _updateFoods(menu) {
            if (menu) {
                // console.debug("_updateFoods:", menu);
                FoodSystem$1.updateFoods(menu);
            }
        }

        static _updateProjects(projects) {
            if (projects) {
                // console.debug("_updateFoods:", menu);
                ProjectSystem.updateProjects(projects);
            }
        }


        static _updateVisitorGift(visitorRewards) {
            if (visitorRewards) {
                SyncSystem.updateVisitorGift(visitorRewards);
            }
        }

        static _updateLetterBox(letterBox) {
            if (letterBox) {
                // console.debug("_updateLetterBox:", letterBox);
                SyncSystem.updateLetterBox(letterBox);
            }
        }
        static _updatewharfLetterBox(wharfLetterBox) {
            if (wharfLetterBox) {
                // console.debug("_updateLetterBox:", letterBox);
                ZhuliSystem.updatewharfLetterBox(wharfLetterBox);
            }
        }

        static _updateZhuliBox() {

            // console.debug("_updateLetterBox:", letterBox);
            SyncSystem.updateZhuliBox();
        }
        //助力他人后奖励领取
        static _ReceiveAidSalary(handler) {
            let dealFunc = function (res) {
                System._updateAidedGroupInfo(res.aidedGroupInfo);
                handler.runWith([res.bag.currency]);
                System._updateFullBag(res.bag, false);
            };
            Service.ReceiveAidSalary(Laya.Handler.create(null, dealFunc));
        }
        //领取助力宝箱奖励
        static _FetchWharfLetterBox(handler, fetchType) {
            let dealFunc = function (res) {
                let wharfLetterBox = res.wharfLetterBox;
                ZhuliSystem.updatewharfLetterBox(wharfLetterBox);
                handler.runWith([res.bag.currency]);
                System._updateFullBag(res.bag, false);
            };
            Service.FetchWharfLetterBox(Laya.Handler.create(null, dealFunc), fetchType);
        }
        static _FetchDailyShell(handler) {
            let dealFunc = function (res) {
                let wharfLetterBox = res.wharfLetterBox;
                ZhuliSystem.updatewharfLetterBox(wharfLetterBox);
                handler.runWith([res.bag.currency]);
                System._updateFullBag(res.bag, false);
            };
            Service.FetchDailyShell(Laya.Handler.create(null, dealFunc));
        }
        static _updateAidedGroupInfo(aidedGroupInfo) {
            if (aidedGroupInfo) {
                let aidedInfoGroup = aidedGroupInfo.aidedInfoGroup;//被助力的信息
                let aidedInfo = aidedGroupInfo.aidInfo;//助力别人的信息
                let slogan = aidedGroupInfo.slogan;
                System._updateZhuliList(aidedInfoGroup);
                System._updateZhuXinxi(aidedInfo);
                System._updateBiaoyu(slogan);
            }
        }
        static _updateZhuliList(aidedInfoGroup) {
            ZhuliSystem.updateZhuliList(aidedInfoGroup);
        }

        static _updateZhuXinxi(aidedInfo) {
            ZhuliSystem.updateZhuXinxi(aidedInfo);
        }
        static _updateBiaoyu(slogan) {
            ZhuliSystem.updateBiaoyu(slogan);
        }

        static _restoreCustomData(customData) {
            // console.debug("_restoreCustomData:", customData);
            SyncSystem.restoreCustomData(customData);
        }

        static _updateFishGameInfo(fishingGameInfo) {
            if (fishingGameInfo) {
                FishingSystem.updateBaseData(fishingGameInfo);
            }
        }

        static _updateFishCollectInfo(fishingCollectInfos) {
            FishingSystem.updateCollectData(fishingCollectInfos);
        }

        static _testNotify() {
            // console.debug("_testNotify", arguments);
        }

        static _reddotNotify(ReddotMsg) {
            // console.debug("_reddotNotify", arguments);
            if (ReddotMsg && ReddotMsg.type) {
                if (ReddotMsg.type == Constants.RedDotType.kTask) {
                    TaskSystem.setDirty(1);
                }
                RedPointSystem.refreshRedPoint(ReddotMsg.type);
            }
        }

        static _onlineCrossDay(OnlineCrossDayMsg) {
            System.onlineCrossDay();
        }

        static _mailMsgNotify(MailMsg) {
            MailSystem.setNewMail(true);
        }

        static _entrustMsgNotify(EntrustMsg) {
            EntrustSystem.updataOne(EntrustMsg.entrust);
        }

        static _miniGameMsgNotify(MiniGameMsg) {
            let miniGame = MiniGameMsg.miniGame;
            MiniGameSystem.startMiniGame(miniGame);
        }

        /**----------------------------------------------------------------------
        直接调用
        */

        static initService() {
            System.init();
            Service.init();
            Service.registerNotifyCallback(1001, null, System._testNotify);
            Service.registerNotifyCallback(1002, null, System._reddotNotify);
            Service.registerNotifyCallback(1003, null, System._onlineCrossDay);
            Service.registerNotifyCallback(1005, null, System._mailMsgNotify);
            Service.registerNotifyCallback(1006, null, System._entrustMsgNotify);
            Service.registerNotifyCallback(1007, null, System._miniGameMsgNotify);
        }
        static pplogin() {
            PLAY68_SDK.start(function (status, player) {
                if (status) {
                    console.log(player.id, 'pp的id');
                    console.log(player.name); // play68 此时可以拿到用户名和头像了。注意：头像可能为空，注意兼容。
                    PlayerSystem.playerInfo.nickName = player.name;
                    PlayerSystem.playerInfo.avatarUrl = player.photo;
                    SdkHelper.sendLioginPP(player.id);
                    console.log(PlayerSystem.playerInfo);
                    var httpRequest = new Laya.HttpRequest();
                    let url = "https://" + WxConfigMgr.getInstance().getGlobalUrl() + "/gatew";
                    //  let url="https://fy.hulai.com:17301/gatew";
                    let param = {
                        "id": player.id,
                        "platform": PLAY68_SDK.getPlatform(),
                        "version": Constants.ppversion,
                    };
                    let obj = JSON.stringify(param);
                    let data = 'opCode=110001&params={\'sdkName\': \'pp\', \'param\':\'' + obj + '\'}';
                    let header = '';
                    console.log(data, 'pp的data');
                    httpRequest.http.timeout = 10000;//设置超时时间；
                    httpRequest.on(Laya.Event.PROGRESS, this, e => {

                    });
                    httpRequest.once(Laya.Event.COMPLETE, this, res => {
                        console.log('pp登录成功', res);
                        let obj = JSON.parse(res.data);
                        let object = JSON.parse(obj.data);
                        System.setToken(object.token);
                        System.setRid(object.rid);
                        System.setServerTag(object.gameserverUrl);
                        System.setOpenId(object.openId);
                        console.log("WX Login done");
                        EventMgr.getInstance().postEvent("LOADING_SHOW_TRANSITION");
                        System.login(object.rid, Laya.Handler.create(this, function () {
                            SceneMgr.getInstance().loadScene(Constants.NauticalSceneId, Laya.Handler.create(this, function (scene1) {
                                console.debug("handler1", 1);
                                SceneMgr.getInstance().loadScene(Constants.BeachSceneId, Laya.Handler.create(this, function (scene2) {
                                    SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId, Laya.Handler.create(this, function (scene3) {
                                        console.debug("handler3", 1);
                                        InGameLoader.getInstance().start();
                                        SceneMgr.getInstance().setSwitchAllowed();
                                        Laya.stage.removeChildByName("LoadTransition");
                                    }));
                                    console.debug("handler2", 2);
                                }));
                                console.debug("handler1", 2);
                            }));
                        }));
                    });
                    httpRequest.once(Laya.Event.ERROR, this, e => {
                        console.log('pp登录失败', e);
                    });
                    httpRequest.send(url, data, 'POST', "json");
                } else {
                    console.log(player); // {code: 1, message: "错误信息"}
                }
            });
        }
        static wxLogin(handler) {
            if (_wxLoginState == 1)
                return;
            _wxLoginState = 1;
            let wx_login_begin = (+ new Date());
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["wx_login_start"]);
            wx.login({
                success(res) {
                    if (res.code) {
                        // console.log(res);
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["wx_login_succ", { cost: (+ new Date()) - wx_login_begin }]);
                        wx.request({
                            url: "https://" + WxConfigMgr.getInstance().getGlobalUrl() + "/gatew",
                            // url: 'https://fy.hulai.com:17305/gatew',
                            method: 'POST',
                            header: {
                                'content-type': 'application/x-www-form-urlencoded'
                            },
                            data: 'opCode=110001&params={\'sdkName\': \'wx\', \'param\':\'' + res.code + '\'}',
                            success(res) {
                                console.log(res);
                                let obj = JSON.parse(res.data.data);
                                console.debug("登录发送code服务器返回:" + JSON.stringify(obj));
                                console.debug(obj);
                                if (obj.code != 0) {
                                    System._needRestart = true;
                                    console.log("gatew 返回", obj.code, obj.error_msg);
                                    if (obj.code == 40003) {
                                        //TODO
                                        EventMgr.getInstance().postEvent("LOADING_SHOW_MAINTAIN");
                                    }
                                    return;
                                }
                                let object = JSON.parse(obj.data);
                                System.setToken(object.token);
                                System.setRid(object.rid);
                                System.setServerTag(object.gameserverUrl);
                                System.setOpenId(object.openId);
                                console.log("WX Login done");
                                console.log("sendLoginLog", {
                                    app_id: _appId,
                                    open_id: _openId,
                                    uid: Number(System.getRid().split('_')[0]),
                                    create_time: Number(object.createTs),
                                    login_times: Number(object.loginTimes)
                                });
                                System._login_times = Number(object.loginTimes);
                                SdkHelper.sendLoginLog({
                                    app_id: _appId,
                                    open_id: _openId,
                                    uid: Number(System.getRid().split('_')[0]),
                                    create_time: Number(object.createTs),
                                    login_times: Number(object.loginTimes)
                                });
                                PlayerSystem.setCreateTime(Number(object.createTs));
                                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["wx_login_gatew_done", { cost: (+ new Date()) - wx_login_begin }]);
                                _wxLoginState = 2;
                                if (handler)
                                    handler.run();
                            },
                            fail() {
                                _wxLoginState = 0;
                                System._needRestart = true;
                                console.debug("登录发送code服务器返回失败!!!");
                                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["wx_login_gatew_fail"]);
                            }
                        });
                    } else {
                        _wxLoginState = 0;
                        System._needRestart = true;
                        console.log('登录失败！' + res.errMsg);
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["wx_login_fail"]);
                    }
                }
            });
        }

        static login(rid, handler) {
            Laya.timer.clear(System, System.login);
            console.log("login initSystems", (+ new Date()));
            System.initSystems();
            console.log("start login", (+ new Date()));

            let deviceType = "";
            if (Laya.Browser.onAndroid) {
                deviceType = "Android";
            } else if (Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) {
                deviceType = "IOS";
            } else {
                deviceType = "diannao";
            }
            let ppInfo = {
                "gameId": "47",
                "channelId": "223",
                "subGameId": "1417",
                "platformId": '2',
                "buildVersion": "" + Constants.ppversion,
                "devId": "211065FE-8E15-49E5-A03C-273B877BE48C",
                "deviceType": deviceType,
                "os": "11.3",
                "source": "2",
                "apiKey": "db96abc4109f6d516b9e5e0a67aa79e9"
            };

            if (Laya.Browser.onMiniGame) {
                if (_wxLoginState == 0) {
                    System.wxLogin(Laya.Handler.create(System, System.login, [rid, handler]));
                } else if (_wxLoginState == 1) {
                    Laya.timer.frameOnce(1, System, System.login, [rid, handler]);
                } else {
                    let server_login_begin = (+ new Date());
                    let dealFunc = function (res) {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["server_login_done", { cost: (+ new Date()) - server_login_begin }]);
                        console.log("loginRes:", res);
                        if (res.extraInfo) {
                            if (res.extraInfo.openId) {
                                System.setOpenId(res.extraInfo.openId);
                            }
                            if (Number(res.extraInfo.loginTimes) <= 1) {
                                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["guide_create_account"]);
                            }
                        }
                        System._dealLoginRes(res);
                        System.setLogined();
                        if (handler) {
                            Laya.LocalStorage.setItem("user_rid", rid);
                            handler.runWith([res]);
                        }
                    };
                    Service.login(System.getServerUrl(), "1", System.getRid(), System.getToken(), PlayerSystem.getPlayerInfo(), null, System.getAuthorized(), Laya.Handler.create(null, dealFunc));

                }
            }
            else if (!Laya.Browser.onMiniGame && ((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) && Constants.shouji)) {
                let dealFunc = function (res) {
                    System._dealLoginRes(res);
                    System.setLogined();
                    if (handler) {
                        Laya.LocalStorage.setItem("user_rid", rid);
                        handler.runWith([res]);
                    }
                };
                System.setRid(rid);
                let a = System.getServerUrl();
                console.log(a);
                Service.login(System.getServerUrl(), "1", System.getRid(), System.getToken(), PlayerSystem.getPlayerInfo(), ppInfo, false, Laya.Handler.create(null, dealFunc));
            }
            else {
                let dealFunc = function (res) {
                    System._dealLoginRes(res);
                    System.setLogined();
                    if (handler) {
                        Laya.LocalStorage.setItem("user_rid", rid);
                        handler.runWith([res]);
                    }
                };
                System.setRid(rid);
                Service.login(System.getServerUrl(), serverId, System.getRid(), System.getToken(), PlayerSystem.getPlayerInfo(), ppInfo, false, Laya.Handler.create(null, dealFunc));
            }
            PaySystem.setTrackData(serverId, System.getRid(), PlayerSystem.getPlayerInfo());
        }

        static onlineCrossDay(handler) {
            let dealFunc = function (res) {
                System._dealLoginRes(res);
                if (handler)
                    handler.runWith([res]);
            };
            Service.onlineCrossDay(Laya.Handler.create(null, dealFunc));
        }

        static debug(code, param, handler) {
            let dealFunc = function (res) {
                System._dealLoginRes(res.loginRes);
                if (handler) {
                    handler.runWith([res]);
                }
            };
            Service.debug(code, param, Laya.Handler.create(null, dealFunc));
        }

        static getFullBag(handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getFullBag(Laya.Handler.create(null, dealFunc));
        }

        static putCustomData(customKV, handler) {
            let dealFunc = function (res) {
                let customKV = res.customKV;
                GuideSystem.updateGuideProgress(Utils.obj2Map(customKV));
                MiniGameSystem.updateOtherData(customKV);
                PetSystem.updataHomeRubbish(customKV);
                if (handler)
                    handler.runWith([res]);
            };
            Service.putCustomData(customKV, Laya.Handler.create(null, dealFunc));
        }

        static delCustomData(keys, handler) {
            let dealFunc = function (res) {
                let customKV = res.customKV;
                GuideSystem.updateGuideProgress(Utils.obj2Map(customKV));
                MiniGameSystem.updateOtherData(customKV);
                PetSystem.updataHomeRubbish(customKV);
                if (handler)
                    handler.runWith([res]);
            };
            Service.delCustomData(keys, Laya.Handler.create(null, dealFunc));
        }

        static getCustomData(handler) {
            let dealFunc = function (res) {
                let customKV = res.customKV;
                GuideSystem.updateGuideProgress(Utils.obj2Map(customKV));
                MiniGameSystem.updateOtherData(customKV);
                PetSystem.updataHomeRubbish(customKV);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getCustomData(Laya.Handler.create(null, dealFunc));
        }

        static viewAd(status, type, handler = null) {
            Service.viewAd(Number(status), String(type), handler);
        }

        static shareReq(status, type, handler = null) {
            Service.shareReq(Number(status), String(type), handler);
        }


        static authorizeRequest(userInfo, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.getAuthorize(userInfo, Laya.Handler.create(null, dealFunc));
        }

        static sendClientLoadingLog(customData, handler = null) {
            Service.sendClientLoadingLog(customData, handler);
        }

        static getAllBuilding(handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let playerBuilding = res.playerBuilding;
                System._updateFullBag(bag);
                System._updateBuilding(playerBuilding);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getAllBuilding(Laya.Handler.create(null, dealFunc));
        }

        static getMileStoneReward(milestoneId, fetchType, handler) {
            let dealFunc = function (res) {
                let finishedMilestone = res.finishedMilestone;
                MileStoneSystem.updateList(finishedMilestone);
                System._dealWithRewards(res.milestoneRewards);
                if (handler)
                    handler.runWith([res]);
            };
            Service.FetchMilestoneReward(milestoneId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        /**----------------------------------------------------------------------
        通过EVENT 调用
        */
        //建筑
        static putBuilding(buildingId, baseId, fetchType, handler) {
            // console.debug("putBuilding", buildingId, baseId, handler)
            let dealFunc = function (res) {
                let bag = res.bag;
                let letterBox = res.letterbox;
                System._updateFullBag(bag);
                System._updateLetterBox(letterBox);
                if (handler)
                    handler.runWith([res]);
                let adRewardsInfo = res.adRewardsInfo;
                if (adRewardsInfo) {
                    PlayerSystem.setAdRewardsInfo(adRewardsInfo);
                }
            };
            Service.putBuilding(buildingId, baseId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        // 这个接口目前还没用过
        static removeBuilding(buildingId, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.removeBuilding(buildingId, Laya.Handler.create(null, dealFunc));
        }

        static bargain(fetchType, fishAmount, buildingId, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.bargain(fetchType, fishAmount, buildingId, Laya.Handler.create(null, dealFunc));
        }

        //任务
        static getAllTasks(handler) {
            let dealFunc = function (res) {
                if (res)
                    System._updateTasks(res.tasks, res.registerDays);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getAllTasks(Laya.Handler.create(null, dealFunc));
        }

        static getDailyTaskReward(taskId, fetchType, handler) {
            let dealFunc = function (res) {
                let tasks = res.tasks;
                let bag = res.bag;
                let reward = res.rewards;
                System._updateTasks(tasks);
                System._updateFullBag(bag, false);
                //TODO show reward
                if (handler)
                    handler.runWith([res]);
                let adRewardsInfo = res.adRewardsInfo;
                if (adRewardsInfo) {
                    PlayerSystem.setAdRewardsInfo(adRewardsInfo);
                }
            };
            Service.getDailyTaskReward(taskId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        static getTrophyTaskReward(taskId, fetchType, handler) {
            let dealFunc = function (res) {
                let tasks = res.tasks;
                let bag = res.bag;
                let reward = res.rewards;
                System._updateTasks(tasks);
                System._updateFullBag(bag, false);
                //TODO show reward
                if (handler)
                    handler.runWith([res]);
                let adRewardsInfo = res.adRewardsInfo;
                if (adRewardsInfo) {
                    PlayerSystem.setAdRewardsInfo(adRewardsInfo);
                }
            };
            Service.getTrophyTaskReward(taskId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        static getMainTaskReward(taskId, handler) {
            let dealFunc = function (res) {
                let tasks = res.tasks;
                let bag = res.bag;
                let reward = res.rewards;
                System._updateTasks(tasks);
                System._updateFullBag(bag, false);
                //TODO show reward
                if (handler)
                    handler.runWith([res]);
            };
            Service.getMainTaskReward(taskId, Laya.Handler.create(null, dealFunc));
        }

        //菜谱
        static getPlayerFoodMenu(handler) {
            let dealFunc = function (res) {
                if (res)
                    System._updateFoods(res.menu);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getPlayerFoodMenu(Laya.Handler.create(null, dealFunc));
        }

        static foodLevelUp(foodListId, fetchType, handler) {
            let dealFunc = function (res) {
                let menu = res.menu;
                let bag = res.bag;
                System._updateFoods(menu);
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
                let adRewardsInfo = res.adRewardsInfo;
                if (adRewardsInfo) {
                    PlayerSystem.setAdRewardsInfo(adRewardsInfo);
                }
            };
            Service.foodLevelUp(foodListId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        //场景同步
        static getLetterBoxInfo(handler) {
            let dealFunc = function (res) {
                SdkHelper.tuisongDays();
                let letterBox = res.letterbox;
                System._updateLetterBox(letterBox);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getLetterBoxInfo(Laya.Handler.create(null, dealFunc));
        }

        static fetchLetterBox(fetchType, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let letterBox = res.letterbox;
                SdkHelper.shanchuTuisong('zhaocai_mao_1');
                let time = Math.floor((Number(letterBox.fullMills) - TimeUtils.getTimeNowOnServer()) / 1000);
                SdkHelper.goTuisong('zhaocai_mao_1', Strings$1.get('tid#WXMessage_TipName'), Strings$1.get('tid#WXMessage_TipDesc'), '' + time);
                System._updateFullBag(bag, false);
                System._updateLetterBox(letterBox);
                if (handler)
                    handler.runWith([res]);
            };
            Service.fetchLetterBox(fetchType == 1 ? true : false, Laya.Handler.create(null, dealFunc));
        }

        static updateSceneInfo(customData, rewards, petRewards, visitorRewards, events, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let letterBox = res.letterbox;
                let wharfLetterBox = res.wharfLetterBox;
                let aidedInfoGroup = res.aidedGroupInfo.aidedInfoGroup;
                let aidedInfo = res.aidedGroupInfo.aidedInfo;
                let slogan = res.aidedGroupInfo.slogan;
                System._updateFullBag(bag);
                System._updateLetterBox(letterBox);
                SyncSystem.dropCache();
                System._updatewharfLetterBox(wharfLetterBox);
                if (aidedInfoGroup) {
                    System._updateZhuliList(aidedInfoGroup);
                }
                else if (aidedInfo) {
                    System._updateZhuXinxi(aidedInfo);
                }
                if (slogan) {
                    System._updateBiaoyu(slogan);
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.updateSceneInfo(customData, rewards, petRewards, visitorRewards, events, Laya.Handler.create(null, dealFunc));
        }

        //宠物 图鉴
        static getPetsInfo(handler) {
            let dealFunc = function (res) {
                let pets = res.playerPets;
                System._updatePets(pets);
                EventMgr.getInstance().postEvent("TASK_POP_SHOW");
                if (handler)
                    handler.runWith([res]);
            };
            Service.getPetsInfo(Laya.Handler.create(null, dealFunc));
        }

        static adoptPet(petId, type, handler) {
            let dealFunc = function (res) {
                if (type == 2) {
                    //打点
                    console.log("钻石打点" + "Adopt");
                    let str = 'gemBuy';
                    let obj = { 'gemUnit': "Adopt" };
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                }
                let currencyBag = res.currencyBag;
                if (currencyBag) {
                    System._updateFullBag(currencyBag);
                }
                PetSystem.doAdoptPet(petId);
                let str = 'catAdopted';
                let number = PetDataMgr.getInstance().PetsAdopted.length;
                let obj = { 'cat': petId, 'totalUnlock': number };
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                if (handler)
                    handler.runWith([res]);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_adopt_cat"]);
            };
            Service.adoptPet(petId, type, Laya.Handler.create(null, dealFunc));
        }

        static storyLock(petId, num, handler) {
            let dealFunc = function (res) {
                PetSystem.updatePet(res.petInfo);
                let rewards = res.rewards.rewards[0].amount;
                PlayerSystem.secretelyCumulateStar(rewards);
                if (handler)
                    handler.runWith([rewards]);
            };
            Service.storyLock(petId, num, Laya.Handler.create(null, dealFunc));
        }

        static petShareGetReward(petId, type, handler) {
            let dealFunc = function (res) {
                let num = res.rewards.rewards[0].amount;
                let rewardType = res.rewards.rewards[0].id;
                if (rewardType == Constants.RewardType.FISH) {
                    PlayerSystem.secretlyCumulateFish(num);
                }
                else {
                    PlayerSystem.secretelyCumulateStar(num);
                }
                if (handler)
                    handler.runWith(res);
            };
            Service.petShareGetReward(petId, type, Laya.Handler.create(null, dealFunc));
        }

        static catLevelUp(petId, handler) {
            let dealFunc = function (res) {
                let pets = res.playerPets;
                System._updatePets(pets);
                System._updateFullBag(res.bag, false);
                if (handler)
                    handler.runWith(res);
            };
            Service.catLevelUp(petId, Laya.Handler.create(null, dealFunc));
        }

        static catStarUp(petId, handler) {
            let dealFunc = function (res) {
                let pets = res.playerPets;
                System._updatePets(pets);
                System._updateFullBag(res.bag, false);
                if (handler)
                    handler.runWith(res);
            };
            Service.catStarUpgrade(petId, Laya.Handler.create(null, dealFunc));
        }

        //长客
        static findVisitorGift(handler) {
            let dealFunc = function (res) {
                let visitorRewards = res.visitorRewards;
                System._updateVisitorGift(visitorRewards);
                if (handler)
                    handler.runWith([res]);
            };
            Service.findVisitorGift(Laya.Handler.create(null, dealFunc));
        }

        static getVisitorGiftAll(fetchType, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let reward = res.rewards;
                System._updateFullBag(bag, false);
                SyncSystem.clearVisitorGift();
                //TODO show reward
                if (handler)
                    handler.runWith([res]);
            };
            Service.getVisitorGiftAll(fetchType, Laya.Handler.create(null, dealFunc));
        }

        static getVisitorGiftSingle(giftId, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let reward = res.rewards;
                let visitorRewards = res.visitorRewards;
                System._updateFullBag(bag, false);
                System._updateVisitorGift(visitorRewards);
                //TODO show reward
                if (handler)
                    handler.runWith([res]);
            };
            Service.getVisitorGiftSingle(giftId, Laya.Handler.create(null, dealFunc));
        }

        static catFoodSubscribe(endTime, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.catFoodSubscribe(endTime, Laya.Handler.create(null, dealFunc));
        }

        //探险
        static getNauticalInfo(handler) {
            let dealFunc = function (res) {
                let nauticalInfo = res.nauticalInfo;
                let petIds = res.petIds;
                let nauticalFoods = res.nauticalFoods;
                let nauticalTools = res.nauticalTools;
                let nauticalTicketNum = res.nauticalTicketNum;
                NauticalSystem$1.updateAll(nauticalInfo, petIds, nauticalFoods, nauticalTools, nauticalTicketNum);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getNauticalInfo(Laya.Handler.create(null, dealFunc));
        }

        static mergeNauticalFood(nauticalFoodId, nauticalFoodCount, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.mergeNauticalFood(nauticalFoodId, nauticalFoodCount, Laya.Handler.create(null, dealFunc));
        }

        static startNauticalTrip(sailIndex, info, handler) {
            let dealFunc = function (res) {
                let nauticalInfo = res.nauticalInfo;
                let petIds = res.petIds;
                let nauticalFoods = res.nauticalFoods;
                let nauticalTools = res.nauticalTools;
                NauticalSystem$1.updateAll(null, petIds, null, nauticalTools);
                NauticalSystem$1.updateOnlyNauticalInfo(sailIndex, nauticalInfo);
                if (handler)
                    handler.runWith([res]);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_begin_sail"]);
            };
            Service.startNauticalTrip(info, Laya.Handler.create(null, dealFunc));
        }

        static finishNauticalTrip(type, tripIndex, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let reward = res.rewards;
                let petInfos = res.petInfos;
                let photoRewards = res.photoRewards;
                let petIds = res.petIds;
                let nauticalFoods = res.nauticalFoods;
                let nauticalTools = res.nauticalTools;
                NauticalSystem$1.updateAll(null, petIds, nauticalFoods, nauticalTools);
                System._dealWithPhotoRewards(photoRewards);
                System._updateFullBag(bag, false);
                PetSystem.updatePets(petInfos);
                //TODO show reward
                NauticalSystem$1.clearNauticalInfo(tripIndex);
                if (handler)
                    handler.runWith([res]);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_end_sail"]);
            };
            Service.finishNauticalTrip(type, tripIndex, Laya.Handler.create(null, dealFunc));
        }

        static previewNauticalTripReq(tripIndex, handler) {
            let dealFunc = function (res) {
                let reward = res.rewards;
                if (handler)
                    handler.runWith([res, tripIndex]);
            };
            Service.previewNauticalTripReq(tripIndex, Laya.Handler.create(null, dealFunc));
        }

        static adReduceNauticalTime(tripIndex, type, handler) {
            let dealFunc = function (res) {
                let nauticalInfo = res.nauticalInfo;
                let nauticalTicketNum = res.nauticalTicketNum;
                NauticalSystem$1.updateNauticalTicketNum(nauticalTicketNum);
                NauticalSystem$1.updateOnlyNauticalInfo(tripIndex, nauticalInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.adReduceNauticalTime(tripIndex, type, Laya.Handler.create(null, dealFunc));
        }

        static unlockNautical(tripIndex, handler) {
            let dealFunc = function (res) {
                let nauticalInfo = res.nauticalInfo;
                NauticalSystem$1.updateOnlyNauticalInfo(tripIndex, nauticalInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.unlockNautical(tripIndex, Laya.Handler.create(null, dealFunc));
        }

        //钓鱼
        static getFishGameInfo(handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let gameInfo = res.gameInfo;
                System._updateFullBag(bag);
                System._updateFishGameInfo(gameInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getFishGameInfo(Laya.Handler.create(null, dealFunc));
        }

        static playFishGame(startType, fishingFishId, fishingBaitId, rewards, fishingFishSize, fetchType, handler, errorHandler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                let gameInfo = res.gameInfo;
                let actualRewards = res.actualRewards;
                // System._updateFullBag(bag, false);
                System._updateFishGameInfo(gameInfo);
                //TODO by zhouhuikun
                System._dealWithRewards(actualRewards);
                if (handler)
                    handler.runWith([res]);
            };
            Service.playFishGame(startType, fishingFishId, fishingBaitId, rewards, fishingFishSize, fetchType, Laya.Handler.create(null, dealFunc), errorHandler);
        }

        static getFishingFishInfo(handler) {
            let dealFunc = function (res) {
                let bag = res.fishInfos;
                System._updateFishCollectInfo(res.fishInfos);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getFishingFishInfo(Laya.Handler.create(null, dealFunc));
        }

        static getFishingAdFreeInfo(handler) {
            let dealFunc = function (res) {
                System._updateFishGameInfo(res.gameInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getFishingAdFreeInfo(Laya.Handler.create(null, dealFunc));
        }


        //家园
        static setPetStatus(params, handler) {
            let dealFunc = function (res) {
                let petInfos = res.petInfos;
                PetSystem.updatePets(petInfos);
                if (handler)
                    handler.runWith([res]);
            };
            Service.setPetStatus(params, Laya.Handler.create(null, dealFunc));
        }

        static cuteInteract(petId, toyId, handler) {
            let dealFunc = function (res) {
                TaskMainSystem.addProgress("HOME_PLAY", 1);
                let increase = res.increase; // 增加的好感度；
                let rewards = res.rewards; // 奖励
                let petInfo = res.petInfo;
                PetSystem.updatePet(petInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.cuteInteract(petId, toyId, Laya.Handler.create(null, dealFunc));
        }

        static commonInteract(petId, toyId, handler) {
            let dealFunc = function (res) {
                TaskMainSystem.addProgress("HOME_PLAY", 1);
                let increase = res.increase; // 增加的好感度；
                let rewards = res.rewards; // 奖励
                let petInfo = res.petInfo;
                PetSystem.updatePet(petInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.commonInteract(petId, toyId, Laya.Handler.create(null, dealFunc));
        }

        static fetchPhotoWall(handler) {
            let dealFunc = function (res) {
                let photoWall = res.photoWall;
                // TODO 更新照片墙信息
                PhotoSystem.updataPhotosData(photoWall);
                if (handler)
                    handler.runWith([res]);
            };
            Service.fetchPhotoWall(Laya.Handler.create(null, dealFunc));
        }

        static fetchHomeInfo(handler) {
            let dealFunc = function (res) {
                let homeInfo = res.homeInfo;
                PetSystem.updateHomeInfo(homeInfo);
                let photoWall = homeInfo.photoWall;
                PhotoSystem.updataPhotosData(photoWall);
                if (handler)
                    handler.runWith([res]);
            };
            Service.fetchHomeInfo(Laya.Handler.create(null, dealFunc));
        }

        static healthyInteract(petId, type, handler) {
            let dealFunc = function (res) {
                //EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["HOME_HEALTH", 1, null]);   
                if (handler)
                    handler.runWith([res]);
            };
            Service.healthyInteract(petId, type, Laya.Handler.create(null, dealFunc));
        }


        //邮件
        static fetchAllMails(handler) {
            let dealFunc = function (res) {
                MailSystem.updataMail(res.mails);
                if (handler)
                    handler.runWith([res]);
            };
            Service.fetchAllMails(Laya.Handler.create(null, dealFunc));
        }

        static readMail(mailId, handler) {
            let dealFunc = function (res) {
                MailSystem.readMail(res.mail);
                if (handler)
                    handler.runWith([res]);
            };
            Service.readMail(mailId, Laya.Handler.create(null, dealFunc));
        }

        static deleteMail(mailId, handler) {
            let dealFunc = function (res) {
                MailSystem.deleteMail(res.mailId);
                if (handler)
                    handler.runWith([res]);
            };
            Service.deleteMail(mailId, Laya.Handler.create(null, dealFunc));
        }

        static deleteReadMails(handler) {
            let dealFunc = function (res) {
                MailSystem.deleteReadMails(res.mails);
                if (handler)
                    handler.runWith([res]);
            };
            Service.deleteReadMails(Laya.Handler.create(null, dealFunc));
        }

        static receiveMailRewards(mailId, handler) {
            let dealFunc = function (res) {
                let rewards = res.rewards;
                System._dealWithRewards(rewards);
                MailSystem.updataMail(res.mails);
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveMailRewards(mailId, Laya.Handler.create(null, dealFunc));
        }

        static fireEvent(type, value, extraInfo, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.fireEvent(type, value, extraInfo, Laya.Handler.create(null, dealFunc));
        }

        static getEntrust(handler) {
            let dealFunc = function (res) {
                let registerDays = res.registerDays;
                let entrusts = res.entrusts;
                if (registerDays != null) {
                    PlayerSystem.setRegisterDays(registerDays);
                }
                EntrustSystem.updateEntrusts(entrusts);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getEntrust(Laya.Handler.create(null, dealFunc));
        }

        static acceptEntrust(listId, handler) {
            let dealFunc = function (res) {
                EntrustSystem.acceptComplete(res.entrust);
                if (handler)
                    handler.runWith([res]);
            };
            Service.acceptEntrust(listId, Laya.Handler.create(null, dealFunc));
        }

        static receiveRewards(listId, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag, false);
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveRewards(listId, Laya.Handler.create(null, dealFunc));
        }

        static sendSuggest(content, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.sendSuggest(content, Laya.Handler.create(null, dealFunc));
        }

        //气球
        static getBalloonInfo(fetchType, handler) {
            let dealFunc = function (res) {
                let balloonNum = res.balloon.dailyCount;
                let rewards = res.rewards.rewards;
                let fishReward = 0;
                for (let i in rewards) {
                    if (rewards[i].id == Constants.RewardType.FISH) {
                        fishReward = rewards[i].amount;
                        PlayerSystem.secretlyCumulateFish(rewards[i].amount);
                    } else if (rewards[i].id == Constants.RewardType.STAR) {
                        PlayerSystem.secretelyCumulateStar(rewards[i].amount);
                    }
                }
                if (handler)
                    handler.runWith([balloonNum, fishReward]);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_end_balloon"]);
            };
            Service.getBalloonInfo(fetchType, Laya.Handler.create(null, dealFunc));
        }

        static getBalloonNum(handler) {
            let dealFunc = function (res) {
                let rewards = res.rewards.rewards;
                let fishReward = 0;
                for (let i in rewards) {
                    if (rewards[i].id == Constants.RewardType.FISH) {
                        fishReward = rewards[i].amount;
                    }
                }
                BalloonSystem.setRewardNum(fishReward);
                if (handler)
                    handler.runWith([fishReward]);
            };
            Service.getBalloonNum(Laya.Handler.create(null, dealFunc));
        }

        //小游戏
        static getMiniGame(handler) {
            let dealFunc = function (res) {
                MiniGameSystem.updataMiniGame(res);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getMiniGame(Laya.Handler.create(null, dealFunc));
        }

        static finishMiniGame(listId, result, fetchType, handler) {
            let dealFunc = function (res) {
                let rewards = res.rewards.rewards;
                for (let i in rewards) {
                    if (rewards[i].id == Constants.RewardType.FISH) {
                        PlayerSystem.secretlyCumulateFish(rewards[i].amount);
                    } else if (rewards[i].id == Constants.RewardType.STAR) {
                        PlayerSystem.secretelyCumulateStar(rewards[i].amount);
                    }
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.finishMiniGame(listId, result, fetchType, Laya.Handler.create(null, dealFunc));

        }

        static ClientTriggerMiniGame(handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.ClientTriggerMiniGame(Laya.Handler.create(null, dealFunc));
        }

        static GiveUpMiniGame(handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.GiveUpMiniGame(Laya.Handler.create(null, dealFunc));
        }


        //挂机
        static queryLuckyBagInfo(handler) {
            let dealFunc = function (res) {
                let luckyBagInfo = res.luckyBagInfo;
                LuckyBagSystem.update(luckyBagInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.queryLuckyBagInfo(Laya.Handler.create(null, dealFunc));
        }

        static receiveLuckyBag(fetchType, handler) {
            let dealFunc = function (res) {
                let luckyBagInfo = res.luckyBagInfo;
                LuckyBagSystem.update(luckyBagInfo);
                let rewards = res.rewards.rewards;
                for (let i in rewards) {
                    if (rewards[i].id == Constants.RewardType.FISH) {
                        PlayerSystem.secretlyCumulateFish(rewards[i].amount);
                    } else if (rewards[i].id == Constants.RewardType.STAR) {
                        PlayerSystem.secretelyCumulateStar(rewards[i].amount);
                    }
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveLuckyBag(fetchType, Laya.Handler.create(null, dealFunc));
        }

        //员工
        static getStaffSkillGroup(staffGroupId, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.getStaffSkillGroup(staffGroupId, Laya.Handler.create(null, dealFunc));
        }

        static skillLevelUp(skillGroupId, staffGroupId, fetchType, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.skillLevelUp(skillGroupId, staffGroupId, fetchType, Laya.Handler.create(null, dealFunc));
        }

        static staffLevelUp(staffGroupId, handler) {
            let dealFunc = function (res) {
                let bag = res.bag;
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.staffLevelUp(staffGroupId, Laya.Handler.create(null, dealFunc));
        }

        static selectSkill(skillGroupId, staffGroupId, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.selectSkill(skillGroupId, staffGroupId, Laya.Handler.create(null, dealFunc));
        }

        static getStaffGroup(handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.getStaffGroup(Laya.Handler.create(null, dealFunc));
        }
        //added by tenkin
        static getCollectGiftInfoRes(handler) {
            let dealFunc = function (res) {
                CollectGiftDataMgr.getInstance().setData(res);
                LevelToySystem.updateData(res.levelToyMap);
                if (handler)
                    handler.runWith([res]);
            };
            Service.GetCollectGiftInfoReq(Laya.Handler.create(null, dealFunc));
        }
        static reduceCDTime(skillGroupId, staffGroupId, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.reduceCDTime(skillGroupId, staffGroupId, Laya.Handler.create(null, dealFunc));
        }
        //助力
        static GetNauticalProjectGroup(handler) {
            let dealFunc = function (res) {
                if (res)
                    //  System._updateFoods(res.menu);
                    if (handler)
                        handler.runWith([res]);
            };
            Service.GetNauticalProjectGroup(Laya.Handler.create(null, dealFunc));
        }
        static projectLevelUp(projectListId, handler) {
            let dealFunc = function (res) {
                let group = res.group;
                let bag = res.bag;
                System._updateProjects(group);
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.ProjectLevelUp(projectListId, Laya.Handler.create(null, dealFunc));
        }

        static FetchLuckyDrawReward(type, rewardInfo, handler) {
            let dealFunc = function (res) {
                let diamound = res.luckyDraw.diamondAmount ? res.luckyDraw.diamondAmount : 0;
                PlayerSystem.updateDiamond(diamound);
                LuckydrawSystem.updateLuckydraw(res.luckyDraw);
                System._dealWithRewards(res.rewards);
                let ticketBuildingInfos = res.ticketBuildingInfos;
                BuildingSystem.updateTicketBuildingInfos(ticketBuildingInfos);
                if (handler)
                    handler.runWith([res]);
            };
            Service.FetchLuckyDrawReward(type, rewardInfo, Laya.Handler.create(null, dealFunc));
        }

        static GetLuckyDrawInfo(handler) {
            let dealFunc = function (res) {
                LuckydrawSystem.updateLuckydraw(res.luckyDraw);
                if (handler)
                    handler.runWith([res]);
            };
            Service.GetLuckyDrawInfo(Laya.Handler.create(null, dealFunc));
        }

        static FetchBuildingTicketReward(ticketIndex, posId, handler) {
            let dealFunc = function (res) {
                let lotteryBag = res.lotteryBag;
                if (lotteryBag) {
                    let ticketBuildingInfos = lotteryBag.ticketBuildingInfos;
                    BuildingSystem.updateTicketBuildingInfos(ticketBuildingInfos);
                }
                let currencyBag = res.currencyBag;
                System._updateFullBag(currencyBag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.FetchBuildingTicketReward(ticketIndex, posId, Laya.Handler.create(null, dealFunc));
        }

        static FinishBuildingCoupon(handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.FinishBuildingCoupon(Laya.Handler.create(null, dealFunc));
        }

        static UseAdCoupon(adCouponType, handler) {
            let dealFunc = function (res) {
                let lotteryBag = res.lotteryBag;
                if (lotteryBag) {
                    let fishingTicketNum = lotteryBag.fishingTicketNum ? lotteryBag.fishingTicketNum : 0;
                    let nauticalSpeedUpNum = lotteryBag.nauticalSpeedUpNum ? lotteryBag.nauticalSpeedUpNum : 0;
                    let skipAdInviteNum = lotteryBag.skipAdInviteNum ? lotteryBag.skipAdInviteNum : 0;
                    let catFoodNum = lotteryBag.catFoodNum ? lotteryBag.catFoodNum : 0;
                    FishingSystem.setFishingTicketNum(fishingTicketNum);
                    NauticalSystem$1.updateNauticalTicketNum(nauticalSpeedUpNum);
                    PaySystem.setSkipAdInviteNum(skipAdInviteNum);
                    PaySystem.setCatFoodNum(catFoodNum);
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.UseAdCoupon(adCouponType, Laya.Handler.create(null, dealFunc));
        }

        static GetLotteryBag(handler) {
            let dealFunc = function (res) {
                let lotteryBag = res.lotteryBag;
                if (lotteryBag) {
                    let fishingTicketNum = lotteryBag.fishingTicketNum ? lotteryBag.fishingTicketNum : 0;
                    let nauticalSpeedUpNum = lotteryBag.nauticalSpeedUpNum ? lotteryBag.nauticalSpeedUpNum : 0;
                    let skipAdInviteNum = lotteryBag.skipAdInviteNum ? lotteryBag.skipAdInviteNum : 0;
                    let catFoodNum = lotteryBag.catFoodNum ? lotteryBag.catFoodNum : 0;
                    FishingSystem.setFishingTicketNum(fishingTicketNum);
                    NauticalSystem$1.updateNauticalTicketNum(nauticalSpeedUpNum);
                    PaySystem.setSkipAdInviteNum(skipAdInviteNum);
                    PaySystem.setCatFoodNum(catFoodNum);
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.GetLotteryBag(Laya.Handler.create(null, dealFunc));
        }

        static SetActivityState(data) {//todo setData
            GameContext.inviteBtnState = data.inviteTask;
            GameContext.imageBtnState = true;
            GameContext.signBtnState = true;
        }


        //万事屋
        static getCatMissionInfo(handler) {
            let dealFunc = function (res) {
                let catMissionInfo = res.catMissionInfo;
                let catMissionPetInfo = res.catMissionPetInfo;
                MissionSystem.updateMissionInfo(catMissionInfo);
                MissionSystem.updatePetInfo(catMissionPetInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.getCatMissionInfo(Laya.Handler.create(null, dealFunc));
        }

        static refreshCatMission(handler) {
            let dealFunc = function (res) {
                let catMissionInfo = res.catMissionInfo;
                MissionSystem.updateMissionInfo(catMissionInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.refreshCatMission(Laya.Handler.create(null, dealFunc));
        }

        static changeRefreshLevel(level, handler) {
            let dealFunc = function (res) {
                MissionSystem.setRefreshLevel(level);
                if (handler)
                    handler.runWith([res]);
            };
            Service.changeRefreshLevel(level, Laya.Handler.create(null, dealFunc));
        }

        static receiveMissionReward(missionIndex, roleIds, handler) {
            let dealFunc = function (res) {
                let catMissionInfo = res.catMissionInfo;
                let catMissionPetInfo = res.catMissionPetInfo;
                let rewards = res.rewards;
                let bag = res.bag;
                MissionSystem.updateMissionInfo(catMissionInfo);
                MissionSystem.updatePetInfo(catMissionPetInfo);
                MissionSystem.clearTeam();
                System._updateFullBag(bag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveMissionReward(missionIndex, roleIds, Laya.Handler.create(null, dealFunc));
        }

        static resetCatStatus(roleId, handler) {
            let dealFunc = function (res) {
                let resetTs = res.resetTs;
                MissionSystem.resetCatStatusData(roleId, resetTs);
                if (handler)
                    handler.runWith([res]);
            };
            Service.resetCatStatus(roleId, Laya.Handler.create(null, dealFunc));
        }

        static adRefreshMission(handler) {
            let dealFunc = function (res) {
                let catMissionInfo = res.catMissionInfo;
                MissionSystem.updateMissionInfo(catMissionInfo);
                if (handler)
                    handler.runWith([res]);
            };
            Service.adRefreshMission(Laya.Handler.create(null, dealFunc));
        }

        //内购
        static createOrder(orderId, purchaseId, handler) {
            let dealFunc = function (res) {
                if (handler)
                    handler.runWith([res]);
            };
            Service.createOrder(orderId, purchaseId, Laya.Handler.create(null, dealFunc));
        }

        static finishOrder(orderId, purchaseId, handler) {
            let dealFunc = function (res) {
                let currencyBag = res.currencyBag;
                let payInfo = res.payInfo;
                let lotteryBag = res.lotteryBag;
                let rewards = res.rewards;
                System._updateFullBag(currencyBag);
                PaySystem.updatePayInfo(payInfo);
                if (lotteryBag) {
                    let fishingTicketNum = lotteryBag.fishingTicketNum ? lotteryBag.fishingTicketNum : 0;
                    let nauticalSpeedUpNum = lotteryBag.nauticalSpeedUpNum ? lotteryBag.nauticalSpeedUpNum : 0;
                    let skipAdInviteNum = lotteryBag.skipAdInviteNum ? lotteryBag.skipAdInviteNum : 0;
                    let catFoodNum = lotteryBag.catFoodNum ? lotteryBag.catFoodNum : 0;
                    FishingSystem.setFishingTicketNum(fishingTicketNum);
                    NauticalSystem$1.updateNauticalTicketNum(nauticalSpeedUpNum);
                    PaySystem.setSkipAdInviteNum(skipAdInviteNum);
                    PaySystem.setCatFoodNum(catFoodNum);
                }
                EventMgr.getInstance().postEvent("PURCHASE_COMPLETE");
                if (handler)
                    handler.runWith([res]);
            };
            Service.finishOrder(orderId, purchaseId, Laya.Handler.create(null, dealFunc));
        }

        static receiveFreeDiamond(handler) {
            let dealFunc = function (res) {
                let currencyBag = res.currencyBag;
                let payInfo = res.payInfo;
                let rewards = res.rewards;
                System._updateFullBag(currencyBag);
                PaySystem.updatePayInfo(payInfo);
                EventMgr.getInstance().postEvent("PURCHASE_COMPLETE", [rewards.rewards, 3]);
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveFreeDiamond(Laya.Handler.create(null, dealFunc));
        }

        static payCardBonus(cardType, handler) {
            let dealFunc = function (res) {
                let currencyBag = res.currencyBag;
                let payInfo = res.payInfo;
                let lotteryBag = res.lotteryBag;
                let rewards = res.rewards;
                System._updateFullBag(currencyBag);
                PaySystem.updatePayInfo(payInfo);
                if (lotteryBag) {
                    let fishingTicketNum = lotteryBag.fishingTicketNum ? lotteryBag.fishingTicketNum : 0;
                    let nauticalSpeedUpNum = lotteryBag.nauticalSpeedUpNum ? lotteryBag.nauticalSpeedUpNum : 0;
                    let skipAdInviteNum = lotteryBag.skipAdInviteNum ? lotteryBag.skipAdInviteNum : 0;
                    let catFoodNum = lotteryBag.catFoodNum ? lotteryBag.catFoodNum : 0;
                    FishingSystem.setFishingTicketNum(fishingTicketNum);
                    NauticalSystem$1.updateNauticalTicketNum(nauticalSpeedUpNum);
                    PaySystem.setSkipAdInviteNum(skipAdInviteNum);
                    PaySystem.setCatFoodNum(catFoodNum);
                }
                if (handler)
                    handler.runWith([res]);
            };
            Service.payCardBonus(cardType, Laya.Handler.create(null, dealFunc));
        }

        //资源购买
        static resourceShop(purchaseId, handler) {
            let dealFunc = function (res) {

                //打点
                console.log("钻石打点" + purchaseId);
                let str = 'gemBuy';
                let obj = { 'gemUnit': purchaseId };
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);

                let currencyBag = res.currencyBag;
                let payInfo = res.payInfo;
                let rewards = res.rewards;
                System._updateFullBag(currencyBag);
                PaySystem.updatePayInfo(payInfo);
                EventMgr.getInstance().postEvent("PURCHASE_COMPLETE");
                if (handler)
                    handler.runWith([res]);
            };
            Service.resourceShop(purchaseId, Laya.Handler.create(null, dealFunc));
        }

        //外卖
        // 获取外卖公司信息 请求
        static fetchTakeoutInfo(handler) {
            let dealFunc = function (res) {
                TakeoutSystem.fetchTakeoutInfo(res);

                if (handler)
                    handler.runWith([res]);
            };
            Service.fetchTakeoutInfo(Laya.Handler.create(null, dealFunc));
        }

        // 配送 请求
        static delivery(pets, free, handler) {
            let dealFunc = function (res) {
                TakeoutSystem.delivery(res);
                if (handler)
                    handler.runWith([res]);
            };
            Service.delivery(pets, free, Laya.Handler.create(null, dealFunc));
        }

        // 领取配送奖励 请求
        static receiveOrderRewards(handler) {
            let dealFunc = function (res) {
                TakeoutSystem.receiveRewards(res);
                let currencyBag = res.currencyBag;
                System._updateFullBag(currencyBag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.receiveOrderRewards(Laya.Handler.create(null, dealFunc));
        }

        // 加速配送 请求
        static speedUpDelivery(free, handler) {
            let dealFunc = function (res) {

                //打点
                if (!free) {
                    console.log("钻石打点" + "OrderCD");
                    let str = 'gemBuy';
                    let obj = { 'gemUnit': "OrderCD" };
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);
                }

                TakeoutSystem.speedUpDelivery(res);
                let currencyBag = res.currencyBag;
                System._updateFullBag(currencyBag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.speedUpDelivery(free, Laya.Handler.create(null, dealFunc));
        }

        // 升级外卖公司 请求
        static levelUpTakeout(handler) {
            let dealFunc = function (res) {
                TakeoutSystem.levelUpTakeout(res);
                let currencyBag = res.currencyBag;
                System._updateFullBag(currencyBag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.levelUpTakeout(Laya.Handler.create(null, dealFunc));
        }

        // 放置宣传员 请求
        static putAdvertiserCats(pets, handler) {
            let dealFunc = function (res) {
                TakeoutSystem.putAdvertiserCats(res);

                if (handler)
                    handler.runWith([res]);
            };
            Service.putAdvertiserCats(pets, Laya.Handler.create(null, dealFunc));
        }

        // 购买订单数量
        static buyOrder(handler) {
            let dealFunc = function (res) {
                //打点
                console.log("钻石打点" + "OrderTimes");
                let str = 'gemBuy';
                let obj = { 'gemUnit': "OrderTimes" };
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT_PP", [str, obj]);

                TakeoutSystem.buyOrder(res);
                let currencyBag = res.currencyBag;
                System._updateFullBag(currencyBag);
                if (handler)
                    handler.runWith([res]);
            };
            Service.buyOrder(Laya.Handler.create(null, dealFunc));
        }

    }

    class BezierPath{
        static CreateBezierPoints(anchorpoints, pointsAmount) {
            var points = [];
            for (var i = 0; i < pointsAmount; i++) {
                var point = this.MultiPointBezier(anchorpoints, i / pointsAmount);
                points.push(point);
            }
            return points;
        }
     
        static MultiPointBezier(points, t) {
            let len = points.length;
            let x = 0, y = 0;
            for (let i = 0; i < len; i++) {
                let point = points[i];
                x += point.x * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (this.erxiangshi(len - 1, i));
                y += point.y * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (this.erxiangshi(len - 1, i));
            }
            return { x: x, y: y };
        }
     
        static erxiangshi(start, end) {
                let cs = 1, bcs = 1;
                while (end > 0) {
                    cs *= start;
                    bcs *= end;
                    start--;
                    end--;
                }
                return (cs / bcs);
        }
    }

    class Balloon$1 {
        //调用顺序：beginBalloon --> timeOn --> _nextBalloonTime --> setPoint --> _loadBalloon --> _onClick --> system --> dataMgr --> flyRewards
        constructor() {
            EventMgr.getInstance().registEvent("BALLOON_SCENE_MGR", this, this.balloonVisible);
            EventMgr.getInstance().registEvent("BALLOON_FLY_YUGAN", this, this.flyRewards);
            EventMgr.getInstance().registEvent("BALLOON_DESTROY", this, this.toDestroy);
            EventMgr.getInstance().registEvent("BALLOON_AD", this, this.dealBalloonClick);
            this.stop = 0;
        }

        static getInstance() {
            return Balloon$1.instance ? Balloon$1.instance : Balloon$1.instance = new Balloon$1();
        }

        begining(){
            this.qiqiuBox=new Laya.Sprite();
            this.qiqiu = new Spine();
            this.clickImage=new Laya.Sprite();
            this.qiqiu.loadAnim("anim2/qiqiumao.sk");
            this.clickImage.loadImage("comp/image.png");
            this.clickImage.width=210;
            this.clickImage.height=210;
            this.clickImage.pivot(105,105);
            this.clickImage.mouseEnabled=true;
            this.clickImage.mouseThrough=false;
            this.clickImage.on(Laya.Event.CLICK,this,this.onClickImageClick);
            this.qiqiuScale=0;
            this.speed=BalloonSystem.getBalloonSpeed();
            this.balloonTime=BalloonSystem.getNextTime();
            this.timerMgr();
            this.qiqiuClick=0;
        }

        timerMgr()
        {
            Laya.timer.clear(this,this.runBalloon);
            Laya.timer.once(this.balloonTime,this,this.runBalloon);
        }

        runBalloon(){
            let playqiqiu=BalloonSystem.balloonSwitch();
            if(playqiqiu){
                let array = this.setPoint();
                this.playBalloon(array);
            }
            else {
                this.balloonTime=BalloonSystem.getLaterTime();
                this.timerMgr();
            }
        }

        playBalloon(array){
            this.stop = 0;
            this.qiqiuClick=0;
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_appear_balloon"]);
            EventMgr.getInstance().postEvent("SERVICE_GET_BALLOON_NUM");
            this.qiqiuBox.addChild(this.qiqiu);
            this.qiqiuBox.addChild(this.clickImage);
            Laya.stage.addChild(this.qiqiuBox);
            this.qiqiuBox.x =array[0].x;
            this.qiqiuBox.y =array[0].y;
            this.playQiQiuMaoNormal();
            for(var i=1;i<array.length;i++){
                let x=array[i].x;
                let y=array[i].y;
                Laya.timer.once(this.speed*i,this,function(){
                    if(this.stop){
                        x = this.qiqiuBox.x;
                        y = this.qiqiuBox.y;
                    }
                    Laya.Tween.to(this.qiqiuBox,{x:x,y:y},this.speed);
                });
            }
            Laya.timer.once(this.speed*(i+1),this,function(){
                if(!this.stop){
                    this.qiqiuBox.removeSelf();
                    this.balloonTime=BalloonSystem.getNextTime();
                    this.timerMgr();
                }
                else {
                    this.stop = 0;
                }
            });
        }

        mayWatchAD(){
            if(0 && BalloonSystem.checkWatchAD()){
                this.stop = 1;
                let str = Strings$1.get("tid#UI_046");
                Laya.Dialog.close("ui/BalloonAdvertising.scene");
                Laya.Dialog.open("ui/BalloonAdvertising.scene",false,[Constants.ShareId.Balloon,"BALLOON_AD",str,str,null,Constants.RewardType.FISH,BalloonSystem.getRewardNum()]);
            }
            else {
                this.dealBalloonClick();
            }
        }

        onClickImageClick(){
            if(this.qiqiuClick==0){
                this.qiqiuClick=1;
                this.clickImage.removeSelf();
                this.mayWatchAD();
            }
        }

        dealBalloonClick(type){
            this.addBalloonNum();
            this.playQiQiuMaoExplosion();
            this.x=this.qiqiuBox.x;
            this.y=this.qiqiuBox.y;
            var duoyugan = new Laya.Skeleton();
            Laya.stage.addChild(duoyugan);
            Laya.timer.once(1000,this,function(){
                this.stop = 0;
                this.qiqiu.removeSelf();
                this.qiqiuBox.removeSelf();
            });
            if(this.stop){
                this.balloonTime=BalloonSystem.getNextTime();
                this.timerMgr();
                BalloonSystem.onADBalloonClick(type);
            }
            else {
                BalloonSystem.onBalloonClick();
            }
        }

        flyRewards(fishNum, fetchType) {    //飞鱼干
            if (fetchType == Constants.FetchType.AD) {
                ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, fishNum,Constants.FlyType.TYPE3,this.x-33, this.y-24);
            } else {
                ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, fishNum,Constants.FlyType.TYPE5,this.x-33, this.y-24);
            }
        }

        addBalloonNum(){
            BalloonSystem.addBalloonNum();
        }

        balloonVisible(num){
            BalloonSystem.setSceneLicense(num);
            this.qiqiuBox.visible = num==1 ? true : false;
        }

        playQiQiuMaoNormal(){
            this.qiqiu.playAnim("fly1", true);
        }

        playQiQiuMaoExplosion(){
            this.qiqiu.playAnim("fly2", false);
        }

        adjustQiQiuScale(num){
            if(num==1){
                this.qiqiu.scale(-1,1);
            }
            else {
                this.qiqiu.scale(1,1);
            }
        }

        setPoint(){ //设置气球运行路径
            let begin=Math.random()*(Laya.stage.height-400)+200;
            let end=Math.random()*(Laya.stage.height-400)+200;
            let points = [];
            let ran=Math.random();
            if(ran<0.5){
                this.adjustQiQiuScale(1);
                var point0= new Laya.Point(-200,begin); // 起点
                var point1 = new Laya.Point(200,begin); // 起点
                var point2 = new Laya.Point(0,Laya.stage.height);
                var point3 = new Laya.Point(Laya.stage.width/2+200,0);
                var point4 = new Laya.Point(Laya.stage.width,0);
                var point5 = new Laya.Point(Laya.stage.width-200, end);//终点
                var point6 = new Laya.Point(Laya.stage.width+200, end);//终点
            }
            else {
                this.adjustQiQiuScale(0);
                var point6 = new Laya.Point(-200,begin); //终点
                var point5 = new Laya.Point(200,begin); // 起点
                var point4 = new Laya.Point(0,Laya.stage.height);
                var point3 = new Laya.Point(Laya.stage.width/2+200,0);
                var point2 = new Laya.Point(Laya.stage.width,0);
                var point1 = new Laya.Point(Laya.stage.width-200, end);// 起点
                var point0 = new Laya.Point(Laya.stage.width+200, end);// 起点
            }
            points.push(point0);
            points.push(point1);
            points.push(point2);
            points.push(point3);
            points.push(point4);
            points.push(point5);
            points.push(point6);
            return BezierPath.CreateBezierPoints(points, 200);
        }

        toDestroy(){
            this.qiqiuBox.removeSelf();
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("BALLOON_SCENE_MGR", this, this.balloonVisible);
            EventMgr.getInstance().removeEvent("BALLOON_DESTROY", this, this.toDestroy);
            EventMgr.getInstance().removeEvent("BALLOON_FLY_YUGAN", this, this.flyRewards);
        }
    }

    class TipBox extends Laya.Box {

        constructor() {
            super();
            this.visible = false;
        }

        showTips(str) {
            console.log(str);
            str = str.replace("<star>", "评价");
            this.tipBox = this.getChildByName("tipBox");
            this.tipLabel = this.getChildByName("tipLabel");
            this.tipLabel.text = str;
            this.visible = true;
            
        }
    }

    /**
    * 本示例采用非脚本的方式实现，而使用继承页面基类，实现页面逻辑。在IDE里面设置场景的Runtime属性即可和场景进行关联
    * 相比脚本方式，继承式页面类，可以直接使用页面定义的属性（通过IDE内var属性定义），比如this.tipLbll，this.scoreLbl，具有代码提示效果
    * 建议：如果是页面级的逻辑，需要频繁访问页面内多个元素，使用继承式写法，如果是独立小模块，功能单一，建议用脚本方式实现，比如子弹脚本。
    */

    class CoffeeScene extends Laya.Scene {
        constructor() {
            super();
            this.sceneId = Constants.CoffeeSceneId;//1
            this.buildingMap = new Map();
        }

        onAwake() {
            SdkHelper.tuisongDays();
            Laya.stage.removeChildByName("startbg");
            Laya.stage.removeChildByName("startbg1");
            Laya.stage.removeChildByName("startbg2");
            // ZhuliSystem.AidPlayer('cat005_4');
            ZhuliSystem.zhuliPanding(SdkHelper.getLaunchOptionsSync());
            this.signOpen=0; 
            if(System._loader_begin != null & System._loader_end != null) {
                System._transition_end = (+ new Date());
                let data = {isNew: System._login_times==1};
                data.loading_time = System._loader_end - System._loader_begin;
                if(System._transition_end != null && System._transition_begin != null) {
                    data.transition_time = System._transition_end - System._transition_begin;
                }
                if(System._load_subpackage_end != null && System._load_subpackage_begin != null) {
                    data.subpackage_time = System._load_subpackage_end - System._load_subpackage_begin;
                }
                if(System.transition_time != null) {
                    data.total_time = data.loading_time + data.transition_time;
                }
                System._loader_begin = null;
                System._loader_end = null;
                System._transition_begin = null;
                System._transition_end = null;
                System._load_subpackage_begin = null;
                System._load_subpackage_end = null;

                data.brand = System._brand!=null?System._brand:"null";
                data.model = System._model!=null?System._model:"null";
                data.version = System._wxversion!=null?System._wxversion:"null";
                data.systemVersion = System._systemVersion!=null?System._systemVersion:"null";
                data.platform = System._platform!=null?System._platform:"null";
                data.benchmarkLevel = System._benchmarkLevel!=null?System._benchmarkLevel:"null";
                data.rid = System.getRid() != null ? System.getRid() : "null";
                data.openid = System.getOpenId() != null ? System.getOpenId() : "null";

                let customData = JSON.stringify(data);
                console.log("loadinglog", customData);
                EventMgr.getInstance().postEvent("SERVICE_SEND_LOADINGLOG", customData);
            }
            SoundManager.getInstance().play("Sound_bgm1", 0);
            EventMgr.getInstance().registEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().registEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().registEvent("UPDATE_DROPFISH", this, this.updateDropFish);
            EventMgr.getInstance().registEvent("UPDATE_DROPFISH_ALL", this, this.updateDropFishAll);
            EventMgr.getInstance().registEvent("SHOW_COFFEE_SCENE", this, this.__show);
            EventMgr.getInstance().registEvent("HIDE_COFFEE_SCENE", this, this.__hide);
            EventMgr.getInstance().registEvent("ZCM_AUTO", this, this.autoFortuneCatLogic);
            EventMgr.getInstance().registEvent("USING_SKILL1", this, this.pickUpFishBySkill);
            EventMgr.getInstance().registEvent("USING_SKILL2", this, this.catEatBySkill);
            EventMgr.getInstance().registEvent("ZHULI_TIPS",this,this.zhuliTips);
            EventMgr.getInstance().registEvent("UPDATE_STAR",this,this.checkOnGiveMarking);
            
            this.dev_show = this.getChildByName("dev_show");
            this.dev_show.visible = Constants.dev;
            Laya.timer.clear(this, this.delayAwake);
            Laya.timer.frameOnce(1, this, this.delayAwake);
            Balloon$1.getInstance().begining();
            CatNewComeMgr.getInstance();
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[1]);
            EventMgr.getInstance().postEvent("SERVICE_SIGN_INFN",[0]);
            this.waitGiveMarking = 0;
            SdkHelper.getPurchasesAsync();//刷新购买信息
        }

        delayAwake() {
            if(System.isLogined()) {
                BarrageMgr.getInstance().randomStart("BarrageLogin", "BarrageTriggerTime1");
                GuideSystem.startCheckGuide();
                BarrageMgr.getInstance().autoRandomPlay();
                EntrustSystem.startCheck();
                EventMgr.getInstance().postEvent("SERVICE_GET_MINIGAME");
                TaskMainSystem.startCheck();
                StaffSystem.getStaffGroup();
                MileStoneSystem.begin();
                let openRes = SdkHelper.getLaunchOptionsSync();
                TaskSystem.collectionTask(openRes);
                MileStoneSystem.loadData();
                NauticalSystem$1.getNauticalInfoReq();
                //this.getPurchaseCard();//周卡月卡自动弹出（待排序）
                //SdkHelper.getLaunchOptionsSync();
                DialogStatusMgr.getInstance().init();
                
            }else {
                Laya.timer.clear(this, this.delayAwake);
                Laya.timer.frameOnce(1, this, this.delayAwake);
            }
        }

        __show() {
            Laya.timer.clear(this, this.____show);
            Laya.timer.frameOnce(1, this, this.____show);
        }
        ____show() {
            this.visible = true;
        }

        __hide() {
            this.visible = false;
        }

        onEnable() {
            this.visible = true;
            EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
        }

        onDisable() {
            Laya.timer.clear(this, this.____show);
            Laya.timer.clear(this, this.delayOpen);
        }

        onDestroy() {
            Laya.timer.clear(this, this.____show);
            Laya.timer.clear(this, this.delayAwake);
            EventMgr.getInstance().removeEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().removeEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().removeEvent("UPDATE_DROPFISH", this, this.updateDropFish);
            EventMgr.getInstance().removeEvent("UPDATE_DROPFISH_ALL", this, this.updateDropFishAll);
            EventMgr.getInstance().removeEvent("SHOW_COFFEE_SCENE", this, this.__show);
            EventMgr.getInstance().removeEvent("HIDE_COFFEE_SCENE", this, this.__hide);
            EventMgr.getInstance().removeEvent("ZCM_AUTO", this, this.autoFortuneCatLogic);
            EventMgr.getInstance().removeEvent("USING_SKILL1", this, this.pickUpFishBySkill);
            EventMgr.getInstance().removeEvent("USING_SKILL2", this, this.catEatBySkill);
            EventMgr.getInstance().removeEvent("ZHULI_TIPS",this,this.zhuliTips);
            EventMgr.getInstance().removeEvent("UPDATE_STAR",this,this.checkOnGiveMarking);
            if(this.jianzao) {
                this.jianzao.destroy();
                this.jianzao = null;
            }
            this._jianzhaoLoadCompleta = false;
        }

        onInitBuilding() {
            let buildingList = BuildingSystem.getBuildingDataListInfo(this.sceneId).list;
            for (var i = 0; i < buildingList.length; ++i) {
                let data = buildingList[i];
                let buildingId = data.getId();
                if (this.buildingMap.get(buildingId) == null && data.getLocationData() != null) {
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root, Constants.CoffeeBuildingScale, Laya.Handler.create(this, this._onBuildingAdded));
                    this.buildingMap.set(buildingId, building);
                }
            }
        }

        initCats() {
            let cats = CoffeeCatMgr.getInstance().allCats();
            for(const cat of cats) {
                if(cat.buildingId == null || cat.buildingId == undefined) {
                    if(cat.parent!=this.root) {
                        cat.removeSelf();
                        this.root.addChild(cat);
                        cat.posInMap(cat._toX, cat._toY);
                        let curAnim = cat.getCurAnim();
                        curAnim = curAnim!=null ? curAnim : "lick1";
                        cat.playAnim(curAnim);
                    }
                }else {
                    this.addCatToBuilding(cat);
                }
            }
        }

        onOpened() {
            this.root = this.getChildByName("root");
            // Laya.timer.clear(this, this.delayOpen);
            // Laya.timer.frameOnce(1, this, this.delayOpen);
            this.delayOpen();
        }

        delayOpen() {
            if(System.isLogined()) {
                this.onInitBuilding();
                this.initCats();
                CoffeeCatMgr.getInstance().clearRemoved();
            }else {
                Laya.timer.clear(this, this.delayOpen);
                Laya.timer.frameOnce(1, this, this.delayOpen);
            }
        }

        checkOnGiveMarking(){
            if(this.waitGiveMarking){
                return ;
            }
            this.loginDays = Number(PlayerSystem.getLoginDays());
            if(this.loginDays > 0){
                let todayGiveMarkingSign = Number(Laya.LocalStorage.getJSON('todayGiveMarkingSign'));
                let nowStarNum = PlayerSystem.getStar();
                let popSign = 0;
                let ShopSuggestionTime =Const.get("ShopSuggestionTime");//弹出所需的星星数量
                /////////////////////星星条件//////////////////////
                if(this.nowStarNum < ShopSuggestionTime && nowStarNum >= ShopSuggestionTime){
                    popSign = 1;
                }
                this.nowStarNum = nowStarNum;
                //////////////////////天数条件//////////////////////
                if(this.loginDays == 3 || this.loginDays == 5 || (this.loginDays > 7 &&this.loginDays % 7 == 1)){
                    popSign = 1;
                }
                if(popSign && todayGiveMarkingSign != this.loginDays){////////////////
                    Laya.timer.once(3000,this,function(){
                        this.onGiveMarking();
                    });
                }
            }
        }

        onGiveMarking(){
            if(DialogStatusMgr.getInstance().getStatus() == 0){
                Laya.LocalStorage.setJSON('todayGiveMarkingSign',this.loginDays);
                SdkHelper.onGiveMarking();
            }
            else {
                this.waitGiveMarking = 1;
                Laya.timer.once(3000,this,function(){
                    this.onGiveMarking();
                });
            }
        }

        addCatToBuilding(cat) {
            let buildingExtInfo = this.getSpaceInfo(cat.buildingId, cat.spaceId);
            if (buildingExtInfo == null) {
                cat.removeSelf();
                return;
            }
            if(cat.parent==buildingExtInfo.getBuildingRoot())
                return;
            let type = cat._eventType;
            let foodId = cat._foodId;
            let time = cat._eventTime;
            cat.removeSelf();
            buildingExtInfo.addToBuildingRoot(cat);
            cat.alpha = 1;
            let actionSetting=buildingExtInfo.randomCatMove();
            cat.playAnim(actionSetting.name);
            cat.x = buildingExtInfo.getBuildingRootHalfWidth();
            cat.y = buildingExtInfo.getBuildingRootHalfHeight();
            if(actionSetting.flip){
                cat.scaleX = -1 * Math.abs(cat.scaleX);
            }else {
                cat.scaleX = Math.abs(cat.scaleX);
            }
            if (actionSetting.hide) {
                buildingExtInfo.hideOthers();
            }
            if(type == 1) {
                if(cat.catMenuTips){
                    cat.catMenuTips.destroy();
                    cat.catMenuTips = null;
                }
                cat.catMenuTips = new ShowCatMenuTip(cat, foodId, buildingExtInfo, time);
            }
            EventMgr.getInstance().postEvent("GUIDE_NEXT_STEP", ["GuideGroup_3"]);
            EventMgr.getInstance().postEvent("GUIDE_NEXT_STEP", ["GuideGroup_6"]);
        }

        _onBuildingAdded(building) {
            let cats = CoffeeCatMgr.getInstance().catsInBuilding(building.getId());
            for(const cat of cats) {
                this.addCatToBuilding(cat);
            }
            this._updateDropFish(building);
        }

        updateDropFish(buildingId, sceneId, spaceId,multiple) {
            let building = this.getBuilding(buildingId);
            if (building == null) {
                return;
            }
            this._updateDropFish(building, spaceId, multiple);
        }

        _updateDropFish(building, spaceId = null, multiple = 1) {
            let buildingId = building.getId();
            let buildingBaseId = building.getBuildingBaseId();
            let fish = SyncSystem.getCachedFish(buildingBaseId);
            let extInfoMgr = building.getBuildingExtInfoMgr();
            if (extInfoMgr.getDropFishPoint() == null || fish <= 0) {

            } else {
                let scene = this;
                let p = extInfoMgr.getDropFishPointPosition();
                let point = new Laya.Point(p.x, p.y);
                extInfoMgr.getDropFishPoint().localToGlobal(point, false, scene.root);
                let buildingRoot = null;
                let globalStart = null;
                let fishtStart = null;
                if (spaceId != null) {
                    buildingRoot = this.getSpaceInfo(buildingId, spaceId).getBuildingRoot();
                    globalStart = new Laya.Point(buildingRoot.x + buildingRoot.width / 2 * buildingRoot.scaleX, buildingRoot.y + buildingRoot.height * buildingRoot.scaleY / 2-100);
                    buildingRoot.parent.localToGlobal(globalStart);
                    fishtStart = scene.root.globalToLocal(globalStart);
                }
                for (let index = 0; index < multiple; index++) {
                    let delay = index * 20;
                    let dropFish = new DropFish(buildingBaseId);
                    extInfoMgr.setDropFish(dropFish);
                    dropFish.setDropFish(fish);
                    scene.root.addChild(dropFish);
                    dropFish.x = point.x + Math.round(Utils.randomExt2(-7, 7)) * 3;
                    dropFish.y = point.y + Math.round(Utils.randomExt2(-10, 10)) * 2;
                    dropFish.zOrder = building.zOrder;
                    if (spaceId == null) {
                        continue;
                    }
                    dropFish.saveStartPoint(fishtStart.x, fishtStart.y);
                    // let tweenTime = 10 * (Math.abs(point.x - fishtStart.x));
                    Laya.Tween.to(dropFish, { factor: 1 }, 500, null, Laya.Handler.create(this, function () {
                        let guide = GuideSystem.getGuide("GuideGroup_4");
                        if (guide) {
                            Laya.timer.once(1200,this,function(){
                                EventMgr.getInstance().postEvent("GUIDE_NEXT_STEP", ["GuideGroup_4"]);
                                guide.registerObj("btn1", this);
                                guide.registerClick("btn1", extInfoMgr.getDropFishPoint(), null, [dropFish]);
                                EventMgr.getInstance().postEvent('GOUDE_COLSE_TALK');
                            });
                        }
                    }),delay);
                }
            }
        }

        updateDropFishAll() {
            for (var [key, value] of this.buildingMap) {
                this._updateDropFish(value);
            }
        }

        addBuilding(data, isNew) {
            if (data && data.getLocationData && data.getBelongScene) {
                if (data.getBelongScene() == this.sceneId) {
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root, Constants.CoffeeBuildingScale, Laya.Handler.create(this, this._onBuildingAdded));
                    this.buildingMap.set(data.getId(), building);

                    if (this.jianzao && !this.jianzao.destroyed) {
                        this.jianzao.destroy();
                    }
                    this.jianzao = new Spine();
                    this.root.addChild(this.jianzao);
                    let locationData = data.getLocationData();
                    let scale = Number(locationData.getBaseSize());
                    if (isNew) {
                        this.jianzao.loadAnim("anim1/jianzao1.sk");
                        this.jianzao.playAnim("jianzao1", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale * 2;
                    } else {
                        this.jianzao.loadAnim("anim3/jianzao.sk");
                        this.jianzao.playAnim("yanwu", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale / 2;
                    }
                    this.jianzao.visible = true;
                    this.jianzao.addCompleteHandler(Laya.Handler.create(this, function () {
                        this.jianzao.destroy();
                        this.jianzao = null;
                    }));
                    this.jianzao.pos(Number(locationData.getSelectionX()) + 152 * scale, Number(locationData.getSelectionY()) + 80 * scale);
                    this.jianzao.zOrder = Math.floor(this.jianzao.y) + Number(locationData.getZOrder());
                }
            }
        }

        removeBuilding(data) {
            if (data && data.getBelongScene && data.getBelongScene() == this.sceneId) {
                if (this.buildingMap.has(data.getId())) {
                    let building = this.buildingMap.get(data.getId());
                    building.cleanBuilding();
                    this.buildingMap.delete(data.getId());
                    building.destroy();
                }
            }
        }

        getBuilding(id) {
            for (var [key, value] of this.buildingMap) {
                if (Number(key) == Number(id)) {
                    if (value.isLoaded()) {
                        return value;
                    }
                    return null;
                }
            }
            return null;
        }

        getSpaceInfo(buildingId, spaceId) {
            let building = this.getBuilding(buildingId);
            if (building) {
                return building.getBuildingExtInfoMgr().getBuildingExtInfo(spaceId);
            }

        }

        playFishNumber(globalX, golbalY, num) {
            let txt = Laya.Pool.getItemByClass("addNumber", Laya.Label);
            txt.color = "#5e4d2c";
            txt.fontSize = 45;
            txt.text = "+" + Strings$1.dealFishNumberText(num);
            txt.bold = true;
            txt.pivotX = txt.width / 2;
            txt.pivotY = txt.height / 2;
            txt.x = globalX;
            txt.y = golbalY;
            txt.scaleX = txt.scaleY = 1;
            Laya.stage.addChild(txt);
            Laya.Tween.to(txt, { y: golbalY - 60}, 750, null,
                Laya.Handler.create(this, function () {
                    txt.removeSelf();
                    Laya.Pool.recover("addNumber", txt);
                }));
        }

        autoFortuneCatLogic() {
            this.MoneyCatLoginShowLimit = Const.get("MoneyCatLoginShowLimit");
            this.MoneyCatLoginShowWeight = Const.get("MoneyCatLoginShowWeight");
            let yes = 0;
            if (SyncSystem.getCacheFish() >= SyncSystem.getCacheFishCapacity() * this.MoneyCatLoginShowWeight) {
                yes++;
            }
            if (PlayerDataMgr.getInstance().getStar() >= this.MoneyCatLoginShowLimit) {
                yes++;
            }
            if (yes < 2) {
                return;
            }
            Laya.Dialog.close("ui/FortuneCatUI.scene");
            if (!GuideSystem.isGuiding()) {
                EventMgr.getInstance().postEvent("DIALOG_CHANGE_STATUS",[Constants.DialogStatus.ForturneCat,1]);
                Laya.Dialog.open("ui/FortuneCatUI.scene", false);
            }
        }

        pickUpFishBySkill(x, y, zOrder) {
            this.buildingMap.forEach((value, key) => {
                let buildingExtInfoMgr = value.getBuildingExtInfoMgr();
                let dropFishPoint = buildingExtInfoMgr.getDropFishPoint();
                if (dropFishPoint == null) {
                    return;
                }
                if ((zOrder > 10000 && value.zOrder < 10000) || (zOrder < 10000 && value.zOrder > 10000)) {
                    return;
                }
                let dropPointPoistion = buildingExtInfoMgr.getDropFishPointPosition();
                if (dropPointPoistion == null) {
                    return;
                }
                let point = new Laya.Point(dropPointPoistion.x, dropPointPoistion.y);
                dropFishPoint.localToGlobal(point, false, this.root);
                let d = Math.sqrt((point.x - x) * (point.x - x) + (point.y - y) * (point.y - y));
                if (d <= 100) {
                    dropFishPoint.event(Laya.Event.CLICK);
                }
            });
        }

        catEatBySkill() {
            let buildings = [BuildingSystem.getBuildingDataByPid(107), BuildingSystem.getBuildingDataByPid(108), BuildingSystem.getBuildingDataByPid(109)];
            for (const building of buildings) {
                if (building==null) {
                    continue;
                }
                let cats = CoffeeCatMgr.getInstance().catsInBuilding(building.getId());
                for (const cat of cats) {
                    if (cat && cat.catMenuTips && cat.catMenuTips.TipClicked !=true && cat.catMenuTips.TipVisible != false) {
                        cat.catMenuTips.skillEat();
                        return;
                    }
                }
            }
        }

        getAllBuildings() {
            return this.buildingMap;
        }

        zhuliTips(tips){
            if(tips==0)
            {   
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon47");  
            }
            else if(tips==1){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon43");  
            }else if(tips==2){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon44");  
            }else if(tips==3){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon40");  
            }else if(tips==4){
            }

            }
    }

    class FishingScene extends Laya.Scene {

        constructor() {
            super();

            this.remainTimes = 0;
            this.ticaketNums = 0;
            this.maxTimes = 0;
            this.nextUpdateSecs = 0;        //距离下次刷新钓鱼次数的秒数
            this.showRedPoint = true;         //控制一下发送红点事件的次数
            this.hideRedPoint = true;         //控制一下发送红点事件的次数
        }

        onAwake() {
            this.root = this.getChildByName("root");
            this.startBtn = this.root.getChildByName("start_btn");
            this.startBtnAd = this.root.getChildByName("fishAd");
            this.startBtnSh = this.root.getChildByName("fishSh");
            this.endBtn = this.root.getChildByName("end_btn");
            this.fishVernier = this.root.getChildByName("fish_vernier");
            this.closeBtn = this.root.getChildByName("close");
            this.collectBtn = this.root.getChildByName("collect");
            this.baitBtn = this.root.getChildByName("baits");
            this.baitImg = this.baitBtn.getChildByName("baitsImg");
            this.baitsRedPoint = this.baitBtn.getChildByName("baitsRedPoint");
            this.tryBtn = this.root.getChildByName("try");
            this.tryQuitBtn = this.root.getChildByName("tryQuit");
            this.tryStartBtn = this.root.getChildByName("tryStart_btn");
            this.tryEndBtn = this.root.getChildByName("tryEnd_btn");
            this.timesFont = this.root.getChildByName("times_show");
            this.tips = this.root.getChildByName("tips");
            this.resultShow = this.root.getChildByName("resultShow");

            this.slider0 = this.root.getChildByName("slider_0");
            this.slider1 = this.root.getChildByName("slider_1");
            this.slider2 = this.root.getChildByName("slider_2");

            this.leftLimit = this.root.getChildByName("left_limit").x;
            this.rightLimit = this.root.getChildByName("right_limit").x;
            this.targetX = 0;
            this.speed = 0;
            this.velocity = -3.001;
            this.initSpeed = 40;
            this.gameState = 1; //0 未开始 1 进行中 2 完成 3 结束
            this.playStatus = 0;// 0 正常  1 试玩

            this.coupon_btn = this.root.getChildByName("coupon_btn");
            this.couponNum = this.coupon_btn.getChildByName("couponNum");
            this.coupon_btn.on(Laya.Event.CLICK, this, this.onClickCouponStart);

            this.initDefaultBaitId();
            this.initLimit();
            this.initAnims();
            this.resetGame();
            Laya.timer.loop(30, this, this.onUpdate);
            EventMgr.getInstance().registEvent("FISHING_BAITS_SELECTED_CHANGE", this, this.initFishingSlideData);
            EventMgr.getInstance().registEvent("GUIDE2_FISHGAME", this, this.Guide2_FishGame);
            EventMgr.getInstance().registEvent("FINISH_GAME_TO_SERVER", this, this.finishGameToServer);
            EventMgr.getInstance().registEvent("FINISH_GAME_TO_TRY", this, this.finishTry);
            EventMgr.getInstance().registEvent("FINISH_GAME_FREE_AGAIN", this, this.freeAgain);
            EventMgr.getInstance().registEvent("CHECK_BAITS_REDPOINT", this, this.checkBaitsRedPoint);

            this.startBtnAd.visible = false;
            this.startBtnSh.visible = false;
            this.tryBtn.visible = false;
            this.tryQuitBtn.visible = false;

            this.fishTimes = 0;
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("FISHING_BAITS_SELECTED_CHANGE", this, this.initFishingSlideData);
            EventMgr.getInstance().removeEvent("GUIDE2_FISHGAME", this, this.Guide2_FishGame);
            EventMgr.getInstance().removeEvent("FINISH_GAME_TO_SERVER", this, this.finishGameToServer);
            EventMgr.getInstance().removeEvent("FINISH_GAME_TO_TRY", this, this.finishTry);
            EventMgr.getInstance().removeEvent("FINISH_GAME_FREE_AGAIN", this, this.freeAgain);
            EventMgr.getInstance().removeEvent("CHECK_BAITS_REDPOINT", this, this.checkBaitsRedPoint);
            EventMgr.getInstance().removeEvent("SET_BAITS_IMAGE", this, this.setBaitImg);
        }

        onRefresh() {
            this.onOpened();
            this.onEnable();
        }

        setBaitImg() {
            let baitId = FishingSystem.getLastBaitId();
            let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", baitId);
            this.baitImg.skin = baitConfig.img;
        }

        //查找并设置默认的鱼饵，看玩家数据，如果数据里没有则默认选择表里的第一个
        initDefaultBaitId() {
            if (FishingSystem.getLastBaitId() == "") {
                let star = PlayerSystem.getStar();
                let config = ConfigMgr$1.getInstance().getConfig("FishingBaits");
                for (const [baitId, baitConfig] of config) {
                    if (star >= baitConfig.StarLimit) {
                        FishingSystem.setLastBaitId(baitId);
                        break;
                    }
                }
            }
        }

        onOpened() {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
            EventMgr.getInstance().postEvent("CLEAR_ALL_BARRAGES");
            this.startBtn.on(Laya.Event.CLICK, this, this.onClickStartGame);
            this.tryStartBtn.on(Laya.Event.CLICK, this, this.onClickStartGame);
            this.startBtnAd.on(Laya.Event.CLICK, this, this.onClickStartGameAd);
            this.startBtnSh.on(Laya.Event.CLICK, this, this.onClickStartGameSh);
            this.coupon_btn.on(Laya.Event.CLICK, this, this.onClickCouponStart);
            this.endBtn.on(Laya.Event.CLICK, this, this.onClickEndGame);
            this.tryEndBtn.on(Laya.Event.CLICK, this, this.onClickEndGame);
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseBtn);
            this.collectBtn.on(Laya.Event.CLICK, this, this.onShowCollect);
            this.baitBtn.on(Laya.Event.CLICK, this, this.onShowBaitsSelect);
            this.tryBtn.on(Laya.Event.CLICK, this, this.onTry);
            this.tryQuitBtn.on(Laya.Event.CLICK, this, this.onTryQuit);
            this.tryBtn.visible = false;
            this.tryQuitBtn.visible = false;
            this.nextUpdateSecs = FishingSystem.getNextUpdateSecs();
            this.remainTimes = FishingSystem.getRemainTimes();
            this.ticaketNums = FishingSystem.getFishingTicketNum();
            this.maxTimes = FishingSystem.getMaxTimes();
            this.fishingRetryFreeTimes = FishingSystem.getFishingRetryFreeTimes();
            this.timesFont.value = this.remainTimes.toString() + "/" + this.maxTimes.toString();
            this.couponNum.text = Strings$1.get("tid#UI_117") + this.ticaketNums;
            if (this.remainTimes >= this.maxTimes || this.nextUpdateSecs < 0) {
                this.tips.visible = false;
                Laya.timer.clear(this, this.tickerRefresh);
            } else {
                //未达到满次数时，开始显示恢复时间
                this.showTimeTips();
                Laya.timer.loop(1000, this, this.tickerRefresh);
            }
            this.checkBaitsRedPoint();
            this._setBtns();
            FishingSystem.updateServerCollectsInfo();

            //初始化游戏条相关内容
            this.initFishingSlideData();
            this.guide13();
        }

        checkBaitsRedPoint() {
            this.baitsRedPoint.visible = false;
            if (FishingNewBaitMgr.getInstance().haveHigherBait() > 0) {
                this.baitsRedPoint.visible = true;
            }
        }

        freeAgain() {
            this.onRefresh();
            this.fishAgain--;
            this._startType = 1;
            this.startGame();
        }

        onTry() {
            this.playStatus = 1;
            this.tryBtn.visible = false;
            this.tryQuitBtn.visible = true;
            this.tryStartBtn.visible = true;
            this.tryEndBtn.visible = false;
            this.timesFont.visible = false;
            this.startBtn.visible = false;
            this.startBtnSh.visible = false;
            this.startBtnAd.visible = false;
            this.realBeginBtn = this.tryStartBtn;
            this.tips.visible = false;
            this.coupon_btn.visible = false;
        }

        onTryQuit() {
            this.playStatus = 0;
            this.tryBtn.visible = true;
            this.tryQuitBtn.visible = false;
            this.tryStartBtn.visible = false;
            this.tryEndBtn.visible = false;
            this.timesFont.visible = true;
            this.startBtnSh.visible = false;
            this.startBtnAd.visible = false;
            this.realBeginBtn = this.startBtn;
            if (this.remainTimes >= this.maxTimes || this.nextUpdateSecs < 0) {
                this.startBtn.visible = true;
                this.tips.visible = false;
                Laya.timer.clear(this, this.tickerRefresh);
            } else {
                //未达到满次数时，开始显示恢复时间
                this._setBtns();
                this.showTimeTips();
                Laya.timer.loop(1000, this, this.tickerRefresh);
            }
        }

        finishTry() {
            this.onRefresh();
            this.onTry();
        }

        tickerRefresh() {
            this.nextUpdateSecs = FishingSystem.getNextUpdateSecs();
            this.remainTimes = FishingSystem.getRemainTimes();
            this.ticaketNums = FishingSystem.getFishingTicketNum();
            this.maxTimes = FishingSystem.getMaxTimes();
            if (this.remainTimes > 0) {
                if (this.showRedPoint == false || this.hideRedPoint == true) {
                    EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [2, true]);
                    this.showRedPoint = true;
                    this.hideRedPoint = false;
                }
            }
            else {
                if (this.showRedPoint == true) {
                    EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [2, false]);
                    this.showRedPoint = false;
                }
            }

            if (this.nextUpdateSecs > 0 && this.maxTimes > this.remainTimes && this.playStatus == 0) {
                this.showTimeTips();
            } else {
                this.tips.visible = false;
                Laya.timer.clear(this, this.tickerRefresh);
            }

            this.timesFont.value = this.remainTimes.toString() + "/" + this.maxTimes.toString();
            this.couponNum.text = Strings$1.get("tid#UI_117") + this.ticaketNums;
            this._setBtns();
        }

        _setBtns() {
            if (this.playStatus == 0) {
                this._startType = 0;
                this.showBtns();
            }
        }

        showTimeTips() {
            this.tips.visible = true;
            let leftHour = parseInt(this.nextUpdateSecs / 3600);
            let leftMin = parseInt(this.nextUpdateSecs / 60) % 60;
            let leftSec = this.nextUpdateSecs % 60;
            if (leftHour > 0) {
                this.tips.text = "Free: " + leftHour.toString() + "h " + leftMin.toString() + "m " + leftSec.toString() + "s";
            } else if (leftMin > 0) {
                this.tips.text = "Free: " + leftMin.toString() + "m " + leftSec.toString() + "s";
            } else {
                this.tips.text = "Free: " + leftSec.toString() + "s";
            }
        }

        initAnims() {
            let outSelf = this;
            this.animCat = new Spine();
            this.root.addChild(this.animCat);
            this.animCat.loadAnim("anim2/diaoyu.sk");
            this.animCat.x = 860;
            this.animCat.y = 1330;
            this.animCat.playAnim("diaoyu4", true);
            this.animCatLoad = true;
            

            let sceneAnim = new Spine();
            sceneAnim.loadAnim("anim2/diaoyuchangjing.sk");
            sceneAnim.playAnim("changjing", true);
            sceneAnim.x = 600;
            sceneAnim.y = 1200;
            this.root.addChild(sceneAnim);
            console.log(this.animCat);
        }

        //初始化游戏条，根据鱼饵设置难度和区间线
        initFishingSlideData(selectBaitId) {
            if (this.dataMap) {
                this.dataMap.clear();
            } else {
                this.dataMap = new Map();
            }

            let baitId = selectBaitId ? selectBaitId : FishingSystem.getLastBaitId();
            let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", baitId);
            this.speed = baitConfig.speed * 10;         //TODO 要不要乘以10，回头问策划
            let zoneConfig = ConfigMgr$1.getInstance().getItemById("FishingHitZone", baitConfig.hitZoneID);
            let totalSize = this.rightLimit - this.leftLimit;
            let midX = this.leftLimit + totalSize / 2;
            //这种写法相当于，HitZone 的第一个数字配不配都没影响，就是0
            if (zoneConfig.HitZone.length >= 2) {
                this.slider2.x = this.leftLimit + zoneConfig.HitZone[1] * totalSize / 2;
                this.slider2.width = (1 - zoneConfig.HitZone[1]) * totalSize;
            } else {
                this.slider2.width = 0;
            }

            if (zoneConfig.HitZone.length >= 3) {
                this.slider1.x = this.leftLimit + zoneConfig.HitZone[2] * totalSize / 2;
                this.slider1.width = (1 - zoneConfig.HitZone[2]) * totalSize;
            } else {
                this.slider1.width = 0;
            }

            if (zoneConfig.HitZone.length >= 4) {
                this.slider0.x = this.leftLimit + zoneConfig.HitZone[3] * totalSize / 2;
                this.slider0.width = (1 - zoneConfig.HitZone[3]) * totalSize;
            } else {
                this.slider0.width = 0;
            }

            let l1 = this.slider2.width == 0 ? midX : this.slider2.x;
            let r1 = this.slider2.width == 0 ? midX : this.slider2.x + this.slider2.width;
            let l2 = this.slider1.width == 0 ? midX : this.slider1.x;
            let r2 = this.slider1.width == 0 ? midX : this.slider1.x + this.slider1.width;
            let l3 = this.slider0.width == 0 ? midX : this.slider0.x;
            let r3 = this.slider0.width == 0 ? midX : this.slider0.x + this.slider0.width;

            let a1 = new Array();
            a1.push({ min: l3, max: r3 });
            this.dataMap.set(1, a1);

            let a2 = new Array();
            a2.push({ min: l2, max: l3 });
            a2.push({ min: r3, max: r2 });
            this.dataMap.set(2, a2);

            let a3 = new Array();
            a3.push({ min: l1, max: l2 });
            a3.push({ min: r2, max: r1 });
            this.dataMap.set(3, a3);

            let a4 = new Array();
            a4.push({ min: r1, max: 99999 });
            a4.push({ min: -99999, max: l1 });
            this.dataMap.set(4, a4);
        }

        getResult() {
            let ret = 0;
            let x = this.fishVernier.x;
            this.dataMap.forEach((value, key) => {
                for (let i = 0; i < value.length; ++i) {
                    let d = value[i];
                    if (x <= d.max && x >= d.min) {
                        ret = key;
                        break;
                    }
                }
            });
            return ret;
        }

        //根据最后的结果区间，取得奖励内容和钓到的鱼
        getReward(baitId, result) {
            let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", baitId);
            let resultConfig = ConfigMgr$1.getInstance().getItemById("FishingResult", String(result));
            //确定是高级鱼还是普通鱼
            //1 普通鱼 2 高级鱼
            let a = 1;
            let r = Utils.randomExt(100);
            if (r < resultConfig.specialRate) {
                a = 2;
            }
            //确定鱼种
            let m = new Map();
            let array = new Array();
            let d = baitConfig.normal;
            if (a == 2) {
                d = baitConfig.special;
            }
            for (let i = 0; i < d.length; ++i) {
                let c = d[i];
                let arr = c.split(":");
                m.set(arr[1], Number(arr[0]));
            }
            let weightedSum = 0;
            m.forEach((value, key) => {
                weightedSum += Number(value);
                array.push({ w: value, n: key });
            });
            array.sort(function (a, b) {
                return a.w < b.w;
            });

            let ret = Utils.calcRandomWeight(array, weightedSum);

            let fishId = ret.n;
            console.debug("钓鱼Id: " + String(fishId));
            let fishConfig = ConfigMgr$1.getInstance().getItemById("FishingFishes", String(fishId));

            //鱼尺寸
            let sizeMin = resultConfig.sizeIncrease[0];
            let sizeMax = resultConfig.sizeIncrease[1];
            let ss = Utils.randomExt2(Number(sizeMin * 100), Number(sizeMax * 100)) / 100.0;
            console.debug(fishConfig);
            let finalSize = Math.floor((Number(fishConfig.size) + ss * Number(fishConfig.size) * Number(fishConfig.sizeTimes)) * 100) / 100;
            //鱼收益
            let incomeMin = resultConfig.incomeIncrease[0];
            let incomeMax = resultConfig.incomeIncrease[1];
            let ss2 = Utils.randomExt2(Number(incomeMin * 100), Number(incomeMax * 100)) / 100.0;
            let finalIncome = Number(fishConfig.income) + Math.floor(ss2 * Number(fishConfig.income) * Number(fishConfig.incomeTimes));

            //口粮奖励
            let rFood = Utils.randomExt(100);
            let foodReward = null;
            if (rFood < fishConfig.NauticalFoodsRate) {
                let mm = new Map();
                let aa = new Array();
                let cc = JSON.parse(fishConfig.NauticalFoods);
                for (let i = 0; i < cc.length; ++i) {
                    let c = cc[i];
                    mm.set(c, c["weight"]);
                }
                let weightedSum = 0;
                mm.forEach((value, key) => {
                    weightedSum += Number(value);
                    aa.push({ w: value, n: key });
                });
                aa.sort(function (a, b) {
                    return a.w < b.w;
                });

                let ret = Utils.calcRandomWeight(aa, weightedSum);
                foodReward = ret.n;
            }

            return [fishId, finalSize, finalIncome, foodReward, fishConfig.size, fishConfig.sizeMax];
        }

        startGame() {
            this.resetGame();
            this.fishVernier.visible = true;
            this.gameState = 1;
            this.realBeginBtn.visible = false;
            this.timesFont.visible = false;
            if (this.playStatus == 0) {
                this.endBtn.visible = true;
                this.tryEndBtn.visible = false;
            }
            else if (this.playStatus == 1) {
                this.tryEndBtn.visible = true;
                this.endBtn.visible = false;
            }
            // this.targetX = this.rightLimit;
            this.targetX = Math.random() >= 0.5 ? this.rightLimit : this.leftLimit;
            this.animCat.play("diaoyu1", true);
            Laya.timer.once(1300, this, function () {
                if (this.gameState == 1) {
                    this.animCat.play("diaoyu2", true);
                }
            });
            Laya.timer.loop(30, this, this.onUpdate);

            //把其余几个按钮都暂时屏蔽掉
            this.closeBtn.off(Laya.Event.CLICK, this, this.onCloseBtn);
            this.collectBtn.off(Laya.Event.CLICK, this, this.onShowCollect);
            this.baitBtn.off(Laya.Event.CLICK, this, this.onShowBaitsSelect);
            this.tryBtn.off(Laya.Event.CLICK, this, this.onTry);
            this.tryQuitBtn.off(Laya.Event.CLICK, this, this.onTryQuit);

            //把倒计时屏蔽
            Laya.timer.clear(this, this.tickerRefresh);
            this.tips.visible = false;
        }

        onCloseBtn() {
            // this.close();
            this.onTryQuit();
            SoundManager.getInstance().play("Sound_clickSound", 1);
                if (this.remainTimes == 0 && this.fishTimes != 0) {
                    let times = Math.floor((FishingSystem.getFullMills()-TimeUtils.getTimeNowOnServer())/1000);
                    SdkHelper.shanchuTuisong('diaoyu_tixing_1');
                    let til=Strings$1.get('tid#WXMessage_FishingGameName');
                    let str=Strings$1.get('tid#WXMessage_FishingGameDesc');
                    SdkHelper.goTuisong('diaoyu_tixing_1', Strings$1.get('tid#WXMessage_FishingGameName'), Strings$1.get('tid#WXMessage_FishingGameDesc'), '' + times);
                }
            if (this.remainTimes > 0) {
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [2, true]);
            }
            else {
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [2, false]);
            }
            SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
            // this.showWxMessage();
        }
        onShowBaitsSelect() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            Laya.Dialog.close("ui/FishingBaitsSelect.scene");
            Laya.Dialog.open("ui/FishingBaitsSelect.scene");
        }

        onShowCollect() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            let afterHandler = Laya.Handler.create(null, function () {
                Laya.Dialog.close("ui/FishingCollectPop.scene");
                Laya.Dialog.open("ui/FishingCollectPop.scene");
            });

            FishingSystem.updateServerCollectsInfo(afterHandler);
        }

        //每30毫秒执行一次的update函数
        onUpdate() {
            let sign = Utils.getSign(this.targetX - this.fishVernier.x);
            this.fishVernier.x += sign * (this.speed + 0.0001);
            if (this.fishVernier.x > this.rightLimit) {
                this.fishVernier.x = this.rightLimit;
                this.targetX = this.leftLimit;
            }
            if (this.fishVernier.x < this.leftLimit) {
                this.fishVernier.x = this.leftLimit;
                this.targetX = this.rightLimit;
            }
            if (this.gameState == 2) {
                this.speed += this.velocity;
                if (this.speed <= 0) {
                    this.gameState = 3;
                    Laya.timer.clear(this, this.onUpdate);
                    this.animCat.play("diaoyu3", true);
                    Laya.timer.once(900, this, function () {
                        this.animCat.play("diaoyu4", true);
                        let d = this.getResult();
                        let baitId = FishingSystem.getLastBaitId();
                        let reward = this.getReward(baitId, d);
                        let fishReward = { "type": "Currency", "id": Constants.RewardType.FISH, "amount": reward[2] };
                        let rewards = reward[3] == null ? { "rewards": [fishReward] } : { "rewards": [fishReward, reward[3]] };
                        this._reward = reward;
                        this._baitId = baitId;
                        this._rewards = rewards;
                        this._result = d;
                        Laya.Dialog.close("ui/FishingDialog.scene");
                        Laya.Dialog.open("ui/FishingDialog.scene", false, { reward: reward, rewards: rewards.rewards, playStatus: this.playStatus, color: d, fishAgain: this.fishAgain });
                        SoundManager.getInstance().play("Sound_fishingPerfectSuccess");
                    });
                }
            }
        }

        finishGameToServer(fetchType = Constants.FetchType.DIRECT) {
            let errorHandler = Laya.Handler.create(this, function (res) {
                //一个通用的错误展示，然后退出到主界面 TODO
                console.log("====== fishing game error ======= " + res);
                TipsPopMgr.getInstance().showTipsById("tid#Tips_fishingError");
                this.onRefresh();
            });
            let afterHandler = Laya.Handler.create(this, function (res) {
                let fishNum = 0;
                res.actualRewards.rewards.forEach(element => {
                    if (element.id == Constants.RewardType.FISH) {
                        fishNum = Number(element.amount);
                    }
                });
                SoundManager.getInstance().play("Sound_clickReward");
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_fish_reward_get_success"]);
                if (fetchType == Constants.FetchType.AD) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, fishNum, Constants.FlyType.TYPE3, Laya.stage.width / 2, Laya.stage.height / 2);
                } else {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, fishNum, Constants.FlyType.TYPE5, Laya.stage.width / 2, Laya.stage.height / 2);
                }
                this.onRefresh();
            });
            if (this == null || this.destroyed) {
                return;
            }
            FishingSystem.finishGameToServer(this._startType, this._reward[0], this._baitId, this._rewards, this._reward[1], fetchType, afterHandler, errorHandler);
            TaskMainSystem.addProgress("FISHGAME_TIMES", 1);
        }

        resetGame() {
            Laya.timer.clear(this, this.onUpdate);
            // this.speed = this.initSpeed;
            this.gameState = 0;
            // this.fishVernier.x = this.leftLimit;
            this.fishVernier.x = this.leftLimit + Math.random() * (this.rightLimit - this.leftLimit);
            this.fishVernier.visible = false;

            this.endBtn.visible = false;

            this.showBtns();
            this.fishing_req = false;
            if (this.animCatLoad) {
                this.animCat.play("diaoyu4", true);
            }
            this._reward = null;
            this._baitId = null;
            this._rewards = null;
            this._result = null;
        }

        initLimit() {
            let left = this.root.getChildByName("left_limit");
            let right = this.root.getChildByName("right_limit");
            this.leftLimit = left.x;
            this.rightLimit = right.x;
        }

        onClickStartGame() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            if (this.gameState == 0 && (this.remainTimes > 0 || this.playStatus == 1)) {
                this._startType = 1;
                this.fishAgain = this.fishingRetryFreeTimes;
                this.startGame();
            }
        }

        onClickCouponStart() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            if (this.gameState == 0 && this.ticaketNums > 0) {
                this._startType = 4;
                this.fishAgain = this.fishingRetryFreeTimes;
                this.startGame();
            }
        }

        done(type, shareId) {
            if (shareId == Constants.ShareId.Fishing) {
                if (type == "share") {
                    this._startType = 3;
                }
                else {
                    this._startType = 2;
                }
                if (this.gameState == 0) {
                    EventMgr.getInstance().postEvent("SERVICE_AD_FREE_FISHING_GAME");
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon18");
                    return;
                }
            }
        }

        onClickStartGameAd() {
            // if(this._startType == 2) {
            //     if(this.gameState == 0) {
            //         this.startGame();
            //         return;
            //     }
            // }
            SdkHelper.showRewardVideoAd(Constants.ShareId.Fishing);
            // EventMgr.getInstance().postEvent("SDK_VIDEO_DONE", Constants.ShareId.Fishing);
        }

        onClickStartGameSh() {
            // if(this._startType == 3) {
            //     if(this.gameState == 0) {
            //         this.startGame();
            //         return;
            //     }
            // }
            SdkHelper.shareAppMessage(Constants.ShareId.Fishing);
            // EventMgr.getInstance().postEvent("SDK_SHARE_DONE", Constants.ShareId.Fishing);
        }

        onClickEndGame() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.gameState = 2;
            this.fishTimes++;
        }

        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Fishing, 1);
            this.resetGame();
            this.setBaitImg();
            SoundManager.getInstance().play("Sound_bgm2");
            EventMgr.getInstance().registEvent("SET_BAITS_IMAGE", this, this.setBaitImg);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done, ["ad"]);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done, ["share"]);
            TaskMainSystem.setTaskMainVisible(false);
        }

        onDisable() {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [1]);
            EventMgr.getInstance().postEvent("RESTART_BARRAGES");
            this.resetGame();
            this.animCat.stop();
            if (AutoZhaolanMgr.getInstance().getFast()) {
                SoundManager.getInstance().play("Sound_bgm3", 0);
            } else {
                SoundManager.getInstance().play("Sound_bgm1", 0);
            }

            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SHOW_WX_MESSAGE", this, this.showWxMessage);
            TaskMainSystem.startCheck(0);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Fishing, 0);
            Laya.timer.clearAll(this);
        }

        showWxMessage() {
            if (this.remainTimes != 0 || this.fishTimes == 0) {
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                return;
            }
            this.fishTimes = 0;
            let settingHandler = Laya.Handler.create(this, function (res) {
                if (!res) {
                    let handler = Laya.Handler.create(this, function (e) {
                        SdkHelper.requestSubscribeMessage([Constants.WxMessageTmplIdKeys.FISHING]);
                        SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                    });
                    let handler1 = Laya.Handler.create(this, function (e) {
                        //   SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                    });
                    SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                    //屏蔽
                    Laya.Dialog.open("ui/TipsDialog.scene", false, [Strings$1.get("tid#WXMessageFishingGame"), handler, handler1]);
                }
                else {
                    SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                }
            });
            SdkHelper.checkSubscriptions(Constants.WxMessageTmplIdKeys.FISHING, settingHandler);
        }

        guide13() {
            let guide = GuideSystem.getGuide("GuideGroup_13");
            if (guide) {
                guide.registerObj("btn0", this);
                guide.registerClick("btn0", this.baitBtn);
                let rect = new Laya.Rectangle(295, 1733, 1260, 702);
                guide.registerObj("btn1", this);
                guide.registerClick("btn1", this.baitBtn, rect);
                guide.registerObj("btn2", this);
                guide.registerClick("btn2", this.startBtn);
                guide.registerObj("btn3", this);
                guide.registerClick("btn3", this.endBtn);
                guide.registerObj("btn4", this);
                guide.registerClick("btn4", this.resultShow);
                guide.start();
            }
        }

        Guide2_FishGame(eventString) {
            if (eventString == "Guide2_FishGame_3") {
                this.onShowBaitsSelect();
            } else if (eventString == "Guide2_FishGame_4") {
                Laya.Dialog.closeAll();
            } else if (eventString == "Guide2_FishGame_5") {
                Laya.Dialog.closeAll();
            } else if (eventString == "Guide2_FishGame_7") {
                this.onClickStartGame();
            } else if (eventString == "Guide2_FishGame_10") {
                this.onClickEndGame();
            }
        }

        showBtns() {
            this.coupon_btn.visible = false;
            this.startBtn.visible = false;
            this.startBtnSh.visible = false;
            this.startBtnAd.visible = false;
            this.timesFont.visible = true;
            if (this.remainTimes <= 0) {
                if (FishingSystem.getFishingTicketNum() > 0) {
                    this.coupon_btn.visible = true;
                    this.realBeginBtn = this.coupon_btn;
                    this.timesFont.visible = false;
                    return;
                }
                let dailyFreeTimes = FishingSystem.getDailyFreeTimes();
                if (dailyFreeTimes < Number(Const.get("FishingAdsTimeMax"))) {
                    if (SdkHelper.hasVideo()) {
                        this.startBtnAd.visible = true;
                        this.realBeginBtn = this.startBtnAd;
                    } else {
                        this.startBtnSh.visible = true;
                        this.realBeginBtn = this.startBtnSh;
                    }
                    return;
                }
            }
            this.startBtn.visible = true;
            this.realBeginBtn = this.startBtn;
        }

    }

    class HomeRubbish extends Spine {
        constructor(data) {
            super();
            this._rubbishData = data;
            this.loadAnim("anim2/jiayuanhudong.sk");
            if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_1) {
                this.playAnim("laji1");
            } else if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_2) {
                this.playAnim("laji2");
            } else if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_3) {
                this.playAnim("laji3");
            }
            this.x = this._rubbishData._x;
            this.y = this._rubbishData._y;
            this._sprite = new Laya.Sprite();
            this._sprite.width = 100;
            this._sprite.height = 100;
            this._sprite.pivotX = 50;
            this._sprite.pivotY = 50;
            this.addChild(this._sprite);
        }

        pickUp(handler) {
            this.addCompleteHandler(Laya.Handler.create(this, function () {
                if (handler) {
                    handler.run();
                }
                this.destroy();
            }));
            if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_1) {
                this.playAnim("laji1-1", false);
            } else if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_2) {
                this.playAnim("laji2-1", false);
            } else if (this._rubbishData.type == Constants.HomeRubbishType.Rubbush_3) {
                this.playAnim("laji3-1", false);
            }
        }

        onDestroy() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }

        onClick(fun, obj) {
            this._sprite.offAll();
            this._sprite.on(Laya.Event.CLICK, obj, fun);
        }

        set enabled(value) {
            this._sprite.mouseEnabled = value;
            if (!this._sprite.mouseEnabled) {
                this._sprite.offAll();
            }
        }

        get enabled() {
            return this._sprite.mouseEnabled;
        }
    }

    /**
    * 本示例采用非脚本的方式实现，而使用继承页面基类，实现页面逻辑。在IDE里面设置场景的Runtime属性即可和场景进行关联
    * 相比脚本方式，继承式页面类，可以直接使用页面定义的属性（通过IDE内var属性定义），比如this.tipLbll，this.scoreLbl，具有代码提示效果
    * 建议：如果是页面级的逻辑，需要频繁访问页面内多个元素，使用继承式写法，如果是独立小模块，功能单一，建议用脚本方式实现，比如子弹脚本。
    */

    class Homeland extends Laya.Scene {
        constructor() {
            super();
            this.sceneId = Constants.HomelandId;//4
        }

        onAwake() {
            // EventMgr.getInstance().registEvent("ADD_BUILDING", this, this.addBuilding);
            this.root = this.getChildByName("root");

            this._minX = (this.root.width + Math.min(Laya.stage.width, this.root.width)) / 2 - this.map_touch.width;
            this._maxX = (this.root.width - Math.min(Laya.stage.width, this.root.width)) / 2;

            this._bg1MinX = 500;
            this._bg1MaxX = 1000;

            this._bg2MinX = 500;
            this._bg2MaxX = 900;

            this.onBg();
            this.initPictures();

            this.initMapObjects();
            this.initBtns();
            // this.initPets();

            this.map_touch.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            this.map_touch.on(Laya.Event.MOUSE_UP, this, this.onMouseUp, [1]);
            this.map_touch.on(Laya.Event.MOUSE_OUT, this, this.onMouseUp, [2]);

            this.MapX = this.MapX;

            this.petBtn.on(Laya.Event.CLICK, this, this.onPetBtnClick);
            this.photoWall.on(Laya.Event.CLICK, this, this.onPhotoWallClick);
            this.housekeeperBtn.on(Laya.Event.CLICK, this, this.onHousekeeperBtnClick);

            // this.photoWallBtn.on(Laya.Event.CLICK, this, this.onPhotoWallClick);
            this.petBtn.x = (Laya.stage.width - 1132) / 2 + 128;
            this.housekeeperBtn.x = this.dragBtn.x = this.pettingBtn.x = this.toyBtn.x = (Laya.stage.width + 1132) / 2 - this.dragBtn.width * this.dragBtn.scaleX + 60;
        }

        onEnable() {
            Laya.MouseManager.multiTouchEnabled = false;
            EventMgr.getInstance().registEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            this.checkGuide("GuideGroup_14");
            EventMgr.getInstance().postEvent("SERVICE_HOME_FETCH_PHOTOWALL");
            EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
            this.createRubbish();
            //Laya.LocalStorage.removeItem("seePhoto");
            let seePhoto = Laya.LocalStorage.getJSON("seePhoto");
            if (!seePhoto) {
                this.tipBox = new Laya.Box();
                this.photoWall.addChild(this.tipBox);
                this.tipBox.zOrder = 10000;
                this.tipBox.pos(600, -80);
                this.tipBox.scale(4, 4);
                this.anim1 = new Spine();
                this.tipBox.addChild(this.anim1);
                this.anim1.loadAnim("anim3/jiayuanjiantou.sk");
                this.anim1.playAnim("jiantoudonghua1", true);
            }
            EventMgr.getInstance().registEvent("START_HOUSEKEEP_SERVICE", this, this.onHousekeeperAd);
            TaskMainUIMgr.getInstance().destroyBtnAnim();
            if(this.pettingBtn.visible==true){
                this.yindao=1;
                TaskMainUIMgr.getInstance().registButton("Click_Play", this.pettingBtn);
            }
            else if(this.dragBtn.visible==true){
                this.yindao=1;
                TaskMainUIMgr.getInstance().registButton("Click_Play", this.dragBtn);
            }
            else if(this.toyBtn.visible==true){
                this.yindao=1;
                TaskMainUIMgr.getInstance().registButton("Click_Play", this.toyBtn);
            }
           
        }

        onDisable() {
            if (this.tipBox) {
                this.tipBox.destroy();
            }
            Laya.MouseManager.multiTouchEnabled = true;
            EventMgr.getInstance().removeEvent("CAMERA_TYPE_SWITCH", this, this.cameraSwitch);
            EventMgr.getInstance().removeEvent("START_HOUSEKEEP_SERVICE", this, this.onHousekeeperAd);
            this._rubbishMap.forEach((value, key) => {
                value.destroy();
            });
            this._rubbishMap.clear();
        }

        onPetBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            Laya.Dialog.close("ui/HomePetPop.scene");
            Laya.Dialog.open("ui/HomePetPop.scene", false);
        }

        onBg() {
            if (!this.bgAnim) {
                this.bgAnim = new Laya.Skeleton();
                setTimeout(() => {
                    this.bgAnim.load("anim2/yutangdonghua.sk", Laya.Handler.create(this, function () {
                        this.map_touch.addChild(this.bgAnim);
                        this.bgAnim.x = 1357;
                        this.bgAnim.y = 1579;
                        this.bgAnim.play("animation", true);
                        this.bgAnim.zOrder = -1;
                    }));
                }, 100);
            }
        }

        initPictures() {

        }

        initBtns() {
            this.dragBtn.visible = true;
            this.pettingBtn.visible = false;
            this.toyBtn.visible = false;

            GameContext.HomelandInteractState = Constants.InteractState.drag;

            this.dragBtn.on(Laya.Event.CLICK, this, this.onDragBtnClick);
            this.pettingBtn.on(Laya.Event.CLICK, this, this.onPettingBtnClick);
            this.toyBtn.on(Laya.Event.CLICK, this, this.onToyBtnClick);
        }

        onDragBtnClick() {
            if( this.yindao==1){
                TaskMainUIMgr.getInstance().destroyBtnAnim();
            }
            this.dragBtn.visible = false;
            this.pettingBtn.visible = true;
            this.toyBtn.visible = false;

            GameContext.HomelandInteractState = Constants.InteractState.petting;
        }

        onPettingBtnClick() {
            if( this.yindao==1){
                TaskMainUIMgr.getInstance().destroyBtnAnim();
            }
            this.dragBtn.visible = false;
            this.pettingBtn.visible = false;
            this.toyBtn.visible = true;

            GameContext.HomelandInteractState = Constants.InteractState.toy;
        }

        onToyBtnClick() {
            if( this.yindao==1){
                TaskMainUIMgr.getInstance().destroyBtnAnim();
            }
            this.dragBtn.visible = true;
            this.pettingBtn.visible = false;
            this.toyBtn.visible = false;

            GameContext.HomelandInteractState = Constants.InteractState.drag;
        }

        initMapObjects() {
            let decorate = Utils.obj2Map(MapInfo.decorate);
            for (const [k, v] of decorate) {
                this.objects.getChildByName(k).zOrder = v + 3;
            }
            this._blockMap = new BlockMap(MapInfo.width, MapInfo.height, MapInfo.block);
        }

        addPet(pet, skipFade = false) {
            this.objects.addChild(pet);
            if (!skipFade) {
                pet.alpha = 0;
                Utils.fadein(pet, 500);
            }
        }

        initPets() {
            let pets = HomePetMgr.getInstance().allPets();
            for (const pet of pets) {
                if (pet.parent == null)
                    this.addPet(pet, true);
                else
                    pet.alpha = 1;
            }
        }

        onOpened() {
            this.initPets();
            HomePetMgr.getInstance().clearRemoved();
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SCENE_BACK"]);
        }

        onClosed() {
            EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "SCENE_HIDE"]);
            Laya.timer.clear(this, this.autoPickUpRubbish);
            Laya.timer.clear(this, this.autoClearCat);
            this._rubbishMap.forEach(element => {
                element.destroy();
            });
            this._rubbishMap.clear();
        }

        set MapX(x) {
            // console.debug(x, this._minX, this._maxX, Laya.stage.width, Laya.stage.height, this.root.width);
            if (x < this._minX)
                x = this._minX;
            else if (x > this._maxX)
                x = this._maxX;
            this.bg1.x = this._bg1MaxX - (x - this._minX) / (this._maxX - this._minX) * (this._bg1MaxX - this._bg1MinX);
            this.bg2.x = this._bg2MaxX - (x - this._minX) / (this._maxX - this._minX) * (this._bg2MaxX - this._bg2MinX);
            this.map_touch.x = x;
        }

        get MapX() {
            return this.map_touch.x;
        }

        onMouseDown() {
            // console.debug("onMouseDown");
            this._moving = true;
            if (this.mousePoint) {
                this.mousePoint.x = Laya.stage.mouseX;
                this.mousePoint.y = Laya.stage.mouseY;
            } else {
                this.mousePoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            }

            this._startX = this.MapX;

            this.map_touch.offAll(Laya.Event.MOUSE_MOVE);
            this.map_touch.on(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
        }

        onMouseMove() {
            // console.debug("onMouseMove");
            if (this._moving) {
                let offsetX = Laya.stage.mouseX - this.mousePoint.x;
                this.MapX = this._startX + offsetX;
            }
        }

        onMouseUp(d) {
            // console.debug("onMouseUp", d);
            this._moving = false;
            this.map_touch.offAll(Laya.Event.MOUSE_MOVE);
        }

        onPhotoWallClick() {
            Laya.LocalStorage.setJSON("seePhoto", 1);
            if (this.tipBox) {
                this.tipBox.destroy();
            }
            Laya.Dialog.close("ui/PhotoWallDialog.scene");
            Laya.Dialog.open("ui/PhotoWallDialog.scene");
        }

        cameraSwitch(type) {
            if (type) {
                this.petBtn.visible = false;
                this.dragBtn.visible = false;
                this.pettingBtn.visible = false;
                this.toyBtn.visible = false;
                this.housekeeperBtn.visible = false;
            } else {
                this.petBtn.visible = true;
                this.dragBtn.visible = false;
                this.pettingBtn.visible = false;
                this.toyBtn.visible = false;
                this.housekeeperBtn.visible = true;
                if (GameContext.HomelandInteractState == Constants.InteractState.drag) {
                    this.dragBtn.visible = true;
                } else if (GameContext.HomelandInteractState == Constants.InteractState.petting) {
                    this.pettingBtn.visible = true;
                } else if (GameContext.HomelandInteractState == Constants.InteractState.toy) {
                    this.toyBtn.visible = true;
                }
            }
        }

        checkGuide(eventString) {
            if (eventString == "GuideGroup_14") {
                let guide = GuideSystem.getGuide("GuideGroup_14");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.petBtn);
                    guide.registerObj("btn5", this);
                    guide.registerClick("btn5", this.dragBtn);
                }
            }
        }

        createRubbish() {
            let rubbishData = PetSystem.creatRubbish();
            if (rubbishData == null) {
                return;
            }
            this._rubbishMap = new Map();
            rubbishData.forEach((value, key) => {
                let rubbish = new HomeRubbish(value);
                this._rubbishMap.set(key, rubbish);
                this.objects.addChild(rubbish);
                let id = key;
                rubbish.onClick(function () {
                    let reward = PetSystem.getRubbishRewards(id);
                    let point = new Laya.Point(rubbish.x, rubbish.y);
                    point = this.objects.localToGlobal(point);
                    if (reward.FISH > 0) {
                        ItemFlyMgr.getInstance().fly(Constants.RewardType.FISH, Number(reward.FISH), Constants.FlyType.ENHANCE, point.x, point.y);
                    }
                    if (reward.STAR > 0) {
                        ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, Number(reward.STAR), Constants.FlyType.ENHANCE, point.x, point.y);
                    }
                    if (reward.MEDAL > 0) {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.MEDAL, Number(reward.MEDAL), null, point.x, point.y);
                    }
                    if (reward.BOOK > 0) {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.BOOK, Number(reward.BOOK), null, point.x, point.y);
                    }
                    rubbish.enabled = false;
                    let handler = Laya.Handler.create(this, function () {
                        this._rubbishMap.delete(id);
                        EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["HOME_CLEAN", 1, null]);
                    });
                    rubbish.pickUp(handler);
                }, this);
                rubbish.alpha = 0;
                Utils.fadein(rubbish, 500);
            });
        }

        onHousekeeperBtnClick() {
            let needHelp = null;
            if (this._rubbishMap.size > 0) {
                needHelp = true;
            }
            let pets = HomePetMgr.getInstance().allPets();
            pets.forEach(pet => {
                if (pet.petData.hunger || pet.petData.thirst || pet.petData.dirty) {
                    needHelp = true;
                }
            });
            Laya.Dialog.open("ui/HousekeeperPop.scene", true, needHelp);
        }

        onHousekeeperAd() {
            this.startAutoPickUpRubbish();
            this.startAutoClearCats();
        }

        startAutoPickUpRubbish() {
            Laya.timer.clear(this, this.autoPickUpRubbish);
            if (this._rubbishMap.size <= 0 || this.destroyed) {
                return;
            }
            let iterableIterator = this._rubbishMap.keys();
            this.autoPickUpRubbish(iterableIterator);
            Laya.timer.loop(200, this, this.autoPickUpRubbish, [iterableIterator]);
        }

        autoPickUpRubbish(iterableIterator) {
            let iterator = iterableIterator.next();
            if (iterator.done) {
                Laya.timer.clear(this, this.autoPickUpRubbish);
                return;
            }
            let key = iterator.value;
            if (this._rubbishMap.has(key)) {
                let rubbish = this._rubbishMap.get(key);
                let reward = PetSystem.getRubbishRewards(key, Constants.FetchType.AD);
                let point = new Laya.Point(rubbish.x, rubbish.y);
                point = this.objects.localToGlobal(point);
                if (reward.FISH > 0) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.FISH, Number(reward.FISH), Constants.FlyType.ENHANCE, point.x, point.y);
                }
                if (reward.STAR > 0) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, Number(reward.STAR), Constants.FlyType.ENHANCE, point.x, point.y);
                }
                if (reward.MEDAL > 0) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.MEDAL, Number(reward.MEDAL), null, point.x, point.y);
                }
                if (reward.BOOK > 0) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.BOOK, Number(reward.BOOK), null, point.x, point.y);
                }
                rubbish.enabled = false;
                this._rubbishMap.delete(key);
                EventMgr.getInstance().postEvent("SERVICE_EVENT_FIRE_EVENT", ["HOME_CLEAN", 1, null]);
                rubbish.pickUp();
            }
        }

        startAutoClearCats() {
            let pets = HomePetMgr.getInstance().allPets();
            if (pets.length <= 0) {
                return;
            }
            this.autoClearCat(pets);
            Laya.timer.loop(200, this, this.autoClearCat, [pets]);
        }

        autoClearCat(pets) {
            let doing = null;
            let allComplete = true;
            pets.forEach(pet => {
                if (doing || pet.getIsHealthyIng()) {
                    allComplete = false;
                    return;
                }
                if (pet.petData.hunger || pet.petData.thirst || pet.petData.dirty) {
                    allComplete = false;
                    doing = true;
                    EventMgr.getInstance().postEvent("SIMULATOR_SEND_CMD", [Constants.HomelandId, "PLAY_BE_HEALTHY", { "petId": pet.getId(), "adType": Constants.FetchType.AD }]);
                }
            });
            if (doing == null && allComplete) {
                Laya.timer.clear(this, this.autoClearCat);
            }
        }

    }

    class PhotoItem extends Laya.Image {

        constructor() {
            super();
        }

        onEnable() {
            this.mouseEnabled = false;
        }

        setValue(frameId, photoData, width, height) {
            this.frameId = frameId;
            this.photoData = photoData;
            if (this.photoData.getType() == 1) {
                this.skin = this.photoData.getImage();
                if (this.catArr && this.catArr.length != 0) {
                    for (const cat of this.catArr) {
                        cat.destroy();
                    }
                }
                this.catArr = null;
            } else {
                let config = this.photoData.getSystemConfig();
                this.skin = "systemPhotoBg/"+config.Background+".jpg";
                if (this.catArr && this.catArr.length != 0) {
                    for (const cat of this.catArr) {
                        cat.destroy();
                    }
                }
                this.catArr = [];
                let catSkins = JSON.parse(config.Cat);
                let catCatroute = "systemPhoto/"+config.Catroute;
                for (const catSkin of catSkins) {
                    let cat = new Laya.Image();
                    cat.skin = catCatroute + "/" + catSkin + ".png";
                    cat.width = width;
                    cat.height = height;
                    this.addChild(cat);
                    this.catArr.push(cat);
                }
            }
            this.anchorX = 0.5;
            this.anchorY = 0.5;
            this.width = width;
            this.height = height;
            this.scaleX = this.scaleY = 1;
            this.pos(width / 2, height / 2);
        }

        getPhotoId() {
            if (this.photoData) {
                return this.photoData.getId();
            }
        }


        clear() {
            this.offAll();
            this.removeSelf();
            this.destroy();
        }
    }

    class PhotoFrame extends Laya.Script {

        constructor() {
            super();
            this.boxWidth = null;
            this.boxHeight = null;
            this.framId = null;
            /** @prop {name:boxWidth,type:int}*/
            /** @prop {name:boxHeight,type:int}*/
            /** @prop {name:framId,type:int}*/
        }

        onAwake() {
            this.scale = this.boxWidth / this.boxWidth;
            this.owner.width = this.boxWidth;
            this.owner.height = this.boxHeight;
            this.frame = this.owner.getChildByName("frame");
            this.frame.width = this.boxWidth / 2 + 10;
            this.frame.height = this.boxHeight / 2 + 10;
            this.photo = this.owner.getChildByName("photo");
            this.photo.scaleX = this.scale;
            this.photo.scaleY = this.scale;
            this.light = this.owner.getChildByName("light");
            this.light.visible = false;
            this.light.width = this.frame.width * 2 - 10;
            this.light.height = this.frame.height * 2 - 9;
            this.drageLimit = 100;
            EventMgr.getInstance().registEvent("LIGHT_FRAME", this, this.lightFrame);
        }

        onEnable() {
            this.downClick = false;
            if (this.alphaPhotoItem) {
                this.alphaPhotoItem.removeSelf();
                this.alphaPhotoItem.destroy();
            }
            if (this.photoData) {
                if (this.photoItem == null || this.photoItem.destroyed) {
                    this.photoItem = new PhotoItem();
                    this.photoItem.setValue(this.framId, this.photoData, this.boxWidth, this.boxHeight);
                    this.photo.addChild(this.photoItem);
                } else {
                    this.photoItem.setValue(this.framId, this.photoData, this.boxWidth, this.boxHeight);
                }
                this.owner.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            } else {
                if (this.photoItem) {
                    this.photoItem.clear();
                }
                this.owner.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            }
            this.owner.on(Laya.Event.CLICK, this, this.openPhotoAlbum);
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("LIGHT_FRAME", this, this.lightFrame);
        }

        initPhoto(photoData) {
            this.photoData = photoData;
        }

        updataPhoto(photoData) {
            this.photoData = photoData;
            if (this.alphaPhotoItem) {
                this.alphaPhotoItem.removeSelf();
                this.alphaPhotoItem.destroy();
            }
            if (this.photoData) {
                if (this.photoItem == null) {
                    this.photoItem = new PhotoItem();
                    this.photoItem.setValue(this.framId, this.photoData, this.boxWidth, this.boxHeight);
                    this.photo.addChild(this.photoItem);
                } else {
                    this.photoItem.setValue(this.framId, this.photoData, this.boxWidth, this.boxHeight);
                }
                this.owner.on(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            } else {
                if (this.photoItem) {
                    this.photoItem.clear();
                    this.photoItem = null;
                }
                this.owner.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            }
            this.downClick = false;
            this.owner.off(Laya.Event.CLICK, this, this.openPhotoAlbum);
            this.owner.on(Laya.Event.CLICK, this, this.openPhotoAlbum);
            this.light.visible = false;
            EventMgr.getInstance().postEvent("UPDATE_BOX_ANIM");
        }

        openPhotoAlbum() {
            this.downClick = false;
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            this.downPoint = null;
            Laya.Dialog.close("ui/PhotoAlbumDialog.scene.scene");
            Laya.Dialog.open("ui/PhotoAlbumDialog.scene", false);
        }


        onMouseDown() {
            this.downClick = true;
            this.downPoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
        }

        checkDragStart() {
            if (!this.downClick) {
                return;
            }
            if (Laya.stage.mouseX - this.downPoint.x > this.drageLimit) {
                //右
                this.photoItem.x += this.drageLimit / 2;
                this.moveStart();
                Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            } else if (this.downPoint.x - Laya.stage.mouseX > this.drageLimit) {
                //左
                this.photoItem.x -= this.drageLimit / 2;
                this.moveStart();
                Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            } else if (this.downPoint.y - Laya.stage.mouseY > this.drageLimit) {
                //上
                this.photoItem.y -= this.drageLimit / 2;
                this.moveStart();
                Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            } else if (Laya.stage.mouseY - this.downPoint.y > this.drageLimit) {
                //下
                this.photoItem.y += this.drageLimit / 2;
                this.moveStart();
                Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            }
        }

        moveStart() {
            this.removeEvent();
            let clickPoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            clickPoint = this.owner.parent.globalToLocal(clickPoint);
            this.owner.off(Laya.Event.CLICK, this, this.openPhotoAlbum);
            this.alphaPhotoItem = new PhotoItem();
            this.alphaPhotoItem.setValue(this.framId, this.photoData, this.boxWidth, this.boxHeight);
            this.alphaPhotoItem.alpha = 0.3;
            this.photo.addChild(this.alphaPhotoItem);
            this.photoData = null;
            EventMgr.getInstance().postEvent("PHOTO_START_DRAG", [this.photoItem]);
            this.photoItem = null;
            this.downClick = false;
            this.light.visible = true;
            Utils.fadein(this.light, 500);
        }

        onDisable() {
            this.removeEvent();
        }

        lightFrame(frameId) {
            if (frameId != null && frameId == this.framId) {
                this.light.visible = true;
            } else {
                this.light.visible = false;;
            }
        }

        removeEvent() {
            this.downClick = false;
            this.downPoint = null;
            this.owner.off(Laya.Event.CLICK, this, this.openPhotoAlbum);
            this.owner.off(Laya.Event.MOUSE_DOWN, this, this.onMouseDown);
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.checkDragStart);
            EventMgr.getInstance().postEvent("UPDATE_BOX_ANIM");
        }
    }

    class PhotoWall extends Laya.Box {

        constructor() {
            super();
        }

        onEnable() {
            this.frame1 = this.getChildByName("frame1").getComponent(PhotoFrame);
            this.frame2 = this.getChildByName("frame2").getComponent(PhotoFrame);
            this.frame3 = this.getChildByName("frame3").getComponent(PhotoFrame);
            this.frame4 = this.getChildByName("frame4").getComponent(PhotoFrame);
            this.frames = [this.frame1, this.frame2, this.frame3, this.frame4];
            EventMgr.getInstance().registEvent("UPDATA_PHOTO_WALL", this, this.updata);
            this.init();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATA_PHOTO_WALL", this, this.updata);
        }

        updata() {
            this.photosData = PhotoSystem.getPhotoFrameData();
            for (let i = 0; i < this.photosData.length; i++) {
                this.frames[i].updataPhoto(PhotoSystem.getPhotoData(this.photosData[i]));
            }
        }
        init() {
            this.photosData = PhotoSystem.getPhotoFrameData();
            for (let i = 0; i < this.photosData.length; i++) {
                this.frames[i].initPhoto(PhotoSystem.getPhotoData(this.photosData[i]));
            }
        }

    }

    /**
     * 本示例采用非脚本的方式实现，而使用继承页面基类，实现页面逻辑。在IDE里面设置场景的Runtime属性即可和场景进行关联
     * 相比脚本方式，继承式页面类，可以直接使用页面定义的属性（通过IDE内var属性定义），比如this.tipLbll，this.scoreLbl，具有代码提示效果
     * 建议：如果是页面级的逻辑，需要频繁访问页面内多个元素，使用继承式写法，如果是独立小模块，功能单一，建议用脚本方式实现，比如子弹脚本。
     */

    class LoginScene extends Laya.Scene {
        constructor() {
            super();
            this.sceneId = 0;
        }


        onEnable() {
            this.input = this.getChildByName("input");
            let loginButton = this.getChildByName("loginButton");
            loginButton.on(Laya.Event.CLICK, this, this.onLoginClick);
            let rid = Laya.LocalStorage.getItem("user_rid");
            if(rid!=null&&rid!=""){
                this.input.text = rid;
            }
        }

        onLoginClick() {
            let text=this.input.text;
            if(text){
               // System.pplogin();
               System.login(text , Laya.Handler.create(this, this.onLogin));
            }else {
                TipsPopMgr.getInstance().showTipsById("tid#loginAccount");
            }
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onLogin() {
            SceneMgr.getInstance().loadScene(Constants.NauticalSceneId, Laya.Handler.create(this, function(scene1) {
                console.debug("handler1", 1);
                SceneMgr.getInstance().loadScene(Constants.BeachSceneId, Laya.Handler.create(this, function(scene2) {
                    SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId, Laya.Handler.create(this, function(scene3) {
                        console.debug("handler3", 1);
                        InGameLoader.getInstance().start();
                        SceneMgr.getInstance().setSwitchAllowed();
                    }));
                    console.debug("handler2", 2);
                }));
                console.debug("handler1", 2);
            }));
        }
    }

    /**
     * 本示例采用非脚本的方式实现，而使用继承页面基类，实现页面逻辑。在IDE里面设置场景的Runtime属性即可和场景进行关联
     * 相比脚本方式，继承式页面类，可以直接使用页面定义的属性（通过IDE内var属性定义），比如this.tipLbll，this.scoreLbl，具有代码提示效果
     * 建议：如果是页面级的逻辑，需要频繁访问页面内多个元素，使用继承式写法，如果是独立小模块，功能单一，建议用脚本方式实现，比如子弹脚本。
     */

    class NauticalScene extends Laya.Scene {
        constructor() {
            super();
            this.sceneId = Constants.NauticalSceneId;//3
            this.buildingMap = new Map();
        }

        onAwake() {
            EventMgr.getInstance().registEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().registEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().registEvent("SWITCH_SCENE_COMPLETE", this, this.switchComplete);
            this.onBg();
        }

        onDestroy(){
            EventMgr.getInstance().removeEvent("ADD_BUILDING", this, this.addBuilding);
            EventMgr.getInstance().removeEvent("REMOVE_BUILDING", this, this.removeBuilding);
            EventMgr.getInstance().removeEvent("SWITCH_SCENE_COMPLETE", this, this.switchComplete);
            if(this.jianzao) {
                this.jianzao.destroy();
                this.jianzao = null;
            }
        }

        onBg() {
            if(!this.bgAnim) {
                this.bgAnim = new Laya.Skeleton();
                setTimeout(() => {
                    this.bgAnim.load("anim1/hailangdonghua.sk", Laya.Handler.create(this, function() {
                        this.root.addChild(this.bgAnim);
                        this.bgAnim.zOrder = -10;
                        this.bgAnim.scaleX = 1.2;
                        this.bgAnim.scaleY = 1.2;
                        this.bgAnim.x = 890;
                        this.bgAnim.y = 1175;
                        this.bgAnim.play("sea2", true);
                    }));
                }, 100);
            }
        }

        onInitBuilding() {
            let buildingList = BuildingSystem.getBuildingDataListInfo(this.sceneId).list;
            for(var i = 0 ; i < buildingList.length ; ++i) {
                let data = buildingList[i];
                let buildingId = data.getId();
                if(this.buildingMap.get(buildingId) == null && data.getLocationData() != null) {
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root);
                    this.buildingMap.set(buildingId, building);
                }
            }
        }

        onEnable() {
            if (AutoZhaolanMgr.getInstance().getFast()) {
                SoundManager.getInstance().play("Sound_bgm3", 0);
            } else {
                SoundManager.getInstance().play("Sound_bgm1", 0);
            }
            this.root = this.getChildByName("root");
            this.onInitBuilding();
            NauticalSystem$1.getNauticalInfoReq(Laya.Handler.create(this, this.updateNautical));
            let buildingShip = this.root.getChildByName("building_ship");
            buildingShip.on(Laya.Event.CLICK, this, this.onBuildingShipClick);
        }

        onBuildingShipClick(){
            // if(GameContext.Sliding)
            //     return;
            // Laya.Dialog.close("ui/PickUI.scene");
            // Laya.Dialog.open("ui/PickUI.scene");
        }

        updateNautical(res) {
            EventMgr.getInstance().postEvent("BOAT_UPDATE_INFO");
        }


        addBuilding(data, isNew) {
            if (data && data.getLocationData && data.getBelongScene) {
                if (data.getBelongScene() == this.sceneId) {
                    let building = BuildingFactory.getInstance().createBuilding(data, this.root, Constants.CoffeeBuildingScale, Laya.Handler.create(this, this._onBuildingAdded));
                    this.buildingMap.set(data.getId(), building);

                    if (this.jianzao && !this.jianzao.destroyed) {
                        this.jianzao.destroy();
                    }
                    this.jianzao = new Spine();
                    this.root.addChild(this.jianzao);
                    let locationData = data.getLocationData();
                    let scale = Number(locationData.getBaseSize());
                    if (isNew) {
                        this.jianzao.loadAnim("anim1/jianzao1.sk");
                        this.jianzao.playAnim("jianzao1", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale * 2;
                    } else {
                        this.jianzao.loadAnim("anim3/jianzao.sk");
                        this.jianzao.playAnim("yanwu", false);
                        this.jianzao.scaleX = this.jianzao.scaleY = scale / 2;
                    }
                    this.jianzao.visible = true;
                    this.jianzao.addCompleteHandler(Laya.Handler.create(this, function () {
                        this.jianzao.destroy();
                        this.jianzao = null;
                    }));
                    this.jianzao.pos(Number(locationData.getSelectionX()) + 152 * scale, Number(locationData.getSelectionY()) + 80 * scale);
                    this.jianzao.zOrder = Math.floor(this.jianzao.y) + Number(locationData.getZOrder());
                }
            }
        }

        removeBuilding(data) {
            if(data && data.getBelongScene && data.getBelongScene()==this.sceneId){
                if(this.buildingMap.has(data.getId())) {
                    let building = this.buildingMap.get(data.getId());
                    building.cleanBuilding();
                    this.buildingMap.delete(data.getId());
                    building.destroy();
                }
            }
        }

        getBuilding(id) {
            for (var [key, value] of this.buildingMap) {
                if(Number(key) == Number(id)) {
                    if(value.isLoaded()) {
                        return value;
                    }
                    return null;
                }
            }
            return null;
        }

        lockTypeChange(lock) {
            if (lock != null && lock) {
                if (this.lockBox) {
                    this.lockBox.visible = true;
                    //this.mouseEnabled = false;
                } else {
                    this.lockBox = this.getChildByName("SceneLock");
                    let mid = new Laya.Point(Laya.stage.width / 2, 0);
                    mid = this.globalToLocal(mid);
                    this.lockBox.x = mid.x;
                    mid = null;
                    this.get=this.lockBox.getChildByName("get");
                    this.get.text=Strings$1.get('tid#UI_170');
                    this.lockNum = this.lockBox.getChildByName("lockNum");
                    let item = ConfigMgr$1.getInstance().getItemById("Scene", Constants.NauticalSceneId.toString());
                    if (item) {
                        let unlockStar = JSON.parse(item.Unlock);
                        this.lockNum.text = "" + unlockStar[0].Star;
                    }
                    this.lockBox.visible = true;
                    //this.mouseEnabled = false;
                }
            } else {
                this.mouseEnabled = true;
                if (this.lockBox) {
                    this.lockBox.visible = false;
                    this.lockBox.removeSelf();
                    this.lockBox.destroy();
                }
            }
        }

        switchComplete(id) {
            if (id == Constants.NauticalSceneId) {
                let zhuliUI = this.getChildByName("zhuliUI");
                zhuliUI.visible = true;
                zhuliUI.getComponent(Laya.Script).open();
            }
        }

        onDisable() {
            let zhuliUI = this.getChildByName("zhuliUI");
            zhuliUI.visible = false;
        }

    }

    /**
     * 助力UI
     */
    class ZhuliUI extends Laya.Script {
        constructor() {
            super();
        }

        onStart() {
        }

        onEnable() {
            EventMgr.getInstance().registEvent("SHOW_ZHULI_SHOW_FIRST", this, this.openjia);
            EventMgr.getInstance().registEvent("UPDATE_ZHULI_LIST", this, this.showzhulinum);
        }

        open() {
            let width = Laya.stage.width;
            if (BuildingSystem.hasBuildingDataByBid(3000) || BuildingSystem.hasBuildingDataByBid(3001) || BuildingSystem.hasBuildingDataByBid(3002)) {
                this.owner.visible = true;
            } else {
                this.owner.visible = false;
                return;
            }
            this.zhuliBtn = this.owner.getChildByName('zhuliBtn');
            this.zhuliBtn.visible=false;
            //this.zhuliBtn.on(Laya.Event.CLICK, this, this.invite);

            this.zhulinum=this.zhuliBtn.getChildByName('zhulinum');
            this.zhulinum.text=ZhuliSystem.getZhulirenshu()+'/3';

            this.tanchuang = true;
            if (ZhuliSystem.getZhuliOverTime()) {
                let zhuliOverTime = ZhuliSystem.getZhuliOverTime() * 1000;
                let nowTime = TimeUtils.getTimeNowOnServer() + 3000;
                let zhuliitem = Number(Const.get('QuayAssistWorkTime')) * 1000;
                let ifGet = ZhuliSystem.getHasReceive();//判断是否已经领取 true 已经领取了
                Laya.timer.once(500, this, function () {
                   this.opended=1;
                    if (!ifGet) {
                        if (zhuliOverTime < nowTime) {
                            this.tanchuang = false;
                            Laya.Dialog.close("ui/ZhuliPop.scene");
                            Laya.Dialog.open("ui/ZhuliPop.scene", false, [1, 1]);
                        }
                        else {
                            Laya.timer.once((zhuliOverTime - nowTime + 3000), this, function () {
                                Laya.Dialog.close("ui/ZhuliPop.scene");
                                Laya.Dialog.open("ui/ZhuliPop.scene", false, [1]);
                            });
                        }
                    }
                    if (this.tanchuang) {
                        if (Number(ZhuliSystem.getCacheFish()) / Number(ZhuliSystem.getCapacity()) >= 0.5) {
                            if (!Constants.Baoxiangtanchuan) {
                                Laya.Dialog.close("ui/ZhuliPopAward.scene");
                                Laya.Dialog.open("ui/ZhuliPopAward.scene", false, [1]);
                                Constants.Baoxiangtanchuan = 1;
                            }
                        }
                    }
                });

            }
            // EventMgr.getInstance().registEvent("UPDATE_WHARFLETTER_BOX",this,this.showAward);
        }
        openjia() {
            let width = Laya.stage.width;
            if (BuildingSystem.hasBuildingDataByBid(3000) || BuildingSystem.hasBuildingDataByBid(3001) || BuildingSystem.hasBuildingDataByBid(3002)) {
                this.owner.visible = true;
            } else {
                this.owner.visible = false;
                return;
            }
            this.zhuliBtn = this.owner.getChildByName('zhuliBtn');
            this.zhuliBtn.on(Laya.Event.CLICK, this, this.invite);
            this.zhuliBtn.visible=false;
            this.zhulinum=this.zhuliBtn.getChildByName('zhulinum');
            this.zhulinum.text=ZhuliSystem.getZhulirenshu()+'/3';
            // EventMgr.getInstance().registEvent("UPDATE_WHARFLETTER_BOX",this,this.showAward);
        }
        showzhulinum(){
            if(this.zhulinum){
                this.zhulinum.text=ZhuliSystem.getZhulirenshu()+'/3';
            }
        }
        invite() {
            if (ZhuliSystem.getZhulishangxian()) {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon39");
            }
            else {
                Laya.Dialog.close("ui/ZhuliPop.scene");
                Laya.Dialog.open("ui/ZhuliPop.scene", false, [0]);
            }
        }
        onDestroy() {
            Laya.Tween.clearAll(this);
        }
        getAward() {
            if (ZhuliSystem.getCacheFish() != 0 || ZhuliSystem.getCacheShell() != 0 || ZhuliSystem.getShareFish() != 0 || ZhuliSystem.getShareShell() != 0) {
                Laya.Dialog.close("ui/ZhuliPopAward.scene");
                Laya.Dialog.open("ui/ZhuliPopAward.scene", false);
            }
        }
        
        onDisable() {
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("UPDATE_ZHULI_LIST", this, this.showzhulinum);
            EventMgr.getInstance().removeEvent("SHOW_ZHULI_SHOW_FIRST", this, this.openjia);
        }
    }

    class AdoptAD extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            this.petId = this._param;
            this._shareId = Constants.ShareId.AdoptPre;
            this.close1.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.adoptS.on(Laya.Event.CLICK, this, this.onAdoptSClick);
            this.adoptV.on(Laya.Event.CLICK, this, this.onAdoptVClick);
            if(SdkHelper.hasVideo()) {
                this.setBtn(2);
            }else {
                this.setBtn(1);
            }    

            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }

        setBtn(num){
            if(num==1){
                this.adoptS.visible = true;
                this.adoptV.visible = false;
                this.adText.text = Strings$1.get("tid#UI_038");
            }
            else if(num==2){
                this.adoptS.visible = false;
                this.adoptV.visible = true;
                this.adText.text = Strings$1.get("tid#UI_037");
            }
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }

        onAdoptSClick(){
            SdkHelper.shareAppMessage(this._shareId);
        }

        onAdoptVClick(){
            SdkHelper.showRewardVideoAd(this._shareId);
        }

        doneS(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent("PRE_ADOPT", [1]);
                this.close();
            }
        }

        doneV(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent("PRE_ADOPT", [1]);
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, ["error"]);
                this.close();
            }
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }

    }

    class AdoptDiamond extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable(){
            this.cost = Number(Const.get("CatAdoptADDiamond"));
            this.text1.text = Strings$1.get("tid#UI_226");
            this.text2.text = this.cost + "";
            this.text3.text = Strings$1.get("tid#UI_226_1");
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.sureBtn.on(Laya.Event.CLICK,this,this.onSureBtnClick);
            this.text2.color = this.cost > PlayerSystem.getDiamond() ? "#ff0000" : "#72584a";
            this.setPos();
        }

        setPos(){
            let space = 10;
            let w1 = this.text1.width;
            let w2 = this.diamond.width * this.diamond.scaleX;
            let w3 = this.text2.width;      
            let w4 = this.text3.width;
            let allWidth = w1 + w2 + w3 + w4 + space * 3;
            let startX = (this.width - allWidth) / 2;
            this.text1.x = startX;
            this.diamond.x = this.text1.x + w1 + space;
            this.text2.x = this.diamond.x + w2 + space;
            this.text3.x = this.text2.x + w3 + space;
        }

        onSureBtnClick(){
            if(this.cost > PlayerSystem.getDiamond()){
                Laya.Dialog.close("ui/OrderBuy.scene");
                Laya.Dialog.open("ui/OrderBuy.scene", false,[0]);
                // TipsPopMgr.getInstance().showTipsById("tid#TipsCommon88");
            }
            else {
                EventMgr.getInstance().postEvent("PRE_ADOPT", [2]);
                
            }
            this.close();
        }
    }

    /**
    * 确认放置
    */
    class AdoptSuccess extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().registEvent("SDK_SHARE_FAIL", this, this.fail);
        }

        onAwake() {
            this._shareId = Constants.ShareId.AdoptSuccess;
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.adoptBox = this.getChildByName("adoptBox");
            this.adoptName = this.adoptBox.getChildByName("adoptName");
            let closeBtn = this.adoptBox.getChildByName("close");
            closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.catBox = this.adoptBox.getChildByName("catBox");
            this.bgBox = this.getChildByName("bgBox");
            this.shareBtn = this.adoptBox.getChildByName("share");
            this.shareBtn.on(Laya.Event.CLICK, this, this.share);
            this.rewardText = this.adoptBox.getChildByName("rewardText");
            this.rewardImg = this.rewardText.getChildByName("rewardImg");
            this.adoptText = this.adoptBox.getChildByName("adoptLabel");
            this.adoptText.text = "";
            this.adoptBox.visible = false;
        }

        onOpened(petId) {
            this.petId = petId;
            let pet = PetSystem.getPetInfo(petId);
            let adoptReward = JSON.parse(pet.config.ShareReward).AdoptReward;
            if(adoptReward.id == Constants.RewardType.FISH){
                this.rewardImg.skin = "topui/yugan.png";
            }
            else if(adoptReward.id == Constants.RewardType.STAR){
                this.rewardImg.skin = "topui/xingxing.png";
            }
            this.rewardText.text = "+" + adoptReward.amount;
            if(adoptReward.amount < 100){
                this.rewardText.x += 25;
            }else if(adoptReward.amount >= 100000){
                this.rewardText.x -= 25;
            }
            this.adoptName.text = Strings$1.get(pet.getName());
            this.cat = new Spine();
            this.cat.x = this.catBox.width / 2;
            this.cat.y = this.catBox.height / 2;
            this.cat.scale(1.3,1.3);
            this.catBox.addChild(this.cat);
            this.cat.loadAnim(pet.getSource() + ".sk", pet.getSkin(), 0);
            this.cat.playAnim("play_beg", true);
            this.bgAnim = new Spine();
            this.bgAnim.loadAnim("anim3/shengjidonghua.sk",Constants.TipAnimType.ADOPT);
            this.bgAnim.playAnim("lingyang1", false);
            this.bgAnim.x = this.adoptBox.width / 2;
            this.bgAnim.y = this.adoptBox.height / 2;
            this.bgBox.addChild(this.bgAnim);
            this.bgAnim.addCompleteHandler(Laya.Handler.create(this,function(){
                this.adoptBox.visible = true;
                this.bgAnim.playAnim("lingyang2", true);
            }));
            this.adoptText.text = Strings$1.get(pet.getName()) + " " + Strings$1.get("tid#UI_073");
           // this.adoptText.text = ;
            let adoptSound = pet.getAdoptSound();
            if (adoptSound)
                SoundManager.getInstance().play(adoptSound, 1);
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }


        onClosed() {
            if(this.cat) {
                this.cat.destroy();
                this.cat = null;
            }
            this.catBox.removeChildren();
            this.adoptText.text = "";
            EventMgr.getInstance().postEvent("CHECK_GUIDEGROUP", ["GuideGroup_14", true]);
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().removeEvent("SDK_SHARE_FAIL", this, this.fail);
        }

        share() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            if(Laya.Browser.onMiniGame) {
                // wx.shareAppMessage({
                //     title: "快来一起养宠物吧!",
                //     imageUrl: "https://download-fy.hulai.com/share/cat/share1.jpg",
                //     success(res){
                //         console.log("success",res);
                //     },
                //     fail(res){
                //         console.log("fail",res);
                //     }
                // })
                SdkHelper.shareAppMessage(this._shareId);
            }
        }

        flyYugan(res){
            let num = res.rewards.rewards[0].amount;
            let rewardType = res.rewards.rewards[0].id;
            let x = Laya.stage.width/2;
            let y = Laya.stage.height/2;
            ItemFlyMgr.getInstance().fly(rewardType, num,Constants.FlyType.ST,x-33, y-24);
        }

        doneS(shareId) {
            if (shareId == this._shareId) {
                let handler=Laya.Handler.create(this,this.flyYugan);
                EventMgr.getInstance().postEvent("SERVICE_PET_SHAREGETREWARD",[this.petId,Constants.PeacockShareType.AdoptSuccess,handler] );
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, ["error"]);
                this.close();
            }
        }
    }

    class ButtonImageRp extends Laya.Script {

       constructor() {
          super();
          this.tubiao = null;
          this.award = null;
          this.animBox = null;
          this.skin = 0;
          /** @prop {name:tubiao,type:Node}*/
          /** @prop {name:award,type:Node}*/
          /** @prop {name:skin,type:int}*/
       }
       onEnable(){
       
       }
       onStart() {
          this.setPifu();
       }
       //21 -30 新增倍率   
       setPifu() {
          if (this.award) {
             if (this.skin == 0) {
                this.award.skin = 'pinjie_btn/double.png';// 双倍领取
             } else if (this.skin == 1) {
                this.award.skin = 'pinjie_btn/double_fish.png'; //鱼干双倍
             } else if (this.skin == 2) {
                this.award.skin = 'pinjie_btn/again.png'; //再来一次
             } else if (this.skin == 3) {
                this.award.skin = 'pinjie_btn/catfood.png'; //高级猫粮
             } else if (this.skin == 4) {
                this.award.skin = 'pinjie_btn/speedup_staff.png';//修炼加速
             } else if (this.skin == 5) {
                this.award.skin = 'pinjie_btn/speedup_sail.png';//出航加速
             } else if (this.skin == 6) {
                this.award.skin = 'pinjie_btn/preadopt.png';//提前领养
             } else if (this.skin == 7) {
                this.award.skin = 'pinjie_btn/OK.png';//好哒
             } else if (this.skin == 8) {
                this.award.skin = 'pinjie_btn/toshare.png';//去分享
             } else if (this.skin == 9) {
                this.award.skin = 'pinjie_btn/toauthorize.png';//去授权  
             } else if (this.skin == 10) {
                this.award.skin = 'pinjie_btn/showoff.png';//炫耀一下
             }
             else if (this.skin == 13) {
                this.award.skin = 'pinjie_btn/subscribe.png';//通知订阅
             }
             // 放置  小号  按钮    大小   190   80
             else if (this.skin == 11) {
                this.award.skin = 'pinjie_btn/on.png';//放置   
             } else if (this.skin == 12) {
                this.award.skin = 'pinjie_btn/off.png';//卸下
             }



             else if (this.skin == 21) {
                this.award.skin = 'pinjie_btn/triple_fish.png';//三倍鱼干
             }
             else if (this.skin == 23) {
                this.award.skin = 'pinjie_btn/triple.png';//三倍奖励
             }
             else if (this.skin == 25) {
                this.award.skin = 'pinjie_btn/quintuple.png';//五倍奖励
             } else if (this.skin == 30) {
                this.award.skin = 'pinjie_btn/tenTimes.png';//十倍奖励
             }

             else if (this.skin == 31) {
                this.award.skin = 'pinjie_btn/buchajia_ad.png';//补差价电视机
             } else if (this.skin == 32) {
                this.award.skin = 'pinjie_btn/buchajia_sh.png';//补差价分享
             }

             else if (this.skin == 33) {
                this.award.skin = 'pinjie_btn/ADdraw.png';//视频抽奖
             } else if (this.skin == 34) {
                this.award.skin = 'pinjie_btn/SHdraw.png';//分享抽奖
             }

             else if (this.skin > 1000) {
                this.award.skin = this.getVideoAdvertisingType(this.skin);
                this.animBox = this.owner.getChildByName("adbtn").getChildByName("animBox");
                if(this.animBox){
                   this.tubiao.visible = false;
                   this.anim1 = new Spine();
                   this.anim1.loadAnim("anim1/anniutexiao.sk");
                   this.anim1.playAnim("anniutexiao1",true);
                   this.anim1.pos(50,30);
                   this.anim1.scale(0.5,0.5);
                   this.animBox.addChild(this.anim1);
                   this.anim2 = new Spine();
                   this.anim2.loadAnim("anim1/anniutexiao.sk");
                   this.anim2.playAnim("anniutexiao2",true);
                   this.anim2.pos(90,35);
                   this.anim2.scale(0.5,0.5);
                   this.animBox.addChild(this.anim2);
                }
                
             }


             if (this.skin == 31 || this.skin == 32) {

             }
             else if (this.skin == 11 || this.skin == 12) {
                this.award.y = 40 - this.award.height / 2;
                this.award.x = (95 - this.award.width) / 2;
             } else {

                //设置宽度 可能需要适配
                this.owner.width = 370;
                this.owner.height = 140;
             Laya.timer.frameOnce(2,this,function(){
                this.award.y = (70 - this.award.height) / 2;
                if (this.tubiao) {
                   this.tubiao.y = (70 -  this.tubiao.height) / 2;
                   this.tubiao.x = (185 - ( this.tubiao.width + this.award.width + 5)) / 2;
                   this.award.x = this.tubiao.x +  this.tubiao.width + 5;
                } else {
                   this.award.x = (185 - this.award.width) / 2;
                }
             });
             }
          }
       }

       getVideoAdvertisingType(type){
          let num = 0;
          let skin = "";
          switch(type){
             case 1001: num = Number(Const.get("ADRewardRateLetterBox"));break;
             case 1002: num = Number(Const.get("ADRewardRateBeachCatGift"));break;
             case 1003: num = Number(Const.get("ADRewardRateTask"));break;
             case 1004: num = Number(Const.get("ADRewardLuckyBagTask"));break;
             case 1005: num = Number(Const.get("ADRewardRateMiniGame"));break;
             case 1006: num = Number(Const.get("ADRewardRateQuayAssist"));break;
             case 1007: num = Number(Const.get("ADRewardRateSail"));break;
             case 1008: num = Number(Const.get("ADRewardRateFishing"));break;
             case 1009: num = Number(Const.get("ADRewardRateBalloon"));break;
          }

          switch(num){
             case 2: skin = 'pinjie_btn/double.png';break;//双倍领取
             case 3: skin = 'pinjie_btn/triple.png';break;//三倍领取
             case 5: skin = 'pinjie_btn/quintuple.png';break;//五倍领取
             case 10: skin = 'pinjie_btn/tenTimes.png';break;//十倍领取
          }
          return skin;
       }
    }

    class AdvertisingPart extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            if (this._param == null) {
                return;
            }
            this._shareId = this._param[0];
            this._event = this._param[1];
            this._textVideo = this._param[2];
            this._textShare = this._param[3];
            this._bannnerId = this._param[4];
            this._rewardType = this._param[5];
            this._rewardNum = this._param[6];
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);


            let videoSharetext = this.getChildByName("adLabel");
            this.rewardLabel = this.getChildByName("rewardLabel");
            this.rewardImage = this.getChildByName("rewardImage");
            this.rewardNum = this.getChildByName("rewardNum");
            
            this.getDoubleV = this.getChildByName("video");
            this.getDoubleS = this.getChildByName("share");
            
                                                         // overseasGetReward
            this.overseasGetReward = this.getChildByName("overseasGetReward");
            this.overseasGetReward.visible = true;
            // this.getDirectly = this.getChildByName("oneGet");
            // this.getDirectly.text=Strings.get('tid#Btn_005');
            // this.getDirectly.visible = false;
            
            // let time = PlayerSystem.getDirectlyReceiveTime();
            // Laya.timer.once(time, this, function () {
            //     this.getDirectly.visible = true;
            // });

            // this.getDirectly.on(Laya.Event.CLICK, this, this.onGetDirectlyClick);
            this.overseasGetReward.on(Laya.Event.CLICK, this, this.onGetDirectlyClick);
            this.getDoubleS.on(Laya.Event.CLICK, this, this.onGetDoubleSClick);
            this.getDoubleV.on(Laya.Event.CLICK, this, this.onGetDoubleVClick);

            if (this._rewardType != null && this._rewardNum != null) {
                this.rewardLabel.visible = true;
                this.rewardImage.visible = true;
                this.rewardNum.visible = true;
                this.rewardNum.text = String(this._rewardNum);
                this.rewardImage.scaleX = this.rewardImage.scaleY = 2;
                if (this._rewardType == Constants.RewardType.FISH) {
                    this.rewardImage.skin = "topui/yugan.png";
                } else if (this._rewardType == Constants.RewardType.STAR) {
                    this.rewardImage.skin = "topui/xingxing.png";
                } else if (this._rewardType == Constants.RewardType.DIAMOND) {
                    this.rewardImage.skin = "topui/zuanshi.png";
                }
                this.rewardImage.x = (this.width - this.rewardLabel.width - this.rewardImage.width * this.rewardImage.scaleX - this.rewardNum.width - 10) / 2;
                this.rewardLabel.x = this.rewardImage.x +this.rewardImage.scaleX *  this.rewardImage.width + 5;
                this.rewardNum.x = this.rewardLabel.x + this.rewardLabel.width + 5;
            } else {
                this.rewardLabel.visible = false;
                this.rewardImage.visible = false;
                this.rewardNum.visible = false;
            }
            
            if (this._bannnerId != null && Laya.Browser.onMiniGame) {
                let wxWidth = wx.getSystemInfoSync().windowWidth;
                let wxHeight = wx.getSystemInfoSync().windowHeight;
                let banner_left = 0;
                let banner_width = wxWidth;
                let banner_height = 376 / Laya.stage.height * wxHeight;
                let banner_top = wxHeight - banner_height;
                SdkHelper.showBanner(this._bannnerId, banner_left, banner_top, banner_width, banner_height);
            }

            if(SdkHelper.hasVideo()) {
                this.getDoubleS.visible = false;
                this.getDoubleV.visible = true;
                videoSharetext.text = this._textVideo;
            }else {
                this.getDoubleS.visible = true;
                this.getDoubleV.visible = false;
                videoSharetext.text = this._textShare;
            }
            if(videoSharetext.text.length<14){
                videoSharetext.y=1020;
            }
            else {
                videoSharetext.y=987;
            }
        }
        onGetDirectlyClick() {
            EventMgr.getInstance().postEvent(this._event, [false]);
            this.close();
        }
        onGetDoubleSClick() {
            SdkHelper.shareAppMessage(this._shareId);
        }
        onGetDoubleVClick() {
            EventMgr.getInstance().postEvent("ADVERTISING_AD_CLICK", [this._shareId]);
            SdkHelper.showRewardVideoAd(this._shareId);
        }

        done(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, [true]);
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, ["error"]);
                this.close();
            }
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
            if (this._bannnerId != null) {
                SdkHelper.hideBanner(this._bannnerId);
            }
            Laya.timer.clearAll(this);
        }
    }

    class BargainingDataMgr{
        constructor(){
            this.loadData();
        }

        static getInstance() {
            return BargainingDataMgr.instance ? BargainingDataMgr.instance : BargainingDataMgr.instance = new BargainingDataMgr();
        }

        loadData(){
            this._itemAddLimit = 0;
            this.BarginTimesLimit = Const.get("BarginTimesLimit");
            this.BarginPercentMax = Number(Const.get("BarginPercentMax"));
            this.BarginPriceRateEach =JSON.parse(Const.get("BarginPriceRateEach")) ;
            this.BarginPriceRateFree = JSON.parse(Const.get("BarginPriceRateFree"));
        }

        beginNum(num){
            this.cut = new Array();
            this.cut.push(this.getCutData());
            this.cut.push(this.getCut1Data());
            this.sumNum = 0;
            this.nowNum = 0;
            this.oriNum = num;
            this.rate = 0;
            this.canCut = 1;
            this.residueTime = Number(this.BarginTimesLimit) + this._itemAddLimit;
        }

        getBarginTimesLimit() {
            return Number(this.BarginTimesLimit) + this._itemAddLimit;
        }

        getResidueTime(){
            return this.residueTime;
        }

        getCutData(){
            let buildingNum = BuildingDataMgr.getInstance().getNowBuildingNumber();
            for(let i = 0; i < this.BarginPriceRateEach.length; i++){
                if(buildingNum >= this.BarginPriceRateEach[i].Buildings[0] && buildingNum <= this.BarginPriceRateEach[i].Buildings[1] || this.BarginPriceRateFree[i].Buildings[1] == -1){
                    return this.BarginPriceRateEach[i].Cut;
                }
            }
        }

        getCut1Data(){
            let buildingNum = BuildingDataMgr.getInstance().getNowBuildingNumber();
            for(let i = 0; i < this.BarginPriceRateFree.length; i++){
                if(buildingNum >= this.BarginPriceRateFree[i].Buildings[0] && buildingNum <= this.BarginPriceRateFree[i].Buildings[1] || this.BarginPriceRateFree[i].Buildings[1] == -1){
                    return this.BarginPriceRateFree[i].Cut;
                }
            }
        }

        getBargaining(num){
            let r = Math.random();
            for(var i=0;i<this.cut[num].length;i++){
                if(r<this.cut[num][i].rate){
                    let ran = Math.random();
                    return this.cut[num][i].value[0] + (this.cut[num][i].value[1]-this.cut[num][i].value[0]) * ran | 0;
                }
            }
        }

        checkBargaining(num){
            let money = this.getBargaining(num);
            money = (money + this.sumNum < this.oriNum * this.BarginPercentMax) ? money : this.oriNum * this.BarginPercentMax - this.sumNum;
            this.sumNum += money;
            if(num == 0){
                this.residueTime = this.residueTime > 0 ? this.residueTime - 1 : 0;
            }
            this.rate = (this.sumNum * 100 / this.oriNum).toFixed(0);
            if(this.rate >= this.BarginPercentMax * 100){
                this.canCut = 0;
            }
            return money | 0;
        }

        checkBargainTimes(bargainTimes){
            this.residueTime = bargainTimes ? bargainTimes : 0;
        }

        getSumNum(){
            return this.sumNum;
        }

        getCanCut(){
            return this.canCut;
        }

        getRate(){
            return this.rate;
        }

        addBarginTimesLimit(times) {
            this._itemAddLimit = times;
        }
    }

    class BargainingSystem{
        constructor(){
        }

        static sendBargain(fetchType, fishAmount, buildingId){
            EventMgr.getInstance().postEvent("SERVICE_BARGAIN",[fetchType, fishAmount, buildingId, Laya.Handler.create(this, this.checkBargainTimes)]);
        }

        static setOriPrice(num){
            BargainingDataMgr.getInstance().beginNum(num);
        }

        static getMoney(style){
            return BargainingDataMgr.getInstance().checkBargaining(style);
        }

        static getCanCut(){
            return BargainingDataMgr.getInstance().getCanCut();
        }

        static getSumNum(){
            return BargainingDataMgr.getInstance().getSumNum();
        }
        
        static getRate(){
            return BargainingDataMgr.getInstance().getRate();
        }

        static getBarginTimesLimit(){
            return BargainingDataMgr.getInstance().getBarginTimesLimit();
        }

        static getResidueTime(){
            return BargainingDataMgr.getInstance().getResidueTime();
        }

        static checkBargainTimes(res){
            let bargainTimes = res.bargainTimes;
            BargainingDataMgr.getInstance().checkBargainTimes(bargainTimes);
        }

        static addBarginTimesLimit(times) {
            BargainingDataMgr.getInstance().addBarginTimesLimit(times);
        }
    }

    class Bargaining  extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            this.onClick = 1;
            this.buildingId = this._param.Id;
            let anim = new Laya.Skeleton();
            anim.load("anim1/kanbanniang.sk",null,0);
            this.anim.addChild(anim);
            let anim1 = new Laya.Skeleton();
            anim1.load("anim1/kanbanniang.sk",null,0);
            this.anim1.addChild(anim1);
            this.oriPrice.text = Strings$1.dealFishNumberText(this._param.Cost);
            this.aftPrice.text = Strings$1.dealFishNumberText(this._param.Cost);
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.finish.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.bargainingV.on(Laya.Event.CLICK,this,this.onBargainingVClick);
            this.bargainingS.on(Laya.Event.CLICK,this,this.onBargainingSClick);
            this.tip.on(Laya.Event.CLICK,this,this.onTipClick);
            this.free.on(Laya.Event.CLICK,this,this.onFreeClick);
            this.words.text = Strings$1.get("tid#UI_053");
            this.finishText.text = Strings$1.get("tid#Btn_002");
            this.freeText.text = Strings$1.get("tid#Btn_001");
            this.vText.text = Strings$1.get("tid#Btn_003");
            this.sText.text = Strings$1.get("tid#Btn_003");
            this.delLine.width = this.oriPrice.width;
            this.biaoqian.visible = false;
            //道具加成
            let items = PetSystem.getItemBySpecialType(5);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
            });
            BargainingSystem.addBarginTimesLimit(itemAddTimes);
            this.BarginTimesLimit = BargainingSystem.getBarginTimesLimit();
            this.residueTime.text = Strings$1.get("tid#UI_074") + this.BarginTimesLimit + "/" + this.BarginTimesLimit + Strings$1.get("tid#UI_075");
            this.leijikanjia.text = Strings$1.get("tid#UI_076");
            this.yuanjia.text = Strings$1.get("tid#UI_077");
            this.xianjia.text = Strings$1.get("tid#UI_078");
            this.bencikanjia.text = Strings$1.get("tid#UI_079");
            BargainingSystem.setOriPrice(this._param.Cost);
            Laya.timer.once(100,this,function(){
                anim.play("mao1-2",true);
                anim1.play("texiao",true);
            });
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
        }

        setSDKbutton(){
            if(SdkHelper.hasVideo()) {
                this.bargainingS.visible = false;
                this.bargainingV.visible = true;
            }else {
                this.bargainingS.visible = true;
                this.bargainingV.visible = false;
            }
        }

        onTipClick(){
            if(BargainingSystem.getResidueTime() <= 0){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon49");
            }
            else if(BargainingSystem.getRate() >= 70){
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon48");
            }
        }

        onFreeClick(){
            SoundManager.getInstance().play("Sound_clickSound");
            if(this.onClick){
                this.changeClick();
                this.free.visible = false;
                this.finish.visible = true;
                this.residueTime.visible = true;
                this.setSDKbutton();
                this.style = 1;
                this.fetchType = Constants.FetchType.DIRECT;
                this.playBargaining();
            }
        }

        onBargainingVClick(){
            SoundManager.getInstance().play("Sound_clickSound");
            if(this.onClick){
                this.changeClick();
                SdkHelper.showRewardVideoAd(Constants.ShareId.Bargaining);
            }
        }

        onBargainingSClick(){
            SoundManager.getInstance().play("Sound_clickSound");
            if(this.onClick){
                this.changeClick();
                SdkHelper.shareAppMessage(Constants.ShareId.Bargaining);
            }
        }

        done(shareId){
            if(shareId == Constants.ShareId.Bargaining){
                this.style = 0;
                this.fetchType = Constants.FetchType.AD;
                this.playBargaining();
            }
        }

        playBargaining(){
            this.biaoqian.visible = true;
            let money = BargainingSystem.getMoney(this.style);
            let sum = this.sumNum ? this.sumNum : 0;
            this.sumNum = BargainingSystem.getSumNum();
            let n = this.sumNum - sum;
            for(var i = 0;i<26;i++){
                let j = i;
                Laya.timer.once(30*j,this,function(){
                    this.nowPrice.text = Strings$1.dealFishNumberText(money*j/25|0);
                    this.sumPrice.text = Strings$1.dealFishNumberText(n*j/25 + sum |0);
                });
            }
            let afterPrice = this._param.Cost - this.sumNum;
            this.aftPrice.text = Strings$1.dealFishNumberText(afterPrice);
            let time = BargainingSystem.getResidueTime();
            //this.residueTime.text = "剩余：" + time + "/" + this.BarginTimesLimit + "次";
            this.residueTime.text = Strings$1.get("tid#UI_074") + time + "/" + this.BarginTimesLimit + Strings$1.get("tid#UI_075");
            if(time == 0){
                this.bargainingS.disabled = true;
                this.bargainingV.disabled = true;
                this.tip.visible = true;
            }
            let rate = BargainingSystem.getRate();
            this.rate.text = rate + "%";
            if(rate == 70){
                this.tip.visible = true;
                this.free.disabled = true;
                this.bargainingS.disabled = true;
                this.bargainingV.disabled = true;
            }
            else {
                this.rate.color = "#ffffff";
                this.off1.color = "#ffffff";
            }
            Laya.Tween.to(this.biaoqian,{scaleX:3.5,scaleY:3.5},200,null,Laya.Handler.create(this, function (){
                Laya.Tween.to(this.biaoqian,{scaleX:2.5,scaleY:2.5},200);
            }));
            this.setSDKbutton();
            BargainingSystem.sendBargain(this.fetchType, money, this.buildingId);
        }

        changeClick(){
            this.onClick = 0;
            Laya.timer.once(500,this,function(){
                this.onClick = 1;
            });
        }

        putBuilding(){
            Laya.Dialog.closeAll();
            if(GameContext.CurrentShopBuildingBase){
                BuildingSystem.placeBuilding(this.buildingId, GameContext.CurrentShopBuildingBase.getId(), true);
                SoundManager.getInstance().play("Sound_placeBuilding");
            }else {
                GameContext.PlacingBuilding = this.buildingId;
                GameContext.PlacingNewBuilding = true;
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                EventMgr.getInstance().postEvent("PLACE_BUILDING", this.buildingId);
                SoundManager.getInstance().play("Sound_clickSound");
            }
        }

        onCloseClick(){
            this.close();
            EventMgr.getInstance().postEvent("CHANGE_SHOP_MASK",0);
            this.putBuilding();
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
        }
    }

    class BowlAdPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            if (this._param == null) {
                return;
            }
            this._shareId = this._param[0];
            this._event = this._param[1];
            this._textVideo = this._param[2];
            this._textShare = this._param[3];
            this._bannnerId = this._param[4];
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);


            let videoSharetext = this.getChildByName("adLabel");
            this.getDoubleV = this.getChildByName("video");
            this.piaoBtn = this.getChildByName("piaoBtn");
            this.piaoBtn.on(Laya.Event.CLICK, this, this.useLiangpiao);
            // this.getDirectly = this.getChildByName("oneGet");
            // this.getDirectly.visible = false;
            this.overseasGetReward = this.getChildByName("overseasGetReward");
            this.overseasGetReward.visible = true;

            this.overseasGetReward.on(Laya.Event.CLICK, this, this.onGetDirectlyClick);
            // this.getDirectly.on(Laya.Event.CLICK, this, this.onGetDirectlyClick);
            this.getDoubleV.on(Laya.Event.CLICK, this, this.onGetDoubleVClick);

            if (this._bannnerId != null && Laya.Browser.onMiniGame) {
                let wxWidth = wx.getSystemInfoSync().windowWidth;
                let wxHeight = wx.getSystemInfoSync().windowHeight;
                let banner_left = 0;
                let banner_width = wxWidth;
                let banner_height = 376 / Laya.stage.height * wxHeight;
                let banner_top = wxHeight - banner_height;
                SdkHelper.showBanner(this._bannnerId, banner_left, banner_top, banner_width, banner_height);
            }

            if (SdkHelper.hasVideo()) {
                this.getDoubleV.visible = true;
                videoSharetext.text = this._textVideo;
            } else {
                this.getDoubleV.visible = false;
                videoSharetext.text = this._textShare;
            }
            if (GuideSystem.isGuiding() && GuideSystem.getConditionGroupComplete("GuideGroup_9")) {
                let guide = GuideSystem.getGuide("GuideGroup_9");
                if (guide) {
                    guide.registerObj("btn1", this);
                    let bg = this.getChildByName("bg");
                    let btn = this.overseasGetReward;
                    btn.on(Laya.Event.CLICK, null, function () {
                        EventMgr.getInstance().postEvent("GUIDE8_ADD_FOOD");
                        EventMgr.getInstance().postEvent("GUIDE_GUIDE8_AD");
                        EventMgr.getInstance().postEvent("DESTORY_VIDEO");
                    }); 
                    this.getDoubleV.visible = true;
                    guide.registerClick("btn1", btn, null, [bg, videoSharetext, this.getDoubleV, btn]);
                    this.getChildByName("close").visible = false;      
                }
            } else {
                this.getChildByName("close").visible = true;
                this.overseasGetReward.visible = true;
                // Laya.timer.once(1500, this, function () {
                // this.getDirectly.visible = true;
                // })
            }
            //屏蔽
            // this.subscriptionBox = this.getChildByName("subscriptionBox");
            // this.subscriptionBox.setValue(Constants.WxMessageTmplIdKeys.CAT_FOOD,Strings.get("tid#WXMessageCatFoood"));
            // this.getDirectly.text = Strings.get("tid#Btn_035");

            //粮票
            this.liangpiao = this.getChildByName('liangpiao');
            this.piaoNum = this.getChildByName('piaoNum');
            this.liangpiaoup();

        }
        onGetDirectlyClick() {
            EventMgr.getInstance().postEvent(this._event, [false]);
            this.close();
        }
        onGetDoubleSClick() {
            SdkHelper.shareAppMessage(this._shareId);
        }
        onGetDoubleVClick() {
            EventMgr.getInstance().postEvent("ADVERTISING_AD_CLICK", [this._shareId]);
            SdkHelper.showRewardVideoAd(this._shareId);
        }

        done(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, [true]);
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, ["error"]);
                this.close();
            }
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
            if (this._bannnerId != null) {
                SdkHelper.hideBanner(this._bannnerId);
            }
            Laya.timer.clearAll(this);
        }
        useLiangpiao() {
            let handler = Laya.Handler.create(this, this.liangPiaohandler);
            EventMgr.getInstance().postEvent("SERVICE_USE_AD_COUPON", [0, handler]);
        }
        liangpiaoup() {
            let piaozi = Number(PaySystem.getCatFoodNum());
            this.piaoNum.text ="X"+ piaozi;
            if (piaozi > 0) {
                this.liangpiao.visible = true;
                this.piaoNum.visible = true;
                this.piaoBtn.visible = true;
                this.getDoubleV.visible = false;
            } else {
                this.liangpiao.visible = false;
                this.piaoNum.visible = false;
                this.piaoBtn.visible = false;
                this.getDoubleV.visible = true;
            }
        }
        liangPiaohandler() {
            this.liangpiaoup();
            this.done(this._shareId);
        }
    }

    class SubscriptionBox extends Laya.Box {
        constructor() {
            super();
            this.visible = false;
        }

        onEnable() {
            let clickBox = this.getChildByName("clickBox");
            clickBox.on(Laya.Event.CLICK, this, function () {
                let hook = this.getChildByName("hook");
                hook.visible = !hook.visible;
                SdkHelper.requestSubscribeMessage([this.key]);
                this.visible = false;
            });
        }

        setTip(str) {
            let tip = this.getChildByName("tip");
            tip.text = str;
            let checkbox = this.getChildByName("checkbox");
            let hook = this.getChildByName("hook");
            let clickBox = this.getChildByName("clickBox");
            clickBox.width = checkbox.width * checkbox.scaleX + tip.width;
            clickBox.x = checkbox.x = hook.x = (this.width - clickBox.width) / 2;
            tip.x = checkbox.x + checkbox.width * checkbox.scaleX;
        }

        setValue(key, str) {
            let handler = Laya.Handler.create(this, function (res) {
                this.subscriptionType = res;
                if (!res) {
                    this.setTip(str);
                    this.visible = true;
                }
            });
            this.key = key;
            SdkHelper.checkSubscriptions(key, handler);
        }

        getSubscriptionsType() {
            let hook = this.getChildByName("hook");
            return hook.visible || this.subscriptionType;
        }
    }

    class BuildingCouponPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onAwake() {
            this.buildingPosArr = [659, 916, 1170];
            this.tip = this.getChildByName("tip");
            this.tip1 = this.getChildByName("tip1");
            this.closeBtn = this.getChildByName("closeBtn");
            this.closeBtn.on(Laya.Event.CLICK, this, this.end);
            this.closeBtn.label = Strings$1.get("tid#Btn_002");
            this.buildings = [this.getChildByName("building1"), this.getChildByName("building2"), this.getChildByName("building3")];
        }
        onEnable() {
            EventMgr.getInstance().registEvent("COUPON_CHOOSE_BUILD", this, this.chooseOne);
            EventMgr.getInstance().registEvent("COUPON_BUILD_DISENABLED", this, this.offBuildings);
            this.tip.text = Strings$1.get("tid#UI_225");
            this.tip1.text = Strings$1.get("tid#UI_228");
            this.tip1.visible = false;
            this.refresh();
            this.showReward();
        }

        onDisable() {
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("COUPON_CHOOSE_BUILD", this, this.chooseOne);
            EventMgr.getInstance().removeEvent("COUPON_BUILD_DISENABLED", this, this.offBuildings);
        }

        chooseOne(ticaketId) {
            let ticketBuildingInfos = BuildingSystem.getTicketBuildingInfos();
            let ticaketData = ticketBuildingInfos[ticaketId];
            this.showStar = ticaketData.star ? true : false;
            this.closeBtn.visible = true;
            BuildingSystem.buyBuildingByTicket(ticaketData.buildingId, ticaketId + 1, Laya.Handler.create(this, this.refresh));
        }

        end() {
            EventMgr.getInstance().postEvent("SERVICE_FINISH_BUILDING_COUPON");
            Laya.LocalStorage.removeItem("building_ticket_pos");
            this.close();
        }

        refresh() {
            let ticketBuildingInfos = BuildingSystem.getTicketBuildingInfos();
            if (ticketBuildingInfos.length == 0) {
                this.closeBtn.label = Strings$1.get("tid#Btn_009");
                Laya.timer.once(2000, this, this.end);
                return;
            }
            let ad = false;
            for (let index = 0; index < this.buildings.length; index++) {
                this.buildings[index].setItem(ticketBuildingInfos[index], index);
                ad = ticketBuildingInfos[index].alreadyGet || ad;
            }
            if (ad) {
                this.tip1.visible = this.showStar;
                this.tip.text = SdkHelper.hasVideo() ? Strings$1.get("tid#UI_226") : Strings$1.get("tid#UI_227");
                this.buildings.forEach(build => {
                    build.chooseOne();
                    build.setEnabled(true);
                });
            }
            this.closeBtn.visible = ad;
        }

        showReward() {
            let posInedxs = Laya.LocalStorage.getJSON("building_ticket_pos");
            if (posInedxs != null && posInedxs.length == 3 && this.closeBtn.visible) {
                for (let i = 0; i < this.buildings.length; i++) {
                    this.buildings[i].hideReward();
                    this.buildings[i].setEnabled(true);
                    this.buildings[i].x = this.buildingPosArr[posInedxs[i]];
                }
                return;
            }
            this.buildings.forEach(building => {
                building.showReward();
            });
            Laya.timer.once(1500, this, this.randomPos);
        }

        randomPos() {
            let arr = [0, 1, 2];
            let arr1 = [arr[0]];
            let arr2 = [arr[1]];
            let arr3 = [arr[2]];
            let indexs = [arr1, arr2, arr3];
            let index0 = 0;
            let index1 = 1;
            let index2 = 2;
            let a = 0;
            for (let i = 0; i < 9; i++) {
                index0 = Utils.randomExt(3);
                index1 = (index0 + 1) % 3;
                index2 = (index0 + 2) % 3;
                a = arr[index1];
                arr[index1] = arr[index2];
                arr[index2] = a;
                arr1.push(arr[0]);
                arr2.push(arr[1]);
                arr3.push(arr[2]);
            }
            let posInedxs = arr;
            let time = 200;
            for (let index = 0; index < this.buildings.length; index++) {
                const building = this.buildings[index];
                building.hideReward();
                let array = indexs[index];
                for (let i = 0; i < array.length; i++) {
                    const xx = this.buildingPosArr[array[i]];
                    Laya.Tween.to(building, { x: xx }, time, null, null, i * time);
                }
            }
            Laya.timer.once(time * 9, this, function () {
                Laya.LocalStorage.setJSON("building_ticket_pos", posInedxs);
                this.buildings.forEach(building => {
                    building.setEnabled(true);
                });
            });
        }

        offBuildings() {
            this.buildings.forEach(building => {
                building.setEnabled(false);
            });
        }
    }

    class BuildingCouponItem extends Laya.Box {
        constructor() {
            super();
        }

        onAwake() {
            this.buildBox = this.getChildByName("buildBox");
            this.starBox = this.getChildByName("starBox");
            this.luckyBox = this.getChildByName("luckyBox");
            this.ad = this.luckyBox.getChildByName("ad");
            this.luckyBox.on(Laya.Event.CLICK, this, this.onClick);
            this.luckyBox.mouseEnabled = false;
        }

        onEnable() {
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdDone);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onAdDone);
        }

        setItem(data, ticketIndex) {
            this._data = data ? data : this._data;
            this._ticketIndex = ticketIndex;
            this.hideReward();
        }

        onClick() {
            if (this._adShow) {
                EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdDone);
                EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onAdDone);
                SdkHelper.showRewardVideoAd(Constants.ShareId.BuildingCoupon);
                return;
            }
            this.showAnim();
        }

        setEnabled(v) {
            this.luckyBox.mouseEnabled = v;
        }

        chooseOne() {
            this._adShow = true;
            this.ad.visible = true;
        }

        onAdDone(shareId) {
            if (shareId != Constants.ShareId.BuildingCoupon) {
                return;
            }
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdDone);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onAdDone);
            this.showAnim();
        }

        getOne() {
            EventMgr.getInstance().postEvent("COUPON_CHOOSE_BUILD", this._ticketIndex);
        }

        showReward() {
            this.luckyBox.visible = false;
            this.starBox.visible = this._data.star && this._data.star != 0 ? true : false;
            this.buildBox.visible = this._data.buildingId && this._data.buildingId != "" ? true : false;
        }

        hideReward() {
            this.luckyBox.visible = !this._data.alreadyGet;
            if (this._data.buildingId != null && this._data.buildingId != "") {
                let item = ConfigMgr$1.getInstance().getItemById("Building", this._data.buildingId);
                let building = this.buildBox.getChildByName("building");
                building.skin = item.ShopItems;
                let fishNum = this.buildBox.getChildByName("fishNum");
                fishNum.text = Strings$1.dealFishNumberText(item.Cost, 1);
                this.buildBox.visible = this._data.alreadyGet;
            } else {
                let num = this.starBox.getChildByName("num");
                num.text = this._data.star;
                this.starBox.visible = this._data.alreadyGet;
            }
        }

        showAnim() {
            EventMgr.getInstance().postEvent("COUPON_BUILD_DISENABLED");
            this.luckyBox.offAll(Laya.Event.CLICK);
            let time = 200;
            Laya.Tween.to(this, { scaleX: 0 }, time, null, Laya.Handler.create(this, function () {
                this.luckyBox.visible = false;
                this.starBox.visible = this._data.star && this._data.star != 0 ? true : false;
                this.buildBox.visible = this._data.buildingId && this._data.buildingId != "" ? true : false;
            }));
            Laya.Tween.to(this, { scaleX: 1 }, time, null, null, time);
            let anim = new Spine();
            anim.loadAnim("anim3/wanshiwu.sk");
            anim.viewport = false;
            this.addChild(anim);
            anim.zOrder = 2;
            anim.visible = false;
            anim.x = this.width / 2;
            anim.scaleX = anim.scaleY = 2;
            anim.y = this.height / 2;
            anim.addCompleteHandler(Laya.Handler.create(this, function () {
                this.getOne();
                anim.destroy();
            }));
            Laya.timer.once(time * 3, this, function () {
                anim.visible = true;
                anim.playAnim("texiao", false);
            });
        }

    }

    class Camera {
        constructor() {

        }
        //destWidth 所要保存的目标图片宽度
        //destHeight 所要保存的目标图片高度
        //返回值，临时图片的路径
        //小程序关闭时会释放图片
        static screenshot(destWidth, destHeight, destX = 0, destY = 0, baseSrite = null, succeeeHandler = null, failHandler = null) {
            if (Laya.Browser.onMiniGame) {
                let w = 0;
                let h = 0;
                if (destWidth == null || destWidth == undefined) {
                    w = Laya.stage.width;
                } else {
                    w = destWidth;
                }
                if (destHeight == null || destHeight == undefined) {
                    h = Laya.stage.height;
                } else {
                    h = destHeight;
                }
                var canvas = Laya.Browser.window.canvas;
                // let tempFilePath = canvas.toTempFilePathSync({
                //     x: destX,
                //     y: destY,
                //     width: w,
                //     height: h,
                //     destWidth: w,
                //     destHeight: h,
                //     fileType: 'png',
                //     quality: 1.0
                // });
                canvas.toTempFilePath({
                    x: destX,
                    y: destY,
                    width: w,
                    height: h,
                    destWidth: w,
                    destHeight: h,
                    fileType: 'png',
                    quality: 1.0,
                    success: (res) => {
                        if (succeeeHandler) {
                            succeeeHandler.runWith(res);
                        }
                    },
                    fail: (res) => {
                        if (failHandler) {
                            failHandler.runWith(res);
                        }
                        console.log(res);
                    }
                });
                // return tempFilePath;
            } else {
                let w = 0;
                let h = 0;
                if (destWidth == null || destWidth == undefined) {
                    w = Laya.stage.width;
                } else {
                    w = destWidth;
                }
                if (destHeight == null || destHeight == undefined) {
                    h = Laya.stage.height;
                } else {
                    h = destHeight;
                }
                if (baseSrite == null) {
                    baseSrite = Laya.stage;
                }
                let htmlC = baseSrite.drawToCanvas(w, h, 0 - destX, 0 - destY);
                let url = htmlC.toBase64("png", 1);
                return url;
            }
        }

        //tempFilePath 想要保存图片的路径
        static saveIamgeToPhotosAlbum(tempFilePath, obj, failCallback) {
            if (Laya.Browser.onMiniGame) {
                wx.getSetting({
                    success(res) {
                        if (!res.authSetting['scope.writePhotosAlbum']) {
                            wx.authorize({
                                scope: 'scope.writePhotosAlbum',
                                success: function () {
                                    Laya.Browser.window.wx.saveImageToPhotosAlbum({
                                        filePath: tempFilePath,
                                        success(res) {
                                        },
                                        fail() {
                                        }
                                    });
                                },
                                fail: function () {
                                    //没有授权过，我们调用showModal弹出弹窗，告诉用户请求给予我们授权。
                                    var alertParam = {
                                        'title': '微信授权',
                                        'content': '保存到相册需要您的授权，请给予我们授权',
                                        'showCancel': true,
                                        'canelColor': '#666',
                                        'confirmText': '去授权',
                                        'confirmColor': '#666',
                                        success: function (ssa) {
                                            //打开设置界面，设置界面只会出现小程序已经向用户请求过的权限。我们请求过保存图片的权限，所以里面会有相册的按钮，允许后点返回。则执行下面的success方法。调用保存图片的接口
                                            if (ssa.confirm == true) {
                                                Laya.Browser.window.wx.openSetting({
                                                    complete: function () {
                                                        wx.getSetting({
                                                            success(res) {
                                                                if (res.authSetting['scope.writePhotosAlbum']) {
                                                                    Laya.Browser.window.wx.saveImageToPhotosAlbum({
                                                                        filePath: tempFilePath,
                                                                        success(res) {
                                                                        },
                                                                        fail() {
                                                                        }
                                                                    });
                                                                } else {
                                                                    failCallback.bind(obj)();
                                                                }
                                                            }
                                                        });
                                                    }
                                                });
                                            } else {
                                                failCallback.bind(obj)();
                                            }
                                        },
                                    };
                                    //这里调用弹窗。
                                    wx.showModal(alertParam);
                                },
                            });
                        } else {
                            Laya.Browser.window.wx.saveImageToPhotosAlbum({
                                filePath: tempFilePath,
                                success(res) {
                                },
                                fail() {
                                }
                            });
                        }
                    }
                });
            } else {
                console.debug("此平台不支持相册");
            }
        }

        /** destWidth:destHeight=5:4 */
        static shareScreenshot(destWidth, destHeight, destX = 0, destY = 0, shareTitle, shareId, completeHandler) {
            if (Laya.Browser.onMiniGame) {
                let handler = Laya.Handler.create(null, function (res) {
                    SdkHelper.shareAppMessage(shareId, shareTitle, res.tempFilePath);
                    if (completeHandler) {
                        completeHandler.run();
                    }
                });
                let failHandler = Laya.Handler.create(null, function () {
                    SdkHelper.shareAppMessage(shareId);
                    if (completeHandler) {
                        completeHandler.run();
                    }
                });
                Camera.screenshot(destWidth, destHeight, destX, destY, null, handler, failHandler);
            }
        }

    }

    class CameraUI extends Laya.Dialog {

        constructor() {
            super();
        }

        onAwake() {
            if (GameContext.Sliding) {
                this.back();
                return;
            }
            this.popupEffect = null;
            this.isShowEffect = false;
            this.closeEffect = false;
            this.bgTouch = this.getChildByName("bgTouch");
            this.homeCameraSprite = this.getChildByName("homeCameraSprite");
            this.homeShotBtn = this.homeCameraSprite.getChildByName("homeShot");
            this.btnBox = this.getChildByName("btnBox");
            this.bcakBtn = this.getChildByName("bcak");
            this.cameraBtn = this.btnBox.getChildByName("cameraBtn");
            this.screenshotBtn = this.btnBox.getChildByName("screenshotBtn");
            this.homeCameraBtn = this.btnBox.getChildByName("homeCameraBtn");
            this.homeCameraBtn.selected = false;
            this.cameraBox = this.getChildByName("cameraBox");
            this.photoShow = this.getChildByName("photoShow");
            this.photoShowBgBox = this.photoShow.getChildByName("bgBox");
            this.catBox = this.photoShowBgBox.getChildByName("catBox");
            this.photoBg = this.photoShow.getChildByName("photoBg");
            this.photo = this.photoShow.getChildByName("photo");
            this.shareBtn = this.photoShow.getChildByName("share");
            this.shareBtn.visible = true;
            this.cameraPhotoShowBox = this.getChildByName("cameraPhotoShow");
            this.cameraPhoto = this.cameraPhotoShowBox.getChildByName("photo");
            this.cameraDate = this.cameraPhotoShowBox.getChildByName("date");
            this.btnBox.visible = false;
            this.bcakBtn.visible = false;
            let starty = 185;
            let ySpace = (this.catBox.height - 350) / 4;
            let xSpace = (this.catBox.width - 400) / 2;
            let startx = 120;
            let startx1 = 320;
            for (let i = 0; i < 5; i++) {
                let n = 3;
                for (let j = 0; j < n; j++) {
                    let image = new Laya.Image("camera/cameraBg1.png");
                    image.width = 348;
                    image.height = 210;
                    image.pivotX = 174;
                    image.pivotY = 105;
                    image.x = i % 2 == 0 ? startx + xSpace * j : startx1 + xSpace * j;
                    image.y = starty;
                    this.catBox.addChild(image);
                }
                starty += ySpace;
            }
            this.bcakBtn.on(Laya.Event.CLICK, this, this.back);
            this.cameraBtn.on(Laya.Event.CLICK, this, this.camera);
            this.screenshotBtn.on(Laya.Event.CLICK, this, this.screenshot);
            this.homeCameraBtn.on(Laya.Event.CLICK, this, this.homeCamera);
            this.cameraBox.on(Laya.Event.CLICK, this, this.snapshot);
            this.cameraBox.on(Laya.Event.MOUSE_DOWN, this, this.startMove);
            this.homeShotBtn.on(Laya.Event.CLICK, this, this.homeShot);
            this.shareBtn.on(Laya.Event.CLICK, this, this.share);
            this.limitPoint1 = new Laya.Point(0, 0);
            this.limitPoint2 = new Laya.Point(Laya.stage.width, Laya.stage.height);
            this.limitPoint1 = this.globalToLocal(this.limitPoint1);
            this.limitPoint2 = this.globalToLocal(this.limitPoint2);
            this.limitPoint2.x -= this.cameraBox.width * this.cameraBox.scaleX;
            this.limitPoint2.y -= this.cameraBox.height * this.cameraBox.scaleY;
            this.timer = new Laya.Timer();
            if (Laya.Browser.onMiniGame) {
                let obj = this;
                wx.onShow(function () {
                    obj.back();
                });
            }
        }

        onEnable() {
            if (GameContext.Sliding) {
                this.back();
                return;
            }
            if(!Laya.LocalStorage.getJSON('nofirstPhoto')){
                Laya.LocalStorage.setJSON('nofirstPhoto',1);
            }
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Photo,1);
            EventMgr.getInstance().postEvent("CLEAR_ALL_BARRAGES");
            this._disabled = false;
            this.mouseEnabled = true;
            this.bgTouch.visible = true;
            this.cameraBox.visible = true;
            this.photoShow.visible = false;
            this.homeCameraSprite.visible = false;
            this.tempFilePath = null;
            this.cameraPhotoShowBox.visible = false;
            this.cameraPhoto.skin = null;
            if (SceneMgr.getInstance().getCurrentSceneId() == Constants.HomelandId) {
                this.homeCameraBtn.visible = false;
            } else {
                this.homeCameraBtn.visible = false;
            }
            EventMgr.getInstance().postEvent("CAMERA_TYPE_SWITCH", [true]);
            TaskMainSystem.setTaskMainVisible(false);
            Laya.timer.once(1, this, this.stopAll);
        }

        onDisable() {
            this._disabled = true;
            EventMgr.getInstance().postEvent("RESTART_BARRAGES");
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Photo,0);
        }

        stopAll() {
            let dialogNum = Laya.Dialog.manager.numChildren;
            if (this.destroyed || this._disabled || GameContext.Sliding || dialogNum > 1) {
                this.back();
                return;
            }
            this.btnBox.visible = true;
            this.bcakBtn.visible = true;
            // Laya.stage.renderingEnabled = false;
            if (SceneMgr.getInstance().getCurrentSceneId() == Constants.FishingSceneId) {
                this.back();
            }
            Laya.timer.pause();
        }

        back() {
            // Laya.stage.renderingEnabled = true;
            if (this.currentSprite && this.currentSpriteParent) {
                this.currentSprite.scaleX = this.currentSprite.scaleY = 1;
                this.currentSprite.x = this.currentSprite.y = 0;
                this.currentSpriteParent.addChild(this.currentSprite);
                this.currentSprite.mouseEnabled = true;
                this.currentSprite = this.currentSpriteParent = null;
            }
            Laya.timer.resume();
            EventMgr.getInstance().postEvent("CAMERA_TYPE_SWITCH", [false]);
            TaskMainSystem.setTaskMainVisible(true);
            this.close();
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [1]);
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [1]);
        }

        homeCamera() {
            if (this.homeCameraBtn.selected) {
                // this.bgTouch.visible = false;
                this.homeCameraSprite.visible = true;
            } else {
                this.bgTouch.visible = true;
                this.homeCameraSprite.visible = false;
            }
        }
        camera() {
            if (PhotoSystem.getUserPhotoNum() >= 20) {
                TipsPopMgr.getInstance().showTips("照片数量达到上限");
            }
            this.bgTouch.visible = true;
            this.cameraBox.visible = true;
        }

        //全屏截图
        screenshot() {
            this.homeCameraBtn.selected = false;
            this.bgTouch.visible = true;
            this.homeCameraSprite.visible = false;
            this.shareBtn.visible = false;
            this.shareBtn.y = 1935;
            this.tempFilePath = null;
            this.btnBox.visible = false;
            this.bcakBtn.visible = false;
            EventMgr.getInstance().postEvent('ZHUAPAICLICK',[1]);
            this.timer.frameOnce(5, this, function () {
                Camera.screenshot(Laya.stage.width, Laya.stage.height, 0, 0, null, shotSuccessHandele);
            });
            let shotSuccessHandele = Laya.Handler.create(this, function (res) {
                Camera.saveIamgeToPhotosAlbum(res.tempFilePath, this, this.back);
                this.timer.frameOnce(5, this, function () {
                    let height = Laya.stage.width / 5 * 4;
                    let y = (Laya.stage.height - height) / 2;
                    let successHandler = Laya.Handler.create(this, function (res) {
                        this.tempFilePath = res.tempFilePath;
                        this.shareBtn.visible = true;
                        this.timer.frameOnce(5, this, this.screeenshotShow);
                    });
                    let failHandler = Laya.Handler.create(this, function () {
                        this.tempFilePath = null;
                        this.shareBtn.visible = true;
                        this.screeenshotShow();

                    });
                    Camera.screenshot(Laya.stage.width, height, 0, y, null, successHandler, failHandler);
                });
            });
        }

        screeenshotShow() {
            if (this.destroyed || this._disabled) {
                return;
            }
            this.bcakBtn.visible = true;
            this.photoBg.width = Laya.stage.width / this.photoBg.scaleX;
            this.photoBg.height = Laya.stage.height / this.photoBg.scaleY;
            this.photoShow.visible = true;
            this.photoShowBgBox.visible = false;
            let currentSceneId = SceneMgr.getInstance().getCurrentSceneId();
            this.currentSprite = SceneMgr.getInstance().getScene(currentSceneId);
            this.photo.height = Laya.stage.height - 185;
            this.currentSpriteParent = this.currentSprite.parent;
            this.currentSprite.scaleX = this.currentSprite.scaleY = this.photo.height / this.currentSprite.height;
            this.photo.width = Laya.stage.width * this.currentSprite.scaleX;
            this.currentSprite.x = this.currentSprite.x * this.currentSprite.scaleX;
            this.currentSprite.mouseEnabled = false;
            this.currentSprite.anchorX = 0;
            this.currentSprite.pivotX = 0;
            this.photo.addChild(this.currentSprite);
        }

        //抓拍不可用
        snapshot() {
            this.cameraBox.visible = false;
            let point = new Laya.Point(this.cameraBox.x, this.cameraBox.y);
            let rightDownpoint = new Laya.Point(this.cameraBox.x + this.cameraBox.width * this.cameraBox.scaleX, this.cameraBox.y + this.cameraBox.height * this.cameraBox.scaleY);
            rightDownpoint = this.cameraBox.parent.localToGlobal(rightDownpoint);
            point = this.cameraBox.parent.localToGlobal(point);
            this.mouseEnabled = false;
            this.timer.frameOnce(5, this, function () {
                Laya.timer.resume();
                this.tempFilePath = Camera.screenshot(rightDownpoint.x - point.x, rightDownpoint.y - point.y, point.x, point.y);
                PhotoSystem.pushPhotoData(this.tempFilePath, 1);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_picture_take"]);
                this.snapShow();
                this.cameraPhoto.skin = this.tempFilePath;
            });
        }

        snapShow() {
            this.cameraDate.text = Strings$1.getTimeByUTCMilliseconds(TimeUtils.getTimeNowOnServer());
            this.cameraPhotoShowBox.x = this.cameraBox.x + this.cameraBox.width * this.cameraBox.scaleX / 2;
            this.cameraPhotoShowBox.y = this.cameraBox.y + this.cameraBox.height * this.cameraBox.scaleY / 2;
            this.cameraPhotoShowBox.scaleX = 1;
            this.cameraPhotoShowBox.scaleY = 1;
            this.cameraPhotoShowBox.visible = true;
            if(SceneMgr.getInstance().getCurrentSceneId()==Constants.HomelandId){
                Laya.Tween.to(this.cameraPhotoShowBox, { x: this.btnBox.x + this.cameraBtn.x + this.cameraBtn.width * this.cameraBtn.scaleX / 2, y: this.btnBox.y + this.cameraBtn.y + this.cameraBtn.height * this.cameraBtn.scaleY / 2, scaleX: 0, scaleY: 0 },
                    600, null, Laya.Handler.create(this, function () {
                        this.back();
                        // TipsPopMgr.getInstance().showTipsById("tid#TipsCommon8");
                    }));
            }else {
                Laya.Tween.to(this.cameraPhotoShowBox, { x: this.btnBox.x + this.cameraBtn.x + this.cameraBtn.width * this.cameraBtn.scaleX / 2-400, y: this.btnBox.y + this.cameraBtn.y + this.cameraBtn.height * this.cameraBtn.scaleY / 2, scaleX: 0, scaleY: 0 },
                    600, null, Laya.Handler.create(this, function () {
                        this.back();
                        // TipsPopMgr.getInstance().showTipsById("tid#TipsCommon8");
                    }));
            }
       
        }

        //家园截图
        homeShot() {
            this.btnBox.visible = false;
            this.bcakBtn.visible = false;
            this.homeCameraSprite.visible = false;
            let baseSprite = SceneMgr.getInstance().getScene(Constants.HomelandId).map_touch;
            this.photo.width = 1040 - 46 * 2;
            this.photo.height = this.photo.width / baseSprite.width * 2436;
            this.photoBg.width = 1040 / this.photoBg.scaleX;
            this.photoBg.height = (this.photo.height + 284) / this.photoBg.scaleY;
            this.photoShowBgBox.visible = true;
            this.photoShow.visible = true;
            this.currentSprite = baseSprite;
            this.currentSprite.x = this.currentSprite.y = 0;
            this.currentSpriteParent = this.currentSprite.parent;
            this.currentSprite.scaleY = this.currentSprite.scaleX = this.photo.height / this.currentSprite.height;
            this.currentSprite.mouseEnabled = false;
            this.photo.addChild(this.currentSprite);
            this.shareBtn.visible = false;
            this.shareBtn.y = 1650;
            this.tempFilePath = null;
            if (Laya.Browser.onMiniGame) {
                this.timer.frameOnce(5, this, function () {
                    let point = new Laya.Point(this.currentSprite.x, this.currentSprite.y);
                    let rightDownpoint = new Laya.Point(this.currentSprite.x + this.currentSprite.width * this.currentSprite.scaleX, this.currentSprite.y + this.currentSprite.height * this.currentSprite.scaleY);
                    point.x += 17;
                    point.y += 10;
                    rightDownpoint.x -= 22;
                    rightDownpoint.y -= 10;
                    rightDownpoint = this.currentSprite.parent.localToGlobal(rightDownpoint);
                    point = this.currentSprite.parent.localToGlobal(point);
                    let handler = Laya.Handler.create(this, function (res) {
                        this.tempFilePath = res.tempFilePath;
                        Camera.saveIamgeToPhotosAlbum(this.tempFilePath, this, this.back);
                        this.shareBtn.visible = true;
                    });
                    let failHandler = Laya.Handler.create(this, function () {
                        this.tempFilePath = null;
                        this.shareBtn.visible = true;
                    });
                    Camera.screenshot(rightDownpoint.x - point.x, rightDownpoint.y - point.y, point.x, point.y, null, handler, failHandler);
                    this.bcakBtn.visible = true;
                });
            } else {
                this.tempFilePath = Camera.screenshot(baseSprite.width, Laya.stage.height, 0, 0, baseSprite);
            }
        }

        share() {
            this.bgTouch.visible = true;
            this.photoShow.visible = false;
            this.close();
            if (Laya.Browser.onMiniGame) {
                if (this.tempFilePath == null) {
                    SdkHelper.shareAppMessage(4, null, null, null, false);
                } else {
                    let str = Strings$1.get("tid#ShareTips6");
                    SdkHelper.shareAppMessage(4, str, this.tempFilePath, null, false);
                }
            }
        }

        startMove() {
            this.bgTouch.on(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
            this.bgTouch.on(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            if (this.mousePoint == null) {
                this.mousePoint = new Laya.Point(Laya.stage.mouseX, Laya.stage.mouseY);
            }
            this.mousePoint.x = Laya.stage.mouseX;
            this.mousePoint.y = Laya.stage.mouseY;
            this.mousePoint = this.globalToLocal(this.mousePoint);
            if (this.offsetPoint == null) {
                this.offsetPoint = new Laya.Point(this.mousePoint.x - this.cameraBox.x, this.mousePoint.y - this.cameraBox.y);
            } else {
                this.offsetPoint.x = this.mousePoint.x - this.cameraBox.x;
                this.offsetPoint.y = this.mousePoint.y - this.cameraBox.y;
            }
        }

        onMouseMove() {
            this.cameraBox.mouseEnabled = false;
            this.mousePoint.x = Laya.stage.mouseX;
            this.mousePoint.y = Laya.stage.mouseY;
            this.mousePoint = this.globalToLocal(this.mousePoint);
            let x = this.mousePoint.x - this.offsetPoint.x;
            let y = this.mousePoint.y - this.offsetPoint.y;
            x = Math.max(this.limitPoint1.x, x);
            x = Math.min(this.limitPoint2.x, x);
            y = Math.max(this.limitPoint1.y, y);
            y = Math.min(this.limitPoint2.y, y);
            this.cameraBox.x = x;
            this.cameraBox.y = y;
        }

        onMouseUp() {
            this.bgTouch.off(Laya.Event.MOUSE_MOVE, this, this.onMouseMove);
            this.bgTouch.off(Laya.Event.MOUSE_UP, this, this.onMouseUp);
            this.cameraBox.mouseEnabled = true;
        }

        onDestroy() {
            Laya.timer.clearAll(this);
        }
    }

    class ButtonChangeBg extends Laya.Script {

        constructor() {
           super();
           this.skin = 0;
           /** @prop {name:skin,type:int}*/
        }

        onStart() {
            this.bg = this.owner.getChildByName("bg");
            this.bg.on(Laya.Event.MOUSE_DOWN,this,this.mouseDown);
            this.bg.on(Laya.Event.MOUSE_UP,this,this.mouseUp);
            this.bg.on(Laya.Event.MOUSE_OUT,this,this.mouseUp);
        }

        mouseDown(){
            if(this.skin == 0){
                this.bg.skin = "pinjie_btn_2/diban_2.png";
            }
        }
         
        mouseUp(){
            if(this.skin == 0){
                this.bg.skin = "pinjie_btn_2/diban_1.png";
            }
        }
    }

    class CatArchives extends Laya.Dialog {
        constructor() {
            super();
        }


        onEnable(){
            this.lockClickSign = 1;

            this.close1.on(Laya.Event.CLICK, this, this.close);
            this.btn1.on(Laya.Event.CLICK, this, this.openStory,[1]);
            this.btn2.on(Laya.Event.CLICK, this, this.openStory,[2]);
            this.btn3.on(Laya.Event.CLICK, this, this.openStory,[3]);
            this.lockBtn.on(Laya.Event.CLICK, this, this.playUnlock);
            this.help.on(Laya.Event.MOUSE_DOWN, this, this.onHelpClick);
            this.help.on(Laya.Event.MOUSE_UP, this, this.offHelpClick);
            this.help.on(Laya.Event.MOUSE_OUT, this, this.offHelpClick);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.like1bg.visible=false;
            this.like2bg.visible=false;
            this.like3bg.visible=false;
            this.lock1.visible=false;
            this.lock2.visible=false;
            this.lock3.visible=false;
            this.story.visible=false;
            this.lock.visible=false;
            this.unlock.visible=false;
            this.helpImg.visible = false;
            this.helpText.visible = false;
            this.helpText.text = Strings$1.get("tid#UI_034");
            this.lockAnim=new Spine();
            this.lockAnim.loadAnim("anim2/jiesuodonghua.sk");
            this.lockAnim.playAnim("jiesuo1",true);
            this.lockAnim.x=this.lockBtn.width/2;
            this.lockAnim.y=this.lockBtn.height/2;
            this.lockBtn.addChild( this.lockAnim);
            this.redPoint1.visible = false;
            this.redPoint2.visible = false;
            this.redPoint3.visible = false;
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
            this.xingming.text = Strings$1.get("tid#UI_080");
            this.shenfen.text = Strings$1.get("tid#UI_081");
            this.wanju.text = Strings$1.get("tid#UI_082");
            this.shiwu.text = Strings$1.get("tid#UI_083");
            this.btnText1.text = Strings$1.get("tid#UI_084");
            this.btnText2.text = Strings$1.get("tid#UI_085");
            this.btnText3.text = Strings$1.get("tid#UI_086");
            this.lockjiesuo.text = Strings$1.get("tid#UI_087");
            this.unlockjiesuo.text = Strings$1.get("tid#UI_087");
            this.jiesuo1.text = "to " + Strings$1.get("tid#UI_088");
            this.jiesuo2.text = "to " + Strings$1.get("tid#UI_088");
            this.jiesuo3.text = "to " + Strings$1.get("tid#UI_088");
            this.unlock1.text = Strings$1.get("tid#UI_093");
            this.unlock2.text = Strings$1.get("tid#UI_093");
            this.unlock3.text = Strings$1.get("tid#UI_093");
        }

        onOpened(data){
            this.catId=data.id;
            this.loadBaseData();
            this.loadLockData();
            this.openStory(1);
            // this.showFavorBuilding();
        }

        onDisable(){
            this.lockAnim.destroy();
        }

        loadBaseData(){
            this.petData=PetSystem.getPetInfo(this.catId);
            this.name.text=Strings$1.get("tid#catName_"+this.catId);
            this.catImg.skin = "cat_collect/normal/" + this.petData.config.Icon + ".png";
            this.toyid=JSON.parse(this.petData.config.FavorToys)[0].NeedToyId;
            this.toy.text=Strings$1.get(ConfigMgr$1.getInstance().getItemById("CatToys", this.toyid).Name);
            this.intimacyGrade =this.petData.getIntimacyGrade();
            this.basicInfor=JSON.parse(ConfigMgr$1.getInstance().getItemById("Role", this.catId).BasicInfor);
            // this.favorBuilding=ConfigMgr.getInstance().getItemById("Role", this.catId).FavorBuilding;
            this.identity.text=Strings$1.get(this.basicInfor.Identity);
            this.food.text=Strings$1.get(this.basicInfor.favorfood);
            this.character1.skin=this.basicInfor.Personality[0];
            this.character2.skin=this.basicInfor.Personality[1];
            this.dataStatus=1;
        }

        // showFavorBuilding(){
        //     let buildingId = new Array();
        //     for(let i=0;i<this.favorBuilding.length;i++){
        //         if(buildingId.length==3){
        //             break;
        //         }
        //         if(BuildingSystem.hasBuildingDataByBid(this.favorBuilding[i])){
        //             buildingId.push(this.favorBuilding[i]);
        //         }
        //     }
        //     this.lock1.visible = true;
        //     this.lock2.visible = true;
        //     this.lock3.visible = true;
        //     this.like1bg.visible = false;
        //     this.like2bg.visible = false;
        //     this.like3bg.visible = false;
        //     if(buildingId.length>0){
        //         switch(buildingId.length){
        //             case 3:
        //                 this.like3.skin=BuildingSystem.getBuildingDataByBid(buildingId[2]).config.ShopItems;
        //                 this.like3bg.visible = true;
        //                 this.lock3.visible = false;
        //             case 2:
        //                 this.like2.skin=BuildingSystem.getBuildingDataByBid(buildingId[1]).config.ShopItems;
        //                 this.like2bg.visible = true;
        //                 this.lock2.visible = false;
        //             case 1:
        //                 this.like1.skin=BuildingSystem.getBuildingDataByBid(buildingId[0]).config.ShopItems;
        //                 this.like1bg.visible = true;
        //                 this.lock1.visible = false;
        //         }
        //     }
        // }

        onHelpClick(){
            this.helpImg.visible = true;
            this.helpText.visible = true;
        }

        offHelpClick(){
            this.helpImg.visible = false;
            this.helpText.visible = false;
        }

        onShareClick(){
            SdkHelper.shareAppMessage(Constants.ShareId.CatArchives);
        }

        doneS(shareId) {
            if(shareId == Constants.ShareId.CatArchives) {
                //this.getReward(Constants.FetchType.AD);
                this.close();
            }
        }

        fail(shareId){
            if (shareId == Constants.ShareId.CatArchives) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["catArchives_shareId_fail"]);
            }
        }

        openStory(num){
            num=1;
            if(this.dataStatus && this.dataStatus != num){
                this.storyPanel._vScrollBar.stopScroll();
                this.storyPanel._vScrollBar.value = 0;
            }
            this.lockClickSign = 1;
            switch(num){
                case 1:
                    this.btn1.skin="collect/storytap2.png";this.btnText1.color="#b97084";
                    this.btn2.skin="collect/storytap1.png";this.btnText2.color="#926757";
                    this.btn3.skin="collect/storytap1.png";this.btnText3.color="#926757";
                    this.unlock1.visible = false;
                    this.unlock2.visible = false;
                    this.unlock3.visible = false;
                    break;
                case 2:
                    this.btn1.skin="collect/storytap1.png";this.btnText1.color="#926757";
                    this.btn2.skin="collect/storytap2.png";this.btnText2.color="#b97084";
                    this.btn3.skin="collect/storytap1.png";this.btnText3.color="#926757";
                    this.unlock1.visible = false;
                    this.unlock2.visible = true;
                    this.unlock3.visible = false;
                    break;
                case 3:
                    this.btn1.skin="collect/storytap1.png";this.btnText1.color="#926757";
                    this.btn2.skin="collect/storytap1.png";this.btnText2.color="#926757";
                    this.btn3.skin="collect/storytap2.png";this.btnText3.color="#b97084";
                    this.unlock1.visible = false;
                    this.unlock2.visible = false;
                    this.unlock3.visible = true;
                    break;
            }
            this.dataStatus = num;
            // this.setRedPoint(num);
            // if(this.intimacyGrade<num){
            //     this.unlock.visible = true;
            //     this.lock.visible = false;
            //     this.story.visible = false;
            //     this.story.text = "";
            //     this.unlock.text = this.StoryUnlockReward[this.dataStatus-1];
            // }else if(this.storyUnlock[this.dataStatus-1]==0){
            //     this.unlock.visible = false;
            //     this.lock.visible = true;
            //     this.lockAnim.playAnim("jiesuo1",true);
            //     this.story.visible = false;
            //     this.story.text = "";
            //     this.lock.text = this.StoryUnlockReward[this.dataStatus-1];
            // }else{
                this.unlock.visible = false;
                this.lock.visible = false;
                this.story.visible = true;
                this.story.text = Strings$1.get(this.basicInfor.Stories[this.dataStatus-1]).replace(new RegExp("ger","gm"),"       ");
           // }
        }

        setRedPoint(num){
            this.redPoint1.visible = false;
            this.redPoint2.visible = false;
            this.redPoint3.visible = false;
            switch(this.intimacyGrade){
                case 3: 
                    if(this.storyUnlock[2]==0 && num!=3){
                        //this.redPoint3.visible = true;
                    }
                case 2: 
                    if(this.storyUnlock[1]==0 && num!=2){
                      //  this.redPoint2.visible = true;
                    }
                case 1:
                    if(this.storyUnlock[0]==0 && num!=1){
                       // this.redPoint1.visible = true;
                    }
            }
        }

        loadLockData(){
            this.storyUnlock= new Array();
            for(let i=0;i<0;i++){
                if(this.petData.storyUnlock.hasOwnProperty(i+1)){
                    this.storyUnlock[i]=1;
                }
                else {
                    this.storyUnlock[i]=0;
                }
            }
            this.StoryUnlockReward = JSON.parse(Const.get("StoryUnlockReward"));
        }

        playUnlock(){
            if(this.lockClickSign > 0){
                this.lockClickSign = 0;
                this.lockAnim.clearCompleteHandler();
                let handler=Laya.Handler.create(this,this.showStory);
                this.lockAnim.addCompleteHandler(handler);
                this.lockAnim.playAnim("jiesuo2",false);
                this.storyLock(this.dataStatus);
            }
        }

        storyLock(dataStatus) {
            if (this.destroyed||this.dataStatus!=dataStatus) {
                return;
            }
            let handler=Laya.Handler.create(this,this.flyReward);
            EventMgr.getInstance().postEvent("SERVICE_STORY_LOCK", [this.catId,dataStatus,handler]);
        }

        showStory(){
            this.unlock.visible = false;
            this.lock.visible = false;
            this.story.visible = true;   
        }

        flyReward(rewards){
            this.story.text = Strings$1.get(this.basicInfor.Stories[this.dataStatus-1]).replace(new RegExp("ger","gm"),"       ");
            ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR,rewards, Constants.FlyType.STENHANCE,Laya.stage.width/2,Laya.stage.height/2+250);
            this.storyUnlock[this.dataStatus-1]=1;
            EventMgr.getInstance().postEvent("SET_CATARCHIVES_REDPOINT",this.catId);
            EventMgr.getInstance().postEvent("UPDATE_PETS");
        }
    }

    class RedPoint extends Laya.Sprite {

        constructor() {
            super();
            this.otherRed = null;       //这个红点关联的其他红点，例如task的会关联到updown
        }

        setCheckFunc(func) {
            this.checkFunc = func;
        }

        refreshRedPoint(data) {
            if(this.checkFunc) {
                let ret = this.checkFunc();//用handler总是在这行莫名报错，所以换成了func
                this.visible = ret?true:false;
                if(this.otherRed != null) {
                    EventMgr.getInstance().postEvent("REDPOINT_REFRESH", [this.otherRed]);
                }
            }
        }

        onStart() {
            this.visible = false;
        }

    }

    class RedPointType extends Laya.Script {

        constructor() {
            super();
            /** @prop {name:redPointType1, tips:"红点注册类型", type:int, default:-1}*/
            this.redPointType1 = -1;
            /** @prop {name:redPointType2, tips:"红点注册类型", type:String, default:null}*/
            this.redPointType2 = null;
            /** @prop {name:otherRed, tips:"其他关联红点", type:String, default:null}*/
            this.otherRed = null;
        }

        onEnable() {
            if(this.redPointType1!=-1)
                RedPointSystem.registerObject(this.redPointType1, this.owner);
            if(this.redPointType2!=null&&this.redPointType2!="null")
                RedPointSystem.registerObject(this.redPointType2, this.owner);
            if(this.otherRed!=null&&this.otherRed!="null")
                this.owner.otherRed = this.otherRed;
        }

        onDisable() {
            if(this.redPointType1!=-1)
                RedPointSystem.unregisterObject(this.redPointType1, this.owner);
            if(this.redPointType2!=null&&this.redPointType2!="null")
                RedPointSystem.unregisterObject(this.redPointType2, this.owner);
        }
    }

    class CatNewCome extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable(){
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.CatNewcome,1);
            this.petId = this._param;
            this.petData = PetSystem.getPetInfo(this.petId);
            this.name.text=Strings$1.get("tid#catName_"+this.petId);
            this.catImg.skin="cat_collect/normal/"+this.petData.config.Icon+".png";
            this.basicInfor =JSON.parse(this.petData.config.BasicInfor);
            console.log(this.basicInfor.Personality);
            this.character1.skin =this.basicInfor.Personality[0];
            this.character2.skin =this.basicInfor.Personality[1];
            this.words.text = "    " + Strings$1.get("tid#UI_043");
            this.close1.on(Laya.Event.CLICK,this,function(){
                this.close();
            });
        }

        // onShareClick(){
        //     SdkHelper.shareAppMessage(Constants.ShareId.CatNewCome);
        // }

        // doneS(shareId) {
        //     if(shareId == Constants.ShareId.CatNewCome) {
        //         let handler=Laya.Handler.create(this,this.flyYugan);
        //         EventMgr.getInstance().postEvent("SERVICE_PET_SHAREGETREWARD",[this.petId,Constants.PeacockShareType.CatNewCome,handler] );
        //         this.close();
        //     }
        // }

        // flyYugan(res){
        //     let num = res.rewards.rewards[0].amount;
        //     let rewardType = res.rewards.rewards[0].id;
        //     let x = Laya.stage.width/2;
        //     let y = Laya.stage.height/2;
        //     var duoyugan = new Spine();
        //     duoyugan.zOrder = 100000;
        //     Laya.stage.addChild(duoyugan);
        //     ItemFlyMgr.getInstance().fly(rewardType, num,Constants.FlyType.LW,x-33, y-24);
        // }
        onDisable(){
            // EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.CatNewcome,0);
        }

    }

    /**
    * 收藏教程
    */
    class CollectionTutorialPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            if (this._param == null) {
                this.close();
                return;
            }
            this._type = this._param[0];
            this.collect = this.getChildByName("collect");
            if (this._param[1] == "fromTip") {
                this.collect.label = "知道了";
                this.collect.label = Strings$1.get("tid#Btn_012");
                this._showTip = null;
            } else {
                this._showTip = true;
                this.collect.label = "去收藏";
                this.collect.label = Strings$1.get("tid#Btn_013");
            }
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCollectClick);
            this.lesson = this.getChildByName("lesson");
            this.next = this.getChildByName("next");
            this.next.on(Laya.Event.CLICK, this, this.nextLesson);
            this.collect.on(Laya.Event.CLICK, this, this.onCollectClick);
            this.title = this.getChildByName("title");
        }

        onEnable() {
            this.chooseBtn(this._type);
        }

        onDisable() {

        }

        chooseBtn(btnId) {
            this._type = btnId;
            this.lesson.skin = "bg1/lesson_" + btnId + "_1.jpg";
            this.next.visible = true;
            this.collect.visible = false;
            if (btnId == 3) {
                this.title.text = "添加到桌面";
                this.title.text = Strings$1.get("tid#UI_089");
            } else if (btnId == 2) {
                this.title.text = "添加到我的小程序";
                this.title.text = Strings$1.get("tid#UI_090");
            } else {
                this.title.text = "添加到浮窗";
                this.title.text = Strings$1.get("tid#UI_091");
            }
        }

        onCloseClick() {
            this.close();
        }

        nextLesson() {
            this.lesson.skin = "bg1/lesson_" + this._type + "_2.jpg";
            this.next.visible = false;
            this.collect.visible = true;
        }

        onCollectClick() {
            this.close();
            if (this._showTip) {
                EventMgr.getInstance().postEvent("RIGHT_UI_SHOW_COLLECTION_TIP", [this._type]);
            }
        }
    }

    class CollectPop extends Laya.Dialog {
        constructor() {
            super();
            GameContext.showPetId = "";
        }

        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI, 1);
            this.show = this.getChildByName("show");
            this.text = this.getChildByName("text");
            this.text.text = "???";
            this.furniture = this.getChildByName("furniture");
            this.button = this.getChildByName("adopt");
            this.button.visible = false;
            this.button.on(Laya.Event.CLICK, this, this.onAdoptPet);
            // TaskMainUIMgr.getInstance().registButton("Click_Adopt", this.button);
            this.number = this.getChildByName("number");
            this.numberQinmi = this.getChildByName("numberQinmi");
            this.qinmiImage = this.getChildByName("qinmiImage");
            this.number1 = this.getChildByName("number1");
            this.number2 = this.number1.getChildByName("number2");
            this.point = this.number1.getChildByName("point");
            this.baifenhao = this.number1.getChildByName("baifenhao");
            this.array1 = new Array(this.number1.x, this.number2.x, this.point.x, this.baifenhao.x);
            this.array2 = new Array(this.number1.x - 30, this.number2.x - 25, this.point.x - 27, this.baifenhao.x - 25);
            this.array3 = new Array(this.number1.x + 30, this.number2.x + 30, this.point.x + 25, this.baifenhao.x + 30);
            this.showVisitTimes(0);
            this.heartBox = this.getChildByName("heartBox");
            this.heart1 = this.heartBox.getChildByName("heart1").getChildByName("red");
            this.heart2 = this.heartBox.getChildByName("heart2").getChildByName("red");
            this.heart3 = this.heartBox.getChildByName("heart3").getChildByName("red");
            this.showIntimacyGrade(0);

            this.visitTimesText = this.getChildByName("visitTimes");
            this.adoptPre = this.getChildByName("adoptPre");
            this.adoptNo = this.getChildByName("adoptNo");
            this.adoptPre.on(Laya.Event.CLICK, this, this.onAdoptPre);

            this.close_show = this.getChildByName("close_show");
            this.close_show.visible = Constants.dev;

            this.closeBtn = this.getChildByName("close");
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);

            this.catArchivesBtn = this.getChildByName("catArchives");
            this.catArchivesBtn.on(Laya.Event.CLICK, this, this.onCatArchivesBtnClick);
            this.catArchivesRedPoint = this.catArchivesBtn.getChildByName("redPoint");

            this.findDefaultShow();

            let config = ConfigMgr$1.getInstance().getConfig("Role");
            let array = new Array();
            config.forEach((listConfig, petId) => {
                array.push(petId);
            });
            function sortByOrder(a, b) {
                return config.get(a).Rank - config.get(b).Rank;
            }
            array.sort(sortByOrder);

            this.list.array = array;
            this.list.scrollBar.on(Laya.Event.END, this, this.onSelect);
            this.list.on(Laya.Event.RENDER, this, this.onSelect);
            Laya.timer.frameOnce(1, this, function () {
                for (let i = 0; i < this.list.array.length; i++) {
                    let canadopt = PetSystem.getPetInfo(this.list.array[i]).canAdopt();
                    if (canadopt) {
                        this.canguideid = this.list.array[i];
                        break;
                    }
                }
                for (const cell of this.list.cells) {
                    if (!this.canguideid) {
                        break;
                    }
                    if (!cell.dataSource) {
                        break;
                    }
                    let canadopt = PetSystem.getPetInfo(cell.dataSource).canAdopt();
                    if (cell.dataSource == this.canguideid) {
                        if (cell.dataSource == "Cat1") {
                            this.onSelectBtn();
                        } else {
                            this.onSelectCell(cell);
                        }
                        cell.on(Laya.Event.CLICK, this,this.onSelectBtn);
                        this.canguidecell = cell;
                    } else {
                        if (canadopt) {
                            cell.on(Laya.Event.CLICK, this, this.onSelectBtn);
                        } else {
                            cell.on(Laya.Event.CLICK, this,this.onselectDouble); 
                        }
                    }
                }
            });
            this._nauticalInfo = NauticalSystem$1.NauticalInfo;

            this.confirmAdoptSignal = 1;
            this.adoptNo.on(Laya.Event.CLICK, this, function () {
                if (this.starNum < this.catAdoptStars) {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon72", [this.catAdoptStars]);
                } else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon87", [this.catFacorsLimit, this.visitLimit]);
                }
            });
            EventMgr.getInstance().registEvent("UPDATE_PETS", this, this.updatePets);
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().registEvent("UPDATE_UNLOCKNUM", this, this.showUnlockNum);
            EventMgr.getInstance().registEvent("PRE_ADOPT", this, this.confirmAdopt);
            EventMgr.getInstance().registEvent("SET_CATARCHIVES_REDPOINT", this, this.showCatArchivesRedPoint);
            EventMgr.getInstance().registEvent("COLLECTPOP_SHOW_PET", this, this.showPet);
            EventMgr.getInstance().registEvent("COLLECTPOP_CHANGE_SHOW_PET", this, this.changeShowPet);
            //this.adoptNo.label = Strings.get("tid#Btn_014");
            //this.adoptPre.label = Strings.get("tid#Btn_015");
            //this.button.label = Strings.get("tid#Btn_016");
            this.questionLabel.text = Strings$1.get("tid#UI_225");
            this.daoptTxt.text = Strings$1.get("tid#UI_224");
            this.desc.on(Laya.Event.MOUSE_DOWN, this, this.onQuestionBtnClick);
            this.desc.on(Laya.Event.MOUSE_UP, this, this.onQuestionBtnUp);
            this.desc.on(Laya.Event.MOUSE_OUT, this, this.onQuestionBtnUp);
        }
        onSelectBtn() {
            TaskMainUIMgr.getInstance().destroyBtnAnim();
            TaskMainUIMgr.getInstance().registButton("Click_Adopt", this.button);
            this.btnclick = 1;
            this.selcetCell = false;
        }
        onSelectCell(cell) {
            if (!this.selcetCell) {
                TaskMainUIMgr.getInstance().destroyBtnAnim();
                TaskMainUIMgr.getInstance().registButton("Click_Adopt", cell);
                this.canguidecell = cell;
                this.btnclick = 0;
                this.selcetCell = true;
            }
        }
        onselectDouble(){
            this.btnclick = 0;
            if (this.canguidecell) {
                if (this.canguidecell.dataSource == this.canguideid) {
                    this.onSelectCell(this.canguidecell);
                }
            }
            else {
                TaskMainUIMgr.getInstance().destroyBtnAnim();
                this.selcetCell = false;
            }
        }
        onSelect() {
            if (!this.canguideid) {
                return;
            }
            for (const cell of this.list.cells) {
                if (!cell.dataSource) {
                    continue;
                }
                let canadopt = PetSystem.getPetInfo(cell.dataSource).canAdopt();
                if (!this.canguideid) {
                    break;
                }
                if (cell.dataSource == this.canguideid) {
                    if (!this.canguidecell) {
                        this.onSelectCell(cell);
                        this.canguidecell = cell;
                    }
                        cell.off(Laya.Event.CLICK, this,  this.onSelectCell);
                        cell.off(Laya.Event.CLICK, this,  this.onselectDouble);
                        cell.on(Laya.Event.CLICK, this,  this.onSelectBtn);
                   
                } else {
                    if (canadopt) {
                        cell.off(Laya.Event.CLICK, this,  this.onSelectCell);
                        cell.off(Laya.Event.CLICK, this,  this.onselectDouble);
                        cell.on(Laya.Event.CLICK, this, this.onSelectBtn);
                    } else {
                        cell.off(Laya.Event.CLICK, this,  this.onSelectCell);
                        cell.off(Laya.Event.CLICK, this,  this.onSelectBtn);
                        cell.on(Laya.Event.CLICK, this, this.onselectDouble);
                    }
                }
            }
            if (!this.canguidecell) {
                return;
            }
            if (this.btnclick == 0) {
                if (this.canguidecell.dataSource == this.canguideid) {
                    this.onSelectCell(this.canguidecell);
                }
                else {
                    TaskMainUIMgr.getInstance().destroyBtnAnim();
                    this.selcetCell = false;
                }
            }
                // this.addWeakGuideAnim(this.list, this.list.width - 100);
                // if (this.weakGuideAnim.getCurAnim() != "tuodong2") {
                //     this.weakGuideAnim.playAnim("tuodong2", true);
                // }
                // this.addWeakGuideAnim(this.list, this.list.width - 100);
                // if (this.weakGuideAnim.getCurAnim() != "tuodong1") {
                //     this.weakGuideAnim.playAnim("tuodong1", true);
                // }
                
     
                // this.removeWeakGuideAnim();
            
        }
        addWeakGuideAnim(node, x, y) {
            if (this.weakGuideAnim == null || this.weakGuideAnim.destroyed) {
                this.weakGuideAnim = new Spine();
                this.weakGuideAnim.loadAnim("anim1/xinshouyindao.sk");
            }
            if (this.weakGuideAnim.parent == null || this.weakGuideAnim.parent != node) {
                this.weakGuideAnim.removeSelf();
                if (x == null) {
                    this.weakGuideAnim.x = node.width / 2;
                } else {
                    this.weakGuideAnim.x = x;
                }
                if (y == null) {
                    this.weakGuideAnim.y = node.height / 2;
                } else {
                    this.weakGuideAnim.y = y;
                }
                this.weakGuideAnim.scaleX = 1 / node.scaleX;
                this.weakGuideAnim.scaleY = 1 / node.scaleY;
                node.addChild(this.weakGuideAnim);
            }
        }
        onQuestionBtnClick() {
            this.questionBox.visible = true;
        }

        onQuestionBtnUp() {
            this.questionBox.visible = false;
        }
        showUnlockNum() {
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            let Catnum = 0;
            let CatUnlock = 0;
            config.forEach((listConfig, petId) => {
                Catnum++;
                if (PetSystem.getPetInfo(petId).visitTimes > 0) {
                    CatUnlock++;
                }
            });
            let haveUnlock = CatUnlock * 1000 / Catnum;
            this.number1.value = haveUnlock / 10 | 0;
            this.number2.value = haveUnlock % 10 | 0;
            if (haveUnlock == 1000) {
                this.number1.x = this.array3[0];
                this.number2.x = this.array3[1];
                this.point.x = this.array3[2];
                this.baifenhao.x = this.array3[3];
            }
            else if (haveUnlock >= 100) {
                this.number1.x = this.array1[0];
                this.number2.x = this.array1[1];
                this.point.x = this.array1[2];
                this.baifenhao.x = this.array1[3];
            }
            else {
                this.number1.x = this.array2[0];
                this.number2.x = this.array2[1];
                this.point.x = this.array2[2];
                this.baifenhao.x = this.array2[3];
            }
        }


        findDefaultShow() {
            if (GameContext.showPetId != "") return;

            let config = ConfigMgr$1.getInstance().getConfig("Role");
            for (const [petId, listConfig] of config) {
                let petData = PetSystem.getPetInfo(petId);
                if (petData != null && petData.getVisitTimes() > 0) {
                    GameContext.showPetId = petId;
                    break;
                }
            }
        }

        //查找指定petId的cell
        findTargetCell(showPetId) {
            for (const cell of this.list.cells) {
                if (cell.dataSource == showPetId) {
                    return cell;
                }
            }
            return null;
        }

        onCloseClick() {
            //close 前清除当前选中的item的redPointStatus
            let petData = PetSystem.getPetInfo(GameContext.showPetId);
            if (petData != null) {
                petData.changeRedPointRead();
            }
            // PetSystem.clearAllRedPoint();
            //close前把所有的 petData 的 newCome 清除
            PetSystem.clearAllNewCome();
            EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["collect"]);
            EventMgr.getInstance().postEvent("CLICK_ADOPT");
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [1]);
        }

        onDisable() {
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("UPDATE_PETS", this, this.updatePets);
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("UPDATE_UNLOCKNUM", this, this.showUnlockNum);
            EventMgr.getInstance().removeEvent("PRE_ADOPT", this, this.confirmAdopt);
            EventMgr.getInstance().removeEvent("SET_CATARCHIVES_REDPOINT", this, this.showCatArchivesRedPoint);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI, 0);
            EventMgr.getInstance().removeEvent("COLLECTPOP_SHOW_PET", this, this.showPet);
            EventMgr.getInstance().removeEvent("COLLECTPOP_CHANGE_SHOW_PET", this, this.changeShowPet);
        }

        onCatArchivesBtnClick() {
            Laya.Dialog.close("ui/CatArchives.scene");
            Laya.Dialog.open("ui/CatArchives.scene", false, { id: GameContext.showPetId });
        }

        updatePets() {
            this.list.refresh();
        }

        showCatArchivesRedPoint(petId) {
            let pet = PetSystem.getPetInfo(petId);
            this.catArchivesRedPoint.visible = false;
            if (pet.visitTimes < 1) {
                return;
            }
            let red = 0;
            for (let i = 0; i < 0; i++) {
                if (pet.storyUnlock && !pet.storyUnlock.hasOwnProperty(i + 1) && pet.getIntimacyGrade() > i) {
                    red = 1;
                }
            }
            if (red == 1) {
                this.catArchivesRedPoint.visible = true;
            }
        }

        //展示新的猫咪，传入的pet是PetData类型
        showPet(pet) {
            this.showCatArchivesRedPoint(pet.configId);
            if (pet.adopted) {
                this.catArchivesBtn.visible = true;
            }
            else {
                this.catArchivesBtn.visible = false;
            }
            //切换动画
            if (this.cat != null) {
                //先把原来的暂停了并销毁，再创建新的
                this.cat.stop();
                this.cat.destroy(true);
            }

            //切换记录的展示猫咪、名字、是否可领取、造访次数、心数等信息
            let visitTimes = pet.getVisitTimes();
            let favor = pet.getFavor();
            let adoptLimit = JSON.parse(pet.config.AdoptLimit);
            let visitLimit = adoptLimit.VisitTimes;
            this.visitLimit = visitLimit ? visitLimit : 0;
            let catFacorsLimit = adoptLimit.CatFavors;
            this.catFacorsLimit = catFacorsLimit ? catFacorsLimit : 0;
            let needToyId = adoptLimit.CatToys;
            let hasToy = true;
            if (needToyId) {
                hasToy = PetSystem.hasToy(needToyId);
            }
            let CatAdoptADCondition = Const.get("CatAdoptADCondition");
            let CatAdoptADFavorsCondition = Const.get("CatAdoptADFavorsCondition");
            let starNum = PlayerSystem.getStar();
            this.starNum = starNum;
            let catAdoptStars = Const.get("CatAdoptStars");
            this.catAdoptStars = catAdoptStars;
            if (pet.getAdopted()) {
                this.furniture.visible = true;
                this.heartBox.visible = true;
                this.numberQinmi.visible = true;
                this.qinmiImage.visible = true;
                let intimacyGrade = pet.getIntimacyGrade();
                this.showIntimacyGrade(intimacyGrade);
                this.button.visible = false;
                this.adoptNo.visible = false;
                this.adoptPre.visible = false;
                this.visitTimesText.visible = false;
                this.showCat(pet);
                this.visitNnum.visible = false;
                this.visitNew.visible = false;
                this.xiaolian.visible = false;
                this.probg.visible = false;
                this.xiaolianPro.visible = false;
                this.xiaolianNume.visible = false;
                this.visitsImg.visible = true;
                this.number.visible = true;
                this.textbg.visible = true;
                this.text.visible = true;
                this.desc.visible = false;
            } else {
                this.desc.visible = true;
                this.textbg.visible = false;
                this.text.visible = false;
                this.visitNnum.visible = true;
                this.visitNew.visible = true;
                this.xiaolian.visible = true;
                this.probg.visible = true;
                this.xiaolianPro.visible = true;
                this.xiaolianNume.visible = true;
                if (Number(favor) < 100) {
                    this.xiaolianNume.value = favor;
                } else {
                    this.xiaolianNume.value = 100;
                }

                this.visitsImg.visible = false;
                this.number.visible = false;
                if (catFacorsLimit) {
                    if (favor >= catFacorsLimit) {
                        this.xiaolianNume.skin = "collect/numberNew.png";
                        this.xiaolianPro.value = 1;
                    } else {
                        this.xiaolianNume.skin = "collect/numberred.png";
                        this.xiaolianPro.value = Number(favor) / Number(catFacorsLimit);
                    }
                }
                else {
                    this.xiaolianNume.skin = "collect/numberNew.png";
                    this.xiaolianPro.value = 1;
                }
                if (visitLimit) {
                    if (visitTimes >= visitLimit) {
                        this.visitNnum.skin = "collect/numberNew.png";
                        this.visitNnum.value = visitTimes + "/" + visitLimit;
                    } else {
                        this.visitNnum.skin = "collect/numberred.png";
                        this.visitNnum.value = visitTimes + "/" + visitLimit;
                    }
                } else {
                    this.visitNnum.skin = "collect/numberNew.png";
                    this.visitNnum.value = visitTimes;
                }
                if (visitTimes > 0) {
                    this.showCat(pet);
                    this.furniture.visible = false;
                    this.heartBox.visible = false;
                    this.numberQinmi.visible = false;
                    this.qinmiImage.visible = false;
                    // this.visitTimesText.text=Strings.get("tid#UI_092");+visitTimes+"/"+visitLimit+Strings.get("tid#UI_075");
                    if (starNum < catAdoptStars) {
                        this.button.visible = false;
                        this.adoptNo.visible = true;
                        this.adoptPre.visible = false;
                        this.visitTimesText.visible = false;
                    } else {
                        if (visitLimit) {
                            this.visitTimesText.text = "来访" + visitTimes + "/" + visitLimit + "次";
                            this.visitTimesText.text = Strings$1.get("tid#UI_092") + visitTimes + "/" + visitLimit;
                            if (visitTimes >= visitLimit) {
                                if (hasToy) {
                                    if (catFacorsLimit) {
                                        if (favor >= catFacorsLimit) {
                                            this.button.visible = true;
                                            this.visitTimesText.visible = false;
                                            this.adoptNo.visible = false;
                                            this.adoptPre.visible = false;
                                        }
                                        else if (favor < catFacorsLimit && favor >= catFacorsLimit * CatAdoptADFavorsCondition) {
                                            this.button.visible = false;
                                            this.visitTimesText.visible = false;
                                            this.adoptNo.visible = false;
                                            this.adoptPre.visible = true;
                                        }
                                        else {
                                            this.visitTimesText.visible = false;
                                            this.button.visible = false;
                                            this.adoptNo.visible = true;
                                            this.adoptPre.visible = false;
                                        }

                                    } else {
                                        this.button.visible = true;
                                        this.visitTimesText.visible = false;
                                        this.adoptNo.visible = false;
                                        this.adoptPre.visible = false;
                                    }
                                } else {
                                    let config = ConfigMgr$1.getInstance().getItemById("CatToys", String(needToyId));
                                    this.visitTimesText.text = Strings$1.get("tid#UI_164", [Strings$1.get(config.Name)]);
                                    this.visitTimesText.visible = false;
                                    this.button.visible = false;
                                    this.adoptNo.visible = true;
                                    this.adoptPre.visible = false;
                                }
                            } else if (visitTimes >= visitLimit * CatAdoptADCondition) {
                                if (hasToy) {
                                    if (catFacorsLimit) {
                                        if (favor >= catFacorsLimit * CatAdoptADFavorsCondition) {
                                            this.visitTimesText.visible = false;
                                            this.button.visible = false;
                                            this.adoptNo.visible = false;
                                            this.adoptPre.visible = true;
                                        }
                                        else {
                                            this.visitTimesText.visible = false;
                                            this.button.visible = false;
                                            this.adoptNo.visible = true;
                                            this.adoptPre.visible = false;
                                        }

                                    } else {
                                        this.visitTimesText.visible = false;
                                        this.button.visible = false;
                                        this.adoptNo.visible = false;
                                        this.adoptPre.visible = true;
                                    }
                                } else {
                                    let config = ConfigMgr$1.getInstance().getItemById("CatToys", String(needToyId));
                                    this.visitTimesText.text = Strings$1.get("tid#UI_164", [Strings$1.get(config.Name)]);
                                    this.visitTimesText.visible = false;
                                    this.button.visible = false;
                                    this.adoptNo.visible = true;
                                    this.adoptPre.visible = false;
                                }
                            }
                            else {
                                this.visitTimesText.visible = false;
                                this.button.visible = false;
                                this.adoptNo.visible = true;
                                this.adoptPre.visible = false;
                            }
                        } else {
                            if (hasToy) {
                                this.button.visible = true;
                                this.visitTimesText.visible = false;
                                this.adoptNo.visible = false;
                                this.adoptPre.visible = false;
                            } else {
                                let config = ConfigMgr$1.getInstance().getItemById("CatToys", String(needToyId));
                                this.visitTimesText.text = Strings$1.get("tid#UI_164", [Strings$1.get(config.Name)]);
                                this.visitTimesText.visible = false;
                                this.button.visible = false;
                                this.adoptNo.visible = true;
                                this.adoptPre.visible = false;
                            }
                        }
                    }
                } else {
                    this.button.visible = false;
                    this.text.text = "???";
                    this.furniture.visible = false;
                    this.heartBox.visible = false;
                    this.adoptNo.visible = false;
                    this.adoptPre.visible = false;
                    this.visitTimesText.visible = false;
                    this.traveling.visible = false;
                    this.catArchivesBtn.visible = false;
                    this.numberQinmi.visible = false;
                    this.qinmiImage.visible = false;


                    this.visitNnum.visible = false;
                    this.visitNew.visible = false;
                    this.xiaolian.visible = false;
                    this.probg.visible = false;
                    this.xiaolianPro.visible = false;
                    this.xiaolianNume.visible = false;
                    this.desc.visible = false;
                }
            }
            let button_redPoint = this.button.getChildByName("redPoint");
            if (pet.needRedPoint()) {
                button_redPoint.visible = true;
            } else {
                button_redPoint.visible = false;
            }
            //这里对访问次数的展示做一个限制，最大值展示99999
            if (visitTimes > 99999) visitTimes = 99999;
            this.showVisitTimes(visitTimes);

            this.close_show.text = "亲密度: " + pet.getIntimacy();
            this.close_show.text = Strings$1.get("tid#UI_093");
            this.numberQinmi.value = Math.floor(pet.getIntimacy());
            this.qinmiImage.x = this.heartBox.x + (this.heartBox.width - (this.qinmiImage.width * 2.5 + this.numberQinmi.value.length * 40 + 90)) / 2 - 10;
            this.numberQinmi.x = this.qinmiImage.x + this.qinmiImage.width * 2.5 + 10;

            this.giftbg.visible = false;
            this.gift.visible = false;
            this.giftLock.visible = false;
            this.giftbg.offAll();
            if (!pet.getAdopted()) {
                return;
            }
            let gift = pet.getGift();
            if (gift && gift.length > 0) {
                let item = gift[0].CatGift[0];
                if (item) {
                    switch (item.type) {
                        case Constants.ItemType.NauticalTools:
                            let tool = NauticalSystem$1.getToolById(item.id);
                            if (tool.count > 0) {
                                this.giftLock.visible = false;
                            } else {
                                this.giftLock.visible = true;
                            }
                            this.giftbg.visible = true;
                            this.gift.visible = true;
                            this.gift.skin = tool.Icon;
                            this.giftbg.on(Laya.Event.CLICK, this, this.showInfo, [item.id, "tool"]);
                            break;
                        case Constants.ItemType.FishingBaits:
                            if (FishingSystem.isBaitUnlocked(item.id)) {
                                this.giftLock.visible = false;
                            } else {
                                this.giftLock.visible = true;
                            }
                            let config = ConfigMgr$1.getInstance().getItemById("FishingBaits", item.id);
                            if (config) {
                                this.giftbg.visible = true;
                                this.gift.visible = true;
                                this.gift.skin = config.img;
                                this.giftbg.on(Laya.Event.CLICK, this, this.showInfo, [item.id, "bait"]);
                            }
                            break;
                        case Constants.ItemType.CatToys:
                            if (PetSystem.hasToy(item.id)) {
                                this.giftLock.visible = false;
                            } else {
                                this.giftLock.visible = true;
                            }
                            let config1 = ConfigMgr$1.getInstance().getItemById("CatToys", item.id);
                            if (config1) {
                                this.giftbg.visible = true;
                                this.gift.visible = true;
                                this.gift.skin = config1.Icon;
                                this.giftbg.on(Laya.Event.CLICK, this, this.showInfo, [item.id, "toy"]);
                            }
                            break;
                    }
                }
            }
        }

        showCat(pet) {
            let j = 0;
            for (let i = 0; i < this._nauticalInfo.length; i++) {
                if (this._nauticalInfo[i].hasPet(pet.getId())) {
                    j = 1;
                }
            }
            if (j) {
                this.traveling.visible = true;
            } else {
                this.traveling.visible = false;
                this.cat = new BaseCat();
                this.cat.scaleX = this.cat.scaleY = 1.1;
                this.show.addChild(this.cat);
                this.cat.x = this.show.width / 2;
                this.cat.y = this.show.height / 2;
                this.cat.loadAnim(pet.getSource() + ".sk", pet.getSkin(), 0);
                this.cat.playAnim(pet.getDefaultAction(), true);
            }
            this.text.text = Strings$1.get(pet.getName());
        }

        showInfo(id, type) {
            GameContext.CurrentInfoId = id;
            GameContext.CurrentInfoType = type;
            Laya.Dialog.close("ui/InfoPop.scene");
            Laya.Dialog.open("ui/InfoPop.scene", false);
        }

        showVisitTimes(visitTimes) {
            this.number.value = visitTimes;
        }

        showIntimacyGrade(intimacyGrade) {
            this.heart1.visible = false;
            this.heart2.visible = false;
            this.heart3.visible = false;
            if (intimacyGrade > 0) {
                this.heart1.visible = true;
            }
            if (intimacyGrade > 1) {
                this.heart2.visible = true;
            }
            if (intimacyGrade > 2) {
                this.heart3.visible = true;
            }
        }

        //切换展示的猫咪，修改缓存记录，传入的pet是PetData类型
        changeShowPet(pet) {
            if (pet.getSource() == GameContext.showPetId) {
                return;
            }

            //先把原来的 cell 切换下状态
            let oldShowCell = this.findTargetCell(GameContext.showPetId);
            if (oldShowCell != null) oldShowCell.updateSelected(false);

            GameContext.showPetId = pet.getId();
            this.showPet(pet);
        }

        onAdoptPet() {
            if (GameContext.showPetId == "") {
                console.error("show pet id is empty!!!");
                return;
            }
            let minStar = Number(Const.get("CatAdoptStars"));
            if (minStar > PlayerSystem.getStar()) {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon19", [minStar]);
                return;
            }
            let petData = PetSystem.getPetInfo(GameContext.showPetId);
            if (!petData.canAdopt()) {
                console.log("pet [" + GameContext.showPetId + "] can not adopt");
                return;
            }

            if (PetSystem.getIfNeedUpdataToAdopt(GameContext.showPetId)) {
                SyncSystem.updateSceneInfo(Laya.Handler.create(this, this.confirmAdopt, [0]));
            } else {
                this.confirmAdopt(0);
            }
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onAdoptPre() {
            // Laya.Dialog.close("ui/AdoptAD.scene");
            // Laya.Dialog.open("ui/AdoptAD.scene", false, GameContext.showPetId);
            Laya.Dialog.close("ui/AdoptDiamond.scene");
            Laya.Dialog.open("ui/AdoptDiamond.scene", false, GameContext.showPetId);
        }

        confirmAdopt(type) {
            if (this.confirmAdoptSignal == 1 && PetSystem.getPetInfo(GameContext.showPetId).getAdopted() == false) {
                this.confirmAdoptSignal--;
                let dealHandler = Laya.Handler.create(this, function () {
                    this.button.visible = false;
                    this.furniture.visible = true;
                    this.heartBox.visible = true;
                    this.numberQinmi.visible = true;
                    this.qinmiImage.visible = true;
                    this.catArchivesBtn.visible = true;
                    this.visitTimesText.visible = false;
                    this.adoptPre.visible = false;
                    this.xiaolian.visible = false;
                    this.xiaolianPro.visible = false;
                    this.probg.visible = false;
                    this.xiaolianNume.visible = false;
                    this.desc.visible = false;
                    this.visitNew.visible = false;
                    this.visitNnum.visible = false;
                    this.visitsImg.visible = true;
                    this.number.visible = true;
                    let intimacyGrade = PetSystem.getPetInfo(GameContext.showPetId).getIntimacyGrade();
                    this.showIntimacyGrade(intimacyGrade);
                    PetSystem.getPetInfo(GameContext.showPetId).setAdopted(true);
                    PetSystem.getPetInfo(GameContext.showPetId).setRedPointStatus(2);  //将红点状态置为2，不再亮红点

                    let adoptCell = this.findTargetCell(GameContext.showPetId);
                    if (adoptCell != null) adoptCell.clearRedPoint();
                    Laya.Dialog.close("ui/AdoptSuccess.scene");
                    Laya.Dialog.open("ui/AdoptSuccess.scene", false, GameContext.showPetId);
                    TaskMainSystem.addProgress("CAT_COLLECT", 1);
                    Laya.timer.once(30, this, function () {
                        this.confirmAdoptSignal++;
                    });
                    EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["orderLevelUp"]);
                });
                PetSystem.adoptPet(GameContext.showPetId, type, dealHandler);
            }

        }

        checkGuide(eventString) {
            if (eventString == "GuideGroup_14") {
                let guide = GuideSystem.getGuide("GuideGroup_14");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.closeBtn);
                }
            }
        }
    }

    class CommonList extends Laya.List {

        constructor() {
            super();
            this.vScrollBarSkin = "";
             
            this.renderHandler = Laya.Handler.create(this, this.updateItem, null, false);
        }

        onSelect(index) {
        }

        updateItem(cell, index)
        {
            cell.setItem(cell.dataSource);
        }
    }

    class CollectItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(petId) {
            EventMgr.getInstance().postEvent("UPDATE_UNLOCKNUM", this, this.showUnlockNum);
            this.petId = petId;
            this.pet = PetSystem.getPetInfo(this.petId);
            this.normalPic = this.pet.getNormalPic();
            this.grayPic = this.pet.getGrayPic();
            this.home = this.getChildByName("home");
            this.home.visible = this.pet.homeStatus;
            let icon = this.getChildByName("image");
            if(this.pet.getVisitTimes() > 0) {
                //访问大于0，展示正常形象
                icon.skin = this.normalPic;
                icon.scale(2,2);
                //需要注册一个点击事件
            } else {
                //展示灰色形象, 需放大一倍
                icon.skin = this.grayPic;
                icon.scale(2,2);
            }
            this.on(Laya.Event.CLICK, this, this._onClickItem);

            if(GameContext.showPetId==petId) {
                EventMgr.getInstance().postEvent("COLLECTPOP_SHOW_PET", this.pet);
                this.updateSelected(true);
            } else {
                this.updateSelected(false);
            }

            //展示新
            let newPic = this.getChildByName("new");
            if(this.pet.getNewCome() && this.pet.getVisitTimes() > 0) {
                newPic.visible = true;
            } else {
                newPic.visible = false;
            }

            //红点状态
            let redPoint = this.getChildByName("redPoint");
            if(this.pet.needRedPoint() && PetSystem.toyUnlock(this.pet.config)) {
                redPoint.visible = true;
            } else {
                redPoint.visible = false;
            }
        }

        _onClickItem() {
            if (GameContext.showPetId != this.pet) {
                EventMgr.getInstance().postEvent("COLLECTPOP_CHANGE_SHOW_PET", this.pet);
                this.updateSelected(true);
                this.pet.changeRedPointRead();
            }
        }

        updateSelected(selected) {
            if(selected) {
                let selected = this.getChildByName("selected");
                let hook = this.getChildByName("hook");
                selected.visible = true;
                hook.visible = true;
            } else {
                let selected = this.getChildByName("selected");
                let hook = this.getChildByName("hook");
                
                //如果之前是选中状态，则可以直接将红点消掉
                if(selected.visible) {
                    let redPoint = this.getChildByName("redPoint");
                    redPoint.visible = false;
                    this.pet.changeRedPointRead();
                }

                selected.visible = false;
                hook.visible = false;
            }
        }

        clearRedPoint() {
            let redPoint = this.getChildByName("redPoint");
            redPoint.visible = false;
        }

    }

    /**
     * 顶部UI
     */
    class SceneTopUI extends Laya.Script {
        constructor() {
            super();
        }
        onEnable() {
            this.xingxing = this.owner.getChildByName("xingxing");
            this.xiaoyugan = this.owner.getChildByName("xiaoyugan");
            this.zuanshi = this.owner.getChildByName("zuanshi");
            console.log(this.owner);
            this.beike =this.owner.getChildByName('beike');
            this.fishText = this.xiaoyugan.getChildByName("fish_text");
            this.starText = this.xingxing.getChildByName("star_text");
            this.diamondText = this.zuanshi.getChildByName("zuanshi_text");
            this.fishAddLabel=this.owner.getChildByName("fishAddLabel");
            this.purchaseBtn = this.owner.getChildByName("purchaseBtn");
            this.purchaseBtn.on(Laya.Event.CLICK,this,this.onPurchaseBtnClick);
            
            //如果有贝壳展示的话
           if(this.beike){
            this.beike.visible=false;
            this.beikeImage=this.beike.getChildByName('beikeImage');
            this.beikeText=this.beike.getChildByName('beike_text');
            this.updateBeike(PlayerSystem.getShell());
           }
     
            this.updateFish(PlayerSystem.getFish());
            this.updateStar(PlayerSystem.getStar());
            this.updateDiamond(PlayerSystem.getDiamond());
         
            if (SceneMgr.getInstance().getCurrentSceneId() == Constants.NauticalSceneId ){
                if(BuildingSystem.hasBuildingDataByBid(3000) ||BuildingSystem.hasBuildingDataByBid(3001) || BuildingSystem.hasBuildingDataByBid(3002)){
                if(Constants.Yuledating&&Constants.Yuledating==1){
                    this.showBeike(true);
                    Constants.Yuledating=0;
                }else {
                    this.showBeike(false);
                }       
             }
            }
            EventMgr.getInstance().registEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().registEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().registEvent("SHOW_ADD_ST", this, this.showAdd);
            EventMgr.getInstance().registEvent('SHOW_BEIKE_SHOP',this,this.showBeike);
            EventMgr.getInstance().registEvent("UPDATE_BEIKE", this, this.updateBeike);//贝壳
            EventMgr.getInstance().registEvent("UPDATE_ZUANSHI", this, this.updateDiamond);
        }

        showAdd(addNum, type, handler) {
            if (type == Constants.RewardType.FISH) {
                if (!this.fishAddLabel) {
                    this.fishAddLabel = this.owner.getChildByName("fishAddLabel");
                    this.fishAddLabel.visible = true;
                }
                this.fishAddLabel.text = "+" + addNum;
                this.fishAddLabel.y = 139;
                this.fishAddLabel.visible = true;
                this.fishAddLabel.alpha = 0;
                Laya.Tween.to(this.fishAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.FISH, handler]));
            } else {
                if (!this.starAddLabel) {
                    this.starAddLabel = this.owner.getChildByName("starAddLabel");
                    this.starAddLabel.visible = true;
                }
                this.starAddLabel.text = "+" + addNum;
                this.starAddLabel.y = 139;
                this.starAddLabel.visible = true;
                this.starAddLabel.alpha = 0;
                Laya.Tween.to(this.starAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.STAR, handler]));
            }
            this._addTween(type);
        }

        _addTween(type) {
            if (type == Constants.RewardType.FISH) {
                this.startFish = this.fishNum;
                this.endFish = PlayerSystem.getFish();
                let addNum = this.endFish - this.startFish;
                if (Math.abs(addNum) < 6) {
                    this.maxTimes = addNum;
                } else {
                    this.maxTimes = 6;
                }
                this.times = 1;
                this.updataFishSixTimes();
            } else {
                this.startStar = this.starNum;
                this.endStar = PlayerSystem.getStar();
                let addNum = this.endFish - this.startFish;
                if (Math.abs(addNum) < 6) {
                    this.maxStarTimes = addNum;
                } else {
                    this.maxStarTimes = 6;
                }
                this.starTimes = 1;
                this.updataStarSixTimes();
            }
        }

        updataStarSixTimes() {
            if (this.starTimes == null) {
                return;
            }
            this.starNum = Math.floor(this.startStar + (this.endStar - this.startStar) / this.maxStarTimes * this.starTimes);
            if (this.starTimes < this.maxStarTimes) {
                this.starTimes++;
                Laya.timer.once(50, this, this.updataStarSixTimes);
            }else {
                this.starNum = PlayerSystem.getStar();
            }
            this.updateStar(this.starNum);
        }

        updataFishSixTimes() {
            if (this.times == null) {
                return;
            }
            this.fishNum = Math.floor(this.startFish + (this.endFish - this.startFish) / this.maxTimes * this.times);
            if (this.times < this.maxTimes) {
                this.times++;
                Laya.timer.once(50, this, this.updataFishSixTimes);
            } else {
                this.fishNum = PlayerSystem.getFish();
            }
            this.updateFish(this.fishNum);
        }

        _addLabelFadeout(type, handler) {
            if (type == Constants.RewardType.FISH) {
                Laya.Tween.to(this.fishAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            } else {
                Laya.Tween.to(this.starAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            }
        }

        updateFish(fish) {
            this.fishNum = PlayerSystem.getFish();
            this.fishText.text = Strings$1.dealFishNumberText(fish);
        }

        updateStar(star) {
            this.starNum = PlayerSystem.getStar();
            this.starText.text = String(star);
        }

        updateBeike(shell){
           // this.starNum = PlayerSystem.getStar();
           if(this.beikeText){
            this.beikeText.text = String(shell);
           }
        }

        updateDiamond(diamond){
            this.diamondText.text = String(diamond);
        }

        onPurchaseBtnClick(){
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene",false,[1]);
        }

        showBeike(show){
            this.xiaoyugan.visible=!show; 
            if(this.fishAddLabel){
                this.fishAddLabel.visible=!show;
            }  
            if(this.beike){
            this.beike.visible=show;  
            }   
            // if(show){
            //     this.xingxing.x=256;
            // }else{
            //     this.xingxing.x=336;
            // }
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().removeEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().removeEvent("SHOW_ADD_ST", this, this.showAdd);
            EventMgr.getInstance().removeEvent('SHOW_BEIKE_SHOP',this,this.showBeike);
            EventMgr.getInstance().removeEvent("UPDATE_BEIKE", this, this.updateBeike);//贝壳
            EventMgr.getInstance().removeEvent("UPDATE_ZUANSHI", this, this.updateDiamond);
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().removeEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().removeEvent("SHOW_ADD_ST", this, this.showAdd);
            EventMgr.getInstance().removeEvent('SHOW_BEIKE_SHOP',this,this.showBeike);
            EventMgr.getInstance().removeEvent("UPDATE_BEIKE", this, this.updateBeike);//贝壳
            EventMgr.getInstance().removeEvent("UPDATE_ZUANSHI", this, this.updateDiamond);
        }
        
    }

    /**
    * 确认购买
    */
    class ConfirmPop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.confirm = this.getChildByName("confirm");
            this.confirm.on(Laya.Event.CLICK, this, this.onConfirmFreeClick);
            this.confirm.visible = true;
            this.sbcj = this.getChildByName("sbcj");
            this.sbcj.on(Laya.Event.CLICK, this, this.onSbcjClick);
            this.vbcj = this.getChildByName("vbcj");
            this.vbcj.on(Laya.Event.CLICK, this, this.onVbcjClick);
            this.yuganNum = this.getChildByName("yugannum");
            this.yuganNum.visible = false;
            this.vbcj.visible = false;
            this.sbcj.visible = false;
            this.buyfish.visible = false;
            let anim = new Spine();
            anim.loadAnim("anim3/buchajia.sk");
            anim.playAnim("buchajia");
            anim.pos(this.vbcj.width / 2, this.vbcj.height / 2 + 30);
            this.vbcj.addChild(anim);

            let animbuy = new Spine();
            animbuy.loadAnim("anim3/buyfish.sk");
            animbuy.playAnim("1");
            animbuy.pos(this.buyfish.width / 2, 80);
            this.bofang = 0;
            this.buyfish.addChild(animbuy);
            this.buyfish.on(Laya.Event.MOUSE_DOWN, this, function () {
                animbuy.playAnim("2", false);
            });
            this.buyfish.on(Laya.Event.MOUSE_UP, this, function () {
                animbuy.playAnim("1");
            });
            this.buyfish.on(Laya.Event.MOUSE_OUT, this, function () {
                animbuy.playAnim("1");
            });
            this.buyfish.on(Laya.Event.CLICK, this, function () {
                this.goNeigo();
            });

            this.showInfo(GameContext.CurrentShopBuilding);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onVideo);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().registEvent("BCJ_VIDEO_CLICK", this, this.onVbcjClick);
            this.tigaopingjia.text = Strings$1.get("tid#UI_097");
            this.xiaofeiText.text = Strings$1.get("tid#UI_098");
            this.unlockText.text = Strings$1.get("tid#UI_099");
            this.yanjiu.text = Strings$1.get("tid#UI_096");
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onVideo);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("BCJ_VIDEO_CLICK", this, this.onVbcjClick);
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
                EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_BUILDING_ITEM", [true]);
            }
        }

        onSbcjClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.Building);
        }

        onVbcjClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.Building);
        }

        showInfo(buildingId) {
            if (buildingId == null) {
                this.close();
                return;
            }
            let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
            if (item == null) {
                this.close();
                return;
            }
            this.title.text = Strings$1.get("tid#BuildingName_" + buildingId);
            this.desc1.text = Strings$1.get("tid#UI_220");
            let pid = BuildingSystem.getPidByBid(buildingId);
            if (pid == "114") {
                this.xiaofeiNum.visible = true;
                this.xiaofeiText.visible = true;
                this.xiaofeiItem.visible = true;
                this.xiaofeiText.text = Strings$1.get("tid#UI_169");
                this.xiaofeiNum.text = "+" + Strings$1.dealFishNumberText(Number(item.SpecialValue));
            } else if (item.AutoIncome && item.AutoIncome > 0) {
                this.xiaofeiNum.text = "+" + (item.AutoIncome * 60) + "/h";
                this.xiaofeiNum.visible = true;
                this.xiaofeiText.visible = true;
                this.xiaofeiItem.visible = true;
            } else {
                this.xiaofeiNum.visible = false;
                this.xiaofeiText.visible = false;
                this.xiaofeiItem.visible = false;
            }
            if (item.Limit && item.Limit > 0) {
                this.unlockText.visible = true;
                this.unlockItem.visible = true;
                this.unlockNum.visible = true;
                this.unlockNum.text = String(item.Limit);
            } else {
                this.unlockText.visible = false;
                this.unlockItem.visible = false;
                this.unlockNum.visible = false;
            }
            this.star.text = "+" + item.StarReward;
            this.cost.text = Strings$1.dealFishNumberText(item.Cost, 1);
            this.icon.skin = item.ShopItems;
            let scalex = 150 / this.icon.width;
            let scaley = 150 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = 1.8;
                this.icon.scaleY = 1.8;
            } else {
                this.icon.scaleX = 1.8;
                this.icon.scaleY = 1.8;
            }
            this.btnFish.x = this.confirm.x + (this.confirm.width * this.confirm.scaleX - this.btnFish.width * this.btnFish.scaleX - this.cost.width - this.yanjiu.width - 10) / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
            this.cost.x = this.btnFish.x + this.btnFish.width * this.btnFish.scaleX + 5;
            this.yanjiu.x = this.cost.x + this.cost.width + 5;
            this.buildingId = buildingId;
            if (!BuildingSystem.checkBuyBuildingAvailable(buildingId)) {
                this.setGray();
                this.checkAdBtnShow(buildingId);
            } else {
                this.checkShowWeakGuideAnim();
            }
            this.showManyi(item);
        }

        setGray() {
            this.confirm.gray = true;
            this.confirm.disabled = true;
        }
        showManyi(building) {
            let quanzhongs;
            if (building.CustomerInvite.length != 0) {
                quanzhongs = building.CatFavors;
            } else {
                this.xiaolianList.visible = false;
                this.manyiImg.visible = false;
                this.left.visible = false;
                this.right.visible = false;
                this.desc1.visible = false;
                let y_ = (this.line.y - this.left.y);
                this.line.y -= y_;
                this.tigaopingjia.y -= y_;
                this.star.y -= y_;
                this.xiaofeiText.y -= y_;
                this.xiaofeiNum.y -= y_;
                this.unlockText.y -= y_;
                this.unlockNum.y -= y_;
                this.confirm.y -= y_;
                this.btnFish.y -= y_;
                this.cost.y -= y_;
                this.yanjiu.y -= y_;
                this.yuganNum.y -= y_;
                this.vbcj.y -= y_;
                this.xing1.y -= y_;
                this.xiaofeiItem.y -= y_;
                this.unlockItem.y -= y_;
                this.buyfish.y -= y_;
                this.bg1.height = 520;

                return;
            }
            this.catWeightedArr = [];
            if (quanzhongs) {
                quanzhongs.forEach(element => {
                    if (element != "") {
                        this.catWeightedArr.push(element);
                    }
                });
            }
            this.xiaolianList.array = this.quanzhongSan();
            if (this.xiaolianList.length == 1) {
                this.xiaolianList.x = 255 + 110 + 150;
            }
            else if (this.xiaolianList.length == 2) {
                this.xiaolianList.x = 255 + (110 + 150) / 2;
            } else {
                this.xiaolianList.x = 255;
            }
            this.xiaolianList.renderHandler = new Laya.Handler(this, this.updateItem);
        }
        updateItem(cell, index) {
            let item = ConfigMgr$1.getInstance().getItemById("Role", cell.dataSource.split(":")[0]);
            cell.getChildByName("head").skin = "cat_collect/head/" + item.HeadIcon + ".png";

            let manyi = Number(cell.dataSource.split(":")[1]);
            cell.getChildByName("chengdu").value = manyi;
            //笑脸的分界线
            let quanzhong = Const.get('CatFavorsIconsShow');
            let quanzhongs = JSON.parse(quanzhong);
            if (manyi >= quanzhongs[0] && manyi < quanzhongs[1]) {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/xiaoxiao.png";
            } else if (manyi >= quanzhongs[1] && manyi < quanzhongs[2]) {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/erxiao.png";
            } else {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/daxiao.png";
            }
        }
        //取权重前三的
        quanzhongSan() {
            let catArr = this.catWeightedArr;
            for (let i = 0; i < catArr.length - 1; i++) {
                for (let j = 0; j < catArr.length - 1 - i; j++) {
                    if (Number(catArr[j].split(":")[1]) < Number(catArr[j + 1].split(":")[1])) {
                        let temp = catArr[j];
                        catArr[j] = catArr[j + 1];
                        catArr[j + 1] = temp;
                    }
                }
            }
            let quanzhongArr = [];
            if (catArr[0])
                quanzhongArr.push(catArr[0]);
            if (catArr[1])
                quanzhongArr.push(catArr[1]);
            if (catArr[2])
                quanzhongArr.push(catArr[2]);
            return quanzhongArr;
        }
        onOpened(param) {
            EventMgr.getInstance().postEvent("SAVE_SHOP_MASK");
            EventMgr.getInstance().postEvent("CHANGE_SHOP_MASK", 1);
            if (Number(this.buildingId) == 1060) {
                let guide = GuideSystem.getGuide("GuideGroup_2");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                }
            } else if (Number(this.buildingId) == 1030) {
                let guide = GuideSystem.getGuide("GuideGroup_5");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                }
            } else if (Number(this.buildingId) == 2080) {
                let guide = GuideSystem.getGuide("GuideGroup_8");
                if (guide) {
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                }
            } else if (Number(this.buildingId) == 2020) {
                let guide = GuideSystem.getGuide("GuideGroup_11");
                if (guide) {
                    // let rect = new Laya.Rectangle(69, 444, 986, 1132);
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                }
            }
        }

        onCloseClick() {
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            // Laya.Dialog.open("ui/shopPop.scene");
        }

        onConfirmFreeClick() {
            if (this.vbcj.visible) {
                Laya.Dialog.close("ui/Makeup.scene");
                Laya.Dialog.open("ui/Makeup.scene", false);
                return;
            }
            else {
                this.onConfirmClick();
            }
        }

        onConfirmClick(AD = null) {
            let fetchType = Constants.FetchType.DIRECT;
            if (AD == true) {
                fetchType = Constants.FetchType.AD;
            }
            this.close();
            TaskMainSystem.addProgress("BUILDING_AMOUNT", 1);
            let star = PlayerSystem.getStar();
            if (GameContext.CurrentShopBuildingBase) {
                let buildingId = this.buildingId;
                BuildingSystem.buyBuilding(this.buildingId, fetchType, function (res) {
                    let guide = ConfirmPop.checkGuide(star, PlayerSystem.getStar());
                    SoundManager.getInstance().play("Sound_buyItemSuccess");
                    EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.GOUMAI);
                    let buildData = BuildingSystem.getBuildingDataByBid(buildingId);
                    if (buildData) {
                        let scene = BuildingSystem.getBuildingDataByBid(buildingId).getBelongScene();
                        if (scene == Constants.CoffeeSceneId) {
                            EventMgr.getInstance().postEvent("UPDATA_COFFEE_BUILDINGS");
                        } else {
                            EventMgr.getInstance().postEvent("UPDATA_NAUTICAL_BUILDINGS");
                            if (Number(buildingId) >= 3010 && Number(buildingId) < 3020) {
                                NauticalSystem$1.updateBtnListAfterBuy();
                            }
                        }
                    } else {
                        EventMgr.getInstance().postEvent("UPDATA_COFFEE_BUILDINGS");
                        EventMgr.getInstance().postEvent("UPDATA_NAUTICAL_BUILDINGS");
                    }
                    if (res.ifCanBargain && !guide && !GuideSystem.isGuiding()) {
                        let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
                        Laya.Dialog.close("ui/Bargaining.scene");
                        Laya.Dialog.open("ui/Bargaining.scene", false, item);
                    }
                    else {
                        EventMgr.getInstance().postEvent("CHANGE_SHOP_MASK", 0);
                        Laya.Dialog.closeAll();
                        BuildingSystem.placeBuilding(buildingId, GameContext.CurrentShopBuildingBase.getId(), true);
                        SoundManager.getInstance().play("Sound_placeBuilding");
                    }
                });

            } else {
                // Laya.Dialog.closeAll();
                let buildingId = this.buildingId;
                BuildingSystem.buyBuilding(this.buildingId, fetchType, function (res) {
                    let guide = ConfirmPop.checkGuide(star, PlayerSystem.getStar());
                    SoundManager.getInstance().play("Sound_buyItemSuccess");
                    EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.GOUMAI);
                    if (res.ifCanBargain && !guide && !GuideSystem.isGuiding()) {
                        let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
                        Laya.Dialog.close("ui/Bargaining.scene");
                        Laya.Dialog.open("ui/Bargaining.scene", false, item);
                    }
                    else {
                        EventMgr.getInstance().postEvent("CHANGE_SHOP_MASK", 0);
                        Laya.Dialog.closeAll();
                        GameContext.PlacingBuilding = buildingId;
                        GameContext.PlacingNewBuilding = true;
                        SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                        EventMgr.getInstance().postEvent("PLACE_BUILDING", buildingId);
                    }
                });
            }
        }

        static checkGuide(before, after) {
            let guideId = JSON.parse(Const.get("BarginIgnoreGuideGruop"));
            let starNum = new Array();
            for (let i = 0; i < guideId.length; i++) {
                let condition = JSON.parse(ConfigMgr$1.getInstance().getItemById("GuideGroup", guideId[i]).Condition);
                let star = condition.Star;
                starNum.push(star);
                if (before < star && star <= after) {
                    return true;
                }
            }
            return false;
        }

        done(shareId) {
            if (shareId == Constants.ShareId.Building) {
                this.onConfirmClick(true);
            }
        }

        onVideo(shareId) {
            if (shareId == Constants.ShareId.Building) {
                this.onConfirmClick(true);
            }
        }

        checkAdBtnShow(buildingId) {
            let needFish = BuildingSystem.checkAdShowFish(buildingId)[0];
            let costFish = BuildingSystem.checkAdShowFish(buildingId)[1];
            if (needFish <= 0) {
                return;
            }
            let yugan = costFish - needFish;
            let adShoppingCondition = Const.get("ADShoppingCondition");

            this.yuganNum.visible = true;
            this.yuganNum.text = Strings$1.get("tid#UI_094") + '  ' + Strings$1.dealFishNumberText(needFish, 1) + '  ' + Strings$1.get("tid#UI_095");
            let maxAd = Number(Const.get("ADShoppingTimes"));
            let adShoppingRange = JSON.parse(Const.get("ADShoppingRange"));
            let showAd = false;
            let buildingNum = BuildingDataMgr.getInstance().getNowBuildingNumber();
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].BUILDING_AMOUNT[0];
                let max = adShoppingRange[i].BUILDING_AMOUNT[1];
                if (max < 0) {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                } else {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                }
            }
            let items = PetSystem.getItemBySpecialType(4);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
            });
            if (showAd && PlayerSystem.getMakeUpTimes() < maxAd + itemAddTimes) {
                if (SdkHelper.hasVideo()) {
                    this.vbcj.visible = true;
                    this.sbcj.visible = false;
                    this.confirm.visible = true;
                    this.confirm.gray = false;
                    this.confirm.disabled = false;
                    this.buyfish.visible = false;
                } else {
                    this.vbcj.visible = false;
                    this.sbcj.visible = true;
                    let cost = this.sbcj.getChildByName("cost");
                    cost.text = this.cost.text + "建造";
                    cost.text = Strings$1.get("tid#UI_096");
                    this.confirm.visible = false;
                }
            } else if (!BuildingSystem.checkBuyBuildingAvailable(buildingId)) {
                this.vbcj.visible = false;
                this.sbcj.visible = false;
                this.confirm.visible = true;
                this.confirm.gray = true;
                this.confirm.disabled = true;
                this.buyfish.visible = true;
            }
            else {
                this.vbcj.visible = false;
                this.sbcj.visible = false;
                this.confirm.visible = true;
                this.buyfish.visible = false;
            }
        }

        checkShowWeakGuideAnim() {
            if (!GuideSystem.isGuiding() && TaskMainSystem.getIsTasking()) {
                let task = TaskMainSystem.getCurrentData();
                if (task.getGuide() == "Building" && !TaskMainSystem.checkEnought()) {
                    if (this.weakAnim == null || this.weakAnim.destroyed) {
                        this.weakAnim = new Spine();
                        this.weakAnim.loadAnim("anim1/xinshouyindao.sk");
                        this.weakAnim.playAnim("zhuazi2", true);
                        this.weakAnim.x = this.confirm.x + this.confirm.width * this.confirm.scaleX / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
                        this.weakAnim.y = this.confirm.y + this.confirm.height * this.confirm.scaleX / 2 - this.confirm.anchorY * this.confirm.height * this.confirm.scaleY;
                        this.weakAnim.zOrder = 5;
                        this.addChild(this.weakAnim);
                        EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_BUILDING_ITEM", [false]);
                        return;
                    }
                }
            }
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
            }
        }
        goNeigo() {
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene", true, [1]);
        }
    }

    class EntrustPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            this.talkBox = this.getChildByName("talkBox");
            this.catName = this.talkBox.getChildByName("catName");
            this.talk = this.talkBox.getChildByName("talk");
            this.taskBox = this.getChildByName("taskBox");
            this.title = this.taskBox.getChildByName("title");
            this.content = this.taskBox.getChildByName("content");
            this.reward = this.taskBox.getChildByName("reward");
            this.rewardNum = this.taskBox.getChildByName("rewardNum");
            this.receiveBtn = this.taskBox.getChildByName("receiveBtn");
            this.accaptBtn = this.taskBox.getChildByName("accaptBtn");
            this.time = this.taskBox.getChildByName("time");
            this.closeBtn = this.taskBox.getChildByName("closeBtn");
            this.receiveBtn.on(Laya.Event.CLICK, this, this.receive);
            this.accaptBtn.on(Laya.Event.CLICK, this, this.accapt);
            this.talkBox.on(Laya.Event.CLICK, this, this.onTalkBoxClick);
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.subscriptions = false;
            SdkHelper.checkSubscriptions(Constants.WxMessageTmplIdKeys.ENTRUST, Laya.Handler.create(this, function (result) {
                this.subscriptions = result;
            }));
            this.lingqu.text = Strings$1.get("tid#Btn_017");
            this.jieshou.text = Strings$1.get("tid#Btn_018");
            this.jiangli.text = Strings$1.get("tid#UI_100");
            this.catName.text = Strings$1.get("tid#UI_101");
        }

        onEnable() {
            this.entrustData = this._param;
            this.currentTalkId = 0;
            this.talkArr = [];
            this.talkBox.visible = true;
            this.taskBox.visible = false;
            if (this.entrustData.status == 0) {
                this.accaptBtn.visible = true;
                this.receiveBtn.visible = false;
                this.time.visible = true;
                this.talkArr = this.entrustData.getAcceptTalk();
                let conditions = this.entrustData.getConditions();
                let strs = [];
                conditions.forEach(condition => {
                    condition.forEach(element => {
                        if (element.eventType == "TIME") {
                            this.miaowangzitime = element.target;
                            strs.push(Strings$1.getLeftTimeBySeconds(element.target));
                        } else if (element.eventType == "CATS_VISIT") {
                            strs.push(element.target);
                        } else {
                            let catId = "Cat" + element.eventType.substring(16);
                            let name = Strings$1.get("tid#catName_" + catId);
                            strs.push(name);
                            strs.push(element.target);
                        }
                    });
                });
                this.time.text = Strings$1.get(this.entrustData.getConditionDesc(), strs);
            } else {
                this.accaptBtn.visible = false;
                this.receiveBtn.visible = true;
                this.time.visible = false;
                this.talkArr = this.entrustData.getFinishTalk();
            }
            this.title.text = Strings$1.get(this.entrustData.getName());
            this.catName.text = Strings$1.get("tid#UI_101");
            // this.catName.text = "喵拉伯王子";
            this.content.text = Strings$1.get(this.entrustData.getDesc());
            let reward = this.entrustData.getReward();
            if (reward.Reward.id == Constants.RewardType.FISH) {
                this.reward.skin = "topui/yugan.png";
            } else {
                this.reward.skin = "topui/xingxing.png";
            }
            this.rewardNum.text = "+" + (reward.Reward.amount + reward.RewardRate * PetSystem.getMaxValueBase());
            if (this.entrustData.getCatTalkShow()) {
                this.talkBox.visible = false;
                this.taskBox.visible = true;
            } else {
                this.talkShow();
            }
            this.entrustData.setCatTalkShow();
        }

        talkShow() {
            if (this.currentTalkId < this.talkArr.length) {
                this.talkStr = Strings$1.get(this.talkArr[this.currentTalkId]);
                this.currentCharNum = 0;
                Laya.timer.loop(1, this, this.showTalk);
                this.currentTalkId++;
            } else {
                if (this._end) {
                    this.close();
                    //屏蔽
                    //Laya.Dialog.close("ui/WxMessagePop.scene");
                    //Laya.Dialog.open("ui/WxMessagePop.scene", false, [Constants.WxMessageTmplIdKeys.ENTRUST]);
                    return;
                }
                this.talkBox.visible = false;
                this.taskBox.visible = true;
                Laya.Tween.from(this.taskBox, { scaleX: 0, scaleY: 0 }, 200);
            }
        }

        receive() {
            let id = this.entrustData.getListId();
            let point = new Laya.Point(this.reward.x, this.reward.y);
            point = this.reward.parent.localToGlobal(point);
            EntrustSystem.receiveRewards(id, Laya.Handler.create(this, function () {
                let reward = this.entrustData.getReward();
                ItemFlyMgr.getInstance().fly(Constants.RewardType.FISH, Number(reward.Reward.amount + reward.RewardRate * PetSystem.getMaxValueBase()), Constants.FlyType.NORM, point.x, point.y);
            }));
            SpecialCatSystem.catLeave();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }

        accapt() {
            SpecialCatSystem.catLeave();
            let id = this.entrustData.getListId();
            EntrustSystem.acceptEntrust(id);
            SoundManager.getInstance().play("Sound_clickSound", 1);
            SdkHelper.shanchuTuisong('miao_wangzi_1');
            SdkHelper.goTuisong('miao_wangzi_1', Strings$1.get('tid#WXMessage_EntrustmentName'), Strings$1.get('tid#WXMessage_EntrustmentDesc'), '' + this.miaowangzitime);
            this.subscriptions = true;
            if (this.subscriptions) {
                //SdkHelper.requestSubscribeMessage([Constants.WxMessageTmplIdKeys.ENTRUST]);
                this.close();
            } else {
                this._end = true;
                this.talkArr = this.entrustData.getWXMessageTalk();
                this.currentTalkId = 0;
                this.talkBox.visible = true;
                this.taskBox.visible = false;
                this.talkShow();
            }
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }

        //一个字一个字蹦
        showTalk(showAll = false) {
            if (showAll) {
                this.talk.text = this.talkStr;
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                return;
            }
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkShow();
                return;
            }
            this.talk.text = this.talkStr.substr(0, ++this.currentCharNum);
            if (this.currentCharNum == this.talkStr.length) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
            }
        }

        onTalkBoxClick() {
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                this.talkShow();
            } else {
                this.showTalk(true);
            }
        }

        onDisable() {
            Laya.Tween.clearAll(this.taskBox);
            Laya.timer.clearAll(this);
        }

    }

    class FishingBaitsSelect extends Laya.Dialog {

        onOpened() {
        }

        onEnable() {
            FishingNewBaitMgr.getInstance().loadData();
            let config = ConfigMgr$1.getInstance().getConfig("FishingBaits");
            this.configArray = new Array();
            config.forEach((listConfig, baitId) => {
                this.configArray.push(listConfig);
            });

            let bg_pannel = this.getChildByName("bg_panel");
            bg_pannel.on(Laya.Event.CLICK, this, function() {
                this.close();
            });

            this.baitLabel.text = Strings$1.get("tid#UI_102");
            this.left_arrow = this.getChildByName("left_arrow");
            this.right_arrow = this.getChildByName("right_arrow");
            this.redPointL = this.left_arrow.getChildByName("redPointL");
            this.redPointR = this.right_arrow.getChildByName("redPointR");


            this.list.on(Laya.Event.MOUSE_UP,this,this.changeListItems);
            this.list.on(Laya.Event.MOUSE_OUT,this,this.changeListItems1);
            this.list.scrollBar.rollRatio = 0;
            
            this.showBaitList();

            EventMgr.getInstance().registEvent("FISHING_BAITS_SELECTED_CHANGE", this, this.changeSelectBaitId);
        }

        onDisable(){
            FishingNewBaitMgr.getInstance().seeBait();
            EventMgr.getInstance().removeEvent("FISHING_BAITS_SELECTED_CHANGE", this, this.changeSelectBaitId);
            this.list.off(Laya.Event.MOUSE_UP,this,this.changeListItems);
            Laya.Tween.clearAll(this);
            EventMgr.getInstance().postEvent("CHECK_BAITS_REDPOINT");
        }

        loadList(){
            let index = 0;
            let baitId = FishingSystem.getLastBaitId();
            for(const config of this.configArray) {
                if(config.Id == baitId) {
                    break;
                }
                index++;
            }
            this._index = index;
            this.startIndex = 0;
            this.endIndex = this.configArray.length;
            let showList = new Array();
            let l = this.configArray.slice(this.startIndex, this.endIndex);
            for(const config of l) {
                showList.push(config.Id);
            }
            this.list.array = showList;
        }

        //展示的鱼饵列表
        showBaitList() {
            this.loadList();
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        showUpdate(){
            this.loadList();
            this.changeListItems();
        }

        adjustList(){
            let w = this.list.getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w+s) + 0.5 | 0;
            Laya.Tween.to(this.list.scrollBar,{value:this._index*(w+s)},100);
            let order = FishingNewBaitMgr.getInstance().haveHigherBait();
            this.redPointL.visible = false;
            this.redPointR.visible = false;
            if(order > 0 ){
                if(this._index > order - 1){
                    this.redPointL.visible = true;
                    this.redPointR.visible = false;
                }
                else if(this._index + 2 < order - 1){
                    this.redPointL.visible = false;
                    this.redPointR.visible = true;
                }
            }
        }

        changeListItems1(){
            this.list.scrollBar.stopScroll();
            this.changeListItems();
        }

        changeListItems() {
            // this.list.scrollBar.stopScroll();
            this.adjustList();            
            //修改left_arrow 和 right_arrow 的展示
            if(this._index == 0) {
                this.left_arrow.visible = false;
            } else {
                this.left_arrow.visible = true;
                this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            }
            if(this._index == this.endIndex -3) {
               this.right_arrow.visible = false;
            } else {
                this.right_arrow.visible = true;
                this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);
            }
        }

        _onLeftArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - 3 > 0 ? this._index - 3 : 0;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        _onRightArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + 3 < this.configArray.length ? this._index + 3 : this.configArray.length - 2;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        //切换选择的鱼饵ID
        changeSelectBaitId(baitId) {
            //更改记录的鱼饵ID
            FishingSystem.setLastBaitId(baitId);
            EventMgr.getInstance().postEvent("SET_BAITS_IMAGE");
            //重新刷新展示的list页面
            this.showUpdate();
        }

    }

    class CommonList$1 extends Laya.List {

        constructor() {
            super();
            this.hScrollBarSkin = "";
             
            this.renderHandler = Laya.Handler.create(this, this.updateItem, null, false);
        }

        onSelect(index) {
        }

        updateItem(cell, index)
        {
            cell.setItem(cell.dataSource);
        }
    }

    class FishingBaitsItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(baitId) {
            this.baitId = baitId;
            let config = ConfigMgr$1.getInstance().getItemById("FishingBaits", baitId);
            let playerStar = PlayerSystem.getStar();
            let bait_img = this.getChildByName("bait_img");
            let star_img = this.getChildByName("star_img");
            let need_star = this.getChildByName("need_star");
            let bait_name = this.getChildByName("bait_name");
            let bait_introduce = this.getChildByName("bait_introduce");
            let hook = this.getChildByName("hook");
            let newLable = this.getChildByName("newLable");
            newLable.visible = false;
            let qualityImage = this.getChildByName("quality");
            qualityImage.visible = false;
            bait_introduce.text = Strings$1.get(config.introduce);

            if (FishingSystem.isBaitUnlocked(baitId)) {
                //判断是是否有足够的铃铛去解锁
                if (config.StarLimit <= playerStar) {
                    //已解锁的，就不展示 star 相关
                    this.lingdangs = 1;//判断是否有铃铛锁
                    star_img.visible = false;
                    need_star.visible = false;
                    bait_img.skin = config.img;
                    bait_introduce.align = "left";
                    bait_name.text = Strings$1.get(config.name);
                    //注册点击事件
                    this.on(Laya.Event.CLICK, this, this.onSelected);
                }
                else {
                    //已解锁的，就不展示 star 相关
                    this.lingdangs = 0;
                    star_img.visible = true;
                    need_star.visible = true;
                    need_star.text = config.StarLimit.toString();
                    this.StartLimit = config.StarLimit;
                    bait_img.skin = config.img;
                    bait_introduce.align = "center";
                    bait_introduce.text = "-";
                    bait_name.text = "???";
                    //注册点击事件
                    this.on(Laya.Event.CLICK, this, this.onSelected);
                }
                let quality = config.Quality;
                if (quality != null) {
                    qualityImage.visible = true;
                    qualityImage.skin = "shopitems/quality" + quality + ".png";
                }
            } else {
                star_img.visible = false;
                // need_star.text = config.StarLimit.toString();
                need_star.visible = false;
                bait_img.skin = config.imgShadow;
                bait_introduce.align = "center";
                bait_introduce.text = "-";
                bait_name.text = "???";
                //取消点击事件
                this.off(Laya.Event.CLICK, this, this.onSelected);
            }

            //如果当前玩家选择的鱼饵是本鱼饵，则展示hook
            if (FishingSystem.getLastBaitId() == baitId) {
                hook.visible = true;
            } else {
                hook.visible = false;
            }

            let newData = FishingNewBaitMgr.getInstance().getBaitStatus(baitId);
            if (newData && newData.Value == 1) {
                newLable.visible = true;
            }
            FishingNewBaitMgr.getInstance().setSeeArr(baitId);
        }
        onSelected() {
            if (this.lingdangs == 1) {
                EventMgr.getInstance().postEvent("FISHING_BAITS_SELECTED_CHANGE", [this.baitId]);
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon24", [this.StartLimit]);
            }
        }
    }

    class FishingCollectPop extends Laya.Dialog {

        onEnable() {

            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);

            GameContext.showFishId = "";

            this.show = this.getChildByName("show");
            this.name_text = this.getChildByName("name_text");
            this.name_text.text = "???";
            this.times_text = this.getChildByName("times_text");
            this.times_text.text = "0";
            this.record_text = this.getChildByName("record_text");
            this.record_text.text = "???";
            this.share = this.getChildByName("share");
            this.share.visible=false;
            //this.share.on(Laya.Event.CLICK,this,this.onShareClick);

            this.findDefaultShow();

            let config = ConfigMgr$1.getInstance().getConfig("FishingFishes");
            let array = new Array();
            config.forEach((listConfig, fishId) => {
                array.push(fishId);
            });
            // this.list 
            this.list.array = array;
            EventMgr.getInstance().registEvent("FISHING_COLLECT_SHOW_FISH", this, this.showFish);
            EventMgr.getInstance().registEvent("FISHING_COLLECT_CHANGE_SHOW_FISH", this, this.changeShowFish);
            this.diaohuo = this.getChildByName("diaohuo");
            this.diaohuo.text = Strings$1.get("tid#UI_157");
            this.zuidajilu = this.getChildByName("zuidajilu");
            this.zuidajilu.text = Strings$1.get("tid#UI_158");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("FISHING_COLLECT_SHOW_FISH", this, this.showFish);
            EventMgr.getInstance().removeEvent("FISHING_COLLECT_CHANGE_SHOW_FISH", this, this.changeShowFish);
        }

        onShareClick(){
            SdkHelper.shareAppMessage(Constants.ShareId.FishNewCome);
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }

        findDefaultShow() {
            if(GameContext.showFishId != "") return;

            let config = ConfigMgr$1.getInstance().getConfig("FishingFishes");
            for(const [fishId, listConfig] of config) {
                let fishCollectData = FishingSystem.getCollectById(fishId);
                if(fishCollectData != null && fishCollectData.getAcquireTimes() > 0) {
                    GameContext.showFishId = fishId;
                    break;
                }
            }
        }

        //查找指定fishId的cell
        findTargetCell(showFishId) {
            for(const cell of this.list.cells) {
                if(cell.dataSource == showFishId) {
                    return cell;
                }
            }
            return null;
        }

        updateFishes() {
            this.list.refresh();
        }

        //展示新的鱼图鉴，传入的fish是FishingCollectData类型
        showFish(fish) {
            //切换记录的展示土建、名字、次数等信息
            this.name_text.text = Strings$1.get(fish.getName());
            this.times_text.text = fish.getAcquireTimes();

            /**冲突 标记 */
            //this.times_text.text = Strings.get("tid#UI_075");
            //this.record_text.text = fish.getMaxSize() + "M";

            this.record_text.text = fish.getMaxSize() + Strings$1.get("tid#fishingFishSizeUnit");

            this.show.skin = fish.getSource();
            this.show.scaleX = this.show.scaleY = 2.375;
        }

        //切换展示的图鉴，修改缓存记录，传入的fish是FishingCollectData类型
        changeShowFish(fish) {
            if(fish.getId() == GameContext.showFishId) {
                return;
            }

            //先把原来的 cell 切换下状态
            let oldShowCell = this.findTargetCell(GameContext.showFishId);
            if(oldShowCell != null) oldShowCell.updateSelected(false);

            GameContext.showFishId = fish.getId();
            if (fish.getAcquireTimes() > 0) {
                this.showFish(fish);
            } else {
                this.showLockFish(fish);
            }
        }

        //展示未解锁的鱼图鉴，传入的fish是FishingCollectData类型
        showLockFish(fish) {
            //切换记录的展示土建、名字、次数等信息
            this.name_text.text = "???";
            this.times_text.text ='0';
            this.record_text.text = "???";
            this.show.skin = fish.getGrayPic();
            this.show.scaleX = this.show.scaleY = 4;
        }

    }

    class FishingCollectItem extends Laya.Sprite {
        constructor() {
            super();
        }

        setItem(fishId) {
            this.fishId = fishId;
            this.fish = FishingSystem.getCollectById(fishId);
            if(!this.fish){
                console.log("error fishId", fishId);
                this.visible = false;
                return;
            }
            this.visible = true;
            this.normalPic = this.fish.getSource();
            this.grayPic = this.fish.getGrayPic();

            let icon = this.getChildByName("image");
            if(this.fish.getAcquireTimes() > 0) {
                //访问大于0，展示正常形象
                icon.skin = this.normalPic;
                icon.scaleX = icon.scaleY = 1.2;
            } else {
                //展示灰色形象, 需放大一倍
                icon.skin = this.grayPic;
                icon.scaleX = icon.scaleY = 2;
            }
            //需要注册一个点击事件
            this.on(Laya.Event.CLICK, this, this.onClick);

            if(GameContext.showFishId==fishId) {
                this.updateSelected(true);
                EventMgr.getInstance().postEvent("FISHING_COLLECT_SHOW_FISH", this.fish);
            } else {
                this.updateSelected(false);
            }

            //展示新
            let newPic = this.getChildByName("new");
            if(this.fish.getNewCome() && this.fish.getAcquireTimes() > 0) {
                newPic.visible = true;
            } else {
                newPic.visible = false;
            }
        }

        onClick() {
            if(GameContext.showFishId!=this.fishId) {
                this.updateSelected(true);
                EventMgr.getInstance().postEvent("FISHING_COLLECT_CHANGE_SHOW_FISH", this.fish);
            }
        }

        updateSelected(selected) {
            if(selected) {
                let selected = this.getChildByName("selected");
                let hook = this.getChildByName("hook");
                selected.visible = true;
                hook.visible = true;
            } else {
                let selected = this.getChildByName("selected");
                let hook = this.getChildByName("hook");
                selected.visible = false;
                hook.visible = false;
            }
        }
    }

    class FishingDialog extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable() {
            this.popupEffect = null;
            this.data = this._param;
            this.videoBtn.on(Laya.Event.CLICK, this, this.onVideoClick);
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareClick);
            this.overseasGetReward.on(Laya.Event.CLICK, this, this.onGetClick);
            this.overseasGetReward.visible = false;
            // this.getBtn.on(Laya.Event.CLICK, this, this.onGetClick);
            // this.getBtn.visible = false;
            this.sureBtn.on(Laya.Event.CLICK, this, this.onSureClick);
            this.sureBtn.label=Strings$1.get('tid#Btn_009');
            this.freeAgain.on(Laya.Event.CLICK, this, this.onfreeAgainClick);
            this.sizeText.text = Strings$1.get("tid#fishingUI1");
            this.historyText.text = Strings$1.get("tid#fishingUI2");
            this.tryText.text = Strings$1.get("tid#UI_160");
            // this.getBtn.text = Strings.get("tid#AdsText1");

            this.color = this.data.color;
            this.fishAgain = this.data.fishAgain;
            this.playStatus = this.data.playStatus;

            this.list.visible = false;
            this.shareBtn.visible = false;
            this.videoBtn.visible = false;
            // this.getBtn.visible = false;

            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            this.again = 0;
            this.animTime = 33;
            this.playDialogAnim();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            this.signAni.stop();
            if (this.fishNewCome && this.playStatus == 0 && this.again == 0) {
                Laya.Dialog.close("ui/FishNewCome.scene");
                Laya.Dialog.open("ui/FishNewCome.scene", false, this._param);
            }
            this.again = 0;
            Laya.Tween.clearAll(this.levelImg);
            Laya.Tween.clearAll(this.blackBg);
            Laya.Tween.clearAll(this.part1);
            Laya.Tween.clearAll(this.whiteBg);
            Laya.Tween.clearAll(this.part2);
            Laya.Tween.clearAll(this.part3);
            Laya.Tween.clearAll(this.list);
            Laya.timer.clearAll(this);
        }

        playDialogAnim(){
            this.setLevelImg();
            Laya.timer.once(this.animTime * 20,this,function(){ //预览动画第115帧
                this.bg.visible = true;
                this.playBlackBg();
                Laya.timer.once(this.animTime * 10,this,function(){ //预览动画第125帧
                    this.playDetail();
                    Laya.timer.once(this.animTime * 15,this,function(){ //预览动画第140帧
                        this.playWhiteBg();
                        Laya.timer.once(this.animTime * 10,this,function(){ //预览动画第150帧
                            this.playList();
                            Laya.timer.once(this.animTime * 11,this,function(){ //预览动画第161帧
                                this.playBtn();
                            });
                        });
                    });
                });
            });
            FishingSystem.updateServerCollectsInfo();
        }

        setLevelImg(){
            switch(this.color){
                case 1: this.levelImg.skin = "fishing_game/ui/perfect.png";break;
                case 2: this.levelImg.skin = "fishing_game/ui/great.png";break;
                case 3: this.levelImg.skin = "fishing_game/ui/good.png";break;
                case 4: this.levelImg.skin = "fishing_game/ui/cool.png";break;
            }
            this.playLevelImg();
        }

        playLevelImg(){
            this.levelImg.scale(0,0);
            this.levelImg.visible = true;
            Laya.Tween.to(this.levelImg,{scaleX:3.5,scaleY:3.5},this.animTime * 4,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(this.levelImg,{scaleX:2,scaleY:2},this.animTime * 3,null,Laya.Handler.create(this,function(){
                    Laya.Tween.to(this.levelImg,{scaleX:2.5,scaleY:2.5},this.animTime * 3);
                }));
            }));
        }

        playBlackBg(){
            this.blackBg.x = 2800;
            this.blackBg.height = 10;
            this.blackBg.visible = true;
            Laya.Tween.to(this.blackBg,{x:912},this.animTime * 3,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(this.blackBg,{height:250},this.animTime * 4,null,Laya.Handler.create(this,function(){
                    Laya.Tween.to(this.blackBg,{height:220},this.animTime * 3);
                }));
                Laya.Tween.to(this.levelImg,{y:809},this.animTime * 5);
            }));
        }

        playDetail(){
            this.setDetail();
            this.part1.alpha = 0;
            this.part1.x = 250;
            this.part1.visible = true;
            Laya.Tween.to(this.part1,{x:0},this.animTime * 15);
            Laya.Tween.to(this.part1,{alpha:1},this.animTime * 15,Laya.Ease.cubicIn);
            Laya.timer.once(this.animTime * 10,this,function(){
                this.part2.alpha = 0;
                this.part2.visible = true;
                Laya.Tween.to(this.part2,{alpha:1},this.animTime * 10,Laya.Ease.cubicIn);
            });
        }

        setDetail(){
            let fishId = this.data.reward[0];
            let fishSize = this.data.reward[1];
            let fishConfig = ConfigMgr$1.getInstance().getItemById("FishingFishes", String(fishId));
            let sizeMin = this.data.reward[4];
            let sizeMax = this.data.reward[5];

            this.fish_img.skin = fishConfig.img;

            //鱼名字
            this.name_text.text = Strings$1.get(fishConfig.name) ;
            this.size1.text = fishSize.toFixed(2) + Strings$1.get("tid#fishingFishSizeUnit");

            //新纪录
            let fishCollect = FishingSystem.getCollectById(fishId);
            let oldRecord = fishCollect == null || fishCollect == undefined ? 0 : fishCollect.getMaxSize();
            this.fishNewCome = 0;
            if (fishCollect.acquireTimes <= 0) {
                this.fishNewCome = 1;
            }
            if (this.fishNewCome || fishSize > oldRecord) {
                this.new_record.visible = true;
                this.history.color = "#db68ec";
            } else {
                this.new_record.visible = false;
                this.history.color = "#ffffff";
            }
            oldRecord = oldRecord > fishSize ? oldRecord : fishSize;
            this.history.text = oldRecord.toFixed(2) + Strings$1.get("tid#fishingFishSizeUnit");

            let rankPercent = ((oldRecord - sizeMin) / (sizeMax - sizeMin) * 100).toFixed(0) + "%";
            this.rank.text = Strings$1.get("tid#fishingUI3",[rankPercent]);

            //给案板上加个动画
            this.signAni = new Spine();
            this.anban.addChild(this.signAni);
            this.signAni.load("anim3/qiandao.sk", Laya.Handler.create(this, function () {
                this.signAni.x = 110;
                this.signAni.y = -50;
                this.signAni.scaleY = this.signAni.scaleX = 0.8;
                this.signAni.play('texiao', true);
            }));
        }

        playWhiteBg(){
            this.whiteBg.x = 2800;
            this.whiteBg.height = 10;
            this.whiteBg.visible = true;
            Laya.Tween.to(this.whiteBg,{x:915},this.animTime * 4,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(this.whiteBg,{height:220},this.animTime * 4,null,Laya.Handler.create(this,function(){
                    Laya.Tween.to(this.whiteBg,{height:205},this.animTime * 3);
                }));
            }));
        }

        setList(){
            let rewardL = this.data.rewards;
            //调整list，排序让fish在前面
            rewardL.sort(function (a, b) {
                if (a.id == "FISH") return -1;
                if (b.id == "FISH") return 1;
                return 0;
            });
            if(this.playStatus == 0){
                if (rewardL.length == 1) {
                    this.list.repeatX = 1;
                    this.list.visible = true;
                    this.list.width = 202;
                } else if (rewardL.length == 2) {
                    this.list.repeatX = 2;
                    this.list.visible = true;
                    this.list.width = 497;
                } else if (rewardL.length == 3) {
                    this.list.repeatX = 3;
                    this.list.visible = true;
                    this.list.width = 781;
                } else {
                    this.list.visible = false;
                }
                this.list.array = rewardL;
            }
        }

        playList(){
            this.list.scale(0,0);
            this.setList();
            Laya.Tween.to(this.list,{scaleX:1.2,scaleY:1.2},this.animTime * 6,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(this.list,{scaleX:0.9,scaleY:0.9},this.animTime * 3,null,Laya.Handler.create(this,function(){
                    Laya.Tween.to(this.list,{scaleX:1,scaleY:1},this.animTime * 2);
                }));
            }));
        }

        setBtn(){
            if (this.playStatus == 1) {
                this.list.visible = false;
                this.shareBtn.visible = false;
                this.videoBtn.visible = false;
                // this.getBtn.visible = false;
                this.overseasGetReward.visible = false;
                this.sureBtn.visible = true;
                this.tryText.visible = true;
            }
            else {
                this.overseasGetReward.visible=true;
                this.freeAgain.visible = false;
                // this.freeAgain.visible = this.fishAgain;
                 if (SdkHelper.hasVideo()) {
                    this.shareBtn.visible = false;
                    this.videoBtn.visible = true;
                } else {
                    this.shareBtn.visible = true;
                    this.videoBtn.visible = false;
                }
            }
        }

        playBtn(){
            this.setBtn();
            this.part3.alpha = 0;
            this.part3.mouseEnabled = false;
            this.part3.visible = true;
            Laya.Tween.to(this.part3,{alpha:1},this.animTime * 14,Laya.Ease.cubicIn,Laya.Handler.create(this,function(){
                this.part3.mouseEnabled = true;
            }));
        }

        onfreeAgainClick() {
            this.again = 1;
            EventMgr.getInstance().postEvent("FINISH_GAME_FREE_AGAIN");
            this.close();
        }

        onSureClick() {
            EventMgr.getInstance().postEvent("FINISH_GAME_TO_TRY");
            this.close();
        }

        onVideoClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.FishReward);
        }

        onShareClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.FishReward);
        }

        onGetClick() {
            EventMgr.getInstance().postEvent("FINISH_GAME_TO_SERVER", [Constants.FetchType.DIRECT]);
            this.close();
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.FishReward) {
                EventMgr.getInstance().postEvent("FINISH_GAME_TO_SERVER", [Constants.FetchType.AD]);
                this.close();
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.FishReward) {
                EventMgr.getInstance().postEvent("FINISH_GAME_TO_SERVER", [Constants.FetchType.AD]);
                this.close();
            }
        }
    }

    class RewardItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(reward) {
            //支持所有的reward
            this.reward = reward;
            let icon_img = this.getChildByName("icon");
            let amount_text = this.getChildByName("amount");
            amount_text.text = Strings$1.dealFishNumberText(reward.amount, 1);
            if (reward.type == Constants.ItemType.Currency) {
                if (reward.id == Constants.RewardType.FISH) {
                    icon_img.skin = "topui/yugan1.png";
                } else if (reward.id == Constants.RewardType.STAR) {
                    icon_img.skin = "topui/xingxing1.png";
                } else if (reward.id == Constants.RewardType.DIAMOND) {
                    icon_img.skin = "topui/diamond.png";
                } else if (reward.id == Constants.RewardType.SHELL) {
                    icon_img.skin = "topui/beike1.png";
                } else if (reward.id == Constants.RewardType.MEDAL) {
                    icon_img.skin = "topui/huizhang1.png";
                } else if (reward.id == Constants.RewardType.BOOK) {
                    icon_img.skin = "topui/miji1.png";
                }
            } else if (reward.type == Constants.ItemType.NauticalFoods) {
                let foodConfig = ConfigMgr$1.getInstance().getItemById("NauticalFoods", reward.id);
                icon_img.skin = foodConfig.Icon;
            } else if (reward.type == Constants.ItemType.NauticalTools) {
                let toolConfig = ConfigMgr$1.getInstance().getItemById("NauticalTools", reward.id);
                icon_img.skin = toolConfig.Icon;
            }
            else if (reward.type == Constants.ItemType.Coupon){
                switch(reward.id){
                    case "ADINVITE": this.rewardSkin = "purchase/kitty.png";break;
                    case "SPEED": this.rewardSkin = "purchase/voucher_speed.png";break;
                    case "FISH": this.rewardSkin = "purchase/voucher_fish.png";break;
                    case "CATFOOD": this.rewardSkin = "purchase/voucher_catfood.png";break;
                }
            }
            this.rewardSkin = icon_img.skin;
        }

        showItemFly(fetchType) {
            if (this.reward && this.reward.type == Constants.ItemType.Currency) {
                let localPoint = new Laya.Point(this.width / 2, this.height / 2);
                localPoint = this.localToGlobal(localPoint);
                let type = Constants.FlyType.TYPE5;
                if (fetchType == Constants.FetchType.AD) {
                    type = Constants.FlyType.TYPE3;
                }
                if (this.reward.id == "FISH") {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, Number(this.reward.amount), type, localPoint.x, localPoint.y);
                } else {
                    ItemFlyMgr.getInstance().fly(this.reward.id, Number(this.reward.amount), Constants.FlyType.NORM, localPoint.x, localPoint.y);
                }
            }
        }

        showRewardTip() {
            if (this.reward == null || !this.visible) {
                return;
            }
            let str = "<img src='" + this.rewardSkin + "' style='width:55; height:55'></img>" + this.reward.amount;
            EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon59", [str]]);
        }
    }

    class FishNewCome extends Laya.Dialog {
        constructor() { 
            super(); 
        }

        onEnable(){
            this.fishId = this._param.reward[0];
            let fishConfig = ConfigMgr$1.getInstance().getItemById("FishingFishes", String(this.fishId));
            let shareReward = JSON.parse(fishConfig.ShareReward);
            let rewardType = shareReward.FirstVisitReward.id;
            let rewardAmount = shareReward.FirstVisitReward.amount;
            if(rewardType == Constants.RewardType.FISH){
                this.rewardImg.skin = "topui/yugan.png";
            }
            else if(rewardType == Constants.RewardType.STAR){
                this.rewardImg.skin = "topui/xingxing.png";
            }
            this.rewards.visible=false;
            this.rewards.text = "+" + Strings$1.dealFishNumberText(rewardAmount);
            this.icon.skin = fishConfig.img;
            this.name.text = Strings$1.get(fishConfig.name);
            this.close1.on(Laya.Event.CLICK,this,this.close);
            this.share.visible=false;
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.words.text = "       " + Strings$1.get("tid#TipsCommon57");
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }

        onShareClick(){
            SdkHelper.shareAppMessage(Constants.ShareId.FishNewCome);
        }

        doneS(shareId) {
            if(shareId == Constants.ShareId.FishNewCome) {
                let handler=Laya.Handler.create(this,this.flyYugan);
                EventMgr.getInstance().postEvent("SERVICE_PET_SHAREGETREWARD",[this.fishId,Constants.PeacockShareType.FishNewCome,handler] );
                this.close();
            }
        }

        flyYugan(res){
            let num = res.rewards.rewards[0].amount;
            let rewardType = res.rewards.rewards[0].id;
            let x = Laya.stage.width/2;
            let y = Laya.stage.height/2;
            ItemFlyMgr.getInstance().fly(rewardType, num,Constants.FlyType.LW,x-33, y-24);
        }

        fail(shareId){
            if (shareId == Constants.ShareId.FishNewCome) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["FishNewCome_shareId_fail"]);
            }
        }
    }

    /**
     * 确认购买
     */
    class FoodLevelUpPop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.confirm = this.getChildByName("confirm");
            this.confirm.on(Laya.Event.CLICK, this, this.onConfirmFreeClick);
            this.confirm.visible = true;
            this.sbcj = this.getChildByName("sbcj");
            this.sbcj.on(Laya.Event.CLICK, this, this.onSbcjClick);
            this.vbcj = this.getChildByName("vbcj");
            this.vbcj.on(Laya.Event.CLICK, this, this.onVbcjClick);
            this.vbcj.visible = false;
            this.sbcj.visible = false;
            let anim = new Spine();
            anim.loadAnim("anim3/buchajia.sk");
            anim.playAnim("buchajia");
            anim.pos(this.vbcj.width/2,this.vbcj.height/2 + 30);
            this.vbcj.addChild(anim);
            this.showInfo(GameContext.CurrentFoodListId);
            this.highest.text = Strings$1.get("tid#UI_008");
            this.timeLabel.text = Strings$1.get("tid#UI_002");
            this.fishLabel.text = Strings$1.get("tid#UI_003");
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onAdClick);
            EventMgr.getInstance().registEvent("BCJ_VIDEO_CLICK", this, this.onVbcjClick);
            //this.reason.text = Strings.get("tid#UI_103");

            let animbuy = new Spine();
            animbuy.loadAnim("anim3/buyfish.sk");
            animbuy.playAnim("1");
            animbuy.pos(this.buyfish.width / 2, 80);
            this.bofang=0;
            this.buyfish.addChild(animbuy);
            this.buyfish.on(Laya.Event.MOUSE_DOWN,this,function(){
                animbuy.playAnim("2",false);
            });
            this.buyfish.on(Laya.Event.MOUSE_UP,this,function(){
                animbuy.playAnim("1");
            });
            this.buyfish.on(Laya.Event.MOUSE_OUT,this,function(){
                animbuy.playAnim("1");
            });
            this.buyfish.on(Laya.Event.CLICK,this,function(){
                this.goNeigo();
            });
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onAdClick);
            EventMgr.getInstance().removeEvent("BCJ_VIDEO_CLICK", this, this.onVbcjClick);
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
                EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_COOK_ITEM", [true]);
            }
        }

        onSbcjClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.FoodLevelUp);
        }

        onVbcjClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.FoodLevelUp);
        }

        showInfo(foodListId) {
            if (foodListId == null) {
                this.close();
                return;
            }
            let food = FoodSystem$1.getCurFoodInList(foodListId);
            this.title.text = Strings$1.get(food.getName());
            this.desc1.text = Strings$1.get(food.getDesc1());
            this.time.text = food.getCookingTime() + "s";
            this.fish.text = food.getIncome();
            this.icon.skin = food.getShopItems();

            let nextFood = FoodSystem$1.getNextLevelUpFoodInList(foodListId);

            if (nextFood == null) {
                this.confirm.disabled = true;
                this.btnFish.visible = this.cost.visible = this.yanjiu.visible = false;
                this.highest.visible = true;
                this.timeArrow.visible = false;
                this.fishArrow.visible = false;
                this.nextTime.visible = false;
                this.nextFish.visible = false;
                this.starLabel.visible = false;
                this.star.visible = false;
                this.starImage.visible = false;
            } else {
                this.confirm.disabled = false;
                this.btnFish.visible = this.cost.visible = this.yanjiu.visible = true;
                this.highest.visible = false;
                this.cost.text = Strings$1.dealFishNumberText(nextFood.getCost(), 1);
                this.setGray(FoodSystem$1.checkLevelUpUnAvailable(foodListId));
                this.timeArrow.visible = true;
                this.fishArrow.visible = true;
                this.nextTime.visible = true;
                this.nextFish.visible = true;
                this.fishArrow.x = this.fish.x + this.fish.width + 5;
                this.nextFish.x = this.fishArrow.x + this.fishArrow.width * this.fishArrow.scaleX + 5;
                this.timeArrow.x = this.time.x + this.time.width + 5;
                this.nextTime.x = this.timeArrow.x + this.timeArrow.width * this.timeArrow.scaleX + 5;
                this.nextFish.text = nextFood.getIncome();
                this.nextTime.text = nextFood.getCookingTime() + "s";
                this.starLabel.visible = true;
                this.star.visible = true;
                this.star.text = String(nextFood.getLimit());
                this.starImage.visible = true;
                this.checkAdBtnShow(foodListId);
                if (!this.confirm.gray) {
                    if (food.isLocked()) {
                        this.checkShowWeakGuideAnim("Food_Unlock");
                    } else {
                        this.checkShowWeakGuideAnim("Food_LvUp");
                    }
                }
            }

            if (food.isLocked()) {
                this.starLabel.text = Strings$1.get("tid#UI_004");
                this.yanjiu.text = Strings$1.get("tid#UI_006");
                this.timeArrow.visible = false;
                this.fishArrow.visible = false;
                this.nextTime.visible = false;
                this.nextFish.visible = false;
            } else {
                this.starLabel.text = Strings$1.get("tid#UI_005");
                this.yanjiu.text = Strings$1.get("tid#UI_007");
            }

            let scalex = 180 / this.icon.width;
            let scaley = 180 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }

            this.btnFish.x = this.confirm.x + (this.confirm.width * this.confirm.scaleX - this.btnFish.width * this.btnFish.scaleX - this.cost.width - this.yanjiu.width - 10) / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
            this.cost.x = this.btnFish.x + this.btnFish.width * this.btnFish.scaleX + 5;
            this.yanjiu.x = this.cost.x + this.cost.width + 5;

            this.foodListId = foodListId;
        }

        setGray(ret) {
            if (ret) {
                this.confirm.gray = true;
                this.confirm.disabled = true;
                this.reason.visible = true;
                this.reason.text = ret;
                this.reason.x = 563.5 - this.reason.width / 2;
            } else {
                this.confirm.gray = false;
                this.confirm.disabled = false;
                this.reason.visible = false;
            }
        }

        onOpened(param) {
            if (GuideSystem.isGuiding()) {
                let guide = GuideSystem.getGuide("GuideGroup_3");
                if (guide) {
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
                }
            }
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/shopPop.scene");
        }

        onConfirmFreeClick(){
            if(this.vbcj.visible){
                Laya.Dialog.close("ui/Makeup.scene");
                Laya.Dialog.open("ui/Makeup.scene",false);
                return ;
            }
            else {
                this.onConfirmClick();
            }
        }

        onConfirmClick() {
            this.close();

            let foodListId = this.foodListId;
            let food = FoodSystem$1.getCurFoodInList(foodListId);
            if (food.isLocked()) {
                TaskMainSystem.addProgress("FOOD_UNLOCK", 1);
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.YANJIU);
            } else {
                TaskMainSystem.addProgress("FOOD_LVUP", 1);
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.GAILIANG);
            }
            FoodSystem$1.foodLevelUp(foodListId, Constants.FetchType.DIRECT);
            SoundManager.getInstance().play("Sound_buyItemSuccess");
        }

        checkAdBtnShow(foodListId) {
            let needFish = FoodSystem$1.checkAdShowFish(foodListId) ? FoodSystem$1.checkAdShowFish(foodListId)[0] : 0;
            let costFish = FoodSystem$1.checkAdShowFish(foodListId) ? FoodSystem$1.checkAdShowFish(foodListId)[1] : 0;
            let yugan = costFish - needFish;
            let adShoppingCondition = Const.get("ADShoppingCondition");
            let maxAd = Number(Const.get("ADShoppingTimes"));
            let adShoppingRange = JSON.parse(Const.get("ADShoppingRange"));
            let showAd = false;
            let buildingNum = BuildingDataMgr.getInstance().getNowBuildingNumber();
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].BUILDING_AMOUNT[0];
                let max = adShoppingRange[i].BUILDING_AMOUNT[1];
                if (max < 0) {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                } else {
                    if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan >= costFish * adShoppingCondition) {
                        showAd = true;
                        break;
                    }
                }
            }
            let items = PetSystem.getItemBySpecialType(4);
            let itemAddTimes = 0;
            items.forEach(item => {
                itemAddTimes += Number(LevelToySystem.getDataById(item.Id)[3]);
            });
            if (showAd && PlayerSystem.getMakeUpTimes() < maxAd + itemAddTimes) {
                if (SdkHelper.hasVideo()) {
                    this.vbcj.visible = true;
                    this.buyfish.visible=false;
                    this.sbcj.visible = false;
                    this.confirm.visible = true;
                    this.confirm.gray = false;
                    this.confirm.disabled = false;
                } else {
                    this.vbcj.visible = false;
                    this.sbcj.visible = true;
                    let cost = this.sbcj.getChildByName("cost");
                    cost.text = this.cost.text + "研究";
                    cost.text = Strings$1.get("tid#Btn_020");
                    let adImage = this.sbcj.getChildByName("adImage");
                    let fish = this.sbcj.getChildByName("btnFish");
                    fish.x = 15 + (adImage.x - fish.width - cost.width - 20) / 2;
                    cost.x = fish.x + fish.width * fish.scaleX ;
                    this.confirm.visible = false;
                }
            }else if(FoodSystem$1.checkLevelUpUnAvailable(foodListId)&&FoodSystem$1.checkLevelUpStarEnough(foodListId)){
                this.buyfish.visible=true;
            }
                else {
                this.buyfish.visible=false;
                this.vbcj.visible = false;
                this.sbcj.visible = false;
                this.confirm.visible = true;
            }
        }

        done(shareId) {
            if (shareId == Constants.ShareId.FoodLevelUp) {
                this.close();
                let food = FoodSystem$1.getCurFoodInList(this.foodListId);
                if (food.isLocked()) {
                    TaskMainSystem.addProgress("FOOD_UNLOCK", 1);
                } else {
                    TaskMainSystem.addProgress("FOOD_LVUP", 1);
                }
                FoodSystem$1.foodLevelUp(this.foodListId, Constants.FetchType.AD);
                SoundManager.getInstance().play("Sound_buyItemSuccess");
            }
        }

        onAdClick(shareId) {
            if (shareId == Constants.ShareId.FoodLevelUp) {
                this.close();
                let food = FoodSystem$1.getCurFoodInList(this.foodListId);
                if (food.isLocked()) {
                    TaskMainSystem.addProgress("FOOD_UNLOCK", 1);
                } else {
                    TaskMainSystem.addProgress("FOOD_LVUP", 1);
                }
                FoodSystem$1.foodLevelUp(this.foodListId, Constants.FetchType.AD);
                SoundManager.getInstance().play("Sound_buyItemSuccess");
            }
        }

        checkShowWeakGuideAnim(guideEvent) {
            if (!GuideSystem.isGuiding() && TaskMainSystem.getIsTasking()) {
                let task = TaskMainSystem.getCurrentData();
                if (task.getGuide() == guideEvent && !TaskMainSystem.checkEnought()) {
                    if (this.weakAnim == null || this.weakAnim.destroyed) {
                        this.weakAnim = new Spine();
                        this.weakAnim.loadAnim("anim1/xinshouyindao.sk");
                        this.weakAnim.playAnim("zhuazi2", true);
                        this.weakAnim.x = this.confirm.x + this.confirm.width * this.confirm.scaleX / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
                        this.weakAnim.y = this.confirm.y + this.confirm.height * this.confirm.scaleX / 2 - this.confirm.anchorY * this.confirm.height * this.confirm.scaleY;
                        this.weakAnim.zOrder = 5;
                        this.addChild(this.weakAnim);
                        EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_COOK_ITEM", [false]);
                        return;
                    }
                }
            }
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
            }
        }
        goNeigo() {
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene",true,[1]);
        }
    }

    class FortuneCatUI extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.ForturneCat,1);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onVideoCallBack);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doubleGet);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            this._yugan = this.getChildByName("yugan");
            this._yugan.value = SyncSystem.getCacheFish() * Number(Const.get("ADRewardRateLetterBox"));
            let cur = this.getChildByName("cur");
            let max = this.getChildByName("max");
            cur.text = Strings$1.dealFishNumberText(SyncSystem.getCacheFish());
            max.text = "/" + Strings$1.dealFishNumberText(SyncSystem.getCacheFishCapacity());
            max.x = cur.x + cur.width;
            let catTalk = this.getChildByName("catTalk");
            catTalk.text = Strings$1.get("tid#UI_167");
            let catAd = this.getChildByName("catAd");//广告
            catAd.on(Laya.Event.CLICK, this, this.onCatAdClick);

            let catSh = this.getChildByName("catSh");//分享
            catSh.on(Laya.Event.CLICK, this, this.onCatShClick);
            let overseasGetReward = this.getChildByName("overseasGetReward");//直接领取，海外按钮
            overseasGetReward.on(Laya.Event.CLICK, this, this.onOneGetClick);
            // let oneGet = this.getChildByName("oneGet");//直接领取
            // oneGet.on(Laya.Event.CLICK, this, this.onOneGetClick);
            if(SdkHelper.hasVideo()) {
                catSh.visible = false;
                catAd.visible = true;
            }else {
                catSh.visible = true;
                catAd.visible = false;
            }
            // Laya.timer.once(2000,this,function(){
            //     oneGet.visible=true;
            // })
            //分享屏蔽
            // this.subscriptionBox = this.getChildByName("subscriptionBox");
            // this.subscriptionBox.setValue(Constants.WxMessageTmplIdKeys.FORTUNE_CAT, Strings.get("tid#WXMessageTip"));
            //  oneGet.text = Strings.get("tid#Btn_019");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onVideoCallBack);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doubleGet);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.ForturneCat, 0);
            this._getReward = null;
        }

        onCloseClick(){
            this.close();
        }

        onCatAdClick() {
            if (this._getReward) {
                return;
            }
            SdkHelper.showRewardVideoAd(Constants.ShareId.FortuneCat);
        }
        onCatShClick() {
            if (this._getReward) {
                return;
            }
            SdkHelper.shareAppMessage(Constants.ShareId.FortuneCat);
        }

        onVideoCallBack(shareId) {
            if (shareId == Constants.ShareId.FortuneCat) {
                this.doubleGet(shareId);
            }
        }

        doubleGet(shareId) {
            if (shareId == Constants.ShareId.FortuneCat) {
                Laya.timer.frameOnce(2, this, function () {
                    this.fetch(Constants.FetchType.AD);
                });
            }
        }
        onOneGetClick() {
            if (this._getReward) {
                return;
            }
            this.fetch(Constants.FetchType.DIRECT);
        }

        fetch(fetchType) {
            let cacheFish =  SyncSystem.getCacheFish();
            if (cacheFish > 0) {
                SoundManager.getInstance().play("Sound_clickReward");
                this._getReward = true;
            }
            let localPoint = new Laya.Point(this._yugan.x , this._yugan.y );
            localPoint = this.localToGlobal(localPoint);
            let type = Constants.FlyType.TYPE2;
            if (fetchType == Constants.FetchType.AD) {
                let ad = Number(Const.get("ADRewardRateLetterBox"));
                cacheFish = cacheFish * ad;
                type = Constants.FlyType.TYPE3;
            }
            SyncSystem.fetchLetterBox(fetchType, Laya.Handler.create(this, function () {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_tip_get_success"]);
                ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, cacheFish, type, localPoint.x, localPoint.y, Laya.Handler.create(this, function () {
                    this.close();
                }));
            }));
            EventMgr.getInstance().postEvent("FETCH_LETTER_BOX_UPDATA_LIMIT");
        }
    }

    class FriendSubscribe extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            let subscribe = this.getChildByName("subscribe");
            subscribe.on(Laya.Event.CLICK, this, this.onSubscribeClick);
            let share = this.getChildByName("share");
            share.on(Laya.Event.CLICK, this, this.onShareClick);
            share.visible = false;
            subscribe.visible = false;
            this.zOrder = 1;
            let fLabel = this.getChildByName("fLabel");
            SdkHelper.checkSubscriptions(Constants.WxMessageType.INTERACTIVE, Laya.Handler.create(this, function (res) {
                if (res) {
                    fLabel.text = Strings$1.get("tid#rankUItips7");
                    share.x = 692;
                    subscribe.visible = false;
                    share.visible = true;
                } else {
                    fLabel.text = Strings$1.get("tid#rankUItips11");
                    share.x = 932;
                    subscribe.visible = true;
                    share.visible = true;
                }
            }));
        }

        onSubscribeClick(event) {
            SdkHelper.requestSubscribeSystemMessage(Constants.WxMessageType.INTERACTIVE);
            this.close();
        }

        onShareClick(event) {
            SdkHelper.shareAppMessage(Constants.ShareId.FriendShare);
            this.close();
        }

        onDisable() {
            EventMgr.getInstance().postEvent("OPENVIEW_RANK_CLOSE_TOUCHENABLED");
        }

    }

    class GlobalSharePop extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            this.shareId = this._param[0];
            this.shareText.text = this._param[1];
            this.shareTitle = this._param[2];
            this.shareImage = this._param[3];
            this.shareBtn.on(Laya.Event.CLICK, this, this.share);
            this.zOrder = 1;
            let closeBtn = this.getChildByName("closeBtn");
            closeBtn.on(Laya.Event.CLICK, this, this.close);
        }

        share() {
            SdkHelper.shareAppMessage(this.shareId, this.shareTitle, this.shareImage);
        }

    }

    class GuessGamePop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            // bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.quit.on(Laya.Event.CLICK, this, this.onQuitClick);
            this.guessBox = this.getChildByName("guessBox");
            this.animBox = this.guessBox.getChildByName("animBox");
            this.catAnim = new Laya.Skeleton();
            this.catAnim.load("anim2/caiquannpc.sk", Laya.Handler.create(this, function () {
                this.catAnim.play("npc", true);
                this.animBox.addChild(this.catAnim);
            }));
            this.coinBox1 = this.guessBox.getChildByName("coinBox1");
            this.coinBox2 = this.guessBox.getChildByName("coinBox2");
            this.coinBox3 = this.guessBox.getChildByName("coinBox3");
            this.coinBox1.on(Laya.Event.CLICK, this, this.onCoinBoxClick,[1]);
            this.coinBox2.on(Laya.Event.CLICK, this, this.onCoinBoxClick,[2]);
            this.coinBox3.on(Laya.Event.CLICK, this, this.onCoinBoxClick,[3]);
            this.anim1 = new Spine();
            this.anim1.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim1.playAnim("jiantoudonghua1",true);
            this.anim1.scale(2,2);
            this.anim1.pos(100,50);
            this.coinBox1.addChild(this.anim1);
            this.anim1.visible = false;
            this.anim2 = new Spine();
            this.anim2.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim2.playAnim("jiantoudonghua1",true);
            this.anim2.scale(2,2);
            this.anim2.pos(100,50);
            this.coinBox2.addChild(this.anim2);
            this.anim2.visible = false;
            this.anim3 = new Spine();
            this.anim3.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim3.playAnim("jiantoudonghua1",true);
            this.anim3.scale(2,2);
            this.anim3.pos(100,50);
            this.coinBox3.addChild(this.anim3);
            this.anim3.visible = false;

            this.startBtn = this.guessBox.getChildByName("startBtn");
            this.startBtn.label=Strings$1.get('tid#Btn_037');
            this.startBtn.on(Laya.Event.CLICK, this, this.onStartCick);
            this.guessText = this.guessBox.getChildByName("guessText");

            this.resultBox = this.getChildByName("resultBox");
            this.winBox = this.resultBox.getChildByName("winBox");
            this.rewardNum = this.winBox.getChildByName("rewardNum");
            this.rewardText = this.winBox.getChildByName("rewardText");
            this.rewardImage = this.winBox.getChildByName("rewardImage");
            this.getBtn = this.winBox.getChildByName("getBtn");
            this.getBtn.on(Laya.Event.CLICK, this, this.onGetBtnClick);
            this.shareDoubleGetBtn = this.winBox.getChildByName("shareBtn");
            this.shareDoubleGetBtn.on(Laya.Event.CLICK, this, this.onShareDoubleClick);
            this.videoDoubleGetBtn = this.winBox.getChildByName("videoBtn");
            this.videoDoubleGetBtn.on(Laya.Event.CLICK, this, this.onVideoDoubleClick);
            this.moreShareBtn.on(Laya.Event.CLICK, this, this.onMoreShareBtnClick);
            this.moreVideoBtn.on(Laya.Event.CLICK, this, this.onMoreAdBtnClick);
            this.close1.on(Laya.Event.CLICK, this, this.onClose1Click);
            let winLabel = this.winBox.getChildByName("winLabel");
            winLabel.text = Strings$1.get("tid#UI_021");

            this.failBox = this.resultBox.getChildByName("failBox");
            let overBtn = this.failBox.getChildByName("overBtn");
            overBtn.on(Laya.Event.CLICK, this, this.onOverBtnClick);
            overBtn.label=Strings$1.get('tid#Btn_028');
            this.adBox = this.resultBox.getChildByName("adBox");
            let adBackBtn = this.adBox.getChildByName("adBackBtn");
            adBackBtn.on(Laya.Event.CLICK, this, this.onBackBtnClick);
            this.adBtn = this.adBox.getChildByName("adBtn");
            this.adBtn.on(Laya.Event.CLICK, this, this.onAdBtnClick);
            this.shareBtn = this.adBox.getChildByName("shareBtn");
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareBtnClick);
            this.rewardText.text = Strings$1.get("tid#UI_100");
            this.next = this.failBox.getChildByName("next");
             this.next.text = Strings$1.get("tid#UI_017");
            // this.nextVideo = this.adBox.getChildByName("nextVideo");
            // this.nextVideo.text = Strings.get("tid#UI_019");
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.quit.visible = true;
            this.playTimes = 0;
            this._choose = null;
            this.resetStart();
            this.minigameData = this._param;
            let reward_g = this.minigameData.getReward();
            if (reward_g.WinReward[0].id == Constants.RewardType.DIAMOND) {
                this.rewardImage.skin = "topui/zuanshi.png";
            } else if (reward_g.WinReward[0].id == Constants.RewardType.STAR) {
                this.rewardImage.skin = "topui/xingxing.png";
            }
            this.rewardNum.text = Strings$1.dealFishNumberText(reward_g.WinReward[0].amount + (reward_g.WinRewardRate ? reward_g.WinRewardRate : 0) * PetSystem.getMaxValueBase());
            this.rewardText.x = (this.winBox.width - this.rewardText.width - this.rewardNum.width - this.rewardImage.width * this.rewardImage.scaleX) / 2;
            this.rewardImage.x = this.rewardText.x + this.rewardText.width;
            this.rewardNum.x = this.rewardImage.x + this.rewardImage.width * this.rewardImage.scaleX;
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            if (SdkHelper.hasVideo()) {
                this.shareBtn.visible = false;
                this.adBtn.visible = true;
                this.shareDoubleGetBtn.visible = false;
                this.videoDoubleGetBtn.visible = true;
            } else {
                this.shareBtn.visible = true;
                this.adBtn.visible = false;
                this.shareDoubleGetBtn.visible = true;
                this.videoDoubleGetBtn.visible = false;
            }
        }

        onStartCick() {
            this.startBtn.visible = false;
            this.quit.visible = false;
            this.startPlay();
        }

        onQuitClick(){
            if(this.playTimes > 0){
                this.gameComplete();
            }
            else {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 3);
                this.close();
            }
        }

        onClose1Click(){
            Laya.Tween.to(this.adGuessMoreBox,{scaleX:0,scaleY:0},100);
        }

        onDisable() {
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
        }

        onCoinBoxClick(num){
            if(this.playTimes == 0){
                switch(num){
                    case 1:this.onCoinBox1Click();break;
                    case 2:this.onCoinBox2Click();break;
                    case 3:this.onCoinBox3Click();break;
                }
            }
            else {
                this._choose = num;
                this.adGuessMoreBox.visible = true;
                this.adGuessMoreBox.scale(0,0);
                Laya.Tween.to(this.adGuessMoreBox,{scaleX:1,scaleY:1},300,Laya.Ease.backOut);
                if (SdkHelper.hasVideo()) {
                    this.moreVideoBtn.visible = true;
                    this.moreShareBtn.visible = false;
                    this.label1.text = Strings$1.get("tid#UI_208");
                } else {
                    this.moreShareBtn.visible = true;
                    this.moreVideoBtn.visible = false;
                    this.label1.text = Strings$1.get("tid#UI_210");
                }
            }
        }

        onCoinBox1Click() {
            this.playTimes++;
            this._choose = 1;
            this._resultType = Constants.MiniGameResult.Fail;
            this.coinBox1.mouseEnabled = false;
            this.coinBox2.mouseEnabled = false;
            this.coinBox3.mouseEnabled = false;
            this.anim1.visible = false;
            this.coinBox1.upPot(Laya.Handler.create(this, function () {
                Laya.timer.once(1000, this, this.afterChoose);
            }));
        }

        onCoinBox2Click() {
            this.playTimes++;
            this._choose = 2;
            this._resultType = Constants.MiniGameResult.Win;
            this.coinBox1.mouseEnabled = false;
            this.coinBox2.mouseEnabled = false;
            this.coinBox3.mouseEnabled = false;
            this.anim2.visible = false;
            this.coinBox2.upPot(Laya.Handler.create(this, function () {
                Laya.timer.once(1000, this, this.afterChoose);
            }));
        }

        onCoinBox3Click() {
            this.playTimes++;
            this._choose = 3;
            this._resultType = Constants.MiniGameResult.Fail;
            this.coinBox1.mouseEnabled = false;
            this.coinBox2.mouseEnabled = false;
            this.coinBox3.mouseEnabled = false;
            this.anim3.visible = false;
            this.coinBox3.upPot(Laya.Handler.create(this, function () {
                Laya.timer.once(1000, this, this.afterChoose);
            }));
        }

        afterChoose(){
            if(this._resultType == Constants.MiniGameResult.Win || this.playTimes > 0){
                this.gameComplete();
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon77"); 
                if(this._choose != 1){
                    this.coinBox1.mouseEnabled = true;
                }
                if(this._choose != 2){
                    this.coinBox2.mouseEnabled = true;
                }
                if(this._choose != 3){
                    this.coinBox3.mouseEnabled = true;
                }
                this.quit.visible = true;
            }
        }

        randomPot() {
            let arr = [1, 2, 3];
            arr.sort(function () {
                return 0.5 - Math.random();
            });
            let arr1 = [arr[0]];
            let arr2 = [arr[1]];
            let arr3 = [arr[2]];
            let index0 = Utils.randomExt(3);
            let index1 = (index0 + 1) % 3;
            let index2 = (index0 + 2) % 3;
            let a = 0;
            for (let i = 0; i < 9; i++) {
                index0 = Utils.randomExt(3);
                index1 = (index0 + 1) % 3;
                index2 = (index0 + 2) % 3;
                a = arr[index1];
                arr[index1] = arr[index2];
                arr[index2] = a;
                arr1.push(arr[0]);
                arr2.push(arr[1]);
                arr3.push(arr[2]);
            }
            this.coinBox1.startRunWithArr(arr1);
            this.coinBox2.startRunWithArr(arr2, Laya.Handler.create(this, this.randomPotComplete));
            this.coinBox3.startRunWithArr(arr3);
        }

        randomPotComplete() {
            this.startBtn.visible = false;
            this.guessText.visible = true;
            this.coinBox1.mouseEnabled = true;
            this.coinBox2.mouseEnabled = true;
            this.coinBox3.mouseEnabled = true;
            this.anim1.visible = true;
            this.anim2.visible = true;
            this.anim3.visible = true;
        }

        startPlay() {
            this.coinBox1.upPot();
            this.coinBox2.upPot(Laya.Handler.create(this, this.downPots));
            this.coinBox3.upPot();
        }

        downPots() {
            Laya.timer.once(1000, this, function () {
                this.coinBox1.downPot();
                this.coinBox2.downPot(Laya.Handler.create(this, this.randomPot));
                this.coinBox3.downPot();
            });
        }

        gameComplete() {
            this.quit.visible = false;
            if (this._resultType == Constants.MiniGameResult.Win) {
                this.guessBox.visible = false;
                Laya.Tween.clearAll(this.resultBox);
                Laya.Tween.from(this.resultBox, { scaleX: 0, scaleY: 0 }, 200);
                this.resultBox.visible = true;
                this.winBox.visible = true;
            } else {
                this.guessBox.visible = false;
                Laya.Tween.clearAll(this.resultBox);
                Laya.Tween.from(this.resultBox, { scaleX: 0, scaleY: 0 }, 200);
                this.resultBox.visible = true;           
                this.failBox.visible = true;
            }
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.GuessGame) {
                this.playTimes++;
                this.guessBox.visible = true;
                this.resultBox.visible = false;
                this.winBox.visible = false;
                this.failBox.visible = false;
                this.adBox.visible = false;
                if (this._choose == 1) {
                    this.coinBox2.mouseEnabled = this.coinBox3.mouseEnabled = true;
                } else if (this._choose == 3) {
                    this.coinBox2.mouseEnabled = this.coinBox1.mouseEnabled = true;
                }
            } else if (shareId == Constants.ShareId.GuessGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_coin_reward_success"]);
                this.close();
                Laya.Dialog.close("ui/MiniGameTalkPop.scene");
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
            }
            else if(shareId == Constants.ShareId.GuessGameMore){
                switch(this._choose){
                    case 1:this.onCoinBox1Click();break;
                    case 2:this.onCoinBox2Click();break;
                    case 3:this.onCoinBox3Click();break;
                }
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.GuessGame) {
                this.playTimes++;
                this.guessBox.visible = true;
                this.resultBox.visible = false;
                this.winBox.visible = false;
                this.failBox.visible = false;
                this.adBox.visible = false;
                if (this._choose == 1) {
                    this.coinBox2.mouseEnabled = this.coinBox3.mouseEnabled = true;
                } else if (this._choose == 3) {
                    this.coinBox2.mouseEnabled = this.coinBox1.mouseEnabled = true;
                }
            } else if (shareId == Constants.ShareId.GuessGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_coin_reward_success"]);
                this.close();
                Laya.Dialog.close("ui/MiniGameTalkPop.scene");
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
            }
            else if(shareId == Constants.ShareId.GuessGameMore){
                switch(this._choose){
                    case 1:this.onCoinBox1Click();break;
                    case 2:this.onCoinBox2Click();break;
                    case 3:this.onCoinBox3Click();break;
                }
            }
        }

        onAdBtnClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.GuessGame);
        }

        onShareBtnClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.GuessGame);
        }

        onMoreAdBtnClick() {
            this.adGuessMoreBox.visible = false;
            SdkHelper.showRewardVideoAd(Constants.ShareId.GuessGameMore);
        }

        onMoreShareBtnClick() {
            this.adGuessMoreBox.visible = false;
            SdkHelper.shareAppMessage(Constants.ShareId.GuessGameMore);
        }

        onBackBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 2, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onGetBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_coin_reward_success"]);
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onOverBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 0, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        showFly(fetchType, res) {
            let point = new Laya.Point(this.rewardImage.x, this.rewardImage.y);
            point = this.rewardImage.parent.localToGlobal(point);
            let rewards = res.rewards.rewards;
            for (let i in rewards) {
                if (rewards[i].amount <= 0) {
                    continue;
                }
                if (rewards[i].id == Constants.RewardType.FISH) {
                    if (fetchType == Constants.FetchType.AD) {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE3, point.x, point.y);
                    } else {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE5, point.x, point.y);
                    }
                } else {
                    ItemFlyMgr.getInstance().fly(rewards[i].id, rewards[i].amount, Constants.FlyType.ZCM, point.x, point.y);
                }
            }
        }

        resetStart() {
            this.coinBox1.reset(1);
            this.coinBox2.reset(2);
            this.coinBox3.reset(3);
            this.coinBox2.showCoin(true);
            this.coinBox1.mouseEnabled = false;
            this.coinBox2.mouseEnabled = false;
            this.coinBox3.mouseEnabled = false;

            this.guessText.visible = false;
            this.startBtn.visible = true;
            this.guessBox.visible = true;

            this.resultBox.visible = false;
            this.winBox.visible = false;
            this.failBox.visible = false;
            this.adBox.visible = false;

            //0:平局,1:负,2:胜
            this._resultType = null;
        }

        onCloseClick() {
            if (this.playTimes == 0) {
                this.close();
            }
        }

        onShareDoubleClick(shareId) {
            SdkHelper.shareAppMessage(Constants.ShareId.GuessGameDouble);
        }

        onVideoDoubleClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.GuessGameDouble);
        }

    }

    class GuessCoinBox extends Laya.Box {
        constructor() {
            super();
        }

        onAwake() {
        }

        onEnable() {
        }

        showCoin(visible) {
            this.getChildByName("coin").visible = visible;
        }

        upPot(handler) {
            let pot = this.getChildByName("pot");
            Laya.Tween.clearAll(pot);
            if (handler != null) {
                Laya.Tween.to(pot, { y: 2 }, 200, null, handler);
            } else {
                Laya.Tween.to(pot, { y: 2 }, 200);
            }
        }

        downPot(handler) {
            let pot = this.getChildByName("pot");
            Laya.Tween.clearAll(pot);
            if (handler != null) {
                Laya.Tween.to(pot, { y: 63 }, 200, null, handler);
            } else {
                Laya.Tween.to(pot, { y: 63 }, 200);
            }
        }

        setPointIndex(index) {
            this.pointIndex = index;
            if (index == 1) {
                this.x = 56;
            } else if (index == 2) {
                this.x = 290;
            } else {
                this.x = 514;
            }
            this.y = 357;
        }

        reset(index) {
            this.setPointIndex(index);
            this.getChildByName("pot").y = 63;
            this.showCoin(false);
            this.pointIndexs = [];
            this.overHandler = null;
        }

        tweenToPoint(index, time) {
            let toX = this.getIndexPointX(index);
            let midX = (this.x + toX) / 2;
            time /= 2;
            let toY;
            this.zOrder = index;
            if (this.pointIndex > index) {
                toY = 280;
            } else if (this.pointIndex < index) {
                toY = 414;
            } else {
                toY = 357;
            }
            Laya.Tween.clearAll(this);
            Laya.Tween.to(this, { x: midX, y: toY }, time);
            Laya.Tween.to(this, { x: toX, y: 357 }, time, null, Laya.Handler.create(this, function () {
                this.setPointIndex(index);
                this.tweenStart();
            }), time);

        }

        getIndexPointX(index) {
            if (index == 1) {
                return 56;
            } else if (index == 2) {
                return 290;
            } else {
                return 514;
            }
        }

        startRunWithArr(indexs, overHandler) {
            this.pointIndexs = indexs;
            this._midIndex = Math.floor(indexs.length / 2);
            this.overHandler = overHandler;
            this.currentIndex = 0;
            this.tweenStart();
        }

        tweenStart() {
            if (this.currentIndex >= this.pointIndexs.length) {
                if (this.overHandler != null) {
                    this.overHandler.run();
                }
                return;
            }
            let currentPointIndex = this.pointIndexs[this.currentIndex];
            let time = (Math.abs(this._midIndex - this.currentIndex)) * 80 + 230;
            this.currentIndex++;
            this.tweenToPoint(currentPointIndex, time);
        }
        onDestroy(){
            Laya.Tween.clearAll(this);
        }
    }

    function sortByOrder(a,b)
    {
        return a.Rank-b.Rank;
    }

    class HomePetPop extends Laya.Dialog {

        onOpened() {
            this.checkGuide("GuideGroup_14");
        }

        onEnable() {
            this.noLabel.text = Strings$1.get("tid#UI_010");

            this.bg_panel.on(Laya.Event.CLICK, this, this.doClose);

            this.confirm.on(Laya.Event.CLICK, this, this.changePets);

            EventMgr.getInstance().registEvent("HOME_PET_SELECTED_CHANGE" , this, this.changeSelectPetId);

            this.hide();

            this.initData();

            this.showPetList();
            this.title.text = Strings$1.get("tid#UI_104");
            this.confirm.label = Strings$1.get("tid#Btn_009");
            this.unlock_label.text = Strings$1.get("tid#UI_105");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("HOME_PET_SELECTED_CHANGE" , this, this.changeSelectPetId);
            Laya.Tween.clearAll(this);
        }

        changePets() {
            let pets = PetSystem.PetsAdopted;
            let arr = new Array();
            for(const pet of pets) {
                if(pet.homeStatus!=pet.fakeInHome) {
                    arr.push({"petId":pet.getId(), "inHome":pet.fakeInHome});
                }
            }
            if(arr.length>0) {
                PetSystem.changePetInHome(arr);
            }
            for(const pet of pets) {
                pet.fakeInHome = null;
            }
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        hide() {
            this._curSelectedIndex = null;
        }

        initData() {
            let pets = PetSystem.PetsAdopted;
            for(const pet of pets) {
                pet.fakeInHome = null;
            }
        }

        setIcon(icon, skin) {
            icon.skin = skin;
            let scalex = 125 / icon.width;
            let scaley = 125 / icon.height;
            if (scalex <= scaley) {
                icon.scaleX = scalex;
                icon.scaleY = scalex;
            } else {
                icon.scaleX = scaley;
                icon.scaleY = scaley;
            }
        }

        calcPet(limit) {
            if(!limit)
                limit = PetSystem.HomePetsLimit;
            let pets = PetSystem.PetsFakeInHome;
            this.pet_count.text = Strings$1.get("tid#UI_009", [String(pets.length) + "/" + String(limit)]);
            let nextInfo = PetSystem.HomePetsNextLimitInfo;
            if(nextInfo) {
                this.unlock.visible = true;
                // this.unlock_label.visible = true;
                this.unlock_label.visible = false;
                this.unlock_star.visible = true;
                this.unlock.text = nextInfo.Star + "  "+Strings$1.get("tid#UI_106")+'+'+String(Number(nextInfo.Amount)-limit);
                this.unlock_star.x = this.unlock.x - this.unlock.width - this.unlock_star.width * this.unlock_star.scaleX - 10;
            }else {
                this.unlock.visible = false;
                this.unlock_label.visible = false;
                this.unlock_star.visible = false;
            }
        }

        loadList(){
            this.left_arrow = this.pets.getChildByName("left_arrow");
            this.right_arrow = this.pets.getChildByName("right_arrow");
            this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);

            this.calcPet();

            let index = 0;
            let pets = PetSystem.PetIdsAdopted;
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            function sortByRankOrder(a,b)
            {
                return (config.get(a).Rank)-(config.get(b).Rank);
               // return (config.get(a).Rank+PetSystem.getPetInfo(a).homeStatus?1000000:0)-(config.get(b).Rank+PetSystem.getPetInfo(b).homeStatus?1000000:0);
            }
            pets.sort(sortByRankOrder);
            this.array = new Array();
            for(const petId of pets) {
                this.array.push(petId);
            }
            this.list = this.petList;
            this.list.on(Laya.Event.MOUSE_UP,this,this.changeListItems);
            this.list.on(Laya.Event.MOUSE_OUT,this,this.changeListItems);
            this.list.scrollBar.rollRatio = 0;

            if (this.array.length > 0) {
                this.noBox.visible = false;
            } else {
                this.noBox.visible = true;
            }
            this._index = index;
            this.startIndex = 0;
            this.endIndex =  this.array.length;
            let showList = this.array.slice(this.startIndex, this.endIndex);
            // console.debug("changeListItems", this.startIndex, this.endIndex, showList, this.array);
            this.list.array = showList;
        }

        adjustList(){
            let w = this.list.getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w+s) + 0.5 | 0;
            Laya.Tween.to(this.list.scrollBar,{value:this._index*(w+s)},100);
        }

        showPetList() {
            this.loadList();
            this.changeListItems();
        }

        changeListItems() {
            this.adjustList();
            //修改left_arrow 和 right_arrow 的展示
            if(this._index == 0) {
                this.left_arrow.visible = false;
            } else {
                this.left_arrow.visible = true;
            }

            if(this._index == this.endIndex -3) {
                this.right_arrow.visible = false;
            } else {
                this.right_arrow.visible = true;
            }
        }

        _onLeftArrowChange() {
            this._index = this._index - 3 > 0 ? this._index - 3 : 0;
            this.list.scrollTo(this._index);
            this.changeListItems();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        _onRightArrowChange() {
            this._index = this._index + 3 < this.array.length ? this._index + 3 : this.array.length - 2;
            this.list.scrollTo(this._index);
            this.changeListItems();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }


        changeSelectPetId(petId) {
            let pet = PetSystem.getPetInfo(petId);
            let petsFakeInHome = PetSystem.PetsFakeInHome;
            let limit = PetSystem.HomePetsLimit;
            if(pet.fakeInHome) {
                pet.fakeInHome = false;
            }else {
                if(petsFakeInHome.length>=limit) {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon6");
                    return;
                }
                pet.fakeInHome = true;
            }
            this.calcPet(limit);
            this.petList.refresh();
        }

        doClose() {
            this.close();
        }

        // EventMgr.getInstance().postEvent("BOAT_START_TRIP");
        checkGuide(eventString) {
            if (eventString == "GuideGroup_14") {
                let guide = GuideSystem.getGuide("GuideGroup_14");
                if (guide) {
                    guide.registerObj("btn3", this.petList.getCell(0));
                    guide.registerClick("btn3", this.petList.getCell(0));
                    guide.registerObj("btn4", this);
                    guide.registerClick("btn4", this.confirm);
                }
            }
        }

    }

    class PetSelectItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(petId) {
            this.offAll();
            // this.name = this.getChildByName("name");
            this.icon = this.getChildByName("icon");
            // this.count = this.getChildByName("count");
            this.hook = this.getChildByName("hook");
            // this.desc = this.getChildByName("desc");
            // this.count.visible = false;
            this.hook.visible = false;
            this.traveling = this.getChildByName("traveling");
            this.traveling.visible = false;
            this.showPet(petId);
        }

        showPet(petId) {
            let config = ConfigMgr$1.getInstance().getItemById("Role", petId);
            // this.name.text = Strings.get(config.Name);
            if(config.HeadIcon)
                this.setIcon("cat_collect/head/"+config.HeadIcon+".png");

            if(PetSystem.getPetInfo(petId).fakeInHome) {
                this.hook.visible= true;
            }

            if(PetSystem.getPetInfo(petId).inNautical) {
                this.traveling.visible= true;
            }
            this.on(Laya.Event.CLICK, this, this.onSelected, [petId]);
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 169 / this.icon.width;
            let scaley = 169 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        onSelected(petId) {
            EventMgr.getInstance().postEvent("HOME_PET_SELECTED_CHANGE", [petId]);
        }
    }

    class HousekeeperPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.needHelp = this._param;
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
            this.okBtn = this.getChildByName("okBtn");
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            this.label1.text = Strings$1.get("tid#UI_107");
            this.label2.text = Strings$1.get("tid#UI_108");
        }

        done(shareId) {
            if (shareId == Constants.ShareId.Housekeeper) {
                EventMgr.getInstance().postEvent("START_HOUSEKEEP_SERVICE");
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == Constants.ShareId.Housekeeper) {
                this.close();
            }
        }

        onOkBtnClick() {
            if (this.needHelp) {
                SdkHelper.showRewardVideoAd(Constants.ShareId.Housekeeper);
            } else {
                this.close();
                let note = Strings$1.get("tid#TipsCommon69");
                EventMgr.getInstance().postEvent("SHOW_TIPS", [note]);
            }
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }
    }

    class ImageLibPop extends Laya.Dialog {
        constructor() {
            super();
            this.name = "ImageLibPop";
        }


        onEnable() {
            this.closeBtn = this.getChildByName("close");
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseUI);
            this.handrad = this.getChildByName("Percent");
            this.intPart = this.getChildByName("CollectedRatio");
            this.fraction = this.intPart.getChildByName("fraction");
            this.point = this.intPart.getChildByName("Point");
            let memoryBtn = this.getChildByName("huiyiButton");
            memoryBtn.on(Laya.Event.CLICK, this, this.onMemoryBtnClicked);
            this.memoryBtn = memoryBtn;
            let specialItemBtn = this.getChildByName("xinwuButton");
            specialItemBtn.on(Laya.Event.CLICK, this, this.onSpecialItemClicked);
            this.specialItemBtn = specialItemBtn;
            this.petName = this.getChildByName("Name");
            this.selectBtn = this.getChildByName("selectBtn");
            this.selectBtn.on(Laya.Event.CLICK, this, this.showPetsUI);
            this.buffText = this.getChildByName("buffText");
            this.allNumber = this.getChildByName("allNumber");
            this.petsUI = null;
            this.collectGiftUI = null;
            this.imagelibUI = null;
            this.tipsUI = null;
            this.imageDetailUI = null;
            this.initPos = 0;
            this.onSpecialItemClicked(true);
            this.petuiVisible = true;
            EventMgr.getInstance().registEvent("Refresh_CollectGiftUI", this, this.RefreshCollectGiftUI);
            EventMgr.getInstance().registEvent("SetInitPos_Event", this, this.setInitPos);
            EventMgr.getInstance().registEvent("ShowPetPhotos_Event", this, this.showPetPhotos);
            EventMgr.getInstance().registEvent("ShowAllPhotos_Event", this, this.ShowAllPhotos);
        }
        onDestroy() {
            EventMgr.getInstance().removeEvent("Refresh_CollectGiftUI", this, this.RefreshCollectGiftUI);
            EventMgr.getInstance().removeEvent("SetInitPos_Event", this, this.setInitPos);
            EventMgr.getInstance().removeEvent("ShowPetPhotos_Event", this, this.showPetPhotos);
            EventMgr.getInstance().removeEvent("ShowAllPhotos_Event", this, this.ShowAllPhotos);
        }
        getInitPos() {
            return this.initPos;
        }
        setInitPos(value) {
            this.initPos = value;
        }
        onCloseUI() {
            CollectGiftDataMgr.getInstance().clear();
            this.tipsUI = null;
            this.close();

        }
        setListTypeData(data) {
            this.list.array = data;
            this.list.refresh();
        }
        setBuffData(){
            let data = PhotoSystem.getSystemPhotosData();
            let allConfig = ConfigMgr$1.getInstance().getConfig("CatPhotos");
            var allNum = 0;
            allConfig.forEach(element => {
                allNum ++;
            });
            this.allNumber.text = data.length + " / " + allNum;
            let sum = 0;
            for(let i = 0;i<data.length;i++){
                let config = data[i].getSystemConfig();
                if(config.SpecialFunctionType == "1"){
                    sum += JSON.parse(config.SpecialValue).Rate;
                }
            }
            let CatPhotosTipsAdditionLimit = Number(Const.get("CatPhotosTipsAdditionLimit"));
            if(sum < CatPhotosTipsAdditionLimit){
                this.buffText.text = Strings$1.get("tid#CatPhotoEffect_1",[(sum * 100).toFixed(0) + "%"]);
            }
            else if(sum >= CatPhotosTipsAdditionLimit){
                this.buffText.text = Strings$1.get("tid#CatPhotoEffect_1",[(CatPhotosTipsAdditionLimit * 100).toFixed(0) + "%"]) + Strings$1.get("tid#CatPhotoEffect_2");
            }

        }
        onMemoryBtnClicked(skipSound) {
            this.intPart.visible = false;
            this.handrad.visible = false;
            this.buffText.visible = true;
            this.allNumber.visible = true;
            this.setBuffData();
            if (this.collectGiftUI != null) {
                this.collectGiftUI.visible = false;
                this.collectGiftUI.close();
            }
            if (this.tipsUI != null) {
                this.tipsUI.close();
            }
            this.selectBuildingGroupButton(this.memoryBtn, skipSound);
            this.showImageLibUI();
            this.showAllTips();
        }
        showPetPhotos(petId) {
            if (this.tipsUI != null) {
                this.tipsUI.visible = false;
                this.tipsUI.close();
                this.tipsUI = null;
            }
            let curPetPothosCnt = 0;
            let catPhotos = [];
            let data2 = PhotoSystem.getSystemPhotosData();
            let config = ConfigMgr$1.getInstance().getConfig("NauticalPhotoReward");
            let hasCat = false;
            this.setPetName(petId);
            config.forEach((configCell, cfgKey) => {
                var catRequire = JSON.parse(configCell.CatRequire);
                var reward = JSON.parse(configCell.Reward);
                if (catRequire.includes(petId)) {
                    let photos = reward.length;
                    curPetPothosCnt += photos;
                }
                for (let i = 0; i < data2.length; i++) {
                    if (reward.includes(data2[i]._id)) {

                        if (catRequire.includes(petId)) {
                            let flag = 0;
                            for(let j=0;j<catPhotos.length;j++){
                                if(catPhotos[j]._id == data2[i]._id){
                                    flag = 1;
                                }
                            }
                            if(!flag){
                                catPhotos.push(data2[i]);
                            }
                        }
                    }
                }

            });
            if (catPhotos == null || catPhotos.length == 0) {
                if (this.imagelibUI != null) {
                    this.imagelibUI.visible = false;
                }
                hasCat = PetSystem.getPetInfo(petId).adopted;
                if (hasCat)//type=1 ,有猫，提示出航， type=2 无猫，提示领养
                    this.loadTips("ImageTipsUI", "prefab/ui/ImageTipsUI.prefab", 1);
                else
                    this.loadTips("ImageTipsUI", "prefab/ui/ImageTipsUI.prefab", 2);

            }
            else {
                if (this.imagelibUI != null) {
                    this.imagelibUI.visible = true;
                    this.imagelibUI.list.array = catPhotos;
                    this.imagelibUI.list.refresh();
                    let state = this.imagelibUI.list.array > 8 ? true : false;
                }

            }
            this.calCatCollectRatio(curPetPothosCnt, catPhotos.length);
        }

        calCatCollectRatio(imgAmount, ownCnt) {
            if (imgAmount == 0) {
                this.SetCollectRatio(0, 0);
            }
            else {
                let ratio = 100 * ownCnt / imgAmount;
                let newRatio = Math.round(ratio * Math.pow(10, 1)) / Math.pow(10, 1);
                let inte = Math.floor(newRatio);
                let fraction = Math.floor(10 * (newRatio - inte));
                this.SetCollectRatio(inte, fraction);
            }

        }
        ShowAllPhotos() {
            if (this.tipsUI != null) {
                this.tipsUI.visible = false;
                this.tipsUI.close();
                this.tipsUI = null;
            }
            this.petName.text = "[ALL]";
            this.petName.visible = true;
            if (this.petsUI != null)
                // this.petsUI.catName.text = "猫咪";
                // this.petsUI.catName.text = Strings.get("tid#UI_104");
            if (this.imagelibUI != null) {
                this.imagelibUI.visible = true;
                this.imagelibUI.list.array = PhotoSystem.getSystemPhotosData();
                function sortByOrder(a, b) {
                    return ConfigMgr$1.getInstance().getItemById("CatPhotos", a._id).Order - ConfigMgr$1.getInstance().getItemById("CatPhotos", b._id).Order;
                }
                this.imagelibUI.list.array.sort(sortByOrder);
                this.imagelibUI.list.refresh();
                this.imagelibUI.CalculateAll();
            }
            this.showAllTips();
        }

        showAllTips(){
            let catPhotos = PhotoSystem.getSystemPhotosData();
            if (catPhotos == null || catPhotos.length == 0) {
            let pets=PetDataMgr.getInstance().getAllPets();
            pets.forEach(element => {
                if(element.adopted==true){
                    this.youmao=1;
                }
                else {

                }
            });
            if(!this.youmao){
                this.loadTips("ImageTipsUI", "prefab/ui/ImageTipsUI.prefab", 2);
            }else {
                this.loadTips("ImageTipsUI", "prefab/ui/ImageTipsUI.prefab", 1);
            }
        }
    }
        setPetName(petId) {
            var petData = PetDataMgr.getInstance().getPet(petId);
            let petname = "[" + Strings$1.get(petData.getName()) + "]";
            this.petName.text = petname;
            this.petName.visible = true;

        }
        setPetName2(name) {
            this.petName.text = name;
            this.petName.visible = true;
        }
        loadTips(name, path, type) {
            Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                if (name == "ImageTipsUI") {
                    this.tipsUI = prefabDef.create();
                    this.tipsUI.zOrder = 2;
                    this.addChild(this.tipsUI);
                    this.tipsUI.x = 0;
                    this.tipsUI.y = 548;
                    this.tipsUI.setNullPhotoType(type);
                }
            }));
        }
        onSpecialItemClicked(skipSound) {
            this.intPart.visible = true;
            this.handrad.visible = true;
            this.buffText.visible = false;
            this.allNumber.visible = false;
            this.selectBtn.visible = false;
            if (this.imagelibUI) {
                this.imagelibUI.visible = false;
                this.imagelibUI.close();
                this.imagelibUI = null;
            }
            if (this.petsUI != null) {
                this.petsUI.close();
                this.petsUI = null;
            }

            if (this.tipsUI != null)
                this.tipsUI.close();
            this.petName.visible = false;

            CollectGiftDataMgr.getInstance().clear();
            this.selectBuildingGroupButton(this.specialItemBtn, skipSound);
            EventMgr.getInstance().postEvent("SERVICE_GetCollectGiftInfoRes");
        }

        selectBuildingGroupButton(button, skipSound) {
            this.selectGroupBtn(button, [this.memoryBtn, this.specialItemBtn]);

            if (skipSound != true)
                SoundManager.getInstance().play("Sound_clickSound", 1);
        }
        selectGroupBtn(selected, btns) {
            for (const btn of btns) {
                if (btn == selected) {
                    btn.disabled = true;
                    btn.gray = false;
                    btn.selected = true;
                    btn.y = 1861;
                } else {
                    btn.disabled = false;
                    btn.selected = false;
                    btn.y = 1872;
                }
            }
        }
        showPetsUI() {
            if (this.petsUI == null) { this.loadRes("PetsSelectUI", "prefab/ui/PetsSelectUI.prefab"); }
            else {
                this.petuiVisible = !this.petuiVisible;
                this.petsUI.visible = this.petuiVisible;
            }
        }
        loadRes(name, path) {

            {
                Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                    if (name == "PetsSelectUI") {
                        this.petsUI = prefabDef.create();
                        this.petsUI.zOrder = 2;
                        this.addChild(this.petsUI);
                        this.petsUI.y = 2000;
                    }
                }));
            }
        }
        showImageLibUI() {
            this.selectBtn.visible = true;
            this.closeBtn.visible = true;
            if (this.tipsUI != null)
                this.tipsUI.close();
            this.tipsUI = null;
            if (this.collectGiftUI != null) {
                this.collectGiftUI.close();
            }
            if (this.imagelibUI == null) {
                this.selectBuildingGroupButton(this.memoryBtn, "Sound_clickSound");
                this.loadImagelib("ImageLibUI", "prefab/ui/ImageLibUI.prefab");
            }
            else {
                this.imagelibUI.visible = true;
                this.imagelibUI.list.array = PhotoSystem.getSystemPhotosData();
                this.imagelibUI.list.refresh();
                let state = this.imagelibUI.list.array.length > 8 ? true : false;

            }

        }
        loadImagelib(name, path) {
            Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                if (name == "ImageLibUI") {
                    this.imagelibUI = prefabDef.create();
                    this.imagelibUI.zOrder = 2;
                    this.addChild(this.imagelibUI);
                    this.imagelibUI.x = 0;
                    this.imagelibUI.y = 578;
                    this.imagelibUI.scrollBar.value = 0;
                }
            }));
        }
        ShowImageDetailUI() {
            this.imageDetailUI.visible = true;
        }

        RefreshCollectGiftUI() {
            if (this.petsUI != null)
                this.petsUI.visible = false;

            this.loadCollectGiftUI("CollectGiftUI", "prefab/ui/CollectGiftUI.prefab");
        }
        loadCollectGiftUI(name, path) {
            Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                if (name == "CollectGiftUI") {
                    this.collectGiftUI = prefabDef.create();
                    this.collectGiftUI.zOrder = 2;
                    this.addChild(this.collectGiftUI);
                    this.collectGiftUI.x = 0;
                    this.collectGiftUI.y = 530;
                }
            }));

        }
        SetCollectRatio(inte, fraction) {
            this.fraction.visible = true;
            this.point.visible = true;
            this.intPart.value = inte;
            this.fraction.value = fraction;
            if (inte >= 100) {
                this.intPart.value = 100;
                this.intPart.x =876;
                this.handrad.x = 977;
                this.fraction.visible = false;
                this.point.visible = false;
            }
            if (inte > 9 && inte < 100) {
                this.fraction.visible = true;
                this.point.visible = true;
                this.intPart.x = 819;
                this.point.x = 38;
                this.fraction.x = 50;
                this.handrad.x = 1018.5;
            }
            else if (inte < 10) {
                this.fraction.visible = true;
                this.intPart.x = 781;
                this.point.visible = true;
                this.point.x = 38;
                this.fraction.x = 50;
                this.handrad.x = 977;
            }
        }

        setPageTipsData(current, amount) {
            if (current > amount)
                current = amount;
            this.pageTips.text = current + "/" + amount;
        }
    }

    /**
    * 确认放置
    */
    class InfoPop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            if(GameContext.CurrentInfoType=="food")
                this.showFood(GameContext.CurrentInfoId);
            else if (GameContext.CurrentInfoType=="tool")
                this.showTool(GameContext.CurrentInfoId);
            else if (GameContext.CurrentInfoType=="bait")
                this.showBait(GameContext.CurrentInfoId);
            else if (GameContext.CurrentInfoType=="toy")
                this.showToy(GameContext.CurrentInfoId);
            else if (GameContext.CurrentInfoType == "luckyDraw")
                this.showLuckyDraw(GameContext.CurrentInfoId);

                this.label1.text = Strings$1.get("tid#UI_111");
                this.label2.text = Strings$1.get("tid#UI_112");
                this.cost.text = Strings$1.get("tid#Btn_023");
                this.highest.text = Strings$1.get("tid#UI_008");
                this.unlock1.text =Strings$1.get("tid#UI_113");
            
        }

        showFood(foodId) {
            if(foodId==null){
                this.close();
                return;
            }
            let item = NauticalSystem$1.getFoodById(foodId);
            if(item==null){
                this.close();
                return;
            }
            this.title.text = Strings$1.get(item.Name);
            this.desc.text = Strings$1.get(item.Desc);
            this.value1.text = "+"+item.Point;
            this.value2.text = "+"+item.Intimacy;
            this.count.text = String(item.count);
            this.count.visible = true;
            this.highest.visible = false;

            this.setIcon(item.Icon);

            this.confirm.disabled = false;
            this.unlock.visible = false;
            this.unlock1.visible = false;
            this.xiaoxinxin.visible = false;
            this.xiaoxinxin1.visible = false;
            this.xiaoxinxin2.visible = false;

            let nextItem = NauticalSystem$1.getFoodCanMergeInfo(foodId);
            if (nextItem) {
                this.confirm.on(Laya.Event.CLICK, this, this.showMerge, [foodId]);
                this.confirm.disabled = false;
                this.highest.visible = false;
                this.cost.visible = true;
            } else {
                this.confirm.disabled = true;
                this.highest.visible = true;
                this.cost.visible = false;
            }

        }

        showTool(toolId) {
            if(toolId==null){
                this.close();
                return;
            }
            let item = NauticalSystem$1.getToolById(toolId);
            if(item==null){
                this.close();
                return;
            }
            this.title.visible = false;
            this.title1.visible = true;
            this.title1.text = Strings$1.get(item.Name);
            this.desc.text = Strings$1.get(item.Desc);
            this.value1.text = "X" + Math.floor(item.PointRate * 100) + "%";
            this.value2.text = "X" + Math.floor(item.IntimacyRate * 100) + "%";
            this.count.visible = false;

            this.setIcon(item.Icon);

            this.confirm.visible = false;
            this.cost.visible = false;
            this.highest.visible = false;

            let tool = NauticalSystem$1.getToolById(toolId);
            let config = ConfigMgr$1.getInstance().getItemById("NauticalTools", toolId);
            this.xiaoxinxin.visible = false;
            this.xiaoxinxin1.visible = false;
            this.xiaoxinxin2.visible = false;
            if (tool.count == 0) {
                this.unlock.y = 1320;
                this.unlock1.y=1400;
                if(config.ProduceDesc){
                    this.unlock.visible = true;
                    this.unlock.text = Strings$1.get(config.ProduceDesc);
                }
                else {
                    this.unlock.visible = false;
                }
                let desc=Strings$1.get(config.Requirement).split("<");
                this.unlock1.visible=false;
                this.unlock1.text=desc[0];
                this.unlock1.x=(this.unlock.width-this.unlock1.width-(this.xiaoxinxin.width+10)*(desc.length-1))/2+ this.unlock1.x;
            } else {
                this.unlock.visible = false;
                this.unlock1.visible=false;
            }

        }

        showBait(baitId) {
            if(baitId==null){
                this.close();
                return;
            }
            let config = ConfigMgr$1.getInstance().getItemById("FishingBaits", baitId);
            if(config==null){
                this.close();
                return;
            }

            this.setIcon(config.img);
            this.title.visible = false;
            this.desc.visible = false;
            this.title1.visible = true;
            this.desc1.visible = true;
            this.title1.text = Strings$1.get(config.name);
            this.desc1.text  = Strings$1.get(config.introduce);
            this.value1.visible = false;
            this.value2.visible = false;
            this.label1.visible = false;
            this.label2.visible = false;
            this.sprite1.visible = false;
            this.sprite2.visible = false;
            this.count.visible = false;

            this.confirm.visible = false;
            this.cost.visible = false;
            this.highest.visible = false;
            if(FishingSystem.isBaitUnlocked(baitId)) {
                this.unlock.visible = false;
                this.unlock1.visible=false;
                this.xiaoxinxin.visible=false;
                this.xiaoxinxin1.visible=false;
                this.xiaoxinxin2.visible=false;
            }else {
                if(config.ProduceDesc){
                    this.unlock.visible = true;
                    this.unlock.text = Strings$1.get(config.ProduceDesc);
                }
                else {
                    this.unlock.visible = false;
                }
                this.unlock1.text='meiqiuhaoxihuan';
                this.unlock1.x=(this.unlock.width-this.unlock1.width)/2+ this.unlock1.x;
                this.xiaoxinxin.x= this.unlock1.x+this.unlock1.width+15;
                this.unlock.y = 1380;
                this.unlock1.y=1400;
                this.xiaoxinxin.y=1405;
                this.unlock1.visible=false;
                this.xiaoxinxin.visible=false;
                this.xiaoxinxin1.visible=false;
                this.xiaoxinxin2.visible = false;
            }
            let quality = config.Quality;
            if (quality != null) {
                this.quality.visible = true;
                this.quality.skin = "shopitems/quality" + quality + ".png";
            }
        }

        showToy(toyId) {
            if(toyId==null){
                this.close();
                return;
            }
            let config = ConfigMgr$1.getInstance().getItemById("CatToys", toyId);
            if(config==null){
                this.close();
                return;
            }

            this.setIcon(config.Icon);
            this.title.visible = false;
            this.desc.visible = false;
            this.title1.visible = true;
            this.desc1.visible = true;
            this.title1.text = Strings$1.get(config.Name);
            if (config.Type == "ITEM" && config.FunctionDesc != null) {
                this.desc1.visible = false;
                let div = new Laya.HTMLDivElement();
                div.style.fontSize = 50;
                div.style.width = this.desc1.width;
                div.style.leading = 10;
                div.style.color = this.desc1.color;
                div.style.align = this.desc1.align;
                div.style.valign = this.desc1.valign;
                div.style.lineHeight = 50;
                div.style.wordWrap = true;
                div.x = this.desc1.x;
                div.y = this.desc1.y;
                this.addChild(div);
                let str = "";
                switch (config.SpecialFunctionType) {
                    case 1: {
                        let obj = JSON.parse(config.SpecialValue);
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [Strings$1.get("tid#BuildingName_" + obj.NeedBId), obj.Rate[0] * 100 + "%"]) + "</span>";
                        break;
                    }
                    case 3:
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc) + "</span>";
                        break;
                    case 6: {
                        let obj = JSON.parse(config.SpecialValue);
                        let money = "";
                        obj.forEach(element => {
                            if (element.id == Constants.RewardType.FISH) {
                                money = "<img src='topui/yugan.png' style='width:55; height:55'></img>" + element.amount;
                            } else if (element.id == Constants.RewardType.STAR) {
                                money = "<img src='topui/xingxing.png' style='width:55; height:55'></img>" + element.amount;
                            }
                        });
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [money]) + "</span>";
                        break;
                    }
                    case 7: {
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [Math.round(Number(JSON.parse(config.SpecialValue).Rate[0]) / Number(Const.get("CatFoodKeepTime")) * 1000) / 10 + "%"]) + "</span>";
                        break;
                    }
                    case 9: {
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [Math.round(Number(JSON.parse(config.SpecialValue).Rate[0]) / Number(Const.get("FishingTimeIncrease")) * 1000) / 10 + "%"]) + "</span>";
                        break;
                    }
                    case 11: {
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [JSON.parse(config.SpecialValue).Rate[0] * 100 + "%"]) + "</span>";
                        break;
                    }
                    default:
                        str = Strings$1.get(config.Desc) + "<br /><br />" + "<span color='#80a734'>" + Strings$1.get(config.FunctionDesc, [JSON.parse(config.SpecialValue).Rate[0]]) + "</span>";
                        break;
                }
                str.replace("\n", "<br />");
                div.innerHTML = str;
            } else {
                this.desc1.text = Strings$1.get(config.Desc);
            }
            this.value1.visible = false;
            this.value2.visible = false;
            this.label1.visible = false;
            this.label2.visible = false;
            this.sprite1.visible = false;
            this.sprite2.visible = false;
            this.count.visible = false;

            this.confirm.visible = false;
            this.cost.visible = false;
            this.highest.visible = false;
            if (PetSystem.hasToy(toyId)) {
                this.unlock.visible = false;
                this.unlock1.visible=false;
                this.xiaoxinxin.visible=false;
                this.xiaoxinxin1.visible=false;
                this.xiaoxinxin2.visible = false;
            }else {
                if(config.ProduceDesc){
                    this.unlock.visible = true;
                    this.unlock.text = Strings$1.get(config.ProduceDesc);
                }
                else {
                    this.unlock.visible = false;
                }
                this.unlock1.text='meiqiuhaoxihuan';
                this.unlock1.x=(this.unlock.width-this.unlock1.width)/2+ this.unlock1.x;
                this.xiaoxinxin.x = this.unlock1.x + this.unlock1.width + 15;
                if (config.Type == "ITEM") {
                    this.unlock.y = 1453;
                } else {
                    this.unlock.y = 1380;
                }
                this.unlock1.y=1400;
                this.xiaoxinxin.y=1405;
                this.unlock1.visible=false;
                this.xiaoxinxin.visible=false;
                this.xiaoxinxin1.visible=false;
                this.xiaoxinxin2.visible = false;
            }
            let quality = config.Quality;
            if (quality != null) {
                this.quality.visible = true;
                this.quality.skin = "shopitems/quality" + quality + ".png";
            }
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 180 / this.icon.width;
            let scaley = 180 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        setGray() {
            this.confirm.gray = true;
            this.confirm.disabled = true;
        }

        onOpened(param) {
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/shopPop.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        showMerge() {
            this.close();
            Laya.Dialog.close("ui/MergePop.scene");
            Laya.Dialog.open("ui/MergePop.scene", false);
        }

        showLuckyDraw(id) {

            if(id==null){
                this.close();
                return;
            }
            let items = LuckydrawSystem.getPrizeWheelReward();
            let item = items[id];
            if(item==null){
                this.close();
                return;
            }
            this.title.visible = false;
            this.desc.visible = false;
            this.title1.visible = true;
            this.desc1.visible = true;
            this.value1.visible = false;
            this.value2.visible = false;
            this.label1.visible = false;
            this.label2.visible = false;
            this.sprite1.visible = false;
            this.sprite2.visible = false;
            this.count.visible = false;
            this.confirm.visible = false;
            this.cost.visible = false;
            this.highest.visible = false;
            this.unlock.visible = false;
            this.unlock1.visible = false;
            this.xiaoxinxin.visible = false;
            this.xiaoxinxin1.visible = false;
            this.xiaoxinxin2.visible = false;
            switch (item.type) {
                case "Packet0": {
                    this.setIcon("luckydraw/question.png");
                    this.title1.text = Strings$1.get("tid#UI_171");
                    this.desc1.text = Strings$1.get("tid#TipsCommon70");
                    break;
                }
                case "Packet1": {
                    this.setIcon("luckydraw/top_packet.png");
                    this.title1.text = Strings$1.get("tid#UI_114");
                    this.desc1.text = Strings$1.get("tid#TipsCommon66", [Const.get("Packet1GameTime")]);
                    break;
                }
                case "Packet2": {
                    this.setIcon("luckydraw/toy_packet.png");
                    this.title1.text = Strings$1.get("tid#UI_115");
                    this.desc1.text = Strings$1.get("tid#TipsCommon67");
                    break;
                }
                case "Packet3": {
                    this.setIcon("luckydraw/sail_packet.png");
                    this.title1.text = Strings$1.get("tid#UI_116");
                    this.desc1.text = Strings$1.get("tid#TipsCommon68");
                    break;
                }
            }
        }
    }

    class IntimacyUp extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            this.petId = this._param[0];
            this.beforeHeartNum = this._param[1];
            this.beforeIntimacy = this._param[2];
            this.nowHeartNum = this._param[3];
            this.nowIntimacy = this._param[4];
            this.text1.text = Strings$1.get("tid#UI_221");
            this.text2.text = Strings$1.get("tid#UI_222");
            this.text3.text = Strings$1.get("tid#UI_223");
            this.config = ConfigMgr$1.getInstance().getItemById("Role", String(this.petId));
            this.before.text = this.config.ValueBase[this.beforeHeartNum];
            this.after.text = this.config.ValueBase[this.nowHeartNum];

            this.closeBtn.on(Laya.Event.CLICK,this,this.onCloseBtnClick);

            this.bgAnim = new Spine();
            this.bgAnim.loadAnim("anim3/shengjidonghua.sk",Constants.TipAnimType.INTIMACYUP);
            this.bgAnim.playAnim("lingyang1", false);
            this.bgAnim.x = this.width / 2;
            this.bgAnim.y = this.height / 2;
            this.animBox.addChild(this.bgAnim);
            this.bgAnim.addCompleteHandler(Laya.Handler.create(this,function(){
                this.imageBox.visible = true;
                this.bgAnim.playAnim("lingyang2", true);
            }));

            let pet = PetSystem.getPetInfo(this.petId);
            this.cat = new Spine();
            this.cat.x = this.catBox.width / 2;
            this.cat.y = this.catBox.height / 2;
            this.cat.scale(1.3,1.3);
            this.catBox.addChild(this.cat);
            this.cat.loadAnim(pet.getSource() + ".sk", pet.getSkin(), 0);
            this.cat.playAnim("play_beg", true);
            this.showHearts();
        }

        showHearts(){
            if(this.beforeHeartNum <=1){
                this.heart1.visible = true;
                this.heart2.visible = false;
                this.heart1.x = 891;
                this.heart1.skin = this.beforeHeartNum ? "collect/heart.png" : "collect/grayheart.png";
            }
            else if(this.beforeHeartNum == 2){
                this.heart1.visible = true;
                this.heart2.visible = true;
                this.heart1.x = 857;
                this.heart2.x = 924;
            }
            if(this.nowHeartNum == 1){
                this.heart3.visible = true;
                this.heart4.visible = false;
                this.heart5.visible = false;
                this.heart3.x = 1092;
            }
            else if(this.nowHeartNum == 2){
                this.heart3.visible = true;
                this.heart4.visible = true;
                this.heart5.visible = false;
                this.heart3.x = 1092;
                this.heart4.x = 1160;
            }
            else if(this.nowHeartNum == 3){
                this.heart3.visible = true;
                this.heart4.visible = true;
                this.heart5.visible = true;
                this.heart3.x = 1058;
                this.heart4.x = 1126;
                this.heart5.x = 1193;
            }
        }

        onCloseBtnClick(){
            this.close();
            EventMgr.getInstance().postEvent("SHOW_INTIMACYUP");
        }
    }

    class LevelToyInfoPop extends Laya.Dialog {
        constructor() {
             super(); 
        }

        onEnable() {
            this.Id = GameContext.CurrentInfoId;
            this.toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", GameContext.CurrentInfoId);
            this.lock.text = Strings$1.get("tid#UI_215");
            this.setIcon(this.toyConfig.Icon);
            this.setLevelData();
            this.title.text = Strings$1.get(this.toyConfig.Name);
            this.desc.text = Strings$1.get(this.toyConfig.Desc);
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 180 / this.icon.width;
            let scaley = 180 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        setLevelData(){
            let levelData = LevelToySystem.getDataById(this.Id);
            if(levelData[0]){
                this.level.text =  "LV"+ levelData[0];
                this.lock.visible = false;
                this.progressBar.visible = true;
                this.level.visible = true;
                this.exp.visible = true;
                if(levelData[2] == -1){
                    this.exp.text = Strings$1.get("tid#UI_214");
                    this.progressBar.value = 1;
                }
                else {
                    this.exp.text = levelData[1] + "/" + levelData[2];
                    this.progressBar.value =  levelData[1] / levelData[2];
                }
            }
            else {
                this.lock.visible = true;
                this.progressBar.visible = false;
                this.level.visible = false;
                this.exp.visible = false;
            }
            this.setFunctionDesc(levelData[0]);
        }

        setFunctionDesc(level){
            level = level ? level : 1;
            let specialValue = JSON.parse(this.toyConfig.SpecialValue).Rate[level - 1];
            let str = this.toyConfig.FunctionDesc;
            let num = 0;
            switch(this.toyConfig.SpecialFunctionType){
                case 1:
                    let bName = Strings$1.get("tid#BuildingName_" + JSON.parse(this.toyConfig.SpecialValue).NeedBId);
                    this.detail.text = Strings$1.get(str,[bName,specialValue * 100 + "%"]);
                    break;
                case 7:
                    // let CatFoodKeepTime = Const.get("CatFoodKeepTime");
                    num = specialValue / 60 >  (specialValue / 60 | 0) ? (specialValue / 60).toFixed(1) : (specialValue / 60).toFixed(0);
                    this.detail.text = Strings$1.get(str,[num]);
                    break;
                case 9:
                    let FishingTimeIncrease = Const.get("FishingTimeIncrease");
                    num = specialValue / FishingTimeIncrease * 100 > (specialValue / FishingTimeIncrease * 100 | 0) ? (specialValue / FishingTimeIncrease * 100).toFixed(1) :  (specialValue / FishingTimeIncrease * 100).toFixed(0);
                    this.detail.text = Strings$1.get(str,[num + "%"]);
                    break;
                case 11:
                    num = specialValue * 100 >  (specialValue * 100 | 0 + 0.05) ? (specialValue * 100).toFixed(1) : (specialValue * 100).toFixed(0);
                    this.detail.text = Strings$1.get(str,[num + "%"]);
                    break;
                default:
                    this.detail.text = Strings$1.get(str,[specialValue]);
            }
        }

    }

    class GapLoadingUI extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable() {
            this.panel = this.getChildByName("panel");
            this.panel.alpha = 0.5;

            this.loadingAni = new Laya.Skeleton();
            this.addChild(this.loadingAni);
            this.loadingAni.load("anim1/loading2.sk", Laya.Handler.create(this, function () {
                this.loadingAni.x = 1827 / 2;
                this.loadingAni.y = 2436 / 2 - 100;
            }));

            let duration = 1000;
            if(GameContext.hideTime!=null && GameContext.showTime!=null) {
                let hideTime = GameContext.hideTime;
                let showTime = GameContext.showTime;
                let gap = showTime - hideTime;
                let arr = Const.getArray("ScreenRecallWaitTimes");
                for(const info of arr) {
                    if(gap>=Number(info.gaps[0])*1000 && (gap < Number(info.gaps[1])*1000 || Number(info.gaps[1]) == -1)) {
                        duration = Number(info.wait_times)*1000;
                        break;
                    }
                }
            }
            console.log("showTime", GameContext.showTime);
            console.log("hideTime", GameContext.hideTime);
            GameContext.showTime = null;
            GameContext.hideTime = null;

            Laya.timer.once(duration, this, this.checkStopLoading);
        }

        onDisable() {
            Laya.timer.clear(this, this.checkStopLoading);
        }

        checkStopLoading() {
            this.loadingAni.stop();
            this.close();
        }
    }

    class ReqLoadingUI extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable() {
            this.zOrder = 2;
            EventMgr.getInstance().registEvent("REQ_LOADING_END", this, this.checkStopLoading);
            EventMgr.getInstance().registEvent("REQ_LOADING_TIMEOUT", this, this.timeout);
            this.closeEffect=null;
            this.panel = this.getChildByName("panel");
            this.panel.alpha = 0.5;

            this.loadingAni = new Laya.Skeleton();
            this.addChild(this.loadingAni);
            this.loadingAni.load("anim1/loading2.sk", Laya.Handler.create(this, function () {
                this.loadingAni.x = 1827 / 2;
                this.loadingAni.y = 2436 / 2 - 100;
            }));

            Laya.timer.loop(5000, this, this.checkStopLoading);
        }

        onOpened() {
            if(NetLoadingMgr.getInstance().ifLoadingIdsEmpty()) {
                if(this.loadingAni) this.loadingAni.stop();
                this.close();
            }
        }

        close() {
            console.log("..... close loading ui .....");
            Laya.timer.clear(this, this.checkStopLoading);
            super.close();
        }

        onDisable() {
            Laya.timer.clear(this, this.checkStopLoading);
            EventMgr.getInstance().removeEvent("REQ_LOADING_END", this, this.checkStopLoading);
            EventMgr.getInstance().removeEvent("REQ_LOADING_TIMEOUT", this, this.timeout);
        }

        checkStopLoading() {
            console.log(".... check stop loading ....");
            console.log(NetLoadingMgr.getInstance().getMap());
            if(NetLoadingMgr.getInstance().ifLoadingIdsEmpty()) {
                this.loadingAni.stop();
                this.close();
            }
        }

        timeout() {
            console.log("..... loading ui time out .....");
            this.loadingAni.stop();
            TipsPopMgr.getInstance().showTipsById("tid#Tips_serversError_2");
            Laya.timer.once(2500, null, function () {
                EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
            });
        }

    }

    class LuckydrawLevelReward extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            this.anim.source = "homeland/ui/texiao1.png,homeland/ui/texiao2.png";
            this.reward = this._param.rewards.rewards[0];
            this.onlyRewards = this._param.onlyRewards.rewards[0];
            this.newItem = this._param.newItem;
            this.levelToy = this._param.levelToy;
            this.addToyExp = this._param.addToyExp;
            this.sure.label = Strings$1.get("tid#Btn_009");
            this.sure.on(Laya.Event.CLICK,this,this.getReward);
            this.toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", this.onlyRewards.id);
            this.showType = 0;
            this.barWidthMax = 185;
            this.PrizeWheelSubReward = Const.get("PrizeWheelSubReward");
            this.maxValueBase = LuckydrawSystem.getFishReward()[2];
            this.load();
        }

        load(){
            this.loadBaseData();
            this.loadDetailImage();
            this.showNumber();
        }

        loadBaseData(){
            this.type = this.toyConfig.SpecialFunctionType ? this.toyConfig.SpecialFunctionType : 0;
            this.tipsImg = this.toyConfig.Icon;
            this.tipsWords = Strings$1.get(this.toyConfig.Name);
            this.icon.skin = this.tipsImg;
            this.title.text = this.tipsWords;
            this.loadToyQuality();
        }

        loadToyQuality(){
            if(this.toyConfig.Quality){
                this.quality.skin = "shopitems/quality" + this.toyConfig.Quality + ".png";
            }
            this.new.visible = true;
            if(this.newItem.newCatToy){
                this.new.scale(2.5,2.5);
                this.new.skin = "wharf/ui/new.png";
                this.new.rotation = 290;
            }
            else {
                if(this.reward.id == Constants.RewardType.FISH){
                    this.new.scale(4,4);
                    this.new.skin = "luckydraw/owned.png";
                    this.new.rotation = 0;
                }
                else {
                    this.new.visible = false;
                }
            }
        }

        loadLevelData(){ //有等级的进入此函数，包括满级和未满级
            //等级、经验、上限、加成数值率
            let beforeLevel = 0;
            let beforeExp = 0;
            let beforeCost = 0;
            let beforeRate = 0;
            let nowLevel = 0;
            let nowExp = 0;
            let nowCost = 0;
            let nowRate = 0;
            let Rate = JSON.parse(this.toyConfig.SpecialValue).Rate;
            if(this.addToyExp){//未已满级
                /////////////////deal with before//////////////
                if(this.levelToy.exp - this.addToyExp < 0){ //升级
                    this.upLevel = 1;
                    beforeCost = this.toyConfig.LvUpCost[this.levelToy.level - 1];
                    beforeLevel = this.levelToy.level - 1;
                    beforeExp = beforeCost + this.levelToy.exp - this.addToyExp;
                    if(this.levelToy.level > 1){
                        beforeRate = Rate[this.levelToy.level - 2];
                    }
                    else {
                        beforeRate = 0;
                    }
                }
                else { //未升级
                    this.upLevel = 0;
                    beforeCost = this.toyConfig.LvUpCost[this.levelToy.level];
                    beforeLevel = this.levelToy.level;
                    beforeExp = this.levelToy.exp - this.addToyExp;
                }
                ////////////////deal with now////////////////
                nowLevel = this.levelToy.level;
                nowExp = this.levelToy.exp;
                nowCost = this.toyConfig.LvUpCost.length > this.levelToy.level ? this.toyConfig.LvUpCost[this.levelToy.level] : -1;
                nowRate = Rate[this.levelToy.level - 1];
            }
            else {//已满级
                nowLevel = this.toyConfig.LvUpCost.length;
                nowExp = 0;
                nowCost = -1;
                nowRate = this.PrizeWheelSubReward * this.maxValueBase;
            }
            this.before = new Array(beforeLevel,beforeExp,beforeCost,beforeRate);
            this.now = new Array(nowLevel,nowExp,nowCost,nowRate);
        }

        loadDetailImage(){//仅读取内容，数值和位置另行处理
            if(this.toyConfig.LvUpCost.length){ //有等级
                this.levelBox.visible = true;
                this.loadLevelData();
                if(!this.levelToy){ //已满级
                    this.obj2.visible = true;
                    this.obj2.skin = "luckydraw/ui/yimanji.png";
                    this.obj1.skin = "topui/yugan1.png";
                    this.obj1.scale(0.8,0.8);
                    this.obj1.y +=10;
                    this.sign.skin = "luckydraw/ui/add.png";
                    this.showType = 4;
                }
                else if(this.type){ //有类型
                    this.loadTypeToy();
                }
            }
            else { //无等级
                this.levelBox.visible = false;
                if(this.type){ //有类型          PS:无类型的直接不显示相关信息
                    if(this.newItem.newCatToy){ //不重复
                        let rate = this.type == 3 ? 0 : JSON.parse(this.toyConfig.SpecialValue).Rate[0];
                        this.before = new Array(0,0,0,0);
                        this.now = new Array(0,0,0,rate);
                        this.loadTypeToy();
                    }
                    else {
                        this.obj2.visible = true;
                        this.obj2.skin = "luckydraw/ui/yiyongyou.png";
                        this.obj1.skin = "topui/yugan1.png";
                        this.obj1.scale(0.8,0.8);
                        this.obj1.y +=10;
                        this.sign.skin = "luckydraw/ui/add.png";
                        this.showType = 4;
                        this.before = new Array(0,0,0,this.PrizeWheelSubReward * this.maxValueBase);
                        this.now = new Array(0,0,0,this.PrizeWheelSubReward * this.maxValueBase);
                    }
                }
                else if(!this.newItem.newCatToy){
                    this.obj2.visible = true;
                    this.obj2.skin = "luckydraw/ui/yiyongyou.png";
                    this.obj1.skin = "topui/yugan1.png";
                    this.obj1.scale(0.8,0.8);
                    this.obj1.y +=10;
                    this.sign.skin = "luckydraw/ui/add.png";
                    this.showType = 4;
                    this.before = new Array(0,0,0,this.PrizeWheelSubReward * this.maxValueBase);
                    this.now = new Array(0,0,0,this.PrizeWheelSubReward * this.maxValueBase);
                }
            }
            
            
        }

        loadTypeToy(){
            this.obj1.scale(2.5,2.5);
            if(this.type == 1){
                this.obj2.visible = true;
                this.showType = 3;
                let NeedBId = JSON.parse(this.toyConfig.SpecialValue).NeedBId;
                switch(NeedBId){
                    case 1130: this.obj2.skin = "luckydraw/ui/zhaocaijinbao.png";break;
                    case 1131: this.obj2.skin = "luckydraw/ui/danyunbowu.png";break;
                    case 1132: this.obj2.skin = "luckydraw/ui/yinghuazhiwu.png";break;
                    case 1133: this.obj2.skin = "luckydraw/ui/kaiyundamo.png";break;
                    case 1134: this.obj2.skin = "luckydraw/ui/dianshichengjin.png";break;
                    case 1135: this.obj2.skin = "luckydraw/ui/fuhaifangzhu.png";break;
                    case 1136: this.obj2.skin = "luckydraw/ui/xiadaozhiying.png";break;
                    case 1137: this.obj2.skin = "luckydraw/ui/xiajindandemao.png";break;
                }
                this.obj1.skin = "luckydraw/ui/xiaofeishangxian.png";
                this.sign.skin = "luckydraw/ui/add.png";
                this.sign1.skin = "luckydraw/ui/baifenhao.png";
                this.before[3] *= 100;
                this.now[3] *= 100;
            }
            else {
                this.obj2.visible = false;
                if(this.type == 2){
                    this.showType = 2;
                    this.obj1.skin = "luckydraw/ui/yuleshouyishangxian.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                }
                else if(this.type == 3){
                    this.showType = 1;
                    this.obj1.skin = "luckydraw/ui/maomaolingyangbibei.png";
                }
                else if(this.type == 4){
                    this.showType = 2;
                    this.obj1.skin = "luckydraw/ui/buchajiacishu.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                }
                else if(this.type == 5){
                    this.showType = 2;
                    this.obj1.skin = "luckydraw/ui/kanjiacishu.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                }
                else if(this.type == 7){
                    this.showType = 3;
                    this.obj1.skin = "luckydraw/ui/maoliangshijian.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                    this.sign1.skin = "luckydraw/ui/min.png";
                    this.before[3] /= 60;
                    this.now[3] /= 60;
                    this.before[3].toFixed(0);
                    this.now[3].toFixed(0);
                }
                else if(this.type == 8){
                    this.showType = 2;
                    this.obj1.skin = "luckydraw/ui/fishTimes.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                }
                else if(this.type == 9){
                    this.showType = 3;
                    this.obj1.skin = "luckydraw/ui/fishCD.png";
                    this.sign.skin = "luckydraw/ui/sub.png";
                    this.sign1.skin = "luckydraw/ui/baifenhao.png";
                    let FishingTimeIncrease = Const.get("FishingTimeIncrease");
                    this.before[3] = this.before[3] / FishingTimeIncrease * 100;
                    this.now[3] = this.now[3] / FishingTimeIncrease * 100;
                    this.before[3].toFixed(0);
                    this.now[3].toFixed(0);
                }
                else if(this.type == 11){
                    this.obj2.visible = true;
                    this.showType = 3;
                    this.obj2.skin = "luckydraw/ui/zhaocaimao.png";
                    this.obj1.skin = "luckydraw/ui/xiaofeishangxian.png";
                    this.sign.skin = "luckydraw/ui/add.png";
                    this.sign1.skin = "luckydraw/ui/baifenhao.png";
                    this.before[3] *= 100;
                    this.now[3] *= 100;
                }
            }
        }

        showNumber(){ 
            if(this.toyConfig.LvUpCost.length){ //有等级
                this.setExpBar();
            }
            else {
                if(this.type || !this.newItem.newCatToy){
                    this.setDetailImgPos(this.now[3]);
                }
                else {
                    this.setDetailImgPos();
                }
            }
        }

        setExpBar(){
            if(!this.addToyExp){   //已满级
                this.level.text = this.toyConfig.LvUpCost.length;
                this.exp.text = Strings$1.get("tid#UI_214");
                this.bar.width = this.barWidthMax;
                this.setDetailImgPos(this.now[3]);
            }
            else {
                this.exp.text = this.before[1] + "/" + this.before[2];
                this.bar.width = this.barWidthMax * this.before[1] / this.before[2];
                if(this.upLevel){  //升级
                    this.setDetailImgPos(this.before[3]);
                    this.level.text = this.before[0];
                    Laya.Tween.to(this.bar,{width:this.barWidthMax},600,null,Laya.Handler.create(this,function(){
                        if(this.now[0] < this.toyConfig.LvUpCost.length){
                            this.exp.text = this.now[1] + "/" + this.now[2];
                            this.bar.width = 0;
                        }
                        else {
                            this.exp.text = Strings$1.get("tid#UI_214");
                        }
                        this.level.text = this.now[0];
                        Laya.Tween.to(this.level,{scaleX : 1.5,scaleY : 1.5},300,null,Laya.Handler.create(this,function(){
                            Laya.Tween.to(this.level,{scaleX : 1,scaleY : 1},300,null,Laya.Handler.create(this,function(){
                                this.setDetailImgPos(this.now[3]);
                                Laya.Tween.to(this.number,{scaleX : 3,scaleY : 3},300,null,Laya.Handler.create(this,function(){
                                    Laya.Tween.to(this.number,{scaleX : 2.5,scaleY : 2.5},300);
                                }));
                            }));
                        }));
                        if(this.now[1] > 0 && this.now[0] < this.toyConfig.LvUpCost.length){
                            Laya.Tween.to(this.bar,{width:this.barWidthMax * this.now[1] / this.now[2]},500);
                        }
                    }));
                }
                else {
                    this.level.text = this.now[0];
                    this.setDetailImgPos(this.now[3]);
                    Laya.Tween.to(this.bar,{width:this.barWidthMax * this.now[1] / this.now[2]},600,null,Laya.Handler.create(this,function(){
                        this.exp.text = this.now[1] + "/" + this.now[2];
                    }));
                }
            } 
        }

        setDetailImgPos(value){
            if(value >= 0){
                if(this.showType != 4){
                    this.number.value = value.toFixed(0);
                }
                else {
                    this.fishNumber.text = "+" + value;
                }
            }
            let space = 20;
            let w1 = this.obj1.width * this.obj1.scaleX;
            let w2 = this.sign.width * this.sign.scaleX;
            let w3 = this.getNumberWidth(value) + 20;
            let w4 = this.sign1.width * this.sign1.scaleX;
            let w5 = this.fishNumber.width;
            if(!this.showType){
                this.detailBox.visible = false;
            }
            else {
                this.detailBox.visible = true;
                if(this.showType == 1){
                    this.obj1.visible = true;
                    this.sign.visible = false;
                    this.number.visible = false;
                    this.sign1.visible = false;
                    this.fishNumber.visible = false;
                    this.obj1.x = (this.detailBox.width - w1) / 2;
                }
                else if(this.showType == 2){
                    this.obj1.visible = true;
                    this.sign.visible = true;
                    this.number.visible = true;
                    this.sign1.visible = false;
                    this.fishNumber.visible = false;
                    let starPos = (this.detailBox.width - w1 - w2 - w3 - 2 * space) / 2;
                    this.obj1.x = starPos;
                    this.sign.x = this.obj1.x + w1 + space;
                    this.number.x = this.sign.x + w2 + space + w3 / 2;
                }
                else if(this.showType == 3){
                    this.obj1.visible = true;
                    this.sign.visible = true;
                    this.number.visible = true;
                    this.sign1.visible = true;
                    this.fishNumber.visible = false;
                    let starPos = (this.detailBox.width - w1 - w2 - w3 - w4 - 3 * space) / 2;
                    this.obj1.x = starPos;
                    this.sign.x = this.obj1.x + w1 + space;
                    this.number.x = this.sign.x + w2 + space + w3 / 2;
                    this.sign1.x = this.number.x + w3 / 2 + space;
                }
                else if(this.showType == 4){
                    this.obj1.visible = true;
                    this.sign.visible = false;
                    this.number.visible = false;
                    this.sign1.visible = false;
                    this.fishNumber.visible = true;
                    let starPos = (this.detailBox.width - w1 - w5 -  space) / 2;
                    this.obj1.x = starPos;
                    this.fishNumber.x = this.obj1.x + w1 + space;
                }
            } 
        }

        getNumberWidth(value){
            let base = 72.5;
            let i = 0;
            for(i = 1;value >= 10;i++){
                value /= 10;
            }
            return i * base;
        }

        getReward(){
            this.close();
            EventMgr.getInstance().postEvent("FINISH_LUCKYDRAW_REWARD");
            if(this.reward.id == Constants.RewardType.FISH){
                this.tipsImg = "topui/yugan.png";
                this.tipsWords = "" + this.reward.amount;
            }
            let str = "<img src='" + this.tipsImg + "' style='width:55; height:55'></img>" + this.tipsWords;
            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon59", [str]);
        }
    }

    class LuckydrawReward extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            this.reward = this._param.rewards.rewards;
            this.changeRewardOrder();
            this.onlyRewards = this._param.onlyRewards.rewards;
            this.newItem = this._param.newItem;
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.closeText.text = Strings$1.get("tid#LuckyDrawBtn5");
            this.title.text = Strings$1.get("tid#LuckyDrawUI_2");
            this.tips1.text = Strings$1.get("tid#LuckyDrawUI_3");
            this.num.visible = false;
            this.tips1.visible = false;
            this.description.visible = false;
            this.icon.width = this.icon.height = 80;
            if(this.onlyRewards && this.onlyRewards.length >=1){
                this.oneReward.visible = true;
                this.getTipImg(this.onlyRewards[0]);
                this.toy.visible = true;
                this.icon.skin = this.tipsImg;
                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", this.onlyRewards[0].id);
                if (toyConfig && toyConfig.Quality) {
                    this.quality.visible = true;
                    this.quality.skin = "shopitems/quality" + toyConfig.Quality + ".png";
                }
                if(!this.newItem.newCatToy){
                    this.detail.visible = false;
                    this.num.visible = true;
                    this.tips1.visible = true;
                    this.num.text = "+" + Strings$1.dealFishNumberText(this.reward[0].amount);
                    this.new.skin = "luckydraw/owned.png";
                    this.new.rotation = 0;
                }
                else {
                    this.num.visible = false;
                    this.tips1.visible = false;
                    this.detail.text = this.tipsWords;
                    this.new.skin = "wharf/ui/new.png";
                    this.new.rotation = 290;
                    let str = toyConfig.FunctionDesc;
                    let type = toyConfig.SpecialFunctionType;
                    if(str){
                        let specialValue = JSON.parse(toyConfig.SpecialValue);
                        if(type == 1){
                                let bName = Strings$1.get("tid#BuildingName_" + specialValue.NeedBId);
                                this.description.text = Strings$1.get(str,[bName,specialValue.Rate * 100 + "%"]);
                        }
                        else  if(type == 3){
                            this.description.text = Strings$1.get(str);
                        }
                        else  if(type == 7){
                            let CatFoodKeepTime = Const.get("CatFoodKeepTime");
                            let num = specialValue / CatFoodKeepTime * 100 ;
                            this.description.text = Strings$1.get(str,[num + "%"]);
                        }
                        else  if(type == 9){
                            let FishingTimeIncrease = Const.get("FishingTimeIncrease");
                            let num = specialValue / FishingTimeIncrease * 100 ;
                            this.description.text = Strings$1.get(str,[num + "%"]);
                        }
                        else {
                            this.description.text = Strings$1.get(str,[specialValue]);
                        }
                        this.description.visible = true;
                        this.toy.y = -91;
                        this.tips1.y = 1564;
                    }
                    else {
                        this.toy.y = -20;
                        this.tips1.y = 1493;
                    }
                }
            }
            else {
                if(this.reward.length == 1){
                    this.oneReward.visible = true;
                    if (this.reward[0].type == Constants.ItemType.Currency) {
                        this.reward0.visible = true;
                        this.reward0.setItem(this.reward[0]);
                    }
                    else {
                        this.getTipImg(this.reward[0]);
                        this.toy.visible = true;
                        this.icon.skin = this.tipsImg;
                        this.detail.text = this.tipsWords;
                    }
                }
                else if(this.reward.length == 2){
                    this.twoReward.visible = true;
                    this.reward1.setItem(this.reward[0]);
                    this.reward2.setItem(this.reward[1]);
                }
            }
        }

        changeRewardOrder(){
            if(this.reward.length > 1){
                function sortByOrder(a,b)
                {
                    function getIndex(c) {
                        let d = 0;
                        if (Constants.ItemType.Currency != c.type) {
                            d += 10;
                        } else if (c.id == Constants.RewardType.FISH) {
                            d += 1;
                        } else if (c.id == Constants.RewardType.STAR) {
                            d += 2;
                        } else if (c.id == Constants.RewardType.SHELL) {
                            d += 3;
                        }
                        return d;
                    }
                    return getIndex(a)-getIndex(b);
                }
                this.reward.sort(sortByOrder);
            }
        }

        onCloseClick(){
            this.close();
            EventMgr.getInstance().postEvent("FINISH_LUCKYDRAW_REWARD");
            for(let i = 0; i< this.reward.length; i++){
                let n = i;
                Laya.timer.once(100 * n,this,function(){
                    this.getTipImg(this.reward[n]);
                    if(this.reward[n].id == Constants.CouponType.BUILD){
                        return;
                    }
                    this.getReward();
                });
            }
            Laya.timer.once(this.reward.length * 100, null, BuildingSystem.checkShowBuildingCouponPop);
        }

        getTipImg(reward){
            if(reward.type == Constants.ItemType.Currency) {
                if(reward.id == Constants.RewardType.FISH) {
                    this.tipsImg = "topui/yugan.png";
                    this.tipsWords = Strings$1.dealFishNumberText(reward.amount,1);
                } 
                else if(reward.id == Constants.RewardType.STAR) {
                    this.tipsImg = "topui/xingxing.png";
                    this.tipsWords = "" + reward.amount;
                } else if (reward.id == Constants.RewardType.SHELL) {
                    this.tipsImg = "topui/beike.png";
                    this.tipsWords = "" + reward.amount;
                } else if (reward.id == Constants.RewardType.MEDAL) {
                    this.tipsImg = "topui/huizhang1.png";
                    this.tipsWords = "" + reward.amount;
                } else if (reward.id == Constants.RewardType.BOOK) {
                    this.tipsImg = "topui/miji1.png";
                    this.tipsWords = "" + reward.amount;
                }
            }
            else if(reward.type == Constants.ItemType.NauticalFoods) {
                let foodConfig = ConfigMgr$1.getInstance().getItemById("NauticalFoods", reward.id);
                this.tipsImg = foodConfig.Icon;
                this.tipsWords = Strings$1.get(foodConfig.Name);
            }
            else if(reward.type == Constants.ItemType.NauticalTools) {
                let toolConfig = ConfigMgr$1.getInstance().getItemById("NauticalTools", reward.id);
                this.tipsImg = toolConfig.Icon;
                this.tipsWords = Strings$1.get(toolConfig.Name);
            }
            else if(reward.type == Constants.ItemType.FishingBaits) {
                let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", reward.id);
                this.tipsImg = baitConfig.Icon;
                this.tipsWords = Strings$1.get(baitConfig.name);
            }
            else if(reward.type == Constants.ItemType.CatToys) {
                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", reward.id);
                this.tipsImg = toyConfig.Icon;
                this.tipsWords = Strings$1.get(toyConfig.Name);
            }
            else if (reward.type == Constants.ItemType.Coupon) {
                this.tipsImg = "coupon/coupon_" + reward.id + ".png";
                this.new.skin = "wharf/ui/new.png";
                this.new.rotation = 290;

                if (reward.id == Constants.CouponType.BUILD) {
                    this.tipsWords = Strings$1.get("tid#UI_173");
                    this.description.text = Strings$1.get("tid#UI_176");
                }
                else if (reward.id == Constants.CouponType.FISHING) {
                    this.tipsWords = Strings$1.get("tid#UI_174");
                    this.description.text = Strings$1.get("tid#UI_177");
                } else if (reward.id == Constants.CouponType.SPEED) {
                    this.tipsWords = Strings$1.get("tid#UI_172");
                    let time = Math.round(Number(Const.get("NauticalADReduceTime")) / 60);
                    this.description.text = Strings$1.get("tid#UI_175", [time]);
                }
                this.description.visible = true;

            } 
            // else if(reward.type == Constants.ItemType.CatPhotos) {
                //     let catPhotosConfig = ConfigMgr.getInstance().getItemById("CatPhotos", reward.id);
                //     this.tipsImg = catPhotosConfig.Icon;
                // }
        }

        getReward(){
            let str = "<img src='" + this.tipsImg + "' style='width:55; height:55'></img>" + this.tipsWords;
            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon59", [str]);
        }

    }

    class LuckyBagDialog extends Laya.Dialog{
        constructor() {
            super();
        }

        onEnable(){
            this.luckyClickSign = 1;
            this.close1 = this.getChildByName("close");
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.numText = this.getChildByName("num");
            this.adSbtn = this.getChildByName("adS");
            this.adSbtn.on(Laya.Event.CLICK,this,this.onShareClick);
            this.adVbtn = this.getChildByName("adV");
            this.adVbtn.on(Laya.Event.CLICK,this,this.onAdClick);
            // this.directlyGet = this.getChildByName("directlyGet");
            // this.directlyGet.on(Laya.Event.CLICK,this,this.onDirectlyGetClick);
            this.overseasGetReward = this.getChildByName("overseasGetReward");
            this.overseasGetReward.on(Laya.Event.CLICK,this,this.onDirectlyGetClick);
            this.waitingBtn = this.getChildByName("waiting");
            this.timeText = this.waitingBtn.getChildByName("time");
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().registEvent("CLOSE_LUCKYBAG_DIALOG", this, this.onCloseClick);
            EventMgr.getInstance().registEvent("UPDATE_LUCKYBAG_DIALOG", this, this.loadLuckyBagData);
            this.loadLuckyBagData();
            //  this.directlyGet.text = Strings.get("tid#Btn_019");
             this.detail = this.getChildByName("detail");
             this.detail.text = Strings$1.get("tid#UI_030");
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
            EventMgr.getInstance().removeEvent("CLOSE_LUCKYBAG_DIALOG", this, this.onCloseClick);
            EventMgr.getInstance().removeEvent("UPDATE_LUCKYBAG_DIALOG", this, this.loadLuckyBagData);
        }

        changeClickSign(){
            this.luckyClickSign = 0;
            Laya.timer.once(500,this,function(){
                this.luckyClickSign = 1;
            });
        }

        loadLuckyBagData(){
            let num = LuckyBagSystem.getReward();
            this.numText.text = "+" + Strings$1.dealFishNumberText(num);
            this.showTime();
            Laya.timer.loop(1000,this,this.showTime);
        }

        showTime(){
            let str = "";
            this.remainingTime = LuckyBagSystem.getRemainingTime();
            if(this.remainingTime > 0){
                this.waitingBtn.visible = true;
                this.timeText.visible = true;
                this.adSbtn.visible = false;
                this.adVbtn.visible = false;
                this.overseasGetReward.visible = false;
                // this.directlyGet.visible = false;

                let minute = this.remainingTime / 60 | 0;
                let second = this.remainingTime % 60;
                if(minute<10){
                    str += "0";
                }
                str += minute + ":";
                if(second<10){
                    str += "0";
                }
                str += second;
                this.timeText.text = str;
            }
            else {
                this.timeText.visible = false;
                this.waitingBtn.visible = false;
                // this.directlyGet.visible = true;
                this.overseasGetReward.visible = true;
                if(SdkHelper.hasVideo()) {
                    this.adSbtn.visible = false;
                    this.adVbtn.visible = true;
                }else {
                    this.adSbtn.visible = true;
                    this.adVbtn.visible = false;
                }
            }
            
        }

        onCloseClick(){
            this.close();
        }

        getReward(fetchType){
            let handler=Laya.Handler.create(this,this.flyYugan,[fetchType]);
            LuckyBagSystem.receiveLuckyBag(fetchType, handler);
        }

        flyYugan(fetchType, res) {
            EventMgr.getInstance().postEvent("CHECK_CLOSE_LUCKYBAG");
            let num = res.rewards.rewards[0].amount;
            let rewardType = res.rewards.rewards[0].id;
            let x = Laya.stage.width/2;
            let y = Laya.stage.height/2;
            if (fetchType == Constants.FetchType.AD) {
                ItemFlyMgr.getInstance().flyReward(rewardType, num,Constants.FlyType.TYPE4,x-33, y-24);
            } else {
                ItemFlyMgr.getInstance().flyReward(rewardType, num,Constants.FlyType.TYPE5,x-33, y-24);
            }
            this.luckyClickSign = 1;
        }

        onDirectlyGetClick(){
            if(this.luckyClickSign > 0){
                this.changeClickSign();
                this.getReward(Constants.FetchType.DIRECT);
                this.close();
            }
        }

        //广告分享
        onAdClick(){
            if(this.luckyClickSign > 0){
                this.changeClickSign();
                SdkHelper.showRewardVideoAd(Constants.ShareId.LuckyBagDouble);
            } 
        }

        onShareClick(){
            if(this.luckyClickSign > 0){
                this.changeClickSign();
                SdkHelper.shareAppMessage(Constants.ShareId.LuckyBagDouble);
            }
        }

        doneV(shareId) {
            if(shareId == Constants.ShareId.LuckyBagDouble) {
                this.getReward(Constants.FetchType.AD);
                this.close();
            }
        }

        doneS(shareId) {
            if(shareId == Constants.ShareId.LuckyBagDouble) {
                this.getReward(Constants.FetchType.AD);
                this.close();
            }
        }
    }

    class LuckyDrawPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            EventMgr.getInstance().postEvent("SERVICE_GET_LUCKYDRAWINFO");
            this.clickSign = 1;
            this.whirling = 0;
            this.whirlType = 0;
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.free.on(Laya.Event.CLICK,this,this.onFreeClick);
            this.once1.on(Laya.Event.CLICK,this,this.onOnceClick);
            this.video.on(Laya.Event.CLICK,this,this.onVideoClick);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.freeText.text = Strings$1.get("tid#LuckyDrawBtn1");
            this.costWords.text = Strings$1.get("tid#UI_117") + ":";
            this.tip1.text = Strings$1.get("tid#LuckyDrawUI_1");
            this.updateUI();
            this.onceText.text = Strings$1.get("tid#LuckyDrawBtn2");
            let DrawLimitRefreshTime = JSON.parse(Const.get("DrawLimitRefreshTime"));
            let str = DrawLimitRefreshTime[0] + "";
            for(let i = 1; i < DrawLimitRefreshTime.length; i++){
                str += "/" + DrawLimitRefreshTime[i];
            }
            this.timesTip.text = Strings$1.get("tid#LuckyDrawUI_4",[str]);
            this.zuanshi.x = this.currency.x  + this.currency.width / 2 + 5;
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().registEvent("FINISH_LUCKYDRAW_REWARD", this, this.finishReward);
            EventMgr.getInstance().registEvent("UPDATE_ LUCKYDRAWPOP_UI", this, this.updateUI);
            let packet1GameTimeData = LuckydrawSystem.getPacket1GameTimeData();
            this.Packet1GameTime = packet1GameTimeData[0] * 1000;
            this.Packet1GameAutoTime = packet1GameTimeData[1] * 1000;
            this.countdownTime.visible = false;
            this.maskBox.visible = false;
        }

        updateUI(){
            this.fishReward = LuckydrawSystem.getFishReward();
            this.maxValueBase = this.fishReward[2];
            let rewards = LuckydrawSystem.getPrizeWheelReward();
            for (let index = 0; index < rewards.length; index++) {
                const element = rewards[index];
                let item = this.luckydraw.getChildByName("drawItem" + index);
                let num = item.getChildByName("num");
                num.fontSize = 40;
                let icon = item.getChildByName("icon");
                icon.rotation = 0;
                let i = index;
                item.rotation = index * 45;
                item.offAll(Laya.Event.CLICK);
                item.on(Laya.Event.CLICK, this, function () {
                    GameContext.CurrentInfoType = "luckyDraw";
                    GameContext.CurrentInfoId = i;
                    Laya.Dialog.close("ui/InfoPop.scene");
                    Laya.Dialog.open("ui/InfoPop.scene", false);
                });
                switch (element.type) {
                    case "FishHigh": {
                        item.mouseEnabled = false;
                        num.text = "+" + Strings$1.dealFishNumberText(this.fishReward[1], 1);
                        icon.skin = "topui/yugan2.png";
                        icon.scale(2, 2);
                        icon.rotation = -35;
                        break;
                    }
                    case "FishLow": {
                        item.mouseEnabled = false;
                        num.text = "+" + Strings$1.dealFishNumberText(this.fishReward[0], 1);
                        icon.skin = "topui/yugan1.png";
                        icon.scale(1, 1);
                        icon.rotation = -35;
                        break;
                    }
                    case "Packet0": {
                        item.mouseEnabled = true;
                        num.text = Strings$1.get("tid#UI_171");
                        icon.skin = "luckydraw/question.png";
                        icon.scale(2.5, 2.5);
                        break;
                    }
                    case "Packet1": {
                        item.mouseEnabled = true;
                        num.text = Strings$1.get("tid#UI_114");
                        icon.skin = "luckydraw/top_packet.png";
                        icon.scale(2, 2);
                        break;
                    }
                    case "Packet2": {
                        item.mouseEnabled = true;
                        num.text = Strings$1.get("tid#UI_115");
                        icon.skin = "luckydraw/toy_packet.png";
                        icon.scale(2, 2);
                        break;
                    }
                    case "Packet3": {
                        item.mouseEnabled = true;
                        num.text = Strings$1.get("tid#UI_116");
                        icon.skin = "luckydraw/sail_packet.png";
                        icon.scale(2, 2);
                        break;
                    }
                }
            }
            this.free.visible = false;
            this.once1.visible = false;
            this.video.visible = false;
            this.share.visible = false;
            let free = LuckydrawSystem.getFreeTimes();
            this.coinNum = PlayerSystem.getDiamond();
            this.costNum = LuckydrawSystem.getDiamondDrawSpend();
            let videoData = LuckydrawSystem.getVideoTimes();
            this.videoTimes = videoData[0];
            this.videoMax = videoData[1];
            if(free){
                this.free.visible = true;
            }
            else if(this.coinNum >= this.costNum){
                this.once1.visible = true;
            }
            else {
                if(SdkHelper.hasVideo()) {
                    this.share.visible = false;
                    this.video.visible = true;
                }else {
                    this.share.visible = true;
                    this.video.visible = false;
                }
                this.share.gray = this.videoTimes >= this.videoMax;
                this.video.gray = this.videoTimes >= this.videoMax;
            }
            this.times.color = this.videoTimes < this.videoMax ? "#72584a" : "#ff0000";
            this.currency.text = this.coinNum + "/" +  this.costNum;
            this.currency.color = this.coinNum >= this.costNum ? "#72584a" : "#ff0000";
            this.times.text = (this.videoMax - this.videoTimes) + "/" + this.videoMax;
            this.tip1.x = (this.width - this.times.width - this.tip1.width) / 2;
            this.times.x = this.tip1.x + this.tip1.width;
        }

        changeUI(){
            switch(this.whirlType){
                case 1:
                    this.free.visible = false;
                    break;
                case 2:
                    this.videoTimes ++;
                    break;
                case 3:
                    this.coinNum -= this.costNum;
                    this.once1.visible = false;
                    break;
            }
            if(LuckydrawSystem.getFreeTimes()){
                this.free.visible = true;
                this.once1.visible = false;
                this.share.visible = false;
                this.video.visible = false;
            }
            else if(this.coinNum >= this.costNum){
                this.once1.visible = true;
            }
            else {
                if(SdkHelper.hasVideo()) {
                    this.share.visible = false;
                    this.video.visible = true;
                }else {
                    this.share.visible = true;
                    this.video.visible = false;
                }
                this.times.color = this.videoTimes < this.videoMax ? "#72584a" : "#ff0000";
                this.share.gray = this.videoTimes >= this.videoMax;
                this.video.gray = this.videoTimes >= this.videoMax;
            }
            this.currency.text = this.coinNum + "/" +  this.costNum;
            this.currency.color = this.coinNum >= this.costNum ? "#72584a" : "#ff0000";
            this.times.text =  (this.videoMax - this.videoTimes) + "/" + this.videoMax;
        }

        finishReward(){
            if(this.anim){
                this.anim.destroy();
            }
            this.bg.visible = false;
            this.whirling = 0;
            this.updateUI();
        }

     
        playWhirling() {
            this.maskBox.visible = true;
            this.whirling = 1;
            this.changeUI();
            this.luckydraw.rotation = 0;
            let PrizeWheelData = LuckydrawSystem.getPrizeWheelNum();
            this.num = PrizeWheelData[0];
            this.rewardType = PrizeWheelData[1];
            let ran = Math.random() * 35 - 22.5;
            this.v = this.luckydraw.rotation + 3600 - this.num * 45 - ran -5;
            this.r = 0;
            Laya.Tween.to(this,{r : this.v},2000,Laya.Ease.circOut,Laya.Handler.create(this, function (){
                this.needle.rotation = 0;
                if (this.rewardType == "Packet1") {
                    this.playInteractive();
                }
                else {
                    this.sendReward();
                }
            }));
        }

        sendReward(){
            let json = {
                "FISH": this.fishCilckamount,
                "STAR": this.lingdangCilckamount,
                "fishAmount":this.fishCilckNum,
                "starAmount":this.lingdangCilckNum
            };
            if(this.rewardType == "Packet1"){
                var rewardInfo = {
                    "type":this.rewardType,
                    "maxCatValue":this.maxValueBase,
                    "json": JSON.stringify(json)
                };
            }
            else {
                var rewardInfo = {
                    "type":this.rewardType,
                    "maxCatValue":this.maxValueBase,
                    "json": "{}"
                };
            }
            LuckydrawSystem.fetchLuckyDrawReward(this.whirlType, rewardInfo ,Laya.Handler.create(this,this.fetchReward));
        }

        fetchReward(res){
            if (this.rewardType == "Packet2" || this.rewardType == "Packet3") {
                this.bg.visible = true;
                this.anim = new Spine();
                this.anim.loadAnim("anim3/libaodonghua.sk");
                this.anim.addCompleteHandler(Laya.Handler.create(this, function () {
                    this.maskBox.visible = false;
                    this.anim.destroy();
                    if(this.num == 5){
                        Laya.Dialog.close("ui/LuckdrawLevelReward.scene");
                        Laya.Dialog.open("ui/LuckdrawLevelReward.scene",false,res);
                    }
                    else {
                        Laya.Dialog.close("ui/LuckdrawReward.scene");
                        Laya.Dialog.open("ui/LuckdrawReward.scene",false,res);
                    }
                }));
                this.animBox.addChild(this.anim);
                if (this.rewardType == "Packet2") {
                    this.anim.playAnim("libao2", false);
                }
                else {
                    this.anim.playAnim("libao1", false);
                }
            }
            else {
                this.maskBox.visible = false;
                if (this.rewardType != "Packet1") {
                    this.bg.visible = true;
                }
                Laya.Dialog.close("ui/LuckdrawReward.scene");
                Laya.Dialog.open("ui/LuckdrawReward.scene",false,res);
            } 
            EventMgr.getInstance().postEvent("CHECK_LUCKYDRAWREDPOINT");
        }

        /////////////////////////////////////////互动礼包////////////////////////////////////////////
        playInteractive(){
            this.fishCilckNum = 0;
            this.lingdangCilckNum = 0;
            this.fishCilckamount = 0;
            this.lingdangCilckamount = 0;
            this.bg.visible = true;
            this.anim = new Spine();
            this.anim1 = new Spine();
            this.anim2 = new Spine();
            this.anim2.loadAnim("anim2/duibo.sk");
            this.anim2.playAnim("go",false);
            this.animBox.addChild(this.anim);
            this.animBox.addChild(this.anim1);
            this.animBox.addChild(this.anim2);
            this.anim.addCompleteHandler(Laya.Handler.create(this,function(){
                this.maskBox.visible = false;
                this.bg.on(Laya.Event.CLICK,this,this.onInteractiveClick);
                this.remainingTime = this.Packet1GameTime / 1000 | 0 + 1;
                this.countdownTime.visible = true;
                this.setCountdownTime();
                Laya.timer.loop(this.Packet1GameAutoTime,this,this.onInteractiveClick);
                Laya.timer.loop(1000,this,this.setCountdownTime);
                Laya.timer.once(this.Packet1GameTime,this,this.finishInteractive);
            }));
            this.anim2.addCompleteHandler(Laya.Handler.create(this,function(){
                this.anim2.destroy();
                this.anim.loadAnim("anim3/libaodonghua.sk");
                this.anim.playAnim("libao3-1",false);
                this.anim1.loadAnim("anim2/duibo.sk");
                this.anim1.playAnim("dianjitishi");
            }));
        }

        finishInteractive(){
            this.bg.off(Laya.Event.CLICK,this,this.onInteractiveClick);
            Laya.timer.clearAll(this);
            this.anim1.destroy();
            this.countdownTime.visible = false;
            this.sendReward();
        }

        onInteractiveClick(){
            this.anim.play("libao3-2",false);
            this.raiseCurrency();
        }

        raiseCurrency(){
            let currencyData = LuckydrawSystem.getPacket1Currency();
            let currency = currencyData[0];
            let currencyAmount = Number(currencyData[1]);
            let skin = "";
            if(currency == Constants.RewardType.FISH){
                skin = "topui/yugan1.png";
                this.fishCilckNum ++;
                this.fishCilckamount += currencyAmount;
            }
            else if(currency == Constants.RewardType.STAR){
                skin = "topui/xingxing1.png";
                this.lingdangCilckNum ++;
                this.lingdangCilckamount += currencyAmount;
            }
            let img = new Laya.Image();
            img.skin = skin;
            img.scale(1.2,1.2);
            this.imgBox.addChild(img);
            Laya.Tween.to(img,{y:img.y - 200},100,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(img,{y:img.y - 200,alpha:0},400,Laya.Ease.circIn);
            }));
        }

        setCountdownTime(){
            if(this.remainingTime >= 0){
                this.remainingTime -= 1;
                this.countdownTime.text = Strings$1.get("tid#UI_118")+ this.remainingTime + Strings$1.get("tid#UI_119");
            }
        }

        /////////////////////////////////////////按钮点击事件////////////////////////////////////////////
        onCloseClick(){
            this.close();
        }

        onFreeClick(){
            if(this.clickSign > 0 && !this.whirling){
                this.changeClickSign();
                this.whirlType = 1;
                this.playWhirling();
            }
        }

        onOnceClick(){
            if(this.clickSign > 0 && !this.whirling){
                this.changeClickSign();
                this.whirlType = 3;
                this.playWhirling();
            }
        }

        onShareClick(){
            if(this.clickSign > 0 && !this.whirling){
                this.changeClickSign();
                if(Number(this.videoTimes) < Number(this.videoMax)){
                    SdkHelper.shareAppMessage(Constants.ShareId.Luckydraw);
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#LuckyDrawTips_1");
                }
            }
        }

        onVideoClick(){
            if(this.clickSign > 0 && !this.whirling){
                this.changeClickSign();
                if(this.videoTimes < this.videoMax){
                    SdkHelper.showRewardVideoAd(Constants.ShareId.Luckydraw);
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#LuckyDrawTips_1");
                }
            } 
        }

        changeClickSign(){
            this.clickSign = 0;
            Laya.timer.once(500,this,function(){
                this.clickSign = 1;
            });
        }

        done(shareId) {
            if (shareId == Constants.ShareId.Luckydraw) {
                this.whirlType = 2;
                this.playWhirling();
            }
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("FINISH_LUCKYDRAW_REWARD", this, this.finishReward);
            EventMgr.getInstance().removeEvent("UPDATE_ LUCKYDRAWPOP_UI", this, this.updateUI);
        }

        set r(num){
            this._r = num;
            this.luckydraw.rotation = num;
            let a = num / this.v * 360;
            let rate = 1.1 - a / 360;
            this.needle.rotation = Math.sin(a) * 10 * rate;
        }

        get r(){
            return this._r
        }
    }

    /**
    * 邮件
    */
    class MailListPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,1);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.close);
            this.none = this.getChildByName("none");
            this.none.visible = false;
            this.none.text = Strings$1.get("tid#UI_121");
            this.list = this.getChildByName("list");
            this.list.visible = false;
            EventMgr.getInstance().postEvent("SERVICE_MAIL_FETCH_ALL_MAILS");
            EventMgr.getInstance().registEvent("UDATA_MAIL", this, this.updataMails);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UDATA_MAIL", this, this.updataMails);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,0);
        }

        updataMails() {
            let mails = MailSystem.getMails();
            //未读
            let array = new Array();
            //已读已领奖
            let receiveArray = new Array();
            //已读未领奖
            let unreceiveArray = new Array();
            mails.forEach((mail) => {
                if (mail.read) {
                    if (mail.receive) {
                        receiveArray.push(mail);
                    } else {
                        unreceiveArray.push(mail);
                    }
                } else {
                    array.push(mail);
                }
            });
            function sortByTime(a, b) {
                return a.expireTime - b.expireTime;
            }
            array.sort(sortByTime);
            receiveArray.sort(sortByTime);
            unreceiveArray.sort(sortByTime);
            array = array.concat(unreceiveArray, receiveArray);
            unreceiveArray = null;
            receiveArray = null;
            this.list.array = array;
            if(array.length > 0){
                this.none.visible = false;
                this.list.visible = true;
            }
            else {
                this.none.visible = true;
            }
            this.list.refresh();
        }

    }

    class MailItem extends Laya.Box {

        constructor() {
            super();
        }

        onEnable() {
            this.readImage = this.getChildByName("read");
            this.unReadImage = this.getChildByName("unRead");
            this.title = this.getChildByName("title");
            this.receiveTime = this.getChildByName("receiveTime");
            this.liveTime = this.getChildByName("liveTime");
            this.giftList = this.getChildByName("giftList");
            this.on(Laya.Event.CLICK, this, this.read);
        }

        setItem(mailData) {
            if (mailData == null) {
                return;
            }
            this.mailData = mailData;
            this.updataView();
        }

        updataView() {
            this.title.text = this.mailData.title;
            let now = TimeUtils.getTimeNowOnServer() / 1000;
            this.liveTime.text = Strings$1.getDaysBySeconds(this.mailData.expireTime - now) + Strings$1.get("tid#UI_120");
            // this.liveTime.text = Strings.get("tid#UI_120");
            this.receiveTime.text = Strings$1.getTimeBySeconds(now - this.mailData.sendTime);
            if (this.mailData.read) {
                this.readImage.visible = true;
                this.unReadImage.visible = false;
            } else {
                this.readImage.visible = false;
                this.unReadImage.visible = true;
            }
            if (this.mailData.receive) {
                this.giftList.visible = false;
            } else {
                this.giftList.visible = true;
                this.giftList.array = this.mailData.rewards;
            }
        }

        read() {
            if (this.mailData == null) {
                return;
            }
            Laya.Dialog.close("ui/MailPop.scene");
            Laya.Dialog.open("ui/MailPop.scene", false, this.mailData);
        }
    }

    /**
    * 邮件照片展示
    */
    class MailPhotoShow extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.close);
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.photoBg = this.getChildByName("photoBg");
            this.photo = this.getChildByName("photo");
            this.photoBg.skin = "systemPhotoBg/"+this._param.Background+".jpg";
            let catSkins = JSON.parse(this._param.Cat);
            let catCatroute = "systemPhoto/"+this._param.Catroute;
            for (const catSkin of catSkins) {
                let cat = new Laya.Image();
                cat.skin = catCatroute + "/" + catSkin + ".png";
                cat.width = this.photo.width;
                cat.height = this.photo.height;
                this.photo.addChild(cat);
            }
        }

        onDisable() {
            this.photo.destroyChildren();
        }

    }

    /**
    * 邮件
    */
    class MailPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.close);
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.mailData = this._param;
            this.konwBtn = this.getChildByName("konwBtn");
            this.konwBtn.on(Laya.Event.CLICK, this, this.onKnowClick);
            this.konwBtn.label = Strings$1.get("tid#Btn_024");
            let title = this.getChildByName("title");
            this.contextPanel = this.getChildByName("contextPanel");
            let content = this.contextPanel.getChildByName("content");
            title.text = this.mailData.title;
            content.text = this.mailData.content;
            if (!this.mailData.read) {
                EventMgr.getInstance().postEvent("SERVICE_MAIL_READ_MAIL", this.mailData.id);
            }
            console.log(this.mailData.mailItems);
            this.rewardItems = [];
            if (this.mailData.mailItems && this.mailData.mailItems.rewards && this.mailData.mailItems.rewards.length > 0) {
                this.loadRewardPrefab();
                this.contextPanel.height = 590;
            } else {
                this.contextPanel.height = 760;
            }
        }

        loadRewardPrefab(completeHandler) {
            this.loadResource("prefab/ui/MailReward.prefab", completeHandler);
        }

        loadResource(path, completeHandler) {
            if (path.indexOf(".prefab") != -1) {
                Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                    let startX = (this.width - 160 * this.mailData.mailItems.rewards.length) / 2;
                    for (let i = 0; i < this.mailData.mailItems.rewards.length; i++) {
                        let mailPrefab = prefabDef.create();
                        mailPrefab.y = 1483;
                        mailPrefab.x = startX + i * 160;
                        this.addChild(mailPrefab);
                        let rewardImage = mailPrefab.getChildByName("reward");
                        let reward = this.mailData.mailItems.rewards[i];
                        let rewardNum = mailPrefab.getChildByName("rewardNum");
                        let received = mailPrefab.getChildByName("received");
                        this.rewardItems.push(mailPrefab);
                        rewardNum.text = Strings$1.dealFishNumberText(reward.amount);
                        if (this.mailData.receive) {
                            received.visible = true;
                        } else {
                            received.visible = false;
                        }
                        switch (reward.type) {
                            case Constants.ItemType.Currency: {
                                if (reward.id == Constants.RewardType.FISH) {
                                    rewardImage.skin = "topui/yugan1.png";
                                } else if (reward.id == Constants.RewardType.STAR) {
                                    rewardImage.skin = "topui/xingxing1.png";
                                } else if (reward.id == Constants.RewardType.DIAMOND || reward.id == "PAY_DIAMOND") {
                                    rewardImage.skin = "topui/diamond.png";
                                } else if (reward.id == Constants.RewardType.SHELL) {
                                    rewardImage.skin = "topui/beike1.png";
                                } else if (reward.id == Constants.RewardType.MEDAL) {
                                    rewardImage.skin = "topui/huizhang1.png";
                                } else if (reward.id == Constants.RewardType.BOOK) {
                                    rewardImage.skin = "topui/miji1.png";
                                }
                                break;
                            }
                            case Constants.ItemType.NauticalFoods: {
                                let config = ConfigMgr$1.getInstance().getItemById("NauticalFoods", reward.id);
                                    rewardImage.skin = config.Icon;
                                break;
                            }
                            case Constants.ItemType.NauticalTools:{
                                let toolConfig = ConfigMgr$1.getInstance().getItemById("NauticalTools", reward.id);
                                rewardImage.skin = toolConfig.Icon;
                                break;
                            }
                            case Constants.ItemType.FishingBaits:{
                                let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", reward.id);
                                rewardImage.skin = baitConfig.Icon;
                                break;
                            }
                            case Constants.ItemType.CatToys:{
                                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", reward.id);
                                rewardImage.skin = toyConfig.Icon;
                                break;
                            }
                            case Constants.ItemType.CatPhotos: {
                                let config = ConfigMgr$1.getInstance().getItemById("CatPhotos", reward.id);
                                rewardImage.skin = "systemPhotoBg/"+config.Background+".jpg";
                                rewardImage.height = 80;
                                let catSkins = JSON.parse(config.Cat);
                                let catCatroute = "systemPhoto/" + config.Catroute;
                                for (const catSkin of catSkins) {
                                    let cat = new Laya.Image();
                                    cat.skin = catCatroute + "/" + catSkin + ".png";
                                    cat.width = 100;
                                    cat.height = 80;
                                    cat.anchorX = 0.5;
                                    cat.anchorY = 0.5;
                                    cat.x = rewardImage.x;
                                    cat.y = rewardImage.y;
                                    cat.zOrder = 2;
                                    mailPrefab.addChild(cat);
                                }
                                let magnifier = new Laya.Image();
                                magnifier.zOrder = 3;
                                magnifier.x = 80;
                                magnifier.y = 20;
                                magnifier.scaleX = magnifier.scaleY = 2.5;
                                magnifier.skin = "mail/magnifier.png";
                                mailPrefab.addChild(magnifier);
                                mailPrefab.on(Laya.Event.CLICK, this, function () {
                                    Laya.Dialog.open("ui/MailPhotoShow.scene", false, config);
                                });
                                break;
                            }
                            // default: {
                            //     skin = "topui/yugan.png";
                            //     break;
                            // }
                        }
                    }
                    if (completeHandler != null) {
                        completeHandler.run();
                    }
                }));
            }
        }

        onKnowClick() {
            if (this.mailData.mailItems && this.mailData.mailItems.rewards && this.mailData.mailItems.rewards.length > 0 && !this.mailData.receive) {
                EventMgr.getInstance().postEvent("SERVICE_RECEIVE_MAIL_REWARDS", [this.mailData.id]);
            }
            this.close();
        }

        onDisable() {
            for (const rewardItem of this.rewardItems) {
                rewardItem.destroy();
            }
        }

    }

    class MakeupPop extends Laya.Dialog {
        constructor(){ 
             super();
         }

         onEnable(){
            this.close1.on(Laya.Event.CLICK,this,this.close);
            this.video.on(Laya.Event.CLICK,this,this.onVideoClick);
            this.detail.text = Strings$1.get("tid#TipsCommon61");
         }
        
         onVideoClick(){
            this.close();
            EventMgr.getInstance().postEvent("BCJ_VIDEO_CLICK");
         }
    }

    /**
    * 确认放置
    */
    class MergePop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.slider.bar.stateNum = 1;
            this.slider._bg.skin="menu/progress2.png";
            this.slider._bg.height=27;
            this.slider._bg.sizeGrid="5,6,5,5";
            let pro=new Laya.Image("menu/progress1$bar.png");
            pro.sizeGrid="5,6,5,5";
            pro.height=27;
            this.slider._progress = pro;
            this.slider.addChild(this.slider._progress);
            this.slider.setChildIndex(this.slider._progress,1);
            this.slider.setChildIndex(this.kuang,2);
            this.slider.setChildIndex(this.slider.bar,3);
            this.on(Laya.Event.MOUSE_UP,this,this.jiaozheng);
            this.on(Laya.Event.MOUSE_OUT,this,this.jiaozheng);
            if(GameContext.CurrentInfoType=="food")
                this.showFood(GameContext.CurrentInfoId);
            else
                this.close();

            this.cost.text = Strings$1.get("tid#Btn_023");
        }

        showFood(foodId) {
            if(foodId==null){
                this.close();
                return;
            }
            let item = NauticalSystem$1.getFoodCanMergeInfo(foodId);
            if(item==null){
                this.close();
                return;
            }
            this.label2.text = Strings$1.get(item.Name);
            this.count2.text = "0";
            this.setIcon(this.icon2, item.Icon);

            let upgradeLine = JSON.parse(item.UpgradeLine)[0];
            this._mergeUnit = Number(upgradeLine.amount);
            this._prefood = NauticalSystem$1.getFoodById(foodId);
            this._mergeMax = Math.floor(this._prefood.count/this._mergeUnit);
            this.slider.max = this._mergeMax;
            this.slider.min = 0;
            this.slider.tick = 0.1;
            this.slider.value = 0;
            this.slider.changeHandler = Laya.Handler.create(this, this.updataSlider, null, false);

            this.label1.text = Strings$1.get(this._prefood.Name);
            this.count1.text = "0/"+String(this._prefood.count);
            this.setIcon(this.icon1, this._prefood.Icon);

            this.confirm.on(Laya.Event.CLICK, this, this.onMerge, [item.Id]);
            this.plus.on(Laya.Event.CLICK, this, this.onPlus);
            this.minus.on(Laya.Event.CLICK, this, this.onMinus);

            this._mergeCount = 0;
            if(this._mergeMax>this._mergeCount) {
                this.plus.disabled=false;
                this.minus.disabled=false;
                this.confirm.disabled=false;
                this.slider.disabled=false;
            }
            else {
                this.plus.disabled=true;
                this.minus.disabled=true;
                this.confirm.disabled=true;
                this.slider.disabled=true;
            }
            this.onMax();
     
        }

        setIcon(icon, skin) {
            icon.skin = skin;
            let scalex = 150 / icon.width;
            let scaley = 150 / icon.height;
            if (scalex <= scaley) {
                icon.scaleX = scalex;
                icon.scaleY = scalex;
            } else {
                icon.scaleX = scaley;
                icon.scaleY = scaley;
            }
        }

        setGray() {
            this.confirm.gray = true;
            this.confirm.disabled = true;
        }

        onOpened(param) {
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/shopPop.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onMerge(foodId) {
            if(!this._mergeCount||this._mergeCount==0) {
                return;
            }
            this.close();
            // Laya.Dialog.closeAll();
            NauticalSystem$1.mergeNauticalFood(foodId, this._mergeCount, Laya.Handler.create(this, function() {
                EventMgr.getInstance().postEvent("NAUTICAL_FOODS_CHANGED");
                EventMgr.getInstance().postEvent("NAUTICAL_FOOD_UPDATE");
            }));
            TipsPopMgr.getInstance().showTipsById("tid#CatFoodTips1");
        }
        onMax() {
            this._mergeCount=this._mergeMax;     
            this.count1.text = String(this._mergeUnit*this._mergeCount)+"/"+String(this._prefood.count);
            this.count2.text = String(this._mergeCount);
            this.slider.value = this._mergeCount;
        }
        onPlus() {
            if(this._mergeMax>this._mergeCount) {
                ++this._mergeCount;
                this.plus.disabled=false;
                this.minus.disabled=false;
            }
            else {
                this.plus.disabled=true;
            }
            
            this.count1.text = String(this._mergeUnit*this._mergeCount)+"/"+String(this._prefood.count);
            this.count2.text = String(this._mergeCount);
            this.slider.value = this._mergeCount;
        }

        onMinus() {
            if(this._mergeCount>0) {
                --this._mergeCount;
                this.minus.disabled=false;
                this.plus.disabled=false;
            }
            else {
                this.minus.disabled=true;
            }
            this.count1.text = String(this._mergeUnit*this._mergeCount)+"/"+String(this._prefood.count);
            this.count2.text = String(this._mergeCount);
            this.slider.value = this._mergeCount;
        }

        updataSlider(value) {
           if(this.slider.value<=this._mergeMax && this.slider.value>=0) {
                this._mergeCount = this.slider.value;
                this.count1.text = String(this._mergeUnit *Math.floor(this._mergeCount)) + "/" + String(this._prefood.count);
                this.count2.text = String(Math.floor(this._mergeCount));
             }
             if(this.slider.value>=this._mergeMax){
                this.minus.disabled=false;
                this.plus.disabled=true;
                this.confirm.disabled = false;
             }else if(this.slider.value<=0){
                this.minus.disabled=true;
                this.plus.disabled=false;
                this.confirm.disabled = true;
             }else {
                this.minus.disabled=false;
                this.plus.disabled=false;
                this.confirm.disabled = false;
             }
             if(this._mergeMax<=0){
                this.minus.disabled=true;
                this.plus.disabled=true;
                this.confirm.disabled = true;
             }
             this.slider.value =value;
    }
        jiaozheng(){
        let value=this.slider.value;
        if(value-Math.floor(value)>=0.5){
            value=Math.ceil(value);
        }else {
            value=Math.floor(value);
        }
        this.updataSlider(value);
       };
        onClosed(){
            this.slider.changeHandler.recover();
        }
    }

    class MileStoneNewsPaper extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.MileStone,1);
            this.Id = this._param;
            this.configData = MileStoneSystem.getConfig(this.Id);
            this.title.skin = this.configData.ContentTitle;
            this.type = this.configData.Type;
            let bottom = this.configData.BottomConfig;
            this.img1.skin = this.configData.TopImage;
            this.img1_1.skin = this.configData.TopImage1;
            this.img1_2.skin = this.configData.TopImage2;
            this.detail1_1.text = "       " + Strings$1.get(this.configData.Desc1);
            this.detailBg.skin = Const.get("MilestoneStructureRoute") + this.type + ".jpg";
            this.type1.visible = false;
            this.type2.visible = false;
            this.type3.visible = false;
            if(this.type == 1){
                this.type1.visible = true;
                this.img2.loadImage(bottom,Laya.Handler.create(this,function(){
                    this.img2.skin = bottom;
                    let scale = 142 / this.img2.height;
                    this.img2.scale(scale,scale);
                }));
            }
            else if(this.type == 2){
                this.type2.visible = true;
                this.img3.loadImage(bottom,Laya.Handler.create(this,function(){
                    this.img3.skin = bottom;
                    let scale = 142 / this.img3.height;
                    this.img3.scale(scale,scale);
                }));
            }
            else if(this.type == 3){
                this.type3.visible = true;
                this.img4.loadImage(bottom,Laya.Handler.create(this,function(){
                    this.img4.skin = bottom;
                    let scale = 142 / this.img4.height;
                    this.img4.scale(scale,scale);
                }));
            }
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.date.text = Strings$1.get("tid#UI_122");
        }

        onCloseClick(){
            this.close();
            if(JSON.parse(this.configData.LetterReward)[0]){
                Laya.Dialog.close("ui/MileStoneReward.scene");
                Laya.Dialog.open("ui/MileStoneReward.scene",false,this.configData);
            }
            else {
                MileStoneSystem.getMileStoneReward(this.Id ,Constants.FetchType.DIRECT);
            }
        }

    }

    class MileStoneReward extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.configData = this._param;
            this.close1.on(Laya.Event.CLICK,this,this.onCloseClick);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.title.text = Strings$1.get(this.configData.LetterTitle);
            this.detail.text = "       " + Strings$1.get(this.configData.LetterContent);
            this.from.text = Strings$1.get(this.configData.LetterSign);
            this.reward = JSON.parse(this.configData.LetterReward)[0];
            this.tipsImg = "";
            this.tipsWords = "";
            if(this.reward.type == "CatToys"){
                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", this.reward.id);
                this.icon.skin = toyConfig.Icon;
                this.name.text = Strings$1.get(toyConfig.Name);
                this.name.visible = true;
                this.number.visible = false;
                this.tipsImg = toyConfig.Icon;
                this.tipsWords = this.name.text;
            }
            else if(this.reward.type == "Currency"){
                if(this.reward.id == Constants.RewardType.FISH){
                    this.currency.skin = "topui/yugan1.png";
                    this.number.text = Strings$1.dealFishNumberText(this.reward.amount);
                }
                else if(this.reward.id == Constants.RewardType.STAR){
                    this.currency.skin = "topui/xingxing1.png";
                    this.number.text = Strings$1.dealFishNumberText(this.reward.amount);
                }
                else if(this.reward.id == Constants.RewardType.SHELL){
                    this.currency.skin = "topui/beike1.png";
                    this.number.text = Strings$1.dealFishNumberText(this.reward.amount);
                }
                else if(this.reward.id == Constants.RewardType.DIAMOND){
                    this.currency.skin = "topui/diamond.png";
                    this.number.text = Strings$1.dealFishNumberText(this.reward.amount);
                }
                this.number.visible = true;
                this.name.visible = false;
                this.tipsImg = this.currency.skin;
                this.tipsWords = this.number.text;
            }
            else if(this.reward.type == "FishingBaits"){
                let baitConfig = ConfigMgr$1.getInstance().getItemById("FishingBaits", this.reward.id);
                this.icon.skin = baitConfig.img;
                this.name.text = Strings$1.get(baitConfig.name);
                this.name.visible = true;
                this.number.visible = false;
                this.tipsImg = baitConfig.img;
                this.tipsWords = this.name.text;
            }
            else if(this.reward.type == "NauticalTools"){
                let toolConfig = ConfigMgr$1.getInstance().getItemById("NauticalTools", this.reward.id);
                this.icon.skin = toolConfig.Icon;
                this.name.text = Strings$1.get(toolConfig.Name);
                this.name.visible = true;
                this.number.visible = false;
                this.tipsImg = toolConfig.Icon;
                this.tipsWords = this.name.text;
            }
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
        }

        onOpened(){
            let anim1 = new Spine();
            anim1.loadAnim("anim2/caidai.sk");
            this.animBox1.addChild(anim1);
            anim1.playAnim("caidai",false);
            let anim2 = new Spine();
            anim2.loadAnim("anim2/caidai.sk");
            this.animBox2.addChild(anim2);
            anim2.playAnim("caidai",false);
        }

        onCloseClick(){
            this.done();
        }

        onShareClick(){
            SdkHelper.shareAppMessage(Constants.ShareId.MileStone);
        }

        done(){
            this.close();
            let handler=Laya.Handler.create(this,this.getReward);
            MileStoneSystem.getMileStoneReward(this.configData.Id ,Constants.FetchType.DIRECT,handler);
        }

        getReward(res){
            let str = "&nbsp;<img src='" + this.tipsImg + "' style='width:50; height:50'></img>&nbsp;"+ this.tipsWords;
            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon59", [str]);
        }
        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.MileStone,0);
        }
    }

    class MiniGameTalkPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            this.talkBox = this.getChildByName("talkBox");
            this.catName = this.talkBox.getChildByName("catName");
            this.talk = this.talkBox.getChildByName("talk");
            this.cat = this.talkBox.getChildByName("cat");
            this.lvImage = this.talkBox.getChildByName("lvImage");
            this.lvImage.visible = false;
            this.talkBox.on(Laya.Event.CLICK, this, this.onTalkBoxClick);
        }

        onEnable() {
            this.minigameData = this._param[0];
            let result = this._param[1];
            this.currentTalkId = 0;
            this.talkArr = [];
            this._gameActive = MiniGameSystem.getActive();
            if (this.minigameData.getType() == Constants.MiniGameType.STRENGTH) {
                let powerGameData = MiniGameSystem.getOtherData().powerGameData;
                let talkConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameStrengthRole", powerGameData.playerId);
                if (this._gameActive) {
                    if (powerGameData.isFirst) {
                        let talka= JSON.parse(talkConfig.FirstStartDialog);
                        let suij=Math.random();
                        let i=Math.floor(suij*talka.length);
                        this.talkArr=talka[i];
                        MiniGameSystem.setTalkID(powerGameData.playerId);
                    } else {
                        let talka= JSON.parse(talkConfig.StartDialog);
                        let suij=Math.random();
                        let i=Math.floor(suij*talka.length);
                        this.talkArr=talka[i];
                        MiniGameSystem.setTalkID(powerGameData.playerId);
                    }
                } else {
                    if (result == Constants.MiniGameResult.KO) {
                        let talkConfigKO = ConfigMgr$1.getInstance().getItemById("SpecialGameStrengthRole",this.KOLastID());
                        let talka= JSON.parse(talkConfigKO.BigWinDialog);
                        let suij=Math.random();
                        let i=Math.floor(suij*talka.length);
                        this.talkArr=talka[i];
                    } else if (result == Constants.MiniGameResult.Win) {
                        let talka= JSON.parse(talkConfig.WinDialog);
                        let suij=Math.random();
                        let i=Math.floor(suij*talka.length);
                        this.talkArr=talka[i];
                    } else {
                        let talka= JSON.parse(talkConfig.FailDialog);
                        let suij=Math.random();
                        let i=Math.floor(suij*talka.length);
                        this.talkArr=talka[i];
                    }
                }
            } else {
                if (this._gameActive) {
                    this.talkArr = this.minigameData.getStartTalk();
                } else {
                    this.talkArr = this.minigameData.getFinishTalk();
                }
            }
            this.catName.text = Strings$1.get(this.minigameData.getCatName());
          
           
            if (this.minigameData.getType() == Constants.MiniGameType.MORA) {
                this.cat.skin = "miniGame/gameCat1.png";
            } else if (this.minigameData.getType() == Constants.MiniGameType.COIN) {
                this.cat.skin = "miniGame/gameCat2.png";
            } else if (this.minigameData.getType() == Constants.MiniGameType.STRENGTH) {
                this.cat.skin = "miniGame/gameCat3.png";
                this.lvImage.visible = true;
                let powerGameData = MiniGameSystem.getOtherData().powerGameData;;
                let playerId = powerGameData.playerId;
               // let playerLevel = Number(playerId.substr(11));
               let talkConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameStrengthRole", powerGameData.playerId);
               // let playerLevel = Number(playerId.substr(11));
        
               this.catName.text =Strings$1.get(talkConfig.Name);
            
               let playerLevel = talkConfig.Armband;
               if(result){
                if(result == Constants.MiniGameResult.KO){
                    let talkConfigKO = ConfigMgr$1.getInstance().getItemById("SpecialGameStrengthRole",this.KOLastID());
                    this.catName.text =Strings$1.get(talkConfigKO.Name);
                    if(playerLevel){
                        let playerLevel_1 = talkConfigKO.Armband;
                        this.lvImage.skin = "miniGame/lv" + playerLevel_1;
                    }else {
                        this.lvImage.skin = "miniGame/lv1.png";
                    }
                }
                else {
                    if(playerLevel){
                        this.lvImage.skin = "miniGame/lv" + playerLevel;
                    }else {
                        this.lvImage.skin = "miniGame/lv1.png";
                    }
                }
               }
               else {
                if(playerLevel){
                    this.lvImage.skin = "miniGame/lv" + playerLevel;
                }else {
                    this.lvImage.skin = "miniGame/lv1.png";
                }
               }        
            }
            this.talkShow();
        }

        talkShow() {
            if (this.currentTalkId < this.talkArr.length) {
                this.talkStr = Strings$1.get(this.talkArr[this.currentTalkId]);
                this.currentCharNum = 0;
                Laya.timer.loop(1, this, this.showTalk);
                this.currentTalkId++;
            } else {
                this.close();
                if (this._gameActive) {
                    if (this.minigameData.getType() == Constants.MiniGameType.MORA) {
                        Laya.Dialog.close("ui/MoraGamePop.scene");
                        Laya.Dialog.open("ui/MoraGamePop.scene", true, this.minigameData);
                    } else if (this.minigameData.getType() == Constants.MiniGameType.COIN) {
                        Laya.Dialog.close("ui/GuessGamePop.scene");
                        Laya.Dialog.open("ui/GuessGamePop.scene", true, this.minigameData);
                    } else if (this.minigameData.getType() == Constants.MiniGameType.STRENGTH) {
                        Laya.Dialog.close("ui/PowerGamePop.scene");
                        Laya.Dialog.open("ui/PowerGamePop.scene", true, this.minigameData);
                    }
                }
            }
        }

        //一个字一个字蹦
        showTalk(showAll = false) {
            if (showAll) {
                this.talk.text = this.talkStr;
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                return;
            }
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkShow();
                return;
            }
            this.talk.text = this.talkStr.substr(0, ++this.currentCharNum);
            if (this.currentCharNum == this.talkStr.length) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
            }
        }

        onTalkBoxClick() {
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                this.talkShow();
            } else {
                this.showTalk(true);
            }
        }

        onDisable() {
            this.minigameData = null;
            Laya.timer.clearAll(this);
        }
        KOLastID(){
             if(MiniGameSystem.getTalkID()!=null){
                 return MiniGameSystem.getTalkID();
             }
        
            let powerGameData = MiniGameSystem.getOtherData().powerGameData;  
            let  playerLevel=powerGameData.playerLevel;
            if(playerLevel>1){
                playerLevel-=1;
            }
            let configs = ConfigMgr$1.getInstance().getConfig("SpecialGameStrengthRole");
            let lastId = null;
            for (const [id, powerConfig] of configs) {
                if (Number(playerLevel) == Number(powerConfig.Order)) {
                    lastId = powerConfig.Id;
                    break;
                }
            }
            return lastId;
        }

    }

    class MissionCatCD extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            this.clickSign = 1;
            this.petId = this._param[0];
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.video.on(Laya.Event.CLICK,this,this.onAdClick);
            if(SdkHelper.hasVideo()) {
                this.share.visible = false;
                this.video.visible = true;
                this.label.text = Strings$1.get("tid#UI_223");
            }else {
                this.share.visible = true;
                this.video.visible = false;
                this.label.text = Strings$1.get("tid#UI_224");
            }
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        changeClickSign(){
            this.clickSign = 0;
            Laya.timer.once(500,this,function(){
                this.clickSign = 1;
            });
        }

        onShareClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.shareAppMessage(Constants.ShareId.MissionCatCD);
            } 
        }

        onAdClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.showRewardVideoAd(Constants.ShareId.MissionCatCD);
            } 
        }

        doneV(shareId){
            if(shareId == Constants.ShareId.MissionCatCD) {
                this.done();
                this.close();
            }
        }

        doneS(shareId){
            if(shareId == Constants.ShareId.MissionCatCD) {
                this.done();
                this.close();
            }
        }

        done(){
            MissionSystem.resetCatStatus(this.petId);
            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon90");
            this.close();
        }
    }

    class MissionCompletePop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this._rewards = this._param;
            this.okBtn = this.getChildByName("okBtn");
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            let closeBtn = this.getChildByName("closeBtn");
            closeBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onOkBtnClick);

            this.btn_text.text = Strings$1.get("tid#LuckyDrawBtn5");
            this.reward1 = this.getChildByName("reward1");
            this.reward2 = this.getChildByName("reward2");
            this.reward3 = this.getChildByName("reward3");
            this.showRewards();
        }

        showRewards() {
            let rewards = this._rewards.rewards;
            //924 1218 780 1070
            this.reward1.setItem(rewards[0]);
            if (rewards.length == 2) {
                this.reward2.setItem(rewards[1]);
                this.reward2.visible = true;
                this.reward2.x = 945;
                this.reward1.x = 611;
            } else if (rewards.length >= 3) {
                this.reward2.setItem(rewards[1]);
                this.reward2.visible = true;
                this.reward3.setItem(rewards[2]);
                this.reward3.visible = true;
                this.reward2.x = 776;
                this.reward1.x = 465;
            }
        }

        onOkBtnClick() {
            // this.reward1.showRewardTip();
            // this.reward2.showRewardTip();
            // this.reward3.showRewardTip();
            MissionSystem.getMissionInfo(Laya.Handler.create(this, function () {
                Laya.Dialog.open("ui/MissionMain.scene");
            }));
        }

    }

    class MissionFailPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.okBtn = this.getChildByName("okBtn");
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            let closeBtn = this.getChildByName("closeBtn");
            closeBtn.on(Laya.Event.CLICK, this, this.back);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.back);

            this.desc.text = Strings$1.get("tid#UI_203");
            this.btn_text.text = Strings$1.get("tid#Btn_043");
        }

        onOkBtnClick() {
            this.close();
            Laya.Dialog.open("ui/CollectPop.scene");
        }

        back() {
            this.close();
            let cur = MissionSystem.getCurrentMissionIndex();
            Laya.Dialog.open("ui/MissionTeam.scene", true, cur);
        }

    }

    class MissionGame extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            if (this._param == null) {
                this.close();
                return
            }
            this._missionIndex = this._param;

            let okBtn = this.getChildByName("okBtn");
            okBtn.on(Laya.Event.CLICK, this, this.skip);

            this.question1 = this.getChildByName("question1");
            this.question2 = this.getChildByName("question2");
            this.question3 = this.getChildByName("question3");

            this.pkCat = this.getChildByName("pkCat");
            this.pkPc = this.getChildByName("pkPc");

            this.waitCat1 = this.getChildByName("waitCat1");
            this.waitCat2 = this.getChildByName("waitCat2");
            this.waitCat3 = this.getChildByName("waitCat3");

            this.eventName = this.getChildByName("eventName");
            this.fighting = this.getChildByName("fighting");
            this.needValue = this.getChildByName("needValue");
            this.catValueName = this.getChildByName("catValueName");
            this.catValue = this.getChildByName("catValue");
            this.resultImage = this.getChildByName("resultImage");
            this._skiped = null;
            this.anim = new Spine();
            this.anim.visible = false;
            this.anim.loadAnim("anim3/wanshiwu.sk");
            this.addChild(this.anim);

            this.initFight();
            this.resetUI();
            this.showPc();
            this.showCats();
            this.eventFight();
        }
        onDisable() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }

        initFight() {
            let missionInfo = MissionSystem.getMissionByIndex(this._missionIndex);
            this.mission = MissionSystem.getMissionById(missionInfo.id);

            this.team = [];
            let team = MissionSystem.Team;
            team.forEach(catId => {
                if (catId) {
                    this.team.push(catId);
                }
            });

            this.fightResult = MissionSystem.fight(this._missionIndex, this.team);
            this.fightInfo = this.fightResult.infos;
            this.pcValue = this.fightResult.pcValue;

            this._eventIndex = 0;

            this.question1.visible = this.fightInfo.length >= 1;
            this.question2.visible = this.fightInfo.length >= 2;
            this.question3.visible = this.fightInfo.length >= 3;
        }

        eventFight() {
            if (this._skiped) {
                return;
            }
            this._curInfo = this.fightInfo[this._eventIndex];
            if (this._curInfo == null) {
                this.skip();
                return;
            }
            this._catIndex = 0;
            this._curEvent = this._curInfo.event;

            this.dialogues = JSON.parse(this._curEvent.Dialogue);

            this.eventName.text = Strings$1.get(this._curEvent.Name);
            this.pkPc.skin = "mission/" + this._curEvent.Image;
            this.question1.skin = this._eventIndex == 0 ? "mission/question2.png" : "mission/question1.png";
            this.question2.skin = this._eventIndex == 1 ? "mission/question2.png" : "mission/question1.png";
            this.question3.skin = this._eventIndex == 2 ? "mission/question2.png" : "mission/question1.png";
            this._eventIndex++;
            this.catFight();
        }

        catFight() {
            if (this._skiped) {
                return;
            }
            this._curCat = this._curInfo.cats[this._catIndex];
            this.resetUI();
            if (this._curCat == null) {
                this.eventFight();
                return;
            }
            this._catIndex++;
            let petData = this._curCat.petData;
            this.pkCat.skin = petData.getNormalPic();
            this._dialogueIndex = 0;
            this._fightIndex = 0;
            Laya.timer.once(1000, this, this.showDialogue);
        }

        resetUI() {
            this.fighting.text = "";
            this.needValue.visible = false;
            this.catValueName.visible = false;
            this.catValue.visible = false;
            this.anim.visible = false;
        }


        startFight() {
            this._curFight = this._curCat.result[this._fightIndex];
            if (this._curFight == null) {
                this.catFight();
                return;
            }
            this.showPcCatValue();
        }

        showPc() {
            this.pkPc.skin = "mission/" + this.mission.Icon;
        }

        showCats() {
            let teamCats = this.team.length;
            this.waitCat1.visible = teamCats >= 1;
            this.waitCat2.visible = teamCats >= 2;
            this.waitCat3.visible = teamCats >= 3;
            let x = (this.width - 237 * (teamCats - 1)) / 2;
            for (let index = 0; index < this.team.length; index++) {
                const petId = this.team[index];
                let petData = PetSystem.getPetInfo(petId);
                this["waitCat" + (index + 1)].skin = petData.getNormalPic();
                this["waitCat" + (index + 1)].x = x + index * 237;
            }
        }


        showDialogue() {
            if (this._skiped) {
                return;
            }
            let start = Math.max(0, this._dialogueIndex - 3);
            this.fighting.text = "";
            for (let index = start; index <= this._dialogueIndex; index++) {
                const element = this.dialogues[index];
                this.fighting.text += Strings$1.get(element) + "\n";
            }
            this._dialogueIndex++;
            if (this._dialogueIndex >= this.dialogues.length) {
                Laya.timer.once(500, this, this.startFight);
                return;
            }
            Laya.timer.once(1000, this, this.showDialogue);
        }

        showPcCatValue() {
            if (this._skiped) {
                return;
            }
            let condition = this._curFight.condition;
            let s = "";
            let n = "";
            if (condition == "STA") {
                s = Strings$1.get("tid#UI_180");
                n = Strings$1.get("tid#UI_199");
            } else if (condition == "INT") {
                s = Strings$1.get("tid#UI_181");
                n = Strings$1.get("tid#UI_200");
            } else if (condition == "SPE") {
                s = Strings$1.get("tid#UI_182");
                n = Strings$1.get("tid#UI_201");
            }
            this.needValue.text = n + ":" + this.pcValue[condition];
            this.needValue.visible = true;
            this.catValueName.text = s;
            this.catValue.text = 0;
            Laya.timer.once(200, this, function () {
                this.showCatValue();
            });
        }

        showCatValue() {
            if (this._skiped) {
                return;
            }
            let condition = this._curFight.condition;
            this.catValueName.visible = true;
            this.catValue.visible = true;
            this.catValue.text = 0;
            this.catValue.color = "#ffffff";
            let petData = this._curCat.petData;
            Laya.Tween.to(this, { catValueText: petData.attribute[condition] }, 500, null, Laya.Handler.create(this, this.showCurCatResult));
        }

        set catValueText(value) {
            this.catValue.text = Math.round(value);
        }

        get catValueText() {
            return Number(this.catValue.text);
        }

        showCurCatResult() {
            this.catValue.color = "#d6ba27";
            this.anim.x = this.catValue.x + this.catValue.width / 2;
            this.anim.y = this.catValue.y + this.catValue.height / 2;
            this.anim.visible = true;
            this.anim.addCompleteHandler(Laya.Handler.create(this, this.texiaoComplete));
            this.anim.playAnim("texiao", false);
        }

        texiaoComplete() {
            let action = this._curFight.success ? "chenggong" : "taotai";
            this["waitCat" + (this.team.indexOf(this._curCat.petId) + 1)].gray = !this._curFight.success;
            this.anim.x = 643;
            this.anim.y = 1133;
            Laya.timer.frameOnce(40, this, this.resultAnimComplete);
            this.anim.playAnim(action, false);
        }

        resultAnimComplete() {
            this.anim.visible = false;
            this._fightIndex++;
            this.startFight();
        }

        skip() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
            this._skiped = true;
            if (this.fightResult.success) {
                let handler = Laya.Handler.create(this, function (res) {
                    Laya.Dialog.open("ui/MissionCompletePop.scene", false, res.rewards);
                });
                MissionSystem.receiveMissionReward(this._missionIndex, this.team, handler);
            } else {
                Laya.Dialog.open("ui/MissionFailPop.scene", false);
            }
        }
    }

    let level_text = [
        "tid#UI_188",
        "tid#UI_189",
        "tid#UI_190",
        "tid#UI_191",
        "tid#UI_192"
    ];

    class MissionLevelChange extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.btnArr = [this.missionLevelBtn1 ,this.missionLevelBtn2 ,this.missionLevelBtn3 ,this.missionLevelBtn4 ,this.missionLevelBtn5];
            this.levelLimit = MissionSystem.getLevel();//当前解锁的最高等级
            this.nowLevel = MissionSystem.getRefreshLevel();//当前选择的等级
            this.chooseLevel = this.nowLevel;
            this.describe.text = Strings$1.get("tid#UI_216");
            this.sure.on(Laya.Event.CLICK,this,this.onSureCliclk);
            this.bg.on(Laya.Event.CLICK,this,this.close);
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.missionLevelBtn1.on(Laya.Event.CLICK, this, this.levelSelected, [1]);
            this.missionLevelBtn2.on(Laya.Event.CLICK, this, this.levelSelected, [2]);
            this.missionLevelBtn3.on(Laya.Event.CLICK, this, this.levelSelected, [3]);
            this.missionLevelBtn4.on(Laya.Event.CLICK, this, this.levelSelected, [4]);
            this.missionLevelBtn5.on(Laya.Event.CLICK, this, this.levelSelected, [5]);
            this.setLevelUpData();
            this.showBtn();
        }

        setLevelUpData(){
            let exp = MissionSystem.getExp();
            this.levelUpCost = JSON.parse(Const.get("CatMissionLvUpCondition"))[this.levelLimit];
            if(this.levelLimit < level_text.length){
                this.levelDetail.text = Strings$1.get("tid#UI_217",[this.levelLimit,exp,this.levelUpCost]);
                this.levelDetail.visible = true;
            }
            else {
                this.levelDetail.visible = false;
            }
            
        }

        onSureCliclk(){
            if(this.chooseLevel != this.nowLevel){
                if (!MissionSystem.canChangeRefreshLevel()) {
                    EventMgr.getInstance().postEvent("SHOW_TIPS", [Strings$1.get("tid#TipsCommon74", [String(Math.round(MissionSystem.getChangeRefreshLevelTime()))])]);
                    return;
                }
                this.close();
                MissionSystem.changeRefreshLevel(this.chooseLevel);
                EventMgr.getInstance().postEvent("MISSION_REFRESH",[this.chooseLevel]);
                EventMgr.getInstance().postEvent("SHOW_TIPS", [Strings$1.get("tid#TipsCommon83")]);
            }
            else {
                this.close();
            }
        }

        showBtn(){
            for(let i = 0;i < this.btnArr.length; i ++){
                this.btnArr[i].label = Strings$1.get(level_text[i]);
                if(i >= this.levelLimit){
                    this.setUnlock(this.btnArr[i]);
                }
                else if(i == this.nowLevel - 1){
                    this.setIsHook(this.btnArr[i]);
                }
                else {
                    this.setUnHook(this.btnArr[i]);
                }
               
            }
        }

        setUnlock(obj){
            obj.skin = "mission/unlock.png";
            obj.getChildByName("hook").visible = false;
            obj.getChildByName("lock").visible = true;
        }

        setIsHook(obj){
            obj.skin = "mission/isHook.png";
            obj.getChildByName("hook").visible = true;
            obj.getChildByName("lock").visible = false;
        }

        setUnHook(obj){
            obj.skin = "mission/unHook.png";
            obj.getChildByName("hook").visible = false;
            obj.getChildByName("lock").visible = false;
        }

        levelSelected(level) {
            if (level == this.chooseLevel) {
                return;
            }
            if (level <= MissionSystem.getLevel()) {
                this.setUnHook(this.btnArr[this.chooseLevel-1]);
                this.chooseLevel = level;
                this.setIsHook(this.btnArr[this.chooseLevel-1]);
            } else {
                EventMgr.getInstance().postEvent("SHOW_TIPS", [Strings$1.get("tid#TipsCommon73")]);
            }
        }
    }

    let level_text$1 = [
        "tid#UI_188",
        "tid#UI_189",
        "tid#UI_190",
        "tid#UI_191",
        "tid#UI_192"
    ];

    class MissionMain extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.close1.on(Laya.Event.CLICK, this, this.close);
            this.refresh_btn.on(Laya.Event.CLICK, this, this.onRefreshBtnClick);
            this.showLevelChange.on(Laya.Event.CLICK, this, this.onShowLevelChange);

            EventMgr.getInstance().registEvent("MISSION_REFRESH", this, this.showMissions);
            this.showMissions();
            this.startGuide();
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("MISSION_REFRESH", this, this.showMissions);
        }

        onShowLevelChange() {
            Laya.Dialog.close("ui/MissionLevelChange.scene");
            Laya.Dialog.open("ui/MissionLevelChange.scene", false);
        }

        showMissions(level = 0) {
            let mission_refresh_level = level ? level : MissionSystem.getRefreshLevel();
            this.label.text = Strings$1.get(level_text$1[mission_refresh_level - 1]);
            let missionList = MissionSystem.getMissionList();
            this.mission1.visible = false;
            this.mission2.visible = false;
            this.mission3.visible = false;
            let length = missionList.length;
            if (length >= 1) {
                this.mission1.visible = true;
                this.mission1.setItem(missionList[0]);
            }
            if (length >= 2) {
                this.mission2.visible = true;
                this.mission2.setItem(missionList[1]);
            }
            if (length >= 3) {
                this.mission3.visible = true;
                this.mission3.setItem(missionList[2]);
            }
        }

        onRefreshBtnClick() {
            if(MissionSystem.getRefreshTimes() || MissionSystem.getAdRefreshTimes()){
                Laya.Dialog.open("ui/MissionRefreshPop.scene", false);
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon91");
            }
        }

        startGuide() {
            let guide = GuideSystem.getGuide("GuideGroup_17");
            if (guide) {
                guide.registerObj("btn0", this);
                guide.registerClick("btn0", this.mission3);
                guide.registerObj("btn1", this.mission3);
                guide.registerClick("btn1", this.mission3.getChildByName("mission_node").getChildByName("ok_btn"));
                guide.start();
            }
        }

    }

    let star_bgs = [
        "mission/mission_green.png",
        "mission/mission_purple.png",
        "mission/mission_orange.png"
    ];

    class MissionItem extends Laya.Box {

        constructor() {
            super();
        }

        onEnable() {
            this.mission_node = this.getChildByName("mission_node");
            this.level_pic = this.mission_node.getChildByName("level_pic");
            // this.mission_pic = this.mission_node.getChildByName("mission_pic");
            this.star1 = this.mission_node.getChildByName("star1");
            this.star2 = this.mission_node.getChildByName("star2");
            this.star3 = this.mission_node.getChildByName("star3");
            let ok_btn = this.mission_node.getChildByName("ok_btn");
            let ok_text = ok_btn.getChildByName("ok_text");
            this.name = this.mission_node.getChildByName("name");
            this.level = this.mission_node.getChildByName("level");
            this.sta_text = this.mission_node.getChildByName("sta_text");
            this.int_text = this.mission_node.getChildByName("int_text");
            this.spe_text = this.mission_node.getChildByName("spe_text");

            this.countdown_node = this.getChildByName("countdown_node");
            // this.question_pic = this.countdown_node.getChildByName("question_pic");
            let countdown_label = this.countdown_node.getChildByName("countdown_label");
            this.countDown_text = this.countdown_node.getChildByName("countdown_text");

            ok_btn.on(Laya.Event.CLICK, this, this.startMission);
            ok_text.text = Strings$1.get("tid#Btn_040");
            countdown_label.text = Strings$1.get("tid#UI_202");
        }

        setItem(missionInfo) {
            //支持所有的reward
            this.missionInfo = missionInfo;

            let resetTime = missionInfo.resetTime;
            let now = TimeUtils.getTimeNowOnServer();

            Laya.timer.clear(this, this.countDown);

            if (resetTime * 1000 >= now) {
                this.resetTime = resetTime;
                this.mission_node.visible = false;
                this.countdown_node.visible = true;
                Laya.timer.loop(1000, this, this.countDown);
                this.countDown();
            } else {
                this.mission_node.visible = true;
                this.countdown_node.visible = false;
                let mission = MissionSystem.getMissionById(missionInfo.id);
                if (mission == null) {
                    //没用id但cd时间到了同步数据
                    this.countDown();
                    return;
                }
                let level = mission.MissionLv;
                let star = mission.StarLv;
                let name = mission.Name;
                this.name.text = Strings$1.get(name);
                this.level.text = "" + level + Strings$1.get("tid#UI_193");
                this.star1.visible = (star & 2) == 2;
                this.star2.visible = (star & 1) == 1;
                this.star3.visible = (star & 2) == 2;
                if (!this.star2.visible) {
                    this.star1.x = 92;
                    this.star3.x = 146;
                }

                this.level_pic.skin = star_bgs[star - 1];

                let ValueRange = mission.ValueRange;
                for (const key in ValueRange) {
                    if (ValueRange.hasOwnProperty(key)) {
                        const element = ValueRange[key];
                        if (key == "STA") {
                            this.sta_text.text = Strings$1.get("tid#UI_180") + " " + element[0] + "-" + element[1];
                        } else if (key == "INT") {
                            this.int_text.text = Strings$1.get("tid#UI_181") + " " + element[0] + "-" + element[1];
                        } else if (key == "SPE") {
                            this.spe_text.text = Strings$1.get("tid#UI_182") + " " + element[0] + "-" + element[1];
                        }
                    }
                }
                this.sta_text.x = this.int_text.x = this.spe_text.x = (287 - Math.max(this.sta_text.width, this.int_text.width, this.spe_text.width)) / 2;
            }

        }


        checkTime(i) {
            if (i < 10) {
                i = "0" + i;
            }
            return i;
        }

        countDown() {
            let resetTime = this.missionInfo.resetTime * 1000;
            let now = TimeUtils.getTimeNowOnServer();
            if (resetTime < now) {
                Laya.timer.clear(this, this.countDown);
                let dealFunc = function (res) {
                    EventMgr.getInstance().postEvent("MISSION_REFRESH");
                };
                MissionSystem.getMissionInfo(Laya.Handler.create(null, dealFunc));
            } else {
                let time = Math.floor(resetTime - now) / 1000;
                let second = Math.floor(time % 60);
                let minute = Math.floor(time / 60) % 60;
                let hour = Math.floor(time / 3600);
                this.countDown_text.text = "" + this.checkTime(hour) + ":" + this.checkTime(minute) + ":" + this.checkTime(second);
            }
        }

        startMission() {
            Laya.Dialog.open("ui/MissionTeam.scene", true, this.missionInfo.index);
        }
    }

    class MissionRefreshPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.clickSign = 1;
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.video.on(Laya.Event.CLICK,this,this.onAdClick);
            this.title.text = Strings$1.get("tid#UI_194");
            this.desc.text = Strings$1.get("tid#UI_195");
            let timesLimit = Number(Const.get("CatMissionRefreshTimes"));
            let CatMissionRefreshAD = JSON.parse(Const.get("CatMissionRefreshAD"));
            let times = MissionSystem.getRefreshTimes();
            let adRefreshTimes = MissionSystem.getAdRefreshTimes();//本日剩余看广告刷新次数
            this.adsToLook = MissionSystem.getAdsToLook();//还需看多少次广告刷新
            this.times.text = Strings$1.get("tid#UI_196", [times + "/" + timesLimit]);
            this.needTimesLabel.text = Strings$1.get("tid#UI_222", [this.adsToLook]);
            this.ok_text.text = Strings$1.get("tid#Btn_009");
            if (times <= 0) {
                this.dailyBox.visible = false;
                this.adBox.visible = true;
                if(SdkHelper.hasVideo()) {
                    this.share.visible = false;
                    this.video.visible = true;
                    this.allTimesLabel.text = Strings$1.get("tid#UI_220", [CatMissionRefreshAD[CatMissionRefreshAD.length - adRefreshTimes]]);
                }else {
                    this.share.visible = true;
                    this.video.visible = false;
                    this.allTimesLabel.text = Strings$1.get("tid#UI_221", [CatMissionRefreshAD[CatMissionRefreshAD.length - adRefreshTimes]]);
                }
            }
            else {
                this.dailyBox.visible = true;
                this.adBox.visible = false;
            }
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        onOkBtnClick() {
            let dealFunc = function (res) {
                EventMgr.getInstance().postEvent("MISSION_REFRESH");
            };
            MissionSystem.refreshMission(Laya.Handler.create(null, dealFunc));
            this.close();
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.doneV);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        changeClickSign(){
            this.clickSign = 0;
            Laya.timer.once(500,this,function(){
                this.clickSign = 1;
            });
        }

        
        onShareClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.shareAppMessage(Constants.ShareId.MissionCatCD);
            } 
        }

        onAdClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.showRewardVideoAd(Constants.ShareId.MissionCatCD);
            } 
        }

        doneV(shareId){
            if(shareId == Constants.ShareId.MissionCatCD) {
                this.done();
                if(this.adsToLook > 1){
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon88");
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon87");
                }
            }
        }

        doneS(shareId){
            if(shareId == Constants.ShareId.MissionCatCD) {
                this.done();
                if(this.adsToLook > 1){
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon89");
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon87");
                }
            }
        }

        done(){
            let dealFunc = function (res) {
                EventMgr.getInstance().postEvent("MISSION_REFRESH");
            };
            MissionSystem.adRefreshMission(Laya.Handler.create(null, dealFunc));
            this.close();
        }
    }

    let level_text$2 = [
        "tid#UI_188",
        "tid#UI_189",
        "tid#UI_190",
        "tid#UI_191",
        "tid#UI_192"
    ];

    class MissionTeam extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this._missionIndex = this._param;
            MissionSystem.setCurrentMissionIndex(this._missionIndex);
            this.close1.on(Laya.Event.CLICK, this, this.closeThis);
            this.ok_btn.on(Laya.Event.CLICK, this, this.enter);
            this.auto.on(Laya.Event.CLICK, this, this.autoSetPets);
            this.toAdopt.on(Laya.Event.CLICK, this, this.onToAdoptClick);

            EventMgr.getInstance().registEvent("MISSION_CAT_SELECTED_CHANGE", this, this.changeSelectCatId);
            EventMgr.getInstance().registEvent("MISSION_CAT_BOX_SELECTED_CHANGE", this, this.changeSelectBox);
            EventMgr.getInstance().registEvent("MISSION_CAT_UNSELECTED", this, this.unSelectCatId);
            EventMgr.getInstance().registEvent("MISSION_CAT_LIST_UPDATE", this, this.updateCatView);

            this.list = this.catList;

            this.ok_text.text = Strings$1.get("tid#Btn_041");
            this.auto_text.text = Strings$1.get("tid#Btn_048");
            this.toAdopt_text.text = Strings$1.get("tid#Btn_022");
            this.noLabel.text = Strings$1.get("tid#UI_219");
            // this.reward_label.text = Strings.get("")//奖励：

            // this.level_node.visible = false;
            // this.level_btn.visible = true;
            this.showMission();


            this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);

            this.box1.setIndex(0);
            this.box2.setIndex(1);
            this.box3.setIndex(2);
            this._index = 0;
            this._boxIndex = 0;
            this.boxs = [this.box1, this.box2, this.box3];
            this.initCatBox();

            this.showCatList();
            this.startGuide();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("MISSION_CAT_SELECTED_CHANGE", this, this.changeSelectCatId);
            EventMgr.getInstance().removeEvent("MISSION_CAT_BOX_SELECTED_CHANGE", this, this.changeSelectBox);
            EventMgr.getInstance().removeEvent("MISSION_CAT_LIST_UPDATE", this, this.updateCatView);
        }

        onToAdoptClick(){
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
            SyncSystem.updateSceneInfo();
            PetSystem.getPetsInfo(Laya.Handler.create(this, function () {
                Laya.Dialog.close("ui/CollectPop.scene");
                Laya.Dialog.open("ui/CollectPop.scene");
            }));
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        showMission() {
            let missionInfo = MissionSystem.getMissionByIndex(this._missionIndex);
            let mission = MissionSystem.getMissionById(missionInfo.id);
            let level = mission.MissionLv;
            let star = mission.StarLv;
            let name = mission.Name;

            this.mission_name.visible = false;
            this.name.text = Strings$1.get(name);
            this.desc.text = Strings$1.get(mission.Desc);
            this.star1.visible = (star & 2) == 2;
            this.star2.visible = (star & 1) == 1;
            this.star3.visible = (star & 2) == 2;
            if (!this.star2.visible) {
                this.star1.x = 570;
                this.star3.x = 609;
            }

            this.icon.skin = "mission/" + mission.Icon;

            let ValueRange = mission.ValueRange;
            for (const key in ValueRange) {
                if (ValueRange.hasOwnProperty(key)) {
                    const element = ValueRange[key];
                    if (key == "STA") {
                        this.attr1.text = Strings$1.get("tid#UI_180") + " " + element[0] + "-" + element[1];
                    } else if (key == "INT") {
                        this.attr2.text = Strings$1.get("tid#UI_181") + " " + element[0] + "-" + element[1];
                    } else if (key == "SPE") {
                        this.attr3.text = Strings$1.get("tid#UI_182") + " " + element[0] + "-" + element[1];
                    }
                }
            }

            let rewards = mission.Reward;
            this.reward1.setItem(rewards[0]);
            this.reward2.visible = false;
            this.reward3.visible = false;
            if (rewards.length == 2) {
                this.reward2.setItem(rewards[1]);
                this.reward2.visible = true;
            } else if (rewards.length >= 3) {
                this.reward2.setItem(rewards[1]);
                this.reward2.visible = true;
                this.reward3.setItem(rewards[2]);
                this.reward3.visible = true;
            }
        }

        calcCat() {
            let bp = 0;
            for (let i = 0; i < 3; ++i) {
                let petId = this.team[i];
                if (petId != null) {
                    bp = bp + this.getPetBp(petId);
                }
            }
            this.bp.text = Strings$1.get("tid#UI_197") + "" + Math.floor(bp);
        }

        showCatList() {
            let index = 0;
            this.calcCat();
            let pets = PetSystem.PetIdsAdopted;
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            function sortByRankOrder(a, b) {
                return config.get(a).Rank - config.get(b).Rank;
            }
            pets.sort(sortByRankOrder);
            this.array = new Array();
            for (const petId of pets) {
                this.array.push(petId);
            }
            // this.array = ["Cat1","Cat2","Cat3","Cat4"];
            this.noBox.visible = false;
            let length = this.array.length;
            if (length > 1 && length < 9) {
                let arr = [null, null, null, null, null, null, null, null, null];
                for (let index = 0; index < this.array.length; index++) {
                    if (index < 5) {
                        arr[index * 2] = this.array[index];
                    } else {
                        arr[(index % 5) * 2 + 1] = this.array[index];
                    }
                }
                this.array = arr;
            }
            else if(this.array.length == 0){
                this.noBox.visible = true;
            }
            this.list = this.catList;

            this.catList.hScrollBarSkin = "";
            this.catList.off(Laya.Event.MOUSE_UP, this, this.changeListItems1);
            this.catList.on(Laya.Event.MOUSE_UP, this, this.changeListItems1);
            this.catList.off(Laya.Event.MOUSE_OUT, this, this.changeListItems2);
            this.catList.on(Laya.Event.MOUSE_OUT, this, this.changeListItems2);
            this.catList.scrollBar.rollRatio = 0;

            this.startIndex = 0;
            this.endIndex = Math.ceil(this.array.length / 2);
            this.endIndex = this.array.length;

            this.changeListItems();
        }

        adjustList() {
            let w = this.list.cells[0].width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w + s) + 0.5 | 0;
            Laya.Tween.to(this.list.scrollBar, { value: this._index * (w + s) }, 100);
        }

        changeListItems2() {
            this.list.scrollBar.stopScroll();
            this.changeListItems1();
        }

        changeListItems1() {
            this.loadList();
            this.loadArrow();
        }

        changeListItems() {
            this.list.array = this.loadList();
            //this.list.scrollBar.stopScroll();
            //修改left_arrow 和 right_arrow 的展示
            this.loadArrow();
        }

        loadList() {
            // let showList = this.array.slice(this.startIndex, this.endIndex * 2);
            let showList = this.array.slice(this.startIndex, this.endIndex);
            this.adjustList();
            return showList;
        }

        loadArrow() {
            if (this._index == 0) {
                this.left_arrow.visible = false;
            } else {
                this.left_arrow.visible = true;
            }
            if (this._index * 2 >= this.endIndex - 10 || this.endIndex < 11) {
                this.right_arrow.visible = false;
            } else {
                this.right_arrow.visible = true;
            }
        }

        _onLeftArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - 5 > 0 ? this._index - 5 : 0;
            this.list.scrollTo(this._index * 2);
            this.changeListItems();
        }

        _onRightArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + 5 < this.endIndex ? this._index + 5 : this.endIndex - 4;
            this.list.scrollTo(this._index * 2);
            this.changeListItems();
        }

        updateCatView(){
            this.catList.refresh();
        }

        changeSelectCatId(catId) {
            this.boxs[this._boxIndex].setItem(catId);
            MissionSystem.setPet(catId, this._boxIndex);
            this.calcCat();
            this.catList.refresh();
            this.selectNextEmptyBox();
        }

        unSelectCatId(catId, index) {
            MissionSystem.removePet(index);
            this.boxs[index].setEmpty();
            this.calcCat();
            this.catList.refresh();
            this.selectNextEmptyBox();
        }

        changeSelectBox(index) {
            this.boxs[this._boxIndex].setSelected(index);
            this._boxIndex = index;
            this.boxs[this._boxIndex].setSelected(index);
        }

        selectNextEmptyBox() {
            for (let index = 0; index < 3; index++) {
                const petId = this.team[index];
                if (petId == null) {
                    this.changeSelectBox(index);
                    return;
                }
            }
        }

        initCatBox() {
            this.team = MissionSystem.Team;
            this.box1.setSelected(this._boxIndex);
            this.box2.setSelected(this._boxIndex);
            this.box3.setSelected(this._boxIndex);
            for (let index = 0; index < this.boxs.length; index++) {
                const element = this.boxs[index];
                if (this.team[index]) {
                    element.setItem(this.team[index]);
                } else {
                    element.setEmpty();
                }
            }
        }


        enter() {
            let count = 0;
            for (let i = 0; i < 3; ++i) {
                let petId = this.team[i];
                if (petId != null) {
                    count = count + 1;
                }
            }
            if (count > 0) {
                this.close();
                Laya.Dialog.open("ui/MissionGame.scene", false, this._missionIndex);
            } else {
                EventMgr.getInstance().postEvent("SHOW_TIPS", [Strings$1.get("tid#TipsCommon76")]);
            }
        }

        autoSetPets() {
            let team = MissionSystem.Team;
            let teamCats = 0;
            team.forEach(element => {
                teamCats += element ? 1 : 0;
            });
            // if (teamCats >= 3) {
            //     return;
            // }
            let pets = PetSystem.PetIdsAdopted.concat();
            if (pets.length <= teamCats) {
                return;
            }
            let _this = this;
            function sortByValue(a, b) {
                return _this.getPetBp(b) - _this.getPetBp(a);
            }
            pets.sort(sortByValue);
            let arr = [];
            pets.forEach(petId => {
                if (arr.length >= 3) {
                    return;
                }
                let reset_time = MissionSystem.getPetRestTime(petId) * 1000;
                let now = TimeUtils.getTimeNowOnServer();
                if (reset_time != null && reset_time >= now) {
                    return;
                }
                arr.push(petId);
            });
            // if (arr.length > teamCats) {
            for (let index = 0; index < 3; index++) {
                const pid = arr[index];
                if (pid != null) {
                    MissionSystem.setPet(pid, index);
                } else {
                    MissionSystem.removePet(index);
                }
            }
            if (arr.length == 0) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon84");
            }
            // }
            this.initCatBox();
            this.selectNextEmptyBox();
        }


        getPetBp(petId) {
            let petData = PetSystem.getPetInfo(petId);
            let attr = petData.attribute;
            return attr["STA"] + attr["INT"] + attr["SPE"];
        }

        closeThis() {
            MissionSystem.getMissionInfo(Laya.Handler.create(this, function () {
                Laya.Dialog.open("ui/MissionMain.scene");
            }));
        }

        startGuide() {
            let guide = GuideSystem.getGuide("GuideGroup_17");
            if (guide) {
                guide.registerObj("btn2", this);
                guide.registerClick("btn2", this.box1, null, this.boxs);
                guide.registerObj("btn3", this);
                guide.registerClick("btn3", this.list);
                guide.registerObj("btn4", this);
                guide.registerClick("btn4", this.ok_btn);
            }
        }
    }

    class MissionCatBox extends Laya.Sprite {

        constructor() {
            super();
        }

        onEnable() {
            this.empty_node = this.getChildByName("empty_node");
            this.selected1 = this.empty_node.getChildByName("selected");

            this.cat_node = this.getChildByName("cat_node");
            this.icon = this.cat_node.getChildByName("icon");
            this.selected2 = this.cat_node.getChildByName("selected");
            this.potential = this.cat_node.getChildByName("potential");
            // this.star1 = this.cat_node.getChildByName("star1");
            // this.star2 = this.cat_node.getChildByName("star2");
            // this.star3 = this.cat_node.getChildByName("star3");
            // this.star4 = this.cat_node.getChildByName("star4");
            // this.star5 = this.cat_node.getChildByName("star5");
            this.attr1 = this.cat_node.getChildByName("attr1");
            this.attr2 = this.cat_node.getChildByName("attr2");
            this.attr3 = this.cat_node.getChildByName("attr3");
            this.offload_btn = this.cat_node.getChildByName("offload_btn");

            this.offload_btn.on(Laya.Event.CLICK, this, this.onOffload);
            this.on(Laya.Event.CLICK, this, this.onSelected);
        }

        setIndex(index) {
            this._index = index;
        }

        setEmpty() {
            this.cat_node.visible = false;
            this.empty_node.visible = true;
        }

        setSelected(index) {
            this.selected1.visible = this._index == index;
            this.selected2.visible = this._index == index;
            this.offload_btn.visible = this._index == index;
        }

        setItem(petId) {
            this.petId = petId;
            // this.offAll();
            this.cat_node.visible = true;
            this.empty_node.visible = false;
            this.showCat(petId);
        }

        showCat(petId) {
            let petData = PetSystem.getPetInfo(petId);
            this.setIcon(petData.getNormalPic());
            // let star = petData.star;
            // this.star1.skin = star>=1 ? "common/xingxing.png": "common/xingxing1.png";
            // this.star2.skin = star>=2 ? "common/xingxing.png": "common/xingxing1.png";
            // this.star3.skin = star>=3 ? "common/xingxing.png": "common/xingxing1.png";
            // this.star4.skin = star>=4 ? "common/xingxing.png": "common/xingxing1.png";
            // this.star5.skin = star>=5 ? "common/xingxing.png": "common/xingxing1.png";

            this.potential.skin = Constants.PotentialPic[petData.Potential];
            this.attr1.text = petData.level + Strings$1.get("tid#UI_193");
            this.attr2.text = petData.level + Strings$1.get("tid#UI_193");
            this.attr3.text = petData.level + Strings$1.get("tid#UI_193");
            let attribute = petData.attribute;
            this.attr1.text = Strings$1.get("tid#UI_180") + " " + Math.floor(attribute["STA"]);
            this.attr2.text = Strings$1.get("tid#UI_181") + " " + Math.floor(attribute["INT"]);
            this.attr3.text = Strings$1.get("tid#UI_182") + " " + Math.floor(attribute["SPE"]);
        }

        checkTime(i) {
            if (i < 10) {
                i = "0" + i;
            }
            return i;
        }


        setIcon(skin) {
            this.icon.skin = skin;
        }

        onOffload() {
            // this.setEmpty();
            EventMgr.getInstance().postEvent("MISSION_CAT_UNSELECTED", [this.petId, this._index]);
        }

        onSelected() {
            EventMgr.getInstance().postEvent("MISSION_CAT_BOX_SELECTED_CHANGE", [this._index]);
        }
    }

    class MissionCatItem extends Laya.Sprite {

        constructor() {
            super();
        }

        onEnable() {
            this.icon = this.getChildByName("icon");
            this.hook = this.getChildByName("hook");
            this.potential = this.getChildByName("potential");
            this.star1 = this.getChildByName("star1");
            this.star2 = this.getChildByName("star2");
            this.star3 = this.getChildByName("star3");
            this.star4 = this.getChildByName("star4");
            this.star5 = this.getChildByName("star5");
            this.grayBox = this.getChildByName("grayBox");
            this.rest = this.getChildByName("rest");
            this.rest_time = this.getChildByName("rest_time");
            this.icon_box = this.getChildByName("icon_box");
            this.rest.text = Strings$1.get("tid#UI_198");
        }

        openAD(){
            Laya.Dialog.close("ui/MissionCatCD.scene");
            Laya.Dialog.open("ui/MissionCatCD.scene",false,[this.petId]);
        }

        setItem(petId) {
            if (petId == null) {
                this.visible = false;
                return;
            }
            this.visible = true;
            this.petId = petId;
            this.icon_box.offAll();
            this.hook.visible = false;
            this.showCat(petId);
        }

        showCat(petId) {
            let petData = PetSystem.getPetInfo(petId);
            this.setIcon(petData.getHeadIcon());
            let star = petData.star;

            this.star1.skin = star >= 1 ? "common/xingxing.png" : "common/xingxing1.png";
            this.star2.skin = star >= 2 ? "common/xingxing.png" : "common/xingxing1.png";
            this.star3.skin = star >= 3 ? "common/xingxing.png" : "common/xingxing1.png";
            this.star4.skin = star >= 4 ? "common/xingxing.png" : "common/xingxing1.png";
            this.star5.skin = star >= 5 ? "common/xingxing.png" : "common/xingxing1.png";

            this.potential.skin = Constants.PotentialPic[petData.Potential];
            Laya.timer.clear(this, this.countDown);
            let reset_time = MissionSystem.getPetRestTime(petId) * 1000;
            let now = TimeUtils.getTimeNowOnServer();
            if (reset_time != null && reset_time >= now) {
                this.grayBox.visible = true;
                this.rest.visible = true;
                this.rest_time.visible = true;
                Laya.timer.loop(1000, this, this.countDown);
                this.countDown();
            } else {
                this.grayBox.visible = false;
                this.rest.visible = false;
                this.rest_time.visible = false;
                if (MissionSystem.hasPet(petId)) {
                    this.hook.visible = true;
                }
            }
            this.icon_box.on(Laya.Event.CLICK, this, this.onSelected, [petId]);
        }

        checkTime(i) {
            if (i < 10) {
                i = "0" + i;
            }
            return i;
        }

        countDown() {
            let reset_time = MissionSystem.getPetRestTime(this.petId) * 1000;
            let now = TimeUtils.getTimeNowOnServer();
            if (reset_time < now) {
                Laya.timer.clear(this, this.countDown);
                this.setItem(this.petId);
            } else {
                let time = Math.floor(reset_time - now) / 1000;
                let second = Math.floor(time % 60);
                let minute = Math.floor(time / 60) % 60;
                let hour = Math.floor(time / 3600);
                this.rest_time.text = "" + this.checkTime(hour) + ":" + this.checkTime(minute) + ":" + this.checkTime(second);
            }
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 169 / this.icon.width;
            let scaley = 169 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        onSelected(petId) {
            let reset_time = MissionSystem.getPetRestTime(petId) * 1000;
            let now = TimeUtils.getTimeNowOnServer();
            if (reset_time != null && reset_time >= now) {
                this.openAD();
                // EventMgr.getInstance().postEvent("SHOW_TIPS", [Strings.get("tid#TipsCommon75")]);
                return;
            }
            if (MissionSystem.hasPet(petId)) {
                let index = MissionSystem.getCatIndex(petId);
                if (index != null) {
                    EventMgr.getInstance().postEvent("MISSION_CAT_UNSELECTED", [petId, index]);
                }
                return;
            }
            EventMgr.getInstance().postEvent("MISSION_CAT_SELECTED_CHANGE", [petId]);
        }
    }

    class MoraGamePop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            // bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.quit.on(Laya.Event.CLICK, this, this.onQuitClick);
            this.animBox = this.getChildByName("animBox");
            this.machineBox = this.animBox.getChildByName("machineBox");
            this.resultBox = this.animBox.getChildByName("resultBox");
            this.lightBox = this.animBox.getChildByName("lightBox");
            this.shitou = this.animBox.getChildByName("shitou");
            this.jiandao = this.animBox.getChildByName("jiandao");
            this.bu = this.animBox.getChildByName("bu");
            this.machine = new Laya.Skeleton();
            this.machine.load("anim2/caiquanji.sk", Laya.Handler.create(this, function () {
                this.machine.play("daiji", true);
                this.machineBox.addChild(this.machine);
            }));
            this.result = new Laya.Skeleton();
            this.result.load("anim2/caiquanji.sk", Laya.Handler.create(this, function () {
                this.result.play("ping", false);
                this.resultBox.addChild(this.result);
            }));
            this.light = new Laya.Skeleton();
            this.light.load("anim2/caiquanji.sk", Laya.Handler.create(this, function () {
                this._lightLoadComplete = true;
                this.light.play("deng1", true);
                this.lightBox.addChild(this.light);
            }));
            this.rewardBox = this.getChildByName("rewardBox");
            this.reward = this.rewardBox.getChildByName("reward");
            this.fishImage = this.rewardBox.getChildByName("fishImage");
            this.btnBox = this.getChildByName("btnBox");
            this.btn1 = this.btnBox.getChildByName("btn1");
            this.btn1.on(Laya.Event.CLICK, this, this.onBtn1Click);
            this.btn2 = this.btnBox.getChildByName("btn2");
            this.btn2.on(Laya.Event.CLICK, this, this.onBtn2Click);
            this.btn3 = this.btnBox.getChildByName("btn3");
            this.btn3.on(Laya.Event.CLICK, this, this.onBtn3Click);
            this.showBox = this.getChildByName("showBox");
            this.welcome = this.showBox.getChildByName("welcome");
            this.adBox = this.showBox.getChildByName("adBox");
            this.adLabel = this.adBox.getChildByName("adLabel");
            this.drawAdLabel = this.adBox.getChildByName("drawAdLabel");
            this.shareBtn = this.adBox.getChildByName("share");
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareBtnClick);
            this.videoBtn = this.adBox.getChildByName("video");
            this.videoBtn.on(Laya.Event.CLICK, this, this.onVideoBtnClick);
            // this.backBtn = this.adBox.getChildByName("backBtn");
            // this.backBtn.on(Laya.Event.CLICK, this, this.onBackBtnClick);

            this.overBox = this.showBox.getChildByName("overBox");
            this.overLabel = this.overBox.getChildByName("overLabel");
            this.drawLabel = this.overBox.getChildByName("drawLabel");
            this.overBtn = this.overBox.getChildByName("overBtn");
            this.overBtn.on(Laya.Event.CLICK, this, this.onOverBtnClick);
            this.overBtn.label=Strings$1.get('tid#Btn_028');

            this.winBox = this.showBox.getChildByName("winBox");
            this.winLabel = this.winBox.getChildByName("winLabel");
            // this.getBtn = this.winBox.getChildByName("getBtn");
            // this.getBtn.on(Laya.Event.CLICK, this, this.onGetBtnClick);
            this.shareDoubleBtn = this.winBox.getChildByName("shareBtn");
            this.shareDoubleBtn.on(Laya.Event.CLICK, this, this.onShareDoubleBtnClick);
            this.videoDoubleBtn = this.winBox.getChildByName("videoBtn");
            this.videoDoubleBtn.on(Laya.Event.CLICK, this, this.onvideoDoubleBtnClick);

            this.overLabel.text = Strings$1.get("tid#UI_017");
            this.drawLabel.text = Strings$1.get("tid#UI_018");
            this.welcome.text = Strings$1.get("tid#UI_013");
            this.winLabel.text = Strings$1.get("tid#UI_014");
            this.rateObj = JSON.parse(Const.get("SpecialGameMoraWin"));
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.quit.visible = true;
            this.playTimes = 0;
            this.winRate = this.rateObj.First;
            this._playerPlayData = null;
            this._catPlayData = null;
            //0:平局,1:负,2:胜
            this._resultType = null;
            this.minigameData = this._param;
            this.result.visible = false;
            this.shitou.visible = false;
            this.jiandao.visible = false;
            this.bu.visible = false;
            this.welcome.visible = true;
            this.adBox.visible = false;
            this.overBox.visible = false;
            this.winBox.visible = false;
            this.btnBox.mouseEnabled = true;
            this.lightBox.visible = true;

            MoraGamePop.shitou.next = MoraGamePop.jiandao;
            MoraGamePop.jiandao.next = MoraGamePop.bu;
            MoraGamePop.bu.next = MoraGamePop.shitou;

            MoraGamePop.shitou.previous = MoraGamePop.bu;
            MoraGamePop.jiandao.previous = MoraGamePop.shitou;
            MoraGamePop.bu.previous = MoraGamePop.jiandao;
            let reward_m = this.minigameData.getReward();
            if (reward_m.WinReward[0].id == Constants.RewardType.DIAMOND) {
                this.fishImage.skin = "topui/zuanshi.png";
            } else if (reward_m.WinReward[0].id == Constants.RewardType.STAR) {
                this.fishImage.skin = "topui/xingxing.png";
            }
            this.reward.text = Strings$1.dealFishNumberText(reward_m.WinReward[0].amount + (reward_m.WinRewardRate ? reward_m.WinRewardRate : 0) * PetSystem.getMaxValueBase());
            this.fishImage.x = (this.rewardBox.width - this.fishImage.width * this.fishImage.scaleX - this.reward.width - 10) / 2;
            this.reward.x = this.fishImage.x + this.fishImage.width * this.fishImage.scaleX + 10;
            if (this._lightLoadComplete) {
                this.light.play("deng1", true);
            }
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            if (SdkHelper.hasVideo()) {
                this.shareBtn.visible = false;
                this.videoBtn.visible = true;
                this.adLabel.text = Strings$1.get("tid#UI_015");
                this.drawAdLabel.text = Strings$1.get("tid#UI_015");
                this.shareDoubleBtn.visible = false;
                this.videoDoubleBtn.visible = true;
            } else {
                this.shareBtn.visible = true;
                this.videoBtn.visible = false;
                this.adLabel.text = Strings$1.get("tid#UI_016");
                this.drawAdLabel.text = Strings$1.get("tid#UI_016");
                this.shareDoubleBtn.visible = true;
                this.videoDoubleBtn.visible = false;
            }
        }

        talkShow() {
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
        }

        onQuitClick(){
            if(this.playTimes > 0){
                if (this._resultType == Constants.MiniGameResult.Win){
                    this.onGetBtnClick();
                }
                else {
                    this.onBackBtnClick();
                }
            }
            else {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 3);
                this.close();
            }
        }

        //石头
        onBtn1Click() {
            this._playerPlayData = MoraGamePop.shitou;
            this.randomCatPlay();
            this.btnBox.mouseEnabled = false;
            this.shitou.visible = true;
            this.jiandao.visible = false;
            this.bu.visible = false;
            Laya.Tween.clearAll(this.shitou);
            Laya.Tween.from(this.shitou, { scaleX: 0, scaleY: 0 }, 200);
            this.startPlay();
        }
        //剪刀
        onBtn2Click() {
            this._playerPlayData = MoraGamePop.jiandao;
            this.randomCatPlay();
            this.btnBox.mouseEnabled = false;
            this.shitou.visible = false;
            this.jiandao.visible = true;
            Laya.Tween.clearAll(this.jiandao);
            Laya.Tween.from(this.jiandao, { scaleX: 0, scaleY: 0 }, 200);
            this.bu.visible = false;
            this.startPlay();
        }
        //布
        onBtn3Click() {
            this._playerPlayData = MoraGamePop.bu;
            this.randomCatPlay();
            this.btnBox.mouseEnabled = false;
            this.shitou.visible = false;
            this.jiandao.visible = false;
            this.bu.visible = true;
            Laya.Tween.clearAll(this.bu);
            Laya.Tween.from(this.bu, { scaleX: 0, scaleY: 0 }, 200);
            this.startPlay();
        }

        randomCatPlay() {
            this.playTimes++;
            let random = Utils.randomExt(100);
            if (random <= this.winRate) {
                this._resultType = Constants.MiniGameResult.Win;
                this._catPlayData = this._playerPlayData.next;
            } else if (random > this.winRate + (100 - this.winRate) / 2) {
                this._resultType = Constants.MiniGameResult.Draw;
                this._catPlayData = this._playerPlayData;
            } else {
                this._resultType = Constants.MiniGameResult.Fail;
                this._catPlayData = this._playerPlayData.previous;
            }
        }

        startPlay() {
            this.quit.visible = false;
            this.machine.play(this._catPlayData.value, false);
            this.machine.on(Laya.Event.STOPPED, this, this.gameComplete);
            this.light.play("deng2", false);
        }

        gameComplete() {
            this.welcome.visible = false;
            this.quit.visible = true;
            if (this._resultType == Constants.MiniGameResult.Draw) {
                this.lightBox.visible = false;
                this.result.visible = true;
                this.result.play("ping", false);
                this.overBox.visible = true;
                this.overLabel.visible = false;
                this.drawLabel.visible = true;
                // if (this.playTimes < 2) {
                //     this.adBox.visible = true;
                //     this.overBox.visible = false;
                //     this.winBox.visible = false;
                //     this.adLabel.visible = false;
                //     this.drawAdLabel.visible = true;
                // } else {
                //     this.overBox.visible = true;
                //     this.overLabel.visible = false;
                //     this.drawLabel.visible = true;
                // }
            } else if (this._resultType == Constants.MiniGameResult.Win) {
                this.light.play("deng3", true);
                this.lightBox.visible = true;
                this.adBox.visible = false;
                this.overBox.visible = false;
                this.winBox.visible = true;
                this.result.visible = true;
                this.result.play("sheng", false);
            } else {
                this.lightBox.visible = false;
                this.result.visible = true;
                this.result.play("fu", false);
                this.overBox.visible = true;
                this.overLabel.visible = true;
                this.drawLabel.visible = false;
                // if (this.playTimes < 2) {
                //     this.adBox.visible = true;
                //     this.overBox.visible = false;
                //     this.winBox.visible = false;
                //     this.adLabel.visible = true;
                //     this.drawAdLabel.visible = false;
                // } else {
                //     this.overBox.visible = true;
                //     this.overLabel.visible = true;
                //     this.drawLabel.visible = false;
                // }

            }
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.MoraGame) {
                this.winRate = this.rateObj.Repeat;
                this.resetStart();
            } else if (shareId == Constants.ShareId.MoraGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_mora_reward_success"]);
                this.close();
                Laya.Dialog.close("ui/MiniGameTalkPop.scene");
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.MoraGame) {
                this.winRate = this.rateObj.Repeat;
                this.resetStart();
            } else if (shareId == Constants.ShareId.MoraGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_mora_reward_success"]);
                this.close();
                Laya.Dialog.close("ui/MiniGameTalkPop.scene");
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
            }
        }

        onBackBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 2, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onGetBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_mora_reward_success"]);
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onOverBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 0, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            this.close();
            Laya.Dialog.close("ui/MiniGameTalkPop.scene");
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData]);
        }

        onShareBtnClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.MoraGame);
        }

        onVideoBtnClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.MoraGame);
        }

        showFly(fetchType, res) {
            let point = new Laya.Point(this.fishImage.x, this.fishImage.y);
            point = this.fishImage.parent.localToGlobal(point);
            let rewards = res.rewards.rewards;
            for (let i in rewards) {
                if (rewards[i].amount <= 0) {
                    continue;
                }
                if (rewards[i].id == Constants.RewardType.FISH) {
                    if (fetchType == Constants.FetchType.AD) {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE3, point.x, point.y);
                    } else {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE5, point.x, point.y);
                    }
                } else {
                    ItemFlyMgr.getInstance().fly(rewards[i].id, rewards[i].amount, Constants.FlyType.ZCM, point.x, point.y);
                }
            }
        }

        resetStart() {
            this._playerPlayData = null;
            this._catPlayData = null;
            this._resultType = null;
            this.result.visible = false;
            this.shitou.visible = false;
            this.jiandao.visible = false;
            this.bu.visible = false;
            this.welcome.visible = true;
            this.adBox.visible = false;
            this.overBox.visible = false;
            this.winBox.visible = false;
            this.machine.play("daiji", true);
            this.btnBox.mouseEnabled = true;
            this.lightBox.visible = true;
        }

        onCloseClick() {
            if (this.playTimes == 0) {
                this.close();
            }
        }
        onDestroy() {
            Laya.Tween.clearAll(this);
        }

        onShareDoubleBtnClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.MoraGameDouble);
        }

        onvideoDoubleBtnClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.MoraGameDouble);
        }

    }
    MoraGamePop.shitou = { value: "shitou", next: null, previous: null };
    MoraGamePop.jiandao = { value: "jiandao", next: null, previous: null };
    MoraGamePop.bu = { value: "bu", next: null, previous: null };

    class MultiLoginDialog extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            this.bgPanel = this.getChildByName("bg_panel");
            this.bgPanel.alpha = 0.2;
            this.bgPanel.on(Laya.Event.CLICK, this, function() {
                this.close();
                EventMgr.getInstance().postEvent("NETWORD_DISCONNECT");
            });
        }
    }

    class ViewAlign extends Laya.Script {

        constructor() {
            super();
        }

        onStart() {
            this.owner.width = Laya.Browser.width * 2436 / Laya.Browser.height;
        }
    }

    class NauticalTravelingPop extends Laya.Dialog {

        constructor(){
            super();
        }

        onEnable() {
            this.close1.on(Laya.Event.CLICK,this,this.close);
            this.title.text = Strings$1.get("tid#UI_205");
            this.detail.text = Strings$1.get("tid#UI_206");
            this.tips.text = Strings$1.get("tid#UI_207");
            this.goFishing.label = Strings$1.get("tid#Btn_045");
            this.goLuckydraw.label = Strings$1.get("tid#Btn_046");
            this.goFishing.on(Laya.Event.CLICK,this,this.onGoFishingClick);
            this.goLuckydraw.on(Laya.Event.CLICK,this,this.onGoLuckydrawClick);
        }

        onGoFishingClick(){
            let afterHandler = Laya.Handler.create(this, function (res) {
                SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
                this.close();
                EventMgr.getInstance().postEvent("NAUTICAL_STARTPOP_CLOSE");
            });
            FishingSystem.updateServerBaseInfo(afterHandler);
        }

        onGoLuckydrawClick(){
            SceneMgr.getInstance().switchScene(Constants.CoffeeSceneId);
            this.close();
            EventMgr.getInstance().postEvent("NAUTICAL_STARTPOP_CLOSE");
            Laya.Dialog.close("ui/LuckydrawPop.scene");
            Laya.Dialog.open("ui/LuckydrawPop.scene");
        }

    }

    class IntimacyUpDataMgr{
        constructor(){

        }

        static getInstance() {
            return IntimacyUpDataMgr.instance ? IntimacyUpDataMgr.instance : IntimacyUpDataMgr.instance = new IntimacyUpDataMgr();
        }

        setChangeFlag(num){
            if(num >= 1){
                this.changeFlag = num;
            }
            else if(num == 0){
                Laya.timer.once(500,this,function(){
                    this.changeFlag = 0;
                });
            }
        }

        getChangeFlag(){
            return this.changeFlag;
        }

        checkIntimacyUp(petId,beforeIntimacy,nowIntimacy){
            if(!this.changeFlag){
                return ;
            }
            let beforeNum = this.getIntimacyGrade(petId,beforeIntimacy);
            let nowNum = this.getIntimacyGrade(petId,nowIntimacy);
            if(beforeNum < nowNum){
                if(!this.waitList){
                    this.waitList = new Array();
                }
                let obj = {
                    "petId" : petId,
                    "beforeIntimacy": beforeIntimacy,
                    "beforeNum":beforeNum,
                    "nowNum":nowNum,
                    "nowIntimacy":nowIntimacy
                };
                this.waitList.push(obj);
            }
        }

        getIntimacyGrade(petId,intimacy) {
            let config = ConfigMgr$1.getInstance().getItemById("Role", String(petId));
            for(const k in config.Intimacy) {
                let v = config.Intimacy[k];
                if(intimacy < v){
                    return k - 1;
                }
            }
            return config.Intimacy.length - 1;
        }

        getWaitList(){
            return this.waitList;
        }

        setWaitList(waitList){
            this.waitList = waitList;
        }

        setBeginFlag(num){
            this.begin = num;
        }

        getBeginFlag(){
            return this.begin;
        }
    }

    class IntimacyUpSystem{
        static init(){
            EventMgr.getInstance().registEvent("CHECK_INTIMACYUP", null, IntimacyUpSystem.checkIntimacyUp);
            EventMgr.getInstance().registEvent("MAY_SHOW_INTIMACYUP", null, IntimacyUpSystem.mayShowIntimacyUp);
            EventMgr.getInstance().registEvent("SHOW_INTIMACYUP", null, IntimacyUpSystem.showIntimacyUp);
            IntimacyUpDataMgr.getInstance().setBeginFlag(0);
        }

        static setChangeFlag(num){
            IntimacyUpDataMgr.getInstance().setChangeFlag(num);
        }

        static checkIntimacyUp(petId,beforeIntimacy,nowIntimacy){
            IntimacyUpDataMgr.getInstance().checkIntimacyUp(petId,beforeIntimacy,nowIntimacy);
        }

        static mayShowIntimacyUp(){
            if(!IntimacyUpDataMgr.getInstance().getBeginFlag()){
                IntimacyUpDataMgr.getInstance().setBeginFlag(1);
                IntimacyUpSystem.showIntimacyUp();
            }
            
        }

        static showIntimacyUp(){
            this.waitList = IntimacyUpDataMgr.getInstance().getWaitList();
            if(this.waitList && this.waitList.length > 0){
                let obj = this.waitList[0];
                this.waitList.splice(0, 1); 
                IntimacyUpDataMgr.getInstance().setWaitList(this.waitList);
                Laya.Dialog.close("ui/IntimacyUp.scene");
                Laya.Dialog.open("ui/IntimacyUp.scene",false,[obj.petId,obj.beforeNum,obj.beforeIntimacy,obj.nowNum,obj.nowIntimacy]);       
            }
            else {
                IntimacyUpDataMgr.getInstance().setBeginFlag(0);
            }
        }

    }

    class NauticalRewardPop extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable() {
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onVideoCallBack);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doubleGet);
            let rewards = this._param[0].rewards.rewards;
            let tripIndex = this._param[1];
            this.bg_panel1 = this.getChildByName("bg_panel1");
            this.bg_panel1.visible = false;
            this.confirm = this.rewardBox.getChildByName("confirm");
            this.confirm.on(Laya.Event.CLICK, this, this.doClose,[0,tripIndex]);
            this.confirm.visible=false;
            this.sure = this.rewardBox.getChildByName("sure");
            this.sure.on(Laya.Event.CLICK, this, this.doClose,[0,tripIndex]);
            this.sure.visible=false;
            this.sureText = this.sure.getChildByName("sureText");
            this.sureText.text = Strings$1.get("tid#Btn_009");
            this.photoConfirm = this.photoBox.getChildByName("photoConfirm");
            this.photoConfirm.on(Laya.Event.CLICK, this, this.closePhoto);
            this.close1 = this.rewardBox.getChildByName("close1");
            this.close1.on(Laya.Event.CLICK,this,this.doClose,[0,tripIndex]);
            this.leftBtn = this.photoBox.getChildByName("left");
            this.leftBtn.on(Laya.Event.CLICK, this, this.onLeftClick);
            this.rightBtn = this.photoBox.getChildByName("right");
            this.rightBtn.on(Laya.Event.CLICK, this, this.onRightClick);
            this.photoShare = this.photoBox.getChildByName("photoShare");
            this.photoShare.on(Laya.Event.CLICK, this, this.sharePhoto);
            this.photoShare.visible=false;
            this.photoItem = this.rewardBox.getChildByName("photoItem");
            this.photoItem.on(Laya.Event.CLICK,this,this.onPhotoClick);

            this.v_fishDouble=this.rewardBox.getChildByName('v_fishDouble');
            this.v_fishDouble.visible = false;
            this.v_fishDouble.on(Laya.Event.CLICK, this, this.adFishDouble);
            this.s_fishDouble=this.rewardBox.getChildByName('s_fishDouble');
            this.s_fishDouble.visible = false;
            this.s_fishDouble.on(Laya.Event.CLICK, this, this.shareFishDouble);
            this.buffBox = this.photoBox.getChildByName("buffBox");
            this.newImg = this.photoBox.getChildByName("newImg");
            this.adRewardRate = Number(Const.get("ADRewardRateSail"));
            SdkHelper.shanchuTuisong('yongshi_guihang_1');
            this.list.array = rewards;
            this.photoArr = this._param[0].photoRewards;
            this.photoAmount = this.photoItem.getChildByName("amount");
            this.photoAmount.text = this.photoArr.length + "";
            Laya.timer.once(300,this,function(){
                if(this.photoArr.length > 0){
                    this.list.x = 192;
                    this.list.visible = true;
                    Laya.timer.once(300,this,this.playPhoto);
                }
                else {
                    this.list.x = 352;
                    this.list.visible = true;
                    if(this.adRewardRate == 1){
                        this.s_fishDouble.visible = false;
                        this.v_fishDouble.visible = false;
                        this.sure.visible=true;
                        this.close1.visible = true;
                    }
                    else {
                        if(SdkHelper.hasVideo()) {
                            this.s_fishDouble.visible = false;
                            this.v_fishDouble.visible = true;
                            this.sure.visible = false;
                            this.close1.visible = true;
                        }else {
                            this.s_fishDouble.visible = true;
                            this.v_fishDouble.visible = false;
                        }
                        Laya.timer.once(1500,this,function(){
                            this.confirm.visible=true;
                            this.close1.visible = true;
                        });
                    }
                }
            });
            this.confirm.text = Strings$1.get("tid#Btn_019");
            IntimacyUpSystem.setChangeFlag(1);
        }

        onPhotoClick(){
            this.bg_panel1.visible = true;
            this.photoBox.visible = true;
            Laya.Tween.to(this.photoBox,{scaleX:1,scaleY:1},100);
        }

        playPhoto(){
            this.bg_panel1.visible = true;
            this.photoBox.scaleX = 1;
            this.photoBox.scaleY = 1;
            if (this.photoArr && this.photoArr.length > 0) {
                this.photoBox.visible = false;
                this.photoAnim.visible = true;
                this.currentPhotoId = 0;
                let str = "";
                for (let i = 0; i < this.photoArr.length - 1; i++) {
                    str += ",";
                }
                this.tab.labels = str;
                this.tab.x = (this.photoBox.width - this.tab.width * this.tab.scaleX) / 2;
                this.checkLeftRightShow();
                this.photoShow();
                if (this.animLoadComplete) {
                    this.anim.play("piaochu", false);
                    return;
                }
                this.anim = new Laya.Skeleton();
                this.anim.load("anim3/zhaopian.sk", Laya.Handler.create(this, function () {
                    this.anim.on(Laya.Event.STOPPED, this, this.animPlayComplete);
                    this.animLoadComplete = true;
                    this.anim.play("piaochu", false);
                    this.anim.x = this.photoAnim.width / 2;
                    this.anim.y = this.photoAnim.height / 2;
                    this.photoAnim.addChild(this.anim);
                }));
            } else {
                this.photoBox.visible = false;
                this.photoAnim.visible = false;
            }
            this.close1.visible = true;
            if(this.adRewardRate == 1){
                this.s_fishDouble.visible = false;
                this.v_fishDouble.visible = false;
                this.sure.visible = true;
            }
            else {
                if(SdkHelper.hasVideo()) {
                    this.s_fishDouble.visible = false;
                    this.v_fishDouble.visible = true;
                }else {
                    this.s_fishDouble.visible = true;
                    this.v_fishDouble.visible = false;
                }
            }
        }

        doClose(guanggao,tripIndex) {
            EventMgr.getInstance().postEvent("left_RED_POINT_SHOW", [true]);  
            EventMgr.getInstance().postEvent("BOAT_GET_REWARD", [guanggao,tripIndex,Laya.Handler.create(this, function (res) {
                this.list.array = res.rewards.rewards;
                Laya.timer.frameOnce(1,this,function(){
                    let cells = this.list.cells;
                    cells.forEach(element => {
                        if (element) {
                            element.showItemFly(guanggao);
                        }
                    }); 
                });
                TaskMainSystem.addProgress("SAIL_TIMES", 1);
            })]);
            SoundManager.getInstance().play("Sound_fishingPerfectSuccess");
            this.close();
        }

        closePhoto() {
            this.bg_panel1.visible = false;
            this.photoItem.visible = true;
            this.photoAnim.visible = false;
            Laya.Tween.to(this.photoBox,{scaleX:0,scaleY:0},100);
            Laya.timer.once(100,this,function(){
                if(this.adRewardRate > 1){
                    this.confirm.visible=true;
                }
                this.photoBox.visible = false;
            });
        }

        animPlayComplete() {
            this.photoBox.visible = true;
            this.photoAnim.visible = false;
        }

        onLeftClick() {
            if (this.currentPhotoId <= 0) {
                return;
            }
            this.currentPhotoId--;
            this.checkLeftRightShow();
            this.photoShow();
        }

        onRightClick() {
            if (this.currentPhotoId >= this.photoArr.length - 1) {
                return;
            }
            this.currentPhotoId++;
            this.checkLeftRightShow();
            this.photoShow();
        }

        checkLeftRightShow() {
            
            if (this.currentPhotoId <= 0) {
                this.leftBtn.visible = false;
            } else {
                this.leftBtn.visible = true;
            }
            if (this.currentPhotoId >= this.photoArr.length - 1) {
                this.rightBtn.visible = false;
            } else {
                this.rightBtn.visible = true;
            }
        }

        photoShow() {
            let id = this.photoArr[this.currentPhotoId].photoId;
            let currentPhotoConfig = ConfigMgr$1.getInstance().getItemById("CatPhotos", id);
            let catSkins = JSON.parse(currentPhotoConfig.Cat);
            if (this.catArr && this.catArr.length != 0) {
                for (const cat of this.catArr) {
                    cat.destroy();
                }
            }
            this.catArr = [];
            let catCatroute ="systemPhoto/"+currentPhotoConfig.Catroute;
            for (const catSkin of catSkins) {
                let cat = new Laya.Image();
                cat.skin = catCatroute + "/" + catSkin + ".png";
                cat.width = this.photo.width;
                cat.height = this.photo.height;
                this.photo.addChild(cat);
                this.catArr.push(cat);
            }
            this.photoBg.skin = "systemPhotoBg/"+currentPhotoConfig.Background+".jpg";
            this.tab.selectedIndex = this.currentPhotoId;
            this.buffBox.setType(id,1);
            let data = PhotoSystem.getSystemPhotosData();
            this.newImg.visible = true;
            for(let i = 0;i<data.length;i++){
                if(id == data[i].getId()){
                    this.newImg.visible = false;
                }
            }
        }

        onDisable() {
            EventMgr.getInstance().postEvent("MAY_SHOW_INTIMACYUP");
            IntimacyUpSystem.setChangeFlag(0);
        }

        sharePhoto() {
            this.photoConfirm.visible = false;
            let title = Strings$1.get("tid#ShareTips4");
            let photoBg = this.photoBox.getChildByName("photoBg");
            let width = photoBg.height * photoBg.scaleY / 4 * 5;
            let x = photoBg.x + (photoBg.width * photoBg.scaleX - width) / 2;
            let point = new Laya.Point(x, photoBg.y);
            let rightDownPoint = new Laya.Point(x + width, photoBg.y + photoBg.height * photoBg.scaleY);
            point = photoBg.parent.localToGlobal(point);
            rightDownPoint = photoBg.parent.localToGlobal(rightDownPoint);
            let handler = Laya.Handler.create(this, function () {
                this.photoConfirm.visible = true;
            });
            Laya.timer.frameOnce(5, this, function () {
                Camera.shareScreenshot(rightDownPoint.x - point.x, rightDownPoint.y - point.y, point.x, point.y, title, Constants.ShareId.NuticalPhoto, handler);
            });
        }
        shareFishDouble(){
            SdkHelper.shareAppMessage(Constants.ShareId.NauticalReward);
        }
        adFishDouble(){
            SdkHelper.showRewardVideoAd(Constants.ShareId.NauticalReward);
        }
        onVideoCallBack(shareId) {
            if (shareId == Constants.ShareId.NauticalReward) {
                this.doubleGet(shareId);
            }
        }

        doubleGet(shareId) {
            if (shareId == Constants.ShareId.NauticalReward) { 
                this.doClose(Constants.FetchType.AD,tripIndex);
            }
        }
    }

    class BuffBox extends Laya.Box{
        constructor(){
            super();
        }

        setType(Id,showStatus){//showStatus 0 相册 1 出航
            showStatus = showStatus ? 1:0;
            let type1 = this.getChildByName("type1");
            let type2 = this.getChildByName("type2");
            let number = type1.getChildByName("number");
            let data = PhotoSystem.getSystemPhotosData();
            let sum = 0;
            for(let i = 0;i<data.length;i++){
                if(showStatus){
                    if(Id == data[i].getId()){
                        this.visible = false;
                        return ;
                    }
                }
                let config = data[i].getSystemConfig();
                if(config.SpecialFunctionType == "1"){
                    sum += JSON.parse(config.SpecialValue).Rate;
                }
            }
            this.visible = true;
            let CatPhotosTipsAdditionLimit = Const.get("CatPhotosTipsAdditionLimit");
            let incomeRate = JSON.parse(ConfigMgr$1.getInstance().getItemById("CatPhotos",Id).SpecialValue).Rate;
            if(showStatus && incomeRate + sum > CatPhotosTipsAdditionLimit){
                type1.visible = false;
                type2.visible = true;
            }
            else {
                type1.visible = true;
                type2.visible = false;
                number.value = incomeRate * 100 | 0;
            }
        }
    }

    function sortByOrder$1(a,b)
    {
        return a.Rank-b.Rank;
    }

    class NauticalStartPop extends Laya.Dialog {

        onOpened() {
            this.showFoodList(1);
        }

        onEnable() {
            this.clearSelectCat();
            this.nauticalIndex = this._param;
            this.maohangli1=0;
            this.qinmidu1=0;
            this.maohangli2=0;
            this.qinmidu2=0;
            this.maohangliRate=1;
            this.qinmiduRate=1;
            this.hangLabel.visible=false;
            this.xinXLabel.visible=false;
            this.maoXin.visible=false;
            this.maoHangL.visible=false;
            this.miaoHangSprite.visible=false;
            this.hideFriends = 1;

            this.bg_panel.on(Laya.Event.CLICK, this, this.doClose);

            this.cat_panel.on(Laya.Event.CLICK, this, this.hide);
            this.DianXinSprite.on(Laya.Event.CLICK, this,function(){
               //事件穿透了
            });
            this.start.on(Laya.Event.CLICK, this, this.showCatList);

            this.food_bg1.on(Laya.Event.CLICK, this, this.showFoodList, [1]);
            this.food_bg2.on(Laya.Event.CLICK, this, this.showFoodList, [2]);
            this.tool_bg.on(Laya.Event.CLICK, this, this.showToolList);

            this.noLabelBtn1.on(Laya.Event.CLICK, this,this.toFishing);
            this.noLabelBtn2.on(Laya.Event.CLICK, this,this.toFriend);
            this.noLabelBtn3.on(Laya.Event.CLICK, this,this.toShop);
            this.noLabelBtn5.on(Laya.Event.CLICK, this,function(){
                Laya.Dialog.close("ui/NauticalGetFood.scene");
                Laya.Dialog.open("ui/NauticalGetFood.scene",false);
            });

            this.start_trip.on(Laya.Event.CLICK, this, this.startTrip);
            this.start_trip.label = Strings$1.get("tid#Btn_025");

            EventMgr.getInstance().registEvent("NAUTICAL_FOOD_SELECTED_CHANGE", this, this.changeSelectFoodId);
            EventMgr.getInstance().registEvent("NAUTICAL_TOOL_SELECTED_CHANGE", this, this.changeSelectToolId);
            EventMgr.getInstance().registEvent("NAUTICAL_CAT_SELECTED_CHANGE" , this, this.changeSelectCatId);
            EventMgr.getInstance().registEvent("NAUTICAL_FOOD_DESC_SHOW" , this, this.showFoodDesc);
            EventMgr.getInstance().registEvent("NAUTICAL_FOOD_DESC_HIDE" , this, this.hideFoodDsec);
            EventMgr.getInstance().registEvent("NAUTICAL_FOOD_UPDATE", this, this.showFoodList);
            EventMgr.getInstance().registEvent("NAUTICAL_STARTPOP_CLOSE", this, this.close);

            this.hide();
            this.initView();
            if(!this._selectAnim){
                let anim = new Spine();
                this._selectAnim = anim;
                anim.playAnim("xuanzhong", true);
                this.addChild(anim);
                anim.loadAnim("anim3/matouui.sk", null, 0);
                this._selectAnim.visible = false;
            }
            this.hangTLabel.text = Strings$1.get("tid#UI_111");
            this.qinTLabel.text = Strings$1.get("tid#UI_112");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("NAUTICAL_FOOD_SELECTED_CHANGE", this, this.changeSelectFoodId);
            EventMgr.getInstance().removeEvent("NAUTICAL_TOOL_SELECTED_CHANGE", this, this.changeSelectToolId);
            EventMgr.getInstance().removeEvent("NAUTICAL_CAT_SELECTED_CHANGE" , this, this.changeSelectCatId);
            EventMgr.getInstance().removeEvent("NAUTICAL_FOOD_DESC_SHOW" , this, this.showFoodDesc);
            EventMgr.getInstance().removeEvent("NAUTICAL_FOOD_DESC_HIDE" , this, this.hideFoodDsec);
            EventMgr.getInstance().removeEvent("NAUTICAL_FOOD_UPDATE", this, this.showFoodList);
            EventMgr.getInstance().removeEvent("NAUTICAL_STARTPOP_CLOSE", this, this.close);
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }

        toFishing(){
            if(GameContext.Sliding)
                return;
            let afterHandler = Laya.Handler.create(this, function (res) {
                SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
                this.destroy();
            });
            FishingSystem.updateServerBaseInfo(afterHandler);
        }

        toFriend(){
            SoundManager.getInstance().play("Sound_clickSound", 1);
            if (Laya.Browser.onMiniGame) {
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
            }
        }

        toShop(){
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
            SyncSystem.updateSceneInfo();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            Laya.Dialog.close("ui/shopPop.scene");
            Laya.Dialog.open("ui/shopPop.scene",true,6);
        }

        startTrip() {
            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            if(nauticalInfo.team.length>0) {
                this.close();
                NauticalSystem$1.startNauticalTrip(this.nauticalIndex);
                Laya.Dialog.close("ui/NauticalTravelingPop.scene");
                Laya.Dialog.open("ui/NauticalTravelingPop.scene", true, ["chuhang",this.nauticalIndex]);
                SoundManager.getInstance().play("Sound_setSail");
                EventMgr.getInstance().postEvent("left_RED_POINT_SHOW",[false]);
                Laya.timer.once(100,this,function(){
                    EventMgr.getInstance().postEvent("CHECK_NAUTICALREWARD_REDPOINT");
                });
                
            }else {
                TipsPopMgr.getInstance().showTipsById("tid#sailMinCatNumber");
            }
        }

        hide() {
            this._curSelectedIndex = null;
            this.panel.visible = false;
            this.title.visible = false;
            this.goods.visible = false;
            this.cats.visible = false;
            this.cat_panel.visible = false;
        }

        initView() {
            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            NauticalSystem$1.checkFood(nauticalInfo);
            NauticalSystem$1.checkTool(nauticalInfo);
            NauticalSystem$1.checkPets(nauticalInfo);

            this.setFood1(nauticalInfo.food1);
            this.setFood2(nauticalInfo.food2);

            this.setTool(nauticalInfo.tool);

        }


        setIcon(icon, skin) {
            icon.skin = skin;
            let scalex = 125 / icon.width;
            let scaley = 125 / icon.height;
            if (scalex <= scaley) {
                icon.scaleX = scalex;
                icon.scaleY = scalex;
            } else {
                icon.scaleX = scaley;
                icon.scaleY = scaley;
            }
        }

        setSelectPos(node) {
            let point = new Laya.Point(node.x, node.y);
            node.parent.localToGlobal(point);
            this.globalToLocal(point);
            this._selectAnim.pos(point.x, point.y);
        }


        setFood1(foodId) {
            if(foodId) {
                let food = NauticalSystem$1.getFoodById(foodId);
                this._foodId1 = foodId;
                this.setIcon(this.food1, food.Icon);
                this.food_bg1_2.visible = true;
                this.showMaoHL(food,1);
            }else {
                this._foodId1 = foodId;
                this.setIcon(this.food1, null);
                this.food_bg1_2.visible = false;
                this.showMaoHL(foodId,4);
            }
        }

        setFood2(foodId) {
            if(foodId) {
                let food = NauticalSystem$1.getFoodById(foodId);
                this._foodId2 = foodId;
                this.setIcon(this.food2, food.Icon);
                this.food_bg2_2.visible = true;
                this.showMaoHL(food,2);
            }else {
                this._foodId2 = foodId;
                this.setIcon(this.food2, null);
                this.food_bg2_2.visible = false;
                this.showMaoHL(foodId,5);
            }
        }

        setTool(toolId) {
            if(toolId) {
                let tool = NauticalSystem$1.getToolById(toolId);
                this._toolId = toolId;
                this.setIcon(this.tool, tool.Icon);
                this.tool_bg_2.visible = true;
                this.showMaoHL(tool,3);
            }else {
                this._toolId = toolId;
                this.setIcon(this.tool, null);
                this.tool_bg_2.visible = false;
                this.showMaoHL(toolId,6);
            }
        }

        calcCat(limit) {
            if(!limit){
                for(let i = 0;i<NauticalSystem$1.btnList.length;i++){
                    if(NauticalSystem$1.btnList[i].nauticalIndex == this.nauticalIndex){
                        limit = NauticalSystem$1.PetsLimit[i];
                        break;
                    }
                }
            }
            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            let pets = nauticalInfo.team;
            this.cat_count.text = Strings$1.get("tid#UI_009", [String(pets.length) + "/" + String(limit)]);
        }

        showFoodList(selectIndex) {
            // this.noLabel1.visible = true;
            // this.noLabel2.visible = true;
            this.noLabel3.visible = false;
            this.noLabel4.visible = false;
            this.noLabel5.visible = true;
            if(this._curSelectedIndex == selectIndex) {
                if(selectIndex==1&&this._foodId1) {
                    this.setFood1(null);
                    NauticalSystem$1.NauticalInfo[this.nauticalIndex].food1 = null;
                    this.goodsList.refresh();
                    return;
                }else if(selectIndex==2&&this._foodId2) {
                    this.setFood2(null);
                    NauticalSystem$1.NauticalInfo[this.nauticalIndex].food2 = null;
                    this.goodsList.refresh();
                    return;
                }
            }
            this._curSelectedIndex = selectIndex;
            this._selectAnim.visible = true;
            let excludeFoodId = NauticalSystem$1.NauticalInfo[this.nauticalIndex].food1;
            let foodType = "food2";
            if(selectIndex==1) {
                this.setSelectPos(this.food1);
                excludeFoodId = NauticalSystem$1.NauticalInfo[this.nauticalIndex].food2;
                foodType = "food1";
            }else {
                this.setSelectPos(this.food2);
            }
            this.panel.visible = true;
            this.title.visible = true;
            this.goods.visible = true;
            this.title.text = "口粮";
            this.title.text = Strings$1.get("tid#UI_123");
            this.cats.visible = false;
            this.cat_panel.visible = false;
            this.left_arrow = this.goods.getChildByName("left_arrow");
            this.right_arrow = this.goods.getChildByName("right_arrow");
            this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);

            let index = 0;
            let foods = NauticalSystem$1.Foods;
            this.array = new Array();

            foods.sort(sortByOrder$1);

            for(const food of foods) {
                if(food.count > 1 || (food.count==1&&food.Id!=excludeFoodId))
                    this.array.push([food.Id, foodType,this.nauticalIndex]);
            }

            // this.array = [["CatFood1", "food"]];
            this.list = this.goodsList;
            this.noLabel1.text = Strings$1.get("tid#TipsCommon1");
            this.noLabelBtn1.label = Strings$1.get("tid#Btn_011");
            this.noLabel2.text = Strings$1.get("tid#TipsCommon1_2");
            if(this.hideFriends){
                this.noLabel2.visible = false;
                this.noLabel1.y = -5;
                this.noLabelBtn1.y = 125;
                this.noLabelBtn1.x = 308;
            }

            this.goodsList.off(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.goodsList.on(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.goodsList.off(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.goodsList.on(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.goodsList.scrollBar.rollRatio = 0;

            this.noLabel5.text = Strings$1.get("tid#UI_209");
            this.noLabelBtn5.label = Strings$1.get("tid#Btn_047");
            this.noBox.y = 2010;

            this.startIndex = 0;
            this.endIndex =this.array.length;

            this.changeListItems();
            if (this._curFoodIndex == null) {
                return;
            }
            this._index = this._curFoodIndex;
            this.goodsList.scrollTo(this._curFoodIndex);
            this.loadArrow();
        }

        showToolList() {
            this.noLabel1.visible = false;
            this.noLabel2.visible = false;
            this.noLabel3.visible = true;
            this.noLabel4.visible = false;
            this.noLabel5.visible = false;
            if(this._curSelectedIndex == 3) {
                if(this._toolId) {
                    this.setTool(null);
                    NauticalSystem$1.NauticalInfo[this.nauticalIndex].tool = null;
                    this.goodsList.refresh();
                    return;
                }
            }
            this._curSelectedIndex = 3;
            this._selectAnim.visible = true;
            this.setSelectPos(this.tool);
            this.panel.visible = true;
            this.title.visible = true;
            this.goods.visible = true;
            this.title.text = "道具";
            this.title.text = Strings$1.get("tid#UI_124");
            this.cats.visible = false;
            this.cat_panel.visible = false;
            this.left_arrow = this.goods.getChildByName("left_arrow");
            this.right_arrow = this.goods.getChildByName("right_arrow");
            this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);

            let index = 0;
            let tools = NauticalSystem$1.Tools;
            this.array = new Array();
            tools.sort(sortByOrder$1);
            for(const tool of tools) {
                this.array.push([tool.Id, "tool",this.nauticalIndex]);
            }
            // this.array = [["CatItem1", "tool"]];
            this.list = this.goodsList;
            this.goodsList.scrollTo(0);
            this.goodsList.off(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.goodsList.on(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.goodsList.off(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.goodsList.on(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.goodsList.scrollBar.rollRatio = 0;
            this.noLabel3.text = Strings$1.get("tid#TipsCommon2");
            this.noBox.y = 2010;

            this.startIndex = 0 ;
            this.endIndex =this.array.length;

            this.changeListItems();
        }

        showCatList() {
            if(this._foodId1==null && this._foodId2==null) {
                TipsPopMgr.getInstance().showTipsById("tid#sailFoodLack");
                let anim = new Spine();
                anim.loadAnim("anim3/matouui.sk");
                anim.playAnim("UItixing", false);
                this.animBox.addChild(anim);
                anim.pos(this.animBox.width/2,this.animBox.height/2);
                anim.addCompleteHandler(Laya.Handler.create(this,function(){
                    anim.destroy();
                }));
                return;
            }
            if(this._curSelectedIndex==4)
                return;
            this.noLabel1.visible = false;
            this.noLabel2.visible = false;
            this.noLabel3.visible = false;
            this.noLabel4.visible = true;
            this.noLabel5.visible = false;
            this._curSelectedIndex = 4;
            this._selectAnim.visible = false;
            this.panel.visible = true;
            this.title.visible = true;
            this.goods.visible = false;
            this.title.text = "猫猫";
            this.title.text = Strings$1.get("tid#UI_104");
            this.cats.visible = true;
            this.cat_panel.visible = false;
            this.left_arrow = this.cats.getChildByName("left_arrow");
            this.right_arrow = this.cats.getChildByName("right_arrow");
            this.left_arrow.on(Laya.Event.CLICK, this, this._onLeftArrowChange);
            this.right_arrow.on(Laya.Event.CLICK, this, this._onRightArrowChange);

            this.calcCat();

            let index = 0;
            let pets = NauticalSystem$1.Pets;
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            function sortByRankOrder(a,b)
            {
                return config.get(a).Rank-config.get(b).Rank;
            }
            pets.sort(sortByRankOrder);
            this.array = new Array();
            for(const petId of pets) {
                this.array.push(petId);
            }
            // this.array = ["Cat1"];
            this.list = this.catList;

            this.catList.off(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.catList.on(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.catList.off(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.catList.on(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.catList.scrollBar.rollRatio = 0;
            this.noLabel4.text = Strings$1.get("tid#TipsCommon3");
            this.noBox.y = 2010;

            this.startIndex = 0;
            this.endIndex =this.array.length;

            this.changeListItems();
        }

        adjustList(save){
            let w = this.list.getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w + s) + 0.5 | 0;
            if (save) { 
                this.saveFoodIndex();
            }
            Laya.Tween.to(this.list.scrollBar,{value:this._index*(w+s)},100);
        }

        changeListItems2(){
            this.list.scrollBar.stopScroll();
            this.changeListItems1();
        }

        changeListItems1() {
            this.loadList(true);
            this.loadArrow();
        }

        changeListItems() {
            this.list.array = this.loadList();
            //this.list.scrollBar.stopScroll();
            //修改left_arrow 和 right_arrow 的展示
            this.loadArrow();
        }

        loadList(save){
            let showList = this.array.slice(this.startIndex, this.endIndex);
            this.adjustList(save); 
            if (showList.length > 0) {
                this.noBox.visible = false;
                this.catNum.visible = true;
                this.cat_count.visible = true;
                this.start_trip.visible = true;
            } else {
                this.noBox.visible = true;
                this.catNum.visible = false;
                this.cat_count.visible = false;
                this.start_trip.visible = false;
            }
            
            return showList;
        }

        loadArrow(){
            if(this._index == 0) {
                this.left_arrow.visible = false;
            } else {
                this.left_arrow.visible = true;
            }
            if(this._index >= this.endIndex -3 || this.endIndex < 4) {
                this.right_arrow.visible = false;
            } else {
                this.right_arrow.visible = true;
            }
        }

        _onLeftArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - 3 > 0 ? this._index - 3 : 0;
            this.saveFoodIndex();
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        _onRightArrowChange() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + 3 < this.endIndex ? this._index + 3 : this.endIndex - 2;
            this.saveFoodIndex();
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        changeSelectFoodId(foodId, foodType) {
            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            if(foodType == "food1") {
                nauticalInfo.food1 = foodId;
                this.setFood1(foodId);
            }else if (foodType == "food2"){
                nauticalInfo.food2 = foodId;
                this.setFood2(foodId);
            }

            this.goodsList.refresh();
        }

        changeSelectToolId(toolId) {

            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];

            nauticalInfo.tool = toolId;
            this.setTool(toolId);

            this.goodsList.refresh();
        }

        clearSelectCat(){
            for(let i = 0; i< NauticalSystem$1.NauticalInfo.length;i++){
                NauticalSystem$1.NauticalInfo[i].removeAllPet();
            }
        }

        changeSelectCatId(catId) {
            let nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            for(let i = 0;i<NauticalSystem$1.btnList.length;i++){
                if(NauticalSystem$1.btnList[i].nauticalIndex == this.nauticalIndex){
                    var limit = NauticalSystem$1.PetsLimit[i];
                    break;
                }
            }
            if(nauticalInfo.hasPet(catId)) {
                nauticalInfo.removePet(catId);
            }else {
                let team = nauticalInfo.team;
                if(team.length>=limit) {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon6");
                    return;
                }
                nauticalInfo.addPet(catId);
            }
            this.calcCat(limit);
            this.catList.refresh();
        }

        doClose() {
            this.close();
        }
        showMaoHL(food,type){
            if(type==1){
                this.maohangli1=Number(food.Point);
                this.qinmidu1=Number(food.Intimacy);     
            }
            else if(type==2){
                this.maohangli2=Number(food.Point);
                this.qinmidu2=Number(food.Intimacy);     
            }
            else if(type==3){
                this.maohangliRate=Number(food.PointRate);
                this.qinmiduRate=Number(food.IntimacyRate);   
            }
            else if(type==4){
                this.maohangli1=0;
                this.qinmidu1=0;

            }
            else if(type==5){
                this.maohangli2=0;
                this.qinmidu2=0;
            }
            else if(type==6){
                this.maohangliRate=1;
                this.qinmiduRate=1;
            }
            let max=0;
            let maohangli= Math.floor((this.maohangli1+this.maohangli2)*this.maohangliRate);
            let maoqinmu=Math.floor((this.qinmidu1+this.qinmidu2)*this.qinmiduRate);
            let configs = ConfigMgr$1.getInstance().getConfig("Role");
            for(const [id, config] of configs) {
                for(let intimacy of config.Intimacy){

                    if(intimacy>max){
                        max=intimacy;
                    }
                };
            }
            if(maohangli>0||maoqinmu>0){
                this.hangLabel.visible=true;
                this.xinXLabel.visible=true;
                this.maoXin.visible=true;
                this.maoHangL.visible=true;
            }
            else {
                this.hangLabel.visible=false;
                this.xinXLabel.visible=false;
                this.maoXin.visible=false;
                this.maoHangL.visible=false;
            }
            this.hangLabel.text=maohangli;

            this.xinXLabel.text=maoqinmu>max?"max":maoqinmu+"";
               
           
            this.maoHangL.x=(this.DianXinSprite.width-(this.maoHangL.width+5+ this.hangLabel.width+20+this.maoXin.width+this.xinXLabel.width+5))/2;

            this.hangLabel.x=this.maoHangL.x+this.maoHangL.width+5;
            this.maoXin.x=this.hangLabel.x+this.hangLabel.width+20;
            this.xinXLabel.x=this.maoXin.x+this.maoXin.width+5;
        }
        showFoodDesc(foodId,type,zuobiao){
            if((zuobiao-this._index*325)%2==0&&(zuobiao-this._index*325)/2!=0){
                this.tipHbg.scaleX=-2.5;
                this.tipHbg.x=595;
                this.miaoHangSprite.x=zuobiao-this._index*325+150;
            }
            else {
                this.tipHbg.scaleX=2.5;
                this.tipHbg.x=0;
                this.miaoHangSprite.x=400+zuobiao-this._index*325;
            }
          
            this.miaoHangSprite.visible=true;
            if(type=="tool"){
                let tool = NauticalSystem$1.getToolById(foodId);
                this.miaoHL.text="x"+Math.floor(Number(tool.PointRate)*100)+"%";
                this.miaoXL.text='x'+Math.floor(Number(tool.IntimacyRate)*100)+"%"; 
                this.qinTLabel.x=this.hangTLabel.x=(595-(this.hangTLabel.width+this.miaoH.width+this.miaoHL.width+10+10))/2;
                this.miaoX.x=this.miaoH.x=this.hangTLabel.x+10+this.hangTLabel.width;
                this.miaoXL.x=this.miaoHL.x=this.miaoH.x+10+this.miaoH.width;

                this.miaoHL.y=this.hangTLabel.y=(258*0.92-55*2-30)/2;
                this.miaoXL.y=this.qinTLabel.y=this.hangTLabel.y+85;
                this.miaoH.y=this.miaoHL.y+0.5;
                this.miaoX.y=this.miaoXL.y+0.5;
            }
            else {
                let food = NauticalSystem$1.getFoodById(foodId);
                this.miaoHL.text="+"+Number(food.Point);
                this.miaoXL.text='+'+Number(food.Intimacy);  
                this.qinTLabel.x=this.hangTLabel.x=(595-(this.hangTLabel.width+this.miaoH.width+this.miaoHL.width+20))/2;
                this.miaoX.x=this.miaoH.x=this.hangTLabel.x+this.hangTLabel.width;
                this.miaoXL.x=this.miaoHL.x=this.miaoH.x+20+this.miaoH.width;
                        
                this.miaoHL.y=this.hangTLabel.y=(258*0.92-55*2-30)/2;
                this.miaoXL.y=this.qinTLabel.y=this.hangTLabel.y+85;
                this.miaoH.y=this.miaoHL.y+0.5;
                this.miaoX.y=this.miaoXL.y+0.5;
                
            }
        }
        hideFoodDsec(){
            this.miaoHangSprite.visible=false;
        }

        // EventMgr.getInstance().postEvent("BOAT_START_TRIP");
        saveFoodIndex() {
            if (this._curSelectedIndex == 1 || this._curSelectedIndex == 2) {
                this._curFoodIndex = this._index;
            }
        }

    }

    class GoodsSelectItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(data) {
            let id = data[0];
            let type = data[1];
            let nauticalIndex = data[2];
            // console.debug("GoodsSelectItem", id, type);
            this.icon_box=this.getChildByName('icon_box');
            this.icon_box.offAll();
            this.name = this.getChildByName("name");
            this.icon = this.getChildByName("icon");
            this.count = this.getChildByName("count");
            this.hook = this.getChildByName("hook");
            this.desc = this.getChildByName("desc");
            this.merge = this.getChildByName("merge");
            this.count.visible = false;
            this.hook.visible = false;
            this.merge.visible = false;

            this._nauticalInfo = NauticalSystem$1.NauticalInfo[nauticalIndex];
            this.clclick=0;

            if(type=="tool")
                this.showTool(id);
            else {
                this.merge.on(Laya.Event.CLICK, this, this.openMerge,[id]);
                this.showFood(id, type);
            }
        }

        showFood(foodId, foodType) {
            this.icon_box.offAll();
            let food = NauticalSystem$1.getFoodById(foodId);
            let item = NauticalSystem$1.getFoodCanMergeInfo(foodId);
            this.name.text = Strings$1.get(food.Name);
            this.desc.text = Strings$1.get(food.Desc);
            this.count.visible = true;
            let usedCount = 0;
            if(foodId==this._nauticalInfo.food1)
                ++usedCount;
            if(foodId==this._nauticalInfo.food2)
                ++usedCount;
            this.count.text = String(food.count-usedCount);
            if(food.count-usedCount==0) {
                this.count.color = "#ea1a1a";
            }else {
                this.count.color = "#363232";
            }
            if(item){
                let upgradeLineNum = JSON.parse(item.UpgradeLine)[0].amount;
                if (food.count - usedCount >= upgradeLineNum) {
                    this.merge.visible = true;
                }
            }

            this.setIcon(food.Icon);
            this.icon_box.on(Laya.Event.MOUSE_DOWN, this, this.showFoodDesc,[foodId, foodType]);
            if((foodType=="food1"&&foodId==this._nauticalInfo.food1)||(foodType=="food2"&&foodId==this._nauticalInfo.food2)) {
                // this.hook.visible= true;
            }else {
                this.icon_box.on(Laya.Event.CLICK, this, this.onFoodSelected, [foodId, foodType]);
            }
        }


        showTool(toolId) {
            this.icon_box.offAll();
            let tool = NauticalSystem$1.getToolById(toolId);
            this.name.text = Strings$1.get(tool.Name);
            this.desc.text = Strings$1.get(tool.Desc);
            this.setIcon(tool.Icon);
            this.icon_box.on(Laya.Event.MOUSE_DOWN, this, this.showFoodDesc,[toolId,"tool"]);
            if(this._nauticalInfo.hasTool(toolId)) {
                this.hook.visible= true;
            }else {
                this.icon_box.on(Laya.Event.CLICK, this, this.onToolSelected, [toolId]);
            }
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 169 / this.icon.width;
            let scaley = 169 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        openMerge(foodId){
            GameContext.CurrentInfoId = foodId;
            GameContext.CurrentInfoType = "food";
            Laya.Dialog.close("ui/MergePop.scene");
            Laya.Dialog.open("ui/MergePop.scene", false);
        }


        onFoodSelected(foodId, foodType) {
            if(this.clclick==0){
                EventMgr.getInstance().postEvent("NAUTICAL_FOOD_SELECTED_CHANGE", [foodId, foodType]);
            }
        }

        onToolSelected(toolId) {
            if(this.clclick==0){
            EventMgr.getInstance().postEvent("NAUTICAL_TOOL_SELECTED_CHANGE", [toolId]);
            }
        }
        showFoodDesc(foodId,type){
            this.clclick=0;
            Laya.timer.once(200, this, this.onHold,[foodId,type]);
            this.icon_box.on(Laya.Event.MOUSE_UP, this, this.releaseShowFood);
            this.icon_box.on(Laya.Event.MOUSE_OUT, this, this.releaseShowFood);
            
        }
        onHold(foodId,type){
            this.clclick=1;
            EventMgr.getInstance().postEvent("NAUTICAL_FOOD_DESC_SHOW",[foodId,type,this.x]);
        }
        releaseShowFood(){
            EventMgr.getInstance().postEvent("NAUTICAL_FOOD_DESC_HIDE");
            Laya.timer.once(1,this,function(){
                this.clclick=0;
            });
            Laya.timer.clear(this, this.onHold);
            this.icon_box.off(Laya.Event.MOUSE_UP, this, this.releaseShowFood);
            this.icon_box.off(Laya.Event.MOUSE_OUT, this, this.releaseShowFood);
        }
    }

    class CatSelectItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(catId) {
            this.offAll();
            // this.name = this.getChildByName("name");
            this.icon = this.getChildByName("icon");
            // this.count = this.getChildByName("count");
            this.hook = this.getChildByName("hook");
            // this.desc = this.getChildByName("desc");
            // this.count.visible = false;
            this.hook.visible = false;
            this.showCat(catId);
        }

        showCat(catId) {
            let config = ConfigMgr$1.getInstance().getItemById("Role", catId);
            // this.name.text = Strings.get(config.Name);
            if(config.HeadIcon)
                this.setIcon("cat_collect/head/"+config.HeadIcon+".png");

            if(NauticalSystem$1.NauticalInfo[0].hasPet(catId) || NauticalSystem$1.NauticalInfo[1].hasPet(catId) || NauticalSystem$1.NauticalInfo[2].hasPet(catId)) {
                this.hook.visible= true;
            }
            this.on(Laya.Event.CLICK, this, this.onSelected, [catId]);
        }

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 169 / this.icon.width;
            let scaley = 169 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        onSelected(catId) {
            EventMgr.getInstance().postEvent("NAUTICAL_CAT_SELECTED_CHANGE", [catId]);
        }
    }

    class NauticalTravelingPop$1 extends Laya.Dialog {

        onEnable() {
            this.nauticalIndex = this._param[1];
            this.bgPanel = this.getChildByName("bg_panel");
            this.bgPanel.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.close1 = this.getChildByName("close1");
            this.close1.on(Laya.Event.CLICK, this, function () {
                this.close();
            });
            this.animPanel = this.getChildByName("animPanel");
            this.time = this.getChildByName("time");
            this.yinying = this.getChildByName("yinying");
            this.jiasu = this.getChildByName("jiasu");
            this.jiasu.text = Strings$1.get("tid#UI_044", [Math.round(Number(Const.get("NauticalADReduceTime")) / 60)]);
            this.videBtn = this.jiasu.getChildByName("video");
            this.videBtn.on(Laya.Event.CLICK, this, this.onVideoClick);
            this.shareBtn = this.jiasu.getChildByName("share");
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareClick);
            this.remainingTimeText = this.getChildByName("remainingTime");
            this.CD = this.getChildByName("CD");
            this.CD.on(Laya.Event.CLICK, this, this.onCDClick);
            this.CDtext = this.CD.getChildByName("CDtext");
            this.NauticalADTimesCD = Const.getArray("NauticalADTimesCD");

            this.couponNum = NauticalSystem$1.getNauticalTicketNum();
            this.couponBox = this.getChildByName("couponBox");
            this.couponBtn = this.couponBox.getChildByName("couponBtn");
            this.couponNumLab = this.couponBox.getChildByName("couponNum");
            this.couponNumLab.text = "x" + this.couponNum;
            this.couponBtn.on(Laya.Event.CLICK, this, this.reduceTimeLottery);

            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);

            //屏蔽
            // this.subscriptionBox = this.getChildByName("subscriptionBox");
            // this.subscriptionBox.setValue(Constants.WxMessageTmplIdKeys.NAUTICAL_START, Strings.get("tid#WXMessageSail"));
            this.nauticalCDBeginTime = Laya.LocalStorage.getJSON("nauticalCDBeginTimes");
            if (!this.nauticalCDBeginTime) {
                this.nauticalCDBeginTime = new Array(0, 0, 0);
            }

            this._nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            this.startTime = this._nauticalInfo.startSec * 1000;
            this.endTime = this._nauticalInfo.endSec * 1000 + 500;
            if (this._nauticalInfo.originEndSec != null) {
                this.originEndTime = this._nauticalInfo.originEndSec * 1000 + 500;
            } else {
                this.originEndTime = this.endTime;
            }
            this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
            SdkHelper.shanchuTuisong('yongshi_guihang_1');
            SdkHelper.goTuisong('yongshi_guihang_1', Strings$1.get('tid#WXMessage_SailName'), Strings$1.get('tid#WXMessage_SailDesc'), '' + Math.ceil(this.leftTime / 1000));
            if (this.leftTime < 0) {
                Laya.timer.once(50, this, function () {
                    this._nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
                    this.startTime = this._nauticalInfo.startSec * 1000;
                    this.endTime = this._nauticalInfo.endSec * 1000 + 500;
                    if (this._nauticalInfo.originEndSec != null) {
                        this.originEndTime = this._nauticalInfo.originEndSec * 1000 + 500;
                    } else {
                        this.originEndTime = this.endTime;
                    }
                    this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
                    this.showTriping();
                    this.showRemainingTime();
                });
            }
            else {
                this.showTriping();
                this.showRemainingTime();
            }

        }

        updateTime() {
            this._nauticalInfo = NauticalSystem$1.NauticalInfo[this.nauticalIndex];
            this.startTime = this._nauticalInfo.startSec * 1000;
            this.endTime = this._nauticalInfo.endSec * 1000 + 500;
            this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
            SdkHelper.shanchuTuisong('yongshi_guihang_1');
            SdkHelper.goTuisong('yongshi_guihang_1', Strings$1.get('tid#WXMessage_SailName'), Strings$1.get('tid#WXMessage_SailDesc'), '' + Math.ceil(this.leftTime / 1000));
        }

        _checkTime() {
            if (this._nauticalInfo.inTrip) {
                if (TimeUtils.getTimeNowOnServer() > this._nauticalInfo.endSec * 1000) {
                    this.close();
                } else {
                    Laya.timer.once(1000, this, this._checkTime, null, true);
                }
            } else {
                this.close();
            }
        }

        showTriping() {
            this._currentAction = null;
            this._currentPlayTimes = 1;
            if (this._param && this._param[0] == "chuhang") {
                this.chuhang();
                this.nauticalCDBeginTime[this.nauticalIndex] = 0;
                Laya.LocalStorage.setJSON("nauticalCDBeginTimes", this.nauticalCDBeginTime);
            } else {
                this.playAnim("daiji");
            }
            this.showBtns();
        }

        onDisable() {
            Laya.timer.clear(this, this._checkTime);
            this.leftTime = 0;
            Laya.Tween.clearAll(this);
            if (this.anim) {
                this.anim.stop();
            }
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
        }

        onCloseClick() {
            this.close();
        }

        showRemainingTime() {
            this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
            this.showTime(this.leftTime);
            Laya.timer.loop(100, this, function () {
                this.leftTime = this.endTime - TimeUtils.getTimeNowOnServer();
                if (this.leftTime <= 0) {
                    this.close();
                }
                else {
                    this.showTime(this.leftTime);
                    if (this.couponNum > 0) {
                        this.jiasu.visible = true;
                        this.couponBox.visible = true;
                        this.CD.visible = false;
                        return;
                    }
                    this.showCDBtn();
                }
            });
        }

        showTime(time) {
            let str = "";
            this.remainingTime = time / 1000 | 0;
            if (this.remainingTime > 0) {
                this.remainingTimeText.visible = true;
                this.time.visible = true;
                this.yinying.visible = true;
                let minute = this.remainingTime / 60 | 0;
                let second = this.remainingTime % 60;
                if (minute < 10) {
                    str += "0";
                }
                str += minute + ":";
                if (second < 10) {
                    str += "0";
                }
                str += second;
                let reTime = (this._nauticalInfo.originEndSec - this._nauticalInfo.endSec) * 1000;
                let rate = 1 - time / (this.endTime - this.startTime + reTime);
                this.time.value = rate;
            } else {
                this.remainingTimeText.visible = false;
                this.time.visible = false;
                this.yinying.visible = false;
                this.close();
            }
            if (str != "") {
                this.remainingTimeText.text = str;
            }
        }

        playAnim(action) {
            if (this.anim == null) {
                this.anim = new Laya.Skeleton();
                this.animPanel.getChildByName("animNode").addChild(this.anim);
                this.anim.load("anim3/matouui.sk", Laya.Handler.create(this, function () {
                    this.anim.play(action, false);
                    this.anim.on(Laya.Event.STOPPED, this, this.playCompleteCall);
                }));
            } else {
                this.anim.play(action, false);
            }
            this._currentAction = action;
        }

        playCompleteCall() {
            if (this._currentAction == "chuhang") {
                this._currentAction = "daiji";
                //SoundManager.getInstance().play("Sound_setSail");
            }
            this._currentPlayTimes--;
            if (this._currentPlayTimes > 0) {
                this.playAnim(this._currentAction);
            } else {
                this.randomEvent();
                this.randomPlayTimes();
                this.playAnim(this._currentAction);
            }
        }

        randomEvent() {
            if (this._currentAction == "daiji") {
                this._currentAction = "shijian" + Math.floor(Utils.randomExt2(1, 5));
            } else {
                this._currentAction = "daiji";
            }

        }

        randomPlayTimes() {
            if (this._currentAction == "daiji") {
                this._currentPlayTimes = Math.floor(Utils.randomExt2(3, 9));
            } else if (this._currentAction == "shijian3") {
                this._currentPlayTimes = 2;
            } else {
                this._currentPlayTimes = 1;
            }
        }

        chuhang() {
            this.playAnim("chuhang");
        }

        onVideoClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.Traveling);
        }

        onShareClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.Traveling);
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.Traveling) {
                EventMgr.getInstance().postEvent("SERVICE_AD_REDUCE_NAUTICAL_TIME", [this.nauticalIndex, 0, Laya.Handler.create(this, function (res) {
                    EventMgr.getInstance().postEvent("BOAT_AD_CHECK_TIME");
                    let str = "";
                    if (res.isDirectFinish) {
                        str = Strings$1.get("tid#TipsCommon17");
                    } else {
                        let time = (Number(Const.get("NauticalADReduceTime")) / 3600).toFixed(1);
                        str = Strings$1.get("tid#TipsCommon16", [String(time)]);
                    }
                    EventMgr.getInstance().postEvent("SHOW_TIPS", [str]);
                    this.setCDtime();
                    EventMgr.getInstance().postEvent("CHECK_NAUTICALREWARD_REDPOINT");
                })]);
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.Traveling) {
                EventMgr.getInstance().postEvent("SERVICE_AD_REDUCE_NAUTICAL_TIME", [this.nauticalIndex, 0, Laya.Handler.create(this, function (res) {
                    EventMgr.getInstance().postEvent("BOAT_AD_CHECK_TIME");
                    let str = "";
                    if (res.isDirectFinish) {
                        str = Strings$1.get("tid#TipsCommon17");
                    } else {
                        let time = (Number(Const.get("NauticalADReduceTime")) / 3600).toFixed(1);
                        str = Strings$1.get("tid#TipsCommon16", [String(time)]);
                    }
                    EventMgr.getInstance().postEvent("SHOW_TIPS", [str]);
                    this.setCDtime();
                    EventMgr.getInstance().postEvent("CHECK_NAUTICALREWARD_REDPOINT");
                })]);
            }
        }

        onCDClick() {
            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon56");
        }

        setCDtime() {
            let now = TimeUtils.getTimeNowOnServer();
            this.nauticalCDBeginTime[this.nauticalIndex] = now;
            Laya.LocalStorage.setJSON("nauticalCDBeginTimes", this.nauticalCDBeginTime);
            this.cdSign = 1;
            this.updateTime();
        }

        showCDBtn() {
            if (this.cdSign == 0) {
                return;
            }
            if (this.leftTime <= 0) {
                this.close();
                return;
            }
            let maxTimes = Number(Const.get("NauticalADTimes"));
            if (NauticalSystem$1.getAdReduceTimes(this.nauticalIndex) < maxTimes) {
                let now = TimeUtils.getTimeNowOnServer();
                let time = (this.nauticalCDBeginTime[this.nauticalIndex] - now) / 1000 + this.NauticalADTimesCD + 1 | 0;
                if (time > 0) {
                    this.CDtext.text = time + Strings$1.get("tid#UI_119");
                    this.CD.visible = true;
                    this.jiasu.visible = false;
                }
                else {
                    this.CD.visible = false;
                    this.jiasu.visible = true;
                    if (SdkHelper.hasVideo()) {
                        this.videBtn.visible = true;
                        this.shareBtn.visible = false;
                    } else {
                        this.videBtn.visible = false;
                        this.shareBtn.visible = true;
                    }
                    this.cdSign = 0;
                    Laya.timer.clear(this, this.showCDBtn);
                }
            }
            else {
                Laya.timer.clear(this, this.showCDBtn);
                this.CD.visible = false;
                this.jiasu.visible = false;
            }
        }

        reduceTimeLottery() {
            EventMgr.getInstance().postEvent("SERVICE_AD_REDUCE_NAUTICAL_TIME", [this.nauticalIndex, 1, Laya.Handler.create(this, function (res) {
                EventMgr.getInstance().postEvent("BOAT_AD_CHECK_TIME");
                this.couponNum = NauticalSystem$1.getNauticalTicketNum();
                this.couponNumLab.text = "x" + this.couponNum;
                let str = "";
                if (res.isDirectFinish) {
                    str = Strings$1.get("tid#TipsCommon17");
                } else {
                    let time = (Number(Const.get("NauticalADReduceTime")) / 3600).toFixed(1);
                    str = Strings$1.get("tid#TipsCommon16", [String(time)]);
                }
                EventMgr.getInstance().postEvent("SHOW_TIPS", [str]);
                this.updateTime();
                this.showBtns();
                EventMgr.getInstance().postEvent("CHECK_NAUTICALREWARD_REDPOINT");
            })]);
        }

        showBtns() {
            let now = TimeUtils.getTimeNowOnServer();
            let time = (this.nauticalCDBeginTime[this.nauticalIndex] - now) / 1000 + this.NauticalADTimesCD + 1 | 0;
            this.jiasu.visible = false;
            if (time <= 0) {
                if (this.couponNum > 0) {
                    this.jiasu.visible = true;
                    this.couponBox.visible = true;
                    return;
                }
                this.couponBox.visible = false;
                let maxTimes = Number(Const.get("NauticalADTimes"));
                if (NauticalSystem$1.getAdReduceTimes(this.nauticalIndex) < maxTimes) {
                    this.jiasu.visible = true;
                    if (SdkHelper.hasVideo()) {
                        this.videBtn.visible = true;
                        this.shareBtn.visible = false;
                    } else {
                        this.videBtn.visible = false;
                        this.shareBtn.visible = true;
                    }
                }
            }
        }
    }

    class Order extends Laya.Dialog {
        constructor() {
            super();
        }
        onAwake() {
            this.baseDeliveryBox = this.getChildByName("baseDeliveryBox");
            let closeBtn = this.baseDeliveryBox.getChildByName("close");
            closeBtn.on(Laya.Event.CLICK, this, this.close);
            this.tagBg = this.baseDeliveryBox.getChildByName("tagBg");
            this.tagLabel = this.baseDeliveryBox.getChildByName("tagLabel");
            this.tagLabel.text = Strings$1.get("tid#OrderSystemDesc3");
            this.bgimage = this.baseDeliveryBox.getChildByName("bgImage");


            this.tagImage = this.baseDeliveryBox.getChildByName("tagImage");
            // this.efficiencyNumLabel = this.baseDeliveryBox.getChildByName("efficiencyNumLabel");
            this.efficiencyLabel = this.baseDeliveryBox.getChildByName("efficiencyLabel");
            this.levelBtn = this.baseDeliveryBox.getChildByName("level");
            this.levelBtn.on(Laya.Event.CLICK, this, this.onLevelClick);
            let redPoint = this.levelBtn.getChildByName("redPoint");
            redPoint.setCheckFunc(TakeoutSystem.checkLevelUpRedPoint);
            redPoint.refreshRedPoint();
            this.deliveringAnim = new Spine();
            this.deliveringAnim.loadAnim("anim3/waimai.sk");
            this.deliveringAnim.playAnim("paizi");
            this.deliveringBox.addChild(this.deliveringAnim);
            this.carAnim = new Spine();
            this.carAnim.loadAnim("anim3/waimai.sk");
            this.carAnim.x = this.car.width / 2;
            this.carAnim.y = this.car.height;
            this.car.addChild(this.carAnim);
            this.deliveryBox = this.baseDeliveryBox.getChildByName("deliveryBox");
            this.deliveryList = this.deliveryBox.getChildByName("deliveryList");
            let orderLabel = this.deliveryBox.getChildByName("orderLabel");
            orderLabel.text = Strings$1.get("tid#OrderSystemDesc11");
            this.orderNumLabel = this.deliveryBox.getChildByName("orderNumLabel");
            this.orderMaxNumLabel = this.deliveryBox.getChildByName("orderMaxNumLabel");
            this.deliveryBtn = this.deliveryBox.getChildByName("deliveryBtn");
            this.deliveryBtn.on(Laya.Event.CLICK, this, this.onDeliveryBtnClick);
            this.orderTime = this.deliveryBox.getChildByName("orderTime");
            this.addOrder = this.deliveryBox.getChildByName("addOrder");
            this.addOrder.on(Laya.Event.CLICK, this, this.buyOrder);

            this.rewardLabel = this.baseDeliveryBox.getChildByName("rewardLabel");
            this.rewardLabel.text = Strings$1.get("tid#OrderSystemDesc5");
            this.rewardNum = this.baseDeliveryBox.getChildByName("rewardNum");
            this.fishImage = this.baseDeliveryBox.getChildByName("fishImage");
            this.rewardBg = this.baseDeliveryBox.getChildByName("rewardBg");
            this.tip = this.baseDeliveryBox.getChildByName("tip");
            this.tip.text = Strings$1.get("tid#OrderSystemDesc12");

            this.deliveringList = this.baseDeliveryBox.getChildByName("deliveringList");
            this.ingBox = this.baseDeliveryBox.getChildByName("ingBox");
            this.deliveryTime = this.ingBox.getChildByName("deliveryTime");
            this.speed = this.ingBox.getChildByName("speed");
            this.speed.on(Laya.Event.CLICK, this, this.onSpeedClick);
            this.freeSpeed = this.ingBox.getChildByName("freeSpeed");
            this.freeSpeed.on(Laya.Event.CLICK, this, this.onFreeSpeedClick);
            this.zuanshi = this.speed.getChildByName("zuanshi");
            this.zuanshi.text = Const.get("OrderSystemClearCDCost");
            this.zuanshiImage = this.speed.getChildByName("zuanshiImage");

            this.endBox = this.baseDeliveryBox.getChildByName("endBox");
            this.getReward = this.endBox.getChildByName("getReward");
            this.getReward.on(Laya.Event.CLICK, this, this.onGetRewardClick);

            //宣传
            this.promoterBox = this.getChildByName("promoterBox");
            this.promoter = this.promoterBox.getChildByName("promoter");
            this.orders = this.promoterBox.getChildByName("orders");
            this.promoterList = this.promoterBox.getChildByName("promoterList");
            this.promoterList.hScrollBarSkin = null;
            this.deliveryList.hScrollBarSkin = null;
            this.deliveringList.hScrollBarSkin = null;
        }
        onEnable() {
            this.guide();
            this.autoPush = this._param;
            EventMgr.getInstance().registEvent("UPDATA_TAKEOUT_INFO_UI", this, this.updateUI);
            this.updateUI(this.autoPush);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATA_TAKEOUT_INFO_UI", this, this.updateUI);
            Laya.timer.clearAll(this);
        }

        updateUI(autoPush) {
            this.levelBtn.label = Strings$1.get("tid#UI_136") + TakeoutSystem.getLevel();

            this.tagImage.skin = "collect/" + TakeoutSystem.getDeliveryTag() + ".png";
            this.efficiencyLabel.text = Strings$1.get("tid#OrderSystemDesc4") + "0%";
            switch (TakeoutSystem.getCurrentType()) {
                case 0: {
                    this.deliveryBox.visible = true;
                    this.ingBox.visible = false;
                    this.endBox.visible = false;
                    this.deliveringList.visible = false;
                    if (autoPush) {
                        TakeoutSystem.pushDeliveryRecommendCat();
                    }
                    this.efficiencyLabel.text = Strings$1.get("tid#OrderSystemDesc4") + Math.floor(TakeoutSystem.getLocalEfficiencyNum() * 100) + "%";
                    let localPets = TakeoutSystem.getDeliveryLocalPets();
                    let pets = [];
                    pets = pets.concat(localPets);
                    // for (let index = pets.length; index < TakeoutSystem.getMaxCourierStaff(); index++) {
                    //     pets.push(null);
                    // }
                    this.deliveryList.array = pets;
                    let max = TakeoutSystem.getMaxCourierStaff();
                    this.deliveryList.x = (this.deliveryBox.width - 280 * max - this.deliveryList.spaceX * (max - 1)) / 2;
                    this.startUpdateOrderTime();
                    this.rewardNum.text = TakeoutSystem.getLoclTrueReward();
                    break
                }
                case 1: {
                    this.freeSpeed.visible = GuideSystem.isGuiding();
                    this.speed.visible = !GuideSystem.isGuiding();
                    this.deliveryBox.visible = false;
                    this.ingBox.visible = true;
                    this.endBox.visible = false;
                    this.deliveringList.visible = true;
                    let pets = TakeoutSystem.getDeliveryPets();
                    this.deliveringList.array = pets;
                    this.efficiencyLabel.text = Strings$1.get("tid#OrderSystemDesc4") + Math.floor(TakeoutSystem.getEfficiencyNum() * 100) + "%";
                    this.startUpdateDeliveryTime();
                    let max = TakeoutSystem.getDeliveryPets().length;
                    this.deliveringList.x = (this.baseDeliveryBox.width - 154 * max - this.deliveringList.spaceX * (max - 1)) / 2;
                    this.rewardNum.text = TakeoutSystem.getDeliveryRewardAmount();
                    break
                }
                case 2: {
                    this.deliveryBox.visible = false;
                    this.ingBox.visible = false;
                    this.endBox.visible = true;
                    this.deliveringList.visible = true;
                    let pets = TakeoutSystem.getDeliveryPets();
                    this.deliveringList.array = pets;
                    this.efficiencyLabel.text = Strings$1.get("tid#OrderSystemDesc4") + Math.floor(TakeoutSystem.getEfficiencyNum() * 100) + "%";
                    let max = TakeoutSystem.getDeliveryPets().length;
                    this.deliveringList.x = (this.baseDeliveryBox.width - 154 * max - this.deliveringList.spaceX * (max - 1)) / 2;
                    this.rewardNum.text = TakeoutSystem.getDeliveryRewardAmount();
                    break
                }
            }
            let str = "daiji" + TakeoutSystem.getStaffCar();
            this.carAnim.playAnim(str);
            this.car.visible = !this.ingBox.visible;
            this.rewardLabel.x = (this.baseDeliveryBox.width - this.rewardLabel.width - this.fishImage.width * this.fishImage.scaleX - this.rewardNum.width) / 2;
            this.fishImage.x = this.rewardLabel.x + this.rewardLabel.width;
            this.rewardNum.x = this.fishImage.x + this.fishImage.width * this.fishImage.scaleX;
            this.rewardBg.width = (80 + this.rewardLabel.width + this.fishImage.width * this.fishImage.scaleX + this.rewardNum.width) / this.rewardBg.scaleX;
            if (TakeoutSystem.getMaxAdvertisingStaff() <= 0) {
                this.bgimage.height = 732;
                this.baseDeliveryBox.y = 411;
                this.tip.visible = true;
                this.promoterBox.visible = false;
            } else {
                this.bgimage.height = 705;
                this.baseDeliveryBox.y = 272;
                this.tip.visible = false;
                this.promoterBox.visible = true;

                //宣传
                this.updatePromoter();
            }

        }

        onLevelClick() {
            if (TakeoutSystem.getNextLevelConfig() == null) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#UI_214");
                return;
            }
            Laya.Dialog.open("ui/OrderLevelUp.scene", false);
        }

        onSpeedClick() {
            if (PlayerSystem.getDiamond() < Number(Const.get("OrderSystemClearCDCost"))) {
                // EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#ConsumeUI_8");
                Laya.Dialog.open("ui/OrderBuy.scene", false, [0]);
                return;
            }
            EventMgr.getInstance().postEvent("SERVICE_SPEED_UP_DELIVERY", [false]);
        }

        onFreeSpeedClick() {
            EventMgr.getInstance().postEvent("SERVICE_SPEED_UP_DELIVERY", [true]);
        }

        onGetRewardClick() {
            let point = new Laya.Point(this.fishImage.x + this.fishImage.width * this.fishImage.scaleX / 2, this.fishImage.y + this.fishImage.height * this.fishImage.scaleY / 2);
            point = this.baseDeliveryBox.localToGlobal(point);
            EventMgr.getInstance().postEvent("SERVICE_ORDER_RECEIVE_REWARDS", Laya.Handler.create(this, function (res) {
                let reward = res.rewards.rewards[0];
                ItemFlyMgr.getInstance().flyReward(reward.id, reward.amount, Constants.FlyType.TYPE3, point.x, point.y);
            }));
        }

        startUpdateOrderTime() {
            Laya.timer.clear(this, this.updateOrderTime);
            if (TakeoutSystem.getOrderUpdaeTimeLeft_S() != null) {
                Laya.timer.loop(1000, this, this.updateOrderTime);
            }
            this.updateOrderTime();
        }

        updateOrderTime() {
            let t = TakeoutSystem.getOrderUpdaeTimeLeft_S();
            this.orderNumLabel.text = TakeoutSystem.getOrderNum();
            this.orderMaxNumLabel.color = this.orderNumLabel.color = TakeoutSystem.getOrderNum() == 0 ? "#e93636" : "#72584a";
            if (t && TakeoutSystem.getCurrentType() == 0) {
                this.orderTime.text = Strings$1.get("tid#OrderSystemDesc15") + this.addZero(Math.floor(t / 60), 2) + ":" + this.addZero(Math.ceil(t % 60), 2);
                this.orderMaxNumLabel.text = "/" + TakeoutSystem.getMaxOrderNum();
                this.addOrder.x = this.orderMaxNumLabel.x + this.orderMaxNumLabel.width + this.addOrder.width * this.addOrder.scaleX * this.addOrder.anchorX + 10;
                this.addOrder.visible = true;
            } else {
                this.orderTime.text = "";
                this.orderMaxNumLabel.text = "/" + TakeoutSystem.getMaxOrderNum();
                Laya.timer.clear(this, this.updateOrderTime);
                this.addOrder.visible = false;
            }
        }

        startUpdateDeliveryTime() {
            Laya.timer.clear(this, this.updateDeliveryTime);
            if (TakeoutSystem.getDeliveryTimeLeft_S() != null) {
                Laya.timer.loop(1000, this, this.updateDeliveryTime);
            }
            this.updateDeliveryTime();
        }

        updateDeliveryTime() {
            let t = TakeoutSystem.getDeliveryTimeLeft_S();
            if (t && TakeoutSystem.getCurrentType() == 1) {
                this.deliveryTime.visible = true;
                this.deliveryTime.text = this.addZero(Math.floor(t / 60), 2) + ":" + this.addZero(Math.ceil(t % 60), 2);
            } else {
                this.deliveryTime.visible = false;
                Laya.timer.clear(this, this.updateDeliveryTime);
            }
        }

        updatePromoter() {
            let pets = TakeoutSystem.getLoaclPublicityPets();
            this.promoter.text = Strings$1.get("tid#OrderSystemDesc7") + TakeoutSystem.getPublicityPets().length + "/" + TakeoutSystem.getMaxAdvertisingStaff();
            this.orders.text = Strings$1.get("tid#OrderSystemDesc8") + TakeoutSystem.getOrderSpeedUp() + "%";
            this.promoterList.array = pets;
        }

        onDeliveryBtnClick() {
            if (TakeoutSystem.getOrderNum() <= 0) {
                if (TakeoutSystem.getBoughtTimes() >= TakeoutSystem.getMaxBuyNum()) {
                    EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#OrderSystemDesc18");
                    return;
                }
                this.buyOrder();
                return;
            }
            let free = GuideSystem.isGuiding();
            let sucess = TakeoutSystem.startDelivery2Service(free);
            if (sucess) {
                Laya.Dialog.close("ui/OrderAssignDialog.scene");
                Laya.Dialog.open("ui/OrderAssignDialog.scene", false);
            }
        }

        addZero(num, length) {
            for (var len = (num + "").length; len < length; len = num.length) {
                num = "0" + num;
            }
            return num;
        }

        guide() {
            let guide = GuideSystem.getGuide("GuideGroup_19");
            if (guide) {
                this.freeSpeed.visible = true;
                this.speed.visible = false;
                guide.registerObj("btn0", this);
                guide.registerClick("btn0", this.deliveryList);
                guide.registerObj("btn1", this);
                guide.registerClick("btn1", this.tagBg, null, [this.tagBg, this.tagLabel, this.tagImage, this.efficiencyLabel]);
                guide.registerObj("btn2", this);
                guide.registerClick("btn2", this.deliveryBtn);
                guide.registerObj("btn3", this);
                guide.registerClick("btn3", this.freeSpeed);
                guide.registerObj("btn4", this);
                guide.registerClick("btn4", this.freeSpeed);
                guide.registerObj("btn5", this);
                guide.registerClick("btn5", this.getReward);
                guide.registerObj("btn6", this);
                guide.registerClick("btn6", this.levelBtn);
                guide.start();
            }
        }

        buyOrder() {
            if (TakeoutSystem.getBoughtTimes() >= TakeoutSystem.getMaxBuyNum()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#OrderSystemDesc32");
                return;
            }
            Laya.Dialog.open("ui/OrderBuy.scene", false);
        }
    }

    class OrderDeliveryCatBox extends Laya.Box {
        constructor() {
            super();
        }
        onAwake() {
            let baseBox = this.getChildByName("baseBox");
            this.catBox = baseBox.getChildByName("catBox");
            this.label1 = this.catBox.getChildByName("tag1");
            this.label2 = this.catBox.getChildByName("tag2");
            this.icon = this.catBox.getChildByName("cat");
            this.heart1 = this.catBox.getChildByName("h1");
            this.heart2 = this.catBox.getChildByName("h2");
            this.heart3 = this.catBox.getChildByName("h3");
            this.tuijian = this.catBox.getChildByName("tuijian");
            this.tuijianAnim = new Spine();
            this.tuijianAnim.loadAnim("anim3/waimai.sk");
            this.tuijianAnim.x = this.tuijian.width / 2;
            this.tuijianAnim.y = this.tuijian.height / 2 + 40;
            this.tuijian.addChild(this.tuijianAnim);
            // this.tuijianAnim.scale(0.83,0.83);
            this.tuijianAnim.playAnim("tuijian");
            this.on(Laya.Event.CLICK, this, this.onClick);
            this.on(Laya.Event.MOUSE_DOWN, this, function () {
                baseBox.scaleX = baseBox.scaleY = 0.9;
            });
            this.on(Laya.Event.MOUSE_UP, this, function () {
                baseBox.scaleX = baseBox.scaleY = 1;
            });
            this.on(Laya.Event.MOUSE_OUT, this, function () {
                baseBox.scaleX = baseBox.scaleY = 1;
            });
        }
        setItem(data) {
            this.petId = data.petId;
            this.lock = data.lock;
            this.itemIndex = data.Id;
            if (this.lock) {
                this.visible = false;
                return;
            }
            if (this.petId == null) {
                this.catBox.visible = false;
                return;
            }
            this.catBox.visible = true;
            let pet = PetSystem.getPetInfo(this.petId);
            let tags = pet.getTag();
            this.label1.skin = "collect/" + tags[0] + ".png";
            this.label2.skin = "collect/" + tags[1] + ".png";
            this.icon.skin = pet.getHeadIcon();
            let level = pet.getIntimacyGrade();
            this.heart1.skin = level >= 1 ? "collect/redheart.png" : "collect/grayheart.png";
            this.heart2.skin = level >= 2 ? "collect/redheart.png" : "collect/grayheart.png";
            this.heart3.skin = level >= 3 ? "collect/redheart.png" : "collect/grayheart.png";
            this.tuijian.visible = TakeoutSystem.checkIfDeliveryRecommendCat(this.petId);
        }

        onClick() {
            if (!this.lock) {
                Laya.Dialog.open("ui/OrderChoose.scene", false, [true, this.petId, this.itemIndex]);
            }
        }
    }

    class OrderDeliveringCatBox extends Laya.Box {
        constructor() {
            super();
        }
        setItem(petId) {
            // this.petId = data.petId;
            // let lock = data.lock;
            // if (lock || this.petId == null) {
            //     this.visible = false;
            //     return;
            // }
            let pet = PetSystem.getPetInfo(petId);
            let icon = this.getChildByName("cat");
            icon.skin = pet.getHeadIcon();
        }

    }

    class OrderPromoterBox extends Laya.Box {
        constructor() {
            super();
        }
        onAwake() {
            this.on(Laya.Event.CLICK, this, this.onClick);
            this.heart1 = this.getChildByName("h1");
            this.heart2 = this.getChildByName("h2");
            this.heart3 = this.getChildByName("h3");
            this.change = this.getChildByName("change");
        }
        setItem(data) {
            this.petId = data.petId;
            this.lock = data.lock;
            this.itemIndex = data.Id;
            let icon = this.getChildByName("cat");
            let lockImage = this.getChildByName("lock");
            lockImage.visible = this.lock;
            if (this.petId == null || this.lock) {
                icon.visible = false;
                this.change.visible = false;
                this.heart1.visible = this.heart2.visible = this.heart3.visible = false;
                return;
            }
            this.change.visible = true;
            let pet = PetSystem.getPetInfo(this.petId);
            icon.skin = pet.getHeadIcon();
            icon.visible = true;
            let level = pet.getIntimacyGrade();
            this.heart1.visible = this.heart2.visible = this.heart3.visible = true;
            this.heart1.skin = level >= 1 ? "collect/redheart.png" : "collect/grayheart.png";
            this.heart2.skin = level >= 2 ? "collect/redheart.png" : "collect/grayheart.png";
            this.heart3.skin = level >= 3 ? "collect/redheart.png" : "collect/grayheart.png";
        }

        onClick() {
            if (!this.lock) {
                Laya.Dialog.open("ui/OrderChoose.scene", false, [false, this.petId, this.itemIndex]);
            } else {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#OrderSystemDesc17"]);
            }
        }
    }

    class OrderAssignDialog extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable(){
            let anim = new Spine();
            anim.loadAnim("anim3/waimai.sk");
            anim.playAnim("chufa",false);
            anim.x = this.animBox.width / 2;
            anim.y = this.animBox.height / 2;
            this.animBox.addChild(anim);
            anim.addCompleteHandler(Laya.Handler.create(this,function(){
                this.close();
            }));
        }

    }

    class OrderBuy extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable() {
            if (this._param == null) {
                this.toShopBox.visible = false;
                this.buyBox.visible = true;
            } else {
                this.toShopBox.visible = true;
                this.buyBox.visible = false;
                if(this._param[0] == 1){
                    this.tipsBtn.visible = true;
                    this.toShopBtn.visible = false;
                }
                else {
                    this.tipsBtn.visible = false;
                    this.toShopBtn.visible = true;
                }
            }
            this.cost = TakeoutSystem.getCurrentBuyOrderCost();
            this.toshopText.text = Strings$1.get("tid#OrderSystemDesc29");
            this.toShopBtn.on(Laya.Event.CLICK, this, this.toShop);
            this.tipsBtn.on(Laya.Event.CLICK, this, this.close);

            this.text1.text = Strings$1.get("tid#OrderSystemDesc26");
            this.text2.text = Strings$1.get("tid#OrderSystemDesc27");
            this.diamondNum.text = String(this.cost);
            this.diamondNum.color = PlayerSystem.getDiamond() < this.cost ? "#ff0000" : "#72584a";
            this.buyBtn.on(Laya.Event.CLICK, this, this.onBuyBtnClick);
            this.buyBtn.mouseEnabled = true;
            this.orderNuim.text = (TakeoutSystem.getMaxBuyNum() - TakeoutSystem.getBoughtTimes()) + "/" + TakeoutSystem.getMaxBuyNum();
            this.tips.text = Strings$1.get("tid#OrderSystemDesc28");
            this.setPos();

            this.closeBtn.on(Laya.Event.CLICK, this, this.close);
        }

        setPos() {
            let space = 10;
            let w1 = this.text1.width;
            let w2 = this.diamond.width * this.diamond.scaleX;
            let w3 = this.text2.width;
            let w4 = this.diamondNum.width;
            let allWidth = w1 + w2 + w3 + w4 + space * 3;
            let startX = (this.buyBox.width - allWidth) / 2;
            this.text1.x = startX;
            this.diamond.x = this.text1.x + w1 + space;
            this.diamondNum.x = this.diamond.x + w2 + space;
            this.text2.x = this.diamondNum.x + w4 + space;
        }

        onBuyBtnClick() {
            if (PlayerSystem.getDiamond() < this.cost) {
                this.toShopBox.visible = true;
                this.buyBox.visible = false;
                return;
            }
            if (TakeoutSystem.getBoughtTimes() >= TakeoutSystem.getMaxBuyNum()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#OrderSystemDesc32");
                return;
            }
            if (TakeoutSystem.getOrderNum() >= TakeoutSystem.getMaxOrderNum()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#OrderSystemDesc33");
                this.close();
                return;
            }
            this.buyBtn.mouseEnabled = false;
            EventMgr.getInstance().postEvent("SERVICE_BUY_ORDER", Laya.Handler.create(this, function () {
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.GOUMAI);
                if (this != null && !this.destroyed) {
                    this.close();
                }
            }));
        }

        // 点击前往则跳转商城（默认钻石页签）
        toShop() {
            this.close();
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene", true, [1]);
        }

    }

    class OrderChoose extends Laya.Dialog {
        constructor() {
            super();
        }
        onAwake() {
            this.closeBtn.on(Laya.Event.CLICK, this, this.closeThis);
        }
        onEnable() {
            EventMgr.getInstance().registEvent("ORDER_REMOVE_CAT", this, this.removeCat);
            EventMgr.getInstance().registEvent("ORDER_USE_CAT", this, this.useCat);
            this.isDelivery = this._param[0];
            let petId = this._param[1];
            this.curChooseIndex = this._param[2];
            if (this.isDelivery) {
                this.title.text = Strings$1.get("tid#OrderSystemDesc10");
                this.tips.text = Strings$1.get("tid#OrderSystemDesc19");
            } else {
                this.title.text = Strings$1.get("tid#OrderSystemDesc9");
                this.tips.text = Strings$1.get("tid#OrderSystemDesc20");
            }
            let obj = TakeoutSystem.getAllCanUsePets(this.isDelivery, petId);
            let pets = obj.pets;
            GameContext.curRecommendOrderCatIds = obj.recommendIds;
            GameContext.curChooseOrderCatId = petId;
            GameContext.orderChooseIsDelivery = this.isDelivery;
            this.list.array = pets;
            this.promotorRemovePetId = null;
        }
        onDisable() {
            GameContext.curChooseOrderCatId = null;
            GameContext.curRecommendOrderCatIds = null;
            GameContext.orderChooseIsDelivery = null;
            this.promotorRemovePetId = null;
            EventMgr.getInstance().removeEvent("ORDER_REMOVE_CAT", this, this.removeCat);
            EventMgr.getInstance().removeEvent("ORDER_USE_CAT", this, this.useCat);
        }

        removeCat(petId) {
            GameContext.curChooseOrderCatId = null;
            if (this.isDelivery) {
                TakeoutSystem.removeDeliveryLocalPet(petId);
                EventMgr.getInstance().postEvent("UPDATA_TAKEOUT_INFO_UI");
            } else {
                this.promotorRemovePetId = petId;
            }
        }

        useCat(petId) {
            if (this.isDelivery) {
                TakeoutSystem.addDeliveryLoclPet(petId, this.curChooseIndex);
                EventMgr.getInstance().postEvent("UPDATA_TAKEOUT_INFO_UI");
            } else {
                if (this.promotorRemovePetId == petId) {
                    this.promotorRemovePetId = null;
                }
                TakeoutSystem.putAdvertiserCats2Service(petId, this.curChooseIndex);
                this.promotorRemovePetId = null;
            }
            this.close();
        }

        closeThis() {
            if (!this.isDeliver && this.promotorRemovePetId) {
                TakeoutSystem.removeAdvertiserCats2Service(this.promotorRemovePetId);
            }
            this.close();
            // Laya.Dialog.open("ui/Order.scene");
        }
    }

    class OrderChooseItem extends Laya.Box {
        constructor() {
            super();
        }
        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onThisClick);
            this.light = this.getChildByName("light");
            let btn = this.light.getChildByName("btn");
            btn.on(Laya.Event.CLICK, this, this.onBtnClick);
            this.tuijian = this.getChildByName("tuijian");
            this.tuijianAnim = new Spine();
            this.tuijianAnim.loadAnim("anim3/waimai.sk");
            this.tuijianAnim.x = this.tuijian.width / 2;
            this.tuijianAnim.y = this.tuijian.height / 2 + 35;
            this.tuijian.addChild(this.tuijianAnim);
            this.tuijianAnim.scale(0.83, 0.83);
            this.tuijianAnim.playAnim("tuijian");
        }
        setItem(petId) {
            let pet = PetSystem.getPetInfo(petId);
            this.petId = petId;
            let label1 = this.getChildByName("label1");
            let label2 = this.getChildByName("label2");
            let tags = pet.getTag();
            label1.skin = "collect/" + tags[0] + ".png";
            label2.skin = "collect/" + tags[1] + ".png";
            let icon = this.getChildByName("icon");
            icon.skin = pet.getHeadIcon();
            this.tuijian.visible = GameContext.curRecommendOrderCatIds.indexOf(this.petId) != -1;
            let heart1 = this.getChildByName("heart1");
            let heart2 = this.getChildByName("heart2");
            let heart3 = this.getChildByName("heart3");
            this.light.visible = this.petId == GameContext.curChooseOrderCatId;
            let level = pet.getIntimacyGrade();
            heart1.skin = level >= 1 ? "collect/redheart.png" : "collect/grayheart.png";
            heart2.skin = level >= 2 ? "collect/redheart.png" : "collect/grayheart.png";
            heart3.skin = level >= 3 ? "collect/redheart.png" : "collect/grayheart.png";
            label1.visible = label2.visible = GameContext.orderChooseIsDelivery;
        }

        onBtnClick() {
            this.light.visible = false;
            EventMgr.getInstance().postEvent("ORDER_REMOVE_CAT", this.petId);
        }

        onThisClick() {
            if (this.petId != GameContext.curChooseOrderCatId) {
                this.light.visible = true;
                EventMgr.getInstance().postEvent("ORDER_USE_CAT", this.petId);
            }
        }
    }

    class OrderLevelUp extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
        }

        onEnable() {
            this.levelUp.on(Laya.Event.CLICK, this, this.onLevelUpClick);
            this.IntimacyLabel.text = Strings$1.get("tid#OrderSystemDesc1");
            this.updateDesc();
        }

        updateDesc() {
            let level = TakeoutSystem.getLevel();
            this.curConfig = TakeoutSystem.getConfig();
            this.title1.text = Strings$1.get("tid#OrderSystemLvUpDesc1") + this.curConfig.RewardBase;
            this.Rider1.text = Strings$1.get("tid#OrderSystemLvUpDesc2", [this.curConfig.Order]);
            this.orderNum1.text = Strings$1.get("tid#OrderSystemLvUpDesc3", [this.curConfig.CourierStaff]);
            this.propaganda1.text = this.curConfig.AdvertisingStaff > 0 ? Strings$1.get("tid#OrderSystemLvUpDesc4", [this.curConfig.AdvertisingStaff]) : Strings$1.get("tid#OrderSystemLvUpDesc5");
            this.costTime1.text = Strings$1.get("tid#OrderSystemLvUpDesc6") + ":" + this.curConfig.SpendTime + "s";

            this.nextConfig = TakeoutSystem.getNextLevelConfig();
            this.title.visible = false;
            this.nowLevel.visible = false;
            this.nextLevel.visible = false;

            if(this.nextConfig != null){
                this.to.visible = true;
                this.nowLevel.visible = true;
                this.nextLevel.visible = true;
                this.maxTip.visible = false;
                this.nowLevel.text = Strings$1.get("tid#UI_136") + level;
                this.nextLevel.text = Strings$1.get("tid#UI_136") + (level + 1);
                // if (this.nextConfig == null) {
                //     this.levelUp.visible = false;
                // }
                this.title2.text = Strings$1.get("tid#OrderSystemLvUpDesc1") + this.nextConfig.RewardBase;
                this.Rider2.text = Strings$1.get("tid#OrderSystemLvUpDesc2", [this.nextConfig.Order]);
                this.orderNum2.text = Strings$1.get("tid#OrderSystemLvUpDesc3", [this.nextConfig.CourierStaff]);
                this.propaganda2.text = this.nextConfig.AdvertisingStaff > 0 ? Strings$1.get("tid#OrderSystemLvUpDesc4", [this.nextConfig.AdvertisingStaff]) : Strings$1.get("tid#OrderSystemLvUpDesc5");
                this.costTime2.text = Strings$1.get("tid#OrderSystemLvUpDesc6") + ":" + this.nextConfig.SpendTime + "s";
                this.IntimacyLevel.text = PetSystem.PetsAdopted.length + "/" + this.curConfig.LvUpLimit;
                this.IntimacyLabel.x = (this.width - this.IntimacyLevel.width - this.IntimacyLabel.width) / 2;
                this.IntimacyLevel.x = this.IntimacyLabel.x + this.IntimacyLabel.width;
                this.IntimacyLevel.color = PetSystem.PetsAdopted.length < this.curConfig.LvUpLimit ? "#ff0000" : "#67544b";
                this.levelUpLabel.text = this.curConfig.LvUpCost + Strings$1.get("tid#OrderSystemDesc2");
                this.IntimacyLevel.visible = true;
                this.IntimacyLabel.visible = true;
                this.levelUp.visible = true;
            }
            else {
                this.to.visible = false;
                this.title.visible = true;
                this.maxTip.visible = true;
                this.title.text = Strings$1.get("tid#UI_136") + level;
                this.title2.text = Strings$1.get("tid#OrderSystemDesc21") ;
                this.Rider2.text = Strings$1.get("tid#OrderSystemDesc22") ;
                this.orderNum2.text = Strings$1.get("tid#OrderSystemDesc23") ;
                this.propaganda2.text = Strings$1.get("tid#OrderSystemDesc24") ;
                this.costTime2.text = Strings$1.get("tid#OrderSystemDesc25") ;
                this.IntimacyLevel.visible = false;
                this.IntimacyLabel.visible = false;
                this.levelUp.visible = false;
            }
            
        }

        onLevelUpClick() {
            if (PetSystem.PetsAdopted.length < this.curConfig.LvUpLimit) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#OrderSystemDesc14", [this.curConfig.LvUpLimit]]);
                return;
            }
            if (this.curConfig.LvUpCost > PlayerSystem.getFish()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon34");
                return;
            }
            EventMgr.getInstance().postEvent("SERVICE_LEVEL_UP_TAKEOUT", Laya.Handler.create(this, function () {
                // Laya.Dialog.open("ui/Order.scene");
                this.updateDesc();
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.LEVELUP);
            }));
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/Order.scene");
        }

    }

    class PayPop extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            this.type = this._param[0];
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.closePanel.on(Laya.Event.CLICK,this,this.close);
            this.Title.text = Strings$1.get("tid#ConsumeUI_24");
            switch(this.type){
                case 1:{
                    this.detail.text = Strings$1.get("tid#ConsumeUI_25");
                    this.icon.skin = "purchase/voucher_fish1.png";
                    break;
                }
                case 2:{
                    this.detail.text = Strings$1.get("tid#ConsumeUI_26");
                    this.icon.skin = "purchase/voucher_speed1.png";
                    break;
                }
                case 3:{
                    this.detail.text = Strings$1.get("tid#ConsumeUI_27");
                    this.icon.skin = "purchase/voucher_catfood1.png";
                    break;
                }
                case 4:{
                    this.detail.text = Strings$1.get("tid#ConsumeUI_28");
                    this.icon.skin = "purchase/kitty1.png";
                    break;
                }
            }
            console.log(this.type);
        }
    }

    class KanBanNiangDataMgr{
        constructor() {
            this.beginLoad();
        }

        beginLoad(){
            this.kbnBuilding_Standby=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_Standby");
            this.kbnBuilding_Start=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_Start");
            this.kbnBuilding_Angry=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_Angry");
            this.kbnBuilding_CoffeeAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_CoffeeAuto");
            this.kbnBuilding_CoffeeClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_CoffeeClick");
            this.kbnBuilding_BeachAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_BeachAuto");
            this.kbnBuilding_BeachClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_BeachClick");
            this.kbnBuilding_WharfAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_WharfAuto");
            this.kbnBuilding_WharfClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Building_WharfClick");
            this.kbnGoods_Standby=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_Standby");
            this.kbnGoods_Start=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_Start");
            this.kbnGoods_Angry=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_Angry");
            this.kbnGoods_FoodsAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_FoodsAuto");
            this.kbnGoods_FoodsClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_FoodsClick");
            this.kbnGoods_NauticalFoodsAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_NauticalFoodsAuto");
            this.kbnGoods_NauticalFoodsClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_NauticalFoodsClick");
            this.kbnGoods_NauticalToolsAuto=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_NauticalToolsAuto");
            this.kbnGoods_NauticalToolsClick=ConfigMgr$1.getInstance().getItemById("Kanban", "Goods_NauticalToolsClick");

            this.kbnConsume_Standby=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_Standby");
            this.kbnConsume_Angry=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_Angry");
            this.kbnConsume_PackStart=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_PackStart");
            this.kbnConsume_CurrencyStart=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_CurrencyStart");
            this.kbnConsume_Auto=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_Auto");
            this.kbnConsume_Click=ConfigMgr$1.getInstance().getItemById("Kanban", "Consume_Click");

            this.KanbanAutoTime = Const.get("KanbanAutoTime");
            this.KanbanContentTime = Const.get("KanbanContentTime");
            this.KanbanAngryConfig = JSON.parse(Const.get("KanbanAngryConfig"));
        }

        static getInstance() {
            return KanBanNiangDataMgr.instance ? KanBanNiangDataMgr.instance : KanBanNiangDataMgr.instance = new KanBanNiangDataMgr();
        }

        getStr0(){
            let r=Math.random()*this.kbnBuilding_Start.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_Start.Dialog[r]);
        }

        getAutoStr1(){
            let r=Math.random()*this.kbnBuilding_CoffeeAuto.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_CoffeeAuto.Dialog[r]);
        }

        getClickStr1(){
            let r=Math.random()*this.kbnBuilding_CoffeeClick.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_CoffeeClick.Dialog[r]);
        }

        getAutoStr2(){
            let r=Math.random()*this.kbnBuilding_BeachAuto.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_BeachAuto.Dialog[r]);
        }

        getClickStr2(){
            let r=Math.random()*this.kbnBuilding_BeachClick.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_BeachClick.Dialog[r]);
        }

        getAutoStr3(){
            let r=Math.random()*this.kbnBuilding_WharfAuto.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_WharfAuto.Dialog[r]);
        }

        getClickStr3(){
            let r=Math.random()*this.kbnBuilding_WharfClick.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_WharfClick.Dialog[r]);
        }

        getAutoStr4(){
            let r=Math.random()*this.kbnGoods_FoodsAuto.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_FoodsAuto.Dialog[r]);
        }

        getClickStr4(){
            let r=Math.random()*this.kbnGoods_FoodsClick.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_FoodsClick.Dialog[r]);
        }

        getAutoStr5(){
            let r=Math.random()*this.kbnGoods_NauticalFoodsAuto.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_NauticalFoodsAuto.Dialog[r]);
        }

        getClickStr5(){
            let r=Math.random()*this.kbnGoods_NauticalFoodsClick.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_NauticalFoodsClick.Dialog[r]);
        }

        getAutoStr6(){
            let r=Math.random()*this.kbnGoods_NauticalToolsAuto.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_NauticalToolsAuto.Dialog[r]);
        }

        getClickStr6(){
            let r=Math.random()*this.kbnGoods_NauticalToolsClick.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_NauticalToolsClick.Dialog[r]);
        }

        getAngryStr1(){
            let r=Math.random()*this.kbnBuilding_Angry.Dialog.length|0;
            return Strings$1.get(this.kbnBuilding_Angry.Dialog[r]);
        }

        getAngryStr2(){
            let r=Math.random()*this.kbnGoods_Angry.Dialog.length|0;
            return Strings$1.get(this.kbnGoods_Angry.Dialog[r]);
        }

        getAction0(){
            let r=Math.random()*this.kbnBuilding_Start.Action.length|0;
            return this.kbnBuilding_Start.Action[r];
        }

        getAutoAction1(){
            let r=Math.random()*this.kbnBuilding_CoffeeAuto.Action.length|0;
            return this.kbnBuilding_CoffeeAuto.Action[r];
        }

        getClickAction1(){
            let r=Math.random()*this.kbnBuilding_CoffeeClick.Action.length|0;
            return this.kbnBuilding_CoffeeClick.Action[r];
        }

        getAutoAction2(){
            let r=Math.random()*this.kbnBuilding_BeachAuto.Action.length|0;
            return this.kbnBuilding_BeachAuto.Action[r];
        }

        getClickAction2(){
            let r=Math.random()*this.kbnBuilding_BeachClick.Action.length|0;
            return this.kbnBuilding_BeachClick.Action[r];
        }

        getAutoAction3(){
            let r=Math.random()*this.kbnBuilding_WharfAuto.Action.length|0;
            return this.kbnBuilding_WharfAuto.Action[r];
        }

        getClickAction3(){
            let r=Math.random()*this.kbnBuilding_WharfClick.Action.length|0;
            return this.kbnBuilding_WharfClick.Action[r];
        }

        getAutoAction4(){
            let r=Math.random()*this.kbnGoods_FoodsAuto.Action.length|0;
            return this.kbnGoods_FoodsAuto.Action[r];
        }

        getClickAction4(){
            let r=Math.random()*this.kbnGoods_FoodsClick.Action.length|0;
            return this.kbnGoods_FoodsClick.Action[r];
        }

        getAutoAction5(){
            let r=Math.random()*this.kbnGoods_NauticalFoodsAuto.Action.length|0;
            return this.kbnGoods_NauticalFoodsAuto.Action[r];
        }

        getClickAction5(){
            let r=Math.random()*this.kbnGoods_NauticalFoodsClick.Action.length|0;
            return this.kbnGoods_NauticalFoodsClick.Action[r];
        }

        getAutoAction6(){
            let r=Math.random()*this.kbnGoods_NauticalToolsAuto.Action.length|0;
            return this.kbnGoods_NauticalToolsAuto.Action[r];
        }

        getClickAction6(){
            let r=Math.random()*this.kbnGoods_NauticalToolsClick.Action.length|0;
            return this.kbnGoods_NauticalToolsClick.Action[r];
        }

        getAngryActionL1(){
            return this.kbnBuilding_Angry.Action[0];
        }

        // getAngryActionL2(){
        //     return this.kbnBuilding_Angry.Action[1];
        // }

        getAngryActionR1(){
            return this.kbnGoods_Angry.Action[0];
        }

        // getAngryActionR2(){
        //     return this.kbnGoods_Angry.Action[1];
        // }

        getCommonL(){
        return this.kbnBuilding_Standby.Action[0];
        }

        getCommonR(){
            return this.kbnGoods_Standby.Action[0];
        }

        getGapTime(){
            return this.KanbanAutoTime;
        }
        getContentTime(){
            return this.KanbanContentTime;
        }
        getAngryConfig(){
            return this.KanbanAngryConfig;
        }
        //内购
        getPackStartAction(){
            let r=Math.random()*this.kbnConsume_PackStart.Action.length|0;
            return this.kbnConsume_PackStart.Action[r];      
        }
        getPackStartStr(){
            return Strings$1.get(this.kbnConsume_PackStart.Dialog[0]);
        }

        getCurrencyStartAction(){
            let r=Math.random()*this.kbnConsume_CurrencyStart.Action.length|0;
            return this.kbnConsume_CurrencyStart.Action[r];      
        }
        getCurrencyStartStr(){
            let r=Math.random()*this.kbnConsume_CurrencyStart.Dialog.length|0;
            return Strings$1.get(this.kbnConsume_CurrencyStart.Dialog[r]);
        }

        getConsumeAngryAction(){
            let r=Math.random()*this.kbnConsume_Angry.Action.length|0;
            return this.kbnConsume_Angry.Action[r];      
        }
        getConsumeAngryStr(){
            let r=Math.random()*this.kbnConsume_Angry.Dialog.length|0;
            return Strings$1.get(this.kbnConsume_Angry.Dialog[r]);
        }
        getAutoAction10(){
            let r=Math.random()*this.kbnConsume_Auto.Action.length|0;
            return this.kbnConsume_Auto.Action[r];
        }
        getAutoStr10(){
            let r=Math.random()*this.kbnConsume_Auto.Dialog.length|0;
            return  Strings$1.get(this.kbnConsume_Auto.Dialog[r]);
        }

        getConsumeClickAction(){
            let r=Math.random()*this.kbnConsume_Click.Action.length|0;
            return this.kbnConsume_Click.Action[r];      
        }
        getConsumeClickStr(){
            let r=Math.random()*this.kbnConsume_Click.Dialog.length|0;
            return Strings$1.get(this.kbnConsume_Click.Dialog[r]);
        }
    }

    class KanBanNiang{
        constructor() {
            this.beginLoad();
        }

        beginLoad(){
            this.openDialogBoxTime=200;
            this.playRandomTime=3000;
            this.playCommonTime=5000;
            this.kbnStart=0;
            this.auto=0;
            this.name = "mao1-2";
            this.gapSkeletonTime=KanBanNiangDataMgr.getInstance().getGapTime()*1000;
            this.autoSkeletonTime=KanBanNiangDataMgr.getInstance().getContentTime()*1000;
            this.AngryConfig=KanBanNiangDataMgr.getInstance().getAngryConfig();
            this.loadSkeleton();
            this.loadClickImage();
            this.loadDialogBox();
            this.loadDialogText();
            this.loadKBNImage();
        }

        static getInstance() {
            return KanBanNiang.instance ? KanBanNiang.instance : KanBanNiang.instance = new KanBanNiang();
        }

        loadClickImage(){
            this.leftClickImage=new Laya.Sprite();
            this.leftClickImage.loadImage("comp/image.png");
            this.leftClickImage.width=352;
            this.leftClickImage.height=372;
            this.leftClickImage.pos(400,350);
            this.rightClickImage=new Laya.Sprite();
            this.rightClickImage.loadImage("comp/image.png");
            this.rightClickImage.width=352;
            this.rightClickImage.height=372;
            this.rightClickImage.pos(1000,350);
            this.leftClickImage.mouseEnabled=true;
            this.leftClickImage.mouseThrough=false;
            this.rightClickImage.mouseEnabled=true;
            this.rightClickImage.mouseThrough=false;
            this.leftAngry=new Array();
            this.rightAngry=new Array();
        }

        loadDialogBox(){
            this.leftDialogBox=new Laya.Sprite();
            this.leftDialogBox.loadImage("shopui/kanbanniangdh.png");
            this.leftDialogBox.scale(2,2);
            this.leftDialogBox.pivot(0,120);
            this.leftDialogBox.pos(651,525);
            this.leftDialogBox.visible=false;
            this.rightDialogBox=new Laya.Sprite();
            this.rightDialogBox.loadImage("shopui/kanbanniangdh.png");
            this.rightDialogBox.pos(1177,525);
            this.rightDialogBox.pivot(0,120);
            this.rightDialogBox.scale(0,0);
            this.rightDialogBox.visible=false;
        }

        loadDialogText(){
            this.textL=new Laya.Text();
            this.textR=new Laya.Text();
            this.textL.fontSize=40;
            this.textL.color="#5f493d";
            this.textL.wordWrap=true;
            this.textL.pos(698,336);
            this.textL.width=405;
            this.textL.height=139;
            //this.textL.text="请随意挑选，如果故意乱动的话我可要生气了啊";
            this.textL.zOrder=1000;
            this.textR.fontSize=40;
            this.textR.color="#5f493d";
            this.textR.wordWrap=true;
            this.textR.pos(731,336);
            this.textR.width=405;
            this.textR.height=139;
            //this.textR.text="请随意挑选，如果故意乱动的话我可要生气了啊";
            this.textR.zOrder=1000;
            this.textR.visible = false;
        }

        loadSkeleton(){
            this.kanBanNiangL =new Laya.Skeleton();
            this.kanBanNiangL.pos(567,660);
            this.kanBanNiangL.zOrder=1000;
            this.kanBanNiangL.visible = false;
            this.kanBanNiangL.load("anim1/kanbanniang.sk",null,0);
            this.angryL =new Laya.Skeleton();
            this.angryL.pos(500,470);
            this.angryL.zOrder=1001;
            this.angryL.load("anim1/kanbanniang.sk",null,0);
            this.angryL.visible = false;
            this.kanBanNiangR = new Laya.Skeleton();
            this.kanBanNiangR.pos(1255,660);
            this.kanBanNiangR.zOrder=1000;
            this.kanBanNiangR.visible = false;
            this.kanBanNiangR.load("anim1/kanbanniang.sk",null,0);
            this.angryR =new Laya.Skeleton();
            this.angryR.pos(1330,450);
            this.angryR.rotation = 100;
            this.angryR.zOrder=1001;
            this.angryR.load("anim1/kanbanniang.sk",null,0);
            this.angryR.visible = false;
        }

        loadKBNImage(){
            this.leftImage=new Laya.Sprite();
            this.leftImage.loadImage("shopui/facility2.png");
            this.leftImage.scale(2,2);
            this.leftImage.pos(436,336);
            this.leftImage.zOrder=1000;
            this.leftImage.visible=false;
            this.rightImage=new Laya.Sprite();
            this.rightImage.loadImage("shopui/wupin2.png");
            this.rightImage.scale(2.5,2.5);
            this.rightImage.pos(1150,318);
            this.rightImage.zOrder=1000;
            this.rightImage.visible=false;
        }

        setID(ID){
            this.kbnId=ID;
        }
        getID(){
             return  this.kbnId;
        }

        setStart(num){
            this.kbnStart=num;
        }

        onClick(){
            Laya.timer.clearAll(this);
            this.playClickSkeleton();
        }

        onLeftClick(){
            this.rightAngry.length=0;
            this.leftImage.visible=false;
            this.rightImage.visible=true;
        }

        onRightClick(){
            this.leftAngry.length=0;
            this.leftImage.visible=true;
            this.rightImage.visible=false;
        }

        leftAngryCount(neigou=0){
            let num=this.leftAngry.length;
            this.leftAngry[num]=Laya.systemTimer.currTimer;
            if(num>this.AngryConfig.AngryTimes){
                if(this.leftAngry[num]-this.leftAngry[num-this.AngryConfig.AngryTimes]
                    <this.AngryConfig.TimeInterval*1000){
                        if(this.kbnId!=7&&!this.kbnId!=11){
                            this.id1=this.kbnId?this.kbnId:this.kbnStart;
                        }
                        if(neigou==0){
                            this.kbnId=7;
                        }else {
                            this.kbnId=11;
                        }
                        
                        this.leftAngry.length=0;
                        this.playClickSkeleton();
                }
            }
        }

        rightAngryCount(){
            let num=this.rightAngry.length;
            this.rightAngry[num]=Laya.systemTimer.currTimer;
            if(num>this.AngryConfig.AngryTimes){
                if(this.rightAngry[num]-this.rightAngry[num-this.AngryConfig.AngryTimes]
                    <this.AngryConfig.TimeInterval*1000){
                        if(this.kbnId!=8){
                            this.id1=this.kbnId;
                        }
                        this.kbnId=8;
                        this.rightAngry.length=0;
                        this.playClickSkeleton();
                }
            }
        }

        setText(){
            if(this.auto==0){
                switch(this.kbnId){
                    case 0:this.textL.text=KanBanNiangDataMgr.getInstance().getStr0();break;
                    case 1:this.textL.text=KanBanNiangDataMgr.getInstance().getClickStr1();break;
                    case 2:this.textL.text=KanBanNiangDataMgr.getInstance().getClickStr2();break;
                    case 3:this.textL.text=KanBanNiangDataMgr.getInstance().getClickStr3();break;
                    case 4:this.textR.text=KanBanNiangDataMgr.getInstance().getClickStr4();break;
                    case 5:this.textR.text=KanBanNiangDataMgr.getInstance().getClickStr5();break;
                    case 6:this.textR.text=KanBanNiangDataMgr.getInstance().getClickStr6();break;
                    case 7:this.textL.text=KanBanNiangDataMgr.getInstance().getAngryStr1();break;
                    case 8:this.textR.text=KanBanNiangDataMgr.getInstance().getAngryStr2();break;
                    case 9:this.textL.text=KanBanNiangDataMgr.getInstance().getPackStartStr();break;
                    case 10:this.textL.text=KanBanNiangDataMgr.getInstance().getCurrencyStartStr();break;
                    case 11:this.textL.text=KanBanNiangDataMgr.getInstance().getConsumeAngryStr();break;
                    case 12:this.textL.text=KanBanNiangDataMgr.getInstance().getConsumeClickStr();break;
                }
            }
            else if(this.auto==1){
                switch(this.kbnId){
                    case 1:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr1();break;
                    case 2:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr2();break;
                    case 3:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr3();break;
                    case 4:this.textR.text=KanBanNiangDataMgr.getInstance().getAutoStr4();break;
                    case 5:this.textR.text=KanBanNiangDataMgr.getInstance().getAutoStr5();break;
                    case 6:this.textR.text=KanBanNiangDataMgr.getInstance().getAutoStr6();break;
                    case 9:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr10();break;
                    case 10:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr10();break;
                    case 11:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr10();break;
                    case 12:this.textL.text=KanBanNiangDataMgr.getInstance().getAutoStr10();break;
                }
            }
        }

        playClickSkeleton(){
            this.auto=0;
            this.textL.visible=false;
            this.textR.visible=false;
            switch(this.kbnId){
                case 0:this.name=KanBanNiangDataMgr.getInstance(). getAction0();break;
                case 1:this.name=KanBanNiangDataMgr.getInstance().getClickAction1();break;
                case 2:this.name=KanBanNiangDataMgr.getInstance().getClickAction2();break;
                case 3:this.name=KanBanNiangDataMgr.getInstance().getClickAction3();break;
                case 4:this.name=KanBanNiangDataMgr.getInstance().getClickAction4();break;
                case 5:this.name=KanBanNiangDataMgr.getInstance().getClickAction5();break;
                case 6:this.name=KanBanNiangDataMgr.getInstance().getClickAction6();break;
                case 7:this.name=KanBanNiangDataMgr.getInstance().getAngryActionL1();this.angryL.visible = true;this.angryL.play("angry",false);break;
                case 8:this.name=KanBanNiangDataMgr.getInstance().getAngryActionR1();this.angryR.visible = true;this.angryR.play("angry",false);break;
                case 9:this.name=KanBanNiangDataMgr.getInstance().getPackStartAction();break;
                case 10:this.name=KanBanNiangDataMgr.getInstance().getCurrencyStartAction();break;
                case 11:this.name=KanBanNiangDataMgr.getInstance().getConsumeAngryAction();this.angryL.visible = true;this.angryL.play("angry",false);break;
                case 12:this.name=KanBanNiangDataMgr.getInstance().getConsumeClickAction();
            }
            this.playSkeleton();
        }

        playAutoSkeleton(){
            this.auto=1;
            this.textL.visible=false;
            this.textR.visible=false;
            switch(this.kbnId){
                case 1:this.name=KanBanNiangDataMgr.getInstance().getAutoAction1();break;
                case 2:this.name=KanBanNiangDataMgr.getInstance().getAutoAction2();break;
                case 3:this.name=KanBanNiangDataMgr.getInstance().getAutoAction3();break;
                case 4:this.name=KanBanNiangDataMgr.getInstance().getAutoAction4();break;
                case 5:this.name=KanBanNiangDataMgr.getInstance().getAutoAction5();break;
                case 6:this.name=KanBanNiangDataMgr.getInstance().getAutoAction6();break;
                case 9:this.name=KanBanNiangDataMgr.getInstance().getAutoAction10();break;
                case 10:this.name=KanBanNiangDataMgr.getInstance().getAutoAction10();break;
                case 11:this.name=KanBanNiangDataMgr.getInstance().getAutoAction10();break;
                case 12:this.name=KanBanNiangDataMgr.getInstance().getAutoAction10();break;

            }
            this.playSkeleton();
        }

        playSkeleton(){
            if(this.kbnId<=3||this.kbnId==7||this.kbnId>=9){
                this.openDialogBox(1);
                this.kanBanNiangL.visible = true;
                this.kanBanNiangR.visible = false;
                this.angryR.visible = false;
                this.kanBanNiangL.play(this.name,false);
                Laya.timer.once(this.openDialogBoxTime,this,function(){
                    this.setText();
                    this.textL.visible=true;
                });
                this.name=KanBanNiangDataMgr.getInstance().getCommonL();
                Laya.timer.once(this.playRandomTime,this,function(){
                    if(this.kbnId==7||this.kbnId==11){
                        this.kbnId=this.id1;
                    }
                    this.kanBanNiangL.play(this.name,true);
                });
                Laya.timer.once(this.autoSkeletonTime,this,function(){
                    this.closeDialogBox(1);
                    if(this.kbnId==0){
                        this.kbnId=this.kbnStart;
                    }
                    Laya.timer.once(this.gapSkeletonTime,this,function(){
                        this.playAutoSkeleton();
                    });
                });
            }
            else if(this.kbnId<=6||this.kbnId==8){
                this.openDialogBox(2);
                this.kanBanNiangR.visible = true;
                this.kanBanNiangL.visible = false;
                this.angryL.visible = false;
                this.kanBanNiangR.play(this.name,false);
                Laya.timer.once(this.openDialogBoxTime,this,function(){
                    this.setText(this.kbnId);
                    this.textR.visible=true;
                });
                this.name=KanBanNiangDataMgr.getInstance().getCommonR();
                Laya.timer.once(this.playRandomTime,this,function(){
                    if(this.kbnId==8){
                        this.kbnId=this.id1;
                    }
                    this.kanBanNiangR.play(this.name,true);
                });
                Laya.timer.once(this.autoSkeletonTime,this,function(){
                    this.closeDialogBox(2);
                    Laya.timer.once(this.gapSkeletonTime,this,function(){
                        this.playAutoSkeleton();
                    });
                });
            }
        }

        openDialogBox(num){
            if(num==1){
                this.leftDialogBox.scale(0,0);
                this.rightDialogBox.visible=false;
                this.leftDialogBox.visible=true;
                Laya.Tween.to(this.leftDialogBox,{scaleX:2,scaleY:2},this.openDialogBoxTime);
            }
            else if(num==2){
                this.rightDialogBox.scale(0,0);
                this.leftDialogBox.visible=false;
                this.rightDialogBox.visible=true;
                Laya.Tween.to(this.rightDialogBox,{scaleX:-2,scaleY:2},this.openDialogBoxTime);
            }
        }

        closeDialogBox(num){
            this.textL.visible=false;
            this.textR.visible=false;
            if(num==1){
                this.rightDialogBox.visible=false;
                this.leftDialogBox.visible=true;
                Laya.Tween.to(this.leftDialogBox,{scaleX:0,scaleY:0},this.openDialogBoxTime);
            }
            else if(num==2){
                this.leftDialogBox.visible=false;
                this.rightDialogBox.visible=true;
                Laya.Tween.to(this.rightDialogBox,{scaleX:0,scaleY:0},this.openDialogBoxTime);
            }
        }

        getLeftClickImage(){
            return this.leftClickImage;
        }

        getRightClickImage(){
            return this.rightClickImage;
        }

        getLeftDialogBox(){
            return this.leftDialogBox;
        }

        getRightDialogBox(){
            return this.rightDialogBox;
        }

        getLeftDialogText(){
            return this.textL;
        }

        getRightDialogText(){
            return this.textR;
        }

        getLeftSkeleton(){
            return this.kanBanNiangL;
        }

        getAngryLSkeleton(){
            return this.angryL;
        }

        getAngryRSkeleton(){
            return this.angryR;
        }

        getRightSkeleton(){
            return this.kanBanNiangR;
        }

        getLeftImage(){
            return this.leftImage;
        }

        getRightImage(){
            return this.rightImage;
        }

        toDestory(){
            Laya.timer.clearAll(this);
            Laya.Tween.clearAll(this);
        }
    }

    class PayPop$1 extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            this.PayTimeFlag = 0;
            this.payPanel.vScrollBarSkin = "";
            this.resourePanel.vScrollBarSkin = "";
            this.PurchaseFISHAmountLow = Number(Const.get("PurchaseFISHAmountLow"));
            this.PurchaseFISHAmountHigh = Number(Const.get("PurchaseFISHAmountHigh"));
            this.PurchaseFISHAmount = JSON.parse(Const.get("PurchaseFISHAmount"));
            this.dailyNum.text = Const.get("PurchaseDailyDiamond");
            this.freeDiamondReceiveMills = PaySystem.getPayInfo().freeDiamondReceiveMills;
            this.label1.text = Strings$1.get("tid#ConsumeUI_19");
            this.label2.text = Strings$1.get("tid#UI_095");
            this.label3.text = Strings$1.get("tid#ConsumeUI_21");
            this.fishTips.text = Strings$1.get("tid#ConsumeUI_10");
            this.diamondsTips.text = Strings$1.get("tid#ConsumeUI_23");
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.purchaseConfig = ConfigMgr$1.getInstance().getConfig("Purchase");
            this.setDataArray();
            this.updateCurrency();
            this.weeklyCard.setCard(this.weekArray[0]);
            this.monthlyCard.setCard(this.monthArray[0]);
            this.giftBtn.on(Laya.Event.CLICK,this,this.onGiftBtnClick);
            this.cardBtn.on(Laya.Event.CLICK,this,this.changePanel,[0]);
            this.resoureBtn.on(Laya.Event.CLICK,this,this.changePanel,[1]);
            EventMgr.getInstance().registEvent("PURCHASE_BY_MONEY",this,this.purchaseByMoney);
            EventMgr.getInstance().registEvent("PURCHASE_BY_DIAMOND",this,this.purchaseByDiamond);
            EventMgr.getInstance().registEvent("PURCHASE_COMPLETE",this,this.purchaseComplete);
            EventMgr.getInstance().registEvent("PURCHASE_UPDATE_CURRENCY",this,this.updateCurrency);
            EventMgr.getInstance().registEvent("PURCHASE_REFRESHLIST",this,this.refreshList);
            EventMgr.getInstance().registEvent("PURCHASE_SHOWFAILED",this,this.showFailed);
            EventMgr.getInstance().registEvent("PURCHASE_CHANGEPAYTIMEFLAG",this,this.changePayTimeFlag);
            EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
            this.anim = new Spine();
            this.anim.loadAnim("anim2/meirilingqu.sk");
            this.animBox.addChild(this.anim);
            this.anim.pos(this.animBox.width/2,this.animBox.height/2);
            this.anim.scale(0.5,0.5);
            this.anim.playAnim("1",true);
            this.waittingPanel.visible = false;
            this.waittingBox.visible = false;
            if(PaySystem.checkDay(this.freeDiamondReceiveMills)){
                this.claimedLabel.visible = false;
                this.dailyLabel.visible = true;
                this.anim.visible = true;
            }
            else {
                this.claimedLabel.visible = true;
                this.dailyLabel.visible = false;
                this.anim.visible = false;
            }
            this.loadKanBanNiangData();
            this.updateResouresPos();
            this.changePanel(this._param[0]);
            
        }

        showFailed(type){
            type = type ? type : 0;
            this.showWatting(false);
            if(type == 0){
                TipsPopMgr.getInstance().showTipsById("tid#ConsumeTips_1");
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#ConsumeTips_2");
            }
            this.changePayTimeFlag(2);
        }

        showWatting(visible){
            if(visible){
                if(!this.anim1){
                    this.anim1 = new Spine();
                    this.anim1.loadAnim("anim1/loading2.sk");
                    this.waittingBox.addChild(this.anim1);
                }
                this.waittingPanel.visible = true;
                this.waittingBox.visible = true;
                this.anim1.playAnim("1",true);
            }
            else {
                this.waittingPanel.visible = false;
                this.waittingBox.visible = false;
            }
        }

        loadKanBanNiangData(){
            KanBanNiang.getInstance().beginLoad();
            this.kbnStart=0;
            this.leftClickImage=KanBanNiang.getInstance().getLeftClickImage();
            this.addChild(this.leftClickImage);
            this.leftClickImage.on(Laya.Event.CLICK,this,this.onKBNLeftClick);
            this.dhL=KanBanNiang.getInstance().getLeftDialogBox();
            this.addChild(this.dhL);
            this.textL=KanBanNiang.getInstance().getLeftDialogText();
            this.addChild(this.textL);
            this.kanBanNiangL=KanBanNiang.getInstance().getLeftSkeleton();
            this.addChild(this.kanBanNiangL);
            this.angryL=KanBanNiang.getInstance().getAngryLSkeleton();
            this.addChild(this.angryL);
            this.kbnLeftImage=KanBanNiang.getInstance().getLeftImage();
            this.addChild(this.kbnLeftImage);
            KanBanNiang.getInstance().setID(9);
            KanBanNiang.getInstance().onClick();
        }
        changePanel(num){
            if(num==0){
                KanBanNiang.getInstance().setID(9);
                KanBanNiang.getInstance().onClick();
            }else {
                KanBanNiang.getInstance().setID(10);
                KanBanNiang.getInstance().onClick();
            }
            this.cardBtn.selected = !num;
            this.resoureBtn.selected = num;
            this.payPanel.visible = !num;
            this.resourePanel.visible = num;
        }

        updateResouresPos(){
            this.diamondsTitle.y = 15;
            this.diamondList.y = 105;
            this.fishTitle.y = 882;
            this.fishList.y = 1012;
            this.shellTitle.y = 1378;
            this.shellList.y = 1469;
            let add = 40;
            this.diamondsTips.visible = false;
            if(PaySystem.getPayInfo().firstPay.length < 6){
                this.diamondList.y += add;
                this.fishTitle.y += add;
                this.fishList.y += add;
                this.shellTitle.y += add;
                this.shellList.y += add;
                this.diamondsTips.visible = true;
            }
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("PURCHASE_BY_MONEY");
            EventMgr.getInstance().removeEvent("PURCHASE_UPDATE_CURRENCY");
            EventMgr.getInstance().removeEvent("PURCHASE_BY_DIAMOND");
            EventMgr.getInstance().removeEvent("PURCHASE_COMPLETE");
            EventMgr.getInstance().removeEvent("PURCHASE_REFRESHLIST");
            EventMgr.getInstance().removeEvent("PURCHASE_SHOWFAILED");
            EventMgr.getInstance().removeEvent("PURCHASE_CHANGEPAYTIMEFLAG");
            KanBanNiang.getInstance().toDestory();
        }

        setDataArray(){
            this.diamondArray = new Array();
            this.fishArray = new Array();
            this.shellArray = new Array();
            this.weekArray = new Array();
            this.monthArray = new Array();
            this.purchaseConfig.forEach((listConfig, Id) => {
                if(listConfig.Type == Constants.RewardType.DIAMOND){
                    this.diamondArray.push(listConfig);
                }
                else if(listConfig.Type == Constants.RewardType.FISH){
                    this.fishArray.push(listConfig);
                }
                else if(listConfig.Type == Constants.RewardType.SHELL){
                    this.shellArray.push(listConfig);
                }
                else if(listConfig.Type == "WEEK"){
                    this.weekArray.push(listConfig);
                }
                else if(listConfig.Type == "MONTH"){
                    this.monthArray.push(listConfig);
                }
            });
            function sortByOrder(a,b)
            {
                return a.Order-b.Order;
            }
            this.diamondArray.sort(sortByOrder);
            this.fishArray.sort(sortByOrder);
            this.shellArray.sort(sortByOrder);
            this.weekArray.sort(sortByOrder);
            this.monthArray.sort(sortByOrder);
            this.diamondList.array = this.diamondArray;
            this.fishList.array = this.fishArray;
            this.shellList.array = this.shellArray;
            
            // console.log(this.diamondArray,this.fishArray,this.shellArray,this.weekArray,this.monthArray)
        }

        refreshList(){
            this.diamondList.refresh();
            this.fishList.refresh();
            this.shellList.refresh();
            this.weeklyCard.setCard(this.weekArray[0]);
            this.monthlyCard.setCard(this.monthArray[0]);
        }

        onGiftBtnClick(){
            this.freeDiamondReceiveMills = PaySystem.getPayInfo().freeDiamondReceiveMills;
            if(PaySystem.checkDay(this.freeDiamondReceiveMills)){
                this.receiveFreeDiamond();
                this.claimedLabel.visible = true;
                this.dailyLabel.visible = false;
                this.anim.visible = false;
                EventMgr.getInstance().postEvent("UPDATE_LEFTUI_BTNORDER");
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#ConsumeUI_3");
            }
        }

        checkTimeOut(){
            if(this.PayTimeFlag == 0){
                this.showFailed(1);
            }
            this.changePayTimeFlag(0);
        }

        changePayTimeFlag(num){
            this.PayTimeFlag = num;
        }

        //花钱内购
        purchaseByMoney(goodsId){
            this.showWatting(true);
            this.changePayTimeFlag(0);
            Laya.timer.once(Number(Const.get("PurchasePayTime")) * 1000,this,function(){
                this.checkTimeOut();
            });
            this.rewards = this.purchaseConfig.get(goodsId);
            let productID = "";
            if(Laya.Browser.onAndroid){
                productID = this.rewards.AndroidPoint;
            }
            else if(Laya.Browser.onIOS&&Constants.shouji){
                productID = this.rewards.IOSPoint;
            }
            this.rewardType = this.rewards.Type == "WEEK" || this.rewards.Type == "MONTH" ? 1 :  PaySystem.checkDoubleSign(this.rewards) ? 2 : 0;;
            SdkHelper.showPay(goodsId,productID);
            // this.showPayTest(goodsId);//测试
        }

        purchaseComplete(rewards,rewardType){
            this.showWatting(false);
            rewards = rewards ? rewards : this.rewards;
            rewardType = rewardType? rewardType :this.rewardType;
            Laya.Dialog.close("ui/PayRewards.scene");
            Laya.Dialog.open("ui/PayRewards.scene",false,[rewards,rewardType]);
            this.rewardType = 0;
            this.updateResouresPos();
        }

        showPayTest(goodsId){
            let orderId = "wzlTest" + TimeUtils.getTimeNowOnServer();
            Laya.timer.once(10000,this,function(){
                EventMgr.getInstance().postEvent("PURCHASEASYNC_COMPLETE",[orderId,goodsId]);
                Laya.timer.once(1000,this,function(){
                    EventMgr.getInstance().postEvent("CONSUMEPURCHASEASYNC_COMPLETE",[orderId,goodsId]);
                });
            });
        }

        //钻石内购
        purchaseByDiamond(goodsId){
            this.rewards = this.purchaseConfig.get(goodsId);
            PaySystem.resourceShop(goodsId);
        }

        //领取免费钻石
        receiveFreeDiamond(){
            PaySystem.receiveFreeDiamond();
        }

        //领取周卡月卡奖励
        payCardBonus(cardType){
            PaySystem.payCardBonus(cardType);
        }

        updateCurrency(){
            this.fish_text.text = Strings$1.dealFishNumberText(PlayerSystem.getFish());
            this.star_text.text = Strings$1.dealFishNumberText(PlayerSystem.getStar());
            this.beike_text.text = Strings$1.dealFishNumberText(PlayerSystem.getShell());
            this.zuanshi_text.text = Strings$1.dealFishNumberText(PlayerSystem.getDiamond());
        }
        onKBNLeftClick(){
            if( KanBanNiang.getInstance().getID()!=11){
                KanBanNiang.getInstance().setID(12);
            } 
                KanBanNiang.getInstance().onClick();
                KanBanNiang.getInstance().leftAngryCount(1);
        }
    }

    class PayCard extends Laya.Sprite{
        constructor(){
            super();
        }

        setCard(listConfig){
            this.cardBg = this.getChildByName("cardBg");
            this.totalText = this.getChildByName("totalText");
            this.totalNum = this.getChildByName("totalNum");
            this.Obtain1 = this.getChildByName("Obtain1");
            this.ObtainNum = this.getChildByName("ObtainNum");
            this.immediatly = this.getChildByName("immediatly");
            this.Obtain2 = this.getChildByName("Obtain2");
            this.detail1 = this.getChildByName("detail1");
            this.detail2 = this.getChildByName("detail2");
            this.detail3 = this.getChildByName("detail3");
            this.detail4 = this.getChildByName("detail4");
            this.detail5 = this.getChildByName("detail5");
            this.daysTips = this.getChildByName("daysTips");
            this.icon1 = this.getChildByName("icon1");
            this.icon2 = this.getChildByName("icon2");
            this.icon3 = this.getChildByName("icon3");
            this.icon4 = this.getChildByName("icon4");
            this.icon5 = this.getChildByName("icon5");
            this.q1 = this.getChildByName("q1");
            this.q2 = this.getChildByName("q2");
            this.q3 = this.getChildByName("q3");
            this.q4 = this.getChildByName("q4");
            this.q5 = this.getChildByName("q5");
            this.q1.on(Laya.Event.CLICK,this,this.onQClick,[1]);
            this.q2.on(Laya.Event.CLICK,this,this.onQClick,[2]);
            this.q3.on(Laya.Event.CLICK,this,this.onQClick,[3]);
            this.q4.on(Laya.Event.CLICK,this,this.onQClick,[4]);
            this.q5.on(Laya.Event.CLICK,this,this.onQClick,[5]);
            this.QArray = [this.q1,this.q2,this.q3,this.q4,this.q5];
            this.cardImg = this.getChildByName("cardImg");
            this.opened = this.getChildByName("opened");
            this.claimed = this.getChildByName("claimed");
            this.buyBtn = this.getChildByName("buy");
            this.buyBtn.on(Laya.Event.CLICK,this,this.onBuyBtnClick,[listConfig]);      
            this.detailArray = [this.detail1,this.detail2,this.detail3,this.detail4,this.detail5];
            this.iconArray = [this.icon1,this.icon2,this.icon3,this.icon4,this.icon5];

            this.totalText.text = Strings$1.get("tid#ConsumeUI_6");
            this.Obtain1.text = Strings$1.get("tid#ConsumeUI_15_1");
            this.immediatly.text = Strings$1.get("tid#ConsumeUI_15_2");
            this.Obtain2.text = Strings$1.get("tid#ConsumeUI_16");

            let directReward = JSON.parse(listConfig.DirectReward);
            let dailyReawrd = JSON.parse(listConfig.DailyReawrd);
            let PurchaseWeekDays = Const.get("PurchaseWeekDays");
            let PurchaseMonthDays = Const.get("PurchaseMonthDays");
            this.ObtainNum.text = directReward[0].amount;
            this.buyBtn.label = "$ " + listConfig.Price;
            this.dailyReawrd = dailyReawrd;

            for(let i = 0;i < dailyReawrd.length;i++){
                if(dailyReawrd[i].type == Constants.ItemType.Currency){
                    this.QArray[i].visible = false;
                    switch(dailyReawrd[i].id){
                        case "FISH": this.detailArray[i].text = dailyReawrd[0].amount + " " + Strings$1.get("tid#UI_095");this.iconArray[i].skin = "topui/yugan.png";break;
                        case "STAR": this.detailArray[i].text = dailyReawrd[0].amount + " " + Strings$1.get("tid#ConsumeUI_20");this.iconArray[i].skin = "topui/xingxing.png";break;
                        case "SHELL": this.detailArray[i].text = dailyReawrd[0].amount + " " + Strings$1.get("tid#ConsumeUI_21");this.iconArray[i].skin = "topui/beike.png";break;
                        case "DIAMOND": this.detailArray[i].text = dailyReawrd[0].amount + " " + Strings$1.get("tid#ConsumeUI_19");this.iconArray[i].skin = "topui/zuanshi.png";break;
                    }
                }
                else if(dailyReawrd[i].type == Constants.ItemType.Coupon){
                    this.QArray[i].visible = true;
                    switch(dailyReawrd[i].id){
                        case "ADINVITE": this.detailArray[i].text = Strings$1.get("tid#ConsumeUI_22") + " +" + dailyReawrd[i].amount;this.iconArray[i].skin = "purchase/kitty.png";break;
                        case "SPEED": this.detailArray[i].text = Strings$1.get("tid#ConsumeUI_12") + " +" + dailyReawrd[i].amount;this.iconArray[i].skin = "purchase/voucher_speed.png";break;
                        case "FISHING": this.detailArray[i].text = Strings$1.get("tid#ConsumeUI_13") + " +" + dailyReawrd[i].amount;this.iconArray[i].skin = "purchase/voucher_fish.png";break;
                        case "CATFOOD": this.detailArray[i].text = Strings$1.get("tid#ConsumeUI_14") + " +" + dailyReawrd[i].amount;this.iconArray[i].skin = "purchase/voucher_catfood.png";break;
                    }
                }
            }

            let payCardInfo = PaySystem.getPayInfo().payCardInfo;
            this.buyBtn.visible = true;
            this.opened.visible = false;
            this.claimed.visible = false;
            
            if(listConfig.Type == "WEEK"){
                this.cardBg.skin = "purchase/bg_week.png";
                this.cardImg.skin = "purchase/weekcard.png";
                this.totalNum.text = Number(dailyReawrd[0].amount) * PurchaseWeekDays + Number(directReward[0].amount);
                this.daysTips.text = Strings$1.get("tid#ConsumeUI_4").replace("%1%",PurchaseWeekDays);
                if(payCardInfo && payCardInfo.weekCardEndMills){
                    if(payCardInfo.weekCardEndMills >= TimeUtils.getTimeNowOnServer()){
                        this.buyBtn.visible = false;
                        this.opened.visible = true;
                        this.daysTips.text = Strings$1.get("tid#ConsumeUI_5").replace("%1%",PaySystem.getRemainDays(payCardInfo.weekCardEndMills));
                        if(!PaySystem.checkDay(payCardInfo.weekCardReceiveMills)){
                            this.claimed.visible = true;
                        }
                    }
                }
            }
            else if(listConfig.Type == "MONTH"){
                this.cardBg.skin = "purchase/bg_month.png";
                this.cardImg.skin = "purchase/monthcard.png";
                this.totalNum.text = Number(dailyReawrd[0].amount) * PurchaseMonthDays + Number(directReward[0].amount);
                this.daysTips.text = Strings$1.get("tid#ConsumeUI_4").replace("%1%",PurchaseMonthDays);
                if(payCardInfo && payCardInfo.monthCardEndMills ){
                    if(payCardInfo.monthCardEndMills  >= TimeUtils.getTimeNowOnServer()){
                        this.buyBtn.visible = false;
                        this.opened.visible = true;
                        this.daysTips.text = Strings$1.get("tid#ConsumeUI_5").replace("%1%",PaySystem.getRemainDays(payCardInfo.monthCardEndMills));
                        if(!PaySystem.checkDay(payCardInfo.monthCardReceiveMills)){
                            this.claimed.visible = true;
                        }
                    }
                }
            }
        }

        onBuyBtnClick(listConfig){
            if(PaySystem.getPayLock()){
                // PaySystem.setPayLock(0);
                // Laya.timer.once(Number(Const.get("PurchasePayTime")) * 1000,this,function(){
                //     PaySystem.setPayLock(1);
                // })
                let goodsId = listConfig.Id;
                EventMgr.getInstance().postEvent("PURCHASE_BY_MONEY",[goodsId]);
            }
        }

        onQClick(num){
            if(this.dailyReawrd[num - 1].type == Constants.ItemType.Coupon){
                let type = 0;
                switch(this.dailyReawrd[num - 1].id){
                    case "ADINVITE": type = 4;break;
                    case "SPEED": type = 2;break;
                    case "FISHING": type = 1;break;
                    case "CATFOOD": type = 3;break;
                }
                Laya.Dialog.close("ui/PayHelper.scene");
                Laya.Dialog.open("ui/PayHelper.scene",false,[type]);
            }
        }
    }

    class PayItem extends Laya.Image{
        constructor(){
            super();
            this.on(Laya.Event.CLICK,this,this.onPayItemClick);
        }

        setItem(listConfig){
            this.listConfig = listConfig;
            this.directReward = this.getChildByName("directReward");
            this.extraReward = this.getChildByName("extraReward");
            this.icon = this.getChildByName("icon");
            this.icon.skin = "purchase/" + listConfig.Picture;
            this.moneyLabel = this.getChildByName("moneyLabel");
            this.diamondLabel = this.getChildByName("diamondLabel");
            this.diamondImage = this.getChildByName("diamondImage");
            this.double = this.getChildByName("double");
            this.double.visible = PaySystem.checkDoubleSign(listConfig);

            let directRewardAmount = JSON.parse(listConfig.DirectReward)[0].amount;
            let extraRewardAmount = JSON.parse(listConfig.ExtraReward).length ? JSON.parse(listConfig.ExtraReward)[0].amount : 0;
            let directRewardAmountRate = JSON.parse(listConfig.DirectReward)[0].amountRate;
            let extraRewardAmountRate = JSON.parse(listConfig.ExtraReward).length ? JSON.parse(listConfig.ExtraReward)[0].amountRate : 0;
            if(directRewardAmountRate){
                directRewardAmount = (directRewardAmountRate * PaySystem.getSingleFish()).toFixed(0);
                if(extraRewardAmountRate){
                    extraRewardAmount = (extraRewardAmountRate * PaySystem.getSingleFish()).toFixed(0);
                }
            }
            this.extraReward.visible = false;
            if(directRewardAmount){
                this.directReward.text = directRewardAmount;
                if(!this.double.visible && extraRewardAmount){
                    this.extraReward.visible = true;
                    this.extraReward.text = extraRewardAmount;
                }
                else if(this.double.visible){
                    this.extraReward.visible = true;
                    this.extraReward.text = directRewardAmount;
                }
            }
            if(listConfig.CostType == "1"){
                this.moneyLabel.visible = true;
                this.diamondLabel.visible = false;
                this.diamondImage.visible = false;
                this.moneyLabel.text = "$ "+listConfig.Price;
            }
            else if(listConfig.CostType == "2"){
                this.moneyLabel.visible = false;
                this.diamondLabel.visible = true;
                this.diamondImage.visible = true;
                this.diamondLabel.text = listConfig.Price;
            }

            let w1 = this.diamondImage.width * this.diamondImage.scaleX;
            let w2 = this.diamondLabel.width;
            let startX = (this.width - w1 - w2 - 10)/2;
            this.diamondImage.x = startX;
            this.diamondLabel.x = startX + w1 + 10;
        }

        onPayItemClick(){
            if(this.listConfig.Type == Constants.RewardType.DIAMOND){
                if(!this.payLock){
                    this.setPayLock();
                    EventMgr.getInstance().postEvent("PURCHASE_BY_MONEY",[this.listConfig.Id]);
                }
                else {
                    let PurchaseCDTime = Number(Const.get("PurchaseCDTime")) * 1000;
                    let time = (PurchaseCDTime - TimeUtils.getTimeNowOnServer() + this.unlockTime) / 1000 | 0 + 1;
                    TipsPopMgr.getInstance().showTipsById("tid#ConsumeUI_9",[time]);
                }
            }
            else {
                Laya.Dialog.close("ui/PayTipsPop.scene");
                Laya.Dialog.open("ui/PayTipsPop.scene",false,[this.listConfig]);
            }
        }

        setPayLock(){
            let PurchaseCDTime = Number(Const.get("PurchaseCDTime")) * 1000;
            this.payLock = 1;
            this.unlockTime = TimeUtils.getTimeNowOnServer();
            Laya.timer.once(PurchaseCDTime,this,function(){
                this.payLock = 0;
            });
        }
    }

    class PayRewards extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.PayRewards, 1);
            this.closeBtn.on(Laya.Event.CLICK,this,this.getReward);
            this.getBtn.on(Laya.Event.CLICK,this,this.getReward);
            this.title.text = Strings$1.get("tid#LuckyDrawUI_2");
            this.getBtn.label = Strings$1.get("tid#Btn_017");
            this.rewardItemList = [this.rewardItem1,this.rewardItem2,this.rewardItem3,this.rewardItem4,this.rewardItem5];
            this.rewardsConfig = this._param[0];
            this.rewardsType = this._param[1];
            if(this.rewardsType == 3){
                this.rewards = this.rewardsConfig;
            }
            else if(this.rewardsConfig.Type == "WEEK" || this.rewardsConfig.Type == "MONTH"){
                if(this.rewardsType == 1){
                    this.rewards = JSON.parse(this.rewardsConfig.DirectReward);
                }
                else {
                    this.rewards = JSON.parse(this.rewardsConfig.DailyReawrd);
                    let type = this.rewardsConfig.Type == "WEEK" ? 0 : 1;
                    PaySystem.payCardBonus(type);
                }
            }
            else {
                this.rewards = JSON.parse(this.rewardsConfig.DirectReward);
                let ExtraReward = JSON.parse(this.rewardsConfig.ExtraReward);
                if(this.rewardsType == 2){
                    let n = this.rewards.length;
                    for(let i = 0; i < n ; i ++){
                        let doubleStr = JSON.stringify(this.rewards[i]);
                        let double = JSON.parse(doubleStr);
                        double.gift = 1;
                        this.rewards.push(double);
                    }
                }
                else {
                    for(let i = 0; i < ExtraReward.length ; i++){
                        ExtraReward[i].gift = 1;
                        this.rewards.push(ExtraReward[i]);
                    }
                }
                if(this.rewardsConfig.Type == Constants.RewardType.FISH){
                    for(let i = 0; i < this.rewards.length ; i ++){
                        this.rewards[i].amount = this.rewards[i].amountRate * PaySystem.getSingleFish();
                    }
                }
            }
            for(let i = 0;i < this.rewardItemList.length; i++){
                if(i < this.rewards.length){
                    this.rewardItemList[i].visible = true;
                    this.rewardItemList[i].setItem(this.rewards[i]);
                }
                else {
                    this.rewardItemList[i].visible = false;
                }
            }
            this.setPos();
        }

        onDisable(){
            if(this.rewardsConfig.Type == "WEEK"){
                EventMgr.getInstance().postEvent("GET_PURCHASE_CARD");
            }
        }

        getReward(){
            this.close();
            EventMgr.getInstance().postEvent("PURCHASE_REFRESHLIST");
            if(this.rewardsType == 1){
                Laya.Dialog.open("ui/PayRewards.scene",false,[this.rewardsConfig,0]);
            }
        }

        setPos(){
            let Y = new Array(910,1213,1080,1420,1571);
            let X = new Array(567,813,1061,696,943);
            let H = new Array(312,452);
            if(this.rewards.length <= 0){
                return ;
            }
            else if(this.rewards.length == 1){
                this.rewardItemList[0].pos(X[1],Y[2]);
            }
            else if(this.rewards.length == 2){
                this.rewardItemList[0].pos(X[3],Y[2]);
                this.rewardItemList[1].pos(X[4],Y[2]);
            }
            else if(this.rewards.length == 3){
                this.rewardItemList[0].pos(X[0],Y[2]);
                this.rewardItemList[1].pos(X[1],Y[2]);
                this.rewardItemList[2].pos(X[2],Y[2]);
            }
            else if(this.rewards.length == 4){
                this.rewardItemList[0].pos(X[0],Y[0]);
                this.rewardItemList[1].pos(X[1],Y[0]);
                this.rewardItemList[2].pos(X[2],Y[0]);
                this.rewardItemList[3].pos(X[1],Y[1]);
            }
            else if(this.rewards.length == 5){
                this.rewardItemList[0].pos(X[0],Y[0]);
                this.rewardItemList[1].pos(X[1],Y[0]);
                this.rewardItemList[2].pos(X[2],Y[0]);
                this.rewardItemList[3].pos(X[3],Y[1]);
                this.rewardItemList[4].pos(X[4],Y[1]);
            }

            if(this.rewards.length <= 3){
                this.bg.height = H[0];
                this.getBtn.y = Y[3];
            }
            else if(this.rewards.length <= 5){
                this.bg.height = H[1];
                this.getBtn.y = Y[4];
            }
        }
        onDisable(){
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.PayRewards, 1);
        }

    }

    class PayRewardItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(reward){
            let icon_img = this.getChildByName("icon");
            let amount_text = this.getChildByName("amount");
            amount_text.text = Strings$1.dealFishNumberText(reward.amount, 1);
            let gift = this.getChildByName("gift");
            gift.visible = reward.gift;
            if (reward.type == Constants.ItemType.Currency) {
                if (reward.id == Constants.RewardType.FISH) {
                    icon_img.skin = "topui/yugan1.png";
                } else if (reward.id == Constants.RewardType.STAR) {
                    icon_img.skin = "topui/xingxing1.png";
                } else if (reward.id == Constants.RewardType.DIAMOND || reward.id == "PAY_DIAMOND") {
                    icon_img.skin = "topui/diamond.png";
                } else if (reward.id == Constants.RewardType.SHELL) {
                    icon_img.skin = "topui/beike1.png";
                } else if (reward.id == Constants.RewardType.MEDAL) {
                    icon_img.skin = "topui/huizhang1.png";
                } else if (reward.id == Constants.RewardType.BOOK) {
                    icon_img.skin = "topui/miji1.png";
                }
            } else if (reward.type == Constants.ItemType.NauticalFoods) {
                let foodConfig = ConfigMgr.getInstance().getItemById("NauticalFoods", reward.id);
                icon_img.skin = foodConfig.Icon;
            } else if (reward.type == Constants.ItemType.NauticalTools) {
                let toolConfig = ConfigMgr.getInstance().getItemById("NauticalTools", reward.id);
                icon_img.skin = toolConfig.Icon;
            }
            else if (reward.type == Constants.ItemType.Coupon){
                switch(reward.id){
                    case "ADINVITE": icon_img.skin = "purchase/kitty1.png";break;
                    case "SPEED": icon_img.skin = "purchase/voucher_speed1.png";break;
                    case "FISHING": icon_img.skin = "purchase/voucher_fish1.png";break;
                    case "CATFOOD": icon_img.skin = "purchase/voucher_catfood1.png";break;
                }
            }
        }

    }

    class PayTipsPop extends Laya.Dialog{
        constructor(){
            super();
        }

        onEnable(){
            this.config = this._param[0];
            let DirectReward = JSON.parse(this.config.DirectReward)[0];
            let amount = DirectReward.amount ? DirectReward.amount : DirectReward.amountRate * PaySystem.getSingleFish();
            this.text1.text = Strings$1.get("tid#ConsumeUI_7_1");
            this.costText.text = this.config.Price + " " ;
            this.text2.text = Strings$1.get("tid#ConsumeUI_7_2");
            this.text3.text = amount + "?";
            this.sure.label = Strings$1.get("tid#ConsumeBtn_3");
            this.cancel.label = Strings$1.get("tid#ConsumeBtn_4");
            if(this.config.CostType == 2){
                this.costImg.skin = "topui/zuanshi.png";
                if(PlayerSystem.getDiamond() < this.config.Price){
                    this.costText.color = "#ff0000";
                }
            }           
            switch(DirectReward.id){
                case Constants.RewardType.FISH : this.rewardImg.skin = "topui/yugan.png";break;
                case Constants.RewardType.SHELL : this.rewardImg.skin = "topui/beike.png";break;
            }
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.cancel.on(Laya.Event.CLICK,this,this.close);
            this.OkBtn.on(Laya.Event.CLICK,this,this.onSureClick);
            this.setPos();
        }

        onSureClick(){
            if(PlayerSystem.getDiamond() < this.config.Price){
                Laya.Dialog.close("ui/OrderBuy.scene");
                Laya.Dialog.open("ui/OrderBuy.scene", false,[1]);
                // TipsPopMgr.getInstance().showTipsById("tid#ConsumeUI_8");
            }
            else {
                EventMgr.getInstance().postEvent("PURCHASE_BY_DIAMOND",[this.config.Id]);
            }
            this.close();
        }

        setPos(){
            let widthArray = new Array(this.text1.width,this.costImg.width * this.costImg.scaleX,this.costText.width,this.text2.width,this.rewardImg.width * this.rewardImg.scaleX,this.text3.width);
            let space = 10;
            let allWidth = space * 4;
            for(let i = 0;i < widthArray.length; i ++){
                allWidth += widthArray[i];
            }
            let beginX = (this.width - allWidth) / 2;
            this.text1.x = beginX;
            this.costImg.x = this.text1.x + widthArray[0] + space;
            this.costText.x = this.costImg.x + widthArray[1] + space;
            this.text2.x = this.costText.x + widthArray[2] + space;
            this.rewardImg.x = this.text2.x + widthArray[3] + space;
            this.text3.x = this.rewardImg.x + widthArray[4];
        }
    }

    class PetStarUpPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this._petId = this._param;
            this.okBtn = this.getChildByName("okBtn");
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkBtnClick);
            this.cancelBtn = this.getChildByName("cancelBtn");
            this.cancelBtn.on(Laya.Event.CLICK, this, this.onCancelBtnClick);
            this.title.text = Strings$1.get("tid#UI_186");
            this.ok_text.text = Strings$1.get("tid#Btn_009");
            this.cancel_text.text = Strings$1.get("tid#Btn_039");

            let book = PlayerSystem.getBook();
            let pet = PetSystem.getPetInfo(this._petId);
            let starUpCondition = Const.getMap("CatCultivateQualityUpCondition");
            let cost = Math.floor(starUpCondition[String(pet.star + 1)].Cost * Const.getArray("CatCultivateQualityUpCostRate")[pet.Potential]);
            this.cost.text = Strings$1.get("tid#UI_183") + book + "/" + cost;
            this.cost_img.x = this.cost.x + this.cost.width / 2 + 5;
            if (cost > book) {
                this.okBtn.gray = true;
            }
        }


        onOkBtnClick() {
            let book = PlayerSystem.getBook();
            let pet = PetSystem.getPetInfo(this._petId);
            let starUpCondition = Const.getMap("CatCultivateQualityUpCondition");
            let cost = Math.floor(starUpCondition[String(pet.star + 1)].Cost * Const.getArray("CatCultivateQualityUpCostRate")[pet.Potential]);
            if (cost > book) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon79", [cost - book]]);
                return;
            }
            let dealFunc = function (res) {
                EventMgr.getInstance().postEvent("PET_STAR_UP");
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.SHENGXING);
            };
            PetSystem.starUp(this._petId, Laya.Handler.create(null, dealFunc));
            EventMgr.getInstance().postEvent("SHOW_CAT_LEVELUPTIPS");
            this.close();
        }

        onCancelBtnClick() {
            this.close();
        }

    }

    class PhotoAlbumDialog extends Laya.Dialog {
        constructor() {
            super();
            this._type = null;
        }
        onAwake() {
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.huiyiButton = this.getChildByName("huiyiButton");
            this.zhuapaiButton = this.getChildByName("zhuapaiButton");
            this.huiyiButton.on(Laya.Event.CLICK, this, this.huiyi);
            this.zhuapaiButton.on(Laya.Event.CLICK, this, this.zhuapai);
            this.huiyiBox = this.getChildByName("huiyiBox");
            this.zhuapaiBox = this.getChildByName("zhuapaiBox");
            this.noBox = this.getChildByName("noBox");
            this.lable1 = this.noBox.getChildByName("lable1");
            this.lable2 = this.noBox.getChildByName("lable2");
            this.lable1.text = Strings$1.get("tid#UI_070");
            this.lable2.text = Strings$1.get("tid#UI_071");
            this.toNautical = this.noBox.getChildByName("toNautical");
            this.toNautical.on(Laya.Event.CLICK, this, this.goNautical);
            this.toNautical.getChildByName("toNauticalText").text = Strings$1.get("tid#Btn_011");
            this.photoNum= this.zhuapaiBox.getChildByName("photoNum");
            this.userPhotos = [];
            for (let i = 0; i < 8; i++) {
                let name = "userPhoto" + (i + 1);
                let userPhoto = this.zhuapaiBox.getChildByName(name);
                this.userPhotos.push(userPhoto);
            }
            this.systemPhotos = [];
            this.systemPhotos.push(this.huiyiBox.getChildByName("systemPhoto1"));
            this.systemPhotos.push(this.huiyiBox.getChildByName("systemPhoto2"));
            this.leftBtn = this.getChildByName("leftButton");
            this.rightBtn = this.getChildByName("rightButton");
            this.leftBtn.on(Laya.Event.CLICK, this, this.onLeftClick);
            this.rightBtn.on(Laya.Event.CLICK, this, this.onRightClick);
        }

        onEnable() {
            EventMgr.getInstance().registEvent("PHOTOS_UPDATA", this, this.updata);
            this._startSystemPhotoId = 0;
            this._startUserPhotoId = 0;
            this.huiyi();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("PHOTOS_UPDATA", this, this.updata);
        }

        onCloseClick() {
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        zhuapai() {
            if (this._type == 1) {
                return;
            }
            this.huiyiButton.selected = false;
            this.huiyiButton.disabled = false;
            this.zhuapaiButton.selected = true;
            this.zhuapaiButton.disabled = true;
            this.zhuapaiButton.gray = false;
            this.zhuapaiBox.visible = true;
            this.huiyiBox.visible = false;
            this._type = 1;
            this.noboxshow=2;
            this.noBox.visible = false;
            this.updata();
        }

        huiyi() {
            if (this._type == 2) {
                return;
            }
            this.huiyiButton.selected = true;
            this.huiyiButton.disabled = true;
            this.huiyiButton.gray = false;
            this.zhuapaiButton.selected = false;
            this.zhuapaiButton.disabled = false;
            this.zhuapaiBox.visible = false;
            this.huiyiBox.visible = true;
            this._type = 2;
            this.noboxshow=1;
            this.updata();
        }

        goNautical(){
            Laya.Dialog.closeAll();
            SceneMgr.getInstance().switchScene(Constants.NauticalSceneId);
            let hasBoat = BuildingSystem.hasBuildingDataByBid("3010") || BuildingSystem.hasBuildingDataByBid("3011") || BuildingSystem.hasBuildingDataByBid("3012");
            let index = NauticalSystem$1.getBoatCanSail();
            if(hasBoat){              
                Laya.timer.once(50,this,function(){
                    if(index == -2){
                        TipsPopMgr.getInstance().showTipsById("tid#TipsCommon29");
                    }
                    else if(index == -3){
                        TipsPopMgr.getInstance().showTipsById("tid#TipsCommon71");
                    }
                    else {
                        Laya.Dialog.close("ui/NauticalStartPop.scene");
                        Laya.Dialog.open("ui/NauticalStartPop.scene", false,index);
                    }
                });
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon28");
            }
        }

        onLeftClick() {
            if (this._type == 1) {
                if (this._startUserPhotoId > 7) {
                    this._startUserPhotoId -= 8;
                }
            } else {
                if (this._startSystemPhotoId > 1) {
                    this._startSystemPhotoId -= 2;
                }
            }
            this.updata();
        }

        onRightClick() {
            if (this._type == 1) {
                if (this._startUserPhotoId < this.userPhotoData.length - 8) {
                    this._startUserPhotoId += 8;
                }
            } else {
                if (this._startSystemPhotoId < this.systemPhotoData.length - 2) {
                    this._startSystemPhotoId += 2;
                }
            }
            this.updata();
        }

        updata() {
            if (this._type == 1) {
                this.userPhotoData = PhotoSystem.getUserPhotosData();
                this.photoNum.text=this.userPhotoData.length+'/'+Const.get('PhotoWallCaptureLimit');
                if(Number(this.userPhotoData.length)>=Number(Const.get('PhotoWallCaptureLimit'))){
                    this.photoNum.color='#f31919';
                }else {
                    this.photoNum.color='#5a361d';
                }
                let endIndex = Math.min(this._startUserPhotoId + 8, this.userPhotoData.length);
                let currentData = this.userPhotoData.slice(this._startUserPhotoId, endIndex);
                currentData.length = 8;
                for (let i in this.userPhotos) {
                    this.userPhotos[i].updata(currentData[i]);
                }
                this.leftBtn.visible = this._startUserPhotoId > 7;
                this.rightBtn.visible = this._startUserPhotoId < this.userPhotoData.length - 8;
            } else {
                this.systemPhotoData = PhotoSystem.getSystemPhotosData();
                let endIndex = Math.min(this._startSystemPhotoId + 2, this.systemPhotoData.length);
                let currentData = this.systemPhotoData.slice(this._startSystemPhotoId, endIndex);
                currentData.length = 2;
                for (let i in this.systemPhotos) {
                    this.systemPhotos[i].updata(currentData[i]);
                }
                this.leftBtn.visible = this._startSystemPhotoId > 1;
                this.rightBtn.visible = this._startSystemPhotoId < this.systemPhotoData.length - 2;
                this.noBox.visible = false;
                if(this.systemPhotoData.length <= 0){
                    this.huiyiBox.visible = false;
                    if(this.noboxshow==1){
                        this.noBox.visible = true;
                    }
                }
            }
            EventMgr.getInstance().postEvent("UPDATE_BOX_ANIM");
        }


    }

    class UserPhoto extends Laya.Box {

        constructor() {
            super();
        }

        onAwake() {
            this.photo = this.getChildByName("photo");
            this.selectedBox = this.getChildByName("selectedBox");
            this.unSelectedBox = this.getChildByName("unSelectedBox");
            this.place = this.selectedBox.getChildByName("place");
            this.remove = this.selectedBox.getChildByName("remove");
            this.delete = this.selectedBox.getChildByName("delete");
            this.place.on(Laya.Event.CLICK, this, this.onPlaceClick);
            this.remove.on(Laya.Event.CLICK, this, this.onRemoveClick);
            this.delete.on(Laya.Event.CLICK, this, this.onDeleteClick);
            this.placed = this.unSelectedBox.getChildByName("placed");
            this.timeLabel = this.unSelectedBox.getChildByName("time");
            this._selected = false;
            this.unSelectedBox.on(Laya.Event.CLICK, this, this.onSelectedClick);
        }

        onEnable() {
            this.selectedBox.visible = false;
            EventMgr.getInstance().registEvent("USER_PHOTO_SELECTED", this, this.selectedByName);
        }

        updata(photoData) {
            this.photoData = photoData;
            this.setSelected(false);
            if (this.photoData) {
                let image = new Laya.Image();
                image.width = this.photo.width;
                image.height = this.photo.height;
                image.visible = true;
                image.skin = this.photoData.getImage();
                this.photo.addChild(image);
                this.photo.visible=true;
                this.placed.visible = this.photoData.getInFrame();
                this.timeLabel.visible = !this.photoData.getInFrame();
                this.timeLabel.text = Strings$1.getTimeByUTCMilliseconds(this.photoData.getTime());
            } else {
                this.photo.skin = null;
                this.photo.visible=false;
                this.placed.visible = false;
                this.timeLabel.visible = false;
            }
        }

        setSelected(value) {
            if (this.photoData) {
                if (this._selected != value) {
                    this._selected = value;
                    if (this._selected) {
                        this.selectedBox.visible = true;
                        this.unSelectedBox.visible = false;
                        this.remove.visible = this.photoData.getInFrame();
                        this.place.visible = !this.photoData.getInFrame();
                    } else {
                        this.selectedBox.visible = false;
                        this.unSelectedBox.visible = true;
                    }
                }
            } else {
                this.selectedBox.visible = false;
                this.unSelectedBox.visible = true;
            }
        }

        selectedByName(name) {
            if (name != this.name) {
                this.setSelected(false);
            }
        }

        onSelectedClick() {
            if (this._selected) {
                return;
            }
            this.setSelected(true);
            EventMgr.getInstance().postEvent("USER_PHOTO_SELECTED", [this.name]);
        }

        onDisable() {
            //  this.photo.disposeBitmap();
            EventMgr.getInstance().removeEvent("USER_PHOTO_SELECTED", this);
            // this.photo.destroy();
        }

        onPlaceClick() {
            let v = PhotoSystem.placePhoto(this.photoData.getId());
            if (v) {
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon10");
            }
        }

        onRemoveClick() {
            PhotoSystem.removePhoto(this.photoData.getId());
        }

        onDeleteClick() {
            PhotoSystem.deletePhotoData(this.photoData.getId());
        }
    }

    class SystemPhoto extends Laya.Box {

        constructor() {
            super();
        }

        onAwake() {
            this.bgImage = this.getChildByName("bg");
            this.catImageBox = this.getChildByName("cat");
            this.selectedBox = this.getChildByName("selectedBox");
            this.unSelectedBox = this.getChildByName("unSelectedBox");
            this.place = this.selectedBox.getChildByName("place");
            this.remove = this.selectedBox.getChildByName("remove");
            this.place.on(Laya.Event.CLICK, this, this.onPlaceClick);
            this.remove.on(Laya.Event.CLICK, this, this.onRemoveClick);
            this.placed = this.unSelectedBox.getChildByName("placed");
            this._selected = false;
            this.unSelectedBox.on(Laya.Event.CLICK, this, this.onSelectedClick);
        }

        onEnable() {
            this.selectedBox.visible = false;
            EventMgr.getInstance().registEvent("USER_PHOTO_SELECTED", this, this.selectedByName);
        }

        updata(photoData) {
            this.photoData = photoData;
            this.setSelected(false);
            if (this.photoData) {
                let config = this.photoData.getSystemConfig();
                this.bgImage.skin = "systemPhotoBg/"+config.Background+".jpg";
                let catSkins = JSON.parse(config.Cat);
                if (this.catArr && this.catArr.length != 0) {
                    for (const cat of this.catArr) {
                        cat.destroy();
                    }
                }
                this.catArr = [];
                let catCatroute = "systemPhoto/"+ config.Catroute;
                for (const catSkin of catSkins) {
                    let cat = new Laya.Image();
                    cat.skin = catCatroute + "/" + catSkin + ".png";
                    cat.width = this.catImageBox.width;
                    cat.height = this.catImageBox.height;
                    this.catImageBox.addChild(cat);
                    this.catArr.push(cat);
                }
                this.placed.visible = this.photoData.getInFrame();
            } else {
                this.bgImage.skin = null;
                if (this.catArr && this.catArr.length != 0) {
                    for (const cat of this.catArr) {
                        cat.destroy();
                    }
                }
                this.catArr = null;
                this.placed.visible = false;
            }
        }

        setSelected(value) {
            if (this.photoData) {
                if (this._selected != value) {
                    this._selected = value;
                    if (this._selected) {
                        this.selectedBox.visible = true;
                        this.unSelectedBox.visible = false;
                        this.remove.visible = this.photoData.getInFrame();
                        this.place.visible = !this.photoData.getInFrame();
                    } else {
                        this.selectedBox.visible = false;
                        this.unSelectedBox.visible = true;
                    }
                }
            } else {
                this.selectedBox.visible = false;
                this.unSelectedBox.visible = true;
            }
        }

        selectedByName(name) {
            if (name != this.name) {
                this.setSelected(false);
            }
        }

        onSelectedClick() {
            if (this._selected) {
                return;
            }
            this.setSelected(true);
            EventMgr.getInstance().postEvent("USER_PHOTO_SELECTED", [this.name]);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("USER_PHOTO_SELECTED", this);
        }

        onPlaceClick() {
            let v = PhotoSystem.placePhoto(this.photoData.getId());
            if (v) {
                this.setSelected(false);
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon10");
            }
        }

        onRemoveClick() {
            this.setSelected(false);
            PhotoSystem.removePhoto(this.photoData.getId());
        }

        onDeleteClick() {
            this.setSelected(false);
            PhotoSystem.deletePhotoData(this.photoData.getId());
        }

    }

    class PhotoTalkPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onTalkBoxClick);
            this.talkBox = this.getChildByName("talkBox");
            this.talk = this.talkBox.getChildByName("talkLab");
            this.anim = new Spine();
            this.anim.loadAnim("anim1/xinshouyindao.sk");
            this.anim.playAnim("maomao1", true);
            this.anim.x = 150;
            this.anim.y = 120;
            this.talkBox.addChild(this.anim);
            this.talkLab = this.talkBox.getChildByName("talkLab");
            this.hang=1;
            this.str_1='tid#Guide_Camera_1';
            this.str_2='tid#Guide_Camera_2';
            this.str_3='tid#Guide_Camera_3';
            this.str_4='tid#Guide_Camera_4';
            this.timer = new Laya.Timer();
        }

        onEnable() {
            this.talkBox .visible=false;
            this.currentTalkId = 0;
            this.talkArr = [this.str_1,this.str_2,this.str_3,this.str_4];
            this.talkShow();
            this.label1=new Laya.Text();
            this.label1.fontSize=45;
            this.label1.wordWrap=true;
            this.label1.width=585;
        }
        onOpened(){
            Laya.timer.pause();
        }

        talkShow() {
            //哈哈哈哈
            this.lingdang.visible=false;
            if (this.currentTalkId < this.talkArr.length) {
                this.talkSt = Strings$1.get(this.talkArr[this.currentTalkId]);
                if( this.talkSt){
                    this.talkBox.visible=true; 
                }else {
                    this.talkBox.visible=false;
                }
                this.talkSarr =  this.talkSt.split("<star>"); 
                this.talkStr =  this.talkSt.replace("<star>","    ");
                this.currentCharNum = 0;
                this.timer.loop(1, this, this.showTalk);
                this.currentTalkId++;
            } else {
                Laya.timer.resume();
                this.close();
            }
        }

        //一个字一个字蹦
        showTalk(showAll = false) {
            if (showAll) {
                this.talk.text = this.talkStr;
                this.lingdaozuobiao(true);
                this.timer.clear(this, this.showTalk);
                this.talkStr = null;
                return;
            }
            if (this.talkStr == null) {
                this.timer.clear(this, this.showTalk);
                this.talkShow();
                return;
            }
            this.lingdaozuobiao();
            if(this.currentCharNum%2==0){
                ++this.currentCharNum;
            }
            this.talk.text = this.talkStr.substr(0, ++this.currentCharNum);
            if (this.currentCharNum >= this.talkStr.length+4) {
                this.timer.clear(this, this.showTalk);
                this.talkStr = null;
            }
        }

        onTalkBoxClick() {
            if (this.talkStr == null) {
                this.timer.clear(this, this.showTalk);
                this.talkStr = null;
                this.talkShow();
            } else {
                this.showTalk(true);
            }
        }

        onDisable() {
            this.currentData = null;
            Laya.timer.clearAll(this);
            this.timer.clearAll(this);
            Laya.Dialog.close("ui/CameraUI.scene");
            Laya.Dialog.open("ui/CameraUI.scene", true, null);
        }
        
        lingdaozuobiao(showAll = false){
            if(showAll){
                this.label1.text=this.talkStr;
                let hangheight=this.label1.height/45;
               // console.log('dyz',hangheight);
                if(this.talkSarr.length>1){
                    this.lingdang.visible=true;
                    this.lingdang.y=(this.talk.height-(this.talk.fontSize*hangheight))/2-5;
                    let length=new Laya.Label();
                    length.text=this.talkSarr[0];
                    length.fontSize=45;
                    this.lingdang.x=length.width;
                }
                else {
                    this.lingdang.visible=false;
                }
                return;
            }
            if(this.talkSarr.length>1){
                this.label1.text=this.talkStr.substr(0, ++this.currentCharNum);
                let hangheight=this.label1.height/45;
               // console.log('dyz',hangheight);

                this.lingdang.y=(this.talk.height-(this.talk.fontSize*hangheight))/2-5;
                let length=new Laya.Label();
                length.fontSize=45;
                length.text=this.talkSarr[0];
                this.lingdang.x=length.width;
                if(this.currentCharNum>=this.talkSarr[0].length){
                    this.lingdang.visible=true;
                }
            }
            else {
                this.lingdang.visible=false;
            }
        }

    }

    class PhotoWallDialog extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            EventMgr.getInstance().registEvent("PHOTO_START_DRAG", this, this.photoMoveStart);
            EventMgr.getInstance().registEvent("UPDATE_BOX_ANIM", this, this.playBoxAnim);
            this.photoMoveBox = this.getChildByName("photoMoveBox");
            this.photoMoveBox.mouseEnabled = false;
            this.photoBox = this.getChildByName("photoBox");
            let clickTree = this.photoBox.getChildByName("clickTree");
            this.box1 = this.getChildByName("box1");
            this.box2 = this.getChildByName("box2");
            this.box3 = this.getChildByName("box3");
            this.box4 = this.getChildByName("box4");
            clickTree.mouseEnabled = false;
            this.loadAnim();
        }

        loadAnim(){
            this.anim1 = new Spine();
            this.box1.addChild(this.anim1);
            this.anim1.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim1.playAnim("jiantoudonghua2",true);
            this.anim2 = new Spine();
            this.box2.addChild(this.anim2);
            this.anim2.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim2.playAnim("jiantoudonghua2",true);
            this.anim3 = new Spine();
            this.box3.addChild(this.anim3);
            this.anim3.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim3.playAnim("jiantoudonghua2",true);
            this.anim4 = new Spine();
            this.box4.addChild(this.anim4);
            this.anim4.loadAnim("anim3/jiayuanjiantou.sk");
            this.anim4.playAnim("jiantoudonghua2",true);
            this.playBoxAnim();
        }

        playBoxAnim(){
            this.box1.visible = false;
            this.box2.visible = false;
            this.box3.visible = false;
            this.box4.visible = false;
            let arr = PhotoSystem.getPhotoFrameData();
            if(arr[0]==null){
                this.box1.visible = true;
            }
            if(arr[1]==null){
                this.box2.visible = true;
            }
            if(arr[2]==null){
                this.box3.visible = true;
            }
            if(arr[3]==null){
                this.box4.visible = true;
            }
        }

        photoMoveStart(photo) {
            let localPoint = new Laya.Point(photo.x, photo.y);
            let globalPoint = photo.parent.localToGlobal(localPoint);
            globalPoint = this.photoMoveBox.globalToLocal(globalPoint);
            this.movePhoto = photo;
            photo.removeSelf();
            photo.x = globalPoint.x ;
            photo.y = globalPoint.y ;
            photo.scaleX = photo.scaleY = 1.2;
            this.photoMoveBox.mouseEnabled = true;
            this.photoMoveBox.addChild(photo);
            this.movePoint = new Laya.Point();
            this.setMovePoint();
            this.offsetPoint = new Laya.Point(this.movePoint.x - this.movePhoto.x, this.movePoint.y - this.movePhoto.y);
            this.photoMoveBox.on(Laya.Event.MOUSE_MOVE, this, this.onPhotoMove);
            this.photoMoveBox.on(Laya.Event.MOUSE_UP, this, this.onPhotoUp);
        }

        setMovePoint() {
            this.movePoint.x = Laya.stage.mouseX;
            this.movePoint.y = Laya.stage.mouseY;
            this.movePoint = this.photoMoveBox.globalToLocal(this.movePoint, false);
        }

        onPhotoMove() {
            this.setMovePoint();
            this.movePhoto.x = this.movePoint.x - this.offsetPoint.x;
            this.movePhoto.y = this.movePoint.y - this.offsetPoint.y;
            this.movingCheckSetPhoto();
        }

        checkIfSetPhoto() {
            let offsetX = this.photoBox.x - this.photoMoveBox.x;
            let offsetY = this.photoBox.y - this.photoMoveBox.y;
            for (let i = 0; i < this.photoBox.frames.length; i++) {
                let frame = this.photoBox.frames[i].owner;
                if (this.movePoint.x > frame.x + offsetX && this.movePoint.x < frame.x + frame.width + offsetX
                    && this.movePoint.y > frame.y + offsetY && this.movePoint.y < frame.y + frame.height + offsetY) {
                    PhotoSystem.exchangeFrameData(this.movePhoto.frameId, i);
                    return;
                }
            }
            PhotoSystem.deleataPhotoFrameData(this.movePhoto.frameId);
        }

        onPhotoUp() {
            this.checkIfSetPhoto();
            this.movePhoto.removeSelf();
            this.movePhoto.destroy();
            this.offsetPoint = null;
            this.movePoint = null;
            this.photoMoveBox.mouseEnabled = false;
        }

        movingCheckSetPhoto() {
            let offsetX = this.photoBox.x - this.photoMoveBox.x;
            let offsetY = this.photoBox.y - this.photoMoveBox.y;
            for (let i = 0; i < this.photoBox.frames.length; i++) {
                let frame = this.photoBox.frames[i].owner;
                if (this.movePoint.x > frame.x + offsetX && this.movePoint.x < frame.x + frame.width + offsetX
                    && this.movePoint.y > frame.y + offsetY && this.movePoint.y < frame.y + frame.height + offsetY) {
                    EventMgr.getInstance().postEvent("LIGHT_FRAME", [i]);
                    return;
                }
            }
            EventMgr.getInstance().postEvent("LIGHT_FRAME", [null]);
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("PHOTO_START_DRAG", this);
            EventMgr.getInstance().removeEvent("UPDATE_BOX_ANIM", this);
        }
    }

    class PickUI extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            let pick = this.getChildByName("pick");
            pick.on(Laya.Event.CLICK, this, this.onPickClick);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
            this.text1 = Strings$1.get("tid#UI_125");
            this.text2 = Strings$1.get("tid#UI_126");
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }

        onCloseClick() {

        }

        onPickClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.ComeOn);
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.ComeOn) {
                let arr = Const.getArray("NauticalADWatchTips");
                if (!arr) {
                    return;
                }
                let random = Utils.randomExt(arr.length);
                TipsPopMgr.getInstance().showTipsById(arr[random]);
                this.close();
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.ComeOn) {
                let arr = Const.getArray("NauticalADWatchTips");
                if (!arr) {
                    return;
                }
                let random = Utils.randomExt(arr.length);
                TipsPopMgr.getInstance().showTipsById(arr[random]);
                this.close();
            }
        }

            
        fail(shareId){
            if (shareId == Constants.ShareId.FishReward) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["ads_nautical_fail"]);
            }
        }

    }

    /**
    * 确认放置
    */
    class PlacePop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            let confirm = this.getChildByName("confirm");
            confirm.on(Laya.Event.CLICK, this, this.onConfirmClick);
            this.showInfo(GameContext.CurrentShopBuilding);
            this.tigaopingjia.text = Strings$1.get("tid#UI_097");
            this.xiaofeiText.text = Strings$1.get("tid#UI_098");
            this.unlockText.text = Strings$1.get("tid#UI_099");
            this.cost.text = Strings$1.get("tid#Btn_026");
        }

        showInfo(buildingId) {
            if (buildingId == null) {
                this.close();
                return;
            }
            let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
            if (item == null) {
                this.close();
                return;
            }
            this.title.text = Strings$1.get("tid#BuildingName_" + buildingId);
            this.desc1.text = Strings$1.get("tid#UI_220");
            let pid = BuildingSystem.getPidByBid(buildingId);
            if (pid == "114") {
                this.xiaofeiNum.visible = true;
                this.xiaofeiText.visible = true;
                this.xiaofeiItem.visible = true;
                this.xiaofeiText.text = Strings$1.get("tid#UI_169");
                this.xiaofeiNum.text = "+" + Strings$1.dealFishNumberText(Number(item.SpecialValue));
            } else if (item.AutoIncome && item.AutoIncome > 0) {
                this.xiaofeiNum.text = "+" + (item.AutoIncome * 60) + "/h";
                this.xiaofeiNum.visible = true;
                this.xiaofeiText.visible = true;
                this.xiaofeiItem.visible = true;
            } else {
                this.xiaofeiNum.visible = false;
                this.xiaofeiText.visible = false;
                this.xiaofeiItem.visible = false;
            }
            if (item.Limit && item.Limit > 0) {
                this.unlockText.visible = true;
                this.unlockItem.visible = true;
                this.unlockNum.visible = true;
                this.unlockNum.text = String(item.Limit);
            } else {
                this.unlockText.visible = false;
                this.unlockItem.visible = false;
                this.unlockNum.visible = false;
            }
            this.star.text = "+" + item.StarReward;
            //    this.cost.text = String(item.Cost);
            this.icon.skin = item.ShopItems;

            let scalex = 150 / this.icon.width;
            let scaley = 150 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = 1.8;
                this.icon.scaleY = 1.8;
            } else {
                this.icon.scaleX = 1.8;
                this.icon.scaleY = 1.8;
            }

            this.buildingId = buildingId;

            if (GameContext.CurrentShopBuildingBase && GameContext.CurrentShopBuildingBase.getBuildingId() == this.buildingId) {
                this.cost.text = "已放置";
                this.cost.text = Strings$1.get("tid#Btn_027");
                let confirm = this.getChildByName("confirm");
                confirm.disabled = true;
                confirm.gray = false;
            }
            this.showManyi(item);
        }

        setGray() {
            this.confirm.gray = true;
            this.confirm.disabled = true;
        }
        showManyi(building) {
            let quanzhongs;
            if (building.CatFavors.length != 0) {
                quanzhongs = building.CatFavors;
            } else {
                this.xiaolianList.visible = false;
                this.manyiImg.visible = false;
                this.left.visible = false;
                this.right.visible = false;
                this.desc1.visible = false;
                let y_ = (this.line.y - this.left.y);
                this.line.y -= y_;
                this.tigaopingjia.y -= y_;
                this.star.y -= y_;
                this.xiaofeiText.y -= y_;
                this.xiaofeiNum.y -= y_;
                this.unlockText.y -= y_;
                this.unlockNum.y -= y_;
                this.confirm.y -= y_;
                this.cost.y -= y_;
                this.xing1.y -= y_;
                this.xiaofeiItem.y -= y_;
                this.unlockItem.y -= y_;
                this.bg1.height = 520;
                return;
            }
            this.catWeightedArr = [];
            if (quanzhongs) {
                quanzhongs.forEach(element => {
                    if (element != "") {
                        this.catWeightedArr.push(element);
                    }
                });
            }
            this.xiaolianList.array = this.quanzhongSan();
            if (this.xiaolianList.length == 1) {
                this.xiaolianList.x = 255 + 110 + 150;
            }
            else if (this.xiaolianList.length == 2) {
                this.xiaolianList.x = 255 + (110 + 150) / 2;
            }else {
                this.xiaolianList.x = 255;
            }
            this.xiaolianList.renderHandler = new Laya.Handler(this, this.updateItem);
        }
        updateItem(cell, index) {
            let item = ConfigMgr$1.getInstance().getItemById("Role", cell.dataSource.split(":")[0]);
            cell.getChildByName("head").skin = "cat_collect/head/" + item.HeadIcon + ".png";
            let manyi = Number(cell.dataSource.split(":")[1]);
            cell.getChildByName("chengdu").value = manyi;
            //笑脸的分界线
            let quanzhong = Const.get('CatFavorsIconsShow');
            let quanzhongs = JSON.parse(quanzhong);
            if (manyi >= quanzhongs[0] && manyi < quanzhongs[1]) {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/xiaoxiao.png";
            } else if (manyi >= quanzhongs[1] && manyi < quanzhongs[2]) {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/erxiao.png";
            } else {
                cell.getChildByName("xiaolian").skin = "shopui/manyi/daxiao.png";
            }
        }
        //取权重前三的
        quanzhongSan() {
            let catArr = this.catWeightedArr;
            for (let i = 0; i < catArr.length - 1; i++) {
                for (let j = 0; j < catArr.length - 1 - i; j++) {
                    if (Number(catArr[j].split(":")[1]) < Number(catArr[j + 1].split(":")[1])) {
                        let temp = catArr[j];
                        catArr[j] = catArr[j + 1];
                        catArr[j + 1] = temp;
                    }
                }
            }
            let quanzhongArr = [];
            if (catArr[0])
                quanzhongArr.push(catArr[0]);
            if (catArr[1])
                quanzhongArr.push(catArr[1]);
            if (catArr[2])
                quanzhongArr.push(catArr[2]);
            return quanzhongArr;
        }

        onOpened(param) {
            if (Number(this.buildingId) == 1060) {
                let guide = GuideSystem.getGuide("GuideGroup_2");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.confirm, null, [this.confirm, this.cost]);
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.cost]);
                }
            } else if (Number(this.buildingId) == 1030) {
                let guide = GuideSystem.getGuide("GuideGroup_5");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.confirm, null, [this.confirm, this.cost]);
                }
            } else if (Number(this.buildingId) == 2080) {
                let guide = GuideSystem.getGuide("GuideGroup_8");
                if (guide) {
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.cost]);
                }
            } else if (Number(this.buildingId) == 2020) {
                let guide = GuideSystem.getGuide("GuideGroup_11");
                if (guide) {
                    let rect = new Laya.Rectangle(69, 444, 986, 1132);
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.confirm, null, [this.confirm, this.cost]);
                }
            }
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/shopPop.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onConfirmClick() {
            this.close();
            Laya.Dialog.closeAll();
            if (GameContext.CurrentShopBuildingBase) {
                BuildingSystem.placeBuilding(this.buildingId, GameContext.CurrentShopBuildingBase.getId(), false);
                SoundManager.getInstance().play("Sound_placeBuilding");
            } else {
                GameContext.PlacingBuilding = this.buildingId;
                GameContext.PlacingNewBuilding = null;
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                EventMgr.getInstance().postEvent("PLACE_BUILDING", this.buildingId);
                SoundManager.getInstance().play("Sound_clickSound");
            }
        }
    }

    class PowerGamePop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            this.gameBox = this.getChildByName("gameBox");
            let bg = this.gameBox.getChildByName("bg");
            // bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.quit.on(Laya.Event.CLICK, this, this.onQuitClick);
            this.countdown = this.gameBox.getChildByName("countdown");
            this.startBtn = this.gameBox.getChildByName("startBtn");
            this.startBtn.on(Laya.Event.CLICK, this, this.onStartCick);
            this.gameAnimBox = this.gameBox.getChildByName("gameAnimBox");
            this.koAnimBox = this.gameBox.getChildByName("koAnimBox");
            this.tipsAnimBox = this.gameBox.getChildByName("tipsAnimBox");
            this.gameAnim = new Laya.Skeleton();
            this.gameAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.gameAnim.play("duibo", true);
                this.gameAnimBox.addChild(this.gameAnim);
            }));
            this.playerAnim = new Laya.Skeleton();
            this.playerAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.playerAnim.play("huangdian1", true);
                this.gameAnimBox.addChild(this.playerAnim);
            }));
            this.aiAnim = new Laya.Skeleton();
            this.aiAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.aiAnim.play("zidian1", true);
                this.gameAnimBox.addChild(this.aiAnim);
            }));
            this.lightAnim = new Laya.Skeleton();
            this.lightAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.lightAnim.play("guangyun", false);
                this.lightAnim.on(Laya.Event.STOPPED, this, this.hideLight);
                this.gameAnimBox.addChild(this.lightAnim);
            }));
            this.koAnim = new Laya.Skeleton();
            this.koAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.koAnim.play("go", true);
                this.koAnimBox.addChild(this.koAnim);
            }));
            this.koAnim1 = new Laya.Skeleton();
            this.koAnim1.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.koAnim1.play("KO", true);
                this.koAnimBox.addChild(this.koAnim1);
            }));
            this.tipsAnim = new Laya.Skeleton();
            this.tipsAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.tipsAnim.play("dianjitishi", true);
                this.tipsAnimBox.addChild(this.tipsAnim);
            }));

            this.touchBox = this.getChildByName("touchBox");
            this.touchBox.on(Laya.Event.CLICK, this, this.onTouch);
            this.touchAnimBox = this.touchBox.getChildByName("touchAnimBox");
            this.touchAnim = new Laya.Skeleton();
            this.touchAnim.load("anim2/duibo.sk", Laya.Handler.create(this, function () {
                this.touchAnim.play("dianji", false);
                this.touchAnimBox.addChild(this.touchAnim);
                this.touchAnimBox.on(Laya.Event.STOPPED, this, this.hideTouchAnim);
            }));


            this.resultBox = this.getChildByName("resultBox");
            this.winBox = this.resultBox.getChildByName("winBox");
            this.winImage = this.winBox.getChildByName("win");
            this.KOImage = this.winBox.getChildByName("KO");
            this.rewardNum = this.winBox.getChildByName("rewardNum");
            this.rewardText = this.winBox.getChildByName("rewardText");
            this.rewardImage = this.winBox.getChildByName("rewardImage");


            this.getBtn = this.winBox.getChildByName("getBtn");
            this.getBtn.on(Laya.Event.CLICK, this, this.onGetBtnClick);

            //广告分享
            this.shareDoubleBtn = this.winBox.getChildByName("shareBtn");
            this.shareDoubleBtn.on(Laya.Event.CLICK, this, this.onShareDoubleBtnClick);
            this.videoDoubleBtn = this.winBox.getChildByName("videoBtn");
            this.videoDoubleBtn.on(Laya.Event.CLICK, this, this.onVideoDoubleBtnClick);

            this.failBox = this.resultBox.getChildByName("failBox");
            let overCloseBtn = this.failBox.getChildByName("overCloseBtn");
            overCloseBtn.on(Laya.Event.CLICK, this, this.onOverBtnClick);


            this.overBtn = this.failBox.getChildByName("overBtn");
            this.overBtn.on(Laya.Event.CLICK, this, this.onOverBtnClick);

            // this.videoBtn = this.failBox.getChildByName("video");
            // this.videoBtn.on(Laya.Event.CLICK, this, this.onVideoBtnClick);
            // this.shareBtn = this.failBox.getChildByName("share");
            // this.shareBtn.on(Laya.Event.CLICK, this, this.onShareBtnClick);

            //video.on(Laya.Event.CLICK, this, this.onOverBtnClick);

            this.failBox = this.resultBox.getChildByName("failBox");
            this.failImage = this.failBox.getChildByName("fail");
            this.drawImage = this.failBox.getChildByName("draw");

            this.touchPoint = new Laya.Point();
            this.label1 = this.winBox.getChildByName("label1");
            this.label1.text = Strings$1.get("tid#UI_040");
            this.rewardText.text = Strings$1.get("tid#UI_100");
            this.label2 = this.failBox.getChildByName("label2");
            this.label2.text = Strings$1.get("tid#UI_017");
            this.overBtn.label = Strings$1.get("tid#Btn_028");
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.quit.visible = true;
            this.playTimes = 0;
            this.gameBox.visible = true;
            this.startBtn.visible = true;
            this.countdown.visible = false;
            this.gameAnimBox.visible = false;
            this.playerAnim.visible = false;
            this.aiAnim.visible = false;
            this.lightAnim.visible = false;
            this.koAnimBox.visible = false;
            this.tipsAnimBox.visible = false;

            this.touchBox.visible = false;
            this.touchAnimBox.visible = false;

            this.resultBox.visible = false;
            this.winBox.visible = false;
            this.failBox.visible = false;
            this.failImage.visible = false;
            this.drawImage.visible = false;

            /**0:平局,1:负,2:胜,3:KO */
            this._resultType = null;
            this.minigameData = this._param;
            this.playerTouchTimes = 0;
            this.aiTouchTimes = 0;
            /**20次100% */
            this.progress = 0;
            this.maxTime = Number(Const.get("GameStrengthTime")) * 1000;
            this.KO_Times = Number(Const.get("GameStrengthKOTimes"));
            this.aiNextTouchTime = 0;
            let reward_p = this.minigameData.getReward();
            if (reward_p.WinReward[0].id == Constants.RewardType.DIAMOND) {
                this.rewardImage.skin = "topui/zuanshi.png";
            } else if (reward_p.WinReward[0].id == Constants.RewardType.STAR) {
                this.rewardImage.skin = "topui/xingxing.png";
            }
            this.rewardNum.text = reward_p.WinReward[0].amount + (reward_p.WinRewardRate ? reward_p.WinRewardRate : 0) * PetSystem.getMaxValueBase();
            this.rewardText.x = (this.winBox.width - this.rewardText.width - this.rewardNum.width - this.rewardImage.width * this.rewardImage.scaleX) / 2;
            this.rewardImage.x = this.rewardText.x + this.rewardText.width;
            this.rewardNum.x = this.rewardImage.x + this.rewardImage.width * this.rewardImage.scaleX;
            let difficulty = MiniGameSystem.getOtherData().powerGameData.difficulty;
            this.difficultyConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameStrength", String(difficulty));


            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);

            // this.fristAD = true;
            if (SdkHelper.hasVideo()) {
                // this.shareBtn.visible = false;
                // this.videoBtn.visible = true;
                this.shareDoubleBtn.visible = false;
                this.videoDoubleBtn.visible = true;
            } else {
                // this.shareBtn.visible = true;
                // this.videoBtn.visible = false;
                this.shareDoubleBtn.visible = true;
                this.videoDoubleBtn.visible = false;
            }
        }

        onDisable() {
            Laya.timer.clearAll(this);
            Laya.Tween.clearAll(this.gameAnimBox);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
        }

        onStartCick() {
            this.quit.visible = false;
            this.playTimes++;
            this.startBtn.visible = false;
            this.koAnimBox.visible = true;
            this, this.koAnim.visible = true;
            this, this.koAnim1.visible = false;
            this.koAnim.play("go", false);
            this.koAnim.once(Laya.Event.STOPPED, this, this.readyStart);
        }

        onQuitClick(){
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 3);
            this.close();
        }

        onTouch() {
            this.touchPoint.x = Laya.stage.mouseX;
            this.touchPoint.y = Laya.stage.mouseY;
            this.touchPoint = this.touchBox.globalToLocal(this.touchPoint);
            this.touchAnimBox.x = this.touchPoint.x;
            this.touchAnimBox.y = this.touchPoint.y;
            this.touchAnimBox.visible = true;
            this.touchAnim.play("dianji", false);

            this.playerTouchTimes++;
            this.lightAnim.visible = true;
            this.lightAnim.play("guangyun", false);
            if (this.playerTouchTimes == 1) {
                this.startPlay();
            }
        }

        readyStart() {
            this._time = 0;
            this.koAnimBox.visible = false;
            this.gameAnimBox.visible = true;
            this.playerAnim.visible = true;
            this.aiAnim.visible = true;
            this.touchBox.visible = true;
            this.tipsAnimBox.visible = true;
            this.tipsAnim.play("dianjitishi", true);
        }

        startPlay() {
            // this.tipsAnimBox.visible = false;
            // this.tipsAnim.stop();
            this.countdown.visible = true;
            this.minTime = 50;
            this.currentTime = Laya.timer.currTimer;
            this.countdown.value = Math.floor(this.maxTime / 1000);
            Laya.timer.loop(this.minTime, this, this.timerUpdata);
        }

        timerUpdata() {
            this.randomAiTouch();
            this.checkWin();
            this._time += Laya.timer.currTimer - this.currentTime;
            this.countdown.value = Math.floor((this.maxTime - this._time) / 1000);
            this.currentTime = Laya.timer.currTimer;
            Laya.Tween.clearAll(this.gameAnimBox);
            Laya.Tween.to(this.gameAnimBox, { x: this.gameBox.width / 2 + this.progress * 475 }, this.minTime);
        }

        checkWin() {
            this.progress = Math.min(1, (this.playerTouchTimes - this.aiTouchTimes) / this.KO_Times);
            this.progress = Math.max(-1, this.progress);
            if (this.progress == 1 || this._time >= this.maxTime) {
                if (this.progress == 1) {
                    this._resultType = Constants.MiniGameResult.KO;
                } else if (this.progress == 0) {
                    this._resultType = Constants.MiniGameResult.Draw;
                } else if (this.progress > 0 && this.progress != 1) {
                    this._resultType = Constants.MiniGameResult.Win;
                } else {
                    this._resultType = Constants.MiniGameResult.Fail;
                }
                this.gameComplete();
            }
        }

        randomAiTouch() {
            if (this.aiNextTouchTime <= this._time) {
                this.aiTouchTimes++;
                this.lightAnim.visible = true;
                this.lightAnim.play("guangyun", false);
                let speed = 1;
                if (this.progress < 0) {
                    speed = Utils.randomExt2(this.difficultyConfig.ClickSpeed0[0], this.difficultyConfig.ClickSpeed0[1]);
                } else if (this.progress >= 0 && this.progress < 0.5) {
                    speed = Utils.randomExt2(this.difficultyConfig.ClickSpeed1[0], this.difficultyConfig.ClickSpeed1[1]);
                } else if (this.progress >= 0.5 && this.progress < 0.8) {
                    speed = Utils.randomExt2(this.difficultyConfig.Speed2[0], this.difficultyConfig.Speed2[1]);
                } else {
                    speed = Utils.randomExt2(this.difficultyConfig.Speed3[0], this.difficultyConfig.Speed3[1]);
                }
                this.aiNextTouchTime += Math.floor(1000 / speed);
            }
        }

        hideTouchAnim() {
            this.touchAnimBox.visible = false;
        }

        hideLight() {
            this.lightAnim.visible = false;
        }

        gameComplete() {
            this.touchBox.visible = false;
            Laya.timer.clearAll(this);
            if (this._resultType == Constants.MiniGameResult.KO) {
                this.koAnimBox.visible = true;
                this.koAnim.visible = false;
                this.koAnim1.visible = true;
                this.koAnim1.play("KO", false);
                this.koAnim1.once(Laya.Event.STOPPED, this, function () {
                    Laya.timer.once(400, this, function () {
                        this.gameBox.visible = false;
                        this.resultBox.visible = true;
                        this.winBox.visible = true;
                        this.failBox.visible = false;
                        this.KOImage.visible = true;
                        this.winImage.visible = false;
                    });
                });

            }
            else if (this._resultType == Constants.MiniGameResult.Draw) {
                this.gameBox.visible = false;
                this.resultBox.visible = true;
                this.failBox.visible = true;
                this.winBox.visible = false;
                this.drawImage.visible = true;
                this.failImage.visible = false;
            } else if (this._resultType == Constants.MiniGameResult.Fail) {
                this.gameBox.visible = false;
                this.resultBox.visible = true;
                this.failBox.visible = true;
                this.winBox.visible = false;
                this.drawImage.visible = false;
                this.failImage.visible = true;
            } else if (this._resultType == Constants.MiniGameResult.Win) {
                this.gameBox.visible = false;
                this.resultBox.visible = true;
                this.winBox.visible = true;
                this.failBox.visible = false;
                this.KOImage.visible = false;
                this.winImage.visible = true;
            } else {
                this.gameBox.visible = false;
                this.resultBox.visible = true;
                this.winBox.visible = true;
                this.failBox.visible = false;
                this.KOImage.visible = true;
                this.winImage.visible = false;
            }
        }

        onBackBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 2, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            MiniGameSystem.addPowerPlayTimes(this._resultType);
            this.close();
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData, this._resultType]);
        }

        onGetBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            MiniGameSystem.addPowerPlayTimes(this._resultType);
            this.close();
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData, this._resultType]);
        }

        onOverBtnClick() {
            MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 0, Constants.FetchType.DIRECT, Laya.Handler.create(this, this.showFly, [Constants.FetchType.DIRECT]));
            MiniGameSystem.addPowerPlayTimes(this._resultType);
            this.close();
            Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData, this._resultType]);
        }

        onShareBtnClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.PowerGame);
        }

        onVideoBtnClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.PowerGame);
        }
        onShareDoubleBtnClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.PowerGameDouble);
        }

        onVideoDoubleBtnClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.PowerGameDouble);
        }
        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.PowerGame) {
                this.resetStart();
            } else if (shareId == Constants.ShareId.PowerGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                MiniGameSystem.addPowerPlayTimes(this._resultType);
                this.close();
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData, this._resultType]);
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.PowerGame) {
                this.resetStart();
            } else if (shareId == Constants.ShareId.PowerGameDouble) {
                MiniGameSystem.finishGame(this.minigameData.getId(), this.minigameData.getListId(), 1, Constants.FetchType.AD, Laya.Handler.create(this, this.showFly, [Constants.FetchType.AD]));
                MiniGameSystem.addPowerPlayTimes(this._resultType);
                this.close();
                Laya.Dialog.open("ui/MiniGameTalkPop.scene", true, [this.minigameData, this._resultType]);
            }
        }
        
        resetStart() {
            this.touchBox.visible = true;
            this.gameBox.visible = true;
            this.startBtn.visible = true;
            this.countdown.visible = false;
            this.gameAnimBox.visible = false;
            this.playerAnim.visible = false;
            this.aiAnim.visible = false;
            this.lightAnim.visible = false;
            this.koAnimBox.visible = false;
            this.tipsAnimBox.visible = false;

            this.touchBox.visible = false;
            this.touchAnimBox.visible = false;

            this.resultBox.visible = false;
            this.winBox.visible = false;
            this.failBox.visible = false;
            this.failImage.visible = false;
            this.drawImage.visible = false;

            /**0:平局,1:负,2:胜,3:KO */
            this._resultType = null;
            this.playerTouchTimes = 0;
            this.aiTouchTimes = 0;
            /**20次100% */
            this.progress = 0;
            this.maxTime = Number(Const.get("GameStrengthTime")) * 1000;
            this.KO_Times = Number(Const.get("GameStrengthKOTimes"));
            this.aiNextTouchTime = 0;
            let reward_p = this.minigameData.getReward();
            this.rewardNum.text = reward_p.WinReward[0].amount + reward_p.WinRewardRate * PetSystem.getMaxValueBase();
            //= this.minigameData.getReward().WinReward[0].amount;
            let difficulty = MiniGameSystem.getOtherData().powerGameData.difficulty;
            this.difficultyConfig = ConfigMgr$1.getInstance().getItemById("SpecialGameStrength", String(difficulty));
            this.gameAnimBox.x = 913;
            // this.fristAD = false;

            // if (this.fristAD == false) {
            //     this.shareBtn.visible = false;
            //     this.videoBtn.visible = false;
            //     this.overBtn.visible = true;

            // }

        }
        showFly(fetchType, res) {
            let point = new Laya.Point(this.rewardImage.x, this.rewardImage.x);
            point = this.rewardImage.parent.localToGlobal(point);
            let rewards = res.rewards.rewards;
            for (let i in rewards) {
                if (rewards[i].amount <= 0) {
                    continue;
                }
                if (rewards[i].id == Constants.RewardType.FISH) {
                    if (fetchType == Constants.FetchType.AD) {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE3, point.x, point.y);
                    } else {
                        ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, rewards[i].amount, Constants.FlyType.TYPE5, point.x, point.y);
                    }
                } else {
                    ItemFlyMgr.getInstance().fly(rewards[i].id, rewards[i].amount, Constants.FlyType.ZCM, point.x, point.y);
                }
            }
        }
        onCloseClick() {
            if (this.playTimes == 0) {
                this.close();
            }
        }
    }

    /**
     * 确认购买
     */
    class ProjectLevelUpPop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.confirm = this.getChildByName("confirm");
            this.confirm.on(Laya.Event.CLICK, this, this.onConfirmClick);
            // this.confirm.visible = true;
            this.sbcj = this.getChildByName("sbcj");
            // this.sbcj.on(Laya.Event.CLICK, this, this.onSbcjClick);
            this.vbcj = this.getChildByName("vbcj");
            this.sbcj.visible = false;
            this.vbcj.visible = false;
            // this.vbcj.on(Laya.Event.CLICK, this, this.onVbcjClick);
            // this.vbcj.visible = false;
            // this.sbcj.visible = false;
            this.showInfo(GameContext.CurrentProjectListId);
            this.highest.text = Strings$1.get("tid#UI_008");
            // this.timeLabel.text = Strings.get("tid#UI_002");
            // this.fishLabel.text = Strings.get("tid#UI_003");
            // EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            // EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onAdClick);
            this.menpiaoLabel.text = Strings$1.get("tid#UI_061");
            this.starLabel.text = Strings$1.get("tid#UI_062");
            this.sheshiLabel.text = Strings$1.get("tid#UI_159");
            //this.reason.text = Strings.get("tid#UI_103");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onAdClick);
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
                EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_COOK_ITEM", [true]);
            }
        }

        onSbcjClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.FoodLevelUp);
        }

        onVbcjClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.FoodLevelUp);
        }

        showInfo(projectListId) {
            if (projectListId == null) {
                this.close();
                return;
            }
            let project = ProjectSystem.getCurProjectInList(projectListId);
            this.title.text = Strings$1.get(project.getName());
            this.desc1.text = Strings$1.get(project.getDesc1());

            //  this.time.text = food.getCookingTime()+"s";
            this.fish.text = project.getAutoIncome() * 60 + '/h';
            this.icon.skin = project.getShopItems();

            let nextProject = ProjectSystem.getNextLevelUpProjectInList(projectListId);




            if (nextProject == null) {
                this.confirm.disabled = true;
                this.btnBeike.visible = this.cost.visible = this.yanjiu.visible = false;
                this.highest.visible = true;
                this.fishArrow.visible = false;
                this.nextFish.visible = false;
                this.starLabel.visible = false;
                this.star.visible = false;
                this.starImage.visible = false;
                this.sheshiLabel.visible = false;
                this.menpiao.visible = false;//门票储量
                this.chuliangyu.visible = false;
                this.chuliangjian.visible = false;
                this.chuliangnex.visible = false;
            } else {
                this.confirm.disabled = false;
                this.btnBeike.visible = this.cost.visible = this.yanjiu.visible = true;
                this.highest.visible = false;
                this.cost.text = Strings$1.dealFishNumberText(nextProject.getCost());
                this.setGray(ProjectSystem.checkLevelUpUnAvailable(projectListId));
                this.fishArrow.visible = true;
                this.nextFish.visible = true;
                this.fishArrow.x = this.fish.x + this.fish.width + 5;
                this.nextFish.x = this.fishArrow.x + this.fishArrow.width * this.fishArrow.scaleX + 5;
                this.nextFish.text = nextProject.getAutoIncome() * 60 + '/h';
                this.starLabel.visible = true;
                this.star.visible = true;
                this.star.text = String(nextProject.getStarLimit());
                this.starImage.visible = true;
                this.sheshiLabel.visible = true;
                this.menpiao.visible = true;//门票储量
                this.menpiao.text = Strings$1.dealFishNumberText(project.getSpecialValue());
                this.chuliangnex.text = Strings$1.dealFishNumberText(nextProject.getSpecialValue());
                this.chuliangyu.visible = true;
                this.chuliangjian.visible = true;
                this.chuliangnex.visible = true;
                this.chuliangjian.x = this.menpiao.x + this.menpiao.width + 5;
                this.chuliangnex.x = this.chuliangjian.x + this.chuliangjian.width * this.chuliangjian.scaleX + 5;
                if (Number(nextProject.getSpecialValue()) == 0 && Number(project.getSpecialValue() == 0)) {
                    this.chuliangnex.visible = false;
                    this.chuliangjian.visible = false;
                    this.chuliangyu.visible = false;
                    this.menpiao.visible = false;
                    this.chuliangyu.visible = false;
                    this.sheshiLabel.visible = false;
                } else {
                    this.chuliangnex.visible = true;
                    this.chuliangjian.visible = true;
                    this.chuliangyu.visible = true;
                    this.menpiao.visible = true;
                    this.chuliangyu.visible = true;
                    this.sheshiLabel.visible = true;
                }
                // if(project.)


                //this.checkAdBtnShow(projectListId);
            }

            if (project.isLocked()) {
                //this.starLabel.text = Strings.get("tid#UI_004");
                this.yanjiu.text = '开发';
                this.yanjiu.text = Strings$1.get("tid#Btn_006");
                this.fishArrow.visible = false;
                this.nextFish.visible = false;
            } else {
                // this.starLabel.text = Strings.get("tid#UI_005");
                this.yanjiu.text = '升级';
                this.yanjiu.text = Strings$1.get("tid#Btn_007");
            }

            let scalex = 180 / this.icon.width;
            let scaley = 180 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
            //使用新按钮后 坐标偏移了  
            this.btnBeike.x = this.confirm.x + (this.confirm.width * this.confirm.scaleX - this.btnBeike.width * this.btnBeike.scaleX - this.cost.width - this.yanjiu.width - 10) / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
            this.cost.x = this.btnBeike.x + this.btnBeike.width * this.btnBeike.scaleX + 5;
            this.yanjiu.x = this.cost.x + this.cost.width + 5;

            this.projectListId = projectListId;
        }

        setGray(ret) {
            if (ret) {
                this.confirm.gray = true;
                this.confirm.disabled = true;
                this.reason.visible = true;
                this.reason.text = ret;
                this.reason.visible = true;

                this.reason.x = 563.5 - this.reason.width / 2;
            } else {
                this.confirm.gray = false;
                this.confirm.disabled = false;
                this.reason.visible = false;
            }
        }

        onOpened(param) {
            // if (GuideSystem.isGuiding()) {
            //     if (GuideSystem.getConditionGroupComplete("GuideGroup_3")) {
            //         let guide = GuideSystem.getGuide("GuideGroup_3");
            //         if (guide) {
            //             guide.registerObj("btn3", this)
            //             guide.registerClick("btn3", this.confirm, null, [this.confirm, this.btnFish, this.cost, this.yanjiu]);
            //         }
            //     }
            // }
        }

        onCloseClick() {
            this.close();
            // Laya.Dialog.open("ui/shopPop.scene");
        }

        onConfirmClick() {
            this.close();

            let projectListId = this.projectListId;
            let project = ProjectSystem.getCurProjectInList(projectListId);
            if (project.isLocked()) {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon53");
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon54");
            }
            ProjectSystem.projectLevelUp(projectListId);
            SoundManager.getInstance().play("Sound_buyItemSuccess");
        }

        checkAdBtnShow(projectListId) {
            this.vbcj.visible = false;
            this.sbcj.visible = false;
            this.confirm.visible = true;
            return;
            // let needFish = FoodSystem.checkAdShowFish(foodListId) ? FoodSystem.checkAdShowFish(foodListId)[0] : 0;
            // let costFish = FoodSystem.checkAdShowFish(foodListId) ? FoodSystem.checkAdShowFish(foodListId)[1] : 0;
            // let yugan = costFish - needFish;
            // let adShoppingCondition = Const.get("ADShoppingCondition");
            // let maxAd = Number(Const.get("ADShoppingTimes"));
            // let adShoppingRange =JSON.parse(Const.get("ADShoppingRange"));
            // let showAd=false;
            // let buildingNum= BuildingDataMgr.getInstance().getNowBuildingNumber();
            // for (let i = 0; i < adShoppingRange.length; i++) {
            //     let min = adShoppingRange[i].BUILDING_AMOUNT[0];
            //     let max = adShoppingRange[i].BUILDING_AMOUNT[1];
            //     if (max < 0) {
            //         if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
            //             showAd = true;
            //             break;
            //         }
            //     } else {
            //         if (needFish && adShoppingRange[i].Fish >= needFish && buildingNum <= max && buildingNum >= min && yugan>=costFish*adShoppingCondition) {
            //             showAd = true;
            //             break;
            //         }
            //     }
            // }
            // if (showAd && PlayerSystem.getMakeUpTimes() < maxAd) {
            //     if (SdkHelper.hasVideo()) {
            //         this.vbcj.visible = true;
            //         this.sbcj.visible = false;
            //         let cost=this.vbcj.getChildByName("cost");
            //         cost.text = this.cost.text + "研究";
            //         let adImage = this.vbcj.getChildByName("adImage");
            //         let fish = this.vbcj.getChildByName("btnFish");
            //         fish.x = 15 + (adImage.x - fish.width - cost.width - 20) / 2;
            //         cost.x = fish.x + fish.width + 5;
            //     } else {
            //         this.vbcj.visible = false;
            //         this.sbcj.visible = true;
            //         let cost = this.sbcj.getChildByName("cost");
            //         cost.text = this.cost.text + "研究";
            //         let adImage = this.sbcj.getChildByName("adImage");
            //         let fish = this.sbcj.getChildByName("btnFish");
            //         fish.x = 15 + (adImage.x - fish.width - cost.width - 20) / 2;
            //         cost.x = fish.x + fish.width + 5;
            //     }
            //     this.confirm.visible = false;
            // } else {
            //     this.vbcj.visible = false;
            //     this.sbcj.visible = false;
            //     this.confirm.visible = true;
            // }
        }

        done(shareId) {
            if (shareId == Constants.ShareId.FoodLevelUp) {
                this.close();
                let food = FoodSystem$1.getCurFoodInList(this.foodListId);
                if (food.isLocked()) {
                    TaskMainSystem.addProgress("LOBBY_FUN", 1);
                } else {
                    TaskMainSystem.addProgress("LOBBY_FUN", 1);
                }
                FoodSystem$1.foodLevelUp(this.foodListId, Constants.FetchType.AD);
                SoundManager.getInstance().play("Sound_buyItemSuccess");
            }
        }

        onAdClick(shareId) {
            if (shareId == Constants.ShareId.FoodLevelUp) {
                this.close();
                let food = FoodSystem$1.getCurFoodInList(this.foodListId);
                if (food.isLocked()) {
                    TaskMainSystem.addProgress("LOBBY_FUN", 1);
                } else {
                    TaskMainSystem.addProgress("LOBBY_FUN", 1);
                }
                FoodSystem$1.foodLevelUp(this.foodListId, Constants.FetchType.AD);
                SoundManager.getInstance().play("Sound_buyItemSuccess");
            }
        }

        checkShowWeakGuideAnim(guideEvent) {
            if (!GuideSystem.isGuiding() && TaskMainSystem.getIsTasking()) {
                let task = TaskMainSystem.getCurrentData();
                if (task.getGuide() == guideEvent && !TaskMainSystem.checkEnought()) {
                    if (this.weakAnim == null || this.weakAnim.destroyed) {
                        this.weakAnim = new Spine();
                        this.weakAnim.loadAnim("anim1/xinshouyindao.sk");
                        this.weakAnim.playAnim("zhuazi2", true);
                        this.weakAnim.x = this.confirm.x + this.confirm.width * this.confirm.scaleX / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
                        this.weakAnim.y = this.confirm.y + this.confirm.height * this.confirm.scaleX / 2 - this.confirm.anchorX * this.confirm.width * this.confirm.scaleX;
                        this.weakAnim.zOrder = 5;
                        this.addChild(this.weakAnim);
                        EventMgr.getInstance().postEvent("SHOW_WEAK_GUIDE_COOK_ITEM", [false]);
                        return;
                    }
                }
            }
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
                this.weakAnim = null;
            }
        }
    }

    class SailDialog extends Laya.Dialog {

        constructor() {
            super();
        }

        onEnable(){
            this.clickSign = 1;
            this.sailIndex = this._param;
            this.NauticalSailLineADCondition = JSON.parse(Const.get("NauticalSailLineADCondition"));
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.video.on(Laya.Event.CLICK,this,this.onVideoClick);
            this.share.on(Laya.Event.CLICK,this,this.onShareClick);
            this.updateView();
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
        }

        updateView(){
            this.nauticalInfo = NauticalSystem$1.NauticalInfo[this.sailIndex];
            this.adLockMax = this.NauticalSailLineADCondition[this.sailIndex];
            let str = this.adLockMax - this.nauticalInfo.adLock + "/" + this.adLockMax;
            if(this.nauticalInfo.adLock == 0){
                this.close();
            }
            else {
                if(SdkHelper.hasVideo()) {
                    this.share.visible = false;
                    this.video.visible = true;
                    this.detail.text = Strings$1.get("tid#UI_165",[str]);
                }else {
                    this.share.visible = true;
                    this.video.visible = false;
                    this.detail.text = Strings$1.get("tid#UI_166",[str]);
                }
            }
        }

        onVideoClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.showRewardVideoAd(Constants.ShareId.SailUnlock);
            } 
        }

        onShareClick(){
            if(this.clickSign > 0){
                this.changeClickSign();
                SdkHelper.shareAppMessage(Constants.ShareId.SailUnlock);
            }
        }

        done(shareId) {
            if(shareId == Constants.ShareId.SailUnlock) {
                let str = this.adLockMax - this.nauticalInfo.adLock + 1 + "/" + this.adLockMax;
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon64",[str]);
                NauticalSystem$1.unlockNautical(this.sailIndex,Laya.Handler.create(this,this.updateView));
            }
        }


        changeClickSign(){
            this.clickSign = 0;
            Laya.timer.once(500,this,function(){
                this.clickSign = 1;
            });
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
        }
    }

    /**
    * 设置
    */
    class SettingPop extends Laya.Dialog {
        constructor() { super(); }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,1);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.close);
            this.music_btn = this.getChildByName("music_btn");
            this.music_btn.on(Laya.Event.CLICK, this, this.musicSwitch);
            this.sound_btn = this.getChildByName("sound_btn");
            this.sound_btn.on(Laya.Event.CLICK, this, this.soundSwitch);
            this.music_btn.selected = SoundManager.getInstance().getMusicMuted();
            this.sound_btn.selected = SoundManager.getInstance().getSoundMuted();
            this.danmu_btn=this.getChildByName('danmu_btn');
            this.danmu_btn.on(Laya.Event.CLICK, this, this.danmuSetting);
            this.danmu_btn.selected=BarrageMgr.getInstance().getOpenDown();
            this.huazhi_btn=this.getChildByName('huazhi_btn');
            this.huazhi_btn.selected=!this.gethuazhi();
            this.huazhi_btn.on(Laya.Event.CLICK, this, this.sethuazhi);
            this.feedback_btn = this.getChildByName("feedback_btn");
            this.feedback_btn.on(Laya.Event.CLICK, this, this.feedback);
            this.suggest_btn = this.getChildByName("suggest_btn");
            this.suggest_btn.on(Laya.Event.CLICK, this, this.suggest);
            this.gaihuazhi=this.huazhi_btn.selected;
            this.UID.value=System.getRid().split('_')[0];
            if (Laya.Browser.onMiniGame) {
                let feedbackPoint = new Laya.Point(this.feedback_btn.x - (this.feedback_btn.pivotX ? this.feedback_btn.pivotX * this.feedback_btn.scaleX : (this.feedback_btn.anchorX ? this.feedback_btn.anchorX * this.feedback_btn.width * this.feedback_btn.scaleX : 0)),
                this.feedback_btn.y - (this.feedback_btn.pivotY ? this.feedback_btn.pivotY * this.feedback_btn.scaleY : (this.feedback_btn.anchorY ? this.feedback_btn.anchorY * this.feedback_btn.height * this.feedback_btn.scaleY : 0)));
                let feedbackRightDown = new Laya.Point(feedbackPoint.x + this.feedback_btn.width * this.feedback_btn.scaleX, feedbackPoint.y + this.feedback_btn.height * this.feedback_btn.scaleY);
                feedbackPoint = this.localToGlobal(feedbackPoint);
                feedbackRightDown = this.localToGlobal(feedbackRightDown);
                let wxWidth = wx.getSystemInfoSync().windowWidth;
                let wxHeight = wx.getSystemInfoSync().windowHeight;
                let btn_left = feedbackPoint.x / Laya.stage.width * wxWidth;
                let btn_top = feedbackPoint.y / Laya.stage.height * wxHeight;
                let btn_width = (feedbackRightDown.x - feedbackPoint.x) / Laya.stage.width * wxWidth;
                let btn_height = (feedbackRightDown.y - feedbackPoint.y) / Laya.stage.height * wxHeight;
                this.wxButton = wx.createFeedbackButton({
                    type: 'text',
                    text: '',
                    style: {
                        left: btn_left,
                        top: btn_top,
                        width: btn_width,
                        height: btn_height,
                        lineHeight: 40,
                        backgroundColor: '',
                        color: '#ffffff',
                        textAlign: 'center',
                        fontSize: 16,
                        borderRadius: 4
                    }
                });
                let self = this;
                this.wxButton.onTap(function () {
                    self.feedback_btn.selected = true;
                    self.close();
                });
            }
        }

        onOpened(param) {
            if (Laya.Browser.onMiniGame) {
                this.wxButton.show();
            }
        }

        onClosed() {
            if (Laya.Browser.onMiniGame) {
                this.wxButton.hide();
            }
            this.feedback_btn.selected = false;
        }



        musicSwitch() {
            SoundManager.getInstance().setMusicMuted(!SoundManager.getInstance().getMusicMuted());
        }

        soundSwitch() {
            SoundManager.getInstance().setSoundMuted(!SoundManager.getInstance().getSoundMuted());
        }
        danmuSetting(){
             BarrageMgr.getInstance().setOpenDown(!BarrageMgr.getInstance().getOpenDown());
        }

        feedback() {
            this.close();
        }

        suggest() {
            Laya.Dialog.close("ui/SuggestPop.scene");
            Laya.Dialog.open("ui/SuggestPop.scene", true);
        }
        gethuazhi(){
            if(Laya.LocalStorage.getJSON("huanzhi")==null){
                Laya.LocalStorage.setJSON("huanzhi",WxConfigMgr.getInstance().compressed); 
             }
             if(Laya.LocalStorage.getJSON("huanzhi")==false){
              return false;
             }
             else if(Laya.LocalStorage.getJSON("huanzhi")==true){
              return true;
             }  
             else {
                Laya.LocalStorage.setJSON("huanzhi",false); 
                return false; 
             }
        }
        sethuazhi(){
            let huazhi=!this.gethuazhi();
            Laya.LocalStorage.setJSON("huanzhi",huazhi);
            Laya.LocalStorage.setJSON('gaihuazhi',true);
        }
        close(){
            if(this.huazhi_btn.selected!=this.gaihuazhi){
                Laya.LocalStorage.setJSON('gaihuazhi',true);
                Laya.Dialog.close("ui/SettingTalkPop.scene");
                Laya.Dialog.open("ui/SettingTalkPop.scene", true);
            }
            else {
                Laya.LocalStorage.setJSON('gaihuazhi',false);
            }
            this.gaihuazhi=0; 
            super.close();
        }

        onDisable(){
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,0);
        }
    }

    class SettingTalkPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onTalkBoxClick);
            this.talkLab.text=Strings$1.get("tid#TipsCommon26");
        }

        onEnable() {
            if (this._param == null) {
                this.close;
                return;
            }
        }
        onTalkBoxClick(){
            //  let str=Strings.get("tid#TipsCommon27");
            //  this.talkLab.text=str.replace('<more>','        ');
            //  this.wxImage.visible=true;
            //  this.wxImage.x=this.talkLab.fontSize*5+27;
            //  let hang=(str.length-11)/Math.floor(this.talkLab.width/this.talkLab.fontSize)
            //  this.wxImage.y=(this.talkLab.height-this.talkLab.fontSize*Math.ceil(hang))/2;
            //  this.clicktimes=1;
            //  this.completeImage.visible=false;
            this.close();
        }

    }

    class ViewUtils {

        static selectGroupBtn(selected, btns) {
            for(const btn of btns) {
                if(btn==selected) {
                    btn.disabled = true;
                    btn.gray = false;
                    btn.selected = true;
                }else {
                    btn.disabled = false;
                    btn.selected = false;
                }
            }
        }

        static selectTab(selected, btns) {
            for(const btn of btns) {
                if(btn==selected) {
                    btn.disabled = true;
                    btn.gray = false;
                    btn.selected = true;
                }else {
                    btn.disabled = false;
                    btn.selected = false;
                }
            }
        }

        static showNode(selected, nodes, selectedVisible) {
            for (const node of nodes) {
                if (node == selected) {
                    node.visible = selectedVisible;
                } else {
                    node.visible = !selectedVisible;
                }
            }
        }
    }

    /**
     * 商店
     */
    class ShopPop extends Laya.Dialog {
        constructor() {
            super();
            this.name="shopPop";
            Laya.loader.load("res/shopitems2.atlas");
            Laya.loader.load("res/cookbook.atlas");
            Laya.loader.load("res/nautical_project.atlas");
            Laya.loader.load("res/nautical_food.atlas");
            Laya.loader.load("res/nautical_tool.atlas");
        }

        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,1);
            this.YG = this.getChildByName("YG");
            this.LD = this.getChildByName("LD");
            this.fishMask_text = this.YG.getChildByName("fish_text");
            this.starMask_text = this.LD.getChildByName("star_text");
            this.changeMask(0);

            this.turnId = this._param;
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);

            let beach = this.getChildByName("beachButton");
            beach.on(Laya.Event.CLICK, this, this.onBeachClicked);
            this.beach = beach;
            this.beachRedBox = this.beach.getChildByName("redBox");
            this.beachRedPoint = this.beachRedBox.getChildByName("redPoint");
            this.beachRedPoint.setCheckFunc(BuildingSystem.checkBeachCanBuyBuilding);
            this.beachRedPoint.refreshRedPoint();

            let coffee = this.getChildByName("coffeeButton");
            coffee.on(Laya.Event.CLICK, this, this.onCoffeeClicked);
            this.coffee = coffee;
            this.coffeeRedBox = this.coffee.getChildByName("redBox");
            this.coffeeRedPoint = this.coffeeRedBox.getChildByName("redPoint");
            this.coffeeRedPoint.setCheckFunc(BuildingSystem.checkCoffeeCanBuyBuilding);
            this.coffeeRedPoint.refreshRedPoint();

            let nautical = this.getChildByName("nauticalButton");
            nautical.on(Laya.Event.CLICK, this, this.onNauticalClicked);
            this.nautical = nautical;
            this.nauticalRedBox = this.nautical.getChildByName("redBox");
            this.nauticalRedPoint = this.nauticalRedBox.getChildByName("redPoint");
            this.nauticalRedPoint.setCheckFunc(BuildingSystem.checkNauticalCanBuyBuilding);
            this.nauticalRedPoint.refreshRedPoint();

            let cook = this.getChildByName("cookButton");
            cook.on(Laya.Event.CLICK, this, this.onCookClicked);
            this.cook = cook;
            this.cookRedBox = this.cook.getChildByName("redBox");
            this.cookRedPoint = this.cookRedBox.getChildByName("redPoint");
            this.cookRedPoint.setCheckFunc(FoodSystem$1.checkFoodShopRedShow);
            this.cookRedPoint.refreshRedPoint();

            let food = this.getChildByName("foodButton");
            food.on(Laya.Event.CLICK, this, this.onFoodClicked);
            this.food = food;

            let tool = this.getChildByName("toolButton");
            tool.on(Laya.Event.CLICK, this, this.onToolClicked);
            this.tool = tool;
            this.toolRedBox = this.tool.getChildByName("redBox");
            this.toolRedPoint = this.toolRedBox.getChildByName("redPoint");
            this.toolRedPoint.setCheckFunc(ProjectSystem.checkProjectShopRedShow);
            this.toolRedPoint.refreshRedPoint();

            this.getFood.text = Strings$1.get("tid#TipsCommon25");
            this.getFoodBtn.on(Laya.Event.CLICK,this,this.onGetFoodBtnClick);
            this.getFoodBtn.label = Strings$1.get("tid#Btn_029");


            let goods = this.getChildByName("goods");
            goods.on(Laya.Event.CLICK, this, this.onGoodsTabClicked);
            this.goods = goods;
            this.goodsRedBox = this.goods.getChildByName("redBox");
            this.goodsRedPoint = this.goodsRedBox.getChildByName("redPoint");
            this.goodsRedPoint.setCheckFunc(function(){
                let ret =  FoodSystem$1.checkFoodShopRedShow() || ProjectSystem.checkProjectShopRedShow();
                return ret
            });
          
            this.goodsRedPoint.refreshRedPoint();
            
            let facility = this.getChildByName("facility");
            facility.on(Laya.Event.CLICK, this, this.onFacilityTabClicked);
            this.facility = facility;
            this.facilityRedBox = this.facility.getChildByName("redBox");
            this.facilityRedPoint = this.facilityRedBox.getChildByName("redPoint");
            this.facilityRedPoint.setCheckFunc(function () {
                let ret = BuildingSystem.checkCoffeeCanBuyBuilding() || BuildingSystem.checkBeachCanBuyBuilding() || BuildingSystem.checkNauticalCanBuyBuilding();
                return ret;
            });
            this.facilityRedPoint.refreshRedPoint();

            EventMgr.getInstance().registEvent("UPDATE_COOKMENU", this, this.updateList, ["cook"]);
            EventMgr.getInstance().registEvent("UPDATA_NAUTICAL_FOODS", this, this.updateList, ["food"]);
            EventMgr.getInstance().registEvent("UPDATA_NAUTICAL_TOOLS", this, this.updateList, ["tool"]);
            EventMgr.getInstance().registEvent("UPDATA_COFFEE_BUILDINGS", this, this.updateList, ["coffee"]);
            EventMgr.getInstance().registEvent("UPDATA_NAUTICAL_BUILDINGS", this, this.updateList, ["nautical"]);
            EventMgr.getInstance().registEvent("NAUTICAL_FOODS_CHANGED", this, this.nauticalFoodsChanged);
            EventMgr.getInstance().registEvent("UPDATE_PROJECTGROUP", this, this.onToolClicked);
            EventMgr.getInstance().registEvent("CHANGE_SHOP_MASK", this, this.changeMask);
            EventMgr.getInstance().registEvent("SAVE_SHOP_MASK", this, this.saveMaskText);
            this.onFacilityTabClicked(true);
            if (SceneMgr.getInstance().getSceneLock(Constants.BeachSceneId)) {
                this.beach.visible = false;
                this.food.visible = false;
            }
            if (SceneMgr.getInstance().getSceneLock(Constants.NauticalSceneId)) {
                this.nautical.visible = false;
                this.tool.visible = false;
            }
            this.loadKanBanNiangData();
            this.list.scrollBar.on(Laya.Event.END, this, this.checkTaskMainGuide);
            this.list.on(Laya.Event.RENDER, this, this.checkTaskMainGuide);
            if(this._param){
                if(this._param[0]==1){
                    if (SceneMgr.getInstance().getCurrentSceneId() == Constants.NauticalSceneId ){
                        if(BuildingSystem.hasBuildingDataByBid(3000) ||BuildingSystem.hasBuildingDataByBid(3001) || BuildingSystem.hasBuildingDataByBid(3002)){
                            this.onGoodsTabClicked();
                            this.onToolClicked();
                        }
                    }
                }
            } 
        }

        changeMask(style){
            if(style){
                this.YG.visible = true;
                this.LD.visible = true;
            }
            else {
                this.YG.visible = false;
                this.LD.visible = false;
            }
        }

        saveMaskText(){
            let fishNum = PlayerSystem.getFish();
            this.fishMask_text.text = Strings$1.dealFishNumberText(fishNum);
            let starNum = PlayerSystem.getStar();
            this.starMask_text.text = Strings$1.dealFishNumberText(starNum);
        }

        loadKanBanNiangData(){
            KanBanNiang.getInstance().beginLoad();
            this.kbnStart=0;
            this.leftClickImage=KanBanNiang.getInstance().getLeftClickImage();
            this.addChild(this.leftClickImage);
            this.leftClickImage.on(Laya.Event.CLICK,this,this.onKBNLeftClick);
            this.rightClickImage=KanBanNiang.getInstance().getRightClickImage();
            this.addChild(this.rightClickImage);
            this.rightClickImage.on(Laya.Event.CLICK,this,this.onKBNRightClick);
            this.dhL=KanBanNiang.getInstance().getLeftDialogBox();
            this.addChild(this.dhL);
            this.dhR=KanBanNiang.getInstance().getRightDialogBox();
            this.addChild(this.dhR);
            this.textL=KanBanNiang.getInstance().getLeftDialogText();
            this.addChild(this.textL);
            this.textR=KanBanNiang.getInstance().getRightDialogText();
            this.addChild(this.textR);
            this.kanBanNiangL=KanBanNiang.getInstance().getLeftSkeleton();
            this.addChild(this.kanBanNiangL);
            this.angryL=KanBanNiang.getInstance().getAngryLSkeleton();
            this.addChild(this.angryL);
            this.angryR=KanBanNiang.getInstance().getAngryRSkeleton();
            this.addChild(this.angryR);
            this.kanBanNiangR=KanBanNiang.getInstance().getRightSkeleton();
            this.addChild(this.kanBanNiangR);
            this.kbnLeftImage=KanBanNiang.getInstance().getLeftImage();
            this.addChild(this.kbnLeftImage);
            this.kbnRightImage=KanBanNiang.getInstance().getRightImage();
            this.addChild(this.kbnRightImage);
            KanBanNiang.getInstance().setID(0);
        }

        onKBNLeftClick(){
            if(this.facility.selected){
                KanBanNiang.getInstance().onClick();
                KanBanNiang.getInstance().leftAngryCount();
            }
        }

        onKBNRightClick(){
            if(this.goods.selected){
                KanBanNiang.getInstance().onClick();
                KanBanNiang.getInstance().rightAngryCount();
            }
        }

        onGetFoodBtnClick(){
            if(BuildingSystem.hasBuildingDataByBid(2020)){
                if(GameContext.Sliding)
                return;
                let afterHandler = Laya.Handler.create(this, function (res) {
                    SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
                    this.destroy();
                });
                FishingSystem.updateServerBaseInfo(afterHandler);
            }
            else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon30");
            }
        }

        onOpened(){
            if (GuideSystem.isGuiding()) {
                let guide = null;
                guide = GuideSystem.getGuide("GuideGroup_3");
                if (guide) {
                    let jiagoods=this.getChildByName('jiagoods');
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1",this.goods);
                    guide.registerObj("btn4", this);
                    guide.registerClick("btn4", this.getChildByName("close"));
                }
                guide = GuideSystem.getGuide("GuideGroup_5");
                if (guide) {
                    let index = 0;
                    for (let i in this.list.array) {
                        if (this.list.array[i].getId() == "103") {
                            index = Number(i);
                            break;
                        }
                    }
                    this.list.scrollTo(index);
                }
                guide = GuideSystem.getGuide("GuideGroup_11");
                if (guide) {
                    let index = 0;
                    for (let i in this.list.array) {
                        for (const buildingId of this.list.array[i].buildings) {
                            if (buildingId == "2020") {
                                index = Number(i);
                                break;
                            }
                        }
                        if (index != 0) {
                            break;
                        }
                    }
                    this.list.scrollTo(index);
                }
                guide = GuideSystem.getGuide("GuideGroup_15");
                if (guide) {
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", this.coffee);
                    this.list.scrollTo(0);
                }
            }
            EventMgr.getInstance().postEvent("HIDE_COFFEE_SCENE");
        }

        onDisable() {
            KanBanNiang.getInstance().toDestory();
            this.saveListScrollHistory();
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[1]);
            EventMgr.getInstance().removeEvent("NAUTICAL_FOODS_CHANGED", this, this.nauticalFoodsChanged);
            EventMgr.getInstance().removeEvent("UPDATE_COOKMENU", this, this.updateList);
            EventMgr.getInstance().removeEvent("UPDATA_NAUTICAL_FOODS", this, this.updateList);
            EventMgr.getInstance().removeEvent("UPDATA_NAUTICAL_TOOLS", this, this.updateList);
            EventMgr.getInstance().removeEvent("UPDATA_COFFEE_BUILDINGS", this, this.updateList);
            EventMgr.getInstance().removeEvent("UPDATA_NAUTICAL_BUILDINGS", this, this.updateList);
            EventMgr.getInstance().removeEvent("UPDATE_PROJECTGROUP", this, this.onToolClicked);
            EventMgr.getInstance().removeEvent("CHANGE_SHOP_MASK", this, this.changeMask);
            EventMgr.getInstance().removeEvent("SAVE_SHOP_MASK", this, this.saveMaskText);
            Laya.timer.clearAll(this);
            EventMgr.getInstance().postEvent("SHOW_COFFEE_SCENE");
            this.removeWeakGuideAnim();
            this.list.scrollBar.off(Laya.Event.END, this, this.checkTaskMainGuide);
            this.list.off(Laya.Event.RENDER, this, this.checkTaskMainGuide);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,0);
        }

        onCloseClick() {
            EventMgr.getInstance().postEvent("SHOW_COFFEE_SCENE");
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }
        saveListScrollHistory(){
            if (GuideSystem.isGuiding()) {
                return;
            }
            if(GameContext.ListClickType)
            {
                if (GameContext.ListClickType == 1)
                    GameContext.CfHis = this.list.getScrollerHeight();
                else if (GameContext.ListClickType == 2)
                    GameContext.BcHis = this.list.getScrollerHeight();
                    GameContext.ListClickType=0;
            }
        }

        selectBuildingGroupButton(button, skipSound) {
            ViewUtils.selectGroupBtn(button, [this.coffee, this.beach, this.nautical]);
            this.showBtnRedBox(button.getChildByName("redBox"));
            if(skipSound!=true)
                SoundManager.getInstance().play("Sound_clickSound", 1);
            }

        selectGoodsGroupButton(button, skipSound) {
            ViewUtils.selectGroupBtn(button, [this.cook, this.food, this.tool]);
            if(skipSound!=true)
                SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        selectTab(selectedTab, skipSound) {
            ViewUtils.selectTab(selectedTab, [this.facility, this.goods]);
            if(skipSound!=true)
                SoundManager.getInstance().play("Sound_clickSound", 1);
            }

        setListTypeAndData(listType, data, label) {
            this.list.setListType(listType, label);
            this.list.arrayCache = data;
            this._label = label;
        }

        updateList(label) {
            if(this._label==label) {
                this.list.refresh();
            }
        }

        onCoffeeClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                if(this.kbnStart){
                    KanBanNiang.getInstance().setID(1);
                }
                else {
                    KanBanNiang.getInstance().setStart(1);
                    this.kbnStart=1;
                }
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            GameContext.ListClickType=1;
            this.selectBuildingGroupButton(this.coffee, skipSound);
            let data = BuildingSystem.getLocationByBelongScene(1);
            this.setListTypeAndData(1, data, "coffee");
            let CFHis=GameContext.CfHis?GameContext.CfHis:0;
            if (GuideSystem.isGuiding() && GuideSystem.getCurrentGuideId() == "GuideGroup_15") {
                this.list.scrollTo(0);
            } else {
                this.list.scrollToHeight(CFHis);
            }
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }


        onBeachClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                if(this.kbnStart){
                    KanBanNiang.getInstance().setID(2);
                }
                else {
                    KanBanNiang.getInstance().setStart(2);
                    this.kbnStart=1;
                }
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            GameContext.ListClickType=2;
            this.selectBuildingGroupButton(this.beach, skipSound);

            let data = BuildingSystem.getLocationByBelongScene2();
            this.setListTypeAndData(1, data, "beach");
            let BCHis=GameContext.BcHis?GameContext.BcHis:0;
            this.list.scrollToHeight(BCHis);
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }

        onNauticalClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                if(this.kbnStart){
                    KanBanNiang.getInstance().setID(3);
                }
                else {
                    KanBanNiang.getInstance().setStart(3);
                    this.kbnStart=1;
                }
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            this.selectBuildingGroupButton(this.nautical, skipSound);
            let data = BuildingSystem.getLocationByBelongScene(3);

            this.setListTypeAndData(1, data, "nautical");
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }

        onCookClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                KanBanNiang.getInstance().setID(4);
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            this.selectGoodsGroupButton(this.cook, skipSound);
            this.showBtnRedBox(this.cookRedBox);
            let config = ConfigMgr$1.getInstance().getConfig("FoodsList");
            let array = new Array();
            config.forEach((listConfig, foodListId) => {
                array.push(foodListId);
            });
            // function sortByOrder(a,b)
            // {
            //     return config.get(a).Order-config.get(b).Order;
            // }
            // array.sort(sortByOrder);

            this.setListTypeAndData(3, array, "cook");
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }


        onFoodClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                KanBanNiang.getInstance().setID(5);
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            this.selectGoodsGroupButton(this.food, skipSound);
            this.cookRedBox.visible = true;
            this.toolRedBox.visible=true;
            let config = ConfigMgr$1.getInstance().getConfig("NauticalFoods");
            let array = new Array();
            config.forEach((listConfig, foodId) => {
                if (NauticalSystem$1.getFoodById(foodId).count > 0) {
                    array.push(foodId);
                }
            });
            function sortByOrder(a,b)
            {
                return config.get(a).Rank-config.get(b).Rank;
            }
            array.sort(sortByOrder);
            if(array.length<1){
                this.getFood.visible = true;
            }
            this.setListTypeAndData(3, array, "food");
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }


        nauticalFoodsChanged() {
            //this.saveListScrollHistory();
            let config = ConfigMgr$1.getInstance().getConfig("NauticalFoods");
            let array = new Array();
            config.forEach((listConfig, foodId) => {
                if (NauticalSystem$1.getFoodById(foodId).count > 0) {
                    array.push(foodId);
                }
            });
            function sortByOrder(a,b)
            {
                return config.get(a).Rank-config.get(b).Rank;
            }
            array.sort(sortByOrder);
            this.setListTypeAndData(3, array, "food");
        }
        //游戏大厅
        onToolClicked(skipSound) {
            this.getFood.visible = false;
            Laya.timer.once(80,this,function(){
                KanBanNiang.getInstance().setID(6);
                KanBanNiang.getInstance().onClick();
            });
            this.saveListScrollHistory();
            this.selectGoodsGroupButton(this.tool, skipSound);
            this.showBtnRedBox(this.toolRedBox);
            let config = ConfigMgr$1.getInstance().getConfig("NauticalProjectList");
            let array = new Array();
            let array1 = new Array();
            let array2= new Array();
            let array3 = new Array();
            config.forEach((listConfig, projectId) => {
                let a=(ProjectSystem.getlistType(projectId));
                if (Number(a)==1) {
                array1.push(projectId);
            }
            else if(Number(a)==2){
                array2.push(projectId);
            }
            else {
                array3.push(projectId);
            }
        
        
        });
            function sortByOrder(a,b)
            {
                return config.get(a).Order-config.get(b).Order;
            }
            array1.sort(sortByOrder);
            array2.sort(sortByOrder);
            array3.sort(sortByOrder);
            array=[array1,array2,array3];
            this.setListTypeAndData(4, array, "projected");
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[true]);
        }

        onFacilityTabClicked(skipSound) {
            KanBanNiang.getInstance().onLeftClick();
            this.saveListScrollHistory();
            this.selectTab(this.facility, skipSound);
            this.showTabRedBox(this.facilityRedBox);
            this.cook.visible = false;
            this.food.visible = false;
            this.tool.visible = false;
            this.coffee.visible = true;
            if (!SceneMgr.getInstance().getSceneLock(Constants.BeachSceneId)) {
                this.beach.visible = true;
            }
            if (!SceneMgr.getInstance().getSceneLock(Constants.NauticalSceneId)) {
                this.nautical.visible = true;
            }
            if (SceneMgr.getInstance().getCurrentSceneId() == Constants.BeachSceneId && !SceneMgr.getInstance().getSceneLock(Constants.BeachSceneId)) {
                this.onBeachClicked(true);
            } else if (SceneMgr.getInstance().getCurrentSceneId() == Constants.NauticalSceneId && !SceneMgr.getInstance().getSceneLock(Constants.NauticalSceneId)) {
                this.onNauticalClicked(true); 
            } else {
                this.onCoffeeClicked(true);
            }
            if(this.turnId == 4){
                Laya.timer.once(50,this,function(){
                    this.onGoodsTabClicked(true);
                    this.turnId = 0;
                });
            }
            if(this.turnId == 6){
                this.onGoodsTabClicked(true);
                Laya.timer.once(50,this,function(){
                    this.onToolClicked(true);
                    this.turnId = 0;
                });
            }
            this.checkTaskMainGuide();
            EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
        }

        onGoodsTabClicked(skipSound) {
            KanBanNiang.getInstance().onRightClick();
            this.saveListScrollHistory();
            this.selectTab(this.goods, skipSound);
            this.showTabRedBox(this.goodsRedBox);
            this.cook.visible = true;
            if (!SceneMgr.getInstance().getSceneLock(Constants.BeachSceneId)) {
                this.food.visible = true;
            }
            if (!SceneMgr.getInstance().getSceneLock(Constants.NauticalSceneId)) {
                this.tool.visible = true;
            }
            this.coffee.visible = false;
            this.beach.visible = false;
            this.nautical.visible = false;
            if (SceneMgr.getInstance().getCurrentSceneId() == Constants.NauticalSceneId ){
                if(BuildingSystem.hasBuildingDataByBid(3000) ||BuildingSystem.hasBuildingDataByBid(3001) || BuildingSystem.hasBuildingDataByBid(3002)){
                     this.onToolClicked(true);
                     EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[true]);
                }
                else {
                    this.onCookClicked(true);
                    EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
                }
            }else {
                this.onCookClicked(true);
                EventMgr.getInstance().postEvent('SHOW_BEIKE_SHOP',[false]);
            } 
            this.checkTaskMainGuide();
       
        }


        showBtnRedBox(redBox) {
            ViewUtils.showNode(redBox, [this.cookRedBox, this.coffeeRedBox, this.beachRedBox, this.nauticalRedBox,this.toolRedBox], false);
        }

        showTabRedBox(redBox) {
            ViewUtils.showNode(redBox, [this.goodsRedBox, this.facilityRedBox], false);
        }

        checkTaskMainGuide() {
            if (GuideSystem.isGuiding()) {
                this.removeWeakGuideAnim();
                return;
            }
            if (TaskMainSystem.getIsTasking()) {
                if (TaskMainSystem.checkEnought()) {
                    this.removeWeakGuideAnim();
                    return;
                }
                let task = TaskMainSystem.getCurrentData();
                let guideType = task.getGuide();
                if (guideType == "Building") {
                    let building = BuildingSystem.checkCanBuyCheapestBuilding();
                    if (building) {
                        if (this.facility.selected) {
                            let pid=BuildingSystem.getPidByBid(building.Id);
                            if (building.BelongScene == Constants.CoffeeSceneId) {
                                if (!this.coffee.selected) {
                                    this.addWeakGuideAnim(this.coffee);
                                } else {
                                    this.checkUpDown(pid);
                                }
                            } else if (building.BelongScene == Constants.BeachSceneId) {
                                if (!this.beach.selected) {
                                    this.addWeakGuideAnim(this.beach);
                                } else {
                                    this.checkUpDown(building.Id);
                                }
                            } else {
                                if (!this.nautical.selected) {
                                    this.addWeakGuideAnim(this.nautical);
                                } else {
                                    this.checkUpDown(pid);
                                }
                            }
                        } else {
                            this.addWeakGuideAnim(this.facility, 85, 118);
                        }
                    } else {
                        this.removeWeakGuideAnim();
                    }
                } else if (guideType == "Food_Unlock") {
                    let food = FoodSystem$1.checkCanBuyCheapestFood();
                    if (food) {
                        if (this.goods.selected) {
                            if (!this.cook.selected) {
                                this.addWeakGuideAnim(this.cook);
                            } else {
                                this.checkUpDown(food.getType());
                            }
                        } else {
                            this.addWeakGuideAnim(this.goods, 75, 150);
                        }
                    }else {
                        this.removeWeakGuideAnim();
                    }
                } else if (guideType == "Food_LvUp") {
                    let food = FoodSystem$1.checkLevelUpCheapestFood();
                    if (food) {
                        if (this.goods.selected) {
                            if (!this.cook.selected) {
                                this.addWeakGuideAnim(this.cook);
                            } else {
                                this.checkUpDown(food.getType());
                            }
                        } else {
                            this.addWeakGuideAnim(this.goods, 75, 150);
                        }
                    }else {
                        this.removeWeakGuideAnim();
                    }
                }
            }
        }

        addWeakGuideAnim(node, x, y) {
            if (this.weakGuideAnim == null || this.weakGuideAnim.destroyed) {
                this.weakGuideAnim = new Spine();
                this.weakGuideAnim.loadAnim("anim1/xinshouyindao.sk");
                this.weakGuideAnim.playAnim("zhuazi2", true);
            }
            if (this.weakGuideAnim.parent == null || this.weakGuideAnim.parent != node) {
                this.weakGuideAnim.removeSelf();
                this.weakGuideAnim.playAnim("zhuazi2", true);
                if (x == null) {
                    this.weakGuideAnim.x = node.width / 2;
                } else {
                    this.weakGuideAnim.x = x;
                }
                if (y == null) {
                    this.weakGuideAnim.y = node.height / 2;
                } else {
                    this.weakGuideAnim.y = y;
                }
                this.weakGuideAnim.scaleX = 1 / node.scaleX;
                this.weakGuideAnim.scaleY = 1 / node.scaleY;
                node.addChild(this.weakGuideAnim);
            }
        }

        removeWeakGuideAnim(){
            if (this.weakGuideAnim != null && !this.weakGuideAnim.destroyed) {
                this.weakGuideAnim.destroy();
            }
            this.weakGuideAnim = null;
        }

        checkUpDown(id,lines) {
            let index = null;
            for (let i in this.list.array) {
                if ((this.list.array[i] == id) || (this.list.array[i].getId && this.list.array[i].getId() != null && this.list.array[i].getId() == id)) {
                    index = Number(i);
                    break;
                } else if (this.list.array[i].buildings != null && this.list.array[i].buildings.length > 0) {
                    let buildings = this.list.array[i].buildings;
                    for (const buildingId of buildings) {
                        if (buildingId == id) {
                            index = Number(i);
                            break;
                        }
                    }
                    if (index != null) {
                        break;
                    }
                }
            }
            if (index != null && index < this.list.getFirstScrollIndex()) {
                this.addWeakGuideAnim(this.list, this.list.width - 100);
                if (this.weakGuideAnim.getCurAnim() != "tuodong2") {
                    this.weakGuideAnim.playAnim("tuodong2", true);
                }
            } else if (index != null && index > this.list.getLastScrollIndex()) {
                this.addWeakGuideAnim(this.list, this.list.width - 100);
                if (this.weakGuideAnim.getCurAnim() != "tuodong1") {
                    this.weakGuideAnim.playAnim("tuodong1", true);
                }
            } else {
                this.removeWeakGuideAnim();
            }
        }
    }

    class ShopItem extends Laya.Box {

        constructor() {
            super();
            this._buildingItems = [];
            this._buildingMap = new Map();
            this.fangzhi=-1;//判断当前放置的是哪一个物品
        }

        onEnable(){
            this.upDownButton = this.getChildByName("upDown");
            this.upDownButton.on(Laya.Event.CLICK, this, this.onUpDowmClick);
            this.upDownButton.selected = false;
            this.redPoint = this.upDownButton.getChildByName("redPoint");
            this.redPoint.visible = false;
            this._selected = false;
        }

        set itemHeight(height) {
            this.trueHeight = height;
            EventMgr.getInstance().postEvent("SHOP_LIST_CHANGE_ITEM_HEIGHT", [this.__index, height]);
        }

        get itemHeight() {
            return this.trueHeight;
        }

        static getItemHeight(location) {
            if (location.shopItemHeight != null) {
                return location.shopItemHeight;
            } else if (location[0] && location[0].shopItemHeight != null) {
                return location[0].shopItemHeight;
            }
        }

        setItem(location, itemType, index, itemHeight) {
            this.__index = index;
            if(location.getListName) {
                this._type = 1;
                this.location = location;
                this.buildings = [];
                let buildings = this.location.getFixedBuildings();
                let buildingOpenCondition = JSON.parse(Const.get("BuildingOpenCondition"));
                for (let buildingId of buildings) {
                    let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
                    if (item != null && buildingOpenCondition != null) {
                        let starLimit = buildingOpenCondition[String(item.OpenOrder)];
                        if (starLimit != null && PlayerSystem.getStar() >= starLimit) {
                            this.buildings.push(buildingId);
                        }
                    }
                }

                let title = this.getChildByName("title");
                title.text = Strings$1.get(location.getListName());
                Laya.loader.load("prefab/ui/CommonComponentItem.json", Laya.Handler.create(this, this.onPrefabLoaded), null, Laya.Loader.JSON);
            }else {
                this._type = 2;
                this.location = location;
                this.buildings = [];
                let buildings = this.location.buildings;
                let buildingOpenCondition = JSON.parse(Const.get("BuildingOpenCondition"));
                for (let buildingId of buildings) {
                    let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
                    if (item != null && buildingOpenCondition != null) {
                        let starLimit = buildingOpenCondition[String(item.OpenOrder)];
                        if (starLimit != null && PlayerSystem.getStar() >= starLimit) {
                            this.buildings.push(buildingId);
                        }
                    }
                }
                let bid1 = location.buildings[0];
                let config = ConfigMgr$1.getInstance().getItemById("Building", String(bid1));
                let title = this.getChildByName("title");
                let SeriesNames = Const.getArray("BuildingShopSeriseName");
                title.text = Strings$1.get(SeriesNames[config.SeriesType]);
                Laya.loader.load("prefab/ui/CommonComponentItem.json", Laya.Handler.create(this, this.onPrefabLoaded2), null, Laya.Loader.JSON);
            }
            if (location.shopItemHeight != null) {
                this.itemHeight = location.shopItemHeight;
            } else if (location.shopItemHeight != null) {
                this.itemHeight = location.shopItemHeight;
            } else {
                this.itemHeight = 380;
            }
            if (this.itemHeight > 380) {
                this._selected = true;
                this.upDownButton.selected = true;
            } else {
                this._selected = false;
                this.upDownButton.selected = false;
            }
        }

        onPrefabLoaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
            let x = 0;
            let y = 112;
            this.fangzhi=-1;
            for (const index in this.buildings) {
                if (this._buildingMap.has(this.buildings[index])) {
                    let buildingItem = this._buildingMap.get(this.buildings[index]);
                    if (buildingItem.hookb && this.fangzhi == -1) {
                        this.fangzhi = index;
                    };
                    continue;
                }
                let buildingItem = Laya.Pool.getItemByCreateFun("CommonComponentItem", prefab.create, prefab);
                x = (index % 4) * 226;
                y = 112 + Math.floor(index / 4) * (224 + 44);
                buildingItem.x = x;
                buildingItem.y = y;
                this.addChild(buildingItem);
                this._buildingItems.push(buildingItem);
                this._buildingMap.set(this.buildings[index], buildingItem);
                buildingItem.setBuildingId(this.buildings[index], this.location);
                if(buildingItem.hookb&&this.fangzhi==-1){
                    this.fangzhi=index;
                };
            }
            this.onUpDowmClick();
            if (this.buildings.length > 4) {
                this.upDownButton.visible = true;
            } else {
                this.upDownButton.visible = false;
            }
            Laya.timer.once(200, this, function () {
                let guide = GuideSystem.getGuide("GuideGroup_15");
                if (guide && this.location.getId && this.location.getOrder() == 1) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.upDownButton);
                    guide.registerObj("btn3", this);
                    guide.registerClick("btn3", this.upDownButton);
                }
            });
        }

        onPrefabLoaded2(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
            let x = 0;
            let y = 112;
            this.fangzhi=-1;
            for (const index in this.buildings) {
                if (this._buildingMap.has(this.buildings[index])) {
                    continue;
                }
                let buildingItem = Laya.Pool.getItemByCreateFun("CommonComponentItem", prefab.create, prefab);
                x = (index % 4) * 226;
                y = 112 + Math.floor(index / 4) * (224 + 44);
                buildingItem.x = x;
                buildingItem.y = y;
                this.addChild(buildingItem);
                this._buildingItems.push(buildingItem);
                this._buildingMap.set(this.buildings[index], buildingItem);
                buildingItem.setBuildingId(this.buildings[index]);
                if(buildingItem.hookb&&this.fangzhi==-1){
                    this.fangzhi=index;
                };
            }
            this.onUpDowmClick();
            if (this.buildings.length > 4) {
                this.upDownButton.visible = true;
            } else {
                this.upDownButton.visible = false;
            }
        }
        clear() {
            this._buildingItems.forEach(buildingItem => {
                this.removeChild(buildingItem);
            });
            this._buildingItems = [];
            this._buildingMap.clear();
            this.buildings = [];
        }

        onUpDowmClick() {
            this._selected = this.upDownButton.selected;
            if (this._selected) {
                this.itemHeight = 112 + Math.floor((this.buildings.length - 1) / 4 + 1) * (224 + 44);
                for (const index in this._buildingItems) {
                    if (this._buildingItems[index]) {
                        let y = 112 + Math.floor(index / 4) * (224 + 44);
                        this._buildingItems[index].y = y;
                        this._buildingItems[index].visible = true;
                    }
                }
                this.redPoint.visible = false;
            } else {
                let fangz=this.fangzhi>=0?this.fangzhi:0;
                let min= 4*Math.floor(fangz/4);
                let max= 4*Math.floor(fangz/4+1);


                this.itemHeight = 380;
                for (const index in this._buildingItems) {
                    if (this._buildingItems[index]) {
                        if (index >=min&&index <max) {
                            this._buildingItems[index].visible = true;
                            this._buildingItems[index].y=112;
                        } else {
                            this._buildingItems[index].visible = false;
                        }
                    }
                }
                this.checkRedPoint();
            }
            this.location.shopItemHeight = this.trueHeight;
        }

        checkRedPoint() {
            this.redPoint.visible = false;
            if (this.buildings.length <= 4) {
                return;
            }
            let fangz = this.fangzhi >= 0 ? this.fangzhi : 0;
            let min = 4 * Math.floor(fangz / 4);
            let max = 4 * Math.floor(fangz / 4 + 1);
            for (let i = 0; i < min; i++) {
                let buildingId = this.buildings[i];
                if (BuildingSystem.checkBuildingUnlocked(buildingId) && ((BuildingSystem.checkBuyBuildingAvailable(buildingId) || BuildingSystem.checkADBuy(buildingId)) && (PlayerSystem.getStar() < Number(Const.get("ShopRedDot"))))) {
                    this.redPoint.visible = true;
                    return;
                }
            }
            for (let i = max; i < this.buildings.length; i++) {
                let buildingId = this.buildings[i];
                if (BuildingSystem.checkBuildingUnlocked(buildingId) && ((BuildingSystem.checkBuyBuildingAvailable(buildingId) || BuildingSystem.checkADBuy(buildingId)) && (PlayerSystem.getStar() < Number(Const.get("ShopRedDot"))))) {
                    this.redPoint.visible = true;
                    return;
                }
            }
        }
    }

    class ShopList extends Laya.List {

        constructor() {
    		super();
    		this._heightMap = new Map();
    		this.repeatX = 1;
    		this.spaceX = 0;
    		this._rendererType = 0;
    		// this.vScrollBarSkin = "";
    		// this.repeatY = 4;
    		// this.vScrollBarSkin = "shopui/gundongtiao2.png";
    		// this.itemRender = ShopItem;
    		// this.selectEnable = true;
    		// this.selectHandler = Laya.Handler.create(this, this.onSelect);

    		this.renderHandler = Laya.Handler.create(this, this.updateItem, null, false);

    		// Laya.loader.load("prefab/ui/CommonComponentItem.json", Laya.Handler.create(this, this.onPrefab1Loaded), null, Laya.Loader.JSON);
    		Laya.loader.load("prefab/ui/GoodsItem.json", Laya.Handler.create(this, this.onPrefab2Loaded), null, Laya.Loader.JSON);
    		Laya.loader.load("prefab/ui/projectItem.json", Laya.Handler.create(this, this.onPrefab3Loaded), null, Laya.Loader.JSON);
    	}

    	onEnable() {
    		this.x = 454;
    		// console.debug("scrollbar", this.scrollBar);
    		this.scrollBar.sizeGrid = "8,6,8,6,0";
    		EventMgr.getInstance().registEvent("SHOP_LIST_CHANGE_ITEM_HEIGHT", this, this.changeItemHeight);
    	}

    	onDisable() {
    		EventMgr.getInstance().removeEvent("SHOP_LIST_CHANGE_ITEM_HEIGHT", this, this.changeItemHeight);
    	}

        setListType(type, itemType) {
    		if(!this._itemRender1){
    			this._itemRender1 =this.itemRender;
    		}
    		if(this._rendererType==type && this._itemType==itemType){
    			return;
    		}
    		this.array = null;
    		this._arrayCache = null;
    		this._rendererType=type;
    		this._itemType = itemType;
    		if(type==1){
    			this.repeatX = 1;
    			this.spaceX = 0;
    			this.spaceY = 0;
    			this.itemRender = this._itemRender1;

    			// this.repeatY = data.length;
    		}else if(type==2){
    			this.repeatX = 4;
    			this.spaceX = 10;
    			this.spaceY = 20;
    			if(this._itemRender2){
    				this.itemRender = this._itemRender2;
    			}
    		}else if(type==3){
    			this.repeatX = 4;
    			this.spaceX = 10;
    			this.spaceY = 20;
    			if(this._itemRender3){
    				this.itemRender = this._itemRender3;
    			}
    		}
    		else {
    			this.repeatX = 1;
    			this.spaceX = 0;
    			this.spaceY = 0;
    			if(this._itemRender4){
    				this.itemRender = this._itemRender4;
    			}
    		}
    	}

    	get arrayCache() {
    		return this._arrayCache;
    	}

    	set arrayCache(data) {
    		this._arrayCache = data;
    		this.clearHeightMap();
    		if(this._rendererType==1) {
    			for(let i = 0; i < this.arrayCache.length; i++) {
    				let location = this.arrayCache[i];
    				let height = ShopItem.getItemHeight(location);
    				if(height) {
    					this._heightMap.set(i, height);
    				}
    			}
    			this.restoreCache();
    			this._cellSize=380;
    			let height = data.length * 380 - 1530;
    			this._heightMap.forEach((value, key) => {
    				height += (value - 380);
    			});
    			height=Math.max(0,height);
    			this._scrollBar.slider.max = height;
    		}else if(this._rendererType==2) {
    			if(this._itemRender2) {
    				this.restoreCache();
    			}
    		}else if(this._rendererType==3){
    			if(this._itemRender3) {
    				this.restoreCache();
    			}
    		}else {
    			if(this._itemRender4) {
    				this.restoreCache();
    			}
    		}
    	}

    	restoreCache() {
    		this.array = this.arrayCache;
    	}

    	onPrefab1Loaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
    		this._prefab1 = prefab;
    		this._itemRender2 = function() {
    			let buildingItem = Laya.Pool.getItemByCreateFun("CommonComponentItem", prefab.create, prefab);
    			return buildingItem;
    		};
    		if(this._rendererType==2){
    			this.array = null;
    			this.itemRender = this._itemRender2;
    			this.restoreCache();
    		}
    	}

    	onPrefab2Loaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
    		this._prefab2 = prefab;
    		this._itemRender3 = function() {
    			let goodsItem = Laya.Pool.getItemByCreateFun("GoodsItem", prefab.create, prefab);
    			return goodsItem;
    		};
    		if(this._rendererType==3){
    			this.array = null;
    			this.itemRender = this._itemRender3;
    			this.restoreCache();
    		}
    	}
    	onPrefab3Loaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
    		this._prefab3 = prefab;
    		this._itemRender4 = function() {
    			let projectListItem = Laya.Pool.getItemByCreateFun("projectItem", prefab.create, prefab);
    			return projectListItem;
    		};
    		if(this._rendererType==4){
    			this.array = null;
    			this.itemRender = this._itemRender4;
    			this.restoreCache();
    		}
        }

        onSelect(index) {
        }

        updateItem(cell, index)
    	{
    		if (cell.clear && typeof (cell.clear) == "function") {
    			cell.clear();
    		}
    		cell.setItem(cell.dataSource, this._itemType, index, this._heightMap.get(index));
    	}

    	changeItemHeight(index, height) {
    		let oldHeight = this._heightMap.get(index);
    		if(height==oldHeight ||(oldHeight==null && height==this._cellSize)) {
    			return;
    		}
    		if(height==this._cellSize) {
    			this._heightMap.delete(index);
    		}else {
    			this._heightMap.set(index, height);
    		}
    		// TODO
    		if(this._scrollBar && this._cellSize) {
    			if (oldHeight) {
    				this._scrollBar.slider.max = Math.max(0, this._scrollBar.slider.max + height - oldHeight);
    			} else {
    				this._scrollBar.slider.max = Math.max(0, this._scrollBar.slider.max + height - this._cellSize);
    			}
    		}

    		for(let i = index + 1; i < this._startIndex + this._cells.length; i++) {
    		    this.posCell(this.getCell(i), i);
    		}
    	}

    	clearHeightMap() {
    		this._heightMap.clear();
    	}

    	changeCells() {
    		if(this.repeatX==1  && this._heightMap.size > 0) {
    		    this._cellChanged = false;
    		    if (this._itemRender) {
    		        this.scrollBar = this.getChildByName("scrollBar");
    		        var cell = this._getOneCell();
    		        var cellWidth = (cell.width + this._spaceX) || 1;
    		        var cellHeight = (cell.height + this._spaceY) || 1;
    		        if (this._width > 0)
    		            this._repeatX2 = this._isVertical ? Math.round(this._width / cellWidth) : Math.ceil(this._width / cellWidth);
    		        if (this._height > 0)
    		            this._repeatY2 = this._isVertical ? Math.ceil(this._height / cellHeight) : Math.round(this._height / cellHeight);
    		        var listWidth = this._width ? this._width : (cellWidth * this.repeatX - this._spaceX);
    		        var listHeight = this._height ? this._height : (cellHeight * this.repeatY - this._spaceY);
    		        this._cellSize = this._isVertical ? cellHeight : cellWidth;
    		        this._cellOffset = this._isVertical ? (cellHeight * Math.max(this._repeatY2, this._repeatY) - listHeight - this._spaceY) : (cellWidth * Math.max(this._repeatX2, this._repeatX) - listWidth - this._spaceX);
    		        if (this._isVertical && this.vScrollBarSkin) {
    		            this._scrollBar.height = listHeight;
    		        }
    		        else if (!this._isVertical && this.hScrollBarSkin)
    		            this._scrollBar.width = listWidth;
    		        this.setContentSize(listWidth, listHeight);
    		        var numX = this._isVertical ? this.repeatX : this.repeatY;
    		        var numY = (this._isVertical ? this.repeatY : this.repeatX) + (this._scrollBar ? 1 : 0);
    		        this._createItems(0, numX, numY);
    		        this._createdLine = numY;
    		        if (this._array) {
    		            this.array = this._array;
    		            this.runCallLater(this.renderItems);
    		        }
    		    }
    		}else {
    			super.changeCells();
    		}
    	}

    	__calcIndexPos(index) {
    		let value = index * this._cellSize;
    		for (let i = 0; i < index; i++) {
    			let height = this._heightMap.get(i);
    			if(height!=null) {
    				value = value + height - this._cellSize;
    			}
    		}
    		return value;
    	}

        scrollTo(index) {
            if (this.repeatX == 1 && this._heightMap.size > 0) {
                if (this._scrollBar) {
                    this._scrollBar.value = this.__calcIndexPos(index);
                }
                else {
                    this.startIndex = index;
                }
            } else {
                super.scrollTo(index);
            }
        }

        scrollToHeight(height) {
            this._scrollBar.value = height;
        }
        
        getFirstScrollIndex() {
            if (this._scrollBar && this.array && this.repeatX == 1) {
                let scrollHeight = this._scrollBar.value;
                for (let i = 0; i < this.array.length; i++) {
                    let height = this._heightMap.get(i);
                    if (height != null) {
                        if (scrollHeight < height / 2) {
                            return i;
                        }
                        scrollHeight -= height;
                    } else {
                        if (scrollHeight < this._cellSize / 2) {
                            return i;
                        }
                        scrollHeight -= this._cellSize;
                    }
                }
            }
            return this._startIndex;
        }

        getScrollerHeight() {
            return this._scrollBar.value;
        }

        getLastScrollIndex() {
            if (this._scrollBar && this.array && this.repeatX == 1) {
                let scrollHeight = this._scrollBar.value + this.height;
                let i = 0;
                for (; i < this.array.length; i++) {
                    let height = this._heightMap.get(i);
                    if (height != null) {
                        if (scrollHeight < height / 2) {
                            return i;
                        }
                        scrollHeight -= height;
                    } else {
                        if (scrollHeight < this._cellSize / 2) {
                            return i;
                        }
                        scrollHeight -= this._cellSize;
                    }
                }
                return i - 1 > 0 ? i - 1 : 0;
            }
            return this._startIndex + this.cells.length - 2 * this.repeatX;
        }

    	tweenTo(index, time = 200, complete = null) {
    		if(this.repeatX==1  && this._heightMap.size > 0) {
    		    if (this._scrollBar) {
    		        this._scrollBar.stopScroll();
    		        let value = this.__calcIndexPos(index);
    		        Laya.Tween.to(this._scrollBar, { value: value}, time, null, complete, 0, true);
    		    }
    		    else {
    		        this.startIndex = index;
    		        if (complete)
    		            complete.run();
    		    }
    		}else {
    			super.tweenTo(index, time, complete);
    		}
    	}

    	onScrollBarChange(e = null) {
    		if(this.repeatX==1  && this._heightMap.size > 0) {
    		    this.runCallLater(this.changeCells);
    		    var scrollValue = this._scrollBar.value;
    		    var lineX = (this._isVertical ? this.repeatX : this.repeatY);
    		    var lineY = (this._isVertical ? this.repeatY : this.repeatX);
    			var scrollLine = Math.floor(scrollValue / this._cellSize);
    			scrollLine = Math.max(0, scrollLine);
    		    let tmpHeight = 0;
                for(let i = 0; i<=scrollLine; i ++) {
                    let height = this._heightMap.get(i);
                    if(height!=null) {
                        tmpHeight += height;
                    }else {
                        tmpHeight += this._cellSize;
                    }
                    if(tmpHeight>=scrollValue) {
                        scrollLine = i;
                        break;
                    }
                }
    		    if (!this.cacheContent) {
    		        var index = scrollLine * lineX;
    		        var num = 0;
    		        if (index > this._startIndex) {
    		            num = index - this._startIndex;
    		            var down = true;
    		            var toIndex = this._startIndex + lineX * (lineY + 1);
    		            this._isMoved = true;
    		        }
    		        else if (index < this._startIndex) {
    		            num = this._startIndex - index;
    		            down = false;
    		            toIndex = this._startIndex - 1;
    		            this._isMoved = true;
    		        }
    		        for (var i = 0; i < num; i++) {
    		            if (down) {
    		                var cell = this._cells.shift();
    		                this._cells[this._cells.length] = cell;
    		                var cellIndex = toIndex + i;
    		            }
    		            else {
    		                cell = this._cells.pop();
    		                this._cells.unshift(cell);
    		                cellIndex = toIndex - i;
    		            }
    		            var pos = this.__calcIndexPos(cellIndex);
    		            this._isVertical ? cell.y = pos : cell.x = pos;
    		            this.renderItem(cell, cellIndex);
    		        }
    		        this._startIndex = index;
    		        this.changeSelectStatus();
    		    }
    		    else {
    		        num = (lineY + 1);
    		        if (this._createdLine - scrollLine < num) {
    		            this._createItems(this._createdLine, lineX, this._createdLine + num);
    		            this.renderItems(this._createdLine * lineX, 0);
    		            this._createdLine += num;
    		        }
    		    }
    		    var r = this._content._style.scrollRect;
    		    if (this._isVertical) {
    		        r.y = scrollValue - this._offset.y;
    		        r.x = -this._offset.x;
    		    }
    		    else {
    		        r.y = -this._offset.y;
    		        r.x = scrollValue - this._offset.x;
    		    }
    		    this._content.scrollRect = r;
    		}else {
    			super.onScrollBarChange(e);
    		}
    	}

    	posCell(cell, cellIndex) {
    		if(this.repeatX==1  && this._heightMap.size > 0) {
    	        if (!this._scrollBar)
    	            return;
    	        var pos = this.__calcIndexPos(cellIndex);
    	        cell._y = pos;
    	    }else {
    	    	super.posCell(cell, cellIndex);
    	    }
    	}
    	refresh() {
    		super.refresh();
    		if (this._rendererType != 1) {
    			return
    		}
    		this._cellSize = 380;
    		let height = this.array.length * 380 - 1530;
    		this._heightMap.forEach((value, key) => {
    			height += (value - 380);
    		});
    		height = Math.max(0, height);
    		this._scrollBar.slider.max = height;
    	}
    }

    /**
    * 签到
    */
    class SignPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onOpened() {

        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Sign, 1);
            this.onopenAuto = 0;
            if (this._param) {
                this.onopenAuto = this._param[0];
            }

            this.qiandaotianshu = SignSystem.getSignDays();
            this.config = ConfigMgr$1.getInstance().getConfig("Sign");
            this.signIdArr = [];
            this.config.forEach((value, key) => {
                this.signIdArr.push(key);
            });

            let bg = this.getChildByName("bg");
            let closeBtn = this.getChildByName('close');
            closeBtn.on(Laya.Event.CLICK, this, this.closeSign);
            this.signBtn = this.getChildByName('signBtn');
            this.signAni = new Spine();
            this.addChild(this.signAni);
            this.signAni.loadAnim("anim3/qiandao.sk");
            this.signAni.addLoadedHandler(Laya.Handler.create(this, function () {
                if (!SignSystem.ifQiandao()) {
                    this.signAni.x = this.daysItem[this.qiandaotianshu].x + this.daysItem[0].width / 2;
                    this.signAni.y = this.daysItem[this.qiandaotianshu].y + this.daysItem[0].height / 2;
                    this.signAni.play('texiao', true);
                    this.signAni.visible = true;
                }
                else {
                    this.signAni.visible = false;
                }

            }));
            this.signBtn.on(Laya.Event.CLICK, this, this.clickSign);
            this.daysItem = [];
            this.initDays();

            this.qipao = this.getChildByName('qipao');
            this.qipaoLabel = this.qipao.getChildByName('qipaoLabel');
            this.qipaoZhuangtai();

            this.ifSign(SignSystem.ifQiandao());

        }

        initDays() {
            for (let i = 0; i < 7; i++) {
                this.daysItem[i] = this.getChildByName('signDay' + (i + 1));
                let daytype = 0;
                if (i < this.qiandaotianshu) {
                    daytype = 2;
                }
                else if (i == this.qiandaotianshu) {
                    daytype = 1;
                }
                else {
                    daytype = 0;
                }
                let ID = this.signIdArr[i];
                DaysItem.getInstance().itemShow(this.daysItem[i], daytype, ID);
            }
        }
        //签到气泡
        qipaoZhuangtai() {
            let qiandaotianshu = this.qiandaotianshu;
            for (let i = qiandaotianshu; i < 7; i++) {
                let ID = this.signIdArr[i];
                let signConfig = ConfigMgr$1.getInstance().getItemById("Sign", ID);
                if (signConfig.Bubble) {
                    this.qipaoLabel.text = Strings$1.get(signConfig.Bubble);
                    this.qipao.x = this.daysItem[i].x + 80;
                    this.qipao.y = this.daysItem[i].y - 145;
                    this.qipao.visible = true;
                    return;
                } else {
                    this.qipao.visible = false;
                }
            }
        }
        closeSign() {
            this.close();
        }
        clickSign() {
            this.ifSign(true);
            this.signAni.playAnim('gaizhang', false);
            let time = 0;
            SignSystem.qiandao();
            this.signAni.addCompleteHandler(Laya.Handler.create(this,function(){
                if(time == 0){
                    time ++;
                    let ID = this.signIdArr[this.qiandaotianshu];
                    DaysItem.getInstance().qiandao(this.daysItem[this.qiandaotianshu], 1, ID);
                    this.signAni.visible = false;
                    this.signAni.stop();
                    Laya.timer.once(1000, this, function () {
                        this.closeSign();
                    });
                }
            }));
            EventMgr.getInstance().postEvent("SHOW_SIGNPOINT");
        }
        /**判断是否签到 false 没有 true 签了 */
        ifSign(sign) {
            if (sign) {
                this.signBtn.label = '已签到';
                this.signBtn.label = Strings$1.get("tid#Btn_030");
                this.signBtn.disabled = true;
            } else {
                this.signBtn.label = '签到';
                this.signBtn.label = Strings$1.get("tid#Btn_031");
                this.signBtn.disabled = false;
            }
        }

        onDisable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.Sign, 0);
        }
    }





    class DaysItem {
        constructor() {
        }


        static getInstance() {
            return DaysItem.instance ? DaysItem.instance : DaysItem.instance = new DaysItem();
        }
        //
        itemShow(item, type, id) {
            item.signdi = item.getChildByName('signdi');
            item.signwdi = item.getChildByName('signwdi');
            item.signddi = item.getChildByName('signddi');
            item.sinknk = item.getChildByName('sinknk');
            item.reward = item.getChildByName('reward');
            item.rewardLabel = item.getChildByName('rewardLabel');
            item.signet = item.getChildByName('signet');
            item.sightLight = item.getChildByName('sightLight');
            item.xingxing = item.sinknk.getChildByName('xingxing');


            item.rewardLabel.text = 1000;

            item.reward.visible = true;
            item.rewardLabel.visible = true;
            item.sightLight.visible = false;
            let data = SignSystem.getReward(id);
            this.MainReward(data, item);
            this.subReward(data, item);
            this.qiandaodiban(item, type);
        }
        //签到
        qiandao(item, type, id) {
            this.qiandaodiban(item, 2);
            let data = SignSystem.getReward(id);
            if (data.MainReward) {
                this.feiyuFeimao(item, data.MainReward.id, data.MainReward.amount);
            }
            if (data.SubReward) {
                if (data.MainReward.id == 'StaffUnlock' && data.SubReward.id == 'SkillAmount') {
                    return;
                }
                this.feiyuFeimao(item, data.SubReward.id, 1);
            }
        }
        //主奖励
        MainReward(data, item) {
            if (data.MainReward) {
                if (data.MainReward.id == Constants.RewardType.FISH) {
                    item.reward.skin = 'topui/yugan.png';
                    item.reward.scale(2.5, 2.5);
                }
                else if (data.MainReward.id == Constants.RewardType.STAR) {
                    item.reward.skin = 'topui/xingxing.png';
                    item.reward.scale(2.5, 2.5);
                }
                else if (data.MainReward.id == 'StaffUnlock') {
                    item.reward.skin = 'signui/renmao.png';
                    item.reward.scale(1, 1);
                }
                else if (data.MainReward.id == 'SkillAmount') {
                    item.reward.skin = 'signui/skilljuan.png';
                    item.reward.scale(1, 1);
                }
                else if (data.MainReward.id == Constants.RewardType.DIAMOND) {
                    item.reward.skin = "topui/zuanshi.png";
                    item.reward.scale(2.5, 2.5);
                }
                item.reward.x = (item.signdi.width * item.signdi.scaleX - item.reward.width * item.reward.scaleX) / 2;
                item.reward.y = (item.signdi.height * item.signdi.scaleY - item.reward.height * item.reward.scaleY) / 2;
                if (data.MainReward.amount) {
                    item.rewardLabel.text = Strings$1.dealFishNumberText(data.MainReward.amount,1);
                    item.rewardLabel.visible = true;
                    item.rewardLabel.y += 10;
                } else {
                    item.rewardLabel.visible = false;
                    item.reward.y += 20;
                }
            }
        }
        //副奖励
        subReward(data, item) {
            if (data.SubReward) {
                item.xingxing.scaleX = item.xingxing.scaleY = 1;

                if (data.SubReward.id == 'FISH') {
                    item.xingxing.scaleX = item.xingxing.scaleY = 0.8;
                } else if (data.SubReward.id == 'STAR') {
                    item.xingxing.scaleX = item.xingxing.scaleY = 0.8;
                }
                else if (data.SubReward.id == 'StaffUnlock') {

                }
                else if (data.SubReward.id == 'SkillAmount') {
                    item.xingxing.x = item.xingxing.y = 4.5;
                    item.xingxing.width = item.xingxing.height = 25;
                    item.xingxing.skin = 'signui/skilljuan.png';
                }
                else if (data.SubReward.id == Constants.RewardType.DIAMOND) {
                    item.xingxing.scaleX = item.xingxing.scaleY = 0.8;
                    item.xingxing.skin = "topui/zuanshi.png";
                }
            }
            else {
                item.sinknk.visible = false;
            }
        }
        /**tyoe  0 未签到  1签到中  2签到了 */
        qiandaodiban(item, type) {
            if (type == 0) {
                item.signet.visible = false;
                item.signdi.visible = false;
                item.signwdi.visible = true;
                item.signddi.visible = false;
            } else if (type == 1) {
                item.signet.visible = false;

                item.signdi.visible = true;
                item.signwdi.visible = false;
                item.signddi.visible = false;
            } else {
                item.signet.visible = true;
                item.signdi.visible = false;
                item.signwdi.visible = false;
                item.signddi.visible = true;
            }
        }
        /**飞鱼干或者技能*/
        feiyuFeimao(item, id, amoumnt) {
            let point = new Laya.Point(item.x, item.y);
            point = item.parent.localToGlobal(point);
            if (id == Constants.RewardType.FISH || id == Constants.RewardType.DIAMOND || id == Constants.RewardType.STAR) {
                ItemFlyMgr.getInstance().fly(id, amoumnt, Constants.FlyType.ZCM, point.x, point.y);
            }
            else if (id == 'SkillAmount') {
                let handler = Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("FISRST_GET_STAFF");

                });
                ItemFlyMgr.getInstance().flyjuanzhou(point.x, point.y, 'SkillAmount', handler);
            }
            else if (id == 'StaffUnlock') {
                let handler = Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("FISRST_GET_STAFF");

                });
                ItemFlyMgr.getInstance().flyjuanzhou(point.x, point.y, 'StaffUnlock', handler);
            }


        }

    }

    /**
    * 员工技能升级
    */
    class StaffLevelUp extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let closeBtn = this.getChildByName("close");
            closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.skillImage = this.getChildByName("skillImage");
            this.skillName = this.getChildByName("skillName");
            this.desc = this.getChildByName("desc");
            this.level = this.getChildByName("level");
            this.duration = this.getChildByName("duration");
            this.cdTime = this.getChildByName("CD");
            this.lock = this.getChildByName("lock");
            this.levelLimit = this.getChildByName("levelLimit");
            this.staffName = this.getChildByName("staffName");

            this.nextBox = this.getChildByName("nextBox");
            this.levelBox = this.nextBox.getChildByName("levelBox");
            this.durationBox = this.nextBox.getChildByName("durationBox");
            this.cdBox = this.nextBox.getChildByName("cdBox");


            this.levelUpBox = this.getChildByName("levelUpBox");
            this.levelUpBtn = this.levelUpBox.getChildByName("levelUpBtn");
            this.levelUpBtn.on(Laya.Event.CLICK, this, this.levelUp);
            this.maxLabel = this.levelUpBox.getChildByName("maxLabel");
            this.maxLabel.text = Strings$1.get("tid#UI_047");
        
            this.upingBox = this.getChildByName("upingBox");
            this.adBtn = this.upingBox.getChildByName("adBtn");
            this.shareBtn = this.upingBox.getChildByName("shareBtn");
            this.adBtn.on(Laya.Event.CLICK, this, this.onAdClick);
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareClick);
            this.progressBar = this.upingBox.getChildByName("progressBar");
            let upingText = this.upingBox.getChildByName("upingText");
            upingText.text = Strings$1.get("tid#UI_045", [Math.floor(Number(Const.get("StafLvUpADReduceTime")) / 3600)]);

            this.unlockBox = this.getChildByName("unlockBox");
            this.unlockBtn = this.unlockBox.getChildByName("unlockBtn");
            this.unlockBtn.on(Laya.Event.CLICK, this, this.unlock);
            this.hasNum = this.unlockBox.getChildByName("hasNum");

            this.maxBox = this.getChildByName("maxBox");
            this.maxLabel1 = this.maxBox.getChildByName("maxLabel");
            this.maxLabel1.text = Strings$1.get("tid#UI_008");
            this.label1 = this.getChildByName("label1");
            this.label1.text = Strings$1.get("tid#UI_049");
            this.lock.text = Strings$1.get("tid#UI_142");
        }

        onEnable() {
            if (this._param == null) {
                this.close();
                return;
            }
            this.guide();
            this.skillData = this._param;
            this.updataView();

            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().registEvent("GUIDE16_CLOSE_SKILL", this, this.close);
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().removeEvent("GUIDE16_CLOSE_SKILL", this, this.close);
            this._uping = null;
        }

        onCloseClick() {
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        updataLevelBar() {
            let skillLevelUpEndTimer = this.skillData.skillLevelUpEndTimer;
            let needTime = this.skillData.getStaffLvUpTime();
            let time = Math.floor(skillLevelUpEndTimer - TimeUtils.getTimeNowOnServer() / 1000);
            if (time < 0) {
                Laya.timer.clear(this, this.updataLevelBar);
                this.levelUpBox.visible = false;
                this.updataView();
                return;
            }
            this.progressBar.value = (1 - Math.min(time / needTime, 1));
            this.progressBar.valueText = this.addZero(Math.floor(time / 3600), 2) + ":" + this.addZero(Math.floor(time % 3600 / 60), 2) + ":" + this.addZero(Math.floor(time % 60), 2);
        }

        addZero(num, length) {
            for (var len = (num + "").length; len < length; len = num.length) {
                num = "0" + num;
            }
            return num;
        }

        levelUp() {
            if (this._uping) {
                return;
            }
            this._uping = true;
            if (PlayerSystem.getFish() < this.skillData.getStaffLvUpCost()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon34");
                this._uping = null;
                return;
            }
            if (this.staffData.getSkillLvLimite() <= this.skillData.getLevel()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon31");
                this._uping = null;
                return;
            }
            let handler = Laya.Handler.create(this, function () {
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.SHENGJI);
                this.updataView();
                this._uping = null;
            });
            StaffSystem.skillLevelUp(this.skillData.getGroupId(), this.skillData.getStaffId(), Constants.FetchType.DIRECT, handler);
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.StaffSkillId) {
                let handler = Laya.Handler.create(this, function () {
                    this.updataView();
                    // EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon63", [Number(Const.get("StafLvUpADReduceTime") / 3600)]]);
                    EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.JIASU);
                });
                StaffSystem.reduceCDTime(this.skillData.getGroupId(), this.skillData.getStaffId(), handler);
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.StaffSkillId) {
                let handler = Laya.Handler.create(this, function () {
                    this.updataView();
                    // EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", ["tid#TipsCommon63", [Number(Const.get("StafLvUpADReduceTime") / 3600)]]);
                    EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.JIASU);
                });
                StaffSystem.reduceCDTime(this.skillData.getGroupId(), this.skillData.getStaffId(), handler);
            }
        }

        onAdClick() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.StaffSkillId);
        }

        onShareClick() {
            SdkHelper.shareAppMessage(Constants.ShareId.StaffSkillId);
        }

        unlock() {
            if (this.staffData.getSkillAmount() <= 0) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon32");
                return;
            }
            let handler = Laya.Handler.create(this, this.updataView);
            StaffSystem.unlockSkill(this.skillData.getGroupId(), this.skillData.getStaffId(), handler);
        }

        updataView() {
            this.staffData = StaffSystem.getCurrendStaffData();
            if (this.staffData == null) {
                this.close();
                return;
            }
            this.skillName.text = Strings$1.get(this.skillData.getName());
            this.desc.text = Strings$1.get(this.skillData.getDesc());
            this.level.text =Strings$1.get("tid#UI_135") + Strings$1.get("tid#UI_136") + this.skillData.getLevel();
            let skillData = this.skillData;
            let lock = !this.skillData.getUnlock();
            if (lock) {
                let groupId = this.skillData.getGroupId();
                let nextId = StaffSystem.getNextLevelSkillId(this.skillData.getGroupId());
                let staffGroupId = this.skillData.getStaffId();
                skillData = new StaffSkillData(groupId, nextId, staffGroupId);
                this.level.text = "等级：未解锁";
                this.level.text = Strings$1.get("tid#UI_137");
            }
            let skillTimeData = JSON.parse(skillData.getSkillTimeData());
            if (skillData.getEffect() != "Invite") {
                this.duration.text = "持续：" + skillTimeData.EffectTime + "s";
                this.duration.text = Strings$1.get("tid#UI_138")+':'+skillTimeData.EffectTime + "s";;
            } else {
                let autoInviteTime = Number(Const.get("StaffAutoInviteTime"));
                this.duration.text = "招揽：" + Math.floor(skillTimeData.EffectTime / autoInviteTime) + "只";
                this.duration.text = Strings$1.get("tid#UI_139")+':'+ + Math.floor(skillTimeData.EffectTime / autoInviteTime) +  Strings$1.get("tid#UI_140");
            }
            this.cdTime.text = "冷却：" + skillTimeData.CDTime + "s";
            this.cdTime.text = Strings$1.get("tid#UI_141")+':'+ skillTimeData.CDTime + "s";;
            this.maxBox.visible = false;

            this.levelLimit.text = this.skillData.getLevel() + "/" + this.staffData.getSkillLvLimite();
            this.staffName.text = "(" + Strings$1.get(this.staffData.getStaffName()) + ")";
            this.staffName.x = this.levelLimit.x + this.levelLimit.width;

            if (!lock) {
                let nextId = StaffSystem.getNextLevelSkillId(skillData.getGroupId(), skillData.getConfigId());
                let nextConfig = null;
                if (nextId == null) {
                    this.nextBox.visible = false;
                    this.maxBox.visible = true;
                    this.levelUpBox.visible = false;
                    this.upingBox.visible = false;
                } else {
                    nextConfig = ConfigMgr$1.getInstance().getItemById("StaffSkill", nextId);
                    let nextSkillData = JSON.parse(nextConfig.SkillData);
                    this.levelBox.getChildByName("txt").text = "Lv"+String(skillData.getLevel() + 1);
                    if (skillData.getEffect() != "Invite") {
                        this.durationBox.getChildByName("txt").text = String(nextSkillData.EffectTime) + "s";
                    } else {
                        let autoInviteTime = Number(Const.get("StaffAutoInviteTime"));
                        this.durationBox.getChildByName("txt").text = Math.floor(nextSkillData.EffectTime / autoInviteTime) + " cats";
                    }
                    this.cdBox.getChildByName("txt").text = String(nextSkillData.CDTime) + "s";
                    this.nextBox.visible = true;
                    this.nextBox.x = Math.max(955, this.level.x + this.level.width + 5, this.cdTime.x + this.cdTime.width + 5, this.duration.x + this.duration.width + 5);

                    let skillLevelUpEndTimer = skillData.skillLevelUpEndTimer;
                    if (skillLevelUpEndTimer == null || skillLevelUpEndTimer <= TimeUtils.getTimeNowOnServer() / 1000) {
                        let fishImage = this.levelUpBtn.getChildByName("fish");
                        let fishNum = this.levelUpBtn.getChildByName("fishNum");
                        fishNum.text = Strings$1.dealFishNumberText(skillData.getStaffLvUpCost(),1) + Strings$1.get("tid#Btn_010");
                        fishImage.x = (this.levelUpBtn.width - fishImage.width * fishImage.scaleX - fishNum.width) / 2;
                        fishNum.x = fishImage.x + fishImage.width * fishImage.scaleX;
                        if (PlayerSystem.getFish() < skillData.getStaffLvUpCost()) {
                            this.levelUpBtn.gray = true;
                        } else {
                            this.levelUpBtn.gray = false;
                        }
                        this.levelUpBox.visible = true;
                        this.maxLabel.visible = this.staffData.getSkillLvLimite() <= this.skillData.getLevel();
                        this.upingBox.visible = false;
                    } else {
                        this.levelUpBox.visible = false;
                        this.updataLevelBar();
                        Laya.timer.clear(this, this.updataLevelBar);
                        Laya.timer.loop(1000, this, this.updataLevelBar);
                        this.upingBox.visible = true;
                    }
                }
                this.unlockBox.visible = false;
                this.lock.visible = false;
            } else {
                this.maxBox.visible = false;
                this.nextBox.visible = false;
                this.upingBox.visible = false;
                this.unlockBox.visible = true;
                this.hasNum.text = "X" + this.staffData.getSkillAmount();
                if (this.staffData.getSkillAmount() <= 0) {
                    this.hasNum.color = "#f2483b";
                    this.unlockBtn.gray = true;
                } else {
                    this.hasNum.color = "#72584a";
                    this.unlockBtn.gray = false;
                }
                this.levelUpBox.visible = false;
                this.lock.visible = false;
            }

            if (skillData.getEffect() == "FishGet") {
                this.skillImage.skin = "staff/skill1.png";
            } else if (skillData.getEffect() == "Order") {
                this.skillImage.skin = "staff/skill2.png";
            } else if (skillData.getEffect() == "Invite") {
                this.skillImage.skin = "staff/skill3.png";
            }
            if (SdkHelper.hasVideo()) {
                this.adBtn.visible = true;
                this.shareBtn.visible = false;
            } else {
                this.shareBtn.visible = true;
                this.adBtn.visible = false;
            }
        }

        guide() {
            if (GuideSystem.isGuiding()) {
                let guide = GuideSystem.getGuide("GuideGroup_16");
                if (guide) {
                    let skillItem = this.unlockBox.getChildByName("skillItem");

                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", skillItem, null, [skillItem, this.hasNum]);
                }
            }
        }
    }

    /**
    * 通用提示面板
    */
    class ProgressBar extends Laya.Image {
        constructor() {
            super();
        }

        onAwake() {
            this.bar = this.getChildByName("bar");
            this.maxWidth = this.bar.width;
            this._valueText = this.getChildByName("valueText");
            this._valueText.visible = false;
        }

        set value(value) {
            this._value = value;
            this.bar.width = this.maxWidth * this._value;
        }

        get value() {
            return this._value;
        }

        set valueText(str) {
            this._valueText.text = str;
            this._valueText.visible = true;
        }

        get valueText() {
            return this._valueText.text;
        }

    }

    /**
    * 员工
    */
    class StaffPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let closeBtn = this.getChildByName("close");
            closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.staffAnimBox = this.getChildByName("staffAnimBox");
            this.staffName = this.getChildByName("staffName");
            this.staffAnimBox.on(Laya.Event.MOUSE_DOWN, this, this.onQuestionBtnClick);
            this.staffAnimBox.on(Laya.Event.MOUSE_UP, this, this.onQuestionBtnUp);
            this.staffAnimBox.on(Laya.Event.MOUSE_OUT, this, this.onQuestionBtnUp);
            this.desc = this.getChildByName("desc");
            this.level = this.getChildByName("level");
            this.levelUpBtn = this.getChildByName("levelUpBtn");
            this.levelUpBtn.on(Laya.Event.CLICK, this, this.levelUp);
            this.skillList = this.getChildByName("list");
            this.questionBox = this.getChildByName("questionBox");
            let label = this.questionBox.getChildByName("questionLabel");
            label.text = Strings$1.get("tid#TipsCommon33");
            let questionBg = this.questionBox.getChildByName("questionBg");
            questionBg.height = (label.height + label.y - questionBg.y + 40) / questionBg.scaleY;
            this.label1 = this.getChildByName("label1");
            this.label1.text = Strings$1.get("tid#UI_048");
        }

        onEnable() {
            this.staffData = StaffSystem.getCurrendStaffData();
            if (!this.staffData) {
                this.close();
                return;
            }
            this.initAnim();
            this.guide();
            this.updataView();
            EventMgr.getInstance().registEvent("UPDATA_SATFFPOP_LIST", this, this.updataList);
            EventMgr.getInstance().registEvent("UPDATA_STAFF_DATA", this, this.updataView);
            EventMgr.getInstance().registEvent("GUIDE16_OPEN_SKILL", this, this.guideOpenSkill);
        }

        updataView() {
            this.staffName.text = Strings$1.get(this.staffData.getStaffName());
            this.desc.text = Strings$1.get(this.staffData.getStaffDesc());
            let fish = this.levelUpBtn.getChildByName("fish");
            let levelUpLabel = this.levelUpBtn.getChildByName("levelUpLabel");
            let skillLevelNum = this.staffData.getSkillLevelNum();
            this.level.text = this.staffData.getNextStaff() ? skillLevelNum + "/" + this.staffData.getLvUpCondition() : String(skillLevelNum);
            levelUpLabel.text = Strings$1.dealFishNumberText(this.staffData.getPrice()) + "升阶";
            levelUpLabel.text =  Strings$1.dealFishNumberText(this.staffData.getPrice(),1) +Strings$1.get("tid#Btn_032");
            fish.x = (this.levelUpBtn.width - fish.width * fish.scaleX - levelUpLabel.width) / 2;
            levelUpLabel.x = fish.x + fish.width * fish.scaleX;
            if (this.staffData.getNextStaff() && this.staffData.getLvUpCondition() != null && skillLevelNum >= this.staffData.getLvUpCondition()) {
                this.level.color = "#867373";
                this.levelUpBtn.gray = this.staffData.getPrice() > PlayerSystem.getFish();
            } else {
                this.level.color = "#c1675d";
                this.levelUpBtn.gray = true;
            }
            this.levelUpBtn.visible = this.staffData.getNextStaff();
            let skills = this.staffData.getSkillData();
            let values = Array.from(skills.values());
            this.skillList.array = values;
            this.questionBox.visible = false;
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATA_SATFFPOP_LIST", this, this.updataList);
            EventMgr.getInstance().removeEvent("UPDATA_STAFF_DATA", this, this.updataView);
            EventMgr.getInstance().removeEvent("GUIDE16_OPEN_SKILL", this, this.guideOpenSkill);
        }

        onCloseClick() {
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        initAnim() {
            this.staffAnimBox.removeChildren();
            let anim = new Spine();
            anim.loadAnim("anim2/renzhemao.sk");
            anim.playAnim("lick1");
            anim.x = this.staffAnimBox.width / 2;
            anim.y = 250;
            this.staffAnimBox.addChild(anim);
        }

        onQuestionBtnClick() {
            this.questionBox.visible = true;
        }

        onQuestionBtnUp() {
            this.questionBox.visible = false;
        }

        levelUp() {
            if (!this.staffData.getNextStaff() || this.staffData.getLvUpCondition() != null && this.staffData.getSkillLevelNum() < this.staffData.getLvUpCondition()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon36");
                return;
            }
            if (this.staffData.getPrice() > PlayerSystem.getFish()) {
                EventMgr.getInstance().postEvent("SHOW_TIPS_BY_ID", "tid#TipsCommon35");
                return;
            }
            let handler = Laya.Handler.create(this, function () {
                this.updataView();
                EventMgr.getInstance().postEvent("SHOW_ANIM_TIP", Constants.TipAnimType.SHENGJIE);
            });
            StaffSystem.staffLevelUp(this.staffData.getId(), handler);
        }

        updataList() {
            this.skillList.refresh();
        }

        guideOpenSkill() {
            let skillData = this.staffData.getSkillDataByGroupId("SkillGroup_1");
            Laya.Dialog.open("ui/StaffLevelUp.scene", false, skillData);
        }

        guide() {
            let guide = GuideSystem.getGuide("GuideGroup_16");
            if (guide) {
                guide.start();
            }
        }
    }

    /**
    * 员工item
    */
    class StaffSkillBox extends Laya.Box {
        constructor() {
            super();
        }

        setItem(data) {
            this.skillData = data;
            let lock = !this.skillData.getUnlock();
            let lockImage = this.getChildByName("lock");
            lockImage.visible = lock;
            let skillData = this.skillData;
            if (lock) {
                let groupId = this.skillData.getGroupId();
                let nextId = StaffSystem.getNextLevelSkillId(this.skillData.getGroupId());
                let staffGroupId = this.skillData.getStaffId();
                skillData = new StaffSkillData(groupId, nextId, staffGroupId);
            }
            let skillImage = this.getChildByName("skillImage");
            skillImage.gray = lock;
            let skillName = this.getChildByName("skillName");
            skillName.text = Strings$1.get(skillData.getName());
            let skillTimeData = JSON.parse(skillData.getSkillTimeData());
            let duration = this.getChildByName("duration");
            if (skillData.getEffect() != "Invite") {
                duration.text = "持续：" + skillTimeData.EffectTime + "s";
                duration.text = Strings$1.get("tid#UI_138")+' '+ skillTimeData.EffectTime + "s";;
            } else {
                let autoInviteTime = Number(Const.get("StaffAutoInviteTime"));
                duration.text = "招揽：" + Math.floor(skillTimeData.EffectTime / autoInviteTime) + "只";
                duration.text = Strings$1.get("tid#UI_139")+' '+ Math.floor(skillTimeData.EffectTime / autoInviteTime) + Strings$1.get("tid#UI_140");
            }

            let cd = this.getChildByName("CD");
            cd.text = "冷却：" + skillTimeData.CDTime + "s";
            cd.text = Strings$1.get("tid#UI_141")+' ' + skillTimeData.CDTime + "s";

            let skillLevelUpEndTimer = skillData.skillLevelUpEndTimer;
            let levelBox = this.getChildByName("lvBox");

            if (lock) {
                levelBox.visible = false;
            } else {
                levelBox.visible = true;
                let level = levelBox.getChildByName("level");
                level.text = "Lv" + String(this.skillData.getLevel());
            }

            let staffData = StaffSystem.getCurrendStaffData();
            let redPoint = this.button.getChildByName("redPoint");
            redPoint.visible = (!lock && ((staffData.getSkillLvLimite() > skillData.getLevel() && PlayerSystem.getFish() >= skillData.getStaffLvUpCost()) || (skillLevelUpEndTimer != null && skillLevelUpEndTimer > TimeUtils.getTimeNowOnServer() / 1000)))
                || (lock && staffData.getSkillAmount() > 0);

            if (skillData.getEffect() == "FishGet") {
                skillImage.skin = "staff/skill1.png";
            } else if (skillData.getEffect() == "Order") {
                skillImage.skin = "staff/skill2.png";
            } else if (skillData.getEffect() == "Invite") {
                skillImage.skin = "staff/skill3.png";
            }
            this.showProgress();
            this.guide();
        }

        onEnable() {
            this.skillProgressImage = this.getChildByName("skillProgress");
            this.skillProgress = this.skillProgressImage.mask;
            this.skillProgressImage.visible = false;
            this.button = this.getChildByName("levelUp");
            this.button.on(Laya.Event.CLICK, this, function () {
                Laya.Dialog.close("ui/StaffLevelUp.scene");
                Laya.Dialog.open("ui/StaffLevelUp.scene", false, this.skillData);
            });
            Laya.timer.clearAll(this);
            this.button.label = Strings$1.get("tid#Btn_033");
        }

        onDisable() {
            Laya.timer.clearAll(this);
        }

        onDestroy() {
            Laya.timer.clearAll(this);
        }

        guide() {
            if (GuideSystem.isGuiding() && this.skillData.getGroupId() == "SkillGroup_1") {
                let guide = GuideSystem.getGuide("GuideGroup_16");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.button);
                }
            }
        }

        showProgress() {
            if (!this.skillData) {
                return;
            }
            let skillData = this.skillData;
            Laya.timer.clearAll(this);
            if (skillData.skillCdTimer == null) {
                this.skillProgress.graphics.clear();
                this.skillProgressImage.visible = false;
                return;
            }
            this.skillData = skillData;
            let timeData = skillData.getSkillTimeData();
            let alltime = timeData ? JSON.parse(timeData).CDTime * 1000 : 2000;
            Laya.timer.frameLoop(1, this, function () {
                if (this.destroyed) {
                    Laya.timer.clearAll(this);
                    return;
                }
                if (this.skillProgress == null) {
                    return;
                }
                let angle = 360 - (this.skillData.skillCdTimer / alltime) * 360;
                if (this.skillData.skillCdTimer == null || this.skillData.skillCdTimer <= 0) {
                    this.skillProgress.graphics.clear();
                    this.skillProgressImage.visible = false;
                    Laya.timer.clearAll(this);
                    return;
                }
                this.skillProgress.graphics.clear();
                this.skillProgress.graphics.drawPie(0, 0, 45, angle - 90, 270, "#000000");
                this.skillProgressImage.visible = true;
            });
        }
    }

    /**
    * 建议
    */
    class ShopPop$1 extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);

            let close = this.getChildByName("closeBtn");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);

            let sendBtn = this.getChildByName("sendBtn");
            sendBtn.on(Laya.Event.CLICK, this, this.send);

            this.textInput = this.getChildByName("input");
            this.textInput.maxChars = Number(Const.get("GameAdviseLengthMax"));
            this.textInput.text = Strings$1.get("tid#GameAdvise1");
            this.textInput.on(Laya.Event.BLUR, this, this.focusShowInputTip);
            this.textInput.on(Laya.Event.CLICK, this, this.focusClearInput);
            this.textInput.color = "#c3b7a1";
            this.textInput.visible = true;
            this.label1 = this.getChildByName("label1");
            this.label1.text = Strings$1.get("tid#UI_143");
            sendBtn.text = Strings$1.get("tid#Btn_034");
        }

        onDisable() {
        }

        onCloseClick() {
            this.close();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            Laya.Dialog.close("ui/Setting.scene");
            Laya.Dialog.open("ui/Setting.scene", false);
        }

        send() {
            let now = TimeUtils.getTimeNowOnServer();
            let currentStr = this.textInput.text;
            if (currentStr.length < Number(Const.get("GameAdviseLength")) || currentStr == Strings$1.get("tid#GameAdvise1")) {
                TipsPopMgr.getInstance().showTipsById("tid#GameAdvise2");
                return;
            }
            if (GameContext.sendTime != null) {
                if (now - GameContext.sendTime < Number(Const.get("GameAdviseCD")) * 1000) {
                    TipsPopMgr.getInstance().showTipsById("tid#GameAdvise3");
                    return;
                }
            }
            GameContext.sendTime = now;
            EventMgr.getInstance().postEvent("SERVICE_SEND_SUGGEST", currentStr);
            this.textInput.text = "";
            Laya.Dialog.closeAll();
            TipsPopMgr.getInstance().showTipsById("tid#GameAdvise4");
        }

        focusClearInput() {
            if (this.textInput.text == Strings$1.get("tid#GameAdvise1")) {
                this.textInput.text = "";
                this.textInput.visible = false;
            }
            this.textInput.color = "#aa6f55";
        }

        focusShowInputTip() {
            this.textInput.visible = true;
            if (this.textInput.text == "") {
                this.textInput.text = Strings$1.get("tid#GameAdvise1");
                this.textInput.color = "#c3b7a1";
            }
        }
    }

    class TaskMainTalkPop extends Laya.Dialog {
        constructor() {
            super();
        }

        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onTalkBoxClick);
            this.talkBox = this.getChildByName("talkBox");
            this.talk = this.talkBox.getChildByName("talkLab");
            this.anim = new Spine();
            this.anim.loadAnim("anim1/xinshouyindao.sk");
            this.anim.playAnim("maomao1", true);
            this.anim.x = 150;
            this.anim.y = 120;
            this.talkBox.addChild(this.anim);
            this.talkLab = this.talkBox.getChildByName("talkLab");
            this.hang=1;
        }

        onEnable() {
            this.talkBox .visible=false;
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.TaskMain,1);
            if (this._param == null) {
                this.close;
                return;
            }
            this.currentData = this._param[0];
            this.currentTalkId = 0;
            this.talkArr = [];
            this._isStartTalk = this._param[1];
            if (this._isStartTalk) {
                this.talkArr = this.currentData.getStartDialog();
            } else {
                this.talkArr = this.currentData.getEndDialog();
            }
            this.talkShow();
            this.label1=new Laya.Text();
            this.label1.fontSize=45;
            this.label1.wordWrap=true;
            this.label1.width=585;
        }

        talkShow() {
            this.lingdang.visible=false;
            if (this.currentTalkId < this.talkArr.length) {
                this.talkSt = Strings$1.get(this.talkArr[this.currentTalkId]);
                if( this.talkSt){
                    this.talkBox.visible=true; 
                }else {
                    this.talkBox.visible=false;
                }
                if( this.talkSt){
                    this.talkSarr =  this.talkSt.split("<star>"); 
                    this.talkStr =  this.talkSt.replace("<star>","    ");
                }
                this.currentCharNum = 0;
                Laya.timer.loop(1, this, this.showTalk);
                this.currentTalkId++;
            } else {
                if (this._isStartTalk) {
                   EventMgr.getInstance().postEvent("TASK_MAIN_BOX_HIDE");
                } else {
                    TaskMainSystem.completeTask();
                }
                this.close();
            }
        }

        //一个字一个字蹦
        showTalk(showAll = false) {
            if (showAll) {
                this.talk.text = this.talkStr;
                this.lingdaozuobiao(true);
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                return;
            }
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkShow();
                return;
            }
            this.lingdaozuobiao();
            if(this.currentCharNum%2==0){
                ++this.currentCharNum;
            }
            this.talk.text = this.talkStr.substr(0, ++this.currentCharNum);
            if (this.currentCharNum >= this.talkStr.length+4) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
            }
        }

        onTalkBoxClick() {
            if (this.talkStr == null) {
                Laya.timer.clear(this, this.showTalk);
                this.talkStr = null;
                this.talkShow();
            } else {
                this.showTalk(true);
            }
        }

        onDisable() {
            this.currentData = null;
            Laya.timer.clearAll(this);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.TaskMain,0);
        }
        
        lingdaozuobiao(showAll = false){
            if(showAll){
                this.label1.text=this.talkStr;
                let hangheight=this.label1.height/45;
               // console.log('dyz',hangheight);
                if(this.talkSarr.length>1){
                    this.lingdang.visible=true;
                    this.lingdang.y=(this.talk.height-(this.talk.fontSize*hangheight))/2-5;
                    let length=new Laya.Label();
                    length.text=this.talkSarr[0];
                    length.fontSize=45;
                    this.lingdang.x=length.width;
                }
                else {
                    this.lingdang.visible=false;
                }
                return;
            }
            if(this.talkSarr.length>1){
                this.label1.text=this.talkStr.substr(0, ++this.currentCharNum);
                let hangheight=this.label1.height/45;
               // console.log('dyz',hangheight);

                this.lingdang.y=(this.talk.height-(this.talk.fontSize*hangheight))/2-5;
                let length=new Laya.Label();
                length.fontSize=45;
                length.text=this.talkSarr[0];
                this.lingdang.x=length.width;
                if(this.currentCharNum>=this.talkSarr[0].length){
                    this.lingdang.visible=true;
                }
            }
            else {
                this.lingdang.visible=false;
            }
        }

    }

    /**
    * 任务 目标
    */
    class TaskPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,1);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            // this.list.array = [1,2,3,4,5,7,9,10,12];

            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);

            EventMgr.getInstance().registEvent("CLOSE_TASKS", this, this.onCloseClick);

            this.achievement = this.getChildByName("achievementBtn");
            this.achievement.on(Laya.Event.CLICK, this, this.onAchievementClicked);
            this.img1 = this.achievement.getChildByName("img1");

            let achievementRed = this.achievement.getChildByName("redPoint");
            achievementRed.setCheckFunc(TaskSystem.hasCompletedTrophyTask);
            achievementRed.refreshRedPoint();

            this.daily = this.getChildByName("dailyBtn");
            this.daily.on(Laya.Event.CLICK, this, this.onDailyClicked);
            this.img2 = this.daily.getChildByName("img2");

            let dailyRed = this.daily.getChildByName("redPoint");
            dailyRed.setCheckFunc(TaskSystem.hasCompletedDailyTask);
            dailyRed.refreshRedPoint();

            this.extrust = this.getChildByName("extrustBtn");
            this.extrust.on(Laya.Event.CLICK, this, this.onExtrustClicked);
            this.img3 = this.extrust.getChildByName("img3");

            this.zongbg=this.getChildByName("zongbg");
            this.protext=this.zongbg.getChildByName("protext");
            this.prozong=this.zongbg.getChildByName("prozong");
            this.onTaskRefresh();

            TaskSystem.getAllTasks(Laya.Handler.create(this, this.onTaskRefresh));
            EventMgr.getInstance().registEvent("UPDATE_TASKS", this, this.updateTasks);
            this.entrustList.array=[];
            EntrustSystem.getAllEntrusts(Laya.Handler.create(this, this.onEntrustRefresh));
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATE_TASKS", this, this.updateTasks);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.BottomUI,0);
        }

        onTaskRefresh() {
            if (TaskSystem.hasCompletedDailyTask()) {
                this.daily.selected = true;
                this.onDailyClicked(true);
            } else if (TaskSystem.hasCompletedTrophyTask()) {
                this.achievement.selected = true;
                this.onAchievementClicked(true);
            } else {
                this.daily.selected = true;
                this.onDailyClicked(true);
            }
        }

        updateTasks() {
            this.list.refresh();
            this.zongjindu();
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this.close();
        }

        onDailyClicked(skipSound = false) {
            this.list.height=1336;
            this.achievement.selected = false;
            this.achievement.disabled = false;
            this.daily.disabled = true;
            this.daily.gray = false;
            this.extrust.selected = false;
            this.extrust.disabled = false;
            this.img1.skin = "task/achievement1.png";
            this.img2.skin = "task/daily2.png";
            this.img3.skin = "task/entrust1.png";

            this.list.visible = true;
            this.entrustList.visible = false;

            let config = ConfigMgr$1.getInstance().getConfig("TaskList");
            let array = new Array();
            config.forEach((listConfig, key) => {
                if (listConfig.TaskType == Constants.TaskType.kDaily && TaskSystem.getCurrentUnlock(key))
                    array.push(key);
            });
            function sortByOrder(a, b) {
                let aOrder = config.get(a).Order;
                let bOrder = config.get(b).Order;
                let task1 = TaskSystem.getCurrentTask(a);
                if (task1.getStatus() == Constants.TaskStatus.kRewarded) {
                    aOrder += 10000;
                } else if (task1.getStatus() == Constants.TaskStatus.kCompleted) {
                    aOrder -= 10000;
                }
                let task2 = TaskSystem.getCurrentTask(b);
                if (task2.getStatus() == Constants.TaskStatus.kRewarded) {
                    bOrder += 10000;
                } else if (task2.getStatus() == Constants.TaskStatus.kCompleted) {
                    bOrder -= 10000;
                }
                return aOrder - bOrder;
            }
            array.sort(sortByOrder);
            this.list.array = array;
            if (!skipSound)
                SoundManager.getInstance().play("Sound_clickSound", 1);
            this.zongbg.visible=false;
        }

        onAchievementClicked(skipSound = false) {
            this.list.height=1200;
            this.achievement.disabled = true;
            this.achievement.gray = false;
            this.daily.selected = false;
            this.daily.disabled = false;
            this.extrust.selected = false;
            this.extrust.disabled = false;
            this.img1.skin = "task/achievement2.png";
            this.img2.skin = "task/daily1.png";
            this.img3.skin = "task/entrust1.png";

            this.list.visible = true;
            this.entrustList.visible = false;

            let config = ConfigMgr$1.getInstance().getConfig("TaskList");
            let array = new Array();
            config.forEach((listConfig, key) => {
                if (listConfig.TaskType == Constants.TaskType.kTrophy && TaskSystem.getCurrentUnlock(key))
                    array.push(key);
            });
            function sortByOrder(a, b) {
                let aOrder = config.get(a).Order;
                let bOrder = config.get(b).Order;
                let task1 = TaskSystem.getCurrentTask(a);
                if (task1.getStatus() == Constants.TaskStatus.kRewarded) {
                    aOrder += 10000;
                } else if (task1.getStatus() == Constants.TaskStatus.kCompleted) {
                    aOrder -= 10000;
                }
                let task2 = TaskSystem.getCurrentTask(b);
                if (task2.getStatus() == Constants.TaskStatus.kRewarded) {
                    bOrder += 10000;
                } else if (task2.getStatus() == Constants.TaskStatus.kCompleted) {
                    bOrder -= 10000;
                }
                return aOrder - bOrder;
            }
            array.sort(sortByOrder);
            this.list.array = array;
            if (!skipSound)
                SoundManager.getInstance().play("Sound_clickSound", 1);
             this.zongbg.visible=true;
             this.zongjindu();
        }
        zongjindu(){
            let config1 = ConfigMgr$1.getInstance().getConfig("TaskList");
            let numZong=0;
            let wancheng=0;
            config1.forEach((listConfig, key) => {
                let task=TaskSystem.getCurrentTask(key);
                if(listConfig.TaskType==1){
                   numZong+=listConfig.List.length;
                   for(let i=0;i<listConfig.List.length;i++){
                       if(i==listConfig.List.length-1){
                        if(task.getStatus()==Constants.TaskStatus.kRewarded){
                            wancheng++;
                        }
                       }
                       if(task.config.Id==listConfig.List[i]){
                           break;
                       }
                       wancheng++;
                   }
                }
           });
           this.protext.text=Math.floor(wancheng/numZong*1000)/10+"%";
           this.prozong.value=wancheng/numZong;
        }

        onExtrustClicked() {
            this.extrust.disabled = true;
            this.extrust.gray = false;
            this.daily.selected = false;
            this.daily.disabled = false;
            this.achievement.selected = false;
            this.achievement.disabled = false;
            this.img1.skin = "task/achievement1.png";
            this.img2.skin = "task/daily1.png";
            this.img3.skin = "task/entrust2.png";

            this.list.visible = false;
            this.entrustList.visible = true;
            this.zongbg.visible=false;
        }

        onEntrustRefresh() {
            this.entrustList.array = EntrustSystem.getEntrusts();
        }
    }

    class TaskItem extends Laya.Sprite {

        constructor() {
            super();
        }

        onEnable() {
            this.receiveBtn = this.getChildByName("receive");
            this.receiveBtn.on(Laya.Event.CLICK, this, this.onReceiveBtn);
            this.goBtn = this.getChildByName("goBtn");
            this.goBtn.on(Laya.Event.CLICK, this, this.onGoBtn);
            this.receiveBtn.label = Strings$1.get("tid#Btn_017");
            this.goBtn.label = Strings$1.get("tid#Btn_011");
        }

        setItem(taskListId) {
            this.taskListId = taskListId;
            let task = TaskSystem.getCurrentTask(taskListId);
            this.taskId = task.getId();
            let title = this.getChildByName("title");
            title.text = Strings$1.get(task.getName());
            let desc = this.getChildByName("desc");
            desc.text = Strings$1.get(task.getDesc(),[Strings$1.short(task.getTarget())]);
            let bar = this.getChildByName("bar");
            bar.value = task.getCurrent()/ task.getTarget();
            let progress = this.getChildByName("progress");
            let headbg=this.getChildByName("headbg1");
            progress.text = Strings$1.dealFishNumberText(task.getCurrent())+"/"+Strings$1.dealFishNumberText(task.getTarget());
            let reward = this.getChildByName("reward");
            this.rewardNum = task.getReward();
            reward.text = Strings$1.short(this.rewardNum);
            this.rewardIcon = this.getChildByName("rewardIcon");
            this.rewardIcon.scaleX = this.rewardIcon.scaleY = 2;
            if (task.getRewardType() == Constants.RewardType.FISH) {
                this.rewardIcon.texture = "topui/yugan.png";
                this.rewardType = Constants.RewardType.FISH;
            } else if (task.getRewardType() == Constants.RewardType.DIAMOND) {
                this.rewardIcon.texture = "topui/zuanshi.png";
                this.rewardType = Constants.RewardType.DIAMOND;
            } else {
                this.rewardIcon.texture = "topui/xingxing.png";
                this.rewardType = Constants.RewardType.STAR;
            }
            let config = ConfigMgr$1.getInstance().getItemById("TaskList",this.taskListId);
            let jump = config.Jump;
            let picture =config.Picture;
            let barbg=bar.getChildByName("barbg");
            let headImg=headbg.getChildByName("headImg");
            let baifenbi=headbg.getChildByName("baifenbi");
            
            this.received = this.getChildByName("received");
            let done=this.received.getChildByName("done");
            let cjdone=this.received.getChildByName("cdone");
            let progressing = this.getChildByName("progressing");
            progressing.visible = jump ? false : task.getStatus()==Constants.TaskStatus.kRunning;
            this.received.visible = task.getStatus()==Constants.TaskStatus.kRewarded;
            this.receiveBtn.visible = task.getStatus()==Constants.TaskStatus.kCompleted;
            this.isReceived=false;
            this.goBtn = this.getChildByName("goBtn");
            this.goBtn.visible = jump && task.getStatus()==Constants.TaskStatus.kRunning;

            if(picture){
                headbg.visible=true;
                headImg.skin=picture;
                headImg.x=(headbg.width-headImg.width)/2;
                title.x=desc.x=232;
                title.y=50;
                desc.y=102;
                progress.x=bar.x=177;
                progress.y=bar.y=150;
                progress.width=380;
                bar.width=barbg.width=132;
                let num=0;
                let list= config.List;
                let id =task.config.Id;
                for(let i=0;i<list.length;i++){
                    if(i==list.length-1){
                        if ( this.received.visible){
                            num++;
                        }
                    }
                    if(id==list[i]){
                        break;
                    }
                    num++;
                } 
                baifenbi.text=Math.floor(num/list.length*100)+"%";
                done.visible=false;
                cjdone.visible=true;
            }else {
                title.x=104;
                desc.x=108;
                title.y=50;
                desc.y=102;
                progress.x=bar.x=48;
                progress.y=bar.y=150;
                progress.width=497;
                bar.width=barbg.width=167;
                headbg.visible=false;
                done.visible=true;
                cjdone.visible=false;
            }
        }

        onGoBtn(){
            let jump = ConfigMgr$1.getInstance().getItemById("TaskList", this.taskListId).Jump;
            if(jump){
                EventMgr.getInstance().postEvent("CLOSE_TASKS");
            }
            if(jump == "ToBuild"){
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
                SyncSystem.updateSceneInfo();
                SoundManager.getInstance().play("Sound_clickSound", 1);
                Laya.Dialog.close("ui/shopPop.scene");
                Laya.Dialog.open("ui/shopPop.scene");
            }else if(jump == "ToFood"){
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
                SyncSystem.updateSceneInfo();
                SoundManager.getInstance().play("Sound_clickSound", 1);
                Laya.Dialog.close("ui/shopPop.scene");
                Laya.Dialog.open("ui/shopPop.scene",true,4);
            }else if(jump == "ToFishing"){
                if(GameContext.Sliding)
                    return;
                if (BuildingSystem.getBuildingDataByPid(202) == null) {
                    let str = Strings$1.get("tid#UI_144");
                    TipsPopMgr.getInstance().showTips(str);
                    return;
                }
                let afterHandler = Laya.Handler.create(this, function (res) {
                    SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
                });
                FishingSystem.updateServerBaseInfo(afterHandler);
            }else if(jump == "ToCatFood"){
                if(GameContext.Sliding)
                    return;
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                let total = Number(Const.get("CatFoodKeepTime"));
                let adTime = Number(Const.get("CatFoodKeepTimeAD"));
                let videoStr = Strings$1.get("tid#UI_027", [String(Math.floor((total + adTime) / 3600))]);
                let shareStr = Strings$1.get("tid#UI_028", [String(Math.floor((total + adTime) / 3600))]);
                Laya.Dialog.close("ui/BowlAdPop.scene");
                Laya.Dialog.open("ui/BowlAdPop.scene", false, [Constants.ShareId.CatFood, "ADD_CAT_FOOD_AD", videoStr, shareStr]);
            }else if(jump == "ToAdopt"){
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[0]);
                SyncSystem.updateSceneInfo();
                PetSystem.getPetsInfo(Laya.Handler.create(this, function () {
                    Laya.Dialog.close("ui/CollectPop.scene");
                    Laya.Dialog.open("ui/CollectPop.scene");
                }));
                SoundManager.getInstance().play("Sound_clickSound", 1);
            }else if(jump == "ToSail"){
                if(GameContext.Sliding)
                    return;
                SceneMgr.getInstance().switchScene(Constants.NauticalSceneId);
                let hasBoat = BuildingSystem.hasBuildingDataByBid("3010") || BuildingSystem.hasBuildingDataByBid("3011") || BuildingSystem.hasBuildingDataByBid("3012");
                let index = NauticalSystem$1.getBoatCanSail();
                if(hasBoat){              
                    Laya.timer.once(50,this,function(){
                        if(index == -2){
                            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon29");
                        }
                        else if(index == -3){
                            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon71");
                        }
                        else {
                            Laya.Dialog.close("ui/NauticalStartPop.scene");
                            Laya.Dialog.open("ui/NauticalStartPop.scene", false,index);
                        }
                    });
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon28");
                }
            } else if (jump == "ToCollectApp") {
                if (GameContext.Sliding) {
                    return;
                }
                Laya.Dialog.close("ui/CollectionTutorialPop.scene");
                Laya.Dialog.open("ui/CollectionTutorialPop.scene", true, [2]);
            } else if (jump == "ToCollectWindow") {
                if (GameContext.Sliding) {
                    return;
                }
                Laya.Dialog.close("ui/CollectionTutorialPop.scene");
                Laya.Dialog.open("ui/CollectionTutorialPop.scene", true, [1]);
            } else if (jump == "ToCollectDesktop") {
                if (GameContext.Sliding) {
                    return;
                }
                Laya.Dialog.close("ui/CollectionTutorialPop.scene");
                Laya.Dialog.open("ui/CollectionTutorialPop.scene", true, [3]);
            } else if (jump == "ToRank") {
                if (GameContext.Sliding)
                    return;
            }
            else if (jump == "ToHome") {
                if (GameContext.Sliding)
                    return;
             SceneMgr.getInstance().switchScene(Constants.HomelandId);
            }
            else if (jump == "ToGift") {
                if (GameContext.Sliding)
                    return;
             SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
             Laya.Dialog.close("ui/VisitorGiftPop.scene");
             Laya.Dialog.open("ui/VisitorGiftPop.scene", true, [2]);
            }
        }

        onReceiveBtn() {
            this._adBtnShow = this.checkAdBtnShow();
            if (this._adBtnShow) {
                EventMgr.getInstance().registEvent("TASK_AD_EVENT", this, this.receive);
                EventMgr.getInstance().registEvent("ADVERTISING_AD_CLICK", this, this.onADclick);
                Laya.Dialog.close("ui/Advertising.scene");
                Laya.Dialog.open("ui/Advertising.scene", false, [Constants.ShareId.Task, "TASK_AD_EVENT",Strings$1.get('tid#UI_068'),Strings$1.get('tid#UI_068'), null, this.rewardType, this.rewardNum]);
            } else {
                this.receive(false);
            }
        }

        receive(AD) {
            let rewardNum=this.rewardNum;
            EventMgr.getInstance().removeEvent("TASK_AD_EVENT", this, this.receive);
            EventMgr.getInstance().removeEvent("ADVERTISING_AD_CLICK", this, this.onADclick);
            if(AD=="error"){
                return;
            }
            if(this.isReceived){
                return;
            }
            this.isReceived=true;
            this.received.visible = true;
            this.receiveBtn.visible = false;
            let config = ConfigMgr$1.getInstance().getItemById("TaskList", this.taskListId);
            let localPoint = new Laya.Point(this.rewardIcon.x, this.rewardIcon.y);
            localPoint = this.localToGlobal(localPoint);
            let fetchType = Constants.FetchType.DIRECT;
            if (AD == true) {
                let ad = Number(Const.get("ADRewardRateTask"));
                fetchType = Constants.FetchType.AD;
                rewardNum=ad*this.rewardNum;
            }
            let currentTaskId = this.taskId;
            let _adBtnShow = this._adBtnShow;
            if (config.TaskType == Constants.TaskType.kDaily) {
                let handler = Laya.Handler.create(this, function () {
                    if (fetchType == Constants.FetchType.AD) {
                        PlayerSystem.addTaskRewardTimes(1);
                        ItemFlyMgr.getInstance().flyReward(this.rewardType, rewardNum, Constants.FlyType.TYPE3, localPoint.x, localPoint.y);
                    } else {
                        ItemFlyMgr.getInstance().flyReward(this.rewardType, rewardNum, Constants.FlyType.TYPE5, localPoint.x, localPoint.y);
                    }
                    if(_adBtnShow){
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_task_predoubled_get_success"]);
                    }
                });
                TaskSystem.getDailyTaskReward(currentTaskId, fetchType, handler);
            } else {
                let handler = Laya.Handler.create(this, function () {
                    if (fetchType == Constants.FetchType.AD) {
                        PlayerSystem.addTaskRewardTimes(1);
                        ItemFlyMgr.getInstance().flyReward(this.rewardType, rewardNum, Constants.FlyType.TYPE3, localPoint.x, localPoint.y);
                    } else {
                        ItemFlyMgr.getInstance().flyReward(this.rewardType, rewardNum, Constants.FlyType.TYPE5, localPoint.x, localPoint.y);
                    }
                    if(_adBtnShow){
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_task_predoubled_get_success"]);
                    }
                });
                TaskSystem.getTrophyTaskReward(currentTaskId, fetchType, handler);
            }
            SoundManager.getInstance().play("Sound_clickReward");
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("TASK_AD_EVENT", this, this.receive);
            EventMgr.getInstance().removeEvent("ADVERTISING_AD_CLICK", this, this.onADclick);
        }

        checkAdBtnShow() {
            let maxAd = Number(Const.get("ADTaskTimes"));
            if (PlayerSystem.getTaskRewardTimes() >= maxAd) {
                return false;
            }
            let type = "Fish";
            if (this.rewardType == Constants.RewardType.FISH) {
                type = "Fish";
            } else if (this.rewardType == Constants.RewardType.STAR) {
                type = "Star";
            } else if (this.rewardType == Constants.RewardType.DIAMOND) {
                type = "Diamond";
            }
            let adShoppingRange = JSON.parse(Const.get("ADTaskRange"));
            let currentStar = PlayerSystem.getStar();
            for (let i = 0; i < adShoppingRange.length; i++) {
                let min = adShoppingRange[i].Stars[0];
                let max = adShoppingRange[i].Stars[1];
                if (max < 0) {
                    if (adShoppingRange[i][type] < this.rewardNum && currentStar >= min) {
                        return true;
                    }
                } else {
                    if (adShoppingRange[i][type] < this.rewardNum && currentStar <= max && currentStar >= min) {
                        return true;
                    }
                }
            }
            return false;
        }

        onADclick(shareId) {
        }

    }

    class FyProgressBar extends Laya.ProgressBar {

        //laya 原生的 progressBar 有点bug，设置了 sizeGrid 之后，显示的尺寸会不对, bar 会覆盖 bg 的边框。。自己定义一个覆盖原来的坐标设置
        changeValue() {
            if (this.sizeGrid) {
                var grid = this.sizeGrid.split(",");
                var left = Number(grid[3]);
                var right = Number(grid[1]);
                var max = this.width - left - right;
                var sw = max * this._value;
                this._bar.width = left + right + sw - 3;
                this._bar.visible = this._bar.width > left + right;
                this._bar.height = this._bg.height - 3;
                this._bar.x = 1.5;
                this._bar.y = 1.5;
            }
            else {
                this._bar.width = this.width * this._value;
            }
        }

    }

    class EntrustItem extends Laya.Sprite {

        constructor() {
            super();
        }

        onEnable() {
            this.title = this.getChildByName("title");
            this.content = this.getChildByName("content");
            this.reward = this.getChildByName("reward");
            this.rewardNum = this.getChildByName("rewardNum");
            this.complete = this.getChildByName("complete");
            this.doiing = this.getChildByName("doiing");
            this.progressBar = this.getChildByName("progressBar");
            this.leftTime = this.getChildByName("leftTime");
            this.jiangli = this.getChildByName("jiangli");
            this.jiangli.text = Strings$1.get("tid#UI_100");
        }

        setItem(data) {
            this.entrustData = data;
            this.title.text = Strings$1.get(data.getName());
            this.content.text = Strings$1.get(data.getDesc());
            let reward = data.getReward();
            if (reward.Reward.id == Constants.RewardType.FISH) {
                this.reward.skin = "topui/yugan.png";
            } else {
                this.reward.skin = "topui/xingxing.png";
            }
            this.progressBar.visible = false;
            this.leftTime.visible = false;
            this.rewardNum.text = "+" + Strings$1.dealFishNumberText( (reward.Reward.amount+reward.RewardRate*PetSystem.getMaxValueBase()));
            let status = data.getStatus();
            if (status <= 1) {
                this.doiing.visible = true;
                this.complete.visible = false;
                if (status == 1) {
                    this.progressBar.visible = true;
                    this.leftTime.visible = true;
                    let obj = data.getFastConditionProgress();
                    this.progressBar.value = obj.progress;
                    let condition = obj.condition;

                    let left = 0;
                    for (const entrustValue of condition) {
                        if (entrustValue.current != null && entrustValue.target > entrustValue.current) {
                            left += entrustValue.target - entrustValue.current;
                        } else {
                            left += entrustValue.target;
                        }
                    }
                    left = Math.max(0, Number(left));
                    if (condition[0].eventType == "TIME") {
                        this.leftTime.text = Strings$1.get("tid#Entrustment_Progress_1", [Strings$1.getLeftTimeBySeconds(left)]);
                    } else {
                        this.leftTime.text = Strings$1.get("tid#Entrustment_Progress_2", [String(left)]);
                    }
                }
            } else {
                this.doiing.visible = false;
                this.complete.visible = true;
            }
        }

    }

    class TipsDialog extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            this.tip.text = this._param[0];
            this.okHandler = this._param[1];
            this.close_Handler = this._param[2];
            this.okBtn.on(Laya.Event.CLICK, this, this.onOkClick);
            this.zOrder = 1;
            this.closeBtn.on(Laya.Event.CLICK, this, this.onCloseClick);
        }

        onOkClick(event) {
            if (this.okHandler) {
                this.okHandler.runWith(event);
            }
            this.close();
        }

        onCloseClick(event) {
            if (this.close_Handler) {
                this.close_Handler.runWith(event);
            }
            this.close();
        }

    }

    class ToyUnlockPop extends Laya.Dialog {

        constructor() {
            super();
        }

        //调用太慢
        // onOpened(data) {

        //     let toyId = data;
        //     let toyConfig = ConfigMgr.getInstance().getItemById("CatToys", toyId);
        //     if(toyConfig) {
        //         this.icon.visible = true;
        //         this.title.visible = true;
        //         this.desc.visible  = true;
        //         this.icon.skin = toyConfig.Icon;
        //         this.title.text = Strings.get(toyConfig.Name);
        //         this.desc.text  = Strings.get(toyConfig.Desc);
        //     }
        // }

        onEnable() {
            // this.icon.visible = false;
            // this.title.visible = false;
            // this.desc.visible  = false;

            let item = this._param.item;
            let petId = this._param.petId;
            let amount = this._param.amount;

            
            this.rewardImg.skin = "topui/yugan.png";
            this.rewardText.text = "+" + amount;
            if(this.rewardText.text.length < 4){
                this.rewardText.x += 25;
            }else if(this.rewardText.text.length >= 7){
                this.rewardText.x -= 25;
            }

            if(item.type == Constants.ItemType.CatToys) {
                let config = ConfigMgr$1.getInstance().getItemById("CatToys", item.id);
                if(config) {
                    // this.icon.visible = true;
                    // this.title.visible = true;
                    // this.desc.visible  = true;
                    this.icon.skin = config.Icon;
                    this.title.text = Strings$1.get(config.Name);
                    this.desc.text  = Strings$1.get(config.Desc);
                }
            }else if(item.type == Constants.ItemType.NauticalTools){
                let config = ConfigMgr$1.getInstance().getItemById("NauticalTools", item.id);
                if(config) {
                    // this.icon.visible = true;
                    // this.title.visible = true;
                    // this.desc.visible  = true;
                    this.icon.skin = config.Icon;
                    this.title.text = Strings$1.get(config.Name);
                    this.desc.text  = Strings$1.get(config.Desc);
                }

            }else if(item.type == Constants.ItemType.FishingBaits) {
                let config = ConfigMgr$1.getInstance().getItemById("FishingBaits", item.id);
                if(config) {
                    // this.icon.visible = true;
                    // this.title.visible = true;
                    // this.desc.visible  = true;
                    this.icon.skin = config.img;
                    this.title.text = Strings$1.get(config.name);
                    this.desc.text  = Strings$1.get(config.introduce);
                }
            }else {
                this.doClose();
                return;
            }

            this.anim.source = "homeland/ui/texiao1.png,homeland/ui/texiao2.png";

            let pet = PetSystem.getPetInfo(petId);
            let anim = new Spine();
            anim.playAnim("play_beg", true);
            this.addChild(anim);
            anim.pos(913, 1350);
            anim.loadAnim(pet.getSource() + ".sk", pet.getSkin(), 0);
            // this.confirm = this.getChildByName("confirm");
            // this.confirm.on(Laya.Event.CLICK, this, this.doClose);
            this.bgPanel = this.getChildByName("bg_panel");
            //this.bgPanel.on(Laya.Event.CLICK, this, this.doClose);
            this.bgPanel.alpha = 0.5;
            this.close1=this.getChildByName("close1");
            this.close1.on(Laya.Event.CLICK, this, this.doClose);
            //this.tip.text = Strings.get("tid#UI_011");
            let shareBtn = this.getChildByName("share");
            shareBtn.on(Laya.Event.CLICK, this, this.toyShare);
            shareBtn.visible=false;
            this.rewardImg.visible = false;
            this.rewardText.visible = false;
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        doClose() {
            this.close();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.doneS);
        }

        toyShare() {
            let title = Strings$1.get("tid#ShareTips5");
            let point = new Laya.Point(400, 500);
            point = this.localToGlobal(point);
            Camera.shareScreenshot(1050, 840, point.x, point.y, title, Constants.ShareId.ToyUnlock);

        }

        doneS(shareId) {
            if(shareId == Constants.ShareId.ToyUnlock) {
                let handler=Laya.Handler.create(this,this.flyYugan);
                EventMgr.getInstance().postEvent("SERVICE_PET_SHAREGETREWARD",[this._param.petId,Constants.PeacockShareType.ToyUnlock,handler] );
                this.close();
            }
        }

        flyYugan(res){
            let num = res.rewards.rewards[0].amount;
            let rewardType = res.rewards.rewards[0].id;
            let x = Laya.stage.width/2;
            let y = Laya.stage.height/2;
            ItemFlyMgr.getInstance().fly(rewardType, num,Constants.FlyType.LW,x-33, y-24);
        }
    }

    class VideoShareUI extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this._shareId = this._param[0];
            this._event = this._param[1];
            this._textVideo = this._param[2];
            this._textShare = this._param[3];
            this._bannnerId = this._param[4];
            this._tips = this._param[5];
            this._knowButtonLable = this._param[6];
            console.debug("VideoShareUI", this._shareId, this._event);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.done);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);
            let videoSharetext = this.getChildByName("videoSharetext");
            let video = this.getChildByName("video");//广告
            video.on(Laya.Event.CLICK, this, this.onVideoClick);
            let juanbtn = this.getChildByName("juanbtn");//猫粮卷
            this.btntext.text=Strings$1.get('tid#ConsumeBtn_6');
            juanbtn.on(Laya.Event.CLICK, this, this.useVideopiao);
            if (SdkHelper.hasVideo()) {
                video.visible = true;
                videoSharetext.text = this._textVideo;
            } else {
                video.visible = false;
                videoSharetext.text = this._textShare;
            }
            video.x = 705;

            if (GuideSystem.isGuiding()) {
                close.visible = false;
            } else {
                close.visible = true;
            }
            let guide = GuideSystem.getGuide("GuideGroup_10");
            if (guide) {
                let btn = null;
                let bg = this.getChildByName("bg");

                btn = video;

                btn.x = 729;
                btn.visible = false;

                let free = new CommonButton("pinjie_btn/btn_blue.png", "");
                free.scaleX = free.scaleY = 2;
                free.sizeGrid = '21,24,23,22';
                free.width = 170;
                free.height = 70;
                free.stateNum = 1;
                free.x = video.x;
                free.y = video.y;
                EventMgr.getInstance().postEvent('GOUDE_COLSE_TALK');

                let freeText = new Laya.Image('pinjie_btn/free.png');
                free.addChild(freeText);
                freeText.x = (170 - 114) / 2;
                freeText.y = (70 - 32) / 2;

                free.on(Laya.Event.CLICK, this, function () {
                    EventMgr.getInstance().postEvent("GUIDE10_FREE_ZHAOLAN");
                    Laya.Dialog.close("ui/VideoShareUI.scene");
                });
                this.addChild(free);

                guide.registerClick("btn1", free, null, [bg, free, videoSharetext]);
            }
            if (this._bannnerId != null && Laya.Browser.onMiniGame) {
                let wxWidth = wx.getSystemInfoSync().windowWidth;
                let wxHeight = wx.getSystemInfoSync().windowHeight;
                let banner_left = 0;
                let banner_width = wxWidth;
                let banner_height = 376 / Laya.stage.height * wxHeight;
                let banner_top = wxHeight - banner_height;
                SdkHelper.showBanner(this._bannnerId, banner_left, banner_top, banner_width, banner_height);
            }

            let tips = this.getChildByName("tips");
            tips.visible = false;
            if (this._tips) {
                tips.visible = true;
                tips.text = this._tips;
            }
            let konw = this.getChildByName("know");
            if (this._knowButtonLable) {
                konw.label = this._knowButtonLable;
                konw.visible = true;
                konw.on(Laya.Event.CLICK, this, this.onVideoClick);
                video.visible = false;
            }
            //电影票
            this.freeText = this.getChildByName('freeText');
            let piaozi = Number(PaySystem.getSkipAdInviteNum());
            this.freeText.text = "Free X "+piaozi;
            if (piaozi > 0) {
                this.freeText.visible = true;
                juanbtn.visible = true;
                konw.visible = false;
            } else {
                this.freeText.visible = false;
                juanbtn.visible = false;
                konw.visible = true;
            }
        }
        useVideopiao() {
            let handler = Laya.Handler.create(this, this.videoPiaohandler);
            EventMgr.getInstance().postEvent("SERVICE_USE_AD_COUPON", [1, handler]);
        }
        videoPiaohandler(){
            this.done(this._shareId);
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.done);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.done);
            if (this._bannnerId != null) {
                SdkHelper.hideBanner(this._bannnerId);
            }
        }

        onCloseClick() {
            this.close();
        }

        done(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event);
                this.close();
            }
        }

        fail(shareId) {
            if (shareId == this._shareId) {
                EventMgr.getInstance().postEvent(this._event, ["error"]);
            }
        }

        onVideoClick() {
            EventMgr.getInstance().postEvent("VIDEO_SHARE_UI", [this._shareId]);
            SdkHelper.showRewardVideoAd(this._shareId);
        }
        onShareClick() {
            SdkHelper.shareAppMessage(this._shareId);
        }
    }

    let animPath = "anim1/duoyugan.sk";
     /**
     * 任务 目标
     */
    class VisitorGiftPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.VisitorGift,1);
            let close = this.getChildByName("close");
            close.on(Laya.Event.CLICK, this, this.onCloseClick);

            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.onCloseClick);

            this.common = this.getChildByName("commonBtn");
            this.common.on(Laya.Event.CLICK, this, this.onCommonClicked);

            this.double = this.getChildByName("doubleBtn");
            this.double.on(Laya.Event.CLICK, this, this.onDoubleClicked);

            this.doubleShareBtn = this.getChildByName("doubleShareBtn");
            this.doubleShareBtn.on(Laya.Event.CLICK, this, this.onDoubleShareClicked);


            this.fish = this.getChildByName("fish");
            this.star = this.getChildByName("star");
            this.capacity = this.getChildByName("capacity");
            this.capacityLabel = this.getChildByName("capacityLabel");

            this.refresh();

            SyncSystem.findVisitorGift(Laya.Handler.create(this, this.refresh));
            EventMgr.getInstance().registEvent("FETCH_ONE_VISITOR_GIFT", this, this._fetchOne);

            this.doubleShareBtn.visible = false;
            this.double.visible = false;
            // if(SdkHelper.hasVideo()) {
            //     this.doubleShareBtn.visible = false;
            //     this.double.visible = true;
            // }else{
            //     this.doubleShareBtn.visible = true;
            //     this.double.visible = false;
            // }
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onDoubleVideoCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onDoubleShareCallback);
            this._active = true;
            this.heji = this.getChildByName("heji");
            this.heji.text = Strings$1.get("tid#UI_145");
            this.capacityLabel.text = Strings$1.get("tid#UI_146");
            this.common.label = Strings$1.get("tid#Btn_019");
        }

        onDisable() {
            this._active = false;
            EventMgr.getInstance().removeEvent("FETCH_ONE_VISITOR_GIFT", this, this._fetchOne);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onDoubleVideoCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onDoubleShareCallback);
            Laya.timer.clearAll(this);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.VisitorGift,0);
        }

        onCloseClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            let gifts = SyncSystem.getVisitorGifts();
            let giftslength=gifts.length;
            let minnumber=Number(Const.get("BeachRewardsRedDot"));
            if(giftslength>minnumber){
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [3,true]);
            }
            else {
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [3,false]);
            }
            this.close();
        }

        refresh() {
            if(this.destroyed)
                return;
            let fish = 0;
            let star = 0;
            let gifts = SyncSystem.getVisitorGifts();
            let giftslength=gifts.length;
            let minnumber=Number(Const.get("BeachRewardsRedDot"));
            if(giftslength>minnumber){
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [3,true]);
            }
            else {
                EventMgr.getInstance().postEvent("RIGHT_RED_POINT_SHOW", [3,false]);
            }
           
            if(gifts) {
                let array = new Array();
                for(const gift of gifts) {
                    if(!gift.fetched) {
                        array.push(gift);
                        let reward = gift.reward;
                        if(reward[1]==Constants.RewardType.FISH){
                            fish +=reward[0];
                        }else if(reward[1]==Constants.RewardType.STAR) {
                            star +=reward[0];
                        }
                    }
                }

                this.list.array = array;
            }
            let max = Number(Const.get("GiftsCountLimit"));
            this.capacity.text = this.list.array.length + "/" + max;
            this.capacityLabel.x = this.capacity.x - this.capacity.width;
            if (this.list.array.length >= max) {
                this.capacity.color = "#cc0000";
            } else {
                this.capacity.color = "#72584a";
            }
            this.fish.text = Strings$1.dealFishNumberText(fish);
            this.star.text = String(star);

            if(fish==0&&star==0) {
                this.common.disabled = true;
                this.double.disabled = true;
                this.doubleShareBtn.disabled = true;
            }else {
                this.common.disabled = false;
                this.double.disabled = false;
                this.doubleShareBtn.disabled = false;
            }
            this.fishNum = fish;
            this.starNum = star;
        }

        _fetchOne() {
            let fish = 0;
            let star = 0;
            if(this.list.array) {
                for(const gift of this.list.array) {
                    if(!gift.fetched) {
                        let reward = gift.reward;
                        if(reward[1]==Constants.RewardType.FISH){
                            fish +=reward[0];
                        }else if(reward[1]==Constants.RewardType.STAR) {
                            star +=reward[0];
                        }
                    }
                }
            }

            this.fish.text = Strings$1.dealFishNumberText(fish);
            this.star.text = String(star);

            if(fish==0&&star==0) {
                this.common.disabled = true;
                this.double.disabled = true;
                this.doubleShareBtn.disabled = true;
            }else {
                this.common.disabled = false;
                this.double.disabled = false;
                this.doubleShareBtn.disabled = false;
            }
            this.fishNum = fish;
            this.starNum = star;
        }

        _fetchAll() {
            if (!this._active) {
                return;
            }
            for(const gift of this.list.array) {
                gift.fetch();
            }
            this.list.refresh();

            this.fish.text = "X0";
            this.star.text = "X0";

            this.common.disabled = true;
            this.double.disabled = true;
            this.doubleShareBtn.disabled = true;
            this.fishNum = 0;
            this.starNum = 0;
        }

        onCommonClicked() {
            this.common.disabled = true;
            this.double.disabled = true;
            this.doubleShareBtn.disabled = true;
            let fishLocalPoint = new Laya.Point(771, 1650);
            let starLocalPoint = new Laya.Point(1009, 1650);
            fishLocalPoint = this.localToGlobal(fishLocalPoint);
            starLocalPoint = this.localToGlobal(starLocalPoint);
            let locX=(fishLocalPoint.x+starLocalPoint.x)/2;
            let locY=(fishLocalPoint.y+starLocalPoint.y)/2;
            EventMgr.getInstance().postEvent("SET_LW",[1]);
            if (this.fishNum > 0 && this.starNum > 0) {
                this.lingdang = new Laya.Skeleton();
                this.addChild(this.lingdang);
                this.lingdang.load(animPath, Laya.Handler.create(this, function() {
                    if(this.lingdang.destroyed)
                        return;
                    this.lingdang.scale(0.83,0.83);
                    this.lingdang.x = 900;
                    this.lingdang.y = 1650;
                    this.lingdang.play("lingdang", false);
                    Laya.timer.once(500, this, function (){
                        this.lingdang.removeSelf();
                    });
                }));
                SyncSystem.getVisitorGiftAll(Constants.FetchType.DIRECT, Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, this.starNum, Constants.FlyType.LW, locX + 9, locY - 1);
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, this.fishNum, Constants.FlyType.TYPE1, locX - 23, locY - 24);
                    this._fetchAll();
                }));


            } else if (this.fishNum > 0) {
                SyncSystem.getVisitorGiftAll(Constants.FetchType.DIRECT, Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, this.fishNum, Constants.FlyType.TYPE1, locX - 23, locY - 24);
                    this._fetchAll();
                }));
            } else {
                this.lingdang = new Laya.Skeleton();
                this.addChild(this.lingdang);
                this.lingdang.load(animPath, Laya.Handler.create(this, function() {
                    if(this.lingdang.destroyed)
                            return;
                    this.lingdang.scale(0.83,0.83);
                    this.lingdang.x = 900;
                    this.lingdang.y = 1650;
                    this.lingdang.play("lingdang", false);
                    Laya.timer.once(500, this, function (){
                        this.lingdang.removeSelf();
                    });
                }));
                SyncSystem.getVisitorGiftAll(Constants.FetchType.DIRECT, Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, this.starNum, Constants.FlyType.LW, locX + 9, locY - 1);
                    this._fetchAll();
                }));
            }
            SoundManager.getInstance().play("Sound_clickReward");
            EventMgr.getInstance().postEvent("SET_LW",[0]);
        }

        onDoubleClicked() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.VisitorGift);
        }

        onDoubleShareClicked() {
            SdkHelper.shareAppMessage(Constants.ShareId.VisitorGift);
        }

        onDoubleShareCallback(shareId){
            if (shareId == Constants.ShareId.VisitorGift) {
                Laya.timer.frameOnce(2,this,this.doDoubleGetVisitorRewards);
            }
        }

        onDoubleVideoCallback(shareId){
            if (shareId == Constants.ShareId.VisitorGift) {
                Laya.timer.frameOnce(2,this,this.doDoubleGetVisitorRewards);
            }
        }

        

        doDoubleGetVisitorRewards() {
            let ad = Number(Const.get("ADRewardRateBeachCatGift"));
            this.common.disabled = true;
            this.double.disabled = true;
            this.doubleShareBtn.disabled = true;
            let fishLocalPoint = new Laya.Point(771, 1650);
            let starLocalPoint = new Laya.Point(1009, 1650);
            fishLocalPoint = this.localToGlobal(fishLocalPoint);
            starLocalPoint = this.localToGlobal(starLocalPoint);
            let locX = (fishLocalPoint.x + starLocalPoint.x) / 2;
            let locY = (fishLocalPoint.y + starLocalPoint.y) / 2;
            EventMgr.getInstance().postEvent("SET_LW",[1]);
            if (this.fishNum > 0 && this.starNum > 0) {
                this.lingdang = new Laya.Skeleton();
                this.addChild(this.lingdang);
                this.lingdang.load(animPath, Laya.Handler.create(this, function () {
                    if (this.lingdang.destroyed)
                        return;
                    this.lingdang.scale(0.83, 0.83);
                    this.lingdang.x = 900;
                    this.lingdang.y = 1650;
                    this.lingdang.play("lingdang", false);
                    Laya.timer.once(500, this, function () {
                        this.lingdang.removeSelf();
                    });
                }));
                SyncSystem.getVisitorGiftAll(Constants.FetchType.AD, Laya.Handler.create(this, function(){
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, this.starNum * ad, Constants.FlyType.LW, locX + 9, locY - 1);
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, this.fishNum * ad, Constants.FlyType.TYPE3, locX - 23, locY - 24);
                    this._fetchAll();
                }));
            } else if (this.fishNum > 0) {
                SyncSystem.getVisitorGiftAll(Constants.FetchType.AD, Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, this.fishNum * ad, Constants.FlyType.TYPE3, locX - 23, locY - 24);
                    this._fetchAll();
                }));
            } else {
                this.lingdang = new Laya.Skeleton();
                this.addChild(this.lingdang);
                this.lingdang.load(animPath, Laya.Handler.create(this, function () {
                    if (this.lingdang.destroyed)
                        return;
                    this.lingdang.scale(0.83, 0.83);
                    this.lingdang.x = 900;
                    this.lingdang.y = 1650;
                    this.lingdang.play("lingdang", false);
                    Laya.timer.once(500, this, function () {
                        this.lingdang.removeSelf();
                    });
                }));
                SyncSystem.getVisitorGiftAll(Constants.FetchType.AD, Laya.Handler.create(this, function () {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["play_count_catgifts_get_success"]);
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.STAR, this.starNum * ad, Constants.FlyType.LW, locX + 9, locY - 1);
                    this._fetchAll();
                }));
            }
            SoundManager.getInstance().play("Sound_clickSound", 1);
            EventMgr.getInstance().postEvent("SET_LW",[0]);
        }
    }

    class VisitorGiftItem extends Laya.Sprite {

        constructor() {
            super();
        }

        onEnable() {
            this.receive = this.getChildByName("receive");
            this.receive.on(Laya.Event.CLICK, this, this.fetch);
            this.receive.label = Strings$1.get("tid#Btn_017");
        }

        setItem(visitorReward) {
            this.visitorReward = visitorReward;
            let catConfig = ConfigMgr$1.getInstance().getItemById("Role", visitorReward.catConfigId);
            let buildingConfig = ConfigMgr$1.getInstance().getItemById("Building", visitorReward.buildingId);

            this.titleText = this.getChildByName("title");
            this.headIcon = this.getChildByName("headIcon");
            this.building = this.getChildByName("building");
            this.reward = this.getChildByName("reward");
            this.rewardIcon = this.getChildByName("rewardIcon");
            this.received = this.getChildByName("received");


            this.titleText.text = Strings$1.get("tid#catName_"+visitorReward.catConfigId);
            if(catConfig.HeadIcon)
                this.headIcon.skin = "cat_collect/head/"+catConfig.HeadIcon+".png";
            this.building.skin = buildingConfig.ShopItems;

            this.rewardConfig = visitorReward.reward;
            this.reward.text = Strings$1.dealFishNumberText(this.rewardConfig[0]);

            if(this.rewardConfig[1]==Constants.RewardType.FISH){
                this.rewardIcon.texture = "topui/yugan.png";
            }else {
                this.rewardIcon.texture = "topui/xingxing.png";
            }


            this.receive.disabled = false;
            this.receive.visible = visitorReward.fetched?false:true;
            this.received.visible = visitorReward.fetched?true:false;
        }

        fetch() {
            this.received.visible = true;
            this.receive.visible = false;
            let localPoint = new Laya.Point(this.rewardIcon.x, this.rewardIcon.y);
            localPoint = this.localToGlobal(localPoint);
            let visitorRewardId = this.visitorReward.id;
            SyncSystem.getVisitorGiftSingle(visitorRewardId, Laya.Handler.create(this, function () {
                this.onFetch();
                ItemFlyMgr.getInstance().fly(this.rewardConfig[1], this.rewardConfig[0], Constants.FlyType.NORM, localPoint.x, localPoint.y);
            }));
            SoundManager.getInstance().play("Sound_clickReward");
        }

        onFetch() {
            console.debug("onFetch");
            this.visitorReward.fetch();
            this.receive.visible = false;
            this.received.visible = true;
            EventMgr.getInstance().postEvent("FETCH_ONE_VISITOR_GIFT");
        }
    }

    class WxMessagePop extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            if (!this._param) {
                return;
            }
            this.messageKeys = this._param;
            let yes = this.getChildByName("yes");
            let talk = this.getChildByName("talk");
            talk.text = Strings$1.get("tid#UI_042");
            yes.on(Laya.Event.CLICK, this, this.onYesClick);
        }

        onYesClick() {
            SdkHelper.requestSubscribeMessage(this.messageKeys);
            this.close();
        }

    }

    class OpenViewMgr {

        constructor() {
            this._creatOpenView();
            EventMgr.getInstance().registEvent("OPENVIEW_RANK_CLOSE_TOUCHENABLED", this, this.openViewTouchEnabled);
        }

        static getInstance() {
            return OpenViewMgr.instance ? OpenViewMgr.instance : OpenViewMgr.instance = new OpenViewMgr();
        }

        _creatOpenView() {
            this.openView = new Laya.WXOpenDataViewer();
            this.openView.width = Laya.stage.width;
            this.openView.height = Laya.stage.height;
            this.openView.x = 0;
            this.openView.y = 0;
        }
        //不要destroy
        getOpenView() {
            if (!this.openView || this.openView.destroyed) {
                this._creatOpenView();
            }
            return this.openView;
        }

        openViewTouchEnabled(enabled = true) {
            if (this.openView != null) {
                this.openView.postMsg({
                    type: "enabledMessage",
                    touchEnabled: enabled
                });
            }
        }

    }

    /**
    * 助力
    */
    class ZhuliPop extends Laya.Dialog {
        constructor() {
            super();
        }
        onOpened(param){
        }
        onAwake() {
        
        }
        
        onEnable() {
            Constants.zhuliOrRank = 0;
            if (this._param == null) {
                this.closePop();
                return;
            }
            let bg = this.getChildByName("bg");
          
            this.zhuliType=this._param[0];
            if(this.zhuliType==0){
            bg.on(Laya.Event.CLICK, this, this.closePop);
            this.title.text="邀请好友助力";
            this.title.text= Strings$1.get("tid#UI_147");
            let string=Strings$1.get('tid#UI_051');
            
            string=string.replace('%1%',Math.floor(Number(Const.get('QuayAssistWorkTime'))/3600));
            string=string.replace('%2%',Number(Const.get('QuayAssistBuff'))*100);
          
            this.desc.text=string;
            this.value1.visible=false;
            this.beikeImg.visible=false;
            this.confirm.label='邀请好友';
            this.confirm.label= Strings$1.get("tid#Btn_008");
            this.confirm.on(Laya.Event.CLICK,this,this.inviteGo);
            this.closeBtn.on(Laya.Event.CLICK,this,this.close);
            this.confirm.y = 1609;
            this.ImgBg.height = 391;
            this.zhuliList.visible = true;

            if (Laya.Browser.onMiniGame) {
                this.openView = OpenViewMgr.getInstance().getOpenView();
                this.addChild(this.openView);
                this.openView.visible = true;
            }
            this.updateBiaoyu();
                
            let array = [];

            for (let data of ZhuliSystem.getZhuliList()) {
                array.push(data[0]);
            }

          
            this.zhuliList.array = array;
            this.zhuliList.renderHandler = new Laya.Handler(this, this.updateItem);
            this.zhuliList.itemRender;
            this.uodateList();
            EventMgr.getInstance().registEvent("SHOW_ZHULI_SHOW", this, this.showzhulishow);
            EventMgr.getInstance().registEvent("UPDATE_ZHULI_LIST", this, this.uodateList);
            EventMgr.getInstance().registEvent("UPDATE_ZHULI_BIAOYU", this, this.updateBiaoyu);

            }else if(this.zhuliType==1){
                this.title.text="助力收益";
                this.title.text= Strings$1.get("tid#UI_065");
                this.desc.text=Strings$1.get('tid#UI_052');
                this.value1.visible=true;
                this.value1.text=Const.get('QuayAssistRewardFriend');
                this.beikeImg.visible=true;
                this.confirm.label='确定';
                this.confirm.label= Strings$1.get("tid#Btn_009");
                this.confirm.on(Laya.Event.CLICK,this,this.getAward);
                this.confirm.y = 1419;
                this.ImgBg.height = 293;
                this.zhuliList.visible = false;
                this.rizhi.visible = false;
                if(this._param[1]){
                    this.awardType=1;
                }
                this.closeBtn.on(Laya.Event.CLICK,this,this.getAward);
            }
        }
        inviteGo(){
            let zhulixinxi= "rid="+System.getRid()+'&zhuli=zhuli';
            let title=Strings$1.get('tid#ShareTips8');
            let img=WxConfigMgr.getInstance().CdnShareUrl + "share_img4.jpg";
            SdkHelper.shareAppMessage(Constants.ShareId.FriendZhuli,title,img,zhulixinxi,false);
            this.closePop();
        }
        getAward(){
            EventMgr.getInstance().postEvent('RECEIVE_AID_SALARY',Laya.Handler.create(this, this.showFly));
            //System._ReceiveAidSalary(Laya.Handler.create(this, this.showFly));
            this.closePop();
        }
        closePop(){
            if( this.awardType==1){
                if(Number(ZhuliSystem.getCacheFish())/ Number(ZhuliSystem.getCapacity())>=0.5){
                    if(!Constants.Baoxiangtanchuan){
                        EventMgr.getInstance().postEvent('OPEN_AWARD');
                        Constants.Baoxiangtanchuan=1;
                    }       
                }
            }
            this.close();
        }
        showFly(res) {
            let point = new Laya.Point(this.beikeImg.x,this.beikeImg.y);
            point = this.confirm.parent.localToGlobal(point);
            let rewards = res;
            for (let i in rewards) {
                let reward=0;
                if(rewards[i].id == Constants.RewardType.SHELL){
                reward=rewards[i].amount-PlayerSystem.getShell();
                }
                if (reward <= 0) {
                    continue;
                }
                if (rewards[i].id == Constants.RewardType.SHELL) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.SHELL,reward, Constants.FlyType.ZCM, point.x, point.y);
                }
            }
        }
        updateBiaoyu() {
            let zhulibiaoyu = ZhuliSystem.getBiaoyu();
            if (zhulibiaoyu[0]) {
                if (zhulibiaoyu[1] == 0) {
                    let str = Strings$1.get('tid#TipsCommon46');
                    if (zhulibiaoyu[3]) {
                        str = str.replace('%2%', zhulibiaoyu[2]);
                        this.rizhi.visible = false;
                        this.showBiaoyu(zhulibiaoyu[3], str, this.rizhi.x);
                    } else {
                        str = str.replace('%1%', zhulibiaoyu[0]);
                        str = str.replace('%2%', zhulibiaoyu[2]);
                        this.rizhi.text = str;
                        this.rizhi.visible = true;
                        this.hidBiaoyu();
                    }
                }
                else if (zhulibiaoyu[1] == 1) {
                    let str = Strings$1.get('tid#TipsCommon45');
                    if (zhulibiaoyu[3]) {
                        str = str.replace('%2%', zhulibiaoyu[2]);
                        this.rizhi.visible = false;
                        this.showBiaoyu(zhulibiaoyu[3], str, this.rizhi.x);
                    } else {
                        str = str.replace('%1%', zhulibiaoyu[0]);
                        str = str.replace('%2%', zhulibiaoyu[2]);
                        this.rizhi.text = str;
                        this.rizhi.visible = true;
                        this.hidBiaoyu();
                    }
                }
            } else {
                this.rizhi.visible = false;
            }
        }
        uodateList() {
            let array = [];
            let zhuliListn = this._mapToJson(ZhuliSystem.getZhuliList());
            for (let data of ZhuliSystem.getZhuliList()) {
                array.push(data[0]);
            }
            this.zhuliList.array = array;
            if (!Constants.zhulilastlist || zhuliListn != Constants.zhulilastlist) {
                Laya.timer.frameOnce(1, this, function () {
                    this.zhuliList.refresh;
                    this.showHead(ZhuliSystem.getZhuliList());
                    Constants.zhulilastlist = zhuliListn;
                });
            }
        }
        updateItem(cell, index) {
            let data = ZhuliSystem.getZhuliList().get(index);
            let headImg = cell.getChildByName("head");
            let diban = cell.getChildByName("bidan");
            headImg.skin = data.imageSkin;
            if (data.imageSkin == 'project/invintFriend.png') {
                headImg.visible = false;
                diban.on(Laya.Event.CLICK, this, this.inviteGo);
            }
            else {
                headImg.visible = true;
            }
        }
        showzhulishow() {
            if (Laya.Browser.onMiniGame) {
                this.openView = OpenViewMgr.getInstance().getOpenView();
                this.addChild(this.openView);
                this.openView.y = 0;
                this.openView.visible = true;
                this.openView.postMsg({
                    type: 'zhulishow',
                });
            }
        }
        _strMapToObj(strMap) {
            let obj = Object.create(null);
            for (let [k, v] of strMap) {
                obj[k] = v;
            }
            return obj;

        }

        /**
      
        *map转换为json
      
        */

        _mapToJson(map) {
            return JSON.stringify(this._strMapToObj(map));
        }
        showHead(list) {
            let listjson = this._mapToJson(list);
            if (Laya.Browser.onMiniGame) {
                if (Constants.zhuliOrRank == 0) {
                    this.openView.y = 0;
                    this.openView.visible = true;
                    this.openView.postMsg({
                        type: "zhulihead",
                        list: listjson,
                        jiacheng: Const.get('QuayAssistBuff')
                    });

                }
            }

        }
        showBiaoyu(openid, biaoyu, weizhi) {
            if (Laya.Browser.onMiniGame) {
                if (Constants.zhuliOrRank == 0) {
                    this.openView.y = 0;
                    this.openView.visible = true;
                    let name = Strings$1.get('tid#TipsCommon58');
                    this.openView.postMsg({
                        type: 'biaoyushow',
                        openid: openid,
                        biaoyu: biaoyu,
                        weizhi: weizhi,
                        shenminame: name
                    });
                }
            }
        }
        hidBiaoyu() {
            if (Laya.Browser.onMiniGame) {
                if (Constants.zhuliOrRank == 0) {
                    this.openView.postMsg({
                        type: "biaoyunull"
                    });
                }
            }
        }
        onDisable() {
            EventMgr.getInstance().removeEvent("SHOW_ZHULI_SHOW", this, this.showzhulishow);
            EventMgr.getInstance().removeEvent("UPDATE_ZHULI_LIST", this, this.uodateList);
            EventMgr.getInstance().removeEvent("UPDATE_ZHULI_BIAOYU", this, this.updateBiaoyu);
            Constants.zhuliOrRank = 1;
            if (Laya.Browser.onMiniGame && this.openView) {
                this.openView.removeSelf();
            }
        }

    }

    /**
    * 助力
    */
    class ZhuliPopAward extends Laya.Dialog {
        constructor() {
            super();
        }
        onOpened(param) {
        }
        onAwake() {
            let bg = this.getChildByName("bg");
            bg.on(Laya.Event.CLICK, this, this.close);
        }

        onEnable() {
            this.title.text = Strings$1.get('tid#UI_064');
            this.shouyi.text = Strings$1.get('tid#UI_066');
            this.jiacheng.text = Strings$1.get('tid#UI_067');
            if (this._param) {
                this.point = this._param[0];
            }
            let yuNum = Number(ZhuliSystem.getCacheFish());
            let zhuliyu = Number(ZhuliSystem.getShareFish());
            let shangxian = Number(ZhuliSystem.getCapacity());
            let beiNum = Number(ZhuliSystem.getShareShell());
            this.desc.text = Strings$1.get('tid#UI_056');
            this.yuganNum.text = Strings$1.dealFishNumberText(yuNum) + '/' + Strings$1.dealFishNumberText(shangxian);
            this.yuganNum.x = this.yuImg.x + this.yuImg.width * this.yuImg.scaleX;
            this.yujiachengnum.text = Strings$1.dealFishNumberText(zhuliyu);
            this.yujiachengnum.x = this.yuImg.x + this.yuImg.width * this.yuImg.scaleX;
            this.beikeImg.x = this.yujiachengnum.x + this.yujiachengnum.width + 30;
            this.beikeNum.text = Strings$1.dealFishNumberText(beiNum);
            this.beikeNum.x = this.beikeImg.x + this.beikeImg.width * this.beikeImg.scaleX + 30;
            this.closebtn.on(Laya.Event.CLICK, this, this.close);
            // this.oneGet.text = Strings.get('tid#Btn_005');
            // this.oneGet.on(Laya.Event.CLICK, this, this.getaward);
            this.overseasGetReward.on(Laya.Event.CLICK, this, this.getaward);
            this.adBtn.on(Laya.Event.CLICK, this, this.adAward);
            this.shareBtn.on(Laya.Event.CLICK, this, this.shareAward);
            this.overseasGetReward.visible = true;
            // Laya.timer.once(1000 * Number(Const.get('QuayRewardSingleAppearTime')), this, function () {
            //     this.oneGet.visible = true;
            // })
            this.adBtn.visible = false;
            this.shareBtn.visible = false;
            // if (SdkHelper.hasVideo()) {
            //     this.adBtn.visible = true;
            //     this.shareBtn.visible = false;
            // } else {
            //     this.adBtn.visible = false;
            //     this.shareBtn.visible = true;
            // }
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            //EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
        }
        shareAward() {
            SdkHelper.shareAppMessage(Constants.ShareId.zhuliBaoxiangAward);
        }
        adAward() {
            SdkHelper.showRewardVideoAd(Constants.ShareId.zhuliBaoxiangAward);
        }
        getaward() {
            EventMgr.getInstance().postEvent('FETCH_WHARF_LETTERBOX', Laya.Handler.create(this, this.showFly,[Constants.FetchType.DIRECT]));
            //  System._FetchWharfLetterBox(Laya.Handler.create(this, this.showFly));
            this.close();
        }
        showFly(ad, res) {
            let rewards = res;
            for (let i in rewards) {
                let reward = 0;
                if (rewards[i].id == Constants.RewardType.FISH) {
                    reward = rewards[i].amount - PlayerSystem.getFish();
                }
                else if (rewards[i].id == Constants.RewardType.SHELL) {
                    reward = rewards[i].amount - PlayerSystem.getShell();
                }
                if (reward <= 0) {
                    continue;
                }
                let type = Constants.FlyType.TYPE2;
                if (ad == Constants.FetchType.AD) {
                    type = Constants.FlyType.TYPE3;
                }
                if (rewards[i].id == Constants.RewardType.FISH) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.FISH, reward, type, this.point.x, this.point.y);
                } else if (rewards[i].id == Constants.RewardType.SHELL) {
                    ItemFlyMgr.getInstance().flyReward(Constants.RewardType.SHELL, reward, type, this.point.x, this.point.y);
                }
            }
        }
        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.zhuliBaoxiangAward) {
                EventMgr.getInstance().postEvent('FETCH_WHARF_LETTERBOX', [Laya.Handler.create(this, this.showFly,[Constants.FetchType.AD]), 1]);
                this.close();
            }
        }
        onShareCallback(shareId) {
            this.onAdCallback(shareId);
        }


        onDisable() {
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
        }

    }

    class BuildingImageScale extends Laya.Script {

        constructor() {
            super();
            this.node1 = null;
            this.node2 = null;
            this.node3 = null;
            this.node4 = null;
            /** @prop {name:node1,type:Node}*/
            /** @prop {name:node2,type:Node}*/
            /** @prop {name:node3,type:Node}*/
            /** @prop {name:node4,type:Node}*/
            this.scale = 1;
        }

        onStart() {
        }

        imageScale() {
            if (this.node1) {
                this.node1.scaleX = this.node1.scaleY = this.scale;
            }
            if (this.node2) {
                this.node2.scaleX = this.node2.scaleY = this.scale;
            }
            if (this.node3) {
                this.node3.scaleX = this.node3.scaleY = this.scale;
            }
            if (this.node4) {
                this.node4.scaleX = this.node4.scaleY = this.scale;
            }
        }

        setScale(scale) {
            this.scale = scale;
        }
    }

    class StaffSkillTip extends Laya.Box {
        constructor() {
            super();
        }


        onEnable() {
            this.ring = this.getChildByName("ring");
            this.ringMask = this.ring.mask;
            this.skillImage = this.getChildByName("skill");
            this.ring.visible = false;
            this.show();
        }

        onDisable() {
            Laya.timer.clearAll(this);
        }
        onDestroy() {
            Laya.timer.clearAll(this);
        }

        addValue(skillData) {
            this.skillData = skillData;
            this.show();
        }

        show() {
            if (!this.skillData) {
                return;
            }
            let skillData = this.skillData;
            Laya.timer.clearAll(this);
            if (skillData.skillTimer == null) {
                this.destroy();
                return;
            }
            this.skillData = skillData;
            let timeData = skillData.getSkillTimeData();
            let alltime = timeData ? JSON.parse(timeData).EffectTime * 1000 : 2000;
            Laya.timer.frameLoop(5, this, function () {
                if (this.destroyed) {
                    Laya.timer.clearAll(this);
                    return;
                }
                let angle = Math.floor(360 - (this.skillData.skillTimer / alltime) * 360);
                if (this.skillData.skillTimer == null) {
                    Laya.timer.clearAll(this);
                    if (!this.destroyed) {
                        this.destroy();
                    }
                }
                if (this.ringMask == null) {
                    return;
                }
                this.ringMask.graphics.clear();
                this.ringMask.graphics.drawPie(0, 0, 31, angle - 90, 270, "#000000");
                this.ring.visible = true;
            });
            if (this.destroyed || this.skillImage == null || this.skillImage.destroyed) {
                Laya.timer.clearAll(this);
                return;
            }
            if (skillData.getEffect() == "FishGet") {
                this.skillImage.skin = "staff/skill1.png";
            } else if (skillData.getEffect() == "Order") {
                this.skillImage.skin = "staff/skill2.png";
            } else if (skillData.getEffect() == "Invite") {
                this.skillImage.skin = "staff/skill3.png";
            }
        }
    }

    class ViewAlignment extends Laya.Script {

        constructor() {
            super();
            // 1 靠右 2 靠左
            /** @prop {name:alignment, tips:"1 靠右", type:Int, default:1}*/
            this.alignment = 1;
            /** @prop {name:offsetX, tips:"偏移量", type:Int, default:0}*/
            this.offsetX = 0;
        }

        onEnable() {
            let renderW = 1827;
            let renderH = 2436;
            let heightScale = Laya.Browser.height/renderH;
            let realW = renderW*heightScale;
            let realH = renderH*heightScale;
            // let globalZero = new Laya.Point((realW/2 -Laya.Browser.width/2)/heightScale, 0);
            // let p = new Laya.Point(this.owner.x, this.owner.y);
            // this.owner.parent.localToGlobal(p);

            if(this.alignment == 1) {
                this.owner.x = Laya.Browser.width/heightScale + this.offsetX;
            } else if (this.alignment == 2) {
                this.owner.x = this.offsetX;
            }
        }

        onDisable() {
        }
    }

    class ArrowUI extends Laya.Script {

        constructor() {
            super();
        }

        switchBtn(sceneId) {
            sceneId = sceneId!=null ? sceneId : SceneMgr.getInstance().getCurrentSceneId();
            this.leftBtn.visible=false;
            this.rightBtn.visible=false;
            this.upBtn.visible=false;
            this.downBtn.visible=false;
            this.leftBtn.selected = false;
            this.rightBtn.selected = false;
            this.upBtn.selected = false;
            this.downBtn.selected = false;
            if(sceneId!=null) {
                let item = ConfigMgr$1.getInstance().getItemById("Scene", sceneId.toString());
                if(item) {
                    item.goto.forEach(element => {
                        if(element != "") {
                            let data = element.split(":");
                            switch (data[0]) {
                                case "left": {
                                    this.btnScene.left = Number(data[1]);
                                    this.leftBtn.visible = true;
                                    this.leftRedPointShow(true);
                                    break;
                                }
                                case "right": {
                                    this.btnScene.right = Number(data[1]);
                                    this.rightBtn.visible = true;
                                    break;
                                }
                                case "up": {
                                    this.btnScene.up = Number(data[1]);
                                    this.upBtn.visible = true;
                                    break;
                                }
                                case "down": {
                                    this.btnScene.down = Number(data[1]);
                                    this.downBtn.visible = true;
                                    break;
                                }
                            }
                        }
                    });
                }
            }
        }

        onEnable() {
            this.leftBtn = this.owner.getChildByName("leftBtn");
            this.rightBtn = this.owner.getChildByName("rightBtn");
            this.upBtn = this.owner.getChildByName("upBtn");
            this.downBtn = this.owner.getChildByName("downBtn");
            this.rightRedP=this.rightBtn.getChildByName('rightRedP');
            this.leftRedP=this.leftBtn.getChildByName('leftRedP');
            this.rightRedP.visible=false;
            this.leftRedP.visible=false;
            this.pointRightBowl1=false;
            this.pointRightBowl2=false;
            this.pointRightBowl3=false;
            this.pointLeftBool=false;
            this.LiveRightPoint();
            let renderH = 2436;
            let heightScale = Laya.Browser.height / renderH;
            let mid = Laya.Browser.width / 2 / heightScale;
            this.upBtn.x = mid - this.upBtn.height / 2 * this.upBtn.scaleY;
            this.downBtn.x = mid + this.downBtn.height / 2 * this.downBtn.scaleY;
            this.btnScene = { left: -1, right: -1, up: -1, down: -1 };
            this.leftBtn.on(Laya.Event.CLICK, this, function () {
                if (SceneMgr.getInstance().isSwitching) {
                    return;
                }
                SoundManager.getInstance().play("Sound_clickSound", 1);
                SceneMgr.getInstance().switchToScene("left", this.btnScene.left);
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[1]);
                EventMgr.getInstance().postEvent("left_RED_POINT_SHOW",[true]);
                DialogStatusMgr.getInstance().clear();
            });

            this.rightBtn.on(Laya.Event.CLICK, this, function () {
                if (SceneMgr.getInstance().isSwitching) {
                    return;
                }
                SoundManager.getInstance().play("Sound_clickSound", 1);
                SceneMgr.getInstance().switchToScene("right", this.btnScene.right);
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR",[1]);
                EventMgr.getInstance().postEvent("left_RED_POINT_SHOW",[true]);
                DialogStatusMgr.getInstance().clear();
            });

            this.upBtn.on(Laya.Event.CLICK, this, function () {
                if (SceneMgr.getInstance().isSwitching) {
                    return;
                }
                SoundManager.getInstance().play("Sound_clickSound", 1);
                SceneMgr.getInstance().switchScene(this.btnScene.up);
            });

            this.downBtn.on(Laya.Event.CLICK, this, function () {
                if (SceneMgr.getInstance().isSwitching) {
                    return;
                }
                SoundManager.getInstance().play("Sound_clickSound", 1);
                SceneMgr.getInstance().switchScene(this.btnScene.down);
            });

            EventMgr.getInstance().registEvent("SWITCH_ARROW_BTN", this, this.switchBtn);
            EventMgr.getInstance().registEvent("LIGHT_ARROW_BTN", this, this.lightBtn);
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().registEvent("RIGHT_RED_POINT_SHOW", this, this.rightRedPointShow);
            EventMgr.getInstance().registEvent("left_RED_POINT_SHOW", this, this.leftRedPointShow);
            EventMgr.getInstance().registEvent("CHECK_NAUTICALREWARD_REDPOINT", this, this.timerForNauticalReward);
            this.timerForNauticalReward();
        }

        lightBtn(type){
            switch (type) {
                case "left": {
                    this.leftBtn.selected = true;
                    this.rightBtn.selected = false;
                    this.upBtn.selected = false;
                    this.downBtn.selected = false;
                    break;
                }
                case "right": {
                    this.rightBtn.selected = true;
                    this.leftBtn.selected = false;
                    this.upBtn.selected = false;
                    this.downBtn.selected = false;
                    break;
                }
                case "up": {
                    this.upBtn.selected = true;
                    this.leftBtn.selected = false;
                    this.rightBtn.selected = false;
                    this.downBtn.selected = false;
                    break;
                }
                case "down": {
                    this.leftBtn.selected = false;
                    this.rightBtn.selected = false;
                    this.upBtn.selected = false;
                    this.downBtn.selected = true;
                    break;
                }
                default :{
                    this.leftBtn.selected = false;
                    this.rightBtn.selected = false;
                    this.upBtn.selected = false;
                    this.downBtn.selected = false;
                }
            }
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SWITCH_ARROW_BTN", this, this.switchBtn);
            EventMgr.getInstance().removeEvent("LIGHT_ARROW_BTN", this, this.lightBtn);
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("RIGHT_RED_POINT_SHOW", this, this.rightRedPointShow);
            EventMgr.getInstance().removeEvent("left_RED_POINT_SHOW", this, this.leftRedPointShow);
            EventMgr.getInstance().removeEvent("CHECK_NAUTICALREWARD_REDPOINT", this, this.timerForNauticalReward);
        }
        checkGuide(eventString){
            if (eventString == "GuideGroup_8") {
                let guide = GuideSystem.getGuide("GuideGroup_8");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.rightBtn);
                    guide.start();
                }
            } else if (eventString == "GuideGroup_11") {
                    let guide = GuideSystem.getGuide("GuideGroup_11");
                    if (guide) {
                        guide.registerObj("btn0", this);
                        guide.registerClick("btn0", this.rightBtn);
                    }
            } else if (eventString == "GuideGroup_12") {
                if (SceneMgr.getInstance().getCurrentSceneId() != Constants.BeachSceneId) {
                    let guide = GuideSystem.getGuide("GuideGroup_12");
                    if (guide) {
                        guide.registerObj("btn0", this);
                        guide.registerClick("btn0", this.rightBtn);
                    }
                }
            }
        }
        //激活一下红点
        LiveRightPoint(){
            let barValue=0;
            let exhaustTime = SyncSystem.getScene2ExhaustTime();
            let now = TimeUtils.getTimeNowOnServer();
            let total = Number(Const.get("CatFoodKeepTime"));
            let full = Number(Const.get("CatFoodNotFull"));
            if(exhaustTime==null || exhaustTime<=now) {
                barValue = 0;
            }else {
                let ratio = (exhaustTime - now)/1000/total;
                barValue = ratio;
            }
            let weakGuideFoodLimitStar = Const.getArray("CatFoodWeekGuideRange");
            let weakGuideFoodLimit = Number(Const.get("CatFoodWeekGuideRate"));
            if (!GuideSystem.isGuiding()&&PlayerSystem.getStar() < weakGuideFoodLimitStar[1] && PlayerSystem.getStar() >weakGuideFoodLimitStar[0] && barValue < weakGuideFoodLimit) {
                this.rightRedP.visible=true;
                this.pointRightBowl1=true;
            }
            let config=ConfigMgr$1.getInstance().getItemById("BuildingBase", '202');
            let diaoyuarray=config.FixedBuildings;
            for(let diaoyutaiID of diaoyuarray){
                if(BuildingSystem.hasBuildingDataByBid(diaoyutaiID)){
                    if( FishingSystem.getRemainTimes()){
                   
                    this.rightRedP.visible=true;
                    this.pointRightBowl2=true;
                     }
                 }
            }
          
            let dealFunc = function(res) {
                let visitorRewards = res.visitorRewards;
                System._updateVisitorGift(visitorRewards);
                let minnumber=Number(Const.get("BeachRewardsRedDot"));
                if(SyncSystem.getVisitorGifts().length>minnumber){
                    this.rightRedP.visible=true;
                    this.pointRightBowl3=true;
                  }
            };
            Service.findVisitorGift(Laya.Handler.create(this, dealFunc));
            let config1=ConfigMgr$1.getInstance().getItemById("BuildingBase", '302');
            let kuaitingarr=config1.FixedBuildings;
            for(let kuaitingID of kuaitingarr){
                if(BuildingSystem.hasBuildingDataByBid(kuaitingID)){
                    let hanghaiFunc = function(res) {
                        if(!res.nauticalInfo.inTrip){
                            this.leftRedP.visible=true;
                            this.pointLeftBool=true;
                        }
                        else {
                            let nowTime=TimeUtils.getTimeNowOnServer();
                            if(res.nauticalInfo.tripEndSec<nowTime/1000){
                                this.leftRedP.visible=true;
                                this.pointLeftBool=true;
                            }
                        }
                    };
                    Service. getNauticalInfo(Laya.Handler.create(this, hanghaiFunc));
                 }
            }
        }
        rightRedPointShow(type,pointBool){
            if(type==1){
                this.pointRightBowl1=pointBool; 
            }else if(type==2){
                this.pointRightBowl2=pointBool; 
            }else {
                this.pointRightBowl3=pointBool; 
            }    
            let pointRightBowl= this.pointRightBowl1|| this.pointRightBowl2|| this.pointRightBowl3;
            if(pointRightBowl){
               this.rightRedP.visible=true;
            }
            else {
               this.rightRedP.visible=false;
            } 
        }
        leftRedPointShow(pointBool){
            this.pointLeftBool=pointBool;
            if (BuildingSystem.getBuildingDataByPid("302") && pointBool) {
                let hanghaiFunc = function(res) {
                    if(((BuildingSystem.hasBuildingDataByBid("3010") && !(res.nauticalInfo[0].inTrip || res.nauticalInfo[0].adLock)) 
                    || !(res.nauticalInfo[1].inTrip || res.nauticalInfo[1].adLock) || !(res.nauticalInfo[2].inTrip || res.nauticalInfo[2].adLock))){
                        this.leftRedP.visible=true;
                        this.pointLeftBool=true;
                    }
                    else {
                        let nowTime=TimeUtils.getTimeNowOnServer();
                        if(res.nauticalInfo[0].tripEndSec<nowTime/1000 && res.nauticalInfo[0].inTrip 
                            || res.nauticalInfo[1].tripEndSec<nowTime/1000 && res.nauticalInfo[1].inTrip 
                            || res.nauticalInfo[2].tripEndSec<nowTime/1000 && res.nauticalInfo[2].inTrip){
                            this.leftRedP.visible=true;
                            this.pointLeftBool=true;
                        }
                        else {
                            this.leftRedP.visible=false; 
                        }
                    }
                };
                Service. getNauticalInfo(Laya.Handler.create(this, hanghaiFunc));
            }
            else {
                this.leftRedP.visible=false;
            }
            
        }

        timerForNauticalReward(){
            for(let i = 0; i < NauticalSystem$1.NauticalInfo.length; i++){
                let endSec = NauticalSystem$1.NauticalInfo[i].endSec * 1000;
                let nowTime = TimeUtils.getTimeNowOnServer();
                let remainingTime = endSec - nowTime;
                remainingTime = remainingTime > 0 ? remainingTime : 0;
                Laya.timer.once(remainingTime,this,this.leftRedPointShow,[true],false);
            }
        }
    }

    /**
     * 左侧下拉UI
     */
    class BottomUI extends Laya.Tree {
        constructor() {
            super();
            this.barMaskMaxHeight = 136;
            this._barValue = 0;
            EventMgr.getInstance().registEvent("RESET_ZHAOLAN_BAR", this, this.resetBarTween);
            EventMgr.getInstance().registEvent("ZHAOLAN_FULL", this, this.showFull);
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().registEvent("STOP_ZHAOLAN_BAR_TWEEN", this, this.stopBarTween);
            EventMgr.getInstance().registEvent("UPDATA_GIFTBTN_SHOW", this, this.giftbtnShow);

            EventMgr.getInstance().registEvent("DO_AD_ZHAOLAN", this, this.playADzhaolanAnim);
            EventMgr.getInstance().registEvent("GUIDE10_FREE_ZHAOLAN", this, this.guideFreeADZhaolan);
            EventMgr.getInstance().registEvent("NORMAL_CAT_ZHAOLAN", this, this.normalCatZhaoLan);
            EventMgr.getInstance().registEvent("SPECIAL_CAT_ZHAOLAN", this, this.specialCatZhaoLan);
            EventMgr.getInstance().registEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().registEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().registEvent("SDK_VIDEO_UNDONE", this, this.fail);
            EventMgr.getInstance().registEvent("SDK_AD_HAS_VIDEO_CHANGE", this, this.adBtnChange);
            EventMgr.getInstance().registEvent("UPDATA_STAFF_DATA", this, this.staffBtnShow);
            EventMgr.getInstance().registEvent("CLICK_ADOPT", this, this.cilickAdopt);
            // EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.starShowBtn);
        }

        onEnable() {
            this.status = 1;    //status 为1是收起down状态，2是展开up状态

            this.giftClickSign = 1;
            this.upBox = this.getChildByName("upBox");
            this.gift = this.getChildByName("giftBtn");
            this.gift.on(Laya.Event.CLICK, this, this.onGiftClick);
            let giftAni = new Laya.Skeleton();
            giftAni.load("anim1/maomizengli.sk");
            giftAni.x = this.gift.width / 2;
            giftAni.y = this.gift.height / 2;
            this.gift.addChild(giftAni);

            this.task = this.upBox.getChildByName("taskBtn");
            this.task.on(Laya.Event.CLICK, this, this.onTaskClick);
            this.shop = this.upBox.getChildByName("shopBtn");
            this.shop.on(Laya.Event.CLICK, this, this.onShopClick);
            this.zhaoLan = this.upBox.getChildByName("zhaolan");
            this.zhaoLan.on(Laya.Event.MOUSE_DOWN, this, this.onZhaoLanClick);
            this.zhaolanBar = this.zhaoLan.getChildByName("zhaolanBar");
            this.barMask = this.zhaolanBar.mask;
            this.barValue = 0;
            this.zhaolanBox = this.zhaoLan.getChildByName("box");
            this.zhaolanUp = this.zhaolanBox.getChildByName("up");
            this.zhaolanDown = this.zhaolanBox.getChildByName("down");
            this.zhaolanUp.visible = true;
            this.zhaolanDown.visible = false;
            this.zhaolanFull = this.zhaoLan.getChildByName("full");
            this.zhaolanFull.visible = false;
            this.zhaolanWord = this.zhaoLan.getChildByName("zhaolan");
            this.zhaolanWord.visible = true;
            this.adBox = this.upBox.getChildByName("adBox");
            this.adBtn = this.adBox.getChildByName("zhaoLan_15");
            this.adBtn.on(Laya.Event.CLICK, this, this.onADClick);
            this.adCD = this.adBox.getChildByName("cd");
            this.adCD.value = 0;
            this.adCD.visible = false;
            this.collect = this.upBox.getChildByName("collectBtn");
            this.collect.on(Laya.Event.CLICK, this, this.onCollectMenuClick);
            this.staffBtn = this.getChildByName("staffBtn");
            this.staffBtn.on(Laya.Event.CLICK, this, this.onStaffClick);

            this.missionBtn = this.getChildByName("missionBtn");
            this.missionBtn.visible = false;
            this.missionBtn.on(Laya.Event.CLICK, this, this.onMissionBtnClick);

            let taskRed = this.task.getChildByName("redPoint");
            taskRed.setCheckFunc(TaskSystem.hasCompletedTask);
            taskRed.refreshRedPoint();

            let collectRed = this.collect.getChildByName("redPoint");
            collectRed.setCheckFunc(PetSystem.needRedPoint);
            collectRed.refreshRedPoint();
            
            let staffRde = this.staffBtn.getChildByName("redPoint");;
            staffRde.setCheckFunc(StaffSystem.checkRedPoint);
            staffRde.refreshRedPoint();

            let shopRed = this.shop.getChildByName("redPoint");
            shopRed.setCheckFunc(function () {
                let ret = BuildingSystem.checkCoffeeCanBuyBuilding() || BuildingSystem.checkBeachCanBuyBuilding() || BuildingSystem.checkNauticalCanBuyBuilding() || FoodSystem$1.checkFoodShopRedShow() || ProjectSystem.checkProjectShopRedShow();
                return ret;
            });
            shopRed.refreshRedPoint();

            let missionRed = this.missionBtn.getChildByName("redPoint");
            missionRed.setCheckFunc(MissionSystem.checkRedPoint);
            missionRed.refreshRedPoint();

            this.adBtnRedPoint = this.adBtn.getChildByName("redPoint");

            this.normalCat = this.getChildByName("normalBox");
            this.specialCat = this.getChildByName("specialBox");
            this._checkGuideMap = new Map();

            //暂时用upBtn代替员工系统的坐标
            Constants.yuangongImagePoint = this.upBox.localToGlobal(new Laya.Point(this.collect.x, this.collect.y - 200));

            TaskMainUIMgr.getInstance().registButton("Building", this.shop);
            TaskMainUIMgr.getInstance().registButton("Click_Manual", this.zhaoLan);
            TaskMainUIMgr.getInstance().registButton("Food_Unlock", this.shop);
            TaskMainUIMgr.getInstance().registButton("Food_LvUp", this.shop);
            TaskMainUIMgr.getInstance().registButton("AD_Invite", this.gift);
            TaskMainUIMgr.getInstance().registButton("Click_Adopt", this.collect);

            this.adBtnChange();
            this.showAdRedPoint();
            this.startCheckGuide();
            this.staffBtnShow();
            if (GuideSystem.isGuiding()) {
                return;
            }
            AutoZhaolanMgr.getInstance().setZhaolanSpeed();
            // this.starShowBtn(PlayerSystem.getStar());
        }
        cilickAdopt(){
            TaskMainUIMgr.getInstance().destroyBtnAnim();
            TaskMainUIMgr.getInstance().registButton("Click_Adopt", this.collect);
        }
        startCheckGuide() {
            if (GuideSystem.isGuiding()) {
                let guideId = GuideSystem.getCurrentGuideId();
                if (guideId != null) {
                    if (!this._checkGuideMap.has(guideId)) {
                        this.checkGuide(guideId);
                    }
                }
            }
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("RESET_ZHAOLAN_BAR", this, this.resetBarTween);
            EventMgr.getInstance().removeEvent("ZHAOLAN_FULL", this, this.showFull);
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("STOP_ZHAOLAN_BAR_TWEEN", this, this.stopBarTween);
            EventMgr.getInstance().removeEvent("UPDATA_GIFTBTN_SHOW", this, this.giftbtnShow);
            EventMgr.getInstance().removeEvent("DO_AD_ZHAOLAN", this, this.playADzhaolanAnim);
            EventMgr.getInstance().removeEvent("NORMAL_CAT_ZHAOLAN", this, this.normalCatZhaoLan);
            EventMgr.getInstance().removeEvent("SPECIAL_CAT_ZHAOLAN", this, this.specialCatZhaoLan);
            EventMgr.getInstance().removeEvent("GUIDE10_FREE_ZHAOLAN", this, this.guideFreeADZhaolan);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_DONE", this, this.onAdCallback);
            EventMgr.getInstance().removeEvent("SDK_SHARE_DONE", this, this.onShareCallback);
            EventMgr.getInstance().removeEvent("SDK_VIDEO_UNDONE", this, this.fail);
            EventMgr.getInstance().removeEvent("SDK_AD_HAS_VIDEO_CHANGE", this, this.adBtnChange);
            EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", this, this.starShowBtn);
            if (this.ani1) {
                this.ani1.stop();
            }
            if (this.ani2) {
                this.ani2.stop();
            }
            if (this.jiayouAnim) {
                this.jiayouAnim.stop();
                this.jiayouAnim.destroy();
                this.jiayouAnim = null;
            }
            Laya.timer.clearAll(this);
        }


        onCollectMenuClick() {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            SyncSystem.updateSceneInfo();
            PetSystem.getPetsInfo(Laya.Handler.create(this, function () {
                Laya.Dialog.close("ui/CollectPop.scene");
                Laya.Dialog.open("ui/CollectPop.scene");
            }));
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onTaskClick() {
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            SyncSystem.updateSceneInfo();
            Laya.Dialog.close("ui/taskPop.scene");
            Laya.Dialog.open("ui/taskPop.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onShopClick() {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            SyncSystem.updateSceneInfo();
            SoundManager.getInstance().play("Sound_clickSound", 1);
            Laya.Dialog.close("ui/shopPop.scene");
            Laya.Dialog.open("ui/shopPop.scene");
        }

        onHomeClick() {
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            SoundManager.getInstance().play("Sound_clickSound", 1);
            SceneMgr.getInstance().switchScene(Constants.HomelandId);
        }

        onZhaoLanClick() {
            this.zhaoLan.scaleX = this.zhaoLan.scaleY = 1.08;
            this.zhaolanUp.visible = false;
            this.zhaolanDown.visible = true;
            this.zhaoLan.once(Laya.Event.MOUSE_UP, this, this.onZhaoLanUp);
            this.zhaoLan.once(Laya.Event.MOUSE_OUT, this, this.onZhaoLanOut);
        }

        onZhaoLanUp() {
            this.zhaoLan.off(Laya.Event.MOUSE_OUT, this, this.onZhaoLanOut);
            this.zhaoLan.scaleX = this.zhaoLan.scaleY = 1;
            this.zhaolanUp.visible = true;
            this.zhaolanDown.visible = false;
            AutoZhaolanMgr.getInstance().progressAdd();
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onZhaoLanOut() {
            this.zhaoLan.off(Laya.Event.MOUSE_UP, this, this.onZhaoLanUp);
            this.zhaolanUp.visible = true;
            this.zhaolanDown.visible = false;
            this.zhaoLan.scaleX = this.zhaoLan.scaleY = 1;
        }

        set barValue(value) {
            this._barValue = value;
            let height = Math.round(value * this.barMaskMaxHeight);
            this.barMask.graphics.clear();
            height = Math.max(1, height);
            this.barMask.graphics.drawRect(0, 0, 131, height, "#000000");
        }

        get barValue() {
            return this._barValue;
        }

        onADClick() {
            /*if (PlayerSystem.getStar() < Number(Const.get("AdsCatInviteLimit"))) {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon9", [Const.get("AdsCatInviteLimit")]);
                return;
            }*/
            if (GuideSystem.isGuiding()) {
                return;
            }
            // let adZhaoLanFirst = Laya.LocalStorage.getJSON("adZhaoLanFirst");
            // if (adZhaoLanFirst) {
            //     SdkHelper.showRewardVideoAd(Constants.ShareId.Zhaolan);
            //     EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["ads_invite_start"]);
            // } else {
            Laya.LocalStorage.setJSON("adZhaoLanFirst", true);
            Laya.Dialog.close("ui/VideoShareUI.scene");
            let shareStr = Strings$1.get("tid#UI_033");
            let videoStr = Strings$1.get("tid#UI_032");
            // let tipStr = Strings.get("tid#UI_057");
            let tipStr = null;
            let btnStr = Strings$1.get("tid#UI_058");
            Laya.Dialog.open("ui/VideoShareUI.scene", true, [Constants.ShareId.Zhaolan_AD_View, "DO_AD_ZHAOLAN", videoStr, shareStr, null, tipStr, btnStr]);
            // }
        }

        doADZhaolan(type) {
            if (!this.jiayouAnim) {
                this.jiayouAnim = new Laya.Skeleton();
                this.jiayouAnim.load("anim2/jiayou.sk", Laya.Handler.create(this, function () {
                    this.jiayouAnim.play("ganbadie", true);
                    this.jiayouAnim.x = this.adBox.width / 2;
                    this.jiayouAnim.y = this.adBox.height / 2;
                    this.adBox.addChild(this.jiayouAnim);
                }));
            } else {
                this.jiayouAnim.play("ganbadie", true);
                this.jiayouAnim.visible = true;
            }
            this.adBtn.visible = false;
            Laya.timer.loop(1000, this, this.timeUpdate);
            let time = Number(Const.get("CatInviteADTime")) + 1;
            this.ADleftTime = time;
            if (type == "free") {
                this.ADleftTime = 30;
            }
            AutoZhaolanMgr.getInstance().startFastZhaolan();
            this.timeUpdate();
        }


        timeUpdate() {
            if (GuideSystem.isGuiding()) {
                this.adBtn.visible = true;
                this.jiayouAnim.visible = false;
                this.adCD.visible = false;
                return;
            }
            this.ADleftTime--;
            if (this.ADleftTime < 0) {
                this.adCD.visible = false;
                this.adBtn.visible = true;
                this.jiayouAnim.visible = false;
                Laya.timer.clear(this, this.timeUpdate);
                AutoZhaolanMgr.getInstance().setZhaolanSpeed();
                if (SceneMgr.getInstance().getCurrentSceneId() != Constants.FishingSceneId) {
                    SoundManager.getInstance().play("Sound_bgm1");
                }
                let catInviteRedPointTime = Number(Const.get("CatInviteRedPointTime")) * 1000;
                this.hideAdRedPoint(catInviteRedPointTime);
            } else {
                this.jiayouAnim.visible = true;
                this.adBtn.visible = false;
                let m = Math.floor(this.ADleftTime / 60);
                let s = this.ADleftTime % 60;
                this.adCD.value = m + ":" + s;
                this.adCD.visible = true;
            }
        }

        onUpClick() {
            this.downBox.visible = true;
            this.upBtn.visible = false;
            this.downBtn.visible = true;
            this.ani2.stop();
            this.ani1.play(0, false);
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onDownClick() {
            this.downBox.visible = false;
            this.upBtn.visible = true;
            this.downBtn.visible = false;
            this.ani1.stop();
            this.ani2.play(0, false);
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onGiftClick() {
            if (this.giftClickSign > 0) {
                this.giftClickSign = 0;
                SoundManager.getInstance().play("Sound_clickSound", 1);
                SyncSystem.findVisitorGift(Laya.Handler.create(this, this.openGiftDialog));
            }
        }

        openGiftDialog() {
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            Laya.Dialog.close("ui/visitorGiftPop.scene");
            Laya.Dialog.open("ui/visitorGiftPop.scene");
            SyncSystem.updateSceneInfo();
            this.giftClickSign = 1;
        }

        //updown 是否需要展示红点，取决于内部各个模块的红点状态
        checkUpdownRed() {
            return PetDataMgr.getInstance().needRedPoint();
        }
        showBtn(type) {
            switch (type) {
                case 1: {
                    this.zhaoLan.visible = true;
                    this.adBox.visible = true;
                    this.gift.visible = false;
                    this.staffBtn.visible = StaffSystem.getStaffUnlock();
                    break;
                }
                case 2: {
                    this.zhaoLan.visible = false;
                    this.adBox.visible = false;
                    this.gift.visible = SyncSystem.getVisitorRewardsExist();
                    this.staffBtn.visible = false;
                    break;
                }
                case 3: {
                    this.zhaoLan.visible = false;
                    this.adBox.visible = false;
                    this.gift.visible = false;
                    this.staffBtn.visible = false;
                    break;
                }
            }
        }

        giftbtnShow() {
            this.showBtn(SceneMgr.getInstance().getCurrentSceneId());
        }

        resetBarTween(value = null) {
            this.stopBarTween();
            if (value != null) {
                this.barValue = value;
            }
            this.startBarTween();
        }

        startBarTween() {
            if (GuideSystem.isGuiding()) {
                return;
            }
            this.barTween = Laya.Tween.to(this, { barValue: 1 }, AutoZhaolanMgr.getInstance().getCatInviteAutoClick() * AutoZhaolanMgr.getInstance().getLeftTimes(), null, Laya.Handler.create(this, this.stopBarTween));
        }

        stopBarTween() {
            if (this.barTween) {
                Laya.Tween.clear(this.barTween);
                this.barTween = null;
            }
        }
        showFull(full) {
            if (full) {
                this.zhaolanFull.visible = true;
                this.zhaolanWord.visible = false;
                this.stopBarTween();
            } else {
                this.zhaolanWord.visible = true;
                this.zhaolanFull.visible = false;
            }
        }

        checkGuide(eventString) {
            this._checkGuideMap.set(eventString, eventString);
            if (eventString == "GuideGroup_2") {
                let guide = GuideSystem.getGuide("GuideGroup_2");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.shop);
                    guide.start();
                }
            }
            if (eventString == "GuideGroup_3") {
                let guide = GuideSystem.getGuide("GuideGroup_3");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.shop);
                    guide.start();
                }
            }
            if (eventString == "GuideGroup_5") {
                let guide = GuideSystem.getGuide("GuideGroup_5");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.shop);
                    guide.start();
                }
            }
            if (eventString == "GuideGroup_6") {
                let guide = GuideSystem.getGuide("GuideGroup_6");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.zhaoLan);
                    guide.start();
                    EventMgr.getInstance().postEvent('GOUDE_COLSE_TALK');
                }
            }
            if (eventString == "GuideGroup_8") {
                let guide = GuideSystem.getGuide("GuideGroup_8");
                if (guide) {
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", this.shop);
                }
            }
            if (eventString == "GuideGroup_9") {
                let guide = GuideSystem.getGuide("GuideGroup_9");
                if (guide) {
                    guide.registerObj("btn2", this);
                    guide.registerClick("btn2", this.gift);
                }
            }
            if (eventString == "GuideGroup_10") {
                let guide = GuideSystem.getGuide("GuideGroup_10");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.adBtn);
                }
            }
            if (eventString == "GuideGroup_11") {
                let guide = GuideSystem.getGuide("GuideGroup_11");
                if (guide) {
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", this.shop);
                }
            }
            if (eventString == "GuideGroup_15") {
                let guide = GuideSystem.getGuide("GuideGroup_15");
                if (guide) {
                    guide.registerObj("btn0", this);
                    guide.registerClick("btn0", this.shop);
                }
            }
        }

        guideFreeADZhaolan() {
            this.playADzhaolanAnim("free");
            let m = Math.floor(this.ADleftTime / 60);
            let s = this.ADleftTime % 60;
            this.adCD.value = m + ":" + s;
        }

        normalCatZhaoLan() {
            if (this.zhaoLan.visible && !this.specialCat.visible) {
                this.normalCat.visible = true;
                this.normalCat.y = 0;
                this.normalCat.alpha = 1;
                Laya.Tween.clearAll(this.normalCat);
                Laya.Tween.to(this.normalCat, { y: -100, alpha: 0 }, 1000, null, Laya.Handler.create(this, function () {
                    this.normalCat.visible = false;
                }));

            }
        }

        specialCatZhaoLan() {
            if (this.zhaoLan.visible) {
                this.specialCat.visible = true;
                this.specialCat.y = 10;
                this.specialCat.alpha = 1;
                this.normalCat.visible = false;
                Laya.Tween.clearAll(this.specialCat);
                Laya.Tween.to(this.specialCat, { y: -90, alpha: 0 }, 1000, null, Laya.Handler.create(this, function () {
                    this.specialCat.visible = false;
                }));
            }
        }

        playADzhaolanAnim(type) {
            if (type == "error") {
                if (GuideSystem.isGuiding()) {
                    EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["ads_invite_guide_fail"]);
                }
                return;
            }
            if (type != "free" && GuideSystem.isGuiding()) {
             
            }
            EventMgr.getInstance().postEvent("GUIDE_GUIDE10_AD");
            SoundManager.getInstance().play("Sound_clickSound", 1);
            SoundManager.getInstance().play("Sound_bgm3");
            this.ADzhaolanAnim = new Spine();
            Laya.stage.addChild(this.ADzhaolanAnim);
            this.ADzhaolanAnim.pos(Laya.stage.width / 2, Laya.stage.height / 2);
            this.ADzhaolanAnim.loadAnim("anim3/jiayou2.sk");
            this.ADzhaolanAnim.playAnim("ganbadie2", false);
            this.ADzhaolanAnim.addCompleteHandler(Laya.Handler.create(this, function () {
                this.onADFinish(type);
            }));
        }

        onAdCallback(shareId) {
            if (shareId == Constants.ShareId.Zhaolan) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["ads_invite_success"]);
                this.playADzhaolanAnim();
            }
        }

        onShareCallback(shareId) {
            if (shareId == Constants.ShareId.Zhaolan) {
                this.playADzhaolanAnim();
            }
        }

        onADFinish(type) {
            this.doADZhaolan(type);
            let catInviteRedPointTime = Number(Const.get("CatInviteRedPointTime")) * 1000;
            this.hideAdRedPoint(catInviteRedPointTime);
            this.ADzhaolanAnim.destroy();
        }

        fail(shareId) {
            if (shareId == Constants.ShareId.Zhaolan) {
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["ads_invite_fail"]);
                let catInviteRedPointADFailTime = Number(Const.get("CatInviteRedPointADFailTime")) * 1000;
                this.hideAdRedPoint(catInviteRedPointADFailTime);
            }
        }

        adBtnChange() {
            if (this.adBtn) {
                let ad = this.adBtn.getChildByName('ad');
                if (SdkHelper.hasVideo()) {
                    ad.skin = 'pinjie_btn_2/ADzhaolan.png';
                    if (this._adRedShow) {
                        this.showAdRedPoint();
                    }
                } else {
                    ad.skin = 'pinjie_btn_2/shzhaolan.png';
                    let catInviteRedPointTime = Number(Const.get("CatInviteRedPointTime")) * 1000;
                    this.hideAdRedPoint(catInviteRedPointTime);
                }
            }
        }

        showAdRedPoint() {
            if (this.ADleftTime != null && this.ADleftTime > 0) {
                let catInviteRedPointTime = Number(Const.get("CatInviteRedPointTime"));
                let leftTime = catInviteRedPointTime * 1000 + this.ADleftTime * 1000;
                this.hideAdRedPoint(leftTime);
                return;
            }
            if (!SdkHelper.hasVideo()) {
                this._adRedShow = true;
                this.adBtnRedPoint.visible = false;
                return;
            }
            this.adBtnRedPoint.visible = true;
        }

        hideAdRedPoint(time) {
            this._adRedShow = null;
            this.adBtnRedPoint.visible = false;
            if (time) {
                Laya.timer.clear(this, this.showAdRedPoint);
                Laya.timer.once(time, this, this.showAdRedPoint);
            }
        }

        onStaffClick() {
            Laya.Dialog.close("ui/StaffPop.scene");
            Laya.Dialog.open("ui/StaffPop.scene");
        }

        staffBtnShow() {
            this.staffBtn.visible = StaffSystem.getStaffUnlock();
        }

        onMissionBtnClick() {
            MissionSystem.getMissionInfo(Laya.Handler.create(this, function () {
                Laya.Dialog.open("ui/MissionMain.scene");
            }));
        }

        starShowBtn(star) {
            let needStar = Const.get("CatMissionUnlockCondition");
            if (star >= needStar) {
                MissionSystem.getMissionInfo();
                this.missionBtn.visible = true;
                EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", this, this.starShowBtn);
            }
        }

    }

    class CollectGiftUI extends Laya.Dialog {
        constructor() {
            super();
        }

        onEnable() {
            this._index = 0;
            this.list = this.getChildByName("list");
            this.scrollBar = this.list.getChildByName("scrollBar1");
            this.list._removePreScrollBar();
            this.list.scrollBar = this.scrollBar;
            this.scrollBar.name = "scrollBar";
            // this.scrollBar.rollRatio = 0;
            let dataMap = CollectGiftDataMgr.getInstance().getGiftData();
            let amount = 0;
            let own = 0;
            this.array = new Array();
            let obj1 = { arr: [], type: 1 };
            let obj2 = { arr: [], type: 2 };
            let obj3 = { arr: [], type: 3 };
            let obj4 = { arr: [], type: 4 };
            dataMap.forEach(
                (data, id) => {
                    if (data._type == 1) {
                        obj1.arr.push(data);
                    } else if (data._type == 2) {
                        obj2.arr.push(data);
                    } else if (data._type == 3) {
                        obj3.arr.push(data);
                    }else if (data._type == 4) {
                        obj4.arr.push(data);
                    }
                    amount ++;
                    if(data.getLock()){
                        own ++;
                    }
                });
            // this.list.array = [obj1, obj2, obj3];
            this.list.arrayCache = [obj1, obj2, obj3,obj4];
            this.scrollBar.value = 0;
            this.parent.calCatCollectRatio(amount, own);
        }

        closeUI() {
            CollectGiftDataMgr.getInstance().clear();
            if (this.parent != null)
                this.parent.close();
        }
    }

    class GiftList extends CommonList {

        constructor() {
            super();
            this.startY = 70;
            this._heightMap = new Map();
        }


        get arrayCache() {
            return this._arrayCache;
        }

        set arrayCache(data) {
            this._arrayCache = data;
            this._heightMap.clear();
            for (let i = 0; i < this.arrayCache.length; i++) {
                let num = this.arrayCache[i].arr.length;
                let height = this.startY + Math.ceil(num / 4) * 205 + 100;
                if (height) {
                    this._heightMap.set(i, height);
                }
            }
            this.array = this.arrayCache;
            let height = 0;
            this._heightMap.forEach((value, key) => {
                height += value;
            });
            height -= this.height;
            height = Math.max(0, height);
            this._scrollBar.setScroll(0,  height - 100, this._scrollBar.value);
        }

        posCell(cell, cellIndex) {
            if (this.repeatX == 1 && this._heightMap.size > 0) {
                if (!this._scrollBar)
                    return;
                var pos = this.__calcIndexPos(cellIndex);
                cell._y = pos;
            } else {
                super.posCell(cell, cellIndex);
            }
        }

        __calcIndexPos(index) {
            let value = 20;
            for (let i = 0; i < index; i++) {
                let height = this._heightMap.get(i);
                if (height != null) {
                    value += height;
                }
            }
            return value;
        }


        // changeCells() {
        //     if (this.repeatX == 1 && this._heightMap.size > 0) {
        //         this._cellChanged = false;
        //         if (this._itemRender) {
        //             this.scrollBar = this.getChildByName("scrollBar");
        //             var cell = this._getOneCell();
        //             var cellWidth = (cell.width + this._spaceX) || 1;
        //             var cellHeight = (cell.height + this._spaceY) || 1;
        //             if (this._width > 0)
        //                 this._repeatX2 = this._isVertical ? Math.round(this._width / cellWidth) : Math.ceil(this._width / cellWidth);
        //             if (this._height > 0)
        //                 this._repeatY2 = this._isVertical ? Math.ceil(this._height / cellHeight) : Math.round(this._height / cellHeight);
        //             var listWidth = this._width ? this._width : (cellWidth * this.repeatX - this._spaceX);
        //             var listHeight = this._height ? this._height : (cellHeight * this.repeatY - this._spaceY);
        //             this._cellSize = this._isVertical ? cellHeight : cellWidth;
        //             this._cellOffset = this._isVertical ? (cellHeight * Math.max(this._repeatY2, this._repeatY) - listHeight - this._spaceY) : (cellWidth * Math.max(this._repeatX2, this._repeatX) - listWidth - this._spaceX);
        //             if (this._isVertical && this.vScrollBarSkin) {
        //                 this._scrollBar.height = listHeight;
        //             }
        //             else if (!this._isVertical && this.hScrollBarSkin)
        //                 this._scrollBar.width = listWidth;
        //             this.setContentSize(listWidth, listHeight);
        //             var numX = this._isVertical ? this.repeatX : this.repeatY;
        //             var numY = (this._isVertical ? this.repeatY : this.repeatX) + (this._scrollBar ? 1 : 0);
        //             this._createItems(0, numX, numY);
        //             this._createdLine = numY;
        //             if (this._array) {
        //                 this.array = this._array;
        //                 this.runCallLater(this.renderItems);
        //             }
        //         }
        //     } else {
        //         super.changeCells();
        //     }
        // }

        changeCells() {
            this._cellChanged = false;
            if (this._itemRender) {
                this.scrollBar = this.getChildByName("scrollBar");
                var cell = this._getOneCell();
                var cellWidth = (cell.width + this._spaceX) || 1;
                var cellHeight = (cell.height + this._spaceY) || 1;
                if (this._width > 0)
                    this._repeatX2 = this._isVertical ? Math.round(this._width / cellWidth) : Math.ceil(this._width / cellWidth);
                if (this._height > 0)
                    this._repeatY2 = this._isVertical ? Math.ceil(this._height / cellHeight) : Math.round(this._height / cellHeight);
                var listWidth = this._width ? this._width : (cellWidth * this.repeatX - this._spaceX);
                var listHeight = this._height ? this._height : (cellHeight * this.repeatY - this._spaceY);
                this._cellSize = this._isVertical ? cellHeight : cellWidth;
                this._cellOffset = this._isVertical ? (cellHeight * Math.max(this._repeatY2, this._repeatY) - listHeight - this._spaceY) : (cellWidth * Math.max(this._repeatX2, this._repeatX) - listWidth - this._spaceX);
                if (this._isVertical && this.vScrollBarSkin)
                    this._scrollBar.height = listHeight;
                else if (!this._isVertical && this.hScrollBarSkin)
                    this._scrollBar.width = listWidth;
                this.setContentSize(listWidth, listHeight);
                var numX = this._isVertical ? this.repeatX : this.repeatY;
                var numY = (this._isVertical ? this.repeatY : this.repeatX) + (this._scrollBar ? 1 : 0);
                this._createItems(0, numX, numY);
                this._createdLine = numY;
                if (this._array) {
                    this.array = this._array;
                    this.runCallLater(this.renderItems);
                }
            }

        }

        onScrollBarChange(e = null) {
    		if(this._heightMap.size > 0) {
    		    this.runCallLater(this.changeCells);
    		    var scrollValue = this._scrollBar.value;
    		    var lineX = (this._isVertical ? this.repeatX : this.repeatY);
    		    var lineY = (this._isVertical ? this.repeatY : this.repeatX);
    			var scrollLine = Math.floor(scrollValue / this._cellSize);
    			scrollLine = Math.max(0, scrollLine);
    		    let tmpHeight = 0;
                for(let i = 0; i<=scrollLine; i ++) {
                    let height = this._heightMap.get(i);
                    if(height!=null) {
                        tmpHeight += height;
                    }else {
                        tmpHeight += this._cellSize;
                    }
                    if(tmpHeight>=scrollValue) {
                        scrollLine = i;
                        break;
                    }
                }
    		    if (!this.cacheContent) {
    		        var index = scrollLine * lineX;
    		        var num = 0;
    		        if (index > this._startIndex) {
    		            num = index - this._startIndex;
    		            var down = true;
    		            var toIndex = this._startIndex + lineX * (lineY + 1);
    		            this._isMoved = true;
    		        }
    		        else if (index < this._startIndex) {
    		            num = this._startIndex - index;
    		            down = false;
    		            toIndex = this._startIndex - 1;
    		            this._isMoved = true;
    		        }
    		        for (var i = 0; i < num; i++) {
    		            if (down) {
    		                var cell = this._cells.shift();
    		                this._cells[this._cells.length] = cell;
    		                var cellIndex = toIndex + i;
    		            }
    		            else {
    		                cell = this._cells.pop();
    		                this._cells.unshift(cell);
    		                cellIndex = toIndex - i;
    		            }
    		            var pos = this.__calcIndexPos(cellIndex);
    		            this._isVertical ? cell.y = pos : cell.x = pos;
    		            this.renderItem(cell, cellIndex);
    		        }
    		        this._startIndex = index;
    		        this.changeSelectStatus();
    		    }
    		    else {
    		        num = (lineY + 1);
    		        if (this._createdLine - scrollLine < num) {
    		            this._createItems(this._createdLine, lineX, this._createdLine + num);
    		            this.renderItems(this._createdLine * lineX, 0);
    		            this._createdLine += num;
    		        }
    		    }
    		    var r = this._content._style.scrollRect;
    		    if (this._isVertical) {
    		        r.y = scrollValue - this._offset.y;
    		        r.x = -this._offset.x;
    		    }
    		    else {
    		        r.y = -this._offset.y;
    		        r.x = scrollValue - this._offset.x;
    		    }
    		    this._content.scrollRect = r;
    		}else {
    			super.onScrollBarChange(e);
    		}
    	}
    }

    class GiftsBox extends Laya.Box {
        constructor() {
            super();
        }
        onEnable() {
            this.nameLab = this.getChildByName("nameLab");
            this.line = this.getChildByName("line");
        }
        setItem(data) {
            let type = data.type;
            switch(type){
                case 1:this.nameLab.text = Strings$1.get("tid#CollectionsTitle_1");break;
                case 2:this.nameLab.text = Strings$1.get("tid#CollectionsTitle_2");break;
                case 3:this.nameLab.text = Strings$1.get("tid#CollectionsTitle_3"); break;
                case 4:this.nameLab.text = Strings$1.get("tid#CollectionsTitle_4");this.line.visible = false; break;
            }
            let array = data.arr;
            this.startY = 80;
            for (let index = 0; index < array.length; index++) {
                const element = array[index];
                if (this.prefabData != null) {
                    this.createGiftItem(this.prefabData, index, element);
                }
                Laya.loader.load("prefab/ui/GiftItem.prefab", Laya.Handler.create(this, function (prefab) {
                    this.prefabData = prefab;
                    this.createGiftItem(this.prefabData, index, element);
                }));
            }
            this.line.y = this.startY + Math.ceil(array.length / 4) * 205 +30;
            this.height = this.line.y;
            this.itemHeight = this.line.y;
        }

        createGiftItem(prefab, index, data) {
            let gift = prefab.create();
            gift.x = (index % 4) * 185 + 15;
            gift.y = Math.floor(index / 4) * 205 + this.startY;
            this.addChild(gift);
            gift.setItem(data);
            return gift;
        }

        set itemHeight(height) {
            this.trueHeight = height;
        }

        get itemHeight() {
            return this.trueHeight;
        }
    }

    class BuildingItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(buildingId) {
            this.offAll();
            this.setBuildingId(buildingId, null);
        }

        setBuildingId(buildingId, location) {
            if(buildingId==null){
                return;
            }
            let item = ConfigMgr$1.getInstance().getItemById("Building", buildingId);
            if(item==null){
                return;
            }
            this.hookb=false;//判定是否放置
            this.buildingId = buildingId;
            this.location = location;
            this.item = this.getChildByName("item");
            this.item1 = this.getChildByName("item1");
            let fish = this.getChildByName("fish");
            let title = this.getChildByName("title");
            title.text = Strings$1.dealFishNumberText(item.Cost,1);
            // title.align = "center";
            fish.x = (this.width - fish.width - title.width - 20) / 2;
            title.x = fish.x + fish.width + 15;
            let hook=this.getChildByName("hook");

            let icon = this.getChildByName("icon");
            this.lock = this.getChildByName("lock");
            this.redPoint = this.getChildByName("redPoint");
            let char = item.ShopItems.substr(item.ShopItems.lastIndexOf(".")-1,1);

            Laya.loader.load(
                [{
                    url: item.ShopItems,
                    type: Laya.Loader.IMAGE
                }],
                Laya.Handler.create(this, this.setIcon, [icon, item.ShopItems, buildingId])
            );

            if(!BuildingSystem.checkBuildingUnlocked(buildingId)){
                // this.disable();
                this.lock.visible=true;
                fish.visible = false;
                title.visible = false;
                hook.visible = false;
                this.hookb=false;
                this.on(Laya.Event.CLICK,this,this.showUnlock, [item.Limit]);
                this.item1.visible = false;
                this.redPoint.visible = false;
            }else {
                this.lock.visible=false;
                if(BuildingSystem.hasBuildingDataByBid(buildingId)){
                    this.item1.visible = true;
                    let buildingData = BuildingSystem.getBuildingDataByBid(buildingId);
                    if(/* location && */ buildingData.getLocationData()){
                        //已经放置
                        hook.visible = true;
                        this.hookb=true;
                    }else {
                        hook.visible = false;
                        this.hookb=false;
                    }
                    this.on(Laya.Event.CLICK,this,this.showPlacePop);
                }else {
                    this.item1.visible = false;
                    hook.visible = false;
                    this.hookb=false;
                    this.on(Laya.Event.CLICK,this,this.showBuyPop);
                    this.redPoint.visible = (BuildingSystem.checkBuyBuildingAvailable(buildingId) || BuildingSystem.checkADBuy(buildingId)) && (PlayerSystem.getStar() < Number(Const.get("ShopRedDot")));
                }
            }
        }

        setIcon(icon, skin, buildingId) {
            if(this.destroyed)
                return;
            if(this.buildingId!=buildingId)
                return;
            icon.skin = skin;
            let scalex = 128 / icon.displayWidth;
            let scaley = 134 / icon.displayHeight;
            if (scalex <= scaley) {
                icon.scaleX = scalex;
                icon.scaleY = scalex;
            } else {
                icon.scaleX = scaley;
                icon.scaleY = scaley;
            }
            this.nextGuide(this.buildingId); 
            this.checkShowWeakGuideAnim();
        }

        disable(){
            let icon = this.getChildByName("icon");
            // icon.visible=false;
            // this.texture = "shopui/xiaomianban2.png";
            this.disabled = true;
        }

        showUnlock(limit) {
            TipsPopMgr.getInstance().showTipsById("tid#starUnlockNumber", [limit]);
        }

        showBuyPop() {
            if(this.disabled==true){
                return;
            }
            GameContext.CurrentShopBuildingBase = this.location;
            GameContext.CurrentShopBuilding = this.buildingId;
            Laya.Dialog.close("ui/confirmPop.scene");
            Laya.Dialog.open("ui/confirmPop.scene", false);
        }

        showPlacePop() {
            if(this.disabled==true){
                return;
            }
            GameContext.CurrentShopBuildingBase = this.location;
            GameContext.CurrentShopBuilding = this.buildingId;
            Laya.Dialog.close("ui/placePop.scene");
            Laya.Dialog.open("ui/placePop.scene", false);
        }

        nextGuide(buildingId) {
            if (!GuideSystem.isGuiding()) {
                return;
            }
            if (Number(buildingId) == 1060) {
                Laya.timer.once(200, this, function () {
                    let guide = GuideSystem.getGuide("GuideGroup_2");
                    if (guide) {
                        guide.registerObj("btn1", this);
                        guide.registerClick("btn1", this);
                    }
                });
            } else if (Number(buildingId) == 1030) {
                Laya.timer.once(500, this, function () {
                    let guide = GuideSystem.getGuide("GuideGroup_5");
                    if (guide) {
                        guide.registerObj("btn1", this);
                        guide.registerClick("btn1", this);
                    }
                });
            } else if (Number(buildingId) == 2080) {
                Laya.timer.once(200, this, function () {
                    let guide = GuideSystem.getGuide("GuideGroup_8");
                    if (guide) {
                        guide.registerObj("btn2", this);
                        guide.registerClick("btn2", this);
                    }
                });
            } else if (Number(buildingId) == 2020) {
                Laya.timer.once(500, this, function () {
                    let guide = GuideSystem.getGuide("GuideGroup_11");
                    if (guide) {
                        guide.registerObj("btn2", this);
                        guide.registerClick("btn2", this);
                    }
                });
            }
        }

        onDestroy(){
            Laya.timer.clearAll(this);
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
            }
            this.weakAnim = null;
        }

        checkShowWeakGuideAnim() {
            if (!GuideSystem.isGuiding() && TaskMainSystem.getIsTasking()) {
                let task = TaskMainSystem.getCurrentData();
                if (task.getGuide() == "Building" && !TaskMainSystem.checkEnought()) {
                    let building = BuildingSystem.checkCanBuyCheapestBuilding();
                    if (building && building.Id == this.buildingId && (this.weakAnim == null || this.weakAnim.destroyed)) {
                        this.weakAnim = new Spine();
                        this.weakAnim.loadAnim("anim1/xinshouyindao.sk");
                        this.weakAnim.playAnim("zhuazi2", true);
                        this.weakAnim.x = this.width / 2;
                        this.weakAnim.y = this.height / 2;
                        this.weakAnim.zOrder = 2;
                        this.weakAnim.scaleX = 1 / this.scaleX;
                        this.weakAnim.scaleY = 1 / this.scaleY;
                        this.addChild(this.weakAnim);
                        EventMgr.getInstance().registEvent("SHOW_WEAK_GUIDE_BUILDING_ITEM", this, this.setWeakGuideAnimVisible);
                        return;
                    }
                }
            }
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
            }
            this.weakAnim = null;
        }

        onDisable(){
            EventMgr.getInstance().removeEvent("SHOW_WEAK_GUIDE_BUILDING_ITEM", this, this.setWeakGuideAnimVisible);
        }

        setWeakGuideAnimVisible(visible) {
            if (visible) {
                this.checkShowWeakGuideAnim();
            } else {
                if (this.weakAnim != null && !this.weakAnim.destroyed) {
                    this.weakAnim.destroy();
                }
                this.weakAnim = null;
            }
        }
    }

    class GetNewCat extends Laya.Box{
        constructor(){
            super();
        }
        getNewCatSkin(imgName){
            Laya.stage.addChild(this);
            let name=this.getChildByName("ImgGetNewCat");
            this.pivot(300,60);
            let locX=Laya.stage.width/2;
            let locY=Laya.stage.height/2;
            this.x=locX-100;
            this.y=locY+200;
            this.alpha=0;
            name.skin=imgName;
            Laya.Tween.to(this,{y:locY-100,alpha:1},600,Laya.Ease.circOut,Laya.Handler.create(this,function(){
                Laya.Tween.to(this,{y:locY-400,alpha:0},600,Laya.Ease.circIn,Laya.Handler.create(this,function(){
                    this.removeSelf();
                }));
            }));
        }
        onDestroy(){
            Laya.Tween.clearAll(this);
        }

        setCatStr(str) {
            let text = this.getChildByName("TextGetNewCat");
            text.text = Strings$1.get("tid#UI_156");
            let bg = this.getChildByName("bg");
            let cat = this.getChildByName("ImgGetNewCat");
            if (str) {
                text.text = str;
                bg.width = text.width + 210;
                bg.x = (this.width - bg.width) / 2;
                cat.x = bg.x + 30;
                text.x = cat.x + 150;
            } 
        }
    }

    class GiftItem extends Laya.Box {
        constructor() {
            super();
        }
        onEnable() {
            this.GiftItem = this.getChildByName("GiftIcon");
            this.Lock = this.getChildByName("Lock");
            this.bgLock = this.getChildByName("bgLock");
            this.bgUnlock = this.getChildByName("bgUnlock");
            this.quality = this.getChildByName("quality");
        }

        setItem(data) {
            if (data._id == -1) {
                this.visible = false;
            } else {
                this.offAll(Laya.Event.CLICK);
                this.quality.visible = false;
                this.visible = true;
                this.GiftItem.skin = data._image;
                this.Lock.visible = (data._lock == 0 ? true : false);
                this.bgLock.visible = (data._lock == 0 ? true : false);
                this.bgUnlock.visible = (data._lock == 0 ? false : true);
                if (data._type == 3) {
                    this.on(Laya.Event.CLICK, this, this.showInfo, [data._realId, "tool"]);
                } else if (data._type == 2) {
                    this.on(Laya.Event.CLICK, this, this.showInfo, [data._realId, "bait"]);
                } else if (data._type == 1 || data._type == 4) {
                    this.on(Laya.Event.CLICK, this, this.showInfo, [data._realId, "toy"]);
                }
                let quality = data.getQuality();
                if (quality != null) {
                    this.quality.visible = true;
                    this.quality.skin = "shopitems/quality" + quality + ".png";
                } else {
                    this.quality.visible = false;
                }
            }


        }

        showInfo(id, type) {
            GameContext.CurrentInfoId = id;
            GameContext.CurrentInfoType = type;
            if(type == "toy"){
                let toyConfig = ConfigMgr$1.getInstance().getItemById("CatToys", id);
                if(toyConfig.LvUpCost.length){
                    Laya.Dialog.close("ui/LevelToyInfoPop.scene");
                    Laya.Dialog.open("ui/LevelToyInfoPop.scene", false);
                    return;
                }
            }
            Laya.Dialog.close("ui/InfoPop.scene");
            Laya.Dialog.open("ui/InfoPop.scene", false);
        }

        showFood(foodId) {
            let food = NauticalSystem.getFoodById(foodId);
            this.count.visible = true;
            this.count.text = Strings.dealFishNumberText(food.count);

            this.setIcon(food.Icon);

            if (food.count == 0) {
                this.setGray(true);
                this.item1.visible = false;
            } else {
                this.item1.visible = true;
            }

            this.on(Laya.Event.CLICK, this, this.showInfo, [foodId, "food"]);
            this.checkShowWeakGuideAnim();
        }

    }

    class GoodsItem extends Laya.Sprite {

        constructor() {
            super();
        }

        setItem(id, type) {
            this.offAll();
            // console.debug("setItem foodId:", id, type);
            this.icon = this.getChildByName("icon");
            this.upIcon = this.getChildByName("upIcon");
            this.count = this.getChildByName("count");
            this.item1 = this.getChildByName("item1");
            this.fish = this.getChildByName("fish");
            this.title = this.getChildByName("title");
            this.lock=this.getChildByName("lock");
            this.redPoint=this.getChildByName("redPoint");
            this.lock.visible=false;
            this.redPoint.visible=false;
            this.upIcon.visible = false;
            this.count.visible = false;
            this.title.visible = false;
            this.fish.visible = false;
            this.setGray(false);
            if(type=="cook")
                this.showCook(id);
            else if(type=="food")
                this.showFood(id);
            else
                this.showGame(id);
        }

        showCook(foodListId) {
            let food = FoodSystem$1.getCurFoodInList(foodListId);

            this.setIcon(food.getShopItems());

            if(food.isLocked()){
                this.item1.visible=false;
                this.title.visible = true;
                this.fish.visible = true;
                this.title.text=Strings$1.dealFishNumberText(food.getCost(),1);
                this.fish.x = (this.width - this.fish.width - this.title.width - 5) / 2;
                this.title.x = this.fish.x + this.fish.width + 5;
                this.on(Laya.Event.CLICK,this,this.showLevelUp, [foodListId]);
                if (food.getLimit() > PlayerSystem.getStar()) {
                    this.lock.visible = true;
                    this.fish.visible = false;
                    this.title.visible = false;
                }
            }else {
                this.lock.visible=false;
                this.item1.visible = true;
                let list = FoodSystem$1.getFoodList(foodListId);
                if (list[food.getLevel()] && FoodSystem$1.checkLevelUpStarEnough(foodListId)) {
                    this.upIcon.visible = true;
                }
                this.on(Laya.Event.CLICK, this, this.showLevelUp, [foodListId]);
            }
            this.redPoint.visible = FoodSystem$1.checkFoodRedPoint(foodListId);
            if (foodListId == "Food1") {
                if (GuideSystem.isGuiding()) {
                    let guide = GuideSystem.getGuide("GuideGroup_3");
                    if (guide) {
                        guide.registerObj("btn2", this);
                        guide.registerClick("btn2", this);
                    }
                }
            }
            this.checkShowWeakGuideAnim(food.getId());
        }

        showFood(foodId) {
            let food = NauticalSystem$1.getFoodById(foodId);
            this.count.visible = true;
            this.count.text = Strings$1.dealFishNumberText(food.count,1);

            this.setIcon(food.Icon);

            if(food.count==0){
                this.setGray(true);
                this.item1.visible=false;
            }else {
                this.item1.visible=true;
            }

            this.on(Laya.Event.CLICK,this,this.showInfo, [foodId, "food"]);
            this.checkShowWeakGuideAnim();
        }

        showTool(toolId) {
            let tool = NauticalSystem$1.getToolById(toolId);

            this.setIcon(tool.Icon);

            if(tool.count==0){
                this.setGray(true);
                this.item1.visible=false;
            }else {
                this.item1.visible=true;
            }
            this.on(Laya.Event.CLICK,this,this.showInfo, [toolId, "tool"]);
            this.checkShowWeakGuideAnim();
        }
     

        setIcon(skin) {
            this.icon.skin = skin;
            let scalex = 150 / this.icon.width;
            let scaley = 150 / this.icon.height;
            if (scalex <= scaley) {
                this.icon.scaleX = scalex;
                this.icon.scaleY = scalex;
            } else {
                this.icon.scaleX = scaley;
                this.icon.scaleY = scaley;
            }
        }

        setGray(b) {
            if(b){
                let icon = this.getChildByName("icon");
                icon.gray = true;
            }else {
                let icon = this.getChildByName("icon");
                icon.gray = false;
            }
        }

        showLevelUp(foodListId) {
            GameContext.CurrentFoodListId = foodListId;
            Laya.Dialog.close("ui/foodLevelUpPop.scene");
            Laya.Dialog.open("ui/foodLevelUpPop.scene", false);
        }

        showInfo(id, type) {
            GameContext.CurrentInfoId = id;
            GameContext.CurrentInfoType = type;
            Laya.Dialog.close("ui/InfoPop.scene");
            Laya.Dialog.open("ui/InfoPop.scene", false);
        }

        checkShowWeakGuideAnim(foodId = null) {
            if (!GuideSystem.isGuiding() && foodId != null && TaskMainSystem.getIsTasking() && !TaskMainSystem.checkEnought()) {
                let task = TaskMainSystem.getCurrentData();
                let food = null;
                if (task.getGuide() == "Food_Unlock") {
                    food = FoodSystem$1.checkCanBuyCheapestFood();
                } else if (task.getGuide() == "Food_LvUp") {
                    food = FoodSystem$1.checkLevelUpCheapestFood();
                }
                if (food && food.getId() == foodId && (this.weakAnim == null || this.weakAnim.destroyed)) {
                    this.weakAnim = new Spine();
                    this.weakAnim.loadAnim("anim1/xinshouyindao.sk");
                    this.weakAnim.playAnim("zhuazi2", true);
                    this.weakAnim.x = this.width / 2;
                    this.weakAnim.y = this.height / 2;
                    this.weakAnim.zOrder = 2;
                    this.weakAnim.scaleX = 1 / this.scaleX;
                    this.weakAnim.scaleY = -1 / this.scaleY;
                    this.zOrder = 2;
                    this.addChild(this.weakAnim);
                    EventMgr.getInstance().registEvent("SHOW_WEAK_GUIDE_COOK_ITEM", this, this.setWeakGuideAnimVisible);
                    return;
                }
            }
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
            }
            this.weakAnim = null;
            this.zOrder = 1;
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("SHOW_WEAK_GUIDE_COOK_ITEM", this, this.setWeakGuideAnimVisible);
            if (this.weakAnim != null && !this.weakAnim.destroyed) {
                this.weakAnim.destroy();
            }
            this.weakAnim = null;
            this.zOrder = 1;
        }

        
        setWeakGuideAnimVisible(visible) {
            if (this.weakAnim) {
                this.weakAnim.visible = visible;
            }
        }

    }

    class ImageDetailUI extends Laya.Dialog {
        constructor() {
            super();

        }
        onEnable() {
            this.bg = this.getChildByName("BgBtn");
            this.bg.on(Laya.Event.CLICK, this, this.closeUI);
            this._index = 0;
            this.leftBtn = this.getChildByName("Left");
            this.leftBtn.on(Laya.Event.CLICK, this, this.OnLeftBtnClick);
            this.rightBtn = this.getChildByName("Right");
            this.rightBtn.on(Laya.Event.CLICK, this, this.OnRightBtnClick);
            this.buffBox = this.getChildByName("buffBox");
            this.initArrowState();
            this.list = this.getChildByName("list");
            this.list.array = this.parent.imagelibUI.list.array;
            this.array = this.list.array;
            function sortByOrder(a,b)
            {
                return ConfigMgr$1.getInstance().getItemById("CatPhotos", a._id).Order-ConfigMgr$1.getInstance().getItemById("CatPhotos", b._id).Order;
            }
        this.array.sort(sortByOrder);
            this.startIndex = 0;
            this.endIndex = this.array.length;
            this.list.scrollBar.rollRatio = 0;
            this.list.scrollBar.value = 0;
            this._index = 0;
            this.initPos = 0;
            this.list.off(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.list.on(Laya.Event.MOUSE_UP,this,this.changeListItems1);
            this.list.off(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.list.on(Laya.Event.MOUSE_OUT,this,this.changeListItems2);
            this.visible=false;
            Laya.timer.once(1,this,this.naviTo);
           
        }

        pushNullPhoto(pt)
        {
         let need= 8-pt.length%8;
         for(let i=0;i<need;i++)
         {
             let pd=new PhotoData(-1,-1,-1);
             this.list.array.push(pd);
         }
        }
        initArrowState()
        {
            this.leftBtn.visible=false;
            this.rightBtn.visible=false;
           
        }
        setNavibtnState(state)
        {
         this.leftBtn.visible=state;
         this.rightBtn.visible=state;
        }
        closeUI()
        {
               this.bg.visible=false;
               this.close();
        }
        onStart() {
            this.naviTo(this.parent.initPos);
        }
        OnLeftBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - 1 > 0 ? this._index - 1 : 0;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        OnRightBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + 1 < this.endIndex ? this._index + 1 : this.endIndex ;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }


        adjustList() {
            let w = this.list.getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w + s) + 0.5 | 0;
            Laya.Tween.to(this.list.scrollBar, { value: this._index * (w + s) }, 100);
        }

        loadList() {
            let showList = this.array.slice(this.startIndex, this.endIndex);
            this.adjustList();
            return showList;
        }

        changeListItems() {
            this.list.array = this.loadList();
            this.loadArrow();
        }
        loadArrow() {
            if (this._index == 0) {
                this.leftBtn.visible = false;
            } else {
                this.leftBtn.visible = true;
            }
            if (this._index >= this.endIndex - 1) {
                this.rightBtn.visible = false;
            } else {
                this.rightBtn.visible = true;
            }
        }
        initImageListData() {
            this.imageListData = PhotoSystem.getSystemPhotosData();
        }
        naviTo() {
            this.list.scrollTo(this.initPos);
            this.changeListItems();
    this.visible=true;
        }
        setInitPos(pos) {
            this.initPos = pos;
        }
        changeListItems1() {
            this.loadList();
            this.loadArrow();
        }
        changeListItems2(){
            this.list.scrollBar.stopScroll();
            this.changeListItems1();
        }
    }

    class ImageDetailItemList extends Laya.List
    {
        constructor()
        {
            super();
            this.array=[];
            this.renderHandler = Laya.Handler.create(this, this.updateItem, null, false);
            
            
        }
        onEnable()
        {
            
        }

        onSelect(index) {
        }

        updateItem(cell, index)
        {
            cell.setItem(cell.dataSource,index);
        }

    }

    class ImageDetailItem extends Laya.Box {
        constructor() {
            super();
        }

        onEnable() {
            this.iconImg = this.getChildByName("IconImg");
            this.bg = this.getChildByName("bg");
            this.catIcons = [];
        }

        setItem(data) {
            let config = ConfigMgr$1.getInstance().getItemById("CatPhotos", data._id);
            this.parent.parent.parent.buffBox.setType(data._id,0);
            if (data._id == -1) {
                this.bg.skin = "systemPhotoBg/sence2.jpg";
                this.bg.visible = false;
                this.visible = false;
            }
            else {
                this.visible = true;

                if (config == null) return;
                var imgs = config.Cat;
                var obj2 = JSON.parse(imgs);
                this.bg.visible = true;
                this.bg.skin = "systemPhotoBg/"+ config.Background+".jpg";
               
            }


            for (let i = 0; i < this.catIcons.length; i++) {
                this.catIcons[i].destroy();
            }
            if (typeof (obj2) != "undefined") {
                for (let i = 0; i < obj2.length; i++) {
                    this.loadSprite(this,"systemPhoto/"+ config.Catroute + "/" + obj2[i] + ".png");
                }
            }

        }

        loadSprite(parent, img) {
            var sp = new Laya.Image();
            sp.skin = img;
            parent.addChild(sp);
            sp.width = 625;
            sp.height = 500;
            sp.x = 14;
            sp.y = 28;
            sp.scaleX = 1.15;
            sp.scaleY = 1.15;
            this.catIcons.push(sp);
            return sp;
        }

        clearCatIcons() {
            this.catIcons.forEach(
                (icon, iconKey) => {
                    icon.skin = "";
                }
            );
        }
    }

    class ImageItem extends Laya.Box {
        constructor() {
            super();
        }

        onEnable() {
            this.image = this.getChildByName("CatIcon");
            this.bg = this.getChildByName("bg");
            this.btn = this.getChildByName("Btn");
            this.btn.on(Laya.Event.CLICK, this, this.showDetail);
            this.catIcons = [];
            this.index = 0;
            this.imgPop=this.parent.parent.parent.parent;
        }
        setItem(imageData, pothoIndex) {
            this.index = pothoIndex ? pothoIndex : 0;
            if (imageData._id == -1) {
                this.bg.skin = "systemPhotoBg/sence2.jpg";
                this.bg.visible = false;
                this.visible = false;
            }
            else {
                this.visible = true;
                let config = ConfigMgr$1.getInstance().getItemById("CatPhotos", imageData._id);
                if (config == null) return;
                var imgs = config.Cat;
                var obj2 = JSON.parse(imgs);
                this.clearCatIcons();

                for (let i = 0; i < this.catIcons.length; i++) {
                    this.catIcons[i].destroy();
                }
                for (let i = 0; i < obj2.length; i++) {
                    this.loadSprite(this,"systemPhoto/"+ config.Catroute + "/" + obj2[i] + ".png");
                }
                this.bg.visible = true;
                this.bg.skin = "systemPhotoBg/"+config.Background+".jpg";
            }

        }
        loadSprite(parent, img) {
            var sp = new Laya.Image();
            sp.skin = img;
            parent.addChild(sp);
            sp.x = this.image.x;
            sp.y = this.image.y;
            sp.width = this.image.width;
            sp.height = this.image.height;
            sp.scaleX = this.image.scaleX;
            sp.scaleY = this.image.scaleY;
            sp.anchorX = sp.anchorY = 0.5;
            this.catIcons.push(sp);
            return sp;
        }

        showDetail() {
            this.loadRes("ImageDetailUI", "prefab/ui/ImageDetailUI.prefab");
            // this.parent.parent.parent.parent.setInitPos(this.index);
            EventMgr.getInstance().postEvent("SetInitPos_Event", this.index);
            
        }

        loadRes(name, path) {
            Laya.loader.create(path, Laya.Handler.create(this, function (prefabDef) {
                if (name == "ImageDetailUI") {
                    this.detailUI = prefabDef.create();
                    this.imgPop.addChild(this.detailUI);
                    this.detailUI.zOrder = 2;
                    this.detailUI.x = 0;
                    this.detailUI.y = 0;
                    this.detailUI.setInitPos(this.index);

                }
            }));
        }
        clearCatIcons() {
            this.catIcons.forEach(
                (icon, iconKey) => {
                    icon.skin = "";
                }
            );
        }
    }

    class ImageLibUI extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this._index = 0;
            this._index2 = 0;
            this.list = this.getChildByName("list");
            this.scrollBar = this.list.getChildByName("scrollBar1");
            this.list._removePreScrollBar();
            this.list.scrollBar = this.scrollBar;
            this.scrollBar.name = "scrollBar";
            // this.scrollBar.rollRatio = 0;
            this.cateName = this.getChildByName("CatName");
            this.array = new Array();
            let data = PhotoSystem.getSystemPhotosData();
            this.array = data;
            this.realListLength = 0;
            function sortByOrder(a, b) {
                return ConfigMgr$1.getInstance().getItemById("CatPhotos", a._id).Order - ConfigMgr$1.getInstance().getItemById("CatPhotos", b._id).Order;
            }
            this.array.sort(sortByOrder);
            this.list.array = this.array;
            this.startIndex = 0;
            this.lastIndex = 0;
            this.endIndex = this.array.length;
            // this.list.scrollBar.value = 0;
            this.initPos = 0;
            // this.list.scrollBar.elasticBackTime = 200;//设置橡皮筋回弹时间。单位为毫秒。
            // this.list.scrollBar.elasticDistance = 50;//设置橡皮筋极限距离。
            this.pageAmount = Math.ceil(this.endIndex / (this.list.repeatX * this.list.repeatY));
            this.scrollBar.value = 0;
            // this.list.scrollBar.scaleX = 2.5;
            // this.list.scrollBar.scaleY = 0.53;
           // this.list.scrollBar.y = 305;
            this.CalculateAll();

            this.setCatName("[ALL]");
            this.imageDetailUI = null;
        }
        pushNullPhoto(pt) {
            let need = 8 - pt.length % 8;
            for (let i = 0; i < need; i++) {
                let pd = new PhotoData(-1, -1, -1);
                this.list.array.push(pd);
            }
            this.realListLength = this.list.array.length - need;
        }
        setNavibtnState(state) {
            this.leftBtn.visible = state;
            this.rightBtn.visible = state;
        }
        initNaviBtnState() {
            if (this.list.array.length == 0) {
                this.leftBtn.visible = false;
                this.rightBtn.visible = false;
            }
            else {
                this.leftBtn.visible = false;
                this.rightBtn.visible = true;
            }

        }
        changeListItems1() {
            this.loadList(2);
            this.loadArrow();
        }
        changeListItems2() {
            this.list.scrollBar.stopScroll();
            this.changeListItems1();
        }
        OnLeftBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - 8 > 0 ? this._index - 8 : 0;
            this.list.scrollTo(this._index);
            this.changeListItems(1);
        }

        OnRightBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + 8 < this.endIndex ? this._index + 8 : this.endIndex - 2;
            this.list.scrollTo(this._index);
            this.changeListItems(1);
        }

        adjustList(type) {
            let w = this.list.getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;

            if (type == 1) {
                this._index = 4 * this.list.scrollBar.value / (w + s) + 0.5 | 0;
                this._index = 8 * Math.ceil(this._index / 8);
                Laya.Tween.to(this.list.scrollBar, { value: 2 * this._index * (w + s) }, 100);
                let currentPage = Math.ceil(this._index / (this.list.repeatX * this.list.repeatY)) + 1;
            }
            else if (type == 2) {

                this._index2 = this.list.scrollBar.value / (w + s) + 0.5 | 0;
                if (this._index2 > this.lastIndex)
                    this._index2 += 1;
                else
                    this._index2 -= 1;
                if (this._index2 % 2 != 0)
                    this._index2 += 1;
                Laya.Tween.to(this.list.scrollBar, { value: this._index2 * (w + s) }, 100);
                this.lastIndex = this._index2;
                let currentPage = Math.ceil(4 * this._index2 / (this.list.repeatX * this.list.repeatY)) + 1;
                this._index = 4 * this._index2;
            }

        }

        loadList(type) {
            let showList = this.array.slice(this.startIndex, this.endIndex);
            this.adjustList(type);
            return showList;
        }

        changeListItems(type) {
            this.list.array = this.loadList(type);
        }
        loadArrow() {
            if (this._index == 0) {
                this.leftBtn.visible = false;
            } else {
                this.leftBtn.visible = true;
            }
            if (this._index >= this.endIndex - 8) {
                this.rightBtn.visible = false;
            } else {
                this.rightBtn.visible = true;
            }
        }
        CalculateAll() {
            let allPhotos = new Map();
            let config = ConfigMgr$1.getInstance().getConfig("NauticalPhotoReward");
            let photoRewards = new Map();
            config.forEach((listConfig, CatRequire) => {
                var re = listConfig.Reward;
                var re2 = JSON.parse(re);
                for (let i = 0; i < re2.length; i++) {
                    if (!photoRewards.has(re2[i]))
                        photoRewards.set(re2[i], re2[i]);
                }
            });
            let imgAmount = photoRewards.size;
            let ownCnt = this.list.array.length;
            let ratio = 100 * ownCnt / imgAmount;
            let newRatio = Math.round(ratio * Math.pow(10, 1)) / Math.pow(10, 1);
            let inte = Math.floor(newRatio);
            let fraction = Math.floor(10 * (newRatio - inte));
            this.parent.SetCollectRatio(inte, fraction);
        }

        setPageData(current, amount) {

        }
        setArrowState(current, amount) {
            if (current == 1)
                this.leftBtn.visible = false;
            if (current == amount)
                this.rightBtn.visible = false;
        }
        initPageData() {
            this.setPageData(1, this.pageAmount);
        }
        setCatName(name) {
            this.parent.setPetName2(name);
        }
    }

    class ImageTipsUI extends Laya.Dialog {
        constructor() {
            super();
        }
        onEnable() {
            this.nullPhotoType = 0;
           
            this.naviBtn = this.getChildByName("naviBtn");
            this.naviBtn.on(Laya.Event.CLICK, this, this.onNavi);
            this.des = this.getChildByName("DesLabel");
            this.des.text = Strings$1.get("tid#UI_055");

        }
        onCloseUI() {
            this.close();
        }
        oncloseAll() {
            if (this.parent != null) {
                this.parent.onCloseUI();
            }
        }
        onNavi() {

            if (this.nullPhotoType == 2)
           
                Laya.Dialog.open("ui/CollectPop.scene");
            else if (this.nullPhotoType == 1) {
               // Laya.Dialog.open("scene/NauticalScene.scene");
               SceneMgr.getInstance().switchScene(Constants.NauticalSceneId);
            }
            else
                console.log("type error");

            this.oncloseAll();
        }
        setNullPhotoType(type) {
            this.onEnable();
            this.nullPhotoType = type;
            if (type == 1) {
                this.des.text = Strings$1.get("tid#UI_055");
                this.naviBtn.label = "去出航";
                this.naviBtn.label = Strings$1.get("tid#Btn_021");
            }

            else if (type == 2) {
                this.des.text = Strings$1.get("tid#UI_054");
                this.naviBtn.label = "去领养";
                this.naviBtn.label = Strings$1.get("tid#Btn_022");
            }
            else
                this.des.text = "error";
        }
    }

    class LeftBtnDrawPop extends Laya.Script {
        constructor() {
            super();
        }

        onEnable() {
            this.owner.x = -428;
            this.clickBg = this.owner.getChildByName("clickBg");
            this.clickBg.visible = false;
            this.pullBtn = this.owner.getChildByName("pullBtn");
            this.pushBtn = this.owner.getChildByName("pushBtn");
            this.shareBtn = this.owner.getChildByName("shareBtn");
            this.settingBtn = this.owner.getChildByName("settingBtn");
            this.mailBtn = this.owner.getChildByName("mailBtn");
            this.collectBtn = this.owner.getChildByName("collectBtn");
            this.signBtn = this.owner.getChildByName("signBtn");
            this.signPoint = this.signBtn.getChildByName("redPoint");
            this.noticeBtn = this.owner.getChildByName("noticeBtn");
            this.clickBg.on(Laya.Event.CLICK, this, this.onPushBtnClick);
            this.pullBtn.on(Laya.Event.CLICK, this, this.onPullBtnClick);
            this.pushBtn.on(Laya.Event.CLICK, this, this.onPushBtnClick);
            this.shareBtn.on(Laya.Event.CLICK, this, this.onShareBtnClick);
            this.settingBtn.on(Laya.Event.CLICK, this, this.onSettingBtnClick);
            this.mailBtn.on(Laya.Event.CLICK, this, this.onMailBtnClick);
            this.collectBtn.on(Laya.Event.CLICK, this, this.onCollectBtnClick);
            this.signBtn.on(Laya.Event.CLICK, this, this.onSignBtnClick);
            this.noticeBtn.on(Laya.Event.CLICK, this, this.onNoticeBtnClick);
            this.pullRedPoint = this.pullBtn.getChildByName("pullRedPoint");


            let mailRed = this.mailBtn.getChildByName("redPoint");
            mailRed.setCheckFunc(MailSystem.hasUnRead);
            mailRed.refreshRedPoint();

            let signRed = this.signBtn.getChildByName("redPoint");
            signRed.setCheckFunc(SignSystem.showRedPoint);
            signRed.refreshRedPoint();

            let pullRed = this.pullBtn.getChildByName("redPoint");
            pullRed.setCheckFunc(function () {
                return MailSystem.hasUnRead() || SignSystem.showRedPoint();
            });
            pullRed.refreshRedPoint();
            this.btnArr = [this.settingBtn, this.mailBtn, this.collectBtn];
            this.shareBtn.visible = false;
            this.rePosBtn();
            EventMgr.getInstance().registEvent('ZHUAPAICLICK', this, this.onoff);
        }
        onoff(zhuapai) {
            if (zhuapai == 1) {
                this.owner.visible = true;
            } else {
                this.owner.visible=false;
            }

        }
        showSignbtn() {
            if (PlayerSystem.getStar() >= Const.get('SignSystemOpenCondition') && SignSystem.getSignDays() < 7) {
                if (this.btnArr.indexOf(this.signBtn) == null || this.btnArr.indexOf(this.signBtn) < 0) {
                    this.btnArr.push(this.signBtn);
                }
                this.signBtn.visible = true;
            } else {
                this.signBtn.visible = false;
            }
            this.rePosBtn();
        }

        onPullBtnClick() {//往右
            this.showSignbtn();
            let time = this.btnArr.length > 4 ? 300 : 150;
            let xx = this.btnArr.length > 4 ? 0 : -190;
            Laya.Tween.to(this.owner, { x: xx }, time, null, Laya.Handler.create(this, function () {
                this.pullBtn.visible = false;
                this.pushBtn.visible = true;
                this.clickBg.visible = true;
            }));
        }

        onPushBtnClick() {//往左
            let time = this.btnArr.length > 4 ? 300 : 150;
            Laya.Tween.to(this.owner, { x: -428 }, time, null, Laya.Handler.create(this, function () {
                this.pullBtn.visible = true;
                this.pushBtn.visible = false;
                this.clickBg.visible = false;
            }));
        }
        onShareBtnClick() {
            if (Laya.Browser.onMiniGame) {
                if (Laya.Browser.onMiniGame) {
                    SdkHelper.shareAppMessage(2);
                }
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#unopenShare");
            }
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onSettingBtnClick() {
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            Laya.Dialog.close("ui/Setting.scene");
            Laya.Dialog.open("ui/Setting.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }


        onMailBtnClick() {
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            Laya.Dialog.close("ui/MailListPop.scene");
            Laya.Dialog.open("ui/MailListPop.scene");
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }

        onCollectBtnClick() {
            Laya.Dialog.close("ui/ImageLibrary.scene");
            Laya.Dialog.open("ui/ImageLibrary.scene", false, null);
        }

        onSignBtnClick() {
            Laya.Dialog.close("ui/Sign.scene");
            Laya.Dialog.open("ui/Sign.scene", false);
        }

        onNoticeBtnClick() {

        }

        onDisable() {

        }

        rePosBtn() {
            let startX = this.btnArr.length > 4 ? 107 : 297;
            let startY = 895;
            for (let index = 0; index < this.btnArr.length; index++) {
                const btn = this.btnArr[index];
                btn.x = startX + Math.floor(index / 4) * 190;
                btn.y = startY + index % 4 * 181;
            }
        }
    }

    /**
     * 左侧下拉UI
     */
    class LeftUI extends Laya.Script {
        constructor() {
            super();
        }
       
        onEnable() {
            this.posPoint();
            this.coffee = this.owner.getChildByName("coffee");
            this.coffee.on(Laya.Event.CLICK, this, this.onCoffeeClick);
            this.home = this.owner.getChildByName("home");
            this.home.on(Laya.Event.CLICK, this, this.onHomeClick);
            this.luckydraw = this.owner.getChildByName("luckydraw");
            this.luckydraw.on(Laya.Event.CLICK, this, this.onLuckydrawClick);
            Constants.luckyButtonPoint = this.owner.localToGlobal(new Laya.Point(this.luckydraw.x, this.luckydraw.y));
            GameContext.luckyButtonPoint=Constants.luckyButtonPoint;
            this.luckyBtn = this.owner.getChildByName("luckyBtn");
            this.luckyBtn.on(Laya.Event.CLICK, this, this.openLuckyBagDialog);
            this.luckyAnim = new Spine();
            this.luckyAnim.loadAnim("anim1/fudaidonghua.sk");
            this.luckyAnim.x = this.luckyBtn.width / 2;
            this.luckyAnim.y = this.luckyBtn.height / 2 - 20;
            this.luckyBtn.addChild(this.luckyAnim);
            this.remainingTimeText = this.luckyBtn.getChildByName("remainingTime");
            this.homeRedP=this.home.getChildByName("homeRedP");
            this.homeRedP.visible=false;
            this.luckydrawRedP = this.luckydraw.getChildByName("luckydrawRedP");
            this.photoBtn = this.owner.getChildByName("photoBtn");
            this.photoBtn.on(Laya.Event.CLICK,this,this.onPhotoBtnClick);
            this.purchaseBtn = this.owner.getChildByName("purchase");
            this.purchaseBtn.on(Laya.Event.CLICK,this,this.onPurchaseBtnClick);
            this.purchaseRedPoint = this.purchaseBtn.getChildByName("purchaseRedPoint");
            this.orderBtn = this.owner.getChildByName("orderBtn");
            this.orderBtn.on(Laya.Event.CLICK,this,this.onOrderBtnClick);
            this.orderAnim = new Spine();
            this.orderAnim.loadAnim("anim3/waimaitubiao.sk");
            this.orderAnim.x = this.orderBtn.width / 2;
            this.orderAnim.y = this.orderBtn.height / 2 - 10;
            this.orderBtn.addChild(this.orderAnim);
            this.orderTimeLabel = this.orderBtn.getChildByName("remainingTime");
            this.orderClickSign = 1;
            this.orderBtn.visible = false;
            let orderRedPoint = this.orderBtn.getChildByName("redPoint");
            orderRedPoint.setCheckFunc(TakeoutSystem.checkOrderRedPoint);
            orderRedPoint.refreshRedPoint();
            this.showHome();
            EventMgr.getInstance().registEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().registEvent("SWITCH_SCENE_COMPLETE", this, this.btnChange);
            EventMgr.getInstance().registEvent("RefreshInviteUI_Event", this, this.ShowInviteUI);
            EventMgr.getInstance().registEvent("CHECK_LUCKYDRAWREDPOINT",this, this.checkLuckydrawRedP);
            EventMgr.getInstance().registEvent("CHECK_CLOSE_LUCKYBAG", this, this.checkCloseluckyBag);
            EventMgr.getInstance().registEvent("UPDATE_LEFTUI_BTNORDER", this, this.updateBtnOrder);
            EventMgr.getInstance().registEvent("GUIDEGROUP_18_START", this, this.guidegroup18Start);
            TaskMainUIMgr.getInstance().registButton("Click_Play", this.home);
            let a =Number(Const.get("HomeRedDot"));
            this.showRedPoint();
            Laya.timer.once(Number(Const.get("HomeRedDot") * 1000 + 6000), this, this.showRedPoint);
            // this.showBtn(Constants.CoffeeSceneId);
            Laya.loader.load("topui/redpoint1.png", Laya.Handler.create(this, function (res) {
                this.luckydrawRedP.texture = res;
            }));
            Laya.timer.once(500, this, function () {
                this.luckyBag();
            });
            this.updateBtnOrder();
        }

        changeOwnerVisible(visible){
            this.owner.visible = visible;
        }
        onPhotoBtnClick(){
            let starNum = PlayerSystem.getStar();
            let catAdoptStars = Const.get("CatAdoptStars");
            if (starNum <catAdoptStars) { 
            TipsPopMgr.getInstance().showTipsById("tid#starUnlockNumber", [catAdoptStars]);
            return;
            }
           let userPhotoData = PhotoSystem.getUserPhotosData().length;
            if(Number(userPhotoData)>=Number(Const.get('PhotoWallCaptureLimit'))){
                TipsPopMgr.getInstance().showTipsById("tid#Guide_Camera_5");
                return;
            }
            EventMgr.getInstance().postEvent('ZHUAPAICLICK',[0]);
            if(!Laya.LocalStorage.getJSON('nofirstPhoto')){
                Laya.Dialog.close("ui/PhotoTalkPop.scene");
                Laya.Dialog.open("ui/PhotoTalkPop.scene", false);
                this.changeOwnerVisible(0);
             //   let customData = { "nofirstPhoto": '1' };
               // EventMgr.getInstance().postEvent("SERVICE_PUT_CUSTOM_DATA", [customData]);
            }
           else {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
            if (GameContext.Sliding) {
                return;
            }
            Laya.Dialog.close("ui/CameraUI.scene");
            Laya.Dialog.open("ui/CameraUI.scene", true, null);
            this.changeOwnerVisible(0);
           }
            SoundManager.getInstance().play("Sound_clickSound", 1);
        }
        updateBtnOrder() {
            if (GuideSystem.isGuiding() && GuideSystem.getCurrentGuideId() == "GuideGroup_18") {
                return;
            }
            //幸运转盘、每日福利、邀请有礼、挂机礼包、内购
            this.luckydraw.visible = false;
            this.luckyBtn.visible = false;
            this.purchaseBtn.visible = false;
            this.orderBtn.visible = false;

            let sceneId = SceneMgr.getInstance().getCurrentSceneId();
            if(sceneId != Constants.HomelandId){
                this.nowPos = 0;
                this.photoBtn.pos(this.posArray[this.nowPos].x,this.posArray[this.nowPos].y);
                this.nowPos ++;

                let purchaseStarLimit = Number(Const.get("PurchaseItemShowStar"));
                if(purchaseStarLimit <= PlayerSystem.getStar()){
                    this.purchaseBtn.visible = true;
                    this.purchaseBtn.pos(this.posArray[this.nowPos].x,this.posArray[this.nowPos].y);
                    this.nowPos ++;
                    this.freeDiamondReceiveMills = PaySystem.getPayInfo().freeDiamondReceiveMills;
                    this.purchaseRedPoint.visible = PaySystem.checkDay(this.freeDiamondReceiveMills);
                }
                // if (LuckydrawSystem.getPrizeWheelUnlockLimits() <= PlayerSystem.getStar()) {
                //     this.luckydraw.visible = true;
                //     this.checkLuckydrawRedP();
                //     this.luckydraw.pos(this.posArray[this.nowPos].x,this.posArray[this.nowPos].y);
                //     this.nowPos ++;
                // }
                if(Const.get("DailyGiftUnlockLimit") <= PlayerSystem.getStar()) {
                    // this.nowPos ++;
                }
                // if(this.luckybageVisible){
                //     this.luckyBtn.visible = true;
                //     this.luckyBtn.pos(this.posArray[this.nowPos].x,this.posArray[this.nowPos].y);
                //     this.nowPos ++;
                // }
                if(Const.get("OrderSystemUnlockLimit") <= PlayerSystem.getStar()){
                    this.orderBtn.visible = true;
                    this.orderBtn.pos(this.posArray[this.nowPos].x,this.posArray[this.nowPos].y);
                    this.nowPos ++;
                    this.setOrderBtnView();
                }
            }
        }

        checkPurchaseVisible(){
            let purchaseStarLimit = Number(Const.get("PurchaseItemShowStar"));
            if(purchaseStarLimit <= PlayerSystem.getStar()){
                let payCardInfo = PaySystem.getPayInfo().payCardInfo;
                let now = TimeUtils.getTimeNowOnServer();
                let weekSign = 0;
                let monthSign = 0;
                if(!payCardInfo.weekCardEndMills || payCardInfo.weekCardEndMills < now){
                    weekSign = 1;
                }
                if(!payCardInfo.monthCardEndMills || payCardInfo.monthCardEndMills < now){
                    monthSign = 1;
                }
                if(weekSign ==1 && monthSign == 1){
                    return true;
                }
            }
            return false;
        }

        posPoint(){
            let startX = 104;
            let addX = 160;
            this.posArray = new Array();
            this.nowPos = 0;
            for(let i = 1; i < 5; i++){
                let p = new Laya.Point(startX + i * addX,277); 
                this.posArray.push(p);
            }
        }

        onOrderBtnClick(){
            if(this.orderClickSign){
                this.orderClickSign = 0;
                switch(TakeoutSystem.getCurrentType()){
                    case 0:
                        this.orderAnim.playAnim("6",false);
                    break;
                    case 1:
                        let s = TakeoutSystem.getDeliveryTimeLeft_S();
                        if(s == null){
                            this.orderAnim.playAnim("6",false);
                        }
                        else {
                            this.orderAnim.playAnim("4",false);
                        }
                    break;
                    case 2:
                        this.orderAnim.playAnim("2",false);
                    break;
                }
                EventMgr.getInstance().postEvent("SERVICE_FETCH_TAKEOUT_INFO", Laya.Handler.create(this, function () {
                    Laya.Dialog.open("ui/Order.scene", true, true);
                }));
                Laya.timer.once(500,this,function(){
                    this.orderClickSign = 1;
                });
            }
            
        }

        setOrderBtnView(){
            this.orderTimeLabel.visible = false;
            switch(TakeoutSystem.getCurrentType()){
                case 0:
                    this.orderAnim.playAnim("5",true);
                break;
                case 1:
                    let s = TakeoutSystem.getDeliveryTimeLeft_S();
                    if(s == null){
                        this.orderAnim.playAnim("5",true);
                    }
                    else {
                        this.orderTimeLabel.visible = true;
                        this.orderAnim.playAnim("3",true);
                        let m = s/60 | 0;
                        s -= m*60;
                        this.orderTimeLabel.text = m + ":" + s;
                        Laya.timer.once(1000,this,this.setOrderBtnView);
                    }
                break;
                case 2:
                    EventMgr.getInstance().postEvent("REDPOINT_REFRESH", ["order"]);
                    this.orderAnim.playAnim("1",true);
                break;
            }
        }

        openLuckyBagDialog() {
            this.playLuckyBagClick();
            Laya.Dialog.close("ui/LuckyBagDialog.scene");
            Laya.Dialog.open("ui/LuckyBagDialog.scene");
        }

        playLuckyBagClick(){
            if(this.remainingTimeText.text == ""){
                this.luckyAnim.playAnim("2",false);
            }
            else {
                this.luckyAnim.playAnim("4",false);
            }
        }

        luckyBag() {
            if (LuckyBagSystem.checkDaysReceiveType() && LuckyBagSystem.checkReceiveTimes()) {
                if (LuckyBagSystem.checkStarReceiveType()) {
                    Laya.timer.loop(1000, this, this.showLuckyBag);
                } else {
                    Laya.timer.once(1000, this, this.luckyBag);
                }
            }
        }

        checkCloseluckyBag() {
            if (LuckyBagSystem.checkDaysReceiveType() && LuckyBagSystem.checkReceiveTimes()) {
                this.luckyBag();
            } else {
                this.luckybageVisible = 0;
                this.updateBtnOrder();
                Laya.timer.clear(this, this.showLuckyBag);
                EventMgr.getInstance().postEvent("CLOSE_LUCKYBAG_DIALOG");
            }
        }

        showLuckyBag() {
            this.remainingTimeText.text = this.loadRemainingTime();
            this.luckybageVisible = 1;
            this.updateBtnOrder();
            if (this.remainingTimeText.text == "") {
                this.luckyAnim.playAnim("1");
            } else {
                this.luckyAnim.playAnim("3");
            }
        }

        loadRemainingTime() {
            let str = "";
            this.remainingTime = LuckyBagSystem.getRemainingTime();
            if (this.remainingTime > 0) {
                let minute = this.remainingTime / 60 | 0;
                let second = this.remainingTime % 60;
                if (minute < 10) {
                    str += "0";
                }
                str += minute + ":";
                if (second < 10) {
                    str += "0";
                }
                str += second;
            }
            return str;
        }

        onCoffeeClick() {
            EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [1]);
            EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [1]);
            SoundManager.getInstance().play("Sound_clickSound", 1);
            EventMgr.getInstance().postEvent("SWITCH_LAST_SCENE");
            this.homeRedP.visible=false;
            Laya.timer.clear(this, this.showRedPoint);
            Laya.timer.once(Number(Const.get("HomeRedDot")*1000+6000),this,this.showRedPoint);
            DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.HomeLand,0);
            IntimacyUpSystem.setChangeFlag(0);
        }
        onHomeClick() {
            let starNum = PlayerSystem.getStar();
            let catAdoptStars = Const.get("CatAdoptStars");
            if (starNum >= catAdoptStars) {
                DialogStatusMgr.getInstance().changeStatus(Constants.DialogStatus.HomeLand,1);  
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
                EventMgr.getInstance().postEvent("CATNEEWCOME_STATUS", [0]);
                SoundManager.getInstance().play("Sound_clickSound", 1);
                EventMgr.getInstance().postEvent("SWITCH_SCENE", Constants.HomelandId);
                IntimacyUpSystem.setChangeFlag(2);
            } else {
                TipsPopMgr.getInstance().showTipsById("tid#TipsCommon23", [catAdoptStars]);
            }
        }

        onLuckydrawClick(){
            Laya.Dialog.close("ui/LuckydrawPop.scene");
            Laya.Dialog.open("ui/LuckydrawPop.scene");
        }

        onPurchaseBtnClick(){
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene",true,[0]);
        }

        ShowInviteUI(){
            Laya.Dialog.close("ui/InviteUI.scene");
            Laya.Dialog.open("ui/InviteUI.scene");
        }
        showHome() {
            this.coffee.visible = false;
            this.home.visible = true;
        }
        showCoffee() {
            this.coffee.visible = true;
            this.home.visible = false;
        }

        checkLuckydrawRedP(){
            this.luckydrawRedP.visible = LuckydrawSystem.getRedPointFlag();
        }


        btnChange(sceneId) {
            if (sceneId == Constants.HomelandId)
                this.showCoffee();
            else
                this.showHome();
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("CHECK_GUIDEGROUP", this, this.checkGuide);
            EventMgr.getInstance().removeEvent("SWITCH_SCENE_COMPLETE", this, this.btnChange);
            EventMgr.getInstance().removeEvent("CHECK_LUCKYDRAWREDPOINT",this, this.checkLuckydrawRedP);
            EventMgr.getInstance().removeEvent("CHECK_CLOSE_LUCKYBAG", this, this.checkCloseluckyBag);
            EventMgr.getInstance().removeEvent("RefreshInviteUI_Event", this, this.ShowInviteUI);
            EventMgr.getInstance().removeEvent("UPDATE_LEFTUI_BTNORDER", this, this.updateBtnOrder);
            EventMgr.getInstance().removeEvent("GUIDEGROUP_18_START", this, this.guidegroup18Start);
        }

        checkGuide(eventString) {
            if (eventString == "GuideGroup_14") {
                let guide = GuideSystem.getGuide("GuideGroup_14");
                if (guide) {
                    guide.registerObj("btn1", this);
                    guide.registerClick("btn1", this.home);
                }
            }
        }
        showRedPoint() {
            let starNum = PlayerSystem.getStar();
            let catAdoptStars = Const.get("CatAdoptStars");
            let arr = HomePetMgr.getInstance().allPets();
            this.homeRedP.visible=false;
            if(starNum >= catAdoptStars && arr.length){
                let chuhang = true;
                for (const info of arr) {
                    if (!PetSystem.getPetInfo(info.petData.configId).inNautical) {
                        chuhang = false;
                    };
                }
                if(!chuhang){
                    this.homeRedP.visible=true;
                }
            }
        }
        
        showSignbtn(sceneIds) {
            let sceneId = SceneMgr.getInstance().getCurrentSceneId();
            if (sceneIds) {
                sceneId = sceneIds;
            }

            if (PlayerDataMgr.getInstance().getStar() >= Const.get('SignSystemOpenCondition') && SignSystem.getSignDays() < 7) {
                this.sign.visible = true;
                if (sceneId == Constants.CoffeeSceneId) {
                    this.sign.visible = true;
                } else if (sceneId == Constants.BeachSceneId) {
                    this.sign.visible = false;
                } else if (sceneId == Constants.NauticalSceneId) {
                    this.sign.visible = false;
                } else if (sceneId == Constants.HomelandId) {
                    this.sign.visible = false;
                }
            } else {
                this.sign.visible = false;
            }
        }

        updateStar(star) {
            let ringCnt = star;
            let constCfg = ConfigMgr$1.getInstance().getItemById("Const", "CollectionHandbookUnlock");
            let ringCntCfg = constCfg.Value;
            if (ringCnt >= ringCntCfg) {
                this.imageLibBtn.visible = true;
            }
            else {
                this.imageLibBtn.visible = false;
            }
           this.resetPos();
        }
     
        // showBtn(sceneId) {
        //     this.luckydraw.visible = false;
        //     if (sceneId == Constants.CoffeeSceneId) {
        //         if (LuckydrawSystem.getPrizeWheelUnlockLimits() <= PlayerSystem.getStar()) {
        //             this.luckydraw.visible = true;
        //             this.checkLuckydrawRedP();
        //         } else {
        //             EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.onStarChange);
        //         }
        //     } else {
        //         EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", this, this.onStarChange);
        //     }
        // }

        // onStarChange(star) {
        //     if (LuckydrawSystem.getPrizeWheelUnlockLimits() <= PlayerSystem.getStar()) {
        //         this.luckydraw.visible = true;
        //         this.checkLuckydrawRedP();
        //     }
        // }
        guidegroup18Start() {
            let guide = GuideSystem.getGuide("GuideGroup_18");
            if (guide) {
                guide.registerObj("btn0", this);
                guide.registerClick("btn0", this.orderBtn);
            }
        }
        
    }

    class PetsSelectUI extends Laya.Dialog {
        constructor() {
            super();
           
        }

        onEnable() {

            GameContext.lastSelectedPet = -1;
            let bg = this.getChildByName("Bg");
            GameContext.catItemArr = new Array();
            this.list = this.getChildByName("list");
            this.leftBtn = this.getChildByName("LeftBtn");
            this._index = 0;
            this.leftBtn.on(Laya.Event.CLICK, this, this.OnLeftBtnClick);
            this.pageCnt = 6;
            this.rightBtn = this.getChildByName("RightBtn");
            this.rightBtn.on(Laya.Event.CLICK, this, this.OnRightBtnClick);
            this.catName = this.getChildByName("CatName");
            let config = ConfigMgr$1.getInstance().getConfig("Role");
            this.array = new Array();
            config.forEach((listConfig, petId) => {
                this.array.push(petId);
            });
            function sortByOrder(a, b) {
                return config.get(a).Rank - config.get(b).Rank;
            }
            this.array.sort(sortByOrder);
            this.array.unshift(-1);
            this.list.off(Laya.Event.MOUSE_UP, this, this.changeListItems1);
            this.list.on(Laya.Event.MOUSE_UP, this, this.changeListItems1);
            this.list.off(Laya.Event.MOUSE_OUT, this, this.changeListItems2);
            this.list.on(Laya.Event.MOUSE_OUT, this, this.changeListItems2);
            this.list.array = this.array;
            this.startIndex = 0;
            this.endIndex = this.array.length;
            this.list.scrollBar.rollRatio = 0;
            this.list.scrollBar.value = 0;

            this.initPetName("[ALL]");
            this.initArrowState();
            this.loadArrow();
            EventMgr.getInstance().registEvent("SetLastSelectedPet_Event", this, this.setLastSelectedPet);
            EventMgr.getInstance().registEvent("SetAllHookState_Event",this, this.setAllHookState);
            this.catName.text = Strings$1.get("tid#UI_104");
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("SetLastSelectedPet_Event", this, this.setLastSelectedPet);
            EventMgr.getInstance().removeEvent("SetAllHookState_Event",this, this.setAllHookState);
            GameContext.lastSelectedPet=-1;
            GameContext.catItemArr=null;
        }
        setLastSelectedPet(petid) {
            GameContext.lastSelectedPet = petid;
        }
        initArrowState() {
            this.leftBtn.visible = false;
            this.rightBtn.visible = false;
        }
        changeListItems2() {
            this.list.scrollBar.stopScroll();
            this.changeListItems1();
        }

        changeListItems1() {
            this.loadList();
            this.loadArrow();
        }
        OnLeftBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index - this.pageCnt > 0 ? this._index - this.pageCnt : 0;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }

        OnRightBtnClick() {
            SoundManager.getInstance().play("Sound_clickSound", 1);
            this._index = this._index + this.pageCnt < this.endIndex ? this._index + this.pageCnt : this.endIndex - 5;
            this.list.scrollTo(this._index);
            this.changeListItems();
        }
        onAllBtnClick() {

        }
        adjustList() {
            let w = this.list.getChildAt(0).getChildAt(0).getChildAt(0).width;
            let s = this.list.spaceX;
            this._index = this.list.scrollBar.value / (w + s) + 0.5 | 0;
            Laya.Tween.to(this.list.scrollBar, { value: this._index * (w + s) }, 100);
        }

        loadList() {
            let showList = this.array.slice(this.startIndex, this.endIndex);
            this.adjustList();
            return showList;
        }

        changeListItems() {
            this.list.array = this.loadList();
            this.loadArrow();
        }
        loadArrow() {
            if (this._index == 0) {
                this.leftBtn.visible = false;
            } else {
                this.leftBtn.visible = true;
            }
            if (this._index >= this.endIndex - this.pageCnt) {
                this.rightBtn.visible = false;
            } else {
                this.rightBtn.visible = true;
            }
        }

        setAllHookState(state) {
            if (this.allHook != null)
                this.allHook.visible = state;

        }
        closeUI() {
            this.visible = false;
            this.close();
        }
        initPetName(name) {
            this.parent.setPetName2(name);
        }
    }

    class CatItem extends Laya.Box {

        constructor() {
            super();
        }
        onEnable() {
            this.icon = this.getChildByName("CatIcon");
            this.hook = this.getChildByName("hook");

            this.hookState = false;
        }
        setItem(petId) {

            this.showPet(petId);
        }

        showPet(petId) {
            this.petId = petId;

            if (petId == -1) {
                this.icon.skin = "ImageLib/ALL.png";
                this.icon.scaleX = 2.5;
                this.icon.scaleY = 2.5;
                this.petId = -1;
            }
            else {
                let config = ConfigMgr$1.getInstance().getItemById("Role", petId);
                if (config.HeadIcon)
                    this.setIcon(config.HeadIcon);


            }
            this.on(Laya.Event.CLICK, this, this.onSelected);
            GameContext.catItemArr.push(this);

            if (GameContext.lastSelectedPet == this.petId) {
                this.hook.visible = true;
            }
            else
                this.hook.visible = false;
        }

        setIcon(skin) {
            this.icon.skin ="cat_collect/head/" +skin+".png";
            this.icon.scaleX = 1;
            this.icon.scaleY = 1;
        }

        onSelected(index) {
            this.hookState = true;
            this.hook.visible = true;
            this.setUnSelected();
            if (this.petId == -1) {
                EventMgr.getInstance().postEvent("ShowAllPhotos_Event", null);
                EventMgr.getInstance().postEvent("SetLastSelectedPet_Event", -1);
            }
            else {
               // this.parent.parent.parent.setAllHookState(false);
               EventMgr.getInstance().postEvent("SetAllHookState_Event", false);
                this.showPetPhotos();
                //this.parent.parent.parent.lastSelectedPet = this.petId;
                EventMgr.getInstance().postEvent("SetLastSelectedPet_Event", this.petId);
            }

        }


        showPetPhotos() {
            // this.parent.parent.parent.parent.showPetPhotos(this.petId);
            EventMgr.getInstance().postEvent("ShowPetPhotos_Event", this.petId);
        }
        setUnSelected() {
            let catItemArr = GameContext.catItemArr;
            for (let i = 0; i < catItemArr.length; i++) {
                if (catItemArr[i] != this) {
                    catItemArr[i].hookState = false;
                    catItemArr[i].hook.visible = false;
                }
            }
        }

    }

    class PetsSelectUIList extends Laya.List
    {
        constructor()
        {
            super();
            //this.hScrollBarSkin = "";
            this.repeatY=1;
            this.array=[];
            this.renderHandler=Laya.Handler.create(this,this.updateItem,null,false);
          this.selectHandler=Laya.Handler.create(this,this.onSelect,null,false);
        }
        updateItem(cell,index)
        {
          // console.log("tkDr updateCell:"+index);
         cell.setItem(cell.dataSource);
         
        }
         onSelect(){
           
            cell.onSelected(index);
        }
    }

    // import BuildingSystem from "../controller/BuildingSystem";
    // import PlayerSystem from "../controller/PlayerSystem";
    // import GuideSystem from "../controller/GuideSystem";


    class ProjectItem extends Laya.Box {

        constructor() {
            super();
            // this._buildingItems = [];
            // this._buildingMap = new Map();
            // this.fangzhi=-1;//判断当前放置的是哪一个物品
        }

        onEnable(){
        }

        set itemHeight(height) {
            this.trueHeight = height;
            EventMgr.getInstance().postEvent("SHOP_LIST_CHANGE_ITEM_HEIGHT", [this.__index, height]);
        }

        get itemHeight() {
            return this.trueHeight;
        }

        static getItemHeight(location) {
            if (location.shopItemHeight != null) {
                return location.shopItemHeight;
            } else if (location[0] && location[0].shopItemHeight != null) {
                return location[0].shopItemHeight;
            }
        }

        setItem(itemlist,type,index) {
            this.projects =itemlist;
            this.__index = index;
            this.titleSprite=this.getChildByName('titleSprite');
            this.title=this.titleSprite.getChildByName('title');
            if(index==0){
                this.title.text='初级项目';
                this.title.text = Strings$1.get("tid#UI_128");
            }
            else if(index==1){
                this.title.text='中级项目';
                this.title.text = Strings$1.get("tid#UI_129");
            }
            else if(index==2){
                this.title.text='高级项目';
                this.title.text = Strings$1.get("tid#UI_130");
            }
                // for (let projectListId of itemlist) {
                //     let project = ProjectSystem.getCurProjectInList(projectListId);
                //     this.projects.push(project);
                // }

                //let title = this.getChildByName("title");
                //title.text = Strings.get(location.getListName());
                Laya.loader.load("prefab/ui/GoodsItem.json", Laya.Handler.create(this, this.onPrefabLoaded), null, Laya.Loader.JSON);
        }

        onPrefabLoaded(prefabJson) {
            let prefab = new Laya.Prefab();
            prefab.json = prefabJson;
            let x = 0;
            let y = 112;
            for (const index in this.projects) {
                let projectItem = Laya.Pool.getItemByCreateFun("GoodsItem", prefab.create, prefab);
                x = (index % 4) * 226;
                y = 112 + Math.floor(index / 4) * (224 + 44);
                projectItem.x = x;
                projectItem.y = y;
                this.addChild(projectItem);
                projectItem.icon=projectItem.getChildByName("icon");
                projectItem.upIcon = projectItem.getChildByName("upIcon");
                projectItem.count = projectItem.getChildByName("count");
                projectItem.item1 = projectItem.getChildByName("item1");
                projectItem.fish = projectItem.getChildByName("fish");
                projectItem.title = projectItem.getChildByName("title");
                projectItem.lock=projectItem.getChildByName("lock");
                projectItem.redPoint=projectItem.getChildByName("redPoint");
                projectItem.lock.visible=false;
                projectItem.redPoint.visible=false;
                projectItem.upIcon.visible = false;
                projectItem.count.visible = false;
                projectItem.title.visible = false;
                projectItem.fish.visible = false;


                this.showGame(projectItem,this.projects[index]);
                // this._buildingItems.push(buildingItem);
                // this._buildingMap.set(this.buildings[index], buildingItem);
                // buildingItem.setBuildingId(this.buildings[index], this.location);
            }
        }
        clear() {
        }
        setIcon(projectItem,skin) {
            projectItem.icon.skin = skin;
            let scalex = 150 / projectItem.icon.width;
            let scaley = 150 / projectItem.icon.height;
            if (scalex <= scaley) {
                projectItem.icon.scaleX = scalex;
                projectItem.icon.scaleY = scalex;
            } else {
                projectItem.icon.scaleX = scaley;
                projectItem.icon.scaleY = scaley;
            }
        }
       //娱乐设施
       showGame(projectItem,projectListId) {

        let project = ProjectSystem.getCurProjectInList(projectListId);

        this.setIcon(projectItem,project.getShopItems());

        if(project.isLocked()){
            projectItem.item1.visible=false;
            projectItem.title.visible = true;
            projectItem.fish.visible = true;
            projectItem.fish.scaleX= projectItem.fish.scaleY=1.5;
            let fishx=projectItem.fish.x;
            let fishy=projectItem.fish.y;
            projectItem.fish.x=fishx-projectItem.fish.width-100;
            projectItem.fish.y=fishy-0.25*projectItem.fish.height;

            projectItem.fish.texture='topui/beike.png';
            projectItem.title.text=Strings$1.dealFishNumberText(project.getCost());
            projectItem.fish.x = (projectItem.width - projectItem.fish.width - projectItem.title.width - 5) / 2-5;
            projectItem.title.x = projectItem.fish.x + projectItem.fish.width + 15;
            projectItem.on(Laya.Event.CLICK,this,this.showLevelUp, [projectListId]);
             if (project.getStarLimit() > PlayerSystem.getStar()||!BuildingSystem.hasBuildingDataByBid(project.getBuildingLimit())) {
                projectItem.lock.visible = true;
                projectItem.fish.visible = false;
                projectItem.title.visible = false;
             }
         }
         else {
             projectItem.lock.visible=false;
             projectItem.item1.visible = true;
             let list = ProjectSystem.getProjectList(projectListId);
            if (list[project.getLevel()] && ProjectSystem.checkLevelUpStarEnough(projectListId)) {
                projectItem.upIcon.visible = true;
            }
            projectItem.on(Laya.Event.CLICK, this, this.showLevelUp,[projectListId]);
        }
        projectItem.redPoint.visible = ProjectSystem.checkProjectRedPoint(projectListId);
        // if (foodListId == "Food1") {
        //     if (GuideSystem.isGuiding()) {
        //         if(GuideSystem.getConditionGroupComplete("GuideGroup_3")){
        //             let guide = GuideSystem.getGuide("GuideGroup_3");
        //             if (guide) {
        //                 guide.registerObj("btn2", this)
        //                 guide.registerClick("btn2", this);
        //             }
        //         }
        // }
        // this.checkShowWeakGuideAnim();
    }
    showLevelUp(projectListId){
        GameContext.CurrentProjectListId = projectListId;
        Laya.Dialog.close("ui/projectLevelUpPop.scene");
        Laya.Dialog.open("ui/projectLevelUpPop.scene", false);
    }
    }

    /**
    * 右侧
    */
    class RightUI extends Laya.Box {
        constructor() {
            super();
        }

        onAwake() {
            this.tipBox = this.getChildByName("tipBox");
            let closeBtn = this.tipBox.getChildByName("closeBtn");
            closeBtn.on(Laya.Event.CLICK, this, this.closeCollectTip);
            this.tip = this.tipBox.getChildByName("tip");
            this.tipBox.visible = false;
            //this.tip.on(Laya.Event.CLICK, this, this.openTutorial);
            if (Laya.Browser.onMiniGame) {
                let menuButtonPos = wx.getMenuButtonBoundingClientRect();
                let wxWidth = wx.getSystemInfoSync().windowWidth;
                let wxHeight = wx.getSystemInfoSync().windowHeight;
                let btnLeft = menuButtonPos.left;
                let btnTop = menuButtonPos.top;
                let btnWidth = menuButtonPos.width;
                let btnHeight = menuButtonPos.height;

                let point = new Laya.Point(btnLeft * Laya.stage.width / wxWidth, btnTop * Laya.stage.height / wxHeight);
                let rightDownPoint = new Laya.Point((btnLeft + btnWidth) * Laya.stage.width / wxWidth, (btnTop + btnHeight) * Laya.stage.height / wxHeight);
                point = this.globalToLocal(point);
                rightDownPoint = this.globalToLocal(rightDownPoint);
                this.tipBox.x = point.x + (rightDownPoint.x - point.x) / 4;
                this.tipBox.y = rightDownPoint.y;
            }
            this.startCheckShowCollect();
        }

        onEnable() {
           // EventMgr.getInstance().registEvent("RIGHT_UI_SHOW_COLLECTION_TIP", this, this.chooseTip);
            // EventMgr.getInstance().registEvent("RIGHT_UI_HIDE_COLLECTION_TIP", this, this.closeCollectTip);
        }

        onDisable() {
           // EventMgr.getInstance().removeEvent("RIGHT_UI_SHOW_COLLECTION_TIP", this, this.chooseTip);
            // EventMgr.getInstance().removeEvent("RIGHT_UI_HIDE_COLLECTION_TIP", this, this.closeCollectTip);
        }

        chooseTip(tipId) {
            if (tipId != null) {
                this._tipType = tipId;
                this.tip.skin = "tutorial/tips" + tipId + ".png";
                this.tipBox.visible = true;
            }
        }

        closeCollectTip() {
            this.tipBox.visible = false;
        }

        startCheckShowCollect() {
            if (Laya.LocalStorage.getJSON("collectionTime")) {
                return;
            }
            EventMgr.getInstance().registEvent("STAR_REALLY_CHANGE", this, this.starShowCollection);
            this.starShowCollection(PlayerSystem.getStar());
        }

        starShowCollection(star) {
            let needStar = Const.get("TaskCollectAppTipsCondition");
            if (star >= needStar) {
               // this.chooseTip(2);
                Laya.LocalStorage.setJSON("collectionTime", 1);
                EventMgr.getInstance().removeEvent("STAR_REALLY_CHANGE", this, this.starShowCollection);
            }
        }

        openTutorial() {
            if (this._tipType != null) {
               // Laya.Dialog.close("ui/CollectionTutorialPop.scene");
                if (this._tipType == 4) {
                   // Laya.Dialog.open("ui/CollectionTutorialPop.scene", true, [2, "fromTip"]);
                } else {
                   // Laya.Dialog.open("ui/CollectionTutorialPop.scene", true, [this._tipType, "fromTip"]);
                }
            }
        }

    }

    class SailBtn extends Laya.Script {

        constructor() {
            super();
            this.imgPath = ["wharf/ui/addbuild.png","wharf/ui/route1.png","wharf/ui/route2.png","wharf/ui/route3.png"];
            this.sailingArray = new Array();
        }

        onEnable(){
            this.sailBtn1 = this.owner.getChildByName("sailBtn1");
            this.sailBtn2 = this.owner.getChildByName("sailBtn2");
            this.sailBtn3 = this.owner.getChildByName("sailBtn3");
            this.sailBtn1.on(Laya.Event.CLICK,this,this.onBtnClick,[0]);
            this.sailBtn2.on(Laya.Event.CLICK,this,this.onBtnClick,[1]);
            this.sailBtn3.on(Laya.Event.CLICK,this,this.onBtnClick,[2]);
            let animBox1 = this.sailBtn1.getChildByName("animBox");
            let animBox2 = this.sailBtn2.getChildByName("animBox");
            let animBox3 = this.sailBtn3.getChildByName("animBox");
            this.anim1 = new Laya.Skeleton();
            this.anim2 = new Laya.Skeleton();
            this.anim3 = new Laya.Skeleton();
            animBox1.addChild(this.anim1);
            animBox2.addChild(this.anim2);
            animBox3.addChild(this.anim3);
            this.anim1.load("anim3/matouui.sk",Laya.Handler.create(this, function(){
                this.anim1.pos(animBox1.width/2,animBox1.height/2);
                this.anim1.play("guilai",true);
            }));
            this.anim2.load("anim3/matouui.sk",Laya.Handler.create(this, function(){
                this.anim2.pos(animBox2.width/2,animBox2.height/2);
                this.anim2.play("guilai",true);
            }));
            this.anim3.load("anim3/matouui.sk",Laya.Handler.create(this, function(){
                this.anim3.pos(animBox3.width/2,animBox3.height/2);
                this.anim3.play("guilai",true);
            }));
            this.updateSailBtnView();
            EventMgr.getInstance().registEvent("UPDATE_SAILBTN", this, this.updateSailBtnView);
        }

        updateSailBtnView(){
            this.sailBtnList = NauticalSystem$1.btnList;
            this.sailBtn1.visible = false;
            this.sailBtn2.visible = false;
            this.sailBtn3.visible = false;
            switch(this.sailBtnList.length){
                case 3:this.setBtn(this.sailBtn3,2);
                case 2:this.setBtn(this.sailBtn2,1);
                case 1:this.setBtn(this.sailBtn1,0);
            }
        }

        setBtn(object,index){
            let bg = object.getChildByName("bg");
            let sp = object.getChildByName("sp");
            let sail = object.getChildByName("sail");
            let timebg = object.getChildByName("timebg");
            let time = object.getChildByName("time");
            let animBox = object.getChildByName("animBox");
            animBox.visible = false;
            bg.visible = false;
            sp.visible = false;
            timebg.visible = false;
            time.visible = false;
            Laya.Tween.clearAll(sail);
            if(this.sailBtnList[index].nauticalInfo.adLock > 0){    //未解锁
                sail.skin = this.imgPath[0];
            }
            else if(this.sailBtnList[index].nauticalInfo.adLock == 0){  //解锁
                let boatIndex = this.sailBtnList[index].nauticalIndex;
                sail.skin = this.imgPath[boatIndex + 1];
                if(this.sailBtnList[index].nauticalInfo.inTrip){    //已出航 考虑是否到时间两种
                    if(this.sailBtnList[index].nauticalInfo.endSec > 0 
                        && TimeUtils.getTimeNowOnServer() >= this.sailBtnList[index].nauticalInfo.endSec * 1000){   //宝箱
                        animBox.visible = true;
                    }
                    else {   //出航中
                        bg.visible = true;
                        sp.visible = true;
                        timebg.visible = true;
                        time.visible = true;
                        this.showRemainingTime(object,index);
                    }
                }
                else {//未出航
                    this.playUnIntrip(sail);
                }
            }
            object.visible = true;
        }

        playUnIntrip(obj){
            let frameTime = 33;
            Laya.Tween.to(obj,{scaleX:2.2,scaleY:2.2},10 * frameTime,null,Laya.Handler.create(this,function(){
                Laya.Tween.to(obj,{scaleX:2,scaleY:2},30 * frameTime,null,Laya.Handler.create(this,function(){
                    this.playUnIntrip(obj);
                }));
            }));
        }

        showRemainingTime(object,index){
            let flag = 0;
            this.sailingArray.forEach(item=>{
                if(item.index==index){
                    flag = 1;
                }
            });
            if(flag == 0){
                let obj = {
                    "index": index,
                    "object": object
                };
                this.sailingArray.push(obj);
            }
            if(this.sailingArray.length <= 1){
                Laya.timer.loop(200,this,this.showAllTime);
            }
        }

        showAllTime(){
            for(let i = 0; i < this.sailingArray.length; i ++){
                this.showTime(this.sailingArray[i].index,this.sailingArray[i].object);
            }
        }

        showTime(index,object){
            let nauticalInfo = this.sailBtnList[index].nauticalInfo;
            let endTime = nauticalInfo.endSec;
            let time = endTime - TimeUtils.getTimeNowOnServer()/1000;
            let str = "";
            let startTime = nauticalInfo.startSec;
            let remainingTime = time.toFixed(0);
            let sp = object.getChildByName("sp");
            let timeText = object.getChildByName("time");
            if(remainingTime > 0 ){
                let minute = remainingTime / 60 | 0;
                let second = remainingTime % 60;
                if(minute<10){
                    str += "0";
                }
                str += minute + ":";
                if(second<10){
                    str += "0";
                }
                str += second;
                let reTime = nauticalInfo.originEndSec - nauticalInfo.endSec;
                let rate = 1 - remainingTime / (endTime - startTime + reTime);
                timeText.text = str;
                sp.graphics.clear();
                sp.graphics.drawPie(80, 80.5, 77, 360 * rate -90 , 270, "#acff6a");
            }
            else {
                if(remainingTime <= 0 && remainingTime > -3){
                    for(let i = 0; i < this.sailingArray.length; i++){
                        if(this.sailingArray[i].index == index){
                            this.sailingArray.splice(i,1);
                        }
                    }
                    if(this.sailingArray.length == 0){
                        Laya.timer.clear(this,this.showTime);
                    }
                    switch(index){
                        case 2:this.setBtn(this.sailBtn3,2);break;
                        case 1:this.setBtn(this.sailBtn2,1);break;
                        case 0:this.setBtn(this.sailBtn1,0);break;
                    }
                }
               
            }
        }

        onBtnClick(index){
            if(GameContext.Sliding)
                    return;
            let tripIndex = this.sailBtnList[index].nauticalIndex;
            if(this.sailBtnList[index].nauticalInfo.adLock > 0){
                Laya.Dialog.close("ui/SailDialog.scene");
                Laya.Dialog.open("ui/SailDialog.scene",true,tripIndex);
            }
            else if(this.sailBtnList[index].nauticalInfo.inTrip ){ //分是否可领取两种
                if(this.sailBtnList[index].nauticalInfo.endSec > 0 
                    && TimeUtils.getTimeNowOnServer() >= this.sailBtnList[index].nauticalInfo.endSec * 1000){   //宝箱
                        NauticalSystem$1.previewNauticalTripReq(tripIndex,Laya.Handler.create(this, this._showReward));
                }
                else {   //出航中
                    Laya.Dialog.close("ui/NauticalTravelingPop.scene");
                    Laya.Dialog.open("ui/NauticalTravelingPop.scene",true,["",tripIndex]);
                }
            }
            else {
                Laya.Dialog.close("ui/NauticalStartPop.scene");
                Laya.Dialog.open("ui/NauticalStartPop.scene",true,tripIndex);
            }
        }

        _showReward(res,tripIndex) {
            Laya.Dialog.close("ui/NauticalRewardPop.scene");
            Laya.Dialog.open("ui/NauticalRewardPop.scene", false, [res,tripIndex]);
        }

        onDisable(){
            this.anim1.destroy();
            this.anim2.destroy();
            this.anim3.destroy();
            EventMgr.getInstance().removeEvent("UPDATE_SAILBTN", this, this.updateSailBtnView);
        }
    }

    class TaskMainBox extends Laya.Box {

        constructor() {
            super();
        }

        onAwake() {
            this.bg = this.getChildByName("bg");
            this.mainBox = this.getChildByName("mainBox");
            this.desc = this.mainBox.getChildByName("desc");
            this.progressBar = this.mainBox.getChildByName("progressBar");
            this.progressText = this.mainBox.getChildByName("progressText");
            this.rewardImage = this.mainBox.getChildByName("rewardImage");
            this.rewardNum = this.mainBox.getChildByName("rewardNum");
            this.getBtn = this.mainBox.getChildByName("getBtn");
            this.getBtn.on(Laya.Event.CLICK, this, this.getReward);
            this.goBtn = this.mainBox.getChildByName("goBtn");
            this.goBtn.on(Laya.Event.CLICK, this, this.goTask);
            this.animBox = this.mainBox.getChildByName("animBox");
            this.anim = new Spine();
            this.animBox.addChild(this.anim);
            this.anim.y = this.animBox.height / 2;
            this.anim.loadAnim("anim1/xinshouyindao.sk");
            this.animBox.on(Laya.Event.CLICK, this, this.onAnimClick);
            /**1:收回,2:显示 */
            this._type = 1;
            this.bg.on(Laya.Event.CLICK, this, this.hide);
            this.goBtn.label = Strings$1.get("tid#Btn_011");
            this.getBtn.label = Strings$1.get("tid#Btn_017");
        }

        onEnable() {
            this.getBtn.mouseEnabled = true;
            this.bg.visible = false;
            this._type = 1;
            this.mainBox.x = -730;
            this.anim.playAnim("maomao3", true);
            this.updatView();
            this.show();
        }

        hide() {
            Laya.Tween.clearAll(this.mainBox);
            Laya.Tween.to(this.mainBox, { x: -730 }, 350);
            this._type = 1;
            this.updatView();
            this.bg.visible = false;
        }

        onAnimClick() {
            if (this._type == 1) {
                this.show();
            } else {
                this.hide();
            }
        }

        show() {
            if(TaskMainSystem.getCurrentData().getGuide()=="Click_Adopt"){
                PetSystem.getPetsInfo();
            }
            Laya.Tween.clearAll(this);
            Laya.Tween.to(this.mainBox, { x: 0 }, 350);
            this._type = 2;
            this.anim.playAnim("maomao3", true);
            this.bg.visible = true;
            if (this.warnAnim) {
                this.warnAnim.x = 572;
                this.warnAnim.y = 156;
            }
        }

        goTask() {
            if (this.currentData.config.Jump == "ToBuild") {
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
                SyncSystem.updateSceneInfo();
                SoundManager.getInstance().play("Sound_clickSound", 1);
                Laya.Dialog.close("ui/shopPop.scene");
                Laya.Dialog.open("ui/shopPop.scene");
            } else if (this.currentData.config.Jump == "ToFood") {
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
                SyncSystem.updateSceneInfo();
                SoundManager.getInstance().play("Sound_clickSound", 1);
                Laya.Dialog.close("ui/shopPop.scene");
                Laya.Dialog.open("ui/shopPop.scene", true, 4);
            } else if (this.currentData.config.Jump == "ToFishing") {
                if (GameContext.Sliding)
                    return;
                let afterHandler = Laya.Handler.create(this, function (res) {
                    SceneMgr.getInstance().switchScene(Constants.FishingSceneId);
                });
                FishingSystem.updateServerBaseInfo(afterHandler);
            } else if (this.currentData.config.Jump == "ToCatFood") {
                if (GameContext.Sliding)
                    return;
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                let total = Number(Const.get("CatFoodKeepTime"));
                let adTime = Number(Const.get("CatFoodKeepTimeAD"));
                let videoStr = Strings$1.get("tid#UI_027", [String(Math.floor((total + adTime) / 3600))]);
                let shareStr = Strings$1.get("tid#UI_028", [String(Math.floor((total + adTime) / 3600))]);
                Laya.Dialog.close("ui/BowlAdPop.scene");
                Laya.Dialog.open("ui/BowlAdPop.scene", false, [Constants.ShareId.CatFood, "ADD_CAT_FOOD_AD", videoStr, shareStr]);
            } else if (this.currentData.config.Jump == "ToAdopt") {
                EventMgr.getInstance().postEvent("BALLOON_SCENE_MGR", [0]);
                SyncSystem.updateSceneInfo();
                PetSystem.getPetsInfo(Laya.Handler.create(this, function () {
                    Laya.Dialog.close("ui/CollectPop.scene");
                    Laya.Dialog.open("ui/CollectPop.scene");
                }));
                SoundManager.getInstance().play("Sound_clickSound", 1);
            } else if (this.currentData.config.Jump == "ToSail") {
                if (GameContext.Sliding)
                    return;
                SceneMgr.getInstance().switchScene(Constants.NauticalSceneId);
                let hasBoat = BuildingSystem.hasBuildingDataByBid("3010") || BuildingSystem.hasBuildingDataByBid("3011") || BuildingSystem.hasBuildingDataByBid("3012");
                let index = NauticalSystem$1.getBoatCanSail();
                if (hasBoat) {
                    Laya.timer.once(50, this, function () {
                        if (index == -2) {
                            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon29");
                        }
                        else if (index == -3) {
                            TipsPopMgr.getInstance().showTipsById("tid#TipsCommon71");
                        }
                        else {
                            Laya.Dialog.close("ui/NauticalStartPop.scene");
                            Laya.Dialog.open("ui/NauticalStartPop.scene", false, index);
                        }
                    });
                }
                else {
                    TipsPopMgr.getInstance().showTipsById("tid#TipsCommon28");
                }
            }
            else if (this.currentData.config.Jump == "ToGift") {
                if (GameContext.Sliding)
                    return;
                SceneMgr.getInstance().switchScene(Constants.BeachSceneId);
                Laya.Dialog.close("ui/VisitorGiftPop.scene");
                Laya.Dialog.open("ui/VisitorGiftPop.scene", true, [2]);
            } else if (this.currentData.config.Jump == "ToHome") {
                if (GameContext.Sliding)
                    return;
                let starNum = PlayerSystem.getStar();
                let catAdoptStars = Const.get("CatAdoptStars");
                if (starNum >= catAdoptStars) {
                    SceneMgr.getInstance().switchScene(Constants.HomelandId);
                }
            }
        }

        getReward() {
            this.getBtn.mouseEnabled = false;
            SyncSystem.updateSceneInfo();
            let reward = JSON.parse(this.currentData.getReward())[0];
            let localPoint = new Laya.Point(this.rewardImage.x, this.rewardImage.y);
            localPoint = this.rewardImage.parent.localToGlobal(localPoint);
            let str = "main_task" + this.currentData.getId().substring(9) + "_4";
            if (str == 'main_task2_4') {
                let str = "ftue_60_taskClaim2";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
            } else if (str == 'main_task4_4') {
                let str = "ftue_65_iceCreamResearch";
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
            }
            let getstr = this.currentData.getId();
            TaskMainSystem.getReward(Laya.Handler.create(null, function () {
                ItemFlyMgr.getInstance().fly(reward.id, Number(reward.amount), Constants.FlyType.NORM, localPoint.x, localPoint.y);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str]);
                console.log('完成任务++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++', getstr);
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [getstr, null, 1]);
            }));
            EventMgr.getInstance().postEvent("CLEAR_MAIN_TASK_UI");
        }

        onDisable() {
            this.anim.stop();
            this._data = null;
        }

        clear() {
            this.bg.visible = false;
            this.anim.stop();
            if (this.warnAnim) {
                this.warnAnim.stop();
            }
            this._data = null;
            Laya.Tween.clearAll(this);
            this.removeSelf();
            this.destroy();
        }

        updatView() {
            this.currentData = TaskMainSystem.getCurrentData();
            if (this.currentData == null) {
                return;
            }
            this.desc.text = Strings$1.get(this.currentData.getDesc(), [this.currentData.getTarget()]);
            let target = this.currentData.getTarget();
            let current = Math.min(this.currentData.getCurrent(), target);
            this.progressBar.value = current / target;
            this.progressText.text = current + "/" + target;
            this.getBtn.visible = (current == target);
            this.goBtn.visible = (current != target) && (this.currentData.config.Jump);
            if (current == target && this._type == 1) {
                this.anim.playAnim("maomao4", true);
            } else {
                this.anim.playAnim("maomao3", true);
            }
            if (this.currentData.config.Guide && current == target && this.currentData.getGuide() != null) {
                if (this.warnAnim == null) {
                    this.warnAnim = new Spine();
                    this.warnAnim.loadAnim("anim1/xinshouyindao.sk");
                    this.mainBox.addChild(this.warnAnim);
                    this.warnAnim.playAnim("zhuazi2", true);
                }
                if (this._type == 1) {
                    this.warnAnim.x = 795;
                    this.warnAnim.y = 100;
                } else {
                    this.warnAnim.x = 572;
                    this.warnAnim.y = 156;
                }
            }
            let reward = JSON.parse(this.currentData.getReward())[0];
            if (reward.id == Constants.RewardType.FISH) {
                this.rewardImage.skin = "topui/yugan.png";
            } else {
                this.rewardImage.skin = "topui/xingxing.png";
            }
            this.rewardNum.text = String(reward.amount);
            this.visible = true;
        }
    }

    /**
    * 通用提示面板
    */
    class TipsPop extends Laya.Script {
        constructor() {
            super();
            this.maxWidth = 500;
            //宽高比例
            this.wh = 2;
            EventMgr.getInstance().registEvent("tipShow", this, this.showTips);
        }

        onEnable() {

        }

        onAwake() {
            this.bg = this.owner.getChildByName("bg");
            this.bg.on(Laya.Event.CLICK, this, this.onCloseClick);
            this.tipBox = this.owner.getChildByName("tipBox");
            this.tipSprite = this.tipBox.getChildByName("tipSprite");
        }

        onStart() {

        }

        onDisable() {

        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("tipShow", this, this.showTips);
        }

        onCloseClick() {
            Laya.timer.clear(this, this.onCloseClick);
            this.owner.removeSelf();
            Laya.Tween.clearAll(this.tipBox);
            this.tipBox.scaleX = 2.5;
            this.tipBox.scaleY = 2.5;
            this.tipBox.alpha = 1;
            this.tipSprite.removeChildren();
            if (this._hideHandler) {
                this._hideHandler.run();
            }
            this._hideHandler = null;
            TipsPopMgr.getInstance().showComplete();
        }

        showTips(obj) {
            let str = obj.tip;
            let hideHandler = obj.hideHandler;
            let time = obj.time;
            let hasBg = obj.hasBg;
            let up = obj.up;
            let offsetY = obj.offsetY;
            this.bg.alpha = hasBg ? 0.2 : 0.001;
            if (!hideHandler) {
                this._hideHandler = null;
                Laya.timer.once(time, this, this.onCloseClick);
            } else {
                this._hideHandler = hideHandler;
            }
            let fontSize = 50;
            let width = str.length * 53;
            let height = 50;
            if (width > 500) {
                let n = Math.abs(Math.pow(width / (8 * (height)), 0.5));
                width /= n;
                width = Math.min(width, Laya.stage.width - 380);
            }
            str = str.replace("<star>", "<img src='topui/xingxing.png' style='width:60; height:60'></img>");
            str = str.replace("<yugan>", "<img src='topui/yugan.png' style='width:49; height:49'></img>");

            let div = new Laya.HTMLDivElement();
            div.style.fontSize = fontSize;
            div.style.width = width;
            div.style.leading = 10;
            div.style.color = "#72584a";
            div.style.align = "center";
            div.style.valign = "middle";
            div.style.lineHeight = 50;
            div.innerHTML = str;
            div.x = - width / 2;
            div.y = 10 - div.contextHeight / 2;

            this.tipBox.width = Math.max(div.contextWidth + 200, 500) / this.tipBox.scaleX;
            this.tipBox.height = Math.max(div.contextHeight + 50, 120) / this.tipBox.scaleY;
            if (this.tipBox.height * this.tipBox.scaleY == 200) {
                this.tipBox.height = 201 / this.tipBox.scaleY;
            }
            this.tipSprite.x = this.tipBox.width / 2;
            this.tipSprite.y = this.tipBox.height / 2;
            this.tipSprite.addChild(div);
            this.tipSprite.scaleX = this.tipSprite.scaleY = 0.4;
            this.tipBox.y = this.owner.height / 2 + offsetY;
            if (up) {
                let toY = this.tipBox.y - 100;
                Laya.Tween.to(this.tipBox, { y: toY }, time / 3, Laya.Ease.backOut);
            } else {
                this.tipBox.y = this.owner.height / 2;
                Laya.Tween.from(this.tipBox, { scaleX: 0, scaleY: 0, alpha: 0.2 }, time / 3, Laya.Ease.backOut);
            }
        }

    }

    class TopBtnsOrder
    {
        constructor()
        {
            this.btn=null;
            this.order=-1;
        }
    }

    /**
     * 顶部UI
     */
    class TopUI extends Laya.Script {
        constructor() {
            super();
        }
       //第2排按钮排序定义:  2:imageLib,3:sign ,4:invitBtn

        onStart() {
        }
       
        onEnable() {
            
            // this.topBtnsArr = [];
            let xingxing = this.owner.getChildByName("xingxing");
            this.xiaoyugan = this.owner.getChildByName("xiaoyugan");
            GameContext.xiaoyuganPos=this.owner.localToGlobal(new Laya.Point(this.xiaoyugan.x, this.xiaoyugan.y));
            this.beike = this.owner.getChildByName("beike");
            this.zuanshi = this.owner.getChildByName("zuanshi");
            let leftClick = this.owner.getChildByName("leftClick");
            let rightClick = this.owner.getChildByName("rightClick");
            let downClick = this.owner.getChildByName('downClick');

            this.fishText = this.xiaoyugan.getChildByName("fish_text");
            this.starText = xingxing.getChildByName("star_text");
            this.beiketext = this.beike.getChildByName("beike_text");
            this.zuanshitext = this.zuanshi.getChildByName("zuanshi_text");
            let fishImage = this.xiaoyugan.getChildByName("fishImage");
            let starImage = xingxing.getChildByName("xingxingImage");
            let beikeImage = this.beike.getChildByName('beikeImage');
            let zuanshiImage = this.zuanshi.getChildByName('zuanshiImage');
            this.yuganImg = this.xiaoyugan.getChildByName("yuganImg");
            this.yuganText =this.xiaoyugan.getChildByName("yuganText");
            this.yuganText.text = Strings$1.get("tid#Tips_Resources1");
            this.xingxingImg = xingxing.getChildByName("xingxingImg");
            this.xingxingText = xingxing.getChildByName("xingxingText");
            this.xingxingText.text = Strings$1.get("tid#Tips_Resources2");

            this.beikebg = this.beike.getChildByName('beikebg');
            this.beikeDesc = this.beike.getChildByName("beikeText");
            this.beikeDesc.text = Strings$1.get("tid#Tips_Resources3");
            this.zuanshibg = this.zuanshi.getChildByName('zuanshibg');
            this.zuanshiDesc = this.zuanshi.getChildByName("zuanshiText");
            this.zuanshiDesc.text = Strings$1.get("tid#Tips_Resources4");
            this.purchaseBtn = this.owner.getChildByName("purchaseBtn");
            this.purchaseBtn.on(Laya.Event.CLICK,this,this.onPurchaseBtnClick);

            //this.btnBox = this.owner.getChildByName("btnBox");
            //modified by tenkin,for UI adjust
            // this.sign = this.owner.getChildByName("signBtn");
            // this.sign.on(Laya.Event.CLICK, this, this.onSignMenuClick);
            // this.imageLibBtn = this.owner.getChildByName("ImageLibBtn");
            // this.imageLibBtn.on(Laya.Event.CLICK, this, this.onImageLibClick);
            // this.signPoint = this.sign.getChildByName("signRedP");
            // this.signPoint.visible = false;
            //this.pushToBtns(this.photo,1);
            // this.dailyRewardBtn=this.owner.getChildByName("DailyRewardBtn");
            // this.dailyRewardBtn.on(Laya.Event.CLICK,this,this.dailyRewardBtnClicked);
            leftClick.on(Laya.Event.MOUSE_DOWN, this, this.onXiaoyuganClick);
            leftClick.on(Laya.Event.MOUSE_UP, this, this.offXiaoyuganClick);
            leftClick.on(Laya.Event.MOUSE_OUT, this, this.offXiaoyuganClick);
            rightClick.on(Laya.Event.MOUSE_DOWN, this, this.onXingxingClick);
            rightClick.on(Laya.Event.MOUSE_UP, this, this.offXingxingClick);
            rightClick.on(Laya.Event.MOUSE_OUT, this, this.offXingxingClick);
            downClick.on(Laya.Event.MOUSE_DOWN, this, this.onBeikeClick);
            downClick.on(Laya.Event.MOUSE_UP, this, this.offBeikeClick);
            downClick.on(Laya.Event.MOUSE_OUT, this, this.offBeikeClick);

            
            Constants.fishImagePoint = this.xiaoyugan.localToGlobal(new Laya.Point(fishImage.x, fishImage.y));
            Constants.starImagePoint = xingxing.localToGlobal(new Laya.Point(starImage.x, starImage.y));
            Constants.shellImagePoint = this.beike.localToGlobal(new Laya.Point(beikeImage.x, beikeImage.y));
            Constants.zuanshiImagePoint = this.zuanshi.localToGlobal(new Laya.Point(zuanshiImage.x, zuanshiImage.y));
            this.updateFish(PlayerSystem.getFish());
            this.updateStar(PlayerSystem.getStar());
            this.updateBeike(PlayerSystem.getShell());
            this.updateZuanshi(PlayerSystem.getDiamond());

            this.maxTimes = 6;
            this.maxStarTimes = 6;
            this.showBeike();
            EventMgr.getInstance().registEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().registEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().registEvent("SHOW_ADD", this, this.showAdd);
            EventMgr.getInstance().registEvent("SHOW_BEIKE", this, this.showBeike);//只在出航界面显示
            EventMgr.getInstance().registEvent("UPDATE_BEIKE", this, this.updateBeike);//贝壳 
            EventMgr.getInstance().registEvent("UPDATE_ZUANSHI", this, this.updateZuanshi);
            // EventMgr.getInstance().registEvent("RefreshInviteUI_Event", this, this.ShowInviteUI);
            // EventMgr.getInstance().registEvent("SHOW_SIGNBTN", this, this.showSignbtn);
            // EventMgr.getInstance().registEvent("SHOW_SIGNPOINT", this, this.showSignPoint);
            // EventMgr.getInstance().registEvent("UnShowSignBtn_Event",this,this.UnShowSignBtn);
            this.startPosx = 600;
            // this.showImageLibBtn();
            // this.showSignbtn();
            // this.showSignPoint();
            // this.ShowInviteBtn();
            // //this.resetPos();
            // this.resetPos2();
            // this.ShowDailyRewardBtn();
        }

        onPurchaseBtnClick(){
            Laya.Dialog.close("ui/PayPop.scene");
            Laya.Dialog.open("ui/PayPop.scene",true,[1]);
        }

        onXiaoyuganClick() {
            this.yuganImg.visible = true;
            this.yuganText.visible = true;
        }

        offXiaoyuganClick() {
            this.yuganImg.visible = false;
            this.yuganText.visible = false;
        }

        onXingxingClick() {
            this.xingxingImg.visible = true;
            this.xingxingText.visible = true;
        }

        offXingxingClick() {
            this.xingxingImg.visible = false;
            this.xingxingText.visible = false;
        }

        onBeikeClick() {
            if(this.beike.visible){
                this.beikebg.visible = true;
                this.beikeDesc.visible = true;
            }
            if(this.zuanshi.visible){
                this.zuanshibg.visible = true;
                this.zuanshiDesc.visible = true;
            }
        }

        offBeikeClick() {
            this.beikebg.visible = false;
            this.beikeDesc.visible = false;
            this.zuanshibg.visible = false;
            this.zuanshiDesc.visible = false;
        }

        onDisable() {
            EventMgr.getInstance().removeEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().removeEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().removeEvent("SHOW_ADD", this, this.showAdd);
            EventMgr.getInstance().removeEvent("SHOW_BEIKE", this, this.showBeike);//只在出航界面显示
            EventMgr.getInstance().removeEvent("UPDATE_BEIKE", this, this.updateBeike);
            EventMgr.getInstance().removeEvent("UPDATE_ZUANSHI", this, this.updateZuanshi);
            // EventMgr.getInstance().removeEvent("SHOW_SIGNPOINT", this, this.showSignPoint);
            // EventMgr.getInstance().removeEvent("UnShowSignBtn_Event",this,this.UnShowSignBtn);
        }

        onDestroy() {
            EventMgr.getInstance().removeEvent("UPDATE_FISH", this, this.updateFish);
            EventMgr.getInstance().removeEvent("UPDATE_STAR", this, this.updateStar);
            EventMgr.getInstance().removeEvent("SHOW_ADD", this, this.showAdd);
            EventMgr.getInstance().removeEvent("SHOW_BEIKE", this, this.showBeike);//只在出航界面显示
            EventMgr.getInstance().removeEvent("UPDATE_BEIKE", this, this.updateBeike);
            EventMgr.getInstance().removeEvent("UPDATE_ZUANSHI", this, this.updateZuanshi);
            // EventMgr.getInstance().removeEvent("SHOW_SIGNPOINT", this, this.showSignPoint);
            // EventMgr.getInstance().removeEvent("UnShowSignBtn_Event",this,this.UnShowSignBtn);
        }

        updateFish(fish, ifTween) {

            if (!ifTween) {
                if (this.endFish != fish) {
                    this.fishNum = fish;
                    this.times = null;
                }
                return;
            }
            this.startFish = this.fishNum;
            this.endFish = fish;
            if (Math.abs(this.endFish - this.startFish) < 6) {
                this.maxTimes = this.endFish - this.startFish;
            } else {
                this.maxTimes = 6;
            }
            this.times = 1;
            this.updataFishSixTimes();
        }

        updateStar(star) {
           
            this.starText.text = String(star);
            // let ringCnt = star;
            // let constCfg = ConfigMgr.getInstance().getItemById("Const", "CollectionHandbookUnlock");
            // let ringCntCfg = constCfg.Value;
            // if (ringCnt >= ringCntCfg) {
            //     this.imageLibBtn.visible = true;
            //     this.pushToBtns(this.imageLibBtn,2);
            // }
            // else {
            //     this.imageLibBtn.visible = false;
            //     this.removeBtn(2);
            // }
            // this.ShowInviteBtn();
            // let sceneId=  SceneMgr.getInstance().getCurrentSceneId();
           // this.ShowDailyRewardBtn();
            // this.showBtn(sceneId);
           
            // this.resetPos2();
        }

        updateBeike(beike) {
            this.beiketext.text = String(beike);
        }

        updateZuanshi(zuanshi) {
            this.zuanshitext.text = String(zuanshi);
        }

        get fishNum() {
            return this.fishNumber;
        }

        set fishNum(fish) {
            this.fishNumber = Math.floor(fish);
            this.fishText.text = Strings$1.dealFishNumberText(fish);
        }

        get starNum() {
            return Number(this.starText.text);
        }

        set starNum(star) {
            this.starText.text = String(Math.floor(star));
        }


        get beikeNum() {
            return Number(this.beiketext.text);
        }

        set beikeNum(beike) {
            this.beiketext.text = String(Math.floor(beike));
        }

        get zuanshiNum() {
            return Number(this.zuanshitext.text);
        }

        set zuanshiNum(zuanshi) {
            this.zuanshitext.text = String(Math.floor(zuanshi));
        }

        updataFishSixTimes() {
            if (this.times == null) {
                return;
            }
            this.fishNum = Math.floor(this.startFish + (this.endFish - this.startFish) / this.maxTimes * this.times);
            if (this.times < this.maxTimes) {
                this.times++;
                Laya.timer.once(50, this, this.updataFishSixTimes);
            } else {
                this.fishNum = PlayerSystem.getFish();
            }
        }

        updataStarSixTimes() {
            if (this.starTimes == null) {
                return;
            }
            this.starNum = Math.floor(this.startStar + (this.endStar - this.startStar) / this.maxStarTimes * this.starTimes);
            if (this.starTimes < this.maxStarTimes) {
                this.starTimes++;
                Laya.timer.once(50, this, this.updataStarSixTimes);
            } else {
                this.starNum = PlayerSystem.getStar();
            }
        }
        updataBeikeSixTimes() {
            if (this.beikeTimes == null) {
                return;
            }
            this.beikeNum = Math.floor(this.startbeike + this.addbeike / this.maxStarTimes * this.starTimes);
            if (this.starTimes < this.maxStarTimes) {
                this.starTimes++;
                Laya.timer.once(50, this, this.updataBeikeSixTimes);
            } else {
                this.beikeNum = PlayerSystem.getShell();
            }
        }

        updataZuanshiSixTimes() {
            if (this.zuanshiTimes == null) {
                return;
            }
            this.zuanshiNum = Math.floor(this.startZuanshi + this.addZuanshi / this.maxStarTimes * this.starTimes);
            if (this.starTimes < this.maxStarTimes) {
                this.starTimes++;
                Laya.timer.once(50, this, this.updataZuanshiSixTimes);
            } else {
                this.zuanshiNum = PlayerSystem.getDiamond();
            }
        }

        showAdd(addNum, type, handler) {
            if (type == Constants.RewardType.FISH) {
                if (!this.fishAddLabel) {
                    this.fishAddLabel = this.owner.getChildByName("fishAddLabel");
                    this.fishAddLabel.visible = true;
                }
                this.fishAddLabel.text = "+" + addNum;
                this.fishAddLabel.y = 139;
                this.fishAddLabel.visible = true;
                this.fishAddLabel.alpha = 0;
                Laya.Tween.to(this.fishAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.FISH, handler]));
            } else if (type == Constants.RewardType.STAR) {
                if (!this.starAddLabel) {
                    this.starAddLabel = this.owner.getChildByName("starAddLabel");
                    this.starAddLabel.visible = true;
                }
                this.starAddLabel.text = "+" + addNum;
                this.starAddLabel.y = 139;
                this.starAddLabel.visible = true;
                this.starAddLabel.alpha = 0;
                Laya.Tween.to(this.starAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.STAR, handler]));
            }
            else if (type == Constants.RewardType.SHELL) {
                if (!this.beikeAddLabel) {
                    this.beikeAddLabel = this.owner.getChildByName("beikeAddLabel");
                    this.beikeAddLabel.visible = true;
                }
                this.addbeike = addNum;
                this.beikeAddLabel.text = "+" + addNum;
                this.beikeAddLabel.y = 231;
                this.beikeAddLabel.visible = true;
                this.beikeAddLabel.alpha = 0;
                Laya.Tween.to(this.beikeAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.SHELL, handler]));
            }
            else if (type == Constants.RewardType.DIAMOND) {
                if (!this.zuanshiAddLabel) {
                    this.zuanshiAddLabel = this.owner.getChildByName("zuanshiAddLabel");
                    this.zuanshiAddLabel.visible = true;
                }
                this.addzuanshi = addNum;
                this.zuanshiAddLabel.text = "+" + addNum;
                this.zuanshiAddLabel.y = 231;
                this.zuanshiAddLabel.visible = true;
                this.zuanshiAddLabel.alpha = 0;
                Laya.Tween.to(this.zuanshiAddLabel, { y: 76, alpha: 1 }, 250, null, Laya.Handler.create(this, this._addLabelFadeout, [Constants.RewardType.DIAMOND, handler]));
            }
            this._addTween(type);
        }

        _addLabelFadeout(type, handler) {
            if (type == Constants.RewardType.FISH) {
                Laya.Tween.to(this.fishAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            } else if (type == Constants.RewardType.STAR) {
                Laya.Tween.to(this.starAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            } else if (type == Constants.RewardType.SHELL) {
                Laya.Tween.to(this.beikeAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            } else if (type == Constants.RewardType.DIAMOND) {
                Laya.Tween.to(this.zuanshiAddLabel, { y: 13, alpha: 0 }, 250, null, handler);
            }
        }
        //表现不增加真实数值
        _addTween(type) {
            if (type == Constants.RewardType.FISH) {
                this.startFish = this.fishNum;
                this.endFish = PlayerSystem.getFish();
                let addNum = this.endFish - this.startFish;
                if (Math.abs(addNum) < 6) {
                    this.maxTimes = addNum;
                } else {
                    this.maxTimes = 6;
                }
                this.times = 1;
                this.updataFishSixTimes();
            } else if (type == Constants.RewardType.STAR) {
                this.startStar = this.starNum;
                this.endStar = PlayerSystem.getStar();
                let addNum = this.endFish - this.startFish;
                if (Math.abs(addNum) < 6) {
                    this.maxStarTimes = addNum;
                } else {
                    this.maxStarTimes = 6;
                }
                this.starTimes = 1;
                this.updataStarSixTimes();
            } else if (type == Constants.RewardType.SHELL) {
                this.startbeike = this.beikeNum;
                let addNum = this.addbeike;
                if (Math.abs(addNum) < 6) {
                    this.maxStarTimes = addNum;
                } else {
                    this.maxStarTimes = 6;
                }
                this.beikeTimes = 1;
                this.updataBeikeSixTimes();
            } else if (type == Constants.RewardType.DIAMOND) {
                this.startZuanshi = this.zuanshiNum;
                let addNum = this.addZuanshi;
                if (Math.abs(addNum) < 6) {
                    this.maxStarTimes = addNum;
                } else {
                    this.maxStarTimes = 6;
                }
                this.zuanshiTimes = 1;
                this.updataZuanshiSixTimes();
            }
        }
        showBeike(sceneId) {
            if (!sceneId) {
                sceneId = SceneMgr.getInstance().getCurrentSceneId();
            }
            this.beike.visible = false;
            this.zuanshi.visible = false;
            this.purchaseBtn.visible = false;
            if (sceneId == Constants.NauticalSceneId) {
                if(BuildingSystem.getBuildingDataByPid(302)){
                    this.beike.visible = true;
                    this.zuanshi.visible = false;
                    this.purchaseBtn.visible = PlayerSystem.getStar() >= Const.get("PurchaseEntranceShowStar");
                   
                }
            } else {
                this.beike.visible = false;
                this.zuanshi.visible = true;
                this.purchaseBtn.visible = PlayerSystem.getStar() >= Const.get("PurchaseEntranceShowStar");
            }
        }
        onDestroy() {
            Laya.Tween.clearAll(this);
        }
    //     ShowInviteUI() {
    //         Laya.Dialog.close("ui/InviteUI.scene");
    //         Laya.Dialog.open("ui/InviteUI.scene");
    //     }
    //     onSignMenuClick() {
    //         Laya.Dialog.close("ui/Sign.scene");
    //         Laya.Dialog.open("ui/Sign.scene", false, this.mailData)
    //     }
    //     onImageLibClick() {
    //         Laya.Dialog.close("ui/ImageLibrary.scene");
    //         Laya.Dialog.open("ui/ImageLibrary.scene", false, null);
    //     }
    //     showSignbtn(sceneId) {
    //         if (!sceneId) {
    //             sceneId = SceneMgr.getInstance().getCurrentSceneId();
    //         }

    //         if (PlayerDataMgr.getInstance().getStar() >= Const.get('SignSystemOpenCondition') && SignSystem.getSignDays() < 7) {
    //             this.sign.visible = true;
    //             if (sceneId == Constants.CoffeeSceneId) {
    //                 this.sign.visible = true;
    //             } else if (sceneId == Constants.BeachSceneId) {
    //                 this.sign.visible = false;
    //             } else if (sceneId == Constants.NauticalSceneId) {
    //                 this.sign.visible = false;
    //             } else if (sceneId == Constants.HomelandId) {
    //                 this.sign.visible = false;
    //             }
    //         } else {
    //             this.sign.visible = false;
    //         }
    //       //  GameContext.signBtnState=this.sign.visible;
    //         if(this.sign.visible)
    //         {
    //             this.pushToBtns(this.sign, 3);
    //         }
    //         else
    //         {
    //             this.removeBtn(3);
    //         }
    //         this.resetPos2();
    //     }
    //     showSignPoint() {
    //         if (SignSystem.ifQiandao() == true) {
    //             this.signPoint.visible = false;

    //         }
    //         else {
    //             this.signPoint.visible = true;
    //         }
    //     }

    //     showImageLibBtn() {
    //         let constCfg = ConfigMgr.getInstance().getItemById("Const", "CollectionHandbookUnlock");
    //         let ringCntCfg = constCfg.Value;
    //         let ringCnt = PlayerSystem.getStar();
    //         if (ringCnt >= ringCntCfg) {
    //             this.imageLibBtn.visible = true;
               
    //         }
    //         else {
    //             this.imageLibBtn.visible = false;
    //         }
           
    //         if(this.imageLibBtn.visible)
    //         {
    //             this.pushToBtns(this.imageLibBtn, 2);
    //         }
    //         else
    //         {
    //             this.removeBtn(2);
    //         }
    //     }
    //     pushToBtns(btn, orderIndex) {
            
    //         let cnt=0;
    //         for(let i=0;i<this.topBtnsArr.length;i++)
    //         {
    //             if(this.topBtnsArr[i].order==orderIndex)
    //             {
    //                 cnt+=1;
    //             }
    //         }
    //         if (cnt==0) { 
    //             let btnOrder = new TopBtnsOrder();
    //         btnOrder.btn = btn;
    //         btnOrder.order = orderIndex;
    //             this.topBtnsArr.push(btnOrder); }

    //     }
    //     removeBtn(orderIndex)
    //     {
    //         let index=this.topBtnsArr.findIndex(item=>item.order===orderIndex);
    //         if(index>-1)
    //       this.topBtnsArr.splice(index,1);
          
    //     }
    //     ShowInviteBtn() {
    //         let constCfg = ConfigMgr.getInstance().getItemById("Const", "InviteTaskUnlockLimit");
    //         let ringCntCfg = constCfg.Value;
    //         let ringCnt = PlayerSystem.getStar();
    // let inviteSecondState=ringCnt>=ringCntCfg?true:false;
    //         this.inviteBtn.visible = GameContext.inviteBtnState&&inviteSecondState;
    //         if(this.inviteBtn.visible)
    //         {
    //             this.pushToBtns(this.inviteBtn, 4);
    //         }
    //         else
    //         {
    //             this.removeBtn(4);
    //         }
    //     }
    //    ShowDailyRewardBtn()
    //    {
    //     let constCfg = ConfigMgr.getInstance().getItemById("Const", "DailyGiftUnlockLimit");
    //     let ringCntCfg = constCfg.Value;
    //     let ringCnt = PlayerSystem.getStar();
    //    let rwdBtnState=ringCnt>=ringCntCfg?true:false;
    //    this.dailyRewardBtn.visible=rwdBtnState;
    //    this.dailyRewardBtn.x=450;
    //    this.dailyRewardBtn.y=270;
    //    }
    //     resetPos2() {
    //         function sortByOrder(a, b) {
    //             return a.order - b.order;
    //         }
    //         this.topBtnsArr.sort(sortByOrder);
    //         for (let i = 0; i < this.topBtnsArr.length; i++) {
               
    //             this.topBtnsArr[i].btn.pos(this.startPosx + i * 112,260) ;
                
    //         }
           
    //     }
    //     resetPos()
    //     {
    //         function sortByOrder(a, b) {
    //             return a.order - b.order;
    //         }
    //         this.topBtnsArr.sort(sortByOrder);
            
    //     }
    //     showBtn(sceneId) {
            
    // let inviteSecondState=this.cfgState("InviteTaskUnlockLimit");
    // let imgSecondState=this.cfgState("CollectionHandbookUnlock");
    // let signSecondState=false;
    // if (PlayerDataMgr.getInstance().getStar() >= Const.get('SignSystemOpenCondition') && SignSystem.getSignDays() < 7)
    // {
    //     signSecondState=true;
    // }
    // let dailyRwdState = PlayerDataMgr.getInstance().getStar()>= ConfigMgr.getInstance().getItemById("Const", "DailyGiftUnlockLimit").Value?true:false;
    //         switch (sceneId) {
    //             case Constants.CoffeeSceneId: {
    //                // this.btnBox.visible = true;
    //                this.imageLibBtn.visible=GameContext.imageBtnState&&imgSecondState;
    //                this.inviteBtn.visible=GameContext.inviteBtnState&&inviteSecondState;
    //                this.sign.visible=GameContext.signBtnState&&signSecondState;
    //                this.dailyRewardBtn.visible=dailyRwdState;
    //                 break;
    //             }
    //             case Constants.BeachSceneId: {
    //                 //this.btnBox.visible = true;
    //                 this.imageLibBtn.visible=false;
    //                 this.inviteBtn.visible=false;
    //                 this.sign.visible=false;
    //                 this.dailyRewardBtn.visible=false;
    //                 break;
    //             }
    //             case Constants.NauticalSceneId: {
    //                // this.btnBox.visible = true;
    //                this.imageLibBtn.visible=false;
    //                this.inviteBtn.visible=false;
    //                this.sign.visible=false;
    //                this.dailyRewardBtn.visible=false;
    //                 break;
    //             }
    //             case Constants.HomelandId: {
    //                 //this.btnBox.visible = true;
    //                 this.imageLibBtn.visible=GameContext.imageBtnState&&imgSecondState;
    //                 this.inviteBtn.visible=false;
    //                 this.sign.visible=false;
    //                 this.dailyRewardBtn.visible=false;
    //                 break;
    //             }
    //             case Constants.FishingSceneId: {
    //                // this.btnBox.visible = false;
    //                this.imageLibBtn.visible=false;
    //                this.inviteBtn.visible=false;
    //                this.sign.visible=false;
    //                this.dailyRewardBtn.visible=false;
    //                 break;
    //             }
    //             default: {
    //                 //this.btnBox.visible = true;

    //             }
               
    //         }
    //         this.ReSetBtns(this.imageLibBtn,2);
    //         this.ReSetBtns(this.sign,3);
    //         this.ReSetBtns(this.inviteBtn,4);
    //         this.showBeike(sceneId);
           
    //     }
    //     cfgState(cfgName)
    //     {
    //         let constCfg = ConfigMgr.getInstance().getItemById("Const", cfgName);
    //         let ringCntCfg = constCfg.Value;
    //         let ringCnt = PlayerSystem.getStar();
    // let SecondState=ringCnt>=ringCntCfg?true:false;
    // return SecondState;
    //     }
    //     ReSetBtns(btn ,orderIndex)
    //     {
    //       if(btn.visible)
    //       {
    //           this.pushToBtns(btn,orderIndex);
    //       }
    //       else
    //       {
    //           this.removeBtn(orderIndex);
    //       }
    //     }

    //     unshowInviteBtn(state)
    //     {
    //         if(state)
    //         {
    //             GameContext.inviteBtnState=false;
    //             this.inviteBtn.visible=false;
    //             this.ReSetBtns(this.inviteBtn,4);
    //             this.resetPos2();
    //         }
    //     }

    //     UnShowSignBtn(state)
    //     {
    //         if(state)
    //         {
    //             GameContext.signBtnState=false;
    //             this.sign.visible=false;
    //             this.ReSetBtns(this.sign,3);
    //             this.resetPos2();
    //         }
    //     }
    //     dailyRewardBtnClicked()
    //     {
    //         Laya.Dialog.close("ui/DailyRewardUI.scene");
    //         Laya.Dialog.open("ui/DailyRewardUI.scene");
    //     }
       
    }

    /**
     * 顶部UI
     */
    let animPath$1 = "anim1/duoyugan.sk";

    class ZhuliBaoxiang extends Laya.Script {
        constructor() {
            super();
        }

        onStart() {
        }
        onAwake() {
            this.baoxiangAnim = this.owner.getChildByName('beikeAnim');
            this.drop_beike = this.owner.getChildByName('drop_beike');
            // Laya.timer.once(1500,this,this.beikedonghua,[true]);
        }
        onEnable() {
            this.dating = this.owner.getChildByName('dating');
            this.zhuliBox = this.owner.getChildByName('zhuliBox');
            this.baoxiang = this.owner.getChildByName('baoxiang');
            this.beikeNum = this.zhuliBox.getChildByName('beikeNum');
            this.fishNum = this.zhuliBox.getChildByName('fishNum');
            this.fish = this.zhuliBox.getChildByName('fish');
            this.beike = this.zhuliBox.getChildByName('beike');
            this.zhuliFish = this.zhuliBox.getChildByName('zhuliFish');
            this.zhuliBeike = this.zhuliBox.getChildByName('zhuliBeike');
            this.tips = this.zhuliBox.getChildByName('tip');
            this.zhuliItem1 = this.owner.getChildByName('zhuliItem1');
            this.zhuliItem2 = this.owner.getChildByName('zhuliItem2');
            this.zhuliItem3 = this.owner.getChildByName('zhuliItem3');
            this.project1 = this.zhuliItem1.getChildByName('project');
            this.project2 = this.zhuliItem2.getChildByName('project');
            this.project3 = this.zhuliItem3.getChildByName('project');
            this.zhuliItem1.visible = false;
            this.zhuliItem2.visible = false;
            this.zhuliItem3.visible = false;

            this.showAward();
            this.baoxiang.on(Laya.Event.MOUSE_DOWN, this, this.xialuoX);
            this.baoxiang.on(Laya.Event.CLICK, this, this.getAward);
            this.dating.on(Laya.Event.MOUSE_DOWN, this, this.xialuoX);
            this.dating.on(Laya.Event.CLICK, this, this.jinrudating);
            this.duoFangBeike();
            EventMgr.getInstance().registEvent("UPDATE_WHARFLETTER_BOX", this, this.showAward);
            EventMgr.getInstance().registEvent("OPEN_AWARD", this, this.getAward);
            this.qipaopiao();//气泡的个数 时间
            this.owner.ani1.play(0, true);
        }
        onDestroy() {
            Laya.Tween.clearAll(this);
            Laya.timer.clearAll(this);
        }
        showAward() {
            let jiangliNume = 2;
            if (ZhuliSystem.getCacheFish() != 0 && ZhuliSystem.getShareShell() != 0) {
                this.fishNum.visible = true;
                this.beikeNum.visible = true;
                this.zhuliFish.visible = false;
                this.zhuliBeike.visible = true;
                this.tips.visible = true;
                this.beike.visible = true;
                this.fish.visible = true;
                jiangliNume = 2;
            } else if (ZhuliSystem.getCacheFish() != 0 && ZhuliSystem.getShareShell() == 0) {
                this.beikeNum.visible = false;
                this.beike.visible = false;
                this.zhuliBeike.visible = false;
                this.fishNum.visible = true;
                this.zhuliFish.visible = false;
                this.tips.visible = true;
                this.fish.visible = true;
                jiangliNume = 1;

            } else if (ZhuliSystem.getCacheFish() == 0 && ZhuliSystem.getShareShell() != 0) {
                this.beikeNum.visible = true;
                this.beike.visible = true;
                this.fishNum.visible = false;
                this.zhuliBeike.visible = true;
                this.zhuliFish.visible = false;
                this.tips.visible = true;
                this.fish.visible = false;
                jiangliNume = 1;
            }
            else {
                this.fishNum.visible = false;
                this.beikeNum.visible = false;
                this.zhuliFish.visible = false;
                this.tips.visible = false;
                this.beike.visible = false;
                this.fish.visible = false;
            }

            this.fishNum.text = Strings$1.dealFishNumberText(Number(ZhuliSystem.getCacheFish()) + Number(ZhuliSystem.getShareFish()));
            this.beikeNum.text = Strings$1.dealFishNumberText(ZhuliSystem.getShareShell());
            if (ZhuliSystem.getShareFish() == 0) {
                this.zhuliFish.visible = false;
            } else {
                this.zhuliFish.visible = false;
            }
            if (ZhuliSystem.getShareShell() == 0) {
                this.zhuliBeike.visible = false;
            } else {
                this.zhuliBeike.visible = false;
            }
            this.fish.x = this.tips.x + 10;
            this.fishNum.x = this.fish.x + 10 + this.fish.width * this.fish.scaleX;

            if (jiangliNume == 2) {
                this.tips.width = 210;
                this.tips.x = 0;
                this.fish.x = this.tips.x;
                this.fish.x = (this.tips.width * this.tips.scaleX - (this.fish.width * this.fish.scaleX + this.beike.width * this.beike.scaleX + 60 + this.fishNum.width + this.beikeNum.width)) / 2 + this.tips.x;
                this.fishNum.x = this.fish.x + 10 + this.fish.width * this.fish.scaleX;
                this.beike.x = this.fishNum.x + this.fishNum.width + 10;

            } else if (jiangliNume == 1) {
                this.tips.width = 150;
                this.tips.x = 60;
                this.fish.x = (this.tips.width * this.tips.scaleX - this.fish.width * this.fish.scaleX - this.fishNum.width - 20) / 2 + this.tips.x;
                this.fishNum.x = this.fish.x + 10 + this.fish.width * this.fish.scaleX;
                this.beike.x = (this.tips.width * this.tips.scaleX - this.beike.width * this.beike.scaleX - this.beikeNum.width - 20) / 2 + this.tips.x;

            }
            this.beikeNum.x = this.beike.x + 10 + this.beike.width * this.beike.scaleX;
            this.zhuliFish.text = '(' + Strings$1.dealFishNumberText(ZhuliSystem.getShareFish()) + ')';
            // this.zhuliBeike.text='('+Strings.dealFishNumberText(ZhuliSystem.getShareShell())+')'; 
            this.zhuliFish.x = this.fishNum.x + 10 + this.fishNum.width;
            // this.zhuliBeike.x= this.beikeNum.x+10+this.beikeNum.width;
            this.beikedonghua();
        }
        //获取下落x
        xialuoX(event) {
            this.xialuo = event.stageX;
        }
        jinrudating(event) {
            let taiqi = event.stageX;
            if (this.xialuo - taiqi > 50) {
                return;
            }
            Constants.Yuledating=1;
            Laya.Dialog.close("ui/shopPop.scene");
            Laya.Dialog.open("ui/shopPop.scene", false, [1]);
        }
        //抬起x
        getAward(event) {
            if (event) {
                let taiqi = event.stageX;
                if (this.xialuo - taiqi > 50) {
                    return;
                }
            }
            if (ZhuliSystem.getCacheFish() != 0 || ZhuliSystem.getShareFish() != 0 || ZhuliSystem.getShareShell() != 0) {
                let point = new Laya.Point(this.baoxiang.x + 70, this.baoxiang.y + 120);
                point = this.baoxiang.parent.localToGlobal(point);
                Laya.Dialog.close("ui/ZhuliPopAward.scene");
                Laya.Dialog.open("ui/ZhuliPopAward.scene", false, [point]);
            }
        }
        getBeikeAward(event) {
            let taiqi = event.stageX;
            if (this.xialuo - taiqi > 50) {
                return;
            }
            if (ZhuliSystem.getCacheShell() != 0) {
                EventMgr.getInstance().postEvent('FETCH_DAILY_SHELL', Laya.Handler.create(this, this.showFly));
                // System._FetchDailyShell(Laya.Handler.create(this, this.showFly));
            }
        }
        showFly(res) {
            this.baoxiangAnim.visible = false;
            let point = new Laya.Point(this.baoxiangAnim.x, this.baoxiangAnim.y);
            point = this.baoxiangAnim.parent.localToGlobal(point);
            let rewards = res;
            for (let i in rewards) {
                let reward = 0;
                if (rewards[i].id == Constants.RewardType.SHELL) {
                    reward = rewards[i].amount - PlayerSystem.getShell();
                }
                if (reward <= 0) {
                    continue;
                }
                if (rewards[i].id == Constants.RewardType.SHELL) {
                    ItemFlyMgr.getInstance().fly(Constants.RewardType.SHELL, reward, Constants.FlyType.ZCM, point.x, point.y);
                }
            }
        }
        beikedonghua() {
            if (ZhuliSystem.getCacheShell() > 0) {
                this.baoxiangAnim.visible = true;
                this.baoxiangAnim.on(Laya.Event.MOUSE_DOWN, this, this.xialuoX);
                this.baoxiangAnim.on(Laya.Event.CLICK, this, this.getBeikeAward);
            } else {
                this.baoxiangAnim.visible = false;
            }
        }
        onDisable() {
            Laya.timer.clearAll(this);
            EventMgr.getInstance().removeEvent("UPDATE_WHARFLETTER_BOX", this, this.showAward);
            EventMgr.getInstance().removeEvent("OPEN_AWARD", this, this.getAward);
            this.baoxiangAnim.removeChildren(0,2);
            this.owner.ani1.stop();
        }
        duoFangBeike() {
            //放3个贝壳
            for (let i = 0; i < 3; i++) {
                let duobeike = new Laya.Skeleton();
                this.baoxiangAnim.addChild(duobeike);
                duobeike.load(animPath$1, Laya.Handler.create(this, function () {
                    // if (this.duobeike.destroyed)
                    //     return;
                    duobeike.scale(0.7, 0.7);
                    if (i > 0) {
                        duobeike.x = 50 + i * 30;
                        duobeike.y = 100;
                    } else {
                        duobeike.x = 90;
                        duobeike.y = 70;
                    }
                    this.palayduobeike(duobeike);
                    Laya.timer.loop(Const.get('QuayDailyShellActionTime')*1000+2000, this, function () {
                    this.palayduobeike(duobeike);
                    });
                }));
            }

        }
        palayduobeike(duobeike){
             duobeike.play("beike", false);
             duobeike.once(Laya.Event.STOPPED, this, function(){
             duobeike.play("beike", false);
             duobeike.once(Laya.Event.STOPPED, this, function(){
             duobeike.play("beike", false);
            }
            );
            });
        }
        //游客大厅气泡
        qipaopiao() {
            this.qipaogeshu = 0;
            this.qipaoshijian = 0;
            let config = Const.get('QuayBubbleAmount');
            let configjson = JSON.parse(config);
            let list = ProjectSystem.getyoulechangList();
            let listnumber = 0;
            let quanzhonghe = 0;
            let projectlist = [];
            list.forEach(element => {
                if (element._unlocked) {
                    listnumber++;
                    let project = [];
                    quanzhonghe += Number(element._config.Weight);
                    project[0] = quanzhonghe;
                    project[1] = "nautical_project/" + element._config.ShopItems + ".png";
                    projectlist.push(project);
                }
            });
            if (quanzhonghe != 0) {
                for (let i = 0; i < 3; i++) {
                    let suiji = Math.random();
                    let quanzhong = suiji * quanzhonghe;
                    quanzhong = Math.floor(quanzhong);
                    let skin;
                    for (let i = 0; i < projectlist.length; i++) {
                        if (quanzhong < projectlist[i][0]) {
                            skin = projectlist[i][1];
                            break;
                        }
                    }
                    if (i == 0) {
                        this.project1.skin = skin;
                    } else if (i == 1) {
                        this.project2.skin = skin;
                    } else {
                        this.project3.skin = skin;
                    }

                }
            }
            if (listnumber > configjson[0].ProjectAmount[0] && listnumber <= configjson[1].ProjectAmount[1]) {
                let zong = 0;
                let quanzhong = [];
                let i = 0;
                for (const key in configjson[0].BubbleAmount) {
                    if (configjson[0].BubbleAmount.hasOwnProperty(key)) {
                        zong += Number(configjson[0].BubbleAmount[key]);
                        quanzhong[i] = configjson[0].BubbleAmount[key];
                        i++;
                    }
                }
                let suiji = Math.random();
                if (suiji < quanzhong[0] / zong) {
                    this.qipaogeshu = 1;
                } else if (suiji > (quanzhong[0] + quanzhong[1]) / zong) {
                    this.qipaogeshu = 3;
                } else {
                    this.qipaogeshu = 2;
                }
                let shijian1 = Number(configjson[0].BubbleTime[0]);
                let shijian2 = Number(configjson[0].BubbleTime[1]);
                this.qipaoshijian = Math.floor(shijian1 + suiji * (shijian2 - shijian1));
            } else if (listnumber > configjson[1].ProjectAmount[0] && listnumber <= configjson[1].ProjectAmount[1]) {
                let zong = 0;
                let quanzhong = [];
                let i = 0;
                for (const key in configjson[0].BubbleAmount) {
                    if (configjson[1].BubbleAmount.hasOwnProperty(key)) {
                        zong += Number(configjson[1].BubbleAmount[key]);
                        quanzhong[i] = configjson[1].BubbleAmount[key];
                        i++;
                    }
                }
                let suiji = Math.random();
                if (suiji < quanzhong[0] / zong) {
                    this.qipaogeshu = 1;
                } else if (suiji > (quanzhong[0] + quanzhong[1]) / zong) {
                    this.qipaogeshu = 3;
                } else {
                    this.qipaogeshu = 2;
                }
                let shijian1 = Number(configjson[1].BubbleTime[0]);
                let shijian2 = Number(configjson[1].BubbleTime[1]);
                this.qipaoshijian = Math.floor(shijian1 + suiji * (shijian2 - shijian1));
            } else if (listnumber > configjson[2].ProjectAmount[0]) {
                let zong = 0;
                let quanzhong = [];
                let i = 0;
                for (const key in configjson[2].BubbleAmount) {
                    if (configjson[0].BubbleAmount.hasOwnProperty(key)) {
                        zong += Number(configjson[2].BubbleAmount[key]);
                        quanzhong[i] = configjson[2].BubbleAmount[key];
                        i++;
                    }
                }
                let suiji = Math.random();
                if (suiji < quanzhong[0] / zong) {
                    this.qipaogeshu = 1;
                } else if (suiji > (quanzhong[0] + quanzhong[1]) / zong) {
                    this.qipaogeshu = 3;
                } else {
                    this.qipaogeshu = 2;
                }
                let shijian1 = Number(configjson[2].BubbleTime[0]);
                let shijian2 = Number(configjson[2].BubbleTime[1]);
                this.qipaoshijian = Math.floor(shijian1 + suiji * (shijian2 - shijian1));
            }
            if (listnumber != 0) {
                this.piaoqipao();//气泡的效果
            }
        }
        piaoqipao() {
            Laya.timer.once(this.qipaoshijian * 1000, this, function () {
                EventMgr.getInstance().postEvent('PLAY_DONGHUA3_YOUKE');
                Laya.timer.once(300, this, function () {
                    if (this.qipaogeshu == 0) {
                        this.zhuliItem1.visible = false;
                        this.zhuliItem2.visible = false;
                        this.zhuliItem3.visible = false;
                    } else if (this.qipaogeshu == 1) {
                        this.zhuliItem1.visible = true;
                        this.zhuliItem2.visible = false;
                        this.zhuliItem3.visible = false;
                    }
                    else if (this.qipaogeshu == 2) {
                        this.zhuliItem1.visible = true;
                        this.zhuliItem2.visible = true;
                        this.zhuliItem3.visible = false;
                    }
                    else if (this.qipaogeshu == 3) {
                        this.zhuliItem1.visible = true;
                        this.zhuliItem2.visible = true;
                        this.zhuliItem3.visible = true;
                    }

                    Laya.Tween.to(this.zhuliItem1, { y: 100, alpha: 0.2 }, 3000, Laya.Ease.circOut, Laya.Handler.create(this, function () {
                        this.zhuliItem1.visible = false;
                        this.zhuliItem1.y = 570;
                        this.zhuliItem1.alpha = 1;
                    }));
                    Laya.Tween.to(this.zhuliItem2, { y: 0, alpha: 0.2 }, 3000, Laya.Ease.circOut, Laya.Handler.create(this, function () {
                        this.zhuliItem2.visible = false;
                        this.zhuliItem2.y = 418;
                        this.zhuliItem2.alpha = 1;
                    }));
                    Laya.Tween.to(this.zhuliItem3, { y: -100, alpha: 0.2 }, 3000, Laya.Ease.circOut, Laya.Handler.create(this, function () {
                        this.zhuliItem3.visible = false;
                        this.zhuliItem3.y = 318;
                        this.zhuliItem3.alpha = 1;
                    }));
                });

                this.qipaopiao();
            });
        }
    }

    /**This class is automatically generated by LayaAirIDE, please do not make any modifications. */

    class GameConfig {
        static init() {
            //注册Script或者Runtime引用
            let reg = Laya.ClassUtils.regClass;
    		reg("scene/BeachScene.js",BeachScene);
    		reg("form/Bowl.js",Bowl);
    		reg("core/component/ViewAdjust.js",ViewAdjust);
    		reg("guide/Guide.js",Guide);
    		reg("scene/CoffeeScene.js",CoffeeScene);
    		reg("scene/FishingScene.js",FishingScene);
    		reg("form/CommonButton.js",CommonButton);
    		reg("scene/Homeland.js",Homeland);
    		reg("form/PhotoWall.js",PhotoWall);
    		reg("form/PhotoFrame.js",PhotoFrame);
    		reg("scene/LoginScene.js",LoginScene);
    		reg("scene/NauticalScene.js",NauticalScene);
    		reg("form/zhuliUI.js",ZhuliUI);
    		reg("form/AdoptAD.js",AdoptAD);
    		reg("form/AdoptDiamond.js",AdoptDiamond);
    		reg("form/AdoptSuccess.js",AdoptSuccess);
    		reg("form/ButtonImageRp.js",ButtonImageRp);
    		reg("form/AdvertisingPart.js",AdvertisingPart);
    		reg("form/Bargaining.js",Bargaining);
    		reg("form/BowlAdPop.js",BowlAdPop);
    		reg("form/SubscriptionBox.js",SubscriptionBox);
    		reg("form/BuildingCouponPop.js",BuildingCouponPop);
    		reg("form/BuildingCouponItem.js",BuildingCouponItem);
    		reg("form/CameraUI.js",CameraUI);
    		reg("form/ButtonChangeBg.js",ButtonChangeBg);
    		reg("form/CatArchives.js",CatArchives);
    		reg("form/RedPoint.js",RedPoint);
    		reg("form/RedPointType.js",RedPointType);
    		reg("form/CatNewCome.js",CatNewCome);
    		reg("form/CollectionTutorialPop.js",CollectionTutorialPop);
    		reg("form/CollectPop.js",CollectPop);
    		reg("form/CommonList.js",CommonList);
    		reg("form/CollectItem.js",CollectItem);
    		reg("form/SceneTopUI.js",SceneTopUI);
    		reg("form/ConfirmPop.js",ConfirmPop);
    		reg("form/EntrustPop.js",EntrustPop);
    		reg("form/FishingBaitsSelect.js",FishingBaitsSelect);
    		reg("form/CommonListH.js",CommonList$1);
    		reg("form/FishingBaitsItem.js",FishingBaitsItem);
    		reg("form/FishingCollectPop.js",FishingCollectPop);
    		reg("form/FishingCollectItem.js",FishingCollectItem);
    		reg("form/FishingDialog.js",FishingDialog);
    		reg("form/RewardItem.js",RewardItem);
    		reg("form/FishNewCome.js",FishNewCome);
    		reg("form/FoodLevelUpPop.js",FoodLevelUpPop);
    		reg("form/FortuneCatUI.js",FortuneCatUI);
    		reg("form/FriendSubscribe.js",FriendSubscribe);
    		reg("form/GlobalSharePop.js",GlobalSharePop);
    		reg("form/GuessGamePop.js",GuessGamePop);
    		reg("form/GuessCoinBox.js",GuessCoinBox);
    		reg("form/HomePetPop.js",HomePetPop);
    		reg("form/PetSelectItem.js",PetSelectItem);
    		reg("form/HousekeeperPop.js",HousekeeperPop);
    		reg("form/ImageLibPop.js",ImageLibPop);
    		reg("form/InfoPop.js",InfoPop);
    		reg("form/IntimacyUp.js",IntimacyUp);
    		reg("form/LevelToyInfoPop.js",LevelToyInfoPop);
    		reg("form/GapLoadingUI.js",GapLoadingUI);
    		reg("form/ReqLoadingUI.js",ReqLoadingUI);
    		reg("form/LuckydrawLevelReward.js",LuckydrawLevelReward);
    		reg("form/LuckydrawReward.js",LuckydrawReward);
    		reg("form/LuckBagDialog.js",LuckyBagDialog);
    		reg("form/LuckyDrawPop.js",LuckyDrawPop);
    		reg("form/MailListPop.js",MailListPop);
    		reg("form/MailItem.js",MailItem);
    		reg("form/MailPhotoShow.js",MailPhotoShow);
    		reg("form/MailPop.js",MailPop);
    		reg("form/MakeupPop.js",MakeupPop);
    		reg("form/MergePop.js",MergePop);
    		reg("form/MileStoneNewsPaper.js",MileStoneNewsPaper);
    		reg("form/MileStoneReward.js",MileStoneReward);
    		reg("form/MiniGameTalkPop.js",MiniGameTalkPop);
    		reg("form/MissionCatCD.js",MissionCatCD);
    		reg("mission/pop/MissionCompletePop.js",MissionCompletePop);
    		reg("mission/pop/MissionFailPop.js",MissionFailPop);
    		reg("mission/view/MissionGame.js",MissionGame);
    		reg("form/MissionLevelChange.js",MissionLevelChange);
    		reg("mission/view/MissionMain.js",MissionMain);
    		reg("mission/view/MissionItem.js",MissionItem);
    		reg("mission/pop/MissionRefreshPop.js",MissionRefreshPop);
    		reg("mission/view/MissionTeam.js",MissionTeam);
    		reg("mission/view/MissionCatBox.js",MissionCatBox);
    		reg("mission/view/MissionCatItem.js",MissionCatItem);
    		reg("form/MoraGamePop.js",MoraGamePop);
    		reg("form/MultiLoginDialog.js",MultiLoginDialog);
    		reg("core/component/ViewMid.js",ViewAlign);
    		reg("form/NauticalGetFood.js",NauticalTravelingPop);
    		reg("form/NauticalRewardPop.js",NauticalRewardPop);
    		reg("form/BuffBox.js",BuffBox);
    		reg("form/NauticalStartPop.js",NauticalStartPop);
    		reg("form/GoodsSelectItem.js",GoodsSelectItem);
    		reg("form/CatSelectItem.js",CatSelectItem);
    		reg("form/NauticalTravelingPop.js",NauticalTravelingPop$1);
    		reg("form/Order.js",Order);
    		reg("form/OrderDeliveryCatBox.js",OrderDeliveryCatBox);
    		reg("form/OrderDeliveringCatBox.js",OrderDeliveringCatBox);
    		reg("form/OrderPromoterBox.js",OrderPromoterBox);
    		reg("form/OrderAssignDialog.js",OrderAssignDialog);
    		reg("form/OrderBuy.js",OrderBuy);
    		reg("form/OrderChoose.js",OrderChoose);
    		reg("form/OrderChooseItem.js",OrderChooseItem);
    		reg("form/OrderLevelUp.js",OrderLevelUp);
    		reg("form/PayHelper.js",PayPop);
    		reg("form/PayPop.js",PayPop$1);
    		reg("form/PayCard.js",PayCard);
    		reg("form/PayItem.js",PayItem);
    		reg("form/PayRewards.js",PayRewards);
    		reg("form/PayRewardItem.js",PayRewardItem);
    		reg("form/PayTipsPop.js",PayTipsPop);
    		reg("form/PetStarUpPop.js",PetStarUpPop);
    		reg("form/PhotoAlbumDialog.js",PhotoAlbumDialog);
    		reg("form/UserPhoto.js",UserPhoto);
    		reg("form/SystemPhoto.js",SystemPhoto);
    		reg("form/PhotoTalkPop.js",PhotoTalkPop);
    		reg("form/PhotoWallDialog.js",PhotoWallDialog);
    		reg("form/PickUI.js",PickUI);
    		reg("form/PlacePop.js",PlacePop);
    		reg("form/PowerGamePop.js",PowerGamePop);
    		reg("form/ProjectLevelUpPop.js",ProjectLevelUpPop);
    		reg("form/SailDialog.js",SailDialog);
    		reg("form/SettingPop.js",SettingPop);
    		reg("form/SettingTalkPop.js",SettingTalkPop);
    		reg("form/ShopPop.js",ShopPop);
    		reg("form/ShopList.js",ShopList);
    		reg("form/ShopItem.js",ShopItem);
    		reg("form/SignPop.js",SignPop);
    		reg("form/StaffLevelUp.js",StaffLevelUp);
    		reg("form/ProgressBar.js",ProgressBar);
    		reg("form/StaffPop.js",StaffPop);
    		reg("form/StaffSkillBox.js",StaffSkillBox);
    		reg("form/SuggestPop.js",ShopPop$1);
    		reg("form/TaskMainTalkPop.js",TaskMainTalkPop);
    		reg("form/TaskPop.js",TaskPop);
    		reg("form/TaskItem.js",TaskItem);
    		reg("form/FyProgressBar.js",FyProgressBar);
    		reg("form/EntrustItem.js",EntrustItem);
    		reg("form/TipsDialog.js",TipsDialog);
    		reg("form/ToyUnlockPop.js",ToyUnlockPop);
    		reg("form/VideoShareUI.js",VideoShareUI);
    		reg("form/VisitorGiftPop.js",VisitorGiftPop);
    		reg("form/VisitorGiftItem.js",VisitorGiftItem);
    		reg("form/WxMessagePop.js",WxMessagePop);
    		reg("form/ZhuliPop.js",ZhuliPop);
    		reg("form/ZhuliPopAward.js",ZhuliPopAward);
    		reg("building/BuildingImageScale.js",BuildingImageScale);
    		reg("form/StaffSkillTip.js",StaffSkillTip);
    		reg("core/component/ViewAlignment.js",ViewAlignment);
    		reg("form/ArrowUI.js",ArrowUI);
    		reg("form/BottomUI.js",BottomUI);
    		reg("form/CollectGiftUI.js",CollectGiftUI);
    		reg("form/GiftList.js",GiftList);
    		reg("form/GiftsBox.js",GiftsBox);
    		reg("form/BuildingItem.js",BuildingItem);
    		reg("form/GetNewCat.js",GetNewCat);
    		reg("form/GiftItem.js",GiftItem);
    		reg("form/GoodsItem.js",GoodsItem);
    		reg("form/ImageDetailUI.js",ImageDetailUI);
    		reg("form/ImageDetailItemList.js",ImageDetailItemList);
    		reg("form/ImageDetailItem.js",ImageDetailItem);
    		reg("form/ImageItem.js",ImageItem);
    		reg("form/ImageLibUI.js",ImageLibUI);
    		reg("form/ImageTipsUI.js",ImageTipsUI);
    		reg("form/LeftBtnDrawPop.js",LeftBtnDrawPop);
    		reg("form/LeftUI.js",LeftUI);
    		reg("form/PetsSelectUI.js",PetsSelectUI);
    		reg("form/PetsSelectUIList.js",PetsSelectUIList);
    		reg("form/CatItem.js",CatItem);
    		reg("form/ProjectItem.js",ProjectItem);
    		reg("form/RightUI.js",RightUI);
    		reg("form/SailBtn.js",SailBtn);
    		reg("form/TaskMainBox.js",TaskMainBox);
    		reg("tips/TipsPop.js",TipsPop);
    		reg("form/TopUI.js",TopUI);
    		reg("form/ZhuliBaoxiang.js",ZhuliBaoxiang);
        }
    }
    GameConfig.width = 1827;
    GameConfig.height = 2436;
    GameConfig.scaleMode ="fixedheight";
    GameConfig.screenMode = "vertical";
    GameConfig.alignV = "middle";
    GameConfig.alignH = "center";
    GameConfig.startScene = "scene/CoffeeScene.scene";
    GameConfig.sceneRoot = "";
    GameConfig.debug = false;
    GameConfig.stat = false;
    GameConfig.physicsDebug = true;
    GameConfig.exportSceneToJson = true;

    GameConfig.init();

    class LoadingUI extends Laya.Panel {
        constructor() {
            super();
            this.animCat = null;
            this.bar = null;
            this.bar2 = null;
            this.bar3 = null;
            this.barSpr = null;
            this.max = 0;

            this.maxValue = 0;
            this.value = 0;
            this.onSet();
        }

        onSet(){
            this.width=Laya.stage.width;
            this.height=Laya.stage.height;

            let heightScale = Laya.Browser.height/2436;
            let realW = 1827*heightScale;

            this.UI=new Laya.Sprite();
            this.UI.x = (Laya.Browser.width/2 - realW/2)/heightScale;



            this.sp=new Laya.Sprite();
            this.sp.width=1827;
            this.sp.height=2436;
            this.sp.loadImage("loading/beijingse.png");

            // this.catBox=new Laya.Box();
            // this.catBox.width=1827;
            // this.catBox.height=2436;

            this.bg=new Laya.Sprite();
            this.bg.loadImage("loading/jindutiaodiban.png");
            this.bg.pos(474,1625);
            this.bg.width=872;
            this.bg.height=100;

            this.barSpr=new Laya.Sprite();
            this.barSpr.loadImage("loading/jindutiao.png");
            this.barSpr.pos(496,1635);
            this.barSpr.width=634;
            this.barSpr.height=78;

            // this.label=new Laya.Label;
            // this.label.text="点击正在钓鱼的猫猫可以去钓鱼哦(*^▽^*)";
            // this.label.fontSize=45;
            // this.label.color="#72584a";
            // this.label.pos(913,1842);
            // this.label.anchorX=0.5;
            // this.label.anchorY=0.5;

            let logo=new Laya.Image("loading/LOGO.png");
            logo.scaleX = logo.scaleY = 2;
            logo.x = 430;
            logo.y = 200;

            this.UI.addChild(this.sp);
            // this.UI.addChild(this.catBox);
            this.UI.addChild(logo);
            this.UI.addChild(this.bg);
            this.UI.addChild(this.barSpr);
            // this.UI.addChild(this.label);
            this.addChild(this.UI);

        }

        setStyle(style) {
            if(style == undefined || style == null) {
                style = 1;
            }
            this.style = style;
            if(this.style == 1) {
                //进度条
                this.initLoadingStyle1();
            } else {
                //其他
            }
        }

        // onAwake() {
        //     let starty = 120;
        //     let ySpace = (this.catBox.height - starty * 2) / 10;
        //     let xSpace = (this.catBox.width - 100) / 7;
        //     let startx = (this.catBox.width - 6 * xSpace - 100) / 2;
        //     this._cats = [];
        //     for (let i = 0; i < 11; i++) {
        //         let n = 8 - i % 2;
        //         for (let j = 0; j < n; j++) {
        //             let image = new Laya.Image("loading/xiaomaolian.png");
        //             image.width = 100;
        //             image.height = 96;
        //             image.pivotX = 50;
        //             image.pivotY = 48;
        //             image.x = 50 + startx * (i % 2) + xSpace * j;
        //             image.y = starty;
        //             this.catBox.addChild(image);
        //             this._cats.push(image);
        //             this._calcImage(image, xSpace, ySpace);
        //         }
        //         if(i==10) {
        //             for(let j = 0; j<n; j++) {
        //                 let image = new Laya.Image("loading/xiaomaolian.png");
        //                 image.width = 100;
        //                 image.height = 96;
        //                 image.pivotX = 50;
        //                 image.pivotY = 48;
        //                 image.x = 50 + startx * (i % 2) + xSpace * (j - 1/2);
        //                 image.y = starty + ySpace;
        //                 this.catBox.addChild(image);
        //                 this._cats.push(image);
        //                 this._calcImage(image, xSpace, ySpace);
        //             }
        //         }else if(i%2==0) {
        //             let image = new Laya.Image("loading/xiaomaolian.png");
        //             image.width = 100;
        //             image.height = 96;
        //             image.pivotX = 50;
        //             image.pivotY = 48;
        //             image.x = 50 + startx * (i % 2) - xSpace/2;
        //             image.y = starty + ySpace;
        //             this.catBox.addChild(image);
        //             this._cats.push(image);
        //             this._calcImage(image, xSpace, ySpace);
        //         }
        //         starty += ySpace;
        //     }

        //     Laya.timer.loop(Number(Const.get("LoadingTipsTime"))*1000, this, this.randomTip);
        //     this.randomTip();
        //     this._moveImage();
        // }

        // _calcImage(image, xSpace, ySpace) {
        //     image._stepX = xSpace/2;
        //     image._stepY = - ySpace;
        //     image._startX = image.x;
        //     image._startY = image.y;
        //     while(image._startX>=0 && image._startY <= this.catBox.height) {
        //         image._startX = image._startX - xSpace/2;
        //         image._startY = image._startY + ySpace;
        //     }

        //     image._endX = image.x;
        //     image._endY = image.y;
        //     while(image._endX<=this.catBox.width && image._endY>=0) {
        //         image._endX = image._endX + xSpace/2;
        //         image._endY = image._endY - ySpace;
        //     }
        // }

        // get imageMove() {
        //     return this._imageMove;
        // }

        // set imageMove(v) {
        //     let delta = v - this._imageMove;
        //     this._imageMove = v;
        //     for(const image of this._cats) {
        //         image.x = image.x + delta * image._stepX;
        //         image.y = image.y + delta * image._stepY;
        //         if(image.x>=image._endX || image.y<=image._endY) {
        //             image.x = image.x - image._endX + image._startX;
        //             image.y = image.y - image._endY + image._startY;
        //         }
        //     }

        // }

        // _moveImage() {
        //     this._imageMove = 0;
        //     let target = this.imageMove + 10;
        //     let duration = 50000;

        //     Laya.Tween.to(this,{imageMove:target}, duration ,null,Laya.Handler.create(this,onTween1));

        //     function onTween1()
        //     {
        //         let target = this.imageMove + 10;
        //         Laya.Tween.clearTween(onTween1);
        //         Laya.Tween.to(this,{imageMove:target},duration, null ,Laya.Handler.create(this,onTween2));
        //     }
        //     function onTween2()
        //     {
        //         let target = this.imageMove + 10;
        //         Laya.Tween.clearTween(onTween2);
        //         Laya.Tween.to(this, {imageMove:target} , duration, null ,Laya.Handler.create(this,onTween1));
        //     }
        // }

        onEnable() {
            this.closeEffect = null;
            this.popupEffect = null;
        }


        initLoadingStyle2() {
            // this.barSpr = this.getChildByName("bar");
            // let bg = this.getChildByName("bg");
            // bg.visible = false;
            // this.barSpr.visible = false;
            // this.animCat = new Laya.Skeleton();
            // this.animCat.load("loading/loading2.sk", Laya.Handler.create(this, function() {
            //     this.animCat.x = 1827/2;
            //     this.animCat.y = 2436/2;
            //     this.animCat.play("1", true);
            // }));
            // this.addChild(this.animCat);
        }

        initLoadingStyle1() {
            this.max = this.barSpr.width;
            this.animCat = new Laya.Skeleton();
            this.animCat.load("loading/loading1.sk", Laya.Handler.create(this, function() {
                this.animCat.x = 1827/2;
                this.animCat.y = 2436/2+100;
                this.animCat.play("1", true);
            }));
            this.UI.addChild(this.animCat);

            this.bar3 = new Laya.Skeleton();
            this.bar3.load("loading/loading1.sk", Laya.Handler.create(this, function() {
                this.bar3.x = 1827/2;
                this.bar3.y = 1772;
                this.bar3.scaleX=this.bar3.scaleY=0.85;
                this.bar3.play("4", true);
            }));
            this.UI.addChild(this.bar3);

            this.bar = new Laya.Skeleton();
            this.bar.load("loading/loading1.sk", Laya.Handler.create(this, function() {
                this.bar.x = 160;
                this.bar.y = 39;
                this.bar.play("2", true);
            }));
            this.barSpr.addChild(this.bar);

            this.bar2 = new Laya.Skeleton();
            this.bar2.load("loading/loading1.sk", Laya.Handler.create(this, function() {
                this.bar2.x = 25;
                this.bar2.y = 36;
                this.bar2.play("3", true);
            }));
            this.barSpr.addChild(this.bar2);
            this.setValue(0);
        }

        onDisable() {
            Laya.timer.clearAll(this);
        }

        setMaxValue(v) {
            this.maxValue = v;
        }

        setValue(v) {
            if(this.style != 1) {
                return;
            }
            if(v == 0) {
                v = 0.000001;
            }
            this.value = v;
            let a = this.value/this.maxValue;
            this.barSpr.width = this.max*a + 160;
            this.bar.x = this.barSpr.width ;
        }

        // setTip(str) {
        //     let label = this.label;
        //     label.text = str;
        // }

        // randomTip() {
        //     try {
        //         let tips = Const.getArray("LoadingTips");
        //         if(tips && tips.length>0) {
        //             let tip = tips[Utils.randomExt(tips.length)];
        //             if(tip) {
        //                 this.setTip(Strings.get(tip));
        //             }
        //         }
        //     }catch(err) {

        //     }
        // }
    }

    class PreGameLoader {

        constructor() {
            this.configs = null;
        }

        static getInstance() {
            return PreGameLoader.instance?PreGameLoader.instance:PreGameLoader.instance = new PreGameLoader();
        }

        reset() {
            this.configArray = ConfigMgr$1.getInstance().getConfigList();
            this.configSize = ConfigMgr$1.getInstance().getConfigCount();
        }

        start(handler) {
            this.reset();
            Laya.stage.removeChildByName("loadingUI");
            let loadingUI=new LoadingUI();
            loadingUI.name = "loadingUI";
            loadingUI.zOrder = 100;
            Laya.stage.addChild(loadingUI);
            // Laya.Dialog.close("ui/LoadingDlg.scene");
            // Laya.Dialog.open("ui/LoadingDlg.scene", false, null, Laya.Handler.create(this, function(ui) {
                this.loadingUI = loadingUI;
                this.loadingUI.setStyle(1);
                let loader = new GameLoaderTaskMgr();
                loader.addTaskInfo({type: "scenePath"});
                for (let i = 0; i < this.configSize; i++) {
                    let config = this.configArray[i];
                    console.log(config);
                    let weight = 1;
                    if(i<=5) {
                        weight = 4;
                    }
                    loader.addTaskInfo({type: "config", source1: config}, weight);
                }
                if(Laya.Browser.onMiniGame)
                    loader.addTaskInfo({type: "login"});

                // if (Laya.Browser.onMiniGame) { //减少本地调试的loading时间
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/denglong_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/denglong_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/denglong_3.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/denglong_4.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/ditan_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/ditan_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/ditan_3.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/ditan_4.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/guitai_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/guitai_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/guitai_3.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/letter_box_dialog.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maojia_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maojia_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maojia_3.png"});

                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo1_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo1_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo2_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo2_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo3_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo3_2.png"});

                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo4_1.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo4_2.png"});
                    // loader.addTaskInfo({type: "image", source1: "component_coffee/maowo5.png"});

                    // loader.addTaskInfo({type: "image", source1: "res/atlas/fishing_game/bait.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/fishing_game/fish.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/fishing_game/ui.png"});

                    // loader.addTaskInfo({type: "image", source1: "res/atlas/collect.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/common.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/cookbook.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/facility.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/menu.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/shopui.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/smallpop.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/task.png"});
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/task.png"});



                    // let i = 0;
                    // let dirTask;
                    // for(const info of loads) {
                    //     if(i%20==0) {
                    //         dirTask = {type: "dir", files:[info]};
                    //         loader.addTaskInfo(dirTask);
                    //     }else{
                    //         dirTask.files.push(info);
                    //     }
                    //     i++;
                    // }

                    // loader.addTaskInfo({type: "dir", files:[{"type": "image", "source1": "sc_coffee1/2lou.png"}, {"type": "image", "source1": "sc_coffee1/leftqiang.png"}, {"type": "image", "source1": "sc_coffee1/rightqiang.png"}, {"type": "image", "source1": "sc_coffee1/yinying.png"}, {"type": "image", "source1": "res/skin_coffee.png"}]})
                    // loader.addTaskInfo({type: "image", source1: "res/scene_coffee.atlas"});
                    // loader.addTaskInfo({type: "image", source1: "res/cookbook.png"});
                    // loader.addTaskInfo({type: "atlas", source1: "res/cookbook.atlas"});
                    // loader.addTaskInfo({type: "scene", source1: "scene/CoffeeScene.json"});
                    // loader.addTaskInfo({type: "scene", source1: "scene/BeachScene.json"});
                    // loader.addTaskInfo({type: "scene", source1: "scene/NauticalScene.json"});
                    // loader.addTaskInfo({type: "scene", source1: "scene/Homeland.json"});

                // }else{
                    // loader.addTaskInfo({type: "image", source1: "res/atlas/cookbook.png"});
                    // loader.addTaskInfo({type: "atlas", source1: "res/atlas/cookbook.atlas"});
                // }

                this.loadingUI.setMaxValue(loader.taskTotalWeight);
                loader.setHandler(Laya.Handler.create(this, function(taskIndexWeight, totalWeight, taskIndex, total) {
                    this.loadingUI.setValue(taskIndexWeight);
                    if(taskIndex==1 ||taskIndex==3 ||taskIndex==5 ||taskIndex==7 || taskIndex==9) {
                        EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_config_"+ +String(taskIndex) + "/" + String(total), {cost : (+ new Date())-this._loader_begin}]);
                    }
                }, null, false),Laya.Handler.create(this, function() {
                    if (handler != null) {
                        this.loadingUI = null;
                        // SceneMgr.getInstance().loadScene(Constants.CoffeeSceneId, Laya.Handler.create(this, function(scene1) {
                            if(Laya.Browser.onMiniGame) {
                                // CommonUIMgr.getInstance().showCommonUI(Constants.CoffeeSceneId, Laya.Handler.create(this, function() {
                                    // CommonUIMgr.getInstance().hideAll();
                                    SdkHelper.sendLoadingLogEnd();
                                    handler.run();
                                    Laya.stage.removeChildByName("loadingUI");
                                // }));
                            }else {
                                SdkHelper.sendLoadingLogEnd();
                                handler.run();
                                Laya.stage.removeChildByName("loadingUI");
                            }
                        // }));
                    }
                }));
                SdkHelper.sendLoadingLogStart();
                loader.start();
                EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_config_start"]);
            // }));
        }

    }

    class LoadingMaintain extends Laya.Panel {

        static getInstance() {
            return LoadingMaintain.instance?LoadingMaintain.instance:LoadingMaintain.instance = new LoadingMaintain();
        }

        onEnable(){
            this.width=Laya.stage.width;
            this.height=Laya.stage.height;
            this.bgColor="ffffff";

            this.failureCat = new Laya.Sprite();
    		this.addChild(this.failureCat);
            this.failureCat.loadImage("loading/failureCat.png");
            this.failureCat.scale(2,2);
            this.failureCat.pos(Laya.stage.width/2-300,Laya.stage.height/2-250);

            this.text1=new Laya.Text();
            this.text1.color = "#906241";
    		this.text1.font = "Impact";
            this.text1.fontSize = 70;
            this.text1.pos(Laya.stage.width/2-250,Laya.stage.height/2+200);
            this.text1.text="服务器维护中.  .  .";
            this.text1.text=Strings$1.get("tid#loading_1");
            this.addChild(this.text1);

        }
    }

    class LoadingTransition extends Laya.Panel {
        constructor() {
            super();
        }

        static getInstance() {
            return LoadingTransition.instance ? LoadingTransition.instance : LoadingTransition.instance = new LoadingTransition();
        }

        onEnable() {
            this.width = Laya.stage.width;
            this.height = Laya.stage.height;
            this.backGround = new Laya.Image();
            this.addChild(this.backGround);
            this.backGround.loadImage("loading/loadingBg.png");
            this.backGround.sizeGrid = "18,16,14,18";
            this.backGround.width = Laya.stage.width + 100;
            this.backGround.height = Laya.stage.height + 100;
            this.backGround.pos(-50, -50);

            this.catPaw = new Laya.Sprite();
            this.addChild(this.catPaw);
            this.catPaw.loadImage("loading/catPaw.png");
            this.catPaw.scale(2.6, 2.6);
            this.catPaw.pos(Laya.stage.width / 2 + 300, 672);

            this.text1 = new Laya.Text();
            this.text1.color = "#7b4c32";
            this.text1.font = "Impact";
            this.text1.fontSize = 80;
            this.text1.pos(Laya.stage.width / 2 - 400, 660);
            this.text1.text = "欢迎来到《猫咪小院》";
            this.text1.text=Strings$1.get("tid#loading_2");
            this.addChild(this.text1);

            this.text1.x = (this.width - this.text1.width - this.catPaw.width * this.catPaw.scaleX) / 2;
            this.catPaw.x = this.text1.x + this.text1.width;

            this.loadingImageSources = {
                "loadingImg1": "loading/loading_1.jpg",
                "loadingImg2": "jpg_sub/loading_2.jpg",
                "loadingImg3": "jpg_sub/loading_3.jpg",
                "loadingImg4": "jpg_sub/loading_4.jpg"
            };
            this.loadIngImage = new Laya.Image("loading/loading_1.jpg");
            this.loadIngImage.scale(2.5, 2.5);
            this.loadIngImage.anchorX = this.loadIngImage.anchorY = 0.5;
            this.loadIngImage.x = this.width / 2;
            this.loadIngImage.y = this.height / 2;
            this.addChild(this.loadIngImage);
            this.hasLoads = {
                "loading/loading_1.jpg": 1
            };

            this.text2 = new Laya.Text();
            this.text2.color = "#906241";
            this.text2.fontSize = 60;
            this.text2.y = 1663;
            this.text2.width = this.width - 200;
            this.text2.x = 100;
            this.text2.align = "center";
            this.text2.wordWrap = true;
            this.addChild(this.text2);

            this.textRandomArr = Object.values(JSON.parse(Const.get("LoadingTextRandom")));
            this.currentTextArr = this.textRandomArr[Utils.randomExt(this.textRandomArr.length)];
            this.currentTextIndex = 0;
            this.textTime = Number(Const.get("LoadingTextTime")) * 1000;
            let createDays = PlayerSystem.getCreateDays();
            let isNew = Laya.LocalStorage.getJSON("newPlayer");
            this.LoadingTextContentNew = [];
            if (isNew != 1) {
                let object = JSON.parse(Const.get("LoadingTextContentNew"));
                for (const key in object) {
                    const obj = object[key];
                    let texts = obj.text;
                    texts.forEach(element => {
                        this.LoadingTextContentNew.push({ i: obj.img, t: element });
                    });
                }
                this.wlecome();
                return;
            }
            for (const key in this.loadingImageSources) {
                const element = this.loadingImageSources[key];
                if (element != "loading/loading_1.jpg") {
                    if (Laya.MiniFileMgr && Laya.MiniFileMgr.getFileInfo(Laya.URL.formatURL(element))) {
                        this.hasLoads[element] = 1;
                    } else {
                        Laya.loader.load(element, Laya.Handler.create(this, function (res) {
                            // console.debug(element, "    ", "load  ", res);
                            this.hasLoads[element] = 1;
                        }));
                    }
                }
            }
            if (createDays != null) {
                this.text2.text = Strings$1.get(Const.get("LoadingTextInitial"), [createDays]);
                Laya.timer.once(this.textTime, this, this.randomWelcome);
            } else {
                this.randomWelcome();
            }
        }

        randomWelcome() {
            this.text2.text = Strings$1.get(this.currentTextArr[this.currentTextIndex]);
            this.currentTextIndex++;
            if (this.currentTextIndex % 2 == 0) {
                let skins = Object.keys(this.hasLoads);
                let skin = skins[Utils.randomExt(skins.length)];
                this.loadIngImage.skin = skin;
            }
            if (this.currentTextIndex >= this.currentTextArr.length) {
                this.currentTextIndex = 0;
                this.currentTextArr = this.textRandomArr[Utils.randomExt(this.textRandomArr.length)];
            }
            Laya.timer.clear(this, this.randomWelcome);
            Laya.timer.once(this.textTime, this, this.randomWelcome);
        }

        wlecome() {
            let obj = this.LoadingTextContentNew[this.currentTextIndex];
            if (obj == null) {
                Laya.timer.clear(this, this.wlecome);
                Laya.timer.once(this.textTime, this, this.wlecome);
                return;
            }
            this.text2.text = Strings$1.get(obj.t);
            this.showNextPic();
            this.loadNextPic();
            this.currentTextIndex++;
            if (this.currentTextIndex >= this.LoadingTextContentNew.length) {
                this.currentTextIndex = 0;
            }
            Laya.timer.clear(this, this.wlecome);
            Laya.timer.once(this.textTime, this, this.wlecome);
        }

        showNextPic() {
            let index = this.currentTextIndex;
            let obj = this.LoadingTextContentNew[index];
            while (this.hasLoads[this.loadingImageSources[obj.i]] == null) {
                index += 1;
                if (index >= this.LoadingTextContentNew.length) {
                    index = 0;
                }
                obj = this.LoadingTextContentNew[index];
            }
            this.loadIngImage.skin = this.loadingImageSources[obj.i];
        }

        loadNextPic() {
            let index = this.currentTextIndex;
            let obj = this.LoadingTextContentNew[index];
            let curImg = obj.i;
            while (obj.i == curImg) {
                index += 1;
                if (index >= this.LoadingTextContentNew.length) {
                    index = 0;
                }
                obj = this.LoadingTextContentNew[index];
                if (obj.i != curImg && this.hasLoads[this.loadingImageSources[obj.i]] == null) {
                    let s = this.loadingImageSources[obj.i];
                    Laya.loader.load(s, Laya.Handler.create(this, function (res) {
                        // console.debug(s, "    ", "load  ", res);
                        this.hasLoads[s] = 1;
                    }));
                    break;
                }
            }
        }

        onDisable() {
            Laya.timer.clearAll(this);
            Laya.LocalStorage.setJSON('newPlayer', 1);
        }
    }

    var ActionPoint = [["Id","ActionsSetting"],["act_rest","[{\"name\":\"lick1\"},{\"name\":\"rest1\"},{\"name\":\"rest2\"}]"],["act_rest_flip","[{\"name\":\"rest1\",\"flip\":\"true\"},{\"name\":\"rest2\",\"flip\":\"true\"}]"],["act_rest_doble","[{\"name\":\"lick1\"},{\"name\":\"rest1\"},{\"name\":\"rest2\"},{\"name\":\"lick1\",\"flip\":\"true\"},{\"name\":\"rest1\",\"flip\":\"true\"},{\"name\":\"rest2\",\"flip\":\"true\"}]"],["act_shatanche","[{\"name\":\"lick1\",\"flip\":\"true\"},{\"name\":\"rest1\",\"flip\":\"true\"},{\"name\":\"rest2\",\"flip\":\"true\"}]"],["act_eat","[{\"name\":\"lick1\",\"flip\":\"true\"}]"],["act_maowo","[{\"name\":\"rest2\",\"flip\":\"true\"}]"],["act_maopajia3","[{\"name\":\"rest4\",\"flip\":\"true\"}]"],["act_fish","[{\"name\":\"fish1\",\"flip\":\"true\"}]"],["act_shadui","[{\"name\":\"rest3\"}]"],["act_shadiao","[{\"name\":\"rest5\"}]"],["act_shadiao_flip","[{\"name\":\"rest5\",\"flip\":\"true\"}]"],["act_paiqiu","[{\"name\":\"play2-1\",\"hide\":\"true\"}]"],["act_paiqiu_doble","[{\"name\":\"play2-1\",\"hide\":\"true\"},{\"name\":\"play2-1\",\"hide\":\"true\",\"flip\":\"true\"}]"],["act_maoxianqiu","[{\"name\":\"play2-1\",\"hide\":\"true\"}]"],["act_maoxianqiu_doble","[{\"name\":\"play2-1\",\"hide\":\"true\"},{\"name\":\"play2-1\",\"hide\":\"true\",\"flip\":\"true\"}]"],["act_lingdang","[{\"name\":\"play2-2\",\"hide\":\"true\"}]"],["act_lingdang_doble","[{\"name\":\"play2-2\",\"hide\":\"true\"},{\"name\":\"play2-2\",\"hide\":\"true\",\"flip\":\"true\"}]"],["act_xigua","[{\"name\":\"play2-2\",\"hide\":\"true\"}]"],["act_xigua_doble","[{\"name\":\"play2-2\",\"hide\":\"true\"},{\"name\":\"play2-2\",\"hide\":\"true\",\"flip\":\"true\"}]"],["act_maozhua","[{\"name\":\"play1\"}]"]];

    var Barrage = [["Id","Desc","Picture","Sort"],["Barrage1","tid#Barrage1",null,1],["Barrage2","tid#Barrage2",null,1],["Barrage3","tid#Barrage3",null,2],["Barrage4","tid#Barrage4",null,2],["Barrage5","tid#Barrage5",null,1],["Barrage6","tid#Barrage6",null,2],["Barrage7","tid#Barrage7",null,1],["Barrage8","tid#Barrage8",null,1],["Barrage9","tid#Barrage9",null,1],["Danmaku1","tid#Barrage10",null,1],["Danmaku2","tid#Barrage11",null,1],["Danmaku3","tid#Barrage12",null,1],["Danmaku4","tid#Barrage13",null,1],["Danmaku5","tid#Barrage14",null,1],["Danmaku6","tid#Barrage15",null,1],["Danmaku7","tid#Barrage16",null,1]];

    var BarrageGroup = [["Id","Group","Star"],["BarrageGroup1",["Barrage8","Barrage9"],5],["BarrageGroup2",["Barrage8","Barrage9","Danmaku6"],5],["BarrageGroup3",["Barrage9","Danmaku1","Danmaku6"],5],["BarrageGroup4",["Barrage1","Barrage4","Barrage5"],0],["BarrageGroup5",["Barrage2","Barrage3","Barrage9"],0],["BarrageGroup6",["Barrage6","Barrage7","Barrage5"],0],["BarrageGroup7",["Barrage6","Barrage7"],0],["BarrageGroup8",["Danmaku6","Danmaku5","Danmaku3"],0],["BarrageGroup9",["Danmaku2","Danmaku7","Danmaku4"],0]];

    var BuildingBase = [["Id","ListName","BaseType","FixedBuildings","BaseSize","Pos","PosSelection","ZOrder","PathPointIndex","Order","BelongScene"],["101","tid#buildingBaseName_101",1,["1000","1001","1002","1003","1004","1005","1006","1007"],2,["575","500"],["310","330"],10000,20,13,1],["102","tid#buildingBaseName_102",1,["1010","1011","1012","1013","1014","1015","1016","1017"],2,["535","1680"],["205","1610"],0,1,14,1],["103","tid#buildingBaseName_103",1,["1030","1031","1032","1033","1034","1035","1036","1037"],1,["500","900"],["361","948"],0,11,5,1],["104","tid#buildingBaseName_104",1,["1020","1021","1022","1023","1024","1025","1026","1027"],1,["1383","134"],["1170","200"],10000,21,6,1],["105","tid#buildingBaseName_105",1,["1040","1041","1042","1043","1044","1045","1046","1047"],1,["715","945"],["585","840"],-100,12,9,1],["106","tid#buildingBaseName_106",1,["1050","1051","1052","1053","1054","1055","1056","1057"],1,["1370","695"],["1225","590"],10000,17,10,1],["107","tid#buildingBaseName_107",1,["1060","1061","1062","1063","1064","1065","1066","1067"],1,["1075","1310"],["905","1255"],0,7,2,1],["108","tid#buildingBaseName_108",1,["1070","1071","1072","1073","1074","1075","1076","1077"],1,["1220","1410"],["1050","1355"],0,6,3,1],["109","tid#buildingBaseName_109",1,["1080","1081","1082","1083","1084","1085","1086","1087"],1,["1400","1480"],["1230","1425"],0,5,4,1],["110","tid#buildingBaseName_110",1,["1090","1091","1092","1093","1094","1095","1096","1097"],2,["1305","1285"],["1035","1160"],0,6,1,1],["111","tid#buildingBaseName_111",1,["1100","1101","1102","1103","1104","1105","1106","1107"],1,["580","1280"],["441","1212"],-100,9,8,1],["112","tid#buildingBaseName_112",1,["1110","1111","1112","1113","1114","1115","1116","1117"],2,["334","1229"],["85","1130"],100,9,12,1],["113","tid#buildingBaseName_113",1,["1120","1121","1122","1123","1124","1125","1126","1127"],1,["1412","1070"],["1260","1024"],1000,12,11,1],["114","tid#buildingBaseName_114",1,["1130","1131","1132","1133","1134","1135","1136","1137"],1,["1307","1937"],["1120","1900"],25200,2,7,1],["115","tid#buildingBaseName_115",1,["1140","1141","1142","1143","1144","1145","1146","1147"],1,["1050","1795"],["898","1715"],-100,2,15,1],["116","tid#buildingBaseName_116",1,["1150","1151","1152","1153","1154","1155","1156","1157"],2,["545","1745"],["280","1500"],-200,2,16,1],["117","tid#buildingBaseName_117",1,["1160","1161","1162","1163","1164","1165","1166","1167"],2,["1605","1695"],["1055","1615"],50,2,17,1],["118","tid#buildingBaseName_118",1,["1170","1171","1172","1173","1174","1175","1176","1177"],1,["810","2220"],["658","2000"],-770,2,18,1],["119","tid#buildingBaseName_119",1,["1180","1181","1182","1183","1184","1185","1186","1187"],1,["625","120"],["465","40"],1000,2,19,1],["120","tid#buildingBaseName_120",1,["1190","1191","1192","1193","1194","1195","1196","1197"],1,["1383","55"],["1231","5"],1000,2,20,1],["201","tid#buildingBaseName_201",2,["2010","2011","2012","2013","2014","2015","2016","2017"],2,["1220","297"],["933","102"],1,1,7,2],["202","tid#buildingBaseName_202",2,["2020","2021","2022","2023","2024","2025","2026","2027"],2,["390","1642"],["190","1822"],10,1,4,2],["203","tid#buildingBaseName_203",2,["2030","2031","2032","2033","2034","2035","2036","2037"],2,["573","722"],["221","642"],3,1,8,2],["204","tid#buildingBaseName_204",2,[],1,["893","2294"],["741","2214"],13,1,6,2],["205","tid#buildingBaseName_205",2,["2050","2051","2052","2053","2054","2055","2056","2057"],1,["958","2123"],["806","2043"],12,1,6,2],["206","tid#buildingBaseName_206",2,["2050","2051","2052","2053","2054","2055","2056","2057"],1,["1208","1967"],["1056","1887"],11,1,6,2],["207","tid#buildingBaseName_207",2,["2060","2061","2062","2063","2080","2081","2082","2083","2084","2085","2086","2087","2110","2111","2112","2113","2114","2115","2116","2117"],1,["653","1467"],["501","1387"],7,1,1,2],["209","tid#buildingBaseName_209",2,["2070","2071","2072","2073","2074","2075","2076","2077"],1,["1094","1177"],["942","1097"],5,1,5,2],["210","tid#buildingBaseName_210",2,["2060","2061","2062","2063","2080","2081","2082","2083","2084","2085","2086","2087","2110","2111","2112","2113","2114","2115","2116","2117"],1,["914","1659"],["762","1579"],8,1,2,2],["211","tid#buildingBaseName_211",2,["2060","2061","2062","2063","2080","2081","2082","2083","2084","2085","2086","2087","2110","2111","2112","2113","2114","2115","2116","2117"],1,["573","1693"],["421","1613"],9,1,2,2],["212","tid#buildingBaseName_212",2,["2060","2061","2062","2063","2100","2101","2102","2103","2104","2105","2106","2107"],1,["1021","1299"],["869","1219"],6,1,3,2],["213","tid#buildingBaseName_213",2,["2100","2101","2102","2103","2104","2105","2106","2107"],1,["1275","1157"],["1123","1077"],4,1,3,2],["214","tid#buildingBaseName_214",2,[],1,["166","347"],["14","267"],2,1,99,2],["215","tid#buildingBaseName_215",2,["2120","2121","2122","2123","2124","2125","2126","2127"],1,["693","1187"],["531","1097"],5,1,9,2],["301","tid#buildingBaseName_301",1,["3000","3001","3002"],2,["904","856"],["722","776"],1,1,29,3],["302","tid#buildingBaseName_302",1,["3010","3011","3012"],1,["1194","1672"],["1042","1592"],1000,1,28,3],["303","tid#buildingBaseName_303",1,[],2,["500","1400"],["348","1320"],1,1,30,4],["304","tid#buildingBaseName_304",1,["3020","3021","3022","3023","3024","3025","3026","3027"],2,["498","1683"],["450","1492"],1,1,31,3],["305","tid#buildingBaseName_305",1,["3030","3031","3032","3033","3034","3035","3036","3037"],1,["1250","650"],["1090","670"],1,1,32,3]];

    var CatToys = [["Id","Type","Name","Desc","Icon","SpecialFunctionType","LvUpCost","SpecialValue","FixedReward","FunctionDesc","Quality","ProduceDesc","Display"],["1","TOY","tid#toyName_1","tid#toyDesc_1","catToys/toy1_1.png",null,[],null,null,null,null,"tid#UI_211",1],["2","TOY","tid#toyName_2","tid#toyDesc_2","catToys/toy1_2.png",null,[],null,null,null,null,"tid#UI_211",1],["3","TOY","tid#toyName_3","tid#toyDesc_3","catToys/toy1_3.png",null,[],null,null,null,null,"tid#UI_211",1],["4","TOY","tid#toyName_4","tid#toyDesc_4","catToys/toy1_4.png",null,[],null,null,null,null,"tid#UI_211",1],["5","TOY","tid#toyName_5","tid#toyDesc_5","catToys/toy2_1.png",null,[],null,null,null,null,"tid#UI_211",1],["6","TOY","tid#toyName_6","tid#toyDesc_6","catToys/toy2_2.png",null,[],null,null,null,null,"tid#UI_211",1],["7","TOY","tid#toyName_7","tid#toyDesc_7","catToys/toy2_3.png",null,[],null,null,null,null,"tid#UI_211",1],["8","TOY","tid#toyName_8","tid#toyDesc_8","catToys/toy2_4.png",null,[],null,null,null,null,"tid#UI_211",1],["9","TOY","tid#toyName_9","tid#toyDesc_9","catToys/toy3_1.png",null,[],null,null,null,null,"tid#UI_211",1],["10","TOY","tid#toyName_10","tid#toyDesc_10","catToys/toy3_2.png",null,[],null,null,null,null,"tid#UI_211",1],["11","TOY","tid#toyName_11","tid#toyDesc_11","catToys/toy3_3.png",null,[],null,null,null,null,"tid#UI_211",1],["12","TOY","tid#toyName_12","tid#toyDesc_12","catToys/toy3_4.png",null,[],null,null,null,null,"tid#UI_211",1],["13","TOY","tid#toyName_13","tid#toyDesc_13","catToys/toy2_5.png",null,[],null,null,null,null,"tid#UI_211",1],["14","TOY","tid#toyName_14","tid#toyDesc_14","catToys/toy3_5.png",null,[],null,null,null,null,"tid#UI_211",1],["10001","ITEM","tid#ItemName_10001","tid#ItemDesc_10001","catToys/item_10001.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10002","ITEM","tid#ItemName_10002","tid#ItemDesc_10002","catToys/item_10004.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10003","ITEM","tid#ItemName_10003","tid#ItemDesc_10003","catToys/item_10008.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10004","ITEM","tid#ItemName_10004","tid#ItemDesc_10004","catToys/item_10012.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10005","ITEM","tid#ItemName_10005","tid#ItemDesc_10005","catToys/item_10019.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10006","ITEM","tid#ItemName_10006","tid#ItemDesc_10006","catToys/item_10027.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10007","ITEM","tid#ItemName_10007","tid#ItemDesc_10007","catToys/item_10033.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10008","ITEM","tid#ItemName_10008","tid#ItemDesc_10008","catToys/item_10036.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10009","ITEM","tid#ItemName_10009","tid#ItemDesc_10009","catToys/item_10041.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10010","ITEM","tid#ItemName_10010","tid#ItemDesc_10010","catToys/item_10066.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10011","ITEM","tid#ItemName_10011","tid#ItemDesc_10011","catToys/item_10045.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10012","ITEM","tid#ItemName_10012","tid#ItemDesc_10012","catToys/item_10070.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10013","ITEM","tid#ItemName_10013","tid#ItemDesc_10013","catToys/item_10050.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10014","ITEM","tid#ItemName_10014","tid#ItemDesc_10014","catToys/item_10054.png",11,[],"{\"Rate\":[0.01]}",null,"tid#ItemFDesc_11",1,"tid#UI_211",1],["10015","ITEM","tid#ItemName_10015","tid#ItemDesc_10015","catToys/item_10011.png",7,[],"{\"Rate\":[300]}",null,"tid#ItemFDesc_7",1,"tid#UI_211",1],["10016","ITEM","tid#ItemName_10016","tid#ItemDesc_10016","catToys/item_10023.png",7,[],"{\"Rate\":[300]}",null,"tid#ItemFDesc_7",1,"tid#UI_211",1],["10017","ITEM","tid#ItemName_10017","tid#ItemDesc_10017","catToys/item_10035.png",7,[],"{\"Rate\":[300]}",null,"tid#ItemFDesc_7",1,"tid#UI_211",1],["10018","ITEM","tid#ItemName_10018","tid#ItemDesc_10018","catToys/item_10067.png",7,[],"{\"Rate\":[300]}",null,"tid#ItemFDesc_7",1,"tid#UI_211",1],["10019","ITEM","tid#ItemName_10019","tid#ItemDesc_10019","catToys/item_10009.png",9,[],"{\"Rate\":[216]}",null,"tid#ItemFDesc_9",1,"tid#UI_211",1],["10020","ITEM","tid#ItemName_10020","tid#ItemDesc_10020","catToys/item_10021.png",9,[],"{\"Rate\":[216]}",null,"tid#ItemFDesc_9",1,"tid#UI_211",1],["10021","ITEM","tid#ItemName_10021","tid#ItemDesc_10021","catToys/item_10032.png",9,[],"{\"Rate\":[216]}",null,"tid#ItemFDesc_9",1,"tid#UI_211",1],["10022","ITEM","tid#ItemName_10022","tid#ItemDesc_10022","catToys/item_10040.png",9,[],"{\"Rate\":[216]}",null,"tid#ItemFDesc_9",1,"tid#UI_211",1],["10023","ITEM","tid#ItemName_10023","tid#ItemDesc_10023","catToys/item_10047.png",9,[],"{\"Rate\":[216]}",null,"tid#ItemFDesc_9",1,"tid#UI_211",1],["10024","ITEM","tid#ItemName_10024","tid#ItemDesc_10024","catToys/item_10057.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10025","ITEM","tid#ItemName_10025","tid#ItemDesc_10025","catToys/item_10061.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10026","ITEM","tid#ItemName_10026","tid#ItemDesc_10026","catToys/item_10002.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10027","ITEM","tid#ItemName_10027","tid#ItemDesc_10027","catToys/item_10003.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10028","ITEM","tid#ItemName_10028","tid#ItemDesc_10028","catToys/item_10005.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10029","ITEM","tid#ItemName_10029","tid#ItemDesc_10029","catToys/item_10007.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10030","ITEM","tid#ItemName_10030","tid#ItemDesc_10030","catToys/item_10010.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10031","ITEM","tid#ItemName_10031","tid#ItemDesc_10031","catToys/item_10013.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10032","ITEM","tid#ItemName_10032","tid#ItemDesc_10032","catToys/item_10015.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10033","ITEM","tid#ItemName_10033","tid#ItemDesc_10033","catToys/item_10018.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10034","ITEM","tid#ItemName_10034","tid#ItemDesc_10034","catToys/item_10020.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10035","ITEM","tid#ItemName_10035","tid#ItemDesc_10035","catToys/item_10022.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10036","ITEM","tid#ItemName_10036","tid#ItemDesc_10036","catToys/item_10025.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10037","ITEM","tid#ItemName_10037","tid#ItemDesc_10037","catToys/item_10028.png",11,[],"{\"Rate\":[0.02]}",null,"tid#ItemFDesc_11",2,"tid#UI_211",1],["10038","ITEM","tid#ItemName_10038","tid#ItemDesc_10038","catToys/item_10052.png",7,[],"{\"Rate\":[480]}",null,"tid#ItemFDesc_7",2,"tid#UI_211",1],["10039","ITEM","tid#ItemName_10039","tid#ItemDesc_10039","catToys/item_10071.png",7,[],"{\"Rate\":[480]}",null,"tid#ItemFDesc_7",2,"tid#UI_211",1],["10040","ITEM","tid#ItemName_10040","tid#ItemDesc_10040","catToys/item_10062.png",7,[],"{\"Rate\":[480]}",null,"tid#ItemFDesc_7",2,"tid#UI_211",1],["10041","ITEM","tid#ItemName_10041","tid#ItemDesc_10041","catToys/item_10051.png",7,[],"{\"Rate\":[480]}",null,"tid#ItemFDesc_7",2,"tid#UI_211",1],["10042","ITEM","tid#ItemName_10042","tid#ItemDesc_10042","catToys/item_10069.png",9,[],"{\"Rate\":[324]}",null,"tid#ItemFDesc_9",2,"tid#UI_211",1],["10043","ITEM","tid#ItemName_10043","tid#ItemDesc_10043","catToys/item_10059.png",9,[],"{\"Rate\":[324]}",null,"tid#ItemFDesc_9",2,"tid#UI_211",1],["10044","ITEM","tid#ItemName_10044","tid#ItemDesc_10044","catToys/item_10064.png",9,[],"{\"Rate\":[324]}",null,"tid#ItemFDesc_9",2,"tid#UI_211",1],["10045","ITEM","tid#ItemName_10045","tid#ItemDesc_10045","catToys/item_10063.png",9,[],"{\"Rate\":[324]}",null,"tid#ItemFDesc_9",2,"tid#UI_211",1],["10046","ITEM","tid#ItemName_10046","tid#ItemDesc_10046","catToys/item_10065.png",9,[],"{\"Rate\":[324]}",null,"tid#ItemFDesc_9",2,"tid#UI_211",1],["10047","ITEM","tid#ItemName_10047","tid#ItemDesc_10047","catToys/item_10029.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10048","ITEM","tid#ItemName_10048","tid#ItemDesc_10048","catToys/item_10031.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10049","ITEM","tid#ItemName_10049","tid#ItemDesc_10049","catToys/item_10034.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10050","ITEM","tid#ItemName_10050","tid#ItemDesc_10050","catToys/item_10037.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10051","ITEM","tid#ItemName_10051","tid#ItemDesc_10051","catToys/item_10039.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10052","ITEM","tid#ItemName_10052","tid#ItemDesc_10052","catToys/item_10042.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10053","ITEM","tid#ItemName_10053","tid#ItemDesc_10053","catToys/item_10043.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10054","ITEM","tid#ItemName_10054","tid#ItemDesc_10054","catToys/item_10044.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10055","ITEM","tid#ItemName_10055","tid#ItemDesc_10055","catToys/item_10046.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10056","ITEM","tid#ItemName_10056","tid#ItemDesc_10056","catToys/item_10048.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10057","ITEM","tid#ItemName_10057","tid#ItemDesc_10057","catToys/item_10049.png",11,[],"{\"Rate\":[0.03]}",null,"tid#ItemFDesc_11",3,"tid#UI_211",1],["10058","ITEM","tid#ItemName_10058","tid#ItemDesc_10058","catToys/item_10053.png",7,[],"{\"Rate\":[600]}",null,"tid#ItemFDesc_7",3,"tid#UI_211",1],["10059","ITEM","tid#ItemName_10059","tid#ItemDesc_10059","catToys/item_10055.png",7,[],"{\"Rate\":[600]}",null,"tid#ItemFDesc_7",3,"tid#UI_211",1],["10060","ITEM","tid#ItemName_10060","tid#ItemDesc_10060","catToys/item_10058.png",7,[],"{\"Rate\":[600]}",null,"tid#ItemFDesc_7",3,"tid#UI_211",1],["10061","ITEM","tid#ItemName_10061","tid#ItemDesc_10061","catToys/item_10060.png",7,[],"{\"Rate\":[600]}",null,"tid#ItemFDesc_7",3,"tid#UI_211",1],["10062","ITEM","tid#ItemName_10062","tid#ItemDesc_10062","catToys/item_10073.png",9,[],"{\"Rate\":[540]}",null,"tid#ItemFDesc_9",3,"tid#UI_211",1],["10063","ITEM","tid#ItemName_10063","tid#ItemDesc_10063","catToys/item_10074.png",9,[],"{\"Rate\":[540]}",null,"tid#ItemFDesc_9",3,"tid#UI_211",1],["10064","ITEM","tid#ItemName_10064","tid#ItemDesc_10064","catToys/item_10075.png",9,[],"{\"Rate\":[540]}",null,"tid#ItemFDesc_9",3,"tid#UI_211",1],["10065","ITEM","tid#ItemName_10065","tid#ItemDesc_10065","catToys/item_10076.png",9,[],"{\"Rate\":[540]}",null,"tid#ItemFDesc_9",3,"tid#UI_211",1],["10066","ITEM","tid#ItemName_10066","tid#ItemDesc_10066","catToys/item_10077.png",9,[],"{\"Rate\":[540]}",null,"tid#ItemFDesc_9",3,"tid#UI_211",1],["10067","ITEM","tid#ItemName_10067","tid#ItemDesc_10067","catToys/item_10006.png",4,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_4",3,"tid#UI_211",1],["10068","ITEM","tid#ItemName_10068","tid#ItemDesc_10068","catToys/item_10014.png",4,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_4",3,"tid#UI_211",1],["10069","ITEM","tid#ItemName_10069","tid#ItemDesc_10069","catToys/item_10017.png",4,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_4",3,"tid#UI_211",1],["10070","ITEM","tid#ItemName_10070","tid#ItemDesc_10070","catToys/item_10024.png",4,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_4",3,"tid#UI_211",1],["10071","ITEM","tid#ItemName_10071","tid#ItemDesc_10071","catToys/item_10072.png",4,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_4",3,"tid#UI_211",1],["10072","ITEM","tid#ItemName_10072","tid#ItemDesc_10072","catToys/item_10016.png",5,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_5",3,"tid#UI_211",1],["10073","ITEM","tid#ItemName_10073","tid#ItemDesc_10073","catToys/item_10026.png",5,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_5",3,"tid#UI_211",1],["10074","ITEM","tid#ItemName_10074","tid#ItemDesc_10074","catToys/item_10038.png",5,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_5",3,"tid#UI_211",1],["10075","ITEM","tid#ItemName_10075","tid#ItemDesc_10075","catToys/item_10030.png",8,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_8",3,"tid#UI_211",1],["10076","ITEM","tid#ItemName_10076","tid#ItemDesc_10076","catToys/item_10068.png",8,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_8",3,"tid#UI_211",1],["10077","ITEM","tid#ItemName_10077","tid#ItemDesc_10077","catToys/item_10056.png",8,[],"{\"Rate\":[1]}",null,"tid#ItemFDesc_8",3,"tid#UI_211",1]];

    var Entrustment = [["Id","Name","Desc","ConditionDesc","Condition","Reward","AcceptTalk","FinishTalk","WXMessage"],["Entrustment_1","tid#EntrustmentName_1","tid#EntrustmentDesc_1","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":0,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":10000}}",["tid#EntrustmentATalk1_1","tid#EntrustmentATalk1_2"],["tid#EntrustmentFTalk1_1","tid#EntrustmentFTalk1_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_2","tid#EntrustmentName_2","tid#EntrustmentDesc_2","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":80,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":10000}}",["tid#EntrustmentATalk2_1","tid#EntrustmentATalk2_2"],["tid#EntrustmentFTalk2_1","tid#EntrustmentFTalk2_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_3","tid#EntrustmentName_3","tid#EntrustmentDesc_3","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":80,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":12000}}",["tid#EntrustmentATalk3_1","tid#EntrustmentATalk3_2"],["tid#EntrustmentFTalk3_1","tid#EntrustmentFTalk3_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_4","tid#EntrustmentName_4","tid#EntrustmentDesc_4","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":90,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":12000}}",["tid#EntrustmentATalk4_1","tid#EntrustmentATalk4_2"],["tid#EntrustmentFTalk4_1","tid#EntrustmentFTalk4_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_5","tid#EntrustmentName_1","tid#EntrustmentDesc_1","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":100,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":15000}}",["tid#EntrustmentATalk1_1","tid#EntrustmentATalk1_2"],["tid#EntrustmentFTalk1_1","tid#EntrustmentFTalk1_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_6","tid#EntrustmentName_2","tid#EntrustmentDesc_2","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":110,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":15000}}",["tid#EntrustmentATalk2_1","tid#EntrustmentATalk2_2"],["tid#EntrustmentFTalk2_1","tid#EntrustmentFTalk2_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_7","tid#EntrustmentName_3","tid#EntrustmentDesc_3","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":120,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":20000}}",["tid#EntrustmentATalk3_1","tid#EntrustmentATalk3_2"],["tid#EntrustmentFTalk3_1","tid#EntrustmentFTalk3_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]],["Entrustment_8","tid#EntrustmentName_4","tid#EntrustmentDesc_4","tid#Entrustment_CDesc_1","[{\"TIME\":64800}]","{\"RewardRate\":130,\"Reward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":20000}}",["tid#EntrustmentATalk4_1","tid#EntrustmentATalk4_2"],["tid#EntrustmentFTalk4_1","tid#EntrustmentFTalk4_2"],["tid#EntrustmentWXMessage1_1","tid#EntrustmentWXMessage1_2"]]];

    var EntrustmentList = [["Id","List","ConditionType","ConditionValue","ListOrder"],["EntrustmentList_1",["Entrustment_1"],"CATS_VISIT","[[35,40]]",1],["EntrustmentList_2",["Entrustment_2"],"CATS_VISIT","[[35,40]]",2],["EntrustmentList_3",["Entrustment_3"],"CATS_VISIT","[[35,40]]",3],["EntrustmentList_4",["Entrustment_4"],"CATS_VISIT","[[40,50]]",4],["EntrustmentList_5",["Entrustment_5"],"CATS_VISIT","[[40,50]]",5],["EntrustmentList_6",["Entrustment_6"],"CATS_VISIT","[[40,50]]",6],["EntrustmentList_7",["Entrustment_7"],"CATS_VISIT","[[40,50]]",7],["EntrustmentList_8",["Entrustment_8"],"CATS_VISIT","[[40,50]]",8],["EntrustmentList_9",["Entrustment_5","Entrustment_6","Entrustment_7","Entrustment_8"],"CATS_VISIT","[[40,50]]",9]];

    var FishingBaits = [["Id","name","introduce","StarLimit","img","imgShadow","normal","special","speed","hitZoneID","Level","Order","Quality","ProduceDesc"],["1","tid#fishingBaitsName1","tid#fishingBaitsDesc1",0,"fishing_game/bait/bait1.png","fishing_game/baitShadow/Shadow_bait1.png",["1000:1","800:2","100:3"],["100:3","300:4"],3,"bait1",1,1,"1",""],["2","tid#fishingBaitsName2","tid#fishingBaitsDesc2",400,"fishing_game/bait/bait2.png","fishing_game/baitShadow/Shadow_bait2.png",["100:1","100:2","300:3","600:4"],["100:4","300:5","100:6"],3,"bait2",2,2,"1",""],["3","tid#fishingBaitsName3","tid#fishingBaitsDesc3",0,"fishing_game/bait/bait3.png","fishing_game/baitShadow/Shadow_bait3.png",["100:3","300:4","200:5","500:6"],["100:6","300:7","100:8"],3,"bait3",3,3,"1","tid#UI_211"],["4","tid#fishingBaitsName4","tid#fishingBaitsDesc4",900,"fishing_game/bait/bait4.png","fishing_game/baitShadow/Shadow_bait4.png",["300:4","500:6","100:7"],["100:8","300:9","100:10"],4,"bait4",4,4,"2",""],["5","tid#fishingBaitsName5","tid#fishingBaitsDesc5",0,"fishing_game/bait/bait5.png","fishing_game/baitShadow/Shadow_bait5.png",["200:6","100:7","200:9","500:10"],["200:9","300:11","100:12"],4,"bait5",5,5,"2","tid#UI_211"],["6","tid#fishingBaitsName6","tid#fishingBaitsDesc6",1600,"fishing_game/bait/bait6.png","fishing_game/baitShadow/Shadow_bait6.png",["200:8","500:10","200:11","500:12"],["200:11","300:13","100:14"],4,"bait6",6,6,"2",""],["7","tid#fishingBaitsName7","tid#fishingBaitsDesc7",0,"fishing_game/bait/bait7.png","fishing_game/baitShadow/Shadow_bait7.png",["200:10","500:12","200:13","500:14"],["200:13","300:15","100:16"],4,"bait7",7,7,"3","tid#UI_211"],["8","tid#fishingBaitsName8","tid#fishingBaitsDesc8",2800,"fishing_game/bait/bait8.png","fishing_game/baitShadow/Shadow_bait8.png",["200:12","500:14","200:15","500:16"],["200:15","300:17","100:18"],4,"bait8",8,8,"3",""],["9","tid#fishingBaitsName9","tid#fishingBaitsDesc9",0,"fishing_game/bait/bait9.png","fishing_game/baitShadow/Shadow_bait9.png",["500:14","500:16","200:17"],["100:18","300:19","100:20"],5,"bait9",9,9,"3","tid#UI_211"],["10","tid#fishingBaitsName10","tid#fishingBaitsDesc10",0,"fishing_game/bait/bait10.png","fishing_game/baitShadow/Shadow_bait10.png",["500:16","300:17","300:18"],["100:17","300:18","300:19"],2,"bait10",10,10,"3","tid#UI_211"]];

    var FishingFishes = [["Id","name","img","imgShadow","size","sizeMax","sizeTimes","income","incomeTimes","NauticalFoodsRate","NauticalFoods","ShareReward"],["1","tid#fishName1","fishing_game/fish/fish1.png","fishing_game/shadow/Shadow_fish1.png",10,40.0,3.0,2600,1.0,0,"[]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":1500}}"],["2","tid#fishName2","fishing_game/fish/fish2.png","fishing_game/shadow/Shadow_fish2.png",8,24.0,2.0,2800,1.0,0,"[]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2700}}"],["3","tid#fishName3","fishing_game/fish/fish3.png","fishing_game/shadow/Shadow_fish3.png",15,30.0,1.0,3100,1.0,0,"[]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4100}}"],["4","tid#fishName4","fishing_game/fish/fish4.png","fishing_game/shadow/Shadow_fish4.png",20,30.0,0.5,3700,1.1,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":100}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":5300}}"],["5","tid#fishName5","fishing_game/fish/fish5.png","fishing_game/shadow/Shadow_fish5.png",3,6.0,1.0,4600,1.1,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":100}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":8100}}"],["6","tid#fishName6","fishing_game/fish/fish6.png","fishing_game/shadow/Shadow_fish6.png",5,12.5,1.5,5100,1.1,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood2\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":9800}}"],["7","tid#fishName7","fishing_game/fish/fish7.png","fishing_game/shadow/Shadow_fish7.png",80,320.0,3.0,5900,1.1,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood2\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":12000}}"],["8","tid#fishName8","fishing_game/fish/fish8.png","fishing_game/shadow/Shadow_fish8.png",10,20.0,1.0,6300,1.1,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":60},{\"type\":\"NauticalFoods\",\"id\":\"CatFood2\",\"amount\":1,\"weight\":40}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":15000}}"],["9","tid#fishName9","fishing_game/fish/fish9.png","fishing_game/shadow/Shadow_fish9.png",50,200.0,3.0,7200,1.15,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1,\"weight\":50},{\"type\":\"NauticalFoods\",\"id\":\"CatFood2\",\"amount\":1,\"weight\":50}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":18000}}"],["10","tid#fishName10","fishing_game/fish/fish10.png","fishing_game/shadow/Shadow_fish10.png",18,27.0,0.5,8900,1.15,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood11\",\"amount\":1,\"weight\":100}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":20000}}"],["11","tid#fishName11","fishing_game/fish/fish11.png","fishing_game/shadow/Shadow_fish11.png",30,75.0,1.5,11000,1.15,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood11\",\"amount\":1,\"weight\":100}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":31000}}"],["12","tid#fishName12","fishing_game/fish/fish12.png","fishing_game/shadow/Shadow_fish12.png",30,45.0,0.5,14000,1.15,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood11\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood12\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":45000}}"],["13","tid#fishName13","fishing_game/fish/fish13.png","fishing_game/shadow/Shadow_fish13.png",8,12.0,0.5,15000,1.2,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood11\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood12\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":57000}}"],["14","tid#fishName14","fishing_game/fish/fish14.png","fishing_game/shadow/Shadow_fish14.png",12,24.0,1.0,16000,1.2,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood21\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood22\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":68000}}"],["15","tid#fishName15","fishing_game/fish/fish15.png","fishing_game/shadow/Shadow_fish15.png",5,7.5,0.5,18000,1.2,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood21\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood22\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":80000}}"],["16","tid#fishName16","fishing_game/fish/fish16.png","fishing_game/shadow/Shadow_fish16.png",100,500.0,4.0,20000,1.25,30,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood21\",\"amount\":1,\"weight\":60},{\"type\":\"NauticalFoods\",\"id\":\"CatFood22\",\"amount\":1,\"weight\":35},{\"type\":\"NauticalFoods\",\"id\":\"CatFood31\",\"amount\":1,\"weight\":5}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":99000}}"],["17","tid#fishName17","fishing_game/fish/fish17.png","fishing_game/shadow/Shadow_fish17.png",80,400.0,4.0,21000,1.25,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood31\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood32\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":110000}}"],["18","tid#fishName18","fishing_game/fish/fish18.png","fishing_game/shadow/Shadow_fish18.png",8,16.0,1.0,22000,1.25,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood31\",\"amount\":1,\"weight\":60},{\"type\":\"NauticalFoods\",\"id\":\"CatFood32\",\"amount\":1,\"weight\":40}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":120000}}"],["19","tid#fishName19","fishing_game/fish/fish19.png","fishing_game/shadow/Shadow_fish19.png",8,16.0,1.0,23000,1.3,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood41\",\"amount\":1,\"weight\":80},{\"type\":\"NauticalFoods\",\"id\":\"CatFood42\",\"amount\":1,\"weight\":20}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":140000}}"],["20","tid#fishName20","fishing_game/fish/fish20.png","fishing_game/shadow/Shadow_fish20.png",3,7.5,1.5,24000,1.3,100,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood41\",\"amount\":1,\"weight\":60},{\"type\":\"NauticalFoods\",\"id\":\"CatFood42\",\"amount\":1,\"weight\":40}]","{\"FirstVisitReward\":{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":150000}}"]];

    var FishingHitZone = [["Id","HitZone"],["1",[0.0,0.4,0.57,0.81]],["2",[0.0,0.43,0.61,0.83]],["3",[0.0,0.46,0.65,0.85]],["4",[0.0,0.48,0.69,0.87]],["5",[0.0,0.51,0.73,0.89]],["6",[0.0,0.54,0.77,0.91]],["7",[0.0,0.57,0.81,0.93]],["8",[0.0,0.6,0.85,0.95]],["9",[0.0,0.62,0.89,0.97]],["10",[0.0,0.65,0.93,0.99]],["bait1_old",[0.0,0.51,0.75,0.91]],["bait2_old",[0.0,0.46,0.72,0.89]],["bait3_old",[0.0,0.42,0.68,0.86]],["bait4_old",[0.0,0.44,0.74,0.93]],["bait5_old",[0.0,0.23,0.53,0.81]],["bait1",[0.0,0.51,0.75,0.91]],["bait2",[0.0,0.46,0.72,0.89]],["bait3",[0.0,0.42,0.7,0.89]],["bait4",[0.0,0.44,0.72,0.87]],["bait5",[0.0,0.4,0.7,0.87]],["bait6",[0.0,0.53,0.78,0.92]],["bait7",[0.0,0.33,0.63,0.85]],["bait8",[0.0,0.36,0.65,0.85]],["bait9",[0.0,0.44,0.74,0.93]],["bait10",[0.0,0.23,0.53,0.8]]];

    var FishingResult = [["Id","specialRate","sizeIncrease","incomeIncrease"],["1",100,[0.7,1.0],[0.55,0.6]],["2",70,[0.4,0.8],[0.45,0.5]],["3",0,[0.1,0.5],[0.35,0.4]],["4",0,[0.0,0.2],[0.25,0.3]]];

    var FoodsList = [["Id","foodsList"],["Food1","[{\"lv\":1,\"foodId\":1},{\"lv\":2,\"foodId\":2},{\"lv\":3,\"foodId\":3}]"],["Food2","[{\"lv\":1,\"foodId\":11},{\"lv\":2,\"foodId\":12},{\"lv\":3,\"foodId\":13}]"],["Food3","[{\"lv\":1,\"foodId\":21},{\"lv\":2,\"foodId\":22},{\"lv\":3,\"foodId\":23}]"],["Food4","[{\"lv\":1,\"foodId\":31},{\"lv\":2,\"foodId\":32},{\"lv\":3,\"foodId\":33},{\"lv\":4,\"foodId\":34}]"],["Food5","[{\"lv\":1,\"foodId\":41},{\"lv\":2,\"foodId\":42},{\"lv\":3,\"foodId\":43},{\"lv\":4,\"foodId\":44}]"],["Food6","[{\"lv\":1,\"foodId\":51},{\"lv\":2,\"foodId\":52},{\"lv\":3,\"foodId\":53},{\"lv\":4,\"foodId\":54}]"],["Food7","[{\"lv\":1,\"foodId\":61},{\"lv\":2,\"foodId\":62},{\"lv\":3,\"foodId\":63},{\"lv\":4,\"foodId\":64},{\"lv\":5,\"foodId\":65}]"],["Food8","[{\"lv\":1,\"foodId\":71},{\"lv\":2,\"foodId\":72},{\"lv\":3,\"foodId\":73},{\"lv\":4,\"foodId\":74},{\"lv\":5,\"foodId\":75}]"],["Food9","[{\"lv\":1,\"foodId\":81},{\"lv\":2,\"foodId\":82},{\"lv\":3,\"foodId\":83},{\"lv\":4,\"foodId\":84},{\"lv\":5,\"foodId\":85}]"],["Food10","[{\"lv\":1,\"foodId\":91},{\"lv\":2,\"foodId\":92},{\"lv\":3,\"foodId\":93},{\"lv\":4,\"foodId\":94},{\"lv\":5,\"foodId\":95}]"],["Food11","[{\"lv\":1,\"foodId\":101},{\"lv\":2,\"foodId\":102},{\"lv\":3,\"foodId\":103},{\"lv\":4,\"foodId\":104},{\"lv\":5,\"foodId\":105}]"],["Food12","[{\"lv\":1,\"foodId\":111},{\"lv\":2,\"foodId\":112},{\"lv\":3,\"foodId\":113},{\"lv\":4,\"foodId\":114},{\"lv\":5,\"foodId\":115}]"],["Food13","[{\"lv\":1,\"foodId\":121},{\"lv\":2,\"foodId\":122},{\"lv\":3,\"foodId\":123},{\"lv\":4,\"foodId\":124},{\"lv\":5,\"foodId\":125}]"],["Food14","[{\"lv\":1,\"foodId\":131},{\"lv\":2,\"foodId\":132},{\"lv\":3,\"foodId\":133},{\"lv\":4,\"foodId\":134},{\"lv\":5,\"foodId\":135}]"],["Food15","[{\"lv\":1,\"foodId\":141},{\"lv\":2,\"foodId\":142},{\"lv\":3,\"foodId\":143},{\"lv\":4,\"foodId\":144},{\"lv\":5,\"foodId\":145}]"],["Food16","[{\"lv\":1,\"foodId\":151},{\"lv\":2,\"foodId\":152},{\"lv\":3,\"foodId\":153},{\"lv\":4,\"foodId\":154},{\"lv\":5,\"foodId\":155}]"],["Food17","[{\"lv\":1,\"foodId\":161},{\"lv\":2,\"foodId\":162},{\"lv\":3,\"foodId\":163},{\"lv\":4,\"foodId\":164},{\"lv\":5,\"foodId\":165}]"],["Food18","[{\"lv\":1,\"foodId\":171},{\"lv\":2,\"foodId\":172},{\"lv\":3,\"foodId\":173},{\"lv\":4,\"foodId\":174},{\"lv\":5,\"foodId\":175}]"],["Food19","[{\"lv\":1,\"foodId\":181},{\"lv\":2,\"foodId\":182},{\"lv\":3,\"foodId\":183},{\"lv\":4,\"foodId\":184},{\"lv\":5,\"foodId\":185}]"],["Food20","[{\"lv\":1,\"foodId\":191},{\"lv\":2,\"foodId\":192},{\"lv\":3,\"foodId\":193},{\"lv\":4,\"foodId\":194},{\"lv\":5,\"foodId\":195}]"]];

    var Guide$1 = [["Id","Type","Content","Hight"],["Guide1_Login_1","1","tid#Guide1_Login_1",1063],["Guide1_Login_2","1","tid#Guide1_Login_2",1063],["Guide1_Build_1","1","tid#Guide1_Build_1",1063],["Guide1_Build_2","1","tid#Guide1_Build_2",1063],["Guide1_Build_3","2","",null],["Guide1_Build_4","2","",null],["Guide1_Build_5","5","tid#Guide1_Build_5",1816],["Guide1_Build_6","2","",null],["Guide1_Food_1","4","",null],["Guide1_Food_1_1","1","tid#Guide1_Food_1_1",1816],["Guide1_Food_2","1","tid#Guide1_Food_2",1816],["Guide1_Food_3","2","",null],["Guide1_Food_4","2","",null],["Guide1_Food_5","2","",null],["Guide1_Food_6","2","",null],["Guide1_Food_7","2","",null],["Guide1_Order_1","3","tid#Guide1_Order_2",1816],["Guide1_Order_2","4","",null],["Guide1_Order_3","3","tid#Guide1_Order_3",1816],["Guide1_Build_2_1","1","tid#Guide1_Build_2_1",1816],["Guide1_Build_2_2","2","",null],["Guide1_Build_2_3","2","",null],["Guide1_Build_2_4","2","",null],["Guide1_Invite_1","3","tid#Guide1_Invite_1",1506],["Guide1_Invite_2","1","tid#Guide1_Invite_2",1506],["Guide1_Invite_3","1","tid#Guide1_Invite_3",1506],["Guide2_Build_1","1","tid#Guide2_Build_1",1063],["Guide2_Build_2","2","",null],["Guide2_Build_3","1","tid#Guide2_Build_3",1063],["Guide2_Build_4","1","tid#Guide2_Build_4",1063],["Guide2_Build_5","2","",null],["Guide2_Build_6","2","",null],["Guide2_Build_7","2","",null],["Guide2_Build_8","2","",null],["Guide2_CatFood_1","3","tid#Guide2_CatFood_1",1063],["Guide2_CatFood_1_1","2","",null],["Guide2_CatFood_2","4","",null],["Guide2_CatReward_1","4","",null],["Guide2_CatReward_2","1","tid#Guide2_CatReward_1",1506],["Guide2_CatReward_3","5","tid#Guide2_CatReward_2",1506],["Guide2_Build2_1","1","tid#Guide2_Build2_1",1063],["Guide2_Build2_2","2","",null],["Guide2_Build2_3","2","",null],["Guide2_Build2_4","2","",null],["Guide2_Build2_5","2","",null],["Guide2_Build2_6","2",null,null],["Guide2_FishEnter_1","2",null,null],["Guide2_FishEnter_2","1","tid#Guide2_FishEnter_2",1816],["Guide2_FishEnter_3","2",null,null],["Guide2_FishEnter_4","1","tid#Guide2_FishEnter_4",1306],["Guide2_FishEnter_5","1","tid#Guide2_FishEnter_5",1306],["Guide4_HouseCat_1","1","tid#Guide4_HouseCat_1",1063],["Guide4_HouseCat_2","1","tid#Guide4_HouseCat_2",1063],["Guide4_HouseCat_3","2","",null],["Guide4_HouseCat_4","2","",null],["Guide4_HouseCat_5","1","tid#Guide4_HouseCat_5",1063],["Guide4_HouseCat_6","1","tid#Guide4_HouseCat_6",1063],["Guide4_HouseCat_7","2","",null],["Guide4_HouseCat_8","2","",null],["Guide4_HouseCat_9","2","",null],["Guide4_HouseCat_10","5","tid#Guide4_HouseCat_10",1816],["Guide4_HouseCat_12","4","",1816],["Guide4_HouseCat_13","1","tid#Guide4_HouseCat_13",1816],["Guide4_HouseCat_14","3","tid#Guide4_HouseCat_14",1816],["Guide1_ADInvite_1","5","tid#Guide1_ADInvite_1",1816],["Guide1_ADInvite_2","4",null,null],["Guide1_ADInvite_3","3","tid#Guide1_ADInvite_3",1816],["Guide4_AdvancedBuilding_1","1","tid#Guide4_AdvancedBuilding_1",1063],["Guide4_AdvancedBuilding_2","2",null,null],["Guide4_AdvancedBuilding_3","2",null,null],["Guide4_AdvancedBuilding_4","5","tid#Guide4_AdvancedBuilding_4",1306],["Guide4_AdvancedBuilding_5","2",null,null],["Guide4_AdvancedBuilding_6","5","tid#Guide4_AdvancedBuilding_6",1306],["Guide4_AdvancedBuilding_7","1","tid#Guide4_AdvancedBuilding_7",1306],["Guide4_Staff_1","1","tid#Guide4_Staff_1",1351],["Guide4_Staff_2","5","tid#Guide4_Staff_2",1351],["Guide4_Staff_3","4",null,null],["Guide4_Staff_4","5","tid#Guide4_Staff_4",1916],["Guide4_Staff_5","1","tid#Guide4_Staff_5",1916],["Guide4_Staff_6","4",null,null],["Guide4_Staff_7","1","tid#Guide4_Staff_7",1916],["Guide4_OrderPre_1","1","tid#Guide4_OrderPre_1",1506],["Guide4_OrderPre_2","5","tid#Guide4_OrderPre_2",1506],["Guide4_Order_1","1","tid#Guide4_Order_1",1816],["Guide4_Order_2","1","tid#Guide4_Order_2",1816],["Guide4_Order_3","5","tid#Guide4_Order_3",1816],["Guide4_Order_4","5","tid#Guide4_Order_4",1816],["Guide4_Order_5","2","",0],["Guide4_Order_6","5","tid#Guide4_Order_6",1816],["Guide4_Order_7","1","tid#Guide4_Order_7",1816],["Guide4_Order_8","2","",0],["Guide4_Order_9","3","tid#Guide4_Order_9",1816],["Guide4_Order_10","5","tid#Guide4_Order_10",1816],["Guide4_Order_11","1","tid#Guide4_Order_11",1816]];

    var GuideGroup = [["Id","Group","Condition"],["GuideGroup_1",["Guide1_Login_1","Guide1_Login_2"],"{\"Star\":0}"],["GuideGroup_2",["Guide1_Build_1","Guide1_Build_3","Guide1_Build_4","Guide1_Build_5","Guide1_Build_6"],"{\"GuideGroup\":\"GuideGroup_1\"}"],["GuideGroup_3",["Guide1_Food_1","Guide1_Food_1_1","Guide1_Food_2","Guide1_Food_3","Guide1_Food_4","Guide1_Food_5","Guide1_Food_6","Guide1_Food_7"],"{\"GuideGroup\":\"GuideGroup_2\"}"],["GuideGroup_4",["Guide1_Order_1","Guide1_Order_2","Guide1_Order_3"],"{\"GuideGroup\":\"GuideGroup_3\"}"],["GuideGroup_5",["Guide1_Build_2_1","Guide1_Build_2_2","Guide1_Build_2_3","Guide1_Build_2_4"],"{\"GuideGroup\":\"GuideGroup_4\"}"],["GuideGroup_6",["Guide1_Invite_1"],"{\"GuideGroup\":\"GuideGroup_5\"}"],["GuideGroup_8",["Guide2_Build_1","Guide2_Build_2","Guide2_Build_3","Guide2_Build_4","Guide2_Build_5","Guide2_Build_6","Guide2_Build_7","Guide2_Build_8"],"{\"Star\":150,\"GuideGroup\":\"GuideGroup_6\"}"],["GuideGroup_9",["Guide2_CatFood_1","Guide2_CatFood_1_1","Guide2_CatFood_2","Guide2_CatReward_1","Guide2_CatReward_2"],"{\"GuideGroup\":\"GuideGroup_8\"}"],["GuideGroup_10",["Guide1_ADInvite_1","Guide1_ADInvite_2","Guide1_ADInvite_3"],"{\"GuideGroup\":\"GuideGroup_6\"}"],["GuideGroup_11",["Guide2_Build2_1","Guide2_Build2_2","Guide2_Build2_3","Guide2_Build2_4","Guide2_Build2_5","Guide2_Build2_6"],"{\"Star\":230,\"GuideGroup\":\"GuideGroup_9\"}"],["GuideGroup_12",["Guide2_FishEnter_1","Guide2_FishEnter_2","Guide2_FishEnter_3","Guide2_FishEnter_4"],"{\"GuideGroup\":\"GuideGroup_11\"}"],["GuideGroup_14",["Guide4_HouseCat_1","Guide4_HouseCat_2","Guide4_HouseCat_3","Guide4_HouseCat_4","Guide4_HouseCat_5","Guide4_HouseCat_6","Guide4_HouseCat_7","Guide4_HouseCat_8","Guide4_HouseCat_9","Guide4_HouseCat_10","Guide4_HouseCat_12","Guide4_HouseCat_13","Guide4_HouseCat_14"],"{\"CatAmount\":1}"],["GuideGroup_15",["Guide4_AdvancedBuilding_1","Guide4_AdvancedBuilding_2","Guide4_AdvancedBuilding_3","Guide4_AdvancedBuilding_4","Guide4_AdvancedBuilding_5","Guide4_AdvancedBuilding_6"],"{\"Star\":2000,\"GuideGroup\":\"GuideGroup_12\"}"],["GuideGroup_16",["Guide4_Staff_1","Guide4_Staff_2","Guide4_Staff_3","Guide4_Staff_4","Guide4_Staff_5","Guide4_Staff_6","Guide4_Staff_7"],"{\"Interface\":\"Staff\"}"],["GuideGroup_18",["Guide4_OrderPre_1","Guide4_OrderPre_2"],"{\"Star\":100}"],["GuideGroup_19",["Guide4_Order_1","Guide4_Order_2","Guide4_Order_3","Guide4_Order_4","Guide4_Order_5","Guide4_Order_6","Guide4_Order_7","Guide4_Order_8","Guide4_Order_9","Guide4_Order_10","Guide4_Order_11"],"{\"Star\":100,\"GuideGroup\":\"GuideGroup_14\",\"Interface\":\"OrderSystem\"}"]];

    var Kanban = [["Id","Dialog","Action"],["Building_Standby",[],["mao1-1"]],["Building_Start",["tid#Building_Start1"],["mao1-2","amao1-3"]],["Building_Angry",["tid#Building_Angry1"],["mao1-3"]],["Building_CoffeeAuto",["tid#Building_CoffeeAuto1","tid#Building_CoffeeAuto2"],["mao1-2","amao1-3"]],["Building_CoffeeClick",["tid#Building_CoffeeClick1","tid#Building_CoffeeClick2","tid#Building_CoffeeClick3","tid#Building_CoffeeClick4","tid#Building_CoffeeClick5"],["mao1-2","amao1-3"]],["Building_BeachAuto",["tid#Building_BeachAuto1","tid#Building_BeachAuto2"],["mao1-2","amao1-3"]],["Building_BeachClick",["tid#Building_BeachClick1","tid#Building_BeachClick2","tid#Building_BeachClick3","tid#Building_BeachClick4","tid#Building_BeachClick5"],["mao1-2","amao1-3"]],["Building_WharfAuto",["tid#Building_WharfAuto1","tid#Building_WharfAuto2"],["mao1-2","amao1-3"]],["Building_WharfClick",["tid#Building_WharfClick1","tid#Building_WharfClick2"],["mao1-2","amao1-3"]],["Goods_Standby",[],["mao2-1"]],["Goods_Start",["tid#Goods_Start1"],["mao2-2","mao2-3"]],["Goods_Angry",["tid#Goods_Angry1"],["mao2-3"]],["Goods_FoodsAuto",["tid#Goods_FoodsAuto1","tid#Goods_FoodsAuto2"],["mao2-2","mao2-3"]],["Goods_FoodsClick",["tid#Goods_FoodsClick1","tid#Goods_FoodsClick2","tid#Goods_FoodsClick3","tid#Goods_FoodsClick4","tid#Goods_FoodsClick5"],["mao2-2","mao2-3"]],["Goods_NauticalFoodsAuto",["tid#Goods_NauticalFoodsAuto1","tid#Goods_NauticalFoodsAuto2"],["mao2-2","mao2-3"]],["Goods_NauticalFoodsClick",["tid#Goods_NauticalFoodsClick1","tid#Goods_NauticalFoodsClick2"],["mao2-2","mao2-3"]],["Goods_NauticalToolsAuto",["tid#Goods_NauticalToolsAuto1","tid#Goods_NauticalToolsAuto2"],["mao2-2","mao2-3"]],["Goods_NauticalToolsClick",["tid#Goods_NauticalToolsClick1","tid#Goods_NauticalToolsClick2"],["mao2-2","mao2-3"]],["Consume_Standby",[],["mao1-1"]],["Consume_Angry",["tid#ConsumeKanban_Angry1"],["mao1-3"]],["Consume_PackStart",["tid#ConsumeKanban_Start1"],["mao1-2","amao1-3"]],["Consume_CurrencyStart",["tid#ConsumeKanban_Start2"],["mao1-2","amao1-3"]],["Consume_Auto",["tid#ConsumeKanban_Auto1","tid#ConsumeKanban_Auto2","tid#ConsumeKanban_Auto3"],["mao1-2","amao1-3"]],["Consume_Click",["tid#ConsumeKanban_Click1","tid#ConsumeKanban_Click2","tid#ConsumeKanban_Click3"],["mao1-2","amao1-3"]]];

    var Mail$1 = [["Id","Name","Content","DeleteTimeLimit","Reward"],["1000","tid#MailTitle_1000","tid#MailContent_1000",604800,"[{\"type\":\"NauticalFoods\",\"id\":\"CatFood1\",\"amount\":1}]"],["beta1","tid#MailTitle_beta1","tid#MailContent_beta1",604800,"[]"],["iosTest","tid#MailTitle_iosTest","tid#MailContent_iosTest",604800,"[]"]];

    var MissionEvent = [["Id","Name","Image","Condition","Dialogue"],["ME_001","tid#NameME_001","ya1.png","[\"INT\"]","[\"tid#ME_001_1\",\"tid#ME_001_2\",\"tid#ME_001_3\"]"],["ME_002","tid#NameME_002","e1.png","[\"STA\"]","[\"tid#ME_002_1\",\"tid#ME_002_2\",\"tid#ME_002_3\",\"tid#ME_002_4\"]"],["ME_003","tid#NameME_003","niao1.png","[\"SPE\"]","[\"tid#ME_003_1\",\"tid#ME_003_2\",\"tid#ME_003_3\",\"tid#ME_003_4\"]"],["ME_004","tid#NameME_004","niao2.png","[\"STA\"]","[\"tid#ME_004_1\",\"tid#ME_004_2\",\"tid#ME_004_3\",\"tid#ME_004_4\"]"],["ME_005","tid#NameME_005","cao1.png","[\"INT\"]","[\"tid#ME_005_1\",\"tid#ME_005_2\",\"tid#ME_005_3\",\"tid#ME_005_4\"]"],["ME_006","tid#NameME_006","cao2.png","[\"SPE\"]","[\"tid#ME_006_1\",\"tid#ME_006_2\",\"tid#ME_006_3\",\"tid#ME_006_4\"]"],["ME_007","tid#NameME_007","niao3.png","[\"SPE\"]","[\"tid#ME_007_1\",\"tid#ME_007_2\",\"tid#ME_007_3\"]"],["ME_008","tid#NameME_008","e2.png","[\"STA\"]","[\"tid#ME_008_1\",\"tid#ME_008_2\",\"tid#ME_008_3\",\"tid#ME_008_4\"]"],["ME_009","tid#NameME_009","kao1.png","[\"INT\"]","[\"tid#ME_009_1\",\"tid#ME_009_2\",\"tid#ME_009_3\"]"],["ME_010","tid#NameME_010","e3.png","[\"STA\"]","[\"tid#ME_010_1\",\"tid#ME_010_2\",\"tid#ME_010_3\",\"tid#ME_010_4\",\"tid#ME_010_5\"]"],["ME_011","tid#NameME_011","cao3.png","[\"SPE\"]","[\"tid#ME_011_1\",\"tid#ME_011_2\",\"tid#ME_011_3\",\"tid#ME_011_4\",\"tid#ME_011_5\"]"],["ME_012","tid#NameME_012","ya2.png","[\"INT\"]","[\"tid#ME_012_1\",\"tid#ME_012_2\",\"tid#ME_012_3\",\"tid#ME_012_4\"]"],["ME_013","tid#NameME_013","ya3.png","[\"INT\"]","[\"tid#ME_013_1\",\"tid#ME_013_2\",\"tid#ME_013_3\"]"],["ME_014","tid#NameME_014","niao1.png","[\"SPE\"]","[\"tid#ME_014_1\",\"tid#ME_014_2\",\"tid#ME_014_3\"]"],["ME_015","tid#NameME_015","e1.png","[\"STA\"]","[\"tid#ME_015_1\",\"tid#ME_015_2\",\"tid#ME_015_3\"]"],["ME_016","tid#NameME_016","niao2.png","[\"INT\"]","[\"tid#ME_016_1\",\"tid#ME_016_2\",\"tid#ME_016_3\"]"],["ME_017","tid#NameME_017","cao1.png","[\"STA\"]","[\"tid#ME_017_1\",\"tid#ME_017_2\",\"tid#ME_017_3\"]"],["ME_018","tid#NameME_018","ya1.png","[\"SPE\"]","[\"tid#ME_018_1\",\"tid#ME_018_2\",\"tid#ME_018_3\",\"tid#ME_018_4\"]"],["ME_019","tid#NameME_019","kao2.png","[\"STA\"]","[\"tid#ME_019_1\",\"tid#ME_019_2\",\"tid#ME_019_3\",\"tid#ME_019_4\"]"],["ME_020","tid#NameME_020","e2.png","[\"INT\"]","[\"tid#ME_020_1\",\"tid#ME_020_2\",\"tid#ME_020_3\",\"tid#ME_020_4\"]"],["ME_021","tid#NameME_021","niao3.png","[\"SPE\"]","[\"tid#ME_021_1\",\"tid#ME_021_2\",\"tid#ME_021_3\",\"tid#ME_021_4\"]"]];

    var NauticalFoods = [["Id","Name","Desc","Icon","Point","Intimacy","UpgradeLine","Rank"],["CatFood1","tid#CatFoodName1","tid#CatFoodDesc1","nautical_food/qundaicai_1.png",10,1,"[]",1],["CatFood2","tid#CatFoodName2","tid#CatFoodDesc2","nautical_food/qundaicai_2.png",27,3,"[{\"id\":\"CatFood1\",\"amount\":3}]",2],["CatFood3","tid#CatFoodName3","tid#CatFoodDesc3","nautical_food/qundaicai_3.png",48,6,"[{\"id\":\"CatFood2\",\"amount\":2}]",3],["CatFood11","tid#CatFoodName11","tid#CatFoodDesc11","nautical_food/yuyou_1.png",20,1,"[]",4],["CatFood12","tid#CatFoodName12","tid#CatFoodDesc12","nautical_food/yuyou_2.png",54,3,"[{\"id\":\"CatFood11\",\"amount\":3}]",5],["CatFood13","tid#CatFoodName13","tid#CatFoodDesc13","nautical_food/yuyou_3.png",97,6,"[{\"id\":\"CatFood12\",\"amount\":2}]",6],["CatFood21","tid#CatFoodName21","tid#CatFoodDesc21","nautical_food/yuluan_1.png",30,1,"[]",7],["CatFood22","tid#CatFoodName22","tid#CatFoodDesc22","nautical_food/yuluan_2.png",81,3,"[{\"id\":\"CatFood21\",\"amount\":3}]",8],["CatFood23","tid#CatFoodName23","tid#CatFoodDesc23","nautical_food/yuluan_3.png",145,6,"[{\"id\":\"CatFood22\",\"amount\":2}]",9],["CatFood31","tid#CatFoodName31","tid#CatFoodDesc31","nautical_food/yurou_1.png",40,1,"[]",10],["CatFood32","tid#CatFoodName32","tid#CatFoodDesc32","nautical_food/yurou_2.png",108,3,"[{\"id\":\"CatFood31\",\"amount\":3}]",11],["CatFood33","tid#CatFoodName33","tid#CatFoodDesc33","nautical_food/yurou_3.png",194,6,"[{\"id\":\"CatFood32\",\"amount\":2}]",12],["CatFood41","tid#CatFoodName41","tid#CatFoodDesc41","nautical_food/jingpinyurou_1.png",50,1,"[]",13],["CatFood42","tid#CatFoodName42","tid#CatFoodDesc42","nautical_food/jingpinyurou_2.png",135,3,"[{\"id\":\"CatFood41\",\"amount\":3}]",14],["CatFood43","tid#CatFoodName43","tid#CatFoodDesc43","nautical_food/jingpinyurou_3.png",243,6,"[{\"id\":\"CatFood42\",\"amount\":2}]",15]];

    var NauticalPhotoReward = [["Id","CatRequire","Reward"],["RewardSence1_00001","[\"Cat1\"]","[\"Sence1_00001\",\"Sence1_10001\",\"Sence1_10002\",\"Sence1_10003\",\"Sence1_10004\"]"],["RewardSence1_00002","[\"Cat2\"]","[\"Sence1_00002\",\"Sence1_10005\",\"Sence1_10006\",\"Sence1_10007\",\"Sence1_10008\"]"],["RewardSence1_00003","[\"Cat4\"]","[\"Sence1_00003\",\"Sence1_10001\",\"Sence1_10005\",\"Sence1_10009\",\"Sence1_10013\",\"Sence1_10017\",\"Sence1_10021\",\"Sence1_10023\"]"],["RewardSence1_00004","[\"Cat7\"]","[\"Sence1_00004\",\"Sence1_10002\",\"Sence1_10006\",\"Sence1_10010\",\"Sence1_10014\",\"Sence1_10018\",\"Sence1_10021\",\"Sence1_10022\"]"],["RewardSence1_00005","[\"Cat10\"]","[\"Sence1_00005\",\"Sence1_10003\",\"Sence1_10007\",\"Sence1_10011\",\"Sence1_10015\",\"Sence1_10019\",\"Sence1_10022\",\"Sence1_10024\"]"],["RewardSence1_00006","[\"Cat12\"]","[\"Sence1_00006\",\"Sence1_10009\",\"Sence1_10010\",\"Sence1_10011\",\"Sence1_10012\"]"],["RewardSence1_00007","[\"Cat16\"]","[\"Sence1_00007\",\"Sence1_10013\",\"Sence1_10014\",\"Sence1_10015\",\"Sence1_10016\"]"],["RewardSence1_00008","[\"Cat20\"]","[\"Sence1_00008\",\"Sence1_10004\",\"Sence1_10008\",\"Sence1_10012\",\"Sence1_10016\",\"Sence1_10020\",\"Sence1_10023\",\"Sence1_10024\"]"],["RewardSence1_00009","[\"Cat25\"]","[\"Sence1_00009\",\"Sence1_10017\",\"Sence1_10018\",\"Sence1_10019\",\"Sence1_10020\"]"],["RewardSence1_10001","[\"Cat1\",\"Cat4\"]","[\"Sence1_10001\",\"Sence1_20001\",\"Sence1_20011\"]"],["RewardSence1_10002","[\"Cat1\",\"Cat7\"]","[\"Sence1_10002\",\"Sence1_20001\",\"Sence1_20006\"]"],["RewardSence1_10003","[\"Cat1\",\"Cat10\"]","[\"Sence1_10003\",\"Sence1_20006\",\"Sence1_20016\"]"],["RewardSence1_10004","[\"Cat1\",\"Cat20\"]","[\"Sence1_10004\",\"Sence1_20011\",\"Sence1_20016\"]"],["RewardSence1_10005","[\"Cat2\",\"Cat4\"]","[\"Sence1_10005\",\"Sence1_20002\",\"Sence1_20012\"]"],["RewardSence1_10006","[\"Cat2\",\"Cat7\"]","[\"Sence1_10006\",\"Sence1_20002\",\"Sence1_20007\"]"],["RewardSence1_10007","[\"Cat2\",\"Cat10\"]","[\"Sence1_10007\",\"Sence1_20007\",\"Sence1_20017\"]"],["RewardSence1_10008","[\"Cat2\",\"Cat20\"]","[\"Sence1_10008\",\"Sence1_20012\",\"Sence1_20017\"]"],["RewardSence1_10009","[\"Cat12\",\"Cat4\"]","[\"Sence1_10009\",\"Sence1_20003\",\"Sence1_20013\"]"],["RewardSence1_10010","[\"Cat12\",\"Cat7\"]","[\"Sence1_10010\",\"Sence1_20003\",\"Sence1_20008\"]"],["RewardSence1_10011","[\"Cat12\",\"Cat10\"]","[\"Sence1_10011\",\"Sence1_20008\",\"Sence1_20018\"]"],["RewardSence1_10012","[\"Cat12\",\"Cat20\"]","[\"Sence1_10012\",\"Sence1_20013\",\"Sence1_20018\"]"],["RewardSence1_10013","[\"Cat16\",\"Cat4\"]","[\"Sence1_10013\",\"Sence1_20004\",\"Sence1_20014\"]"],["RewardSence1_10014","[\"Cat16\",\"Cat7\"]","[\"Sence1_10014\",\"Sence1_20004\",\"Sence1_20009\"]"],["RewardSence1_10015","[\"Cat16\",\"Cat10\"]","[\"Sence1_10015\",\"Sence1_20009\",\"Sence1_20019\"]"],["RewardSence1_10016","[\"Cat16\",\"Cat20\"]","[\"Sence1_10016\",\"Sence1_20014\",\"Sence1_20019\"]"],["RewardSence1_10017","[\"Cat25\",\"Cat4\"]","[\"Sence1_10017\",\"Sence1_20005\",\"Sence1_20015\"]"],["RewardSence1_10018","[\"Cat25\",\"Cat7\"]","[\"Sence1_10018\",\"Sence1_20005\",\"Sence1_20010\"]"],["RewardSence1_10019","[\"Cat25\",\"Cat10\"]","[\"Sence1_10019\",\"Sence1_20010\",\"Sence1_20020\"]"],["RewardSence1_10020","[\"Cat25\",\"Cat20\"]","[\"Sence1_10020\",\"Sence1_20015\",\"Sence1_20020\"]"],["RewardSence1_10021","[\"Cat4\",\"Cat7\"]","[\"Sence1_10021\",\"Sence1_20001\",\"Sence1_20002\",\"Sence1_20003\",\"Sence1_20004\",\"Sence1_20005\"]"],["RewardSence1_10022","[\"Cat10\",\"Cat7\"]","[\"Sence1_10022\",\"Sence1_20006\",\"Sence1_20007\",\"Sence1_20008\",\"Sence1_20009\",\"Sence1_20010\"]"],["RewardSence1_10023","[\"Cat4\",\"Cat20\"]","[\"Sence1_10023\",\"Sence1_20011\",\"Sence1_20012\",\"Sence1_20013\",\"Sence1_20014\",\"Sence1_20015\"]"],["RewardSence1_10024","[\"Cat10\",\"Cat20\"]","[\"Sence1_10024\",\"Sence1_20016\",\"Sence1_20017\",\"Sence1_20018\",\"Sence1_20019\",\"Sence1_20020\"]"],["RewardSence1_20001","[\"Cat1\",\"Cat4\",\"Cat7\"]","[\"Sence1_20001\"]"],["RewardSence1_20002","[\"Cat2\",\"Cat4\",\"Cat7\"]","[\"Sence1_20002\"]"],["RewardSence1_20003","[\"Cat12\",\"Cat4\",\"Cat7\"]","[\"Sence1_20003\"]"],["RewardSence1_20004","[\"Cat16\",\"Cat4\",\"Cat7\"]","[\"Sence1_20004\"]"],["RewardSence1_20005","[\"Cat25\",\"Cat4\",\"Cat7\"]","[\"Sence1_20005\"]"],["RewardSence1_20006","[\"Cat1\",\"Cat10\",\"Cat7\"]","[\"Sence1_20006\"]"],["RewardSence1_20007","[\"Cat2\",\"Cat10\",\"Cat7\"]","[\"Sence1_20007\"]"],["RewardSence1_20008","[\"Cat12\",\"Cat10\",\"Cat7\"]","[\"Sence1_20008\"]"],["RewardSence1_20009","[\"Cat16\",\"Cat10\",\"Cat7\"]","[\"Sence1_20009\"]"],["RewardSence1_20010","[\"Cat25\",\"Cat10\",\"Cat7\"]","[\"Sence1_20010\"]"],["RewardSence1_20011","[\"Cat1\",\"Cat4\",\"Cat20\"]","[\"Sence1_20011\"]"],["RewardSence1_20012","[\"Cat2\",\"Cat4\",\"Cat20\"]","[\"Sence1_20012\"]"],["RewardSence1_20013","[\"Cat12\",\"Cat4\",\"Cat20\"]","[\"Sence1_20013\"]"],["RewardSence1_20014","[\"Cat16\",\"Cat4\",\"Cat20\"]","[\"Sence1_20014\"]"],["RewardSence1_20015","[\"Cat25\",\"Cat4\",\"Cat20\"]","[\"Sence1_20015\"]"],["RewardSence1_20016","[\"Cat1\",\"Cat10\",\"Cat20\"]","[\"Sence1_20016\"]"],["RewardSence1_20017","[\"Cat2\",\"Cat10\",\"Cat20\"]","[\"Sence1_20017\"]"],["RewardSence1_20018","[\"Cat12\",\"Cat10\",\"Cat20\"]","[\"Sence1_20018\"]"],["RewardSence1_20019","[\"Cat16\",\"Cat10\",\"Cat20\"]","[\"Sence1_20019\"]"],["RewardSence1_20020","[\"Cat25\",\"Cat10\",\"Cat20\"]","[\"Sence1_20020\"]"],["RewardSence2_00001","[\"Cat3\"]","[\"Sence2_00001\",\"Sence2_10001\",\"Sence2_10002\",\"Sence2_10003\",\"Sence2_10004\"]"],["RewardSence2_00002","[\"Cat5\"]","[\"Sence2_00002\",\"Sence2_10001\",\"Sence2_10005\"]"],["RewardSence2_00003","[\"Cat8\"]","[\"Sence2_00003\",\"Sence2_10002\",\"Sence2_10006\"]"],["RewardSence2_00004","[\"Cat13\"]","[\"Sence2_00004\",\"Sence2_10003\",\"Sence2_10007\"]"],["RewardSence2_00005","[\"Cat17\"]","[\"Sence2_00005\",\"Sence2_10005\",\"Sence2_10006\",\"Sence2_10007\",\"Sence2_10008\"]"],["RewardSence2_00006","[\"Cat21\"]","[\"Sence2_00006\"]"],["RewardSence2_00007","[\"Cat27\"]","[\"Sence2_00007\",\"Sence2_10004\",\"Sence2_10008\"]"],["RewardSence02_10001","[\"Cat3\",\"Cat5\"]","[\"Sence2_10001\"]"],["RewardSence02_10002","[\"Cat3\",\"Cat8\"]","[\"Sence2_10002\"]"],["RewardSence02_10003","[\"Cat3\",\"Cat13\"]","[\"Sence2_10003\"]"],["RewardSence02_10004","[\"Cat3\",\"Cat27\"]","[\"Sence2_10004\"]"],["RewardSence02_10005","[\"Cat17\",\"Cat5\"]","[\"Sence2_10005\"]"],["RewardSence02_10006","[\"Cat17\",\"Cat8\"]","[\"Sence2_10006\"]"],["RewardSence02_10007","[\"Cat17\",\"Cat13\"]","[\"Sence2_10007\"]"],["RewardSence02_10008","[\"Cat17\",\"Cat27\"]","[\"Sence2_10008\"]"],["RewardSence3_00001","[\"Cat6\"]","[\"Sence3_00001\"]"],["RewardSence3_00002","[\"Cat9\"]","[\"Sence3_00002\"]"],["RewardSence3_00003","[\"Cat14\"]","[\"Sence3_00003\"]"],["RewardSence3_00004","[\"Cat18\"]","[\"Sence3_00004\"]"],["RewardSence3_00005","[\"Cat22\"]","[\"Sence3_00005\"]"],["RewardSence3_00006","[\"Cat24\"]","[\"Sence3_00006\"]"],["RewardSence3_00007","[\"Cat28\"]","[\"Sence3_00007\"]"],["RewardSence03_00001","[\"Cat6\"]","[\"Sence3_00001\"]"],["RewardSence03_00002","[\"Cat9\"]","[\"Sence3_00002\"]"],["RewardSence03_00003","[\"Cat14\"]","[\"Sence3_00003\"]"],["RewardSence03_00004","[\"Cat18\"]","[\"Sence3_00004\"]"],["RewardSence03_00005","[\"Cat22\"]","[\"Sence3_00005\"]"],["RewardSence03_00006","[\"Cat24\"]","[\"Sence3_00006\"]"],["RewardSence03_00007","[\"Cat28\"]","[\"Sence3_00007\"]"],["RewardSence4_00001","[\"Cat1\"]","[\"Sence4_00001\"]"],["RewardSence4_00002","[\"Cat2\"]","[\"Sence4_00002\"]"],["RewardSence4_00003","[\"Cat4\"]","[\"Sence4_00003\"]"],["RewardSence4_00004","[\"Cat7\"]","[\"Sence4_00004\"]"],["RewardSence4_00005","[\"Cat10\"]","[\"Sence4_00005\"]"],["RewardSence4_00006","[\"Cat12\"]","[\"Sence4_00006\"]"],["RewardSence4_00007","[\"Cat16\"]","[\"Sence4_00007\"]"],["RewardSence4_00008","[\"Cat20\"]","[\"Sence4_00008\"]"],["RewardSence4_00009","[\"Cat25\"]","[\"Sence4_00009\"]"],["RewardSence04_00001","[\"Cat1\"]","[\"Sence4_00001\"]"],["RewardSence04_00002","[\"Cat2\"]","[\"Sence4_00002\"]"],["RewardSence04_00003","[\"Cat4\"]","[\"Sence4_00003\"]"],["RewardSence04_00004","[\"Cat7\"]","[\"Sence4_00004\"]"],["RewardSence04_00005","[\"Cat10\"]","[\"Sence4_00005\"]"],["RewardSence04_00006","[\"Cat12\"]","[\"Sence4_00006\"]"],["RewardSence04_00007","[\"Cat16\"]","[\"Sence4_00007\"]"],["RewardSence04_00008","[\"Cat20\"]","[\"Sence4_00008\"]"],["RewardSence04_00009","[\"Cat25\"]","[\"Sence4_00009\"]"],["RewardSence5_00001","[\"Cat11\"]","[\"Sence5_00001\"]"],["RewardSence5_00002","[\"Cat15\"]","[\"Sence5_00002\"]"],["RewardSence5_00003","[\"Cat19\"]","[\"Sence5_00003\"]"],["RewardSence5_00004","[\"Cat23\"]","[\"Sence5_00004\"]"],["RewardSence5_00005","[\"Cat26\"]","[\"Sence5_00005\"]"],["RewardSence5_00006","[\"Cat29\"]","[\"Sence5_00006\"]"],["RewardSence5_00007","[\"Cat30\"]","[\"Sence5_00007\"]"],["RewardSence05_00001","[\"Cat11\"]","[\"Sence5_00001\"]"],["RewardSence05_00002","[\"Cat15\"]","[\"Sence5_00002\"]"],["RewardSence05_00003","[\"Cat19\"]","[\"Sence5_00003\"]"],["RewardSence05_00004","[\"Cat23\"]","[\"Sence5_00004\"]"],["RewardSence05_00005","[\"Cat26\"]","[\"Sence5_00005\"]"],["RewardSence05_00006","[\"Cat29\"]","[\"Sence5_00006\"]"],["RewardSence05_00007","[\"Cat30\"]","[\"Sence5_00007\"]"],["RewardSence6_00001","[\"Cat3\"]","[\"Sence6_00001\",\"Sence6_10001\",\"Sence6_10002\",\"Sence6_10003\",\"Sence6_10004\",\"Sence6_10005\"]"],["RewardSence6_00002","[\"Cat5\"]","[\"Sence6_00002\",\"Sence6_10001\",\"Sence6_10006\",\"Sence6_10011\",\"Sence6_10012\",\"Sence6_10013\"]"],["RewardSence6_00003","[\"Cat8\"]","[\"Sence6_00003\",\"Sence6_10002\",\"Sence6_10007\",\"Sence6_10011\",\"Sence6_10014\"]"],["RewardSence6_00004","[\"Cat13\"]","[\"Sence6_00004\",\"Sence6_10006\",\"Sence6_10007\",\"Sence6_10008\",\"Sence6_10009\",\"Sence6_10010\"]"],["RewardSence6_00005","[\"Cat17\"]","[\"Sence6_00005\",\"Sence6_10003\",\"Sence6_10008\",\"Sence6_10012\",\"Sence6_10015\"]"],["RewardSence6_00006","[\"Cat21\"]","[\"Sence6_00006\",\"Sence6_10004\",\"Sence6_10009\",\"Sence6_10014\",\"Sence6_10015\",\"Sence6_10016\"]"],["RewardSence6_00007","[\"Cat27\"]","[\"Sence6_00007\",\"Sence6_10005\",\"Sence6_10010\",\"Sence6_10013\",\"Sence6_10016\"]"],["RewardSence6_10001","[\"Cat3\",\"Cat5\"]","[\"Sence6_10001\",\"Sence6_20001\",\"Sence6_20002\",\"Sence6_20003\"]"],["RewardSence6_10002","[\"Cat3\",\"Cat8\"]","[\"Sence6_10002\",\"Sence6_20001\",\"Sence6_20007\"]"],["RewardSence6_10003","[\"Cat3\",\"Cat17\"]","[\"Sence6_10003\",\"Sence6_20002\",\"Sence6_20008\"]"],["RewardSence6_10004","[\"Cat3\",\"Cat21\"]","[\"Sence6_10004\",\"Sence6_20007\",\"Sence6_20008\",\"Sence6_20009\"]"],["RewardSence6_10005","[\"Cat3\",\"Cat27\"]","[\"Sence6_10005\",\"Sence6_20003\",\"Sence6_20009\"]"],["RewardSence6_10006","[\"Cat13\",\"Cat5\"]","[\"Sence6_10006\",\"Sence6_20004\",\"Sence6_20005\",\"Sence6_20006\"]"],["RewardSence6_10007","[\"Cat13\",\"Cat8\"]","[\"Sence6_10007\",\"Sence6_20004\",\"Sence6_20010\"]"],["RewardSence6_10008","[\"Cat13\",\"Cat17\"]","[\"Sence6_10008\",\"Sence6_20005\",\"Sence6_20011\"]"],["RewardSence6_10009","[\"Cat13\",\"Cat21\"]","[\"Sence6_10009\",\"Sence6_20010\",\"Sence6_20011\",\"Sence6_20012\"]"],["RewardSence6_10010","[\"Cat13\",\"Cat27\"]","[\"Sence6_10010\",\"Sence6_20006\",\"Sence6_20012\"]"],["RewardSence6_10011","[\"Cat5\",\"Cat8\"]","[\"Sence6_10011\",\"Sence6_20001\",\"Sence6_20004\"]"],["RewardSence6_10012","[\"Cat5\",\"Cat17\"]","[\"Sence6_10012\",\"Sence6_20002\",\"Sence6_20005\"]"],["RewardSence6_10013","[\"Cat5\",\"Cat27\"]","[\"Sence6_10013\",\"Sence6_20003\",\"Sence6_20006\"]"],["RewardSence6_10014","[\"Cat21\",\"Cat8\"]","[\"Sence6_10014\",\"Sence6_20007\",\"Sence6_20010\"]"],["RewardSence6_10015","[\"Cat21\",\"Cat17\"]","[\"Sence6_10015\",\"Sence6_20008\",\"Sence6_20011\"]"],["RewardSence6_10016","[\"Cat21\",\"Cat27\"]","[\"Sence6_10016\",\"Sence6_20009\",\"Sence6_20012\"]"],["RewardSence6_20001","[\"Cat3\",\"Cat5\",\"Cat8\"]","[\"Sence6_20001\"]"],["RewardSence6_20002","[\"Cat3\",\"Cat5\",\"Cat17\"]","[\"Sence6_20002\"]"],["RewardSence6_20003","[\"Cat3\",\"Cat5\",\"Cat27\"]","[\"Sence6_20003\"]"],["RewardSence6_20004","[\"Cat13\",\"Cat5\",\"Cat8\"]","[\"Sence6_20004\"]"],["RewardSence6_20005","[\"Cat13\",\"Cat5\",\"Cat17\"]","[\"Sence6_20005\"]"],["RewardSence6_20006","[\"Cat13\",\"Cat5\",\"Cat27\"]","[\"Sence6_20006\"]"],["RewardSence6_20007","[\"Cat3\",\"Cat21\",\"Cat8\"]","[\"Sence6_20007\"]"],["RewardSence6_20008","[\"Cat3\",\"Cat21\",\"Cat17\"]","[\"Sence6_20008\"]"],["RewardSence6_20009","[\"Cat3\",\"Cat21\",\"Cat27\"]","[\"Sence6_20009\"]"],["RewardSence6_20010","[\"Cat13\",\"Cat21\",\"Cat8\"]","[\"Sence6_20010\"]"],["RewardSence6_20011","[\"Cat13\",\"Cat21\",\"Cat17\"]","[\"Sence6_20011\"]"],["RewardSence6_20012","[\"Cat13\",\"Cat21\",\"Cat27\"]","[\"Sence6_20012\"]"],["RewardSence7_00001","[\"Cat6\"]","[\"Sence7_00001\"]"],["RewardSence7_00002","[\"Cat9\"]","[\"Sence7_00002\"]"],["RewardSence7_00003","[\"Cat14\"]","[\"Sence7_00003\"]"],["RewardSence7_00004","[\"Cat18\"]","[\"Sence7_00004\"]"],["RewardSence7_00005","[\"Cat22\"]","[\"Sence7_00005\"]"],["RewardSence7_00006","[\"Cat24\"]","[\"Sence7_00006\"]"],["RewardSence7_00007","[\"Cat28\"]","[\"Sence7_00007\"]"],["RewardSence8_00001","[\"Cat11\"]","[\"Sence8_00001\",\"Sence8_10001\",\"Sence8_10002\",\"Sence8_10003\",\"Sence8_10004\",\"Sence8_10005\"]"],["RewardSence8_00002","[\"Cat15\"]","[\"Sence8_00002\",\"Sence8_10006\",\"Sence8_10007\",\"Sence8_10008\",\"Sence8_10009\",\"Sence8_10010\"]"],["RewardSence8_00003","[\"Cat19\"]","[\"Sence8_00003\",\"Sence8_10001\",\"Sence8_10006\"]"],["RewardSence8_00004","[\"Cat23\"]","[\"Sence8_00004\",\"Sence8_10002\",\"Sence8_10007\"]"],["RewardSence8_00005","[\"Cat26\"]","[\"Sence8_00005\",\"Sence8_10003\",\"Sence8_10008\"]"],["RewardSence8_00006","[\"Cat29\"]","[\"Sence8_00006\",\"Sence8_10004\",\"Sence8_10009\"]"],["RewardSence8_00007","[\"Cat30\"]","[\"Sence8_00007\",\"Sence8_10005\",\"Sence8_10010\"]"],["RewardSence8_10001","[\"Cat11\",\"Cat19\"]","[\"Sence8_10001\"]"],["RewardSence8_10002","[\"Cat11\",\"Cat23\"]","[\"Sence8_10002\"]"],["RewardSence8_10003","[\"Cat11\",\"Cat26\"]","[\"Sence8_10003\"]"],["RewardSence8_10004","[\"Cat11\",\"Cat29\"]","[\"Sence8_10004\"]"],["RewardSence8_10005","[\"Cat11\",\"Cat30\"]","[\"Sence8_10005\"]"],["RewardSence8_10006","[\"Cat15\",\"Cat19\"]","[\"Sence8_10006\"]"],["RewardSence8_10007","[\"Cat15\",\"Cat23\"]","[\"Sence8_10007\"]"],["RewardSence8_10008","[\"Cat15\",\"Cat26\"]","[\"Sence8_10008\"]"],["RewardSence8_10009","[\"Cat15\",\"Cat29\"]","[\"Sence8_10009\"]"],["RewardSence8_10010","[\"Cat15\",\"Cat30\"]","[\"Sence8_10010\"]"],["RewardSence08_00001","[\"Cat11\"]","[\"Sence8_00001\",\"Sence8_10001\",\"Sence8_10002\",\"Sence8_10003\",\"Sence8_10004\",\"Sence8_10005\"]"],["RewardSence08_00002","[\"Cat15\"]","[\"Sence8_00002\",\"Sence8_10006\",\"Sence8_10007\",\"Sence8_10008\",\"Sence8_10009\",\"Sence8_10010\"]"],["RewardSence08_00003","[\"Cat19\"]","[\"Sence8_00003\",\"Sence8_10001\",\"Sence8_10006\"]"],["RewardSence08_00004","[\"Cat23\"]","[\"Sence8_00004\",\"Sence8_10002\",\"Sence8_10007\"]"],["RewardSence08_00005","[\"Cat26\"]","[\"Sence8_00005\",\"Sence8_10003\",\"Sence8_10008\"]"],["RewardSence08_00006","[\"Cat29\"]","[\"Sence8_00006\",\"Sence8_10004\",\"Sence8_10009\"]"],["RewardSence08_00007","[\"Cat30\"]","[\"Sence8_00007\",\"Sence8_10005\",\"Sence8_10010\"]"],["RewardSence08_10001","[\"Cat11\",\"Cat19\"]","[\"Sence8_10001\"]"],["RewardSence08_10002","[\"Cat11\",\"Cat23\"]","[\"Sence8_10002\"]"],["RewardSence08_10003","[\"Cat11\",\"Cat26\"]","[\"Sence8_10003\"]"],["RewardSence08_10004","[\"Cat11\",\"Cat29\"]","[\"Sence8_10004\"]"],["RewardSence08_10005","[\"Cat11\",\"Cat30\"]","[\"Sence8_10005\"]"],["RewardSence08_10006","[\"Cat15\",\"Cat19\"]","[\"Sence8_10006\"]"],["RewardSence08_10007","[\"Cat15\",\"Cat23\"]","[\"Sence8_10007\"]"],["RewardSence08_10008","[\"Cat15\",\"Cat26\"]","[\"Sence8_10008\"]"],["RewardSence08_10009","[\"Cat15\",\"Cat29\"]","[\"Sence8_10009\"]"],["RewardSence08_10010","[\"Cat15\",\"Cat30\"]","[\"Sence8_10010\"]"],["RewardSence9_deer_00001","[\"Cat1\"]","[\"Sence9_deer_00001\"]"],["RewardSence9_deer_00002","[\"Cat2\"]","[\"Sence9_deer_00002\"]"],["RewardSence9_deer_00003","[\"Cat4\"]","[\"Sence9_deer_00003\"]"],["RewardSence9_deer_00004","[\"Cat7\"]","[\"Sence9_deer_00004\"]"],["RewardSence9_snowball_00001","[\"Cat3\"]","[\"Sence9_snowball_00001\",\"Sence9_snowball_10001\",\"Sence9_snowball_10002\",\"Sence9_snowball_10003\",\"Sence9_snowball_10004\"]"],["RewardSence9_snowball_00002","[\"Cat5\"]","[\"Sence9_snowball_00002\",\"Sence9_snowball_10005\",\"Sence9_snowball_10006\",\"Sence9_snowball_10007\",\"Sence9_snowball_10008\"]"],["RewardSence9_snowball_00003","[\"Cat8\"]","[\"Sence9_snowball_00003\",\"Sence9_snowball_10009\",\"Sence9_snowball_10010\",\"Sence9_snowball_10011\",\"Sence9_snowball_10012\"]"],["RewardSence9_snowball_00004","[\"Cat13\"]","[\"Sence9_snowball_00004\",\"Sence9_snowball_10013\",\"Sence9_snowball_10014\",\"Sence9_snowball_10015\",\"Sence9_snowball_10016\"]"],["RewardSence9_snowball_00005","[\"Cat6\"]","[\"Sence9_snowball_00005\",\"Sence9_snowball_10001\",\"Sence9_snowball_10005\",\"Sence9_snowball_10009\",\"Sence9_snowball_10013\"]"],["RewardSence9_snowball_00006","[\"Cat9\"]","[\"Sence9_snowball_00006\",\"Sence9_snowball_10002\",\"Sence9_snowball_10006\",\"Sence9_snowball_10010\",\"Sence9_snowball_10014\"]"],["RewardSence9_snowball_00007","[\"Cat14\"]","[\"Sence9_snowball_00007\",\"Sence9_snowball_10003\",\"Sence9_snowball_10007\",\"Sence9_snowball_10011\",\"Sence9_snowball_10015\"]"],["RewardSence9_snowball_00008","[\"Cat18\"]","[\"Sence9_snowball_00008\",\"Sence9_snowball_10004\",\"Sence9_snowball_10008\",\"Sence9_snowball_10012\",\"Sence9_snowball_10016\"]"],["RewardSence9_snowman_00001","[\"Cat17\"]","[\"Sence9_snowman_00001\"]"],["RewardSence9_snowman_00002","[\"Cat21\"]","[\"Sence9_snowman_00002\"]"],["RewardSence9_snowman_00003","[\"Cat27\"]","[\"Sence9_snowman_00003\"]"],["RewardSence9_snowman_00004","[\"Cat22\"]","[\"Sence9_snowman_00004\"]"],["RewardSence9_snowman_00005","[\"Cat24\"]","[\"Sence9_snowman_00005\"]"],["RewardSence9_snowman_00006","[\"Cat28\"]","[\"Sence9_snowman_00006\"]"],["RewardSence9_snowball_10001","[\"Cat3\",\"Cat6\"]","[\"Sence9_snowball_10001\"]"],["RewardSence9_snowball_10002","[\"Cat3\",\"Cat9\"]","[\"Sence9_snowball_10002\"]"],["RewardSence9_snowball_10003","[\"Cat3\",\"Cat14\"]","[\"Sence9_snowball_10003\"]"],["RewardSence9_snowball_10004","[\"Cat3\",\"Cat18\"]","[\"Sence9_snowball_10004\"]"],["RewardSence9_snowball_10005","[\"Cat5\",\"Cat6\"]","[\"Sence9_snowball_10005\"]"],["RewardSence9_snowball_10006","[\"Cat5\",\"Cat9\"]","[\"Sence9_snowball_10006\"]"],["RewardSence9_snowball_10007","[\"Cat5\",\"Cat14\"]","[\"Sence9_snowball_10007\"]"],["RewardSence9_snowball_10008","[\"Cat5\",\"Cat18\"]","[\"Sence9_snowball_10008\"]"],["RewardSence9_snowball_10009","[\"Cat8\",\"Cat6\"]","[\"Sence9_snowball_10009\"]"],["RewardSence9_snowball_10010","[\"Cat8\",\"Cat9\"]","[\"Sence9_snowball_10010\"]"],["RewardSence9_snowball_10011","[\"Cat8\",\"Cat14\"]","[\"Sence9_snowball_10011\"]"],["RewardSence9_snowball_10012","[\"Cat8\",\"Cat18\"]","[\"Sence9_snowball_10012\"]"],["RewardSence9_snowball_10013","[\"Cat13\",\"Cat6\"]","[\"Sence9_snowball_10013\"]"],["RewardSence9_snowball_10014","[\"Cat13\",\"Cat9\"]","[\"Sence9_snowball_10014\"]"],["RewardSence9_snowball_10015","[\"Cat13\",\"Cat14\"]","[\"Sence9_snowball_10015\"]"],["RewardSence9_snowball_10016","[\"Cat13\",\"Cat18\"]","[\"Sence9_snowball_10016\"]"],["RewardSence09_deer_00001","[\"Cat1\"]","[\"Sence9_deer_00001\"]"],["RewardSence09_deer_00002","[\"Cat2\"]","[\"Sence9_deer_00002\"]"],["RewardSence09_deer_00003","[\"Cat4\"]","[\"Sence9_deer_00003\"]"],["RewardSence09_deer_00004","[\"Cat7\"]","[\"Sence9_deer_00004\"]"],["RewardSence09_snowball_00001","[\"Cat3\"]","[\"Sence9_snowball_00001\",\"Sence9_snowball_10001\",\"Sence9_snowball_10002\",\"Sence9_snowball_10003\",\"Sence9_snowball_10004\"]"],["RewardSence09_snowball_00002","[\"Cat5\"]","[\"Sence9_snowball_00002\",\"Sence9_snowball_10005\",\"Sence9_snowball_10006\",\"Sence9_snowball_10007\",\"Sence9_snowball_10008\"]"],["RewardSence09_snowball_00003","[\"Cat8\"]","[\"Sence9_snowball_00003\",\"Sence9_snowball_10009\",\"Sence9_snowball_10010\",\"Sence9_snowball_10011\",\"Sence9_snowball_10012\"]"],["RewardSence09_snowball_00004","[\"Cat13\"]","[\"Sence9_snowball_00004\",\"Sence9_snowball_10013\",\"Sence9_snowball_10014\",\"Sence9_snowball_10015\",\"Sence9_snowball_10016\"]"],["RewardSence09_snowball_00005","[\"Cat6\"]","[\"Sence9_snowball_00005\",\"Sence9_snowball_10001\",\"Sence9_snowball_10005\",\"Sence9_snowball_10009\",\"Sence9_snowball_10013\"]"],["RewardSence09_snowball_00006","[\"Cat9\"]","[\"Sence9_snowball_00006\",\"Sence9_snowball_10002\",\"Sence9_snowball_10006\",\"Sence9_snowball_10010\",\"Sence9_snowball_10014\"]"],["RewardSence09_snowball_00007","[\"Cat14\"]","[\"Sence9_snowball_00007\",\"Sence9_snowball_10003\",\"Sence9_snowball_10007\",\"Sence9_snowball_10011\",\"Sence9_snowball_10015\"]"],["RewardSence09_snowball_00008","[\"Cat18\"]","[\"Sence9_snowball_00008\",\"Sence9_snowball_10004\",\"Sence9_snowball_10008\",\"Sence9_snowball_10012\",\"Sence9_snowball_10016\"]"],["RewardSence09_snowman_00001","[\"Cat17\"]","[\"Sence9_snowman_00001\"]"],["RewardSence09_snowman_00002","[\"Cat21\"]","[\"Sence9_snowman_00002\"]"],["RewardSence09_snowman_00003","[\"Cat27\"]","[\"Sence9_snowman_00003\"]"],["RewardSence09_snowman_00004","[\"Cat22\"]","[\"Sence9_snowman_00004\"]"],["RewardSence09_snowman_00005","[\"Cat24\"]","[\"Sence9_snowman_00005\"]"],["RewardSence09_snowman_00006","[\"Cat28\"]","[\"Sence9_snowman_00006\"]"],["RewardSence09_snowball_10001","[\"Cat3\",\"Cat6\"]","[\"Sence9_snowball_10001\"]"],["RewardSence09_snowball_10002","[\"Cat3\",\"Cat9\"]","[\"Sence9_snowball_10002\"]"],["RewardSence09_snowball_10003","[\"Cat3\",\"Cat14\"]","[\"Sence9_snowball_10003\"]"],["RewardSence09_snowball_10004","[\"Cat3\",\"Cat18\"]","[\"Sence9_snowball_10004\"]"],["RewardSence09_snowball_10005","[\"Cat5\",\"Cat6\"]","[\"Sence9_snowball_10005\"]"],["RewardSence09_snowball_10006","[\"Cat5\",\"Cat9\"]","[\"Sence9_snowball_10006\"]"],["RewardSence09_snowball_10007","[\"Cat5\",\"Cat14\"]","[\"Sence9_snowball_10007\"]"],["RewardSence09_snowball_10008","[\"Cat5\",\"Cat18\"]","[\"Sence9_snowball_10008\"]"],["RewardSence09_snowball_10009","[\"Cat8\",\"Cat6\"]","[\"Sence9_snowball_10009\"]"],["RewardSence09_snowball_10010","[\"Cat8\",\"Cat9\"]","[\"Sence9_snowball_10010\"]"],["RewardSence09_snowball_10011","[\"Cat8\",\"Cat14\"]","[\"Sence9_snowball_10011\"]"],["RewardSence09_snowball_10012","[\"Cat8\",\"Cat18\"]","[\"Sence9_snowball_10012\"]"],["RewardSence09_snowball_10013","[\"Cat13\",\"Cat6\"]","[\"Sence9_snowball_10013\"]"],["RewardSence09_snowball_10014","[\"Cat13\",\"Cat9\"]","[\"Sence9_snowball_10014\"]"],["RewardSence09_snowball_10015","[\"Cat13\",\"Cat14\"]","[\"Sence9_snowball_10015\"]"],["RewardSence09_snowball_10016","[\"Cat13\",\"Cat18\"]","[\"Sence9_snowball_10016\"]"],["RewardSence10_00001","[\"Cat11\"]","[\"Sence10_00001\",\"Sence10_10001\"]"],["RewardSence10_00002","[\"Cat15\"]","[\"Sence10_00002\",\"Sence10_10002\"]"],["RewardSence10_00003","[\"Cat19\"]","[\"Sence10_00003\",\"Sence10_10003\"]"],["RewardSence10_00004","[\"Cat23\"]","[\"Sence10_00004\",\"Sence10_10004\"]"],["RewardSence10_00005","[\"Cat26\"]","[\"Sence10_00005\",\"Sence10_10005\"]"],["RewardSence10_00006","[\"Cat29\"]","[\"Sence10_00006\",\"Sence10_10006\"]"],["RewardSence10_00007","[\"Cat30\"]","[\"Sence10_00007\",\"Sence10_10007\"]"],["RewardSence10_10001","[\"Cat11\",\"Cat6\"]","[\"Sence10_10001\"]"],["RewardSence10_10002","[\"Cat15\",\"Cat9\"]","[\"Sence10_10002\"]"],["RewardSence10_10003","[\"Cat19\",\"Cat14\"]","[\"Sence10_10003\"]"],["RewardSence10_10004","[\"Cat23\",\"Cat18\"]","[\"Sence10_10004\"]"],["RewardSence10_10005","[\"Cat26\",\"Cat22\"]","[\"Sence10_10005\"]"],["RewardSence10_10006","[\"Cat29\",\"Cat24\"]","[\"Sence10_10006\"]"],["RewardSence10_10007","[\"Cat30\",\"Cat28\"]","[\"Sence10_10007\"]"]];

    var NauticalProject = [["Id","ShopItems","Type","Weight","BuildingLimit","StarLimit","Cost","AutoIncome","SpecialValue"],["Project011","xiguazhi","Project01",100,"3000",600,40,7,""],["Project012","xiguazhi","Project01",100,"3000",600,100,10,"5000"],["Project013","xiguazhi","Project01",100,"3000",750,120,15,"10000"],["Project021","chonglang","Project02",100,"3000",600,50,8,""],["Project022","chonglang","Project02",100,"3000",660,110,11,"5000"],["Project023","chonglang","Project02",100,"3000",840,140,16,"10000"],["Project031","suidao","Project03",100,"3000",600,80,8,""],["Project032","suidao","Project03",100,"3000",720,120,11,"5000"],["Project033","suidao","Project03",100,"3000",930,80,16,"10000"],["Project041","xishishe","Project04",100,"3000",630,100,9,""],["Project042","xishishe","Project04",100,"3000",810,130,13,"5000"],["Project043","xishishe","Project04",100,"3000",1050,170,19,"10000"],["Project051","paiqiu","Project05",100,"3000",690,110,9,""],["Project052","paiqiu","Project05",100,"3000",900,150,13,"5000"],["Project053","paiqiu","Project05",100,"3001",1130,190,19,"10000"],["Project061","diaochuang","Project06",100,"3000",780,130,9,""],["Project062","diaochuang","Project06",100,"3000",990,160,13,"5000"],["Project063","diaochuang","Project06",100,"3001",1220,210,19,"10000"],["Project071","yezhi","Project07",100,"3000",870,140,10,""],["Project072","yezhi","Project07",100,"3001",1100,190,14,"5000"],["Project073","yezhi","Project07",100,"3001",1320,220,20,"10000"],["Project081","motuo","Project08",100,"3000",960,80,10,""],["Project082","motuo","Project08",100,"3001",1190,200,14,"5000"],["Project083","motuo","Project08",100,"3001",1440,120,20,"10000"],["Project091","riguangyu","Project09",100,"3001",1100,170,10,""],["Project092","riguangyu","Project09",100,"3001",1280,220,14,"5000"],["Project093","riguangyu","Project09",100,"3001",1560,130,21,"10000"],["Project101","xianxiachuan","Project10",100,"3001",1100,180,10,""],["Project102","xianxiachuan","Project10",100,"3001",1400,230,14,"5000"],["Project103","xianxiachuan","Project10",100,"3001",1680,270,21,"10000"],["Project111","anmo","Project11",100,"3001",1100,180,10,""],["Project112","anmo","Project11",100,"3001",1520,250,14,"5000"],["Project113","anmo","Project11",100,"3002",1780,300,21,"10000"],["Project121","shengyu","Project12",100,"3001",1160,140,11,""],["Project122","shengyu","Project12",100,"3001",1640,260,15,"5000"],["Project123","shengyu","Project12",100,"3002",1920,330,22,"10000"],["Project131","dulunche","Project13",100,"3001",1250,210,11,""],["Project132","dulunche","Project13",100,"3002",1740,290,16,"5000"],["Project133","dulunche","Project13",100,"3002",2140,380,23,"10000"],["Project141","luoriyuhui","Project14",100,"3001",1360,230,11,""],["Project142","luoriyuhui","Project14",100,"3002",1860,320,16,"5000"],["Project143","luoriyuhui","Project14",100,"3002",2380,440,23,"10000"],["Project151","shadiaoyishuzhan","Project15",100,"3001",1480,240,11,""],["Project152","shadiaoyishuzhan","Project15",100,"3002",2060,360,16,"5000"],["Project153","shadiaoyishuzhan","Project15",100,"3002",2620,500,23,"10000"],["Project161","pijiu","Project16",100,"3001",1600,260,11,""],["Project162","pijiu","Project16",100,"3002",2300,420,16,"5000"],["Project163","pijiu","Project16",100,"3002",2920,650,23,"10000"],["Project171","huaxiangyi","Project17",100,"3002",1700,280,11,""],["Project172","huaxiangyi","Project17",100,"3002",2540,340,16,"5000"],["Project173","huaxiangyi","Project17",100,"3002",3120,750,23,"10000"],["Project181","fangyingting","Project18",100,"3002",1820,310,11,""],["Project182","fangyingting","Project18",100,"3002",2820,600,16,"5000"],["Project183","fangyingting","Project18",100,"3002",3520,1000,23,"10000"],["Project191","buding","Project19",100,"3002",1980,340,11,""],["Project192","buding","Project19",100,"3002",3020,700,16,"5000"],["Project193","buding","Project19",100,"3002",3820,1200,24,"10000"],["Project201","qianshui","Project20",100,"3002",2220,400,12,""],["Project202","qianshui","Project20",100,"3002",3370,900,17,"5000"],["Project203","qianshui","Project20",100,"3002",4320,1600,25,"10000"],["Project211","mingxiang","Project21",100,"3002",2460,460,12,""],["Project212","mingxiang","Project21",100,"3002",3670,1100,17,"5000"],["Project213","mingxiang","Project21",100,"3002",4720,2000,25,"10000"],["Project221","jiweijiu","Project22",100,"3002",2720,550,12,""],["Project222","jiweijiu","Project22",100,"3002",4120,1400,17,"5000"],["Project223","jiweijiu","Project22",100,"3002",5420,2600,25,"10000"],["Project231","shaokao","Project23",100,"3002",3220,800,12,""],["Project232","shaokao","Project23",100,"3002",4520,1800,17,"5000"],["Project233","shaokao","Project23",100,"3002",5920,3000,25,"10000"],["Project241","kuaiting","Project24",100,"3002",3970,1300,12,""],["Project242","kuaiting","Project24",100,"3002",4920,2200,17,"5000"],["Project243","kuaiting","Project24",100,"3002",6420,3400,25,"10000"]];

    var NauticalProjectList = [["Id","ProjectList","Type","Order"],["Project1","[{\"lv\":1,\"ProjectId\":\"Project011\"},{\"lv\":2,\"ProjectId\":\"Project012\"},{\"lv\":3,\"ProjectId\":\"Project013\"}]",1,1],["Project2","[{\"lv\":1,\"ProjectId\":\"Project021\"},{\"lv\":2,\"ProjectId\":\"Project022\"},{\"lv\":3,\"ProjectId\":\"Project023\"}]",1,2],["Project3","[{\"lv\":1,\"ProjectId\":\"Project031\"},{\"lv\":2,\"ProjectId\":\"Project032\"},{\"lv\":3,\"ProjectId\":\"Project033\"}]",1,3],["Project4","[{\"lv\":1,\"ProjectId\":\"Project041\"},{\"lv\":2,\"ProjectId\":\"Project042\"},{\"lv\":3,\"ProjectId\":\"Project043\"}]",1,4],["Project5","[{\"lv\":1,\"ProjectId\":\"Project051\"},{\"lv\":2,\"ProjectId\":\"Project052\"},{\"lv\":3,\"ProjectId\":\"Project053\"}]",1,5],["Project6","[{\"lv\":1,\"ProjectId\":\"Project061\"},{\"lv\":2,\"ProjectId\":\"Project062\"},{\"lv\":3,\"ProjectId\":\"Project063\"}]",1,6],["Project7","[{\"lv\":1,\"ProjectId\":\"Project071\"},{\"lv\":2,\"ProjectId\":\"Project072\"},{\"lv\":3,\"ProjectId\":\"Project073\"}]",1,7],["Project8","[{\"lv\":1,\"ProjectId\":\"Project081\"},{\"lv\":2,\"ProjectId\":\"Project082\"},{\"lv\":3,\"ProjectId\":\"Project083\"}]",1,8],["Project9","[{\"lv\":1,\"ProjectId\":\"Project091\"},{\"lv\":2,\"ProjectId\":\"Project092\"},{\"lv\":3,\"ProjectId\":\"Project093\"}]",2,1],["Project10","[{\"lv\":1,\"ProjectId\":\"Project101\"},{\"lv\":2,\"ProjectId\":\"Project102\"},{\"lv\":3,\"ProjectId\":\"Project103\"}]",2,2],["Project11","[{\"lv\":1,\"ProjectId\":\"Project111\"},{\"lv\":2,\"ProjectId\":\"Project112\"},{\"lv\":3,\"ProjectId\":\"Project113\"}]",2,3],["Project12","[{\"lv\":1,\"ProjectId\":\"Project121\"},{\"lv\":2,\"ProjectId\":\"Project122\"},{\"lv\":3,\"ProjectId\":\"Project123\"}]",2,4],["Project13","[{\"lv\":1,\"ProjectId\":\"Project131\"},{\"lv\":2,\"ProjectId\":\"Project132\"},{\"lv\":3,\"ProjectId\":\"Project133\"}]",2,5],["Project14","[{\"lv\":1,\"ProjectId\":\"Project141\"},{\"lv\":2,\"ProjectId\":\"Project142\"},{\"lv\":3,\"ProjectId\":\"Project143\"}]",2,6],["Project15","[{\"lv\":1,\"ProjectId\":\"Project151\"},{\"lv\":2,\"ProjectId\":\"Project152\"},{\"lv\":3,\"ProjectId\":\"Project153\"}]",2,7],["Project16","[{\"lv\":1,\"ProjectId\":\"Project161\"},{\"lv\":2,\"ProjectId\":\"Project162\"},{\"lv\":3,\"ProjectId\":\"Project163\"}]",2,8],["Project17","[{\"lv\":1,\"ProjectId\":\"Project171\"},{\"lv\":2,\"ProjectId\":\"Project172\"},{\"lv\":3,\"ProjectId\":\"Project173\"}]",3,1],["Project18","[{\"lv\":1,\"ProjectId\":\"Project181\"},{\"lv\":2,\"ProjectId\":\"Project182\"},{\"lv\":3,\"ProjectId\":\"Project183\"}]",3,2],["Project19","[{\"lv\":1,\"ProjectId\":\"Project191\"},{\"lv\":2,\"ProjectId\":\"Project192\"},{\"lv\":3,\"ProjectId\":\"Project193\"}]",3,3],["Project20","[{\"lv\":1,\"ProjectId\":\"Project201\"},{\"lv\":2,\"ProjectId\":\"Project202\"},{\"lv\":3,\"ProjectId\":\"Project203\"}]",3,4],["Project21","[{\"lv\":1,\"ProjectId\":\"Project211\"},{\"lv\":2,\"ProjectId\":\"Project212\"},{\"lv\":3,\"ProjectId\":\"Project213\"}]",3,5],["Project22","[{\"lv\":1,\"ProjectId\":\"Project221\"},{\"lv\":2,\"ProjectId\":\"Project222\"},{\"lv\":3,\"ProjectId\":\"Project223\"}]",3,6],["Project23","[{\"lv\":1,\"ProjectId\":\"Project231\"},{\"lv\":2,\"ProjectId\":\"Project232\"},{\"lv\":3,\"ProjectId\":\"Project233\"}]",3,7],["Project24","[{\"lv\":1,\"ProjectId\":\"Project241\"},{\"lv\":2,\"ProjectId\":\"Project242\"},{\"lv\":3,\"ProjectId\":\"Project243\"}]",3,8]];

    var NauticalTools = [["Id","Name","Desc","Icon","PointRate","IntimacyRate","Rank","Requirement","ProduceDesc"],["CatItem1","tid#CatItemName1","tid#CatItemDesc1","nautical_tool/lvxinggonglue.png",0.2,0.0,1,"tid#CatItemRequire1","tid#UI_211"],["CatItem2","tid#CatItemName2","tid#CatItemDesc2","nautical_tool/zhinanzhen.png",0.5,0.0,2,"tid#CatItemRequire2","tid#UI_211"],["CatItem3","tid#CatItemName3","tid#CatItemDesc3","nautical_tool/yangmaozhinan.png",0.35,0.2,3,"tid#CatItemRequire3","tid#UI_211"],["CatItem4","tid#CatItemName4","tid#CatItemDesc4","nautical_tool/menghuandao.png",0.65,0.35,3,"tid#CatItemRequire4","tid#UI_211"],["CatItem5","tid#CatItemName5","tid#CatItemDesc5","nautical_tool/kongtinglvxing.png",0.8,0.3,3,"tid#CatItemRequire5","tid#UI_211"],["CatItem6","tid#CatItemName6","tid#CatItemDesc6","nautical_tool/yazhouqianzheng.png",0.85,0.15,3,"tid#CatItemRequire6","tid#UI_211"]];

    var OrderSystem = [["Id","CourierStaff","Order","AdvertisingStaff","SpendTime","RewardBase","RandomLebel","LvUpLimit","LvUpCost","StaffCar"],["1",1,2,0,180,14000,"{\"sweet\":600,\"smart\":200,\"lazy\":100,\"timid\":100,\"sticky\":600,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":200,\"meng\":100,\"naughty\":100}",2,3000,1],["2",1,3,0,270,18000,"{\"sweet\":300,\"smart\":300,\"lazy\":100,\"timid\":100,\"sticky\":300,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":300,\"meng\":100,\"naughty\":100}",5,6000,1],["3",1,3,1,360,25000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",10,10000,1],["4",1,4,1,510,35000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",15,15000,1],["5",1,4,2,660,45000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",20,20000,1],["6",2,4,2,840,55000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",25,30000,2],["7",2,5,2,1020,65000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",30,40000,2],["8",2,5,3,1200,90000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",40,70000,2],["9",2,6,3,1380,120000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",50,120000,2],["10",2,6,4,1560,160000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",60,200000,2],["11",3,6,4,1740,200000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",70,300000,3],["12",3,7,4,1920,240000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",80,400000,3],["13",3,7,5,2100,280000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",90,600000,3],["14",3,8,5,2280,320000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",100,800000,3],["15",3,8,5,2460,360000,"{\"sweet\":100,\"smart\":100,\"lazy\":100,\"timid\":100,\"sticky\":100,\"lively\":100,\"aojiao\":100,\"brave\":100,\"cold\":100,\"elegant\":100,\"meng\":100,\"naughty\":100}",-1,0,3]];

    var Scene$1 = [["Id","Name","Desc1","goto","BuildingList","Unlock"],["1","tid#sceneName1","tid#sceneName1",["left:3","right:2"],"[]","[{\"Star\":0}]"],["2","tid#sceneName2","tid#sceneName2",["left:1"],"[]","[{\"Star\":150}]"],["3","tid#sceneName3","tid#sceneName3",["right:1"],"[]","[{\"Star\":400}]"]];

    var Sign$1 = [["Id","Reward","Bubble"],["1","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":40}}",null],["2","{\"MainReward\":{\"type\":\"Staff\",\"id\":\"StaffUnlock\"},\"SubReward\":{\"type\":\"Staff\",\"id\":\"SkillAmount\",\"amount\":1}}","tid#SignBubble_3"],["3","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":60}}",null],["4","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":60},\"SubReward\":{\"type\":\"Staff\",\"id\":\"SkillAmount\",\"amount\":1}}","tid#SignBubble_5"],["5","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":80}}",null],["6","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":80},\"SubReward\":{\"type\":\"Staff\",\"id\":\"SkillAmount\",\"amount\":1}}","tid#SignBubble_7"],["7","{\"MainReward\":{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":100}}",null]];

    var Sound = [["Id","Type","Sound"],["Sound_bgm1",1,"sound/bgm1.mp3"],["Sound_bgm2",1,"sound/bgm2.mp3"],["Sound_bgm3",1,"sound/bgm3.mp3"],["Sound_setSail",2,"sound/setSail.mp3"],["Sound_buyItemSuccess",2,"sound/buyItemSuccess.mp3"],["Sound_placeBuilding",2,"sound/placeBuilding.mp3"],["Sound_clickReward",2,"sound/clickReward.mp3"],["Sound_clickSound",2,"sound/clickSound.mp3"],["Sound_beachWave",2,"sound/beachWave.mp3"],["Sound_fishingSuccess",2,"sound/fishingSuccess.mp3"],["Sound_fishingPerfectSuccess",2,"sound/fishingPerfectSuccess.mp3"],["Sound_catMeow_1",2,"sound/catMeow_1.mp3"],["Sound_catMeow_2",2,"sound/catMeow_2.mp3"],["Sound_catMeow_3",2,"sound/catMeow_3.mp3"]];

    var SpecialGame = [["Id","Type","RewardConfig","Name","Desc1","Desc2"],["GameMora1_1","MORA","{\"WinRewardRate\":30,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":1700}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora1_2","MORA","{\"WinRewardRate\":40,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2000}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora1_3","MORA","{\"WinRewardRate\":50,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2300}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora2_1","MORA","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora2_2","MORA","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora3","MORA","{\"WinRewardRate\":70,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora4","MORA","{\"WinRewardRate\":80,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora5","MORA","{\"WinRewardRate\":90,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora6","MORA","{\"WinRewardRate\":100,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameMora7","MORA","{\"WinRewardRate\":110,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":3500}],\"FailReward\":[]}","tid#GameMoraRName1",["tid#StartGameMora1_1","tid#StartGameMora1_2"],["tid#EndGameMora1_1","tid#EndGameMora1_2"]],["GameCoin1_1","COIN","{\"WinRewardRate\":30,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":1700}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin1_2","COIN","{\"WinRewardRate\":40,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin1_3","COIN","{\"WinRewardRate\":50,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2300}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin2_1","COIN","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin2_2","COIN","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin3","COIN","{\"WinRewardRate\":70,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin4","COIN","{\"WinRewardRate\":80,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin5","COIN","{\"WinRewardRate\":90,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin6","COIN","{\"WinRewardRate\":100,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameCoin7","COIN","{\"WinRewardRate\":110,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}","tid#GameCoinRName1",["tid#StartGameCoin1_1","tid#StartGameCoin1_2"],["tid#EndGameCoin1_1","tid#EndGameCoin1_2"]],["GameStrength1_2","STRENGTH","{\"WinRewardRate\":40,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2000}],\"FailReward\":[]}",null,[],[]],["GameStrength1_3","STRENGTH","{\"WinRewardRate\":50,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":2300}],\"FailReward\":[]}",null,[],[]],["GameStrength2_1","STRENGTH","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength2_2","STRENGTH","{\"WinRewardRate\":60,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength3","STRENGTH","{\"WinRewardRate\":70,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength4","STRENGTH","{\"WinRewardRate\":80,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength5","STRENGTH","{\"WinRewardRate\":90,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength6","STRENGTH","{\"WinRewardRate\":100,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]],["GameStrength7","STRENGTH","{\"WinRewardRate\":110,\"WinReward\":[{\"type\":\"Currency\",\"id\":\"FISH\",\"amount\":4000}],\"FailReward\":[]}",null,[],[]]];

    var SpecialGameGroup = [["Id","GameGroup","ConditionValue","RegistrationDate"],["GameGroup1",["GameMora1_3","GameCoin1_3","GameStrength1_3"],"{\"CATS_INTIVE\":[[60,70],[240,250],[550,560]]}",[1,1]],["GameGroup2",["GameMora2_2","GameCoin2_2","GameStrength2_2"],"{\"CATS_INTIVE\":[[60,70],[240,250],[550,560]]}",[2,2]],["GameGroup3",["GameMora3","GameCoin3","GameStrength3"],"{\"CATS_INTIVE\":[[70,80],[240,250],[550,560]]}",[3,3]],["GameGroup4",["GameMora4","GameCoin4","GameStrength4"],"{\"CATS_INTIVE\":[[70,80],[240,250],[550,560]]}",[4,4]],["GameGroup5",["GameMora5","GameCoin5","GameStrength5"],"{\"CATS_INTIVE\":[[70,80],[240,250],[550,560]]}",[5,5]],["GameGroup6",["GameMora6","GameCoin6","GameStrength6"],"{\"CATS_INTIVE\":[[70,80],[240,250],[550,560]]}",[6,6]],["GameGroup7",["GameMora7","GameCoin7","GameStrength7"],"{\"CATS_INTIVE\":[[70,80],[240,250],[550,560]]}",[7]]];

    var SpecialGameStrength = [["Id","ClickSpeed0","ClickSpeed1","Speed2","Speed3"],["1",[2,4],[3,5],[2,4],[4,8]],["2",[3,5],[4,6],[3,5],[5,9]],["3",[4,6],[5,7],[4,6],[6,10]],["4",[5,7],[6,8],[5,7],[7,11]],["5",[5,7],[7,9],[6,8],[8,12]],["6",[5,7],[8,10],[7,9],[9,13]],["7",[5,7],[9,11],[8,10],[10,14]],["8",[5,7],[10,12],[9,11],[11,15]],["9",[5,7],[10,12],[9,11],[12,16]],["10",[5,7],[10,12],[9,11],[13,17]]];

    var SpecialGameStrengthRole = [["Id","Name","Order","FirstStartDialog","StartDialog","BigWinDialog","WinDialog","FailDialog","Armband"],["StrengthCat1","tid#StrengthCatName1",1,"[[\"tid#StartSGame1_1\",\"tid#StartSGame1_2\"]]","[[\"tid#StartSGame1_3\",\"tid#StartSGame1_4\"]]","[[\"tid#EndSGame1_1\",\"tid#EndSGame1_2\"]]","[[\"tid#EndSGame1_3\",\"tid#EndSGame1_4\"]]","[[\"tid#EndSGame1_5\",\"tid#EndSGame1_6\"]]","1.png"],["StrengthCat2","tid#StrengthCatName2",2,"[[\"tid#StartSGame2_1\",\"tid#StartSGame2_2\"]]","[[\"tid#StartSGame2_3\",\"tid#StartSGame2_4\"]]","[[\"tid#EndSGame2_1\",\"tid#EndSGame2_2\"]]","[[\"tid#EndSGame2_3\",\"tid#EndSGame2_4\"]]","[[\"tid#EndSGame2_5\",\"tid#EndSGame2_6\"]]","2.png"],["StrengthCat3","tid#StrengthCatName3",3,"[[\"tid#StartSGame3_1\",\"tid#StartSGame3_2\"]]","[[\"tid#StartSGame3_3\",\"tid#StartSGame3_4\"]]","[[\"tid#EndSGame3_1\",\"tid#EndSGame3_2\"]]","[[\"tid#EndSGame3_3\",\"tid#EndSGame3_4\"]]","[[\"tid#EndSGame3_5\",\"tid#EndSGame3_6\"]]","3.png"],["StrengthCat4","tid#StrengthCatName4",4,"[[\"tid#StartSGame4_1\",\"tid#StartSGame4_2\"]]","[[\"tid#StartSGame4_3\",\"tid#StartSGame4_4\"]]","[[\"tid#EndSGame4_1\",\"tid#EndSGame4_2\"]]","[[\"tid#EndSGame4_3\",\"tid#EndSGame4_4\"]]","[[\"tid#EndSGame4_5\",\"tid#EndSGame4_6\"]]","4.png"],["StrengthCat5","tid#StrengthCatName5",5,"[[\"tid#StartSGame5_1\",\"tid#StartSGame5_2\"]]","[[\"tid#StartSGame5_3\",\"tid#StartSGame5_4\"]]","[[\"tid#EndSGame5_1\",\"tid#EndSGame5_2\"]]","[[\"tid#EndSGame5_3\",\"tid#EndSGame5_4\"]]","[[\"tid#EndSGame5_5\",\"tid#EndSGame5_6\"]]","5.png"],["StrengthCat6","tid#StrengthCatName6",6,"[[\"tid#StartSGame6_1\",\"tid#StartSGame6_2\"]]","[[\"tid#StartSGame6_3\",\"tid#StartSGame6_4\"]]","[[\"tid#EndSGame6_1\",\"tid#EndSGame6_2\"]]","[[\"tid#EndSGame6_3\",\"tid#EndSGame6_4\"]]","[[\"tid#EndSGame6_5\",\"tid#EndSGame6_6\"]]","6.png"],["StrengthCat7","tid#StrengthCatName7",7,"[[\"tid#StartSGame7_1\",\"tid#StartSGame7_2\"]]","[[\"tid#StartSGame7_3\",\"tid#StartSGame7_4\"]]","[[\"tid#EndSGame7_1\",\"tid#EndSGame7_2\"]]","[[\"tid#EndSGame7_3\",\"tid#EndSGame7_4\"]]","[[\"tid#EndSGame7_5\",\"tid#EndSGame7_6\"]]","7.png"],["StrengthCat8","tid#StrengthCatName8",8,"[[\"tid#StartSGame8_1\",\"tid#StartSGame8_2\"]]","[[\"tid#StartSGame8_3\",\"tid#StartSGame8_4\"]]","[[\"tid#EndSGame8_1\",\"tid#EndSGame8_2\"]]","[[\"tid#EndSGame8_3\",\"tid#EndSGame8_4\"]]","[[\"tid#EndSGame8_5\",\"tid#EndSGame8_6\"]]","8.png"]];

    var Staff$1 = [["Id","Name","Desc","NextStaff","Price","SkillLvLimite","LvUpCondition","Image","SkillGroup"],["Staff1_1","tid#NameStaff1_1","tid#DescStaff1_1","Staff1_2",15000,5,5,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"],["Staff1_2","tid#NameStaff1_2","tid#DescStaff1_2","Staff1_3",30000,10,15,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"],["Staff1_3","tid#NameStaff1_3","tid#DescStaff1_3","Staff1_4",80000,15,30,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"],["Staff1_4","tid#NameStaff1_4","tid#DescStaff1_4","Staff1_5",150000,20,50,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"],["Staff1_5","tid#NameStaff1_5","tid#DescStaff1_5","Staff1_6",500000,25,70,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"],["Staff1_6","tid#NameStaff1_6","tid#DescStaff1_6",null,null,30,null,"cat_collect/normal/mao1_1.png","[\"SkillGroup_1\",\"SkillGroup_2\",\"SkillGroup_3\"]"]];

    var StaffGroup = [["Id","StaffGroup"],["StaffGroup_1","[\"Staff1_1\",\"Staff1_2\",\"Staff1_3\",\"Staff1_4\",\"Staff1_5\",\"Staff1_6\"]"]];

    var StaffSkill = [["Id","Effect","SkillData","LvUpCost","LvUpTime","Level"],["Skill1_001","FishGet","{\"CDTime\":320,\"EffectTime\":30}",3000,20,1],["Skill1_002","FishGet","{\"CDTime\":310,\"EffectTime\":40}",5000,60,2],["Skill1_003","FishGet","{\"CDTime\":300,\"EffectTime\":50}",8000,180,3],["Skill1_004","FishGet","{\"CDTime\":290,\"EffectTime\":60}",10000,480,4],["Skill1_005","FishGet","{\"CDTime\":280,\"EffectTime\":70}",12000,960,5],["Skill1_006","FishGet","{\"CDTime\":270,\"EffectTime\":80}",14000,1920,6],["Skill1_007","FishGet","{\"CDTime\":260,\"EffectTime\":90}",16000,3600,7],["Skill1_008","FishGet","{\"CDTime\":250,\"EffectTime\":100}",18000,5400,8],["Skill1_009","FishGet","{\"CDTime\":240,\"EffectTime\":110}",20000,7200,9],["Skill1_010","FishGet","{\"CDTime\":230,\"EffectTime\":120}",25000,14400,10],["Skill1_011","FishGet","{\"CDTime\":220,\"EffectTime\":125}",30000,21600,11],["Skill1_012","FishGet","{\"CDTime\":210,\"EffectTime\":130}",35000,28800,12],["Skill1_013","FishGet","{\"CDTime\":200,\"EffectTime\":135}",40000,36000,13],["Skill1_014","FishGet","{\"CDTime\":190,\"EffectTime\":140}",50000,43200,14],["Skill1_015","FishGet","{\"CDTime\":180,\"EffectTime\":145}",60000,50400,15],["Skill1_016","FishGet","{\"CDTime\":170,\"EffectTime\":150}",70000,57600,16],["Skill1_017","FishGet","{\"CDTime\":160,\"EffectTime\":155}",80000,64800,17],["Skill1_018","FishGet","{\"CDTime\":150,\"EffectTime\":160}",100000,72000,18],["Skill1_019","FishGet","{\"CDTime\":140,\"EffectTime\":165}",120000,86400,19],["Skill1_020","FishGet","{\"CDTime\":130,\"EffectTime\":170}",140000,100800,20],["Skill1_021","FishGet","{\"CDTime\":125,\"EffectTime\":175}",180000,115200,21],["Skill1_022","FishGet","{\"CDTime\":120,\"EffectTime\":180}",220000,129600,22],["Skill1_023","FishGet","{\"CDTime\":115,\"EffectTime\":185}",260000,144000,23],["Skill1_024","FishGet","{\"CDTime\":110,\"EffectTime\":190}",300000,172800,24],["Skill1_025","FishGet","{\"CDTime\":105,\"EffectTime\":195}",400000,201600,25],["Skill1_026","FishGet","{\"CDTime\":100,\"EffectTime\":200}",600000,230400,26],["Skill1_027","FishGet","{\"CDTime\":95,\"EffectTime\":205}",800000,259200,27],["Skill1_028","FishGet","{\"CDTime\":90,\"EffectTime\":210}",1000000,288000,28],["Skill1_029","FishGet","{\"CDTime\":85,\"EffectTime\":215}",1500000,316800,29],["Skill1_030","FishGet","{\"CDTime\":80,\"EffectTime\":220}",0,0,30],["Skill2_001","Order","{\"CDTime\":300,\"EffectTime\":40}",3000,20,1],["Skill2_002","Order","{\"CDTime\":290,\"EffectTime\":50}",5000,60,2],["Skill2_003","Order","{\"CDTime\":280,\"EffectTime\":60}",8000,180,3],["Skill2_004","Order","{\"CDTime\":270,\"EffectTime\":70}",10000,480,4],["Skill2_005","Order","{\"CDTime\":260,\"EffectTime\":80}",12000,960,5],["Skill2_006","Order","{\"CDTime\":250,\"EffectTime\":90}",14000,1920,6],["Skill2_007","Order","{\"CDTime\":240,\"EffectTime\":100}",16000,3600,7],["Skill2_008","Order","{\"CDTime\":230,\"EffectTime\":110}",18000,5400,8],["Skill2_009","Order","{\"CDTime\":220,\"EffectTime\":120}",20000,7200,9],["Skill2_010","Order","{\"CDTime\":210,\"EffectTime\":130}",25000,14400,10],["Skill2_011","Order","{\"CDTime\":200,\"EffectTime\":135}",30000,21600,11],["Skill2_012","Order","{\"CDTime\":190,\"EffectTime\":140}",35000,28800,12],["Skill2_013","Order","{\"CDTime\":180,\"EffectTime\":145}",40000,36000,13],["Skill2_014","Order","{\"CDTime\":170,\"EffectTime\":150}",50000,43200,14],["Skill2_015","Order","{\"CDTime\":160,\"EffectTime\":155}",60000,50400,15],["Skill2_016","Order","{\"CDTime\":150,\"EffectTime\":160}",70000,57600,16],["Skill2_017","Order","{\"CDTime\":140,\"EffectTime\":165}",80000,64800,17],["Skill2_018","Order","{\"CDTime\":130,\"EffectTime\":170}",100000,72000,18],["Skill2_019","Order","{\"CDTime\":120,\"EffectTime\":175}",120000,86400,19],["Skill2_020","Order","{\"CDTime\":110,\"EffectTime\":180}",140000,100800,20],["Skill2_021","Order","{\"CDTime\":105,\"EffectTime\":185}",180000,115200,21],["Skill2_022","Order","{\"CDTime\":100,\"EffectTime\":190}",220000,129600,22],["Skill2_023","Order","{\"CDTime\":95,\"EffectTime\":195}",260000,144000,23],["Skill2_024","Order","{\"CDTime\":90,\"EffectTime\":200}",300000,172800,24],["Skill2_025","Order","{\"CDTime\":85,\"EffectTime\":205}",400000,201600,25],["Skill2_026","Order","{\"CDTime\":80,\"EffectTime\":210}",600000,230400,26],["Skill2_027","Order","{\"CDTime\":75,\"EffectTime\":215}",800000,259200,27],["Skill2_028","Order","{\"CDTime\":70,\"EffectTime\":220}",1000000,288000,28],["Skill2_029","Order","{\"CDTime\":65,\"EffectTime\":225}",1500000,316800,29],["Skill2_030","Order","{\"CDTime\":60,\"EffectTime\":230}",0,0,30],["Skill3_001","Invite","{\"CDTime\":390,\"EffectTime\":9}",3000,20,1],["Skill3_002","Invite","{\"CDTime\":380,\"EffectTime\":9}",5000,60,2],["Skill3_003","Invite","{\"CDTime\":370,\"EffectTime\":9}",8000,180,3],["Skill3_004","Invite","{\"CDTime\":360,\"EffectTime\":9}",10000,480,4],["Skill3_005","Invite","{\"CDTime\":350,\"EffectTime\":9}",12000,960,5],["Skill3_006","Invite","{\"CDTime\":345,\"EffectTime\":12}",14000,1920,6],["Skill3_007","Invite","{\"CDTime\":340,\"EffectTime\":12}",16000,3600,7],["Skill3_008","Invite","{\"CDTime\":335,\"EffectTime\":12}",18000,5400,8],["Skill3_009","Invite","{\"CDTime\":330,\"EffectTime\":12}",20000,7200,9],["Skill3_010","Invite","{\"CDTime\":325,\"EffectTime\":12}",25000,14400,10],["Skill3_011","Invite","{\"CDTime\":320,\"EffectTime\":15}",30000,21600,11],["Skill3_012","Invite","{\"CDTime\":315,\"EffectTime\":15}",35000,28800,12],["Skill3_013","Invite","{\"CDTime\":310,\"EffectTime\":15}",40000,36000,13],["Skill3_014","Invite","{\"CDTime\":305,\"EffectTime\":15}",50000,43200,14],["Skill3_015","Invite","{\"CDTime\":300,\"EffectTime\":15}",60000,50400,15],["Skill3_016","Invite","{\"CDTime\":295,\"EffectTime\":18}",70000,57600,16],["Skill3_017","Invite","{\"CDTime\":290,\"EffectTime\":18}",80000,64800,17],["Skill3_018","Invite","{\"CDTime\":285,\"EffectTime\":18}",100000,72000,18],["Skill3_019","Invite","{\"CDTime\":280,\"EffectTime\":18}",120000,86400,19],["Skill3_020","Invite","{\"CDTime\":275,\"EffectTime\":18}",140000,100800,20],["Skill3_021","Invite","{\"CDTime\":270,\"EffectTime\":21}",180000,115200,21],["Skill3_022","Invite","{\"CDTime\":265,\"EffectTime\":21}",220000,129600,22],["Skill3_023","Invite","{\"CDTime\":260,\"EffectTime\":21}",260000,144000,23],["Skill3_024","Invite","{\"CDTime\":255,\"EffectTime\":21}",300000,172800,24],["Skill3_025","Invite","{\"CDTime\":250,\"EffectTime\":21}",400000,201600,25],["Skill3_026","Invite","{\"CDTime\":245,\"EffectTime\":21}",600000,230400,26],["Skill3_027","Invite","{\"CDTime\":240,\"EffectTime\":21}",800000,259200,27],["Skill3_028","Invite","{\"CDTime\":235,\"EffectTime\":21}",1000000,288000,28],["Skill3_029","Invite","{\"CDTime\":230,\"EffectTime\":21}",1500000,316800,29],["Skill3_030","Invite","{\"CDTime\":225,\"EffectTime\":21}",0,0,30]];

    var StaffSkillGroup = [["Id","Name","Desc","Group1"],["SkillGroup_1","tid#NameSkillGroup_1","tid#DescSkillGroup_1","[\"Skill1_001\",\"Skill1_002\",\"Skill1_003\",\"Skill1_004\",\"Skill1_005\",\"Skill1_006\",\"Skill1_007\",\"Skill1_008\",\"Skill1_009\",\"Skill1_010\",\"Skill1_011\",\"Skill1_012\",\"Skill1_013\",\"Skill1_014\",\"Skill1_015\",\"Skill1_016\",\"Skill1_017\",\"Skill1_018\",\"Skill1_019\",\"Skill1_020\",\"Skill1_021\",\"Skill1_022\",\"Skill1_023\",\"Skill1_024\",\"Skill1_025\",\"Skill1_026\",\"Skill1_027\",\"Skill1_028\",\"Skill1_029\",\"Skill1_030\"]"],["SkillGroup_2","tid#NameSkillGroup_2","tid#DescSkillGroup_2","[\"Skill2_001\",\"Skill2_002\",\"Skill2_003\",\"Skill2_004\",\"Skill2_005\",\"Skill2_006\",\"Skill2_007\",\"Skill2_008\",\"Skill2_009\",\"Skill2_010\",\"Skill2_011\",\"Skill2_012\",\"Skill2_013\",\"Skill2_014\",\"Skill2_015\",\"Skill2_016\",\"Skill2_017\",\"Skill2_018\",\"Skill2_019\",\"Skill2_020\",\"Skill2_021\",\"Skill2_022\",\"Skill2_023\",\"Skill2_024\",\"Skill2_025\",\"Skill2_026\",\"Skill2_027\",\"Skill2_028\",\"Skill2_029\",\"Skill2_030\"]"],["SkillGroup_3","tid#NameSkillGroup_3","tid#DescSkillGroup_3","[\"Skill3_001\",\"Skill3_002\",\"Skill3_003\",\"Skill3_004\",\"Skill3_005\",\"Skill3_006\",\"Skill3_007\",\"Skill3_008\",\"Skill3_009\",\"Skill3_010\",\"Skill3_011\",\"Skill3_012\",\"Skill3_013\",\"Skill3_014\",\"Skill3_015\",\"Skill3_016\",\"Skill3_017\",\"Skill3_018\",\"Skill3_019\",\"Skill3_020\",\"Skill3_021\",\"Skill3_022\",\"Skill3_023\",\"Skill3_024\",\"Skill3_025\",\"Skill3_026\",\"Skill3_027\",\"Skill3_028\",\"Skill3_029\",\"Skill3_030\"]"]];

    var TaskInvite = [["Id","Order","NextId","Condition","Reward"],["Invite1_1",1,"Invite1_2","{\"InviteNum\":1}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":100}]"],["Invite1_2",1,"Invite1_3","{\"STAR\":50,\"InviteNum\":1}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":120}]"],["Invite1_3",1,"Invite1_4","{\"STAR\":150,\"InviteNum\":1}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":150}]"],["Invite1_4",1,"","{\"STAR\":500,\"InviteNum\":1}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":200}]"],["Invite2_1",2,"Invite2_2","{\"InviteNum\":3}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":200}]"],["Invite2_2",2,"Invite2_3","{\"STAR\":50,\"InviteNum\":3}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":240}]"],["Invite2_3",2,"Invite2_4","{\"STAR\":150,\"InviteNum\":3}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":300}]"],["Invite2_4",2,"","{\"STAR\":500,\"InviteNum\":3}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":400}]"],["Invite3_1",3,"Invite3_2","{\"InviteNum\":5}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":300}]"],["Invite3_2",3,"Invite3_3","{\"STAR\":50,\"InviteNum\":5}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":360}]"],["Invite3_3",3,"Invite3_4","{\"STAR\":150,\"InviteNum\":5}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":450}]"],["Invite3_4",3,null,"{\"STAR\":500,\"InviteNum\":5}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":600}]"],["Invite4_1",4,"Invite4_2","{\"InviteNum\":7}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":400}]"],["Invite4_2",4,"Invite4_3","{\"STAR\":50,\"InviteNum\":7}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":480}]"],["Invite4_3",4,"Invite4_4","{\"STAR\":150,\"InviteNum\":7}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":600}]"],["Invite4_4",4,null,"{\"STAR\":500,\"InviteNum\":7}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":800}]"],["Invite5_1",5,"Invite5_2","{\"InviteNum\":10}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":500}]"],["Invite5_2",5,"Invite5_3","{\"STAR\":50,\"InviteNum\":10}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":600}]"],["Invite5_3",5,"Invite5_4","{\"STAR\":150,\"InviteNum\":10}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":750}]"],["Invite5_4",5,null,"{\"STAR\":500,\"InviteNum\":10}","[{\"type\":\"Currency\",\"id\":\"DIAMOND\",\"amount\":1000}]"]];

    var BaseConfigs = {
        'ActionPoint'   : ActionPoint,
        'Barrage'   : Barrage,
        'BarrageGroup'   : BarrageGroup,
        'BuildingBase'   : BuildingBase,
        'CatToys'   : CatToys,
        'Entrustment'   : Entrustment,
        'EntrustmentList'   : EntrustmentList,
        'FishingBaits'   : FishingBaits,
        'FishingFishes'   : FishingFishes,
        'FishingHitZone'   : FishingHitZone,
        'FishingResult'   : FishingResult,
        'FoodsList'   : FoodsList,
        'Guide'   : Guide$1,
        'GuideGroup'   : GuideGroup,
        'Kanban'   : Kanban,
        'Mail'   : Mail$1,
        'MissionEvent'   : MissionEvent,
        'NauticalFoods'   : NauticalFoods,
        'NauticalPhotoReward'   : NauticalPhotoReward,
        'NauticalProject'   : NauticalProject,
        'NauticalProjectList'   : NauticalProjectList,
        'NauticalTools'   : NauticalTools,
        'OrderSystem'   : OrderSystem,
        'Scene'   : Scene$1,
        'Sign'   : Sign$1,
        'Sound'   : Sound,
        'SpecialGame'   : SpecialGame,
        'SpecialGameGroup'   : SpecialGameGroup,
        'SpecialGameStrength'   : SpecialGameStrength,
        'SpecialGameStrengthRole'   : SpecialGameStrengthRole,
        'Staff'   : Staff$1,
        'StaffGroup'   : StaffGroup,
        'StaffSkill'   : StaffSkill,
        'StaffSkillGroup'   : StaffSkillGroup,
        'TaskInvite'   : TaskInvite,
    };

    let checkCompressed = function (platform, model, benchmarkLevel) {
    	if (Laya.LocalStorage.getJSON("huanzhi") == false) {
    		return false;
    	} else if (Laya.LocalStorage.getJSON("huanzhi") == true) {
    		return true;
    	}
    	return false;
    	//暂时屏蔽自动检测高低配机型
    	if (platform == "ios") {
    		let model1 = model.split("<")[1].split(",")[0];
    		let lv = 0;
    		if (model1.search("iPad") == -1) {
    			//iphone
    			lv = Number(model1.split("iPhone")[1]);
    			if (lv > 9) {
    				return false;
    			}
    		} else {
    			//ipad
    			lv = Number(model1.split("iPad")[1]);
    			if (lv > 4) {
    				return false;
    			}
    		}
    		return true;
    	} else {
    		if (Number(benchmarkLevel) > 9 || Number(benchmarkLevel) == -1) {
    			return false;
    		}
    		return true;
    	}
    };
    class Main {

    	constructor() {
    		Constants.shouji = 1;
    		//根据IDE设置初始化引擎
    		if (window["Laya3D"]) Laya3D.init(GameConfig.width, GameConfig.height);
    		else Laya.init(GameConfig.width, GameConfig.height, Laya["WebGL"]);
    		Laya["Physics"] && Laya["Physics"].enable();
    		Laya["DebugPanel"] && Laya["DebugPanel"].enable();
    		Laya.stage.scaleMode = GameConfig.scaleMode;
    		Laya.stage.screenMode = GameConfig.screenMode;
    		Laya.stage.alignV = GameConfig.alignV;
    		Laya.stage.alignH = GameConfig.alignH;
    		Laya.stage.frameRate = "slow";
    		Laya.UIConfig.showButtons = false;
    		Laya.SoundManager.autoStopMusic = true;
    		Laya.SoundManager.useAudioMusic = false;
    		this.dabaoceshi();
    		let bg = new Laya.Image();
    		bg.name = 'startbg';

    		bg.loadImage('bg.jpg');
    		bg.sizeGrid = "18,16,14,18";
    		bg.width = Laya.stage.width + 100;
    		bg.height = Laya.stage.height + 100;
    		bg.pos(-50, -50);
    		Laya.stage.addChild(bg);
    		//兼容微信不支持加载scene后缀场景
    		Laya.URL.exportSceneToJson = GameConfig.exportSceneToJson;

    		//if (!((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone )&& Constants.shouji) && !Laya.Browser.onMiniGame) {
    			//  SdkHelper.init();  //用于本地测试
    		//}
    		if((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone )&& Constants.shouji){//
    			//海外版特殊处理，进入loading再接入sdk
    			EventMgr.getInstance().registEvent('PP_INIT_FISHING', this, this.ppinit);
    			System.setAppId(WxConfigMgr.getInstance().getAppid());
    			EventMgr.getInstance().registEvent("LOADING_SHOW_TRANSITION", this, this.showTransition);
    		}
    		else if (Laya.Browser.onMiniGame) {
    			System.setAppId(WxConfigMgr.getInstance().getAppid());
    			SdkHelper.init(WxConfigMgr.getInstance().getAppVersion());
    			SdkHelper.onShareAppMessage(0);
    			SdkHelper.loadRewardVideoAd();
    			let info = wx.getSystemInfoSync();
    			System._brand = info.brand;
    			System._model = info.model;
    			System._wxversion = info.version;
    			System._systemVersion = info.system;
    			System._platform = info.platform;
    			System._benchmarkLevel = info.benchmarkLevel;
    			if (Laya.LocalStorage.getJSON('gaihuazhi') == true) {
    				Laya.LocalStorage.setJSON('gaihuazhi', false);
    				Laya["MiniAdpter"].removeAll();
    			}
    			WxConfigMgr.getInstance().compressed = checkCompressed(info.platform, info.model, info.benchmarkLevel);
    			window.wx.setPreferredFramesPerSecond(60);
    			let width = Laya.stage.width;
    			let height = Laya.stage.height;
    			//如果是微信的话，尝试读取 appid 和 版本号，appid 和版本号决定了资源的 cdn 路径以及 global 地址
    			// let fs = wx.getFileSystemManager();

    			// fs.readFile({
    			// 	filePath: "pub.conf",
    			// 	encoding: "utf-8",
    			// 	success: function(res) {
    			// 		console.debug("..... pub conf: " + res.data);
    			// 		let cc = JSON.parse(res.data);
    			// 		console.debug("..... pub platform: " + cc['platform']);
    			// 		WxConfigMgr.getInstance().setPlatform(cc['platform']);
    			// 	},
    			// 	fail: function(res) {
    			// 		console.error("..... read version failed ....." + res.errMsg)
    			// 	},
    			// });

    			Laya.URL.basePath = WxConfigMgr.getInstance().CdnBaseUrl;
    			Laya["MiniAdpter"].nativefiles = [
    				// "layaNativeDir", "wxlocal",
    				"config/",
    				// "bg/",
    				// "cat_collect/",
    				// "collect/",
    				"prefab/",
    				"scene/",
    				"ui/",
    				// "shopitems/",
    				"loading/",
    				"rank_open",
    				"openProject",
    				"sound/",
    				// "menu/",

    				"js/",
    				"libs/",
    				"fileconfig.json",
    				"game.js",
    				"game.json",
    				"index.js",
    				"project.config.json",
    				"unpack.json",
    				"version.json",
    				"version_compressed.json",
    				"subpackage.json",
    				"subpackage_compressed.json",
    				"weapp-adapter.js"
    			];
    		}
    		//打开调试面板（通过IDE设置调试模式，或者url地址增加debug=true参数，均可打开调试面板）
    		if (GameConfig.debug || Laya.Utils.getQueryString("debug") == "true") Laya.enableDebugPanel();
    		if (GameConfig.physicsDebug && Laya["PhysicsDebugDraw"]) Laya["PhysicsDebugDraw"].enable();
    		if (GameConfig.stat) Laya.Stat.show();
    		Laya.alertGlobalError = true;

    		EventMgr.getInstance().registEvent("WX_USER_INFO_SETTED", this, this.onUserInfoSetted);
    		EventMgr.getInstance().registEvent("LOADING_SHOW_MAINTAIN", this, this.showMaintain);
    		// EventMgr.getInstance().registEvent("LOADING_SHOW_TRANSITION", this, this.showTransition);
    		//激活资源版本控制，version.json由IDE发布功能自动生成，如果没有也不影响后续流程
    		//if (!((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone)&& Constants.shouji)) {
    			if (WxConfigMgr.getInstance().compressed) {
    				Laya.ResourceVersion.enable("version_compressed.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
    			} else {
    				Laya.ResourceVersion.enable("version.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
    			}
    		//}
    		this._initTip();
    	}
    	dabaoceshi() {
                 if(WxConfigMgr.GlobalUrl.pp.url=="game-cats-life.cp.freegames123.com:17405"&&Constants.shouji==1){
    				
    			 }else {
    				var vConsole =new VConsole();
    				let  label=new Laya.Label();
    				label.text='测试';
    				label.color="red";
    				label.fontSize=100;
    				label.x=1000;
    				label.y=100;
    				label.zOrder = 20000;
    				Laya.stage.addChild(label);
    			 }
    	}
    	ppinit() {
    		SdkHelper.loadRewardVideoAd();
    		System.pplogin();
    	}
    	showMaintain() {
    		Laya.stage.removeChildByName("MainTain");
    		let maintain = new LoadingMaintain();
    		maintain.name = "MainTain";
    		maintain.zOrder = 20001;
    		Laya.stage.addChild(maintain);
    	}

    	showTransition() {
    		console.log("压 土豆  ftue_loadingPicture_start");
    		let str = "ftue_loadingPicture_start";
    		EventMgr.getInstance().postEvent("SDK_SEND_EVENT", [str, null, 1]);
    		Laya.stage.removeChildByName("LoadTransition");
    		let transition = new LoadingTransition();
    		transition.name = "LoadTransition";
    		transition.zOrder = 20000;
    		Laya.stage.addChild(transition);
    	}

    	onUserInfoSetted() {
    		console.log("UserInfoSetted");
    		System.setAuthorized(true);
    		GameContext.__authorized = true;
    		this._playerInfoSeted = true;
    		// this.doWXLogin();
    		Laya.stage.removeChildByName("start_btn");
    	}

    	getObj(className) {
    		return className()
    	}

    	onLogin(res) {
    	}

    	onVersionLoaded() {
    		if (Laya.Browser.onMiniGame) {
    			Laya.ResourceVersion.enableSubPackage(WxConfigMgr.getInstance().subPackageCfgName, Laya.Handler.create(this, this.onSubPackageManifestLoaded));
    		} else {
    			this.onSubPackageManifestLoaded();
    		}
    	}
    	onSubPackageManifestLoaded() {
    		if (Laya.Browser.onMiniGame) {
    			window.wx.getSetting({
    				success(res) {
    					console.log(res.authSetting);
    					if (res.authSetting["scope.userInfo"]) {
    						console.log("用户已授权");
    						window.wx.getUserInfo({
    							lang: "zh_CN",
    							success(res) {
    								console.log(res);
    								PlayerSystem.setPlayerInfo(res.userInfo);
    								//此时可进行登录操作
    								EventMgr.getInstance().postEvent("WX_USER_INFO_SETTED");
    							}
    						});
    					} else if (false) {
    						console.log("用户未授权");
    						let wxWidth = wx.getSystemInfoSync().windowWidth;
    						let wxHeight = wx.getSystemInfoSync().windowHeight;
    						let heightScale = Laya.Browser.height / 2436;
    						let img = new Laya.Image();
    						img.skin = "loading/startbtn.png";
    						img.anchorX = 0.5;
    						img.anchorY = 0.5;
    						img.scaleX = 2;
    						img.scaleY = 2;
    						img.x = Laya.Browser.width / 2 / heightScale; //0.5 * Laya.stage.width;
    						img.y = Laya.stage.height - 500;
    						img.zOrder = 10000;
    						Laya.stage.addChild(img);
    						img.name = "start_btn";

    						let btn_width = wxWidth; //400 / Laya.stage.width * wxWidth;
    						let btn_height = wxHeight; //200 / Laya.stage.height * wxHeight;
    						let btn_left = 0; //0.5 * (wxWidth - btn_width);
    						let btn_top = 0; //(img.y-100) / Laya.stage.height * wxHeight;

    						let button = window.wx.createUserInfoButton({
    							type: 'text',
    							text: '', //'授权',
    							style: {
    								left: btn_left,
    								top: btn_top,
    								width: btn_width,
    								height: btn_height,
    								backgroundColor: '#00000000', //'#00ff00ff',//最后两位为透明度
    								color: '#ffffff',
    								fontSize: 20,
    								textAlign: "center",
    								lineHeight: btn_height,
    							}
    						});
    						button.onTap((res) => {
    							if (res.userInfo) {
    								console.log("用户授权:", res);
    								PlayerSystem.setPlayerInfo(res.userInfo);
    								//此时可进行登录操作
    								EventMgr.getInstance().postEvent("WX_USER_INFO_SETTED");
    								button.destroy();
    							} else {
    								console.log("用户拒绝授权:", res);
    								// wx.exitMiniProgram({
    								// 	fail() {
    								// 		console.log("退出游戏失败")
    								// 	}
    								// })
    							}
    						});
    					}
    				}
    			});
    		}
    		//激活大小图映射，加载小图的时候，如果发现小图在大图合集里面，则优先加载大图合集，而不是小图
    		Laya.AtlasInfoManager.enable("fileconfig.json", Laya.Handler.create(this, function () {
    			ConfigMgr$1.getInstance().loadConfig("Const", Laya.Handler.create(this, function () {
    				ConfigMgr$1.getInstance().loadConfig("Translate", Laya.Handler.create(this, this.onConfigLoaded));
    			}));
    		}));
    	}

    	onConfigLoaded() {
    		let logo = new Laya.Image("loading/LOGO.png");
    		logo.name = 'startbg1';
    		logo.scaleX = logo.scaleY = 2;
    		logo.x = (Laya.stage.width - logo.scaleX * 490) / 2+6;
    		logo.y = 200;
    		Laya.stage.addChild(logo);

    		let mao = new Laya.Image();
    		mao.name = 'startbg2';
    		mao.loadImage('loading/mao.png');
    		mao.scaleX = mao.scaleY = 2;
    		mao.x = (Laya.stage.width - mao.scaleX * 399) / 2-15;
    		mao.y = (Laya.stage.height - mao.scaleY * 288) / 2-8;
    		Laya.stage.addChild(mao);
    		for (let name of Object.keys(BaseConfigs)) {
    			ConfigMgr$1.getInstance().insertConfig(name, BaseConfigs[name]);
    		}
    		let configFinish = false;
    		let loginSceneFinish = false;
    		let loginScene = null;
    		System.initService();
    		if (!Laya.Browser.onMiniGame && !((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone)&& Constants.shouji)) {
    			for (const [k, v] of ConfigMgr$1.getInstance().getSubConfigMap()) {
    				ConfigMgr$1.getInstance().loadSubConfig(k);
    			}
    			SceneMgr.getInstance().switchScene(Constants.LoginSceneId, Laya.Handler.create(this, function (scene) {
    				loginSceneFinish = true;
    				loginScene = scene;
    				if (loginScene) {
    					loginScene.visible = configFinish;
    				}
    				// if (Laya.Browser.onMiniGame) {
    				//     loginScene.visible = false;
    				//     if(configFinish)
    				//         loginScene.onLoginClick();
    				// }
    			}));
    		} else if (!Laya.Browser.onMiniGame && ((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) && Constants.shouji)) {
    			for (const [k, v] of ConfigMgr$1.getInstance().getSubConfigMap()) {
    				ConfigMgr$1.getInstance().loadSubConfig(k);
    			}
    		}
    		else {
    			System._load_subpackage_begin = (+ new Date());
    			console.log("start loadSubpackage sub_inloading", System._load_subpackage_begin);
    			System._loadState = 0;
    			var loadTask = wx.loadSubpackage({
    				name: WxConfigMgr.getInstance().subPackageName,
    				success: function (res) {
    					System._load_subpackage_end = (+ new Date());
    					console.log("loadSubpackage", WxConfigMgr.getInstance().subPackageName, "success", System._load_subpackage_begin);
    					System._loadState = 1;
    					Laya.timer.frameOnce(1, null, function () {
    						SoundManager._isLoading = true;
    						let load_subpackage_sound_begin = (+ new Date());
    						wx.loadSubpackage({
    							name: 'sound',
    							success: function (res) {
    								SoundManager._isLoading = null;
    								let load_subpackage_sound_end = (+ new Date());
    								console.log("loadSubpackage sound success", load_subpackage_sound_end - load_subpackage_sound_begin);
    								EventMgr.getInstance().postEvent("SOUND_REPLAY_BGM", "bgm");
    								wx.loadSubpackage({
    									name: 'subconfig',
    									success: function (res) {
    										for (const [k, v] of ConfigMgr$1.getInstance().getSubConfigMap()) {
    											ConfigMgr$1.getInstance().loadSubConfig(k);
    										}
    									},
    									fail: function (res) {
    										console.log("loadSubpackage subconfig fail");
    									}
    								});
    							},
    							fail: function (res) {
    								console.log("loadSubpackage sound fail");
    							}
    						});
    					});
    				},
    				fail: function (res) {
    					System._loadState = -1;
    					console.log("loadSubpackage sub_inloading fail");
    				}
    			});
    			System.wxLogin();
    		}
    		EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_start"]);

    		System._loader_begin = (+ new Date());
    		PreGameLoader.getInstance()._loader_begin = System._loader_begin;
    		PreGameLoader.getInstance().start(Laya.Handler.create(this, function () {
    		if((Laya.Browser.onAndroid || Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone )&& Constants.shouji==1){//
    		SdkHelper.init(WxConfigMgr.getInstance().getAppVersion(), WxConfigMgr.getInstance().getAppid());
    		}
    			System._loader_end = (+ new Date());
    			EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_end", { cost: System._loader_end - System._loader_begin }]);
    			//先初始化加载好WebSocketClient要的proto文件
    			if (!Laya.Browser.onMiniGame) {
    				configFinish = true;
    				System.initSystems();
    				// if (Laya.Browser.onMiniGame) {
    				//     if(loginScene)
    				//         loginScene.onLoginClick();
    				// }else
    				if (loginScene) {
    					loginScene.visible = configFinish;
    				}
    			} else {
    				System._transition_begin = (+ new Date());
    				EventMgr.getInstance().postEvent("SDK_SEND_EVENT", ["loading_resources_start"]);
    				//EventMgr.getInstance().postEvent("LOADING_SHOW_TRANSITION");
    				wx.onShow((res) => {
    					Laya.timer._update();
    					Laya.timer.resume();
    					Laya.updateTimer._update();
    					Laya.updateTimer.resume();
    					GameContext.showTime = TimeUtils.getTimeNowOnServer();
    					console.log("hideTime", GameContext.hideTime);
    					console.log("showTime", GameContext.showTime);
    					if (System._needRestart || GameContext.hideTime == null || GameContext.showTime - GameContext.hideTime >= 180000) {
    						//EventMgr.getInstance().postEvent("LOADING_SHOW_TRANSITION");
    						WebSocketClient.getInstance().clearNetStatus();//先断网，不然屁事多
    						Laya.systemTimer.frameOnce(1, null, function () {
    							System.clear();
    						});
    						Laya.systemTimer.frameOnce(5, null, function () {
    							System.restart();
    						});
    					} else {
    						EventMgr.getInstance().postEvent("SOUND_REPLAY_BGM", "bgm");
    						// Laya.timer.frameOnce(1, null, function() {
    						// 	Laya.Dialog.close("ui/LoadingGap.scene");
    						// 	Laya.Dialog.open("ui/LoadingGap.scene", false);
    						// });
    					}
    					wx.setKeepScreenOn({
    						keepScreenOn: true
    					});
    				});
    				wx.setKeepScreenOn({
    					keepScreenOn: true
    				});
    				wx.onHide((res) => {
    					Laya.timer.pause();
    					Laya.updateTimer.pause();
    					GameContext.hideTime = TimeUtils.getTimeNowOnServer();
    				});
    				this._loadFinished = true;
    				// if(this._playerInfoSeted) {
    				this.doWXLogin();
    				// }
    				// wx.onAudioInterruptionEnd((res)=>{EventMgr.getInstance().postEvent("SOUND_REPLAY_BGM", "bgm");});
    				wx.onMemoryWarning(function () {
    					Laya.Scene.gc();
    					wx.triggerGC();
    					console.debug('运行内存不足');
    				});
    			}
    		}));
    	}

    	doWXLogin() {
    		if (this._loadFinished) {
    			System.enter();
    		}
    	}

    	_initTip() {
    		TipsPopMgr.getInstance().init();
    	}

    	_showToStage(dis, offX = 0, offY = 0) {

    		let rec = dis.getBounds();
    		dis.x = Laya.ILaya.stage.mouseX + offX + this.offsetX;
    		dis.y = Laya.ILaya.stage.mouseY + offY;

    		if (dis.x + rec.width > Laya.ILaya.stage.width) {
    			dis.x -= rec.width + offX;
    		}
    		if (dis.y + rec.height > Laya.ILaya.stage.height) {
    			dis.y -= rec.height + offY;
    		}
    		// console.debug(Laya.ILaya.stage.width, Laya.ILaya.stage.height);
    		// console.debug(dis.x, dis.y);
    	}

    	_hideTip() {
    		// console.debug("hideTip")
    		this._tipBox.visible = false;
    		Laya.ILaya.timer.clear(this, this._hideTip);
    	}
    }
    //激活启动类
    new Main();

}());
